(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   3624648,     136362]
NotebookOptionsPosition[   2644537,     106069]
NotebookOutlinePosition[   3116790,     119133]
CellTagsIndexPosition[   3100425,     118655]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 6.0", "Title",
 TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
 TextAlignment->Center],

Cell[TextData[StyleBox["Reference Guide",
 FontWeight->"Bold"]], "Subtitle",
 PageBreakBelow->True,
 TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
 TextAlignment->Center],

Cell[TextData[{
 "HTML version: ",
 ButtonBox["http://www.feyncalc.org/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
 TextAlignment->Center],

Cell[BoxData[
 ButtonBox[
  RowBox[{
  "Click", " ", "to", " ", "save", " ", "this", " ", "notebook", " ", 
   "without", " ", "cache", " ", "and", " ", "FrontEndVersion", " ", "info"}],
  
  ButtonFunction:>(Module[{fn}, SetDirectory[
      DirectoryName[fn = ToFileName[
         ReplaceAll["FileName", 
          NotebookInformation[
           InputNotebook[]]]]]]; 
    newfn = StringReplace[fn, ".nb" -> "NoCache.nb"]; 
    If[FileNames[newfn] =!= {}, 
      DeleteFile[newfn]]; Put[
      ReplaceAll[
       NotebookGet[
        InputNotebook[]], Condition[FrontEndVersion -> Pattern[v, 
           Blank[]], Head[v] == String] :> Sequence[]], "fcr.nb"]; 
    CopyFile["fcr.nb", newfn]; DeleteFile["fcr.nb"]]),
  Evaluator->Automatic]], NotebookDefault,
 Active->True,
 CellTags->"ClickSave"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"$Version", ",", 
   RowBox[{"Date", "[", "]"}], ",", "$UserName"}], "}"}]], "Input",
 CellLabel->"In[1]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"6.0 for Linux x86 (64-bit) (March 26, 2007)\"\>", ",", 
    RowBox[{"{", 
     RowBox[{
     "2007", ",", "5", ",", "9", ",", "15", ",", "4", ",", 
      "13.672289`7.888416215386728"}], "}"}], ",", "\<\"rolfm\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions", "Section",
 CellTags->"Functions"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell["\<\
This is only for timing purposes and can be omitted in general.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"absolutestarttime", "=", 
  RowBox[{"AbsoluteTime", "[", "]"}]}]], "Input",
 CellLabel->"In[2]:="],

Cell[BoxData[
 FormBox["3.387711853789883`16.282481452911043*^9", 
  TraditionalForm]], "Output",
 CellLabel->"Out[2]="]
}, Open  ]],

Cell["<<HighEnergyPhysics`FeynCalc`", "Input",
 CellLabel->"In[3]:="],

Cell["Abbreviation", "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Abbreviation"],

Cell["\<\
Abbreviation is a function used by OneLoop and PaVeReduce for generating \
smaller files when saving results to the hard disk. The convention is that a \
definition like GP = GluonPropagator should be accompanied by the definition \
Abbreviation[GluonPropagator] = HoldForm[GP].\
\>", "Text",
 CellTags->"Abbreviation"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["$Abbreviations",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Abbreviations",
  ButtonNote->"$Abbreviations"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"Abbreviation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GP", "[", 
  RowBox[{"p", ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", " ", "a"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", " ", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Abbreviation",
 CellLabel->"In[4]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellTags->"Abbreviation",
 CellLabel->"Out[4]="]
}, Open  ]]
}, Open  ]],

Cell["Amplitude", "ObjectName",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amplitude"],

Cell["\<\
Amplitude is a database of Feynman amplitudes. Amplitude[\"name\"] returns \
the amplitude corresponding to the string \"name\". A list of all defined \
names is obtained with Amplitude[]. New amplitudes can be added to the file \
\"Amplitude.m\". It is strongly recommended to use names that reflect the \
process. The option Gauge \[Rule] 1 means `t Hooft Feynman gauge; \
Polarization \[Rule] 0 gives unpolarized OPE-type amplitudes, Polarization \
\[Rule] 1 the polarized ones.\
\>", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amplitude", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[5]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"QuarkMass", "\[Rule]", "0"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"Amplitude"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Amplitude", "[", "]"}], "//", "Length"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[6]:="],

Cell[BoxData[
 FormBox["98", TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[6]="]
}, Open  ]],

Cell["This is the amplitude of a gluon self-energy diagram.", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<se1g1\>\"", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[7]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"c", "d"}], 
     RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"e", "f"}], 
     RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox[
      RowBox[{"-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"-", "q"}],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[7]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[8]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Alpha]", "\[Rho]"}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Beta]", "\[Sigma]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Alpha]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Alpha]",
             TraditionalForm],
            TraditionalForm]]}], ")"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Beta]", "\[Mu]"}]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Beta]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Beta]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Beta]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Sigma]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
     SuperscriptBox["q", "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[8]="]
}, Open  ]],

Cell[TextData[{
 "This is the amplitude for graph 6.2 from the paper Z.Phys C ",
 StyleBox["70:",
  FontWeight->"Bold"],
 "637-654, 1996",
 StyleBox[".",
  FontWeight->"Bold"]
}], "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q1", ",", "q2", ",", 
   RowBox[{"EpsEvaluate", "[", 
    RowBox[{"Trick", "[", 
     RowBox[{"Explicit", "[", 
      RowBox[{"Amplitude", "[", "\"\<gg2\>\"", "]"}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[9]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Global`OverVector", "::", "\<\"shdw\"\>"}], 
   RowBox[{
   ":", " "}], "\<\"Symbol \\!\\(TraditionalForm\\`\\\"OverVector\\\"\\) \
appears in multiple contexts \\!\\(TraditionalForm\\`\\({\\\"Global`\\\", \
\\\"System`\\\"}\\)\\); definitions in context \
\\!\\(TraditionalForm\\`\\\"Global`\\\"\\) may shadow or be shadowed by other \
definitions. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/shdw\\\", ButtonNote -> \
\\\"Global`OverVector::shdw\\\"]\\)\"\>"}], TraditionalForm]], "Message", \
"MSG",
 CellTags->"Amplitude"],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], "q1"}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "q2"}]}], "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
       SubsuperscriptBox["g", "s", "4"], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "1"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q2",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox["\[Nu]",
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], ")"}], " ", 
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "10"]}]]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "10"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox["\[Mu]",
                TraditionalForm],
               TraditionalForm]]}]}], ")"}], " ", 
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "1"], 
            SubscriptBox["\[Lambda]", "11"]}]]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "1"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "1"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "1"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}], " ", 
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "11"], 
            SubscriptBox["\[Lambda]", "12"]}]]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "12"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "5"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "5"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}], " ", 
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "12"]}]]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         FormBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"],
          TraditionalForm]}], ")"}], " ", 
       FormBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q2",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]],
        TraditionalForm], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]]}], ")"}], "/", 
     RowBox[{
      SuperscriptBox["q1", "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[9]="]
}, Open  ]]
}, Open  ]],

Cell["Amputate", "ObjectName",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amputate"],

Cell["\<\
Amputate[exp,q1,q2,...] amputates Eps and DiracGamma. Amputate[exp,q1,q2,Pair\
\[Rule]{p}] amputates also p.q1 and p.q2; Pair\[Rule]All amputates all except \
OPEDelta.\
\>", "Text",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amputate", "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[10]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Amputate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[11]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[11]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amputate", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[12]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       RowBox[{"$AL$165", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$165", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[12]="]
}, Open  ]]
}, Open  ]],

Cell["AnomalousDimension", "ObjectName",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["\<\
AnomalousDimension[name] is a database of anomalous dimensions of twist 2 \
operators.\
\>", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "AnomalousDimension", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[13]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[13]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["Polarized case", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", " ", 
  RowBox[{"AnomalousDimension", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[14]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[14]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[15]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     FractionBox["4", "m"], "+", 
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", "6"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[15]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[16]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["8", "m"], "-", 
     FractionBox["16", 
      RowBox[{"m", "+", "1"}]]}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[16]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[17]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", 
     FractionBox["8", "m"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[17]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[18]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"8", " ", 
     SubscriptBox["T", "f"]}], "3"], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", "m"], "+", 
      FractionBox["16", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["22", "3"]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[18]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"PS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gpsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[19]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", 
      RowBox[{"m", "+", "1"}]], "+", 
     FractionBox["3", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox["2", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
     FractionBox["1", "m"], "-", 
     FractionBox["1", 
      SuperscriptBox["m", "2"]], "+", 
     FractionBox["2", 
      SuperscriptBox["m", "3"]]}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[19]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[20]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        SuperscriptBox["m", "2"]], "+", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "m"], "+", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        RowBox[{"m", "+", "1"}]], "-", 
       RowBox[{"24", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       RowBox[{"32", " ", 
        RowBox[{
         SubscriptBox["S", "12"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       RowBox[{"32", " ", 
        RowBox[{
         SubscriptBox["S", "21"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox[
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "m"], "+", 
       FractionBox[
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        RowBox[{"m", "+", "1"}]], "-", 
       RowBox[{"32", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "3"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       RowBox[{"64", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "12"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "-", 
       FractionBox["40", "m"], "+", 
       FractionBox["40", 
        RowBox[{"m", "+", "1"}]], "+", 
       FractionBox["16", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
       FractionBox["8", 
        SuperscriptBox["m", "3"]], "+", 
       FractionBox["40", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "3"}], ")"}]}], " ", 
    SubsuperscriptBox["C", "F", "2"]}], "-", 
   RowBox[{
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["80", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       FractionBox["16", "3"], " ", 
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["88", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["2", "3"]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "-", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["536", "9"]}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       FractionBox["88", "3"], " ", 
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"16", " ", 
       RowBox[{
        SubscriptBox["S", "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"16", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"32", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox["212", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["748", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["44", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox["4", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["16", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
      FractionBox["17", "3"]}], ")"}], " ", 
    SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[20]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[21]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["14", "m"], "-", 
      FractionBox["19", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["1", 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox["2", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
   RowBox[{"16", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}], "m"]}], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["4", "m"], "+", 
      FractionBox["3", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["3", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["2", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox["12", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[21]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[22]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["15", "m"], "-", 
      FractionBox["6", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["12", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox["3", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
    SubsuperscriptBox["C", "F", "2"]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        RowBox[{"3", " ", "m"}]]}], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["7", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["2", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["2", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "+", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "+", 
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{
          SubsuperscriptBox["S", "1", "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "m"]}], "+", 
      FractionBox[
       RowBox[{
        SubsuperscriptBox["S", "1", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", "m"}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["56", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["20", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["28", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox["38", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox["6", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
    SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[22]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[23]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      RowBox[{
       FractionBox["134", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox["S", "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "21"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["107", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["241", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["58", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox["86", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["8", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox["48", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
      FractionBox["16", "3"]}], ")"}], " ", 
    SubsuperscriptBox["C", "A", "2"]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["5", "9"]}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox["14", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["19", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "+", 
      FractionBox["1", "3"]}], ")"}], " ", 
    SubscriptBox["C", "A"]}], "+", 
   RowBox[{"8", " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["10", 
        RowBox[{"m", "+", "1"}]]}], "+", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "1", "+", 
      FractionBox["10", "m"], "-", 
      FractionBox["10", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox["m", "3"]]}], ")"}], " ", 
    SubscriptBox["T", "f"]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[23]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized (different representation)"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "GSGG1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[24]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{"m", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"m", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"m", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"m", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"48", " ", "m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "469"}], ")"}]}], 
                  "+", "698"}], ")"}]}], "+", "7"}], ")"}]}], "+", "258"}], 
           ")"}]}], "+", "144"}], 
        RowBox[{"9", " ", 
         SuperscriptBox["m", "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
         FormBox[
          FractionBox["m", "2"],
          TraditionalForm], "\[NoBreak]", ")"}]}], 
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      RowBox[{
       SubsuperscriptBox["S", "3", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
       FormBox[
        FractionBox["m", "2"],
        TraditionalForm], "\[NoBreak]", ")"}], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"67", " ", "m", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}], "2"]}], "+", "144"}], ")"}]}],
           "+", "72"}], ")"}], " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      RowBox[{"4", " ", 
       RowBox[{
        SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
         FractionBox["m", "2"],
         TraditionalForm], "\[NoBreak]", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "+", 
      RowBox[{"8", " ", 
       RowBox[{
        OverscriptBox["S", "~"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}]}], ")"}], " ", 
    SubsuperscriptBox["C", "A", "2"]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"m", " ", 
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "m", " ", 
            RowBox[{"(", 
             RowBox[{"m", "+", "1"}], ")"}]}], "+", "13"}], ")"}]}], "-", 
        "3"}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], "9"]}], ")"}], " ", 
    SubscriptBox["C", "A"]}], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     SubscriptBox["C", "F"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"m", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"m", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "5"}], ")"}]}], 
                "+", "1"}], ")"}]}], "-", "8"}], ")"}]}], "+", "2"}], ")"}]}],
        "+", "4"}], ")"}], " ", 
     SubscriptBox["T", "f"]}], 
    RowBox[{
     SuperscriptBox["m", "3"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], TraditionalForm]], "Output",\

 CellTags->"AnomalousDimension",
 CellLabel->"Out[24]="]
}, Open  ]],

Cell[TextData[{
 "Check that all odd moments give the same for the two representations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Gamma]", 
     RowBox[{"S", ",", "gg", " "}], 
     RowBox[{
      RowBox[{"(", "1", ")"}], " "}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"%", "-", "%%"}], "/.", 
    RowBox[{"OPEm", "\[Rule]", "ij"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"ij", ",", "1", ",", "17", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[25]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
    ",", "0"}], "}"}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[25]="]
}, Open  ]]
}, Open  ]],

Cell["AntiCommutator", "ObjectName",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell["\<\
AntiCommutator[x, y] = c defines the anti-commutator of the non commuting \
objects x and y.\
\>", "Text",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "This declares ",
 StyleBox["a",
  FontFamily->"Courier"],
 " and ",
 StyleBox["b",
  FontFamily->"Courier"],
 " as noncommutative variables."
}], "Text",
 CellTags->"AntiCommutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[26]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[27]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "a", ",", "\[MediumSpace]", "b", "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[27]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[28]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "+", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[28]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[29]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[29]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[30]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"a", ".", "a"}]}], "-", 
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"b", ".", "b"}]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[30]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[31]:="],

Cell["Defining {c,d} = z results in replacements of c.d by z-d.c.", "Text",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], " ", "=", " ", "z"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[32]:="],

Cell[BoxData[
 FormBox["z", TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[32]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{"d", " ", ".", " ", "c", " ", ".", " ", "d"}], " ", "]"}]], "Input",\

 CellTags->"AntiCommutator",
 CellLabel->"In[33]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"d", " ", "z"}], "-", 
   RowBox[{"d", ".", "d", ".", "c"}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[33]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], " ", "=", " ", 
  OverscriptBox["z", "~"]}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[34]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["z", "~"], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[34]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{
   OverscriptBox["d", "~"], " ", ".", " ", 
   OverscriptBox["c", "~"], " ", ".", " ", 
   OverscriptBox["d", "~"]}], " ", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[35]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    OverscriptBox["d", "~"], " ", 
    OverscriptBox["z", "~"]}], "-", 
   RowBox[{
    OverscriptBox["c", "~"], ".", 
    OverscriptBox["d", "~"], ".", 
    OverscriptBox["d", "~"]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[35]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[36]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[37]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[38]:="]
}, Open  ]],

Cell["AntiQuarkField", "ObjectName",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell["\<\
AntiQuarkField is the name of a fermionic field. AntiQuarkField is just a \
name with no functional properties. Only typeset rules are attached.\
\>", "Text",
 CellTags->"AntiQuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 ", ",
 ButtonBox["QuarkField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkField",
  ButtonNote->"QuarkField"],
 "."
}], "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell[BoxData["AntiQuarkField"], "Input",
 CellTags->"AntiQuarkField",
 CellLabel->"In[39]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellTags->"AntiQuarkField",
 CellLabel->"Out[39]="]
}, Open  ]]
}, Open  ]],

Cell["AntiSymmetrize", "ObjectName",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell["\<\
AntiSymmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"AntiSymmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[40]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[40]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[41]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[41]="]
}, Open  ]]
}, Open  ]],

Cell["Anti5", "ObjectName",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Anti5"],

Cell[TextData[{
 "Anti5[exp] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "in exp to the right. Anti5[exp, n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "n times to the right. Anti5[exp, -n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " n times to the left."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "The naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "scheme is used."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracOrder",
  ButtonNote->"DiracOrder"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Anti5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"5", ",", "\[Mu]"}], "]"}], " "}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[42]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[42]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[43]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[43]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "1"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[44]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[44]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", 
  RowBox[{
  "5", ",", "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[45]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[45]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", "2"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[46]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[46]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%%", ",", "Infinity"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[47]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[47]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "Infinity"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[48]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[48]="]
}, Open  ]],

Cell[TextData[{
 "In the naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "- scheme D-dimensional ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 "-matrices anticommute with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"5", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[49]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[49]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[50]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[50]="]
}, Open  ]]
}, Open  ]],

Cell["Apart2", "ObjectName",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Apart2"],

Cell["\<\
Apart2[expr] partial fractions FeynAmpDenominators (and FAD's).\
\>", "Text",
 CellTags->"Apart2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 "."
}], "Text",
 CellTags->"Apart2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "M"}], "}"}], ",", 
   RowBox[{"q", "-", "p"}]}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[51]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[51]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart2", "[", "%", "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[52]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
   RowBox[{
    SuperscriptBox["m", "2"], "-", 
    SuperscriptBox["M", "2"]}]], TraditionalForm]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[52]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[53]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "m"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}], "-", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "M"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}]}], 
  RowBox[{
   SuperscriptBox["m", "2"], "-", 
   SuperscriptBox["M", "2"]}]]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[53]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["A0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"A0", "[", 
    SuperscriptBox["m", "2"], "]"}], TraditionalForm]]],
 "is the Passarino-Veltman one-point integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[54]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"A0",
 CellLabel->"Out[54]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[55]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[55]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[56]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[57]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}], " ", 
    SuperscriptBox["m", "2"]}], "+", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[57]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[58]:="],

Cell[TextData[{
 "According to the rules of dimensional regularization ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "(", "0", ")"}], TraditionalForm]]],
 " is set to 0."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", "0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[59]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[59]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  RowBox[{"SmallVariable", "[", 
   RowBox[{"M", "^", "2"}], "]"}], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[60]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[60]="]
}, Open  ]]
}, Open  ]],

Cell["A0ToB0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0ToB0"],

Cell[TextData[{
 "A0ToB0 is an option for A0. If set to True, A0[",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "] is expressed by (1 + B0[0, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "]) ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0ToB0",
 CellLabel->"In[61]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"A0ToB0",
 CellLabel->"Out[61]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0ToB0"]
}, Open  ]],

Cell["BackgroundGluonVertex", "ObjectName",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[TextData[{
 "BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or \
BackgroundGluonVertex[ p,mu,a , q,nu,b , k,la,c ] yields the 3-gluon vertex \
in the background field gauge, where the first set of arguments corresponds \
to the external background field. BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, \
{k,la,c}, {s,si,d}] or BackgroundGluonVertex[{mu,a}, {nu,b}, {la,c}, {",
 "si",
 ",d}] or BackgroundGluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
BackgroundGluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex, \
with {p,mu,a} and {k,la,c} denoting the external background fields. The \
gauge, dimension and the name of the coupling constant are determined by the \
options Gauge, Dimension and CouplingConstant. The Feynman rules are taken \
from L.Abbot ",
 StyleBox["NPB ",
  FontWeight->"Bold"],
 "185 (1981), 189-203; except that all momenta are incoming. Note that Abbots \
coupling constant convention is consistent with the default setting of \
GluonVertex."
}], "Text",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "BackgroundGluonVertex", "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[62]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[62]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[63]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "k", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[63]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[64]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[64]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[65]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", 
          FractionBox["k", "\[Alpha]"], "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", 
          FractionBox["q", "\[Alpha]"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[65]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[66]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Sigma]"}]]}], "\[Alpha]"], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}], "\[Alpha]"], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[66]="]
}, Open  ]]
}, Open  ]],

Cell["Bracket", "ObjectName",
 CellTags->"Bracket"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Bracket"],

Cell["Bracket is an option of Convolute.", "Text",
 CellTags->"Bracket"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"Bracket"]
}, Open  ]],

Cell["BReduce", "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BReduce"],

Cell["\<\
BReduce is an option for B0, B00, B1, B11 determining whether reductions to \
A0 and B0 will be done. \
\>", "Text",
 CellTags->"BReduce"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 "."
}], "Text",
 CellTags->"BReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BReduce"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s"}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[67]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"BReduce",
 CellLabel->"Out[67]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellLabel->"In[68]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[68]="]
}, Open  ]],

Cell["\<\
With BReduce\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[69]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", "s", ")"}], "s"], "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"BReduce",
 CellLabel->"Out[69]="]
}, Open  ]]
}, Open  ]],

Cell["B0", "ObjectName",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0"],

Cell[TextData[{
 "B0[pp, ma^2, mb^2] is the Passarino-Veltman two-point integral ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass^2. If the option \
BReduce is set to True certain B0's are reduced to A0's. Setting the option \
B0Unique to True simplifies B0[a,0,a] and B0[0,0,a]."
}], "Text",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B0", "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[70]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[70]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "p"}], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[71]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[71]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B0", ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[72]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[73]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[73]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[74]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "2"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[74]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", 
   RowBox[{"m", "^", "2"}], ",", 
   RowBox[{"m", "^", "2"}]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[75]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[75]="]
}, Open  ]]
}, Open  ]],

Cell["B0Real", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Real"],

Cell["\<\
B0Real is an option of B0 (default False). If set to True, B0 is assumed to \
be real and the relation B0[a,0,a] = 2 + B0[0,a,a]  is applied.\
\>", "Text",
 CellTags->"B0Real"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Real"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Real"],

Cell["By default the arguments are not assumed real.", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[76]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[76]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[77]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[77]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s", ",", 
   RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[78]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[78]="]
}, Open  ]]
}, Open  ]],

Cell["B0Unique", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Unique"],

Cell["\<\
B0Unique is an option of B0. If set to True, B0[0,0,m2] is replaced with \
(B0[0,m2,m2]+1) and B0[m2,0,m2] simplifies to (B0[0,m2,m2]+2).\
\>", "Text",
 CellTags->"B0Unique"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Unique"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Unique"],

Cell["By default transformation is done.", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[79]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[79]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[80]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[80]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]False, nothing happens.\
\>", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s", ",", 
   RowBox[{"B0Unique", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[81]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "0", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[81]="]
}, Open  ]]
}, Open  ]],

Cell["B00", "ObjectName",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "B00[pp, ma^2,mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "00"], TraditionalForm]]],
 "-function, i.e., the coefficient function of the metric tensor. All \
arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B00", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[82]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B00",
 CellLabel->"Out[82]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B00"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e., ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[83]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[83]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[84]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}], " ", 
    SuperscriptBox["m", "2"]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"]}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["M", "2"],
        TraditionalForm], ")"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["M", "2"], "-", 
             SuperscriptBox["m", "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["B", "0"], "(", 
              FormBox[
               SuperscriptBox[
                FormBox["p",
                 TraditionalForm], "2"],
               TraditionalForm], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}], "-", 
             RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
          RowBox[{"2", " ", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm]}]], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
           SubscriptBox["B", "0"], "(", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm], ",", 
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox["M", "2"], "+", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[84]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[85]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}], " ", 
    SuperscriptBox["m", "2"]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"6", " ", 
       SuperscriptBox["m", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["m", "2"],
        TraditionalForm], ")"}], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[85]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[86]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"00\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[86]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[87]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}], " ", 
    SuperscriptBox["m", "2"]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[87]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[88]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     SuperscriptBox["M", "2"], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}], " ", 
    SuperscriptBox["m", "2"]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[88]="]
}, Open  ]]
}, Open  ]],

Cell["B1", "ObjectName",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B1"],

Cell[TextData[{
 "B1[pp, ma^2, mb^2] the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "1"], TraditionalForm]]],
 "-function. All arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B1", "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[89]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B1",
 CellLabel->"Out[89]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"B1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[90]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["M", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
    RowBox[{"2", " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[90]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[91]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "1"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[91]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B1", ",", 
    RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[92]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[93]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[93]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[94]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[94]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[95]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "1"}], ")"}]}], "+", 
   FractionBox["1", "4"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[95]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"pp", ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   SubsuperscriptBox["m", "2", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[96]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", "pp", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}], " ", 
     SubsuperscriptBox["m", "2", "2"]}], 
    RowBox[{"2", " ", "pp"}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "pp", ",", 
     SubsuperscriptBox["m", "e", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ")"}]}]}], TraditionalForm]], "Output",\

 CellTags->"B1",
 CellLabel->"Out[96]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[97]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "e", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[97]="]
}, Open  ]]
}, Open  ]],

Cell["B11", "ObjectName",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "B11[pp, ma^2, mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "11"], TraditionalForm]]],
 "-function, i.e., the coefficient function of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    SuperscriptBox["p", "\[Nu]"]}], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Null]", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B11", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[98]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[98]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B11"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "2"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[99]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[99]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[100]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["B", "0"], "(", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], ",", 
       SuperscriptBox["m", "2"], ",", 
       SuperscriptBox["M", "2"], ")"}]}], " ", 
     SuperscriptBox["m", "2"]}], "+", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["M", "2"],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox["M", "2"]}], "+", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm]}], ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["M", "2"], "-", 
           SuperscriptBox["m", "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], "2"],
             TraditionalForm], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}], "-", 
           RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
        RowBox[{"2", " ", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm]}]], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         SuperscriptBox["m", "2"], ",", 
         SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", 
       SuperscriptBox["M", "2"], "+", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm]}], ")"}]}]}], 
   RowBox[{"3", " ", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[101]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["m", "2"]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]}], 
   RowBox[{"3", " ", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"11\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[104]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[105]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[106]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    SuperscriptBox["M", "2"], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[106]="]
}, Open  ]]
}, Open  ]],

Cell["CA", "ObjectName",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CA"],

Cell[TextData[{
 "CA is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CA = ",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"CA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell[BoxData["CA"], "Input",
 CellTags->"CA",
 CellLabel->"In[107]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "A"], TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CA", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CA",
 CellLabel->"In[108]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CA",
 CellLabel->"In[109]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[109]="]
}, Open  ]]
}, Open  ]],

Cell["Calc", "ObjectName",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Calc"],

Cell["\<\
Calc[exp] performs several basic simplifications. Calc[exp] is the same as \
DotSimplify[DiracSimplify[Contract[DiracSimplify[Explicit[ \
SUNSimplify[Trick[exp], Explicit\[Rule]False] ]]]]].\
\>", "Text",
 CellTags->"Calc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"Calc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Calc"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 " in 4 dimensions and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[110]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[110]="]
}, Open  ]],

Cell[TextData[{
 "This simplifies ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "abc"], " ", 
    RowBox[{
     SubscriptBox["f", "abe"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "e"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[111]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[111]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[112]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[113]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[113]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "-", "q"}], ",", "3"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[114]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{
       RowBox[{"-", "p"}], "-", "q"}],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[114]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"%", " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "li1"}], "]"}]}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"q", ",", "li2"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "p"}], "-", "q"}], ",", "li3"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[115]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[115]="]
}, Open  ]]
}, Open  ]],

Cell["CalcColorFactor", "ObjectName",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalcColorFactor"],

Cell["\<\
CalcColorFactor[expr] calculates the color factor of expr.  CalcColorFactor \
is useful for application on FeynArts produced amplitudes. CalcColorFactor is \
just a macro function for CalcColorFactor[x_] := 
If[FreeQ2[FeynCalcInternal[x], SUNIndex],  x, 
                    SUNSimplify[SUNSimplify[  (If[ !FreeQ[#1, DiracGamma], \
DiracTrick[#1], #1] & )[  SUNSimplify[x]], Explicit -> False], Explicit -> \
True]].\
\>", "Text",
 CellTags->"CalcColorFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[116]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[116]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<q2se2\>\"", "]"}]], "Input",
 CellLabel->"In[117]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{"2", " ", 
       SubscriptBox["T", "f"], " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[CapitalPi]", "q"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], ".", 
        SubsuperscriptBox["Q", 
         SubscriptBox["c", "9"], 
         SubscriptBox["\[Lambda]", "9"]], ".", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "q"], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], ".", 
        SubsuperscriptBox["Q", 
         SubscriptBox["c", "3"], 
         SubscriptBox["\[Lambda]", "3"]]}]}],
      TraditionalForm], ")"}]}], " ", 
   RowBox[{
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "11"], 
     SubscriptBox["\[Lambda]", "11"]], ".", 
    RowBox[{
     SubscriptBox["\[CapitalPi]", "q"], "(", 
     FormBox[
      RowBox[{"p", "-", "q2"}],
      TraditionalForm], ")"}], ".", 
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "1"], 
     SubscriptBox["\[Lambda]", "1"]]}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "1"], 
      SubscriptBox["c", "3"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], "(", "q2", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "9"], 
      SubscriptBox["c", "11"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]], "(", "q2", ")"}]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[117]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "@", 
  RowBox[{"Explicit", "@", "%"}]}]], "Input",
 CellLabel->"In[118]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "4"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{"-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "9"],
           TraditionalForm],
          TraditionalForm]], ".", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q1", "-", "q2"}],
           TraditionalForm], ")"}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "3"],
           TraditionalForm],
          TraditionalForm]]}]}],
      TraditionalForm], ")"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "11"],
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", "(", 
       FormBox[
        RowBox[{"p", "-", "q2"}],
        TraditionalForm], ")"}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "1"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["1", 
       SuperscriptBox["q2", "2"]], ")"}], "2"], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[118]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["CalculateCounterTerm ***unfinished***", "ObjectName",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell["CalculateCounterTerm[exp, k] calculates the residue of exp.", "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CalculateCounterTerm"}]], "Input",
 CellTags->"CalculateCounterTerm",
 CellLabel->"In[119]:="],

Cell[BoxData[
 StyleBox["\<\"CalculateCounterTerm[exp, k] calculates the residue of \
exp.\"\>", "MSG"]], "Print", "PrintUsage",
 CellTags->{"CalculateCounterTerm", "Info3387711865-3148101"}]
}, Open  ]]
}, Open  ]],

Cell["CancelQP", "ObjectName",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CancelQP"],

Cell[TextData[{
 "CancelQP is an option for OneLoop. If set to True, cancelation of all ",
 StyleBox["q",
  FontSlant->"Italic"],
 ".",
 StyleBox["p",
  FontSlant->"Italic"],
 "'s and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["q", "2"], TraditionalForm]]],
 " is performed."
}], "Text",
 CellTags->"CancelQP"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{
      RowBox[{"p", "+", 
       RowBox[{"2", "q2"}]}], ",", 
      RowBox[{"p", "+", "q2"}]}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "M"}], "}"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[120]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "q2"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{"p", "+", 
      RowBox[{"2", " ", "q2"}]}],
     TraditionalForm], ")"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox["q2", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[120]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[121]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"3", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"7", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"5", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[121]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[122]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox["m", "4"]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox["m", "4"]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"17", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"17", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     RowBox[{
      FractionBox["17", "6"], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "0", ",", 
       SuperscriptBox["m", "2"], ",", 
       SuperscriptBox["M", "2"], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"17", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"13", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"19", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"19", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"8", " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{"{", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           SuperscriptBox["m", "2"]}], "}"}]}], ")"}]}], 
      SuperscriptBox["M", "2"]], "+", 
     FractionBox[
      RowBox[{"8", " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{"{", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"]}], "}"}]}], ")"}]}], 
      SuperscriptBox["M", "2"]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox["4", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[122]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[123]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[123]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[124]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], "-", 
           RowBox[{"2", " ", 
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q1",
              TraditionalForm]}]}], "+", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}],
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox["m", "2"],
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ")"}], " ", 
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q1",
          TraditionalForm]}], " ", 
        SuperscriptBox["m", "2"]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}]}]]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["q1",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ",", "0", ",", "0", ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox[
         SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[124]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[125]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"PaVe", "(", 
        RowBox[{"2", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"], ",", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox["p",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q1",
               TraditionalForm]}]}], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}],
         TraditionalForm], "2"]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"1", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"], ",", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox["p",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q1",
               TraditionalForm]}]}], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"2", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"], ",", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox["p",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q1",
               TraditionalForm]}]}], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox[
         SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "-", 
         RowBox[{"2", " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q1",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[125]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[126]:="]
}, Open  ]],

Cell["Cases2", "ObjectName",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Cases2"],

Cell["\<\
Cases2[expr, f] returns a list of all objects in expr with head f. \
Cases2[expr,f] is equivalent to Cases2[{expr},f[___],Infinity]//Union. \
Cases2[expr, f, g, ...] or Cases2[expr, {f,g, ...}] is equivalent to \
Cases[{expr},f[___] | g[___] ...] .\
\>", "Text",
 CellTags->"Cases2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "a", "]"}], "+", 
    SuperscriptBox[
     RowBox[{"f", "[", "b", "]"}], "2"], "+", 
    RowBox[{"f", "[", 
     RowBox[{"c", ",", "d"}], "]"}]}], ",", "f"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[127]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "(", "a", ")"}], ",", 
    RowBox[{"f", "(", "b", ")"}], ",", 
    RowBox[{"f", "(", 
     RowBox[{"c", ",", "d"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[127]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "y", "]"}]}], ",", "Sin", ",", "g"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[128]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "y", ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "x", "]"}], "+", 
    RowBox[{"g", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"Sin", ",", "g"}], "}"}]}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[129]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ",", 
    RowBox[{"g", "(", 
     RowBox[{"a", ",", "b", ",", "c"}], ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
     RowBox[{"DiracSlash", "[", "q", "]"}]}], "+", 
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "p"}], "]"}]}], ",", "Dot"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[130]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[130]="]
}, Open  ]]
}, Open  ]],

Cell["CF", "ObjectName",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CF"],

Cell[TextData[{
 "CF is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CF = (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "2"], TraditionalForm]]],
 "-1)/(2 ",
 StyleBox["N",
  FontSlant->"Italic"],
 "))."
}], "Text",
 CellTags->"CF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell[BoxData["CF"], "Input",
 CellTags->"CF",
 CellLabel->"In[131]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[131]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CF", ",", " ", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[132]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[132]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[133]:="],

Cell["(-1 + SUNN^2)/(2*SUNN)", "Output",
 CellLabel->"Out[133]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CF",
 CellLabel->"In[134]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[134]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNN", "^", "2"}], "-", "1"}], ",", " ", 
   RowBox[{"SUNNToCACF", " ", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[135]="]
}, Open  ]]
}, Open  ]],

Cell["ChangeDimension", "ObjectName",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
ChangeDimension[exp, dim] changes all LorentzIndex and Momenta in exp to \
dimension dim (and also Levi-Civita-tensors, Dirac slashes and Dirac \
matrices).\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
Remember that LorentzIndex[mu, 4] is simplified to LorentzIndex[mu] and \
Momentum[p, 4] to Momentum[p]. Thus the fullowing objects are defined in four \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChangeDimension", "[", 
  RowBox[{"%", ",", " ", "D"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[137]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[138]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "}"}]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[138]//StandardForm="]
}, Open  ]],

Cell["\<\
This changes all non-4-dimensional objects to 4-dimensional ones.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"%%", ",", " ", "4"}], "]"}], " ", "//", " ", 
  "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[139]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[139]//StandardForm="]
}, Open  ]],

Cell["Consider the following list of 4- and D-dimensional object.", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MT", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
   RowBox[{
    RowBox[{"GAD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"f", "[", "D", "]"}]}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[140]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     RowBox[{"f", "(", "D", ")"}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[141]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%%", ",", "n"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[142]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[142]="]
}, Open  ]],

Cell["\<\
Any explicit occurence of D (like in f(D)) is not replaced by \
ChangeDimension.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[143]:="],

Cell["\<\
The option Dimension of Eps must be changed too, since with the default \
setting Dimension\[Rule]4 the arguments of Eps are automatically changed to 4 \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[144]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[145]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[145]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"a1", ",", "4"}], "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[146]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[146]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   SuperscriptBox["a1", "2"], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[147]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[147]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  SuperscriptBox["a2", "2"], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[148]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[148]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[149]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a1", ",", "a2"}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[150]:="]
}, Open  ]],

Cell["ChargeConjugationMatrix", "ObjectName",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "ChargeConjugationMatrix denotes the charge conjugation matrix ",
 StyleBox["C",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrixInv",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChargeConjugationMatrixInv",
  ButtonNote->"ChargeConjugationMatrixInv"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[151]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[152]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[153]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[153]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[154]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[154]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[155]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[155]="]
}, Open  ]]
}, Open  ]],

Cell["ChargeConjugationMatrixInv", "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell["\<\
ChargeConjugationMatrixInv is the inverse of ChargeConjugationMatrix. It is \
substituted immediately by -ChargeConjugationMatrix.\
\>", "Text",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChargeConjugationMatrix",
  ButtonNote->"ChargeConjugationMatrix"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[156]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[156]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[157]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[157]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[158]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[158]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[159]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[159]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Calc"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[161]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"C", ".", "C"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[161]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrixInv", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[162]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[162]="]
}, Open  ]]
}, Open  ]],

Cell["CheckContext", "ObjectName",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
CheckContext[string] yields True if the packaged associated with string is \
already loaded, and False otherwise.\
\>", "Text",
 CellTags->"CheckContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MakeContext",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MakeContext",
  ButtonNote->"MakeContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 "."
}], "Text",
 CellTags->"CheckContext"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
If a FeynCalc symbol has not been used, the corresponding subpackage has not \
been loaded:\
\>", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[163]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[163]="]
}, Open  ]],

Cell["Using it makes FeynCalc load the subpackage:", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData["CompleteSquare"], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[164]:="],

Cell[BoxData[
 FormBox["CompleteSquare", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[164]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[165]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[165]="]
}, Open  ]]
}, Open  ]],

Cell["CheckDB", "ObjectName",
 CellTags->"CheckDB"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellTags->"CheckDB"],

Cell["\<\
CheckDB[exp, fil] saves [with Put] or retrieves [with Get] exp from a file \
fil. It checks if the setting of the option Directory is a valid directory \
name and if fil is a valid file name and does exist. If it does, Get[fil] is \
executed. If fil does not exist, exp gets evaluated and saved to  fil. Saving \
and evaluating can be further controlled with the options ForceSave and \
NoSave. If the option Check is set to False  the return value is what is \
evaluated [see above]. If Check is set to True the return value is True or \
False depending on whether the evaluation of exp agrees with what is loaded \
from fil or fil does not exist.  Default value of Check : False. NOTICE : If \
fil ends with \".Gen\" or \".Mod\", the   setting of Directory is ignored and \
fil is  saved in the \"CouplingVectors\"  subdirectory of \"Phi\". If fil \
ends with  \".Fac\", the setting of Directory is  ignored and fil is saved in \
the \"Factors\" subdirectory of \"Phi\". If fil is a file   name with full \
path, the setting of  Directory is also ignored.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[166]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Directory", "\[RuleDelayed]", 
     RowBox[{"ToFileName", "[", 
      RowBox[{
       RowBox[{"{", "$FeynCalcDirectory", "}"}], ",", "\<\"fcdb\"\>"}], 
      "]"}]}], ",", 
    RowBox[{"ForceSave", "\[Rule]", "False"}], ",", 
    RowBox[{"NoSave", "\[Rule]", "False"}], ",", 
    RowBox[{"Check", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[166]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[167]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldFirst", "}"}], TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[167]="]
}, Open  ]],

Cell["\<\
The first time the Table function is evaluated and the result saved into the \
test.s file.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"\"\<test \>\"", ",", "i"}], "]"}], ";", "i"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[168]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "1"}],
   SequenceForm["test ", 1],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "2"}],
   SequenceForm["test ", 2],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellLabel->"Out[168]="]
}, Open  ]],

Cell["\<\
Executing the same a second time will just load the result from test.s and \
not evaluate the Table function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"WriteString", "[", 
       RowBox[{"\"\<stdout\>\"", ",", "\"\<test\>\""}], "]"}], ";", "i"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[169]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[169]="]
}, Open  ]],

Cell["This shows the actual saved value of test.s.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Import", "[", 
  RowBox[{
   RowBox[{"ToFileName", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
     "\"\<test.s\>\""}], "]"}], ",", "\"\<Text\>\""}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[170]:="],

Cell[BoxData[
 FormBox["\<\"{1, 2}\\n\"\>", TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[170]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{"ToFileName", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
    "\"\<test.s\>\""}], "]"}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[171]:="]
}, Open  ]],

Cell["ChiralityProjector", "ObjectName",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "ChiralityProjector[+1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 ". ChiralityProjector[-1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 "."
}], "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"+", "1"}], "]"}], ",", 
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[172]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", "%", "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[173]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[173]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"DiracSubstitute67", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[174]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"], "+", 
     FractionBox["1", "2"]}], ",", 
    RowBox[{
     FractionBox["1", "2"], "-", 
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[174]="]
}, Open  ]]
}, Open  ]],

Cell["Chisholm", "ObjectName",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Chisholm"],

Cell["\<\
Chisholm[x] substitutes products of three Dirac matrices or slashes by the \
Chisholm identity.\
\>", "Text",
 CellTags->"Chisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["ChisholmSpinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChisholmSpinor",
  ButtonNote->"ChisholmSpinor"],
 "."
}], "Text",
 CellTags->"Chisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[175]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$305",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$305",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Mu]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Mu]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[176]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[177]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$327",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$327",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[177]="]
}, Open  ]],

Cell["The $MU$ variables are unique indices.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t1", ".", "t1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[178]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[178]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[179]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$346",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$346",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[179]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t2", ".", "t3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[180]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[180]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracSlash", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[181]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", "t4", "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[182]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$373",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["$MU$373",
        TraditionalForm],
       TraditionalForm], 
      FormBox["a",
       TraditionalForm], 
      FormBox["b",
       TraditionalForm], 
      FormBox["c",
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}]}], "-", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["b",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[182]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Kappa]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[183]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Tau]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Kappa]"]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[183]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[184]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$404",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$404",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$415",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$415",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$426",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$426",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$382",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$382",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[184]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[185]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$487",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$487",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$498",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$498",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$509",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$509",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$465",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$465",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[185]="]
}, Open  ]],

Cell["Check that both a1.a1 and a2.a3 give the same.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a1", ".", "a1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[186]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a2", ".", "a3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[187]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[187]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "a1", ",", "a2", ",", "a3"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[188]:="]
}, Open  ]],

Cell["ChisholmSpinor", "ObjectName",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell["\<\
ChisholmSpinor[x] uses the Chisholm identity on a DiraGamma between spinors. \
As an optional second argument 1 or 2 may be given, indicating that \
ChisholmSpinor should only act on the first resp. second part of a product of \
spinor chains.\
\>", "Text",
 CellTags->"ChisholmSpinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p1", ",", "m1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ".", 
  RowBox[{"Spinor", "[", 
   RowBox[{"p2", ",", "m2"}], "]"}]}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m1", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p2",
     TraditionalForm], ",", "m2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ChisholmSpinor"}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"m1", " ", "m2", " ", 
      RowBox[{
       RowBox[{"\[CurlyPhi]", "(", 
        FormBox["p1",
         TraditionalForm], ",", "m1", ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], ".", 
       RowBox[{"\[CurlyPhi]", "(", 
        FormBox["p2",
         TraditionalForm], ",", "m2", ")"}]}]}], 
     RowBox[{
      FormBox["p1",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p2",
       TraditionalForm]}]]}], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["alpha1",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["alpha1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox["p1",
        TraditionalForm], 
       FormBox["p2",
        TraditionalForm]}]]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m2", " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}], " ", 
     SuperscriptBox[
      FormBox["p1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m1", " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}], " ", 
     SuperscriptBox[
      FormBox["p2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[190]="]
}, Open  ]]
}, Open  ]],

Cell["ClearScalarProducts", "ObjectName",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell["\<\
ClearScalarProducts removes all user-performed specific settings for \
ScalarProduct's.\
\>", "Text",
 CellTags->"ClearScalarProducts"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "p"}], "]"}], "=", 
  RowBox[{"m", "^", "2"}]}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[191]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[191]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[192]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[192]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[193]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[194]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[195]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[195]="]
}, Open  ]]
}, Open  ]],

Cell["Collecting", "ObjectName",
 CellTags->"Collecting"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collecting"],

Cell["\<\
Collecting is an option of ScalarProductCancel, Series2, TID and related \
functions. Setting it to True will trigger some kind of collecting of the \
result.\
\>", "Text",
 CellTags->"Collecting"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["Collect2", "ObjectName",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect2"],

Cell["\<\
Collect2[expr, x] collects together terms which are not free of any \
occurrence of x. Collect2[expr, {x1, x2, ...}] (or also Collect2[expr, x1, \
x2, ...]) collects together terms which are not free of any occurrence of x1, \
x2, .... The coefficients are put over a common denominator. If expr is \
expanded before collecting depends on the option Factoring, which may be set \
to Factor, Factor2, or any other function, which is applied to the \
coefficients. If expr is already expanded with respect to x (x1,x2, ...), the \
option Expanding can be set to False.\
\>", "Text",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect2", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[196]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Denominator", "\[Rule]", "False"}], ",", 
    RowBox[{"Dot", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[196]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"t1", "=", 
    RowBox[{"a", "+", 
     RowBox[{"r", " ", "a"}], "+", 
     RowBox[{
      SuperscriptBox["k", "2"], " ", 
      RowBox[{"f", "[", "a", "]"}]}], "-", 
     RowBox[{"k", " ", 
      RowBox[{"f", "[", "a", "]"}]}], "+", 
     FractionBox["x", "2"], "-", 
     FractionBox["y", "w"]}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[197]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "k"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[198]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox["x", "2"], "-", 
   FractionBox["y", "w"], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", "k"}], ")"}], " ", 
    RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[198]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "Factor"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[199]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[199]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "h"}], ")"}], " ", 
    SuperscriptBox["a", "2"]}], "-", 
   RowBox[{
    SuperscriptBox["b", "2"], " ", "e", " ", "a"}], "-", 
   RowBox[{"2", " ", "b", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "h"}], ")"}], " ", "a"}], "+", 
   RowBox[{
    SuperscriptBox["b", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[200]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   "a", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    TagBox[
     RowBox[{"L", "(", "1", ")"}],
     HoldForm], " ", 
    SuperscriptBox["a", "4"]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "1", ")"}],
      HoldForm], "2"], " ", 
    SuperscriptBox["a", "3"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "1", ")"}],
      HoldForm], "3"], " ", 
    SuperscriptBox["a", "2"]}], "+", 
   RowBox[{"5", " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "1", ")"}],
      HoldForm], "4"], " ", "a"}], "-", 
   SuperscriptBox[
    TagBox[
     RowBox[{"L", "(", "1", ")"}],
     HoldForm], "5"]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[201]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[202]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}], " ", 
    SuperscriptBox["a", "4"]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"], " ", 
    SuperscriptBox["a", "3"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"], " ", 
    SuperscriptBox["a", "2"]}], "+", 
   RowBox[{"5", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"], " ", "a"}], "-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
  TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[202]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "L"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[203]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c"}], ")"}], "3"], "]"}], ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "fun"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[204]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"fun", "(", "1", ")"}], " ", 
    SuperscriptBox["a", "3"]}], "+", 
   RowBox[{
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "3"}], " ", "b"}], "-", 
      RowBox[{"3", " ", "c"}]}], ")"}], " ", 
    SuperscriptBox["a", "2"]}], "+", 
   RowBox[{
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["b", "2"]}], "+", 
      RowBox[{"6", " ", "c", " ", "b"}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["c", "2"]}]}], ")"}], " ", "a"}], "+", 
   RowBox[{"fun", "(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["b", "3"]}], "-", 
     RowBox[{"3", " ", "c", " ", 
      SuperscriptBox["b", "2"]}], "-", 
     RowBox[{"3", " ", 
      SuperscriptBox["c", "2"], " ", "b"}], "-", 
     SuperscriptBox["c", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[204]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"fun", "\[Rule]", "FactorTerms"}]}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[205]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "3"], "-", 
   RowBox[{"3", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}], " ", 
    SuperscriptBox["a", "2"]}], "+", 
   RowBox[{"3", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"2", " ", "c", " ", "b"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "-", 
   SuperscriptBox["b", "3"], "-", 
   SuperscriptBox["c", "3"], "-", 
   RowBox[{"3", " ", "b", " ", 
    SuperscriptBox["c", "2"]}], "-", 
   RowBox[{"3", " ", 
    SuperscriptBox["b", "2"], " ", "c"}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[205]="]
}, Open  ]]
}, Open  ]],

Cell["Collect3", "ObjectName",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect3"],

Cell[TextData[{
 "Collect3[expr, {x, y, ...}] collects terms involving the same powers of \
monomials ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["x",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "1"]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["y",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "2"]], TraditionalForm]]],
 " ... An option Factor \[Rule] True/False can be  given, which factors the \
coefficients. The option Head (default Plus) determines the applied function \
to the list of monomials  mulitplied by their coefficients."
}], "Text",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect3", "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[206]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"Head", "\[Rule]", "Plus"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Collect3",
 CellLabel->"Out[206]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[207]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"List", "+", 
   RowBox[{"Internal`DistributedTermsList", "(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", " ", "e"}], "-", "c"}], ")"}]}], " ", 
       SuperscriptBox["b", "2"]}], "+", 
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"2", " ", "a", " ", 
       RowBox[{"(", 
        RowBox[{"b", "-", "a"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"h", "-", "1"}], ")"}]}]}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"CoefficientDomain", "\[Rule]", "RationalFunctions"}]}], ")"}]}],
   TraditionalForm]], "Output",
 CellTags->"Collect3",
 CellLabel->"Out[207]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[208]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"List", "+", 
   RowBox[{"Internal`DistributedTermsList", "(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["a", "5"], "-", 
      RowBox[{"5", " ", "b", " ", 
       SuperscriptBox["a", "4"]}], "-", 
      RowBox[{"5", " ", "c", " ", 
       SuperscriptBox["a", "4"]}], "-", 
      RowBox[{"5", " ", "d", " ", 
       SuperscriptBox["a", "4"]}], "+", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["a", "3"]}], "+", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["a", "3"]}], "+", 
      RowBox[{"10", " ", 
       SuperscriptBox["d", "2"], " ", 
       SuperscriptBox["a", "3"]}], "+", 
      RowBox[{"20", " ", "b", " ", "c", " ", 
       SuperscriptBox["a", "3"]}], "+", 
      RowBox[{"20", " ", "b", " ", "d", " ", 
       SuperscriptBox["a", "3"]}], "+", 
      RowBox[{"20", " ", "c", " ", "d", " ", 
       SuperscriptBox["a", "3"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "3"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "3"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["d", "3"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", "b", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", "b", " ", 
       SuperscriptBox["d", "2"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", "c", " ", 
       SuperscriptBox["d", "2"], " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", "c", " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", "d", " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["c", "2"], " ", "d", " ", 
       SuperscriptBox["a", "2"]}], "-", 
      RowBox[{"60", " ", "b", " ", "c", " ", "d", " ", 
       SuperscriptBox["a", "2"]}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox["b", "4"], " ", "a"}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox["c", "4"], " ", "a"}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox["d", "4"], " ", "a"}], "+", 
      RowBox[{"20", " ", "b", " ", 
       SuperscriptBox["c", "3"], " ", "a"}], "+", 
      RowBox[{"20", " ", "b", " ", 
       SuperscriptBox["d", "3"], " ", "a"}], "+", 
      RowBox[{"20", " ", "c", " ", 
       SuperscriptBox["d", "3"], " ", "a"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["c", "2"], " ", "a"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["d", "2"], " ", "a"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["d", "2"], " ", "a"}], "+", 
      RowBox[{"60", " ", "b", " ", "c", " ", 
       SuperscriptBox["d", "2"], " ", "a"}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["b", "3"], " ", "c", " ", "a"}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["b", "3"], " ", "d", " ", "a"}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["c", "3"], " ", "d", " ", "a"}], "+", 
      RowBox[{"60", " ", "b", " ", 
       SuperscriptBox["c", "2"], " ", "d", " ", "a"}], "+", 
      RowBox[{"60", " ", 
       SuperscriptBox["b", "2"], " ", "c", " ", "d", " ", "a"}], "-", 
      SuperscriptBox["b", "5"], "-", 
      SuperscriptBox["c", "5"], "-", 
      SuperscriptBox["d", "5"], "-", 
      RowBox[{"5", " ", "b", " ", 
       SuperscriptBox["c", "4"]}], "-", 
      RowBox[{"5", " ", "b", " ", 
       SuperscriptBox["d", "4"]}], "-", 
      RowBox[{"5", " ", "c", " ", 
       SuperscriptBox["d", "4"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["c", "3"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["d", "3"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["d", "3"]}], "-", 
      RowBox[{"20", " ", "b", " ", "c", " ", 
       SuperscriptBox["d", "3"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "3"], " ", 
       SuperscriptBox["c", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["b", "3"], " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "3"], " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"30", " ", "b", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", "c", " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"5", " ", 
       SuperscriptBox["b", "4"], " ", "c"}], "-", 
      RowBox[{"5", " ", 
       SuperscriptBox["b", "4"], " ", "d"}], "-", 
      RowBox[{"5", " ", 
       SuperscriptBox["c", "4"], " ", "d"}], "-", 
      RowBox[{"20", " ", "b", " ", 
       SuperscriptBox["c", "3"], " ", "d"}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["c", "2"], " ", "d"}], "-", 
      RowBox[{"20", " ", 
       SuperscriptBox["b", "3"], " ", "c", " ", "d"}]}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"CoefficientDomain", "\[Rule]", "RationalFunctions"}]}], ")"}]}],
   TraditionalForm]], "Output",
 CellTags->"Collect3",
 CellLabel->"Out[208]="]
}, Open  ]]
}, Open  ]],

Cell["Combine", "ObjectName",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Combine"],

Cell["\<\
Combine[expr]puts terms in a sum over a common denominator,and cancels \
factors in the result. Combine is similar to Together, but accepts the option \
Expanding and works usually better than Together for polynomials involving \
rationals with sums in the denominator.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Combine", "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[209]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[209]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Combine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[210]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[210]="]
}, Open  ]],

Cell["\<\
Here the result from Together where the numerator is automatically expanded.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Together", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"b", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[211]="]
}, Open  ]],

Cell["\<\
If the option Expanding is set to True, the result of Combine is the same as \
Together, but uses a slightly different algorithm.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], ",", 
   RowBox[{"Expanding", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[212]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"b", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[212]="]
}, Open  ]]
}, Open  ]],

Cell["CombineGraphs", "ObjectName",
 CellTags->"CombineGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CombineGraphs"],

Cell["CombineGraphs is an option for OneLoopSum.", "Text",
 CellTags->"CombineGraphs"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],

Cell["Commutator", "ObjectName",
 CellTags->"Commutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Commutator"],

Cell["\<\
Commutator[x, y] = c defines the commutator between the non-commuting objects \
x and y. \
\>", "Text",
 CellTags->"Commutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"Commutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[213]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[214]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DotSimplify", "[", 
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{"c", "+", "d"}]}], "]"}], "]"}], " "}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "c"}], "+", 
   RowBox[{"a", ".", "d"}], "+", 
   RowBox[{"b", ".", "c"}], "+", 
   RowBox[{"b", ".", "d"}], "-", 
   RowBox[{"c", ".", "a"}], "-", 
   RowBox[{"c", ".", "b"}], "-", 
   RowBox[{"d", ".", "a"}], "-", 
   RowBox[{"d", ".", "b"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[216]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[217]:="],

Cell["Verify the Jacobi identity.", "Text",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "=", "Commutator"}], ";", " ", 
  RowBox[{"DeclareNonCommutative", "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], ";"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[218]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "[", 
   RowBox[{"x", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"y", ",", "z"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"y", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"z", ",", "x"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"z", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[219]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"[", "\[NoBreak]", "x", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "y", "\[NoBreak]", ",", "z", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "y", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "z", "\[NoBreak]", ",", "x", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "z", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "x", "\[NoBreak]", ",", "y", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[219]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[220]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[220]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "\[Chi]", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[221]:="],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[222]:="]
}, Open  ]],

Cell["CommutatorExplicit", "ObjectName",
 CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell["\<\
CommutatorExplicit[exp] substitutes any Commutator and AntiCommutator in exp \
by their definitions.\
\>", "Text",
 CellTags->"CommutatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[223]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[224]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[224]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{
   RowBox[{"a", "-", "c"}], ",", 
   RowBox[{"b", "-", "d"}]}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   FormBox[
    RowBox[{"a", "-", "c"}],
    TraditionalForm], ",", "\[MediumSpace]", 
   FormBox[
    RowBox[{"b", "-", "d"}],
    TraditionalForm], "}"}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[226]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[227]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[227]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CommutatorExplicit", "[", "%%", "]"}], "//", 
  "DotSimplify"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[228]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"a", ".", "d"}], "+", 
   RowBox[{"b", ".", "a"}], "-", 
   RowBox[{"b", ".", "c"}], "-", 
   RowBox[{"c", ".", "b"}], "+", 
   RowBox[{"c", ".", "d"}], "-", 
   RowBox[{"d", ".", "a"}], "+", 
   RowBox[{"d", ".", "c"}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[228]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[229]:="]
}, Open  ]],

Cell["CompleteSquare", "ObjectName",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[TextData[{
 "Completes the square of a second order polynomial in the momentum x. \
CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 "] \[Rule] -",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " (",
 "b",
 "/(2 ",
 "a",
 ")+",
 "x",
 ")^2. CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 ", ",
 "q",
 "] \[Rule] {-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["q",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 "q",
 "\[Rule]",
 "b",
 "/(2 ",
 "a",
 ")+",
 "p",
 "}."
}], "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"5", 
    RowBox[{"SP", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"2", "p"}], "+", 
       RowBox[{"3", "r"}]}], ",", 
      RowBox[{"p", "+", "r"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellLabel->"In[230]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"5", " ", 
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "r"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{
      RowBox[{"2", " ", "p"}], "+", 
      RowBox[{"3", " ", "r"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[230]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"CompleteSquare", "[", 
   RowBox[{"t1", ",", "p"}], "]"}]}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[231]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"10", " ", 
    RowBox[{"(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")", ".", "(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")"}]}], "-", 
   FractionBox[
    RowBox[{"5", " ", 
     SuperscriptBox[
      FormBox["r",
       TraditionalForm], "2"]}], "8"]}], TraditionalForm]], "Output",
 CellTags->"CompleteSquare",
 CellLabel->"Out[231]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t1", "-", "t2"}], "//", "ScalarProductExpand"}], "//", 
  "Expand"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[232]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"CompleteSquare",
 CellLabel->"Out[232]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteSquare", "[", 
  RowBox[{"t1", ",", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[233]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"10", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], "2"]}], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], "2"]}], "8"]}], ",", 
    RowBox[{"q", "\[Rule]", 
     RowBox[{"p", "+", 
      FractionBox[
       RowBox[{"5", " ", "r"}], "4"]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"CompleteSquare",
 CellLabel->"Out[233]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[234]:="]
}, Open  ]],

Cell["ComplexConjugate", "ObjectName",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell["\<\
ComplexConjugate[expr] complex conjugates expr. It operates on fermion lines, \
i.e., products of Spinor[..] .DiracMatrix[..] . Spinor[..], and changes all \
occuring LorentzIndex[mu] into LorentzIndex[ComplexIndex[mu]]. For taking the \
spin sum (i.e. constructing the traces) use FermionSpinSum. WARNING: In expr \
should be NO explicit I's in denominators!\
\>", "Text",
 CellTags->"ComplexConjugate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ComplexIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ComplexIndex",
  ButtonNote->"ComplexIndex"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 "."
}], "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[235]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Nu]"}]], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[235]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[236]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[236]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "5"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[237]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[237]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[238]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[240]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[241]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[241]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"ComplexConjugate", "[", 
    RowBox[{"MetricTensor", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[242]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[242]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"k1", ",", "m"}], "]"}], ".", 
  RowBox[{"GA", "[", "\[Lambda]", "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{"p1", ",", "m"}], "]"}]}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[243]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "k1", ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Lambda]"], ".", 
   RowBox[{"u", "(", "p1", ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[244]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Lambda]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[244]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell["Contract", "ObjectName",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract"],

Cell["\<\
Contract[expr] contracts pairs of Lorentz indices of metric tensors, \
four-vectors and (depending on the optionEpsContract) of Levi-Civita tensors \
in expr. For the contraction of Dirac matrices with each other use \
DiracSimplify. Contract[exp1, exp2] contracts (exp1*exp2), where exp1 and \
exp2 may be larger products of sums of metric tensors and 4-vectors. \
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Contract", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[245]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract3", "\[Rule]", "True"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "True"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], ",", 
    RowBox[{"MomentumCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"Rename", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Contract",
 CellLabel->"Out[245]="]
}, Open  ]],

Cell["\<\
The option setting Contract3 can be set to True resulting in a faster \
algorithm to contract products of tensors.\
\>", "Text"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"Contract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[246]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[246]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[247]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Nu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[247]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[248]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[248]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[249]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[249]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[250]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Mu]"}]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[250]="]
}, Open  ]],

Cell["The default dimension for MetricTensor is 4.", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[251]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[251]="]
}, Open  ]],

Cell["\<\
A short way to enter D-dimensional metric tensors is given by MTD. The \".\" \
as multiplication operator is not necessary but just convenient for \
typesetting.\
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[252]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[252]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[253]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[254]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"q", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[255]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", " ", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[256]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"a", "-", "b"}], ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[257]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"a", "-", "b"}],
      TraditionalForm], ")"}], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[258]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Sigma]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[259]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[260]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[260]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
  " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " "}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[261]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "6"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
   " ", 
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
    "]"}]}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[263]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Beta]"}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Sigma]"}]]}], 
  TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[263]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "%", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[264]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[264]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[265]:="]
}, Open  ]],

Cell["Contract1", "ObjectName",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract1"],

Cell["\<\
For covariant calculations it is not necessary to distinguish between co- and \
contravariant indices (except for the Levi-Civita tensor).
Therefore normally no distinction is made between upper and lower indices.
Howere, if needed, one can use Upper and Lower and then a distuction is made:
Contract1[exp] contracts Upper and Lower indices. Neither Upper and Upper nor \
Lower and Lower indices are contracted.\
\>", "Text",
 CellTags->"Contract1"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Upper",
  ButtonNote->"Upper"],
 "."
}], "Text",
 CellTags->"Contract1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Lower", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[266]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[266]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[267]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Lower", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[268]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[268]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Upper", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[269]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm], 
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[269]="]
}, Open  ]]
}, Open  ]],

Cell["Convolute", "ObjectName",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Convolute"],

Cell[TextData[{
 "Convolute[f, g, x] convolutes ",
 StyleBox["f",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 ") and ",
 StyleBox["g",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 "), i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubscriptBox[
      StyleBox[
       RowBox[{
        StyleBox["d",
         FontSlant->"Plain"], 
        StyleBox["x",
         FontSlant->"Italic"]}]], "1"], " ", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       SubscriptBox[
        StyleBox[
         RowBox[{
          StyleBox["d",
           FontSlant->"Plain"], "x"}]], "2"], " ", 
       RowBox[{"\[Delta]", "(", 
        RowBox[{"x", " ", "-", " ", 
         RowBox[{
          SubscriptBox["x", "1"], " ", 
          SubscriptBox["x", "2"]}]}], ")"}], " ", 
       RowBox[{"f", "(", 
        SubscriptBox["x", "1"], ")"}], " ", 
       RowBox[{
        RowBox[{"g", "(", 
         SubscriptBox["x", "2"], ")"}], " ", "."}]}]}]}]}], 
   TraditionalForm]]],
 " Convolute[f, g] is equivalent to Convolute[f, g, x]. Convolute[exp, {x1, \
x2}] assumes that exp is polynomial in x1 and x2. Convolute uses \
table-look-up and does not do any integral calculations, only linear \
algebra."
}], "Text",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[270]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[270]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["ConvoluteTable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ConvoluteTable",
  ButtonNote->"ConvoluteTable"],
 "."
}], "Text",
 CellTags->"Convolute"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[271]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[273]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[274]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[274]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"PlusDistribution", "[", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[275]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[275]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[276]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", "x"}], "-", "x", "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[276]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[277]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[278]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["log", "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[280]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", "x", "-", 
   RowBox[{"x", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[281]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[282]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[282]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[283]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[283]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", " ", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[287]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "x", " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[287]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[288]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[288]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[289]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[289]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[290]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "x", "+", 
   RowBox[{"x", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[290]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[291]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{"log", "(", "x", ")"}], "-", "x", "-", 
   RowBox[{"x", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[291]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[292]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[292]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[293]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], "+", 
   RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[293]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[294]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], 
    RowBox[{"1", "-", "x"}]], "+", 
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[294]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[295]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], "+", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", "x", ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "3"], "(", "x", ")"}]}], "-", 
   TagBox[
    RowBox[{"\[Zeta]", "(", 
     TagBox["3",
      Zeta,
      Editable->True,
      Selectable->True], ")"}],
    InterpretTemplate[
     Function[BoxForm`e$, 
      Zeta[BoxForm`e$]]],
    Editable->False,
    Selectable->False]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[295]="]
}, Open  ]]
}, Open  ]],

Cell["ConvoluteTable ", "ObjectName",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell["\<\
ConvoluteTable[f, g, x] yields the convolution of f and g. ConvoluteTable is \
called by Convolute.\
\>", "Text",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[296]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[296]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[297]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[297]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[298]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[299]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[299]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[300]="]
}, Open  ]]
}, Open  ]],

Cell["CovariantFieldDerivative", "ObjectName",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell["\<\
CovariantFieldDerivative[f[x],x,{li1,li2,...},opts] is a covariant derivative \
of f[x] with respect to space-time variables x and with Lorentz indices li1, \
li2,... CovariantFieldDerivative has only typesetting definitions by default. \
The user is must supply his/her own definition of the actual function.\
\>", "Text",
 CellTags->"CovariantFieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["FieldDerivative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldDerivative",
  ButtonNote->"FieldDerivative"],
 "."
}], "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantFieldDerivative", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ",", "x", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantFieldDerivative",
 CellLabel->"In[301]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    TagBox[
     StyleBox["\<\"\[ScriptCapitalD]\"\>",
      FontSlant->"Italic"],
     StyleForm[#, FontSlant -> "Italic"]& ], 
    TagBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm,
     Editable->True]], "(", 
   TagBox[
    FormBox[
     RowBox[{
      SubscriptBox["A", "\[Mu]"], "(", "x", ")"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"CovariantFieldDerivative",
 CellLabel->"Out[301]="]
}, Open  ]]
}, Open  ]],

Cell["CounterT", "ObjectName",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterT"],

Cell["\<\
CounterT is a factor used by GluonPropagator and QuarkPropagator when \
CounterTerms is set to All.\
\>", "Text",
 CellTags->"CounterT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
   RowBox[{"CounterTerm", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellTags->"CounterT",
 CellLabel->"In[302]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"CounterT", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["S", "n"], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"10", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"], "-", 
          RowBox[{
           FractionBox["10", "3"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubsuperscriptBox["g", "s", "2"]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "+", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["S", "n"], " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FractionBox["4", "3"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"]}], "-", 
          FractionBox[
           RowBox[{"4", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubsuperscriptBox["g", "s", "2"]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellTags->"CounterT",
 CellLabel->"Out[302]="]
}, Open  ]]
}, Open  ]],

Cell["CounterTerm", "ObjectName",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterTerm"],

Cell["\<\
CounterTerm[name] is a database of counter terms. CounterTerm is also an \
option for the Feynman rule functions QuarkGluonVertex, GluonPropagator, \
QuarkPropagator.\
\>", "Text",
 CellTags->"CounterTerm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterT",
  ButtonNote->"CounterT"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CounterTerm", "[", "Zm", "]"}]], "Input",
 CellTags->"CounterTerm",
 CellLabel->"In[303]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["C", "F"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"4", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"11", " ", 
             SubscriptBox["C", "A"]}], "2"], "+", 
           FractionBox[
            RowBox[{"9", " ", 
             SubscriptBox["C", "F"]}], "2"], "-", 
           RowBox[{"2", " ", 
            SubscriptBox["N", "f"], " ", 
            SubscriptBox["T", "f"]}]}], ")"}]}], 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]], "+", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"97", " ", 
             SubscriptBox["C", "A"]}], "12"], "+", 
           FractionBox[
            RowBox[{"3", " ", 
             SubscriptBox["C", "F"]}], "4"], "-", 
           FractionBox[
            RowBox[{"5", " ", 
             SubscriptBox["N", "f"], " ", 
             SubscriptBox["T", "f"]}], "3"]}], ")"}]}], 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]]}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "4"]}], 
    RowBox[{"256", " ", 
     SuperscriptBox["\[Pi]", "4"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     SubscriptBox["C", "F"], " ", 
     SubsuperscriptBox["g", "s", "2"]}], 
    RowBox[{"8", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     SuperscriptBox["\[Pi]", "2"]}]], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"CounterTerm",
 CellLabel->"Out[303]="]
}, Open  ]]
}, Open  ]],

Cell["CouplingConstant", "ObjectName",
 CellTags->"CouplingConstant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CouplingConstant"],

Cell["\<\
In general, CouplingConstant is an option for several Feynman rule functions \
and for CovariantD and FieldStrength.
In the convention of the subpackage PHI, CouplingConstant is also the head of \
coupling constants.  CouplingConstant takes three extra optional arguments, \
with head RenormalizationState, RenormalizationScheme and ExpansionState \
respectively.  E.g. CouplingConstant[QED[1]] is the unit charge, \
CouplingConstant[ChPT2[4],1] is the first of the coupling constants of the \
lagrangian ChPT2[4].  CouplingConstant[a_,b_,c___][i_] := \
CouplingConstant[a,b,RenormalizationState[i],c].\
\>", "Text",
 CellTags->"CouplingConstant"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],

Cell["CovariantD", "ObjectName",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantD"],

Cell["\<\
CovariantD[mu] is a generic covariant derivative with Lorentz index mu. With \
the option-setting Explicit \[Rule] True, an explicit expression for a \
fermionic field is returned, depending on the setting on the other options.
CovariantD[x, mu] is a generic covariant derivative with respect to x^mu.
CovariantD[mu, a, b] is a covariant derivative for a bosonic field; acting on \
QuantumField[f,{},{a,b}], where f is some field name and a and b are two \
SU(N) indices. Again, with the option-setting Explicit \[Rule] True, an \
explicit expression is returned, depending on the setting on the other \
options.
CovariantD[OPEDelta, a, b] is a short form for  \
CovariantD[mu,a,b]*FourVector[OPEDelta, mu]. CovariantD[{OPEDelta, a, b}, \
{n}] yields the product of n operators, where n is an integer.  \
CovariantD[OPEDelta, a, b, {m, n}]  gives the expanded form of \
CovariantD[OPEDelta, a, b]^m up to order g^n for the gluon, where n is an \
integer and g the coupling constant  indicated by the setting of the option \
CouplingConstant. CovariantD[OPEDelta, {m, n}] gives the expanded form of  \
CovariantD[OPEDelta]^m up to order g^n of the fermionic field.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CovariantD", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[304]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"DummyIndex", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[304]="]
}, Open  ]],

Cell["\<\
 Possible settings of PartialD are: LeftPartialD, LeftRigthPartialD, \
RightPartialD. The default setting of QuantumField is GaugeField.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"CovariantD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[305]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[305]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[306]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", "\[Mu]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[306]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[307]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[307]="]
}, Open  ]],

Cell["\<\
The first argument of CovariantD is intepreted as type LorentzIndex, except \
for OPEDelta, which is type Momentum.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "OPEDelta", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[308]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[308]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[309]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[310]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "b"}]]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", 
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[311]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], " ", 
      SubscriptBox["\[Delta]", 
       RowBox[{"a", "c14"}]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e1",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"a", "c14", "e1"}]]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], " ", 
      SubscriptBox["\[Delta]", 
       RowBox[{"b", "c14"}]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e2",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"c14", "b", "e2"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[312]="]
}, Open  ]],

Cell[TextData[{
 "This gives",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"m", " ", "times", " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], "\[CapitalDelta]"]}], 
     ",", " "}]}], TraditionalForm]]],
 "the partial derivative ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "contracted with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[CapitalDelta]", "\[Mu]"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "0"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[313]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to first order in the coupling constant ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     RowBox[{"(", "thesumistheFeynCalcOPESum", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[314]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "1"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[314]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to second order in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[315]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"]}], "-", 
   RowBox[{
    SubsuperscriptBox["g", "s", "2"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], "j"], 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              SubscriptBox[
               OverscriptBox["\[PartialD]", "\[RightArrow]"], 
               FormBox["\[CapitalDelta]",
                TraditionalForm]], ")"}], "i"], ".", 
            SubsuperscriptBox[
             FormBox["A",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["c", "1"],
                Subscripted[
                 $CellContext`c[1]],
                Editable->False],
               TraditionalForm],
              TraditionalForm]], ".", 
            SuperscriptBox[
             RowBox[{"(", 
              SubscriptBox[
               OverscriptBox["\[PartialD]", "\[RightArrow]"], 
               FormBox["\[CapitalDelta]",
                TraditionalForm]], ")"}], 
             RowBox[{"j", "-", "i"}]], ".", 
            SubsuperscriptBox[
             FormBox["A",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["c", "2"],
                Subscripted[
                 $CellContext`c[2]],
                Editable->False],
               TraditionalForm],
              TraditionalForm]], ".", 
            SuperscriptBox[
             RowBox[{"(", 
              SubscriptBox[
               OverscriptBox["\[PartialD]", "\[RightArrow]"], 
               FormBox["\[CapitalDelta]",
                TraditionalForm]], ")"}], 
             RowBox[{
              RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}], " ", 
           SubscriptBox["f", 
            RowBox[{
             FormBox[
              FormBox["a",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["c", "1"],
                Subscripted[
                 $CellContext`c[1]],
                Editable->False],
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["e", "1"],
                Subscripted[
                 $CellContext`e[1]],
                Editable->False],
               TraditionalForm],
              TraditionalForm]}]], " ", 
           SubscriptBox["f", 
            RowBox[{
             FormBox[
              FormBox["b",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["c", "2"],
                Subscripted[
                 $CellContext`c[2]],
                Editable->False],
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox[
               InterpretationBox[
                SubscriptBox["e", "1"],
                Subscripted[
                 $CellContext`e[1]],
                Editable->False],
               TraditionalForm],
              TraditionalForm]}]]}]}],
         HoldForm],
        TraditionalForm]}],
      HoldForm],
     TraditionalForm]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "1"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[315]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}], "OPEm"]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[316]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SubsuperscriptBox["D", 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     RowBox[{"a", "b"}]], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[317]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "1"}]], 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], "i"], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}]}],
      HoldForm],
     TraditionalForm]}], "-", 
   RowBox[{
    SubsuperscriptBox["g", "s", "2"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], "j"], 
          RowBox[{
           SubscriptBox["T", 
            FormBox[
             FormBox[
              InterpretationBox[
               SubscriptBox["c", "1"],
               Subscripted[
                $CellContext`c[1]],
               Editable->False],
              TraditionalForm],
             TraditionalForm]], ".", 
           SubscriptBox["T", 
            FormBox[
             FormBox[
              InterpretationBox[
               SubscriptBox["c", "2"],
               Subscripted[
                $CellContext`c[2]],
               Editable->False],
              TraditionalForm],
             TraditionalForm]], ".", 
           SuperscriptBox[
            RowBox[{"(", 
             SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
               TraditionalForm]], ")"}], "i"], ".", 
           SubsuperscriptBox[
            FormBox["A",
             TraditionalForm], 
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox[
              InterpretationBox[
               SubscriptBox["c", "1"],
               Subscripted[
                $CellContext`c[1]],
               Editable->False],
              TraditionalForm],
             TraditionalForm]], ".", 
           SuperscriptBox[
            RowBox[{"(", 
             SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
               TraditionalForm]], ")"}], 
            RowBox[{"j", "-", "i"}]], ".", 
           SubsuperscriptBox[
            FormBox["A",
             TraditionalForm], 
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox[
              InterpretationBox[
               SubscriptBox["c", "2"],
               Subscripted[
                $CellContext`c[2]],
               Editable->False],
              TraditionalForm],
             TraditionalForm]], ".", 
           SuperscriptBox[
            RowBox[{"(", 
             SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
               TraditionalForm]], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}]}],
         HoldForm],
        TraditionalForm]}],
      HoldForm],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[318]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "Gstrong", " ", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "4"],
       Subscripted[
        $CellContext`c[4]],
       Editable->False], "]"}], "]"}], ".", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"GaugeField", ",", 
      RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
      RowBox[{"SUNIndex", "[", 
       InterpretationBox[
        SubscriptBox["c", "4"],
        Subscripted[
         $CellContext`c[4]],
        Editable->False], "]"}]}], "]"}]}]}], "+", 
  RowBox[{"RightPartialD", "[", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}]}]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[318]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"\[Mu]", ",", "a", ",", "b", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[319]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RightPartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "-", 
  RowBox[{"Gstrong", " ", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"GaugeField", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "5"],
       Subscripted[
        $CellContext`c[5]],
       Editable->False], "]"}]}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", 
     InterpretationBox[
      SubscriptBox["c", "5"],
      Subscripted[
       $CellContext`c[5]],
      Editable->False]}], "]"}]}]}]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[319]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["CrossProduct", "ObjectName",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CrossProduct"],

Cell["\<\
CrossProduct[a, b] denotes the three-dimensional cross-product of the \
three-vectors a and b.\
\>", "Text",
 CellTags->"CrossProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"CrossProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CrossProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"CrossProduct", "[", 
    RowBox[{
     RowBox[{"ThreeVector", "[", "b", "]"}], ",", 
     RowBox[{"ThreeVector", "[", "c", "]"}]}], "]"}]}], "]"}]], "Input",
 CellTags->"CrossProduct",
 CellLabel->"In[320]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["c", "\[RightVector]"],
      TraditionalForm]}], " ", 
    OverscriptBox["b", "\[RightVector]"]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["b", "\[RightVector]"],
      TraditionalForm]}], " ", 
    OverscriptBox["c", "\[RightVector]"]}]}], TraditionalForm]], "Output",
 CellTags->"CrossProduct",
 CellLabel->"Out[320]="]
}, Open  ]]
}, Open  ]],

Cell["C0", "ObjectName",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"C0"],

Cell[TextData[{
 "C0[p10, p12, p20, m1^2, m2^2, m3^2] is the scalar Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "0"], TraditionalForm]]],
 "function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2]), the \
first three arguments of C0 are the scalar products p10 = p1^2, p12 = \
(p1-p2).(p1-p2), p20 = p2^2."
}], "Text",
 CellTags->"C0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"C0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", " ", "m12", ",", "m22", ",", "m32"}], 
  "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[321]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[321]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"C0", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
   "]"}], "//", "PaVeOrder"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[322]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[322]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"C0", "[", 
    RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
    "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"c", ",", "a"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[323]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "b", ",", "c", ",", "a", ",", "m22", ",", "m32", 
   ",", "m12", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[323]="]
}, Open  ]]
}, Open  ]],

Cell["DataType", "ObjectName",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DataType"],

Cell["\<\
DataType[exp, type] = True defines the object exp to have data-type type. \
DataType[exp1, exp2, ..., type] defines the objects exp1, exp2, ...to have \
data-type type. The default setting is DataType[__, _] := False. To assign a \
certain data-type, do, e.g., DataType[x, PositiveInteger] = True.\
\>", "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "Currently used DataTypes: ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommutative",
  ButtonNote->"NonCommutative"],
 ", ",
 ButtonBox["PositiveInteger",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PositiveInteger",
  ButtonNote->"PositiveInteger"],
 ", ",
 ButtonBox["NegativeInteger",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NegativeInteger",
  ButtonNote->"NegativeInteger"],
 ", ",
 ButtonBox["PositiveNumber",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PositiveNumber",
  ButtonNote->"PositiveNumber"],
 ", ",
 ButtonBox["FreeIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeIndex",
  ButtonNote->"FreeIndex"],
 ", ",
 ButtonBox["GrassmannParity",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GrassmannParity",
  ButtonNote->"GrassmannParity"]
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "PHI adds the DataTypes: ",
 ButtonBox["UMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UMatrix",
  ButtonNote->"UMatrix"],
 ", ",
 ButtonBox["UScalar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UScalar",
  ButtonNote->"UScalar"],
 "."
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DataType"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DataType"],

Cell["NonCommutative is just a data-type.", "Text",
 CellTags->"DataType"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[324]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", "a"}], ")"}], ".", "f"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[325]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", " ", "a"}], ")"}], ".", "f"}]}], TraditionalForm]], "Output",\

 CellTags->"DataType",
 CellLabel->"Out[325]="]
}, Open  ]],

Cell["\<\
Since \"f \"and \"g\" have DataType NonCommutative the function DotSimplify \
extracts only \"a\" out of the noncommutative product.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[326]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"2", " ", "a", " ", 
    RowBox[{"g", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[326]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "True"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[327]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest1", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "odd"}], "]"}]}], "\[RuleDelayed]", 
     RowBox[{"-", "1"}]}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[328]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest2", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "even"}], "]"}]}], "\[RuleDelayed]", "1"}]}]}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[329]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "a"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "z"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[330]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[330]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest1", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[331]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest2", "[", "%", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[332]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "z"], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[332]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ptest1", ",", "ptest2", ",", "t", ",", "a", ",", "m"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[333]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "ganzeZahl"}], "]"}], "=", "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[334]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "^", "p_"}], "/;", 
       RowBox[{"DataType", "[", 
        RowBox[{"p", ",", "ganzeZahl"}], "]"}]}], "\[RuleDelayed]", "1"}], 
     "}"}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[335]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"test", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", "n"}], " ", "x"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[336]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[336]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "test", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[337]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[337]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f", ",", "test"}], "]"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[338]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "False"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[339]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "False"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[340]:="],

Cell["\<\
Certain FeynCalc objects have DataType PositiveInteger set to True.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[341]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[341]="]
}, Open  ]],

Cell["PowerSimplify uses the DataType information.", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[342]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[342]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", " ", 
     RowBox[{"SO", "[", "q", "]"}]}], ")"}], "^", "OPEm"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[343]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[343]="]
}, Open  ]]
}, Open  ]],

Cell["DB0", "ObjectName",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB0"],

Cell["\<\
DB0[p2, m1^2, m2^2] is the derivative of the two-point function B0[p2, m1^2, \
m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"DB0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B0", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB0",
 CellLabel->"In[344]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"DB0", "(", 
   RowBox[{
    SubscriptBox["p", "2"], ",", 
    SubsuperscriptBox["m", "1", "2"], ",", 
    SubsuperscriptBox["m", "2", "2"]}], ")"}], TraditionalForm]], "Output",
 CellTags->"DB0",
 CellLabel->"Out[344]="]
}, Open  ]]
}, Open  ]],

Cell["DB1", "ObjectName",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB1"],

Cell["\<\
DB1[p2,m1^2,m2^2] is the derivative of B1[p2,m1^2,m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 "."
}], "Text",
 CellTags->"DB1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B1", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB1",
 CellLabel->"In[345]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "2"], "-", 
       SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
     RowBox[{"DB0", "(", 
      RowBox[{
       SubscriptBox["p", "2"], ",", 
       SubsuperscriptBox["m", "1", "2"], ",", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}]}], 
    RowBox[{"2", " ", 
     SubscriptBox["p", "2"]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"DB0", "(", 
     RowBox[{
      SubscriptBox["p", "2"], ",", 
      SubsuperscriptBox["m", "1", "2"], ",", 
      SubsuperscriptBox["m", "2", "2"]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SubscriptBox["p", "2"], ",", 
        SubsuperscriptBox["m", "1", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "1", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "2"], "-", 
       SubsuperscriptBox["m", "1", "2"]}], ")"}]}], 
    RowBox[{"2", " ", 
     SubsuperscriptBox["p", "2", "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"DB1",
 CellLabel->"Out[345]="]
}, Open  ]]
}, Open  ]],

Cell["DeclareNonCommutative", "ObjectName",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell["\<\
DeclareNonCommutative[a, b, ...] declares a,b, ... to be non-commutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to True.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[346]:="],

Cell["\<\
As a side effect of DeclareNonCommutative x is declared to be of data type \
NonCommutative.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[347]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[347]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[348]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[349]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[349]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[350]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[351]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[351]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunction", "ObjectName",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 " DeltaFunction[x] is the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 ".\n*** After version 4 of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", there is a built-in function, DiracDelta, with comparable properties. \
***"
}], "Text",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunction", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[352]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[353]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[353]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", "x", "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[354]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "0", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[354]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[355]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[355]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"DeltaFunction", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[356]:="],

Cell[BoxData[
 FormBox["x", TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[356]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunctionDoublePrime", "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 " DeltaFunctionDoublePrime[1-x] is the second derivative of the Dirac \
delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],

Cell["DeltaFunctionPrime", "ObjectName",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 " DeltaFunctionPrime[1-x] is the derivative of the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunctionPrime", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[357]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[357]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[358]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    SuperscriptBox["x", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[359]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[359]="]
}, Open  ]]
}, Open  ]],

Cell["DenominatorOrder", "ObjectName",
 CellTags->"DenominatorOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DenominatorOrder"],

Cell["\<\
DenominatorOrder is an option for OneLoop, if set to True the \
PropagatorDenominator will be ordered in a standard way.\
\>", "Text",
 CellTags->"DenominatorOrder"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],

Cell["Dimension", "ObjectName",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Dimension"],

Cell["\<\
Dimension is an option of several functions and denotes the number of \
space-time dimensions. Possible settings are: 4, n, d, D, ... ,the variable \
does not matter, but it should have Head Symbol.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "MetricTensor", "]"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[360]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[360]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"m", ",", "n", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[361]="]
}, Open  ]],

Cell["\<\
The dimension of the indices is not shown by default but can be inspected \
easily.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MetricTensor", "[", 
    RowBox[{"m", ",", "n", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
   RowBox[{"DiracMatrix", "[", 
    RowBox[{"\[Alpha]", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[362]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "d"}], "]"}], ",", "d"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"m", ",", "d"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"n", ",", "d"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[362]//StandardForm="]
}, Open  ]],

Cell["\<\
Setting the global variable $LorentzIndices to True will display the \
dimension (if different from 4) as a subscript.\
\>", "Text",
 CellTags->"Dimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[363]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[364]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[364]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[365]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[365]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[366]:="]
}, Open  ]],

Cell["DimensionalReduction", "ObjectName",
 CellTags->"DimensionalReduction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DimensionalReduction"],

Cell["\<\
DimensionalReduction is an option for TID and OneLoopSimplify.\
\>", "Text",
 CellTags->"DimensionalReduction"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],

Cell["DiracBasis", "ObjectName",
 CellTags->"DiracBasis"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracBasis"],

Cell["\<\
DiracBasis[any] is a head which is wrapped around Dirac structures (and the \
1) as a result of the function DiracReduce. Eventually you want to substitute \
DiracBasis by Identity (or set: DiracBasis[1] = S; \
DiracBasis[DiracMatrix[mu]] = P; etc.).\
\>", "Text",
 CellTags->"DiracBasis"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracReduce",
  ButtonNote->"DiracReduce"],
 "."
}], "Text",
 CellTags->"DiracBasis"]
}, Open  ]],

Cell["DiracCanonical", "ObjectName",
 CellTags->"DiracCanonical"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracCanonical"],

Cell["\<\
DiracCanonical is an option for DiracSimplify. If set to True DiracSimplify \
uses the function DiracOrder internally.\
\>", "Text",
 CellTags->"DiracCanonical"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],

Cell["DiracEquation", "ObjectName",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracEquation"],

Cell["\<\
DiracEquation[exp] applies the Dirac equation without expanding exp. If that \
is needed, use DiracSimplify.\
\>", "Text",
 CellTags->"DiracEquation"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracEquation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "p", "]"}], "]"}]}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[367]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracEquation",
 CellLabel->"Out[367]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracEquation", "[", "%", "]"}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[368]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"m", " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracEquation",
 CellLabel->"Out[368]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGamma", "ObjectName",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "DiracGamma[x, dim] is the head of all Dirac matrices and slashes (in the \
internal representation). Use DiracMatrix (or GA, GAD) and DiracSlash (or GS, \
GSD) for manual (short) input. DiracGamma[x, 4] simplifies to DiracGamma[x]. \
DiracGamma[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracGamma[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
   TraditionalForm]]],
 " DiracGamma[7] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGamma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", "5", "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[369]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[369]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[370]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[370]="]
}, Open  ]],

Cell[TextData[{
 "A Dirac-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["q", "\[Mu]"]}], TraditionalForm]]],
 ", is displayed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[371]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[371]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[372]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"q", ",", "D"}], "]"}], ",", "D"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[373]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[373]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[374]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[374]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "=", 
  RowBox[{"DiracGammaExpand", "[", "a1", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[375]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[375]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", "=", 
  RowBox[{
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"GSD", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GSD", "[", "q", "]"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[376]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[376]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a4", "=", 
  RowBox[{"DiracTrick", "[", "a3", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[377]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a5", "=", 
  RowBox[{"DiracSimplify", "[", "a4", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"D", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "-", 
   RowBox[{"D", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"]}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[378]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "]"}]], "Input",\

 CellTags->"DiracGamma",
 CellLabel->"In[379]:="]
}, Open  ]],

Cell["DiracGammaCombine", "ObjectName",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell["\<\
DiracGammaCombine[exp] is (nearly) the inverse operation to DiracGammaExpand.\
\
\>", "Text",
 CellTags->"DiracGammaCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], " ", "+", " ", 
   RowBox[{"GS", "[", "q", "]"}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[380]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[380]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[381]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "+", "q"}], "]"}], "]"}]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[381]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "p", "]"}]}], " ", "-", " ", 
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "q", "]"}]}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[382]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "-", 
     RowBox[{"2", " ", "q"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[382]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[383]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{
    RowBox[{"2", " ", "p"}], "-", 
    RowBox[{"2", " ", "q"}]}], "]"}], "]"}]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[383]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "%%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[384]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[384]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGammaExpand", "ObjectName",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell["\<\
DiracGammaExpand[exp] expands all DiracGamma[Momentum[a+b+..]] in exp into \
(DiracGamma[Momentum[a]] + DiracGamma[Momentum[b]] + ...).\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaCombine",
  ButtonNote->"DiracGammaCombine"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], "]"}]}]}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[385]="]
}, Open  ]],

Cell["\<\
Momentum is the head of p-q, i.e., it is treated as one four-momentum.\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[386]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[386]//StandardForm="]
}, Open  ]],

Cell["With DiracGammaExpand the Momentum[p-q] gets expanded.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[387]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[387]="]
}, Open  ]],

Cell["The inverse operation is DiracGammaCombine.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaCombine", "[", 
   RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[388]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[388]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[389]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}], "-", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "q", "]"}], "]"}]}], ")"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[389]//StandardForm="]
}, Open  ]],

Cell["In order to do non-commutative expansion use DiracSimplify.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[390]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[390]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[391]:="]
}, Open  ]],

Cell["DiracGammaT", "ObjectName",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell["\<\
DiracGammaT[x] denotes the transpose of DiracGamma[x]. \
Transpose[DiracGammaT[x]] gives DiracGamma[x]. Note that x must have Head \
LorentzIndex or Momentum.\
\>", "Text",
 CellTags->"DiracGammaT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 "."
}], "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaT", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[392]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[392]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Transpose", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[393]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[393]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", "Transpose"}]], "Input",\

 CellTags->"DiracGammaT",
 CellLabel->"In[394]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", "\[Gamma]", "\[CenterDot]", 
    FormBox["p",
     TraditionalForm], ")"}], "T"], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[394]="]
}, Open  ]]
}, Open  ]],

Cell["DiracMatrix", "ObjectName",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes a Dirac gamma matrix with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". DiracMatrix[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", ",", "\[Nu]", ","}], " "}], TraditionalForm]]],
 "...] is a product of ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 " matrices with Lorentz indices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 " DiracMatrix[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracMatrix[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "2"}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"+", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 ". DiracMatrix[7] is",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", "2"}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"-", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[395]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[395]="]
}, Open  ]],

Cell[TextData[{
 "This is how to enter the non-commutative product of two ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 ". The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Dot \".\" is used as non-commutative multiplication operator."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[396]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[396]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Alpha]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[397]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[397]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], "//", "FCE"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[398]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[398]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[399]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[399]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[400]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[400]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[401]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[401]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[402]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[402]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"a", " ", ".", " ", "b"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[403]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[404]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "a", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "b", "]"}], "]"}]}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[404]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracOrder", "ObjectName",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracOrder"],

Cell["\<\
DiracOrder[expr] orders the Dirac matrices in expr alphabetically. \
DiracOrder[expr,orderlist] orders the Dirac matrices in expr according to \
orderlist.\
\>", "Text",
 CellTags->"DiracOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Beta]", ",", "\[Alpha]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[405]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[405]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[406]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[406]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[407]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[407]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[408]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[408]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[409]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[409]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t3", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[410]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[410]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[411]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[411]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t4", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[412]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[412]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[413]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[413]="]
}, Open  ]],

Cell[TextData[{
 "This orders the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Alpha]"], 
    SuperscriptBox["\[Gamma]", "\[Beta]"], 
    SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]]],
 " in reverse order."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", 
  RowBox[{"t5", ",", 
   RowBox[{"{", 
    RowBox[{"\[Delta]", ",", "\[Beta]", ",", "\[Alpha]"}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[414]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Delta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Delta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Delta]"}]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Delta]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[414]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "%", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[415]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[415]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[416]:="]
}, Open  ]],

Cell["DiracReduce", "ObjectName",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracReduce"],

Cell["\<\
DiracReduce[exp] reduces all four-dimensional Dirac matrices in exp to the \
standard basis (S,P,V,A,T) using the Chisholm identity (see Chisholm). In the \
result the basic Dirac structures are wrapped with a head DiracBasis. I.e., S \
corresponds to DiracBasis[1], P : DiracBasis[DiracMatrix[5]], V: \
DiracBasis[DiracMatrix[mu]], A: DiracBasis[DiracMatrix[mu, 5]], T: \
DiracBasis[DiracSigma[DiracMatrix[mu, nu]]]. By default DiracBasis is \
substituted to Identity. Notice that the result of DiracReduce is given in \
the FeynCalcExternal-way, i.e.,evtl. you may have to use FeynCalcInternal on \
the result.\
\>", "Text",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracReduce", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[417]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"DiracBasis", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[417]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[418]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[418]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t1", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[419]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[420]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t2", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[421]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      RowBox[{"$MU", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", 
      FormBox[
       RowBox[{"$MU", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[422]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"], 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[422]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracReduce", "[", "t3", "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[423]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    SuperscriptBox["\[Gamma]", "5"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[423]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", " ", "=", 
  RowBox[{"Calc", "[", 
   RowBox[{"DiracSimplify", "[", 
    RowBox[{"DiracSigmaExplicit", "[", 
     RowBox[{"t4", ".", "t4"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t4", ".", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[425]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[425]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[426]:="]
}, Open  ]],

Cell["DiracSigma", "ObjectName",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigma"],

Cell["\<\
DiracSigma[a, b] stands for i/2*(a . b - b . a) in 4 dimensions. a and b must \
have Head DiracGamma, DiracMatrix or DiracSlash. Only antisymmetry is \
implemented.\
\>", "Text",
 CellTags->"DiracSigma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigmaExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSigmaExplicit",
  ButtonNote->"DiracSigmaExplicit"],
 "."
}], "Text",
 CellTags->"DiracSigma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[427]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[427]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[428]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Beta]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[429]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["\[Sigma]", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[429]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], ",", 
    RowBox[{"GS", "[", "q", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[430]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"p", "q"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[430]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[431]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[431]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[432]:="]
}, Open  ]],

Cell["DiracSigmaExplicit", "ObjectName",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell["\<\
DiracSigmaExplicit[exp] inserts in exp for all DiracSigma its definition. \
DiracSigmaExplict is also an option of DiracSimplify.\
\>", "Text",
 CellTags->"DiracSigmaExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSigma",
  ButtonNote->"DiracSigma"],
 "."
}], "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigma", "[", 
  RowBox[{
   RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[433]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[433]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "%", "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[434]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[434]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSimpCombine", "ObjectName",
 CellTags->"DiracSimpCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimpCombine"],

Cell["\<\
DiracSimpCombine is an option for DiracSimplify. If set to True, sums of \
DiracGamma's will be merged as much as possible in DiracGamma[ .. + .. + ]'s.\
\
\>", "Text",
 CellTags->"DiracSimpCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],

Cell["DiracSimplify", "ObjectName",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["\<\
DiracSimplify[expr] simplifies products of Dirac matrices in expr and expands \
non-commutative products. Double Lorentz indices and four vectors are \
contracted. The Dirac equation is applied. All DiracMatrix[5], DiracMatrix[6] \
and DiracMatrix[7] are moved to the right. The order of the other Dirac \
matrices is not changed.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSimplify", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[435]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracCanonical", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSigmaExplicit", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracSimpCombine", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSubstitute67", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
    RowBox[{"InsideDiracTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[435]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[436]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[436]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[437]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[438]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[439]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[440]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[441]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[441]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[442]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[442]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[443]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[444]="]
}, Open  ]],

Cell["\<\
Contrary to DiracTrick DiracSimplify does non-commutative expansion.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[445]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[446]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", " ", "=", " ", 
  RowBox[{
   RowBox[{"SpinorVBar", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"SpinorUBar", "[", "q", "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", "p", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "q", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[447]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t6", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[448]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[449]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"D", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"12", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "2"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "5"], 
      SubscriptBox["\[Nu]", "6"]}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[450]="]
}, Open  ]],

Cell["\<\
 With the option DiracCanonical an alphabetic ordering is done.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}], ",", 
   RowBox[{"DiracCanonical", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[451]="]
}, Open  ]],

Cell["\<\
Setting InsideDiracTrace\[Rule]True assumes that a trace is still to be taken \
later on.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[452]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[453]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[453]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
  "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[454]:="]
}, Open  ]],

Cell["DiracSimplify2", "ObjectName",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "DiracSimplify2[exp] simplifies the Dirac structure but leaves any ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " untouched."
}], "Text",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{
  "\[Mu]", ",", "\[Nu]", ",", "\[Mu]", ",", "5", ",", "\[Alpha]", ",", 
   "\[Beta]", ",", "\[Alpha]"}], "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[455]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify2",
 CellLabel->"Out[455]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify2", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[456]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify2",
 CellLabel->"Out[456]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSlash", "ObjectName",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "DiracSlash[p] is the contraction ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], 
    SubscriptBox["\[Gamma]", 
     RowBox[{"\[Mu]", " "}]]}], TraditionalForm]]],
 "(FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]). Products of those can be entered in the form DiracSlash[p1, p2, ...]."
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSlash", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[457]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[457]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracSlash"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "This is q-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    RowBox[{
     SubscriptBox["q", "\[Mu]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", "q", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[458]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[459]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[459]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[460]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[460]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[461]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[461]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "q", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[462]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", "q", "]"}], "]"}]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[462]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"q", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[463]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "n"}], "]"}], ",", "n"}], "]"}]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[463]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracSpinor", "ObjectName",
 CellTags->"DiracSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSpinor"],

Cell["\<\
DiracSpinor is simply a quantity defined as noncommutative (with \
DeclareNonCommutative[DiracSpinor]). The convention intended is that \
DiracSpinor[p, m, ind] is a Dirac spinor for a fermion with momentum p and \
mass m and indices ind.\
\>", "Text",
 CellTags->"DiracSpinor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],

Cell["DiracSubstitute67", "ObjectName",
 CellTags->"DiracSubstitute67"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSubstitute67"],

Cell["\<\
DiracSubstitute67 is an option for DiracSimplify. If set to True the \
chirality-projectors DiracGamma[6] and DiracGamma[7] are substituted by their \
definitions.\
\>", "Text",
 CellTags->"DiracSubstitute67"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],

Cell["DiracTrace", "ObjectName",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrace"],

Cell["\<\
DiracTrace[expr] is the head of Dirac traces. Whether the trace is evaluated \
depends on the option DiracTraceEvaluate. Direct trace evaluation should be \
performed with TR (or Tr). The argument expr may be a product of Dirac \
matrices or slashes separated by the Mathematica Dot (.).\
\>", "Text",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[464]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[464]="]
}, Open  ]],

Cell[TextData[{
 "For comments regarding ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes see the notes for ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[465]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}],
     "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{
    "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Delta]", 
     ",", "\[Tau]", ",", "5"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[469]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Mu]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Nu]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Rho]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Sigma]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Tau]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[470]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[472]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[472]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracTraceEvaluate", "ObjectName",
 CellTags->"DiracTraceEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTraceEvaluate"],

Cell["\<\
DiracTraceEvaluate is an option for DiracTrace and Tr. If set to False, \
DiracTrace remains unevaluated.\
\>", "Text",
 CellTags->"DiracTraceEvaluate"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],

Cell["DiracTrick", "ObjectName",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["\<\
DiracTrick[exp] contracts gamma matrices with each other and performs several \
simplifications, but no expansion, use Calc or DiracSimplify for \
non-commutative expansion.\
\>", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrick", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[473]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[473]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[474]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t1", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[475]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[476]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t2", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[478]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t3", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[479]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[479]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[480]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[480]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t4", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[481]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[481]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[482]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[482]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[483]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[483]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[484]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[485]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "%", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[486]="]
}, Open  ]]
}, Open  ]],

Cell["Divideout", "ObjectName",
 CellTags->"Divideout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Divideout"],

Cell["\<\
Divideout is an option for OPEInt and OPEInsert. The setting is divided out \
at the end.\
\>", "Text",
 CellTags->"Divideout"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInt",
  ButtonNote->"OPEInt"],
 ", ",
 ButtonBox["OPEInsert",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInsert",
  ButtonNote->"OPEInsert"],
 "."
}], "Text",
 CellTags->"Divideout"]
}, Open  ]],

Cell["DOT", "ObjectName",
 CellTags->"DOT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DOT"],

Cell["\<\
DOT[a, b, ...] is the FeynCalc function for non-commutative multiplication. \
By default it is set to the Mathematica Dot functions. By setting
DOT=.
this can be disabled. Note that then non-commutative products should to be \
entered like DOT[ DiracMatrix[mu], m + DiracSlash[p], DiracMatrix[mu] ], etc.\
\
\>", "Text",
 CellTags->"DOT"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DOT"]
}, Open  ]],

Cell["DotExpand", "ObjectName",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotExpand"],

Cell["DotExpand[expr] expands DOT products in expr.", "Text",
 CellTags->"DotExpand"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DOT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DOT",
  ButtonNote->"DOT"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DotExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[487]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[487]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[488]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[489]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "d"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "e"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "f"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "d"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "e"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "f"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "d"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "e"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[489]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnDeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[490]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[491]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[491]="]
}, Open  ]]
}, Open  ]],

Cell["DotPower", "ObjectName",
 CellTags->"DotPower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotPower"],

Cell["\<\
DotPower is an option for DotSimplify. It determines whether non-commutative \
powers are represented by succesive multiplication or by Power.\
\>", "Text",
 CellTags->"DotPower"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotPower"]
}, Open  ]],

Cell["DotProduct", "ObjectName",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotProduct"],

Cell["\<\
DotProduct[x, y] denotes the three-dimensional dot-product. If x and y have \
Head List, DotProduct[x, a] (where a is a vector) performs Sum[ x[[k]] \
a[[k]], {k, 0, 3}].\
\>", "Text",
 CellTags->"DotProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CrossProduct",
  ButtonNote->"CrossProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"DotProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"3", 
    RowBox[{"ThreeVector", "[", "b", "]"}]}]}], "]"}]], "Input",
 CellTags->"DotProduct",
 CellLabel->"In[492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", " ", 
   RowBox[{
    FormBox[
     OverscriptBox["a", "\[RightVector]"],
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     OverscriptBox["b", "\[RightVector]"],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"DotProduct",
 CellLabel->"Out[492]="]
}, Open  ]]
}, Open  ]],

Cell["DotSimplifyRelations", "ObjectName",
 CellTags->"DotSimplifyRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplifyRelations"],

Cell["\<\
DotSimplifyRelations is an option for DotSimplify. Its setting may be a list \
of substitution rules of the form DotSimplifyRelations \[Rule] {a . b \[Rule] \
c, b^2 \[Rule] 0, ...}.\
\>", "Text",
 CellTags->"DotSimplifyRelations"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],

Cell["DotSimplify", "ObjectName",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplify"],

Cell["\<\
DotSimplify[expr] expands and reorders noncommutative terms in expr. \
Simplifying relations may be specified by the option DotSimplifyRelations or \
by Commutator and AntiCommutator definitions. Whether expr is expanded \
noncommutatively depends on the option Expanding.\
\>", "Text",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DotSimplify", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"DotSimplifyRelations", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DotPower", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[493]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 "."
}], "Text",
 CellTags->"DotSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"GS", "[", "p", "]"}]}], "-", 
     RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
   RowBox[{"GA", "[", "\[Nu]", "]"}]}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "-", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "q"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[494]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[495]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[496]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"z", " ", "c"}]}], ")"}], ".", "a"}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[497]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"c", " ", "z"}]}], ")"}], ".", "a"}], TraditionalForm]], "Output",\

 CellTags->"DotSimplify",
 CellLabel->"Out[497]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[498]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "1"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[499]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[499]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[500]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"(", 
     RowBox[{"a", "+", 
      RowBox[{"c", ".", "a", ".", "a"}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[500]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[501]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[502]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[502]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "c"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[503]:="],

Cell[BoxData[
 FormBox["c", TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[503]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[504]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "c"}], "-", 
   RowBox[{"a", ".", "a", ".", "b"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[504]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[505]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"DotSimplifyRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ".", "c"}], "\[Rule]", 
      RowBox[{"1", "/", "z"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[506]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", "a"}], TraditionalForm]], "Output",\

 CellTags->"DotSimplify",
 CellLabel->"Out[506]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[507]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"x", ".", "x", ".", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[508]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[508]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{
   RowBox[{"x", ".", "x", ".", "x"}], ",", 
   RowBox[{"DotPower", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[509]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[509]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[510]:="]
}, Open  ]],

Cell["DummyIndex", "ObjectName",
 CellTags->"DummyIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DummyIndex"],

Cell["\<\
DummyIndex is an option of CovariantD specifying an index to use as dummy \
summation index. If set to Automatic, unique indices are generated\
\>", "Text",
 CellTags->"DummyIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 "."
}], "Text",
 CellTags->"DummyIndex"]
}, Open  ]],

Cell["D0", "ObjectName",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0"],

Cell[TextData[{
 "D0[ p10, p12, p23, p30, p20, p13, m1^2, m2^2, m3^2, m4^2 ] is the \
Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "0"], TraditionalForm]]],
 " function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2] \
[(q+p3)^2-m4^2] ), the first six arguments of D0 are the scalar products p10 \
= p1^2, p12 = (p1-p2)^2, p23 = (p2-p3)^2, p30 = p3^2, p20 = p2^2, p13 = \
(p1-p3)^2."
}], "Text",
 CellTags->"D0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"D0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0", "[", 
  RowBox[{
  "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
   RowBox[{"m1", "^", "2"}], ",", 
   RowBox[{"m2", "^", "2"}], ",", 
   RowBox[{"m3", "^", "2"}], ",", 
   RowBox[{"m4", "^", "2"}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[511]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[511]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
     RowBox[{"m1", "^", "2"}], ",", 
     RowBox[{"m2", "^", "2"}], ",", 
     RowBox[{"m3", "^", "2"}], ",", 
     RowBox[{"m4", "^", "2"}]}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"p13", ",", "p20"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[512]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p30", ",", "p23", ",", "p12", ",", 
   "p13", ",", "p20", ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[512]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", "%", "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[513]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[513]="]
}, Open  ]]
}, Open  ]],

Cell["D0Convention", "ObjectName",
 CellTags->"D0Convention"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0Convention"],

Cell["\<\
D0Convention is an option for Write2. If set to 1, the convention for the \
arguments of D0 is changed when writing a Fortran file with Write2: The fifth \
and sixth argument of D0 are interchanged and the square root is taken of the \
last four arguments.\
\>", "Text",
 CellTags->"D0Convention"],

Cell[TextData[{
 "See also: ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"D0Convention"]
}, Open  ]],

Cell["Eps", "ObjectName",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Eps"],

Cell[TextData[{
 "Eps[a, b, c, d] is the head of the totally antisymmetric ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "\[Epsilon]"}], TraditionalForm]]],
 " (Levi-Civita) tensor. The a,b, ... may have head LorentzIndex, Momentum or \
Integer. In case of integers the Levi-Civita tensor is evaluated immediately. \
"
}], "Text",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Eps", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[514]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[514]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsEvaluate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsEvaluate",
  ButtonNote->"EpsEvaluate"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 "."
}], "Text",
 CellTags->"Eps"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[515]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[515]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[516]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[516]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Eps", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[517]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Eps", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]}]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[517]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[518]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[518]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"1", ",", "0", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[519]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[519]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[520]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[521]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[521]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1a1", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[522]:="],

Cell[BoxData[
 FormBox["a1a1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[522]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[523]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[524]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[524]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "a2a2", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[525]:="],

Cell[BoxData[
 FormBox["a2a2", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[525]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
   ".", 
   RowBox[{"GAD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[526]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Alpha]"}]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellLabel->"Out[526]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellLabel->"In[527]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellLabel->"Out[527]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
     ".", 
     RowBox[{"GAD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
      "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5p", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}], ".", 
     RowBox[{"GAD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[529]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox[
        SuperscriptBox["\[Mu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Nu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Rho]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Alpha]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[529]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "=", 
  RowBox[{"Factor2", "[", 
   RowBox[{"Calc", "[", 
    RowBox[{"g5", ".", "g5p"}], "]"}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[530]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[530]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "/.", 
  RowBox[{"D", "\[Rule]", "4"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[531]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[531]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "g5", ",", "g5p", ",", "g52"}], 
  "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[532]:="]
}, Open  ]],

Cell["EpsChisholm", "ObjectName",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell["\<\
EpsChisholm[expr] substitutes for a gamma matrix contracted with a \
Levi-Civita tensor (Eps) the Chisholm identity.\
\>", "Text",
 CellTags->"EpsChisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[533]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[ImaginaryI]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$1275",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$1275",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"EpsChisholm",
 CellLabel->"Out[533]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsChisholm", "[", "%", "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[534]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"EpsChisholm",
 CellLabel->"Out[534]="]
}, Open  ]]
}, Open  ]],

Cell["EpsContract", "ObjectName",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsContract"],

Cell["\<\
EpsContract is an option of Contract specifying whether Levi-Civita tensors \
Eps[...] will be contracted, i.e., products of two  Eps are replaced via the \
determinant formula.\
\>", "Text",
 CellTags->"EpsContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"EpsContract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[535]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[535]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[536]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], ")"}], "2"], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[536]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[537]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["D", "4"]}], "+", 
   RowBox[{"6", " ", 
    SuperscriptBox["D", "3"]}], "-", 
   RowBox[{"11", " ", 
    SuperscriptBox["D", "2"]}], "+", 
   RowBox[{"6", " ", "D"}]}], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[537]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "a1", "]"}]], "Input",
 CellLabel->"In[538]:="]
}, Open  ]],

Cell["EpsDiscard", "ObjectName",
 CellTags->"EpsDiscard"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsDiscard"],

Cell["\<\
EpsDiscard is an option for FeynCalc2FORM. If set to True all Levi-Civita \
tensors are replaced by 0 after contraction.\
\>", "Text",
 CellTags->"EpsDiscard"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],

Cell["EpsEvaluate", "ObjectName",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell["\<\
EpsEvaluate[expr] applies total antisymmetry and linearity (w.r.t. \
Momentum's) to all Levi-Civita tensors (Eps') in expr.\
\>", "Text",
 CellTags->"EpsEvaluate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"p", "+", "q"}], ",", "\[Sigma]"}], "]"}]}], "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[539]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[539]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsEvaluate", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[540]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[540]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[541]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}]}], "]"}]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[541]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["EpsUncontract", "ObjectName",
 CellTags->"EpsUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsUncontract"],

Cell["\<\
EpsUncontract does Uncontract on scalar products involving Eps.\
\>", "Text",
 CellTags->"EpsUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],

Cell["Epsilon", "ObjectName",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Epsilon"],

Cell[TextData[{
 "Epsilon is (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-4), where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the space-time dimension. Epsilon stands for a small positive number."
}], "Text",
 CellTags->"Epsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Epsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData["Epsilon"], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[542]:="],

Cell[BoxData[
 FormBox[
  TagBox["\[CurlyEpsilon]",
   TraditionalForm], TraditionalForm]], "Output",
 CellTags->"Epsilon",
 CellLabel->"Out[542]="]
}, Open  ]],

Cell["\<\
Epsilon has no functional properties, but some upvalues are changed:\
\>", "Text",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "4"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "3"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "2"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", "0"}]}], 
  "}"}]], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[543]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
   "}"}], TraditionalForm]], "Output",
 CellTags->"Epsilon",
 CellLabel->"Out[543]="]
}, Open  ]]
}, Open  ]],

Cell["EpsilonOrder", "ObjectName",
 CellTags->"EpsilonOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "EpsilonOrder is an option of OPEIntDelta and RHI. The setting determines \
the order ",
 "n",
 " (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Epsilon", 
    StyleBox["n",
     FontSlant->"Plain"]], TraditionalForm]]],
 ") which should be kept."
}], "Text",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 ", ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],

Cell["Expanding", "ObjectName",
 CellTags->"Expanding"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expanding"],

Cell["\<\
Expanding is an option for Calc, Contract, DiracSimplify,DotSimplify, \
SUNSimplify, etc. As option for Contract it specifies whether expansion \
w.r.t. LorentzIndex is done BEFORE contraction. If set to False in \
DiracSimplify or SUNSimplify, only a limited set of simplifications \
(multiplicative linearity etc.) is performed.\
\>", "Text",
 CellTags->"Expanding"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"Expanding"]
}, Open  ]],

Cell["ExpandPartialD", "ObjectName",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell["\<\
ExpandPartialD[exp] expands all products of QuantumField's and partial \
differentiation operators in exp and applies the Leibniz rule.\
\>", "Text",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ExpandPartialD", "]"}]], "Input",
 CellLabel->"In[544]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a$___", ".", 
       RowBox[{"\[PartialD]", "/", "\[PartialD]", 
        SuperscriptBox[
         FormBox["x$_",
          TraditionalForm], 
         FormBox[
          FormBox["mu$_",
           TraditionalForm],
          TraditionalForm]]}], ".", "b$___"}], "\[RuleDelayed]", 
      RowBox[{"a$", ".", 
       RowBox[{
        SubscriptBox[
         TagBox[
          StyleBox["\<\"\[PartialD]\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ], 
         TagBox[
          FormBox[
           FormBox["mu$",
            TraditionalForm],
           TraditionalForm],
          TraditionalForm,
          Editable->True]], "(", 
        TagBox[
         FormBox[
          RowBox[{"Dot", "[", "b$", "]"}],
          TraditionalForm],
         TraditionalForm,
         Editable->True], ")"}]}]}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[544]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["PartialDRelations",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialDRelations",
  ButtonNote->"PartialDRelations"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[545]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[547]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[547]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[548]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[548]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[Null]"]}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "2"}], ")"}], 
  ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "2"], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[550]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[551]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm]}], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ")"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[PartialD]", 
         FormBox["\[CapitalDelta]",
          TraditionalForm]], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ".", 
       RowBox[{
        SubscriptBox["\[PartialD]", 
         FormBox["\[CapitalDelta]",
          TraditionalForm]], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}]}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm]}], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ")"}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[551]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"8", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "3"}]}]], "Input",\

 CellTags->"ExpandPartialD",
 CellLabel->"In[552]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "3"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[552]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[553]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "3"], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[553]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[554]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "-", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Tau]"}], "]"}], " ", 
  
  RowBox[{"RightPartialD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Mu]", ",", "\[Beta]", ",", "\[Nu]"}], 
   "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Tau]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[556]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[556]="]
}, Open  ]]
}, Open  ]],

Cell["ExpandScalarProduct, ScalarProductExpand", "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell["\<\
ExpandScalarProduct[expr] expands scalar products of sums of momenta in expr. \
ExpandScalarProduct does not use Expand on expr.
ScalarProductExpand=ExpandScalarProduct.\
\>", "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "//", "Options"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[557]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[557]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p1", "+", "p2", "+", "p3"}], ",", 
    RowBox[{"p4", "+", "p5", "+", "p6"}]}], "]"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p1", "+", "p2", "+", "p3"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p4", "+", "p5", "+", "p6"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[558]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ScalarProductExpand"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p1", "+", "p2", "+", "p3"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p4", "+", "p5", "+", "p6"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"p", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[562]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[563]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "-", "r"}]}], "]"}], "//", "FCI"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"q", "-", "r"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"Pair", "\[Rule]", "ExpandScalarProduct"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[565]="]
}, Open  ]]
}, Open  ]],

Cell["Expand2", "ObjectName",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expand2"],

Cell["\<\
Expand2[exp, x] expands all sums containing x. Expand2[exp, {x1, x2, ...}]  \
expands all sums containing x1, x2, ....\
\>", "Text",
 CellTags->"Expand2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x1", "+", "x2", "+", "x3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "x1"}], "+", 
       RowBox[{"3", "x2"}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "y1"}], "+", 
       RowBox[{"3", "y2"}]}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "x2"}], "}"}]}], "]"}]], "Input",
 CellTags->"Expand2",
 CellLabel->"In[566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["x1", "2"]}], "+", 
   RowBox[{"5", " ", "x2", " ", "x1"}], "+", 
   RowBox[{"2", " ", "x3", " ", "x1"}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x2", "2"]}], "+", 
   RowBox[{"3", " ", "x2", " ", "x3"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "y1"}], "+", 
      RowBox[{"3", " ", "y2"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Expand2",
 CellLabel->"Out[566]="]
}, Open  ]]
}, Open  ]],

Cell["Explicit", "ObjectName",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Explicit"],

Cell[TextData[{
 "Explicit is an option for FieldStrength, GluonVertex, SUNF, and \
Twist2GluonOperator. If set to True the full form of the operator is \
inserted. ",
 StyleBox["Explicit[exp]",
  FontVariations->{"CompatibilityType"->0}],
 " inserts explicit expressions of GluonVertex, Twist2GluonOperator, etc., in \
exp. SUNF's are replaced by SUNTrace objects."
}], "Text",
 CellTags->"Explicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "//", "Options"}]], "Input",
 CellLabel->"In[567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[567]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellLabel->"In[568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", "g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellLabel->"Out[568]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[569]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GluonPropagator", "[", 
    RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellLabel->"In[570]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellLabel->"Out[570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
  "p", ",", "\[Mu]", ",", "a", ",", " ", "q", ",", "\[Nu]", ",", "b", ",", 
   "r", ",", "\[Rho]", ",", "c"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[571]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[572]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[573]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[574]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[575]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[575]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[576]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b1",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c16",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c16",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[576]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitLorentzIndex", "ObjectName",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell["\<\
ExplicitLorentzIndex[ind] is an explicit Lorentz index, i.e., ind is an \
integer.\
\>", "Text",
 CellTags->"ExplicitLorentzIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[577]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox["1",
     TraditionalForm], "\[Mu]"}]], TraditionalForm]], "Output",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[577]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[578]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"ExplicitLorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[578]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitPartialD", "ObjectName",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell["\<\
ExplicitPartialD[exp] inserts in exp the definition for LeftRightPartialD[z] \
(and LeftRightPartialD2[z]).\
\>", "Text",
 CellTags->"ExplicitPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 "."
}], "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[579]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[579]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[581]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"16", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "4"}]}]], "Input",\

 CellTags->"ExplicitPartialD",
 CellLabel->"In[582]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "4"]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[582]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[583]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "4"], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[583]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitSUNIndex", "ObjectName",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "ExplicitSUNIndex[ind] is a specific SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") index, i.e., ind is an integer."
}], "Text",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitSUNIndex", "[", "1", "]"}]], "Input",
 CellTags->"ExplicitSUNIndex",
 CellLabel->"In[584]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"ExplicitSUNIndex",
 CellLabel->"Out[584]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "a"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellLabel->"In[585]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellLabel->"Out[585]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExtraFactor", "ObjectName",
 CellTags->"ExtraFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraFactor"],

Cell["\<\
ExtraFactor is an option for FermionSpinSum. The setting ExtraFactor \[Rule] \
fa  multiplies the whole amplitude with the factor fa before squaring.\
\>", "Text",
 CellTags->"ExtraFactor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],

Cell["ExtraVariables", "ObjectName",
 CellTags->"ExtraVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraVariables"],

Cell["\<\
ExtraVariables is an option for OneLoopSum; it may be set to a list of \
variables which are also bracketed out in the result, just like B0, C0, D0 \
and  PaVe.\
\>", "Text",
 CellTags->"ExtraVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],

Cell["FactorFull", "ObjectName",
 CellTags->"FactorFull"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorFull"],

Cell[TextData[{
 "FactorFull is an option of Factor2 (default False). If set to False, \
products like (",
 StyleBox["a",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") (",
 StyleBox["a",
  FontSlant->"Italic"],
 "+",
 StyleBox["b",
  FontSlant->"Italic"],
 ") will be replaced by (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "2"], TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"FactorFull"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorFull"]
}, Open  ]],

Cell["Factoring", "ObjectName",
 CellTags->"Factoring"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factoring"],

Cell["\<\
Factoring is an option for Collect2, Contract, Tr and more functions. If set \
to True, the result will be factored, using Factor2. If set to any function \
f, this function will be used.\
\>", "Text",
 CellTags->"Factoring"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"Factoring"]
}, Open  ]],

Cell["Factorout", "ObjectName",
 CellTags->"Factorout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factorout"],

Cell["Factorout is an option for OPEInt.", "Text",
 CellTags->"Factorout"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInt",
  ButtonNote->"OPEInt"],
 "."
}], "Text",
 CellTags->"Factorout"]
}, Open  ]],

Cell["FactorTime", "ObjectName",
 CellTags->"FactorTime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorTime"],

Cell["\<\
FactorTime is an option for Factor2. It denotes the maximum time (in seconds) \
during which Factor2 tries to factor.\
\>", "Text",
 CellTags->"FactorTime"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorTime"]
}, Open  ]],

Cell["Factor1", "ObjectName",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor1"],

Cell["\<\
Factor1[poly] factorizes common terms  in the summands of poly. It uses \
basically PolynomialGCD.\
\>", "Text",
 CellTags->"Factor1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Factor1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[586]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[586]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor1", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[587]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[588]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[589]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor1", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[590]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[591]:="]
}, Open  ]],

Cell["Factor2", "ObjectName",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor2"],

Cell["\<\
Factor2[poly] factors a polynomial in a standard way. Factor2 works sometimes \
better than Factor on polynomials involving rationals with sums in the \
denominator. Factor2 uses Factor internally and is in general slower than \
Factor. There are four possible settings of the option Method (0,1,2,3). In \
general Factor will work faster than Factor2.\
\>", "Text",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Factor2", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[592]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FactorFull", "\[Rule]", "False"}], ",", 
    RowBox[{"Method", "\[Rule]", "3"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[592]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"Factor2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[593]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor2", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"t3", ",", 
   RowBox[{"FactorFull", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[598]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[599]:="]
}, Open  ]],

Cell["FAD", "ObjectName",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FAD"],

Cell["\<\
FAD is the FeynCalc external form of FeynAmpDenominator and denotes an \
inverse propagator. FAD[q, q-p, ...] is 1/(q^2 (q-p)^2 ...). FAD[{q1,m}, \
{q1-p,m}, q2, ...] is 1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... ). \
Translation into FeynCalc internal form is performed by FeynCalcInternal.\
\>", "Text",
 CellTags->"FAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"FAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[600]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`p - $CellContext`q)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[600]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "}"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[601]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["p", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`p^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", ($CellContext`p - $CellContext`q)^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q", ",", 
      RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[602]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[602]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[603]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], "]"}]], "Output",\

 CellTags->"FAD",
 CellLabel->"Out[603]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", "p", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"p", "-", "q"}], "]"}]}], " ", "//", " ", 
   "FeynAmpDenominatorCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[604]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], "]"}]], "Output",\

 CellTags->"FAD",
 CellLabel->"Out[604]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FC", "ObjectName",
 CellTags->"FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC"],

Cell["\<\
FC changes the output format to FeynCalcForm. To change to InputForm use FI.\
\>", "Text",
 CellTags->"FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FI",
  ButtonNote->"FI"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC"],

Cell[BoxData["FI"], "Input",
 CellTags->"FC",
 CellLabel->"In[605]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[606]:="],

Cell["{DiracGamma[5], DiracGamma[Momentum[p]]}", "Output",
 CellTags->"FC",
 CellLabel->"Out[606]="]
}, Open  ]],

Cell[BoxData["FC"], "Input",
 CellTags->"FC",
 CellLabel->"In[607]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[608]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC",
 CellLabel->"Out[608]="]
}, Open  ]]
}, Open  ]],

Cell["FCE", "ObjectName",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCE"],

Cell["\<\
FCE[exp] translates exp from the internal FeynCalc representation to a short \
form.\
\>", "Text",
 CellTags->"FCE"],

Cell["FCE is equivalent to FeynCalcExternal.", "Text",
 CellTags->"FCE"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FCE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCE", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[609]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[609]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[610]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "5", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[610]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[611]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[611]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[612]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[612]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[613]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[613]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[614]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[614]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[615]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[615]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCI", "ObjectName",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCI"],

Cell["\<\
FCI[exp] translates exp into the internal FeynCalc \
(datatype-)representation.FCI is equivalent to FeynCalcInternal.\
\>", "Text",
 CellTags->"FCI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FCI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[616]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[616]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[617]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[617]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[618]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[618]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[619]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[619]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[620]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[620]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCIntegral", "ObjectName",
 CellTags->"FCIntegral"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegral"],

Cell["\<\
FCIntegral is the head of integrals in a setting of the option IntegralTable \
of FeynAmpDenominatorSimplify. Currently implemented only for 2-loop \
integrals.\
\>", "Text",
 CellTags->"FCIntegral"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IntegralTable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IntegralTable",
  ButtonNote->"IntegralTable"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FCIntegral"]
}, Open  ]],

Cell["FCIntegrate", "ObjectName",
 CellTags->"FCIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegrate"],

Cell["\<\
FCIntegrate is an option of certain Feynman integral related functions. It \
determines which integration function is used to evaluate analytic integrals. \
Possible settings include Integrate, NIntegrate, (Dot[Integratedx@@#2, #1] \
&).\
\>", "Text",
 CellTags->"FCIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCNIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCNIntegrate",
  ButtonNote->"FCNIntegrate"],
 "."
}], "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],

Cell["FCNIntegrate", "ObjectName",
 CellTags->"FCNIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCNIntegrate"],

Cell["\<\
FCNIntegrate is an option of certain Feynman integral related functions which \
may return output containing both integrals that can be evaluated and \
integrals that can only be evaluated numerically. It then determines which \
integration function is used to evaluate numeric integrals. Possible settings \
include NIntegrate, (0*#1)&, (Dot[Integratedx@@#2, #1] &).\
\>", "Text",
 CellTags->"FCNIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCIntegrate",
  ButtonNote->"FCIntegrate"],
 "."
}], "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],

Cell["FC2RHI ", "ObjectName",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2RHI"],

Cell["\<\
FC2RHI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the RHI-integrals.  FC2RHI[exp] is equivalent to FC2RHI[exp,q1,q2]. \
The option IncludePair governs the inclusion  of scalar products p.k1, p.k2 \
and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2RHI", "]"}]], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[621]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC2RHI",
 CellLabel->"Out[621]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q1", ",", 
      RowBox[{"q1", "-", "q2"}], ",", 
      RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}], "//", 
   "FCI"}]}]], "Input",
 CellLabel->"In[622]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[622]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[623]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "0", "0", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellLabel->"Out[623]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[624]:="],

Cell["RHI[{0, 0, OPEm, 0, 0}, {0, 1, 1, 0, 1}]", "Output",
 CellLabel->"Out[624]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"FC2RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2RHI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[625]:="]
}, Open  ]],

Cell["FC2TLI ", "ObjectName",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2TLI"],

Cell["\<\
FC2TLI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the TLI-integrals.  The option IncludePair governs the inclusion  of \
scalar products p.k1, p.k2 and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2TLI", "]"}]], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[626]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC2TLI",
 CellLabel->"Out[626]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FAD"}]], "Input",
 CellLabel->"In[627]:="],

Cell[BoxData[
 StyleBox["\<\"FAD[q, q-p, ...] denotes 1/(q^2 (q-p)^2 ...).\\nFAD[{q1,m}, \
{q1-p,m}, q2, ...] is\\n1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... \
).\\n(Translation into FeynCalc internal form is performed \
by\\nFeynCalcInternal.)\"\>", "MSG"]], "Print", "PrintUsage",
 CellTags->"Info3387711966-3304885"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q1", ",", "m1"}], "}"}], ",", 
     RowBox[{"q2", "-", "q1"}], ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}]}]], "Input",
 CellLabel->"In[628]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "p"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`p + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "q1"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`q1 + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
         "\[InvisibleSpace]", 
         SuperscriptBox["m1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", $CellContext`q1^2, "-", $CellContext`m1^2, "]"],
        Editable->False], ")"}]}]],
    TraditionalForm], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}],
      TraditionalForm], ")"}], "m"]}], TraditionalForm]], "Output",
 CellLabel->"Out[628]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2TLI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[629]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   RowBox[{
    StyleBox["1",
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0, 0]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]]}], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellLabel->"Out[629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[630]:="],

Cell["TLI[{OPEm, 0, 0, 0, 0}, {{1, m1}, 0, 0, 1, 1}]", "Output",
 CellLabel->"Out[630]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TLI2FC", "[", "%", "]"}]], "Input",
 CellLabel->"In[631]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m1", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[631]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"FC2TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2TLI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[632]:="]
}, Open  ]],

Cell["FDS", "ObjectName",
 CellTags->"FDS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FDS"],

Cell["FDS is shorthand for FeynAmpDenominatorSimplify.", "Text",
 CellTags->"FDS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FDS"]
}, Open  ]],

Cell["FermionSpinSum", "ObjectName",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell["\<\
FermionSpinSum[x] constructs the Traces out of squared ampliudes.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FermionSpinSum", "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[633]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SpinPolarizationSum", "\[Rule]", "Identity"}], ",", 
    RowBox[{"SpinorCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"ExtraFactor", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[633]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["ComplexConjugate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ComplexConjugate",
  ButtonNote->"ComplexConjugate"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell[TextData[{
 "Spinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square 2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " or -2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"u", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[634]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{"m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[636]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "m"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[636]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorV", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[637]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"v", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[637]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[638]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", "m"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[638]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[639]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "m"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[639]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"k1", ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p1", ",", "m"}], "]"}]}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[640]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[640]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ct", "=", 
  RowBox[{"ComplexConjugate", "[", "t", "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["p1",
      TraditionalForm], ",", "m", ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["k1",
      TraditionalForm], ",", "m", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[641]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FermionSpinSum", "[", 
  RowBox[{"t", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"m", "+", 
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["k1",
          TraditionalForm]}]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{"m", "+", 
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p1",
          TraditionalForm]}]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[642]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[643]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox["m", "2"]}], "+", 
     RowBox[{
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[643]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[644]:="]
}, Open  ]],

Cell["FeynAmp", "ObjectName",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp[q, amp] is the head of a Feynman amplitude. amp denotes the \
analytical expression for the amplitude and q is the integration variable. \
FeynAmp[q1, q2, amp] denotes a two-loop amplitude.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp has no functional properties and serves just as a head. There are \
however special typesetting rules attached.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"FeynAmp"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
This is a 1-loop gluon self-energy amplitude (ignoring factors of (2 \[Pi])).\
\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"GV", "[", 
     RowBox[{"p", ",", "\[Mu]", ",", "a", ",", " ", 
      RowBox[{"q", "-", "p"}], ",", "\[Alpha]", ",", "c", ",", " ", 
      RowBox[{"-", "q"}], ",", "\[Beta]", ",", "e"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"p", "-", "q"}], ",", " ", "\[Alpha]", ",", "c", ",", " ", 
      "\[Rho]", ",", "d"}], "]"}], 
    RowBox[{"GV", "[", 
     RowBox[{
      RowBox[{"-", "p"}], ",", "\[Nu]", ",", "b", ",", " ", 
      RowBox[{"p", "-", "q"}], ",", "\[Rho]", ",", "d", ",", " ", "q", ",", 
      "\[Sigma]", ",", "f"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
     "q", ",", " ", "\[Beta]", ",", "e", ",", " ", "\[Sigma]", ",", "f"}], 
     "]"}]}]}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[645]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   StyleBox[
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "Global`q"}],
    ZeroWidthTimes->True], "(", 
   FormBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"c", "d"}], 
       RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
      FormBox[
       RowBox[{"p", "-", "q"}],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"e", "f"}], 
       RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox[
        RowBox[{"-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"p", "-", "q"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox["q",
        TraditionalForm]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Alpha]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox["p",
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"q", "-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"-", "q"}],
        TraditionalForm]}], ")"}], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynAmp",
 CellLabel->"Out[645]="]
}, Open  ]],

Cell["This is a generic 2-loop amplitude.", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{
   SubscriptBox["q", "1"], ",", 
   SubscriptBox["q", "2"], ",", "anyexpression"}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[646]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], 
      SubscriptBox["q", "1"]}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], 
     SubscriptBox["q", "2"]}]}], "(", "anyexpression", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmp",
 CellLabel->"Out[646]="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominator", "ObjectName",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell["\<\
FeynAmpDenominator[ PropagatorDenominator[ ... ], PropagatorDenominator[ ... \
], ...] is the head of the denominators of the propagators, i.e., \
FeynAmpDenominator[x] is the representation of 1/x .\
\>", "Text",
 CellTags->"FeynAmpDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[647]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[647]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[648]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["p", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[648]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[649]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCI"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[650]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}], ",", "m"}], "]"}], "]"}]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[650]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCE"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[651]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[651]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[652]:="]
}, Open  ]],

Cell["FeynAmpDenominatorCombine", "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell["\<\
FeynAmpDenominatorCombine[expr] expands expr with respect to \
FeynAmpDenominator and combines products of FeynAmpDenominator in expr into \
one FeynAmpDenominator.\
\>", "Text",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSplit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSplit",
  ButtonNote->"FeynAmpDenominatorSplit"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", " ", "=", " ", 
  RowBox[{
   RowBox[{"FAD", "[", "q", "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"q", "-", "p"}], "]"}]}]}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[653]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False]],
    TraditionalForm], " ", 
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[653]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[654]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}]}], "]"}]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[654]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[655]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[655]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominatorSimplify", "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["\<\
FeynAmpDenominatorSimplify[exp] tries to simplify each PropagatorDenominator \
in a canonical way. FeynAmpDenominatorSimplify[exp, q1] simplifies all \
FeynAmpDenominator's in exp in a canonical way, including some translation of \
momenta. FeynAmpDenominatorSimplify[exp, q1, q2] additionally removes \
integrals with no mass scale.\
\>", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["FDS can be used as an alias.", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData["FDS"], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[656]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominatorSimplify", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[656]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[TextData[{
 "The cornerstone of dimensional regularization is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "n"], "k", " ", 
      RowBox[{
       RowBox[{"f", "(", "k", ")"}], "/", 
       SuperscriptBox["k", 
        RowBox[{"2", "m"}]]}]}]}], "=", " ", 
    RowBox[{"0", " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[657]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[657]="]
}, Open  ]],

Cell[TextData[{
 "This brings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], ")"}]}], 
   TraditionalForm]]],
 " into a standard form."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"k", "-", 
      SubscriptBox["p", "1"]}], ",", 
     RowBox[{"k", "-", 
      SubscriptBox["p", "2"]}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[658]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"], "+", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[658]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", 
     RowBox[{"k", "-", "q"}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[659]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", 
      RowBox[{"k", "-", "q"}], ",", 
      RowBox[{"k", "-", "q"}]}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[660]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"f", "(", 
    RowBox[{"q", "-", "k"}], ")"}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"FeynAmpDenominatorSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FAD", "[", 
      RowBox[{
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"]}], ",", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "2"]}]}], "]"}], " ", 
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[661]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SubsuperscriptBox["p", "2", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r", "=", 
  RowBox[{"SPC", "[", 
   RowBox[{"t", ",", "k", ",", 
    RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[662]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SubsuperscriptBox["p", "2", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "+", 
        SubscriptBox["p", "1"], "-", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"r", ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[663]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SubsuperscriptBox["p", "2", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FDS", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", "p1"}], ",", 
      RowBox[{"k", "-", "p2"}]}], "]"}], 
    RowBox[{
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "OPEDelta"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[664]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p2",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p2",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[664]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", "r"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[665]:="]
}, Open  ]],

Cell["FeynAmpDenominatorSplit", "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell["\<\
FeynAmpDenominatorSplit[expr] splits all FeynAmpDenominator[a,b, ...] in expr \
into FeynAmpDenominator[a]*FeynAmpDenominator[b] ... . \
FeynAmpDenominatorSplit[expr, q1] splits all FeynAmpDenominator in expr into \
a product of two, one containing q1 and other momenta, the second without q1.\
\
\>", "Text",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorCombine",
  ButtonNote->"FeynAmpDenominatorCombine"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], "//", "FCI"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[666]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[666]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Head"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[667]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominator", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSplit", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[668]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], " ", 
    SuperscriptBox["q2", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[669]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q1", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", "-", "q2"}], "]"}], " ", 
  RowBox[{"FAD", "[", "q2", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[669]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", 
    RowBox[{"t", ",", "q1"}], "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[670]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{"q2", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q2"}]}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q1"}], ",", 
    RowBox[{"q1", "-", "q2"}]}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[670]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[671]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", "q2", ",", 
   RowBox[{"q1", "-", "q2"}], ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}]}], "]"}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[671]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[672]:="]
}, Open  ]],

Cell["FeynAmpList", "ObjectName",
 CellTags->"FeynAmpList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList[info][FeynAmp[...], FeynAmp[...], ...] is a head of a list of \
Feynman amplitudes.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList has no functional properties and serves just as a head.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],

Cell["FeynCalc", "ObjectName",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc"],

Cell["FeynCalc is simply a symbol with a usage definition.", "Text",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FeynCalc"}]], "Input",
 CellTags->"FeynCalc",
 CellLabel->"In[673]:="],

Cell[BoxData[
 StyleBox["\<\"For installation notes visit www.feyncalc.org\\n\\nFor a list \
of availabe objects type $FeynCalcStuff, which contains a list of all \
functions and options in StringForm. You can get on-line information by \
?function, e.g. ?Contract.\\n\\nThere are several useful functions for short \
input, type $FCS for a list of short commands. Then type, e.g., \
?GA.\\n\\n\\nTo enable/disable start-up messages, put the \
line\\n\\n$FeynCalcStartupMessages = \
True;\\n\\nor\\n\\n$FeynCalcStartupMessages = False;\\n\\n into your \
\\\"init.m\\\" file or into your \\\"FCConfig.m\\\" file.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"FeynCalc", "Info3387711973-8270308"}]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcExternal", "ObjectName",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell["\<\
FeynCalcExternal[exp] translates exp from the internal FeynCalc \
representation to a shorthand form.\
\>", "Text",
 CellTags->"FeynCalcExternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcExternal", "[", 
  RowBox[{"DiracGamma", "[", "5", "]"}], "]"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[674]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[674]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[675]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "5", "]"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[675]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[676]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[676]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[677]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[677]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[678]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[678]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[679]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[679]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[680]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[680]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcForm", "ObjectName",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "FeynCalcForm[expr] changes the printed output to a an easy-to-read form. It \
allows a readable output also when running a terminal based ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session. Whether the result of FeynCalcForm[expr] is displayed or not, \
depends on the setting of $PrePrint. $PrePrint = FeynCalcForm forces \
displaying everything after applying FeynCalcForm. In order to change to the \
normal (internal) Mathematica OutputForm, do: ($PrePrint=.)."
}], "Text",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell["This is the normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[681]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcForm",
 CellLabel->"Out[681]="]
}, Open  ]],

Cell["\<\
This is the shorthand (terminal) display (easy-to-read form):\
\>", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", " ", "=", " ", "FeynCalcForm"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[682]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "OutputForm"}], ")"}]}]}], 
       ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[683]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[684]:="],

Cell[OutputFormData["\<\
SUNTrace[SUNT[SUNIndex[a]] . SUNT[SUNIndex[b]] . SUNT[SUNIndex[c]]]\
\>", "\<\
tr[T[a] T[b] T[c]]\
\>"], "Output",
 CellTags->"FeynCalcForm",
 CellLabel->"Out[684]="]
}, Open  ]],

Cell["Reset to normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", "=."}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[685]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "TraditionalForm"}], ")"}]}]}],
        ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[686]:="]
}, Open  ]],

Cell["FeynCalcInternal", "ObjectName",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell["\<\
FeynCalcInternal[exp] translates exp into the internal FeynCalc (abstract \
data-type) representation.\
\>", "Text",
 CellTags->"FeynCalcInternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[687]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[687]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[688]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[688]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[689]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[689]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[690]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[690]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[691]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[691]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], ",", 
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"FAD", "[", "q", "]"}], ",", 
    RowBox[{"LC", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}]}],
    "}"}], "]"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[692]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    FractionBox["1", 
     SuperscriptBox["q", "2"]], ",", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[692]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[693]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], ",", 
   RowBox[{"SUND", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"SUNF", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"Eps", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]}], 
  "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[693]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalc2FORM", "ObjectName",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell["\<\
FeynCalc2FORM[expr] displays expr in FORM syntax. FeynCalc2FORM[file, x] \
writes x in FORM syntax to a file. FeynCalc2FORM[file, x==y] writes x=y to a \
file in FORM syntax.\
\>", "Text",
 CellTags->"FeynCalc2FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynCalc2FORM", "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[694]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsDiscard", "\[Rule]", "False"}], ",", 
    RowBox[{"FORMEpilog", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"FORMProlog", "\[Rule]", "\<\"write statistics;\"\>"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\<\"\\\\[Alpha]\"\>", "\[Rule]", "\<\"al\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Beta]\"\>", "\[Rule]", "\<\"be\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Gamma]\"\>", "\[Rule]", "\<\"ga\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Delta]\"\>", "\[Rule]", "\<\"de\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Mu]\"\>", "\[Rule]", "\<\"mu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Nu]\"\>", "\[Rule]", "\<\"nu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Rho]\"\>", "\[Rule]", "\<\"ro\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Sigma]\"\>", "\[Rule]", "\<\"si\"\>"}]}], "}"}]}], 
    ",", 
    RowBox[{"TraceDimension", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[694]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Rho]"}], "]"}], " ", 
  RowBox[{
   RowBox[{"y", "^", "2"}], "/", "d"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[695]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["y", "2"], " ", 
    SuperscriptBox["p", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "d"], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[695]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[696]:="],

Cell["(y^2*d_(mu,nu)*p(ro))/d", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"LC", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]", ",", "\[Rho]"}], 
   "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[697]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Delta]", "\[Rho]"}]], 
  TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[697]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[698]:="],

Cell["(-i_)*e_(al,be,de,ro)", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[699]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[699]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[700]:="],

Cell["g_(0,mu)*g_(0,nu)*g_(0,ro)*g_(0,si)", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], 
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Rho]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[701]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Rho]"]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[701]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[702]:="],

Cell["g_(0,mu)*g_(0,nu)*g_(1,mu)*g_(1,ro)", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
     RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[703]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReadList", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", "String"}],
   "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[704]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "\<\"d_(mu,nu)*p(mu)\"\>", "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[704]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"TR", "[", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
    ".", 
    RowBox[{"GS", "[", 
     RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[705]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[705]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{"L", " ", "==", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[706]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[707]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"Indices \\\\[Mu],\\\\[Nu],\\\\[Rho],\\\\[Sigma];\"\>"},
      {"\<\"Vectors p,q;\"\>"},
      {"\<\"write statistics;\"\>"},
      {"\<\"Local L = ( \"\>"},
      {"\<\"4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)*d_(\
ro,si)*q.p+\"\>"},
      {"\<\"d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*q(mu)-\
\"\>"},
      {"\<\"d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*q(nu)+\
\"\>"},
      {"\<\"d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*q(ro)-\
\"\>"},
      {"\<\"d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*q(si))\
 ); \"\>"},
      {"\<\"   \"\>"},
      {"\<\"Print; \"\>"},
      {"\<\".end\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[707]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FileNames", "[", "\"\<fc2ftest.f\>\"", "]"}], "=!=", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DeleteFile", "[", "\"\<fc2ftest.f\>\"", "]"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[708]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[709]:="]
}, Open  ]],

Cell["FeynCalcToLaTeX", "ObjectName",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],

Cell["\<\
FeynCalcToLaTeX[expr] generates LaTeX with line-breaking  for expr. 
FeynCalcToLaTeX[expr, 500] generates LaTeX for expr where 500 is the Window \
width  setting for the Mathematica frontend. Increasing its value will \
generate less line breaks.\
\>", "Text",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellLabel->"In[710]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"li1", "li2"}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[710]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcToLaTeX", "[", "%", "]"}]], "Input",
 CellLabel->"In[711]:="],

Cell[BoxData[
 FormBox["\<\"%% AMS-LaTeX Created by Wolfram Mathematica \
6.0\\\\documentclass{article}\\n\\\\usepackage{amsmath, amssymb, \
graphics}\\\\newcommand{\\\\mathsym}[1]{{}}\\n\\\\newcommand{\\\\unicode}{{}}\
\\\\begin{document}\\\\noindent\\\\(-\\\\frac{i \
g^{\\\\text{li1}\\\\text{li2}} \\\\delta \
_{\\\\text{ci1}\\\\text{ci2}}}{p^2}\\\\)\"\>", TraditionalForm]], "Output",
 CellLabel->"Out[711]="]
}, Open  ]]
}, Open  ]],

Cell["FeynmanParameterNames", "ObjectName",
 CellTags->"FeynmanParameterNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParameterNames"],

Cell["\<\
FeynmanParameterNames is an option for FeynmanParametrize and \
FeynmanParametrize.\
\>", "Text",
 CellTags->"FeynmanParameterNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 "."
}], "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],

Cell["FeynmanParametrize ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell["\<\
FeynmanParametrize[exp,k] introduces feynman parameters for all one-loop \
integrals in exp (k = integration momentum).\
\>", "Text",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize", "]"}]], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[712]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynmanParameterNames", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanParametrize",
 CellLabel->"Out[712]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[713]:="]
}, Open  ]],

Cell["FeynRule", "ObjectName",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynRule"],

Cell["\<\
FeynRule[lag, {fields}] derives the Feynman rule corresponding to the field \
configuration fields of the lagrangian lag.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynRule", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[714]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Anti5", "\[Rule]", 
     RowBox[{"-", "\[Infinity]"}]}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factor1", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}], ",", 
    RowBox[{"InitialFunction", "\[Rule]", "Identity"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[714]="]
}, Open  ]],

Cell["FeynRule does not calculate propagator Feynman rules.", "Text",
 CellTags->"FeynRule"],

Cell["\<\
The option ZeroMomentumInsertion can be used for twist-2 and higher twist \
operators.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Lagrangian",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lagrangian",
  ButtonNote->"Lagrangian"],
 "."
}], "Text",
 CellTags->"FeynRule"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gou", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[715]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[715]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gop", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[716]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "gop", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[717]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Gamma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Gamma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Beta]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b2",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Gamma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c17",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b2",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c17",
           TraditionalForm],
          TraditionalForm]}]]}]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Alpha]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox["\[CapitalDelta]",
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Alpha]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Alpha]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b3",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["c18",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b3",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c18",
           TraditionalForm],
          TraditionalForm]}]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[717]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{"%", ",", "QuantumField"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b3",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b2",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c17",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["c18",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[718]="]
}, Open  ]],

Cell["2-gluon Feynman rules (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[719]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[719]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2u", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gou", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[720]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], "m"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], "m"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}],
      TraditionalForm], "2"], " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}],
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[720]="]
}, Open  ]],

Cell["2-gluon Feynman rules (polarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[721]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[721]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2p", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gop", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[722]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]], " ", 
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}]}], ")"}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]], " ", 
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}]}], ")"}]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[722]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Calc", "[", 
   RowBox[{"f2p", "/.", 
    RowBox[{"p", "\[Rule]", 
     RowBox[{"-", "q"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[723]="]
}, Open  ]],

Cell["\<\
Compare with the Feynman rule tabulated in Twist2GluonOperator.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"q", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[724]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[724]="]
}, Open  ]],

Cell["quark-quark Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[725]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[725]="]
}, Open  ]],

Cell["quark-quark -gluon-gluon Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[726]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[726]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qggf", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", "QuarkField", "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}], "[", "q", "]"}], 
    ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[727]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FormBox["\[Psi]",
      TraditionalForm], "(", "p", ")"}], ",", 
    RowBox[{
     FormBox[
      OverscriptBox["\[Psi]", "_"],
      TraditionalForm], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[727]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"n4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{"qo", ",", "qggf", ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[728]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[728]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[729]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["s",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[729]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"n4", "-", "t4"}], "/.", 
     RowBox[{"OPEm", "\[Rule]", "5"}]}], "/.", 
    RowBox[{"s", "\[Rule]", 
     RowBox[{
      RowBox[{"-", "p"}], "-", "q", "-", "r"}]}]}], "/.", 
   RowBox[{"D", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[730]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[730]="]
}, Open  ]],

Cell["\<\
 In general equality can be shown by Timing[Factor2[ \
FCE[Calc[ChangeDimension[FCE[OPESumExplicit[n4-t4]],4]/.s\[Rule]-p-q-r]]]] \
but it is a little bit slow ...\
\>", "Text",
 CellTags->"FeynRule"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"qggf", ",", "n2", ",", "n4"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[731]:="],

Cell["QCD vertices", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fii", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[732]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[732]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g3", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fii"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[733]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[733]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[734]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "r"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"r", "-", "p"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[734]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g3", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[735]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[735]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi4", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Sigma]", "}"}], ",", 
       RowBox[{"{", "d", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[736]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[736]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fi4"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[737]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[737]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[738]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u8",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[738]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g4", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[739]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[739]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "f2p", ",", "f2u", ",", "f3", ",", "f32", ",", "fi", ",", "fi4", ",", "fii",
    ",", "g3", ",", "g4", ",", "gop", ",", "gou", ",", "n3", ",", "nf3", ",", 
   "n4", ",", "np2", ",", "npf3", ",", "p33", ",", "pf3", ",", "pn3", ",", 
   "pqo", ",", "qf", ",", "qp", ",", "qgf", ",", "qo", ",", "t4"}], 
  "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[740]:="]
}, Open  ]],

Cell["FI", "ObjectName",
 CellTags->"FI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FI"],

Cell["\<\
FI changes the output format to InputForm. This is useful to see the internal \
representation of FeynCalc objects. To change back to FeynCalcForm use FC.\
\>", "Text",
 CellTags->"FI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FI"]
}, Open  ]],

Cell["FieldDerivative", "ObjectName",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell["\<\
FieldDerivative[f[x],x,li1,li2,...] is the derivative of f[x] with respect to \
space-time variables x and with Lorentz indices li1, li2, ..., where li1, \
li2, ... have head LorentzIndex. FieldDerivative[f[x],x,li1,li2,...] can be \
given as FieldDerivative[f[x],x,{l1,l2,...}], where l1 is li1 without the \
head, ... FieldDerivative is defined only for objects with head \
QuantumField[...]. If the space-time derivative of other objects is wanted, \
the corresponding rule must be specified.\
\>", "Text",
 CellTags->"FieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"B", ",", 
     RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "[", "y", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"C", ",", 
     RowBox[{"{", "\[Rho]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"D", ",", 
     RowBox[{"{", "\[Sigma]", "}"}]}], "]"}], "[", "y", "]"}]}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[741]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["D",
      TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[741]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "x", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[742]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[742]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "y", ",", 
    RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["D",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["B", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["D",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["B", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[743]="]
}, Open  ]]
}, Open  ]],

Cell["FieldStrength", "ObjectName",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[TextData[{
 "FieldStrength[\[Mu], \[Nu], a] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "a"]}], "-", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Nu]"], 
     SubsuperscriptBox["A", "\[Mu]", "a"]}], "+", 
    RowBox[{
     SubscriptBox["g", "s"], " ", 
     SubsuperscriptBox["A", "\[Mu]", "b1"], 
     SubsuperscriptBox["A", "\[Nu]", "c1"]}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["f", 
     RowBox[{"a", " ", "b1", " ", "c1"}]]}], TraditionalForm]]],
 ". FieldStrength[\[Mu], \[Nu]] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "\[Null]"]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "\[Nu]"], 
      SubsuperscriptBox["A", "\[Mu]", "\[Null]"]}], "."}]}], 
   TraditionalForm]]],
 " The name of the field (A) and the coupling constant (g) can be set through \
the options or by additional arguments. The first two indices are interpreted \
as type LorentzIndex, except OPEDelta, which is converted to \
Momentum[OPEDelta]."
}], "Text",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FieldStrength", "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[744]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`fctools`IndexPosition`IndexPosition", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], ",", 
    RowBox[{"Symbol", "\[Rule]", "\<\"F\"\>"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[744]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FieldStrength"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[745]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "\[Null]"], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[745]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[746]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[746]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[747]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[747]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[748]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b12",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c60",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b12", "c60"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[748]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[749]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[749]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[750]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"Momentum", "[", "OPEDelta", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[750]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"CouplingConstant", "\[Rule]", 
    RowBox[{"-", "Gstrong"}]}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[751]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b13",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c61",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b13", "c61"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[751]="]
}, Open  ]]
}, Open  ]],

Cell["FinalFunction", "ObjectName",
 CellTags->"FinalFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalFunction"],

Cell["FinalFunction is an option for OneLoopSum.", "Text",
 CellTags->"FinalFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"FinalFunction"]
}, Open  ]],

Cell["FinalSubstitutions", "ObjectName",
 CellTags->"FinalSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalSubstitutions"],

Cell["\<\
FinalSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"FinalSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],

Cell["FORM", "ObjectName",
 CellTags->"FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM"],

Cell["\<\
FORM is an option for RHI. If set to True a FORM file is generated and run \
from Mathematica (provided R. Hamberg's FORM-program is installed correctly \
... ).\
\>", "Text",
 CellTags->"FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 ", ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM"]
}, Open  ]],

Cell["FORMEpilog", "ObjectName",
 CellTags->"FORMEpilog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMEpilog"],

Cell["\<\
FORMEpilog is an option for FeynCalc2FORM. It may be set to a string which is \
put at the end of the FORM-file.\
\>", "Text",
 CellTags->"FORMEpilog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMProlog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORMProlog",
  ButtonNote->"FORMProlog"],
 "."
}], "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],

Cell["FORMProlog", "ObjectName",
 CellTags->"FORMProlog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMProlog"],

Cell["\<\
FORMProlog is an option for FeynCalc2FORM. It may be set to a string which is \
put after the type declarations of the FORM-file.\
\>", "Text",
 CellTags->"FORMProlog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMEpilog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORMEpilog",
  ButtonNote->"FORMEpilog"],
 "."
}], "Text",
 CellTags->"FORMProlog"]
}, Open  ]],

Cell["FORM2FeynCalc", "ObjectName",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell["\<\
FORM2FeynCalc[expr] translates the FORM expr into FeynCalc notation. \
FORM2FeynCalc[file] translates the FORM expresssions in file into \
FeynCalcnotation. FORM2FeynCalc[file, x1, x2, ...] reads in a file in \
FORM-format and translates the assignments for the variables a, b, ... into \
FeynCalc syntax. If the option Set is True, the variables x1, x2 are assigned \
to the right hand sides defined in the FORM-file.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FORM2FeynCalc", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Dot", "\[Rule]", "Dot"}], ",", 
    RowBox[{"HoldForm", "\[Rule]", "True"}], ",", 
    RowBox[{"LorentzIndex", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"mu", ",", "nu", ",", "al", ",", "be"}], "}"}]}], ",", 
    RowBox[{"Set", "\[Rule]", "False"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Vectors", "\[Rule]", "Automatic"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[752]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p.q + 2*x m^2\>\"", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"x", ".", 
     SuperscriptBox["m", "2"]}]}], "+", 
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[753]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[754]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{"x", ".", 
    SuperscriptBox["m", "2"]}]}], "+", 
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[754]//StandardForm="]
}, Open  ]],

Cell["\<\
Functions are automatically converted right, but bracketed expressions need \
to be substituted explicitly.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<x +f(z)+ log(x)^2+[li2(1-x)]\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<[li2(1-x)]\>\"", "\[Rule]", "\"\<PolyLog[2,1-x]\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[755]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}], "+", "x", "+", 
   RowBox[{"f", "(", "z", ")"}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[755]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[756]:="],

Cell[BoxData[
 RowBox[{"x", "+", 
  RowBox[{"f", "[", "z", "]"}], "+", 
  SuperscriptBox[
   RowBox[{"Log", "[", "x", "]"}], "2"], "+", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[756]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<x + [(1)]*y -[(-1)^m]\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[757]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{
    RowBox[{"Hold", "[", "1", "]"}], ".", "y"}], "-", 
   RowBox[{"Hold", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[757]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[758]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   RowBox[{"1", ".", "y"}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[758]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[759]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["p", "mu"], ".", 
    SuperscriptBox["q", "nu"]}], "+", 
   SuperscriptBox["g", 
    RowBox[{"mu", "nu"}]]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[760]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "mu"}], "]"}], ".", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "nu"}], "]"}]}], "+", 
  RowBox[{"MT", "[", 
   RowBox[{"mu", ",", "nu"}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[760]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mu", "\[Rule]", "\[Mu]"}], ",", 
      RowBox[{"nu", "\[Rule]", "\[Nu]"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[761]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], ".", 
    SuperscriptBox["q", "\[Nu]"]}], "+", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[761]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  "\"\<i_*az*bz*aM^2*D1*[(1)]*b_G1 * ( 4*eperp(mu,nu)*avec.bvec*blam )\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[762]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"4", " ", "\[ImaginaryI]"}], ")"}], ".", "az", ".", "bz", ".", 
   SuperscriptBox["aM", "2"], ".", "D1", ".", 
   RowBox[{"Hold", "[", "1", "]"}], ".", "b$G1", ".", 
   RowBox[{"eperp", "(", 
    RowBox[{"mu", ",", "nu"}], ")"}], ".", 
   RowBox[{"(", 
    FormBox[
     RowBox[{
      FormBox["avec",
       TraditionalForm], "\[CenterDot]", 
      FormBox["bvec",
       TraditionalForm]}],
     TraditionalForm], ")"}], ".", "blam"}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[762]="]
}, Open  ]]
}, Open  ]],

Cell["FourDivergence", "ObjectName",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourDivergence"],

Cell["\<\
FourDivergence[exp, FourVector[p, mu]] calculates the partial derivative of \
exp w.r.t. p(mu). FourDivergence[exp, FourVector[p, mu], FourVector[p,nu], \
...]gives the multiple derivative.\
\>", "Text",
 CellTags->"FourDivergence"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourDivergence"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[763]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[763]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[764]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[764]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "-", "k"}], ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[765]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "k"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[765]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[766]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[766]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[767]:="]
}, Open  ]],

Cell["FourLaplacian", "ObjectName",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[TextData[{
 "FourLaplacian[exp, p, q] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["p", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 "exp."
}], "Text",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FourLaplacian", "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[768]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[768]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[769]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[769]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[770]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "D"}], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SOD", "[", "q", "]"}], "^", "OPEm"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"q", "-", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[771]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", " ", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[772]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"4", " ", "m", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "1"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], "-", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}]}], ")"}]}], 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], "-", 
   FractionBox[
    RowBox[{"2", " ", "D", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "m", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", "D", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellTags->"FourLaplacian",
 CellLabel->"Out[772]="]
}, Open  ]]
}, Open  ]],

Cell["FourVector", "ObjectName",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourVector"],

Cell[TextData[{
 "FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector p with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". A vector with space-time Dimension D is obtained by supplying the option \
Dimension \[Rule] D."
}], "Text",
 CellTags->"FourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[773]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[773]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[774]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[774]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[775]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[775]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[776]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[776]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[777]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[777]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FourVector. Since \
FourVector is turned into Pair internally ExpandScalarProduct may be used.\
\>", "Text",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[778]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[778]="]
}, Open  ]]
}, Open  ]],

Cell["FreeIndex", "ObjectName",
 CellTags->"FreeIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeIndex"],

Cell["\<\
FreeIndex is a datatype which is recognized by Contract. Possible use: \
DataType[mu, FreeIndex] = True.\
\>", "Text",
 CellTags->"FreeIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"FreeIndex"]
}, Open  ]],

Cell["FreeQ2", "ObjectName",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeQ2"],

Cell["\<\
FreeQ2[expr, {form1, form2, ...}] yields True if expr does not contain any \
occurence of form1, form2, ... and False otherwise. FreeQ2[expr, form] is the \
same as FreeQ[expr, form].\
\>", "Text",
 CellTags->"FreeQ2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectFree",
  ButtonNote->"SelectFree"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"FreeQ2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"a", ",", "x"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[779]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[779]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[780]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[780]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{"x", ",", " ", "y"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[781]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[781]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", " ", "f"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[782]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[782]="]
}, Open  ]]
}, Open  ]],

Cell["FRH", "ObjectName",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FRH"],

Cell["\<\
FRH[exp_] := FixedPoint[ReleaseHold, exp], i.e., FRH removes all HoldForm and \
Hold in exp.\
\>", "Text",
 CellTags->"FRH"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"FRH"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "-", "1", " ", "-", " ", 
   RowBox[{"Hold", "[", 
    RowBox[{"2", "-", "2"}], "]"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[783]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Hold", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Hold", "[", 
      RowBox[{"2", "-", "2"}], "]"}]}], "+", "1", "-", "1"}], "]"}], 
  TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[783]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[784]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", "3"}], "-", "x", "-", "1"}], "==", "0"}], "]"}], ",", 
   "x", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "HH"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[785]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "3", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "6", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "7", ")"}],
       HoldForm]}], "}"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", 
  RowBox[{"HH", "[", "3", "]"}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[786]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RadicalBox[
     RowBox[{
      FractionBox["27", "2"], "-", 
      FractionBox[
       RowBox[{"3", " ", 
        SqrtBox["69"]}], "2"]}], "3"]}], "+", 
   FractionBox[
    RadicalBox[
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"9", "+", 
        SqrtBox["69"]}], ")"}]}], "3"], 
    SuperscriptBox["3", 
     RowBox[{"2", "/", "3"}]]]}], TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[786]="]
}, Open  ]]
}, Open  ]],

Cell["FromTFI", "ObjectName",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FromTFi"],

Cell["\<\
FromTFi[expr, q1, q2, p] translates the TFi notatation from the TARCER \
package to the usual FeynCalc notation. See TFi for details on the \
conventions.\
\>", "Text",
 CellTags->"FromTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TFI",
  ButtonNote->"TFI"],
 ", ",
 ButtonBox["ToTFI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToTFI",
  ButtonNote->"ToTFI"],
 "."
}], "Text",
 CellTags->"FromTFI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFI", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[787]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   SubsuperscriptBox[
    StyleBox["F",
     SingleLetterItalics->False,
     FontWeight->"Bold"], 
    RowBox[{
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "M"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "m"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}],
      TraditionalForm]}], 
    RowBox[{"(", 
     FormBox["D",
      TraditionalForm], ")"}]],
   HighEnergyPhysics`Tarcer`TFI[
   D, $CellContext`m^2, {{1, 0}, {1, $CellContext`M}, {1, 0}, {
     1, $CellContext`m}, {1, 0}}],
   Editable->True], TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[787]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[788]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q2", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[788]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[789]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["\[CapitalDelta]",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q2",
     TraditionalForm]}], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[789]="]
}, Open  ]]
}, Open  ]],

Cell["FUNCTION", "ObjectName",
 CellTags->"FUNCTION"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FUNCTION"],

Cell["\<\
FUNCTION[exp, string] is a head of an expression to be declared a function \
(of type string), if used in Write2.\
\>", "Text",
 CellTags->"FUNCTION"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FUNCTION"]
}, Open  ]],

Cell["FunctionalD", "ObjectName",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FunctionalD"],

Cell["\<\
FunctionalD[expr, {QuantumField[name, {mu}, {a}][p], ...}] calculates the \
functional derivative of expr with respect to the field list (with incoming \
momenta p, etc.) and does the fourier transform. FunctionalD[expr, \
{QuantumField[name, {mu},{a}], ...}] calculates the functional derivative and \
does partial integration but omits the x-space delta functions.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["FunctionalD is a low level function used in FeynRule.", "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FunctionalD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]\[Phi]", "(", "x", ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SuperscriptBox["\[Delta]", 
      RowBox[{"(", "D", ")"}]], "(", 
     RowBox[{"x", "-", "y"}], ")"}]}], TraditionalForm]]],
 " the arguments and the \[Delta] function are omitted, i.e., for the program \
for simplicity: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Delta]\[Phi]", "/", "\[Delta]\[Phi]"}], "=", "1"}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[790]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[790]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", "\[Phi]", "]"}], "^", "2"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[791]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[Phi]"}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[791]="]
}, Open  ]],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Delta]", " ", 
       RowBox[{
        SubscriptBox["\[PartialD]", "\[Mu]"], 
        RowBox[{"\[Phi]", "(", "x", ")"}]}]}], ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     RowBox[{
      SuperscriptBox["\[Delta]", 
       RowBox[{"(", "D", ")"}]], "(", 
      RowBox[{"x", "-", "y"}], ")"}]}]}], TraditionalForm]]],
 " the arguments are omitted, and the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 "operator is specified by default to be an integration by parts operator, \
i.e., the right hand side will be just ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     FormBox[
      SubscriptBox["\[PartialD]", "\[Mu]"],
      TraditionalForm]}], ","}], TraditionalForm]]],
 " or, more precisely (by default) ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 "."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[792]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[792]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "\[Phi]", "]"}], " ", "=", 
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"x", " ", "[", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}], " ", 
         RowBox[{
          SuperscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}]}], "-", 
        RowBox[{
         SuperscriptBox["m", "2"], 
         RowBox[{"\[Phi]", "(", "x", ")"}], " ", 
         RowBox[{"\[Phi]", "(", "y", ")"}]}]}], "]"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"s", "[", "\[Phi]", "]"}], "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ".", 
      
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}]}], 
     "-", 
     RowBox[{
      RowBox[{"m", "^", "2"}], " ", 
      RowBox[{
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ".", 
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}]}]}], ")"}], "/", 
   "2"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[793]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}], ".", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], "-", 
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"\[Phi]", ".", "\[Phi]"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[793]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"s", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[794]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[Phi]"}], " ", 
    SuperscriptBox["m", "2"]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[794]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "A", "]"}], " ", "=", " ", 
   RowBox[{"-", 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], "x", " ", 
      FractionBox["1", "4"], " ", 
      RowBox[{
       SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], "(", "x", ")"}], " ", 
      RowBox[{
       SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["First approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F1", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "b", ",", "c"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[795]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[795]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F2", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "d", ",", "e"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[796]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "d", "e"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[796]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"S", "[", "A", "]"}], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"1", "/", "4"}]}], 
   RowBox[{"F1", ".", "F2"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[797]="]
}, Open  ]],

Cell[TextData[{
 "In order to derive the equation of motion the functional derivative of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with respect to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[Sigma]", "g"], TraditionalForm]]],
 "has to be set to zero. Bearing in mind that for FeynCalc we have to be \
precise as to where which operators (coming from the substitution of the \
derivative of the \[Delta] function) act:, act with the functional derivative \
operator on the first field strength:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       FormBox[
        RowBox[{
         FormBox[
          RowBox[{"0", " ", "=", " ", 
           RowBox[{
            FormBox[
             RowBox[{
              RowBox[{"(", "\[Delta]S", ")"}], "/", 
              RowBox[{"(", 
               RowBox[{"\[Delta]", 
                RowBox[{
                 SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
               ")"}]}],
             TraditionalForm], "=", 
            RowBox[{"-", 
             RowBox[{"2", "/", "4"}]}]}]}],
          TraditionalForm], 
         RowBox[{"\[Integral]", 
          RowBox[{
           SuperscriptBox["d", "D"], "x"}]}]}],
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{"\[Delta]", "/", 
         RowBox[{"(", 
          RowBox[{"\[Delta]", 
           RowBox[{
            SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
          ")"}]}]}]}],
      TraditionalForm], " ", 
     RowBox[{
      SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}], ")"}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      SubsuperscriptBox["F", "a", 
       RowBox[{",", "\[Mu]\[Nu]"}]]}],
     TraditionalForm], "(", "x", ")"}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"See", " ", "what", " ", "happens", " ", "with", " ", "just", 
   RowBox[{
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        RowBox[{"S", "[", "A", "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        SubsuperscriptBox["A", "\[Sigma]", "g"]}], ")"}]}],
     TraditionalForm], "."}]}], TraditionalForm]]]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ag", "=", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Sigma]", "}"}], ",", 
    RowBox[{"{", "g", "}"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[798]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox[
    FormBox["\[Sigma]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["g",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[798]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{"F1", ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[799]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["g", 
      RowBox[{"\[Nu]", "\[Sigma]"}]]}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[799]="]
}, Open  ]],

Cell["\<\
In order to minimize the number of dummy indices, replace b \[Rule] c.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"t1", " ", "/.", " ", 
   RowBox[{"b", "\[Rule]", "c"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[800]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["g", 
      RowBox[{"\[Nu]", "\[Sigma]"}]]}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[800]="]
}, Open  ]],

Cell[TextData[{
 "Instead of inserting the definition for the second ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], TraditionalForm]]],
 ", introduce a QuantumField object with antisymmetry built into the Lorentz \
indices:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{"F", "/:", " ", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
   "pard___", ",", "F", ",", "\[Beta]_", ",", "\[Alpha]_", ",", "s_"}], "]"}],
   ":=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"pard", ",", "F", ",", "\[Alpha]", ",", "\[Beta]", ",", "s"}], 
     "]"}]}], "/;", 
   RowBox[{"!", 
    RowBox[{"OrderedQ", "[", 
     RowBox[{"{", 
      RowBox[{"\[Beta]", ",", "\[Alpha]"}], "}"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[801]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"F", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[802]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]}], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[802]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\[Mu]", "\[RuleDelayed]", "\[Nu]"}], ",", 
    RowBox[{"\[Nu]", "\[RuleDelayed]", "\[Mu]"}]}], "}"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[803]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["F", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]}], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[803]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"Contract", "[", 
    RowBox[{"ExpandPartialD", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"1", "/", "2"}]}], " ", 
      RowBox[{"t1", ".", 
       RowBox[{"QuantumField", "[", 
        RowBox[{"F", ",", 
         RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
         RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
         RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]}]}], "]"}], "]"}], "/.",
    
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[804]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["F", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["F", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[804]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", " ", 
  RowBox[{"t2", " ", "/.", " ", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"FCE", "[", "t3", "]"}], "/.", " ", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "c", ",", "g"}], "]"}], "\[Rule]", 
    RowBox[{"-", 
     RowBox[{"SUNF", "[", 
      RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[806]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[806]="]
}, Open  ]],

Cell[TextData[{
 "Since the variational derivative vanishes t4 implies that 0 = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["D", "\[Mu]"], 
    SubsuperscriptBox["F", "g", "\[Mu]\[Sigma]"]}], TraditionalForm]]],
 " ."
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["Second approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell["\<\
It is of course also possible to do the functional deriviate on the S[A] with \
both field strength tensors inserted.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "[", "A", "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[807]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[807]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{
    RowBox[{"S", "[", "A", "]"}], ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[808]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]]}]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[808]="]
}, Open  ]],

Cell["\<\
This is just funcional derivation and partial integration and simple \
contraction of indices. No attempt is made to rename dummy indices (since \
this is difficult in general).\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["\<\
With a general replacement rule only valid for commuting fields the color \
indices can be canonicalized a bit more. The idea is to use the commutative \
properties of the vector fields, and canonicalize the color indices by a \
trick. This function will work on any commuting product of fields.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "symfun", "]"}], ";"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[809]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"symfun", "[", 
   RowBox[{"z_", ",", "fieldname_Symbol"}], "]"}], ":=", 
  RowBox[{"Expand", "[", 
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"FixedPoint", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Collect2", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"DotSimplify", "[", 
            RowBox[{"#1", "/.", 
             RowBox[{"Times", "\[Rule]", "Dot"}]}], "]"}], "/.", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", "qi___", ")"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par1___", ",", "fieldname", ",", "li1_", ",", "sui1_"}], 
                 "]"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par2___", ",", "fieldname", ",", "li2_", ",", "sui2_"}], 
                 "]"}], ".", "qf___"}], " ", ")"}], "any_"}], ")"}], 
            "\[RuleDelayed]", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"qi", ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par1", ",", "A", ",", "li1", ",", "sui2"}], "]"}], 
                 ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par2", ",", "fieldname", ",", "li2", ",", "sui1"}],
                   "]"}], ".", "qf"}], " ", ")"}], 
               RowBox[{"(", 
                RowBox[{"any", "/.", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"sui1", "\[RuleDelayed]", "sui2"}], ",", 
                   RowBox[{"sui2", "\[RuleDelayed]", "sui1"}]}], "}"}]}], 
                ")"}]}], ")"}], "/;", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"!", 
                RowBox[{"(", 
                 RowBox[{"FreeQ2", "[", 
                  RowBox[{"any", ",", 
                   RowBox[{"{", 
                    RowBox[{"sui1", ",", "su2"}], "}"}]}], "]"}], ")"}]}], 
               ")"}], "&&", 
              RowBox[{"!", 
               RowBox[{"(", 
                RowBox[{"OrderedQ", "[", 
                 RowBox[{"{", 
                  RowBox[{"sui1", ",", "sui2"}], "}"}], "]"}], 
                ")"}]}]}]}]}]}], "/.", 
          RowBox[{"Dot", "\[Rule]", "Times"}]}], ",", "QuantumField"}], "]"}],
        "&"}], ",", "z", ",", "42"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[810]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r2", " ", "=", " ", 
  RowBox[{"symfun", "[", 
   RowBox[{"r1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[811]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[811]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r3", "=", 
  RowBox[{"r2", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["3", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[812]="]
}, Open  ]],

Cell[TextData[{
 "Inspection reveals that still terms are the same. Gather the terms with two \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", " "}], TraditionalForm]]],
 "'s:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twof", "=", 
  RowBox[{"Select", "[", 
   RowBox[{"r3", ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Count", "[", 
       RowBox[{"#", ",", 
        RowBox[{"SUNF", "[", "__", "]"}]}], "]"}], "===", "2"}], "&"}]}], 
   "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["3", "4"]}], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[813]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twofnew", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"twof", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
     "+", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"twof", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"b", "\[RuleDelayed]", "c"}], ",", 
         RowBox[{"c", "\[RuleDelayed]", "b"}]}], "}"}]}], ")"}]}], ")"}], "/.",
    
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[RuleDelayed]", "c"}], ",", 
     RowBox[{"c", "\[RuleDelayed]", "a"}]}], "}"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["e",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r4", " ", "=", " ", 
  RowBox[{"r3", "-", "twof", "+", "twofnew"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[815]="]
}, Open  ]],

Cell["\<\
Check that this is now indeed the same as the t4 result from the first \
attempt.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData["t4"], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[816]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w0", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "g", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "a", ",", "b"}], "}"}], ",", "1"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "a", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "b", ",", "d"}], "}"}], ",", "1"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"g", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "a", ",", "b"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"a", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "b", ",", "d"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w1", "=", 
  RowBox[{"Explicit", "[", "w0", "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["g",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["g",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["g",
           TraditionalForm],
          TraditionalForm]}]]}]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]]}]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[818]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w2", "=", 
  RowBox[{
   RowBox[{"ExpandPartialD", "[", "w1", "]"}], "/.", 
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[819]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif1", " ", "=", " ", 
  RowBox[{"w2", "-", "r4"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif2", "=", 
  RowBox[{"symfun", "[", 
   RowBox[{"dif1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[821]="]
}, Open  ]],

Cell["quod erat demonstrandum.", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unset", "[", 
   RowBox[{"s", "[", "\[Phi]", "]"}], "]"}], ";", 
  RowBox[{"Unset", "[", 
   RowBox[{"S", "[", "A", "]"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{
   "Ag", ",", "F1", ",", "F2", ",", "t1", ",", "t2", ",", "t3", ",", "t4", 
    ",", "F", ",", "r1", ",", "r2", ",", "r3", ",", "r4", ",", "symfun", ",", 
    "twof", ",", "twofnew", ",", "w0", ",", "w1", ",", "w2", ",", "dif1", ",",
     "dif2", ",", "dif3"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[822]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples of funcional differentiation as used in FeynRule", \
"Subsubsection",
 CellTags->"FunctionalD"],

Cell["This is a part of the QCD Lagrangian.", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"Gstrong", "*", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li1", "]"}], ",", "  ", 
       RowBox[{"SUNIndex", "[", "si2", "]"}]}], "]"}], ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", "   ", 
       RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}], " ", ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{
       RowBox[{"PartialD", "[", 
        RowBox[{"LorentzIndex", "[", "li1", "]"}], "]"}], ",", " ", 
       "GaugeField", ",", "   ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", 
       RowBox[{"SUNIndex", "[", "si1", "]"}]}], "]"}]}], "*", "  ", 
    RowBox[{"SUNF", "[", 
     RowBox[{
      RowBox[{"SUNIndex", "[", "si1", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si2", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}]}], ")"}], "/", 
  "4"}]], "Input",
 CellLabel->"In[823]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]]}]}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[823]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Rho]", "}"}], ",", 
        RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]]}], ")"}]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Calc", "[", 
   RowBox[{"%", "//", "Calc"}], "]"}], "//", "Factor"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[825]="]
}, Open  ]]
}, Open  ]],

Cell["FV", "ObjectName",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FV"],

Cell[TextData[{
 "FV[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 ". "
}], "Text",
 CellTags->"FV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[826]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[826]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[827]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[827]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[828]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[828]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[829]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[829]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FV; ExpandScalarProduct \
does the job.\
\>", "Text",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[830]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[830]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[831]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[831]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FVD", "ObjectName",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FVD"],

Cell[TextData[{
 "FVD[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the D-dimensional vector ",
 StyleBox["p",
  FontSlant->"Italic"],
 " with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "."
}], "Text",
 CellTags->"FVD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FVD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[832]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[832]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[833]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[833]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FVD", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[834]:="],

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[834]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[835]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[835]//StandardForm="]
}, Open  ]],

Cell["There is no special function to expand momenta in FVD.", "Text",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FVD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[836]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[836]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[837]:="],

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[837]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GA", "ObjectName",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA"],

Cell[TextData[{
 "GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] by FeynCalcInternal (=FCI).GA[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ","}], TraditionalForm]]],
 " ...] is a short form for GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GA[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[838]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[838]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[839]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[839]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GA", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[840]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"GA",
 CellLabel->"Out[840]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[841]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[841]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[842]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Sigma]", "]"}]}]], "Output",
 CellTags->"GA",
 CellLabel->"Out[842]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GA", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[843]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[843]="]
}, Open  ]]
}, Open  ]],

Cell["GAD", "ObjectName",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GAD"],

Cell[TextData[{
 "GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ",D],D] by FeynCalcInternal (=FCI).GAD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 "] is a short form for GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GAD[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[844]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[844]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[845]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[845]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[846]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[846]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[847]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[847]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[848]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Sigma]", "]"}]}]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[848]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GAD", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[849]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[849]="]
}, Open  ]]
}, Open  ]],

Cell["GammaEpsilon", "ObjectName",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
GammaEpsilon[exp] gives a series expansion of Gamma[exp] in Epsilon up to \
order 6 (where EulerGamma is neglected).\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GammaExpand",
  ButtonNote->"GammaExpand"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
If the argument is of the form (1+a Epsilon) the result is not calculated but \
tabulated.\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "+", 
   RowBox[{"a", " ", "Epsilon"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$3266", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "5"]}], " ", 
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["5",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False], " ", 
       SuperscriptBox["a", "5"]}], "-", 
      RowBox[{
       FractionBox["1", "36"], " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["3",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False], " ", 
       SuperscriptBox["a", "5"]}]}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"]}], "+", 
   RowBox[{
    FractionBox["1", "160"], " ", 
    SuperscriptBox["a", "4"], " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "4"]}], "-", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["a", "3"], " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{
    FractionBox["1", "12"], " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox["\[Pi]", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[850]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "-", 
   RowBox[{"Epsilon", "/", "2"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$3267", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], " ", 
        TagBox[
         RowBox[{"\[Zeta]", "(", 
          TagBox["3",
           Zeta,
           Editable->True,
           Selectable->True], ")"}],
         InterpretTemplate[
          Function[BoxForm`e$, 
           Zeta[BoxForm`e$]]],
         Editable->False,
         Selectable->False]}], "1152"], "+", 
      FractionBox[
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["5",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False], "160"]}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"]}], "2560"], "+", 
   RowBox[{
    FractionBox["1", "24"], " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"]}], "48"], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[851]="]
}, Open  ]],

Cell["For other arguments the expansion is calculated.", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "Epsilon", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$4071", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"126", " ", 
        SuperscriptBox["\[Pi]", "4"], " ", 
        RowBox[{
         SuperscriptBox[
          TagBox["\[Psi]",
           PolyGamma], 
          RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], "+", 
       RowBox[{"24", " ", 
        RowBox[{
         SuperscriptBox[
          TagBox["\[Psi]",
           PolyGamma], 
          RowBox[{"(", "6", ")"}]], "(", "1", ")"}]}], "-", 
       RowBox[{"2016", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        TagBox[
         RowBox[{"\[Zeta]", "(", 
          TagBox["5",
           Zeta,
           Editable->True,
           Selectable->True], ")"}],
         InterpretTemplate[
          Function[BoxForm`e$, 
           Zeta[BoxForm`e$]]],
         Editable->False,
         Selectable->False]}]}], ")"}], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "6"]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"61", " ", 
        SuperscriptBox["\[Pi]", "6"]}], "+", 
       RowBox[{"1680", " ", 
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           TagBox["\[Psi]",
            PolyGamma], 
           RowBox[{"(", "2", ")"}]], "(", "1", ")"}], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "5"]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"20", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{
         SuperscriptBox[
          TagBox["\[Psi]",
           PolyGamma], 
          RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], "-", 
       RowBox[{"288", " ", 
        TagBox[
         RowBox[{"\[Zeta]", "(", 
          TagBox["5",
           Zeta,
           Editable->True,
           Selectable->True], ")"}],
         InterpretTemplate[
          Function[BoxForm`e$, 
           Zeta[BoxForm`e$]]],
         Editable->False,
         Selectable->False]}]}], ")"}], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"]}], "1440"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}], "160"], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     SuperscriptBox[
      TagBox["\[Psi]",
       PolyGamma], 
      RowBox[{"(", "2", ")"}]], "(", "1", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "12"], "+", 
   FractionBox["1", 
    FormBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[852]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "x", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[853]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CapitalGamma]", "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[853]="]
}, Open  ]]
}, Open  ]],

Cell["GammaExpand", "ObjectName",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaExpand"],

Cell["\<\
GammaExpand[exp] rewrites Gamma[n + m] in exp (where n has Head Integer).\
\>", "Text",
 CellTags->"GammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaEpsilon",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GammaEpsilon",
  ButtonNote->"GammaEpsilon"],
 "."
}], "Text",
 CellTags->"GammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"2", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[854]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "+", "1"}], ")"}], " ", 
   RowBox[{"\[CapitalGamma]", "(", 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "+", "1"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[854]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{
    RowBox[{"-", "3"}], "+", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[855]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[CapitalGamma]", "(", 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "+", "1"}], ")"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "3"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "1"}], ")"}], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[855]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"1", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[856]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CapitalGamma]", "(", 
   RowBox[{
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "+", "1"}], ")"}], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[856]="]
}, Open  ]]
}, Open  ]],

Cell["Gamma1", "ObjectName",
 CellTags->"Gamma1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma1"],

Cell["\<\
Gamma1[al,ga, be,de] is a special product of Gamma functions expanded up to \
order Epsilon^2.\
\>", "Text",
 CellTags->"Gamma1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma2",
  ButtonNote->"Gamma2"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma1"]
}, Open  ]],

Cell["Gamma2", "ObjectName",
 CellTags->"Gamma2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma2"],

Cell["\<\
Gamma2[x,y] is a special product of Gamma functions expanded up to order \
Epsilon^3 when positive integer arguments are given.\
\>", "Text",
 CellTags->"Gamma2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma2"]
}, Open  ]],

Cell["Gamma3", "ObjectName",
 CellTags->"Gamma3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma3"],

Cell["\<\
Gamma3[al,be,ga,ep] is a special product of Gamma functions expanded up to \
order Epsilon^n when positive integer arguments are given (the order n is \
determined by the option EpsilonOrder).\
\>", "Text",
 CellTags->"Gamma3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma2",
  ButtonNote->"Gamma2"],
 "."
}], "Text",
 CellTags->"Gamma3"]
}, Open  ]],

Cell["Gauge", "ObjectName",
 CellTags->"Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gauge"],

Cell["\<\
Gauge is an option for GluonProgagator. If set to 1 the 't Hooft Feynman \
gauge is used.\
\>", "Text",
 CellTags->"Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Gauge",
  ButtonNote->"$Gauge"],
 ", ",
 ButtonBox["GluonProgagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonProgagator",
  ButtonNote->"GluonProgagator"],
 "."
}], "Text",
 CellTags->"Gauge"]
}, Open  ]],

Cell["GaugeField", "ObjectName",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeField"],

Cell["\<\
GaugeField is just a name. No functional properties are associated with it. \
GaugeField is used as default setting for the option QuantumField of \
FieldStrength.\
\>", "Text",
 CellTags->"GaugeField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"GaugeField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell[BoxData["GaugeField"], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[857]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellTags->"GaugeField",
 CellLabel->"Out[857]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[858]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"GaugeField",
 CellLabel->"Out[858]="]
}, Open  ]]
}, Open  ]],

Cell["GaugeXi", "ObjectName",
 CellTags->"GaugeXi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeXi"],

Cell["GaugeXi is a head for gauge parameters.", "Text",
 CellTags->"GaugeXi"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gauge",
  ButtonNote->"Gauge"],
 ", ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GaugeXi"]
}, Open  ]],

Cell["GA5", "ObjectName",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA5"],

Cell["GA5 is equivalent to DiracGamma[5] and denotes gamma5.", "Text",
 CellTags->"GA5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell[BoxData["GA5"], "Input",
 CellTags->"GA5",
 CellLabel->"In[859]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"GA5",
 CellLabel->"Out[859]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Tr"}]], "Input",
 CellTags->"GA5",
 CellLabel->"In[860]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"GA5",
 CellLabel->"Out[860]="]
}, Open  ]]
}, Open  ]],

Cell["GGV", "ObjectName",
 CellTags->"GGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GGV"],

Cell["GGV is equivalent to GluonGhostVertex.", "Text",
 CellTags->"GGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GGV"]
}, Open  ]],

Cell["GhostPropagator", "ObjectName",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "GhostPropagator[p, a, b] gives the ghost propagator where \"a\" and \"b\" \
are the color indices. GhostPropagator[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GHP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GHP",
  ButtonNote->"GHP"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GhostPropagator", "]"}]], "Input",
 CellLabel->"In[861]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[861]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[862]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[863]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[863]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", "p", "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[864]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "u"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[864]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GhostPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[865]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[865]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GhostPropagator", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    " ", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[866]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[866]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GHP", "ObjectName",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GHP"],

Cell[TextData[{
 "GHP[p, a, b] gives the ghost propagator where \"a\" and \"b\" are the color \
indices. GHP[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[867]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[867]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GHP", "[", "p", "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[868]:="],

Cell[BoxData[
 FormBox[
  FractionBox["\[ImaginaryI]", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[868]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[869]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"ci1", "ci2"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[869]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{
    RowBox[{"GHP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], 
   "]"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[870]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[870]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GluonField", "ObjectName",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonField"],

Cell["GluonField is a name of a gauge field.", "Text",
 CellTags->"GluonField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GluonField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell[BoxData["GluonField"], "Input",
 CellTags->"GluonField",
 CellLabel->"In[871]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellTags->"GluonField",
 CellLabel->"Out[871]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GluonField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GluonField",
 CellLabel->"In[872]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"GluonField",
 CellLabel->"Out[872]="]
}, Open  ]]
}, Open  ]],

Cell["GluonGhostVertex", "ObjectName",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell["\<\
GluonGhostVertex[{p,mu,a}, {q,nu,b}, {k,rho,c}] or GluonGhostVertex[ p,mu,a , \
q,nu,b , k,rho,c ] yields the Gluon-Ghost vertex. The first argument \
represents the gluon and the third argument the outgoing ghost field (but \
incoming four-momentum). The dimension and the name of the coupling constant \
are determined by the options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonGhostVertex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonGhostVertex", "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[873]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[873]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[874]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellLabel->"In[876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[876]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[877]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellLabel->"Out[877]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "Explicit"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[878]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "Gstrong"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"k", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[878]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "k", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[879]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "li1"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[879]="]
}, Open  ]]
}, Open  ]],

Cell["GluonPropagator", "ObjectName",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[TextData[{
 "GluonPropagator[p, {mu, a}, {nu, b}] or GluonPropagator[p, mu, a , nu, b ] \
yields the gluon propagator. GluonPropagator[p, {mu}, {nu}] or \
GluonPropagator[p, mu, nu] omits the SUNDelta. The gauge and the dimension is \
determined by the option Gauge and Dimension. The following settings of Gauge \
are possible: 1 for the Feynman gauge; ",
 Cell[BoxData[
  FormBox["\[Alpha]", TraditionalForm]]],
 " for the general covariant gauge; {Momentum[n] ,1} for the axial gauge."
}], "Text",
 CellTags->"GluonPropagator"],

Cell["GP can be used as an abbreviation of GluonPropagator.", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData["GP"], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[880]:="],

Cell[BoxData[
 FormBox["GluonPropagator", TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[880]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonPropagator", "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[881]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[881]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[882]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellTags->"GluonPropagator",
 CellLabel->"Out[882]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], "]"}], "//",
   "Explicit"}]], "Input",
 CellLabel->"In[883]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[883]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[884]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[884]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Momentum", "[", "n", "]"}], ",", "1"}], "}"}]}]}], "]"}], "//",
   "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[885]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["n",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], 
       RowBox[{
        FormBox["n",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[885]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[886]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[886]="]
}, Open  ]],

Cell["\<\
This is a convenient way to enter amplitudes by hand (GP is an abbreviation \
GluonPropagator).\
\>", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GP", "[", 
    RowBox[{"p", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[887]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li2",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"li1", "li2"}]]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[887]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[888]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"li3", ",", "li4"}], "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[888]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "1"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[889]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"11", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["19", "6"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[889]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "2"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[890]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "3"]}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        FractionBox["1", "6"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[890]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "3"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[891]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["T", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[891]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "4"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[892]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[892]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "5"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[893]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubsuperscriptBox["g", "s", "2"]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["T", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubsuperscriptBox["g", "s", "2"]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[893]="]
}, Open  ]]
}, Open  ]],

Cell["GluonSelfEnergy", "ObjectName",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell["\<\
GluonSelfEnergy[{mu, a}, {nu,b}] yields the 1-loop Gluon selfenergy.\
\>", "Text",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonSelfEnergy", "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[894]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"log", "(", 
         RowBox[{
          SuperscriptBox["\[Mu]", "2"], " ", "_"}], ")"}], "\[RuleDelayed]", 
        "0"}], ",", 
       RowBox[{
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], "\[RuleDelayed]", 
        RowBox[{"log", "(", 
         RowBox[{"4", " ", "\[Pi]"}], ")"}]}]}], "}"}]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[894]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[895]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output",
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[895]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[896]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "2"]}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], "2"]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Xi]"}], ")"}]}], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox["13", "3"], "-", "\[Xi]"}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output",
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[896]="]
}, Open  ]]
}, Open  ]],

Cell["GluonVertex", "ObjectName",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonVertex"],

Cell["Note: All momenta are flowing into the vertex.", "Text",
 CellTags->"GluonVertex"],

Cell["\<\
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or GluonVertex[p,mu,a , q,nu,b , \
k,la,c ] yields the 3-gluon vertex. GluonVertex[{p,mu}, {q,nu}, {k,la}] \
yields the 3-gluon vertex without color structure and the coupling constant. \
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or GluonVertex[{mu,a}, \
{nu,b}, {la,c}, {si,d}] or GluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
GluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex. The \
dimension and the name of the coupling constant are determined by the options \
Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell["GV can be used as an abbreviation of GluonVertex.", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData["GV"], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[897]:="],

Cell[BoxData[
 FormBox["GluonVertex", TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[897]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonVertex", "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[898]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[898]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[899]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[899]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[900]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[900]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[901]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["V", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r"}], ")"}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[901]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[902]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"GluonVertex",
 CellLabel->"Out[902]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[903]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["V", 
    RowBox[{"a", "b", "c", "d"}], 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r", 
    RowBox[{",", " "}], "s"}], ")"}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[903]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellLabel->"In[904]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u9",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[904]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[905]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["W", 
   RowBox[{"a", "b", "c", "d"}]], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[905]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
   "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", "\[Rho]", ",", "c", ",", 
    "\[Sigma]", ",", "d"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[906]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u10",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[906]="]
}, Open  ]],

Cell["\<\
A very convenient way to enter diagrams by hand is to label each line hitting \
a vertex by a number and put this number after the inflowing momentum.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GV", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[907]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[907]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GV", "[", 
   RowBox[{
   "p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3", ",", "s", ",", 
    "4"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[908]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u11",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[908]="]
}, Open  ]]
}, Open  ]],

Cell["GO", "ObjectName",
 CellTags->"GO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GO"],

Cell["GO is equivalent to Twist2GluonOperator.", "Text",
 CellTags->"GO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"GO"]
}, Open  ]],

Cell["GP", "ObjectName",
 CellTags->"GP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GP"],

Cell["GP is equivalent to GluonPropagator.", "Text",
 CellTags->"GP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GP"]
}, Open  ]],

Cell["GrassmannParity", "ObjectName",
 CellTags->"GrassmannParity"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GrassmannParity"],

Cell["\<\
GrassmannParity is a data type. E.g. DataType[F, GrassmannParity] = 1 \
declares F to be of bosonic type and DataType[F, GrassmannParity] = -1 of \
fermionic one.\
\>", "Text",
 CellTags->"GrassmannParity"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],

Cell["GS", "ObjectName",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GS"],

Cell[TextData[{
 "GS[p] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p]] by FeynCalcInternal \
(=FCI). GS[p,q, ...] is a short form for GS[p].GS[q]. ... ."
}], "Text",
 CellTags->"GS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[909]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[909]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[910]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", "p", "]"}], "]"}]], "Output",
 CellTags->"GS",
 CellLabel->"Out[910]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[911]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[911]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[912]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "p", "]"}], ".", 
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"GS", "[", "r", "]"}], ".", 
  RowBox[{"GS", "[", "s", "]"}]}]], "Output",
 CellTags->"GS",
 CellLabel->"Out[912]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GS", "[", "q", "]"}]}]], "Input",
 CellTags->"GS",
 CellLabel->"In[913]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[913]="]
}, Open  ]]
}, Open  ]],

Cell["GSD", "ObjectName",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GSD"],

Cell[TextData[{
 "GSD[p] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p,D],D] by FeynCalcInternal \
(=FCI). GSD[p,q, ...] is a short form for GSD[p].GSD[q]. ... ."
}], "Text",
 CellTags->"GSD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GSD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", "p", "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[914]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[914]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GSD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[915]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[915]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[916]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[916]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[917]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "p", "]"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"GSD", "[", "r", "]"}], ".", 
  RowBox[{"GSD", "[", "s", "]"}]}]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[917]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}]}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[918]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[918]="]
}, Open  ]]
}, Open  ]],

Cell["Gstrong", "ObjectName",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gstrong"],

Cell["Gstrong denotes the strong coupling constant.", "Text",
 CellTags->"Gstrong"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex"],
 "."
}], "Text",
 CellTags->"Gstrong"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Gstrong"],

Cell["\<\
Gstrong has no functional properties. Only a typesetting rule is defined.\
\>", "Text",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell[BoxData["Gstrong"], "Input",
 CellTags->"Gstrong",
 CellLabel->"In[919]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["g", "s"], TraditionalForm]], "Output",
 CellTags->"Gstrong",
 CellLabel->"Out[919]="]
}, Open  ]]
}, Open  ]],

Cell["GTI", "ObjectName",
 CellTags->"GTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GTI"],

Cell["GTI is like RHI, but with no functional properties.", "Text",
 CellTags->"GTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"GTI"]
}, Open  ]],

Cell["GV", "ObjectName",
 CellTags->"GV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GV"],

Cell["GV is equivalent to GluonVertex.", "Text",
 CellTags->"GV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GV"]
}, Open  ]],

Cell["Hill", "ObjectName",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Hill"],

Cell["\<\
Hill[x, y] gives the Hill identity with arguments x and y. The returned \
object is 0.\
\>", "Text",
 CellTags->"Hill"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Hill"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hill", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[920]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"log", "(", "a", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "a"}], ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{"1", "-", "a"}], 
      RowBox[{"1", "-", "b"}]], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"log", "(", "a", ")"}]}], "+", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}], "-", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"1", "-", "b"}]], ")"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}], "-", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"a", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], ")"}], " ", 
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
      RowBox[{"a", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", "a", ")"}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox[
     RowBox[{"1", "-", "a"}], 
     RowBox[{"1", "-", "b"}]], ")"}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", "b", ")"}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox["b", "a"], ")"}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
     RowBox[{"a", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "b"}], ")"}]}]], ")"}], "-", 
   FractionBox[
    SuperscriptBox["\[Pi]", "2"], "6"]}], TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[920]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"%", " ", "/.", " ", 
    RowBox[{"a", "\[RuleDelayed]", " ", ".123"}]}], " ", "/.", " ", 
   RowBox[{"b", "\[RuleDelayed]", " ", ".656"}]}], " ", "//", " ", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[921]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[921]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Hill", "[", 
     RowBox[{"x", ",", 
      RowBox[{"x", " ", "y"}]}], "]"}], "//", "PowerExpand"}], "//", 
   "SimplifyPolyLog"}], "//", "Expand"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[922]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "y"}], ")"}], " ", 
    RowBox[{"log", "(", "y", ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", 
      RowBox[{"x", " ", "y"}]}], ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "y"}], ")"}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"x", " ", "y"}], ")"}], "+", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox[
     RowBox[{"1", "-", "x"}], 
     RowBox[{"1", "-", 
      RowBox[{"x", " ", "y"}]}]], ")"}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], " ", "y"}], 
     RowBox[{"1", "-", 
      RowBox[{"x", " ", "y"}]}]], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[922]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", " ", "/.", " ", 
     RowBox[{"x", "\[RuleDelayed]", " ", ".34"}]}], "/.", " ", 
    RowBox[{"y", "\[Rule]", " ", ".6"}]}], "//", "N"}], "//", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[923]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[923]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricAC", "ObjectName",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["\<\
HypergeometricAC[n][exp] analytically continues Hypergeometric2F1 functions \
in exp. The second argument n refers to the equation number (n) in chapter \
2.10 of \"Higher Transcendental Functions\" by Ergelyi, Magnus, \
Oberhettinger, Tricomi. In case of eq. (6) (p.109) the last line is returned \
for HypergeometricAC[6][exp], while the first equality is given by \
HypergeometricAC[61][exp]. ((2.10.1) is identical to eq. (9.5.7) of \"Special \
Functions & their Applications\" by N.N.Lebedev).\
\>", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricAC", "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[924]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Collect2", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"HypergeometricAC",
 CellLabel->"Out[924]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypExplicit",
  ButtonNote->"HypExplicit"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["These are all transformation rules currently built in.", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "1", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[925]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Gamma]", "-", "\[Beta]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{
              RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+",
               "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           RowBox[{"1", "-", "z"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], ")"}], 
     " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Alpha]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox["\[Beta]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           RowBox[{"1", "-", "z"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[925]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "2", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[926]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Alpha]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           FractionBox["1", "z"],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Beta]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Beta]", "-", "\[Gamma]", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{
              RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           FractionBox["1", "z"],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[926]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "3", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[927]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Alpha]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Gamma]", "-", "\[Beta]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           FractionBox["1", 
            RowBox[{"1", "-", "z"}]],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Beta]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{
              RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           FractionBox["1", 
            RowBox[{"1", "-", "z"}]],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[927]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "4", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[928]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], ")"}], 
     " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["\[Alpha]",
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           RowBox[{"-", 
            FractionBox[
             RowBox[{"1", "-", "z"}], "z"]}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox["z", 
      RowBox[{"-", "\[Alpha]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]], " ", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
         SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", "\[Alpha]"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            TagBox[
             RowBox[{
              RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+",
               "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           Hypergeometric2F1,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           RowBox[{"-", 
            FractionBox[
             RowBox[{"1", "-", "z"}], "z"]}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      Hypergeometric2F1], " ", 
     SuperscriptBox["z", 
      RowBox[{"\[Alpha]", "-", "\[Gamma]"}]]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
     RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[928]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "6", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[929]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Beta]"}]], " ", 
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["\[Beta]",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["\[Gamma]",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"-", 
          FractionBox["z", 
           RowBox[{"1", "-", "z"}]]}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[929]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "61", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[930]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Alpha]"}]], " ", 
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["\[Alpha]",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"\[Gamma]", "-", "\[Beta]"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["\[Gamma]",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"-", 
          FractionBox["z", 
           RowBox[{"1", "-", "z"}]]}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[930]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricIR", "ObjectName",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell["\<\
HypergeometricIR[exp, t] substitutes for all Hypergeometric2F1[a,b,c,z] in \
exp by its Euler integral reprentation. The factor Integratedx[t, 0, 1] can \
be omitted by setting the option Integratedx \[Rule] False.\
\>", "Text",
 CellTags->"HypergeometricIR"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricIR", "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[931]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integratedx", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[931]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[932]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "-", "b"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[932]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"t", "^", "b"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", "c"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "^", "a"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[933]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "1"}], ")"}], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"b", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"b", "+", "c", "+", "2"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}], 
   RowBox[{"\[CapitalGamma]", "(", 
    RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[933]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[934]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
   SuperscriptBox["t", "b"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"t", " ", "z"}], "+", "1"}], ")"}], "a"]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[934]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricSE", "ObjectName",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[TextData[{
 "HypergeometricSE[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions by their series expansion in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 ", running from 0 to ",
 Cell[BoxData[
  FormBox["\[Infinity]", TraditionalForm]]],
 ", is the summation index)."
}], "Text",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricSE", "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[935]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Simplify", "\[Rule]", "FullSimplify"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[935]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[936]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["z", "\[Nu]"], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[936]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[937]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["z", "\[Nu]"], " ", 
    RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[937]="]
}, Open  ]]
}, Open  ]],

Cell["HypExplicit", "ObjectName",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "HypExplicit[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions in exp by their definition in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 " is the summation index)."
}], "Text",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[938]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
      SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["a",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True], ",", 
          TagBox["b",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         TagBox["c",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox["z",
        Hypergeometric2F1,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   Hypergeometric2F1], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[938]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[939]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["z", "\[Nu]"], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[939]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricPFQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[940]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
      SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["a",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["b",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["c",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         RowBox[{
          TagBox["d",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["e",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox["z",
        HypergeometricPFQ,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   HypergeometricPFQ], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[940]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[941]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["z", "\[Nu]"], " ", 
    RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[941]="]
}, Open  ]]
}, Open  ]],

Cell["HypInt", "ObjectName",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypInt"],

Cell["\<\
HypInt[exp, t]  substitutes for all  Hypergeometric2F1[a,b,c,z] in exp \
Gamma[c]/(Gamma[b] Gamma[c-b]) Integratedx[t,0,1]  t^(b-1) (1-t)^(c-b-1) (1-t \
z)^(-a).\
\>", "Text",
 CellTags->"HypInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[942]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
      SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["a",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True], ",", 
          TagBox["b",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         TagBox["c",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox["z",
        Hypergeometric2F1,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   Hypergeometric2F1], TraditionalForm]], "Output",
 CellTags->"HypInt",
 CellLabel->"Out[942]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypInt", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[943]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      FormBox["0",
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]], 
     RowBox[{"\[DifferentialD]", 
      TagBox[
       FormBox["t",
        TraditionalForm],
       TraditionalForm,
       Editable->True], "\[VeryThinSpace]"}]}]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "-", "b"}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"HypInt",
 CellLabel->"Out[943]="]
}, Open  ]]
}, Open  ]],

Cell["IFPD", "ObjectName",
 CellTags->"IFPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPD"],

Cell["IFPD[p, m] denotes (p^2 - m^2).", "Text",
 CellTags->"IFPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"IFPD"]
}, Open  ]],

Cell["IFPDOff", "ObjectName",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOff"],

Cell["\<\
IFPDOff[exp_,q1_, q2_, ...] changes from IFPD representation to \
FeynAmpDenominator[ ...]. The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOff"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOn",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPDOn",
  ButtonNote->"IFPDOn"],
 "."
}], "Text",
 CellTags->"IFPDOff"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[944]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["p", "2"],
    TraditionalForm], 
   FormBox["\<\" - \"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[944]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[945]:="],

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[945]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPDOff", "[", 
  RowBox[{"%", ",", "p"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[946]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[946]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[947]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   SuperscriptBox["m", "2"]}], "+", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[947]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["IFPDOn", "ObjectName",
 CellTags->"IFPDOn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOn"],

Cell["\<\
IFPDOn[exp_,q1_, q2_, ...] changes from FeynAmpDenominator[ ...] \
representation to the IFPD one (Inverse Feynman Propagator Denominator). \
I.e., FeynAmpDenominator[PropagatorDenominator[a,b]] is replaced by \
1/IFPD[a,b] and The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOn"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPDOff",
  ButtonNote->"IFPDOff"],
 "."
}], "Text",
 CellTags->"IFPDOn"]
}, Open  ]],

Cell["IncludePair", "ObjectName",
 CellTags->"IncludePair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IncludePair"],

Cell["\<\
IncludePair is an option for FC2RHI. Possible settings are True and False.\
\>", "Text",
 CellTags->"IncludePair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC2RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC2RHI",
  ButtonNote->"FC2RHI"],
 "."
}], "Text",
 CellTags->"IncludePair"]
}, Open  ]],

Cell["IndexPosition", "ObjectName",
 CellTags->"IndexPosition"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IndexPosition"],

Cell["IndexPosition is an option for FieldStrength.", "Text",
 CellTags->"IndexPosition"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"IndexPosition"]
}, Open  ]],

Cell["Indices", "ObjectName",
 CellTags->"Indices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Indices"],

Cell["\<\
Indices is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a I(ndices) \
statement.\
\>", "Text",
 CellTags->"Indices"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Indices"]
}, Open  ]],

Cell["InitialFunction", "ObjectName",
 CellTags->"InitialFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialFunction"],

Cell["\<\
InitialFunction is an option of FeynRule the setting of which is applied to \
the first argument of FeynRule before anything else\
\>", "Text",
 CellTags->"InitialFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"InitialFunction"]
}, Open  ]],

Cell["InitialSubstitutions", "ObjectName",
 CellTags->"InitialSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialSubstitutions"],

Cell["\<\
InitialSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"InitialSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],

Cell["InsideDiracTrace", "ObjectName",
 CellTags->"InsideDiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InsideDiracTrace"],

Cell["\<\
InsideDiracTrace is and option of DiracSimplify. If set to True, \
DiracSimplify assumes to operate inside a DiracTrace, i.e., products of an \
odd number of Dirac matrices are discarded. Furthermore simple traces are \
calculated (but divided by a factor 4, i.e. :  \
DiracSimplify[DiracMatrix[a,b], InsideDiracTrace\[Rule]True]  yields \
ScalarProduct[a,b]) Traces involving more than four DiracGamma's and \
DiracGamma[5] are not performed.\
\>", "Text",
 CellTags->"InsideDiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],

Cell["IntegralTable", "ObjectName",
 CellTags->"IntegralTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegralTable"],

Cell["\<\
IntegralTable is an option of OneLoopSimplify, TwoLoopSimplify and \
FeynAmpDenominatorSimplify. It may be set to a list of the form : \
{FCIntegral[ ... ] \[RuleDelayed] bla, ...}.\
\>", "Text",
 CellTags->"IntegralTable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TwoLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TwoLoopSimplify",
  ButtonNote->"TwoLoopSimplify"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"IntegralTable"]
}, Open  ]],

Cell["IntegrateByParts", "ObjectName",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell["\<\
IntegrateByParts[(1-t)^(a Epsilon -1) g[t], deriv, t] does an integration by \
parts of the definite integral over t from 0 to 1.\
\>", "Text",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "IntegrateByParts", "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[948]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Hold", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",
 CellTags->"IntegrateByParts",
 CellLabel->"Out[948]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegrateByParts", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[949]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["g", "\[Prime]",
       MultilineFunction->None], "(", "t", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{"a", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]]}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"g", "(", "0", ")"}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"IntegrateByParts",
 CellLabel->"Out[949]="]
}, Open  ]]
}, Open  ]],

Cell["Integratedx", "ObjectName",
 CellTags->"Integratedx"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integratedx"],

Cell["\<\
Integratedx[x, low, up] is a variable representing the integration operator \
Integrate[#, {x,low,up}]&.\
\>", "Text",
 CellTags->"Integratedx"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["CalculateCounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CalculateCounterTerm",
  ButtonNote->"CalculateCounterTerm"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypInt",
  ButtonNote->"HypInt"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 "."
}], "Text",
 CellTags->"Integratedx"]
}, Open  ]],

Cell["Integrate2", "ObjectName",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate2"],

Cell[TextData[{
 "Integrate2 is like Integrate, but : Integrate2[a_Plus, b__] := \
Map[Integrate2[#, b]&, a] ( more linear algebra and partial fraction \
decomposition is done) Integrate2[f[x] DeltaFunction[x], x] \[Rule] f[0] \
Integrate2[f[x] DeltaFunction[x0-x], x] \[Rule] f[x0]. Integrate2[f[x] \
DeltaFunction[a + b x], x] \[Rule] Integrate[f[x] (1/Abs[b]) \
DeltaFunction[a/b + x], x], where abs[b] \[Rule] b, if b is a Symbol, and if \
b = -c, then abs[-c] \[Rule] c, i.e., the variable contained in b is supposed \
to be positive. ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["\[Pi]", "2"]}], TraditionalForm]]],
 " is replaced by 6 Zeta2. Integrate2[1/(1-y),{y,x,1}] is intepreted as \
distribution, i.e. as Integrate2[-1/(1-y)],{y, 0, x}] \[Rule] Log[1-y]. \
Integrate2[1/(1-x),{x,0,1}] \[Rule] 0."
}], "Text",
 CellTags->"Integrate2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate3",
  ButtonNote->"Integrate3"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"Integrate2"],

Cell["\<\
NOTE: Since Integrate2 does do a reordering and partial fraction \
decomposition before calling the integral table of Integrate3 it will in \
general be slower compared to Integrate3 for sums of integrals. I.e., if the \
integrand has already an expanded form and if partial fraction decomposition \
is not necessary it is more effective to use Integrate3.\
\>", "Text",
 CellTags->"Integrate2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[950]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.01600100000000282`", ",", 
    RowBox[{
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False], "-", 
     RowBox[{
      FractionBox["3", "2"], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Integrate2",
 CellLabel->"Out[950]="]
}, Open  ]],

Cell[TextData[{
 "Since Integrate2 uses table-look-up methods it is much faster than ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Integrate."
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"2", ",", 
     RowBox[{"x", "^", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[951]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
   RowBox[{"4", " ", 
    RowBox[{"log", "(", "2", ")"}]}], "-", "4"}], TraditionalForm]], "Output",\

 CellTags->"Integrate2",
 CellLabel->"Out[951]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"-", "x"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[952]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], "4"], "-", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"Integrate2",
 CellLabel->"Out[952]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", "x"}], ")"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"log", "(", "2", ")"}]}], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], "4"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "2", ")"}], "3"], "-", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[953]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[954]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[954]="]
}, Open  ]],

Cell[TextData[{
 "Integrate2 does integration in a Hadamard sense, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 " means acutally expanding the result of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "\[Delta]", 
     RowBox[{"1", "-", "\[Delta]"}]], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 "up do ",
 Cell[BoxData[
  FormBox[
   RowBox[{"O", "(", "\[Delta]", ")"}], TraditionalForm]]],
 " and neglecting all ",
 Cell[BoxData[
  FormBox["\[Delta]", TraditionalForm]]],
 "-dependent terms. E.g. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", "\[Delta]", 
      RowBox[{"1", "-", "\[Delta]"}]], 
     RowBox[{
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "log"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      SubsuperscriptBox["\[VerticalSeparator]", "\[Delta]", 
       RowBox[{"1", "-", "\[Delta]"}]]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"log", "(", "\[Delta]", ")"}]}], "+", 
       RowBox[{"log", "(", "1", ")"}]}], "\[Implies]", "0."}]}], " "}], 
   TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->False,
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[955]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[955]="]
}, Open  ]],

Cell[TextData[{
 "In the physics literature sometimes the \"+\" notation is used. In FeynCalc \
the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]]],
 " is represented by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], TraditionalForm]]],
 " or just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[956]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[956]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"1", "-", "x"}]}], "]"}], "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[957]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[957]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[958]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], "8"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[958]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", "x", "]"}], "^", "2"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[959]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   TagBox[
    RowBox[{"\[Zeta]", "(", 
     TagBox["3",
      Zeta,
      Editable->True,
      Selectable->True], ")"}],
    InterpretTemplate[
     Function[BoxForm`e$, 
      Zeta[BoxForm`e$]]],
    Editable->False,
    Selectable->False]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "x"}]}], "]"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], "4"], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[960]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", "x", "]"}], " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], "4"]}], "+", 
   FractionBox[
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], "2"], "+", 
   FractionBox["3", "16"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[962]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Log", "[", "x", "]"}], "^", "2"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[963]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "4"], "(", 
     FractionBox["1", "2"], ")"}]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "4"], "(", "2", ")"}], "6"], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[963]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", "z"}], ")"}]}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x", "+", 
           RowBox[{"x", " ", "z"}]}], ")"}]}], ")"}], "/", "z"}]}], "]"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x", "+", 
      RowBox[{"x", " ", "z"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "z", ")"}], 
    RowBox[{"6", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"-", "z"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "z"}],
       TraditionalForm], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      RowBox[{"-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      FractionBox["1", 
       RowBox[{"z", "+", "1"}]], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], 
        RowBox[{"z", "+", "1"}]]}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      FractionBox[
       RowBox[{"1", "-", "z"}], 
       RowBox[{"z", "+", "1"}]], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], 
    RowBox[{"1", "-", "z"}]]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[964]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart", "[", 
  RowBox[{
   RowBox[{"Integrate2", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", 
       RowBox[{"(", 
        RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"1", "-", "x"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[965]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], 
     RowBox[{"m", "-", "1"}]]}], "-", 
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], 
    SuperscriptBox["m", "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "2"}],
         TraditionalForm], ")"}]}], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
     RowBox[{
      SubscriptBox["S", "12"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], "+", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], "m"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[965]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     FractionBox[
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}], 
      RowBox[{"1", "+", "x"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"90", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubsuperscriptBox["S", 
        RowBox[{"-", "1"}], "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True,
            Selectable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]],
          Editable->False,
          Selectable->False]}]}], ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
         RowBox[{"log", "(", "4", ")"}], " ", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "1"}]], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}]}], ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"270", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "3"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "2"}], "-", "1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "2", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "1", "-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", "-", "11"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"585", " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False]}], "-", 
     RowBox[{"945", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False]}], "-", 
     RowBox[{"720", " ", 
      RowBox[{
       SubscriptBox[
        InterpretationBox["Li",
         PolyLog,
         Editable->False,
         Selectable->False], "4"], "(", 
       FractionBox["1", "2"], ")"}]}], "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}], "+", 
     RowBox[{"450", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[966]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"OPEm", "\[Rule]", "2"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[967]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"450", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
     RowBox[{"720", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
        RowBox[{"log", "(", "4", ")"}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}]}], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True,
            Selectable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]],
          Editable->False,
          Selectable->False]}]}], ")"}]}], "-", 
     RowBox[{"945", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False]}], "+", 
     RowBox[{"585", " ", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False]}], "-", 
     RowBox[{"720", " ", 
      RowBox[{
       SubscriptBox[
        InterpretationBox["Li",
         PolyLog,
         Editable->False,
         Selectable->False], "4"], "(", 
       FractionBox["1", "2"], ")"}]}], "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"1080", " ", 
      RowBox[{"log", "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}], "-", "2160"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[967]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[968]:="],

Cell[BoxData[
 FormBox["0.0505138354027453`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[968]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", " ", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"Log", "[", "x", "]"}], " ", 
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{"1", "+", "x"}]]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[969]:="],

Cell[BoxData[
 FormBox["0.05051383539905906`", TraditionalForm]], "Output",
 CellLabel->"Out[969]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[970]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "3"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "2"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "3"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "21"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       RowBox[{"-", "1"}], "-", "2"}]], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "12"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "21"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], 
    RowBox[{"8", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], 
    RowBox[{"8", " ", "m"}]]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[970]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "3"}]}]], "Input",
 CellLabel->"In[971]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"log", "(", "2", ")"}]}], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "+", 
   FractionBox[
    RowBox[{"7", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}], "6"], "+", 
   FractionBox["2", "3"]}], TraditionalForm]], "Output",
 CellLabel->"Out[971]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[972]:="],

Cell[BoxData[
 FormBox["0.10641794295256102`", TraditionalForm]], "Output",
 CellLabel->"Out[972]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"3", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[973]:="],

Cell[BoxData[
 FormBox["0.10641794295771256`", TraditionalForm]], "Output",
 CellLabel->"Out[973]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[974]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[974]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[975]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[975]="]
}, Open  ]],

Cell["\<\
This is the polarized non-singlet spin splitting function whose first moment \
vanishes.\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[976]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "4"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "24"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"48", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True,
             Selectable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]],
           Editable->False,
           Selectable->False]}], "+", "3"}], ")"}]}]}], ")"}], " ", 
    SubsuperscriptBox["C", "F", "2"]}], "+", 
   RowBox[{
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "-", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"2", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"4", " ", 
           RowBox[{
            SubscriptBox[
             InterpretationBox["Li",
              PolyLog,
              Editable->False,
              Selectable->False], "2"], "(", 
            RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True,
             Selectable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]],
           Editable->False,
           Selectable->False]}], "+", 
         FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
    SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[976]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{"t", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[977]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.1720109999999977`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[977]="]
}, Open  ]],

Cell["\<\
Expanding t with respect to x yields a form already suitable for Integrate3 \
and therefore the following is faster:\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Integrate3", "[", 
    RowBox[{
     RowBox[{"Expand", "[", 
      RowBox[{"t", ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Expand"}], "//",
   "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[978]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.03600200000000342`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[978]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[979]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[980]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[980]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[981]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["6959", "10800"]}], "+", 
   FractionBox[
    RowBox[{"46", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "45"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[981]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "@", "%"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[982]:="],

Cell[BoxData[
 FormBox["0.06419859938720329`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[982]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[983]:="],

Cell[BoxData[
 FormBox["0.06419859938720252`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[983]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[984]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], " ", 
      RowBox[{
       SubsuperscriptBox["S", "1", "2"], "(", 
       FormBox["m",
        TraditionalForm], ")"}]}], "m"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"m", "-", "1"}], "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox["m", "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "11"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "2", ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}], "m"]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[984]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "4"}]}]], "Input",
 CellLabel->"In[985]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["241", "288"]}], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "3"]}], TraditionalForm]], "Output",
 CellLabel->"Out[985]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[986]:="],

Cell[BoxData[
 FormBox["0.0873906851910381`", TraditionalForm]], "Output",
 CellLabel->"Out[986]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"4", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[987]:="],

Cell[BoxData[
 FormBox["0.0873906851910375`", TraditionalForm]], "Output",
 CellLabel->"Out[987]="]
}, Open  ]]
}, Open  ]],

Cell["Integrate3", "ObjectName",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate3"],

Cell["\<\
Integrate3 contains the integral table used by Integrate2. Integration is \
performed in a distributional sense. Integrate3 works more effectively on a \
sum of expressions if they are expanded or collected with respect to the \
integration variable. See the examples in Integrate2.\
\>", "Text",
 CellTags->"Integrate3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"Integrate3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "OPEm"], " ", 
    RowBox[{"Log", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[988]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[988]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "OPEm"], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}]}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[989]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox["m",
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "-", 
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "+", 
   TagBox[
    RowBox[{"\[Zeta]", "(", 
     TagBox["3",
      Zeta,
      Editable->True,
      Selectable->True], ")"}],
    InterpretTemplate[
     Function[BoxForm`e$, 
      Zeta[BoxForm`e$]]],
    Editable->False,
    Selectable->False]}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[989]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"a", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], " ", 
       RowBox[{"Log", "[", "x", "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}], 
      RowBox[{"1", "-", "x"}]]}], "+", 
    RowBox[{"b", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], 
       RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
         RowBox[{"-", "x"}]}], "]"}]}], 
      RowBox[{"1", "+", "x"}]]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[990]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       SubscriptBox["S", "12"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "-", 
      RowBox[{
       SubscriptBox["S", "21"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "+", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False]}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", "b", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "8"], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "2"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}], " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{
       RowBox[{"log", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox["m",
           TraditionalForm], ")"}], "-", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "3"}]], "(", 
          FormBox["m",
           TraditionalForm], ")"}]}], ")"}]}], "+", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"3", "-", "1"}]], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "1"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}], " ", 
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["3",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False]}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       RowBox[{"log", "(", "2", ")"}], " ", 
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["3",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[990]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"DeltaFunctionPrime", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[991]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[991]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], " ", 
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[992]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[992]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[993]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[993]="]
}, Open  ]]
}, Open  ]],

Cell["IntermediateSubstitutions", "ObjectName",
 CellTags->"IntermediateSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntermediateSubstitutions"],

Cell["\<\
IntermediateSubstitutions is an option for OneLoop. All substitutions \
indicated hereby are done at an intermediate stage of the calculation.\
\>", "Text",
 CellTags->"IntermediateSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],

Cell["InverseMellin", "ObjectName",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "InverseMellin[exp, y] performs the inverse Mellin transform of polynomials \
in OPEm. The inverse transforms are not calculated but a table-lookup is \
done. WARNING: do not \"trust\" the results for the inverse Mellin transform \
involving SumT's; there is an unresolved inconsistency here (related to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "m"}], ")"}], "."}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"InverseMellin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[994]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "-", "1"}]], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[994]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"OPEm", "+", "3"}], ")"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[995]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "+", "2"}]], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[995]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{"1", ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[996]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "y"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[996]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"OPEm", "^", "4"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[997]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SuperscriptBox["log", "3"], "(", "y", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"InverseMellin",
 CellLabel->"Out[997]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "OPEm"}], "+", "1"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[998]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox["y", 
     RowBox[{"m", "-", "1"}]]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[998]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "i"}], "+", "1"}], ",", "y", ",", "i"}], "]"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[999]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox["y", 
     RowBox[{"i", "-", "1"}]]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"i", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[999]="]
}, Open  ]],

Cell["\<\
The inverse operation to InverseMellin is done by Integrate2.\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"InverseMellin", "[", 
    RowBox[{
     RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1000]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", "m"], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1000]="]
}, Open  ]],

Cell["\<\
Below is a list of all built-in basic inverse Mellin transforms .\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[BoxData[
 RowBox[{
  RowBox[{"list", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     FractionBox["1", 
      RowBox[{"OPEm", "+", "n"}]], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "OPEm"}], "+", "n"}]], ",", 
     RowBox[{"PolyGamma", "[", 
      RowBox[{"0", ",", "OPEm"}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "-", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"1", "-", "OPEm"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "+", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      SuperscriptBox["OPEm", "2"]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     FractionBox[
      SuperscriptBox[
       RowBox[{"SumS", "[", 
        RowBox[{"1", ",", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "2"], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{"OPEm", "-", "1"}]}], "]"}], "2"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "3"], ",", 
     RowBox[{
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], " ", 
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], "}"}]}], ";"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[1001]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"im", "[", "z_", "]"}], ":=", 
  RowBox[{"z", " ", "\[LongRightArrow]", 
   RowBox[{"InverseMellin", "[", 
    RowBox[{"z", ",", "y"}], "]"}]}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1002]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1003]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "y", ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1003]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1004]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], " ", 
     RowBox[{"log", "(", "y", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1004]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"PolyGamma", "[", 
   RowBox[{"0", ",", "OPEm"}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1005]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     TagBox["\[Psi]",
      PolyGamma], 
     RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]]}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "y"}],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], "-", 
     RowBox[{
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1005]="]
}, Open  ]],

Cell[BoxData[" "], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1006]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1007]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox["1", 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1007]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1008]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "-", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "2"}]]}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1008]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1009]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "+", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], "-", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "y"}], ")"}], " ", 
      SuperscriptBox["y", "m"]}], "+", 
     RowBox[{
      RowBox[{"log", "(", "y", ")"}], " ", 
      SuperscriptBox["y", "m"]}], "+", 
     SuperscriptBox["y", "m"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1009]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", 
   RowBox[{"OPEm", "^", "2"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1010]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", "y", ")"}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1010]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1011]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"log", "(", "y", ")"}], "-", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1011]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{"1", ",", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "^", "2"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1012]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["S", "1", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "y"}], ")"}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], "2"], "-", 
       RowBox[{"3", " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", 
        RowBox[{"1", "-", "y"}], ")"}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox[
          InterpretationBox["Li",
           PolyLog,
           Editable->False,
           Selectable->False], "2"], "(", "y", ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1012]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"2", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1013]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "2"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox[
        RowBox[{"log", "(", "y", ")"}], 
        RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1013]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", "OPEm"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1014]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", 
        RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1014]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"3", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1015]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}], " ", 
        TagBox[
         RowBox[{"\[Zeta]", "(", 
          TagBox["3",
           Zeta,
           Editable->True,
           Selectable->True], ")"}],
         InterpretTemplate[
          Function[BoxForm`e$, 
           Zeta[BoxForm`e$]]],
         Editable->False,
         Selectable->False]}], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "y"}], ")"}]}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1015]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1016]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "11"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1016]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1017]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True,
            Selectable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]],
          Editable->False,
          Selectable->False]}], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
       FractionBox[
        RowBox[{
         SubscriptBox[
          InterpretationBox["Li",
           PolyLog,
           Editable->False,
           Selectable->False], "2"], "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1017]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", "1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1018]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox[
         RowBox[{"\[Zeta]", "(", 
          TagBox["3",
           Zeta,
           Editable->True,
           Selectable->True], ")"}],
         InterpretTemplate[
          Function[BoxForm`e$, 
           Zeta[BoxForm`e$]]],
         Editable->False,
         Selectable->False], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
       FractionBox[
        RowBox[{
         SubscriptBox[
          InterpretationBox["Li",
           PolyLog,
           Editable->False,
           Selectable->False], "2"], "(", "y", ")"}], 
        RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1018]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1019]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "111"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1019]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"im", ",", "list"}], "]"}], ";"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1020]:="]
}, Open  ]],

Cell["Isolate", "ObjectName",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Isolate"],

Cell["\<\
Isolate[expr] substitutes abbreviations KK[i] for all Plus[...] (sub-sums) in \
expr. The inserted KK[i] have head HoldForm. Isolate[expr, varlist] \
substitutes KK[i] for all subsums in expr which are free of any occurence of \
a member of the list varlist. Instead of KK any other head or a list of names \
of the abbreviations may be specified with the option IsolateNames.\
\>", "Text",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Isolate", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1021]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"IsolateNames", "\[Rule]", "KK"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolatePrint`IsolatePrint", "\[Rule]", 
     "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolateSplit`IsolateSplit", "\[Rule]", 
     "\[Infinity]"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1021]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t0", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{"a", "+", "b"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1022]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"KK", "(", "1", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1022]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}], " ", "f"}], " ", "+", " ", "e"}], ",", 
    "f"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1023]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"e", "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "1", ")"}],
     HoldForm]}], "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "2", ")"}],
     HoldForm]}]}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1023]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t1", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1024]:="],

Cell[BoxData[
 RowBox[{"e", "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "1", "]"}],
    HoldForm]}], "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "2", "]"}],
    HoldForm]}]}]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1024]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t0", ",", " ", "t1", ",", " ", 
   RowBox[{"ReleaseHold", "[", "t1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1025]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     RowBox[{"KK", "(", "1", ")"}],
     HoldForm], ",", 
    RowBox[{"e", "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "1", ")"}],
       HoldForm]}], "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "2", ")"}],
       HoldForm]}]}], ",", 
    RowBox[{"e", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}], " ", "f"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1025]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "+", 
       RowBox[{"c", " ", 
        RowBox[{"(", 
         RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"d", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"y", "+", "z"}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "d"}], "}"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "fF"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1026]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"d", "(", "z", ")"}], " ", 
    TagBox[
     RowBox[{"fF", "(", "1", ")"}],
     HoldForm]}], "+", 
   RowBox[{
    RowBox[{"a", "(", "z", ")"}], " ", 
    TagBox[
     RowBox[{"fF", "(", "2", ")"}],
     HoldForm]}]}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1026]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "fF"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1027]:="],

Cell["Global`fF", "Print", "PrintUsage",
 CellTags->{"Info3387712053-5834299", "Isolate"}],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fF", "[", "1", "]"}], "=", 
         RowBox[{"y", "+", "z"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fF", "[", "2", "]"}], "=", 
         RowBox[{"b", "+", 
          RowBox[{"c", " ", 
           RowBox[{"\<\"HoldForm\"\>", "[", 
            RowBox[{"fF", "[", "1", "]"}], "]"}]}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition["fF"],
  Editable->False]], "Print",
 CellTags->{"Info3387712053-5834299", "Isolate"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"a", "-", "b", "-", "c", "-", "d", "-", "e"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}], ",", 
   RowBox[{"IsolateSplit", "\[Rule]", "15"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1028]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"L", "(", "3", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1028]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"L", "[", "2", "]"}], ",", 
   RowBox[{"L", "[", "1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1029]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "-", "b"}], ",", 
    RowBox[{"L", "(", "1", ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1029]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t0", ",", "t1", ",", "L", ",", "fF"}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1030]:="]
}, Open  ]],

Cell["IsolateHead", "ObjectName",
 CellTags->"IsolateHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateHead"],

Cell["IsolateHead is equivalent to IsolateNames.", "Text",
 CellTags->"IsolateHead"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"IsolateHead"]
}, Open  ]],

Cell["IsolateNames", "ObjectName",
 CellTags->"IsolateNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateNames"],

Cell["\<\
IsolateNames is an option for Isolate and Collect2. Its default setting is \
KK. Instead of a symbol the setting may also be a list with the names of the \
abbrevations.\
\>", "Text",
 CellTags->"IsolateNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"IsolateNames"]
}, Open  ]],

Cell["IsolatePrint", "ObjectName",
 CellTags->"IsolatePrint"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolatePrint"],

Cell["\<\
IsolatePrint is an option of Isolate. If it is set to OutputForm (or any \
other *Form) the definitions of the abbreviations are printed during the \
operation of Isolate. The setting IsolatePrint \[Rule] False suppresses \
printing.\
\>", "Text",
 CellTags->"IsolatePrint"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],

Cell["IsolateSplit", "ObjectName",
 CellTags->"IsolateSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateSplit"],

Cell["\<\
IsolateSplit is an option for Isolate. Its setting determines the maximum \
number of characters of FortranForm[expr] which are abbreviated by Isolate. \
If the expression is larger than the indicated number, it is split into \
smaller pieces and onto each subsum Isolate is applied. With the default \
setting IsolateSplit \[Rule] Infinity no splitting is done.\
\>", "Text",
 CellTags->"IsolateSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],

Cell["KeepOnly", "ObjectName",
 CellTags->"KeepOnly"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KeepOnly"],

Cell["\<\
KeepOnly is an option of OneLoop. It may be set to B0, C0, D0 keeping only \
the corresponding coefficients. The default setting is False. If KeepOnly is \
set to {} then the part of the amplitude which is not coefficient of B0, C0, \
D0 is kept.\
\>", "Text",
 CellTags->"KeepOnly"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"KeepOnly"]
}, Open  ]],

Cell["KK", "ObjectName",
 CellTags->"KK"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KK"],

Cell["\<\
KK[i] is the default setting of IsolateNames, which is the head of \
abbreviations used by Isolate. A KK[i] returned by Isolate is given in \
HoldForm and can be recovered by ReleaseHold[KK[i]].\
\>", "Text",
 CellTags->"KK"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"KK"]
}, Open  ]],

Cell["Kummer", "ObjectName",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Kummer"],

Cell["\<\
Kummer[i][exp] applies Kummer relation number i (i =1, ... 24, 94,95,96) to \
all Hypergeometric2F1 in exp. i = 94 corresponds to eq. 9.131.2, i = 95 to \
eq. 9.132.1 and i = 96 to eq. 9.132.2 in Gradsteyn & Ryzhik.\
\>", "Text",
 CellTags->"Kummer"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 "."
}], "Text",
 CellTags->"Kummer"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "2", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1031]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox["z",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1031]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "3", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1032]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox["z", 
            RowBox[{"1", "-", "z"}]]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1032]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "4", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1033]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox["z", 
            RowBox[{"1", "-", "z"}]]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1033]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1034]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1034]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1035]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"a", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"b", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1035]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1036]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1036]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1037]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"a", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1037]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1038]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1038]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1039]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"b", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1039]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1040]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", "z"],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1040]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1041]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "-", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "-", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", "z"],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1041]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1042]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1042]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1043]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "-", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "-", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1043]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1044]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1044]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1045]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"a", "-", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"1", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"a", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"a", "-", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1045]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1046]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", "z"],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1046]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1047]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "+", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", "z"],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1047]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1048]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1048]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1049]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "+", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1049]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1050]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1050]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1051]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         FractionBox["1", "z"],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "c", "+", "1"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"b", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "+", "b", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["1", 
           RowBox[{"1", "-", "z"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1051]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1052]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"2", "-", "c"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"2", "-", "c"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox["z",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1052]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1053]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox["z",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1053]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1054]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"2", "-", "c"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"a", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"2", "-", "c"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["z", 
           RowBox[{"z", "-", "1"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1054]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1055]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["z", 
           RowBox[{"z", "-", "1"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1055]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1056]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"a", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"b", "-", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{"2", "-", "c"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"b", "-", "c", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"2", "-", "c"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["z", 
           RowBox[{"z", "-", "1"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1056]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1057]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["z", 
           RowBox[{"z", "-", "1"}]],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1057]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1058]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"c", "-", "a"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"c", "-", "b"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1058]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1059]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1059]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1060]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"c", "-", "a"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"c", "-", "b"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"a", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", 
           FractionBox["1", 
            RowBox[{"1", "-", "z"}]]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1060]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1061]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"1", "-", 
           FractionBox["1", 
            RowBox[{"1", "-", "z"}]]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1061]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1062]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox[
            RowBox[{"c", "-", "a"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox[
            RowBox[{"c", "-", "b"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox[
           RowBox[{
            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"b", "-", "c"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"c", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", "b"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{
             RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1062]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1063]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         RowBox[{"1", "-", "z"}],
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"c", "-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox[
            RowBox[{"1", "-", "z"}], "z"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1063]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "94", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1064]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"a", "+", "b", "-", "c"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox[
               RowBox[{"c", "-", "a"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"c", "-", "b"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{
               RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            RowBox[{"1", "-", "z"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c"}]]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "b", ")"}]}]], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c"}], ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["a",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox["b",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            RowBox[{"1", "-", "z"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "a"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1064]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "95", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1065]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"b", "-", "a"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["a",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"c", "-", "b"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{"a", "-", "b", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            FractionBox["1", 
             RowBox[{"1", "-", "z"}]],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "a"}]]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["b",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"c", "-", "a"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{
               RowBox[{"-", "a"}], "+", "b", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            FractionBox["1", 
             RowBox[{"1", "-", "z"}]],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "b"}]]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1065]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "96", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1066]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TagBox[
    TagBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
       SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
      RowBox[{"(", 
       RowBox[{
        TagBox[
         TagBox[
          RowBox[{
           TagBox["a",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True], ",", 
           TagBox["b",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox[
         TagBox[
          TagBox["c",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True],
          InterpretTemplate[{
            SlotSequence[1]}& ]],
         Hypergeometric2F1,
         Editable->False,
         Selectable->False], ";", 
        TagBox["z",
         Hypergeometric2F1,
         Editable->True,
         Selectable->True]}], ")"}]}],
     InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
     Editable->False,
     Selectable->False],
    Hypergeometric2F1], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "a"], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"b", "-", "a"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["a",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"a", "-", "c", "+", "1"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{"a", "-", "b", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            FractionBox["1", "z"],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "a"}]]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "b"], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
          SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["b",
               Hypergeometric2F1,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"b", "-", "c", "+", "1"}],
               Hypergeometric2F1,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             TagBox[
              RowBox[{
               RowBox[{"-", "a"}], "+", "b", "+", "1"}],
              Hypergeometric2F1,
              Editable->True,
              Selectable->True],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            Hypergeometric2F1,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            FractionBox["1", "z"],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       Hypergeometric2F1], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "b"}]]}], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1066]="]
}, Open  ]]
}, Open  ]],

Cell["Lagrangian", "ObjectName",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lagrangian"],

Cell["\<\
Lagrangian[\"oqu\"] gives the unpolarized OPE quark operator. \
Lagrangian[\"oqp\"] gives the polarized quark OPE operator. \
Lagrangian[\"ogu\"] gives the unpolarized gluon OPE operator. \
Lagrangian[\"ogp\"] gives the polarized gluon OPE operator. \
Lagrangian[\"ogd\"] gives the sigma-term part of the QCD lagrangian. \
Lagrangian[\"QCD\"] gives the gluon self interaction part of the QCD \
lagrangian.\
\>", "Text",
 CellTags->"Lagrangian"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"Lagrangian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1067]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1067]="]
}, Open  ]],

Cell["Twist-2 operator product expansion operators", "Text",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1068]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1068]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1069]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1069]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1070]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1070]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1071]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1071]="]
}, Open  ]]
}, Open  ]],

Cell["LC", "ObjectName",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LC"],

Cell["\<\
LC[m,n,r,s] evaluates to 4-dimensional LeviCivita[m,n,r,s] by virtue of \
applying FeynCalcInternal. LC[m,...][p, ...] evaluates to 4-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1072]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1072]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1073]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1073]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1074]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1074]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LC", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1075]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1075]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1076]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1076]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], " "}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1077]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"18", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "-", 
   RowBox[{"24", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1077]="]
}, Open  ]]
}, Open  ]],

Cell["LCD", "ObjectName",
 CellTags->"LCD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LCD"],

Cell["\<\
LCD[m,n,r,s] evaluates to D-dimensional LeviCivita[m,n,r,s] by virtue of \
FeynCalcInternal. LCD[m,...][p, ...] evaluates to D-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LCD"],

Cell["\<\
You need to do SetOptions[Eps, Dimension\[Rule]D] before LCD can be used as \
D-dimensional Levi-Civita input function.\
\>", "Text",
 CellTags->"LCD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 "."
}], "Text",
 CellTags->"LCD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LCD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1078]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LCD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1079]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1079]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1080]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1080]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1081]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1081]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LCD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1082]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1082]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1083]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1083]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1084]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1084]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Eps", ",", 
   RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1085]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1085]="]
}, Open  ]]
}, Open  ]],

Cell["LeftPartialD", "ObjectName",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "LeftPartialD[\[Mu]] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 "acting to the left."
}], "Text",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1086]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1086]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1087]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1087]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1088]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1088]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"LeftPartialD", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1089]:="],

Cell[BoxData[
 RowBox[{"LeftPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1089]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Rho]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1090]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1090]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1091]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1091]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1092]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1092]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD", "ObjectName",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "LeftRightPartialD[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives 1/2 (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] - LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1093]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1093]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1094]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1094]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1095]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1095]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1096]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[Null]"]}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1096]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1097]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1097]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1098]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}]}], "-", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1098]="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD2", "ObjectName",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "LeftRightPartialD2[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD2[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] + LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1099]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1099]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1100]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1101]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox["A", "\[Nu]"]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD2", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Rho]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1104]="]
}, Open  ]]
}, Open  ]],

Cell["LeviCivita ", "ObjectName",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[TextData[{
 "LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "\[Mu]", ",", " ", "\[Nu]", ",", " ", "\[Rho]", ",", " ", "\[Sigma]"}], 
   TraditionalForm]]],
 "] is an input function for the totally antisymmetric Levi-Civita tensor. It \
evaluates automatically to the internal representation Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox[
   FormBox["\[Rho]",
    TraditionalForm], TraditionalForm]]],
 "], LorentzIndex[\[Sigma]]] (or with a second argument in LorentzIndex for \
the Dimension, if the option Dimension of LeviCivita is changed). \
LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    RowBox[{"\[Nu]", " ", "..."}]}], TraditionalForm]]],
 "][ ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", ",", " ", ".."}], TraditionalForm]]],
 ".] evaluates to Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], ..., Momentum[",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "], ...]."
}], "Text",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "LeviCivita", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1105]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LeviCivita"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1106]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Gamma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1106]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"OPEDelta", ",", "p"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1107]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", "]"}], "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1108]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm], 
    FormBox["r",
     TraditionalForm], 
    FormBox["s",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Beta]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1109]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1109]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"OPEDelta", ",", "p"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1110]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1110]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}], ".", 
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}]}], " "}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1111]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1111]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1112]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1113]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", " ", 
   RowBox[{
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}], ".", 
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}]}], "]"}], "//", "Factor2"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1114]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1114]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1115]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1116]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Gamma]", "\[Delta]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1116]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1117]:="]
}, Open  ]],

Cell["LeviCivitaSign", "ObjectName",
 CellTags->"LeviCivitaSign"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivitaSign"],

Cell["\<\
LeviCivitaSign is an option for DiracTrace and EpsChisholm. It determines the \
sign in the result of a Dirac trace of four gamma matrices and gamma5.\
\>", "Text",
 CellTags->"LeviCivitaSign"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 "."
}], "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],

Cell["ListIntegrals  ***unfinished***", "ObjectName",
 CellTags->"ListIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell["\<\
ListIntegrals[exp, {q1, q2}] gives a list of basic integrals  \
(FeynAmpDenominator's multiplied by scalar products involving q1, q2).  Any \
non-integer exponent (e.g. the (OPEm-1) in  \
ScalarProduct[OPEDelta,q1]^(OPEm-1)) is replaced by OPEm.  If the options \
Pair is set to True only integrals involving   scalar products (not counting \
those with OPEDelta) are given; with Pair \[Rule] False those are excluded.\
\>", "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"ListIntegrals",
 CellLabel->"In[1118]:="]
}, Open  ]],

Cell["Li2 ", "ObjectName",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li2"],

Cell["\<\
 Li2 is an abbreviation for the dilog function, i.e., Li2 = PolyLog[2, #]&.\
\>", "Text",
 CellTags->"Li2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li3",
  ButtonNote->"Li3"],
 "."
}], "Text",
 CellTags->"Li2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "[", "x", "]"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1119]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    InterpretationBox["Li",
     PolyLog,
     Editable->False,
     Selectable->False], "2"], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1119]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "//", "StandardForm"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1120]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "#1"}], "]"}], "&"}]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1120]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "x"], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1121]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    InterpretationBox["Li",
     PolyLog,
     Editable->False,
     Selectable->False], "2"], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1121]="]
}, Open  ]]
}, Open  ]],

Cell["Li3 ", "ObjectName",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li3"],

Cell["\<\
 Li3 is an abbreviation for the trilog function, i.e., Li3 = PolyLog[3, #]&.\
\>", "Text",
 CellTags->"Li3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li2",
  ButtonNote->"Li2"],
 "."
}], "Text",
 CellTags->"Li3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "[", "x", "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1122]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    InterpretationBox["Li",
     PolyLog,
     Editable->False,
     Selectable->False], "3"], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1122]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "//", "StandardForm"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1123]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", "#1"}], "]"}], "&"}]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1123]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Li3", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1124]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", "x", ")"}], "x"], 
  TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1124]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox[
    RowBox[{"Li2", "[", "x", "]"}], "x"], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1125]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    InterpretationBox["Li",
     PolyLog,
     Editable->False,
     Selectable->False], "3"], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1125]="]
}, Open  ]]
}, Open  ]],

Cell["Loop", "ObjectName",
 CellTags->"Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Loop"],

Cell["Loop is an option indicating the number of (virtual) loops.", "Text",
 CellTags->"Loop"]
}, Open  ]],

Cell["LorentzIndex", "ObjectName",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["\<\
LorentzIndex[mu] denotes a four dimensional Lorentz index. For other than \
four dimensions: LorentzIndex[mu, D] or LorentzIndex[mu], etc. \
LorentzIndex[mu, 4] simplifies to LorentzIndex[mu].\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChangeDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChangeDimension",
  ButtonNote->"ChangeDimension"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 "."
}], "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["This denotes a four-dimensional Lorentz index.", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1126]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1126]="]
}, Open  ]],

Cell["\<\
An optional second argument can be given for a dimension different from 4.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", 
  RowBox[{"\[Alpha]", ",", "n"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1127]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1127]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["LorentzIndex[ComplexIndex[\[Alpha]]]", "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1128]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Alpha]", "*"], TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1129]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", 
   RowBox[{"ComplexIndex", "[", "\[Beta]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1130]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Beta]", "*"], TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1130]="]
}, Open  ]],

Cell["\<\
Setting $LorentzIndices=True displays the dimension (if different from 4) as \
a subindex.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1131]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "n"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "4"}], "]"}]}], "}"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1132]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     FormBox["\[Alpha]",
      TraditionalForm], 
     FormBox["D",
      TraditionalForm]], ",", 
    SubscriptBox[
     FormBox["\[Nu]",
      TraditionalForm], 
     FormBox["n",
      TraditionalForm]], ",", "\[Beta]"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"LorentzIndex",
 CellLabel->"Out[1132]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1133]:="]
}, Open  ]],

Cell["Lower", "ObjectName",
 CellTags->"Lower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lower"],

Cell["\<\
Lower may be used inside LorentzIndex to indicate an covariant LorentzIndex.\
\>", "Text",
 CellTags->"Lower"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Upper",
  ButtonNote->"Upper"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Lower"]
}, Open  ]],

Cell["MakeContext", "ObjectName",
 CellTags->"MakeContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MakeContext"],

Cell["\<\
MakeContext[string] constructs the context path of string. MakeContext is \
invoked at startup of FeynCalc. MakeContext[a, b] construct the context path \
of b defined in the context of a.\
\>", "Text",
 CellTags->"MakeContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["CheckContext",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CheckContext",
  ButtonNote->"CheckContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"MakeContext"]
}, Open  ]],

Cell["Mandelstam", "ObjectName",
 CellTags->"Mandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "Mandelstam is an option for DiracTrace, OneLoop, OneLoopSum, Tr and \
TrickMandelstam.  A typical setting is Mandelstam \[Rule] {",
 "s",
 ", ",
 "t",
 ", ",
 "u",
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 "}, which stands for  ",
 "s",
 " + ",
 "t",
 " + ",
 "u",
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 ". If other than four-particle processes are calculated the setting should \
be: Mandelstam \[Rule] {}."
}], "Text",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["TrickMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TrickMandelstam",
  ButtonNote->"TrickMandelstam"],
 ", ",
 ButtonBox["SetMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SetMandelstam",
  ButtonNote->"SetMandelstam"],
 "."
}], "Text",
 CellTags->"Mandelstam"]
}, Open  ]],

Cell["Map2", "ObjectName",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Map2"],

Cell["\<\
Map2[f, exp] is equivalent to Map if Nterms[exp] > 0, otherwise Map2[f, exp] \
gives f[exp].\
\>", "Text",
 CellTags->"Map2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NTerms",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NTerms",
  ButtonNote->"NTerms"],
 "."
}], "Text",
 CellTags->"Map2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"a", "-", "b"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1134]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "a", ")"}], "+", 
   RowBox[{"f", "(", 
    RowBox[{"-", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1134]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "x"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1135]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "1"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1137]="]
}, Open  ]]
}, Open  ]],

Cell["MemoryAvailable", "ObjectName",
 CellTags->"MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemoryAvailable"],

Cell["\<\
MemoryAvailable is an option of MemSet. It can be set to an integer n, where \
n is the available amount of main memory in Mega Byte. The default setting is \
$MemoryAvailable.\
\>", "Text",
 CellTags->"MemoryAvailable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemSet",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MemSet",
  ButtonNote->"MemSet"],
 ", ",
 ButtonBox["$MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$MemoryAvailable",
  ButtonNote->"$MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],

Cell["MemSet", "ObjectName",
 CellTags->"MemSet"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemSet"],

Cell[TextData[{
 "MemSet[f[x_], body] is like f[x_] := f[x] = body, but dependend on the \
value of the setting of MemoryAvailable \[Rule] memorycut (memorycut - \
MemoryInUse[]/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", "6"], TraditionalForm]]],
 ") MemSet[f[x_], body] may evaluate as f[x_] := body."
}], "Text",
 CellTags->"MemSet"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MemoryAvailable",
  ButtonNote->"MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemSet"]
}, Open  ]],

Cell["MetricTensor", "ObjectName",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "MetricTensor[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor. The default dimension is 4."
}], "Text",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MetricTensor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1138]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1138]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1139]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1139]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
   RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1140]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1141]:="],

Cell[BoxData[
 FormBox["n", TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1142]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1142]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1143]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"a", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1143]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1144]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1144]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1145]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1145]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MLimit", "ObjectName",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MLimit"],

Cell["\<\
MLimit[expr, lims] takes multiple limits of expr using the limits lims.\
\>", "Text",
 CellTags->"MLimit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MLimit", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"x", "-", "1"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], ",", 
   RowBox[{"x", "\[Rule]", "1"}], ",", 
   RowBox[{"y", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"MLimit",
 CellLabel->"In[1146]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"MLimit", "(", 
   RowBox[{
    RowBox[{
     RowBox[{"y", " ", 
      RowBox[{"log", "(", "y", ")"}]}], "-", 
     FractionBox[
      RowBox[{"sin", "(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"x", "-", "1"}]]}], ",", 
    RowBox[{"x", "\[Rule]", "1"}], ",", 
    RowBox[{"y", "\[Rule]", "0"}]}], ")"}], TraditionalForm]], "Output",
 CellTags->"MLimit",
 CellLabel->"Out[1146]="]
}, Open  ]]
}, Open  ]],

Cell["Momentum", "ObjectName",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Momentum"],

Cell["\<\
Momentum[p] is the head of a four momentum (p). The internal representation \
of a four-dimensional p is Momentum[p]. For other than four dimensions: \
Momentum[p, dim]. Momentum[p, 4] simplifies to Momentum[p].\
\>", "Text",
 CellTags->"Momentum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 "."
}], "Text",
 CellTags->"Momentum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Momentum"],

Cell["This is a four dimensional momentum.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", "p", "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1147]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1147]="]
}, Open  ]],

Cell["\<\
As an optional second argument the dimension must be specified if it is \
different from 4.\
\>", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "D"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1148]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1148]="]
}, Open  ]],

Cell["The dimension index is supressed in the output.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", ",", "d"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1149]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1149]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"Momentum", "[", 
   RowBox[{"-", "q"}], "]"}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1150]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "q"}], TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1150]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1151]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1151]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], " ", "+", " ", 
   RowBox[{"Momentum", "[", 
    RowBox[{"2", "q"}], "]"}]}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1152]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "-", "q"}], ")"}], "+", 
   RowBox[{"2", " ", "q"}]}], TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1153]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1153]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumExpand"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1154]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1154]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1155]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1155]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1156]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1156]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1157]:="]
}, Open  ]],

Cell["MomentumCombine", "ObjectName",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell["\<\
MomentumCombine[expr] is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine combines also Pair`s.\
\>", "Text",
 CellTags->"MomentumCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine2",
  ButtonNote->"MomentumCombine2"],
 "."
}], "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"Momentum", "[", "q", "]"}]}]}], " ", "//", " ", 
   "MomentumCombine"}], " ", "//", " ", "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1158]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1158]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "+", " ", 
   RowBox[{"2", " ", 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", "\[Mu]"}], "]"}]}], " "}]}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1159]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1159]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumCombine", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1161]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1161]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "ExpandScalarProduct"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1162]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1162]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1163]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1163]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1164]:="]
}, Open  ]],

Cell["MomentumCombine2", "ObjectName",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell["\<\
MomentumCombine2[expr]  is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine2 combines also FourVectors.\
\>", "Text",
 CellTags->"MomentumCombine2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 "."
}], "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"3", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1165]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"3", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1165]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine2", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1166]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{
     RowBox[{"3", " ", "p"}], "+", 
     RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output",
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1166]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MomentumExpand", "ObjectName",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell["\<\
MomentumExpand[expr] expands Momentum[a+b+ ...] in expr into Momentum[a] + \
Momentum[b] + ... .\
\>", "Text",
 CellTags->"MomentumExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumExpand", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1167]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1167]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", "r"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1168]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["r",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1168]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1169]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1169]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", "r"}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1170]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox["r",
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1170]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1171]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1171]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", 
    RowBox[{"r", "-", "p"}]}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1172]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox[
     RowBox[{"r", "-", "p"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1173]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "+", 
    RowBox[{"Momentum", "[", "r", "]"}]}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1173]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1174]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1174]="]
}, Open  ]]
}, Open  ]],

Cell["MT", "ObjectName",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MT"],

Cell[TextData[{
 "MT[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in 4 dimensions."
}], "Text",
 CellTags->"MT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1175]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1176]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1177]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1177]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1178]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1178]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MT", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1179]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1179]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MTD", "ObjectName",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MTD"],

Cell[TextData[{
 "MTD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in D dimensions."
}], "Text",
 CellTags->"MTD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 "."
}], "Text",
 CellTags->"MTD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1180]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1180]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1181]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1182]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1182]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1183]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1183]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1184]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1184]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["NegativeInteger", "ObjectName",
 CellTags->"NegativeInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NegativeInteger"],

Cell["\<\
NegativeInteger is a data type. E.g. DataType[n, NegativeInteger] can be set \
to True.\
\>", "Text",
 CellTags->"NegativeInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],

Cell["Nf", "ObjectName",
 CellTags->"Nf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nf"],

Cell["Nf denotes the number of flavors.", "Text",
 CellTags->"Nf"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 ", ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 ", ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 "."
}], "Text",
 CellTags->"Nf"]
}, Open  ]],

Cell["Nielsen", "ObjectName",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nielsen"],

Cell["Nielsen[i,j, x] denotes Nielsen's polylogarithm.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Nielsen", "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1185]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PolyLog", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Nielsen",
 CellLabel->"Out[1185]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Nielsen"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{
     FormBox["1",
      TraditionalForm], 
     FormBox["2",
      TraditionalForm]}]], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1186]="]
}, Open  ]],

Cell["\<\
Numerical evaluation is done via N[Nielsen[n_,p_,x_]] := \
(-1)^(n+p-1)/(n-1)!/p! NIntegrate[Log[1-x t]^p Log[t]^(n-1)/t,{t,0,1}];\
\>", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", ".45"}], "]"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1187]:="],

Cell[BoxData[
 FormBox["0.07287162612104658`", TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1187]="]
}, Open  ]],

Cell["Some special values are built in.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "0"}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"-", "1"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"1", "/", "2"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1188]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0", ",", 
    FractionBox[
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False], "8"], ",", 
    FractionBox[
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False], "8"], ",", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1188]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "+", 
   TagBox[
    RowBox[{"\[Zeta]", "(", 
     TagBox["3",
      Zeta,
      Editable->True,
      Selectable->True], ")"}],
    InterpretTemplate[
     Function[BoxForm`e$, 
      Zeta[BoxForm`e$]]],
    Editable->False,
    Selectable->False]}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "3", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "6"]}], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "4"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1190]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"3", ",", "1", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1191]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    InterpretationBox["Li",
     PolyLog,
     Editable->False,
     Selectable->False], "4"], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1191]="]
}, Open  ]]
}, Open  ]],

Cell["NonCommFreeQ", "ObjectName",
 CellTags->"NonCommFreeQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommFreeQ"],

Cell["\<\
NonCommFreeQ[exp] yields True if exp contains no non-commutative objects \
(i.e. those objects which are listed in $NonComm) or only non-commutative \
objects inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommFreeQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],

Cell["NonCommQ", "ObjectName",
 CellTags->"NonCommQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommQ"],

Cell["\<\
NonCommQ[exp] yields True if exp contains non-commutative objects (i.e. those \
objects which are listed in $NonComm) not inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommFreeQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommFreeQ",
  ButtonNote->"NonCommFreeQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NonCommutative", "ObjectName",
 CellTags->"NonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommutative"],

Cell["\<\
NonCommutative is a data type which may be used, e.g.,  as: DataType[x, \
NonCommutative] = True.\
\>", "Text",
 CellTags->"NonCommutative"],

Cell[TextData[{
 "See also:",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"NonCommutative"]
}, Open  ]],

Cell["NTerms", "ObjectName",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NTerms"],

Cell["\<\
NTerms[x] is equivalent to Length if x is a sum; otherwise NTerms[x] returns \
1, except NTerms[0] \[Rule] 0.\
\>", "Text",
 CellTags->"NTerms"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", "-", "b"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1192]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1192]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", " ", "b", " ", "c"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1193]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1193]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "9", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1194]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "0", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1195]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1195]="]
}, Open  ]]
}, Open  ]],

Cell["NumberOfMetricTensors", "ObjectName",
 CellTags->"NumberOfMetricTensors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumberOfMetricTensors"],

Cell["NumberOfMetricTensors is an option of Tdec.", "Text",
 CellTags->"NumberOfMetricTensors"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tdec",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tdec",
  ButtonNote->"Tdec"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NumericalFactor", "ObjectName",
 CellTags->"NumericalFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericalFactor"],

Cell["\<\
NumericalFactor[expr] gives the overall numerical factor of expr.\
\>", "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],

Cell["NumericQ1", "ObjectName",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericQ1"],

Cell["\<\
NumericQ1[x,{a,b,..}] is like NumericQ, but assumes that {a,b,..} are numeric \
quantities.\
\>", "Text",
 CellTags->"NumericQ1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ", "[", 
  RowBox[{
   RowBox[{"3", " ", "a"}], "+", 
   RowBox[{"Log", "[", "b", "]"}], "+", 
   SuperscriptBox["c", "2"]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1196]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1196]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1197]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1198]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1198]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OneLoop  ***unfinished***", "ObjectName",
 CellTags->"OneLoop"],

Cell[TextData[{
 "See also the old FeynCalc documentation at ",
 ButtonBox["http://www.mertig.com/oldfc/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc/"], None}],
 ". NOTICE: While OneLoop is restricted to 't Hooft Feynman gauge the \
function ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 " does not have this restriction (but is usually slower). OneLoop handles \
selfenergies, vertex and box-graphs (those only up to 3rd rank tensor in the \
integration variable)."
}], "Text",
 CellTags->"OneLoop"],

Cell[TextData[{
 "WARNING: If you encounter anomalies: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "The", " ", "default", " ", "setting", " ", "of", " ", "$West", " ", "is",
      " ", "True"}], ",", " ", 
    RowBox[{"i", ".", "e", "."}], ",", " ", 
    RowBox[{"the", " ", "way", " ", 
     RowBox[{"tr", "(", 
      RowBox[{
       SuperscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["\[Gamma]", "\[Nu]"], 
       SuperscriptBox["\[Gamma]", "\[Rho]"], 
       SuperscriptBox["\[Gamma]", "\[Sigma]"], 
       SuperscriptBox["\[Gamma]", "\[Tau]"], 
       SuperscriptBox["\[Gamma]", "\[Lambda]"], 
       SuperscriptBox["\[Gamma]", "5"]}], ")"}]}]}], TraditionalForm]]],
 " is calculated in D dimensions has changed compared to the old FeynCalc \
version. Please keep in mind that the issue of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes is inherintly tricky."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoop"],

Cell[TextData[{
 "OneLoop[",
 "q",
 ", amplitude] calculates the one-loop Feynman diagram amplitude (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-point, where ",
 StyleBox["n",
  FontSlant->"Italic"],
 "\[LessEqual]4 and the highest tensor rank of the integration momenta (after \
cancellation of scalar products) may be 3; unless OneLoopSimplify is used)."
}], "Text",
 CellTags->"OneLoop"],

Cell["\<\
The argument q denotes the integration variable, i.e., the loop momentum. \
OneLoop[name, q, amplitude] has as first argument a name of the amplitude. If \
the second argument has head FeynAmp then OneLoop[q, FeynAmp[name, k, expr]] \
and OneLoop[FeynAmp[name, k, expr]] tranform to OneLoop[name, k, expr].\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoop", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1199]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Apart2", "\[Rule]", "True"}], ",", 
    RowBox[{"CancelQP", "\[Rule]", "True"}], ",", 
    RowBox[{"DenominatorOrder", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"InitialSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntermediateSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OneLoopSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Prefactor", "\[Rule]", "1"}], ",", 
    RowBox[{"ReduceGamma", "\[Rule]", "False"}], ",", 
    RowBox[{"ReduceToScalars", "\[Rule]", "False"}], ",", 
    RowBox[{"SmallVariables", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOut", "\[Rule]", "False"}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}], ",", 
    RowBox[{"Sum", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1199]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["$LimitTo4",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$LimitTo4",
  ButtonNote->"$LimitTo4"],
 "."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "I"}], "/", 
    RowBox[{"Pi", "^", "2"}]}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["\[ImaginaryI]", 
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1200]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1201]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"mf", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"mf", ",", "TraditionalForm"}], "]"}], "=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "\"\<f\>\""}], "]"}], ",", "mf"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1202]:="],

Cell[TextData[{
 "Remember that FAD[{q,mf},{q-k,mf}] is a fast possibility to enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["q", "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}], " ", 
       RowBox[{"(", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}]}], " ", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"I", " ", 
    FractionBox[
     SuperscriptBox["el", "2"], 
     RowBox[{"16", " ", 
      SuperscriptBox["Pi", "4"], 
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}]}]], 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "mf"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "k"}], ",", "mf"}], "}"}]}], "]"}], 
    RowBox[{"DiracTrace", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", 
         RowBox[{"q", "-", "k"}], "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}]}], "]"}]}], " ", "//", " ", 
   "FCI"}]}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1203]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q", "-", "k"}],
           TraditionalForm], ")"}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], ".", 
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], 
   RowBox[{"16", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "D"}], ")"}], " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox[
        InterpretationBox[
         SubscriptBox["m", "f"],
         $CellContext`mf], "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
       SuperscriptBox[
        InterpretationBox[
         SubscriptBox["m", "f"],
         $CellContext`mf], "2"]}], ")"}]}]}]], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1203]="]
}, Open  ]],

Cell["\<\
The input to OneLoop may be in 4 dimensions, since the function changes the \
dimension of the objects automatically to the setting of the Dimension option \
(D by default).\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "t"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1204]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["el", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["8", "3"]}], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"], ",", 
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"], ",", 
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"], ")"}], " ", 
        SuperscriptBox[
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "2"]}], "-", 
       FractionBox[
        RowBox[{"8", " ", 
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"]}], "3"], "+", 
       RowBox[{
        FractionBox["8", "3"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], "2"],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{
        FractionBox["4", "3"], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"], ",", 
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"], ",", 
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"], ")"}], " ", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"]}], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], "9"]}], ")"}]}], 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1204]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1205]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"6", " ", 
       SuperscriptBox[
        InterpretationBox[
         SubscriptBox["m", "f"],
         $CellContext`mf], "2"]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], "2"],
         TraditionalForm], ")"}]}], "-", 
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], "2"], "+", 
      RowBox[{"3", " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"], ",", 
        SuperscriptBox[
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "2"], ",", 
        SuperscriptBox[
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "2"], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], "2"]}], "+", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], ")"}]}]}], ")"}]}], 
   RowBox[{"36", " ", 
    SuperscriptBox["\[Pi]", "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1205]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1206]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1206]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"k", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1207]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         SubscriptBox["B", "0"], "(", "0", ",", 
         SuperscriptBox["m", "2"], ",", 
         SuperscriptBox["m", "2"], ")"}], " ", 
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        SuperscriptBox["m", "2"]}], 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], ",", "0", ",", 
       SuperscriptBox["m", "2"], ")"}], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1207]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1208]:="]
}, Open  ]],

Cell["OneLoopSimplify", "ObjectName",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell["\<\
OneLoopSimplify[amp, q] simplifies the one-loop amplitude amp. The second \
argument denotes the integration momentum.\
\>", "Text",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoopSimplify", "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1209]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSimplify", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntegralTable", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OPE1Loop", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1209]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1210]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1210]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1211]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"m", "\[Rule]", "0"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1212]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1212]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1213]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1213]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", " ", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1214]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "1", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "1", "2"], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "1", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FCE", "[", "%", "]"}], "/.", 
   RowBox[{
    RowBox[{"SPD", "[", 
     SubscriptBox["p", "1"], "]"}], "\[Rule]", "0"}]}], "//", 
  "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", "2", "2"]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    SubsuperscriptBox["p", 
     FormBox["1",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubsuperscriptBox["p", 
        FormBox["1",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ",", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"k", ",", "l"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["l",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox["l",
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["l",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1216]="]
}, Open  ]]
}, Open  ]],

Cell["OneLoopSum", "ObjectName",
 CellTags->"OneLoopSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSum"],

Cell["\<\
OneLoopSum[ FeynAmp[ ... ], FeynAmp[ ... ] , ...] will calculate a list of \
Feynman amplitudes by replacing FeynAmp step by step by OneLoop.\
\>", "Text",
 CellTags->"OneLoopSum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],

Cell["OPE", "ObjectName",
 CellTags->"OPE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE"],

Cell["\<\
OPE is a convenience variable to separate OPE insertions.
OPE is also an option of several input functions like GluonPropagator.\
\>", "Text",
 CellTags->"OPE"],

Cell["\<\
The OPE variable has the property to vanish for higher powers:\
\>", "Text"],

Cell[CellGroupData[{

Cell["{OPE,OPE^2,OPE^3}", "Input",
 CellLabel->"In[1217]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\[CapitalOmega]", ",", "0", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1217]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"OPE"]
}, Open  ]],

Cell["OPEDelta", "ObjectName",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEDelta"],

Cell["\<\
OPEDelta is a lightlike axial vector as used e.g. in the operator product \
expansion in QCD.\
\>", "Text",
 CellTags->"OPEDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"OPEDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1218]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1218]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1219]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1219]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"OPEDelta", ",", "OPEDelta"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1220]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1220]="]
}, Open  ]]
}, Open  ]],

Cell["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo", "ObjectName",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEi"],

Cell["\<\
OPEi, etc. are variables with DataType PositiveInteger which are used in \
functions relating to the operator product expansion.\
\>", "Text",
 CellTags->"OPEi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData["OPEi"], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1221]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1221]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{
  "OPEi", ",", " ", "OPEj", ",", "OPEk", ",", "OPEl", ",", " ", "OPEm", ",", 
   " ", "OPEn", ",", " ", "OPEo", ",", " ", "PositiveInteger"}], 
  "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1222]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", "True", 
    ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1222]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEi"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEj"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEk"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEl"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEm"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEn"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEo"}], ")"}]}]}], "}"}], "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1223]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
   "}"}], TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1223]="]
}, Open  ]],

Cell["Re has been changed:", "Text",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1224]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1224]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "1"}], ",", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "2"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "1"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "2"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1226]="]
}, Open  ]]
}, Open  ]],

Cell["OPEInsert  ***unfinished***", "ObjectName",
 CellTags->"OPEInsert"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInsert"],

Cell["\<\
OPEInsert[diagram_String, name_, rhifile_String] or OPEInsert[diagram_String, \
name_]. The setting of the option EpsContract (4 or D) determines whether the \
Levi-Civita tensors are contracted in 4 or D dimensions.\
\>", "Text",
 CellTags->"OPEInsert"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInsert"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInsert"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInsert",
 CellLabel->"In[1227]:="]
}, Open  ]],

Cell["OPEInt  ***unfinished***", "ObjectName",
 CellTags->"OPEInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInt"],

Cell["\<\
OPEInt[expr, q, p, x].  The dimension is changed to the one indicated by the \
option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEInt"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInt"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInt",
 CellLabel->"In[1228]:="]
}, Open  ]],

Cell["OPEIntDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "OPEIntDelta[expr, x, m] introduces the delta(1-x) (DeltaFunction[1-x]). The \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntDelta",
 CellLabel->"In[1229]:="]
}, Open  ]],

Cell["OPEIntegrate  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell["\<\
OPEIntegrate[expr, q, x].  The dimension is changed to the one indicated by \
the option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrate",
 CellLabel->"In[1230]:="]
}, Open  ]],

Cell["OPEIntegrateDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrateDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "OPEIntegrateDelta[expr, x, m] introduces the \[Delta](1-x) \
(DeltaFunction[1-x]). The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntegrateDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrateDelta",
 CellLabel->"In[1231]:="]
}, Open  ]],

Cell["OPESumExplicit", "ObjectName",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell["OPESumExplicit[exp] calculates OPESum's.", "Text",
 CellTags->"OPESumExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"A", "^", "i"}], 
     RowBox[{"B", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "i", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", 
      RowBox[{"m", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1232]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{"m", "-", "3"}]], 
    RowBox[{
     SuperscriptBox["A", "i"], " ", 
     SuperscriptBox["B", 
      RowBox[{
       RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1232]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1233]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["A", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox["B", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1233]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1234]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1234]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1235]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"c", " ", 
      FormBox[
       SuperscriptBox["a", 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"a", "-", "c"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["a", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1235]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1236]:="]
}, Open  ]],

Cell["OPESum", "ObjectName",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESum"],

Cell["\<\
OPESum[exp, {i, 0, m}] denotes a symbolic sum.The syntax is the same as for \
Sum.\
\>", "Text",
 CellTags->"OPESum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"SO", "[", "p", "]"}], "^", "OPEi"}], 
     RowBox[{
      RowBox[{"SO", "[", "k", "]"}], "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "OPEi", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{"OPEm", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1237]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{"m", "-", "3"}]], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], "i"]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1237]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1238]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"c", " ", 
      FormBox[
       SuperscriptBox["a", 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"a", "-", "c"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["a", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1240]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1241]:="]
}, Open  ]],

Cell["OPESumSimplify", "ObjectName",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell["OPESumSimplify[exp] simplifies OPESum's in exp.", "Text",
 CellTags->"OPESumSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 "."
}], "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["\<\
OPESum[(-SOD[p])^(OPEi+1) SOD[p-q]^(OPEm-OPEi-2),{OPEi,0,OPEm}]\
\>", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1242]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[CapitalDelta]\"\>",
           TraditionalForm], 
          FormBox["\<\"\[CenterDot]\"\>",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}], ")"}]}], ")"}], 
      RowBox[{"i", "+", "1"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["\<\"(\"\>",
         TraditionalForm], 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], 
        FormBox["\<\")\"\>",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1242]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumSimplify[%]", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1243]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "m"], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", "(", 
           FormBox[
            RowBox[{"p", "-", "q"}],
            TraditionalForm], ")"}], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
      HoldForm],
     TraditionalForm]}], " ", 
   RowBox[{
    FormBox["\[CapitalDelta]",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", "OPEm"}], "}"}], "]"}], " ", 
   RowBox[{"a", "^", "OPEi"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1244]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    FormBox[
     SuperscriptBox["a", "i"],
     TraditionalForm]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1244]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", "i"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "m"}], "}"}]}], "]"}], " ", 
   RowBox[{"a", "^", 
    RowBox[{"(", 
     RowBox[{"j", "-", "i"}], ")"}]}], " ", 
   RowBox[{"b", "^", "i"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1245]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "0"}], "m"], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"i", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["b", "i"]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1245]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1246]:="],

Cell[BoxData[
 RowBox[{"OPESum", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{
      RowBox[{"-", "i"}], "+", "j"}]], " ", 
    SuperscriptBox["b", "i"]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "0", ",", "i"}], "}"}]}], "]"}]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1246]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["OPE1Loop ", "ObjectName",
 CellTags->"OPE1Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell["\<\
OPE1Loop[q1, amp].  OPE1Loop[{q1,q2}, amp] does sub-loop  decomposition.\
\>", "Text",
 CellTags->"OPE1Loop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell[BoxData[""], "Input",
 CellTags->"OPE1Loop",
 CellLabel->"In[1247]:="]
}, Open  ]],

Cell["OPE2TID  ***unfinished***", "ObjectName",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE2TID"],

Cell["\<\
OPE2TID[exp, k1, k2, p]. The setting of the option EpsContract determines the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OPE2TID", "]"}]], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1248]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Uncontract", "\[Rule]", "False"}], ",", 
    RowBox[{"Contract", "\[Rule]", "True"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPE2TID",
 CellLabel->"Out[1248]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE2TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE2TID"],

Cell[BoxData[""], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1249]:="]
}, Open  ]],

Cell["OptionsSelect", "ObjectName",
 CellTags->"OptionsSelect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OptionsSelect"],

Cell["\<\
OptionsSelect[function,opts] returns the option settings of opts accepted by \
function.  When an option occurs several times in opts, the first setting is \
selected.\
\>", "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],

Cell["Pair", "ObjectName",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Pair"],

Cell[TextData[{
 "Pair[x, y] is the head of a special pairing used in the internal \
representation: x and y may have heads LorentzIndex or Momentum. If both x \
and y have head LorentzIndex, the metric tensor is understood. If x and y \
have head Momentum, a scalar product is meant. If one of x and y has head \
LorentzIndex and the other Momentum, a Lorentz vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 " is understood."
}], "Text",
 CellTags->"Pair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"Pair"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Pair"],

Cell["This represents a four-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1250]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1250]="]
}, Open  ]],

Cell["This is a D-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1251]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1251]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1252]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1252]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "q", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1253]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1254]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1255]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "2"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1256]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "3"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1257]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "3"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", 
      RowBox[{"n", "-", "4"}]}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], "//", 
  "Contract"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1258]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1259]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", "+", " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "q", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1260]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1260]="]
}, Open  ]]
}, Open  ]],

Cell["PairCollect", "ObjectName",
 CellTags->"PairCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairCollect"],

Cell["\<\
PairCollect is an option for DiracTrace specifying if the result is collected \
with respect to Pair's.\
\>", "Text",
 CellTags->"PairCollect"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"PairCollect"]
}, Open  ]],

Cell["PairContract", "ObjectName",
 CellTags->"PairContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract"],

Cell["\<\
PairContract is like Pair, but with (local) contraction properties.\
\>", "Text",
 CellTags->"PairContract"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PairContract"]
}, Open  ]],

Cell["PairContract2", "ObjectName",
 CellTags->"PairContract2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract2"],

Cell["\<\
PairContract2 is like Pair, but with local contraction properties among \
PairContract2's.\
\>", "Text",
 CellTags->"PairContract2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract2"]
}, Open  ]],

Cell["PairContract3", "ObjectName",
 CellTags->"PairContract3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract3"],

Cell["\<\
PairContract3 is like Pair, but with local contraction properties among \
PairContract3's.\
\>", "Text",
 CellTags->"PairContract3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract3"]
}, Open  ]],

Cell["PartialD", "ObjectName",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialD"],

Cell[TextData[{
 "PartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes the four-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]],
 " PartialD is used to denote derivative fields. PartialD[LorentzIndex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", 
    StyleBox["D",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "]] denotes the ",
 Cell[BoxData[
  FormBox[
   StyleBox["D",
    FontSlant->"Plain"], TraditionalForm]]],
 "-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1261]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1263]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1263]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["PartialDRelations", "ObjectName",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialDRelations"],

Cell["\<\
PartialDRelations is an option for ExpandPartialD. It is a list of rules \
applied by ExpandPartialD at the end.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialDRelations"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"B", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1264]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["B", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialDRelations",
 CellLabel->"Out[1264]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"A", "\[Rule]", "C"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1265]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["B", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["C",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["B", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"PartialDRelations",
 CellLabel->"Out[1265]="]
}, Open  ]]
}, Open  ]],

Cell["PartialFourVector", "ObjectName",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialFourVector"],

Cell["\<\
PartialFourVector[exp, FourVector[p, mu]]  calculates the partial derivative \
of exp w.r.t. p(mu). PartialFourVector[exp, FourVector[p, mu], \
FourVector[p,nu], ...]  gives the multiple derivative.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialFourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialFourVector", "[", 
  RowBox[{
   RowBox[{"a", " ", 
    SuperscriptBox["\[ExponentialE]", 
     SuperscriptBox[
      RowBox[{"FourVector", "[", 
       RowBox[{"p", ",", "\[Nu]"}], "]"}], "2"]]}], ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"PartialFourVector",
 CellLabel->"In[1266]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "a", " ", 
   SuperscriptBox["\[ExponentialE]", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialFourVector",
 CellLabel->"Out[1266]="]
}, Open  ]]
}, Open  ]],

Cell["PartialIntegrate", "ObjectName",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell["\<\
 PartialIntegrate[exp, ap, t] does a partial integration of the definite \
integral Integrate[exp,{t,0,1}], with ap the factor that is to be integrated \
and exp/ap the factor that is to be differentiated.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PartialIntegrate", "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integrate", "\[Rule]", "Integrate"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1267]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], 
    RowBox[{"g", "[", "x", "]"}]}], ",", 
   RowBox[{"g", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1268]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     TagBox[
      RowBox[{
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], " ", 
        RowBox[{"\[Integral]", 
         RowBox[{
          RowBox[{"g", "(", "x", ")"}], 
          RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[InvisibleSpace]", 
       
       RowBox[{"x", "\[Rule]", "0"}]}],
      HoldForm], ")"}]}], "+", 
   RowBox[{"(", 
    TagBox[
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], " ", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[InvisibleSpace]", 
      RowBox[{"x", "\[Rule]", "1"}]}],
     HoldForm], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}]}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1268]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[1269]:="],

Cell["\<\
-HoldForm[f[x]*Integrate[g[x], x] /. x -> 0] + 
 HoldForm[f[x]*Integrate[g[x], x] /. x -> 1] - 
 Integrate[Integrate[g[x], x]*Derivative[1][f][x], {x, 0, 1}]\
\>", "Output",
 CellLabel->"Out[1269]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"3", "x"}], "+", "2"}], "]"}], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1270]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "3"}], " ", "x"}], "-", "2"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{
      RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1270]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x_", "]"}], "=", 
  RowBox[{"D", "[", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"3", "x"}], "+", "2"}], "]"}]], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["3", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1271]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"PartialIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "x", "]"}], 
       RowBox[{"g", "[", "x", "]"}]}], ",", 
      RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"log", "(", 
     FractionBox["32", "25"], ")"}], 
    RowBox[{
     RowBox[{"log", "(", "5", ")"}], " ", 
     RowBox[{"log", "(", "8", ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "x", "]"}], 
     RowBox[{"g", "[", "x", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["2", 
    RowBox[{"log", "(", "8", ")"}]], "-", 
   FractionBox["5", 
    RowBox[{"log", "(", "125", ")"}]]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1273]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"f", ",", "g"}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1274]:="]
}, Open  ]],

Cell["PartitHead", "ObjectName",
 CellTags->"PartitHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartitHead"],

Cell["\<\
PartitHead[expr, h] returns a list {ex1, h[ex2]} with ex1 free of expressions \
with head h, and h[ex2] having head h.\
\>", "Text",
 CellTags->"PartitHead"]
}, Open  ]],

Cell["PauliSigma", "ObjectName",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PauliSigma"],

Cell["\<\
PauliSigma denotes the vector of the 3 Pauli matrices. PauliSigma[1], \
PauliSigma[2], PauliSigma[3] give the explicit Pauli matrices. PauliSigma[] \
yields {PauliSigma[1], PauliSigma[2], PauliSigma[3]}.\
\>", "Text",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PauliSigma", "[", "]"}], "//", "TableForm"}]], "Input",
 CellTags->"PauliSigma",
 CellLabel->"In[1275]:="],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TagBox[GridBox[{
         {"0"},
         {"1"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"1"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"0"},
         {
          RowBox[{"-", "\[ImaginaryI]"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\[ImaginaryI]"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"1"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"0"},
         {
          RowBox[{"-", "1"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PauliSigma",
 CellLabel->"Out[1275]//TableForm="]
}, Open  ]]
}, Open  ]],

Cell["PaVe", "ObjectName",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVe"],

Cell["\<\
 PaVe[ i,j,... {p10,p12,...},{m1^2, mw^2, ...} ] denotes the invariant (and \
scalar) Passarino-Veltman integrals, i.e. the coefficient functions of the \
tensor integral decomposition. Joining plist and mlist gives the same \
conventions as for A0, B0, C0, D0. Automatic simlifications are performed for \
the coefficient functions of two-point integrals and for the scalar \
integrals.\
\>", "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "For a more detailed description see the manual (no changes from there \
basically): ",
 ButtonBox["http://www.mertig.com/oldfc",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc"], None}]
}], "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVe"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVe"],

Cell["Some of the PaVe's reduce to special cases.", "Text",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}]}], "]"}]], "Input",\

 CellTags->"PaVe",
 CellLabel->"In[1276]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", 
   "f", ",", "m1", ",", "m2", ",", "m3", ",", "m4", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"PaVe",
 CellLabel->"Out[1276]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", "0", ",", 
   RowBox[{"{", "pp", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"m", "^", "2"}], ",", 
     RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"PaVe",
 CellLabel->"In[1277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"PaVe", "(", 
   RowBox[{"0", ",", "0", ",", 
    RowBox[{"{", "pp", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"]}], "}"}]}], ")"}], TraditionalForm]], "Output",\

 CellTags->"PaVe",
 CellLabel->"Out[1277]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeOrderList", "ObjectName",
 CellTags->"PaVeOrderList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrderList"],

Cell["\<\
PaVeOrderList is an option for PaVeOrder and PaVeReduce, specifying in which \
order the arguments of D0 are to be permuted.\
\>", "Text",
 CellTags->"PaVeOrderList"],

Cell[TextData[{
 "See also: ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],

Cell["PaVeOrder", "ObjectName",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "PaVeOrder[expr] orders the arguments of all D0 in expr in a standard way. \
PaVeOrder[expr, PaVeOrderList \[Rule] { {..., s, u, ...}, {... ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "1"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "2"], "2"], TraditionalForm]]],
 " ...}, ...}] orders the arguments of all D0 in expr according to the \
specified ordering lists. The lists may contain only a subsequence of the \
D0-variables."
}], "Text",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1278]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D0", "[", 
     RowBox[{
     "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "m12", ",", 
      "m22", ",", "m32", ",", "m42"}], "]"}], "+", 
    RowBox[{"D0", "[", 
     RowBox[{
     "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
      "me2", ",", "0", ",", "me2", ",", "0"}], "]"}]}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"f", ",", "e"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "a", ",", "d", ",", "c", ",", "b", ",", "f", ",", 
    "e", ",", "m22", ",", "m12", ",", "m42", ",", "m32", ")"}], "+", 
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2",
     ",", "me2", ",", "me2", ",", 
    FormBox["0",
     TraditionalForm], ",", 
    FormBox["0",
     TraditionalForm], ",", "me2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1280]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeReduce", "ObjectName",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell["\<\
PaVeReduce[expr] reduces all Passarino-Veltman integrals (i.e. all PaVe's) in \
expr down to scalar A0, B0, C0 and D0.\
\>", "Text",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PaVeReduce", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "True"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PaVeOrderList", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1281]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "WriteOutPaVe"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1282]:="],

Cell[BoxData[
 StyleBox["\<\"WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If \
set to a string, the results of all Passarino-Veltman PaVe's are stored in \
files with names generated from this string and the arguments of PaVe.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"Info3387712116-6749064", "PaVeReduce"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FRH",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FRH",
  ButtonNote->"FRH"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"IsolateHead", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1283]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", "s"}]}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", "s", ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["M", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     SuperscriptBox["M", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "0", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "4"]}], "-", 
       RowBox[{"10", " ", 
        SuperscriptBox["M", "2"], " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"2", " ", "s", " ", 
        SuperscriptBox["m", "2"]}], "+", 
       RowBox[{
        SuperscriptBox["M", "2"], " ", "s"}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["M", "2"]}], "-", "s"}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "s", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1283]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"L", "[", "10", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"2", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "mw2", ",", "t"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "0", ",", "mw2"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"WriteOutPaVe", "\[Rule]", "\"\<p\>\""}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", "mw2", ",", "mw2", ")"}], 
    RowBox[{"mw2", "-", "t"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["B", "0"], "(", "t", ",", "mw2", ",", "me2", ")"}], 
    RowBox[{"mw2", "-", "t"}]], "+", 
   FractionBox["2", 
    RowBox[{"mw2", "-", "t"}]]}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", 
     "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\""}], ",", "String"}], "]"}], 
  "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1287]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - B0[t, mw2, \
SmallVariable[me2]]/\"\>"},
      {"\<\"  (mw2 - t)\"\>"},
      {"\<\"  ) \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1287]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{
   "FileNames", "[", "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\"", "]"}]}], 
  ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1288]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"se", "=", 
   RowBox[{"SmallVariable", "[", "ME2", "]"}]}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1289]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"d122", "=", 
  RowBox[{"PaVeReduce", "[", 
   RowBox[{
    RowBox[{"PaVe", "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", 
      RowBox[{"{", 
       RowBox[{"se", ",", "MW2", ",", "MW2", ",", "se", ",", "S", ",", "T"}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "se", ",", "0", ",", "se"}], "}"}]}], "]"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"S", ",", "T", ",", "U", ",", 
       RowBox[{"2", " ", "MW2"}]}], "}"}]}], ",", 
    RowBox[{"IsolateHead", "\[Rule]", "F"}]}], "]"}]}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1290]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     SuperscriptBox["T", "2"], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "MW2", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", "T", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}], " ", 
     SuperscriptBox["S", "3"]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], "2"], " ", "T", " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}], " ", 
     SuperscriptBox["S", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}], "3"]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     SuperscriptBox["T", "2"], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "T", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}], " ", 
     SuperscriptBox["S", "2"]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"4", " ", "S", " ", "MW2"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "2"]}], "+", 
       RowBox[{"S", " ", "U"}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "S", ",", "0", ",", "0", ")"}], " ", "S"}],
     
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "5"]}], "-", 
        RowBox[{"5", " ", "S", " ", 
         SuperscriptBox["MW2", "4"]}], "-", 
        RowBox[{"U", " ", 
         SuperscriptBox["MW2", "4"]}], "-", 
        RowBox[{"16", " ", 
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["MW2", "3"]}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["MW2", "2"]}], "+", 
        RowBox[{"4", " ", "S", " ", 
         SuperscriptBox["U", "2"], " ", 
         SuperscriptBox["MW2", "2"]}], "-", 
        RowBox[{"4", " ", 
         SuperscriptBox["S", "2"], " ", "U", " ", 
         SuperscriptBox["MW2", "2"]}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["S", "4"], " ", "MW2"}], "+", 
        RowBox[{"8", " ", 
         SuperscriptBox["S", "3"], " ", "U", " ", "MW2"}], "+", 
        RowBox[{
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["U", "3"]}], "+", 
        RowBox[{
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["U", "2"]}]}], ")"}], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "MW2", ",", "0", ",", "ME2", ")"}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "+", "S"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["MW2", "3"]}], "-", 
       RowBox[{"9", " ", "S", " ", 
        SuperscriptBox["MW2", "2"]}], "-", 
       RowBox[{"U", " ", 
        SuperscriptBox["MW2", "2"]}], "-", 
       RowBox[{"4", " ", "S", " ", "U", " ", "MW2"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "3"]}], "+", 
       RowBox[{"3", " ", "S", " ", 
        SuperscriptBox["U", "2"]}], "+", 
       RowBox[{"5", " ", 
        SuperscriptBox["S", "2"], " ", "U"}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "T", ",", "ME2", ",", "ME2", ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "+", "S"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        SuperscriptBox["MW2", "6"]}], "-", 
       RowBox[{"8", " ", "S", " ", 
        SuperscriptBox["MW2", "5"]}], "-", 
       RowBox[{"2", " ", "T", " ", 
        SuperscriptBox["MW2", "5"]}], "+", 
       RowBox[{"12", " ", 
        SuperscriptBox["S", "2"], " ", 
        SuperscriptBox["MW2", "4"]}], "+", 
       RowBox[{"20", " ", "S", " ", "T", " ", 
        SuperscriptBox["MW2", "4"]}], "-", 
       RowBox[{"8", " ", 
        SuperscriptBox["S", "3"], " ", 
        SuperscriptBox["MW2", "3"]}], "-", 
       RowBox[{"6", " ", "S", " ", 
        SuperscriptBox["T", "2"], " ", 
        SuperscriptBox["MW2", "3"]}], "-", 
       RowBox[{"36", " ", 
        SuperscriptBox["S", "2"], " ", "T", " ", 
        SuperscriptBox["MW2", "3"]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "4"], " ", 
        SuperscriptBox["MW2", "2"]}], "+", 
       RowBox[{"6", " ", 
        SuperscriptBox["S", "2"], " ", 
        SuperscriptBox["T", "2"], " ", 
        SuperscriptBox["MW2", "2"]}], "+", 
       RowBox[{"20", " ", 
        SuperscriptBox["S", "3"], " ", "T", " ", 
        SuperscriptBox["MW2", "2"]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["S", "2"], " ", 
        SuperscriptBox["T", "3"], " ", "MW2"}], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["S", "3"], " ", 
        SuperscriptBox["T", "2"], " ", "MW2"}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "4"], " ", "T", " ", "MW2"}], "-", 
       RowBox[{
        SuperscriptBox["S", "2"], " ", 
        SuperscriptBox["T", "4"]}]}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "MW2", ",", "T", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ")"}]}]}], "-", 
   FractionBox[
    RowBox[{"MW2", "+", "S"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1290]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"\"\<fctd122.for\>\"", ",", 
    RowBox[{"d122res", "==", "d122"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"PaVeReduce",
 CellLabel->"In[1291]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fctd122.for\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1292]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        d122res = (-5D0D-1*(MW2 + S))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)*(MW2**2 - 1D0*S*U)) - \"\>"},
      {"\<\"     &  (5.D-1*(4D0*MW2**5 - 5D0*MW2**4*S - \"\>"},
      {"\<\"     &       16D1*MW2**3*S**2 + 4D0*MW2**2*S**3 + \"\>"},
      {"\<\"     &       4D0*MW2*S**4 - 1D0*MW2**4*U - \"\>"},
      {"\<\"     &       4D0*MW2**2*S**2*U + 8D0*MW2*S**3*U + \"\>"},
      {"\<\"     &       4D0*MW2**2*S*U**2 + S**3*U**2 + S**2*U**3)*\"\>"},
      {"\<\"     &     B0(MW2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(4D0*MW2 - 1D0*T)**2*\"\>"},
      {"\<\"     &     (MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*S*(MW2**2 - 4D0*MW2*S + 2D0*S**2 + S*U)*\"\>"},
      {"\<\"     &     B0(S,0D0,0D0))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (4D0*MW2**3 - 9D0*MW2**2*S + 2D0*S**3 - \"\>"},
      {"\<\"     &       1D0*MW2**2*U - 4D0*MW2*S*U + 5D0*S**2*U + \"\>"},
      {"\<\"     &       3D0*S*U**2)*B0(T,ME2,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**2) - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (2D0*MW2**6 - 8D0*MW2**5*S + 12D1*MW2**4*S**2 - \
\"\>"},
      {"\<\"     &       8D0*MW2**3*S**3 + 2D0*MW2**2*S**4 - \"\>"},
      {"\<\"     &       2D0*MW2**5*T + 20D1*MW2**4*S*T - \"\>"},
      {"\<\"     &       36D1*MW2**3*S**2*T + 20D1*MW2**2*S**3*T - \"\>"},
      {"\<\"     &       2D0*MW2*S**4*T - 6D0*MW2**3*S*T**2 + \"\>"},
      {"\<\"     &       6D0*MW2**2*S**2*T**2 - 6D0*MW2*S**3*T**2 + \"\>"},
      {"\<\"     &       4D0*MW2*S**2*T**3 - 1D0*S**2*T**4)*\"\>"},
      {"\<\"     &     C0(MW2,MW2,T,ME2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**3) + \"\>"},
      {"\<\"     &  ((MW2 - \
1D0*S)**2*S**2*T*C0(MW2,S,ME2,ME2,0D0,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3 - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**2*T**2*\"\>"},
      {"\<\"     &     C0(T,ME2,ME2,ME2,ME2,0D0))/(MW2**2 - 1D0*S*U)**3 + \
\"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**3*T**2*\"\>"},
      {"\<\"     &     D0(MW2,MW2,ME2,ME2,T,S,ME2,0D0,ME2,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3\"\>"},
      {"\<\"                  \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1292]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{"FileNames", "[", "\"\<fctd122.for\>\"", "]"}]}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"d122", ",", "se"}], "]"}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1293]:="]
}, Open  ]],

Cell["PD", "ObjectName",
 CellTags->"PD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PD"],

Cell["PD is an abbreviation for PropagatorDenominator.", "Text",
 CellTags->"PD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PD"]
}, Open  ]],

Cell["PlusDistribution", "ObjectName",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell["\<\
PlusDistribution[1/(1-x)] denotes a distribution (in the sense of the \"+\" \
prescription).\
\>", "Text",
 CellTags->"PlusDistribution"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{"1", "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1294]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1294]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1295]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox[
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1295]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1296]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1296]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1297]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1297]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}], "^", "2"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1298]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1299]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1299]="]
}, Open  ]]
}, Open  ]],

Cell["Polarization", "ObjectName",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Polarization"],

Cell["\<\
Polarization[k] = Polarization[k, I] is the head of a polarization momentum \
with (incoming) momentum k. A slashed polarization vector (e1(k) slash) has \
to be entered as DiracSlash[Polarization[k]]. The internal representation for \
a polarization vector e1 corresponding to a boson with four momentum k is: \
Momentum[ Polarization[ k, I ] ]. With this notation transversality of \
polarization vectors is provided, i.e. , Pair[ Momentum[k], Momentum[ \
Polarization[k, I] ] ] yields 0. Polarization[k,-I] denotes the complex \
conjugate polarization. Polarization is also an option of various functions \
related to the operator product expansion. The setting 0 denotes the \
unpolarized and 1 the polarized case.\
\>", "Text",
 CellTags->"Polarization"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PolarizationVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PolarizationVector",
  ButtonNote->"PolarizationVector"],
 ", ",
 ButtonBox["PolarizationSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PolarizationSum",
  ButtonNote->"PolarizationSum"],
 ", ",
 ButtonBox["DoPolarizationSums",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DoPolarizationSums",
  ButtonNote->"DoPolarizationSums"],
 "."
}], "Text",
 CellTags->"Polarization"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Polarization", "[", "k", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1300]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1301]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1301]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", 
  "ComplexConjugate"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1302]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     SuperscriptBox[
      TagBox["\<\"\[CurlyEpsilon]\"\>",
       Power,
       Editable->True,
       Selectable->True], 
      TagBox["\<\"*\"\>",
       Power,
       Editable->True,
       Selectable->True]],
     InterpretTemplate[Superscript[#, #2]& ],
     Editable->False,
     Selectable->False],
    TraditionalForm], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1302]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1303]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", 
   RowBox[{"-", "\[ImaginaryI]"}]}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1303]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"Polarization", "[", "k", "]"}], "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1304]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1304]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"Polarization", "[", "k", "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1305]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{"Polarization", "[", 
    RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1305]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", "k", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", " ", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", " ", "I"}], "]"}], " ", "]"}]}], " ", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1306]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1306]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationSum", "ObjectName",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell["\<\
PolarizationSum[ mu,nu, ... ] defines (as abbreviations) several polarization \
sums. The first two arguments are the interpreted as Lorentz indices, all \
further ones are momenta. PolarizationSum performs no calculations.\
\>", "Text",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PolarizationSum", "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1307]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1307]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1308]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1308]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1309]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"]], "-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", "n"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1310]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["n",
       TraditionalForm], "2"]}], 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["n",
        TraditionalForm]}],
      TraditionalForm], "2"]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["n",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["n",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["n",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["n",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", 
   RowBox[{
    SubscriptBox["p", "1"], "-", 
    SubscriptBox["p", "2"]}]}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", "1", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", "2", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       FormBox[
        RowBox[{
         SubscriptBox["p", "1"], "-", 
         SubscriptBox["p", "2"]}],
        TraditionalForm], ")"}], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       FormBox[
        RowBox[{
         SubscriptBox["p", "1"], "-", 
         SubscriptBox["p", "2"]}],
        TraditionalForm], ")"}], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1311]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationUncontract", "ObjectName",
 CellTags->"PolarizationUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationUncontract"],

Cell["\<\
PolarizationUncontract does Uncontract on scalar products involving \
polarization vectors.\
\>", "Text",
 CellTags->"PolarizationUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],

Cell["PolarizationVector", "ObjectName",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell["PolarizationVector[p, mu] gives a polarization vector.", "Text",
 CellTags->"PolarizationVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationVector", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1312]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Conjugate", "[", 
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "*"], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1313]="]
}, Open  ]],

Cell[TextData[{
 "A polarization vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
   TraditionalForm]]],
 "is a special four-vector."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "mu"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1314]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "mu", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1314]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "The transverality property ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], "=", "0"}],
    TraditionalForm]]],
 " is built in."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1315]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "k", ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1315]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1316]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1317]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a2", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1318]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1318]="]
}, Open  ]],

Cell[TextData[{
 "Depending on the alphabetical ordering of the momenta simplifcations are \
done, e.g., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
      FormBox[
       RowBox[{"k", "-", "p"}],
       TraditionalForm], ")"}]}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", "p", "-", "k"}], ")"}], "\[Mu]"]}], 
      RowBox[{
       SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
       FormBox[
        RowBox[{"k", "-", "p"}],
        TraditionalForm], ")"}]}], "=", 
     RowBox[{
      RowBox[{
       SuperscriptBox["k", "\[Mu]"], 
       RowBox[{
        SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
        FormBox[
         RowBox[{"k", "-", "p"}],
         TraditionalForm], ")"}]}], "=", 
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       RowBox[{
        RowBox[{"(", "\[NoBreak]", 
         FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
           FormBox[
            RowBox[{"k", "-", "p"}],
            TraditionalForm], "\[NoBreak]", ")"}],
          TraditionalForm], "\[NoBreak]", ")"}], "."}]}]}]}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1319]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1319]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a3", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1320]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1320]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1321]:="]
}, Open  ]],

Cell["PositiveInteger", "ObjectName",
 CellTags->"PositiveInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveInteger"],

Cell["\<\
PositiveInteger is a data type. E.g. DataType[OPEm, PositiveInteger] gives \
True.\
\>", "Text",
 CellTags->"PositiveInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],

Cell["PositiveNumber", "ObjectName",
 CellTags->"PositiveNumber"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveNumber"],

Cell["\<\
PositiveNumber is a data type. E.g. DataType[Epsilon, PositiveNumber] = True \
(by default). \
\>", "Text",
 CellTags->"PositiveNumber"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],

Cell["PowerFactor", "ObjectName",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "PowerFactor[exp] replaces ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "a"], TraditionalForm]]],
 "with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PowerSimplify",
  ButtonNote->"PowerSimplify"],
 "."
}], "Text",
 CellTags->"PowerFactor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "a"], 
  SuperscriptBox["y", "a"]}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1322]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["x", "a"], " ", 
   SuperscriptBox["y", "a"]}], TraditionalForm]], "Output",
 CellTags->"PowerFactor",
 CellLabel->"Out[1322]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerFactor", "[", "%", "]"}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1323]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]], "Output",
 CellTags->"PowerFactor",
 CellLabel->"Out[1323]="]
}, Open  ]]
}, Open  ]],

Cell["PowerSimplify", "ObjectName",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "PowerSimplify[exp] simplifies",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "x"}], ")"}], "a"]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"y", "-", "x"}], ")"}], "n"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], "n"], TraditionalForm]]],
 "; thus assuming that the exponent is an integer (even if it is symbolic). \
Furthermore ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " are expanded and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"2", " ", "m"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 " and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] 1 and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    FormBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}],
     TraditionalForm], "m"], TraditionalForm]]],
 " for ",
 StyleBox["n",
  FontSlant->"Italic"],
 " even and odd respectively and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"-", "n"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ExponentialE]", 
    StyleBox[
     RowBox[{"\[ImaginaryI]", 
      StyleBox["m",
       FontSlant->"Italic"], "\[Pi]"}]]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"OPEm"],
 "."
}], "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1324]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1324]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1325]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1325]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1326]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1326]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"I", "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1327]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1327]="]
}, Open  ]]
}, Open  ]],

Cell["Power2", "ObjectName",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Power2"],

Cell["\<\
Power2[x, y] represents x^y.  Sometimes Power2 is more useful than the \
Mathematica Power. Power2[-a,b] simplifies to (-1)^b Power2[a,b] (if no \
Epsilon is in b ...).\
\>", "Text",
 CellTags->"Power2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerFactor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PowerFactor",
  ButtonNote->"PowerFactor"],
 "."
}], "Text",
 CellTags->"Power2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1328]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "a"}], ")"}], "b"], TraditionalForm]], "Output",
 CellTags->"Power2",
 CellLabel->"Out[1328]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power2", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1329]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "b"], " ", 
   FormBox[
    SuperscriptBox["a", "b"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Power2",
 CellLabel->"Out[1329]="]
}, Open  ]]
}, Open  ]],

Cell["PostFortranFile", "ObjectName",
 CellTags->"PostFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PostFortranFile"],

Cell["\<\
PostFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string,  which will be put at the end of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PostFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PreFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PreFortranFile",
  ButtonNote->"PreFortranFile"],
 "."
}], "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],

Cell["Prefactor", "ObjectName",
 CellTags->"Prefactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Prefactor"],

Cell["\<\
Prefactor is an option for OneLoop and OneLoopSum. If set as option of \
OneLoop, the amplitude is multiplied by Prefactor before calculation; if \
specified as option of OneLoopSum, after calculation in the final result as a \
global factor.\
\>", "Text",
 CellTags->"Prefactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"Prefactor"]
}, Open  ]],

Cell["PreFortranFile", "ObjectName",
 CellTags->"PreFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PreFortranFile"],

Cell["\<\
PreFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string, which will be put at the beginning of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PreFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PostFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PostFortranFile",
  ButtonNote->"PostFortranFile"],
 "."
}], "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],

Cell["PropagatorDenominator", "ObjectName",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "PropagatorDenominator[q, m] is a factor of the denominator of a propagator. \
If p is supposed to be D-dimensional enter: PropagatorDenominator[Momentum[q, \
D], m]. What is meant is ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"q", "^", "2"}]}]}]}], TraditionalForm]]],
 " - ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "). PropagatorDenominator[p] evaluates to PropagatorDenominator[p, 0]. "
}], "Text",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominatorExplicit",
  ButtonNote->"PropagatorDenominatorExplicit"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}]}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1330]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1330]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", "p", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1331]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"q", ",", "m"}], "]"}], " "}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1332]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1332]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", "t1", "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1333]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1333]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"t1", ",", "D"}], "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1334]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1334]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominatorExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1335]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1335]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1336]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["m", "2"]}], "+", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "q", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1336]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1337]:="]
}, Open  ]],

Cell["PropagatorDenominatorExplicit", "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell["\<\
PropagatorDenominatorExplicit[exp] changes each occurence of \
PropagatorDenominator[a,b] in exp into 1/(ScalarProduct[a,a]-b^2) and \
replaces FeynAmpDenominator by Identity.\
\>", "Text",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}], "//", 
  "FCI"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1338]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1338]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PropagatorDenominatorExplicit", "[", "%", "]"}], "//", 
  "FCE"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1339]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm], "-", 
      SuperscriptBox["m", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["m", "2"], "-", 
      RowBox[{"2", " ", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}], "+", 
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1339]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1340]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["m", "2"]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["m", "2"], "-", 
     RowBox[{"2", " ", 
      RowBox[{"SPD", "[", 
       RowBox[{"p", ",", "q"}], "]"}]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}]}]]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1340]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["QGV", "ObjectName",
 CellTags->"QGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QGV"],

Cell["QGV is equivalent to QuarkGluonVertex.", "Text",
 CellTags->"QGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QGV"]
}, Open  ]],

Cell["QO", "ObjectName",
 CellTags->"QO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QO"],

Cell["QO is equivalent to Twist2QuarkOperator.", "Text",
 CellTags->"QO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"QO"]
}, Open  ]],

Cell["QP", "ObjectName",
 CellTags->"QP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QP"],

Cell["\<\
QP is an alias for QuarkPropagator. QP[p] is the massless quark propagator. \
QP[{p,m}] gives the  quark propagator with mass m.\
\>", "Text",
 CellTags->"QP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"QP"]
}, Open  ]],

Cell["QuantumField", "ObjectName",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuantumField"],

Cell[TextData[{
 "QuantumField is the head of quantized fields and their derivatives. \
QuantumField[par, ftype, {lorind}, {sunind}] denotes a quantum field of type \
ftype with (possible) Lorentz-indices lorind and SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices sunind. The optional first argument par denotes a partial \
derivative acting on the field."
}], "Text",
 CellTags->"QuantumField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"QuantumField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuantumField"],

Cell["This denotes a scalar field.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "S", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1341]:="],

Cell[BoxData[
 FormBox["S", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1341]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1342]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1342]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "QuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1343]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1343]="]
}, Open  ]],

Cell["This is a field with a Lorentz index.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"B", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1344]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["B", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1344]="]
}, Open  ]],

Cell["Color indices should be put after the Lorentz ones.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1345]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1345]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1346]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1346]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[CapitalDelta]", "a"], TraditionalForm]]],
 " is a short form for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "^", "\[Mu]"}], " ", 
    RowBox[{
     SubsuperscriptBox["A", "\[Mu]", "a"], "."}]}], TraditionalForm]]],
 " "
}], "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1347]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1347]="]
}, Open  ]],

Cell["\<\
 The first list of indices is usually interpreted as type LorentzIndex, \
except for OPEDelta, which gets converted to type Momentum. \
\>", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "OPEDelta", "}"}], ",", 
    RowBox[{"{", "a", "}"}]}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1348]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1348]//StandardForm="]
}, Open  ]],

Cell["Derivatives of fields are denoted as follows.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "A", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1349]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1349]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "S"}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1350]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm]], 
   SubsuperscriptBox["S", "\[Null]", "\[Null]"]}], TraditionalForm]], "Output",\

 CellTags->"QuantumField",
 CellLabel->"Out[1350]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1351]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\[PartialD]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]],
    TraditionalForm], 
   SubsuperscriptBox["A", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1351]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PartialD", "[", "OPEDelta", "]"}], "^", "OPEm"}], ",", "A", ",", 
   
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[PartialD]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox["m",
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1352]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"QuantumField", "[", "A", "]"}], "]"}], " ", "===", " ", 
  RowBox[{"QuantumField", "[", "A", "]"}]}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1353]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1353]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkField", "ObjectName",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkField"],

Cell["\<\
QuarkField is the name of a fermionic field. QuarkField is just a name with \
no functional properties. Only typesetting rules are attached.\
\>", "Text",
 CellTags->"QuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiQuarkField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiQuarkField",
  ButtonNote->"AntiQuarkField"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"QuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell[BoxData["QuarkField"], "Input",
 CellTags->"QuarkField",
 CellLabel->"In[1354]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellTags->"QuarkField",
 CellLabel->"Out[1354]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkGluonVertex", "ObjectName",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[TextData[{
 "QuarkGluonVertex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    StyleBox["a",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "] gives the Feynman rule for the quark-gluon vertex. "
}], "Text",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkGluonVertex", "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1355]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1355]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a"}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1356]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1356]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1357]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1357]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1358]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1359]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1359]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1360]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "True"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1360]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1361]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkMass", "ObjectName",
 CellTags->"QuarkMass"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkMass"],

Cell["QuarkMass is an option of Amplitude.", "Text",
 CellTags->"QuarkMass"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"QuarkMass"]
}, Open  ]],

Cell["QuarkPropagator", "ObjectName",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell["\<\
QuarkPropagator[p] is the massless quark propagator. QuarkPropagator[{p,m}] \
or gives the quark propagator with mass m.\
\>", "Text",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkPropagator", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1362]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Loop", "\[Rule]", "0"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1362]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", "p", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1363]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1363]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1364]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1364]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1365]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1365]="]
}, Open  ]]
}, Open  ]],

Cell["ReduceGamma", "ObjectName",
 CellTags->"ReduceGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceGamma"],

Cell["\<\
ReduceGamma is an option of OneLoop. If set to True all DiracMatrix[6] and \
DiracMatrix[7] (i.e. all ChiralityProjector) are reduced to Gamma5.\
\>", "Text",
 CellTags->"ReduceGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],

Cell["ReduceToScalars", "ObjectName",
 CellTags->"ReduceToScalars"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceToScalars"],

Cell["\<\
ReduceToScalars is an option for OneLoop  and OneLoopSum that specifies \
whether the result will be reduced to scalar A0, B0, C0 and D0 scalar \
integrals.\
\>", "Text",
 CellTags->"ReduceToScalars"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],

Cell["Rename", "ObjectName",
 CellTags->"Rename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Rename"],

Cell["\<\
Rename is an option for Contract. If set to True, dummy indices in Eps are \
renamed, using $MU[i].\
\>", "Text",
 CellTags->"Rename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Rename"]
}, Open  ]],

Cell["RHI  ***unfinished***", "ObjectName",
 CellTags->"RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI"],

Cell[BoxData[""], "Input",
 CellTags->"RHI",
 CellLabel->"In[1366]:="]
}, Open  ]],

Cell["RHI2FC  ***unfinished***", "ObjectName",
 CellTags->"RHI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI2FC"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI2FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"RHI2FC",
 CellLabel->"In[1367]:="]
}, Open  ]],

Cell["RHM  ***unfinished***", "ObjectName",
 CellTags->"RHM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHM"],

Cell["RHM[] is like RHI[], gives Gamma functions.", "Text",
 CellTags->"RHM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHM"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHM"],

Cell[BoxData[""], "Input",
 CellTags->"RHM",
 CellLabel->"In[1368]:="]
}, Open  ]],

Cell["RHO  ***unfinished***", "ObjectName",
 CellTags->"RHO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHO"],

Cell["\<\
RHO[i] with i from 1 to 4 is an abbreviation for the 4 operators (eq. \
(3.2.17) -- (3.2.20)) defined in R.Hambergs thesis. The Lorentz indices are \
suppressed. The explicit expressions may be recovered by RHO[i, mu, nu, p].\
\>", "Text",
 CellTags->"RHO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHO"],

Cell[BoxData[""], "Input",
 CellTags->"RHO",
 CellLabel->"In[1369]:="]
}, Open  ]],

Cell["RHP  ", "ObjectName",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHP"],

Cell["\<\
RHP[i, mu, nu, p] with i from 1 to 4 gives the projectors for RHO[i]. RHP[mu, \
nu, p] gives Sum[RHP[i,mu, nu, p] RHO[i], {i, 4}] collected with respect to \
mu and nu. See the thesis of Roelof Hamberg (but correcting a misprint).\
\>", "Text",
 CellTags->"RHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHO",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHO",
  ButtonNote->"RHO"],
 "."
}], "Text",
 CellTags->"RHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"1", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1370]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], 
    RowBox[{"D", "-", "2"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", "-", "D"}], ")"}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1370]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"2", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1371]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], 
   SuperscriptBox[
    FormBox[
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}],
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellLabel->"Out[1371]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"3", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1372]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"4", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1373]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[1373]="]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellTags->"RHP",
 CellLabel->"In[1374]:="]
}, Open  ]],

Cell["RTL  ***unfinished***", "ObjectName",
 CellTags->"RTL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RTL"],

Cell["\<\
RTL[exp] inserts the list of known TLI integrals into exp, substitutes D\
\[RightArrow]4+Epsilon and expands around Epsilon up to the finite part.\
\>", "Text",
 CellTags->"RTL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 ", ",
 ButtonBox["Epsilon",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Epsilon",
  ButtonNote->"Epsilon"],
 "."
}], "Text",
 CellTags->"RTL"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RTL"],

Cell[BoxData[""], "Input",
 CellTags->"RTL",
 CellLabel->"In[1375]:="]
}, Open  ]],

Cell["RightPartialD", "ObjectName",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 ", acting to the right."
}], "Text",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 "."
}], "Text",
 CellTags->"RightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1376]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[RightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1376]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1377]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1378]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1379]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1379]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1380]:="],

Cell[BoxData[
 RowBox[{"RightPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1380]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["RussianTrick", "ObjectName",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RussianTrick"],

Cell["\<\
RussianTrick[exp, k, {q1,q2,p}] (=RussianTrick[exp,p,p,{q1,q2,p}]) does the \
integration by parts where p is the external momentum. RussianTrick[exp, k,l, \
{q1,q2,p}] (=RussianTrick[exp,k,l]) does integration by parts where l is the \
momentum to be differentiated.\
\>", "Text",
 CellTags->"RussianTrick"],

Cell["The result is an expression which is vanishing.", "Text",
 CellTags->"RussianTrick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["FourLaplacian",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourLaplacian",
  ButtonNote->"FourLaplacian"],
 "."
}], "Text",
 CellTags->"RussianTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"RHI", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"OPEm", ",", "0", ",", " ", "0", ",", "0", ",", " ", "0"}], 
     "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
     "1", ",", " ", "1", ",", " ", "1", ",", " ", "1", ",", " ", "1"}], 
     "}"}]}], "]"}]}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1381]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "1", "1", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1381]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "RHI2FC"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1382]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1382]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RussianTrick", "[", 
  RowBox[{
   RowBox[{"%", "//", "RHI2FC"}], ",", "q2"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1383]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"], 
     RowBox[{
      SuperscriptBox["q1", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], "-", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"4", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellTags->"RussianTrick",
 CellLabel->"Out[1383]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", "%", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1384]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"0", "1", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "2", "1"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "4"}], ")"}], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "2", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1384]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm], "\[Rule]", 
    FractionBox[
     RowBox[{
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"0", "1", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "2", "1"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       FormBox[
        SubsuperscriptBox[
         StyleBox["T",
          FontFamily->"Times",
          FontWeight->"Bold"], 
         StyleBox[
          RowBox[{"1", "1", "1", "2", "1"}],
          FontColor->RGBColor[0, 0, 1]], 
         RowBox[{
          FormBox["m",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm]}]],
        TraditionalForm]}]}], 
     RowBox[{"4", "-", "D"}]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1385]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1386]:="]
}, Open  ]],

Cell["ScalarGluonVertex", "ObjectName",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell["\<\
ScalarGluonVertex[{p}, {q}, {mu,a}] or ScalarGluonVertex[ p,  q,  mu, a ] \
yields the scalar-scalar-gluon vertex (p and q are incoming momenta).
ScalarGluonVertex[{mu,a}, {nu,b}] yields the scalar-scalar-gluon-gluon vertex \
(p and q are incoming momenta).
The dimension  and the name of the coupling constant are determined by the \
options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ScalarGluonVertex", "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1387]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1387]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", " ", 
   RowBox[{"{", "q", "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"mu", ",", " ", "a"}], "}"}]}], "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1388]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SubscriptBox["g", "s"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], 
    FormBox[
     FormBox["mu",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1388]="]
}, Open  ]]
}, Open  ]],

Cell["ScalarProduct", "ObjectName",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "ScalarProduct[p, q] is the input for scalar product. ScalarProduct[p] is \
equivalent to ScalarProduct[p, p]. Expansion of sums of momenta in \
ScalarProduct is done with ExpandScalarProduct. Scalar product may be set, \
e.g., ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "; but a and b must not contain sums. Note that ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 " actually sets also: Pair[Momentum[a, ___], Momentum[b, ___]] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 ". It is encouraged to always set ScalarProduct's ",
 StyleBox["before",
  FontWeight->"Bold"],
 " any calculation. This improves the performance of FeynCalc ."
}], "Text",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1389]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1389]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", 
   RowBox[{"-", "q"}]}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1390]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{
     FormBox["q",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "+", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1390]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1391]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1391]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", "q", "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1392]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1392]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1393]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1393]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1394]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1394]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"]}], "]"}], " ", "=", " ", 
  RowBox[{"s", "/", "2"}]}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1395]:="],

Cell[BoxData[
 FormBox[
  FractionBox["s", "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1395]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1396]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["s", "2"], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "1"],
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1396]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1397]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["s", "2"], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "1"],
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1397]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1398]:="]
}, Open  ]],

Cell["ScalarProductCancel", "ObjectName",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell["\<\
ScalarProductCancel[exp, q1, q2, ...] cancels scalar products with \
propagators. ScalarProductCancel[exp] cancels simple cases.\
\>", "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData["SPC"], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1399]:="],

Cell[BoxData[
 FormBox["ScalarProductCancel", TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1399]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "//", "Options"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1400]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1400]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SPC"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1401]:="],

Cell[BoxData[
 StyleBox["\<\"SPC is an abbreviation for ScalarProductCancel.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"Info3387712142-8239904", "ScalarProductCancel"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}]}], "//", 
   "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1402]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1402]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "[", 
  RowBox[{"t1", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1403]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}]}]]}], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1404]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1404]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t1", ",", "q", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1405]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1405]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1406]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1406]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t2", ",", "q1", ",", "q2", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1407]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{"4", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox["m", "2"], "2"]}], "-", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "4"], "4"], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["m", "2"]}], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "2"], "4"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1407]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1408]:="]
}, Open  ]],

Cell["ScaleMu", "ObjectName",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScaleMu"],

Cell["\<\
ScaleMu is the mass scale used for dimensional regularization of loop \
integrals.\
\>", "Text",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell[BoxData["ScaleMu"], "Input",
 CellTags->"ScaleMu",
 CellLabel->"In[1409]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellTags->"ScaleMu",
 CellLabel->"Out[1409]="]
}, Open  ]]
}, Open  ]],

Cell["Schouten", "ObjectName",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Schouten"],

Cell["\<\
Schouten[expr] applies the Schouten identity on at most 42 terms in a sum. If \
Schouten should operate on larger expression you can give a second argument, \
e.g.: Schouten[expr, 4711] which will work on sums with less than 4711 terms.
Schouten is also an option of Contract and DiracTrace. It may be set to an \
integer indicating the maximum number of terms onto which the function \
Schouten will be applied .\
\>", "Text",
 CellTags->"Schouten"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"Schouten"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"a", "=", "1"}], "4"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SP", "[", 
         RowBox[{"k", ",", 
          RowBox[{"q", "[", "a", "]"}]}], "]"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", "6"], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "b", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "c", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "d", "]"}]}], "]"}], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "b", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "c", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "d", "]"}], "]"}]}], "]"}]}], ")"}]}], ")"}]}],
      ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "1", ",", "4"}], "}"}]}], "]"}], "-", 
   RowBox[{
    RowBox[{"Eps", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "1", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "2", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "3", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "4", "]"}], "]"}]}], "]"}], "*", 
    RowBox[{"Pair", "[", 
     RowBox[{
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"Momentum", "[", "k", "]"}]}], "]"}]}]}]}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1410]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        RowBox[{"q", "(", "1", ")"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"q", "(", "2", ")"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"q", "(", "3", ")"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"q", "(", "4", ")"}],
        TraditionalForm]}]]}], " ", 
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Schouten",
 CellLabel->"Out[1410]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Schouten"}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1411]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Schouten",
 CellLabel->"Out[1411]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellLabel->"In[1412]:="]
}, Open  ]],

Cell["SD", "ObjectName",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SD"],

Cell[TextData[{
 "SD[i, j] is the (FeynCalc-external) Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices i and j. SD[i,j] is transformed into \
SUNDelta[SUNIndex[i],SUNIndex[j]] by FeynCalcInternal."
}], "Text",
 CellTags->"SD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 "."
}], "Text",
 CellTags->"SD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1413]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1413]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1414]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1414]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1415]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1415]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SelectFree", "ObjectName",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectFree"],

Cell["\<\
SelectFree[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: SelectFree[a, b] returns a and \
SelectFree[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectFree"],

Cell["SelectFree is equivalent to Select1.", "Text",
 CellTags->"SelectFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"SelectFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}], "+", "d"}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1416]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"b", "+", "d"}], TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1416]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"x", " ", "y"}], ",", " ", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1417]:="],

Cell[BoxData[
 FormBox["y", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1417]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", "z", " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1418]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "z"}], TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1418]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1419]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1420]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1421]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1422]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1422]="]
}, Open  ]]
}, Open  ]],

Cell["SelectGraphs", "ObjectName",
 CellTags->"SelectGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectGraphs"],

Cell["\<\
SelectGraphs is an option for OneLoopSum indicating that only a slected set \
of graphs of the list provided to OneLoopSum is to be calculated. Possible \
settings are: SelectGraphs\[RightArrow]{ i, j,  ... } or SelectGraphs\
\[RightArrow]{ a, {b, c}, ... } which indicates the graphs to be taken from \
the list provided to OneLoopSum. In the second setting the list {b, c} \
indicates that all amplitudes from b to c should be taken.\
\>", "Text",
 CellTags->"SelectGraphs"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],

Cell["SelectNotFree", "ObjectName",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, x] returns that part of expr which is not free of any \
occurance of x.\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a, \
b, ...}]&], except the special cases: SelectNotFree[a, b] returns 1 and \
SelectNotFree[a, a] returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["SelectNotFree is equivalent to Select2.", "Text",
 CellTags->"SelectNotFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectFree",
  ButtonNote->"SelectFree"],
 "."
}], "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1423]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", "+", 
   RowBox[{"f", "(", "a", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1423]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", 
    RowBox[{"f", "[", "x", "]"}], " ", "z"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", "y", " ", 
   RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1425]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1425]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "x"}], ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1426]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1426]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1427]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1427]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1428]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1429]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1429]="]
}, Open  ]]
}, Open  ]],

Cell["SelectSplit", "ObjectName",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectSplit"],

Cell["\<\
SelectSplit[l, p] Construct list of mutually exclusive subsets from l in \
which every element li satisfies a criterium pj[li] with pj from p and \
appends the subset of remaining unmatched elements.\
\>", "Text",
 CellTags->"SelectSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "2"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "4"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1430]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     SuperscriptBox["a", "2"], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"]}], "}"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"SelectSplit",
 CellLabel->"Out[1430]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Plus"}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1431]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["a", "2"], ",", 
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SelectSplit",
 CellLabel->"Out[1431]="]
}, Open  ]]
}, Open  ]],

Cell["Select1", "ObjectName",
 CellTags->"Select1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select1"],

Cell["\<\
Select1[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: Select1[a, b] returns a and Select1[a,a] \
returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Select2",
  ButtonNote->"Select2"],
 "."
}], "Text",
 CellTags->"Select1"]
}, Open  ]],

Cell["Select2", "ObjectName",
 CellTags->"Select2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select2"],

Cell["\<\
Select2[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a,b, \
...}]&], except the special cases: Select2[a, b] returns 1 and  Select2[a,a] \
returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Select1",
  ButtonNote->"Select1"],
 "."
}], "Text",
 CellTags->"Select2"]
}, Open  ]],

Cell["Series2", "ObjectName",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series2"],

Cell[TextData[{
 " Series2 performs a series expansion around 0. Series2 is (up to the \
Gamma-bug in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions smaller than 5.0) equivalent to Series, except that it applies \
Normal on the result and has an option FinalSubstitutions. Series2[f, e, n] \
is equivalent to Series2[f, {e, 0, n}]."
}], "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series2", "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1432]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "\[Rule]", "0"}], "}"}]}], ",", 
    RowBox[{"Print", "\[Rule]", "False"}], ",", 
    RowBox[{"SimplifyPolyLog", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1432]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1433]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"Series2",
 CellLabel->"Out[1433]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1434]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1434]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1435]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{
    FractionBox["1", "x"], "-", 
    TagBox["\[DoubledGamma]",
     Function[{}, EulerGamma]], "+", 
    RowBox[{
     FractionBox["1", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"6", " ", 
        SuperscriptBox[
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], "2"]}], "+", 
       SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", "x"}], "+", 
    InterpretationBox[
     RowBox[{"O", "(", 
      SuperscriptBox["x", "2"], ")"}],
     SeriesData[$CellContext`x, 0, {}, -1, 2, 1],
     Editable->False]}],
   SeriesData[$CellContext`x, 0, {
    1, -EulerGamma, Rational[1, 12] (6 EulerGamma^2 + Pi^2)}, -1, 2, 1],
   Editable->False], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1435]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1436]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "3"]}], " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], " ", 
    SuperscriptBox["x", "2"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1436]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2", ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox[
       RowBox[{"\[Zeta]", "(", 
        TagBox["3",
         Zeta,
         Editable->True,
         Selectable->True], ")"}],
       InterpretTemplate[
        Function[BoxForm`e$, 
         Zeta[BoxForm`e$]]],
       Editable->False,
       Selectable->False], " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "3"], " ", 
      SuperscriptBox["x", "2"]}], "+", 
     RowBox[{"3", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "-", 
     RowBox[{"3", " ", 
      TagBox["\[DoubledGamma]",
       Function[{}, EulerGamma]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         SuperscriptBox["x", "2"]}], "+", "2"}], ")"}]}], "+", 
     FractionBox["6", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1437]="]
}, Open  ]],

Cell[TextData[{
 "In earlier ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions the Gamma functions was not expanded far enough, in 5.0 this got \
finally fixed:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Gamma", "[", "x", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"$VersionNumber", "<", "5"}], ",", "4", ",", "2"}], "]"}]}],
        "}"}]}], "]"}], "//", "Normal"}], "//", "Expand"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "3"], " ", 
      SuperscriptBox["x", "2"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], "-", 
        RowBox[{"4", " ", "x", " ", 
         TagBox[
          RowBox[{"\[Zeta]", "(", 
           TagBox["3",
            Zeta,
            Editable->True,
            Selectable->True], ")"}],
          InterpretTemplate[
           Function[BoxForm`e$, 
            Zeta[BoxForm`e$]]],
          Editable->False,
          Selectable->False]}]}], ")"}], " ", "x"}], "+", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "-", 
     RowBox[{
      TagBox["\[DoubledGamma]",
       Function[{}, EulerGamma]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox["x", "2"]}], "+", "12"}], ")"}]}], "+", 
     FractionBox["12", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1438]="]
}, Open  ]],

Cell["\<\
There is a table of expansions of special hypergeometric functions.\
\>", "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"OPEm", "-", "1"}], ",", 
       RowBox[{
        RowBox[{"Epsilon", "/", "2"}], "+", "OPEm"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"OPEm", ",", 
       RowBox[{"OPEm", "+", "Epsilon"}]}], "}"}], ",", "1"}], "]"}], ",", 
   "Epsilon", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{
     SuperscriptBox[
      TagBox["\[Psi]",
       PolyGamma], 
      RowBox[{"(", "1", ")"}]], "(", "m", ")"}], " ", "m"}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{
     SuperscriptBox[
      TagBox["\[Psi]",
       PolyGamma], 
      RowBox[{"(", "1", ")"}]], "(", "m", ")"}]}], "-", 
   FractionBox["2", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1439]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"HypergeometricPFQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", " ", "OPEm", ",", " ", 
        RowBox[{
         RowBox[{"Epsilon", "/", "2"}], " ", "+", " ", "OPEm"}]}], "}"}], ",",
       " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", " ", "+", " ", "OPEm"}], ",", " ", 
        RowBox[{"Epsilon", " ", "+", " ", "OPEm"}]}], "}"}], ",", "  ", "1"}],
      "]"}], ",", "Epsilon", ",", "1"}], "]"}]}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox[
       TagBox["\[Psi]",
        PolyGamma], 
       RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "2"]}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
   RowBox[{
    FractionBox["3", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    RowBox[{
     SuperscriptBox[
      TagBox["\[Psi]",
       PolyGamma], 
      RowBox[{"(", "1", ")"}]], "(", "m", ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    RowBox[{
     SubscriptBox["S", "11"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1440]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"1", ",", " ", "Epsilon", ",", " ", 
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"2", " ", "Epsilon"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1441]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
      SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["1",
           Hypergeometric2F1,
           Editable->True,
           Selectable->True], ",", 
          TagBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm],
           Hypergeometric2F1,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         TagBox[
          RowBox[{
           RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "+", "1"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        Hypergeometric2F1,
        Editable->False,
        Selectable->False], ";", 
       TagBox["x",
        Hypergeometric2F1,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   Hypergeometric2F1], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1441]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1442]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "6"]}], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{"2", " ", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1442]="]
}, Open  ]],

Cell[TextData[{
 "There are over 100 more special expansions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[ThinSpace]", 
    RowBox[{
     SubscriptBox["", "2"], 
     SubscriptBox["F", "1"]}]}], TraditionalForm]]],
 " tabulated in Series2.m. The interested user can consult the source code \
(search for HYPERLIST)."
}], "Text",
 CellTags->"Series2"]
}, Open  ]],

Cell["Series3", "ObjectName",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series3"],

Cell["\<\
Series3 performs a series expansion around 0. Series3 is equivalent to \
Series, except that it applies Normal on the result and that some Series bugs \
are fixed. Series3[f, e, n] is equivalent to  Series3[f, {e, 0, n}].\
\>", "Text",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series3", "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1443]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Series3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1444]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series3", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TagBox["\[DoubledGamma]",
     Function[{}, EulerGamma]]}], "+", "1", "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1445]="]
}, Open  ]]
}, Open  ]],

Cell["SetMandelstam", "ObjectName",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SetMandelstam"],

Cell["\<\
SetMandelstam[s, t, u, p1, p2, p3, p4, m1, m2, m3, m4] defines the Mandelstam \
variables  s=(p1+p2)^2, t=(p1+p3)^2, u=(p1+p4)^2 and sets the pi on-shell: \
p1^2=m1^2, p2^2=m2^2, p3^2=m3^2, p4^2=m4^2. Note that p1 +  p2 + p3 + p4 = 0 \
is assumed.
SetMandelstam[x, {p1, p2, p3, p4, p5}, {m1, m2, m3, m4, m5}] defines x[i, j] \
= (pi+pj)^2 and sets the pi on-shell. The pi satisfy: p1 + p2 + p3 + p4 + p5 \
= 0.\
\>", "Text",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SetMandelstam", "]"}]], "Input",
 CellTags->"SetMandelstam",
 CellLabel->"In[1446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"4", ",", "D", ",", "___"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SetMandelstam",
 CellLabel->"Out[1446]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Mandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Mandelstam",
  ButtonNote->"Mandelstam"],
 "."
}], "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],

Cell["SimplifyDeltaFunction", "ObjectName",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell["\<\
SimplifyDeltaFunction[exp, x] simplifies f[x]*DeltaFunction[1-x] \
toLimit[f[x],x\[Rule]1] DeltaFunction[1-x] and applies a list of \
transformation rules for DeltaFunctionPrime[1-x]*x^(OPEm-1)*f[x] where \
x^(OPEm-1) is suppressed in exp.\
\>", "Text",
 CellTags->"SimplifyDeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunction", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1447]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1448]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     UnderscriptBox["lim", 
      RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
     RowBox[{"g", "(", "x", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1449]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{"g", "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1450]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", " ", 
  RowBox[{"Log", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1451]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1452]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1453]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1453]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1454]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1454]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", "x", "]"}], 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1455]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"log", "(", "x", ")"}], " ", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1455]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1456]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1456]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1457]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1457]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1458]="]
}, Open  ]]
}, Open  ]],

Cell["SimplifyGTI  ***unfinished***", "ObjectName",
 CellTags->"SimplifyGTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell["SimplifyGTI simplifies GTI's.", "Text",
 CellTags->"SimplifyGTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GTI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GTI",
  ButtonNote->"GTI"],
 "."
}], "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell[BoxData[""], "Input",
 CellTags->"SimplifyGTI",
 CellLabel->"In[1459]:="]
}, Open  ]],

Cell["SimplifyPolyLog", "ObjectName",
 CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell["\<\
SimplifyPolyLog[y] performs several simplifications assuming that the \
variables occuring in the Log and PolyLog functions are between 0 and 1. \
\>", "Text",
 CellTags->"SimplifyPolyLog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Nielsen",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Nielsen",
  ButtonNote->"Nielsen"],
 "."
}], "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sip", "[", "y_", "]"}], " ", ":=", " ", 
  RowBox[{"y", " ", "==", " ", 
   RowBox[{"SimplifyPolyLog", "[", "y", "]"}]}]}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1460]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "/", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1461]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox["1", "x"], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1461]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1462]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1462]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", 
     RowBox[{"x", "^", "2"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1463]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"1", "-", 
     SuperscriptBox["x", "2"]}], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"x", "+", "1"}], ")"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1463]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1464]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    SuperscriptBox["x", "2"], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1464]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    RowBox[{"-", 
     FractionBox["x", 
      RowBox[{"1", "-", "x"}]]}], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1465]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "2"], "(", 
    FractionBox["x", 
     RowBox[{"x", "-", "1"}]], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", 
     RowBox[{
      FormBox["1",
       TraditionalForm], 
      FormBox["2",
       TraditionalForm]}]], "(", 
    FormBox[
     RowBox[{"-", 
      FractionBox["x", 
       RowBox[{"1", "-", "x"}]]}],
     TraditionalForm], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"-", 
     FractionBox["1", "x"]}], ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "3"], "(", 
     RowBox[{"-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1469]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1470]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    SuperscriptBox["x", "2"], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{"4", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}]}], "+", 
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "3"], "(", 
      RowBox[{"-", "x"}], ")"}]}], "+", 
    RowBox[{"4", " ", 
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"-", 
     FractionBox["x", 
      RowBox[{"1", "-", "x"}]]}], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1472]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "3"], "(", 
    RowBox[{"1", "-", 
     FractionBox["1", "x"]}], ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    TagBox[
     RowBox[{"\[Zeta]", "(", 
      TagBox["3",
       Zeta,
       Editable->True,
       Selectable->True], ")"}],
     InterpretTemplate[
      Function[BoxForm`e$, 
       Zeta[BoxForm`e$]]],
     Editable->False,
     Selectable->False]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1472]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"4", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1473]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     InterpretationBox["Li",
      PolyLog,
      Editable->False,
      Selectable->False], "4"], "(", 
    RowBox[{"-", 
     FractionBox["x", 
      RowBox[{"1", "-", "x"}]]}], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "24"]}], " ", 
     RowBox[{
      SuperscriptBox["log", "4"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SubscriptBox[
       InterpretationBox["Li",
        PolyLog,
        Editable->False,
        Selectable->False], "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     TagBox[
      RowBox[{"\[Zeta]", "(", 
       TagBox["3",
        Zeta,
        Editable->True,
        Selectable->True], ")"}],
      InterpretTemplate[
       Function[BoxForm`e$, 
        Zeta[BoxForm`e$]]],
      Editable->False,
      Selectable->False], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["3",
        TraditionalForm]}]], "(", "x", ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["2",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     SubscriptBox[
      InterpretationBox["Li",
       PolyLog,
       Editable->False,
       Selectable->False], "4"], "(", "x", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1473]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"a", "+", 
    RowBox[{"b", "/", "c"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"a", "+", 
     FractionBox["b", "c"]}], ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    FractionBox[
     RowBox[{"b", "+", 
      RowBox[{"a", " ", "c"}]}], "c"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1474]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"1", "/", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    FractionBox["1", "x"], ")"}], "\[LongEqual]", 
   RowBox[{"-", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1475]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcTanh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["tanh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", "+", "1"}], 
       RowBox[{"1", "-", "x"}]]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1476]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcSinh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["sinh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{"x", "+", 
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", "1"}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcCosh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["cosh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{"x", "+", 
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "-", "1"}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1478]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "sip", "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1479]:="]
}, Open  ]],

Cell["Simplify2  ***unfinished***", "ObjectName",
 CellTags->"Simplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Simplify2"],

Cell["Simplify2 is a special ordering function.", "Text",
 CellTags->"Simplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Simplify2"],

Cell[BoxData[""], "Input",
 CellTags->"Simplify2",
 CellLabel->"In[1480]:="]
}, Open  ]],

Cell["SmallDelta", "ObjectName",
 CellTags->"SmallDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallDelta"],

Cell["SmallDelta denotes some small positive number.", "Text",
 CellTags->"SmallDelta"]
}, Open  ]],

Cell["SmallEpsilon", "ObjectName",
 CellTags->"SmallEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallEpsilon"],

Cell["SmallEpsilon denotes some small positive number.", "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],

Cell["SmallVariable", "ObjectName",
 CellTags->"SmallVariable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariable"],

Cell["\<\
SmallVariable[me] is the head of small (negligible) variables. This means any \
mass with this head can be neglected if it appears in a sum, but not as an \
argument of Passarino-Veltman (PaVe) functions or PropagatorDenominator.\
\>", "Text",
 CellTags->"SmallVariable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"SmallVariable"]
}, Open  ]],

Cell["SmallVariables", "ObjectName",
 CellTags->"SmallVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariables"],

Cell["\<\
SmallVariables is an option for OneLoop. \"SmallVariables\[Rule]{Melectron}\" \
i.e. will substitute \"SmallVariable[Melectron]\"  for all Melectron's in the \
calculation.\
\>", "Text",
 CellTags->"SmallVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"SmallVariables"]
}, Open  ]],

Cell["SMP", "ObjectName",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMP"],

Cell["\<\
SMP[par] substitutes a symbol for the  Standard Model parameter par.  SMP[] \
gives the list of substitutions. par should be a string; e.g., SMP[\"SW\"] \
gives  sw (in the Global` context).\
\>", "Text",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SMP", "[", "]"}], "//", "InputForm"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1481]:="],

Cell["\<\
{\"EL\" :> e, \"CW\" :> cw, \"ME\" :> me, \"MH\" :> mh, \"MW\" :> mw, \"SW\" \
:> sw}\
\>", "Output",
 CellTags->"SMP",
 CellLabel->"Out[1481]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SMVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SMVertex",
  ButtonNote->"SMVertex"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMP", "[", "\"\<ME\>\"", "]"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1482]:="],

Cell[BoxData[
 FormBox["me", TraditionalForm]], "Output",
 CellTags->"SMP",
 CellLabel->"Out[1482]="]
}, Open  ]]
}, Open  ]],

Cell["SMVertex", "ObjectName",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMVertex"],

Cell["\<\
SMVertex[\"AWW\", p,mu, q,nu, k,rho] gives the photon-W-W vertex (p,mu \
correspond to the photon, q,nu to the (incoming) W+ and k,rho to the \
(incoming) W-. All momenta are flowing into the vertex.  SMVertex[\"HHH\", \
___] give the three-higgs coupling. \
\>", "Text",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "//", "Options"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1483]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SMVertex",
 CellLabel->"Out[1483]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "[", 
  RowBox[{
  "\"\<AWW\>\"", ",", "p", ",", "\[Mu]", ",", " ", "q", ",", "\[Nu]", ",", 
   " ", "k", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "e", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "p"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SMVertex",
 CellLabel->"Out[1484]="]
}, Open  ]]
}, Open  ]],

Cell["Smu  ***unfinished***", "ObjectName",
 CellTags->"Smu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Smu"],

Cell["Smu  is ...", "Text",
 CellTags->"Smu"]
}, Open  ]],

Cell["Sn", "ObjectName",
 CellTags->"Sn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Sn"],

Cell[TextData[{
 "Sn is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Pi]", 
    RowBox[{"n", "/", "2"}]], TraditionalForm]]],
 "/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"2", " ", "\[Pi]"}], ")"}], "n"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Sn"]
}, Open  ]],

Cell["SO", "ObjectName",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SO"],

Cell["\<\
SO[q] is a four-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q], Momentum[OPEDelta] by FCI.\
\>", "Text",
 CellTags->"SO"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", "p", "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1485]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1486]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SO", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1487]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1487]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SOD", "ObjectName",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SOD"],

Cell["\<\
SOD[q] is a D-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q,D], Momentum[OPEDelta,D] by \
FeynCalcInternal.\
\>", "Text",
 CellTags->"SOD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SOD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", "p", "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1488]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1488]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1489]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1489]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1490]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"OPEDelta", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1490]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Solve2", "ObjectName",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve2"],

Cell["\<\
Solve2 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and accepts the options Factoring and \
FinalSubstitutions. Solve2 uses the \"high school algorithm\" and factors \
intermediate results. Therefore it can be drastically more useful than Solve.\
\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve2", "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1491]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1491]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1492]="]
}, Open  ]],

Cell["\<\
If no equation sign is given the polynomials are supposed to be 0.\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "y"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1493]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x", ",", 
   RowBox[{"FinalSubstitutions", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"y", "\[Rule]", "h"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "h"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1494]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", 
   RowBox[{"Factoring", "\[Rule]", "Expand"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["b", "2"], "-", 
      FractionBox["w", "4"]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1495]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1496]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{"d", "+", "p"}]}]}], "}"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1496]="]
}, Open  ]]
}, Open  ]],

Cell["Solve3", "ObjectName",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve3"],

Cell["\<\
Solve3 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and uses the \"high school algorithm\" and is \
sometimes better than Solve for systems involving rational polyonomials.\
\>", "Text",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve3", "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1497]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve3",
 CellLabel->"Out[1497]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Solve2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Solve2",
  ButtonNote->"Solve2"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve3", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve3",
 CellLabel->"Out[1498]="]
}, Open  ]]
}, Open  ]],

Cell["SP", "ObjectName",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SP"],

Cell[TextData[{
 "SP[a, b] denotes a four-dimensional scalar product. SP[a, b] is transformed \
into ScalarProduct[a, b] by FeynCalcInternal. SP[p] is the same as SP[p, p] \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SP", "[", "q", "]"}]}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1499]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1499]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1500]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1500]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1501]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1501]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SP", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1502]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], "2"], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1502]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1503]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1503]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1504]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "a", "]"}], ",", 
   RowBox[{"Momentum", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1504]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1505]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1505]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SPC", "ObjectName",
 CellTags->"SPC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPC"],

Cell["SPC is an abbreviation for ScalarProductCancel.", "Text",
 CellTags->"SPC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 "."
}], "Text",
 CellTags->"SPC"]
}, Open  ]],

Cell["SPD", "ObjectName",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPD"],

Cell[TextData[{
 "SPD[a, b] denotes a D-dimensional scalar product. SPD[a, b] is transformed \
into ScalarProduct[a, b,Dimension\[Rule]D] by FeynCalcInternal. SPD[p] is the \
same as SPD[p,p] ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PD",
  ButtonNote->"PD"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SPD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SPD", "[", "q", "]"}]}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1506]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1506]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1507]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1507]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SPD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1508]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1508]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SPD", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1509]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], "2"], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1509]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1510]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1510]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SPD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1511]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"a", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1511]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1512]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1512]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"p", ",", "q"}], "]"}], ",", " ", "D"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1513]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1513]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Spinor", "ObjectName",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Spinor"],

Cell[TextData[{
 "Spinor[p, m, o] is the head of Dirac spinors. Which of the spinors ",
 StyleBox["u",
  FontSlant->"Italic"],
 ", ",
 StyleBox["v",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{",", " "}], TraditionalForm]]],
 "or ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 "is understood, depends on the sign of the momentum (p) argument and the \
relative position of DiracSlash[p]: Spinor[sign p, mass] is that spinor which \
yields: sign*mass*Spinor[p, mass] if the Dirac equation is applied (by \
DiracEquation or DiracSimplify).\nThe optional argument o can be used for \
additional degrees of freedom. If no optional argument o is supplied, a 1 is \
subsituted in.\nSpinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 StyleBox["u",
  FontSlant->"Italic"],
 "=2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " and  ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 StyleBox["v=",
  FontSlant->"Italic"],
 "-2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"Spinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"Spinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", "p", "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1514]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1514]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1515]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1515]="]
}, Open  ]],

Cell["\<\
FeynCalc uses covariant normalization (as opposed to e.g. the normalization \
used in Bjorken&Drell).\
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}]}], "//", "DiracSimplify"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1516]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "m"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1516]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{
     RowBox[{"-", "p"}], ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1517]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "m"}], " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox[
     RowBox[{"-", "p"}],
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1517]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", "p", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1518]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1518]//StandardForm="]
}, Open  ]],

Cell["\<\
By convention, ChangeDimension does not operate on momenta in Spinor's (but \
on e.g.  DiracSlash[Momentum[p]]). \
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Spinor", "[", "p", "]"}], ",", "D"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1519]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1519]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1520]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1520]//StandardForm="]
}, Open  ]],

Cell["SmallVariable's are discarded by Spinor.", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", 
    RowBox[{"SmallVariable", "[", "m", "]"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1521]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1521]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorCollect", "ObjectName",
 CellTags->"SpinorCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorCollect"],

Cell["\<\
SpinorCollect is an option for FermionSpinSum. If set to False the  argument \
of FermionSpinSum has to be already collected w.r.t. Spinor.\
\>", "Text",
 CellTags->"SpinorCollect"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],

Cell["SpinorUBar", "ObjectName",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "SpinorUBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 "-spinor."
}], "Text",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell["One argument only assumes a massless spinor.", "Text",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1522]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1522]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1523]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1523]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1524]:="],

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1524]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorUBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1525]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorUBar",
 CellLabel->"Out[1525]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorUBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1526]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorUBar",
 CellLabel->"Out[1526]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorU", "ObjectName",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorU"],

Cell["SpinorU[p, m, optarg] denotes a u-spinor.", "Text",
 CellTags->"SpinorU"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorU"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", "p", "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1527]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1527]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorU", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1529]:="],

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1529]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorU", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1530]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorU",
 CellLabel->"Out[1530]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorU", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1531]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorU",
 CellLabel->"Out[1531]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorVBar", "ObjectName",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "SpinorVBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", "_"], TraditionalForm]]],
 "(p,m)-spinor."
}], "Text",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 "."
}], "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1532]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1532]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1533]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1533]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1534]:="],

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1534]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorVBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1535]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1535]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorVBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1536]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1536]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorV", "ObjectName",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorV"],

Cell["SpinorV[p, m, optarg] denotes a v-spinor.", "Text",
 CellTags->"SpinorV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", "p", "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1537]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1537]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1538]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1538]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorV", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1539]:="],

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1539]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorV", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1540]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1540]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorV", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1541]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1541]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinPolarizationSum", "ObjectName",
 CellTags->"SpinPolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinPolarizationSum"],

Cell["\<\
SpinPolarizationSum is an option for FermionSpinSum. The set (pure) function \
acts on the usual spin sum.\
\>", "Text",
 CellTags->"SpinPolarizationSum"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],

Cell["SPL", "ObjectName",
 CellTags->"SPL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPL"],

Cell["SPL is an abbreviation for SimplifyPolyLog.", "Text",
 CellTags->"SPL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"SPL"]
}, Open  ]],

Cell["SplittingFunction", "ObjectName",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell[TextData[{
 "SplittingFunction[pxy] is a database of splitting functions in the ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["MS", "_"], TraditionalForm]]],
 " scheme."
}], "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SplittingFunction", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1542]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Polarization", "\[Rule]", "1"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1542]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 "."
}], "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell["Unpolarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1543]:="],

Cell["\<\
In general the argument should be a string, but if the variables Pqq, etc. \
have no value, you can omit the \"\".\
\>", "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1544]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4"}], " ", "x"}], "+", 
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1544]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{"16", " ", "x"}], "+", "8"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1545]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"4", " ", "x"}], "-", "8", "+", 
     FractionBox["8", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1547]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["x", "2"]}], "+", "x", "+", 
      RowBox[{
       FractionBox["11", "12"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"], "-", "2", "+", 
      FractionBox["1", "x"]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1547]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1548]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"log", "(", "x", ")"}], 
         RowBox[{"1", "-", "x"}]], "+", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], 
           RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1548]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "x"}], "+", "4", "-", 
        FractionBox["4", "x"]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2", "-", 
     FractionBox["4", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "8"}], " ", 
         SuperscriptBox["x", "2"]}], "+", 
        RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1550]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1551]:="],

Cell["Polarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1552]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1553]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4"}], " ", "x"}], "+", 
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1553]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1554]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", "x"}], "-", "8"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{"8", "-", 
     RowBox[{"4", " ", "x"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1556]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "16"}], " ", "x"}], "+", 
      RowBox[{
       FractionBox["22", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{"8", " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", "8"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1556]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1557]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], "+", 
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      RowBox[{"log", "(", "x", ")"}], " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
     RowBox[{"4", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1557]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4"}], " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1558]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agqd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqgd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1562]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aggd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1563]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "16"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "+", "1"}], ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"48", " ", "x"}], "-", "16"}], ")"}], " ", 
      RowBox[{"log", "(", "x", ")"}]}], "+", 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "4"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "24"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"48", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True,
             Selectable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]],
           Editable->False,
           Selectable->False]}], "+", "3"}], ")"}]}]}], ")"}], " ", 
    SubsuperscriptBox["C", "F", "2"]}], "+", 
   RowBox[{
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "-", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"2", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"4", " ", 
           RowBox[{
            SubscriptBox[
             InterpretationBox["Li",
              PolyLog,
              Editable->False,
              Selectable->False], "2"], "(", 
            RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TagBox[
           RowBox[{"\[Zeta]", "(", 
            TagBox["3",
             Zeta,
             Editable->True,
             Selectable->True], ")"}],
           InterpretTemplate[
            Function[BoxForm`e$, 
             Zeta[BoxForm`e$]]],
           Editable->False,
           Selectable->False]}], "+", 
         FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
    SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1565]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "x"}], "-", "2"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{"54", " ", "x"}], "-", 
      RowBox[{"16", " ", "x", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
      RowBox[{"18", " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "44"}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"4", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "8"}], " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"44", " ", "x"}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "4"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], " ", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", 
        RowBox[{"-", "x"}], ")"}]}], "+", "48"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1566]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGQ", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "A"], " ", 
      SubscriptBox["C", "F"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "x"}], "3"], "+", 
         FractionBox["80", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "+", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      FractionBox[
       RowBox[{"280", " ", "x"}], "9"], "+", 
      RowBox[{"16", " ", "x", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"32", "-", 
         RowBox[{"104", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", 
        RowBox[{"-", "x"}], ")"}]}], "+", 
      FractionBox["328", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "8"}], " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"4", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{"32", " ", "x"}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "64"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"36", " ", "x"}], "+", "48"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "68"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], " ", 
      SubscriptBox["T", "f"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"32", " ", "x"}], "9"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"32", " ", "x"}], "3"], "-", 
         FractionBox["64", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      FractionBox["128", "9"]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1567]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["8", 
           RowBox[{"x", "+", "1"}]]}], "+", "32", "+", 
         FractionBox["8", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["232", "3"], "-", 
         FractionBox[
          RowBox[{"536", " ", "x"}], "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         FractionBox["32", 
          RowBox[{"1", "-", "x"}]], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox[
       RowBox[{"388", " ", "x"}], "9"], "+", 
      RowBox[{
       FractionBox["64", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         RowBox[{"16", " ", 
          SubscriptBox[
           RowBox[{"(", 
            FractionBox["1", 
             RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
         FractionBox["16", 
          RowBox[{"x", "+", "1"}]]}], ")"}]}], "+", 
      RowBox[{
       FractionBox["536", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       RowBox[{
        SubscriptBox[
         InterpretationBox["Li",
          PolyLog,
          Editable->False,
          Selectable->False], "2"], "(", 
        RowBox[{"-", "x"}], ")"}]}], "+", 
      RowBox[{"24", " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       TagBox[
        RowBox[{"\[Zeta]", "(", 
         TagBox["3",
          Zeta,
          Editable->True,
          Selectable->True], ")"}],
        InterpretTemplate[
         Function[BoxForm`e$, 
          Zeta[BoxForm`e$]]],
        Editable->False,
        Selectable->False]}], "-", 
      FractionBox["148", "9"]}], ")"}], " ", 
    SubsuperscriptBox["C", "A", "2"]}], "+", 
   RowBox[{
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"608", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["32", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"32", " ", "x"}], "3"]}], "-", 
         FractionBox["32", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      RowBox[{
       FractionBox["160", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["448", "9"]}], ")"}], " ", 
    SubscriptBox["C", "A"]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "16"}], " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "80"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{"80", " ", "x"}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", "80"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1568]="]
}, Open  ]]
}, Open  ]],

Cell["StandardMatrixElement", "ObjectName",
 CellTags->"StandardMatrixElement"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"StandardMatrixElement"],

Cell["\<\
StandardMatrixElement[ ... ] is the head for matrix element abbreviations.\
\>", "Text",
 CellTags->"StandardMatrixElement"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],

Cell["SubContext", "ObjectName",
 CellTags->"SubContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubContext"],

Cell["\<\
SubContext[fun] gives the sub-directory (context) in HighEnergyPhysics.\
\>", "Text",
 CellTags->"SubContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"SubContext"]
}, Open  ]],

Cell["SubLoop", "ObjectName",
 CellTags->"SubLoop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubLoop"],

Cell["\<\
SubLoop is an option for OPE1Loop. If set to True, sub 1-loop tensorintegral \
decomposition is performed.\
\>", "Text",
 CellTags->"SubLoop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"SubLoop"]
}, Open  ]],

Cell["SumP", "ObjectName",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumP"],

Cell[TextData[{
 "SumP[k, m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     RowBox[{
      StyleBox["k",
       FontSlant->"Plain"], "-", "1"}]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{
       StyleBox["i",
        FontSlant->"Plain"], "=", "1"}], 
      RowBox[{"2", 
       StyleBox["m",
        FontSlant->"Plain"]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         StyleBox["i",
          FontSlant->"Plain"]]}], ")"}], "/", 
      SuperscriptBox[
       StyleBox["i",
        FontSlant->"Plain"], 
       StyleBox["k",
        FontSlant->"Plain"]]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SumP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1569]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1570]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1571]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox["m",
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1572]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "8"], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", "i"}]}]], "Input",\

 CellTags->"SumP",
 CellLabel->"In[1573]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{"SumP", "[", 
   RowBox[{"1", ",", 
    RowBox[{"n", "/", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1574]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"n", "-", "1"}], "2"],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox["n", "2"],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"n", "\[Rule]", "8"}]}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1575]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1575]="]
}, Open  ]]
}, Open  ]],

Cell["SumS", "ObjectName",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumS"],

Cell[TextData[{
 "SumS[1, m] is the harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    StyleBox[" ",
     FontSlant->"Italic"], 
    RowBox[{
     StyleBox["i",
      FontSlant->"Italic"], "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"=", 
      RowBox[{
       SubscriptBox["S", "1"], "(", "m", ")"}]}], " ", ")"}], "."}], 
   TraditionalForm]]],
 " SumS[1,1,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    SubscriptBox["S", "1"]}], TraditionalForm]]],
 "(",
 StyleBox["i",
  FontSlant->"Italic"],
 ")/",
 StyleBox["i",
  FontSlant->"Italic"],
 ". SumS[k,l,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", "l"], "(", "i", ")"}], "/", 
     SuperscriptBox["i", "k"]}]}], TraditionalForm]]],
 ". SumS[r, n] represents Sum[Sign[r]^i/i^Abs[r], {i, 1, n}]. SumS[r,s, n] is \
Sum[Sign[r]^k/k^Abs[r] Sign[s]^j/j^Abs[s], {k, 1, n}, {j, 1, k}], etc."
}], "Text",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SumS", "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1576]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Reduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"SumS",
 CellLabel->"Out[1576]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1577]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1577]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1578]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1578]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "m"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1579]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{"-", "1"}]], "(", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1579]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", "m"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}], ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "+", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1581]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1582]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1583]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1583]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1584]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1585]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1585]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{"1", "/", "i"}]}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1586]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1586]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1587]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "1", ",", "1", ",", "11"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1588]:="],

Cell[BoxData[
 FormBox[
  FractionBox["31276937512951", "4260000729600"], TraditionalForm]], "Output",\

 CellTags->"SumS",
 CellLabel->"Out[1588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1589]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1590]="]
}, Open  ]]
}, Open  ]],

Cell["SumT", "ObjectName",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumT"],

Cell[TextData[{
 "SumT[1, m] is the alternative harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", 
     RowBox[{"i", " "}]}]}], TraditionalForm]]],
 " SumT[r, n] represents Sum[(-1)^i/i^r, {i,1,n}], SumT[r,s, n] is Sum[1/k^r \
(-1)^j/j^s, {k, 1, n}, {j, 1, k}]."
}], "Text",
 CellTags->"SumT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 "."
}], "Text",
 CellTags->"SumT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1591]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1591]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1592]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1592]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", "m", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1593]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["S", "~"], "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], "m"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1598]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1599]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1599]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1600]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1600]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], 
   RowBox[{"m", "-", "1"}]], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1601]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        TagBox["\[Psi]",
         PolyGamma], 
        RowBox[{"(", "0", ")"}]], "(", 
       FractionBox["m", "2"], ")"}], "-", 
      RowBox[{
       SuperscriptBox[
        TagBox["\[Psi]",
         PolyGamma], 
        RowBox[{"(", "0", ")"}]], "(", 
       FractionBox[
        RowBox[{"m", "+", "1"}], "2"], ")"}]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{"log", "(", "4", ")"}], "2"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "12"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1602]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["57561743656913", "21300003648000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1602]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"-", "2"}], ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1603]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1603]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{"SumT", ",", "6"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1604]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1604]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{
      RowBox[{"-", "2"}], ",", "1", ",", "#1"}], "]"}], "&"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1605]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1605]="]
}, Open  ]]
}, Open  ]],

Cell["SUND", "ObjectName",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUND"],

Cell[TextData[{
 "SUND[a,b,c] are the symmetric SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["d", "abc"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"SUND"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUND"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1606]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1606]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tt", "=", 
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1607]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1607]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"c", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1608]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1608]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1609]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1609]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1610]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   RowBox[{"(", 
    RowBox[{"4", "-", 
     SubsuperscriptBox["C", "A", "2"]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1610]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1611]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1611]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1612]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"4", "-", 
       SuperscriptBox["N", "2"]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}], "N"]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1612]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUND",
 CellLabel->"In[1613]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1613]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1614]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1614]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1615]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1615]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "tt", "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1616]:="]
}, Open  ]],

Cell["SUNDelta", "ObjectName",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "SUNDelta[a, b] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices a and b."
}], "Text",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SD",
  ButtonNote->"SD"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1617]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1617]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", " ", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}], " ", "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1618]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1618]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1619]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1619]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1620]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1620]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1621]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1621]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNDelta", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1622]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1622]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1623]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1623]//StandardForm="]
}, Open  ]],

Cell["\<\
Symbolic arguments to SUNDelta are transformed into the data type SUNIndex \
and integer arguments are transformed to ExplicitSUNIndex. The difference is \
that SUNSimplify will only sum over symbolic indices.\
\>", "Text",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "2"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "2"}], "]"}]}], "//", "SUNSimplify"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1624]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1624]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1625]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1625]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1626]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"ExplicitSUNIndex", "[", "2", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1626]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNDeltaContract", "ObjectName",
 CellTags->"SUNDeltaContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "SUNDeltaContract[expr] substitues for all SUNDelta in expr \
SUNDeltaContract, contracts the SUN(N) indices and resubstitutes SUNDelta. \
SUNDeltaContract[i, j] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with contraction properties. SUNDeltaContract wraps also the head \
SUNIndex around its arguments."
}], "Text",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],

Cell["SUNF", "ObjectName",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNF"],

Cell[TextData[{
 "SUNF[a,b,c] are the structure constants of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 "). The arguments a,b,c should be of symbolic type."
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNF", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1627]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1627]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNT",
  ButtonNote->"SUNT"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "x"}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"b", ",", "a", ",", "c"}], "]"}], "y"}]}]}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1628]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]]}], "+", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{"b", "a", "c"}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1628]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1629]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1630]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "-", "y"}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1630]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1631]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"a", "a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1631]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "a", ",", "b"}], "]"}], "//", "Calc"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1632]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1632]="]
}, Open  ]],

Cell[TextData[{
 "This is a consequence of the usual choice for the normalization of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "a"], " ", "generatorsin", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"see", "  ", "SUNT"}], ")"}], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1633]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1633]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1634]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUNF",
 CellLabel->"In[1636]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1636]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1637]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1637]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1638]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1638]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1639]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"b", "a", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1639]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"b", ",", "a", ",", "c"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1640]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1640]="]
}, Open  ]]
}, Open  ]],

Cell["SUNFJacobi", "ObjectName",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell["\<\
SUNFJacobi is an option for SUNSimplify, indicating whether the Jacobi \
identity should be used.\
\>", "Text",
 CellTags->"SUNFJacobi"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "False"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1641]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "True"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1642]="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndex", "ObjectName",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "SUNIndex is the head of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices."
}], "Text",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 "."
}], "Text",
 CellTags->"SUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1643]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1643]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1644]:="],

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1644]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1645]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "i", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "j", "]"}]}], "]"}]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1645]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndexRename", "ObjectName",
 CellTags->"SUNIndexRename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndexRename"],

Cell["\<\
SUNIndexRename is an option of SUNSimplify. If set to False, no automatic \
renaming of dummy indices is done.\
\>", "Text",
 CellTags->"SUNIndexRename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],

Cell["SUNN", "ObjectName",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNN"],

Cell[TextData[{
 "SUNN denotes the number of colors. Trick[SUNDelta[a, a]] yields (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["SUNN", "2"], TraditionalForm]]],
 " -1)."
}], "Text",
 CellTags->"SUNN"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNN"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNN",
 CellLabel->"In[1646]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["N", "2"], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"SUNN",
 CellLabel->"Out[1646]="]
}, Open  ]]
}, Open  ]],

Cell["SUNNToCACF", "ObjectName",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell["\<\
SUNNToCACF is an option of SUNSimplify. If set to True, the Casimir operator \
eigenvalues CA (=N) and CF (=(N^2-1)/(2 N)) are introduced.\
\>", "Text",
 CellTags->"SUNNToCACF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNN",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNN",
  ButtonNote->"SUNN"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNNToCACF",
 CellLabel->"In[1647]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNNToCACF",
 CellLabel->"Out[1647]="]
}, Open  ]]
}, Open  ]],

Cell["SUNSimplify", "ObjectName",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell["\<\
SUNSimplify simplifies products of SUNT (and complex conjugated) matrices. \
Basic renaming of dummy indices is done. If the option SUNTrace is set to \
False, then any SUNT-matrices are taken out of DiracTrace[...]; otherwise a \
color-trace is taken (by SUNTrace) before taking the SUN-objects in front of \
DiracTrace[...].\
\>", "Text",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNSimplify", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1648]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "False"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNIndexRename", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNFJacobi", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1648]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["CalcColorFactor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CalcColorFactor",
  ButtonNote->"CalcColorFactor"],
 "."
}], "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1649]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{"a", "b"}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1649]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1650]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1650]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "a", "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1651]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1651]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1652]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1652]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1653]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1653]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "r", ",", "s"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"b", ",", "r", ",", "s"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1654]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "r", "s"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"b", "r", "s"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1654]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1655]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1655]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", ".", " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1656]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], ".", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1656]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1657]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubsuperscriptBox["C", "A", "2"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1657]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1658]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"d", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1658]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1659]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1660]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["d", 
    RowBox[{"b", "c", "d"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t6", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1661]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1662]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1663]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["N", "2"], "-", "4"}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1664]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1664]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1665]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "+", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1665]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"c", ",", "b", ",", "a"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1666]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1666]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t7", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "e"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "e"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "z"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "z"}], "]"}]}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1667]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "e"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "e"}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "z"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "z"}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1668]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNIndexRename", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1669]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1669]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"1", "-", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "i"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1670]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   SubsuperscriptBox["C", "A", "2"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1670]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t8", "=", 
  RowBox[{"DiracTrace", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"SUNIndex", "[", "a", "]"}], "]"}], 
    RowBox[{
     RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
     RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}], "]"}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1671]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1671]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1672]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1672]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1673]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"f", "(", 
    FormBox["a",
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1673]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6", ",", "t7", ",",
    "t8"}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1674]:="]
}, Open  ]],

Cell["SUNT", "ObjectName",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNT"],

Cell[TextData[{
 "SUNT[a] is the SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " generator in the fundamental representation."
}], "Text",
 CellTags->"SUNT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1675]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["T", 
   FormBox["a",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1675]="]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " is a noncommutative object, products have to separated by a dot (.)."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNT", "[", "a", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "b", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "c", "]"}]}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1676]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1676]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1677]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["d",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1677]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1678]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1678]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1679]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["C", "F", "2"], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1679]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1680]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}], " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1680]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1681]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1681]="]
}, Open  ]],

Cell[TextData[{
 "The normalizaton of the generators is chosen in the standard way, therefore \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"tr", "(", 
     RowBox[{
      SubscriptBox["T", "a"], 
      SubscriptBox["T", "b"]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "2"}], " ", 
     RowBox[{
      SubscriptBox["\[Delta]", "ab"], "."}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1682]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1682]="]
}, Open  ]],

Cell[TextData[{
 "In case you want ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ", you need to include a factor 2Tf inside the trace."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"2", " ", "Tf", " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1683]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1683]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNTrace", "[", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1684]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1684]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1685]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"SUNIndex", "[", "a", "]"}], "]"}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1685]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1686]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1686]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNTrace", "ObjectName",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNTrace"],

Cell["SUNTrace[expr] calculates the color-trace.", "Text",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNTrace", "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1687]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1687]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"SUNTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1688]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1688]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1689]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1689]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1690]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], "4"], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1690]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1691]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1691]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1692]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], "e"}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], "e"}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1692]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t1", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1693]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1693]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1694]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SubscriptBox["d", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], "4"], "+", 
       RowBox[{
        FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}]}], ")"}]}], 
    RowBox[{"2", " ", "N"}]], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubscriptBox["d", 
     RowBox[{"c119", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "8"]}], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c120"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c120", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c120"}]]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], "c119"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "8"]}], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c119",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c121"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c121", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c119",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c121"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1694]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1695]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1695]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1696]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1696]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1697]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1697]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "d", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1698]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1698]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1699]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SubsuperscriptBox["C", "A", "2"]}], ")"}]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1699]="]
}, Open  ]]
}, Open  ]],

Cell["SymbolicSum2", "ObjectName",
 CellTags->"SymbolicSum2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum2"],

Cell["\<\
SymbolicSum2 is similar to SymbolicSum (Algegra`SymbolicSum`SymbolicSum was a \
function to do symbolic summation. It was obsolete from version 3 - all \
functionality is now autoloaded by Sum), but extended to several double sums.\
\
\>", "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],

Cell["SymbolicSum3", "ObjectName",
 CellTags->"SymbolicSum3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum3"],

Cell["\<\
SymbolicSymbolicSum3 is similar to SymbolicSum \
(Algegra`SymbolicSum`SymbolicSum was a function to do symbolic summation. It \
was obsolete from version 3 - all functionality is now autoloaded by Sum), \
but extended to several double sums.\
\>", "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],

Cell["Symmetrize", "ObjectName",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Symmetrize"],

Cell["\<\
Symmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"Symmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["AntiSymmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiSymmetrize",
  ButtonNote->"AntiSymmetrize"],
 "."
}], "Text",
 CellTags->"Symmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1700]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Symmetrize",
 CellLabel->"Out[1700]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1701]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Symmetrize",
 CellLabel->"Out[1701]="]
}, Open  ]]
}, Open  ]],

Cell["TBox, Tbox", "ObjectName",
 CellTags->{"TBox", "Tbox"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"TBox", "Tbox"}],

Cell["\<\
TBox[a, b, ...] or Tbox[a, b, ...] produces a RowBox[{a,b, ...}] where a, b, \
... are boxed in TraditionalForm. TBox and Tbox are used internally by \
FeynCalc to produce the typeset output in TraditionalForm\
\>", "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],

Cell["Tdec", "ObjectName",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tdec"],

Cell["\<\
Tdec[{q,mu}, {p}]; Tdec[{{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] or \
Tdec[exp, {{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] calculates the tensorial \
decomposition formulas. The more common ones are saved in TIDL.\
\>", "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tdec", "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1702]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "True"}], ",", 
    RowBox[{"List", "\[Rule]", "True"}], ",", 
    RowBox[{"NumberOfMetricTensors", "\[Rule]", "\[Infinity]"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1702]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"Tdec"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tdec"],

Cell[TextData[{
 "Check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"f", "(", 
       RowBox[{"p", ",", "q"}], ")"}], 
      SuperscriptBox["q", "\[Mu]"]}]}], "=", " ", 
    RowBox[{
     FractionBox[
      SuperscriptBox["p", "\[Mu]"], 
      SuperscriptBox["p", "2"]], 
     RowBox[{"\[Integral]", 
      RowBox[{
       SuperscriptBox["d", "D"], 
       RowBox[{"f", "(", 
        RowBox[{"p", ",", "q"}], ")"}], 
       RowBox[{"p", "\[CenterDot]", "q"}]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tdec", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1703]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"X1", "\[Rule]", 
       SuperscriptBox["m", "2"]}], ",", 
      RowBox[{"X2", "\[Rule]", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}]}], "}"}], ",", 
    FractionBox[
     RowBox[{"X2", " ", 
      SuperscriptBox["p", "\[Mu]"]}], "X1"]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1703]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"%", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1704]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm]}], 
   SuperscriptBox["m", "2"]], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1704]="]
}, Open  ]],

Cell[TextData[{
 "This calculates integral transformation for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "1"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "2"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "3"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"p", ",", 
      RowBox[{
       SubscriptBox["q", 
        RowBox[{"1", ","}]], 
       SubscriptBox["q", "2"]}], ",", 
      SubscriptBox["q", "3"]}], ")"}], " ", 
    SubsuperscriptBox["q", "1", "\[Mu]"], 
    SubsuperscriptBox["q", "2", "\[Nu]"], 
    SubsuperscriptBox["q", "3", "\[Rho]"]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Tdec"],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"Tdec", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "1"], ",", "\[Mu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "2"], ",", "\[Nu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "3"], ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}]}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1705]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"t", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1706]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}], ")"}], " ", 
     SuperscriptBox["p", "\[Rho]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "3"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{"D", "+", "1"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}], ")"}], " ", 
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{"D", "+", "1"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}], ")"}], " ", 
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{"D", "+", "1"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["D", "2"]}], ")"}], " ", 
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["p", "\[Rho]"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["q", "3"],
             TraditionalForm]}],
           TraditionalForm], ")"}]}], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        SuperscriptBox["m", "2"]}], "+", 
       RowBox[{"D", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "+", 
       RowBox[{"2", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{"D", "+", "1"}], ")"}], " ", 
     SuperscriptBox["m", "6"]}]]}], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1706]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{"%", " ", 
     RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}], 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Rho]"}], "]"}]}], "]"}], "//", "Factor"}]], "Input",\

 CellTags->"Tdec",
 CellLabel->"In[1707]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox["m", "6"]}]], 
   RowBox[{
    SuperscriptBox[
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "4"]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "4"]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "4"]}], "-", 
      RowBox[{"3", " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       SuperscriptBox["m", "2"]}], "+", 
      RowBox[{"D", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1707]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1708]:="]
}, Open  ]],

Cell["TensorFunction", "ObjectName",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TensorFunction"],

Cell["\<\
Tensorfunction[t, mu, nu, ...] transform into t[LorentzIndex[mu], \
LorentzIndex[nu], ...], i.e., it can be used as an unspecified tensoriell \
function t. A symmetric tensor can be obtained by Tensorfunction[{t, \"S\"}, \
mu, nu, ...], and an antisymmteric one by Tensorfunction[{t, \"A\"}, mu, nu, \
...].\
\>", "Text",
 CellTags->"TensorFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"TensorFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1709]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1709]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1710]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1710]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "%"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1711]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"Tensorfunction", "(", 
    RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1711]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1712]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", 
  RowBox[{"Tensorfunction", "[", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
   "]"}]}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1712]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Alpha]", ",", "\[Beta]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1713]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Alpha]", ",", 
    "\[Beta]"}], ")"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1713]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1714]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", 
    "\[Alpha]"}], ")"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1714]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1715]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1715]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "f", "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1716]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ClearAttributes", "[", 
  RowBox[{"f", ",", "Orderless"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1717]:="]
}, Open  ]],

Cell["Tf", "ObjectName",
 CellTags->"Tf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tf"],

Cell[TextData[{
 "Tf is the color factor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ". It is 1/2 for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"Tf"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr2",
  ButtonNote->"Tr2"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 "."
}], "Text",
 CellTags->"Tf"]
}, Open  ]],

Cell["TFi", "ObjectName",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TFi"}]], "Input",
 CellLabel->"In[1718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Information", "::", "\<\"notfound\"\>"}], 
   RowBox[{
   ":", " "}], "\<\"Symbol \\!\\(TraditionalForm\\`\\\"TFi\\\"\\) not found. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/notfound\\\", \
ButtonNote -> \\\"Information::notfound\\\"]\\)\"\>"}], 
  TraditionalForm]], "Message", "MSG"]
}, Open  ]],

Cell["\<\
TFi[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is the 2-loop \
d-dimensional integral 1/( (q1^2 - m1^2)^n1 (q2^2 - m2^2)^n2 ((q1-p)^2 - \
m3^2)^n3 *((q2-p)^2 - m4^2)^n4 * ((q1-q2)^2 - m5^2)^n5 ) . TFi[d, pp, \
{x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as additional \
factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z* (q2.p)^v*(q1.q2)^w. \
TFi[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as \
additional factors in the numerator (OPEDelta.q1)^a * (OPEDelta.q2)^b; dp is \
(OPEDelta.p).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "TFi is similar to TFI from the TARCER package, see ",
 " ",
 ButtonBox["hep-ph/9801383.",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}]
}], "Text",
 CellTags->"TFi"],

Cell["\<\
The function TarcerRecurse from the TARCER package recognize TFi (as well as \
TFI, which is defined in the HighEnergyPhysics`Tarcer` context).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ToTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToTFi",
  ButtonNote->"ToTFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"TFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "m1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "m3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m5"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1719]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "m1"},
       {"1", "m2"},
       {"3", "m3"},
       {"1", "m4"},
       {"1", "m5"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"TFi",
 CellLabel->"Out[1719]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1720]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
      "1"}], "}"}]}], ")"}], TraditionalForm]], "Output",
 CellTags->"TFi",
 CellLabel->"Out[1720]="]
}, Open  ]]
}, Open  ]],

Cell["ThreeVector", "ObjectName",
 CellTags->"ThreeVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ThreeVector"],

Cell["ThreeVector[p] is the three dimensional vector p.", "Text",
 CellTags->"ThreeVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CrossProduct",
  ButtonNote->"CrossProduct"],
 "."
}], "Text",
 CellTags->"ThreeVector"]
}, Open  ]],

Cell["TID", "ObjectName",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TID"],

Cell["TID[amp,q] performs a tensorial decomposition.", "Text",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TID", "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1721]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Isolate", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1721]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1722]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1722]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Factor", "/@", 
   RowBox[{"TID", "[", 
    RowBox[{"%", ",", "k"}], "]"}]}], "//", "FCE"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm], " ", 
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", 
         FormBox["2",
          TraditionalForm], "\[Mu]"], " ", 
        FormBox[
         SubsuperscriptBox["p", "1", "2"],
         TraditionalForm]}], "-", 
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["1",
           TraditionalForm], "\[Mu]"]}], "2"]}], ")"}]}], 
    RowBox[{
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}], "-", 
     FractionBox[
      SuperscriptBox["s", "2"], "4"]}]], "+", 
   FractionBox[
    RowBox[{
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm], " ", 
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["2",
           TraditionalForm], "\[Mu]"]}], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", 
         FormBox["1",
          TraditionalForm], "\[Mu]"], " ", 
        FormBox[
         SubsuperscriptBox["p", "2", "2"],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     FractionBox[
      SuperscriptBox["s", "2"], "4"], "-", 
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1723]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1724]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1724]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1725]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1725]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%%", ",", "k", ",", 
   RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "True"}]}], 
  "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1726]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1726]="]
}, Open  ]]
}, Open  ]],

Cell["TIDL", "ObjectName",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TIDL"],

Cell[" TIDL is a database of tensorial reduction formalae.", "Text",
 CellTags->"TIDL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"]
}], "Text",
 CellTags->"TIDL"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[TextData[{
 "In any n-dimensional integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "n"], "\[NegativeVeryThinSpace]", 
     SuperscriptBox["q", "\[Mu]"], " ", 
     RowBox[{"f", "(", 
      RowBox[{"q", ",", "p"}], ")"}], " ", "the", " ", 
     SuperscriptBox["q", "\[Mu]"]}], " "}], TraditionalForm]]],
 "can be replaced by"
}], "Text",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1727]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1727]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1728]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1728]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1729]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      SubsuperscriptBox["p", "1", "2"]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1729]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1730]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", "D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "3"]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "3"]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", "D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1730]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1731]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1732]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"], ",", 
     SubscriptBox["p", "3"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1733]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["3",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}], ")"}]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "3"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "3", "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "3"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "3", "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1733]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1734]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1735]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r", ",", "s"}], "}"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1736]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Length", "[", "%", "]"}], ",", " ", 
   RowBox[{"LeafCount", "[", "%", "]"}], ",", 
   RowBox[{"ByteCount", "[", "%", "]"}]}], "}"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1737]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "29", ",", "536"}], "}"}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1737]="]
}, Open  ]]
}, Open  ]],

Cell["TimedIntegrate", "ObjectName",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["\<\
 TimedIntegrate[exp, vars] is like Integrate, but stops after the number of \
seconds specified by the option Timing. Options of Integrate can be given and \
are passed on.\
\>", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TimedIntegrate", "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1738]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Timing", "\[Rule]", "10"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}], ",", 
    RowBox[{"Integrate", "\[Rule]", "Integrate"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1738]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["This should reach to be done", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", "^", "5"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1739]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "5"}], TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1739]="]
}, Open  ]],

Cell["This shouldn't", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Cos", "[", 
     RowBox[{"x", "^", "5"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "10"}], ",", 
   RowBox[{"Integrate", "\[Rule]", "int"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1740]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"int", "(", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"cos", "(", 
      SuperscriptBox["x", "5"], ")"}], ")"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}]}], ")"}], TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1740]="]
}, Open  ]]
}, Open  ]],

Cell["TLI  ***unfinished***", "ObjectName",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI"],

Cell["\<\
TLI[{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}].  The exponents of the \
numerator scalar product are (dl = OPEDelta):  v: k1.k1, w: k2.k2,  x: p.k1, \
y: p.k2, z: k1.k2.  a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: \
dl.(k1-k2).
TLI[{mu1, ...}, {nu1, ...}][{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}] is a \
tensor integral, where mu1 belongs to k1 and nu1 to k2.
TLI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  TLI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}].
TLI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
TLI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI", "]"}]], "Input",
 CellTags->"TLI",
 CellLabel->"In[1741]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsilonOrder", "\[Rule]", "0"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLI",
 CellLabel->"Out[1741]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI2FC",
  ButtonNote->"TLI2FC"],
 "."
}], "Text",
 CellTags->"TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI"],

Cell[BoxData[""], "Input",
 CellTags->"TLI",
 CellLabel->"In[1742]:="]
}, Open  ]],

Cell["TLIFP  ***unfinished***", "ObjectName",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIFP"],

Cell["TLIFP[exp] does Feynman-Parametrizations of TLI's in exp.", "Text",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIFP", "]"}]], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "t", ",", "u", ",", "s", ",", "y"}], "}"}]}], ",", 
    RowBox[{"GammaExpand", "\[Rule]", "True"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}], ",", 
    RowBox[{"Print", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLIFP",
 CellLabel->"Out[1743]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIFP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIFP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1744]:="]
}, Open  ]],

Cell["TLIHYP  ***unfinished***", "ObjectName",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIHYP"],

Cell["\<\
TLIHYP[exp] expresses TLI's in exp. in terms of hypergeometric functions, \
where possible.\
\>", "Text",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIHYP", "]"}]], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1745]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", "x"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLIHYP",
 CellLabel->"Out[1745]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIHYP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIHYP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1746]:="]
}, Open  ]],

Cell["TLI2", "ObjectName",
 CellTags->"TLI2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2"],

Cell["like TLI, but without any functional properties.", "Text",
 CellTags->"TLI2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLI2"]
}, Open  ]],

Cell["TLI2FC  ***unfinished***", "ObjectName",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2FC"],

Cell["\<\
TLI2FC[exp] transforms all TLI-integrals in exp to the FAD form.\
\>", "Text",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI2FC", "]"}]], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1747]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Momentum", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"q1", ",", "q2", ",", "p"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TLI2FC",
 CellLabel->"Out[1747]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 ".  ",
 ButtonBox["FC2TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC2TLI",
  ButtonNote->"FC2TLI"],
 "."
}], "Text",
 CellTags->"TLI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1748]:="]
}, Open  ]],

Cell["ToDistribution", "ObjectName",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToDistribution"],

Cell["\<\
ToDistribution[exp,x] replaces (1-x)^(a Epsilon - 1) in exp by 1/(a Epsilon) \
DeltaFunction[1-x] + 1/(1-x) + a Epsilon Log[1-x]/(1-x) + 1/2 a^2 Epsilon^2 \
Log[1-x]^2/(1-x)] and (1-x)^(a Epsilon - 2) in exp by -1/(a Epsilon) \
DeltaFunctionPrime[1-x] + 1/(1-x)^2 + (a Epsilon) Log[1-x]/(1-x)^2 + a^2 \
Epsilon^2/2 Log[1-x]^2/(1-x)^2 + a^3 Epsilon^3/6 Log[1-x]^3/(1-x)^2.\
\>", "Text",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ToDistribution", "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1749]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1749]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 "."
}], "Text",
 CellTags->"ToDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}]}], 
  "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1750]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1750]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "1"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1751]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "6"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "3"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]}], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"], "+", 
   FractionBox[
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1751]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1752]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"6", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1753]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"Epsilon", "-", "2"}]], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], ",", 
   "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1754]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1754]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "@", "Integrate"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1755]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Assumptions", "\[RuleDelayed]", "$Assumptions"}], ",", 
    RowBox[{"GenerateConditions", "\[Rule]", "False"}], ",", 
    RowBox[{"PrincipalValue", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1755]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"ToDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "^", 
      RowBox[{"(", 
       RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1756]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1756]="]
}, Open  ]]
}, Open  ]],

Cell["ToHypergeometric", "ObjectName",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "ToHypergeometric[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["t",
      FontSlant->"Plain"], 
     StyleBox["b",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       StyleBox["t",
        FontSlant->"Plain"]}], ")"}], 
     StyleBox["c",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{
        StyleBox["t",
         FontSlant->"Plain"], " ", 
        StyleBox["z",
         FontSlant->"Plain"]}]}], ")"}], 
     StyleBox["a",
      FontSlant->"Plain"]]}], TraditionalForm]],
  FontVariations->{"CompatibilityType"->0}],
 ", ",
 "t",
 "] returns ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"c", "+", "1"}], ")"}], "/", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"b", "+", "c", "+", "2"}], ")"}]}], " ", 
    FormBox[
     RowBox[{
      SubscriptBox["\[Null]", "2"], 
      RowBox[{
       SubscriptBox["F", "1"], "(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "a"}], ",", 
         RowBox[{"b", "+", "1"}]}], ";", 
        RowBox[{"b", "+", "c", "+", "2"}], ";", 
        RowBox[{"-", "z"}]}], ")"}]}],
     TraditionalForm]}], TraditionalForm]]],
 ". Remember that Re ",
 StyleBox["b",
  FontSlant->"Italic"],
 " >0 and Re (",
 StyleBox["c",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") > 0 should hold (need not be set in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 "."
}], "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1757]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "1"}], ")"}], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"b", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"b", "+", "c", "+", "2"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", "z"}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}], 
   RowBox[{"\[CapitalGamma]", "(", 
    RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1757]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1758]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"w", " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "-", "b"}], ")"}], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox["z",
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}], 
   RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1758]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1759]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["u", "a"], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "+", "1"}], ")"}], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"-", "a"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"b", "+", "1"}],
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox[
            RowBox[{"b", "+", "c", "+", "2"}],
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          RowBox[{"-", 
           FractionBox["z", "u"]}],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}], 
   RowBox[{"\[CapitalGamma]", "(", 
    RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1760]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["u", 
     RowBox[{"-", "a"}]], " ", "w", " ", 
    RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"c", "-", "b"}], ")"}], " ", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "2"], 
        SubscriptBox["F", "1"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["a",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True], ",", 
            TagBox["b",
             Hypergeometric2F1,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           TagBox["c",
            Hypergeometric2F1,
            Editable->True,
            Selectable->True],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          Hypergeometric2F1,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          FractionBox["z", "u"],
          Hypergeometric2F1,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     Hypergeometric2F1]}], 
   RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1760]="]
}, Open  ]]
}, Open  ]],

Cell["ToLarin", "ObjectName",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToLarin"],

Cell[TextData[{
 "ToLarin[exp] translates all ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]]],
 " in exp into ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], "/", "6"}], " ", 
    SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Lambda]\[Sigma]"], 
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["\[Gamma]", "\[Lambda]"], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Sigma]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"ToLarin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ToLarin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "5", "]"}]}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1761]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"ToLarin",
 CellLabel->"Out[1761]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToLarin", "[", "%", "]"}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1762]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du1",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du2",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du3",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"ToLarin",
 CellLabel->"Out[1762]="]
}, Open  ]]
}, Open  ]],

Cell["ToTFi", "ObjectName",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToTFi"],

Cell["\<\
ToTFi[expr, q1, q2, p] translates FeynCalc 2-loop self energy type integrals \
into the TFi notatation, which can be used as input for the function \
TarcerRecurse from the TARCER package. See TFi for details on the \
conventions.\
\>", "Text",
 CellTags->"ToTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TFi",
  ButtonNote->"TFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"ToTFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", 
   RowBox[{"q1", "-", "p"}], ",", 
   RowBox[{"{", 
    RowBox[{"q2", ",", "M"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
   RowBox[{"q1", "-", "q2"}]}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1763]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q1^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q2", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, "]"],
      
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1763]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{"%", ",", "q1", ",", "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1764]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"ToTFi", "(", 
   RowBox[{
    FormBox[
     FractionBox["1", 
      RowBox[{
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q1^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
          "\[InvisibleSpace]", 
          SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q2", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm[
         "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, 
          "]"],
         Editable->False], ")"}]}]],
     TraditionalForm], ",", "q1", ",", "q2", ",", "p"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1764]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1765]:="],

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q1"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "p"}], "+", "q2"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1765]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "q1", "]"}], " ", 
   RowBox[{"SOD", "[", "q2", "]"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1766]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1766]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TR", "ObjectName",
 CellTags->"TR"],

Cell[BoxData[
 FormBox["TR", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TR"],

Cell[TextData[{
 "TR[exp] calculates the Dirac trace of exp. Depending on the setting of the \
option SUNTrace also a trace over SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") objects is performed. The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " build-in function Tr is overloaded to call TR if ",
 "any of DiracGamma, DiracSlash, GA, GAD, GS or GSD",
 " are in the expression."
}], "Text",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TR", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1767]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "442"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1767]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"TR"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1768]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1768]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", "%", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1769]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1769]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "q", "]"}], "-", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Nu]", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1770]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}], " ", 
      SuperscriptBox["m", "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
   "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1771]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Xi]", ",", "5"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1772]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Xi]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Xi]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Rho]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Sigma]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Tau]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1772]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$West"], "Input",
 CellTags->"TR",
 CellLabel->"In[1773]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1773]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "$West"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1774]:="],

Cell[BoxData[
 StyleBox["\<\"If $West is set to True (which is the default), traces \
involving more than 4 Dirac matrices and gamma5 are calculated recursively \
according to formula (A.5) from Comp. Phys. Comm 77 (1993) 286-298, which is \
based on the Breitenlohner Maison gamma5 - scheme.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellTags->{"Info3387712206-4027155", "TR"}]
}, Open  ]],

Cell["\<\
$West=False should be set before any calculation is done (because \
intermediate results from trace calculations are cached).\
\>", "Text",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$West", "=", "False"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1775]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1775]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GAD", "[", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], ",", 
     SubscriptBox["\[Nu]", "2"], ",", 
     SubscriptBox["\[Nu]", "3"], ",", 
     SubscriptBox["\[Nu]", "4"], ",", 
     SubscriptBox["\[Nu]", "5"], ",", 
     SubscriptBox["\[Nu]", "6"]}], "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1776]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1776]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1777]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}], "+", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["r",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1777]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "q", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"Factoring", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1778]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m", "2"]}], "-", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1778]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1779]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1779]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1780]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1780]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Beta]", "]"}], ".", 
    RowBox[{"GAD", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1781]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "D", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1781]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1782]:="],

Cell[BoxData[
 RowBox[{"4", " ", "D", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1782]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
   RowBox[{
    RowBox[{"SUNT", "[", "b", "]"}], ".", 
    RowBox[{"SUNT", "[", "c", "]"}]}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "b"}], "]"}]}]}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1783]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox["b",
      TraditionalForm]], ".", 
    SubscriptBox["T", 
     FormBox["c",
      TraditionalForm]]}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1783]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1784]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1785]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1786]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1786]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1787]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1787]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr", "[", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"m", ",", "n"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1788]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1788]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1789]:="]
}, Open  ]],

Cell["TraceDimension", "ObjectName",
 CellTags->"TraceDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceDimension"],

Cell["\<\
TraceDimension is an option for FeynCalc2FORM. If set to 4: trace, if set to \
n: tracen.\
\>", "Text",
 CellTags->"TraceDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"TraceDimension"]
}, Open  ]],

Cell["TraceOfOne", "ObjectName",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell["\<\
TraceOfOne is an option for Tr and DiracTrace. Its setting determines the \
value of the unit trace.\
\>", "Text",
 CellTags->"TraceOfOne"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr1"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1790]:="],

Cell[BoxData[
 FormBox["tr1", TraditionalForm]], "Output",
 CellTags->"TraceOfOne",
 CellLabel->"Out[1790]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr2"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1791]:="],

Cell[BoxData[
 FormBox["tr2", TraditionalForm]], "Output",
 CellTags->"TraceOfOne",
 CellLabel->"Out[1791]="]
}, Open  ]]
}, Open  ]],

Cell["Trick", "ObjectName",
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Trick"],

Cell["\<\
Trick[exp] performs several basic simplifications without expansion. \
Trick[exp] uses Contract, DotSimplify and SUNDeltaContract.\
\>", "Text",
 CellTags->"Trick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Trick"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Trick"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", " ", "\[Nu]"}]], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"GA", "[", "\[Mu]", "]"}], " ", 
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1792]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1792]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1793]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1793]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", "%", "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1794]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "+", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1794]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"c", ".", "b", ".", "a", " ", ".", " ", 
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1795]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", " ", "b", " ", "c", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1795]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1796]:="],

Cell[BoxData[
 RowBox[{"a", " ", "b", " ", "c", " ", 
  RowBox[{
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}]}]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1796]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["TrickIntegrate", "ObjectName",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell["\<\
 TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] does an integration trick for \
the definite integral of ((1-t)^(a Epsilon -1) g[t])  from 0 to 1. \
TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] gives  g[1]/a/Epsilon + \
Hold[Integrate][(1-t)^(a Epsilon-1) (g[t]-g[1]),{t,0,1}]. TrickIntegrate[t^(a \
Epsilon -1) f[t], t] gives f[0]/a/Epsilon + Hold[Integrate][t^(a Epsilon-1) \
(f[t]-f[0]),{t,0,1}], provided g[1] and f[0] do exist.\
\>", "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", "t"}], "]"}]], "Input",
 CellTags->"TrickIntegrate",
 CellLabel->"In[1797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Hold", "[", "Integrate", "]"}], "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"a", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]], " ", 
     RowBox[{"g", "(", "t", ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}], 
  TraditionalForm]], "Output",
 CellTags->"TrickIntegrate",
 CellLabel->"Out[1797]="]
}, Open  ]]
}, Open  ]],

Cell["TrickMandelstam", "ObjectName",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[TextData[{
 "TrickMandelstam[expr, {s, t, u, ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "4", "2"], TraditionalForm]]],
 "}] simplifies all sums in expr so that one of the Mandelstam variables s, t \
or u is eliminated by the relation ",
 StyleBox["s",
  FontSlant->"Italic"],
 " + ",
 StyleBox["t",
  FontSlant->"Italic"],
 " + ",
 StyleBox["u",
  FontSlant->"Italic"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Italic"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " . The trick is that the resulting sum has the most short number of terms."
}], "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TrickMandelstam", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"s", "+", "t", "-", "u"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["M", "W", "2"]}], "-", "t", "-", "u"}], ")"}]}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "t", ",", "u", ",", 
      RowBox[{"2", 
       SubsuperscriptBox["M", "W", "2"]}]}], "}"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1798]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "s", " ", 
   RowBox[{"(", 
    RowBox[{"u", "-", 
     SubsuperscriptBox["M", "W", "2"]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"TrickMandelstam",
 CellLabel->"Out[1798]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickMandelstam", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "s"}], " ", "-", " ", 
    RowBox[{"s", "^", "2"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "t"}], " ", "-", " ", "st", " ", "+", " ", 
    
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "u"}], " ", "-", " ", "su"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t", ",", "u", ",", 
     RowBox[{"2", 
      RowBox[{"M", "^", "2"}]}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1799]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["M", "4"]}], "-", 
   SuperscriptBox["s", "2"], "-", "st", "-", "su"}], 
  TraditionalForm]], "Output",
 CellTags->"TrickMandelstam",
 CellLabel->"Out[1799]="]
}, Open  ]]
}, Open  ]],

Cell["Tr2", "ObjectName",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tr2"],

Cell["\<\
If exp contains DiracTrace's, Tr2[exp] simplifies exp and does the Dirac \
traces unless more that 4 gamma matrices and DiracGamma[5] occur. Tr2[exp] \
also separates the color-strucure, and takes the color trace if Tf occurs in \
exp. If exp does not contain DiracTrace's, Tr2[exp] takes the Dirac trace.\
\>", "Text",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tr2", "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1800]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Factoring", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tr2",
 CellLabel->"Out[1800]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["Tf",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tf",
  ButtonNote->"Tf"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"Tr2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr2", "[", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1801]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "d"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "c"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "c"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "d"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "b"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"c", "d"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[1801]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"a", " ", "Tf"}], ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1802]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "a", " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"Tr2",
 CellLabel->"Out[1802]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2AlienOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell["\<\
Twist2AlienOperator[p, 0] : (7);   Twist2AlienOperator[p1,p2,{p3,mu,a}, 0] \
(p1: incoming quark momentum, p3: incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p", ",", " ", "0"}], "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1803]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"2", " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1803]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", " ", "0"}], 
  "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1804]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
    SubsuperscriptBox["g", "s", "3"], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["mu",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          RowBox[{"m", "-", "1"}]], "-", 
         FractionBox["1", "m"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}], ")"}]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["2", "m"], "-", 
         FractionBox["1", 
          RowBox[{"m", "+", "1"}]], "-", 
         FractionBox["2", 
          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p1",
              TraditionalForm]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]], "-", 
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p3",
                TraditionalForm]}], ")"}]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]]}], ")"}]}]}], ")"}]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1804]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2CounterOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell["\<\
Twist2CounterOperator[p,mu,nu,a,b,5]; Twist2CounterOperator[p, 7] : (7);  \
Twist2CounterOperator[p1,p2,{p3,mu,a}, 1] (p1: incoming quark momentum, p3: \
incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "mu", ",", "nu", ",", "a", ",", "b", ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["24", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["24", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["4", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["mu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["nu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "2"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["6", "m"]}], "+", 
          FractionBox["16", 
           RowBox[{"m", "+", "1"}]], "-", 
          FractionBox["16", 
           RowBox[{"m", "+", "2"}]], "+", 
          FractionBox["6", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["12", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["8", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["8", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"mu", "nu"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"]}]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]]}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "7"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1806]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1806]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", "1"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1807]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["C", "A"], "-", 
       RowBox[{"2", " ", 
        SubscriptBox["C", "F"]}]}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["2", "m"], "-", 
       FractionBox["1", 
        RowBox[{"m", "+", "1"}]], "-", 
       FractionBox["2", 
        RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["mu",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p1",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]], "-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p2",
              TraditionalForm]}], ")"}]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]]}], ")"}], " ", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1807]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2GluonOperator", "ObjectName",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["\<\
Twist2GluonOperator[{p, mu, a}, {nu, b}] or Twist2GluonOperator[p, {mu, a}, \
{nu, b}] or Twist2GluonOperator[p, mu,a, nu,b] yields the 2-gluon operator (p \
is ingoing momentum corresponding to Lorentz index mu). \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}] or Twist2GluonOperator[ \
p,mu,a , q,nu,b , k,la,c ] gives the 3-gluon operator. \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
Twist2GluonOperator[p,mu,a , q,nu,b , k,la,c , s,si,d] yields the 4-Gluon \
operator. The dimension is determined by the option and Dimension. The \
setting of the option Polarization (unpolarized: 0; polarized: 1) determines \
whether the unpolarized or polarized feynman rule is returned. With the \
setting Explicit to False the color-structure and the (1+(-1)^OPEm) (for \
polarized: (1-(-1)^OPEm)) is extracted and the color indices are omitted in \
the arguments of Twist2GluonOperator.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion ", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", " ", 
   RowBox[{"ZeroMomentumInsertion", " ", "\[Rule]", " ", "True"}]}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1808]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1808]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1809]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1809]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1810]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1810]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1811]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1811]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1812]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1813]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm]}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm]}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1815]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c136",
           TraditionalForm],
          TraditionalForm]}]]}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c136",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c136",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c136",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c136",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c136",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1816]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1818]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1819]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1821]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1822]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1822]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1823]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], ")"}], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1823]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1825]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1826]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}], " ", 
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}], " ", 
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}], " ", 
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1826]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1827]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c137",
           TraditionalForm],
          TraditionalForm]}]]}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c137",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c137",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c137",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c137",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c137",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1827]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1828]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1828]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1829]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{"m", "-", "3"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1829]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Examples", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["The setting All for Explicit performs the sums.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1830]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1831]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1832]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1832]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1833]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "-", 
        RowBox[{
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}]}]}], ")"}], " ", 
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm]}]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}], " ", 
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm]}]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1833]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1834]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]]}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], ")"}], "m"]}], "+", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1834]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1835]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c138",
           TraditionalForm],
          TraditionalForm]}]]}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c138",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c138",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c138",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c138",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c138",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1835]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1836]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1836]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1837]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1837]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1838]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1839]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"q", "r"}]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[Mu]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1839]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1840]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"q", "r"}]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[Mu]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1840]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1841]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c139",
           TraditionalForm],
          TraditionalForm]}]]}], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c139",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c139",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c139",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c139",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c139",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1841]="]
}, Open  ]],

Cell["Suppress the lengthy output.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
     RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1842]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], "+", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], ")"}], 
      RowBox[{"m", "-", "2"}]]}], "+", 
    RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["r",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["s",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["s",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}]}], ")"}]}]}],
   Short[#, 6]& ], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1842]//Short="]
}, Open  ]],

Cell["\<\
The setting Explicit \[Rule] All performs the sums.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1843]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1844]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["r",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{"m", "-", "3"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
            "i"]}]}],
         TraditionalForm], ")"}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{"\[LeftSkeleton]", "15", "\[RightSkeleton]"}], "+", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}], ")"}], "2"]], 
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]], " ", 
        SuperscriptBox[
         RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
         RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}], "m"], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
                "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], "3"]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"\[LeftSkeleton]", "2", "\[RightSkeleton]"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], "2"]}], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{"m", "-", "3"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}],
             TraditionalForm], "2"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}]}], ")"}]}]}], "-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "m"]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}]}]]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}],
   Short[#, 5]& ], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1844]//Short="]
}, Open  ]]
}, Open  ]],

Cell["Twist2QuarkOperator", "ObjectName",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
Twist2QuarkOperator[p] or Twist2QuarkOperator[p,_,_] yields the \
quark-antiquark operator (p is momentum in the direction of the incoming \
quark). Twist2QuarkOperator[{p,q}] yields the 2-quark operator for non-zero \
momentum insertion (p is momentum in the direction of the incoming quark). \
Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a] is the quark-antiquark-gluon \
operator, where p1 is the incoming quark, p2 the incoming antiquark and p3 \
denotes the incoming gluon momentum. Twist2QuarkOperator[{p1}, {p2}, {p3, mu, \
a}, {p4, nu, b}] or Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}, {p4, \
nu, b}] or Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a, p4, nu, b] gives the \
Quark-Quark-Gluon-Gluon-operator. The setting of the option Polarization \
(unpolarized: 0; polarized: 1) determines whether the unpolarized or \
polarized operator is returned\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1845]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1845]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1846]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", "5"]}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1846]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1847]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1847]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1848]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubsuperscriptBox["g", "s", "2"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1848]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", " ", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1849]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1849]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["\<\"\[Gamma]\"\>",
     TraditionalForm], "\[CenterDot]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1850]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1851]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1852]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal (FCE) form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Twist2QuarkOperator", "[", "p", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1853]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "OPEDelta", "]"}], " ", 
  SuperscriptBox[
   RowBox[{"SOD", "[", "p", "]"}], 
   RowBox[{
    RowBox[{"-", "1"}], "+", "OPEm"}]]}]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1853]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1854]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "1"}]], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "1"}]]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1854]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1855]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubsuperscriptBox["g", "s", "2"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}]}], ")"}]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1855]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1856]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1856]="]
}, Open  ]],

Cell["\<\
With the setting ZeroMomentumInsertion \[Rule] False a non-zero momentum is \
assumed to flow into the operator vertex. \
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
This is the Feynman rule associated with the quark-antiquark operator, where \
p is the momentum of the incoming quark and q the momentum of the antiquark. \
The momentum flowing into the operator vertex is thus -p-q.\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1857]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1857]="]
}, Open  ]],

Cell["This is the quark-antiquark-gluon operator vertex.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1858]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1858]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "t2", "]"}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1859]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["2", 
   RowBox[{"2", "-", "OPEm"}]], " ", "Gstrong", " ", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"GSD", "[", "OPEDelta", "]"}]}], " ", 
  RowBox[{"FVD", "[", 
   RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"SOD", "[", "k", "]"}]}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], "-", "OPEi", "+", "OPEm"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SOD", "[", "k", "]"}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], "OPEi"]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{
       RowBox[{"-", "2"}], "+", "OPEm"}]}], "}"}]}], "]"}]}]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1859]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1860]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1860]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1861]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1861]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{"m", "-", "2"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1862]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
   "]"}], ";"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1863]:="]
}, Open  ]],

Cell["Twist3QuarkOperator", "ObjectName",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell["\<\
Twist3QuarkOperator[p] or  Twist3QuarkOperator[p,_,_]  yields the  2-quark \
operator (p is momentum in the direction of the fermion number flow). \
Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a] Quark-Quark-Gluon-operator, \
where p1 is the incoming quark, p2 the incoming antiquark and p3 denotes the \
(incoming) gluon momentum.  Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, \
a}, {p4, nu, b}] or Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a, p4, nu, b] \
 gives the Quark-Quark-Gluon-Gluon-operator. The setting of the option \
Polarization (unpolarized: 0; polarized: 1) determines whether the \
uppolarized or polarized operator is returned.\
\>", "Text",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "//", "Options"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1864]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1864]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "[", "p", "]"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1865]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1865]="]
}, Open  ]]
}, Open  ]],

Cell["Twist4GluonOperator", "ObjectName",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell["\<\
Twist4GluonOperator[{oa, ob, oc, od}, {p1,la1,a1}, {p2,la2,a2}, {p3,la3,a3}, \
{p4,la4,a4}].\
\>", "Text",
 CellTags->"Twist4GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist3QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist3QuarkOperator",
  ButtonNote->"Twist3QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist4GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"oa", ",", " ", "ob", ",", " ", "oc", ",", " ", "od"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"p1", ",", "la1", ",", "a1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p2", ",", "la2", ",", "a2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "la3", ",", "a3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p4", ",", "la4", ",", "a4"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist4GluonOperator",
 CellLabel->"In[1866]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Twist4GluonOperator",
 CellLabel->"Out[1866]="]
}, Open  ]]
}, Open  ]],

Cell["TwoLoopSimplify  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"TwoLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell["\<\
TwoLoopSimplify[amplitude,{qu1,qu2}] simplifies the 2-loop amplitude (qu1 and \
qu2 denote the integration momenta). TwoLoopSimplify[amplitude] transforms to \
TwoLoopSimplify[amplitude, {Global`q1, Global`q2}], i.e., the integration \
momenta in amplitude must be named q1 and q2.\
\>", "Text",
 CellTags->"TwoLoopSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell[BoxData[""], "Input",
 CellTags->"TwoLoopSimplify",
 CellLabel->"In[1867]:="]
}, Open  ]],

Cell["Uncontract", "ObjectName",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Uncontract"],

Cell["\<\
Uncontract[exp, q1, q2, ...] uncontracts Eps and DiracGamma. Uncontract[exp, \
q1, q2, Pair \[Rule] {p}] uncontracts also p.q1 and p.q2; the option Pair \
\[Rule] All uncontracts all momenta except OPEDelta.\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Uncontract", "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1868]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1868]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Uncontract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1869]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1869]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1870]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$16364", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16364", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1870]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSlash", "[", "p", "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1871]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1871]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t2", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1872]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox[
      RowBox[{"$AL$16365", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16365", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1872]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1873]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$16368", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$16367", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16367", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16368", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1873]="]
}, Open  ]],

Cell["By default scalar products are not uncontracted.", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", " ", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1874]="]
}, Open  ]],

Cell["\<\
With the option Pair\[Rule]All they are \[OpenCurlyDoubleQuote]uncontracted \
\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16370", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16370", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "q"}], "]"}], "^", "2"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16371", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16371", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16371", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16371", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1876]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1877]:="]
}, Open  ]],

Cell["UnDeclareNonCommutative", "ObjectName",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell["\<\
UnDeclareNonCommutative[a, b, ...] undeclares a,b, ... to be noncommutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to False.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1878]:="],

Cell["\<\
As a side-effect of DeclareNonCommutative x is declared to be of DataType \
NonCommutative.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["DataType[x,NonCommutative]", "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1879]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1879]="]
}, Open  ]],

Cell["The inverse operation is UnDeclareNonCommutative.", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1880]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1881]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1881]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1882]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1883]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1883]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1884]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1885]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1885]="]
}, Open  ]]
}, Open  ]],

Cell["Upper", "ObjectName",
 CellTags->"Upper"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Upper"],

Cell["\<\
Upper may be used inside LorentzIndex to indicate an contravariant \
LorentzIndex.\
\>", "Text",
 CellTags->"Upper"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Upper"]
}, Open  ]],

Cell["UVPart  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"UVPart"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UVPart"],

Cell["\<\
UVPart[exp, q] discards ultraviolet finite integrals  (q = integration \
momentum).\
\>", "Text",
 CellTags->"UVPart"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UVPart"],

Cell[BoxData[""], "Input",
 CellTags->"UVPart",
 CellLabel->"In[1886]:="]
}, Open  ]],

Cell["Vectors", "ObjectName",
 CellTags->"Vectors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Vectors"],

Cell["\<\
Vectors is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a V(ectors) \
statement.\
\>", "Text",
 CellTags->"Vectors"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Vectors"]
}, Open  ]],

Cell["WriteOut", "ObjectName",
 CellTags->"WriteOut"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOut"],

Cell["\<\
WriteOut is an option for OneLoop.  If set to True, the result of OneLoop \
will be written to a file called \"name.res\", where name is the first \
argument of OneLoop.\
\>", "Text",
 CellTags->"WriteOut"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"WriteOut"]
}, Open  ]],

Cell["WriteOutPaVe", "ObjectName",
 CellTags->"WriteOutPaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOutPaVe"],

Cell["\<\
WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If set to a string, \
the results of all Passarino-Veltman PaVe's are stored in files with names \
generated from this string and the arguments of PaVe.\
\>", "Text",
 CellTags->"WriteOutPaVe"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],

Cell["Write2", "ObjectName",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Write2"],

Cell["\<\
Write2[file, val1 = expr1, val2 = expr2, ...] writes the settings val1 = \
expr1, val2 = expr2 in sequence followed by a newline, to the specified \
output file. Setting the option FormatType of Write2 to FortranForm results \
in FORTRAN syntax output.\
\>", "Text",
 CellTags->"Write2"],

Cell["\<\
The continuation character for FORTRAN output can be controlled by changing \
$FortranContinuationCharacter.\
\>", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "$FortranContinuationCharacter", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1887]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   StyleBox["\"\<&\>\"",
    ShowSpecialCharacters->False,
    ShowStringCharacters->True,
    NumberMarks->True],
   FullForm], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1887]//FullForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1888]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"D0Convention", "\[Rule]", "0"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"FortranFormatDoublePrecision", "\[Rule]", "True"}], ",", 
    RowBox[{"PageWidth", "\[Rule]", "62"}], ",", 
    RowBox[{"PostFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"PreFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"StringReplace", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1888]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1889]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldRest", "}"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1889]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"Write2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "c"}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "b"}], ")"}], "2"]}], ")"}], "2"], ",", "a", ",", 
    "Factor"}], "]"}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1890]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}], " ", 
    SuperscriptBox["a", "3"]}], "+", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"c", " ", "b"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}], " ", 
    SuperscriptBox["a", "2"]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}], "2"]}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1890]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tempfilename", "=", 
   RowBox[{
    RowBox[{"ToString", "[", "$SessionID", "]"}], "<>", "\"\<.s\>\""}]}], 
  ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1891]:="],

Cell["This writes the assignment r=t to a file.", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1892]:="],

Cell["This shows the contents of the file.", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1893]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + \"\>"},
      {"\<\" (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1893]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1894]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"x", "+", 
   RowBox[{"Isolate", "[", 
    RowBox[{"t", ",", "a", ",", 
     RowBox[{"IsolateNames", "\[Rule]", "W"}]}], "]"}]}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1895]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    TagBox[
     RowBox[{"W", "(", "1", ")"}],
     HoldForm], " ", 
    SuperscriptBox["a", "3"]}], "+", 
   RowBox[{"8", " ", 
    TagBox[
     RowBox[{"W", "(", "3", ")"}],
     HoldForm], " ", 
    SuperscriptBox["a", "2"]}], "-", 
   RowBox[{"4", " ", 
    TagBox[
     RowBox[{"W", "(", "1", ")"}],
     HoldForm], " ", 
    TagBox[
     RowBox[{"W", "(", "2", ")"}],
     HoldForm], " ", "a"}], "+", 
   SuperscriptBox[
    TagBox[
     RowBox[{"W", "(", "2", ")"}],
     HoldForm], "2"], "+", "x"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1895]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1896]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1897]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"W[1] = (b + c\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[2] = (b^2 + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[3] = (b^2 + b*c + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"r = ( 4*a^4 + x - 8*a^3*HoldForm[W[1]] - \
4*a*HoldForm[W[1]]*\"\>"},
      {"\<\"  HoldForm[W[2]] + HoldForm[W[2]]^2 + 8*a^2*HoldForm[W[3]]\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1897]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1898]:="],

Cell["\<\
This is how to write out the expression t2 in Fortran format.\
\>", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"Write2",
 CellLabel->"In[1899]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1900]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        W(1D0)= W(1D0)\"\>"},
      {"\<\"        W(2D0)= W(2D0)\"\>"},
      {"\<\"        W(3D0)= W(3D0)\"\>"},
      {"\<\"        r = 4D0*a**4 + x - 8D0*a**3*W(1D0) - \"\>"},
      {"\<\"     &  4D0*a*W(1D0)*W(2D0) + W(2D0)**2 + 8D0*a**2*W(3D0)\"\>"},
      {"\<\"                  \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1900]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"t", ",", "t2", ",", "tempfilename"}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1901]:="]
}, Open  ]],

Cell["XYT", "ObjectName",
 CellTags->"XYT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"XYT"],

Cell[TextData[{
 "XYT[exp, x,y] transforms  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "m"], TraditionalForm]]],
 " away ..."
}], "Text",
 CellTags->"XYT"]
}, Open  ]],

Cell["ZeroMomentumInsertion", "ObjectName",
 CellTags->"ZeroMomentumInsertion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],

Cell["\<\
ZeroMomentumInsertion is an option of FeynRule, Twist2GluonOperator and \
Twist2QuarkOperator.\
\>", "Text",
 CellTags->"ZeroMomentumInsertion"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 ", ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],

Cell["Zeta2", "ObjectName",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Zeta2"],

Cell["Zeta2 denotes Zeta[2]. N[Zeta[2]] evaluates to N[Zeta[2]].", "Text",
 CellTags->"Zeta2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Zeta2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell[BoxData["Zeta2"], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1902]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Zeta]", "(", "2", ")"}], TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1902]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "Zeta2", "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1903]:="],

Cell[BoxData[
 FormBox["1.6449340668482262`", TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1903]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyPolyLog", "[", 
  RowBox[{"Pi", "^", "2"}], "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1904]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"6", " ", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1904]="]
}, Open  ]]
}, Open  ]],

Cell["$Abbreviations", "ObjectName",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Abbreviations"],

Cell["\<\
$Abbreviations are a list of string substitution rules used when generating \
names for storing intermediate results. It is used by OneLoop and \
PaVeReduce.The elements of the list should be of the form \"name\" \[Rule] \
\"abbreviation\".\
\>", "Text",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$Abbreviations", "//", "StandardForm"}]], "Input",
 CellTags->"$Abbreviations",
 CellLabel->"In[1905]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\", \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"^\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"{\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"/\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"Subscript\"\>", "\[Rule]", "\<\"su\"\>"}], ",", 
   RowBox[{"\<\"SmallVariable\"\>", "\[Rule]", "\<\"sma\"\>"}], ",", 
   RowBox[{"\<\"}\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"[\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"]\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"*\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\" \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\n\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\r\"\>", "\[Rule]", "\<\"\"\>"}]}], "}"}]], "Output",
 CellTags->"$Abbreviations",
 CellLabel->"Out[1905]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Abbreviation",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Abbreviation",
  ButtonNote->"Abbreviation"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 "."
}], "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],

Cell["$AL", "ObjectName",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$AL"],

Cell["\<\
$AL is the head for dummy indices which may be introduced by Uncontract.\
\>", "Text",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell[BoxData["$AL"], "Input",
 CellTags->"$AL",
 CellLabel->"In[1906]:="],

Cell[BoxData[
 FormBox["$AL", TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1906]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1907]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16387", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$16387", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1907]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=", "\[Mu]"}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1908]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1909]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$16388", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$16388", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1909]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=."}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1910]:="]
}, Open  ]],

Cell["$BreitMaison", "ObjectName",
 CellTags->"$BreitMaison"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "The Breitenlohner-Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " scheme is currently not supported by the Dirac algebra functions.Use \
Tracer if you need it (PHI supplies the functions FCToTracer and \
TracerToFC)."
}], "Text",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$Larin",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Larin",
  ButtonNote->"$Larin"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],

Cell["$Color", "ObjectName",
 CellTags->"$Color"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Color"],

Cell["\<\
$Color is False by default. If set to True, some special variables will be \
colored.\
\>", "Text",
 CellTags->"$Color"]
}, Open  ]],

Cell["$Covariant", "ObjectName",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Covariant"],

Cell["\<\
The boolean setting of $Covariant determines whether lorentz indices are \
displayed as lower indices (True) or as upper ones (False).\
\>", "Text",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell[BoxData["$Covariant"], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1911]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1911]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1912]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1912]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "True"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1913]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1914]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1914]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "False"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1915]:="]
}, Open  ]],

Cell["$FCS", "ObjectName",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCS"],

Cell["\<\
$FCS is a list of functions with a short name. E.g. GA[nu] can be used \
instead of DiracGamma[nu].\
\>", "Text",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell[BoxData["$FCS"], "Input",
 CellTags->"$FCS",
 CellLabel->"In[1916]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"FAD\"\>", ",", "\<\"FV\"\>", ",", "\<\"FVD\"\>", 
    ",", "\<\"GA\"\>", ",", "\<\"GA5\"\>", ",", "\<\"GS\"\>", 
    ",", "\<\"GSD\"\>", ",", "\<\"LC\"\>", ",", "\<\"LCD\"\>", 
    ",", "\<\"MT\"\>", ",", "\<\"MTD\"\>", ",", "\<\"SD\"\>", 
    ",", "\<\"SOD\"\>", ",", "\<\"SP\"\>", ",", "\<\"SPC\"\>", 
    ",", "\<\"SPD\"\>", ",", "\<\"SPL\"\>", ",", "\<\"FCI\"\>", 
    ",", "\<\"FCE\"\>", ",", "\<\"FI\"\>", ",", "\<\"FC\"\>", 
    ",", "\<\"GGV\"\>", ",", "\<\"GP\"\>", ",", "\<\"QGV\"\>", 
    ",", "\<\"QO\"\>", ",", "\<\"FDr\"\>", ",", "\<\"CDr\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"$FCS",
 CellLabel->"Out[1916]="]
}, Open  ]]
}, Open  ]],

Cell["$FCT", "ObjectName",
 CellTags->"$FCT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCT"],

Cell["\<\
Setting the global variable $FCT to True changes the default typesetting \
rules for the noncommutative multiplication operator Dot[].\
\>", "Text",
 CellTags->"$FCT"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "True"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1917]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1918]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"$FCT",
 CellLabel->"Out[1918]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1919]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1920]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"$FCT",
 CellLabel->"Out[1920]="]
}, Open  ]]
}, Open  ]],

Cell["$FeynCalcStuff", "ObjectName",
 CellTags->"$FeynCalcStuff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcStuff"],

Cell["$FeynCalcStuff is the list of availabe stuff in FeynCalc.", "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],

Cell["$FeynCalcVersion", "ObjectName",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcVersion"],

Cell["\<\
$FeynCalcVersion is a string that represents the version of FeynCalc.\
\>", "Text",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell[BoxData["$FeynCalcVersion"], "Input",
 CellTags->"$FeynCalcVersion",
 CellLabel->"In[1921]:="],

Cell[BoxData[
 FormBox["\<\"5.1.0\"\>", TraditionalForm]], "Output",
 CellTags->"$FeynCalcVersion",
 CellLabel->"Out[1921]="]
}, Open  ]]
}, Open  ]],

Cell["$FortranContinuationCharacter", "ObjectName",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],

Cell["\<\
$FortranContinuationCharacter is the continuation character used in Write2.\
\>", "Text",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell[BoxData["$FortranContinuationCharacter"], "Input",
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"In[1922]:="],

Cell[BoxData[
 FormBox["\<\"&\"\>", TraditionalForm]], "Output",
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"Out[1922]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],

Cell["$Gauge", "ObjectName",
 CellTags->"$Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Gauge"],

Cell["\<\
$Gauge(= 1/xi) is a constant specifying the gauge fixing parameter of QED in \
Lorentz gauge.  The usual choice is Feynman gauge, $Gauge=1. Notice that \
$Gauge is used by some functions, the option Gauge by others.\
\>", "Text",
 CellTags->"$Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gauge",
  ButtonNote->"Gauge"],
 "."
}], "Text",
 CellTags->"$Gauge"]
}, Open  ]],

Cell["$Larin", "ObjectName",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Larin"],

Cell[TextData[{
 "If set to True, the Larin-Gorishny-Atkyampo-DelBurgo-scheme for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions is used, i.e. before evaluating traces (but after moving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " anticommuting in all dimensions to the right of the Dirac string) a \
product  gamma[mu].gamma5 is substituted to  -I/6 Eps[mu,al,be,si] \
gamma[al,be,si], where all indices live in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions now. Especially the Levic-Civita tensor is taken to be ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensional, i.e., contraction of two Eps's results in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "'s. This has (FOR ONE AXIAL-VECTOR-CURRENT ONLY, it is not so clear if this \
scheme also works for more than one fermion line involving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ") the same effect as the Breitenlohner-Maison-'t Hooft-Veltman scheme."
}], "Text",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell[BoxData["$Larin"], "Input",
 CellTags->"$Larin",
 CellLabel->"In[1923]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$Larin",
 CellLabel->"Out[1923]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$BreitMaison",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$BreitMaison",
  ButtonNote->"$BreitMaison"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$Larin"]
}, Open  ]],

Cell["$LimitTo4", "ObjectName",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LimitTo4"],

Cell["\<\
$LimitTo4 is a global variable with default setting True. If set to False, no \
limit Dimension \[Rule] 4 is performed after tensor integral decomposition.\
\>", "Text",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell[BoxData["$LimitTo4"], "Input",
 CellTags->"$LimitTo4",
 CellLabel->"In[1924]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"$LimitTo4",
 CellLabel->"Out[1924]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],

Cell["$LorentzIndices", "ObjectName",
 CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LorentzIndices"],

Cell["\<\
$LorentzIndices is a global variable. If set to True the dimension of \
LorentzIndex is displayed as an index.\
\>", "Text",
 CellTags->"$LorentzIndices"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1925]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1926]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1926]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1927]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1927]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Beta]"}], "]"}], 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1928]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Mu]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     SubscriptBox[
      FormBox["\[Beta]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1928]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1929]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1929]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1930]:="]
}, Open  ]],

Cell["$MIntegrate", "ObjectName",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MIntegrate"],

Cell["\<\
$MIntegrate is a global list of integrations done by Mathematica inside \
OPEIntDelta.\
\>", "Text",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell[BoxData["$MIntegrate"], "Input",
 CellTags->"$MIntegrate",
 CellLabel->"In[1931]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellTags->"$MIntegrate",
 CellLabel->"Out[1931]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 "."
}], "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],

Cell["$MU", "ObjectName",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MU"],

Cell["\<\
$MU is the head for dummy indices which may be introduced by Chisholm (and \
evtl. Contract and DiracReduce).\
\>", "Text",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell[BoxData["$MU"], "Input",
 CellTags->"$MU",
 CellLabel->"In[1932]:="],

Cell[BoxData[
 FormBox["$MU", TraditionalForm]], "Output",
 CellTags->"$MU",
 CellLabel->"Out[1932]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"$MU"]
}, Open  ]],

Cell["$MemoryAvailable", "ObjectName",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MemoryAvailable"],

Cell[TextData[{
 "$MemoryAvailable is  a global variable which is set to an integer ",
 StyleBox["n",
  FontSlant->"Italic"],
 ", where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the available amount of main memory in MB. The default is 128. It \
should be increased if possible. The higher $MemoryAvailable can be, the more \
intermediate steps do not have to be repeated by FeynCalc."
}], "Text",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell[BoxData["$MemoryAvailable"], "Input",
 CellTags->"$MemoryAvailable",
 CellLabel->"In[1933]:="],

Cell[BoxData[
 FormBox["256", TraditionalForm]], "Output",
 CellTags->"$MemoryAvailable",
 CellLabel->"Out[1933]="]
}, Open  ]]
}, Open  ]],

Cell["$NonComm", "ObjectName",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$NonComm"],

Cell["\<\
$NonComm contains a list of all non-commutative heads present.\
\>", "Text",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell[BoxData["$NonComm"], "Input",
 CellTags->"$NonComm",
 CellLabel->"In[1934]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "ChiralityProjector", ",", "DiracGamma", ",", "DiracGammaT", ",", 
    "DiracMatrix", ",", "DiracSigma", ",", "DiracSlash", ",", "Spinor", ",", 
    "GAD", ",", "GA", ",", "GSD", ",", "GS", ",", "LeftPartialD", ",", 
    "LeftRightPartialD2", ",", "LeftRightPartialD", ",", "PartialD", ",", 
    "PauliSigma", ",", "QuantumField", ",", "RightPartialD", ",", 
    "SpinorUBar", ",", "SpinorU", ",", "SpinorVBar", ",", "SpinorV", ",", 
    "SUNT", ",", "FieldStrength", ",", "QuarkGluonVertex", ",", 
    "QuarkPropagator", ",", "C", ",", "Twist2QuarkOperator", ",", 
    "CovariantD", ",", "OPESum", ",", "Twist2AlienOperator", ",", 
    "Twist2CounterOperator"}], "}"}], TraditionalForm]], "Output",
 CellTags->"$NonComm",
 CellLabel->"Out[1934]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommutative",
  ButtonNote->"NonCommutative"],
 "."
}], "Text",
 CellTags->"$NonComm"]
}, Open  ]],

Cell["$PairBrackets", "ObjectName",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$PairBrackets"],

Cell["\<\
$PairBrackets determines whether brackets are drawn around scalar products in \
the notebook interface.\
\>", "Text",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell[BoxData["$PairBrackets"], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1935]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1935]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1936]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1936]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "True"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1937]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1937]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1938]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{"(", 
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm], ")"}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{"(", 
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm], ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1938]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "False"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1939]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1939]="]
}, Open  ]]
}, Open  ]],

Cell["$SpinorMinimal", "ObjectName",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$SpinorMinimal"],

Cell["\<\
$SpinorMinimal is a global switch for an additional simplification attempt in \
DiracSimplify for more than one Spinor-line. The default is False, since \
otherwise it costs too much time.\
\>", "Text",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell[BoxData["$SpinorMinimal"], "Input",
 CellTags->"$SpinorMinimal",
 CellLabel->"In[1940]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$SpinorMinimal",
 CellLabel->"Out[1940]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],

Cell["$VeryVerbose", "ObjectName",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$VeryVerbose"],

Cell["\<\
$VeryVerbose is a global variable with default setting 0. If set to 1, 2, \
..., less and more intermediate comments and informations are displayed \
during calculations.\
\>", "Text",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell[BoxData["$VeryVerbose"], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1941]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"$VeryVerbose",
 CellLabel->"Out[1941]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", " ", "=", " ", "3"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1942]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Expand", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "-", "y", "-", "z"}], ")"}], "^", "6"}], "]"}], ",", 
    "x"}], "]"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1943]:="],

Cell["\<\
expanding. inhomogeneous part; LeafCount = 43; factored. #1#2#3#4#5#6\
\>", "Print",
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"length \"\>", "\[InvisibleSpace]", "21", 
    "\[InvisibleSpace]", "\<\".\"\>"}],
   SequenceForm["length ", 21, "."],
   Editable->False], TraditionalForm]], "Print",
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"collecting \"\>", "\[InvisibleSpace]", "6", 
    "\[InvisibleSpace]", "\<\" terms.\"\>"}],
   SequenceForm["collecting ", 6, " terms."],
   Editable->False], TraditionalForm]], "Print",
 CellTags->"$VeryVerbose"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", "=", "0"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1944]:="],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$V0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$V0",
  ButtonNote->"$V0"],
 "."
}], "Text",
 CellTags->"$V0"]
}, Open  ]],

Cell["$West", "ObjectName",
 CellTags->"$West"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$West"],

Cell[TextData[{
 "If $West is set to True (which is the default), traces involving more than \
4 Dirac matrices and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " are calculated recursively according to formula (A.5) from Comp. Phys. \
Comm 77 (1993) 286-298, which is based on the Breitenlohner Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "-scheme."
}], "Text",
 CellTags->"$West"],

Cell[TextData[{
 "See examples in ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"$West"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AbsoluteTime", "[", "]"}], "-", "absolutestarttime"}]], "Input",
 CellLabel->"In[1945]:="],

Cell[BoxData[
 FormBox["378.507177`9.029619112211703", TraditionalForm]], "Output",
 CellLabel->"Out[1945]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemoryInUse", "[", "]"}]], "Input",
 CellLabel->"In[1946]:="],

Cell[BoxData[
 FormBox["57401152", TraditionalForm]], "Output",
 CellLabel->"Out[1946]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input",
 CellLabel->"In[1947]:="],

Cell[BoxData[
 FormBox["57647320", TraditionalForm]], "Output",
 CellLabel->"Out[1947]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input",
 CellLabel->"In[1948]:="],

Cell[BoxData[
 FormBox["87.885493`", TraditionalForm]], "Output",
 CellLabel->"Out[1948]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions (experimental)", "Section",
 CellTags->"Experimental"],

Cell["DoPolarizationSums", "ObjectName",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
***EXPERIMENTAL***
DoPolarizationSums[exp] sums over vector polarizations for expressions with a \
factor of the form
Pair[LorentzIndex[rho1_], Momentum[Polarization[p_, -I]]]*
Pair[LorentzIndex[rho2_], Momentum[Polarization[p_, I]]].\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
In the following (and indeed everywhere else within FeynCalc), notice that \
Lorentz indices written as super or subscripts are not to be taken as such. \
Instead, by convention, when two indices are contracted one is always lower \
and the other upper.\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell["\<\
FeynCalc uses the following normalization of Polarization vectors:\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", 
       RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]}]], "Input",\

 CellTags->"DoPolarizationSums",
 CellLabel->"In[1949]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1949]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1950]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1950]="]
}, Open  ]],

Cell[TextData[{
 "DoPolarizationSums is chosen in such a way as to be consistent with this \
normalization and with ",
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", "Polarizations"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[CurlyEpsilon]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}], " ", 
    RowBox[{
     SubscriptBox[
      RowBox[{"(", 
       SuperscriptBox["\[CurlyEpsilon]", "\[Nu]"], ")"}], 
      FormBox["",
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "We can choose e.g. the following polarization vectors, labeled with a \
subscript, {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "3"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "4"], TraditionalForm]]],
 "} = {(\[ImaginaryI],0,0,0), (0,\[ImaginaryI],0,0), (0,0,\[ImaginaryI],0), \
(0,0,0,\[ImaginaryI])}."
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1951]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ev", "[", "i_", "]"}], ":=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", "I", ",", 
     "i"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1952]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ev", "[", "1", "]"}], ",", 
   RowBox[{"ev", "[", "2", "]"}], ",", 
   RowBox[{"ev", "[", "3", "]"}], ",", 
   RowBox[{"ev", "[", "4", "]"}]}], "}"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\[ImaginaryI]", "0", "0", "0"},
     {"0", "\[ImaginaryI]", "0", "0"},
     {"0", "0", "\[ImaginaryI]", "0"},
     {"0", "0", "0", "\[ImaginaryI]"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1953]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["ee[i][\[Mu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i_", "]"}], "[", "mu_", "]"}], ":=", 
   RowBox[{
    RowBox[{"ev", "[", "i", "]"}], "[", 
    RowBox[{"[", "mu", "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1954]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"i_", ",", "j_", ",", "mu_", ",", "nu_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i", "]"}], "[", "mu", "]"}], 
   RowBox[{
    RowBox[{"ee", "[", "j", "]"}], "[", "nu", "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1955]:="],

Cell[TextData[{
 StyleBox["tt[i,j,\[Mu],\[Nu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "j"], "\[Nu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"i", ",", "j"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "i", ",", "mu", ",", "nu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"mu", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"nu", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1956]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1956]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "i"], "j"],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"mu", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "j", ",", "mu", ",", "mu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1957]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1957]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "=", 
  RowBox[{
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", 
        RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1958]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", 
     FormBox[
      SuperscriptBox["\[Mu]", "*"],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "*"],
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1958]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1959]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox[
    FormBox[
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", 
       FormBox[
        SuperscriptBox["\[Mu]", "*"],
        TraditionalForm]}]],
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1960]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1962]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt", ",", "ev", ",", "ptest"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1963]:="]
}, Open  ]],

Cell["FeynmanDoIntegrals ***unfinished***", "ObjectName",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanDoIntegrals[exp, moms, vars] attempts to evaluate integrals over \
Feynman parameters vars in an expression exp as produced e.g. with \
FeynmanReduce. The variables given must be atomic quantities (AtomQ). If vars \
is omitted all variables in the integrals will be integrated. If vars is \
given, only the variables in vars will be integrated. moms is a list of all \
external momenta. The integrals in exp must be given in the form \
Integratedx[x, low, up].int, where x is the integration variable, low and up \
are the integration limits and int the integrand. The interval [low,up] is \
assumed to include integration bounds put by possible DeltaFunctions and \
moreover it is assumed that up \[GreaterEqual] 0 and that up > low. The two \
options FCIntegrate and FCNIntegrate determine which integration will be \
applied to integrals judged respectively analytically and numerically doable. \
This judging is a very rough one. Using (TimedIntegrate[##, \
Integrate\[Rule]Integrate]&) or (TimedIntegrate[##, \
Integrate\[Rule]NIntegrate]&) as the setting of one or both allows for finer \
judging. Those that are judged neither analytically nor numerically doable \
are left unevaluated, but can of course be attempted evaluated by a simple \
sustitution. Beside the explicit options of FeynmanDoIntegrals options of the \
integration functions specified (FCIntegrate and FCNIntegrate) may be given \
and are passed on to these.\
\>", "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanReduce",
  ButtonNote->"FeynmanReduce"],
 ", ",
 ButtonBox["FeynmanParametrize1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize1",
  ButtonNote->"FeynmanParametrize1"],
 "."
}], "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanDoIntegrals", "]"}]], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"NIntegrate", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsilonOrder", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}]}], ",", 
    RowBox[{"FCIntegrate", "\[Rule]", "TimedIntegrate"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}], ",", 
    RowBox[{"Series", "\[Rule]", "True"}], ",", 
    RowBox[{"FCNIntegrate", "\[Rule]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"DOT", "(", 
        RowBox[{
         RowBox[{"Sequence", "@@", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
             "HighEnergyPhysics`fcdevel`FeynmanDoIntegrals`Private`\
Integratedx", "@@", "#1"}], "&"}], ")"}], "/@", 
           RowBox[{"{", "##2", "}"}]}]}], ",", "#1"}], ")"}], "&"}], 
      ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"Out[1964]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1965]:="]
}, Open  ]],

Cell["FeynmanParametrize1 ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanParametrize1[exp,k,Method->Denominator] introduces Feynman parameters \
for all one-loop integrals in exp (k = integration momentum) using formula \
(11.A.1) from \"The Quantum Theory of Fields\" vol. 1 by Steven Weinberg.  \
FeynmanParametrize1[exp,k,Method->Exp] introduces Feynman parameters for all \
one-loop integrals in exp (k = integration momentum) using 1/(A-I eps) = I \
Integrate[Exp[-I x (A-I eps)],{x,0,Infinity}, \
Assumptions->{Arg[A]==0,Arg[eps]==0}]. In this case, when the option \
Integrate is set to True, odd factors of k-tensors are dropped and even \
factors are replaced according to Itzykson&Zuber (8-117).\
\>", "Text",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize1", "]"}]], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "}"}]}], ",", 
    RowBox[{"Method", "\[Rule]", "Denominator"}], ",", 
    RowBox[{"Integrate", "\[Rule]", "True"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"CompleteSquare", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanParametrize1",
 CellLabel->"Out[1966]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1967]:="]
}, Open  ]],

Cell["FeynmanReduce ***unfinished***", "ObjectName",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanReduce[exp,params] takes a Feynman parameterized expression exp (as \
e.g. generated with FeynmanParametrize1) and a list of Feynman parameters as \
input and attempts to simplify the expression. If no parameters are given, \
Integratedx variables in the expression will be used. Currently, reduction of \
exponentials is implemented. This will work on terms of the form \
E^p1[a,b,c,...]*p2[a,b,c,...], where p1 and p2 are fractions of polynomials \
in the Feynman parameters a,b,c,... If the option Expand is set to True, \
FeynmanReduce will attempt to bring the expression exp into a sum of such \
terms and operate on the terms one by one.\
\>", "Text",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanReduce", "]"}]], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1968]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanReduce",
 CellLabel->"Out[1968]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1969]:="],

Cell["\<\
Other objects that are considered to be ok or under active development:\
\>", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "ChargeConjugationMatrix", ",", " ", "DiracGammaT", ",", " ", 
   "SquareAmplitude"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1970]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"C", ",", "DiracGammaT", ",", "SquareAmplitude"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Experimental",
 CellLabel->"Out[1970]="]
}, Open  ]],

Cell["Objects that are under consideration for being discarded:", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "$OPEWard", ",", "$AchmedRoss", ",", "Contract2", ",", 
   "SpinSumExternalMomentum", ",", "SelectedGraphs"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1971]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "False", ",", "False", ",", "Contract2", ",", "SpinSumExternalMomentum", 
    ",", "SelectedGraphs"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Experimental",
 CellLabel->"Out[1971]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["AbsoluteTime[]-absolutestarttime", "Input",
 CellLabel->"In[1972]:="],

Cell[BoxData[
 FormBox["382.035904`9.033649173623228", TraditionalForm]], "Output",
 CellLabel->"Out[1972]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"AllBrackets",
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
WindowSize->{880, 861},
WindowMargins->{{Automatic, 114}, {Automatic, 18}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"FacingPages"->True,
"FirstPageFace"->Left,
"FirstPageFooter"->True,
"FirstPageHeader"->False,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.5625, 842.375},
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintSelectionHighlighting"->False,
"PrintingMargins"->{{54., 54.}, {72., 72.}}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{
 "MakeContentsNonDefaultOptions" -> {
   AutomaticCellTagging -> FromContent, PageNumbers -> False, SaveOptions -> 
    True, SelectCellStyles -> {"Section", "Subsection"}, 
    TOCCellStyles -> {"Contents", "Index"}}},
CellLabelAutoDelete->True,
Magnification->1.,
FrontEndVersion->"6.0 for Linux x86 (64-bit) (March 26, 2007)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "ClickSave"->{
  Cell[1192, 47, 799, 21, 36, NotebookDefault,
   CellTags->"ClickSave"]},
 "Functions"->{
  Cell[2538, 92, 52, 1, 65, "Section",
   CellTags->"Functions"]},
 "Abbreviation"->{
  Cell[3094, 117, 84, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"Abbreviation"],
  Cell[3203, 123, 63, 1, 36, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[3269, 126, 329, 6, 61, "Text",
   CellTags->"Abbreviation"],
  Cell[3601, 134, 1032, 43, 43, "Text",
   CellTags->"Abbreviation"],
  Cell[4670, 182, 60, 1, 36, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[4755, 187, 264, 8, 28, "Input",
   CellTags->"Abbreviation"],
  Cell[5022, 197, 228, 8, 27, "Output",
   CellTags->"Abbreviation"]},
 "Amplitude"->{
  Cell[5277, 209, 55, 1, 41, "ObjectName",
   CellTags->"Amplitude"],
  Cell[5357, 214, 60, 1, 36, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[5420, 217, 529, 9, 79, "Text",
   CellTags->"Amplitude"],
  Cell[5974, 230, 115, 3, 28, "Input",
   CellTags->"Amplitude"],
  Cell[6092, 235, 328, 10, 27, "Output",
   CellTags->"Amplitude"],
  Cell[6435, 248, 177, 9, 25, "Text",
   CellTags->"Amplitude"],
  Cell[6649, 262, 57, 1, 36, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[6731, 267, 133, 4, 28, "Input",
   CellTags->"Amplitude"],
  Cell[6867, 273, 104, 3, 27, "Output",
   CellTags->"Amplitude"],
  Cell[6986, 279, 93, 1, 25, "Text",
   CellTags->"Amplitude"],
  Cell[7104, 284, 121, 3, 28, "Input",
   CellTags->"Amplitude"],
  Cell[7228, 289, 2216, 95, 29, "Output",
   CellTags->"Amplitude"],
  Cell[9481, 389, 108, 3, 28, "Input",
   CellTags->"Amplitude"],
  Cell[9592, 394, 5105, 187, 49, "Output",
   CellTags->"Amplitude"],
  Cell[14712, 584, 211, 8, 25, "Text",
   CellTags->"Amplitude"],
  Cell[14948, 596, 306, 9, 28, "Input",
   CellTags->"Amplitude"],
  Cell[15257, 607, 667, 14, 40, "Message",
   CellTags->"Amplitude"],
  Cell[15927, 623, 13529, 441, 135, "Output",
   CellTags->"Amplitude"]},
 "Amputate"->{
  Cell[29483, 1068, 53, 1, 41, "ObjectName",
   CellTags->"Amputate"],
  Cell[29561, 1073, 59, 1, 36, "Subsubsection",
   CellTags->"Amputate"],
  Cell[29623, 1076, 215, 5, 43, "Text",
   CellTags->"Amputate"],
  Cell[29863, 1085, 114, 3, 28, "Input",
   CellTags->"Amputate"],
  Cell[29980, 1090, 296, 10, 27, "Output",
   CellTags->"Amputate"],
  Cell[30291, 1103, 656, 28, 25, "Text",
   CellTags->"Amputate"],
  Cell[30984, 1136, 56, 1, 36, "Subsubsection",
   CellTags->"Amputate"],
  Cell[31065, 1141, 170, 5, 28, "Input",
   CellTags->"Amputate"],
  Cell[31238, 1148, 428, 16, 27, "Output",
   CellTags->"Amputate"],
  Cell[31703, 1169, 131, 4, 28, "Input",
   CellTags->"Amputate"],
  Cell[31837, 1175, 721, 27, 27, "Output",
   CellTags->"Amputate"]},
 "AnomalousDimension"->{
  Cell[32585, 1206, 73, 1, 41, "ObjectName",
   CellTags->"AnomalousDimension"],
  Cell[32683, 1211, 69, 1, 36, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[32755, 1214, 143, 4, 25, "Text",
   CellTags->"AnomalousDimension"],
  Cell[32923, 1222, 134, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[33060, 1227, 254, 8, 27, "Output",
   CellTags->"AnomalousDimension"],
  Cell[33329, 1238, 406, 18, 25, "Text",
   CellTags->"AnomalousDimension"],
  Cell[33772, 1261, 66, 1, 36, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[33841, 1264, 63, 1, 25, "Text",
   CellTags->"AnomalousDimension"],
  Cell[33929, 1269, 206, 5, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[34138, 1276, 254, 8, 27, "Output",
   CellTags->"AnomalousDimension"],
  Cell[34407, 1287, 241, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[34673, 1300, 133, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[34809, 1305, 462, 17, 40, "Output",
   CellTags->"AnomalousDimension"],
  Cell[35286, 1325, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35551, 1338, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[35686, 1343, 284, 10, 40, "Output",
   CellTags->"AnomalousDimension"],
  Cell[35985, 1356, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[36250, 1369, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[36385, 1374, 283, 10, 40, "Output",
   CellTags->"AnomalousDimension"],
  Cell[36683, 1387, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[36948, 1400, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[37083, 1405, 594, 21, 40, "Output",
   CellTags->"AnomalousDimension"],
  Cell[37692, 1429, 241, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[37958, 1442, 133, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[38094, 1447, 689, 23, 47, "Output",
   CellTags->"AnomalousDimension"],
  Cell[38798, 1473, 241, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[39064, 1486, 133, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[39200, 1491, 6351, 214, 233, "Output",
   CellTags->"AnomalousDimension"],
  Cell[45566, 1708, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[45831, 1721, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[45966, 1726, 4090, 139, 141, "Output",
   CellTags->"AnomalousDimension"],
  Cell[50071, 1868, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[50336, 1881, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[50471, 1886, 6253, 212, 226, "Output",
   CellTags->"AnomalousDimension"],
  Cell[56739, 2101, 240, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[57004, 2114, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[57139, 2119, 4933, 167, 186, "Output",
   CellTags->"AnomalousDimension"],
  Cell[62087, 2289, 267, 9, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[62379, 2302, 132, 3, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[62514, 2307, 4422, 140, 151, "Output",
   CellTags->"AnomalousDimension"],
  Cell[66951, 2450, 326, 10, 26, "Text",
   CellTags->"AnomalousDimension"],
  Cell[67302, 2464, 295, 9, 28, "Input",
   CellTags->"AnomalousDimension"],
  Cell[67600, 2475, 239, 7, 27, "Output",
   CellTags->"AnomalousDimension"]},
 "AntiCommutator"->{
  Cell[67866, 2486, 65, 1, 41, "ObjectName",
   CellTags->"AntiCommutator"],
  Cell[67956, 2491, 65, 1, 36, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[68024, 2494, 145, 4, 25, "Text",
   CellTags->"AntiCommutator"],
  Cell[68172, 2500, 599, 24, 25, "Text",
   CellTags->"AntiCommutator"],
  Cell[68808, 2529, 62, 1, 36, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[68873, 2532, 200, 9, 25, "Text",
   CellTags->"AntiCommutator"],
  Cell[69076, 2543, 150, 4, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[69251, 2551, 143, 4, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[69397, 2557, 163, 5, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[69597, 2567, 124, 3, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[69724, 2572, 180, 6, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[69941, 2583, 265, 8, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[70209, 2593, 438, 16, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[70684, 2614, 258, 8, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[70945, 2624, 298, 10, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[71258, 2637, 218, 6, 33, "Input",
   CellTags->"AntiCommutator"],
  Cell[71479, 2645, 104, 1, 25, "Text",
   CellTags->"AntiCommutator"],
  Cell[71608, 2650, 177, 5, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[71788, 2657, 109, 3, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[71934, 2665, 182, 5, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[72119, 2672, 190, 6, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[72346, 2683, 250, 8, 33, "Input",
   CellTags->"AntiCommutator"],
  Cell[72599, 2693, 132, 4, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[72768, 2702, 252, 7, 33, "Input",
   CellTags->"AntiCommutator"],
  Cell[73023, 2711, 315, 11, 30, "Output",
   CellTags->"AntiCommutator"],
  Cell[73353, 2725, 240, 6, 33, "Input",
   CellTags->"AntiCommutator"],
  Cell[73596, 2733, 176, 5, 28, "Input",
   CellTags->"AntiCommutator"],
  Cell[73775, 2740, 226, 7, 33, "Input",
   CellTags->"AntiCommutator"]},
 "AntiQuarkField"->{
  Cell[74016, 2750, 65, 1, 41, "ObjectName",
   CellTags->"AntiQuarkField"],
  Cell[74106, 2755, 65, 1, 36, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[74174, 2758, 197, 4, 25, "Text",
   CellTags->"AntiQuarkField"],
  Cell[74374, 2764, 313, 14, 25, "Text",
   CellTags->"AntiQuarkField"],
  Cell[74724, 2783, 62, 1, 36, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[74811, 2788, 93, 2, 28, "Input",
   CellTags->"AntiQuarkField"],
  Cell[74907, 2792, 137, 4, 28, "Output",
   CellTags->"AntiQuarkField"]},
 "AntiSymmetrize"->{
  Cell[75071, 2800, 65, 1, 41, "ObjectName",
   CellTags->"AntiSymmetrize"],
  Cell[75161, 2805, 65, 1, 36, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[75229, 2808, 153, 4, 25, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[75385, 2814, 185, 8, 25, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[75607, 2827, 62, 1, 36, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[75694, 2832, 246, 8, 28, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[75943, 2842, 319, 11, 38, "Output",
   CellTags->"AntiSymmetrize"],
  Cell[76299, 2858, 266, 8, 28, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[76568, 2868, 654, 20, 38, "Output",
   CellTags->"AntiSymmetrize"]},
 "Anti5"->{
  Cell[77249, 2892, 47, 1, 41, "ObjectName",
   CellTags->"Anti5"],
  Cell[77321, 2897, 56, 1, 36, "Subsubsection",
   CellTags->"Anti5"],
  Cell[77380, 2900, 487, 16, 25, "Text",
   CellTags->"Anti5"],
  Cell[77870, 2918, 163, 7, 25, "Text",
   CellTags->"Anti5"],
  Cell[78036, 2927, 423, 19, 25, "Text",
   CellTags->"Anti5"],
  Cell[78496, 2951, 53, 1, 36, "Subsubsection",
   CellTags->"Anti5"],
  Cell[78574, 2956, 154, 5, 28, "Input",
   CellTags->"Anti5"],
  Cell[78731, 2963, 186, 6, 27, "Output",
   CellTags->"Anti5"],
  Cell[78954, 2974, 102, 3, 28, "Input",
   CellTags->"Anti5"],
  Cell[79059, 2979, 290, 11, 27, "Output",
   CellTags->"Anti5"],
  Cell[79386, 2995, 144, 5, 28, "Input",
   CellTags->"Anti5"],
  Cell[79533, 3002, 265, 10, 27, "Output",
   CellTags->"Anti5"],
  Cell[79835, 3017, 194, 6, 28, "Input",
   CellTags->"Anti5"],
  Cell[80032, 3025, 320, 9, 28, "Output",
   CellTags->"Anti5"],
  Cell[80389, 3039, 125, 4, 28, "Input",
   CellTags->"Anti5"],
  Cell[80517, 3045, 320, 9, 28, "Output",
   CellTags->"Anti5"],
  Cell[80874, 3059, 133, 4, 28, "Input",
   CellTags->"Anti5"],
  Cell[81010, 3065, 320, 9, 28, "Output",
   CellTags->"Anti5"],
  Cell[81367, 3079, 151, 5, 28, "Input",
   CellTags->"Anti5"],
  Cell[81521, 3086, 320, 9, 28, "Output",
   CellTags->"Anti5"],
  Cell[81856, 3098, 351, 14, 25, "Text",
   CellTags->"Anti5"],
  Cell[82232, 3116, 127, 4, 28, "Input",
   CellTags->"Anti5"],
  Cell[82362, 3122, 191, 6, 27, "Output",
   CellTags->"Anti5"],
  Cell[82590, 3133, 102, 3, 28, "Input",
   CellTags->"Anti5"],
  Cell[82695, 3138, 290, 11, 27, "Output",
   CellTags->"Anti5"]},
 "Apart2"->{
  Cell[83012, 3153, 49, 1, 41, "ObjectName",
   CellTags->"Apart2"],
  Cell[83086, 3158, 57, 1, 36, "Subsubsection",
   CellTags->"Apart2"],
  Cell[83146, 3161, 108, 3, 25, "Text",
   CellTags->"Apart2"],
  Cell[83257, 3166, 302, 14, 25, "Text",
   CellTags->"Apart2"],
  Cell[83596, 3185, 54, 1, 36, "Subsubsection",
   CellTags->"Apart2"],
  Cell[83675, 3190, 256, 9, 28, "Input",
   CellTags->"Apart2"],
  Cell[83934, 3201, 1188, 30, 48, "Output",
   CellTags->"Apart2"],
  Cell[85159, 3236, 104, 3, 28, "Input",
   CellTags->"Apart2"],
  Cell[85266, 3241, 720, 26, 48, "Output",
   CellTags->"Apart2"],
  Cell[86023, 3272, 140, 4, 28, "Input",
   CellTags->"Apart2"],
  Cell[86166, 3278, 517, 19, 54, "Output",
   CellTags->"Apart2"]},
 "A0"->{
  Cell[86710, 3301, 64, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0"],
  Cell[86799, 3307, 53, 1, 36, "Subsubsection",
   CellTags->"A0"],
  Cell[86855, 3310, 291, 11, 25, "Text",
   CellTags->"A0"],
  Cell[87171, 3325, 107, 3, 28, "Input",
   CellTags->"A0"],
  Cell[87281, 3330, 161, 6, 27, "Output",
   CellTags->"A0"],
  Cell[87457, 3339, 437, 24, 25, "Text",
   CellTags->"A0"],
  Cell[87931, 3368, 50, 1, 36, "Subsubsection",
   CellTags->"A0"],
  Cell[87984, 3371, 246, 11, 25, "Text",
   CellTags->"A0"],
  Cell[88255, 3386, 120, 4, 28, "Input",
   CellTags->"A0"],
  Cell[88378, 3392, 259, 10, 27, "Output",
   CellTags->"A0"],
  Cell[88652, 3405, 186, 6, 28, "Input",
   CellTags->"A0"],
  Cell[88863, 3415, 120, 4, 28, "Input",
   CellTags->"A0"],
  Cell[88986, 3421, 324, 11, 27, "Output",
   CellTags->"A0"],
  Cell[89325, 3435, 187, 6, 28, "Input",
   CellTags->"A0"],
  Cell[89515, 3443, 222, 8, 25, "Text",
   CellTags->"A0"],
  Cell[89762, 3455, 96, 3, 28, "Input",
   CellTags->"A0"],
  Cell[89861, 3460, 97, 3, 27, "Output",
   CellTags->"A0"],
  Cell[89995, 3468, 160, 5, 28, "Input",
   CellTags->"A0"],
  Cell[90158, 3475, 97, 3, 27, "Output",
   CellTags->"A0"]},
 "A0ToB0"->{
  Cell[90282, 3482, 72, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0ToB0"],
  Cell[90379, 3488, 57, 1, 36, "Subsubsection",
   CellTags->"A0ToB0"],
  Cell[90439, 3491, 614, 27, 25, "Text",
   CellTags->"A0ToB0"],
  Cell[91078, 3522, 111, 3, 28, "Input",
   CellTags->"A0ToB0"],
  Cell[91192, 3527, 165, 6, 27, "Output",
   CellTags->"A0ToB0"],
  Cell[91372, 3536, 533, 29, 25, "Text",
   CellTags->"A0ToB0"]},
 "BackgroundGluonVertex"->{
  Cell[91920, 3568, 79, 1, 41, "ObjectName",
   CellTags->"BackgroundGluonVertex"],
  Cell[92024, 3573, 72, 1, 36, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[92099, 3576, 1042, 19, 151, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[93166, 3599, 140, 3, 28, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[93309, 3604, 318, 9, 27, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[93642, 3616, 201, 9, 25, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[93880, 3630, 69, 1, 36, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[93974, 3635, 360, 10, 28, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[94337, 3647, 1460, 57, 31, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[95834, 3709, 436, 12, 28, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[96273, 3723, 3843, 141, 29, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[100153, 3869, 411, 11, 28, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[100567, 3882, 1532, 59, 51, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[102136, 3946, 487, 13, 28, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[102626, 3961, 3921, 143, 46, "Output",
   CellTags->"BackgroundGluonVertex"]},
 "Bracket"->{
  Cell[106574, 4108, 51, 1, 41, "ObjectName",
   CellTags->"Bracket"],
  Cell[106650, 4113, 58, 1, 36, "Subsubsection",
   CellTags->"Bracket"],
  Cell[106711, 4116, 72, 1, 25, "Text",
   CellTags->"Bracket"],
  Cell[106786, 4119, 181, 9, 25, "Text",
   CellTags->"Bracket"]},
 "BReduce"->{
  Cell[106982, 4131, 74, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"BReduce"],
  Cell[107081, 4137, 58, 1, 36, "Subsubsection",
   CellTags->"BReduce"],
  Cell[107142, 4140, 148, 4, 25, "Text",
   CellTags->"BReduce"],
  Cell[107293, 4146, 534, 29, 25, "Text",
   CellTags->"BReduce"],
  Cell[107864, 4180, 55, 1, 36, "Subsubsection",
   CellTags->"BReduce"],
  Cell[107922, 4183, 251, 11, 25, "Text",
   CellTags->"BReduce"],
  Cell[108198, 4198, 134, 4, 28, "Input",
   CellTags->"BReduce"],
  Cell[108335, 4204, 176, 6, 27, "Output",
   CellTags->"BReduce"],
  Cell[108910, 4230, 94, 3, 25, "Text",
   CellTags->"BReduce"],
  Cell[109029, 4237, 183, 5, 28, "Input",
   CellTags->"BReduce"],
  Cell[109215, 4244, 260, 10, 40, "Output",
   CellTags->"BReduce"]},
 "B0"->{
  Cell[109502, 4258, 41, 1, 41, "ObjectName",
   CellTags->"B0"],
  Cell[109568, 4263, 53, 1, 36, "Subsubsection",
   CellTags->"B0"],
  Cell[109624, 4266, 390, 9, 61, "Text",
   CellTags->"B0"],
  Cell[110039, 4279, 102, 3, 28, "Input",
   CellTags->"B0"],
  Cell[110144, 4284, 280, 9, 27, "Output",
   CellTags->"B0"],
  Cell[110439, 4296, 443, 24, 25, "Text",
   CellTags->"B0"],
  Cell[110919, 4325, 50, 1, 36, "Subsubsection",
   CellTags->"B0"],
  Cell[110994, 4330, 234, 8, 28, "Input",
   CellTags->"B0"],
  Cell[111231, 4340, 316, 12, 27, "Output",
   CellTags->"B0"],
  Cell[111562, 4355, 237, 7, 28, "Input",
   CellTags->"B0"],
  Cell[111824, 4366, 154, 5, 28, "Input",
   CellTags->"B0"],
  Cell[111981, 4373, 245, 8, 27, "Output",
   CellTags->"B0"],
  Cell[112263, 4386, 179, 6, 28, "Input",
   CellTags->"B0"],
  Cell[112445, 4394, 245, 8, 27, "Output",
   CellTags->"B0"],
  Cell[112727, 4407, 177, 6, 28, "Input",
   CellTags->"B0"],
  Cell[112907, 4415, 218, 7, 27, "Output",
   CellTags->"B0"]},
 "B0Real"->{
  Cell[113152, 4426, 72, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Real"],
  Cell[113249, 4432, 57, 1, 36, "Subsubsection",
   CellTags->"B0Real"],
  Cell[113309, 4435, 186, 4, 25, "Text",
   CellTags->"B0Real"],
  Cell[113498, 4441, 153, 8, 25, "Text",
   CellTags->"B0Real"],
  Cell[113688, 4454, 54, 1, 36, "Subsubsection",
   CellTags->"B0Real"],
  Cell[113745, 4457, 83, 1, 25, "Text",
   CellTags->"B0Real"],
  Cell[113853, 4462, 133, 4, 28, "Input",
   CellTags->"B0Real"],
  Cell[113989, 4468, 200, 7, 27, "Output",
   CellTags->"B0Real"],
  Cell[114226, 4480, 102, 3, 28, "Input",
   CellTags->"B0Real"],
  Cell[114331, 4485, 282, 9, 27, "Output",
   CellTags->"B0Real"],
  Cell[114628, 4497, 92, 3, 25, "Text",
   CellTags->"B0Real"],
  Cell[114745, 4504, 181, 5, 28, "Input",
   CellTags->"B0Real"],
  Cell[114929, 4511, 200, 7, 27, "Output",
   CellTags->"B0Real"]},
 "B0Unique"->{
  Cell[115156, 4522, 76, 2, 41, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Unique"],
  Cell[115257, 4528, 59, 1, 36, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[115319, 4531, 184, 4, 25, "Text",
   CellTags->"B0Unique"],
  Cell[115506, 4537, 155, 8, 25, "Text",
   CellTags->"B0Unique"],
  Cell[115698, 4550, 56, 1, 36, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[115757, 4553, 73, 1, 25, "Text",
   CellTags->"B0Unique"],
  Cell[115855, 4558, 135, 4, 28, "Input",
   CellTags->"B0Unique"],
  Cell[115993, 4564, 202, 7, 27, "Output",
   CellTags->"B0Unique"],
  Cell[116232, 4576, 104, 3, 28, "Input",
   CellTags->"B0Unique"],
  Cell[116339, 4581, 284, 9, 27, "Output",
   CellTags->"B0Unique"],
  Cell[116638, 4593, 88, 3, 25, "Text",
   CellTags->"B0Unique"],
  Cell[116751, 4600, 186, 5, 28, "Input",
   CellTags->"B0Unique"],
  Cell[116940, 4607, 177, 6, 27, "Output",
   CellTags->"B0Unique"]},
 "B00"->{
  Cell[117144, 4617, 43, 1, 41, "ObjectName",
   CellTags->"B00"],
  Cell[117212, 4622, 54, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[117269, 4625, 290, 8, 43, "Text",
   CellTags->"B00"],
  Cell[117584, 4637, 104, 3, 28, "Input",
   CellTags->"B00"],
  Cell[117691, 4642, 162, 6, 27, "Output",
   CellTags->"B00"],
  Cell[117868, 4651, 346, 19, 25, "Text",
   CellTags->"B00"],
  Cell[118251, 4675, 51, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[118305, 4678, 202, 7, 25, "Text",
   CellTags->"B00"],
  Cell[118532, 4689, 97, 3, 28, "Input",
   CellTags->"B00"],
  Cell[118632, 4694, 156, 6, 27, "Output",
   CellTags->"B00"],
  Cell[118825, 4705, 211, 7, 28, "Input",
   CellTags->"B00"],
  Cell[119039, 4714, 2678, 91, 79, "Output",
   CellTags->"B00"],
  Cell[121754, 4810, 211, 7, 28, "Input",
   CellTags->"B00"],
  Cell[121968, 4819, 1452, 54, 40, "Output",
   CellTags->"B00"],
  Cell[123457, 4878, 261, 8, 28, "Input",
   CellTags->"B00"],
  Cell[123721, 4888, 516, 22, 27, "Output",
   CellTags->"B00"],
  Cell[124274, 4915, 181, 6, 28, "Input",
   CellTags->"B00"],
  Cell[124458, 4923, 614, 22, 40, "Output",
   CellTags->"B00"],
  Cell[125109, 4950, 248, 8, 28, "Input",
   CellTags->"B00"],
  Cell[125360, 4960, 641, 23, 40, "Output",
   CellTags->"B00"]},
 "B1"->{
  Cell[126028, 4987, 41, 1, 41, "ObjectName",
   CellTags->"B1"],
  Cell[126094, 4992, 53, 1, 36, "Subsubsection",
   CellTags->"B1"],
  Cell[126150, 4995, 230, 7, 25, "Text",
   CellTags->"B1"],
  Cell[126405, 5006, 102, 3, 28, "Input",
   CellTags->"B1"],
  Cell[126510, 5011, 161, 6, 27, "Output",
   CellTags->"B1"],
  Cell[126686, 5020, 559, 29, 25, "Text",
   CellTags->"B1"],
  Cell[127282, 5054, 50, 1, 36, "Subsubsection",
   CellTags->"B1"],
  Cell[127357, 5059, 209, 7, 28, "Input",
   CellTags->"B1"],
  Cell[127569, 5068, 1153, 42, 44, "Output",
   CellTags->"B1"],
  Cell[128759, 5115, 259, 8, 28, "Input",
   CellTags->"B1"],
  Cell[129021, 5125, 316, 12, 27, "Output",
   CellTags->"B1"],
  Cell[129352, 5140, 187, 6, 28, "Input",
   CellTags->"B1"],
  Cell[129564, 5150, 209, 7, 28, "Input",
   CellTags->"B1"],
  Cell[129776, 5159, 390, 15, 38, "Output",
   CellTags->"B1"],
  Cell[130203, 5179, 179, 6, 28, "Input",
   CellTags->"B1"],
  Cell[130385, 5187, 403, 14, 38, "Output",
   CellTags->"B1"],
  Cell[130825, 5206, 154, 5, 28, "Input",
   CellTags->"B1"],
  Cell[130982, 5213, 403, 14, 38, "Output",
   CellTags->"B1"],
  Cell[131422, 5232, 238, 7, 28, "Input",
   CellTags->"B1"],
  Cell[131663, 5241, 791, 25, 43, "Output",
   CellTags->"B1"],
  Cell[132491, 5271, 279, 8, 28, "Input",
   CellTags->"B1"],
  Cell[132773, 5281, 419, 14, 38, "Output",
   CellTags->"B1"]},
 "B11"->{
  Cell[133219, 5299, 43, 1, 57, "ObjectName",
   CellTags->"B11"],
  Cell[133287, 5304, 54, 1, 52, "Subsubsection",
   CellTags->"B11"],
  Cell[133344, 5307, 495, 17, 43, "Text",
   CellTags->"B11"],
  Cell[133864, 5328, 104, 3, 28, "Input",
   CellTags->"B11"],
  Cell[133971, 5333, 162, 6, 27, "Output",
   CellTags->"B11"],
  Cell[134148, 5342, 441, 24, 41, "Text",
   CellTags->"B11"],
  Cell[134626, 5371, 51, 1, 52, "Subsubsection",
   CellTags->"B11"],
  Cell[134680, 5374, 215, 7, 25, "Text",
   CellTags->"B11"],
  Cell[134920, 5385, 97, 3, 28, "Input",
   CellTags->"B11"],
  Cell[135020, 5390, 156, 6, 27, "Output",
   CellTags->"B11"],
  Cell[135213, 5401, 212, 7, 28, "Input",
   CellTags->"B11"],
  Cell[135428, 5410, 2677, 94, 52, "Output",
   CellTags->"B11"],
  Cell[138142, 5509, 212, 7, 28, "Input",
   CellTags->"B11"],
  Cell[138357, 5518, 1206, 47, 46, "Output",
   CellTags->"B11"],
  Cell[139600, 5570, 171, 5, 28, "Input",
   CellTags->"B11"],
  Cell[139774, 5577, 164, 6, 27, "Output",
   CellTags->"B11"],
  Cell[139975, 5588, 212, 7, 28, "Input",
   CellTags->"B11"],
  Cell[140190, 5597, 517, 22, 27, "Output",
   CellTags->"B11"],
  Cell[140744, 5624, 170, 5, 28, "Input",
   CellTags->"B11"],
  Cell[140917, 5631, 163, 6, 27, "Output",
   CellTags->"B11"],
  Cell[141117, 5642, 182, 6, 28, "Input",
   CellTags->"B11"],
  Cell[141302, 5650, 269, 9, 38, "Output",
   CellTags->"B11"],
  Cell[141608, 5664, 249, 8, 28, "Input",
   CellTags->"B11"],
  Cell[141860, 5674, 295, 10, 38, "Output",
   CellTags->"B11"]},
 "CA"->{
  Cell[142182, 5688, 41, 1, 57, "ObjectName",
   CellTags->"CA"],
  Cell[142248, 5693, 53, 1, 52, "Subsubsection",
   CellTags->"CA"],
  Cell[142304, 5696, 198, 9, 41, "Text",
   CellTags->"CA"],
  Cell[142505, 5707, 274, 14, 41, "Text",
   CellTags->"CA"],
  Cell[142816, 5726, 50, 1, 36, "Subsubsection",
   CellTags->"CA"],
  Cell[142891, 5731, 70, 2, 28, "Input",
   CellTags->"CA"],
  Cell[142964, 5735, 120, 4, 27, "Output",
   CellTags->"CA"],
  Cell[143121, 5744, 173, 5, 28, "Input",
   CellTags->"CA"],
  Cell[143297, 5751, 98, 3, 27, "Output",
   CellTags->"CA"],
  Cell[143432, 5759, 72, 2, 28, "Input",
   CellTags->"CA"],
  Cell[143507, 5763, 98, 3, 27, "Output",
   CellTags->"CA"]},
 "Calc"->{
  Cell[143632, 5770, 45, 1, 41, "ObjectName",
   CellTags->"Calc"],
  Cell[143702, 5775, 55, 1, 52, "Subsubsection",
   CellTags->"Calc"],
  Cell[143760, 5778, 233, 5, 43, "Text",
   CellTags->"Calc"],
  Cell[143996, 5785, 407, 19, 25, "Text",
   CellTags->"Calc"],
  Cell[144440, 5809, 61, 1, 35, "Subsubsection",
   CellTags->"Calc"],
  Cell[144504, 5812, 346, 13, 25, "Text",
   CellTags->"Calc"],
  Cell[144875, 5829, 284, 9, 28, "Input",
   CellTags->"Calc"],
  Cell[145162, 5840, 147, 5, 27, "Output",
   CellTags->"Calc"],
  Cell[145324, 5848, 213, 9, 25, "Text",
   CellTags->"Calc"],
  Cell[145562, 5861, 258, 8, 28, "Input",
   CellTags->"Calc"],
  Cell[145823, 5871, 352, 15, 27, "Output",
   CellTags->"Calc"],
  Cell[146212, 5891, 328, 11, 28, "Input",
   CellTags->"Calc"],
  Cell[146543, 5904, 424, 16, 27, "Output",
   CellTags->"Calc"],
  Cell[147004, 5925, 101, 3, 28, "Input",
   CellTags->"Calc"],
  Cell[147108, 5930, 547, 23, 27, "Output",
   CellTags->"Calc"],
  Cell[147692, 5958, 215, 6, 28, "Input",
   CellTags->"Calc"],
  Cell[147910, 5966, 985, 44, 27, "Output",
   CellTags->"Calc"],
  Cell[148932, 6015, 382, 13, 28, "Input",
   CellTags->"Calc"],
  Cell[149317, 6030, 100, 3, 27, "Output",
   CellTags->"Calc"]},
 "CalcColorFactor"->{
  Cell[149444, 6037, 67, 1, 41, "ObjectName",
   CellTags->"CalcColorFactor"],
  Cell[149536, 6042, 66, 1, 36, "Subsubsection",
   CellTags->"CalcColorFactor"],
  Cell[149605, 6045, 470, 9, 113, "Text",
   CellTags->"CalcColorFactor"],
  Cell[150078, 6056, 195, 9, 25, "Text",
   CellTags->"CalcColorFactor"]},
 "Collecting"->{
  Cell[155138, 6246, 411, 18, 25, "Text",
   CellTags->"Collecting"],
  Cell[255324, 10232, 57, 1, 57, "ObjectName",
   CellTags->"Collecting"],
  Cell[255406, 10237, 61, 1, 52, "Subsubsection",
   CellTags->"Collecting"],
  Cell[255470, 10240, 207, 5, 41, "Text",
   CellTags->"Collecting"],
  Cell[255680, 10247, 411, 18, 41, "Text",
   CellTags->"Collecting"]},
 "CalculateCounterTerm"->{
  Cell[155564, 6267, 94, 1, 41, "ObjectName",
   CellTags->"CalculateCounterTerm"],
  Cell[155683, 6272, 71, 1, 36, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[155757, 6275, 110, 1, 25, "Text",
   CellTags->"CalculateCounterTerm"],
  Cell[155904, 6281, 77, 1, 35, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[156006, 6286, 123, 3, 28, "Input",
   CellTags->"CalculateCounterTerm"],
  Cell[156132, 6291, 191, 3, 40, "Print",
   CellTags->{"CalculateCounterTerm", "Info3387711865-3148101"}]},
 "Info3387711865-3148101"->{
  Cell[156132, 6291, 191, 3, 40, "Print",
   CellTags->{"CalculateCounterTerm", "Info3387711865-3148101"}]},
 "CancelQP"->{
  Cell[156350, 6298, 53, 1, 41, "ObjectName",
   CellTags->"CancelQP"],
  Cell[156428, 6303, 59, 1, 36, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[156490, 6306, 314, 13, 25, "Text",
   CellTags->"CancelQP"],
  Cell[156841, 6324, 65, 1, 35, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[156931, 6329, 549, 19, 28, "Input",
   CellTags->"CancelQP"],
  Cell[157483, 6350, 667, 24, 50, "Output",
   CellTags->"CancelQP"],
  Cell[158187, 6379, 182, 5, 28, "Input",
   CellTags->"CancelQP"],
  Cell[158372, 6386, 2183, 73, 43, "Output",
   CellTags->"CancelQP"],
  Cell[160592, 6464, 183, 5, 28, "Input",
   CellTags->"CancelQP"],
  Cell[160778, 6471, 6388, 208, 166, "Output",
   CellTags->"CancelQP"],
  Cell[167203, 6684, 555, 18, 28, "Input",
   CellTags->"CancelQP"],
  Cell[167761, 6704, 886, 33, 45, "Output",
   CellTags->"CancelQP"],
  Cell[168684, 6742, 182, 5, 28, "Input",
   CellTags->"CancelQP"],
  Cell[168869, 6749, 6250, 218, 88, "Output",
   CellTags->"CancelQP"],
  Cell[175156, 6972, 183, 5, 28, "Input",
   CellTags->"CancelQP"],
  Cell[175342, 6979, 7177, 246, 88, "Output",
   CellTags->"CancelQP"],
  Cell[182534, 7228, 106, 3, 28, "Input",
   CellTags->"CancelQP"]},
 "Cases2"->{
  Cell[182655, 7234, 49, 1, 41, "ObjectName",
   CellTags->"Cases2"],
  Cell[182729, 7239, 57, 1, 52, "Subsubsection",
   CellTags->"Cases2"],
  Cell[182789, 7242, 293, 6, 59, "Text",
   CellTags->"Cases2"],
  Cell[183119, 7253, 63, 1, 35, "Subsubsection",
   CellTags->"Cases2"],
  Cell[183207, 7258, 305, 10, 28, "Input",
   CellTags->"Cases2"],
  Cell[183515, 7270, 275, 9, 27, "Output",
   CellTags->"Cases2"],
  Cell[183827, 7284, 312, 10, 28, "Input",
   CellTags->"Cases2"],
  Cell[184142, 7296, 279, 9, 27, "Output",
   CellTags->"Cases2"],
  Cell[184458, 7310, 427, 14, 28, "Input",
   CellTags->"Cases2"],
  Cell[184888, 7326, 355, 11, 27, "Output",
   CellTags->"Cases2"],
  Cell[185280, 7342, 327, 10, 28, "Input",
   CellTags->"Cases2"],
  Cell[185610, 7354, 463, 17, 27, "Output",
   CellTags->"Cases2"]},
 "CF"->{
  Cell[186100, 7375, 41, 1, 57, "ObjectName",
   CellTags->"CF"],
  Cell[186166, 7380, 53, 1, 52, "Subsubsection",
   CellTags->"CF"],
  Cell[186222, 7383, 287, 13, 41, "Text",
   CellTags->"CF"],
  Cell[186512, 7398, 274, 14, 41, "Text",
   CellTags->"CF"],
  Cell[186823, 7417, 50, 1, 36, "Subsubsection",
   CellTags->"CF"],
  Cell[186898, 7422, 70, 2, 28, "Input",
   CellTags->"CF"],
  Cell[186971, 7426, 120, 4, 27, "Output",
   CellTags->"CF"],
  Cell[187128, 7435, 178, 5, 28, "Input",
   CellTags->"CF"],
  Cell[187309, 7442, 193, 7, 40, "Output",
   CellTags->"CF"],
  Cell[187738, 7464, 72, 2, 28, "Input",
   CellTags->"CF"],
  Cell[187813, 7468, 98, 3, 27, "Output",
   CellTags->"CF"],
  Cell[187948, 7476, 233, 7, 28, "Input",
   CellTags->"CF"],
  Cell[188184, 7485, 177, 6, 27, "Output",
   CellTags->"CF"]},
 "ChangeDimension"->{
  Cell[188388, 7495, 67, 1, 57, "ObjectName",
   CellTags->"ChangeDimension"],
  Cell[188480, 7500, 66, 1, 52, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[188549, 7503, 210, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[188762, 7510, 519, 24, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[189318, 7539, 63, 1, 52, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[189384, 7542, 219, 5, 43, "Text",
   CellTags->"ChangeDimension"],
  Cell[189628, 7551, 212, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[189843, 7559, 198, 7, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[190078, 7571, 151, 4, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[190232, 7577, 198, 7, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[190467, 7589, 116, 3, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[190586, 7594, 272, 8, 42, "Output",
   CellTags->"ChangeDimension"],
  Cell[190873, 7605, 119, 3, 25, "Text",
   CellTags->"ChangeDimension"],
  Cell[191017, 7612, 201, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[191221, 7620, 222, 6, 42, "Output",
   CellTags->"ChangeDimension"],
  Cell[191458, 7629, 105, 1, 25, "Text",
   CellTags->"ChangeDimension"],
  Cell[191588, 7634, 482, 15, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[192073, 7651, 571, 18, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[192681, 7674, 153, 4, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[192837, 7680, 755, 29, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[193629, 7714, 222, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[193854, 7722, 755, 29, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[194624, 7754, 134, 4, 25, "Text",
   CellTags->"ChangeDimension"],
  Cell[194761, 7760, 208, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[194972, 7768, 219, 5, 43, "Text",
   CellTags->"ChangeDimension"],
  Cell[195194, 7775, 201, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[195420, 7785, 220, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[195643, 7793, 489, 21, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[196169, 7819, 182, 5, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[196354, 7826, 489, 21, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[196880, 7852, 176, 5, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[197059, 7859, 301, 10, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[197397, 7874, 141, 4, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[197541, 7880, 130, 4, 27, "Output",
   CellTags->"ChangeDimension"],
  Cell[197686, 7887, 201, 6, 28, "Input",
   CellTags->"ChangeDimension"],
  Cell[197890, 7895, 157, 5, 28, "Input",
   CellTags->"ChangeDimension"]},
 "ChargeConjugationMatrix"->{
  Cell[198062, 7903, 83, 1, 57, "ObjectName",
   CellTags->"ChargeConjugationMatrix"],
  Cell[198170, 7908, 74, 1, 52, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[198247, 7911, 176, 6, 41, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[198426, 7919, 248, 9, 41, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[198711, 7933, 71, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[198807, 7938, 214, 5, 28, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[199024, 7945, 306, 11, 27, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[199367, 7961, 120, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[199490, 7966, 255, 9, 27, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[199782, 7980, 209, 5, 28, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[199994, 7987, 228, 7, 27, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200259, 7999, 120, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200382, 8004, 192, 6, 27, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200611, 8015, 134, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200748, 8020, 137, 4, 27, "Output",
   CellTags->"ChargeConjugationMatrix"]},
 "ChargeConjugationMatrixInv"->{
  Cell[200912, 8028, 89, 1, 57, "ObjectName",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201026, 8033, 77, 1, 52, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201106, 8036, 195, 4, 41, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201304, 8042, 242, 9, 41, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201583, 8056, 74, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201682, 8061, 217, 5, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[201902, 8068, 309, 11, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[202248, 8084, 123, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[202374, 8089, 258, 9, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[202669, 8103, 212, 5, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[202884, 8110, 231, 7, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203152, 8122, 123, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203278, 8127, 195, 6, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203510, 8138, 165, 4, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203678, 8144, 189, 6, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203904, 8155, 119, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204026, 8160, 164, 5, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204227, 8170, 140, 3, 28, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204370, 8175, 140, 4, 27, "Output",
   CellTags->"ChargeConjugationMatrixInv"]},
 "CheckContext"->{
  Cell[204537, 8183, 61, 1, 57, "ObjectName",
   CellTags->"CheckContext"],
  Cell[204623, 8188, 63, 1, 52, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[204689, 8191, 164, 4, 41, "Text",
   CellTags->"CheckContext"],
  Cell[204856, 8197, 320, 14, 41, "Text",
   CellTags->"CheckContext"],
  Cell[205213, 8216, 60, 1, 52, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[205276, 8219, 142, 4, 25, "Text",
   CellTags->"CheckContext"],
  Cell[205443, 8227, 138, 3, 28, "Input",
   CellTags->"CheckContext"],
  Cell[205584, 8232, 112, 3, 27, "Output",
   CellTags->"CheckContext"],
  Cell[205711, 8238, 87, 1, 25, "Text",
   CellTags->"CheckContext"],
  Cell[205823, 8243, 92, 2, 28, "Input",
   CellTags->"CheckContext"],
  Cell[205918, 8247, 121, 3, 27, "Output",
   CellTags->"CheckContext"],
  Cell[206076, 8255, 138, 3, 28, "Input",
   CellTags->"CheckContext"],
  Cell[206217, 8260, 111, 3, 27, "Output",
   CellTags->"CheckContext"]},
 "CheckDB"->{
  Cell[206355, 8267, 51, 1, 57, "ObjectName",
   CellTags->"CheckDB"],
  Cell[206431, 8272, 106, 3, 52, "Subsubsection",
   CellTags->"CheckDB"],
  Cell[207646, 8296, 113, 3, 28, "Input",
   CellTags->"CheckDB"],
  Cell[207762, 8301, 471, 14, 27, "Output",
   CellTags->"CheckDB"],
  Cell[208270, 8320, 116, 3, 28, "Input",
   CellTags->"CheckDB"],
  Cell[208389, 8325, 134, 4, 27, "Output",
   CellTags->"CheckDB"],
  Cell[209681, 8380, 383, 13, 28, "Input",
   CellTags->"CheckDB"],
  Cell[210067, 8395, 150, 5, 27, "Output",
   CellTags->"CheckDB"],
  Cell[210317, 8407, 303, 9, 28, "Input",
   CellTags->"CheckDB"],
  Cell[210623, 8418, 119, 3, 49, "Output",
   CellTags->"CheckDB"],
  Cell[210757, 8424, 268, 8, 28, "Input",
   CellTags->"CheckDB"]},
 "ChiralityProjector"->{
  Cell[211040, 8435, 73, 1, 57, "ObjectName",
   CellTags->"ChiralityProjector"],
  Cell[211138, 8440, 69, 1, 52, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[211210, 8443, 499, 19, 41, "Text",
   CellTags->"ChiralityProjector"],
  Cell[211712, 8464, 409, 19, 41, "Text",
   CellTags->"ChiralityProjector"],
  Cell[212158, 8488, 66, 1, 36, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[212249, 8493, 262, 8, 28, "Input",
   CellTags->"ChiralityProjector"],
  Cell[212514, 8503, 227, 7, 27, "Output",
   CellTags->"ChiralityProjector"],
  Cell[212778, 8515, 114, 3, 28, "Input",
   CellTags->"ChiralityProjector"],
  Cell[212895, 8520, 227, 7, 27, "Output",
   CellTags->"ChiralityProjector"],
  Cell[213159, 8532, 196, 5, 28, "Input",
   CellTags->"ChiralityProjector"],
  Cell[213358, 8539, 380, 14, 42, "Output",
   CellTags->"ChiralityProjector"]},
 "Chisholm"->{
  Cell[213765, 8557, 53, 1, 57, "ObjectName",
   CellTags->"Chisholm"],
  Cell[213843, 8562, 59, 1, 52, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[213905, 8565, 142, 4, 41, "Text",
   CellTags->"Chisholm"],
  Cell[214050, 8571, 316, 14, 41, "Text",
   CellTags->"Chisholm"],
  Cell[214403, 8590, 56, 1, 36, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[214484, 8595, 206, 6, 28, "Input",
   CellTags->"Chisholm"],
  Cell[214693, 8603, 1988, 78, 28, "Output",
   CellTags->"Chisholm"],
  Cell[216718, 8686, 183, 5, 28, "Input",
   CellTags->"Chisholm"],
  Cell[216904, 8693, 236, 7, 27, "Output",
   CellTags->"Chisholm"],
  Cell[217177, 8705, 139, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[217319, 8711, 1372, 54, 27, "Output",
   CellTags->"Chisholm"],
  Cell[218706, 8768, 77, 1, 25, "Text",
   CellTags->"Chisholm"],
  Cell[218808, 8773, 130, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[218941, 8779, 105, 3, 27, "Output",
   CellTags->"Chisholm"],
  Cell[219083, 8787, 139, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[219225, 8793, 1372, 54, 27, "Output",
   CellTags->"Chisholm"],
  Cell[220634, 8852, 130, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[220767, 8858, 105, 3, 27, "Output",
   CellTags->"Chisholm"],
  Cell[220909, 8866, 169, 5, 28, "Input",
   CellTags->"Chisholm"],
  Cell[221081, 8873, 590, 22, 27, "Output",
   CellTags->"Chisholm"],
  Cell[221708, 8900, 110, 3, 28, "Input",
   CellTags->"Chisholm"],
  Cell[221821, 8905, 1457, 58, 27, "Output",
   CellTags->"Chisholm"],
  Cell[223315, 8968, 232, 7, 28, "Input",
   CellTags->"Chisholm"],
  Cell[223550, 8977, 391, 10, 27, "Output",
   CellTags->"Chisholm"],
  Cell[223978, 8992, 139, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[224120, 8998, 11506, 439, 93, "Output",
   CellTags->"Chisholm"],
  Cell[235663, 9442, 139, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[235805, 9448, 11506, 439, 93, "Output",
   CellTags->"Chisholm"],
  Cell[247326, 9890, 85, 1, 25, "Text",
   CellTags->"Chisholm"],
  Cell[247436, 9895, 130, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[247569, 9901, 125, 4, 27, "Output",
   CellTags->"Chisholm"],
  Cell[247731, 9910, 130, 4, 28, "Input",
   CellTags->"Chisholm"],
  Cell[247864, 9916, 125, 4, 27, "Output",
   CellTags->"Chisholm"],
  Cell[248004, 9923, 192, 6, 28, "Input",
   CellTags->"Chisholm"]},
 "ChisholmSpinor"->{
  Cell[248211, 9932, 65, 1, 57, "ObjectName",
   CellTags->"ChisholmSpinor"],
  Cell[248301, 9937, 65, 1, 52, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[248369, 9940, 296, 6, 59, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[248668, 9948, 304, 14, 41, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[249009, 9967, 62, 1, 36, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[249096, 9972, 312, 9, 28, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[249411, 9983, 427, 15, 27, "Output",
   CellTags->"ChisholmSpinor"],
  Cell[249875, 10003, 117, 3, 28, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[249995, 10008, 2722, 100, 85, "Output",
   CellTags->"ChisholmSpinor"]},
 "ClearScalarProducts"->{
  Cell[252744, 10112, 75, 1, 57, "ObjectName",
   CellTags->"ClearScalarProducts"],
  Cell[252844, 10117, 70, 1, 52, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[252917, 10120, 145, 4, 41, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[253065, 10126, 484, 23, 41, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[253586, 10154, 67, 1, 36, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[253678, 10159, 195, 6, 28, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[253876, 10167, 139, 4, 27, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[254052, 10176, 211, 6, 28, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[254266, 10184, 139, 4, 27, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[254420, 10191, 104, 2, 28, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[254549, 10197, 211, 6, 28, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[254763, 10205, 173, 6, 27, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[254973, 10216, 148, 4, 28, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[255124, 10222, 173, 6, 27, "Output",
   CellTags->"ClearScalarProducts"]},
 "Collect2"->{
  Cell[256106, 10268, 53, 1, 57, "ObjectName",
   CellTags->"Collect2"],
  Cell[256184, 10273, 59, 1, 52, "Subsubsection",
   CellTags->"Collect2"],
  Cell[256246, 10276, 613, 10, 97, "Text",
   CellTags->"Collect2"],
  Cell[256884, 10290, 115, 3, 28, "Input",
   CellTags->"Collect2"],
  Cell[257002, 10295, 399, 11, 27, "Output",
   CellTags->"Collect2"],
  Cell[257416, 10309, 176, 9, 41, "Text",
   CellTags->"Collect2"],
  Cell[257629, 10323, 56, 1, 36, "Subsubsection",
   CellTags->"Collect2"],
  Cell[257710, 10328, 440, 14, 37, "Input",
   CellTags->"Collect2"],
  Cell[258153, 10344, 451, 16, 38, "Output",
   CellTags->"Collect2"],
  Cell[258641, 10365, 185, 5, 28, "Input",
   CellTags->"Collect2"],
  Cell[258829, 10372, 399, 14, 37, "Output",
   CellTags->"Collect2"],
  Cell[259265, 10391, 186, 5, 28, "Input",
   CellTags->"Collect2"],
  Cell[259454, 10398, 451, 16, 38, "Output",
   CellTags->"Collect2"],
  Cell[259942, 10419, 525, 18, 28, "Input",
   CellTags->"Collect2"],
  Cell[260470, 10439, 514, 17, 27, "Output",
   CellTags->"Collect2"],
  Cell[261021, 10461, 320, 10, 29, "Input",
   CellTags->"Collect2"],
  Cell[261344, 10473, 815, 31, 28, "Output",
   CellTags->"Collect2"],
  Cell[262196, 10509, 112, 3, 28, "Input",
   CellTags->"Collect2"],
  Cell[262311, 10514, 825, 27, 28, "Output",
   CellTags->"Collect2"],
  Cell[263151, 10544, 130, 4, 28, "Input",
   CellTags->"Collect2"],
  Cell[263306, 10552, 305, 9, 28, "Input",
   CellTags->"Collect2"],
  Cell[263614, 10563, 935, 31, 27, "Output",
   CellTags->"Collect2"],
  Cell[264586, 10599, 149, 4, 28, "Input",
   CellTags->"Collect2"],
  Cell[264738, 10605, 671, 21, 27, "Output",
   CellTags->"Collect2"]},
 "Collect3"->{
  Cell[265436, 10630, 53, 1, 41, "ObjectName",
   CellTags->"Collect3"],
  Cell[265514, 10635, 59, 1, 36, "Subsubsection",
   CellTags->"Collect3"],
  Cell[265576, 10638, 719, 23, 61, "Text",
   CellTags->"Collect3"],
  Cell[266320, 10665, 115, 3, 28, "Input",
   CellTags->"Collect3"],
  Cell[266438, 10670, 233, 8, 27, "Output",
   CellTags->"Collect3"],
  Cell[266686, 10681, 286, 14, 25, "Text",
   CellTags->"Collect3"],
  Cell[267009, 10700, 56, 1, 36, "Subsubsection",
   CellTags->"Collect3"],
  Cell[267090, 10705, 525, 18, 28, "Input",
   CellTags->"Collect3"],
  Cell[267618, 10725, 704, 22, 27, "Output",
   CellTags->"Collect3"],
  Cell[268359, 10752, 289, 9, 29, "Input",
   CellTags->"Collect3"],
  Cell[268651, 10763, 5476, 142, 137, "Output",
   CellTags->"Collect3"]},
 "Combine"->{
  Cell[274154, 10909, 51, 1, 41, "ObjectName",
   CellTags->"Combine"],
  Cell[274230, 10914, 58, 1, 36, "Subsubsection",
   CellTags->"Combine"],
  Cell[274291, 10917, 315, 6, 61, "Text",
   CellTags->"Combine"],
  Cell[274631, 10927, 113, 3, 28, "Input",
   CellTags->"Combine"],
  Cell[274747, 10932, 171, 6, 27, "Output",
   CellTags->"Combine"],
  Cell[274933, 10941, 175, 9, 25, "Text",
   CellTags->"Combine"],
  Cell[275145, 10955, 55, 1, 36, "Subsubsection",
   CellTags->"Combine"],
  Cell[275225, 10960, 283, 10, 40, "Input",
   CellTags->"Combine"],
  Cell[275511, 10972, 304, 11, 40, "Output",
   CellTags->"Combine"],
  Cell[275830, 10986, 122, 3, 25, "Text",
   CellTags->"Combine"],
  Cell[275977, 10993, 284, 10, 40, "Input",
   CellTags->"Combine"],
  Cell[276264, 11005, 303, 10, 40, "Output",
   CellTags->"Combine"],
  Cell[276582, 11018, 175, 4, 25, "Text",
   CellTags->"Combine"],
  Cell[276782, 11026, 354, 12, 40, "Input",
   CellTags->"Combine"],
  Cell[277139, 11040, 303, 10, 40, "Output",
   CellTags->"Combine"]},
 "CombineGraphs"->{
  Cell[277469, 11054, 63, 1, 41, "ObjectName",
   CellTags->"CombineGraphs"],
  Cell[277557, 11059, 64, 1, 36, "Subsubsection",
   CellTags->"CombineGraphs"],
  Cell[277624, 11062, 86, 1, 25, "Text",
   CellTags->"CombineGraphs"],
  Cell[277713, 11065, 185, 8, 25, "Text",
   CellTags->"CombineGraphs"]},
 "Commutator"->{
  Cell[277913, 11076, 57, 1, 41, "ObjectName",
   CellTags->"Commutator"],
  Cell[277995, 11081, 61, 1, 36, "Subsubsection",
   CellTags->"Commutator"],
  Cell[278059, 11084, 138, 4, 25, "Text",
   CellTags->"Commutator"],
  Cell[278200, 11090, 607, 24, 25, "Text",
   CellTags->"Commutator"],
  Cell[278844, 11119, 58, 1, 36, "Subsubsection",
   CellTags->"Commutator"],
  Cell[278905, 11122, 167, 4, 28, "Input",
   CellTags->"Commutator"],
  Cell[279097, 11130, 136, 4, 28, "Input",
   CellTags->"Commutator"],
  Cell[279236, 11136, 184, 5, 27, "Output",
   CellTags->"Commutator"],
  Cell[279457, 11146, 121, 3, 28, "Input",
   CellTags->"Commutator"],
  Cell[279581, 11151, 177, 6, 27, "Output",
   CellTags->"Commutator"],
  Cell[279795, 11162, 247, 8, 28, "Input",
   CellTags->"Commutator"],
  Cell[280045, 11172, 381, 12, 27, "Output",
   CellTags->"Commutator"],
  Cell[280441, 11187, 169, 4, 28, "Input",
   CellTags->"Commutator"],
  Cell[280613, 11193, 68, 1, 25, "Text",
   CellTags->"Commutator"],
  Cell[280684, 11196, 228, 6, 28, "Input",
   CellTags->"Commutator"],
  Cell[280937, 11206, 463, 15, 28, "Input",
   CellTags->"Commutator"],
  Cell[281403, 11223, 727, 19, 27, "Output",
   CellTags->"Commutator"],
  Cell[282167, 11247, 114, 3, 28, "Input",
   CellTags->"Commutator"],
  Cell[282284, 11252, 106, 3, 27, "Output",
   CellTags->"Commutator"],
  Cell[282405, 11258, 113, 3, 28, "Input",
   CellTags->"Commutator"],
  Cell[282521, 11263, 159, 4, 28, "Input",
   CellTags->"Commutator"]},
 "CommutatorExplicit"->{
  Cell[282695, 11270, 73, 1, 41, "ObjectName",
   CellTags->"CommutatorExplicit"],
  Cell[282793, 11275, 69, 1, 36, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[282865, 11278, 157, 4, 25, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[283025, 11284, 296, 14, 25, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[283358, 11303, 66, 1, 36, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[283427, 11306, 175, 4, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[283627, 11314, 144, 4, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[283774, 11320, 192, 5, 27, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[284003, 11330, 129, 3, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[284135, 11335, 185, 6, 27, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[284357, 11346, 196, 6, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[284556, 11354, 283, 10, 27, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[284876, 11369, 129, 3, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[285008, 11374, 389, 14, 27, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[285434, 11393, 167, 5, 28, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[285604, 11400, 389, 12, 27, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[286008, 11415, 177, 4, 28, "Input",
   CellTags->"CommutatorExplicit"]},
 "CompleteSquare"->{
  Cell[286200, 11422, 65, 1, 57, "ObjectName",
   CellTags->"CompleteSquare"],
  Cell[286290, 11427, 65, 1, 52, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[286358, 11430, 1107, 82, 59, "Text",
   CellTags->"CompleteSquare"],
  Cell[287502, 11517, 62, 1, 36, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[288243, 11551, 170, 5, 28, "Input",
   CellTags->"CompleteSquare"],
  Cell[288416, 11558, 685, 27, 42, "Output",
   CellTags->"CompleteSquare"],
  Cell[289138, 11590, 180, 6, 28, "Input",
   CellTags->"CompleteSquare"],
  Cell[289321, 11598, 110, 3, 27, "Output",
   CellTags->"CompleteSquare"],
  Cell[289468, 11606, 155, 4, 28, "Input",
   CellTags->"CompleteSquare"],
  Cell[289626, 11612, 515, 20, 42, "Output",
   CellTags->"CompleteSquare"],
  Cell[290156, 11635, 137, 4, 28, "Input",
   CellTags->"CompleteSquare"]},
 "ComplexConjugate"->{
  Cell[290308, 11642, 69, 1, 57, "ObjectName",
   CellTags->"ComplexConjugate"],
  Cell[290402, 11647, 67, 1, 52, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[290472, 11650, 414, 7, 77, "Text",
   CellTags->"ComplexConjugate"],
  Cell[290889, 11659, 449, 19, 41, "Text",
   CellTags->"ComplexConjugate"],
  Cell[291375, 11683, 64, 1, 36, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[291464, 11688, 196, 5, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[291663, 11695, 163, 5, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[291863, 11705, 121, 3, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[291987, 11710, 239, 6, 42, "Output",
   CellTags->"ComplexConjugate"],
  Cell[292263, 11721, 152, 4, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[292418, 11727, 249, 7, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[292704, 11739, 125, 3, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[292832, 11744, 427, 16, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[293296, 11765, 182, 5, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[293481, 11772, 531, 22, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[294049, 11799, 125, 3, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[294177, 11804, 531, 22, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[294745, 11831, 190, 5, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[294938, 11838, 380, 17, 27, "Output",
   CellTags->"ComplexConjugate"],
  Cell[295355, 11860, 270, 7, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[295628, 11869, 157, 4, 42, "Output",
   CellTags->"ComplexConjugate"],
  Cell[295822, 11878, 273, 8, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[296098, 11888, 335, 10, 28, "Output",
   CellTags->"ComplexConjugate"],
  Cell[296470, 11903, 125, 3, 28, "Input",
   CellTags->"ComplexConjugate"],
  Cell[296598, 11908, 431, 15, 28, "Output",
   CellTags->"ComplexConjugate"],
  Cell[297044, 11926, 166, 4, 41, "Text",
   CellTags->"ComplexConjugate"]},
 "Contract"->{
  Cell[297225, 11933, 53, 1, 57, "ObjectName",
   CellTags->"Contract"],
  Cell[297303, 11938, 59, 1, 52, "Subsubsection",
   CellTags->"Contract"],
  Cell[297365, 11941, 411, 7, 61, "Text",
   CellTags->"Contract"],
  Cell[297801, 11952, 115, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[297919, 11957, 612, 15, 49, "Output",
   CellTags->"Contract"],
  Cell[298687, 11980, 423, 19, 41, "Text",
   CellTags->"Contract"],
  Cell[299147, 12004, 56, 1, 36, "Subsubsection",
   CellTags->"Contract"],
  Cell[299228, 12009, 233, 7, 28, "Input",
   CellTags->"Contract"],
  Cell[299464, 12018, 319, 13, 27, "Output",
   CellTags->"Contract"],
  Cell[299820, 12036, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[299932, 12041, 236, 10, 27, "Output",
   CellTags->"Contract"],
  Cell[300205, 12056, 199, 6, 28, "Input",
   CellTags->"Contract"],
  Cell[300407, 12064, 376, 16, 27, "Output",
   CellTags->"Contract"],
  Cell[300820, 12085, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[300932, 12090, 221, 8, 27, "Output",
   CellTags->"Contract"],
  Cell[301190, 12103, 144, 4, 28, "Input",
   CellTags->"Contract"],
  Cell[301337, 12109, 155, 5, 27, "Output",
   CellTags->"Contract"],
  Cell[301507, 12117, 83, 1, 25, "Text",
   CellTags->"Contract"],
  Cell[301615, 12122, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[301727, 12127, 104, 3, 27, "Output",
   CellTags->"Contract"],
  Cell[301846, 12133, 208, 5, 43, "Text",
   CellTags->"Contract"],
  Cell[302079, 12142, 231, 7, 28, "Input",
   CellTags->"Contract"],
  Cell[302313, 12151, 233, 8, 27, "Output",
   CellTags->"Contract"],
  Cell[302583, 12164, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[302695, 12169, 104, 3, 27, "Output",
   CellTags->"Contract"],
  Cell[302836, 12177, 231, 7, 28, "Input",
   CellTags->"Contract"],
  Cell[303070, 12186, 233, 8, 27, "Output",
   CellTags->"Contract"],
  Cell[303340, 12199, 227, 7, 28, "Input",
   CellTags->"Contract"],
  Cell[303570, 12208, 405, 18, 27, "Output",
   CellTags->"Contract"],
  Cell[304012, 12231, 114, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[304129, 12236, 206, 8, 27, "Output",
   CellTags->"Contract"],
  Cell[304372, 12249, 277, 9, 28, "Input",
   CellTags->"Contract"],
  Cell[304652, 12260, 513, 22, 27, "Output",
   CellTags->"Contract"],
  Cell[305202, 12287, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[305314, 12292, 575, 24, 27, "Output",
   CellTags->"Contract"],
  Cell[305926, 12321, 271, 8, 28, "Input",
   CellTags->"Contract"],
  Cell[306200, 12331, 667, 29, 27, "Output",
   CellTags->"Contract"],
  Cell[306904, 12365, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[307016, 12370, 439, 19, 27, "Output",
   CellTags->"Contract"],
  Cell[307492, 12394, 313, 9, 28, "Input",
   CellTags->"Contract"],
  Cell[307808, 12405, 922, 40, 28, "Output",
   CellTags->"Contract"],
  Cell[308767, 12450, 109, 3, 28, "Input",
   CellTags->"Contract"],
  Cell[308879, 12455, 204, 7, 28, "Output",
   CellTags->"Contract"],
  Cell[309120, 12467, 426, 13, 28, "Input",
   CellTags->"Contract"],
  Cell[309549, 12482, 301, 9, 28, "Output",
   CellTags->"Contract"],
  Cell[309887, 12496, 139, 4, 28, "Input",
   CellTags->"Contract"],
  Cell[310029, 12502, 352, 12, 28, "Output",
   CellTags->"Contract"],
  Cell[310396, 12517, 194, 6, 28, "Input",
   CellTags->"Contract"]},
 "Contract1"->{
  Cell[310605, 12526, 55, 1, 57, "ObjectName",
   CellTags->"Contract1"],
  Cell[310685, 12531, 60, 1, 52, "Subsubsection",
   CellTags->"Contract1"],
  Cell[310748, 12534, 459, 8, 119, "Text",
   CellTags->"Contract1"],
  Cell[311210, 12544, 389, 18, 41, "Text",
   CellTags->"Contract1"],
  Cell[311636, 12567, 57, 1, 36, "Subsubsection",
   CellTags->"Contract1"],
  Cell[311718, 12572, 349, 12, 28, "Input",
   CellTags->"Contract1"],
  Cell[312070, 12586, 207, 8, 27, "Output",
   CellTags->"Contract1"],
  Cell[312314, 12599, 349, 12, 28, "Input",
   CellTags->"Contract1"],
  Cell[312666, 12613, 478, 20, 27, "Output",
   CellTags->"Contract1"],
  Cell[313181, 12638, 269, 9, 28, "Input",
   CellTags->"Contract1"],
  Cell[313453, 12649, 105, 3, 27, "Output",
   CellTags->"Contract1"],
  Cell[313595, 12657, 269, 9, 28, "Input",
   CellTags->"Contract1"],
  Cell[313867, 12668, 298, 11, 27, "Output",
   CellTags->"Contract1"]},
 "Convolute"->{
  Cell[314192, 12683, 55, 1, 57, "ObjectName",
   CellTags->"Convolute"],
  Cell[314272, 12688, 60, 1, 52, "Subsubsection",
   CellTags->"Convolute"],
  Cell[314335, 12691, 1367, 50, 65, "Text",
   CellTags->"Convolute"],
  Cell[315727, 12745, 117, 3, 28, "Input",
   CellTags->"Convolute"],
  Cell[315847, 12750, 400, 13, 27, "Output",
   CellTags->"Convolute"],
  Cell[316262, 12766, 332, 14, 41, "Text",
   CellTags->"Convolute"],
  Cell[316631, 12785, 57, 1, 36, "Subsubsection",
   CellTags->"Convolute"],
  Cell[316713, 12790, 134, 4, 28, "Input",
   CellTags->"Convolute"],
  Cell[316850, 12796, 154, 5, 27, "Output",
   CellTags->"Convolute"],
  Cell[317041, 12806, 134, 4, 28, "Input",
   CellTags->"Convolute"],
  Cell[317178, 12812, 178, 6, 27, "Output",
   CellTags->"Convolute"],
  Cell[317393, 12823, 134, 4, 28, "Input",
   CellTags->"Convolute"],
  Cell[317530, 12829, 128, 4, 27, "Output",
   CellTags->"Convolute"],
  Cell[317695, 12838, 181, 6, 39, "Input",
   CellTags->"Convolute"],
  Cell[317879, 12846, 215, 7, 27, "Output",
   CellTags->"Convolute"],
  Cell[318131, 12858, 227, 7, 39, "Input",
   CellTags->"Convolute"],
  Cell[318361, 12867, 215, 7, 27, "Output",
   CellTags->"Convolute"],
  Cell[318613, 12879, 181, 6, 39, "Input",
   CellTags->"Convolute"],
  Cell[318797, 12887, 289, 10, 27, "Output",
   CellTags->"Convolute"],
  Cell[319123, 12902, 228, 8, 39, "Input",
   CellTags->"Convolute"],
  Cell[319354, 12912, 532, 19, 40, "Output",
   CellTags->"Convolute"],
  Cell[319923, 12936, 190, 6, 28, "Input",
   CellTags->"Convolute"],
  Cell[320116, 12944, 431, 16, 27, "Output",
   CellTags->"Convolute"],
  Cell[320584, 12965, 216, 7, 28, "Input",
   CellTags->"Convolute"],
  Cell[320803, 12974, 273, 10, 27, "Output",
   CellTags->"Convolute"],
  Cell[321113, 12989, 237, 8, 39, "Input",
   CellTags->"Convolute"],
  Cell[321353, 12999, 369, 12, 27, "Output",
   CellTags->"Convolute"],
  Cell[321759, 13016, 263, 9, 39, "Input",
   CellTags->"Convolute"],
  Cell[322025, 13027, 597, 20, 27, "Output",
   CellTags->"Convolute"],
  Cell[322659, 13052, 239, 8, 39, "Input",
   CellTags->"Convolute"],
  Cell[322901, 13062, 811, 28, 38, "Output",
   CellTags->"Convolute"],
  Cell[323749, 13095, 190, 6, 28, "Input",
   CellTags->"Convolute"],
  Cell[323942, 13103, 218, 8, 27, "Output",
   CellTags->"Convolute"],
  Cell[324197, 13116, 190, 6, 28, "Input",
   CellTags->"Convolute"],
  Cell[324390, 13124, 317, 11, 27, "Output",
   CellTags->"Convolute"],
  Cell[324744, 13140, 260, 9, 39, "Input",
   CellTags->"Convolute"],
  Cell[325007, 13151, 452, 16, 45, "Output",
   CellTags->"Convolute"],
  Cell[325496, 13172, 165, 5, 28, "Input",
   CellTags->"Convolute"],
  Cell[325664, 13179, 229, 9, 38, "Output",
   CellTags->"Convolute"],
  Cell[325930, 13193, 195, 6, 28, "Input",
   CellTags->"Convolute"],
  Cell[326128, 13201, 239, 9, 38, "Output",
   CellTags->"Convolute"],
  Cell[326404, 13215, 212, 7, 39, "Input",
   CellTags->"Convolute"],
  Cell[326619, 13224, 546, 20, 38, "Output",
   CellTags->"Convolute"],
  Cell[327202, 13249, 213, 7, 39, "Input",
   CellTags->"Convolute"],
  Cell[327418, 13258, 422, 16, 38, "Output",
   CellTags->"Convolute"],
  Cell[327877, 13279, 237, 8, 39, "Input",
   CellTags->"Convolute"],
  Cell[328117, 13289, 685, 24, 38, "Output",
   CellTags->"Convolute"],
  Cell[328839, 13318, 213, 7, 39, "Input",
   CellTags->"Convolute"],
  Cell[329055, 13327, 526, 19, 38, "Output",
   CellTags->"Convolute"],
  Cell[329618, 13351, 190, 6, 28, "Input",
   CellTags->"Convolute"],
  Cell[329811, 13359, 218, 8, 27, "Output",
   CellTags->"Convolute"],
  Cell[330066, 13372, 165, 5, 28, "Input",
   CellTags->"Convolute"],
  Cell[330234, 13379, 188, 6, 27, "Output",
   CellTags->"Convolute"],
  Cell[330459, 13390, 286, 10, 39, "Input",
   CellTags->"Convolute"],
  Cell[330748, 13402, 986, 37, 45, "Output",
   CellTags->"Convolute"],
  Cell[331771, 13444, 295, 10, 39, "Input",
   CellTags->"Convolute"],
  Cell[332069, 13456, 1733, 64, 61, "Output",
   CellTags->"Convolute"],
  Cell[336183, 13633, 244, 8, 39, "Input",
   CellTags->"Convolute"],
  Cell[336430, 13643, 551, 20, 38, "Output",
   CellTags->"Convolute"]},
 "ConvoluteTable"->{
  Cell[333829, 13524, 66, 1, 57, "ObjectName",
   CellTags->"ConvoluteTable"],
  Cell[333920, 13529, 65, 1, 52, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[333988, 13532, 152, 4, 25, "Text",
   CellTags->"ConvoluteTable"],
  Cell[334165, 13540, 122, 3, 28, "Input",
   CellTags->"ConvoluteTable"],
  Cell[334290, 13545, 405, 13, 27, "Output",
   CellTags->"ConvoluteTable"],
  Cell[334710, 13561, 322, 14, 41, "Text",
   CellTags->"ConvoluteTable"],
  Cell[335069, 13580, 62, 1, 36, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[335156, 13585, 144, 4, 28, "Input",
   CellTags->"ConvoluteTable"],
  Cell[335303, 13591, 159, 5, 27, "Output",
   CellTags->"ConvoluteTable"],
  Cell[335499, 13601, 144, 4, 28, "Input",
   CellTags->"ConvoluteTable"],
  Cell[335646, 13607, 183, 6, 27, "Output",
   CellTags->"ConvoluteTable"],
  Cell[335866, 13618, 144, 4, 28, "Input",
   CellTags->"ConvoluteTable"],
  Cell[336013, 13624, 133, 4, 27, "Output",
   CellTags->"ConvoluteTable"]},
 "CovariantFieldDerivative"->{
  Cell[337008, 13667, 85, 1, 57, "ObjectName",
   CellTags->"CovariantFieldDerivative"],
  Cell[337118, 13672, 75, 1, 52, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[337196, 13675, 372, 6, 59, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[337571, 13683, 460, 19, 41, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[338068, 13707, 72, 1, 36, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[338165, 13712, 329, 9, 28, "Input",
   CellTags->"CovariantFieldDerivative"],
  Cell[338497, 13723, 543, 21, 27, "Output",
   CellTags->"CovariantFieldDerivative"]},
 "CounterT"->{
  Cell[339067, 13748, 53, 1, 57, "ObjectName",
   CellTags->"CounterT"],
  Cell[339145, 13753, 59, 1, 52, "Subsubsection",
   CellTags->"CounterT"],
  Cell[339207, 13756, 146, 4, 41, "Text",
   CellTags->"CounterT"],
  Cell[339356, 13762, 450, 19, 41, "Text",
   CellTags->"CounterT"],
  Cell[339843, 13786, 56, 1, 36, "Subsubsection",
   CellTags->"CounterT"],
  Cell[339924, 13791, 284, 6, 28, "Input",
   CellTags->"CounterT"],
  Cell[340211, 13799, 3127, 106, 54, "Output",
   CellTags->"CounterT"]},
 "CounterTerm"->{
  Cell[343365, 13909, 59, 1, 57, "ObjectName",
   CellTags->"CounterTerm"],
  Cell[343449, 13914, 62, 1, 36, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[343514, 13917, 216, 5, 43, "Text",
   CellTags->"CounterTerm"],
  Cell[343733, 13924, 578, 24, 25, "Text",
   CellTags->"CounterTerm"],
  Cell[344348, 13953, 59, 1, 36, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[344432, 13958, 116, 3, 28, "Input",
   CellTags->"CounterTerm"],
  Cell[344551, 13963, 1634, 53, 59, "Output",
   CellTags->"CounterTerm"]},
 "CouplingConstant"->{
  Cell[346212, 14020, 69, 1, 41, "ObjectName",
   CellTags->"CouplingConstant"],
  Cell[346306, 14025, 67, 1, 36, "Subsubsection",
   CellTags->"CouplingConstant"],
  Cell[346376, 14028, 657, 11, 105, "Text",
   CellTags->"CouplingConstant"],
  Cell[347036, 14041, 318, 14, 25, "Text",
   CellTags->"CouplingConstant"]},
 "CovariantD"->{
  Cell[347369, 14058, 57, 1, 41, "ObjectName",
   CellTags->"CovariantD"],
  Cell[347451, 14063, 61, 1, 36, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[347515, 14066, 1212, 19, 229, "Text",
   CellTags->"CovariantD"],
  Cell[348752, 14089, 119, 3, 28, "Input",
   CellTags->"CovariantD"],
  Cell[348874, 14094, 439, 12, 27, "Output",
   CellTags->"CovariantD"],
  Cell[349328, 14109, 185, 4, 43, "Text",
   CellTags->"CovariantD"],
  Cell[349516, 14115, 455, 19, 25, "Text",
   CellTags->"CovariantD"],
  Cell[350008, 14139, 58, 1, 36, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[350091, 14144, 117, 3, 28, "Input",
   CellTags->"CovariantD"],
  Cell[350211, 14149, 132, 4, 27, "Output",
   CellTags->"CovariantD"],
  Cell[350380, 14158, 150, 4, 28, "Input",
   CellTags->"CovariantD"],
  Cell[350533, 14164, 161, 5, 27, "Output",
   CellTags->"CovariantD"],
  Cell[350731, 14174, 180, 5, 28, "Input",
   CellTags->"CovariantD"],
  Cell[350914, 14181, 992, 39, 33, "Output",
   CellTags->"CovariantD"],
  Cell[351921, 14223, 164, 4, 25, "Text",
   CellTags->"CovariantD"],
  Cell[352110, 14231, 120, 3, 28, "Input",
   CellTags->"CovariantD"],
  Cell[352233, 14236, 176, 6, 27, "Output",
   CellTags->"CovariantD"],
  Cell[352446, 14247, 153, 4, 28, "Input",
   CellTags->"CovariantD"],
  Cell[352602, 14253, 205, 7, 27, "Output",
   CellTags->"CovariantD"],
  Cell[352844, 14265, 203, 5, 28, "Input",
   CellTags->"CovariantD"],
  Cell[353050, 14272, 939, 36, 33, "Output",
   CellTags->"CovariantD"],
  Cell[354026, 14313, 183, 5, 28, "Input",
   CellTags->"CovariantD"],
  Cell[354212, 14320, 932, 35, 33, "Output",
   CellTags->"CovariantD"],
  Cell[355181, 14360, 187, 5, 28, "Input",
   CellTags->"CovariantD"],
  Cell[355371, 14367, 1339, 48, 36, "Output",
   CellTags->"CovariantD"],
  Cell[356725, 14418, 602, 22, 31, "Text",
   CellTags->"CovariantD"],
  Cell[357352, 14444, 215, 6, 28, "Input",
   CellTags->"CovariantD"],
  Cell[357570, 14452, 513, 20, 34, "Output",
   CellTags->"CovariantD"],
  Cell[358098, 14475, 285, 10, 25, "Text",
   CellTags->"CovariantD"],
  Cell[358408, 14489, 215, 6, 28, "Input",
   CellTags->"CovariantD"],
  Cell[358626, 14497, 2262, 80, 51, "Output",
   CellTags->"CovariantD"],
  Cell[360903, 14580, 189, 7, 25, "Text",
   CellTags->"CovariantD"],
  Cell[361117, 14591, 215, 6, 28, "Input",
   CellTags->"CovariantD"],
  Cell[361335, 14599, 5963, 196, 54, "Output",
   CellTags->"CovariantD"],
  Cell[367335, 14800, 181, 5, 28, "Input",
   CellTags->"CovariantD"],
  Cell[367519, 14807, 261, 9, 28, "Output",
   CellTags->"CovariantD"],
  Cell[367817, 14821, 195, 6, 28, "Input",
   CellTags->"CovariantD"],
  Cell[368015, 14829, 4522, 146, 54, "Output",
   CellTags->"CovariantD"],
  Cell[372574, 14980, 223, 7, 28, "Input",
   CellTags->"CovariantD"],
  Cell[372800, 14989, 788, 24, 62, "Output",
   CellTags->"CovariantD"],
  Cell[373625, 15018, 240, 7, 28, "Input",
   CellTags->"CovariantD"],
  Cell[373868, 15027, 803, 25, 62, "Output",
   CellTags->"CovariantD"]},
 "CrossProduct"->{
  Cell[374698, 15056, 61, 1, 57, "ObjectName",
   CellTags->"CrossProduct"],
  Cell[374784, 15061, 63, 1, 52, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[374850, 15064, 145, 4, 41, "Text",
   CellTags->"CrossProduct"],
  Cell[374998, 15070, 308, 14, 41, "Text",
   CellTags->"CrossProduct"],
  Cell[375343, 15089, 60, 1, 36, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[375428, 15094, 326, 9, 28, "Input",
   CellTags->"CrossProduct"],
  Cell[375757, 15105, 639, 22, 30, "Output",
   CellTags->"CrossProduct"]},
 "C0"->{
  Cell[376423, 15131, 41, 1, 57, "ObjectName",
   CellTags->"C0"],
  Cell[376489, 15136, 53, 1, 52, "Subsubsection",
   CellTags->"C0"],
  Cell[376545, 15139, 448, 10, 59, "Text",
   CellTags->"C0"],
  Cell[376996, 15151, 458, 24, 41, "Text",
   CellTags->"C0"],
  Cell[377491, 15180, 50, 1, 36, "Subsubsection",
   CellTags->"C0"],
  Cell[377566, 15185, 174, 5, 28, "Input",
   CellTags->"C0"],
  Cell[377743, 15192, 260, 8, 27, "Output",
   CellTags->"C0"],
  Cell[378040, 15205, 203, 6, 28, "Input",
   CellTags->"C0"],
  Cell[378246, 15213, 260, 8, 27, "Output",
   CellTags->"C0"],
  Cell[378543, 15226, 326, 10, 28, "Input",
   CellTags->"C0"],
  Cell[378872, 15238, 260, 8, 27, "Output",
   CellTags->"C0"]},
 "DataType"->{
  Cell[379159, 15250, 53, 1, 41, "ObjectName",
   CellTags->"DataType"],
  Cell[379237, 15255, 59, 1, 36, "Subsubsection",
   CellTags->"DataType"],
  Cell[379299, 15258, 344, 6, 61, "Text",
   CellTags->"DataType"],
  Cell[379646, 15266, 835, 32, 42, "Text",
   CellTags->"DataType"],
  Cell[380484, 15300, 291, 13, 25, "Text",
   CellTags->"DataType"],
  Cell[380778, 15315, 218, 9, 25, "Text",
   CellTags->"DataType"],
  Cell[381033, 15329, 56, 1, 36, "Subsubsection",
   CellTags->"DataType"],
  Cell[381092, 15332, 74, 1, 25, "Text",
   CellTags->"DataType"],
  Cell[381169, 15335, 221, 7, 28, "Input",
   CellTags->"DataType"],
  Cell[381415, 15346, 224, 8, 28, "Input",
   CellTags->"DataType"],
  Cell[381642, 15356, 238, 9, 27, "Output",
   CellTags->"DataType"],
  Cell[381895, 15368, 179, 4, 43, "Text",
   CellTags->"DataType"],
  Cell[382099, 15376, 112, 3, 28, "Input",
   CellTags->"DataType"],
  Cell[382214, 15381, 210, 7, 27, "Output",
   CellTags->"DataType"],
  Cell[382439, 15391, 271, 9, 28, "Input",
   CellTags->"DataType"],
  Cell[382713, 15402, 412, 14, 28, "Input",
   CellTags->"DataType"],
  Cell[383128, 15418, 395, 14, 28, "Input",
   CellTags->"DataType"],
  Cell[383548, 15436, 340, 13, 28, "Input",
   CellTags->"DataType"],
  Cell[383891, 15451, 350, 13, 27, "Output",
   CellTags->"DataType"],
  Cell[384278, 15469, 107, 3, 28, "Input",
   CellTags->"DataType"],
  Cell[384388, 15474, 298, 11, 27, "Output",
   CellTags->"DataType"],
  Cell[384723, 15490, 107, 3, 28, "Input",
   CellTags->"DataType"],
  Cell[384833, 15495, 172, 6, 27, "Output",
   CellTags->"DataType"],
  Cell[385020, 15504, 191, 6, 28, "Input",
   CellTags->"DataType"],
  Cell[385214, 15512, 187, 6, 28, "Input",
   CellTags->"DataType"],
  Cell[385404, 15520, 429, 15, 28, "Input",
   CellTags->"DataType"],
  Cell[385858, 15539, 290, 11, 28, "Input",
   CellTags->"DataType"],
  Cell[386151, 15552, 296, 11, 27, "Output",
   CellTags->"DataType"],
  Cell[386484, 15568, 105, 3, 28, "Input",
   CellTags->"DataType"],
  Cell[386592, 15573, 228, 9, 27, "Output",
   CellTags->"DataType"],
  Cell[386835, 15585, 151, 5, 28, "Input",
   CellTags->"DataType"],
  Cell[386989, 15592, 222, 7, 28, "Input",
   CellTags->"DataType"],
  Cell[387214, 15601, 272, 9, 28, "Input",
   CellTags->"DataType"],
  Cell[387489, 15612, 114, 3, 25, "Text",
   CellTags->"DataType"],
  Cell[387628, 15619, 149, 4, 28, "Input",
   CellTags->"DataType"],
  Cell[387780, 15625, 107, 3, 27, "Output",
   CellTags->"DataType"],
  Cell[387902, 15631, 83, 1, 25, "Text",
   CellTags->"DataType"],
  Cell[388010, 15636, 233, 8, 28, "Input",
   CellTags->"DataType"],
  Cell[388246, 15646, 104, 3, 27, "Output",
   CellTags->"DataType"],
  Cell[388387, 15654, 226, 7, 28, "Input",
   CellTags->"DataType"],
  Cell[388616, 15663, 435, 16, 27, "Output",
   CellTags->"DataType"]},
 "DB0"->{
  Cell[389078, 15683, 43, 1, 41, "ObjectName",
   CellTags->"DB0"],
  Cell[389146, 15688, 54, 1, 36, "Subsubsection",
   CellTags->"DB0"],
  Cell[389203, 15691, 146, 4, 25, "Text",
   CellTags->"DB0"],
  Cell[389352, 15697, 156, 9, 25, "Text",
   CellTags->"DB0"],
  Cell[389545, 15711, 51, 1, 36, "Subsubsection",
   CellTags->"DB0"],
  Cell[389621, 15716, 339, 12, 28, "Input",
   CellTags->"DB0"],
  Cell[389963, 15730, 255, 8, 27, "Output",
   CellTags->"DB0"]},
 "DB1"->{
  Cell[390245, 15742, 43, 1, 41, "ObjectName",
   CellTags->"DB1"],
  Cell[390313, 15747, 54, 1, 36, "Subsubsection",
   CellTags->"DB1"],
  Cell[390370, 15750, 117, 3, 25, "Text",
   CellTags->"DB1"],
  Cell[390490, 15755, 156, 9, 41, "Text",
   CellTags->"DB1"],
  Cell[390683, 15769, 51, 1, 36, "Subsubsection",
   CellTags->"DB1"],
  Cell[390759, 15774, 339, 12, 28, "Input",
   CellTags->"DB1"],
  Cell[391101, 15788, 1369, 43, 44, "Output",
   CellTags->"DB1"]},
 "DeclareNonCommutative"->{
  Cell[392497, 15835, 79, 1, 57, "ObjectName",
   CellTags->"DeclareNonCommutative"],
  Cell[392601, 15840, 72, 1, 52, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[392676, 15843, 192, 4, 41, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[392871, 15849, 347, 14, 41, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[393255, 15868, 69, 1, 36, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[393327, 15871, 135, 3, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[393465, 15876, 152, 4, 25, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[393642, 15884, 158, 4, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[393803, 15890, 120, 3, 27, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[393938, 15896, 158, 4, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[394121, 15904, 191, 5, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[394315, 15911, 199, 6, 27, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[394529, 15920, 170, 4, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[394724, 15928, 191, 5, 28, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[394918, 15935, 199, 6, 27, "Output",
   CellTags->"DeclareNonCommutative"]},
 "DeltaFunction"->{
  Cell[395144, 15945, 63, 1, 41, "ObjectName",
   CellTags->"DeltaFunction"],
  Cell[395232, 15950, 64, 1, 36, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[395299, 15953, 353, 11, 51, "Text",
   CellTags->"DeltaFunction"],
  Cell[395655, 15966, 592, 24, 25, "Text",
   CellTags->"DeltaFunction"],
  Cell[396284, 15995, 61, 1, 36, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[396370, 16000, 142, 4, 28, "Input",
   CellTags->"DeltaFunction"],
  Cell[396515, 16006, 203, 7, 27, "Output",
   CellTags->"DeltaFunction"],
  Cell[396755, 16018, 320, 10, 28, "Input",
   CellTags->"DeltaFunction"],
  Cell[397078, 16030, 137, 4, 27, "Output",
   CellTags->"DeltaFunction"],
  Cell[397252, 16039, 294, 9, 28, "Input",
   CellTags->"DeltaFunction"],
  Cell[397549, 16050, 137, 4, 27, "Output",
   CellTags->"DeltaFunction"],
  Cell[397723, 16059, 320, 10, 28, "Input",
   CellTags->"DeltaFunction"],
  Cell[398046, 16071, 137, 4, 27, "Output",
   CellTags->"DeltaFunction"],
  Cell[398220, 16080, 204, 6, 28, "Input",
   CellTags->"DeltaFunction"],
  Cell[398427, 16088, 109, 3, 27, "Output",
   CellTags->"DeltaFunction"]},
 "DeltaFunctionDoublePrime"->{
  Cell[398563, 16095, 85, 1, 41, "ObjectName",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[398673, 16100, 75, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[398751, 16103, 248, 8, 25, "Text",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[399002, 16113, 219, 8, 25, "Text",
   CellTags->"DeltaFunctionDoublePrime"]},
 "DeltaFunctionPrime"->{
  Cell[399236, 16124, 73, 1, 41, "ObjectName",
   CellTags->"DeltaFunctionPrime"],
  Cell[399334, 16129, 69, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[399406, 16132, 227, 7, 25, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[399636, 16141, 576, 23, 25, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[400249, 16169, 66, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[400340, 16174, 152, 4, 28, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[400495, 16180, 240, 8, 27, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[400772, 16193, 330, 10, 28, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[401105, 16205, 203, 6, 27, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[401345, 16216, 326, 10, 28, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[401674, 16228, 114, 3, 27, "Output",
   CellTags->"DeltaFunctionPrime"]},
 "DenominatorOrder"->{
  Cell[401815, 16235, 69, 1, 41, "ObjectName",
   CellTags->"DenominatorOrder"],
  Cell[401909, 16240, 67, 1, 36, "Subsubsection",
   CellTags->"DenominatorOrder"],
  Cell[401979, 16243, 175, 4, 25, "Text",
   CellTags->"DenominatorOrder"],
  Cell[402157, 16249, 327, 13, 25, "Text",
   CellTags->"DenominatorOrder"]},
 "Dimension"->{
  Cell[402499, 16265, 55, 1, 41, "ObjectName",
   CellTags->"Dimension"],
  Cell[402579, 16270, 60, 1, 36, "Subsubsection",
   CellTags->"Dimension"],
  Cell[402642, 16273, 246, 5, 43, "Text",
   CellTags->"Dimension"],
  Cell[402913, 16282, 120, 3, 28, "Input",
   CellTags->"Dimension"],
  Cell[403036, 16287, 243, 8, 27, "Output",
   CellTags->"Dimension"],
  Cell[403316, 16300, 313, 9, 28, "Input",
   CellTags->"Dimension"],
  Cell[403632, 16311, 286, 11, 27, "Output",
   CellTags->"Dimension"],
  Cell[403933, 16325, 131, 4, 25, "Text",
   CellTags->"Dimension"],
  Cell[404089, 16333, 357, 11, 28, "Input",
   CellTags->"Dimension"],
  Cell[404449, 16346, 437, 13, 42, "Output",
   CellTags->"Dimension"],
  Cell[404901, 16362, 166, 4, 25, "Text",
   CellTags->"Dimension"],
  Cell[405070, 16368, 133, 4, 28, "Input",
   CellTags->"Dimension"],
  Cell[405228, 16376, 326, 9, 28, "Input",
   CellTags->"Dimension"],
  Cell[405557, 16387, 662, 28, 28, "Output",
   CellTags->"Dimension"],
  Cell[406256, 16420, 110, 3, 28, "Input",
   CellTags->"Dimension"],
  Cell[406369, 16425, 450, 13, 42, "Output",
   CellTags->"Dimension"],
  Cell[406834, 16441, 134, 4, 28, "Input",
   CellTags->"Dimension"]},
 "DimensionalReduction"->{
  Cell[406983, 16448, 77, 1, 57, "ObjectName",
   CellTags->"DimensionalReduction"],
  Cell[407085, 16453, 71, 1, 52, "Subsubsection",
   CellTags->"DimensionalReduction"],
  Cell[407159, 16456, 121, 3, 41, "Text",
   CellTags->"DimensionalReduction"],
  Cell[407283, 16461, 301, 13, 41, "Text",
   CellTags->"DimensionalReduction"]},
 "DiracBasis"->{
  Cell[407599, 16477, 57, 1, 57, "ObjectName",
   CellTags->"DiracBasis"],
  Cell[407681, 16482, 61, 1, 52, "Subsubsection",
   CellTags->"DiracBasis"],
  Cell[407745, 16485, 299, 6, 59, "Text",
   CellTags->"DiracBasis"],
  Cell[408047, 16493, 184, 8, 41, "Text",
   CellTags->"DiracBasis"]},
 "DiracCanonical"->{
  Cell[408246, 16504, 65, 1, 57, "ObjectName",
   CellTags->"DiracCanonical"],
  Cell[408336, 16509, 65, 1, 52, "Subsubsection",
   CellTags->"DiracCanonical"],
  Cell[408404, 16512, 171, 4, 41, "Text",
   CellTags->"DiracCanonical"],
  Cell[408578, 16518, 194, 8, 41, "Text",
   CellTags->"DiracCanonical"]},
 "DiracEquation"->{
  Cell[408787, 16529, 63, 1, 57, "ObjectName",
   CellTags->"DiracEquation"],
  Cell[408875, 16534, 64, 1, 52, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[408942, 16537, 160, 4, 41, "Text",
   CellTags->"DiracEquation"],
  Cell[409105, 16543, 199, 9, 41, "Text",
   CellTags->"DiracEquation"],
  Cell[409341, 16557, 61, 1, 36, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[409427, 16562, 279, 8, 28, "Input",
   CellTags->"DiracEquation"],
  Cell[409709, 16572, 368, 13, 27, "Output",
   CellTags->"DiracEquation"],
  Cell[410114, 16590, 119, 3, 28, "Input",
   CellTags->"DiracEquation"],
  Cell[410236, 16595, 217, 7, 27, "Output",
   CellTags->"DiracEquation"]},
 "DiracGamma"->{
  Cell[410480, 16606, 57, 1, 57, "ObjectName",
   CellTags->"DiracGamma"],
  Cell[410562, 16611, 61, 1, 52, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[410626, 16614, 766, 25, 59, "Text",
   CellTags->"DiracGamma"],
  Cell[411395, 16641, 681, 29, 41, "Text",
   CellTags->"DiracGamma"],
  Cell[412113, 16675, 58, 1, 36, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[412196, 16680, 113, 3, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[412312, 16685, 137, 4, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[412486, 16694, 159, 4, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[412648, 16700, 214, 8, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[412877, 16711, 332, 13, 25, "Text",
   CellTags->"DiracGamma"],
  Cell[413234, 16728, 167, 5, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[413404, 16735, 223, 8, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[413664, 16748, 280, 8, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[413947, 16758, 468, 17, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[414452, 16780, 218, 7, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[414673, 16789, 223, 8, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[414933, 16802, 208, 7, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[415144, 16811, 341, 12, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[415522, 16828, 144, 4, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[415669, 16834, 593, 22, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[416299, 16861, 300, 9, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[416602, 16872, 430, 13, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[417069, 16890, 138, 4, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[417210, 16896, 430, 13, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[417677, 16914, 141, 4, 28, "Input",
   CellTags->"DiracGamma"],
  Cell[417821, 16920, 1123, 42, 27, "Output",
   CellTags->"DiracGamma"],
  Cell[418959, 16965, 168, 5, 28, "Input",
   CellTags->"DiracGamma"]},
 "DiracGammaCombine"->{
  Cell[419142, 16973, 71, 1, 57, "ObjectName",
   CellTags->"DiracGammaCombine"],
  Cell[419238, 16978, 68, 1, 52, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[419309, 16981, 135, 4, 41, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[419447, 16987, 804, 34, 41, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[420288, 17026, 65, 1, 36, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[420378, 17031, 220, 6, 28, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[420601, 17039, 265, 9, 27, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[420903, 17053, 122, 3, 28, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[421028, 17058, 194, 5, 42, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[421259, 17068, 270, 8, 28, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[421532, 17078, 317, 11, 27, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[421886, 17094, 122, 3, 28, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[422011, 17099, 244, 7, 42, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[422292, 17111, 127, 3, 28, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[422422, 17116, 438, 16, 27, "Output",
   CellTags->"DiracGammaCombine"]},
 "DiracGammaExpand"->{
  Cell[422887, 17136, 69, 1, 57, "ObjectName",
   CellTags->"DiracGammaExpand"],
  Cell[422981, 17141, 67, 1, 52, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[423051, 17144, 190, 4, 41, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[423244, 17150, 806, 34, 41, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[424087, 17189, 64, 1, 36, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[424176, 17194, 314, 9, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[424493, 17205, 474, 17, 27, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[424982, 17225, 125, 3, 25, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[425132, 17232, 121, 3, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[425256, 17237, 291, 8, 42, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[425562, 17248, 101, 1, 25, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[425688, 17253, 125, 3, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[425816, 17258, 599, 22, 27, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[426430, 17283, 90, 1, 25, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[426545, 17288, 209, 5, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[426757, 17295, 291, 8, 42, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[427085, 17308, 164, 4, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[427252, 17314, 394, 11, 42, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[427661, 17328, 106, 1, 25, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[427792, 17333, 122, 3, 28, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[427917, 17338, 534, 20, 27, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[428466, 17361, 114, 3, 28, "Input",
   CellTags->"DiracGammaExpand"]},
 "DiracGammaT"->{
  Cell[428595, 17367, 59, 1, 57, "ObjectName",
   CellTags->"DiracGammaT"],
  Cell[428679, 17372, 62, 1, 52, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[428744, 17375, 208, 5, 59, "Text",
   CellTags->"DiracGammaT"],
  Cell[428955, 17382, 188, 9, 41, "Text",
   CellTags->"DiracGammaT"],
  Cell[429180, 17396, 59, 1, 36, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[429264, 17401, 158, 4, 28, "Input",
   CellTags->"DiracGammaT"],
  Cell[429425, 17407, 220, 8, 27, "Output",
   CellTags->"DiracGammaT"],
  Cell[429682, 17420, 113, 3, 28, "Input",
   CellTags->"DiracGammaT"],
  Cell[429798, 17425, 212, 8, 27, "Output",
   CellTags->"DiracGammaT"],
  Cell[430047, 17438, 167, 6, 28, "Input",
   CellTags->"DiracGammaT"],
  Cell[430217, 17446, 219, 7, 29, "Output",
   CellTags->"DiracGammaT"]},
 "DiracMatrix"->{
  Cell[430463, 17457, 59, 1, 57, "ObjectName",
   CellTags->"DiracMatrix"],
  Cell[430547, 17462, 62, 1, 52, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[430612, 17465, 1220, 46, 59, "Text",
   CellTags->"DiracMatrix"],
  Cell[431835, 17513, 1053, 49, 41, "Text",
   CellTags->"DiracMatrix"],
  Cell[432925, 17567, 59, 1, 36, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[433009, 17572, 119, 3, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[433131, 17577, 212, 8, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[433358, 17588, 539, 20, 43, "Text",
   CellTags->"DiracMatrix"],
  Cell[433922, 17612, 184, 5, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[434109, 17619, 350, 14, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[434496, 17638, 160, 5, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[434659, 17645, 175, 4, 42, "Output",
   CellTags->"DiracMatrix"],
  Cell[434871, 17654, 145, 4, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[435019, 17660, 142, 4, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[435198, 17669, 112, 3, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[435313, 17674, 125, 3, 42, "Output",
   CellTags->"DiracMatrix"],
  Cell[435475, 17682, 111, 3, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[435589, 17687, 142, 4, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[435768, 17696, 138, 4, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[435909, 17702, 221, 6, 42, "Output",
   CellTags->"DiracMatrix"],
  Cell[436167, 17713, 152, 4, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[436322, 17719, 249, 7, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[436608, 17731, 166, 5, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[436777, 17738, 342, 14, 27, "Output",
   CellTags->"DiracMatrix"],
  Cell[437156, 17757, 112, 3, 28, "Input",
   CellTags->"DiracMatrix"],
  Cell[437271, 17762, 269, 7, 42, "Output",
   CellTags->"DiracMatrix"]},
 "DiracOrder"->{
  Cell[437567, 17773, 57, 1, 57, "ObjectName",
   CellTags->"DiracOrder"],
  Cell[437649, 17778, 61, 1, 52, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[437713, 17781, 204, 5, 41, "Text",
   CellTags->"DiracOrder"],
  Cell[437920, 17788, 312, 14, 41, "Text",
   CellTags->"DiracOrder"],
  Cell[438269, 17807, 58, 1, 36, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[438352, 17812, 176, 5, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[438531, 17819, 206, 6, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[438774, 17830, 114, 3, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[438891, 17835, 472, 18, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[439378, 17856, 93, 1, 25, "Text",
   CellTags->"DiracOrder"],
  Cell[439496, 17861, 176, 5, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[439675, 17868, 247, 7, 27, "Output",
   CellTags->"DiracOrder"],
  Cell[439959, 17880, 114, 3, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[440076, 17885, 472, 18, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[440585, 17908, 171, 5, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[440759, 17915, 243, 7, 27, "Output",
   CellTags->"DiracOrder"],
  Cell[441017, 17925, 196, 8, 25, "Text",
   CellTags->"DiracOrder"],
  Cell[441238, 17937, 114, 3, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[441355, 17942, 391, 15, 27, "Output",
   CellTags->"DiracOrder"],
  Cell[441783, 17962, 167, 5, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[441953, 17969, 239, 7, 27, "Output",
   CellTags->"DiracOrder"],
  Cell[442229, 17981, 114, 3, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[442346, 17986, 271, 10, 27, "Output",
   CellTags->"DiracOrder"],
  Cell[442654, 18001, 183, 5, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[442840, 18008, 255, 7, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[443110, 18018, 288, 10, 25, "Text",
   CellTags->"DiracOrder"],
  Cell[443423, 18032, 219, 7, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[443645, 18041, 1207, 45, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[444889, 18091, 113, 3, 28, "Input",
   CellTags->"DiracOrder"],
  Cell[445005, 18096, 477, 19, 28, "Output",
   CellTags->"DiracOrder"],
  Cell[445497, 18118, 188, 6, 28, "Input",
   CellTags->"DiracOrder"]},
 "DiracReduce"->{
  Cell[445700, 18127, 59, 1, 57, "ObjectName",
   CellTags->"DiracReduce"],
  Cell[445784, 18132, 62, 1, 52, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[445849, 18135, 662, 11, 97, "Text",
   CellTags->"DiracReduce"],
  Cell[446536, 18150, 121, 3, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[446660, 18155, 323, 10, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[446998, 18168, 197, 9, 41, "Text",
   CellTags->"DiracReduce"],
  Cell[447232, 18182, 59, 1, 36, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[447316, 18187, 172, 5, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[447491, 18194, 202, 6, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[447730, 18205, 116, 3, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[447849, 18210, 281, 9, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[448167, 18224, 186, 5, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[448356, 18231, 239, 7, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[448632, 18243, 116, 3, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[448751, 18248, 919, 28, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[449707, 18281, 207, 6, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[449917, 18289, 283, 8, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[450237, 18302, 140, 4, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[450380, 18308, 1811, 54, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[452228, 18367, 253, 7, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[452484, 18376, 127, 4, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[452648, 18385, 133, 4, 28, "Input",
   CellTags->"DiracReduce"],
  Cell[452784, 18391, 127, 4, 27, "Output",
   CellTags->"DiracReduce"],
  Cell[452926, 18398, 156, 4, 28, "Input",
   CellTags->"DiracReduce"]},
 "DiracSigma"->{
  Cell[453097, 18405, 57, 1, 57, "ObjectName",
   CellTags->"DiracSigma"],
  Cell[453179, 18410, 61, 1, 52, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[453243, 18413, 213, 5, 41, "Text",
   CellTags->"DiracSigma"],
  Cell[453459, 18420, 211, 9, 41, "Text",
   CellTags->"DiracSigma"],
  Cell[453707, 18434, 58, 1, 36, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[453790, 18439, 236, 7, 28, "Input",
   CellTags->"DiracSigma"],
  Cell[454029, 18448, 169, 5, 28, "Output",
   CellTags->"DiracSigma"],
  Cell[454235, 18458, 122, 3, 28, "Input",
   CellTags->"DiracSigma"],
  Cell[454360, 18463, 784, 29, 38, "Output",
   CellTags->"DiracSigma"],
  Cell[455181, 18497, 236, 7, 28, "Input",
   CellTags->"DiracSigma"],
  Cell[455420, 18506, 189, 6, 28, "Output",
   CellTags->"DiracSigma"],
  Cell[455646, 18517, 223, 7, 28, "Input",
   CellTags->"DiracSigma"],
  Cell[455872, 18526, 156, 5, 27, "Output",
   CellTags->"DiracSigma"],
  Cell[456065, 18536, 122, 3, 28, "Input",
   CellTags->"DiracSigma"],
  Cell[456190, 18541, 950, 33, 38, "Output",
   CellTags->"DiracSigma"],
  Cell[457155, 18577, 144, 4, 28, "Input",
   CellTags->"DiracSigma"]},
 "DiracSigmaExplicit"->{
  Cell[457314, 18584, 73, 1, 57, "ObjectName",
   CellTags->"DiracSigmaExplicit"],
  Cell[457412, 18589, 69, 1, 52, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[457484, 18592, 186, 4, 25, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[457673, 18598, 195, 9, 41, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[457905, 18612, 66, 1, 36, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[457996, 18617, 217, 6, 28, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[458216, 18625, 177, 5, 28, "Output",
   CellTags->"DiracSigmaExplicit"],
  Cell[458430, 18635, 129, 3, 28, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[458562, 18640, 792, 29, 38, "Output",
   CellTags->"DiracSigmaExplicit"]},
 "DiracSimpCombine"->{
  Cell[459381, 18673, 69, 1, 41, "ObjectName",
   CellTags->"DiracSimpCombine"],
  Cell[459475, 18678, 67, 1, 36, "Subsubsection",
   CellTags->"DiracSimpCombine"],
  Cell[459545, 18681, 209, 5, 25, "Text",
   CellTags->"DiracSimpCombine"],
  Cell[459757, 18688, 196, 8, 25, "Text",
   CellTags->"DiracSimpCombine"]},
 "DiracSimplify"->{
  Cell[459968, 18699, 63, 1, 41, "ObjectName",
   CellTags->"DiracSimplify"],
  Cell[460056, 18704, 64, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[460123, 18707, 382, 7, 61, "Text",
   CellTags->"DiracSimplify"],
  Cell[460530, 18718, 125, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[460658, 18723, 604, 14, 49, "Output",
   CellTags->"DiracSimplify"],
  Cell[461277, 18740, 422, 19, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[461736, 18764, 61, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[461800, 18767, 99, 1, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[461924, 18772, 178, 5, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[462105, 18779, 250, 7, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[462392, 18791, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[462515, 18796, 261, 10, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[462791, 18809, 96, 1, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[462912, 18814, 179, 5, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[463094, 18821, 250, 7, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[463381, 18833, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[463504, 18838, 261, 10, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[463802, 18853, 174, 5, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[463979, 18860, 246, 7, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[464240, 18870, 179, 7, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[464444, 18881, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[464567, 18886, 394, 15, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[464998, 18906, 170, 5, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[465171, 18913, 242, 7, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[465450, 18925, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[465573, 18930, 274, 10, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[465884, 18945, 326, 10, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[466213, 18957, 570, 18, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[466798, 18978, 120, 3, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[466943, 18985, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[467066, 18990, 1908, 72, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[469011, 19067, 117, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[469131, 19072, 570, 18, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[469738, 19095, 272, 7, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[470013, 19104, 407, 14, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[470457, 19123, 120, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[470580, 19128, 109, 3, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[470726, 19136, 373, 12, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[471102, 19150, 624, 18, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[471763, 19173, 119, 3, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[471885, 19178, 13907, 530, 93, "Output",
   CellTags->"DiracSimplify"],
  Cell[485807, 19711, 115, 3, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[485947, 19718, 251, 7, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[486201, 19727, 465, 18, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[486681, 19748, 141, 4, 25, "Text",
   CellTags->"DiracSimplify"],
  Cell[486847, 19756, 289, 8, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[487139, 19766, 556, 19, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[487732, 19790, 268, 7, 28, "Input",
   CellTags->"DiracSimplify"],
  Cell[488003, 19799, 109, 3, 27, "Output",
   CellTags->"DiracSimplify"],
  Cell[488127, 19805, 183, 5, 28, "Input",
   CellTags->"DiracSimplify"]},
 "DiracSimplify2"->{
  Cell[488325, 19813, 65, 1, 41, "ObjectName",
   CellTags->"DiracSimplify2"],
  Cell[488415, 19818, 65, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[488483, 19821, 224, 7, 25, "Text",
   CellTags->"DiracSimplify2"],
  Cell[488710, 19830, 200, 9, 25, "Text",
   CellTags->"DiracSimplify2"],
  Cell[488947, 19844, 62, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[489034, 19849, 222, 6, 28, "Input",
   CellTags->"DiracSimplify2"],
  Cell[489259, 19857, 439, 11, 28, "Output",
   CellTags->"DiracSimplify2"],
  Cell[489735, 19873, 121, 3, 28, "Input",
   CellTags->"DiracSimplify2"],
  Cell[489859, 19878, 883, 35, 28, "Output",
   CellTags->"DiracSimplify2"]},
 "DiracSlash"->{
  Cell[490769, 19917, 57, 1, 41, "ObjectName",
   CellTags->"DiracSlash"],
  Cell[490851, 19922, 61, 1, 36, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[490915, 19925, 469, 16, 43, "Text",
   CellTags->"DiracSlash"],
  Cell[491409, 19945, 119, 3, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[491531, 19950, 244, 8, 27, "Output",
   CellTags->"DiracSlash"],
  Cell[491790, 19961, 868, 39, 25, "Text",
   CellTags->"DiracSlash"],
  Cell[492695, 20005, 58, 1, 36, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[492756, 20008, 234, 9, 25, "Text",
   CellTags->"DiracSlash"],
  Cell[493015, 20021, 113, 3, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[493131, 20026, 223, 8, 27, "Output",
   CellTags->"DiracSlash"],
  Cell[493391, 20039, 173, 5, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[493567, 20046, 431, 16, 27, "Output",
   CellTags->"DiracSlash"],
  Cell[494035, 20067, 136, 4, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[494174, 20073, 431, 16, 27, "Output",
   CellTags->"DiracSlash"],
  Cell[494642, 20094, 105, 3, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[494750, 20099, 147, 4, 27, "Output",
   CellTags->"DiracSlash"],
  Cell[494934, 20108, 148, 4, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[495085, 20114, 163, 4, 42, "Output",
   CellTags->"DiracSlash"],
  Cell[495285, 20123, 214, 7, 28, "Input",
   CellTags->"DiracSlash"],
  Cell[495502, 20132, 212, 6, 42, "Output",
   CellTags->"DiracSlash"]},
 "DiracSpinor"->{
  Cell[495741, 20142, 59, 1, 41, "ObjectName",
   CellTags->"DiracSpinor"],
  Cell[495825, 20147, 62, 1, 36, "Subsubsection",
   CellTags->"DiracSpinor"],
  Cell[495890, 20150, 288, 6, 43, "Text",
   CellTags->"DiracSpinor"],
  Cell[496181, 20158, 170, 8, 41, "Text",
   CellTags->"DiracSpinor"]},
 "DiracSubstitute67"->{
  Cell[496366, 20169, 71, 1, 57, "ObjectName",
   CellTags->"DiracSubstitute67"],
  Cell[496462, 20174, 68, 1, 52, "Subsubsection",
   CellTags->"DiracSubstitute67"],
  Cell[496533, 20177, 219, 5, 41, "Text",
   CellTags->"DiracSubstitute67"],
  Cell[496755, 20184, 313, 13, 41, "Text",
   CellTags->"DiracSubstitute67"]},
 "DiracTrace"->{
  Cell[497083, 20200, 57, 1, 57, "ObjectName",
   CellTags->"DiracTrace"],
  Cell[497165, 20205, 61, 1, 52, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[497229, 20208, 336, 6, 61, "Text",
   CellTags->"DiracTrace"],
  Cell[497590, 20218, 119, 3, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[497712, 20223, 664, 16, 49, "Output",
   CellTags->"DiracTrace"],
  Cell[498391, 20242, 283, 12, 41, "Text",
   CellTags->"DiracTrace"],
  Cell[498677, 20256, 163, 9, 41, "Text",
   CellTags->"DiracTrace"],
  Cell[498877, 20270, 58, 1, 36, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[498960, 20275, 174, 5, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[499137, 20282, 270, 9, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[499444, 20296, 209, 6, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[499656, 20304, 370, 11, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[500063, 20320, 149, 4, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[500215, 20326, 635, 22, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[500887, 20353, 296, 8, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[501186, 20363, 566, 23, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[501789, 20391, 334, 9, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[502126, 20402, 8726, 322, 52, "Output",
   CellTags->"DiracTrace"],
  Cell[510889, 20729, 186, 5, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[511078, 20736, 498, 15, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[511613, 20756, 294, 8, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[511910, 20766, 182, 6, 27, "Output",
   CellTags->"DiracTrace"],
  Cell[512129, 20777, 111, 3, 28, "Input",
   CellTags->"DiracTrace"],
  Cell[512243, 20782, 175, 5, 42, "Output",
   CellTags->"DiracTrace"]},
 "DiracTraceEvaluate"->{
  Cell[512445, 20791, 73, 1, 57, "ObjectName",
   CellTags->"DiracTraceEvaluate"],
  Cell[512543, 20796, 69, 1, 52, "Subsubsection",
   CellTags->"DiracTraceEvaluate"],
  Cell[512615, 20799, 162, 4, 41, "Text",
   CellTags->"DiracTraceEvaluate"],
  Cell[512780, 20805, 281, 13, 41, "Text",
   CellTags->"DiracTraceEvaluate"]},
 "DiracTrick"->{
  Cell[513076, 20821, 57, 1, 57, "ObjectName",
   CellTags->"DiracTrick"],
  Cell[513158, 20826, 61, 1, 52, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[513222, 20829, 222, 5, 43, "Text",
   CellTags->"DiracTrick"],
  Cell[513469, 20838, 119, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[513591, 20843, 174, 6, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[513780, 20852, 428, 19, 70, "Text",
   CellTags->"DiracTrick"],
  Cell[514245, 20876, 58, 1, 70, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[514306, 20879, 96, 1, 25, "Text",
   CellTags->"DiracTrick"],
  Cell[514427, 20884, 175, 5, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[514605, 20891, 247, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[514889, 20903, 114, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[515006, 20908, 247, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[515268, 20918, 93, 1, 25, "Text",
   CellTags->"DiracTrick"],
  Cell[515386, 20923, 176, 5, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[515565, 20930, 247, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[515849, 20942, 114, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[515966, 20947, 247, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[516250, 20959, 171, 5, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[516424, 20966, 243, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[516682, 20976, 176, 7, 25, "Text",
   CellTags->"DiracTrick"],
  Cell[516883, 20987, 114, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[517000, 20992, 243, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[517280, 21004, 167, 5, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[517450, 21011, 239, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[517726, 21023, 114, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[517843, 21028, 239, 7, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[518119, 21040, 323, 10, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[518445, 21052, 567, 18, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[519049, 21075, 114, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[519166, 21080, 567, 18, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[519770, 21103, 108, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[519881, 21108, 1905, 72, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[521823, 21185, 370, 12, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[522196, 21199, 621, 18, 27, "Output",
   CellTags->"DiracTrick"],
  Cell[522854, 21222, 113, 3, 28, "Input",
   CellTags->"DiracTrick"],
  Cell[522970, 21227, 621, 18, 27, "Output",
   CellTags->"DiracTrick"]},
 "Divideout"->{
  Cell[523618, 21249, 55, 1, 57, "ObjectName",
   CellTags->"Divideout"],
  Cell[523698, 21254, 60, 1, 52, "Subsubsection",
   CellTags->"Divideout"],
  Cell[523761, 21257, 137, 4, 41, "Text",
   CellTags->"Divideout"],
  Cell[523901, 21263, 281, 13, 41, "Text",
   CellTags->"Divideout"]},
 "DOT"->{
  Cell[524197, 21279, 43, 1, 57, "ObjectName",
   CellTags->"DOT"],
  Cell[524265, 21284, 54, 1, 52, "Subsubsection",
   CellTags->"DOT"],
  Cell[524322, 21287, 348, 8, 93, "Text",
   CellTags->"DOT"],
  Cell[524673, 21297, 177, 8, 41, "Text",
   CellTags->"DOT"]},
 "DotExpand"->{
  Cell[524865, 21308, 55, 1, 57, "ObjectName",
   CellTags->"DotExpand"],
  Cell[524945, 21313, 60, 1, 52, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[525008, 21316, 85, 1, 41, "Text",
   CellTags->"DotExpand"],
  Cell[525096, 21319, 582, 23, 41, "Text",
   CellTags->"DotExpand"],
  Cell[525715, 21347, 57, 1, 36, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[525797, 21352, 313, 10, 28, "Input",
   CellTags->"DotExpand"],
  Cell[526113, 21364, 504, 13, 27, "Output",
   CellTags->"DotExpand"],
  Cell[526632, 21380, 229, 7, 28, "Input",
   CellTags->"DotExpand"],
  Cell[526886, 21391, 313, 10, 28, "Input",
   CellTags->"DotExpand"],
  Cell[527202, 21403, 639, 22, 27, "Output",
   CellTags->"DotExpand"],
  Cell[527856, 21428, 231, 7, 28, "Input",
   CellTags->"DotExpand"],
  Cell[528112, 21439, 313, 10, 28, "Input",
   CellTags->"DotExpand"],
  Cell[528428, 21451, 504, 13, 27, "Output",
   CellTags->"DotExpand"]},
 "DotPower"->{
  Cell[528959, 21468, 53, 1, 57, "ObjectName",
   CellTags->"DotPower"],
  Cell[529037, 21473, 59, 1, 52, "Subsubsection",
   CellTags->"DotPower"],
  Cell[529099, 21476, 189, 4, 41, "Text",
   CellTags->"DotPower"],
  Cell[529291, 21482, 182, 8, 70, "Text",
   CellTags->"DotPower"]},
 "DotProduct"->{
  Cell[529488, 21493, 57, 1, 70, "ObjectName",
   CellTags->"DotProduct"],
  Cell[529570, 21498, 61, 1, 70, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[529634, 21501, 219, 5, 70, "Text",
   CellTags->"DotProduct"],
  Cell[529856, 21508, 312, 14, 41, "Text",
   CellTags->"DotProduct"],
  Cell[530205, 21527, 58, 1, 36, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[530288, 21532, 234, 7, 28, "Input",
   CellTags->"DotProduct"],
  Cell[530525, 21541, 314, 11, 30, "Output",
   CellTags->"DotProduct"]},
 "DotSimplifyRelations"->{
  Cell[530866, 21556, 77, 1, 57, "ObjectName",
   CellTags->"DotSimplifyRelations"],
  Cell[530968, 21561, 71, 1, 52, "Subsubsection",
   CellTags->"DotSimplifyRelations"],
  Cell[531042, 21564, 241, 5, 41, "Text",
   CellTags->"DotSimplifyRelations"],
  Cell[531286, 21571, 194, 8, 41, "Text",
   CellTags->"DotSimplifyRelations"]},
 "DotSimplify"->{
  Cell[531495, 21582, 59, 1, 57, "ObjectName",
   CellTags->"DotSimplify"],
  Cell[531579, 21587, 62, 1, 52, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[531644, 21590, 322, 6, 61, "Text",
   CellTags->"DotSimplify"],
  Cell[531991, 21600, 121, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[532115, 21605, 381, 11, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[532511, 21619, 414, 19, 41, "Text",
   CellTags->"DotSimplify"],
  Cell[532962, 21643, 59, 1, 36, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[533046, 21648, 342, 11, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[533391, 21661, 376, 11, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[533804, 21677, 116, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[533923, 21682, 1013, 39, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[534951, 21724, 158, 4, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[535134, 21732, 209, 7, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[535346, 21741, 217, 8, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[535600, 21754, 116, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[535719, 21759, 223, 7, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[535979, 21771, 161, 5, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[536143, 21778, 107, 3, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[536287, 21786, 116, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[536406, 21791, 279, 10, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[536700, 21804, 161, 5, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[536886, 21813, 116, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[537005, 21818, 223, 7, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[537265, 21830, 165, 5, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[537433, 21837, 107, 3, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[537577, 21845, 116, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[537696, 21850, 257, 8, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[537968, 21861, 165, 5, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[538158, 21870, 299, 9, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[538460, 21881, 166, 6, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[538641, 21890, 125, 3, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[538791, 21897, 148, 4, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[538942, 21903, 140, 4, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[539119, 21912, 213, 6, 28, "Input",
   CellTags->"DotSimplify"],
  Cell[539335, 21920, 140, 4, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[539490, 21927, 170, 4, 28, "Input",
   CellTags->"DotSimplify"]},
 "DummyIndex"->{
  Cell[539675, 21934, 57, 1, 57, "ObjectName",
   CellTags->"DummyIndex"],
  Cell[539757, 21939, 61, 1, 52, "Subsubsection",
   CellTags->"DummyIndex"],
  Cell[539821, 21942, 191, 4, 41, "Text",
   CellTags->"DummyIndex"],
  Cell[540015, 21948, 181, 8, 41, "Text",
   CellTags->"DummyIndex"]},
 "D0"->{
  Cell[540211, 21959, 41, 1, 57, "ObjectName",
   CellTags->"D0"],
  Cell[540277, 21964, 53, 1, 52, "Subsubsection",
   CellTags->"D0"],
  Cell[540333, 21967, 524, 12, 77, "Text",
   CellTags->"D0"],
  Cell[540860, 21981, 458, 24, 41, "Text",
   CellTags->"D0"],
  Cell[541355, 22010, 50, 1, 36, "Subsubsection",
   CellTags->"D0"],
  Cell[541430, 22015, 315, 9, 28, "Input",
   CellTags->"D0"],
  Cell[541748, 22026, 550, 20, 27, "Output",
   CellTags->"D0"],
  Cell[542335, 22051, 484, 14, 28, "Input",
   CellTags->"D0"],
  Cell[542822, 22067, 550, 20, 27, "Output",
   CellTags->"D0"],
  Cell[543409, 22092, 104, 3, 28, "Input",
   CellTags->"D0"],
  Cell[543516, 22097, 550, 20, 27, "Output",
   CellTags->"D0"]},
 "D0Convention"->{
  Cell[544093, 22121, 61, 1, 57, "ObjectName",
   CellTags->"D0Convention"],
  Cell[544179, 22126, 63, 1, 52, "Subsubsection",
   CellTags->"D0Convention"],
  Cell[544245, 22129, 307, 6, 59, "Text",
   CellTags->"D0Convention"],
  Cell[544555, 22137, 263, 13, 41, "Text",
   CellTags->"D0Convention"]},
 "Eps"->{
  Cell[544833, 22153, 43, 1, 57, "ObjectName",
   CellTags->"Eps"],
  Cell[544901, 22158, 54, 1, 52, "Subsubsection",
   CellTags->"Eps"],
  Cell[544958, 22161, 347, 9, 43, "Text",
   CellTags->"Eps"],
  Cell[545330, 22174, 105, 3, 28, "Input",
   CellTags->"Eps"],
  Cell[545438, 22179, 160, 5, 27, "Output",
   CellTags->"Eps"],
  Cell[545613, 22187, 486, 24, 41, "Text",
   CellTags->"Eps"],
  Cell[546136, 22216, 51, 1, 36, "Subsubsection",
   CellTags->"Eps"],
  Cell[546212, 22221, 322, 8, 28, "Input",
   CellTags->"Eps"],
  Cell[546537, 22231, 477, 21, 27, "Output",
   CellTags->"Eps"],
  Cell[547051, 22257, 314, 8, 28, "Input",
   CellTags->"Eps"],
  Cell[547368, 22267, 435, 19, 27, "Output",
   CellTags->"Eps"],
  Cell[547840, 22291, 181, 6, 28, "Input",
   CellTags->"Eps"],
  Cell[548024, 22299, 176, 5, 42, "Output",
   CellTags->"Eps"],
  Cell[548237, 22309, 142, 4, 28, "Input",
   CellTags->"Eps"],
  Cell[548382, 22315, 99, 3, 27, "Output",
   CellTags->"Eps"],
  Cell[548518, 22323, 142, 4, 28, "Input",
   CellTags->"Eps"],
  Cell[548663, 22329, 117, 4, 27, "Output",
   CellTags->"Eps"],
  Cell[548795, 22336, 189, 6, 28, "Input",
   CellTags->"Eps"],
  Cell[549009, 22346, 460, 13, 28, "Input",
   CellTags->"Eps"],
  Cell[549472, 22361, 477, 21, 27, "Output",
   CellTags->"Eps"],
  Cell[549986, 22387, 107, 3, 28, "Input",
   CellTags->"Eps"],
  Cell[550096, 22392, 102, 3, 27, "Output",
   CellTags->"Eps"],
  Cell[550213, 22398, 189, 6, 28, "Input",
   CellTags->"Eps"],
  Cell[550427, 22408, 460, 13, 28, "Input",
   CellTags->"Eps"],
  Cell[550890, 22423, 477, 21, 27, "Output",
   CellTags->"Eps"],
  Cell[551404, 22449, 137, 4, 28, "Input",
   CellTags->"Eps"],
  Cell[551544, 22455, 102, 3, 27, "Output",
   CellTags->"Eps"],
  Cell[553473, 22538, 433, 14, 40, "Input",
   CellTags->"Eps"],
  Cell[553909, 22554, 1119, 45, 38, "Output",
   CellTags->"Eps"],
  Cell[555065, 22604, 978, 28, 40, "Input",
   CellTags->"Eps"],
  Cell[556046, 22634, 1679, 61, 38, "Output",
   CellTags->"Eps"],
  Cell[557762, 22700, 188, 6, 28, "Input",
   CellTags->"Eps"],
  Cell[557953, 22708, 342, 12, 38, "Output",
   CellTags->"Eps"],
  Cell[558332, 22725, 124, 4, 28, "Input",
   CellTags->"Eps"],
  Cell[558459, 22731, 99, 3, 27, "Output",
   CellTags->"Eps"],
  Cell[558573, 22737, 164, 5, 28, "Input",
   CellTags->"Eps"]},
 "EpsChisholm"->{
  Cell[558752, 22745, 59, 1, 57, "ObjectName",
   CellTags->"EpsChisholm"],
  Cell[558836, 22750, 62, 1, 52, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[558901, 22753, 166, 4, 41, "Text",
   CellTags->"EpsChisholm"],
  Cell[559070, 22759, 182, 9, 41, "Text",
   CellTags->"EpsChisholm"],
  Cell[559289, 22773, 59, 1, 36, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[559373, 22778, 208, 6, 28, "Input",
   CellTags->"EpsChisholm"],
  Cell[559584, 22786, 1989, 78, 27, "Output",
   CellTags->"EpsChisholm"],
  Cell[561610, 22869, 115, 3, 28, "Input",
   CellTags->"EpsChisholm"],
  Cell[561728, 22874, 594, 24, 27, "Output",
   CellTags->"EpsChisholm"]},
 "EpsContract"->{
  Cell[562349, 22902, 59, 1, 57, "ObjectName",
   CellTags->"EpsContract"],
  Cell[562433, 22907, 62, 1, 52, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[562498, 22910, 227, 5, 59, "Text",
   CellTags->"EpsContract"],
  Cell[562728, 22917, 277, 14, 41, "Text",
   CellTags->"EpsContract"],
  Cell[563042, 22936, 59, 1, 36, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[563126, 22941, 468, 13, 28, "Input",
   CellTags->"EpsContract"],
  Cell[563597, 22956, 485, 21, 27, "Output",
   CellTags->"EpsContract"],
  Cell[564119, 22982, 205, 6, 28, "Input",
   CellTags->"EpsContract"],
  Cell[564327, 22990, 569, 23, 28, "Output",
   CellTags->"EpsContract"],
  Cell[564933, 23018, 204, 6, 28, "Input",
   CellTags->"EpsContract"],
  Cell[565140, 23026, 320, 11, 27, "Output",
   CellTags->"EpsContract"]},
 "EpsDiscard"->{
  Cell[565574, 23045, 57, 1, 57, "ObjectName",
   CellTags->"EpsDiscard"],
  Cell[565656, 23050, 61, 1, 36, "Subsubsection",
   CellTags->"EpsDiscard"],
  Cell[565720, 23053, 169, 4, 25, "Text",
   CellTags->"EpsDiscard"],
  Cell[565892, 23059, 196, 9, 25, "Text",
   CellTags->"EpsDiscard"]},
 "EpsEvaluate"->{
  Cell[566103, 23071, 59, 1, 41, "ObjectName",
   CellTags->"EpsEvaluate"],
  Cell[566187, 23076, 62, 1, 36, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[566252, 23079, 173, 4, 25, "Text",
   CellTags->"EpsEvaluate"],
  Cell[566428, 23085, 494, 24, 25, "Text",
   CellTags->"EpsEvaluate"],
  Cell[566959, 23114, 59, 1, 36, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[567043, 23119, 335, 10, 28, "Input",
   CellTags->"EpsEvaluate"],
  Cell[567381, 23131, 466, 20, 27, "Output",
   CellTags->"EpsEvaluate"],
  Cell[567884, 23156, 115, 3, 28, "Input",
   CellTags->"EpsEvaluate"],
  Cell[568002, 23161, 466, 20, 27, "Output",
   CellTags->"EpsEvaluate"],
  Cell[568505, 23186, 116, 3, 28, "Input",
   CellTags->"EpsEvaluate"],
  Cell[568624, 23191, 434, 12, 42, "Output",
   CellTags->"EpsEvaluate"]},
 "EpsUncontract"->{
  Cell[569085, 23207, 63, 1, 41, "ObjectName",
   CellTags->"EpsUncontract"],
  Cell[569173, 23212, 64, 1, 36, "Subsubsection",
   CellTags->"EpsUncontract"],
  Cell[569240, 23215, 115, 3, 25, "Text",
   CellTags->"EpsUncontract"],
  Cell[569358, 23220, 285, 14, 25, "Text",
   CellTags->"EpsUncontract"]},
 "Epsilon"->{
  Cell[569658, 23237, 51, 1, 41, "ObjectName",
   CellTags->"Epsilon"],
  Cell[569734, 23242, 58, 1, 36, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[569795, 23245, 237, 9, 25, "Text",
   CellTags->"Epsilon"],
  Cell[570035, 23256, 175, 9, 25, "Text",
   CellTags->"Epsilon"],
  Cell[570247, 23270, 55, 1, 36, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[570327, 23275, 80, 2, 28, "Input",
   CellTags->"Epsilon"],
  Cell[570410, 23279, 148, 5, 27, "Output",
   CellTags->"Epsilon"],
  Cell[570573, 23287, 114, 3, 25, "Text",
   CellTags->"Epsilon"],
  Cell[570712, 23294, 587, 19, 28, "Input",
   CellTags->"Epsilon"],
  Cell[571302, 23315, 199, 6, 27, "Output",
   CellTags->"Epsilon"]},
 "EpsilonOrder"->{
  Cell[571528, 23325, 61, 1, 41, "ObjectName",
   CellTags->"EpsilonOrder"],
  Cell[571614, 23330, 63, 1, 36, "Subsubsection",
   CellTags->"EpsilonOrder"],
  Cell[571680, 23333, 305, 12, 25, "Text",
   CellTags->"EpsilonOrder"],
  Cell[571988, 23347, 287, 14, 25, "Text",
   CellTags->"EpsilonOrder"]},
 "Expanding"->{
  Cell[572290, 23364, 55, 1, 41, "ObjectName",
   CellTags->"Expanding"],
  Cell[572370, 23369, 60, 1, 36, "Subsubsection",
   CellTags->"Expanding"],
  Cell[572433, 23372, 378, 7, 61, "Text",
   CellTags->"Expanding"],
  Cell[572814, 23381, 641, 29, 25, "Text",
   CellTags->"Expanding"]},
 "ExpandPartialD"->{
  Cell[573470, 23413, 65, 1, 41, "ObjectName",
   CellTags->"ExpandPartialD"],
  Cell[573560, 23418, 65, 1, 36, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[573628, 23421, 188, 4, 43, "Text",
   CellTags->"ExpandPartialD"],
  Cell[574998, 23472, 724, 28, 25, "Text",
   CellTags->"ExpandPartialD"],
  Cell[575759, 23505, 62, 1, 36, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[575846, 23510, 373, 10, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[576222, 23522, 345, 12, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[576604, 23539, 121, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[576728, 23544, 773, 30, 27, "Output",
   CellTags->"ExpandPartialD"],
  Cell[577538, 23579, 115, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[577656, 23584, 798, 21, 62, "Output",
   CellTags->"ExpandPartialD"],
  Cell[578491, 23610, 259, 7, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[578753, 23619, 312, 11, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[579102, 23635, 121, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[579226, 23640, 704, 26, 38, "Output",
   CellTags->"ExpandPartialD"],
  Cell[579967, 23671, 433, 13, 50, "Input",
   CellTags->"ExpandPartialD"],
  Cell[580403, 23686, 332, 10, 35, "Output",
   CellTags->"ExpandPartialD"],
  Cell[580772, 23701, 121, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[580896, 23706, 1940, 67, 38, "Output",
   CellTags->"ExpandPartialD"],
  Cell[582873, 23778, 180, 6, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[583056, 23786, 267, 8, 35, "Output",
   CellTags->"ExpandPartialD"],
  Cell[583360, 23799, 123, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[583486, 23804, 438, 14, 38, "Output",
   CellTags->"ExpandPartialD"],
  Cell[583961, 23823, 121, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[584085, 23828, 1877, 59, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[585999, 23892, 314, 9, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[586316, 23903, 1192, 47, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[587545, 23955, 121, 3, 28, "Input",
   CellTags->"ExpandPartialD"],
  Cell[587669, 23960, 110, 3, 27, "Output",
   CellTags->"ExpandPartialD"]},
 "ScalarProductCancel"->{
  Cell[587806, 23967, 121, 1, 41, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[587952, 23972, 95, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588050, 23975, 252, 5, 69, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588327, 23984, 158, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588488, 23989, 214, 6, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588717, 23998, 462, 19, 25, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589216, 24022, 92, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589333, 24027, 257, 7, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589593, 24036, 338, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589968, 24051, 152, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590123, 24056, 338, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590498, 24071, 186, 5, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590687, 24078, 277, 9, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591001, 24092, 156, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591160, 24097, 277, 9, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591474, 24111, 190, 5, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591667, 24118, 227, 7, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591931, 24130, 156, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592090, 24135, 227, 7, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592354, 24147, 239, 7, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592596, 24156, 312, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592945, 24171, 184, 4, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[593132, 24177, 565, 22, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[1705633, 69388, 75, 1, 41, "ObjectName",
   CellTags->"ScalarProductCancel"],
  Cell[1705733, 69393, 70, 1, 36, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1705806, 69396, 186, 4, 25, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1706017, 69404, 89, 2, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1706109, 69408, 134, 3, 27, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1706280, 69416, 134, 3, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1706417, 69421, 392, 10, 49, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1706846, 69436, 106, 3, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1706955, 69441, 179, 3, 40, "Print",
   CellTags->{"Info3387712142-8239904", "ScalarProductCancel"}],
  Cell[1707149, 69447, 743, 33, 25, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1707929, 69485, 67, 1, 36, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1707999, 69488, 244, 9, 25, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1708268, 69501, 429, 15, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1708700, 69518, 454, 17, 43, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1709191, 69540, 156, 4, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1709350, 69546, 874, 33, 52, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1710261, 69584, 162, 4, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1710426, 69590, 708, 26, 52, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1711171, 69621, 185, 5, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1711359, 69628, 708, 26, 52, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1712104, 69659, 578, 18, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1712685, 69679, 898, 33, 45, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1713620, 69717, 197, 5, 28, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1713820, 69724, 3108, 111, 109, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1716943, 69838, 162, 5, 28, "Input",
   CellTags->"ScalarProductCancel"]},
 "ExpandScalarProduct"->{
  Cell[587806, 23967, 121, 1, 41, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[587952, 23972, 95, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588050, 23975, 252, 5, 69, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588327, 23984, 158, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588488, 23989, 214, 6, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[588717, 23998, 462, 19, 25, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589216, 24022, 92, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589333, 24027, 257, 7, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589593, 24036, 338, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[589968, 24051, 152, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590123, 24056, 338, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590498, 24071, 186, 5, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[590687, 24078, 277, 9, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591001, 24092, 156, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591160, 24097, 277, 9, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591474, 24111, 190, 5, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591667, 24118, 227, 7, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[591931, 24130, 156, 3, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592090, 24135, 227, 7, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592354, 24147, 239, 7, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592596, 24156, 312, 10, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[592945, 24171, 184, 4, 28, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[593132, 24177, 565, 22, 27, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]},
 "Expand2"->{
  Cell[593724, 24203, 51, 1, 41, "ObjectName",
   CellTags->"Expand2"],
  Cell[593800, 24208, 58, 1, 36, "Subsubsection",
   CellTags->"Expand2"],
  Cell[593861, 24211, 164, 4, 25, "Text",
   CellTags->"Expand2"],
  Cell[594062, 24220, 55, 1, 36, "Subsubsection",
   CellTags->"Expand2"],
  Cell[594142, 24225, 579, 21, 28, "Input",
   CellTags->"Expand2"],
  Cell[594724, 24248, 574, 19, 27, "Output",
   CellTags->"Expand2"]},
 "Explicit"->{
  Cell[595325, 24271, 53, 1, 41, "ObjectName",
   CellTags->"Explicit"],
  Cell[595403, 24276, 59, 1, 36, "Subsubsection",
   CellTags->"Explicit"],
  Cell[595465, 24279, 401, 9, 61, "Text",
   CellTags->"Explicit"],
  Cell[595869, 24290, 331, 14, 25, "Text",
   CellTags->"Explicit"],
  Cell[596710, 24328, 56, 1, 36, "Subsubsection",
   CellTags->"Explicit"],
  Cell[598564, 24407, 230, 6, 28, "Input",
   CellTags->"Explicit"],
  Cell[598797, 24415, 940, 42, 27, "Output",
   CellTags->"Explicit"],
  Cell[599774, 24462, 109, 3, 28, "Input",
   CellTags->"Explicit"],
  Cell[599886, 24467, 1969, 78, 27, "Output",
   CellTags->"Explicit"],
  Cell[601892, 24550, 184, 5, 28, "Input",
   CellTags->"Explicit"],
  Cell[602079, 24557, 892, 36, 38, "Output",
   CellTags->"Explicit"],
  Cell[603008, 24598, 109, 3, 28, "Input",
   CellTags->"Explicit"],
  Cell[603120, 24603, 2641, 101, 38, "Output",
   CellTags->"Explicit"],
  Cell[605798, 24709, 155, 4, 28, "Input",
   CellTags->"Explicit"],
  Cell[605956, 24715, 163, 5, 27, "Output",
   CellTags->"Explicit"],
  Cell[606156, 24725, 109, 3, 28, "Input",
   CellTags->"Explicit"],
  Cell[606268, 24730, 1742, 77, 27, "Output",
   CellTags->"Explicit"]},
 "ExplicitLorentzIndex"->{
  Cell[608037, 24811, 77, 1, 41, "ObjectName",
   CellTags->"ExplicitLorentzIndex"],
  Cell[608139, 24816, 71, 1, 36, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[608213, 24819, 141, 4, 25, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[608357, 24825, 295, 13, 25, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[608689, 24843, 68, 1, 36, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[608782, 24848, 224, 6, 28, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[609009, 24856, 199, 7, 27, "Output",
   CellTags->"ExplicitLorentzIndex"],
  Cell[609245, 24868, 121, 3, 28, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[609369, 24873, 247, 6, 42, "Output",
   CellTags->"ExplicitLorentzIndex"]},
 "ExplicitPartialD"->{
  Cell[609643, 24883, 69, 1, 41, "ObjectName",
   CellTags->"ExplicitPartialD"],
  Cell[609737, 24888, 67, 1, 36, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[609807, 24891, 162, 4, 25, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[609972, 24897, 476, 18, 25, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[610485, 24920, 64, 1, 36, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[610574, 24925, 183, 4, 28, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[610760, 24931, 525, 19, 38, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[611322, 24955, 184, 4, 28, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[611509, 24961, 430, 16, 33, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[611976, 24982, 186, 4, 28, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[612165, 24988, 461, 15, 38, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[612663, 25008, 183, 6, 28, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[612849, 25016, 270, 8, 35, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[613156, 25029, 125, 3, 28, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[613284, 25034, 440, 14, 38, "Output",
   CellTags->"ExplicitPartialD"]},
 "ExplicitSUNIndex"->{
  Cell[613751, 25052, 69, 1, 41, "ObjectName",
   CellTags->"ExplicitSUNIndex"],
  Cell[613845, 25057, 67, 1, 36, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[613915, 25060, 178, 6, 25, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[614096, 25068, 484, 23, 25, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[614617, 25096, 64, 1, 36, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[614706, 25101, 125, 3, 28, "Input",
   CellTags->"ExplicitSUNIndex"],
  Cell[614834, 25106, 112, 3, 27, "Output",
   CellTags->"ExplicitSUNIndex"]},
 "ExtraFactor"->{
  Cell[615393, 25131, 59, 1, 41, "ObjectName",
   CellTags->"ExtraFactor"],
  Cell[615477, 25136, 62, 1, 36, "Subsubsection",
   CellTags->"ExtraFactor"],
  Cell[615542, 25139, 199, 4, 43, "Text",
   CellTags->"ExtraFactor"],
  Cell[615744, 25145, 194, 8, 25, "Text",
   CellTags->"ExtraFactor"]},
 "ExtraVariables"->{
  Cell[615953, 25156, 65, 1, 41, "ObjectName",
   CellTags->"ExtraVariables"],
  Cell[616043, 25161, 65, 1, 36, "Subsubsection",
   CellTags->"ExtraVariables"],
  Cell[616111, 25164, 213, 5, 43, "Text",
   CellTags->"ExtraVariables"],
  Cell[616327, 25171, 298, 14, 25, "Text",
   CellTags->"ExtraVariables"]},
 "FactorFull"->{
  Cell[616640, 25188, 57, 1, 41, "ObjectName",
   CellTags->"FactorFull"],
  Cell[616722, 25193, 61, 1, 36, "Subsubsection",
   CellTags->"FactorFull"],
  Cell[616786, 25196, 509, 24, 25, "Text",
   CellTags->"FactorFull"],
  Cell[617298, 25222, 178, 9, 25, "Text",
   CellTags->"FactorFull"]},
 "Factoring"->{
  Cell[617491, 25234, 55, 1, 41, "ObjectName",
   CellTags->"Factoring"],
  Cell[617571, 25239, 60, 1, 36, "Subsubsection",
   CellTags->"Factoring"],
  Cell[617634, 25242, 235, 5, 43, "Text",
   CellTags->"Factoring"],
  Cell[617872, 25249, 382, 19, 25, "Text",
   CellTags->"Factoring"]},
 "Factorout"->{
  Cell[618269, 25271, 55, 1, 41, "ObjectName",
   CellTags->"Factorout"],
  Cell[618349, 25276, 60, 1, 36, "Subsubsection",
   CellTags->"Factorout"],
  Cell[618412, 25279, 74, 1, 25, "Text",
   CellTags->"Factorout"],
  Cell[618489, 25282, 174, 9, 25, "Text",
   CellTags->"Factorout"]},
 "FactorTime"->{
  Cell[618678, 25294, 57, 1, 41, "ObjectName",
   CellTags->"FactorTime"],
  Cell[618760, 25299, 61, 1, 36, "Subsubsection",
   CellTags->"FactorTime"],
  Cell[618824, 25302, 166, 4, 25, "Text",
   CellTags->"FactorTime"],
  Cell[618993, 25308, 178, 9, 25, "Text",
   CellTags->"FactorTime"]},
 "Factor1"->{
  Cell[619186, 25320, 51, 1, 41, "ObjectName",
   CellTags->"Factor1"],
  Cell[619262, 25325, 58, 1, 36, "Subsubsection",
   CellTags->"Factor1"],
  Cell[619323, 25328, 144, 4, 25, "Text",
   CellTags->"Factor1"],
  Cell[619470, 25334, 175, 9, 25, "Text",
   CellTags->"Factor1"],
  Cell[619682, 25348, 55, 1, 36, "Subsubsection",
   CellTags->"Factor1"],
  Cell[619762, 25353, 213, 8, 28, "Input",
   CellTags->"Factor1"],
  Cell[619978, 25363, 224, 8, 27, "Output",
   CellTags->"Factor1"],
  Cell[620239, 25376, 222, 7, 28, "Input",
   CellTags->"Factor1"],
  Cell[620464, 25385, 435, 16, 27, "Output",
   CellTags->"Factor1"],
  Cell[620936, 25406, 251, 9, 28, "Input",
   CellTags->"Factor1"],
  Cell[621190, 25417, 176, 6, 27, "Output",
   CellTags->"Factor1"],
  Cell[621403, 25428, 107, 3, 28, "Input",
   CellTags->"Factor1"],
  Cell[621513, 25433, 224, 8, 27, "Output",
   CellTags->"Factor1"],
  Cell[621774, 25446, 108, 3, 28, "Input",
   CellTags->"Factor1"],
  Cell[621885, 25451, 176, 6, 27, "Output",
   CellTags->"Factor1"],
  Cell[622076, 25460, 141, 4, 28, "Input",
   CellTags->"Factor1"]},
 "Factor2"->{
  Cell[622232, 25467, 51, 1, 41, "ObjectName",
   CellTags->"Factor2"],
  Cell[622308, 25472, 58, 1, 36, "Subsubsection",
   CellTags->"Factor2"],
  Cell[622369, 25475, 398, 7, 61, "Text",
   CellTags->"Factor2"],
  Cell[622792, 25486, 113, 3, 28, "Input",
   CellTags->"Factor2"],
  Cell[622908, 25491, 230, 7, 27, "Output",
   CellTags->"Factor2"],
  Cell[623153, 25501, 178, 9, 25, "Text",
   CellTags->"Factor2"],
  Cell[623368, 25515, 55, 1, 36, "Subsubsection",
   CellTags->"Factor2"],
  Cell[623448, 25520, 213, 8, 28, "Input",
   CellTags->"Factor2"],
  Cell[623664, 25530, 224, 8, 27, "Output",
   CellTags->"Factor2"],
  Cell[623925, 25543, 222, 7, 28, "Input",
   CellTags->"Factor2"],
  Cell[624150, 25552, 435, 16, 27, "Output",
   CellTags->"Factor2"],
  Cell[624622, 25573, 251, 9, 28, "Input",
   CellTags->"Factor2"],
  Cell[624876, 25584, 176, 6, 27, "Output",
   CellTags->"Factor2"],
  Cell[625089, 25595, 107, 3, 28, "Input",
   CellTags->"Factor2"],
  Cell[625199, 25600, 224, 8, 27, "Output",
   CellTags->"Factor2"],
  Cell[625460, 25613, 108, 3, 28, "Input",
   CellTags->"Factor2"],
  Cell[625571, 25618, 176, 6, 27, "Output",
   CellTags->"Factor2"],
  Cell[625784, 25629, 173, 5, 28, "Input",
   CellTags->"Factor2"],
  Cell[625960, 25636, 224, 8, 27, "Output",
   CellTags->"Factor2"],
  Cell[626199, 25647, 141, 4, 28, "Input",
   CellTags->"Factor2"]},
 "FAD"->{
  Cell[626355, 25654, 43, 1, 70, "ObjectName",
   CellTags->"FAD"],
  Cell[626423, 25659, 54, 1, 70, "Subsubsection",
   CellTags->"FAD"],
  Cell[626480, 25662, 335, 6, 70, "Text",
   CellTags->"FAD"],
  Cell[626818, 25670, 802, 34, 70, "Text",
   CellTags->"FAD"],
  Cell[627657, 25709, 51, 1, 36, "Subsubsection",
   CellTags->"FAD"],
  Cell[627733, 25714, 146, 5, 28, "Input",
   CellTags->"FAD"],
  Cell[627882, 25721, 700, 20, 48, "Output",
   CellTags->"FAD"],
  Cell[628619, 25746, 197, 7, 28, "Input",
   CellTags->"FAD"],
  Cell[628819, 25755, 819, 22, 48, "Output",
   CellTags->"FAD"],
  Cell[629675, 25782, 245, 9, 28, "Input",
   CellTags->"FAD"],
  Cell[629923, 25793, 161, 5, 42, "Output",
   CellTags->"FAD"],
  Cell[630121, 25803, 215, 8, 28, "Input",
   CellTags->"FAD"],
  Cell[630339, 25813, 527, 16, 62, "Output",
   CellTags->"FAD"],
  Cell[630903, 25834, 280, 9, 28, "Input",
   CellTags->"FAD"],
  Cell[631186, 25845, 527, 16, 62, "Output",
   CellTags->"FAD"]},
 "FC"->{
  Cell[631740, 25865, 41, 1, 70, "ObjectName",
   CellTags->"FC"],
  Cell[631806, 25870, 53, 1, 70, "Subsubsection",
   CellTags->"FC"],
  Cell[631862, 25873, 117, 3, 70, "Text",
   CellTags->"FC"],
  Cell[631982, 25878, 545, 24, 70, "Text",
   CellTags->"FC"],
  Cell[632564, 25907, 50, 1, 36, "Subsubsection",
   CellTags->"FC"],
  Cell[632617, 25910, 70, 2, 28, "Input",
   CellTags->"FC"],
  Cell[632712, 25916, 227, 7, 28, "Input",
   CellTags->"FC"],
  Cell[632942, 25925, 100, 2, 26, "Output",
   CellTags->"FC"],
  Cell[633057, 25930, 70, 2, 28, "Input",
   CellTags->"FC"],
  Cell[633152, 25936, 227, 7, 28, "Input",
   CellTags->"FC"],
  Cell[633382, 25945, 305, 11, 27, "Output",
   CellTags->"FC"]},
 "FCE"->{
  Cell[633714, 25960, 43, 1, 41, "ObjectName",
   CellTags->"FCE"],
  Cell[633782, 25965, 54, 1, 36, "Subsubsection",
   CellTags->"FCE"],
  Cell[633839, 25968, 126, 4, 25, "Text",
   CellTags->"FCE"],
  Cell[633968, 25974, 72, 1, 25, "Text",
   CellTags->"FCE"],
  Cell[634043, 25977, 427, 19, 25, "Text",
   CellTags->"FCE"],
  Cell[634507, 26001, 51, 1, 36, "Subsubsection",
   CellTags->"FCE"],
  Cell[634583, 26006, 262, 8, 28, "Input",
   CellTags->"FCE"],
  Cell[634848, 26016, 225, 8, 27, "Output",
   CellTags->"FCE"],
  Cell[635110, 26029, 104, 3, 28, "Input",
   CellTags->"FCE"],
  Cell[635217, 26034, 190, 6, 42, "Output",
   CellTags->"FCE"],
  Cell[635444, 26045, 437, 13, 28, "Input",
   CellTags->"FCE"],
  Cell[635884, 26060, 547, 18, 29, "Output",
   CellTags->"FCE"],
  Cell[636468, 26083, 104, 3, 28, "Input",
   CellTags->"FCE"],
  Cell[636575, 26088, 452, 13, 42, "Output",
   CellTags->"FCE"],
  Cell[637064, 26106, 95, 3, 28, "Input",
   CellTags->"FCE"],
  Cell[637162, 26111, 862, 34, 29, "Output",
   CellTags->"FCE"],
  Cell[638061, 26150, 104, 3, 28, "Input",
   CellTags->"FCE"],
  Cell[638168, 26155, 893, 24, 82, "Output",
   CellTags->"FCE"],
  Cell[639098, 26184, 134, 4, 28, "Input",
   CellTags->"FCE"],
  Cell[639235, 26190, 452, 13, 42, "Output",
   CellTags->"FCE"]},
 "FCI"->{
  Cell[639714, 26207, 43, 1, 41, "ObjectName",
   CellTags->"FCI"],
  Cell[639782, 26212, 54, 1, 36, "Subsubsection",
   CellTags->"FCI"],
  Cell[639839, 26215, 159, 4, 25, "Text",
   CellTags->"FCI"],
  Cell[640001, 26221, 427, 19, 25, "Text",
   CellTags->"FCI"],
  Cell[640465, 26245, 51, 1, 36, "Subsubsection",
   CellTags->"FCI"],
  Cell[640541, 26250, 437, 13, 28, "Input",
   CellTags->"FCI"],
  Cell[640981, 26265, 547, 18, 29, "Output",
   CellTags->"FCI"],
  Cell[641565, 26288, 104, 3, 28, "Input",
   CellTags->"FCI"],
  Cell[641672, 26293, 452, 13, 42, "Output",
   CellTags->"FCI"],
  Cell[642161, 26311, 95, 3, 28, "Input",
   CellTags->"FCI"],
  Cell[642259, 26316, 862, 34, 29, "Output",
   CellTags->"FCI"],
  Cell[643158, 26355, 104, 3, 28, "Input",
   CellTags->"FCI"],
  Cell[643265, 26360, 893, 24, 82, "Output",
   CellTags->"FCI"],
  Cell[644195, 26389, 134, 4, 28, "Input",
   CellTags->"FCI"],
  Cell[644332, 26395, 452, 13, 42, "Output",
   CellTags->"FCI"]},
 "FCIntegral"->{
  Cell[644811, 26412, 57, 1, 41, "ObjectName",
   CellTags->"FCIntegral"],
  Cell[644893, 26417, 61, 1, 36, "Subsubsection",
   CellTags->"FCIntegral"],
  Cell[644957, 26420, 209, 5, 43, "Text",
   CellTags->"FCIntegral"],
  Cell[645169, 26427, 360, 14, 25, "Text",
   CellTags->"FCIntegral"]},
 "FCIntegrate"->{
  Cell[645544, 26444, 59, 1, 41, "ObjectName",
   CellTags->"FCIntegrate"],
  Cell[645628, 26449, 62, 1, 36, "Subsubsection",
   CellTags->"FCIntegrate"],
  Cell[645693, 26452, 287, 6, 43, "Text",
   CellTags->"FCIntegrate"],
  Cell[645983, 26460, 194, 9, 25, "Text",
   CellTags->"FCIntegrate"]},
 "FCNIntegrate"->{
  Cell[646192, 26472, 61, 1, 41, "ObjectName",
   CellTags->"FCNIntegrate"],
  Cell[646278, 26477, 63, 1, 36, "Subsubsection",
   CellTags->"FCNIntegrate"],
  Cell[646344, 26480, 418, 7, 61, "Text",
   CellTags->"FCNIntegrate"],
  Cell[646765, 26489, 192, 9, 25, "Text",
   CellTags->"FCNIntegrate"]},
 "FC2RHI"->{
  Cell[646972, 26501, 50, 1, 41, "ObjectName",
   CellTags->"FC2RHI"],
  Cell[647047, 26506, 57, 1, 36, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[647107, 26509, 306, 6, 61, "Text",
   CellTags->"FC2RHI"],
  Cell[647438, 26519, 111, 3, 28, "Input",
   CellTags->"FC2RHI"],
  Cell[647552, 26524, 277, 8, 27, "Output",
   CellTags->"FC2RHI"],
  Cell[649588, 26612, 162, 9, 25, "Text",
   CellTags->"FC2RHI"],
  Cell[649787, 26626, 54, 1, 36, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[649844, 26629, 72, 2, 28, "Input",
   CellTags->"FC2RHI"]},
 "FC2TLI"->{
  Cell[649931, 26634, 50, 1, 41, "ObjectName",
   CellTags->"FC2TLI"],
  Cell[650006, 26639, 57, 1, 36, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[650066, 26642, 256, 5, 43, "Text",
   CellTags->"FC2TLI"],
  Cell[650347, 26651, 111, 3, 28, "Input",
   CellTags->"FC2TLI"],
  Cell[650461, 26656, 277, 8, 27, "Output",
   CellTags->"FC2TLI"],
  Cell[654866, 26821, 162, 9, 25, "Text",
   CellTags->"FC2TLI"],
  Cell[655065, 26835, 54, 1, 36, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[655122, 26838, 72, 2, 28, "Input",
   CellTags->"FC2TLI"]},
 "Info3387711966-3304885"->{
  Cell[650849, 26673, 321, 5, 116, "Print",
   CellTags->"Info3387711966-3304885"]},
 "FDS"->{
  Cell[655209, 26843, 43, 1, 41, "ObjectName",
   CellTags->"FDS"],
  Cell[655277, 26848, 54, 1, 36, "Subsubsection",
   CellTags->"FDS"],
  Cell[655334, 26851, 82, 1, 25, "Text",
   CellTags->"FDS"],
  Cell[655419, 26854, 228, 9, 25, "Text",
   CellTags->"FDS"]},
 "FermionSpinSum"->{
  Cell[655662, 26866, 65, 1, 41, "ObjectName",
   CellTags->"FermionSpinSum"],
  Cell[655752, 26871, 65, 1, 36, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[655820, 26874, 118, 3, 25, "Text",
   CellTags->"FermionSpinSum"],
  Cell[655963, 26881, 127, 3, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[656093, 26886, 314, 9, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[656422, 26898, 521, 24, 25, "Text",
   CellTags->"FermionSpinSum"],
  Cell[656980, 26927, 62, 1, 36, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[657045, 26930, 261, 12, 25, "Text",
   CellTags->"FermionSpinSum"],
  Cell[657331, 26946, 296, 9, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[657630, 26957, 353, 13, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[658020, 26975, 143, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[658166, 26981, 330, 12, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[658533, 26998, 143, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[658679, 27004, 133, 4, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[658849, 27013, 296, 9, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[659148, 27024, 353, 13, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[659538, 27042, 143, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[659684, 27048, 330, 12, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[660051, 27065, 143, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[660197, 27071, 152, 5, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[660386, 27081, 334, 10, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[660723, 27093, 508, 17, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[661246, 27113, 164, 4, 25, "Text",
   CellTags->"FermionSpinSum"],
  Cell[661435, 27121, 147, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[661585, 27127, 540, 18, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[662162, 27150, 145, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[662310, 27156, 1074, 36, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[663421, 27197, 148, 4, 28, "Input",
   CellTags->"FermionSpinSum"],
  Cell[663572, 27203, 837, 32, 27, "Output",
   CellTags->"FermionSpinSum"],
  Cell[664424, 27238, 141, 4, 28, "Input",
   CellTags->"FermionSpinSum"]},
 "FeynAmp"->{
  Cell[664580, 27245, 51, 1, 41, "ObjectName",
   CellTags->"FeynAmp"],
  Cell[664656, 27250, 58, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[664717, 27253, 243, 5, 43, "Text",
   CellTags->"FeynAmp"],
  Cell[664963, 27260, 165, 4, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[665131, 27266, 181, 9, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[665349, 27280, 55, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[665407, 27283, 125, 4, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[665557, 27291, 846, 22, 50, "Input",
   CellTags->"FeynAmp"],
  Cell[666406, 27315, 2586, 102, 41, "Output",
   CellTags->"FeynAmp"],
  Cell[669007, 27420, 73, 1, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[669105, 27425, 198, 6, 28, "Input",
   CellTags->"FeynAmp"],
  Cell[669306, 27433, 414, 14, 41, "Output",
   CellTags->"FeynAmp"]},
 "FeynAmpDenominator"->{
  Cell[669747, 27451, 73, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominator"],
  Cell[669845, 27456, 69, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[669917, 27459, 256, 5, 43, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[670176, 27466, 338, 14, 25, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[670551, 27485, 66, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[670642, 27490, 201, 5, 28, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[670846, 27497, 211, 7, 42, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[671094, 27509, 326, 9, 28, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[671423, 27520, 430, 15, 48, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[671868, 27538, 247, 7, 28, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[672140, 27549, 149, 4, 28, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[672292, 27555, 279, 7, 42, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[672608, 27567, 149, 4, 28, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[672760, 27573, 176, 5, 42, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[672951, 27581, 134, 4, 28, "Input",
   CellTags->"FeynAmpDenominator"]},
 "FeynAmpDenominatorCombine"->{
  Cell[673100, 27588, 87, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[673212, 27593, 76, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[673291, 27596, 228, 5, 43, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[673522, 27603, 241, 9, 25, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[673800, 27617, 73, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[673898, 27622, 234, 7, 28, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[674135, 27631, 765, 23, 45, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[674937, 27659, 208, 6, 28, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[675148, 27667, 203, 6, 42, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[675388, 27678, 206, 6, 28, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[675597, 27686, 233, 7, 42, "Output",
   CellTags->"FeynAmpDenominatorCombine"]},
 "FeynAmpDenominatorSimplify"->{
  Cell[675857, 27697, 89, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[675971, 27702, 77, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[676051, 27705, 398, 7, 61, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[676452, 27714, 85, 1, 25, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[676562, 27719, 95, 2, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[676660, 27723, 147, 3, 27, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[676822, 27729, 218, 9, 25, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[677077, 27743, 74, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[677154, 27746, 432, 14, 25, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[677611, 27764, 280, 8, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[677894, 27774, 122, 3, 27, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[678031, 27780, 468, 18, 27, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[678524, 27802, 328, 10, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[678855, 27814, 336, 11, 45, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[679228, 27830, 260, 7, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[679491, 27839, 308, 10, 45, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[679836, 27854, 339, 10, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[680178, 27866, 362, 12, 47, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[680577, 27883, 449, 14, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[681029, 27899, 1091, 40, 48, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[682157, 27944, 215, 6, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[682375, 27952, 795, 29, 47, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[683207, 27986, 157, 4, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[683367, 27992, 795, 29, 47, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[684199, 28026, 373, 13, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[684575, 28041, 1361, 50, 48, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[685951, 28094, 147, 4, 28, "Input",
   CellTags->"FeynAmpDenominatorSimplify"]},
 "FeynAmpDenominatorSplit"->{
  Cell[686113, 28101, 83, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[686221, 28106, 74, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[686298, 28109, 357, 7, 61, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[686658, 28118, 245, 9, 25, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[686940, 28132, 71, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[687036, 28137, 308, 9, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[687347, 28148, 489, 16, 45, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[687873, 28169, 116, 3, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[687992, 28174, 136, 3, 27, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[688165, 28182, 139, 3, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[688307, 28187, 489, 16, 45, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[688833, 28208, 150, 4, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[688986, 28214, 425, 13, 42, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[689448, 28232, 230, 7, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[689681, 28241, 368, 12, 42, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[690086, 28258, 206, 6, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[690295, 28266, 305, 9, 42, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[690615, 28278, 121, 3, 28, "Input",
   CellTags->"FeynAmpDenominatorSplit"]},
 "FeynAmpList"->{
  Cell[690751, 28284, 59, 1, 41, "ObjectName",
   CellTags->"FeynAmpList"],
  Cell[690835, 28289, 62, 1, 36, "Subsubsection",
   CellTags->"FeynAmpList"],
  Cell[690900, 28292, 145, 4, 25, "Text",
   CellTags->"FeynAmpList"],
  Cell[691048, 28298, 117, 3, 25, "Text",
   CellTags->"FeynAmpList"],
  Cell[691168, 28303, 179, 9, 25, "Text",
   CellTags->"FeynAmpList"]},
 "FeynCalc"->{
  Cell[691362, 28315, 53, 1, 41, "ObjectName",
   CellTags->"FeynCalc"],
  Cell[691440, 28320, 59, 1, 36, "Subsubsection",
   CellTags->"FeynCalc"],
  Cell[691502, 28323, 91, 1, 25, "Text",
   CellTags->"FeynCalc"],
  Cell[691618, 28328, 99, 3, 28, "Input",
   CellTags->"FeynCalc"],
  Cell[691720, 28333, 703, 11, 344, "Print",
   CellTags->{"FeynCalc", "Info3387711973-8270308"}]},
 "Info3387711973-8270308"->{
  Cell[691720, 28333, 703, 11, 344, "Print",
   CellTags->{"FeynCalc", "Info3387711973-8270308"}]},
 "FeynCalcExternal"->{
  Cell[692450, 28348, 69, 1, 41, "ObjectName",
   CellTags->"FeynCalcExternal"],
  Cell[692544, 28353, 67, 1, 36, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[692614, 28356, 156, 4, 25, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[692773, 28362, 211, 9, 25, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[693021, 28376, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[693110, 28381, 162, 4, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[693275, 28387, 143, 4, 27, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[693455, 28396, 117, 3, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[693575, 28401, 126, 3, 42, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[693738, 28409, 450, 13, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[694191, 28424, 560, 18, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[694788, 28447, 117, 3, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[694908, 28452, 465, 13, 42, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[695410, 28470, 121, 3, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[695534, 28475, 875, 34, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[696446, 28514, 117, 3, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[696566, 28519, 906, 24, 82, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[697509, 28548, 163, 5, 28, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[697675, 28555, 465, 13, 42, "Output",
   CellTags->"FeynCalcExternal"]},
 "FeynCalcForm"->{
  Cell[698167, 28572, 61, 1, 41, "ObjectName",
   CellTags->"FeynCalcForm"],
  Cell[698253, 28577, 63, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[698319, 28580, 534, 10, 79, "Text",
   CellTags->"FeynCalcForm"],
  Cell[698856, 28592, 433, 19, 25, "Text",
   CellTags->"FeynCalcForm"],
  Cell[699326, 28616, 60, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[699389, 28619, 79, 1, 25, "Text",
   CellTags->"FeynCalcForm"],
  Cell[699493, 28624, 242, 7, 28, "Input",
   CellTags->"FeynCalcForm"],
  Cell[699738, 28633, 527, 22, 27, "Output",
   CellTags->"FeynCalcForm"],
  Cell[700280, 28658, 112, 3, 25, "Text",
   CellTags->"FeynCalcForm"],
  Cell[700395, 28663, 148, 4, 28, "Input",
   CellTags->"FeynCalcForm"],
  Cell[700546, 28669, 647, 19, 72, "Input",
   CellTags->"FeynCalcForm"],
  Cell[701218, 28692, 242, 7, 28, "Input",
   CellTags->"FeynCalcForm"],
  Cell[701463, 28701, 191, 6, 32, "Output",
   CellTags->"FeynCalcForm"],
  Cell[701669, 28710, 76, 1, 25, "Text",
   CellTags->"FeynCalcForm"],
  Cell[701748, 28713, 123, 4, 28, "Input",
   CellTags->"FeynCalcForm"],
  Cell[701874, 28719, 652, 19, 50, "Input",
   CellTags->"FeynCalcForm"]},
 "FeynCalcInternal"->{
  Cell[702541, 28741, 69, 1, 41, "ObjectName",
   CellTags->"FeynCalcInternal"],
  Cell[702635, 28746, 67, 1, 36, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[702705, 28749, 157, 4, 25, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[702865, 28755, 401, 19, 25, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[703303, 28779, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[703392, 28784, 450, 13, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[703845, 28799, 560, 18, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[704442, 28822, 117, 3, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[704562, 28827, 465, 13, 42, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[705064, 28845, 121, 3, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[705188, 28850, 875, 34, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[706100, 28889, 117, 3, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[706220, 28894, 906, 24, 82, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[707163, 28923, 163, 5, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[707329, 28930, 465, 13, 42, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[707831, 28948, 517, 15, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[708351, 28965, 1461, 63, 42, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[709849, 29033, 117, 3, 28, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[709969, 29038, 1128, 30, 102, "Output",
   CellTags->"FeynCalcInternal"]},
 "FeynCalc2FORM"->{
  Cell[711124, 29072, 63, 1, 41, "ObjectName",
   CellTags->"FeynCalc2FORM"],
  Cell[711212, 29077, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[711279, 29080, 226, 5, 43, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[711508, 29087, 199, 9, 25, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[711732, 29100, 125, 3, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[711860, 29105, 1022, 22, 71, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[712931, 29133, 61, 1, 36, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[713017, 29138, 273, 9, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[713293, 29149, 276, 9, 41, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[713606, 29163, 137, 4, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[713746, 29169, 68, 1, 22, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[713851, 29175, 199, 6, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[714053, 29183, 199, 6, 28, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[714289, 29194, 137, 4, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[714429, 29200, 66, 1, 22, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[714532, 29206, 212, 6, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[714747, 29214, 373, 11, 27, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[715157, 29230, 137, 4, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[715297, 29236, 80, 1, 22, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[715414, 29242, 298, 9, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[715715, 29253, 473, 16, 27, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[716225, 29274, 137, 4, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[716365, 29280, 80, 1, 22, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[716460, 29284, 328, 10, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[716813, 29298, 332, 10, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[717148, 29310, 154, 4, 27, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[717339, 29319, 314, 10, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[717656, 29331, 6179, 240, 49, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[723850, 29574, 209, 6, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[724084, 29584, 376, 11, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[724463, 29597, 1338, 36, 233, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[725816, 29636, 313, 10, 28, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[726132, 29648, 129, 4, 28, "Input",
   CellTags->"FeynCalc2FORM"]},
 "FeynCalcToLaTeX"->{
  Cell[726276, 29655, 67, 1, 41, "ObjectName",
   CellTags->"FeynCalcToLaTeX"],
  Cell[726368, 29660, 66, 1, 36, "Subsubsection",
   CellTags->"FeynCalcToLaTeX"],
  Cell[726437, 29663, 300, 6, 69, "Text",
   CellTags->"FeynCalcToLaTeX"]},
 "FeynmanParameterNames"->{
  Cell[727973, 29717, 79, 1, 41, "ObjectName",
   CellTags->"FeynmanParameterNames"],
  Cell[728077, 29722, 72, 1, 36, "Subsubsection",
   CellTags->"FeynmanParameterNames"],
  Cell[728152, 29725, 143, 4, 25, "Text",
   CellTags->"FeynmanParameterNames"],
  Cell[728298, 29731, 362, 14, 25, "Text",
   CellTags->"FeynmanParameterNames"]},
 "FeynmanParametrize"->{
  Cell[728675, 29748, 90, 1, 41, "ObjectName",
   CellTags->"FeynmanParametrize"],
  Cell[728790, 29753, 69, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[728862, 29756, 176, 4, 25, "Text",
   CellTags->"FeynmanParametrize"],
  Cell[729063, 29764, 135, 3, 28, "Input",
   CellTags->"FeynmanParametrize"],
  Cell[729201, 29769, 251, 8, 27, "Output",
   CellTags->"FeynmanParametrize"],
  Cell[729501, 29783, 66, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[729570, 29786, 84, 2, 28, "Input",
   CellTags->"FeynmanParametrize"]},
 "FeynRule"->{
  Cell[729669, 29791, 53, 1, 41, "ObjectName",
   CellTags->"FeynRule"],
  Cell[729747, 29796, 59, 1, 36, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[729809, 29799, 168, 4, 25, "Text",
   CellTags->"FeynRule"],
  Cell[730002, 29807, 115, 3, 28, "Input",
   CellTags->"FeynRule"],
  Cell[730120, 29812, 630, 16, 49, "Output",
   CellTags->"FeynRule"],
  Cell[730765, 29831, 92, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[730860, 29834, 133, 4, 25, "Text",
   CellTags->"FeynRule"],
  Cell[730996, 29840, 185, 9, 25, "Text",
   CellTags->"FeynRule"],
  Cell[731218, 29854, 56, 1, 36, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[731299, 29859, 146, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[731448, 29865, 680, 23, 38, "Output",
   CellTags->"FeynRule"],
  Cell[732165, 29893, 146, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[732314, 29899, 1003, 36, 38, "Output",
   CellTags->"FeynRule"],
  Cell[733354, 29940, 111, 3, 28, "Input",
   CellTags->"FeynRule"],
  Cell[733468, 29945, 4550, 176, 38, "Output",
   CellTags->"FeynRule"],
  Cell[738055, 30126, 141, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[738199, 30132, 2654, 113, 28, "Output",
   CellTags->"FeynRule"],
  Cell[740868, 30248, 74, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[740967, 30253, 503, 16, 28, "Input",
   CellTags->"FeynRule"],
  Cell[741473, 30271, 700, 30, 27, "Output",
   CellTags->"FeynRule"],
  Cell[742210, 30306, 307, 9, 28, "Input",
   CellTags->"FeynRule"],
  Cell[742520, 30317, 3760, 137, 47, "Output",
   CellTags->"FeynRule"],
  Cell[746295, 30457, 72, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[746392, 30462, 503, 16, 28, "Input",
   CellTags->"FeynRule"],
  Cell[746898, 30480, 700, 30, 27, "Output",
   CellTags->"FeynRule"],
  Cell[747635, 30515, 307, 9, 28, "Input",
   CellTags->"FeynRule"],
  Cell[747945, 30526, 4465, 151, 47, "Output",
   CellTags->"FeynRule"],
  Cell[752447, 30682, 218, 7, 28, "Input",
   CellTags->"FeynRule"],
  Cell[752668, 30691, 1044, 42, 28, "Output",
   CellTags->"FeynRule"],
  Cell[753727, 30736, 110, 3, 25, "Text",
   CellTags->"FeynRule"],
  Cell[753862, 30743, 360, 10, 28, "Input",
   CellTags->"FeynRule"],
  Cell[754225, 30755, 1108, 45, 28, "Output",
   CellTags->"FeynRule"],
  Cell[755348, 30803, 77, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[755450, 30808, 145, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[755598, 30814, 618, 22, 28, "Output",
   CellTags->"FeynRule"],
  Cell[756231, 30839, 90, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[756346, 30844, 145, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[756494, 30850, 618, 22, 28, "Output",
   CellTags->"FeynRule"],
  Cell[757149, 30877, 696, 21, 50, "Input",
   CellTags->"FeynRule"],
  Cell[757848, 30900, 893, 37, 29, "Output",
   CellTags->"FeynRule"],
  Cell[758778, 30942, 227, 7, 28, "Input",
   CellTags->"FeynRule"],
  Cell[759008, 30951, 4903, 175, 107, "Output",
   CellTags->"FeynRule"],
  Cell[763948, 31131, 420, 12, 28, "Input",
   CellTags->"FeynRule"],
  Cell[764371, 31145, 4701, 156, 75, "Output",
   CellTags->"FeynRule"],
  Cell[769109, 31306, 362, 12, 28, "Input",
   CellTags->"FeynRule"],
  Cell[769474, 31320, 104, 3, 27, "Output",
   CellTags->"FeynRule"],
  Cell[769593, 31326, 210, 5, 43, "Text",
   CellTags->"FeynRule"],
  Cell[769806, 31333, 144, 4, 28, "Input",
   CellTags->"FeynRule"],
  Cell[769953, 31339, 51, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[770029, 31344, 693, 21, 50, "Input",
   CellTags->"FeynRule"],
  Cell[770725, 31367, 984, 42, 27, "Output",
   CellTags->"FeynRule"],
  Cell[771746, 31414, 212, 7, 28, "Input",
   CellTags->"FeynRule"],
  Cell[771961, 31423, 1969, 78, 27, "Output",
   CellTags->"FeynRule"],
  Cell[773967, 31506, 385, 11, 28, "Input",
   CellTags->"FeynRule"],
  Cell[774355, 31519, 1419, 57, 27, "Output",
   CellTags->"FeynRule"],
  Cell[775811, 31581, 197, 6, 28, "Input",
   CellTags->"FeynRule"],
  Cell[776011, 31589, 104, 3, 27, "Output",
   CellTags->"FeynRule"],
  Cell[776152, 31597, 884, 26, 50, "Input",
   CellTags->"FeynRule"],
  Cell[777039, 31625, 1270, 54, 29, "Output",
   CellTags->"FeynRule"],
  Cell[778346, 31684, 212, 7, 28, "Input",
   CellTags->"FeynRule"],
  Cell[778561, 31693, 3296, 130, 27, "Output",
   CellTags->"FeynRule"],
  Cell[781894, 31828, 461, 13, 28, "Input",
   CellTags->"FeynRule"],
  Cell[782358, 31843, 3461, 131, 27, "Output",
   CellTags->"FeynRule"],
  Cell[785856, 31979, 197, 6, 28, "Input",
   CellTags->"FeynRule"],
  Cell[786056, 31987, 104, 3, 27, "Output",
   CellTags->"FeynRule"],
  Cell[786175, 31993, 418, 9, 50, "Input",
   CellTags->"FeynRule"]},
 "FI"->{
  Cell[786608, 32005, 41, 1, 41, "ObjectName",
   CellTags->"FI"],
  Cell[786674, 32010, 53, 1, 36, "Subsubsection",
   CellTags->"FI"],
  Cell[786730, 32013, 195, 4, 43, "Text",
   CellTags->"FI"],
  Cell[786928, 32019, 545, 24, 25, "Text",
   CellTags->"FI"]},
 "FieldDerivative"->{
  Cell[787488, 32046, 67, 1, 41, "ObjectName",
   CellTags->"FieldDerivative"],
  Cell[787580, 32051, 66, 1, 36, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[787649, 32054, 549, 9, 79, "Text",
   CellTags->"FieldDerivative"],
  Cell[788201, 32065, 314, 14, 25, "Text",
   CellTags->"FieldDerivative"],
  Cell[788552, 32084, 63, 1, 36, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[788640, 32089, 620, 19, 50, "Input",
   CellTags->"FieldDerivative"],
  Cell[789263, 32110, 465, 17, 27, "Output",
   CellTags->"FieldDerivative"],
  Cell[789765, 32132, 218, 6, 28, "Input",
   CellTags->"FieldDerivative"],
  Cell[789986, 32140, 1529, 55, 27, "Output",
   CellTags->"FieldDerivative"],
  Cell[791552, 32200, 218, 6, 28, "Input",
   CellTags->"FieldDerivative"],
  Cell[791773, 32208, 4269, 153, 49, "Output",
   CellTags->"FieldDerivative"]},
 "FieldStrength"->{
  Cell[796069, 32365, 63, 1, 41, "ObjectName",
   CellTags->"FieldStrength"],
  Cell[796157, 32370, 64, 1, 36, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[796224, 32373, 1285, 37, 61, "Text",
   CellTags->"FieldStrength"],
  Cell[797534, 32414, 125, 3, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[797662, 32419, 529, 15, 49, "Output",
   CellTags->"FieldStrength"],
  Cell[798206, 32437, 196, 9, 25, "Text",
   CellTags->"FieldStrength"],
  Cell[798439, 32451, 61, 1, 36, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[798525, 32456, 150, 4, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[798678, 32462, 174, 5, 27, "Output",
   CellTags->"FieldStrength"],
  Cell[798889, 32472, 160, 4, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[799052, 32478, 168, 5, 27, "Output",
   CellTags->"FieldStrength"],
  Cell[799257, 32488, 200, 5, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[799460, 32495, 722, 30, 27, "Output",
   CellTags->"FieldStrength"],
  Cell[800219, 32530, 210, 5, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[800432, 32537, 1503, 65, 27, "Output",
   CellTags->"FieldStrength"],
  Cell[801972, 32607, 241, 6, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[802216, 32615, 547, 15, 62, "Output",
   CellTags->"FieldStrength"],
  Cell[802800, 32635, 244, 6, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[803047, 32643, 545, 15, 62, "Output",
   CellTags->"FieldStrength"],
  Cell[803629, 32663, 291, 7, 28, "Input",
   CellTags->"FieldStrength"],
  Cell[803923, 32672, 1503, 65, 27, "Output",
   CellTags->"FieldStrength"]},
 "FinalFunction"->{
  Cell[805453, 32741, 63, 1, 41, "ObjectName",
   CellTags->"FinalFunction"],
  Cell[805541, 32746, 64, 1, 36, "Subsubsection",
   CellTags->"FinalFunction"],
  Cell[805608, 32749, 86, 1, 25, "Text",
   CellTags->"FinalFunction"],
  Cell[805697, 32752, 190, 9, 25, "Text",
   CellTags->"FinalFunction"]},
 "FinalSubstitutions"->{
  Cell[805902, 32764, 73, 1, 41, "ObjectName",
   CellTags->"FinalSubstitutions"],
  Cell[806000, 32769, 69, 1, 36, "Subsubsection",
   CellTags->"FinalSubstitutions"],
  Cell[806072, 32772, 204, 4, 43, "Text",
   CellTags->"FinalSubstitutions"],
  Cell[806279, 32778, 406, 19, 25, "Text",
   CellTags->"FinalSubstitutions"]},
 "FORM"->{
  Cell[806700, 32800, 45, 1, 41, "ObjectName",
   CellTags->"FORM"],
  Cell[806770, 32805, 55, 1, 36, "Subsubsection",
   CellTags->"FORM"],
  Cell[806828, 32808, 204, 5, 43, "Text",
   CellTags->"FORM"],
  Cell[807035, 32815, 285, 14, 25, "Text",
   CellTags->"FORM"]},
 "FORMEpilog"->{
  Cell[807335, 32832, 57, 1, 41, "ObjectName",
   CellTags->"FORMEpilog"],
  Cell[807417, 32837, 61, 1, 36, "Subsubsection",
   CellTags->"FORMEpilog"],
  Cell[807481, 32840, 161, 4, 25, "Text",
   CellTags->"FORMEpilog"],
  Cell[807645, 32846, 312, 14, 25, "Text",
   CellTags->"FORMEpilog"]},
 "FORMProlog"->{
  Cell[807972, 32863, 57, 1, 41, "ObjectName",
   CellTags->"FORMProlog"],
  Cell[808054, 32868, 61, 1, 36, "Subsubsection",
   CellTags->"FORMProlog"],
  Cell[808118, 32871, 178, 4, 25, "Text",
   CellTags->"FORMProlog"],
  Cell[808299, 32877, 312, 14, 25, "Text",
   CellTags->"FORMProlog"]},
 "FORM2FeynCalc"->{
  Cell[808626, 32894, 63, 1, 41, "ObjectName",
   CellTags->"FORM2FeynCalc"],
  Cell[808714, 32899, 64, 1, 36, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[808781, 32902, 470, 8, 79, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[809254, 32912, 199, 9, 25, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[809478, 32925, 125, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[809606, 32930, 662, 18, 49, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[810317, 32954, 61, 1, 36, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[810403, 32959, 139, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[810545, 32964, 354, 14, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[810936, 32983, 114, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[811053, 32988, 243, 8, 42, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[811311, 32999, 159, 4, 25, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[811495, 33007, 314, 8, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[811812, 33017, 405, 14, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[812254, 33036, 114, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[812371, 33041, 303, 9, 43, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[812711, 33055, 150, 4, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[812864, 33061, 305, 10, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[813206, 33076, 117, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[813326, 33081, 237, 8, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[813600, 33094, 150, 4, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[813753, 33100, 257, 9, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[814047, 33114, 114, 3, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[814164, 33119, 307, 10, 42, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[814508, 33134, 333, 9, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[814844, 33145, 269, 9, 27, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[815150, 33159, 195, 5, 28, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[815348, 33166, 583, 18, 27, "Output",
   CellTags->"FORM2FeynCalc"]},
 "FourDivergence"->{
  Cell[815958, 33188, 65, 1, 41, "ObjectName",
   CellTags->"FourDivergence"],
  Cell[816048, 33193, 65, 1, 36, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[816116, 33196, 242, 5, 43, "Text",
   CellTags->"FourDivergence"],
  Cell[816361, 33203, 197, 9, 25, "Text",
   CellTags->"FourDivergence"],
  Cell[816595, 33217, 62, 1, 36, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[816682, 33222, 167, 5, 28, "Input",
   CellTags->"FourDivergence"],
  Cell[816852, 33229, 212, 8, 27, "Output",
   CellTags->"FourDivergence"],
  Cell[817101, 33242, 211, 6, 28, "Input",
   CellTags->"FourDivergence"],
  Cell[817315, 33250, 242, 10, 27, "Output",
   CellTags->"FourDivergence"],
  Cell[817594, 33265, 192, 6, 28, "Input",
   CellTags->"FourDivergence"],
  Cell[817789, 33273, 247, 9, 27, "Output",
   CellTags->"FourDivergence"],
  Cell[818073, 33287, 237, 7, 28, "Input",
   CellTags->"FourDivergence"],
  Cell[818313, 33296, 110, 3, 27, "Output",
   CellTags->"FourDivergence"],
  Cell[818438, 33302, 112, 3, 28, "Input",
   CellTags->"FourDivergence"]},
 "FourLaplacian"->{
  Cell[818565, 33308, 63, 1, 41, "ObjectName",
   CellTags->"FourLaplacian"],
  Cell[818653, 33313, 64, 1, 36, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[818720, 33316, 426, 16, 25, "Text",
   CellTags->"FourLaplacian"],
  Cell[819171, 33336, 125, 3, 28, "Input",
   CellTags->"FourLaplacian"],
  Cell[819299, 33341, 170, 5, 27, "Output",
   CellTags->"FourLaplacian"],
  Cell[819484, 33349, 324, 14, 25, "Text",
   CellTags->"FourLaplacian"],
  Cell[819845, 33368, 61, 1, 36, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[819931, 33373, 131, 4, 28, "Input",
   CellTags->"FourLaplacian"],
  Cell[820065, 33379, 167, 6, 27, "Output",
   CellTags->"FourLaplacian"],
  Cell[820269, 33390, 152, 4, 28, "Input",
   CellTags->"FourLaplacian"],
  Cell[820424, 33396, 132, 4, 27, "Output",
   CellTags->"FourLaplacian"],
  Cell[820593, 33405, 264, 9, 28, "Input",
   CellTags->"FourLaplacian"],
  Cell[820860, 33416, 440, 16, 48, "Output",
   CellTags->"FourLaplacian"],
  Cell[821337, 33437, 157, 4, 28, "Input",
   CellTags->"FourLaplacian"],
  Cell[821497, 33443, 3872, 136, 95, "Output",
   CellTags->"FourLaplacian"]},
 "FourVector"->{
  Cell[825396, 33583, 57, 1, 41, "ObjectName",
   CellTags->"FourVector"],
  Cell[825478, 33588, 61, 1, 36, "Subsubsection",
   CellTags->"FourVector"],
  Cell[825542, 33591, 339, 10, 43, "Text",
   CellTags->"FourVector"],
  Cell[825884, 33603, 356, 19, 25, "Text",
   CellTags->"FourVector"],
  Cell[826277, 33627, 58, 1, 36, "Subsubsection",
   CellTags->"FourVector"],
  Cell[826360, 33632, 140, 4, 28, "Input",
   CellTags->"FourVector"],
  Cell[826503, 33638, 238, 10, 27, "Output",
   CellTags->"FourVector"],
  Cell[826778, 33653, 164, 5, 28, "Input",
   CellTags->"FourVector"],
  Cell[826945, 33660, 290, 12, 27, "Output",
   CellTags->"FourVector"],
  Cell[827272, 33677, 180, 5, 28, "Input",
   CellTags->"FourVector"],
  Cell[827455, 33684, 225, 6, 42, "Output",
   CellTags->"FourVector"],
  Cell[827717, 33695, 229, 6, 28, "Input",
   CellTags->"FourVector"],
  Cell[827949, 33703, 275, 8, 42, "Output",
   CellTags->"FourVector"],
  Cell[828261, 33716, 212, 6, 28, "Input",
   CellTags->"FourVector"],
  Cell[828476, 33724, 147, 4, 42, "Output",
   CellTags->"FourVector"],
  Cell[828638, 33731, 193, 4, 43, "Text",
   CellTags->"FourVector"],
  Cell[828856, 33739, 212, 6, 28, "Input",
   CellTags->"FourVector"],
  Cell[829071, 33747, 407, 18, 27, "Output",
   CellTags->"FourVector"]},
 "FreeIndex"->{
  Cell[829505, 33769, 55, 1, 41, "ObjectName",
   CellTags->"FreeIndex"],
  Cell[829585, 33774, 60, 1, 36, "Subsubsection",
   CellTags->"FreeIndex"],
  Cell[829648, 33777, 152, 4, 25, "Text",
   CellTags->"FreeIndex"],
  Cell[829803, 33783, 290, 14, 25, "Text",
   CellTags->"FreeIndex"]},
 "FreeQ2"->{
  Cell[830108, 33800, 49, 1, 41, "ObjectName",
   CellTags->"FreeQ2"],
  Cell[830182, 33805, 57, 1, 36, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[830242, 33808, 228, 5, 43, "Text",
   CellTags->"FreeQ2"],
  Cell[830473, 33815, 308, 14, 25, "Text",
   CellTags->"FreeQ2"],
  Cell[830818, 33834, 54, 1, 36, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[830897, 33839, 246, 8, 28, "Input",
   CellTags->"FreeQ2"],
  Cell[831146, 33849, 106, 3, 27, "Output",
   CellTags->"FreeQ2"],
  Cell[831289, 33857, 241, 8, 28, "Input",
   CellTags->"FreeQ2"],
  Cell[831533, 33867, 105, 3, 27, "Output",
   CellTags->"FreeQ2"],
  Cell[831675, 33875, 133, 4, 28, "Input",
   CellTags->"FreeQ2"],
  Cell[831811, 33881, 105, 3, 27, "Output",
   CellTags->"FreeQ2"],
  Cell[831953, 33889, 162, 5, 28, "Input",
   CellTags->"FreeQ2"],
  Cell[832118, 33896, 106, 3, 27, "Output",
   CellTags->"FreeQ2"]},
 "FRH"->{
  Cell[832251, 33903, 43, 1, 41, "ObjectName",
   CellTags->"FRH"],
  Cell[832319, 33908, 54, 1, 36, "Subsubsection",
   CellTags->"FRH"],
  Cell[832376, 33911, 134, 4, 25, "Text",
   CellTags->"FRH"],
  Cell[832513, 33917, 171, 9, 25, "Text",
   CellTags->"FRH"],
  Cell[832721, 33931, 51, 1, 36, "Subsubsection",
   CellTags->"FRH"],
  Cell[832797, 33936, 200, 6, 28, "Input",
   CellTags->"FRH"],
  Cell[833000, 33944, 248, 9, 27, "Output",
   CellTags->"FRH"],
  Cell[833285, 33958, 99, 3, 28, "Input",
   CellTags->"FRH"],
  Cell[833387, 33963, 99, 3, 27, "Output",
   CellTags->"FRH"],
  Cell[833523, 33971, 304, 10, 28, "Input",
   CellTags->"FRH"],
  Cell[833830, 33983, 525, 20, 27, "Output",
   CellTags->"FRH"],
  Cell[834392, 34008, 128, 4, 28, "Input",
   CellTags->"FRH"],
  Cell[834523, 34014, 536, 21, 52, "Output",
   CellTags->"FRH"]},
 "FromTFi"->{
  Cell[835086, 34039, 51, 1, 41, "ObjectName",
   CellTags->"FromTFi"],
  Cell[835162, 34044, 58, 1, 36, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[835223, 34047, 200, 5, 43, "Text",
   CellTags->"FromTFi"],
  Cell[835721, 34072, 55, 1, 36, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[835801, 34077, 431, 14, 28, "Input",
   CellTags->"FromTFi"],
  Cell[836235, 34093, 945, 36, 28, "Output",
   CellTags->"FromTFi"],
  Cell[837217, 34134, 693, 22, 28, "Input",
   CellTags->"FromTFi"],
  Cell[837913, 34158, 639, 22, 48, "Output",
   CellTags->"FromTFi"],
  Cell[838589, 34185, 756, 24, 28, "Input",
   CellTags->"FromTFi"],
  Cell[839348, 34211, 842, 30, 49, "Output",
   CellTags->"FromTFi"]},
 "FromTFI"->{
  Cell[835426, 34054, 258, 13, 25, "Text",
   CellTags->"FromTFI"]},
 "FUNCTION"->{
  Cell[840217, 34245, 53, 1, 41, "ObjectName",
   CellTags->"FUNCTION"],
  Cell[840295, 34250, 59, 1, 36, "Subsubsection",
   CellTags->"FUNCTION"],
  Cell[840357, 34253, 160, 4, 25, "Text",
   CellTags->"FUNCTION"],
  Cell[840520, 34259, 173, 9, 25, "Text",
   CellTags->"FUNCTION"]},
 "FunctionalD"->{
  Cell[840708, 34271, 59, 1, 41, "ObjectName",
   CellTags->"FunctionalD"],
  Cell[840792, 34276, 62, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[840857, 34279, 416, 7, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[841276, 34288, 95, 1, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[841374, 34291, 304, 14, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[841715, 34310, 59, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[841777, 34313, 610, 20, 42, "Text",
   CellTags->"FunctionalD"],
  Cell[842412, 34337, 228, 6, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[842643, 34345, 107, 3, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[842787, 34353, 253, 7, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[843043, 34362, 135, 4, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[843193, 34369, 1269, 44, 49, "Text",
   CellTags->"FunctionalD"],
  Cell[844487, 34417, 295, 8, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[844785, 34427, 273, 10, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[845073, 34440, 778, 24, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[845876, 34468, 668, 22, 50, "Input",
   CellTags->"FunctionalD"],
  Cell[846547, 34492, 735, 26, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[847319, 34523, 217, 6, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[847539, 34531, 626, 25, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[848180, 34559, 489, 15, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[848672, 34576, 97, 2, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[848794, 34582, 315, 8, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[849112, 34592, 1303, 56, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[850452, 34653, 315, 8, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[850770, 34663, 1303, 56, 28, "Output",
   CellTags->"FunctionalD"],
  Cell[852110, 34724, 216, 8, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[852329, 34734, 2956, 115, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[855300, 34852, 560, 13, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[855863, 34867, 1383, 49, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[857249, 34918, 445, 13, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[857719, 34935, 231, 7, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[857953, 34944, 286, 12, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[858276, 34961, 165, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[858444, 34968, 2260, 95, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[860719, 35066, 120, 3, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[860864, 35073, 176, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[861043, 35080, 2260, 95, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[863318, 35178, 290, 8, 43, "Text",
   CellTags->"FunctionalD"],
  Cell[863611, 35188, 523, 16, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[864159, 35208, 230, 7, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[864392, 35217, 384, 17, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[864813, 35239, 240, 7, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[865056, 35248, 416, 18, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[865509, 35271, 583, 17, 50, "Input",
   CellTags->"FunctionalD"],
  Cell[866095, 35290, 2900, 126, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[869032, 35421, 184, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[869219, 35428, 1405, 62, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[870661, 35495, 352, 11, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[871016, 35508, 1159, 50, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[872190, 35561, 269, 9, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[872462, 35572, 98, 2, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[872563, 35576, 167, 4, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[872755, 35584, 105, 3, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[872863, 35589, 2956, 115, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[875856, 35709, 194, 6, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[876053, 35717, 23406, 1004, 140, "Output",
   CellTags->"FunctionalD"],
  Cell[899474, 36724, 227, 5, 43, "Text",
   CellTags->"FunctionalD"],
  Cell[899704, 36731, 345, 6, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[900052, 36739, 132, 4, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[900187, 36745, 2583, 67, 138, "Input",
   CellTags->"FunctionalD"],
  Cell[902795, 36816, 169, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[902967, 36823, 12134, 525, 72, "Output",
   CellTags->"FunctionalD"],
  Cell[915138, 37353, 164, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[915305, 37360, 6459, 285, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[921779, 37648, 212, 8, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[922016, 37660, 315, 11, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[922334, 37673, 2679, 119, 38, "Output",
   CellTags->"FunctionalD"],
  Cell[925050, 37797, 651, 21, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[925704, 37820, 1310, 60, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[927051, 37885, 154, 4, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[927208, 37891, 5152, 227, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[932375, 38121, 131, 4, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[932531, 38129, 79, 2, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[932613, 38133, 1159, 50, 27, "Output",
   CellTags->"FunctionalD"],
  Cell[933809, 38188, 798, 21, 50, "Input",
   CellTags->"FunctionalD"],
  Cell[934610, 38211, 1064, 41, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[935711, 38257, 137, 4, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[935851, 38263, 4174, 167, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[940062, 38435, 204, 6, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[940269, 38443, 6023, 266, 28, "Output",
   CellTags->"FunctionalD"],
  Cell[946329, 38714, 138, 4, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[946470, 38720, 11159, 491, 49, "Output",
   CellTags->"FunctionalD"],
  Cell[957666, 39216, 163, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[957832, 39223, 10288, 452, 49, "Output",
   CellTags->"FunctionalD"],
  Cell[968135, 39678, 66, 1, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[968204, 39681, 539, 13, 50, "Input",
   CellTags->"FunctionalD"],
  Cell[968780, 39699, 110, 2, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[968893, 39703, 79, 1, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[971447, 39800, 765, 22, 50, "Input",
   CellTags->"FunctionalD"],
  Cell[972215, 39824, 8894, 328, 61, "Output",
   CellTags->"FunctionalD"],
  Cell[981146, 40157, 165, 5, 28, "Input",
   CellTags->"FunctionalD"],
  Cell[981314, 40164, 2064, 82, 38, "Output",
   CellTags->"FunctionalD"]},
 "FV"->{
  Cell[983405, 40250, 41, 1, 41, "ObjectName",
   CellTags->"FV"],
  Cell[983471, 40255, 53, 1, 36, "Subsubsection",
   CellTags->"FV"],
  Cell[983527, 40258, 275, 13, 25, "Text",
   CellTags->"FV"],
  Cell[983805, 40273, 562, 29, 25, "Text",
   CellTags->"FV"],
  Cell[984404, 40307, 50, 1, 36, "Subsubsection",
   CellTags->"FV"],
  Cell[984479, 40312, 124, 4, 28, "Input",
   CellTags->"FV"],
  Cell[984606, 40318, 126, 4, 27, "Output",
   CellTags->"FV"],
  Cell[984769, 40327, 148, 5, 28, "Input",
   CellTags->"FV"],
  Cell[984920, 40334, 185, 7, 27, "Output",
   CellTags->"FV"],
  Cell[985142, 40346, 160, 5, 28, "Input",
   CellTags->"FV"],
  Cell[985305, 40353, 139, 4, 42, "Output",
   CellTags->"FV"],
  Cell[985481, 40362, 195, 7, 28, "Input",
   CellTags->"FV"],
  Cell[985679, 40371, 217, 6, 42, "Output",
   CellTags->"FV"],
  Cell[985911, 40380, 130, 4, 25, "Text",
   CellTags->"FV"],
  Cell[986066, 40388, 196, 6, 28, "Input",
   CellTags->"FV"],
  Cell[986265, 40396, 185, 7, 27, "Output",
   CellTags->"FV"],
  Cell[986487, 40408, 107, 3, 28, "Input",
   CellTags->"FV"],
  Cell[986597, 40413, 163, 5, 42, "Output",
   CellTags->"FV"]},
 "FVD"->{
  Cell[986787, 40422, 43, 1, 41, "ObjectName",
   CellTags->"FVD"],
  Cell[986855, 40427, 54, 1, 36, "Subsubsection",
   CellTags->"FVD"],
  Cell[986912, 40430, 312, 15, 25, "Text",
   CellTags->"FVD"],
  Cell[987227, 40447, 560, 29, 25, "Text",
   CellTags->"FVD"],
  Cell[987824, 40481, 51, 1, 36, "Subsubsection",
   CellTags->"FVD"],
  Cell[987900, 40486, 126, 4, 28, "Input",
   CellTags->"FVD"],
  Cell[988029, 40492, 127, 4, 27, "Output",
   CellTags->"FVD"],
  Cell[988193, 40501, 150, 5, 28, "Input",
   CellTags->"FVD"],
  Cell[988346, 40508, 186, 7, 27, "Output",
   CellTags->"FVD"],
  Cell[988569, 40520, 162, 5, 28, "Input",
   CellTags->"FVD"],
  Cell[988734, 40527, 141, 4, 42, "Output",
   CellTags->"FVD"],
  Cell[988912, 40536, 197, 7, 28, "Input",
   CellTags->"FVD"],
  Cell[989112, 40545, 268, 8, 42, "Output",
   CellTags->"FVD"],
  Cell[989395, 40556, 88, 1, 25, "Text",
   CellTags->"FVD"],
  Cell[989508, 40561, 198, 6, 28, "Input",
   CellTags->"FVD"],
  Cell[989709, 40569, 186, 7, 27, "Output",
   CellTags->"FVD"],
  Cell[989932, 40581, 108, 3, 28, "Input",
   CellTags->"FVD"],
  Cell[990043, 40586, 165, 5, 42, "Output",
   CellTags->"FVD"]},
 "GA"->{
  Cell[990235, 40595, 41, 1, 41, "ObjectName",
   CellTags->"GA"],
  Cell[990301, 40600, 53, 1, 36, "Subsubsection",
   CellTags->"GA"],
  Cell[990357, 40603, 654, 24, 43, "Text",
   CellTags->"GA"],
  Cell[991014, 40629, 369, 19, 25, "Text",
   CellTags->"GA"],
  Cell[991420, 40653, 50, 1, 36, "Subsubsection",
   CellTags->"GA"],
  Cell[991495, 40658, 101, 3, 28, "Input",
   CellTags->"GA"],
  Cell[991599, 40663, 133, 4, 27, "Output",
   CellTags->"GA"],
  Cell[991769, 40672, 213, 7, 28, "Input",
   CellTags->"GA"],
  Cell[991985, 40681, 317, 10, 27, "Output",
   CellTags->"GA"],
  Cell[992339, 40696, 171, 5, 28, "Input",
   CellTags->"GA"],
  Cell[992513, 40703, 163, 4, 42, "Output",
   CellTags->"GA"],
  Cell[992713, 40712, 163, 5, 28, "Input",
   CellTags->"GA"],
  Cell[992879, 40719, 289, 8, 27, "Output",
   CellTags->"GA"],
  Cell[993205, 40732, 203, 6, 28, "Input",
   CellTags->"GA"],
  Cell[993411, 40740, 262, 7, 42, "Output",
   CellTags->"GA"],
  Cell[993710, 40752, 250, 8, 28, "Input",
   CellTags->"GA"],
  Cell[993963, 40762, 301, 9, 28, "Output",
   CellTags->"GA"]},
 "GAD"->{
  Cell[994291, 40775, 43, 1, 41, "ObjectName",
   CellTags->"GAD"],
  Cell[994359, 40780, 54, 1, 36, "Subsubsection",
   CellTags->"GAD"],
  Cell[994416, 40783, 671, 24, 43, "Text",
   CellTags->"GAD"],
  Cell[995090, 40809, 367, 19, 25, "Text",
   CellTags->"GAD"],
  Cell[995494, 40833, 51, 1, 36, "Subsubsection",
   CellTags->"GAD"],
  Cell[995570, 40838, 103, 3, 28, "Input",
   CellTags->"GAD"],
  Cell[995676, 40843, 134, 4, 27, "Output",
   CellTags->"GAD"],
  Cell[995847, 40852, 216, 7, 28, "Input",
   CellTags->"GAD"],
  Cell[996066, 40861, 318, 10, 27, "Output",
   CellTags->"GAD"],
  Cell[996421, 40876, 173, 5, 28, "Input",
   CellTags->"GAD"],
  Cell[996597, 40883, 213, 6, 42, "Output",
   CellTags->"GAD"],
  Cell[996847, 40894, 165, 5, 28, "Input",
   CellTags->"GAD"],
  Cell[997015, 40901, 290, 8, 27, "Output",
   CellTags->"GAD"],
  Cell[997342, 40914, 205, 6, 28, "Input",
   CellTags->"GAD"],
  Cell[997550, 40922, 267, 7, 42, "Output",
   CellTags->"GAD"],
  Cell[997854, 40934, 254, 8, 28, "Input",
   CellTags->"GAD"],
  Cell[998111, 40944, 302, 9, 28, "Output",
   CellTags->"GAD"]},
 "GammaEpsilon"->{
  Cell[998440, 40957, 61, 1, 41, "ObjectName",
   CellTags->"GammaEpsilon"],
  Cell[998526, 40962, 63, 1, 36, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[998592, 40965, 167, 4, 25, "Text",
   CellTags->"GammaEpsilon"],
  Cell[998762, 40971, 299, 14, 25, "Text",
   CellTags->"GammaEpsilon"],
  Cell[999098, 40990, 60, 1, 36, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[999161, 40993, 141, 4, 25, "Text",
   CellTags->"GammaEpsilon"],
  Cell[999327, 41001, 170, 5, 28, "Input",
   CellTags->"GammaEpsilon"],
  Cell[999500, 41008, 2073, 75, 40, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1001610, 41088, 170, 5, 28, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1001783, 41095, 1834, 69, 43, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1003632, 41167, 91, 1, 25, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1003748, 41172, 123, 3, 28, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1003874, 41177, 3083, 109, 90, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1006994, 41291, 117, 3, 28, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1007114, 41296, 150, 4, 27, "Output",
   CellTags->"GammaEpsilon"]},
 "GammaExpand"->{
  Cell[1007291, 41304, 59, 1, 41, "ObjectName",
   CellTags->"GammaExpand"],
  Cell[1007375, 41309, 62, 1, 36, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1007440, 41312, 123, 3, 25, "Text",
   CellTags->"GammaExpand"],
  Cell[1007566, 41317, 194, 9, 25, "Text",
   CellTags->"GammaExpand"],
  Cell[1007797, 41331, 59, 1, 36, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1007881, 41336, 187, 5, 28, "Input",
   CellTags->"GammaExpand"],
  Cell[1008071, 41343, 349, 12, 27, "Output",
   CellTags->"GammaExpand"],
  Cell[1008457, 41360, 197, 6, 28, "Input",
   CellTags->"GammaExpand"],
  Cell[1008657, 41368, 652, 23, 44, "Output",
   CellTags->"GammaExpand"],
  Cell[1009346, 41396, 187, 5, 28, "Input",
   CellTags->"GammaExpand"],
  Cell[1009536, 41403, 222, 7, 27, "Output",
   CellTags->"GammaExpand"]},
 "Gamma1"->{
  Cell[1009785, 41414, 49, 1, 41, "ObjectName",
   CellTags->"Gamma1"],
  Cell[1009859, 41419, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma1"],
  Cell[1009919, 41422, 139, 4, 25, "Text",
   CellTags->"Gamma1"],
  Cell[1010061, 41428, 275, 14, 25, "Text",
   CellTags->"Gamma1"]},
 "Gamma2"->{
  Cell[1010351, 41445, 49, 1, 41, "ObjectName",
   CellTags->"Gamma2"],
  Cell[1010425, 41450, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma2"],
  Cell[1010485, 41453, 172, 4, 43, "Text",
   CellTags->"Gamma2"],
  Cell[1010660, 41459, 275, 14, 25, "Text",
   CellTags->"Gamma2"]},
 "Gamma3"->{
  Cell[1010950, 41476, 49, 1, 41, "ObjectName",
   CellTags->"Gamma3"],
  Cell[1011024, 41481, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma3"],
  Cell[1011084, 41484, 237, 5, 43, "Text",
   CellTags->"Gamma3"],
  Cell[1011324, 41491, 275, 14, 25, "Text",
   CellTags->"Gamma3"]},
 "Gauge"->{
  Cell[1011614, 41508, 47, 1, 41, "ObjectName",
   CellTags->"Gauge"],
  Cell[1011686, 41513, 56, 1, 36, "Subsubsection",
   CellTags->"Gauge"],
  Cell[1011745, 41516, 133, 4, 25, "Text",
   CellTags->"Gauge"],
  Cell[1011881, 41522, 301, 14, 25, "Text",
   CellTags->"Gauge"]},
 "GaugeField"->{
  Cell[1012197, 41539, 57, 1, 41, "ObjectName",
   CellTags->"GaugeField"],
  Cell[1012279, 41544, 61, 1, 36, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1012343, 41547, 212, 5, 43, "Text",
   CellTags->"GaugeField"],
  Cell[1012558, 41554, 318, 14, 25, "Text",
   CellTags->"GaugeField"],
  Cell[1012913, 41573, 58, 1, 36, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1012996, 41578, 86, 2, 28, "Input",
   CellTags->"GaugeField"],
  Cell[1013085, 41582, 106, 3, 27, "Output",
   CellTags->"GaugeField"],
  Cell[1013228, 41590, 237, 6, 28, "Input",
   CellTags->"GaugeField"],
  Cell[1013468, 41598, 316, 14, 27, "Output",
   CellTags->"GaugeField"]},
 "GaugeXi"->{
  Cell[1013811, 41616, 51, 1, 41, "ObjectName",
   CellTags->"GaugeXi"],
  Cell[1013887, 41621, 58, 1, 36, "Subsubsection",
   CellTags->"GaugeXi"],
  Cell[1013948, 41624, 77, 1, 25, "Text",
   CellTags->"GaugeXi"],
  Cell[1014028, 41627, 285, 14, 25, "Text",
   CellTags->"GaugeXi"]},
 "GA5"->{
  Cell[1014328, 41644, 43, 1, 41, "ObjectName",
   CellTags->"GA5"],
  Cell[1014396, 41649, 54, 1, 36, "Subsubsection",
   CellTags->"GA5"],
  Cell[1014453, 41652, 88, 1, 25, "Text",
   CellTags->"GA5"],
  Cell[1014544, 41655, 364, 19, 25, "Text",
   CellTags->"GA5"],
  Cell[1014945, 41679, 51, 1, 36, "Subsubsection",
   CellTags->"GA5"],
  Cell[1015021, 41684, 72, 2, 28, "Input",
   CellTags->"GA5"],
  Cell[1015096, 41688, 130, 4, 27, "Output",
   CellTags->"GA5"],
  Cell[1015263, 41697, 94, 3, 28, "Input",
   CellTags->"GA5"],
  Cell[1015360, 41702, 99, 3, 27, "Output",
   CellTags->"GA5"]},
 "GGV"->{
  Cell[1015486, 41709, 43, 1, 41, "ObjectName",
   CellTags->"GGV"],
  Cell[1015554, 41714, 54, 1, 36, "Subsubsection",
   CellTags->"GGV"],
  Cell[1015611, 41717, 72, 1, 25, "Text",
   CellTags->"GGV"],
  Cell[1015686, 41720, 198, 9, 25, "Text",
   CellTags->"GGV"]},
 "GhostPropagator"->{
  Cell[1015899, 41732, 67, 1, 41, "ObjectName",
   CellTags->"GhostPropagator"],
  Cell[1015991, 41737, 66, 1, 36, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1016060, 41740, 324, 9, 25, "Text",
   CellTags->"GhostPropagator"],
  Cell[1016387, 41751, 436, 19, 25, "Text",
   CellTags->"GhostPropagator"],
  Cell[1017147, 41789, 63, 1, 36, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1017235, 41794, 156, 4, 28, "Input",
   CellTags->"GhostPropagator"],
  Cell[1017394, 41800, 193, 6, 27, "Output",
   CellTags->"GhostPropagator"],
  Cell[1017624, 41811, 206, 5, 28, "Input",
   CellTags->"GhostPropagator"],
  Cell[1017833, 41818, 409, 16, 44, "Output",
   CellTags->"GhostPropagator"],
  Cell[1018279, 41839, 123, 3, 28, "Input",
   CellTags->"GhostPropagator"],
  Cell[1018405, 41844, 176, 6, 27, "Output",
   CellTags->"GhostPropagator"],
  Cell[1018618, 41855, 188, 5, 28, "Input",
   CellTags->"GhostPropagator"],
  Cell[1018809, 41862, 413, 16, 44, "Output",
   CellTags->"GhostPropagator"],
  Cell[1019259, 41883, 285, 9, 28, "Input",
   CellTags->"GhostPropagator"],
  Cell[1019547, 41894, 228, 6, 42, "Output",
   CellTags->"GhostPropagator"]},
 "GHP"->{
  Cell[1019802, 41904, 43, 1, 41, "ObjectName",
   CellTags->"GHP"],
  Cell[1019870, 41909, 54, 1, 36, "Subsubsection",
   CellTags->"GHP"],
  Cell[1019927, 41912, 288, 9, 25, "Text",
   CellTags->"GHP"],
  Cell[1020218, 41923, 460, 19, 25, "Text",
   CellTags->"GHP"],
  Cell[1020715, 41947, 51, 1, 36, "Subsubsection",
   CellTags->"GHP"],
  Cell[1020791, 41952, 132, 4, 28, "Input",
   CellTags->"GHP"],
  Cell[1020926, 41958, 181, 6, 27, "Output",
   CellTags->"GHP"],
  Cell[1021144, 41969, 130, 4, 28, "Input",
   CellTags->"GHP"],
  Cell[1021277, 41975, 157, 5, 43, "Output",
   CellTags->"GHP"],
  Cell[1021471, 41985, 132, 4, 28, "Input",
   CellTags->"GHP"],
  Cell[1021606, 41991, 185, 6, 27, "Output",
   CellTags->"GHP"],
  Cell[1021828, 42002, 264, 9, 28, "Input",
   CellTags->"GHP"],
  Cell[1022095, 42013, 216, 6, 42, "Output",
   CellTags->"GHP"]},
 "GluonField"->{
  Cell[1022338, 42023, 57, 1, 41, "ObjectName",
   CellTags->"GluonField"],
  Cell[1022420, 42028, 61, 1, 36, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1022484, 42031, 79, 1, 25, "Text",
   CellTags->"GluonField"],
  Cell[1022566, 42034, 187, 9, 25, "Text",
   CellTags->"GluonField"],
  Cell[1022790, 42048, 58, 1, 36, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1022873, 42053, 86, 2, 28, "Input",
   CellTags->"GluonField"],
  Cell[1022962, 42057, 106, 3, 27, "Output",
   CellTags->"GluonField"],
  Cell[1023105, 42065, 237, 6, 28, "Input",
   CellTags->"GluonField"],
  Cell[1023345, 42073, 316, 14, 27, "Output",
   CellTags->"GluonField"]},
 "GluonGhostVertex"->{
  Cell[1023688, 42091, 69, 1, 41, "ObjectName",
   CellTags->"GluonGhostVertex"],
  Cell[1023782, 42096, 67, 1, 36, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1023852, 42099, 422, 7, 61, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1024277, 42108, 583, 23, 25, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1024885, 42135, 131, 3, 28, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1025019, 42140, 324, 10, 27, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1025392, 42156, 64, 1, 36, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1025481, 42161, 348, 10, 28, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1025832, 42173, 517, 21, 30, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1027844, 42271, 148, 4, 28, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1027995, 42277, 333, 10, 42, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1028365, 42292, 191, 5, 28, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1028559, 42299, 521, 21, 32, "Output",
   CellTags->"GluonGhostVertex"]},
 "GluonPropagator"->{
  Cell[1029107, 42324, 67, 1, 41, "ObjectName",
   CellTags->"GluonPropagator"],
  Cell[1029199, 42329, 66, 1, 36, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1029268, 42332, 535, 10, 70, "Text",
   CellTags->"GluonPropagator"],
  Cell[1029806, 42344, 99, 1, 25, "Text",
   CellTags->"GluonPropagator"],
  Cell[1029930, 42349, 83, 2, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1030016, 42353, 125, 3, 27, "Output",
   CellTags->"GluonPropagator"],
  Cell[1030178, 42361, 129, 3, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1030310, 42366, 482, 13, 27, "Output",
   CellTags->"GluonPropagator"],
  Cell[1030807, 42382, 591, 24, 70, "Text",
   CellTags->"GluonPropagator"],
  Cell[1031435, 42411, 72, 1, 35, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1031532, 42416, 187, 5, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1031722, 42423, 233, 8, 27, "Output",
   CellTags->"GluonPropagator"],
  Cell[1032698, 42466, 271, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1032972, 42475, 1035, 39, 51, "Output",
   CellTags->"GluonPropagator"],
  Cell[1034044, 42519, 357, 10, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1034404, 42531, 2748, 104, 52, "Output",
   CellTags->"GluonPropagator"],
  Cell[1037189, 42640, 199, 6, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1037391, 42648, 271, 9, 43, "Output",
   CellTags->"GluonPropagator"],
  Cell[1037677, 42660, 149, 4, 25, "Text",
   CellTags->"GluonPropagator"],
  Cell[1037851, 42668, 242, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1038096, 42677, 1028, 39, 54, "Output",
   CellTags->"GluonPropagator"],
  Cell[1039161, 42721, 267, 9, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1039431, 42732, 314, 9, 42, "Output",
   CellTags->"GluonPropagator"],
  Cell[1039782, 42746, 275, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1040060, 42755, 1274, 46, 46, "Output",
   CellTags->"GluonPropagator"],
  Cell[1041371, 42806, 275, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1041649, 42815, 1283, 47, 42, "Output",
   CellTags->"GluonPropagator"],
  Cell[1042969, 42867, 275, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1043247, 42876, 1282, 46, 46, "Output",
   CellTags->"GluonPropagator"],
  Cell[1044566, 42927, 275, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1044844, 42936, 1274, 46, 46, "Output",
   CellTags->"GluonPropagator"],
  Cell[1046155, 42987, 275, 7, 28, "Input",
   CellTags->"GluonPropagator"],
  Cell[1046433, 42996, 2422, 88, 46, "Output",
   CellTags->"GluonPropagator"]},
 "GluonSelfEnergy"->{
  Cell[1048882, 43088, 67, 1, 41, "ObjectName",
   CellTags->"GluonSelfEnergy"],
  Cell[1048974, 43093, 66, 1, 36, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1049043, 43096, 122, 3, 25, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1049190, 43103, 129, 3, 28, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1049322, 43108, 803, 24, 27, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1050140, 43135, 591, 24, 25, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1050768, 43164, 72, 1, 35, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1050865, 43169, 252, 8, 28, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1051120, 43179, 2388, 91, 40, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1053545, 43275, 300, 9, 28, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1053848, 43286, 2755, 103, 46, "Output",
   CellTags->"GluonSelfEnergy"]},
 "GluonVertex"->{
  Cell[1056630, 43393, 59, 1, 41, "ObjectName",
   CellTags->"GluonVertex"],
  Cell[1056714, 43398, 62, 1, 36, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1056779, 43401, 88, 1, 25, "Text",
   CellTags->"GluonVertex"],
  Cell[1056870, 43404, 621, 10, 97, "Text",
   CellTags->"GluonVertex"],
  Cell[1057494, 43416, 91, 1, 25, "Text",
   CellTags->"GluonVertex"],
  Cell[1057610, 43421, 79, 2, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1057692, 43425, 117, 3, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1057846, 43433, 121, 3, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1057970, 43438, 378, 11, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1058363, 43452, 337, 14, 25, "Text",
   CellTags->"GluonVertex"],
  Cell[1058737, 43471, 59, 1, 36, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1058821, 43476, 338, 10, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1059162, 43488, 943, 42, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1060142, 43535, 379, 12, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1060524, 43549, 1972, 78, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1062533, 43632, 308, 10, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1062844, 43644, 272, 9, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1063153, 43658, 349, 12, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1063505, 43672, 1682, 65, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1065224, 43742, 414, 12, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1065641, 43756, 352, 11, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1069939, 43922, 374, 12, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1070316, 43936, 160, 5, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1070513, 43946, 259, 7, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1070775, 43955, 3470, 131, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1074260, 44089, 200, 4, 43, "Text",
   CellTags->"GluonVertex"],
  Cell[1074485, 44097, 172, 5, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1074660, 44104, 942, 42, 27, "Output",
   CellTags->"GluonVertex"],
  Cell[1075639, 44151, 230, 7, 28, "Input",
   CellTags->"GluonVertex"],
  Cell[1075872, 44160, 3422, 131, 27, "Output",
   CellTags->"GluonVertex"]},
 "GO"->{
  Cell[1079321, 44295, 41, 1, 41, "ObjectName",
   CellTags->"GO"],
  Cell[1079387, 44300, 53, 1, 36, "Subsubsection",
   CellTags->"GO"],
  Cell[1079443, 44303, 73, 1, 25, "Text",
   CellTags->"GO"],
  Cell[1079519, 44306, 206, 9, 25, "Text",
   CellTags->"GO"]},
 "GP"->{
  Cell[1079740, 44318, 41, 1, 41, "ObjectName",
   CellTags->"GP"],
  Cell[1079806, 44323, 53, 1, 36, "Subsubsection",
   CellTags->"GP"],
  Cell[1079862, 44326, 69, 1, 25, "Text",
   CellTags->"GP"],
  Cell[1079934, 44329, 194, 9, 25, "Text",
   CellTags->"GP"]},
 "GrassmannParity"->{
  Cell[1080143, 44341, 67, 1, 41, "ObjectName",
   CellTags->"GrassmannParity"],
  Cell[1080235, 44346, 66, 1, 36, "Subsubsection",
   CellTags->"GrassmannParity"],
  Cell[1080304, 44349, 216, 5, 43, "Text",
   CellTags->"GrassmannParity"],
  Cell[1080523, 44356, 186, 9, 25, "Text",
   CellTags->"GrassmannParity"]},
 "GS"->{
  Cell[1080724, 44368, 41, 1, 41, "ObjectName",
   CellTags->"GS"],
  Cell[1080790, 44373, 53, 1, 36, "Subsubsection",
   CellTags->"GS"],
  Cell[1080846, 44376, 635, 18, 43, "Text",
   CellTags->"GS"],
  Cell[1081484, 44396, 482, 24, 25, "Text",
   CellTags->"GS"],
  Cell[1082003, 44425, 50, 1, 36, "Subsubsection",
   CellTags->"GS"],
  Cell[1082078, 44430, 97, 3, 28, "Input",
   CellTags->"GS"],
  Cell[1082178, 44435, 139, 4, 27, "Output",
   CellTags->"GS"],
  Cell[1082354, 44444, 162, 6, 28, "Input",
   CellTags->"GS"],
  Cell[1082519, 44452, 155, 4, 42, "Output",
   CellTags->"GS"],
  Cell[1082711, 44461, 140, 4, 28, "Input",
   CellTags->"GS"],
  Cell[1082854, 44467, 412, 13, 27, "Output",
   CellTags->"GS"],
  Cell[1083303, 44485, 179, 6, 28, "Input",
   CellTags->"GS"],
  Cell[1083485, 44493, 242, 7, 42, "Output",
   CellTags->"GS"],
  Cell[1083764, 44505, 237, 8, 28, "Input",
   CellTags->"GS"],
  Cell[1084004, 44515, 361, 12, 27, "Output",
   CellTags->"GS"]},
 "GSD"->{
  Cell[1084392, 44531, 43, 1, 41, "ObjectName",
   CellTags->"GSD"],
  Cell[1084460, 44536, 54, 1, 36, "Subsubsection",
   CellTags->"GSD"],
  Cell[1084517, 44539, 644, 18, 43, "Text",
   CellTags->"GSD"],
  Cell[1085164, 44559, 483, 24, 25, "Text",
   CellTags->"GSD"],
  Cell[1085684, 44588, 51, 1, 36, "Subsubsection",
   CellTags->"GSD"],
  Cell[1085760, 44593, 99, 3, 28, "Input",
   CellTags->"GSD"],
  Cell[1085862, 44598, 140, 4, 27, "Output",
   CellTags->"GSD"],
  Cell[1086039, 44607, 164, 6, 28, "Input",
   CellTags->"GSD"],
  Cell[1086206, 44615, 205, 6, 42, "Output",
   CellTags->"GSD"],
  Cell[1086448, 44626, 142, 4, 28, "Input",
   CellTags->"GSD"],
  Cell[1086593, 44632, 413, 13, 27, "Output",
   CellTags->"GSD"],
  Cell[1087043, 44650, 181, 6, 28, "Input",
   CellTags->"GSD"],
  Cell[1087227, 44658, 247, 7, 42, "Output",
   CellTags->"GSD"],
  Cell[1087511, 44670, 241, 8, 28, "Input",
   CellTags->"GSD"],
  Cell[1087755, 44680, 362, 12, 27, "Output",
   CellTags->"GSD"]},
 "Gstrong"->{
  Cell[1088144, 44696, 51, 1, 41, "ObjectName",
   CellTags->"Gstrong"],
  Cell[1088220, 44701, 58, 1, 36, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1088281, 44704, 83, 1, 25, "Text",
   CellTags->"Gstrong"],
  Cell[1088367, 44707, 334, 15, 25, "Text",
   CellTags->"Gstrong"],
  Cell[1088738, 44727, 55, 1, 36, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1088796, 44730, 119, 3, 25, "Text",
   CellTags->"Gstrong"],
  Cell[1088940, 44737, 80, 2, 28, "Input",
   CellTags->"Gstrong"],
  Cell[1089023, 44741, 125, 4, 27, "Output",
   CellTags->"Gstrong"]},
 "GTI"->{
  Cell[1089175, 44749, 43, 1, 41, "ObjectName",
   CellTags->"GTI"],
  Cell[1089243, 44754, 54, 1, 36, "Subsubsection",
   CellTags->"GTI"],
  Cell[1089300, 44757, 85, 1, 25, "Text",
   CellTags->"GTI"],
  Cell[1089388, 44760, 159, 9, 25, "Text",
   CellTags->"GTI"]},
 "GV"->{
  Cell[1089562, 44772, 41, 1, 41, "ObjectName",
   CellTags->"GV"],
  Cell[1089628, 44777, 53, 1, 36, "Subsubsection",
   CellTags->"GV"],
  Cell[1089684, 44780, 65, 1, 25, "Text",
   CellTags->"GV"],
  Cell[1089752, 44783, 182, 9, 25, "Text",
   CellTags->"GV"]},
 "Hill"->{
  Cell[1089949, 44795, 45, 1, 41, "ObjectName",
   CellTags->"Hill"],
  Cell[1090019, 44800, 55, 1, 36, "Subsubsection",
   CellTags->"Hill"],
  Cell[1090077, 44803, 129, 4, 25, "Text",
   CellTags->"Hill"],
  Cell[1090209, 44809, 196, 9, 25, "Text",
   CellTags->"Hill"],
  Cell[1090442, 44823, 52, 1, 36, "Subsubsection",
   CellTags->"Hill"],
  Cell[1090519, 44828, 124, 4, 28, "Input",
   CellTags->"Hill"],
  Cell[1090646, 44834, 2639, 95, 83, "Output",
   CellTags->"Hill"],
  Cell[1093322, 44934, 270, 8, 28, "Input",
   CellTags->"Hill"],
  Cell[1093595, 44944, 100, 3, 27, "Output",
   CellTags->"Hill"],
  Cell[1093732, 44952, 262, 9, 28, "Input",
   CellTags->"Hill"],
  Cell[1093997, 44963, 1517, 57, 43, "Output",
   CellTags->"Hill"],
  Cell[1095551, 45025, 273, 9, 28, "Input",
   CellTags->"Hill"],
  Cell[1095827, 45036, 100, 3, 27, "Output",
   CellTags->"Hill"]},
 "HypergeometricAC"->{
  Cell[1095954, 45043, 69, 1, 41, "ObjectName",
   CellTags->"HypergeometricAC"],
  Cell[1096048, 45048, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1096118, 45051, 553, 9, 79, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1096696, 45064, 131, 3, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1096830, 45069, 177, 6, 27, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1097022, 45078, 598, 24, 25, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1097657, 45107, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1097724, 45110, 101, 1, 25, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1097850, 45115, 264, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1098117, 45124, 3978, 124, 48, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1102132, 45253, 264, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1102399, 45262, 3941, 124, 49, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1106377, 45391, 264, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1106644, 45400, 3997, 126, 49, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1110678, 45531, 264, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1110945, 45540, 4289, 133, 94, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1115271, 45678, 264, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1115538, 45687, 1560, 55, 36, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1117135, 45747, 265, 7, 28, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1117403, 45756, 1561, 55, 36, "Output",
   CellTags->"HypergeometricAC"]},
 "HypergeometricIR"->{
  Cell[1118991, 45815, 69, 1, 41, "ObjectName",
   CellTags->"HypergeometricIR"],
  Cell[1119085, 45820, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1119155, 45823, 269, 5, 43, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1119427, 45830, 479, 19, 25, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1119931, 45853, 131, 3, 28, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1120065, 45858, 182, 6, 27, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1120296, 45870, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1120385, 45875, 241, 7, 28, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1120629, 45884, 676, 22, 47, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1121342, 45911, 394, 14, 28, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1121739, 45927, 1737, 58, 44, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1123513, 45990, 148, 4, 28, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1123664, 45996, 352, 13, 27, "Output",
   CellTags->"HypergeometricIR"]},
 "HypergeometricSE"->{
  Cell[1124043, 46013, 69, 1, 41, "ObjectName",
   CellTags->"HypergeometricSE"],
  Cell[1124137, 46018, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1124207, 46021, 420, 13, 43, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1124652, 46038, 131, 3, 28, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1124786, 46043, 186, 6, 27, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1124987, 46052, 211, 9, 25, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1125235, 46066, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1125324, 46071, 245, 7, 28, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1125572, 46080, 679, 18, 44, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1126288, 46103, 351, 11, 28, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1126642, 46116, 955, 24, 44, "Output",
   CellTags->"HypergeometricSE"]},
 "HypExplicit"->{
  Cell[1127624, 46144, 59, 1, 41, "ObjectName",
   CellTags->"HypExplicit"],
  Cell[1127708, 46149, 62, 1, 36, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1127773, 46152, 325, 10, 43, "Text",
   CellTags->"HypExplicit"],
  Cell[1128101, 46164, 206, 9, 25, "Text",
   CellTags->"HypExplicit"],
  Cell[1128344, 46178, 59, 1, 36, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1128428, 46183, 164, 4, 28, "Input",
   CellTags->"HypExplicit"],
  Cell[1128595, 46189, 1241, 46, 27, "Output",
   CellTags->"HypExplicit"],
  Cell[1129873, 46240, 142, 4, 28, "Input",
   CellTags->"HypExplicit"],
  Cell[1130018, 46246, 674, 18, 44, "Output",
   CellTags->"HypExplicit"],
  Cell[1130729, 46269, 262, 8, 28, "Input",
   CellTags->"HypExplicit"],
  Cell[1130994, 46279, 1495, 55, 27, "Output",
   CellTags->"HypExplicit"],
  Cell[1132526, 46339, 142, 4, 28, "Input",
   CellTags->"HypExplicit"],
  Cell[1132671, 46345, 950, 24, 44, "Output",
   CellTags->"HypExplicit"]},
 "HypInt"->{
  Cell[1133648, 46373, 49, 1, 41, "ObjectName",
   CellTags->"HypInt"],
  Cell[1133722, 46378, 57, 1, 36, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1133782, 46381, 206, 5, 43, "Text",
   CellTags->"HypInt"],
  Cell[1134025, 46391, 54, 1, 36, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1134104, 46396, 159, 4, 28, "Input",
   CellTags->"HypInt"],
  Cell[1134266, 46402, 1236, 46, 27, "Output",
   CellTags->"HypInt"],
  Cell[1135539, 46453, 128, 4, 28, "Input",
   CellTags->"HypInt"],
  Cell[1135670, 46459, 981, 34, 50, "Output",
   CellTags->"HypInt"]},
 "IFPD"->{
  Cell[1136678, 46497, 45, 1, 41, "ObjectName",
   CellTags->"IFPD"],
  Cell[1136748, 46502, 55, 1, 36, "Subsubsection",
   CellTags->"IFPD"],
  Cell[1136806, 46505, 66, 1, 25, "Text",
   CellTags->"IFPD"],
  Cell[1136875, 46508, 214, 9, 25, "Text",
   CellTags->"IFPD"]},
 "IFPDOff"->{
  Cell[1137104, 46520, 51, 1, 41, "ObjectName",
   CellTags->"IFPDOff"],
  Cell[1137180, 46525, 58, 1, 36, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1137241, 46528, 182, 4, 43, "Text",
   CellTags->"IFPDOff"],
  Cell[1137426, 46534, 270, 14, 25, "Text",
   CellTags->"IFPDOff"],
  Cell[1137733, 46553, 55, 1, 36, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1137813, 46558, 163, 5, 28, "Input",
   CellTags->"IFPDOff"],
  Cell[1137979, 46565, 384, 16, 27, "Output",
   CellTags->"IFPDOff"],
  Cell[1138400, 46586, 108, 3, 28, "Input",
   CellTags->"IFPDOff"],
  Cell[1138511, 46591, 178, 5, 42, "Output",
   CellTags->"IFPDOff"],
  Cell[1138726, 46601, 130, 4, 28, "Input",
   CellTags->"IFPDOff"],
  Cell[1138859, 46607, 212, 8, 27, "Output",
   CellTags->"IFPDOff"],
  Cell[1139108, 46620, 108, 3, 28, "Input",
   CellTags->"IFPDOff"],
  Cell[1139219, 46625, 283, 9, 42, "Output",
   CellTags->"IFPDOff"]},
 "IFPDOn"->{
  Cell[1139529, 46638, 49, 1, 41, "ObjectName",
   CellTags->"IFPDOn"],
  Cell[1139603, 46643, 57, 1, 36, "Subsubsection",
   CellTags->"IFPDOn"],
  Cell[1139663, 46646, 317, 6, 61, "Text",
   CellTags->"IFPDOn"],
  Cell[1139983, 46654, 272, 14, 25, "Text",
   CellTags->"IFPDOn"]},
 "IncludePair"->{
  Cell[1140270, 46671, 59, 1, 41, "ObjectName",
   CellTags->"IncludePair"],
  Cell[1140354, 46676, 62, 1, 36, "Subsubsection",
   CellTags->"IncludePair"],
  Cell[1140419, 46679, 124, 3, 25, "Text",
   CellTags->"IncludePair"],
  Cell[1140546, 46684, 176, 9, 25, "Text",
   CellTags->"IncludePair"]},
 "IndexPosition"->{
  Cell[1140737, 46696, 63, 1, 41, "ObjectName",
   CellTags->"IndexPosition"],
  Cell[1140825, 46701, 64, 1, 36, "Subsubsection",
   CellTags->"IndexPosition"],
  Cell[1140892, 46704, 89, 1, 25, "Text",
   CellTags->"IndexPosition"],
  Cell[1140984, 46707, 199, 9, 25, "Text",
   CellTags->"IndexPosition"]},
 "Indices"->{
  Cell[1141198, 46719, 51, 1, 41, "ObjectName",
   CellTags->"Indices"],
  Cell[1141274, 46724, 58, 1, 36, "Subsubsection",
   CellTags->"Indices"],
  Cell[1141335, 46727, 205, 5, 43, "Text",
   CellTags->"Indices"],
  Cell[1141543, 46734, 193, 9, 25, "Text",
   CellTags->"Indices"]},
 "InitialFunction"->{
  Cell[1141751, 46746, 67, 1, 41, "ObjectName",
   CellTags->"InitialFunction"],
  Cell[1141843, 46751, 66, 1, 36, "Subsubsection",
   CellTags->"InitialFunction"],
  Cell[1141912, 46754, 183, 4, 25, "Text",
   CellTags->"InitialFunction"],
  Cell[1142098, 46760, 186, 9, 25, "Text",
   CellTags->"InitialFunction"]},
 "InitialSubstitutions"->{
  Cell[1142299, 46772, 77, 1, 41, "ObjectName",
   CellTags->"InitialSubstitutions"],
  Cell[1142401, 46777, 71, 1, 36, "Subsubsection",
   CellTags->"InitialSubstitutions"],
  Cell[1142475, 46780, 208, 4, 25, "Text",
   CellTags->"InitialSubstitutions"],
  Cell[1142686, 46786, 408, 19, 25, "Text",
   CellTags->"InitialSubstitutions"]},
 "InsideDiracTrace"->{
  Cell[1143109, 46808, 69, 1, 41, "ObjectName",
   CellTags->"InsideDiracTrace"],
  Cell[1143203, 46813, 67, 1, 36, "Subsubsection",
   CellTags->"InsideDiracTrace"],
  Cell[1143273, 46816, 498, 9, 61, "Text",
   CellTags->"InsideDiracTrace"],
  Cell[1143774, 46827, 318, 14, 25, "Text",
   CellTags->"InsideDiracTrace"]},
 "IntegralTable"->{
  Cell[1144107, 46844, 63, 1, 41, "ObjectName",
   CellTags->"IntegralTable"],
  Cell[1144195, 46849, 64, 1, 36, "Subsubsection",
   CellTags->"IntegralTable"],
  Cell[1144262, 46852, 233, 5, 25, "Text",
   CellTags->"IntegralTable"],
  Cell[1144498, 46859, 500, 19, 25, "Text",
   CellTags->"IntegralTable"]},
 "IntegrateByParts"->{
  Cell[1145013, 46881, 69, 1, 41, "ObjectName",
   CellTags->"IntegrateByParts"],
  Cell[1145107, 46886, 67, 1, 36, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1145177, 46889, 184, 4, 25, "Text",
   CellTags->"IntegrateByParts"],
  Cell[1145386, 46897, 131, 3, 28, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1145520, 46902, 172, 5, 27, "Output",
   CellTags->"IntegrateByParts"],
  Cell[1145741, 46913, 64, 1, 36, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1145830, 46918, 539, 19, 28, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1146372, 46939, 648, 23, 41, "Output",
   CellTags->"IntegrateByParts"]},
 "Integratedx"->{
  Cell[1147047, 46966, 59, 1, 41, "ObjectName",
   CellTags->"Integratedx"],
  Cell[1147131, 46971, 62, 1, 36, "Subsubsection",
   CellTags->"Integratedx"],
  Cell[1147196, 46974, 154, 4, 25, "Text",
   CellTags->"Integratedx"],
  Cell[1147353, 46980, 852, 34, 43, "Text",
   CellTags->"Integratedx"]},
 "Integrate2"->{
  Cell[1148220, 47017, 57, 1, 41, "ObjectName",
   CellTags->"Integrate2"],
  Cell[1148302, 47022, 61, 1, 36, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1148366, 47025, 866, 17, 97, "Text",
   CellTags->"Integrate2"],
  Cell[1149235, 47044, 508, 24, 25, "Text",
   CellTags->"Integrate2"],
  Cell[1149746, 47070, 405, 7, 61, "Text",
   CellTags->"Integrate2"],
  Cell[1150188, 47082, 58, 1, 36, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1150271, 47087, 425, 15, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1150699, 47104, 594, 22, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1151308, 47129, 190, 6, 25, "Text",
   CellTags->"Integrate2"],
  Cell[1151523, 47139, 281, 9, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1151807, 47150, 232, 8, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1152076, 47163, 276, 9, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1152355, 47174, 574, 22, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1152966, 47201, 336, 11, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1153305, 47214, 775, 28, 41, "Output",
   CellTags->"Integrate2"],
  Cell[1154117, 47247, 317, 10, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1154437, 47259, 134, 4, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1154586, 47266, 1589, 53, 55, "Text",
   CellTags->"Integrate2"],
  Cell[1156200, 47323, 270, 9, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1156473, 47334, 106, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1156594, 47340, 649, 24, 44, "Text",
   CellTags->"Integrate2"],
  Cell[1157268, 47368, 317, 10, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1157588, 47380, 106, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1157731, 47388, 386, 13, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1158120, 47403, 164, 5, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1158321, 47413, 417, 14, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1158741, 47429, 397, 16, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1159175, 47450, 328, 11, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1159506, 47463, 369, 15, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1159912, 47483, 354, 12, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1160269, 47497, 527, 20, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1160833, 47522, 313, 10, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1161149, 47534, 189, 6, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1161375, 47545, 281, 9, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1161659, 47556, 514, 20, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1162210, 47581, 445, 15, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1162658, 47598, 598, 21, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1163293, 47624, 721, 24, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1164017, 47650, 5640, 201, 129, "Output",
   CellTags->"Integrate2"],
  Cell[1169694, 47856, 471, 16, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1170168, 47874, 1013, 37, 42, "Output",
   CellTags->"Integrate2"],
  Cell[1171218, 47916, 567, 19, 39, "Input",
   CellTags->"Integrate2"],
  Cell[1171788, 47937, 6108, 208, 121, "Output",
   CellTags->"Integrate2"],
  Cell[1177933, 48150, 142, 4, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1178078, 48156, 2556, 85, 77, "Output",
   CellTags->"Integrate2"],
  Cell[1180671, 48246, 104, 3, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1180778, 48251, 124, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1181497, 48281, 684, 23, 39, "Input",
   CellTags->"Integrate2"],
  Cell[1182184, 48306, 4132, 157, 81, "Output",
   CellTags->"Integrate2"],
  Cell[1188141, 48543, 151, 4, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1188295, 48549, 109, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1188441, 48557, 368, 12, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1188812, 48571, 106, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1188933, 48577, 137, 4, 25, "Text",
   CellTags->"Integrate2"],
  Cell[1189095, 48585, 147, 4, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1189245, 48591, 5979, 193, 215, "Output",
   CellTags->"Integrate2"],
  Cell[1195261, 48789, 230, 8, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1195494, 48799, 174, 6, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1195683, 48808, 165, 4, 25, "Text",
   CellTags->"Integrate2"],
  Cell[1195873, 48816, 326, 11, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1196202, 48829, 175, 6, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1196392, 48838, 126, 4, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1196543, 48846, 317, 10, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1196863, 48858, 134, 4, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1197034, 48867, 325, 11, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1197362, 48880, 260, 9, 40, "Output",
   CellTags->"Integrate2"],
  Cell[1197659, 48894, 99, 3, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1197761, 48899, 125, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1197923, 48907, 325, 11, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1198251, 48920, 125, 3, 27, "Output",
   CellTags->"Integrate2"],
  Cell[1198413, 48928, 381, 13, 28, "Input",
   CellTags->"Integrate2"],
  Cell[1198797, 48943, 4862, 184, 125, "Output",
   CellTags->"Integrate2"]},
 "Integrate3"->{
  Cell[1204773, 49181, 57, 1, 70, "ObjectName",
   CellTags->"Integrate3"],
  Cell[1204855, 49186, 61, 1, 70, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1204919, 49189, 331, 6, 70, "Text",
   CellTags->"Integrate3"],
  Cell[1205253, 49197, 187, 9, 70, "Text",
   CellTags->"Integrate3"],
  Cell[1205477, 49211, 58, 1, 36, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1205560, 49216, 280, 9, 28, "Input",
   CellTags->"Integrate3"],
  Cell[1205843, 49227, 225, 8, 45, "Output",
   CellTags->"Integrate3"],
  Cell[1206105, 49240, 400, 13, 41, "Input",
   CellTags->"Integrate3"],
  Cell[1206508, 49255, 722, 29, 27, "Output",
   CellTags->"Integrate3"],
  Cell[1207267, 49289, 676, 23, 41, "Input",
   CellTags->"Integrate3"],
  Cell[1207946, 49314, 2758, 99, 70, "Output",
   CellTags->"Integrate3"],
  Cell[1210741, 49418, 266, 8, 28, "Input",
   CellTags->"Integrate3"],
  Cell[1211010, 49428, 106, 3, 27, "Output",
   CellTags->"Integrate3"],
  Cell[1211153, 49436, 322, 10, 28, "Input",
   CellTags->"Integrate3"],
  Cell[1211478, 49448, 195, 6, 27, "Output",
   CellTags->"Integrate3"],
  Cell[1211710, 49459, 270, 9, 28, "Input",
   CellTags->"Integrate3"],
  Cell[1211983, 49470, 106, 3, 27, "Output",
   CellTags->"Integrate3"]},
 "IntermediateSubstitutions"->{
  Cell[1212116, 49477, 87, 1, 70, "ObjectName",
   CellTags->"IntermediateSubstitutions"],
  Cell[1212228, 49482, 76, 1, 70, "Subsubsection",
   CellTags->"IntermediateSubstitutions"],
  Cell[1212307, 49485, 206, 4, 70, "Text",
   CellTags->"IntermediateSubstitutions"],
  Cell[1212516, 49491, 193, 9, 70, "Text",
   CellTags->"IntermediateSubstitutions"]},
 "InverseMellin"->{
  Cell[1212724, 49503, 63, 1, 70, "ObjectName",
   CellTags->"InverseMellin"],
  Cell[1212812, 49508, 64, 1, 70, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1212879, 49511, 531, 14, 70, "Text",
   CellTags->"InverseMellin"],
  Cell[1213413, 49527, 609, 29, 70, "Text",
   CellTags->"InverseMellin"],
  Cell[1214059, 49561, 61, 1, 36, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1214145, 49566, 169, 5, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1214317, 49573, 157, 5, 27, "Output",
   CellTags->"InverseMellin"],
  Cell[1214511, 49583, 220, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1214734, 49592, 157, 5, 27, "Output",
   CellTags->"InverseMellin"],
  Cell[1214928, 49602, 142, 4, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1215073, 49608, 280, 10, 27, "Output",
   CellTags->"InverseMellin"],
  Cell[1215390, 49623, 194, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1215587, 49631, 293, 11, 38, "Output",
   CellTags->"InverseMellin"],
  Cell[1215917, 49647, 194, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1216114, 49655, 360, 13, 27, "Output",
   CellTags->"InverseMellin"],
  Cell[1216511, 49673, 203, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1216717, 49682, 360, 13, 27, "Output",
   CellTags->"InverseMellin"],
  Cell[1217092, 49698, 113, 3, 25, "Text",
   CellTags->"InverseMellin"],
  Cell[1217230, 49705, 294, 9, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1217527, 49716, 131, 4, 39, "Output",
   CellTags->"InverseMellin"],
  Cell[1217673, 49723, 117, 3, 25, "Text",
   CellTags->"InverseMellin"],
  Cell[1217793, 49728, 3132, 100, 164, "Input",
   CellTags->"InverseMellin"],
  Cell[1220928, 49830, 244, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1221197, 49841, 179, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1221379, 49849, 398, 14, 40, "Output",
   CellTags->"InverseMellin"],
  Cell[1221814, 49868, 179, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1221996, 49876, 356, 12, 40, "Output",
   CellTags->"InverseMellin"],
  Cell[1222389, 49893, 172, 5, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1222564, 49900, 796, 28, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1223375, 49931, 81, 2, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1223481, 49937, 192, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1223676, 49945, 500, 19, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1224213, 49969, 270, 9, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1224486, 49980, 502, 18, 41, "Output",
   CellTags->"InverseMellin"],
  Cell[1225025, 50003, 270, 9, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1225298, 50014, 693, 24, 41, "Output",
   CellTags->"InverseMellin"],
  Cell[1226028, 50043, 267, 9, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1226298, 50054, 838, 30, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1227173, 50089, 246, 9, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1227422, 50100, 549, 20, 41, "Output",
   CellTags->"InverseMellin"],
  Cell[1228008, 50125, 274, 10, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1228285, 50137, 1140, 39, 45, "Output",
   CellTags->"InverseMellin"],
  Cell[1229462, 50181, 192, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1229657, 50189, 700, 25, 43, "Output",
   CellTags->"InverseMellin"],
  Cell[1230394, 50219, 221, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1230618, 50228, 841, 30, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1231496, 50263, 192, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1231691, 50271, 1047, 38, 45, "Output",
   CellTags->"InverseMellin"],
  Cell[1232775, 50314, 202, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1232980, 50322, 544, 20, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1233561, 50347, 223, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1233787, 50356, 1354, 48, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1235178, 50409, 223, 7, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1235404, 50418, 1299, 46, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1236740, 50469, 212, 6, 28, "Input",
   CellTags->"InverseMellin"],
  Cell[1236955, 50477, 632, 23, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1237602, 50503, 158, 5, 28, "Input",
   CellTags->"InverseMellin"]},
 "Isolate"->{
  Cell[1237775, 50511, 51, 1, 41, "ObjectName",
   CellTags->"Isolate"],
  Cell[1237851, 50516, 58, 1, 36, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1237912, 50519, 422, 7, 61, "Text",
   CellTags->"Isolate"],
  Cell[1238359, 50530, 114, 3, 28, "Input",
   CellTags->"Isolate"],
  Cell[1238476, 50535, 404, 12, 49, "Output",
   CellTags->"Isolate"],
  Cell[1238929, 50553, 55, 1, 36, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1239009, 50558, 156, 5, 28, "Input",
   CellTags->"Isolate"],
  Cell[1239168, 50565, 158, 6, 27, "Output",
   CellTags->"Isolate"],
  Cell[1239363, 50576, 378, 13, 28, "Input",
   CellTags->"Isolate"],
  Cell[1239744, 50591, 305, 12, 27, "Output",
   CellTags->"Isolate"],
  Cell[1240086, 50608, 114, 3, 28, "Input",
   CellTags->"Isolate"],
  Cell[1240203, 50613, 282, 11, 42, "Output",
   CellTags->"Isolate"],
  Cell[1240522, 50629, 182, 5, 28, "Input",
   CellTags->"Isolate"],
  Cell[1240707, 50636, 640, 25, 27, "Output",
   CellTags->"Isolate"],
  Cell[1241384, 50666, 561, 19, 28, "Input",
   CellTags->"Isolate"],
  Cell[1241948, 50687, 355, 14, 27, "Output",
   CellTags->"Isolate"],
  Cell[1242340, 50706, 94, 3, 28, "Input",
   CellTags->"Isolate"],
  Cell[1242437, 50711, 90, 1, 41, "Print",
   CellTags->{"Info3387712053-5834299", "Isolate"}],
  Cell[1242530, 50714, 1014, 30, 52, "Print",
   CellTags->{"Info3387712053-5834299", "Isolate"}],
  Cell[1243581, 50749, 278, 7, 28, "Input",
   CellTags->"Isolate"],
  Cell[1243862, 50758, 157, 6, 27, "Output",
   CellTags->"Isolate"],
  Cell[1244056, 50769, 178, 6, 28, "Input",
   CellTags->"Isolate"],
  Cell[1244237, 50777, 206, 7, 27, "Output",
   CellTags->"Isolate"],
  Cell[1244458, 50787, 152, 4, 28, "Input",
   CellTags->"Isolate"]},
 "Info3387712053-5834299"->{
  Cell[1242437, 50711, 90, 1, 41, "Print",
   CellTags->{"Info3387712053-5834299", "Isolate"}],
  Cell[1242530, 50714, 1014, 30, 52, "Print",
   CellTags->{"Info3387712053-5834299", "Isolate"}]},
 "IsolateHead"->{
  Cell[1244625, 50794, 59, 1, 41, "ObjectName",
   CellTags->"IsolateHead"],
  Cell[1244709, 50799, 62, 1, 36, "Subsubsection",
   CellTags->"IsolateHead"],
  Cell[1244774, 50802, 84, 1, 25, "Text",
   CellTags->"IsolateHead"],
  Cell[1244861, 50805, 194, 9, 25, "Text",
   CellTags->"IsolateHead"]},
 "IsolateNames"->{
  Cell[1245070, 50817, 61, 1, 41, "ObjectName",
   CellTags->"IsolateNames"],
  Cell[1245156, 50822, 63, 1, 36, "Subsubsection",
   CellTags->"IsolateNames"],
  Cell[1245222, 50825, 220, 5, 43, "Text",
   CellTags->"IsolateNames"],
  Cell[1245445, 50832, 290, 14, 25, "Text",
   CellTags->"IsolateNames"]},
 "IsolatePrint"->{
  Cell[1245750, 50849, 61, 1, 41, "ObjectName",
   CellTags->"IsolatePrint"],
  Cell[1245836, 50854, 63, 1, 36, "Subsubsection",
   CellTags->"IsolatePrint"],
  Cell[1245902, 50857, 284, 6, 43, "Text",
   CellTags->"IsolatePrint"],
  Cell[1246189, 50865, 180, 9, 25, "Text",
   CellTags->"IsolatePrint"]},
 "IsolateSplit"->{
  Cell[1246384, 50877, 61, 1, 41, "ObjectName",
   CellTags->"IsolateSplit"],
  Cell[1246470, 50882, 63, 1, 36, "Subsubsection",
   CellTags->"IsolateSplit"],
  Cell[1246536, 50885, 413, 7, 61, "Text",
   CellTags->"IsolateSplit"],
  Cell[1246952, 50894, 180, 9, 25, "Text",
   CellTags->"IsolateSplit"]},
 "KeepOnly"->{
  Cell[1247147, 50906, 53, 1, 41, "ObjectName",
   CellTags->"KeepOnly"],
  Cell[1247225, 50911, 59, 1, 36, "Subsubsection",
   CellTags->"KeepOnly"],
  Cell[1247287, 50914, 293, 6, 43, "Text",
   CellTags->"KeepOnly"],
  Cell[1247583, 50922, 452, 24, 25, "Text",
   CellTags->"KeepOnly"]},
 "KK"->{
  Cell[1248050, 50949, 41, 1, 41, "ObjectName",
   CellTags->"KK"],
  Cell[1248116, 50954, 53, 1, 36, "Subsubsection",
   CellTags->"KK"],
  Cell[1248172, 50957, 235, 5, 43, "Text",
   CellTags->"KK"],
  Cell[1248410, 50964, 292, 14, 25, "Text",
   CellTags->"KK"]},
 "Kummer"->{
  Cell[1248717, 50981, 49, 1, 41, "ObjectName",
   CellTags->"Kummer"],
  Cell[1248791, 50986, 57, 1, 36, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1248851, 50989, 260, 5, 43, "Text",
   CellTags->"Kummer"],
  Cell[1249114, 50996, 201, 9, 25, "Text",
   CellTags->"Kummer"],
  Cell[1249352, 51010, 54, 1, 36, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1249431, 51015, 337, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1249771, 51026, 2763, 98, 29, "Output",
   CellTags->"Kummer"],
  Cell[1252571, 51129, 337, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1252911, 51140, 2781, 99, 36, "Output",
   CellTags->"Kummer"],
  Cell[1255729, 51244, 337, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1256069, 51255, 2781, 99, 36, "Output",
   CellTags->"Kummer"],
  Cell[1258887, 51359, 388, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1259278, 51372, 2771, 98, 27, "Output",
   CellTags->"Kummer"],
  Cell[1262086, 51475, 479, 13, 28, "Input",
   CellTags->"Kummer"],
  Cell[1262568, 51490, 2864, 99, 27, "Output",
   CellTags->"Kummer"],
  Cell[1265469, 51594, 388, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1265860, 51607, 2758, 98, 40, "Output",
   CellTags->"Kummer"],
  Cell[1268655, 51710, 479, 13, 28, "Input",
   CellTags->"Kummer"],
  Cell[1269137, 51725, 2873, 100, 40, "Output",
   CellTags->"Kummer"],
  Cell[1272047, 51830, 388, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1272438, 51843, 2758, 98, 40, "Output",
   CellTags->"Kummer"],
  Cell[1275233, 51946, 479, 13, 28, "Input",
   CellTags->"Kummer"],
  Cell[1275715, 51961, 2873, 100, 40, "Output",
   CellTags->"Kummer"],
  Cell[1278625, 52066, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1279115, 52083, 2940, 104, 40, "Output",
   CellTags->"Kummer"],
  Cell[1282092, 52192, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1282724, 52213, 3068, 107, 40, "Output",
   CellTags->"Kummer"],
  Cell[1285829, 52325, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1286319, 52342, 2915, 104, 41, "Output",
   CellTags->"Kummer"],
  Cell[1289271, 52451, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1289903, 52472, 3043, 107, 41, "Output",
   CellTags->"Kummer"],
  Cell[1292983, 52584, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1293473, 52601, 2891, 103, 40, "Output",
   CellTags->"Kummer"],
  Cell[1296401, 52709, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1297033, 52730, 3100, 108, 40, "Output",
   CellTags->"Kummer"],
  Cell[1300170, 52843, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1300660, 52860, 2940, 104, 40, "Output",
   CellTags->"Kummer"],
  Cell[1303637, 52969, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1304269, 52990, 3125, 109, 40, "Output",
   CellTags->"Kummer"],
  Cell[1307431, 53104, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1307921, 53121, 2865, 102, 40, "Output",
   CellTags->"Kummer"],
  Cell[1310823, 53228, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1311455, 53249, 3050, 107, 40, "Output",
   CellTags->"Kummer"],
  Cell[1314542, 53361, 487, 15, 28, "Input",
   CellTags->"Kummer"],
  Cell[1315032, 53378, 2865, 102, 40, "Output",
   CellTags->"Kummer"],
  Cell[1317934, 53485, 629, 19, 50, "Input",
   CellTags->"Kummer"],
  Cell[1318566, 53506, 3167, 110, 40, "Output",
   CellTags->"Kummer"],
  Cell[1321770, 53621, 531, 15, 50, "Input",
   CellTags->"Kummer"],
  Cell[1322304, 53638, 2914, 102, 29, "Output",
   CellTags->"Kummer"],
  Cell[1325255, 53745, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1325596, 53756, 2763, 98, 29, "Output",
   CellTags->"Kummer"],
  Cell[1328396, 53859, 531, 15, 50, "Input",
   CellTags->"Kummer"],
  Cell[1328930, 53876, 2985, 104, 36, "Output",
   CellTags->"Kummer"],
  Cell[1331952, 53985, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1332293, 53996, 2753, 98, 36, "Output",
   CellTags->"Kummer"],
  Cell[1335083, 54099, 531, 15, 50, "Input",
   CellTags->"Kummer"],
  Cell[1335617, 54116, 2985, 104, 36, "Output",
   CellTags->"Kummer"],
  Cell[1338639, 54225, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1338980, 54236, 2753, 98, 36, "Output",
   CellTags->"Kummer"],
  Cell[1341770, 54339, 582, 17, 50, "Input",
   CellTags->"Kummer"],
  Cell[1342355, 54358, 2967, 103, 27, "Output",
   CellTags->"Kummer"],
  Cell[1345359, 54466, 389, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1345751, 54479, 2771, 98, 27, "Output",
   CellTags->"Kummer"],
  Cell[1348559, 54582, 582, 17, 50, "Input",
   CellTags->"Kummer"],
  Cell[1349144, 54601, 3083, 107, 40, "Output",
   CellTags->"Kummer"],
  Cell[1352264, 54713, 389, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1352656, 54726, 2816, 100, 40, "Output",
   CellTags->"Kummer"],
  Cell[1355509, 54831, 582, 17, 50, "Input",
   CellTags->"Kummer"],
  Cell[1356094, 54850, 3025, 105, 40, "Output",
   CellTags->"Kummer"],
  Cell[1359156, 54960, 389, 11, 28, "Input",
   CellTags->"Kummer"],
  Cell[1359548, 54973, 2758, 98, 40, "Output",
   CellTags->"Kummer"],
  Cell[1362343, 55076, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1362684, 55087, 5080, 165, 71, "Output",
   CellTags->"Kummer"],
  Cell[1367801, 55257, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1368142, 55268, 5159, 169, 48, "Output",
   CellTags->"Kummer"],
  Cell[1373338, 55442, 338, 9, 28, "Input",
   CellTags->"Kummer"],
  Cell[1373679, 55453, 5177, 169, 48, "Output",
   CellTags->"Kummer"]},
 "Lagrangian"->{
  Cell[1378883, 55626, 57, 1, 70, "ObjectName",
   CellTags->"Lagrangian"],
  Cell[1378965, 55631, 61, 1, 36, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1379029, 55634, 454, 9, 70, "Text",
   CellTags->"Lagrangian"],
  Cell[1379486, 55645, 181, 9, 25, "Text",
   CellTags->"Lagrangian"],
  Cell[1379704, 55659, 58, 1, 36, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1379787, 55664, 124, 3, 28, "Input",
   CellTags->"Lagrangian"],
  Cell[1379914, 55669, 332, 11, 38, "Output",
   CellTags->"Lagrangian"],
  Cell[1380261, 55683, 85, 1, 25, "Text",
   CellTags->"Lagrangian"],
  Cell[1380371, 55688, 124, 3, 28, "Input",
   CellTags->"Lagrangian"],
  Cell[1380498, 55693, 683, 23, 38, "Output",
   CellTags->"Lagrangian"],
  Cell[1381218, 55721, 124, 3, 28, "Input",
   CellTags->"Lagrangian"],
  Cell[1381345, 55726, 1006, 36, 38, "Output",
   CellTags->"Lagrangian"],
  Cell[1382388, 55767, 124, 3, 28, "Input",
   CellTags->"Lagrangian"],
  Cell[1382515, 55772, 621, 22, 28, "Output",
   CellTags->"Lagrangian"],
  Cell[1383173, 55799, 124, 3, 28, "Input",
   CellTags->"Lagrangian"],
  Cell[1383300, 55804, 664, 23, 28, "Output",
   CellTags->"Lagrangian"]},
 "LC"->{
  Cell[1383991, 55831, 41, 1, 41, "ObjectName",
   CellTags->"LC"],
  Cell[1384057, 55836, 53, 1, 36, "Subsubsection",
   CellTags->"LC"],
  Cell[1384113, 55839, 240, 5, 43, "Text",
   CellTags->"LC"],
  Cell[1384356, 55846, 274, 14, 25, "Text",
   CellTags->"LC"],
  Cell[1384667, 55865, 50, 1, 36, "Subsubsection",
   CellTags->"LC"],
  Cell[1384742, 55870, 164, 5, 28, "Input",
   CellTags->"LC"],
  Cell[1384909, 55877, 184, 6, 27, "Output",
   CellTags->"LC"],
  Cell[1385130, 55888, 95, 3, 28, "Input",
   CellTags->"LC"],
  Cell[1385228, 55893, 477, 21, 27, "Output",
   CellTags->"LC"],
  Cell[1385742, 55919, 104, 3, 28, "Input",
   CellTags->"LC"],
  Cell[1385849, 55924, 337, 8, 42, "Output",
   CellTags->"LC"],
  Cell[1386223, 55937, 181, 6, 28, "Input",
   CellTags->"LC"],
  Cell[1386407, 55945, 169, 5, 27, "Output",
   CellTags->"LC"],
  Cell[1386613, 55955, 130, 4, 28, "Input",
   CellTags->"LC"],
  Cell[1386746, 55961, 367, 10, 42, "Output",
   CellTags->"LC"],
  Cell[1387150, 55976, 381, 13, 28, "Input",
   CellTags->"LC"],
  Cell[1387534, 55991, 396, 16, 27, "Output",
   CellTags->"LC"]},
 "LCD"->{
  Cell[1387957, 56011, 43, 1, 41, "ObjectName",
   CellTags->"LCD"],
  Cell[1388025, 56016, 54, 1, 36, "Subsubsection",
   CellTags->"LCD"],
  Cell[1388082, 56019, 234, 5, 43, "Text",
   CellTags->"LCD"],
  Cell[1388319, 56026, 161, 4, 25, "Text",
   CellTags->"LCD"],
  Cell[1388483, 56032, 272, 14, 25, "Text",
   CellTags->"LCD"],
  Cell[1388792, 56051, 51, 1, 36, "Subsubsection",
   CellTags->"LCD"],
  Cell[1388846, 56054, 190, 6, 28, "Input",
   CellTags->"LCD"],
  Cell[1389061, 56064, 166, 5, 28, "Input",
   CellTags->"LCD"],
  Cell[1389230, 56071, 185, 6, 27, "Output",
   CellTags->"LCD"],
  Cell[1389452, 56082, 96, 3, 28, "Input",
   CellTags->"LCD"],
  Cell[1389551, 56087, 478, 21, 27, "Output",
   CellTags->"LCD"],
  Cell[1390066, 56113, 105, 3, 28, "Input",
   CellTags->"LCD"],
  Cell[1390174, 56118, 438, 12, 42, "Output",
   CellTags->"LCD"],
  Cell[1390649, 56135, 183, 6, 28, "Input",
   CellTags->"LCD"],
  Cell[1390835, 56143, 170, 5, 27, "Output",
   CellTags->"LCD"],
  Cell[1391042, 56153, 131, 4, 28, "Input",
   CellTags->"LCD"],
  Cell[1391176, 56159, 418, 12, 42, "Output",
   CellTags->"LCD"],
  Cell[1391631, 56176, 400, 13, 28, "Input",
   CellTags->"LCD"],
  Cell[1392034, 56191, 397, 15, 27, "Output",
   CellTags->"LCD"],
  Cell[1392468, 56211, 170, 5, 28, "Input",
   CellTags->"LCD"],
  Cell[1392641, 56218, 161, 5, 27, "Output",
   CellTags->"LCD"]},
 "LeftPartialD"->{
  Cell[1392829, 56227, 61, 1, 41, "ObjectName",
   CellTags->"LeftPartialD"],
  Cell[1392915, 56232, 63, 1, 36, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1392981, 56235, 307, 12, 31, "Text",
   CellTags->"LeftPartialD"],
  Cell[1393291, 56249, 573, 24, 25, "Text",
   CellTags->"LeftPartialD"],
  Cell[1393901, 56278, 69, 1, 35, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1393995, 56283, 253, 7, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1394251, 56292, 306, 11, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1394594, 56308, 120, 3, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1394717, 56313, 355, 14, 27, "Output",
   CellTags->"LeftPartialD"],
  Cell[1395109, 56332, 118, 3, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1395230, 56337, 291, 7, 42, "Output",
   CellTags->"LeftPartialD"],
  Cell[1395558, 56349, 161, 4, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1395722, 56355, 176, 4, 42, "Output",
   CellTags->"LeftPartialD"],
  Cell[1395935, 56364, 372, 10, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1396310, 56376, 344, 12, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1396691, 56393, 120, 3, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1396814, 56398, 774, 30, 27, "Output",
   CellTags->"LeftPartialD"],
  Cell[1397625, 56433, 118, 3, 28, "Input",
   CellTags->"LeftPartialD"],
  Cell[1397746, 56438, 799, 21, 62, "Output",
   CellTags->"LeftPartialD"]},
 "LeftRightPartialD"->{
  Cell[1398572, 56463, 71, 1, 41, "ObjectName",
   CellTags->"LeftRightPartialD"],
  Cell[1398668, 56468, 68, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1398739, 56471, 592, 21, 49, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1399334, 56494, 831, 33, 25, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1400202, 56532, 65, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1400292, 56537, 132, 3, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1400427, 56542, 260, 9, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1400724, 56556, 127, 3, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1400854, 56561, 527, 19, 38, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1401418, 56585, 263, 7, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1401684, 56594, 316, 11, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1402037, 56610, 125, 3, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1402165, 56615, 708, 26, 38, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1402910, 56646, 382, 10, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1403295, 56658, 354, 12, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1403686, 56675, 125, 3, 28, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1403814, 56680, 902, 33, 38, "Output",
   CellTags->"LeftRightPartialD"]},
 "LeftRightPartialD2"->{
  Cell[1404743, 56717, 73, 1, 70, "ObjectName",
   CellTags->"LeftRightPartialD2"],
  Cell[1404841, 56722, 69, 1, 70, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1404913, 56725, 591, 21, 70, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1405507, 56748, 698, 29, 70, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1406242, 56782, 66, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1406333, 56787, 134, 3, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1406470, 56792, 261, 9, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1406768, 56806, 128, 3, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1406899, 56811, 433, 16, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1407369, 56832, 265, 7, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1407637, 56841, 317, 11, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1407991, 56857, 126, 3, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1408120, 56862, 600, 23, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1408757, 56890, 384, 10, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1409144, 56902, 355, 12, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1409536, 56919, 126, 3, 28, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1409665, 56924, 780, 30, 27, "Output",
   CellTags->"LeftRightPartialD2"]},
 "LeviCivita"->{
  Cell[1410472, 56958, 58, 1, 70, "ObjectName",
   CellTags->"LeviCivita"],
  Cell[1410555, 56963, 61, 1, 36, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1410619, 56966, 1212, 41, 79, "Text",
   CellTags->"LeviCivita"],
  Cell[1411856, 57011, 120, 3, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1411979, 57016, 168, 5, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1412162, 57024, 463, 24, 25, "Text",
   CellTags->"LeviCivita"],
  Cell[1412662, 57053, 58, 1, 36, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1412745, 57058, 187, 5, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1412935, 57065, 492, 21, 28, "Output",
   CellTags->"LeviCivita"],
  Cell[1413464, 57091, 204, 6, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1413671, 57099, 411, 17, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1414119, 57121, 180, 5, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1414302, 57128, 313, 13, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1414652, 57146, 186, 5, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1414841, 57153, 107, 3, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1414985, 57161, 242, 7, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1415230, 57170, 332, 8, 42, "Output",
   CellTags->"LeviCivita"],
  Cell[1415599, 57183, 365, 10, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1415967, 57195, 932, 40, 28, "Output",
   CellTags->"LeviCivita"],
  Cell[1416936, 57240, 113, 3, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1417052, 57245, 126, 4, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1417193, 57252, 316, 9, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1417534, 57265, 403, 11, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1417940, 57278, 297, 10, 27, "Output",
   CellTags->"LeviCivita"],
  Cell[1418252, 57291, 316, 9, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1418593, 57304, 179, 5, 28, "Input",
   CellTags->"LeviCivita"],
  Cell[1418775, 57311, 199, 6, 28, "Output",
   CellTags->"LeviCivita"],
  Cell[1418989, 57320, 110, 3, 28, "Input",
   CellTags->"LeviCivita"]},
 "LeviCivitaSign"->{
  Cell[1419114, 57326, 65, 1, 41, "ObjectName",
   CellTags->"LeviCivitaSign"],
  Cell[1419204, 57331, 65, 1, 36, "Subsubsection",
   CellTags->"LeviCivitaSign"],
  Cell[1419272, 57334, 203, 4, 43, "Text",
   CellTags->"LeviCivitaSign"],
  Cell[1419478, 57340, 310, 14, 25, "Text",
   CellTags->"LeviCivitaSign"]},
 "ListIntegrals"->{
  Cell[1419803, 57357, 81, 1, 41, "ObjectName",
   CellTags->"ListIntegrals"],
  Cell[1419909, 57362, 64, 1, 36, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1419976, 57365, 470, 8, 79, "Text",
   CellTags->"ListIntegrals"],
  Cell[1420483, 57378, 61, 1, 36, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1420547, 57381, 80, 2, 28, "Input",
   CellTags->"ListIntegrals"]},
 "Li2"->{
  Cell[1420642, 57386, 44, 1, 41, "ObjectName",
   CellTags->"Li2"],
  Cell[1420711, 57391, 54, 1, 36, "Subsubsection",
   CellTags->"Li2"],
  Cell[1420768, 57394, 117, 3, 25, "Text",
   CellTags->"Li2"],
  Cell[1420888, 57399, 153, 8, 25, "Text",
   CellTags->"Li2"],
  Cell[1421078, 57412, 51, 1, 36, "Subsubsection",
   CellTags->"Li2"],
  Cell[1421154, 57417, 100, 3, 28, "Input",
   CellTags->"Li2"],
  Cell[1421257, 57422, 236, 9, 27, "Output",
   CellTags->"Li2"],
  Cell[1421530, 57436, 107, 3, 28, "Input",
   CellTags->"Li2"],
  Cell[1421640, 57441, 162, 5, 42, "Output",
   CellTags->"Li2"],
  Cell[1421839, 57451, 253, 9, 40, "Input",
   CellTags->"Li2"],
  Cell[1422095, 57462, 236, 9, 27, "Output",
   CellTags->"Li2"]},
 "Li3"->{
  Cell[1422358, 57475, 44, 1, 41, "ObjectName",
   CellTags->"Li3"],
  Cell[1422427, 57480, 54, 1, 36, "Subsubsection",
   CellTags->"Li3"],
  Cell[1422484, 57483, 118, 3, 25, "Text",
   CellTags->"Li3"],
  Cell[1422605, 57488, 153, 8, 25, "Text",
   CellTags->"Li3"],
  Cell[1422795, 57501, 51, 1, 36, "Subsubsection",
   CellTags->"Li3"],
  Cell[1422871, 57506, 100, 3, 28, "Input",
   CellTags->"Li3"],
  Cell[1422974, 57511, 236, 9, 27, "Output",
   CellTags->"Li3"],
  Cell[1423247, 57525, 107, 3, 28, "Input",
   CellTags->"Li3"],
  Cell[1423357, 57530, 162, 5, 42, "Output",
   CellTags->"Li3"],
  Cell[1423556, 57540, 152, 5, 28, "Input",
   CellTags->"Li3"],
  Cell[1423711, 57547, 266, 11, 40, "Output",
   CellTags->"Li3"],
  Cell[1424014, 57563, 204, 7, 41, "Input",
   CellTags->"Li3"],
  Cell[1424221, 57572, 236, 9, 27, "Output",
   CellTags->"Li3"]},
 "Loop"->{
  Cell[1424484, 57585, 45, 1, 41, "ObjectName",
   CellTags->"Loop"],
  Cell[1424554, 57590, 55, 1, 36, "Subsubsection",
   CellTags->"Loop"],
  Cell[1424612, 57593, 94, 1, 25, "Text",
   CellTags->"Loop"]},
 "LorentzIndex"->{
  Cell[1424721, 57597, 61, 1, 41, "ObjectName",
   CellTags->"LorentzIndex"],
  Cell[1424807, 57602, 63, 1, 36, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1424873, 57605, 243, 5, 43, "Text",
   CellTags->"LorentzIndex"],
  Cell[1425119, 57612, 314, 14, 25, "Text",
   CellTags->"LorentzIndex"],
  Cell[1425470, 57631, 60, 1, 36, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1425533, 57634, 89, 1, 25, "Text",
   CellTags->"LorentzIndex"],
  Cell[1425647, 57639, 125, 3, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1425775, 57644, 116, 3, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1425906, 57650, 125, 3, 25, "Text",
   CellTags->"LorentzIndex"],
  Cell[1426056, 57657, 148, 4, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1426207, 57663, 116, 3, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1426360, 57671, 106, 2, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1426469, 57675, 140, 4, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1426646, 57684, 165, 4, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1426814, 57690, 113, 3, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1426964, 57698, 207, 5, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1427174, 57705, 139, 4, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1427328, 57712, 141, 4, 25, "Text",
   CellTags->"LorentzIndex"],
  Cell[1427472, 57718, 137, 4, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1427634, 57726, 347, 10, 28, "Input",
   CellTags->"LorentzIndex"],
  Cell[1427984, 57738, 385, 16, 27, "Output",
   CellTags->"LorentzIndex"],
  Cell[1428384, 57757, 148, 4, 28, "Input",
   CellTags->"LorentzIndex"]},
 "Lower"->{
  Cell[1428547, 57764, 47, 1, 41, "ObjectName",
   CellTags->"Lower"],
  Cell[1428619, 57769, 56, 1, 36, "Subsubsection",
   CellTags->"Lower"],
  Cell[1428678, 57772, 120, 3, 25, "Text",
   CellTags->"Lower"],
  Cell[1428801, 57777, 402, 19, 25, "Text",
   CellTags->"Lower"]},
 "MakeContext"->{
  Cell[1429218, 57799, 59, 1, 41, "ObjectName",
   CellTags->"MakeContext"],
  Cell[1429302, 57804, 62, 1, 36, "Subsubsection",
   CellTags->"MakeContext"],
  Cell[1429367, 57807, 238, 5, 43, "Text",
   CellTags->"MakeContext"],
  Cell[1429608, 57814, 530, 24, 25, "Text",
   CellTags->"MakeContext"]},
 "Mandelstam"->{
  Cell[1430153, 57841, 57, 1, 41, "ObjectName",
   CellTags->"Mandelstam"],
  Cell[1430235, 57846, 61, 1, 36, "Subsubsection",
   CellTags->"Mandelstam"],
  Cell[1430299, 57849, 1575, 73, 61, "Text",
   CellTags->"Mandelstam"],
  Cell[1431877, 57924, 758, 34, 25, "Text",
   CellTags->"Mandelstam"]},
 "Map2"->{
  Cell[1432650, 57961, 45, 1, 41, "ObjectName",
   CellTags->"Map2"],
  Cell[1432720, 57966, 55, 1, 36, "Subsubsection",
   CellTags->"Map2"],
  Cell[1432778, 57969, 135, 4, 25, "Text",
   CellTags->"Map2"],
  Cell[1432916, 57975, 169, 9, 25, "Text",
   CellTags->"Map2"],
  Cell[1433122, 57989, 52, 1, 36, "Subsubsection",
   CellTags->"Map2"],
  Cell[1433199, 57994, 149, 5, 28, "Input",
   CellTags->"Map2"],
  Cell[1433351, 58001, 202, 7, 27, "Output",
   CellTags->"Map2"],
  Cell[1433590, 58013, 125, 4, 28, "Input",
   CellTags->"Map2"],
  Cell[1433718, 58019, 129, 4, 27, "Output",
   CellTags->"Map2"],
  Cell[1433884, 58028, 184, 6, 28, "Input",
   CellTags->"Map2"],
  Cell[1434071, 58036, 191, 7, 27, "Output",
   CellTags->"Map2"],
  Cell[1434299, 58048, 125, 4, 28, "Input",
   CellTags->"Map2"],
  Cell[1434427, 58054, 129, 4, 27, "Output",
   CellTags->"Map2"]},
 "MemoryAvailable"->{
  Cell[1434583, 58062, 67, 1, 41, "ObjectName",
   CellTags->"MemoryAvailable"],
  Cell[1434675, 58067, 66, 1, 36, "Subsubsection",
   CellTags->"MemoryAvailable"],
  Cell[1434744, 58070, 230, 5, 43, "Text",
   CellTags->"MemoryAvailable"],
  Cell[1434977, 58077, 314, 14, 25, "Text",
   CellTags->"MemoryAvailable"]},
 "MemSet"->{
  Cell[1435306, 58094, 49, 1, 41, "ObjectName",
   CellTags->"MemSet"],
  Cell[1435380, 58099, 57, 1, 36, "Subsubsection",
   CellTags->"MemSet"],
  Cell[1435440, 58102, 345, 9, 43, "Text",
   CellTags->"MemSet"],
  Cell[1435788, 58113, 198, 9, 25, "Text",
   CellTags->"MemSet"]},
 "MetricTensor"->{
  Cell[1436001, 58125, 61, 1, 41, "ObjectName",
   CellTags->"MetricTensor"],
  Cell[1436087, 58130, 63, 1, 36, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1436153, 58133, 214, 7, 25, "Text",
   CellTags->"MetricTensor"],
  Cell[1436370, 58142, 584, 29, 25, "Text",
   CellTags->"MetricTensor"],
  Cell[1436991, 58176, 60, 1, 36, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1437076, 58181, 154, 4, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1437233, 58187, 165, 5, 28, "Output",
   CellTags->"MetricTensor"],
  Cell[1437435, 58197, 137, 4, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1437575, 58203, 109, 3, 27, "Output",
   CellTags->"MetricTensor"],
  Cell[1437721, 58211, 202, 5, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1437926, 58218, 165, 5, 28, "Output",
   CellTags->"MetricTensor"],
  Cell[1438128, 58228, 137, 4, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1438268, 58234, 109, 3, 27, "Output",
   CellTags->"MetricTensor"],
  Cell[1438414, 58242, 181, 5, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1438598, 58249, 228, 6, 42, "Output",
   CellTags->"MetricTensor"],
  Cell[1438863, 58260, 230, 6, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1439096, 58268, 278, 8, 42, "Output",
   CellTags->"MetricTensor"],
  Cell[1439411, 58281, 213, 6, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1439627, 58289, 146, 4, 42, "Output",
   CellTags->"MetricTensor"],
  Cell[1439810, 58298, 263, 7, 28, "Input",
   CellTags->"MetricTensor"],
  Cell[1440076, 58307, 147, 4, 42, "Output",
   CellTags->"MetricTensor"]},
 "MLimit"->{
  Cell[1440250, 58315, 49, 1, 41, "ObjectName",
   CellTags->"MLimit"],
  Cell[1440324, 58320, 57, 1, 36, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1440384, 58323, 116, 3, 25, "Text",
   CellTags->"MLimit"],
  Cell[1440537, 58331, 54, 1, 36, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1440616, 58336, 419, 14, 28, "Input",
   CellTags->"MLimit"],
  Cell[1441038, 58352, 430, 14, 43, "Output",
   CellTags->"MLimit"]},
 "Momentum"->{
  Cell[1441495, 58370, 53, 1, 41, "ObjectName",
   CellTags->"Momentum"],
  Cell[1441573, 58375, 59, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1441635, 58378, 258, 5, 43, "Text",
   CellTags->"Momentum"],
  Cell[1441896, 58385, 530, 24, 25, "Text",
   CellTags->"Momentum"],
  Cell[1442463, 58414, 56, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1442522, 58417, 75, 1, 25, "Text",
   CellTags->"Momentum"],
  Cell[1442622, 58422, 110, 3, 28, "Input",
   CellTags->"Momentum"],
  Cell[1442735, 58427, 105, 3, 27, "Output",
   CellTags->"Momentum"],
  Cell[1442855, 58433, 138, 4, 25, "Text",
   CellTags->"Momentum"],
  Cell[1443018, 58441, 133, 4, 28, "Input",
   CellTags->"Momentum"],
  Cell[1443154, 58447, 105, 3, 27, "Output",
   CellTags->"Momentum"],
  Cell[1443274, 58453, 86, 1, 25, "Text",
   CellTags->"Momentum"],
  Cell[1443385, 58458, 169, 5, 28, "Input",
   CellTags->"Momentum"],
  Cell[1443557, 58465, 148, 4, 42, "Output",
   CellTags->"Momentum"],
  Cell[1443742, 58474, 153, 5, 28, "Input",
   CellTags->"Momentum"],
  Cell[1443898, 58481, 123, 4, 27, "Output",
   CellTags->"Momentum"],
  Cell[1444058, 58490, 111, 3, 28, "Input",
   CellTags->"Momentum"],
  Cell[1444172, 58495, 143, 4, 42, "Output",
   CellTags->"Momentum"],
  Cell[1444352, 58504, 259, 8, 28, "Input",
   CellTags->"Momentum"],
  Cell[1444614, 58514, 201, 7, 27, "Output",
   CellTags->"Momentum"],
  Cell[1444852, 58526, 111, 3, 28, "Input",
   CellTags->"Momentum"],
  Cell[1444966, 58531, 231, 7, 42, "Output",
   CellTags->"Momentum"],
  Cell[1445234, 58543, 148, 4, 28, "Input",
   CellTags->"Momentum"],
  Cell[1445385, 58549, 183, 5, 42, "Output",
   CellTags->"Momentum"],
  Cell[1445605, 58559, 149, 4, 28, "Input",
   CellTags->"Momentum"],
  Cell[1445757, 58565, 231, 7, 42, "Output",
   CellTags->"Momentum"],
  Cell[1446025, 58577, 216, 7, 28, "Input",
   CellTags->"Momentum"],
  Cell[1446244, 58586, 148, 4, 42, "Output",
   CellTags->"Momentum"],
  Cell[1446407, 58593, 132, 4, 28, "Input",
   CellTags->"Momentum"]},
 "MomentumCombine"->{
  Cell[1446554, 58600, 67, 1, 41, "ObjectName",
   CellTags->"MomentumCombine"],
  Cell[1446646, 58605, 66, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1446715, 58608, 183, 4, 43, "Text",
   CellTags->"MomentumCombine"],
  Cell[1446901, 58614, 585, 23, 25, "Text",
   CellTags->"MomentumCombine"],
  Cell[1447523, 58642, 63, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1447611, 58647, 303, 9, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1447917, 58658, 214, 6, 42, "Output",
   CellTags->"MomentumCombine"],
  Cell[1448168, 58669, 288, 9, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1448459, 58680, 218, 7, 27, "Output",
   CellTags->"MomentumCombine"],
  Cell[1448714, 58692, 125, 3, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1448842, 58697, 218, 7, 27, "Output",
   CellTags->"MomentumCombine"],
  Cell[1449097, 58709, 163, 5, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1449263, 58716, 259, 8, 42, "Output",
   CellTags->"MomentumCombine"],
  Cell[1449559, 58729, 170, 5, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1449732, 58736, 218, 7, 27, "Output",
   CellTags->"MomentumCombine"],
  Cell[1449987, 58748, 121, 3, 28, "Input",
   CellTags->"MomentumCombine"],
  Cell[1450111, 58753, 259, 8, 42, "Output",
   CellTags->"MomentumCombine"],
  Cell[1450385, 58764, 115, 3, 28, "Input",
   CellTags->"MomentumCombine"]},
 "MomentumCombine2"->{
  Cell[1450515, 58770, 69, 1, 41, "ObjectName",
   CellTags->"MomentumCombine2"],
  Cell[1450609, 58775, 67, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1450679, 58778, 192, 4, 43, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1450874, 58784, 589, 23, 25, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1451500, 58812, 64, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1451589, 58817, 434, 13, 28, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1452026, 58832, 476, 20, 27, "Output",
   CellTags->"MomentumCombine2"],
  Cell[1452539, 58857, 164, 5, 28, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1452706, 58864, 309, 9, 42, "Output",
   CellTags->"MomentumCombine2"]},
 "MomentumExpand"->{
  Cell[1453042, 58877, 65, 1, 41, "ObjectName",
   CellTags->"MomentumExpand"],
  Cell[1453132, 58882, 65, 1, 36, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1453200, 58885, 149, 4, 25, "Text",
   CellTags->"MomentumExpand"],
  Cell[1453352, 58891, 349, 14, 25, "Text",
   CellTags->"MomentumExpand"],
  Cell[1453738, 58910, 62, 1, 36, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1453825, 58915, 218, 6, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1454046, 58923, 189, 5, 42, "Output",
   CellTags->"MomentumExpand"],
  Cell[1454272, 58933, 168, 5, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1454443, 58940, 248, 9, 27, "Output",
   CellTags->"MomentumExpand"],
  Cell[1454728, 58954, 116, 3, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1454847, 58959, 247, 7, 42, "Output",
   CellTags->"MomentumExpand"],
  Cell[1455131, 58971, 211, 6, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1455345, 58979, 358, 15, 27, "Output",
   CellTags->"MomentumExpand"],
  Cell[1455740, 58999, 116, 3, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1455859, 59004, 284, 8, 42, "Output",
   CellTags->"MomentumExpand"],
  Cell[1456180, 59017, 236, 7, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1456419, 59026, 384, 16, 27, "Output",
   CellTags->"MomentumExpand"],
  Cell[1456840, 59047, 116, 3, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1456959, 59052, 367, 11, 42, "Output",
   CellTags->"MomentumExpand"],
  Cell[1457363, 59068, 112, 3, 28, "Input",
   CellTags->"MomentumExpand"],
  Cell[1457478, 59073, 558, 23, 27, "Output",
   CellTags->"MomentumExpand"]},
 "MT"->{
  Cell[1458063, 59100, 41, 1, 41, "ObjectName",
   CellTags->"MT"],
  Cell[1458129, 59105, 53, 1, 36, "Subsubsection",
   CellTags->"MT"],
  Cell[1458185, 59108, 182, 7, 25, "Text",
   CellTags->"MT"],
  Cell[1458370, 59117, 604, 29, 25, "Text",
   CellTags->"MT"],
  Cell[1459011, 59151, 50, 1, 36, "Subsubsection",
   CellTags->"MT"],
  Cell[1459086, 59156, 134, 4, 28, "Input",
   CellTags->"MT"],
  Cell[1459223, 59162, 155, 5, 28, "Output",
   CellTags->"MT"],
  Cell[1459415, 59172, 263, 8, 28, "Input",
   CellTags->"MT"],
  Cell[1459681, 59182, 99, 3, 27, "Output",
   CellTags->"MT"],
  Cell[1459817, 59190, 157, 5, 28, "Input",
   CellTags->"MT"],
  Cell[1459977, 59197, 136, 4, 42, "Output",
   CellTags->"MT"],
  Cell[1460150, 59206, 189, 6, 28, "Input",
   CellTags->"MT"],
  Cell[1460342, 59214, 218, 6, 42, "Output",
   CellTags->"MT"],
  Cell[1460597, 59225, 225, 8, 28, "Input",
   CellTags->"MT"],
  Cell[1460825, 59235, 136, 4, 42, "Output",
   CellTags->"MT"]},
 "MTD"->{
  Cell[1460988, 59243, 43, 1, 41, "ObjectName",
   CellTags->"MTD"],
  Cell[1461056, 59248, 54, 1, 36, "Subsubsection",
   CellTags->"MTD"],
  Cell[1461113, 59251, 184, 7, 25, "Text",
   CellTags->"MTD"],
  Cell[1461300, 59260, 602, 29, 25, "Text",
   CellTags->"MTD"],
  Cell[1461939, 59294, 51, 1, 36, "Subsubsection",
   CellTags->"MTD"],
  Cell[1462015, 59299, 136, 4, 28, "Input",
   CellTags->"MTD"],
  Cell[1462154, 59305, 156, 5, 28, "Output",
   CellTags->"MTD"],
  Cell[1462347, 59315, 266, 8, 28, "Input",
   CellTags->"MTD"],
  Cell[1462616, 59325, 100, 3, 27, "Output",
   CellTags->"MTD"],
  Cell[1462753, 59333, 175, 6, 28, "Input",
   CellTags->"MTD"],
  Cell[1462931, 59341, 151, 4, 42, "Output",
   CellTags->"MTD"],
  Cell[1463119, 59350, 207, 7, 28, "Input",
   CellTags->"MTD"],
  Cell[1463329, 59359, 282, 8, 42, "Output",
   CellTags->"MTD"],
  Cell[1463648, 59372, 235, 8, 28, "Input",
   CellTags->"MTD"],
  Cell[1463886, 59382, 146, 4, 42, "Output",
   CellTags->"MTD"]},
 "NegativeInteger"->{
  Cell[1464059, 59390, 67, 1, 41, "ObjectName",
   CellTags->"NegativeInteger"],
  Cell[1464151, 59395, 66, 1, 36, "Subsubsection",
   CellTags->"NegativeInteger"],
  Cell[1464220, 59398, 141, 4, 25, "Text",
   CellTags->"NegativeInteger"],
  Cell[1464364, 59404, 186, 9, 25, "Text",
   CellTags->"NegativeInteger"]},
 "Nf"->{
  Cell[1464565, 59416, 41, 1, 41, "ObjectName",
   CellTags->"Nf"],
  Cell[1464631, 59421, 53, 1, 36, "Subsubsection",
   CellTags->"Nf"],
  Cell[1464687, 59424, 66, 1, 25, "Text",
   CellTags->"Nf"],
  Cell[1464756, 59427, 685, 29, 25, "Text",
   CellTags->"Nf"]},
 "Nielsen"->{
  Cell[1465456, 59459, 51, 1, 41, "ObjectName",
   CellTags->"Nielsen"],
  Cell[1465532, 59464, 58, 1, 36, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1465593, 59467, 86, 1, 25, "Text",
   CellTags->"Nielsen"],
  Cell[1465704, 59472, 114, 3, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1465821, 59477, 169, 6, 27, "Output",
   CellTags->"Nielsen"],
  Cell[1466005, 59486, 199, 9, 25, "Text",
   CellTags->"Nielsen"],
  Cell[1466241, 59500, 55, 1, 36, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1466321, 59505, 141, 4, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1466465, 59511, 251, 10, 27, "Output",
   CellTags->"Nielsen"],
  Cell[1466731, 59524, 176, 4, 25, "Text",
   CellTags->"Nielsen"],
  Cell[1466932, 59532, 172, 5, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1467107, 59539, 123, 3, 27, "Output",
   CellTags->"Nielsen"],
  Cell[1467245, 59545, 71, 1, 25, "Text",
   CellTags->"Nielsen"],
  Cell[1467341, 59550, 467, 14, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1467811, 59566, 992, 40, 42, "Output",
   CellTags->"Nielsen"],
  Cell[1468840, 59611, 190, 5, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1469033, 59618, 984, 38, 38, "Output",
   CellTags->"Nielsen"],
  Cell[1470054, 59661, 190, 5, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1470247, 59668, 1144, 42, 40, "Output",
   CellTags->"Nielsen"],
  Cell[1471428, 59715, 190, 5, 28, "Input",
   CellTags->"Nielsen"],
  Cell[1471621, 59722, 240, 9, 27, "Output",
   CellTags->"Nielsen"]},
 "NonCommFreeQ"->{
  Cell[1471888, 59735, 61, 1, 70, "ObjectName",
   CellTags->"NonCommFreeQ"],
  Cell[1471974, 59740, 63, 1, 70, "Subsubsection",
   CellTags->"NonCommFreeQ"],
  Cell[1472040, 59743, 244, 5, 70, "Text",
   CellTags->"NonCommFreeQ"],
  Cell[1472287, 59750, 519, 24, 70, "Text",
   CellTags->"NonCommFreeQ"]},
 "NonCommQ"->{
  Cell[1472821, 59777, 53, 1, 70, "ObjectName",
   CellTags->"NonCommQ"],
  Cell[1472899, 59782, 59, 1, 70, "Subsubsection",
   CellTags->"NonCommQ"],
  Cell[1472961, 59785, 203, 4, 70, "Text",
   CellTags->"NonCommQ"],
  Cell[1473167, 59791, 527, 24, 70, "Text",
   CellTags->"NonCommQ"],
  Cell[1476091, 59935, 167, 9, 70, "Text",
   CellTags->"NonCommQ"]},
 "NonCommutative"->{
  Cell[1473709, 59818, 65, 1, 70, "ObjectName",
   CellTags->"NonCommutative"],
  Cell[1473799, 59823, 65, 1, 70, "Subsubsection",
   CellTags->"NonCommutative"],
  Cell[1473867, 59826, 150, 4, 70, "Text",
   CellTags->"NonCommutative"],
  Cell[1474020, 59832, 333, 14, 70, "Text",
   CellTags->"NonCommutative"]},
 "NTerms"->{
  Cell[1474368, 59849, 49, 1, 70, "ObjectName",
   CellTags->"NTerms"],
  Cell[1474442, 59854, 57, 1, 70, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1474502, 59857, 154, 4, 70, "Text",
   CellTags->"NTerms"],
  Cell[1474693, 59866, 54, 1, 36, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1474772, 59871, 129, 4, 28, "Input",
   CellTags->"NTerms"],
  Cell[1474904, 59877, 103, 3, 27, "Output",
   CellTags->"NTerms"],
  Cell[1475044, 59885, 139, 4, 28, "Input",
   CellTags->"NTerms"],
  Cell[1475186, 59891, 103, 3, 27, "Output",
   CellTags->"NTerms"],
  Cell[1475326, 59899, 106, 3, 28, "Input",
   CellTags->"NTerms"],
  Cell[1475435, 59904, 103, 3, 27, "Output",
   CellTags->"NTerms"],
  Cell[1475575, 59912, 106, 3, 28, "Input",
   CellTags->"NTerms"],
  Cell[1475684, 59917, 103, 3, 27, "Output",
   CellTags->"NTerms"]},
 "NumberOfMetricTensors"->{
  Cell[1475814, 59924, 79, 1, 70, "ObjectName",
   CellTags->"NumberOfMetricTensors"],
  Cell[1475918, 59929, 72, 1, 70, "Subsubsection",
   CellTags->"NumberOfMetricTensors"],
  Cell[1475993, 59932, 95, 1, 70, "Text",
   CellTags->"NumberOfMetricTensors"]},
 "NumericalFactor"->{
  Cell[1476273, 59947, 67, 1, 70, "ObjectName",
   CellTags->"NumericalFactor"],
  Cell[1476365, 59952, 66, 1, 52, "Subsubsection",
   CellTags->"NumericalFactor"],
  Cell[1476434, 59955, 119, 3, 70, "Text",
   CellTags->"NumericalFactor"]},
 "NumericQ1"->{
  Cell[1476568, 59961, 55, 1, 57, "ObjectName",
   CellTags->"NumericQ1"],
  Cell[1476648, 59966, 60, 1, 52, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1476711, 59969, 139, 4, 25, "Text",
   CellTags->"NumericQ1"],
  Cell[1476887, 59978, 57, 1, 36, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1476969, 59983, 224, 7, 28, "Input",
   CellTags->"NumericQ1"],
  Cell[1477196, 59992, 110, 3, 27, "Output",
   CellTags->"NumericQ1"],
  Cell[1477343, 60000, 271, 9, 28, "Input",
   CellTags->"NumericQ1"],
  Cell[1477617, 60011, 110, 3, 27, "Output",
   CellTags->"NumericQ1"],
  Cell[1477764, 60019, 311, 10, 28, "Input",
   CellTags->"NumericQ1"],
  Cell[1478078, 60031, 109, 3, 27, "Output",
   CellTags->"NumericQ1"]},
 "OneLoop"->{
  Cell[1478236, 60040, 69, 1, 41, "ObjectName",
   CellTags->"OneLoop"],
  Cell[1478308, 60043, 619, 16, 61, "Text",
   CellTags->"OneLoop"],
  Cell[1478930, 60061, 968, 26, 61, "Text",
   CellTags->"OneLoop"],
  Cell[1479935, 60092, 58, 1, 36, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1479996, 60095, 391, 12, 43, "Text",
   CellTags->"OneLoop"],
  Cell[1480390, 60109, 352, 6, 61, "Text",
   CellTags->"OneLoop"],
  Cell[1480767, 60119, 114, 3, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1480884, 60124, 1275, 29, 93, "Output",
   CellTags->"OneLoop"],
  Cell[1482174, 60156, 781, 39, 25, "Text",
   CellTags->"OneLoop"],
  Cell[1482992, 60200, 55, 1, 36, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1483072, 60205, 275, 10, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1483350, 60217, 323, 11, 43, "Output",
   CellTags->"OneLoop"],
  Cell[1483710, 60233, 131, 4, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1483844, 60239, 266, 10, 27, "Output",
   CellTags->"OneLoop"],
  Cell[1484125, 60252, 363, 11, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1484491, 60265, 588, 20, 27, "Text",
   CellTags->"OneLoop"],
  Cell[1485104, 60289, 908, 30, 70, "Input",
   CellTags->"OneLoop"],
  Cell[1486015, 60321, 1804, 62, 52, "Output",
   CellTags->"OneLoop"],
  Cell[1487834, 60386, 219, 5, 43, "Text",
   CellTags->"OneLoop"],
  Cell[1488078, 60395, 131, 4, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1488212, 60401, 2148, 72, 50, "Output",
   CellTags->"OneLoop"],
  Cell[1490397, 60478, 113, 3, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1490513, 60483, 1540, 53, 43, "Output",
   CellTags->"OneLoop"],
  Cell[1492090, 60541, 333, 11, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1492426, 60554, 442, 17, 46, "Output",
   CellTags->"OneLoop"],
  Cell[1492905, 60576, 131, 4, 28, "Input",
   CellTags->"OneLoop"],
  Cell[1493039, 60582, 1917, 67, 44, "Output",
   CellTags->"OneLoop"],
  Cell[1494971, 60652, 106, 3, 28, "Input",
   CellTags->"OneLoop"]},
 "OneLoopSimplify"->{
  Cell[1495092, 60658, 67, 1, 41, "ObjectName",
   CellTags->"OneLoopSimplify"],
  Cell[1495184, 60663, 66, 1, 36, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1495253, 60666, 172, 4, 25, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1495450, 60674, 130, 3, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1495583, 60679, 737, 18, 49, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1496335, 60700, 376, 19, 25, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1496748, 60724, 63, 1, 36, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1496836, 60729, 341, 11, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1497180, 60742, 450, 17, 46, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1497667, 60764, 147, 4, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1497817, 60770, 450, 17, 46, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1498304, 60792, 203, 6, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1498510, 60800, 369, 14, 46, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1498916, 60819, 401, 12, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1499320, 60833, 580, 22, 47, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1499937, 60860, 152, 4, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1500092, 60866, 7072, 260, 94, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1507201, 61131, 253, 9, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1507457, 61142, 4303, 163, 48, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1511797, 61310, 424, 15, 28, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1512224, 61327, 1510, 58, 49, "Output",
   CellTags->"OneLoopSimplify"]},
 "OneLoopSum"->{
  Cell[1513761, 61389, 57, 1, 41, "ObjectName",
   CellTags->"OneLoopSum"],
  Cell[1513843, 61394, 61, 1, 36, "Subsubsection",
   CellTags->"OneLoopSum"],
  Cell[1513907, 61397, 190, 4, 43, "Text",
   CellTags->"OneLoopSum"],
  Cell[1514100, 61403, 178, 9, 25, "Text",
   CellTags->"OneLoopSum"]},
 "OPE"->{
  Cell[1514293, 61415, 43, 1, 41, "ObjectName",
   CellTags->"OPE"],
  Cell[1514361, 61420, 54, 1, 36, "Subsubsection",
   CellTags->"OPE"],
  Cell[1514418, 61423, 170, 4, 51, "Text",
   CellTags->"OPE"],
  Cell[1514936, 61446, 174, 9, 25, "Text",
   CellTags->"OPE"]},
 "OPEDelta"->{
  Cell[1515125, 61458, 53, 1, 41, "ObjectName",
   CellTags->"OPEDelta"],
  Cell[1515203, 61463, 59, 1, 36, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1515265, 61466, 140, 4, 25, "Text",
   CellTags->"OPEDelta"],
  Cell[1515408, 61472, 212, 9, 25, "Text",
   CellTags->"OPEDelta"],
  Cell[1515657, 61486, 56, 1, 36, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1515738, 61491, 146, 4, 28, "Input",
   CellTags->"OPEDelta"],
  Cell[1515887, 61497, 251, 10, 27, "Output",
   CellTags->"OPEDelta"],
  Cell[1516175, 61512, 133, 4, 28, "Input",
   CellTags->"OPEDelta"],
  Cell[1516311, 61518, 105, 3, 27, "Output",
   CellTags->"OPEDelta"],
  Cell[1516453, 61526, 141, 4, 28, "Input",
   CellTags->"OPEDelta"],
  Cell[1516597, 61532, 105, 3, 27, "Output",
   CellTags->"OPEDelta"]},
 "OPEi"->{
  Cell[1516729, 61539, 81, 1, 41, "ObjectName",
   CellTags->"OPEi"],
  Cell[1516835, 61544, 55, 1, 36, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1516893, 61547, 171, 4, 25, "Text",
   CellTags->"OPEi"],
  Cell[1517101, 61556, 52, 1, 36, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1517178, 61561, 75, 2, 28, "Input",
   CellTags->"OPEi"],
  Cell[1517256, 61565, 101, 3, 27, "Output",
   CellTags->"OPEi"],
  Cell[1517394, 61573, 259, 7, 28, "Input",
   CellTags->"OPEi"],
  Cell[1517656, 61582, 228, 7, 27, "Output",
   CellTags->"OPEi"],
  Cell[1517921, 61594, 1076, 40, 50, "Input",
   CellTags->"OPEi"],
  Cell[1519000, 61636, 202, 6, 27, "Output",
   CellTags->"OPEi"],
  Cell[1519217, 61645, 55, 1, 25, "Text",
   CellTags->"OPEi"],
  Cell[1519297, 61650, 519, 17, 28, "Input",
   CellTags->"OPEi"],
  Cell[1519819, 61669, 529, 19, 27, "Output",
   CellTags->"OPEi"],
  Cell[1520385, 61693, 534, 19, 28, "Input",
   CellTags->"OPEi"],
  Cell[1520922, 61714, 413, 15, 27, "Output",
   CellTags->"OPEi"],
  Cell[1521372, 61734, 519, 17, 28, "Input",
   CellTags->"OPEi"],
  Cell[1521894, 61753, 529, 19, 27, "Output",
   CellTags->"OPEi"]},
 "OPEInsert"->{
  Cell[1522450, 61776, 73, 1, 41, "ObjectName",
   CellTags->"OPEInsert"],
  Cell[1522548, 61781, 60, 1, 36, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1522611, 61784, 264, 5, 43, "Text",
   CellTags->"OPEInsert"],
  Cell[1522878, 61791, 165, 9, 25, "Text",
   CellTags->"OPEInsert"],
  Cell[1523080, 61805, 57, 1, 36, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1523140, 61808, 76, 2, 28, "Input",
   CellTags->"OPEInsert"]},
 "OPEInt"->{
  Cell[1523231, 61813, 67, 1, 41, "ObjectName",
   CellTags->"OPEInt"],
  Cell[1523323, 61818, 57, 1, 36, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1523383, 61821, 257, 5, 43, "Text",
   CellTags->"OPEInt"],
  Cell[1523643, 61828, 162, 9, 25, "Text",
   CellTags->"OPEInt"],
  Cell[1523842, 61842, 54, 1, 36, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1523899, 61845, 73, 2, 28, "Input",
   CellTags->"OPEInt"]},
 "OPEIntDelta"->{
  Cell[1523987, 61850, 77, 1, 41, "ObjectName",
   CellTags->"OPEIntDelta"],
  Cell[1524089, 61855, 62, 1, 36, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1524154, 61858, 478, 10, 87, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1524635, 61870, 167, 9, 25, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1524839, 61884, 59, 1, 36, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1524901, 61887, 78, 2, 28, "Input",
   CellTags->"OPEIntDelta"]},
 "OPEIntegrate"->{
  Cell[1524994, 61892, 79, 1, 41, "ObjectName",
   CellTags->"OPEIntegrate"],
  Cell[1525098, 61897, 63, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1525164, 61900, 266, 5, 43, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1525433, 61907, 168, 9, 25, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1525638, 61921, 60, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1525701, 61924, 79, 2, 28, "Input",
   CellTags->"OPEIntegrate"]},
 "OPEIntegrateDelta"->{
  Cell[1525795, 61929, 89, 1, 41, "ObjectName",
   CellTags->"OPEIntegrateDelta"],
  Cell[1525909, 61934, 68, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1525980, 61937, 499, 10, 87, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1526482, 61949, 173, 9, 25, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1526692, 61963, 65, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1526760, 61966, 84, 2, 28, "Input",
   CellTags->"OPEIntegrateDelta"]},
 "OPESumExplicit"->{
  Cell[1526859, 61971, 65, 1, 41, "ObjectName",
   CellTags->"OPESumExplicit"],
  Cell[1526949, 61976, 65, 1, 36, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1527017, 61979, 85, 1, 25, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1527105, 61982, 307, 14, 25, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1527449, 62001, 62, 1, 36, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1527536, 62006, 387, 13, 28, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1527926, 62021, 385, 14, 51, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1528348, 62040, 123, 3, 28, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1528474, 62045, 408, 16, 40, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1528919, 62066, 554, 18, 28, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1529476, 62086, 678, 26, 53, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1530191, 62117, 123, 3, 28, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1530317, 62122, 2216, 83, 86, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1532548, 62208, 157, 5, 28, "Input",
   CellTags->"OPESumExplicit"]},
 "OPESum"->{
  Cell[1532720, 62216, 49, 1, 70, "ObjectName",
   CellTags->"OPESum"],
  Cell[1532794, 62221, 57, 1, 70, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1532854, 62224, 127, 4, 70, "Text",
   CellTags->"OPESum"],
  Cell[1532984, 62230, 323, 14, 70, "Text",
   CellTags->"OPESum"],
  Cell[1533344, 62249, 54, 1, 36, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1533423, 62254, 460, 15, 28, "Input",
   CellTags->"OPESum"],
  Cell[1533886, 62271, 833, 30, 51, "Output",
   CellTags->"OPESum"],
  Cell[1534756, 62306, 115, 3, 28, "Input",
   CellTags->"OPESum"],
  Cell[1534874, 62311, 1604, 60, 45, "Output",
   CellTags->"OPESum"],
  Cell[1536515, 62376, 546, 18, 28, "Input",
   CellTags->"OPESum"],
  Cell[1537064, 62396, 670, 26, 53, "Output",
   CellTags->"OPESum"],
  Cell[1537771, 62427, 115, 3, 28, "Input",
   CellTags->"OPESum"],
  Cell[1537889, 62432, 2208, 83, 86, "Output",
   CellTags->"OPESum"],
  Cell[1540112, 62518, 149, 5, 28, "Input",
   CellTags->"OPESum"]},
 "OPESumSimplify"->{
  Cell[1540276, 62526, 65, 1, 70, "ObjectName",
   CellTags->"OPESumSimplify"],
  Cell[1540366, 62531, 65, 1, 70, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1540434, 62534, 92, 1, 70, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1540529, 62537, 307, 14, 70, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1540873, 62556, 62, 1, 36, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1540960, 62561, 143, 4, 28, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1541106, 62567, 1052, 37, 50, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1542195, 62609, 89, 2, 28, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1542287, 62613, 1099, 38, 50, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1543423, 62656, 273, 8, 28, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1543699, 62666, 283, 11, 50, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1544019, 62682, 439, 14, 28, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1544461, 62698, 553, 22, 50, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1545051, 62725, 116, 3, 28, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1545170, 62730, 404, 13, 42, "Output",
   CellTags->"OPESumSimplify"]},
 "OPE1Loop"->{
  Cell[1545601, 62747, 54, 1, 41, "ObjectName",
   CellTags->"OPE1Loop"],
  Cell[1545680, 62752, 59, 1, 36, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1545742, 62755, 119, 3, 25, "Text",
   CellTags->"OPE1Loop"],
  Cell[1545864, 62760, 173, 9, 25, "Text",
   CellTags->"OPE1Loop"],
  Cell[1546074, 62774, 56, 1, 36, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1546133, 62777, 75, 2, 28, "Input",
   CellTags->"OPE1Loop"]},
 "OPE2TID"->{
  Cell[1546223, 62782, 69, 1, 41, "ObjectName",
   CellTags->"OPE2TID"],
  Cell[1546317, 62787, 58, 1, 36, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1546378, 62790, 184, 4, 43, "Text",
   CellTags->"OPE2TID"],
  Cell[1546587, 62798, 114, 3, 28, "Input",
   CellTags->"OPE2TID"],
  Cell[1546704, 62803, 343, 10, 27, "Output",
   CellTags->"OPE2TID"],
  Cell[1547062, 62816, 172, 9, 25, "Text",
   CellTags->"OPE2TID"],
  Cell[1547271, 62830, 55, 1, 36, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1547329, 62833, 74, 2, 28, "Input",
   CellTags->"OPE2TID"]},
 "OptionsSelect"->{
  Cell[1547418, 62838, 63, 1, 41, "ObjectName",
   CellTags->"OptionsSelect"],
  Cell[1547506, 62843, 64, 1, 36, "Subsubsection",
   CellTags->"OptionsSelect"],
  Cell[1547573, 62846, 219, 5, 43, "Text",
   CellTags->"OptionsSelect"]},
 "Pair"->{
  Cell[1547807, 62854, 45, 1, 41, "ObjectName",
   CellTags->"Pair"],
  Cell[1547877, 62859, 55, 1, 36, "Subsubsection",
   CellTags->"Pair"],
  Cell[1547935, 62862, 499, 11, 61, "Text",
   CellTags->"Pair"],
  Cell[1548437, 62875, 989, 49, 25, "Text",
   CellTags->"Pair"],
  Cell[1549463, 62929, 52, 1, 36, "Subsubsection",
   CellTags->"Pair"],
  Cell[1549518, 62932, 84, 1, 25, "Text",
   CellTags->"Pair"],
  Cell[1549627, 62937, 218, 6, 28, "Input",
   CellTags->"Pair"],
  Cell[1549848, 62945, 157, 5, 28, "Output",
   CellTags->"Pair"],
  Cell[1550020, 62953, 73, 1, 25, "Text",
   CellTags->"Pair"],
  Cell[1550118, 62958, 268, 8, 28, "Input",
   CellTags->"Pair"],
  Cell[1550389, 62968, 157, 5, 28, "Output",
   CellTags->"Pair"],
  Cell[1550583, 62978, 208, 6, 28, "Input",
   CellTags->"Pair"],
  Cell[1550794, 62986, 236, 10, 27, "Output",
   CellTags->"Pair"],
  Cell[1551067, 63001, 197, 6, 28, "Input",
   CellTags->"Pair"],
  Cell[1551267, 63009, 203, 8, 27, "Output",
   CellTags->"Pair"],
  Cell[1551507, 63022, 197, 6, 28, "Input",
   CellTags->"Pair"],
  Cell[1551707, 63030, 159, 6, 27, "Output",
   CellTags->"Pair"],
  Cell[1551903, 63041, 222, 7, 28, "Input",
   CellTags->"Pair"],
  Cell[1552128, 63050, 238, 9, 27, "Output",
   CellTags->"Pair"],
  Cell[1552403, 63064, 223, 7, 28, "Input",
   CellTags->"Pair"],
  Cell[1552629, 63073, 223, 9, 28, "Output",
   CellTags->"Pair"],
  Cell[1552889, 63087, 223, 7, 28, "Input",
   CellTags->"Pair"],
  Cell[1553115, 63096, 223, 9, 28, "Output",
   CellTags->"Pair"],
  Cell[1553375, 63110, 308, 10, 28, "Input",
   CellTags->"Pair"],
  Cell[1553686, 63122, 101, 3, 27, "Output",
   CellTags->"Pair"],
  Cell[1553824, 63130, 272, 8, 28, "Input",
   CellTags->"Pair"],
  Cell[1554099, 63140, 292, 12, 27, "Output",
   CellTags->"Pair"],
  Cell[1554428, 63157, 383, 12, 28, "Input",
   CellTags->"Pair"],
  Cell[1554814, 63171, 101, 3, 27, "Output",
   CellTags->"Pair"]},
 "PairCollect"->{
  Cell[1554942, 63178, 59, 1, 41, "ObjectName",
   CellTags->"PairCollect"],
  Cell[1555026, 63183, 62, 1, 36, "Subsubsection",
   CellTags->"PairCollect"],
  Cell[1555091, 63186, 153, 4, 25, "Text",
   CellTags->"PairCollect"],
  Cell[1555247, 63192, 286, 14, 25, "Text",
   CellTags->"PairCollect"]},
 "PairContract"->{
  Cell[1555548, 63209, 61, 1, 41, "ObjectName",
   CellTags->"PairContract"],
  Cell[1555634, 63214, 63, 1, 36, "Subsubsection",
   CellTags->"PairContract"],
  Cell[1555700, 63217, 118, 3, 25, "Text",
   CellTags->"PairContract"],
  Cell[1555821, 63222, 275, 13, 25, "Text",
   CellTags->"PairContract"]},
 "PairContract2"->{
  Cell[1556111, 63238, 63, 1, 41, "ObjectName",
   CellTags->"PairContract2"],
  Cell[1556199, 63243, 64, 1, 36, "Subsubsection",
   CellTags->"PairContract2"],
  Cell[1556266, 63246, 142, 4, 25, "Text",
   CellTags->"PairContract2"],
  Cell[1556411, 63252, 398, 18, 25, "Text",
   CellTags->"PairContract2"]},
 "PairContract3"->{
  Cell[1556824, 63273, 63, 1, 41, "ObjectName",
   CellTags->"PairContract3"],
  Cell[1556912, 63278, 64, 1, 36, "Subsubsection",
   CellTags->"PairContract3"],
  Cell[1556979, 63281, 142, 4, 25, "Text",
   CellTags->"PairContract3"],
  Cell[1557124, 63287, 398, 18, 25, "Text",
   CellTags->"PairContract3"]},
 "PartialD"->{
  Cell[1557537, 63308, 53, 1, 41, "ObjectName",
   CellTags->"PartialD"],
  Cell[1557615, 63313, 59, 1, 36, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1557677, 63316, 864, 35, 43, "Text",
   CellTags->"PartialD"],
  Cell[1558544, 63353, 581, 24, 25, "Text",
   CellTags->"PartialD"],
  Cell[1559162, 63382, 65, 1, 35, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1559252, 63387, 233, 7, 28, "Input",
   CellTags->"PartialD"],
  Cell[1559488, 63396, 302, 11, 33, "Output",
   CellTags->"PartialD"],
  Cell[1559827, 63412, 116, 3, 28, "Input",
   CellTags->"PartialD"],
  Cell[1559946, 63417, 351, 14, 27, "Output",
   CellTags->"PartialD"],
  Cell[1560334, 63436, 114, 3, 28, "Input",
   CellTags->"PartialD"],
  Cell[1560451, 63441, 287, 7, 42, "Output",
   CellTags->"PartialD"]},
 "PartialDRelations"->{
  Cell[1560765, 63452, 71, 1, 41, "ObjectName",
   CellTags->"PartialDRelations"],
  Cell[1560861, 63457, 68, 1, 36, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1560932, 63460, 191, 5, 25, "Text",
   CellTags->"PartialDRelations"],
  Cell[1561126, 63467, 700, 29, 25, "Text",
   CellTags->"PartialDRelations"],
  Cell[1561863, 63501, 74, 1, 35, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1561962, 63506, 344, 10, 28, "Input",
   CellTags->"PartialDRelations"],
  Cell[1562309, 63518, 348, 12, 33, "Output",
   CellTags->"PartialDRelations"],
  Cell[1562694, 63535, 256, 7, 28, "Input",
   CellTags->"PartialDRelations"],
  Cell[1562953, 63544, 855, 34, 27, "Output",
   CellTags->"PartialDRelations"]},
 "PartialFourVector"->{
  Cell[1563835, 63582, 71, 1, 41, "ObjectName",
   CellTags->"PartialFourVector"],
  Cell[1563931, 63587, 68, 1, 36, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1564002, 63590, 277, 6, 43, "Text",
   CellTags->"PartialFourVector"],
  Cell[1564282, 63598, 402, 19, 25, "Text",
   CellTags->"PartialFourVector"],
  Cell[1564721, 63622, 74, 1, 35, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1564820, 63627, 382, 11, 33, "Input",
   CellTags->"PartialFourVector"],
  Cell[1565205, 63640, 399, 15, 28, "Output",
   CellTags->"PartialFourVector"]},
 "PartialIntegrate"->{
  Cell[1565631, 63659, 69, 1, 41, "ObjectName",
   CellTags->"PartialIntegrate"],
  Cell[1565725, 63664, 67, 1, 36, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1565795, 63667, 283, 6, 43, "Text",
   CellTags->"PartialIntegrate"],
  Cell[1566103, 63677, 132, 3, 28, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1566238, 63682, 185, 6, 27, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1566472, 63694, 73, 1, 35, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1566570, 63699, 326, 10, 28, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1566899, 63711, 1239, 41, 45, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1568519, 63771, 275, 9, 28, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1568797, 63782, 507, 19, 38, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1569341, 63806, 293, 10, 41, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1569637, 63818, 378, 14, 44, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1570052, 63837, 443, 14, 28, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1570498, 63853, 302, 10, 46, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1570837, 63868, 320, 11, 28, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1571160, 63881, 246, 8, 42, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1571421, 63892, 138, 4, 28, "Input",
   CellTags->"PartialIntegrate"]},
 "PartitHead"->{
  Cell[1571574, 63899, 57, 1, 41, "ObjectName",
   CellTags->"PartitHead"],
  Cell[1571656, 63904, 61, 1, 36, "Subsubsection",
   CellTags->"PartitHead"],
  Cell[1571720, 63907, 167, 4, 25, "Text",
   CellTags->"PartitHead"]},
 "PauliSigma"->{
  Cell[1571902, 63914, 57, 1, 41, "ObjectName",
   CellTags->"PauliSigma"],
  Cell[1571984, 63919, 61, 1, 36, "Subsubsection",
   CellTags->"PauliSigma"],
  Cell[1572048, 63922, 252, 5, 43, "Text",
   CellTags->"PauliSigma"],
  Cell[1572325, 63931, 141, 4, 28, "Input",
   CellTags->"PauliSigma"],
  Cell[1572469, 63937, 3957, 118, 123, "Output",
   CellTags->"PauliSigma"]},
 "PaVe"->{
  Cell[1576453, 64059, 45, 1, 41, "ObjectName",
   CellTags->"PaVe"],
  Cell[1576523, 64064, 55, 1, 36, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1576581, 64067, 430, 8, 61, "Text",
   CellTags->"PaVe"],
  Cell[1577014, 64077, 266, 8, 25, "Text",
   CellTags->"PaVe"],
  Cell[1577283, 64087, 181, 9, 25, "Text",
   CellTags->"PaVe"],
  Cell[1577501, 64101, 52, 1, 36, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1577556, 64104, 78, 1, 25, "Text",
   CellTags->"PaVe"],
  Cell[1577659, 64109, 303, 10, 28, "Input",
   CellTags->"PaVe"],
  Cell[1577965, 64121, 304, 9, 27, "Output",
   CellTags->"PaVe"],
  Cell[1578306, 64135, 271, 9, 28, "Input",
   CellTags->"PaVe"],
  Cell[1578580, 64146, 311, 11, 27, "Output",
   CellTags->"PaVe"]},
 "PaVeOrderList"->{
  Cell[1578918, 64161, 63, 1, 41, "ObjectName",
   CellTags->"PaVeOrderList"],
  Cell[1579006, 64166, 64, 1, 36, "Subsubsection",
   CellTags->"PaVeOrderList"],
  Cell[1579073, 64169, 176, 4, 25, "Text",
   CellTags->"PaVeOrderList"],
  Cell[1579252, 64175, 487, 23, 25, "Text",
   CellTags->"PaVeOrderList"]},
 "PaVeOrder"->{
  Cell[1579754, 64201, 55, 1, 41, "ObjectName",
   CellTags->"PaVeOrder"],
  Cell[1579834, 64206, 60, 1, 36, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1579897, 64209, 564, 16, 61, "Text",
   CellTags->"PaVeOrder"],
  Cell[1580464, 64227, 186, 9, 25, "Text",
   CellTags->"PaVeOrder"],
  Cell[1580687, 64241, 57, 1, 36, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1580769, 64246, 413, 12, 28, "Input",
   CellTags->"PaVeOrder"],
  Cell[1581185, 64260, 382, 12, 27, "Output",
   CellTags->"PaVeOrder"],
  Cell[1581604, 64277, 413, 12, 28, "Input",
   CellTags->"PaVeOrder"],
  Cell[1582020, 64291, 382, 12, 27, "Output",
   CellTags->"PaVeOrder"],
  Cell[1582439, 64308, 706, 20, 50, "Input",
   CellTags->"PaVeOrder"],
  Cell[1583148, 64330, 623, 18, 27, "Output",
   CellTags->"PaVeOrder"]},
 "PaVeReduce"->{
  Cell[1583798, 64352, 57, 1, 41, "ObjectName",
   CellTags->"PaVeReduce"],
  Cell[1583880, 64357, 61, 1, 36, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1583944, 64360, 167, 4, 25, "Text",
   CellTags->"PaVeReduce"],
  Cell[1584136, 64368, 120, 3, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1584259, 64373, 446, 13, 27, "Output",
   CellTags->"PaVeReduce"],
  Cell[1584742, 64391, 106, 3, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1584851, 64396, 332, 5, 59, "Print",
   CellTags->{"Info3387712116-6749064", "PaVeReduce"}],
  Cell[1585198, 64404, 279, 14, 25, "Text",
   CellTags->"PaVeReduce"],
  Cell[1585514, 64423, 58, 1, 36, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1585597, 64428, 520, 16, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1586120, 64446, 3016, 104, 92, "Output",
   CellTags->"PaVeReduce"],
  Cell[1589173, 64555, 106, 3, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1589282, 64560, 136, 4, 27, "Output",
   CellTags->"PaVeReduce"],
  Cell[1589455, 64569, 107, 3, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1589565, 64574, 136, 4, 27, "Output",
   CellTags->"PaVeReduce"],
  Cell[1589738, 64583, 491, 15, 50, "Input",
   CellTags->"PaVeReduce"],
  Cell[1590232, 64600, 445, 14, 40, "Output",
   CellTags->"PaVeReduce"],
  Cell[1590714, 64619, 398, 12, 50, "Input",
   CellTags->"PaVeReduce"],
  Cell[1591115, 64633, 792, 23, 75, "Output",
   CellTags->"PaVeReduce"],
  Cell[1591922, 64659, 209, 7, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1592134, 64668, 162, 5, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1592321, 64677, 633, 17, 50, "Input",
   CellTags->"PaVeReduce"],
  Cell[1592957, 64696, 8271, 259, 311, "Output",
   CellTags->"PaVeReduce"],
  Cell[1601243, 64958, 267, 8, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1601535, 64970, 375, 11, 28, "Input",
   CellTags->"PaVeReduce"],
  Cell[1601913, 64983, 2978, 58, 659, "Output",
   CellTags->"PaVeReduce"],
  Cell[1604906, 65044, 253, 7, 28, "Input",
   CellTags->"PaVeReduce"]},
 "Info3387712116-6749064"->{
  Cell[1584851, 64396, 332, 5, 59, "Print",
   CellTags->{"Info3387712116-6749064", "PaVeReduce"}]},
 "PD"->{
  Cell[1605174, 65054, 41, 1, 41, "ObjectName",
   CellTags->"PD"],
  Cell[1605240, 65059, 53, 1, 36, "Subsubsection",
   CellTags->"PD"],
  Cell[1605296, 65062, 81, 1, 25, "Text",
   CellTags->"PD"],
  Cell[1605380, 65065, 310, 14, 25, "Text",
   CellTags->"PD"]},
 "PlusDistribution"->{
  Cell[1605705, 65082, 69, 1, 41, "ObjectName",
   CellTags->"PlusDistribution"],
  Cell[1605799, 65087, 67, 1, 36, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1605869, 65090, 147, 4, 25, "Text",
   CellTags->"PlusDistribution"],
  Cell[1606019, 65096, 193, 9, 25, "Text",
   CellTags->"PlusDistribution"],
  Cell[1606249, 65110, 64, 1, 36, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1606338, 65115, 198, 6, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1606539, 65123, 208, 7, 41, "Output",
   CellTags->"PlusDistribution"],
  Cell[1606784, 65135, 254, 8, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1607041, 65145, 268, 9, 44, "Output",
   CellTags->"PlusDistribution"],
  Cell[1607346, 65159, 329, 10, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1607678, 65171, 113, 3, 27, "Output",
   CellTags->"PlusDistribution"],
  Cell[1607828, 65179, 389, 12, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1608220, 65193, 113, 3, 27, "Output",
   CellTags->"PlusDistribution"],
  Cell[1608370, 65201, 417, 13, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1608790, 65216, 113, 3, 27, "Output",
   CellTags->"PlusDistribution"],
  Cell[1608940, 65224, 307, 10, 28, "Input",
   CellTags->"PlusDistribution"],
  Cell[1609250, 65236, 376, 13, 44, "Output",
   CellTags->"PlusDistribution"]},
 "Polarization"->{
  Cell[1609653, 65253, 61, 1, 41, "ObjectName",
   CellTags->"Polarization"],
  Cell[1609739, 65258, 63, 1, 36, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1609805, 65261, 769, 12, 115, "Text",
   CellTags->"Polarization"],
  Cell[1610577, 65275, 484, 19, 25, "Text",
   CellTags->"Polarization"],
  Cell[1611098, 65299, 60, 1, 36, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1611183, 65304, 118, 3, 28, "Input",
   CellTags->"Polarization"],
  Cell[1611304, 65309, 151, 4, 27, "Output",
   CellTags->"Polarization"],
  Cell[1611492, 65318, 153, 4, 28, "Input",
   CellTags->"Polarization"],
  Cell[1611648, 65324, 168, 4, 42, "Output",
   CellTags->"Polarization"],
  Cell[1611853, 65333, 160, 5, 28, "Input",
   CellTags->"Polarization"],
  Cell[1612016, 65340, 487, 19, 27, "Output",
   CellTags->"Polarization"],
  Cell[1612540, 65364, 196, 6, 28, "Input",
   CellTags->"Polarization"],
  Cell[1612739, 65372, 187, 5, 42, "Output",
   CellTags->"Polarization"],
  Cell[1612963, 65382, 155, 4, 28, "Input",
   CellTags->"Polarization"],
  Cell[1613121, 65388, 270, 9, 27, "Output",
   CellTags->"Polarization"],
  Cell[1613428, 65402, 194, 6, 28, "Input",
   CellTags->"Polarization"],
  Cell[1613625, 65410, 243, 6, 42, "Output",
   CellTags->"Polarization"],
  Cell[1613905, 65421, 302, 8, 28, "Input",
   CellTags->"Polarization"],
  Cell[1614210, 65431, 109, 3, 27, "Output",
   CellTags->"Polarization"]},
 "PolarizationSum"->{
  Cell[1614346, 65438, 67, 1, 41, "ObjectName",
   CellTags->"PolarizationSum"],
  Cell[1614438, 65443, 66, 1, 36, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1614507, 65446, 277, 5, 43, "Text",
   CellTags->"PolarizationSum"],
  Cell[1614809, 65455, 130, 3, 28, "Input",
   CellTags->"PolarizationSum"],
  Cell[1614942, 65460, 173, 5, 27, "Output",
   CellTags->"PolarizationSum"],
  Cell[1615130, 65468, 198, 9, 25, "Text",
   CellTags->"PolarizationSum"],
  Cell[1615365, 65482, 63, 1, 36, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1615453, 65487, 155, 4, 28, "Input",
   CellTags->"PolarizationSum"],
  Cell[1615611, 65493, 183, 6, 27, "Output",
   CellTags->"PolarizationSum"],
  Cell[1615831, 65504, 165, 4, 28, "Input",
   CellTags->"PolarizationSum"],
  Cell[1615999, 65510, 605, 25, 43, "Output",
   CellTags->"PolarizationSum"],
  Cell[1616641, 65540, 175, 4, 28, "Input",
   CellTags->"PolarizationSum"],
  Cell[1616819, 65546, 1775, 76, 43, "Output",
   CellTags->"PolarizationSum"],
  Cell[1618631, 65627, 247, 7, 28, "Input",
   CellTags->"PolarizationSum"],
  Cell[1618881, 65636, 4326, 175, 48, "Output",
   CellTags->"PolarizationSum"]},
 "PolarizationUncontract"->{
  Cell[1623234, 65815, 81, 1, 41, "ObjectName",
   CellTags->"PolarizationUncontract"],
  Cell[1623340, 65820, 73, 1, 36, "Subsubsection",
   CellTags->"PolarizationUncontract"],
  Cell[1623416, 65823, 152, 4, 25, "Text",
   CellTags->"PolarizationUncontract"],
  Cell[1623571, 65829, 321, 14, 25, "Text",
   CellTags->"PolarizationUncontract"]},
 "PolarizationVector"->{
  Cell[1623907, 65846, 73, 1, 41, "ObjectName",
   CellTags->"PolarizationVector"],
  Cell[1624005, 65851, 69, 1, 36, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1624077, 65854, 103, 1, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1624183, 65857, 409, 18, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1624629, 65880, 66, 1, 36, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1624720, 65885, 157, 4, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1624880, 65891, 258, 9, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1625175, 65905, 194, 5, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1625372, 65912, 268, 9, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1625655, 65924, 241, 9, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1625921, 65937, 190, 5, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1626114, 65944, 310, 8, 42, "Output",
   CellTags->"PolarizationVector"],
  Cell[1626439, 65955, 440, 18, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1626904, 65977, 284, 8, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1627191, 65987, 426, 17, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1627654, 66009, 121, 3, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1627778, 66014, 115, 3, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1627930, 66022, 310, 9, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1628243, 66033, 488, 20, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1628768, 66058, 121, 3, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1628892, 66063, 336, 12, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1629243, 66078, 1511, 51, 42, "Text",
   CellTags->"PolarizationVector"],
  Cell[1630779, 66133, 310, 9, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1631092, 66144, 488, 20, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1631617, 66169, 121, 3, 28, "Input",
   CellTags->"PolarizationVector"],
  Cell[1631741, 66174, 336, 12, 27, "Output",
   CellTags->"PolarizationVector"],
  Cell[1632092, 66189, 153, 4, 28, "Input",
   CellTags->"PolarizationVector"]},
 "PositiveInteger"->{
  Cell[1632260, 66196, 67, 1, 41, "ObjectName",
   CellTags->"PositiveInteger"],
  Cell[1632352, 66201, 66, 1, 36, "Subsubsection",
   CellTags->"PositiveInteger"],
  Cell[1632421, 66204, 136, 4, 25, "Text",
   CellTags->"PositiveInteger"],
  Cell[1632560, 66210, 186, 9, 25, "Text",
   CellTags->"PositiveInteger"]},
 "PositiveNumber"->{
  Cell[1632761, 66222, 65, 1, 41, "ObjectName",
   CellTags->"PositiveNumber"],
  Cell[1632851, 66227, 65, 1, 36, "Subsubsection",
   CellTags->"PositiveNumber"],
  Cell[1632919, 66230, 146, 4, 25, "Text",
   CellTags->"PositiveNumber"],
  Cell[1633068, 66236, 185, 9, 25, "Text",
   CellTags->"PositiveNumber"]},
 "PowerFactor"->{
  Cell[1633268, 66248, 59, 1, 41, "ObjectName",
   CellTags->"PowerFactor"],
  Cell[1633352, 66253, 62, 1, 36, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1633417, 66256, 375, 16, 25, "Text",
   CellTags->"PowerFactor"],
  Cell[1633795, 66274, 197, 9, 25, "Text",
   CellTags->"PowerFactor"],
  Cell[1634029, 66288, 59, 1, 36, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1634113, 66293, 144, 5, 28, "Input",
   CellTags->"PowerFactor"],
  Cell[1634260, 66300, 181, 6, 27, "Output",
   CellTags->"PowerFactor"],
  Cell[1634478, 66311, 116, 3, 28, "Input",
   CellTags->"PowerFactor"],
  Cell[1634597, 66316, 181, 6, 27, "Output",
   CellTags->"PowerFactor"]},
 "PowerSimplify"->{
  Cell[1634805, 66326, 63, 1, 41, "ObjectName",
   CellTags->"PowerSimplify"],
  Cell[1634893, 66331, 64, 1, 36, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1634960, 66334, 2559, 112, 63, "Text",
   CellTags->"PowerSimplify"],
  Cell[1637522, 66448, 282, 14, 25, "Text",
   CellTags->"PowerSimplify"],
  Cell[1637841, 66467, 61, 1, 36, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1637927, 66472, 234, 8, 28, "Input",
   CellTags->"PowerSimplify"],
  Cell[1638164, 66482, 110, 3, 27, "Output",
   CellTags->"PowerSimplify"],
  Cell[1638311, 66490, 239, 8, 28, "Input",
   CellTags->"PowerSimplify"],
  Cell[1638553, 66500, 178, 6, 27, "Output",
   CellTags->"PowerSimplify"],
  Cell[1638768, 66511, 239, 8, 28, "Input",
   CellTags->"PowerSimplify"],
  Cell[1639010, 66521, 178, 6, 27, "Output",
   CellTags->"PowerSimplify"],
  Cell[1639225, 66532, 190, 6, 28, "Input",
   CellTags->"PowerSimplify"],
  Cell[1639418, 66540, 178, 6, 27, "Output",
   CellTags->"PowerSimplify"]},
 "Power2"->{
  Cell[1639623, 66550, 49, 1, 41, "ObjectName",
   CellTags->"Power2"],
  Cell[1639697, 66555, 57, 1, 36, "Subsubsection",
   CellTags->"Power2"],
  Cell[1639757, 66558, 213, 5, 43, "Text",
   CellTags->"Power2"],
  Cell[1639973, 66565, 186, 9, 25, "Text",
   CellTags->"Power2"],
  Cell[1640196, 66579, 54, 1, 36, "Subsubsection",
   CellTags->"Power2"],
  Cell[1640275, 66584, 147, 5, 28, "Input",
   CellTags->"Power2"],
  Cell[1640425, 66591, 171, 6, 29, "Output",
   CellTags->"Power2"],
  Cell[1640633, 66602, 148, 5, 28, "Input",
   CellTags->"Power2"],
  Cell[1640784, 66609, 257, 10, 29, "Output",
   CellTags->"Power2"]},
 "PostFortranFile"->{
  Cell[1641068, 66623, 67, 1, 41, "ObjectName",
   CellTags->"PostFortranFile"],
  Cell[1641160, 66628, 66, 1, 36, "Subsubsection",
   CellTags->"PostFortranFile"],
  Cell[1641229, 66631, 229, 5, 43, "Text",
   CellTags->"PostFortranFile"],
  Cell[1641461, 66638, 308, 14, 25, "Text",
   CellTags->"PostFortranFile"]},
 "Prefactor"->{
  Cell[1641784, 66655, 55, 1, 41, "ObjectName",
   CellTags->"Prefactor"],
  Cell[1641864, 66660, 60, 1, 36, "Subsubsection",
   CellTags->"Prefactor"],
  Cell[1641927, 66663, 290, 6, 43, "Text",
   CellTags->"Prefactor"],
  Cell[1642220, 66671, 293, 14, 25, "Text",
   CellTags->"Prefactor"]},
 "PreFortranFile"->{
  Cell[1642528, 66688, 65, 1, 41, "ObjectName",
   CellTags->"PreFortranFile"],
  Cell[1642618, 66693, 65, 1, 36, "Subsubsection",
   CellTags->"PreFortranFile"],
  Cell[1642686, 66696, 232, 5, 43, "Text",
   CellTags->"PreFortranFile"],
  Cell[1642921, 66703, 310, 14, 25, "Text",
   CellTags->"PreFortranFile"]},
 "PropagatorDenominator"->{
  Cell[1643246, 66720, 79, 1, 41, "ObjectName",
   CellTags->"PropagatorDenominator"],
  Cell[1643350, 66725, 72, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1643425, 66728, 549, 16, 61, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1643977, 66746, 493, 19, 25, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1644507, 66770, 69, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1644601, 66775, 178, 5, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1644782, 66782, 215, 7, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1645034, 66794, 136, 3, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1645173, 66799, 164, 5, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1645374, 66809, 189, 5, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1645566, 66816, 215, 7, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1645818, 66828, 158, 4, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1645979, 66834, 235, 6, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1646251, 66845, 194, 5, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1646448, 66852, 235, 6, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1646720, 66863, 145, 3, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1646868, 66868, 253, 9, 42, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1647158, 66882, 127, 3, 28, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1647288, 66887, 325, 10, 56, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1647628, 66900, 121, 3, 28, "Input",
   CellTags->"PropagatorDenominator"]},
 "PropagatorDenominatorExplicit"->{
  Cell[1647764, 66906, 95, 1, 70, "ObjectName",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1647884, 66911, 80, 1, 70, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1647967, 66914, 243, 5, 70, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1648213, 66921, 379, 14, 70, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1648629, 66940, 77, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1648731, 66945, 308, 11, 28, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1649042, 66958, 357, 12, 45, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1649436, 66975, 181, 5, 28, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1649620, 66982, 766, 29, 42, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1650423, 67016, 131, 3, 28, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1650557, 67021, 547, 18, 56, "Output",
   CellTags->"PropagatorDenominatorExplicit"]},
 "QGV"->{
  Cell[1651131, 67043, 43, 1, 70, "ObjectName",
   CellTags->"QGV"],
  Cell[1651199, 67048, 54, 1, 70, "Subsubsection",
   CellTags->"QGV"],
  Cell[1651256, 67051, 72, 1, 70, "Text",
   CellTags->"QGV"],
  Cell[1651331, 67054, 198, 9, 70, "Text",
   CellTags->"QGV"]},
 "QO"->{
  Cell[1651544, 67066, 41, 1, 70, "ObjectName",
   CellTags->"QO"],
  Cell[1651610, 67071, 53, 1, 70, "Subsubsection",
   CellTags->"QO"],
  Cell[1651666, 67074, 73, 1, 70, "Text",
   CellTags->"QO"],
  Cell[1651742, 67077, 206, 9, 70, "Text",
   CellTags->"QO"]},
 "QP"->{
  Cell[1651963, 67089, 41, 1, 70, "ObjectName",
   CellTags->"QP"],
  Cell[1652029, 67094, 53, 1, 52, "Subsubsection",
   CellTags->"QP"],
  Cell[1652085, 67097, 169, 4, 70, "Text",
   CellTags->"QP"],
  Cell[1652257, 67103, 194, 9, 41, "Text",
   CellTags->"QP"]},
 "QuantumField"->{
  Cell[1652466, 67115, 61, 1, 57, "ObjectName",
   CellTags->"QuantumField"],
  Cell[1652552, 67120, 63, 1, 52, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1652618, 67123, 404, 9, 59, "Text",
   CellTags->"QuantumField"],
  Cell[1653025, 67134, 421, 19, 41, "Text",
   CellTags->"QuantumField"],
  Cell[1653483, 67158, 60, 1, 52, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1653546, 67161, 71, 1, 25, "Text",
   CellTags->"QuantumField"],
  Cell[1653642, 67166, 118, 3, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1653763, 67171, 109, 3, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1653909, 67179, 131, 3, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1654043, 67184, 137, 4, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1654217, 67193, 127, 3, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1654347, 67198, 114, 3, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1654476, 67204, 80, 1, 25, "Text",
   CellTags->"QuantumField"],
  Cell[1654581, 67209, 169, 5, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1654753, 67216, 135, 4, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1654903, 67223, 94, 1, 25, "Text",
   CellTags->"QuantumField"],
  Cell[1655022, 67228, 212, 6, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1655237, 67236, 319, 14, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1655593, 67255, 114, 3, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1655710, 67260, 255, 6, 42, "Output",
   CellTags->"QuantumField"],
  Cell[1655980, 67269, 358, 13, 25, "Text",
   CellTags->"QuantumField"],
  Cell[1656363, 67286, 206, 6, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1656572, 67294, 259, 10, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1656846, 67307, 185, 4, 43, "Text",
   CellTags->"QuantumField"],
  Cell[1657056, 67315, 244, 7, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1657303, 67324, 245, 6, 42, "Output",
   CellTags->"QuantumField"],
  Cell[1657563, 67333, 88, 1, 25, "Text",
   CellTags->"QuantumField"],
  Cell[1657676, 67338, 219, 6, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1657898, 67346, 355, 14, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1658290, 67365, 184, 5, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1658477, 67372, 257, 9, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1658771, 67386, 259, 7, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1659033, 67395, 404, 16, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1659474, 67416, 291, 9, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1659768, 67427, 413, 16, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1660218, 67448, 232, 6, 28, "Input",
   CellTags->"QuantumField"],
  Cell[1660453, 67456, 112, 3, 27, "Output",
   CellTags->"QuantumField"]},
 "QuarkField"->{
  Cell[1660592, 67463, 57, 1, 57, "ObjectName",
   CellTags->"QuarkField"],
  Cell[1660674, 67468, 61, 1, 36, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1660738, 67471, 189, 4, 43, "Text",
   CellTags->"QuarkField"],
  Cell[1660930, 67477, 321, 14, 25, "Text",
   CellTags->"QuarkField"],
  Cell[1661288, 67496, 58, 1, 36, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1661371, 67501, 87, 2, 28, "Input",
   CellTags->"QuarkField"],
  Cell[1661461, 67505, 112, 3, 27, "Output",
   CellTags->"QuarkField"]},
 "QuarkGluonVertex"->{
  Cell[1661600, 67512, 69, 1, 41, "ObjectName",
   CellTags->"QuarkGluonVertex"],
  Cell[1661694, 67517, 67, 1, 36, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1661764, 67520, 260, 9, 25, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1662049, 67533, 132, 3, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1662184, 67538, 491, 13, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1662690, 67554, 196, 9, 25, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1662923, 67568, 64, 1, 36, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1663012, 67573, 153, 4, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1663168, 67579, 149, 4, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1663354, 67588, 208, 5, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1663565, 67595, 149, 4, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1663751, 67604, 208, 5, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1663962, 67611, 149, 4, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1664148, 67620, 208, 5, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1664359, 67627, 149, 4, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1664545, 67636, 315, 9, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1664863, 67647, 366, 11, 27, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1665266, 67663, 316, 9, 28, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1665585, 67674, 367, 11, 27, "Output",
   CellTags->"QuarkGluonVertex"]},
 "QuarkMass"->{
  Cell[1665979, 67689, 55, 1, 41, "ObjectName",
   CellTags->"QuarkMass"],
  Cell[1666059, 67694, 60, 1, 36, "Subsubsection",
   CellTags->"QuarkMass"],
  Cell[1666122, 67697, 76, 1, 25, "Text",
   CellTags->"QuarkMass"],
  Cell[1666201, 67700, 183, 9, 25, "Text",
   CellTags->"QuarkMass"]},
 "QuarkPropagator"->{
  Cell[1666399, 67712, 67, 1, 41, "ObjectName",
   CellTags->"QuarkPropagator"],
  Cell[1666491, 67717, 66, 1, 36, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1666560, 67720, 174, 4, 25, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1666759, 67728, 130, 3, 28, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1666892, 67733, 534, 14, 27, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1667441, 67750, 341, 14, 25, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1667819, 67769, 63, 1, 36, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1667907, 67774, 124, 3, 28, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1668034, 67779, 177, 6, 27, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1668248, 67790, 171, 5, 28, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1668422, 67797, 177, 6, 27, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1668636, 67808, 157, 4, 28, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1668796, 67814, 177, 6, 27, "Output",
   CellTags->"QuarkPropagator"]},
 "ReduceGamma"->{
  Cell[1669000, 67824, 59, 1, 41, "ObjectName",
   CellTags->"ReduceGamma"],
  Cell[1669084, 67829, 62, 1, 36, "Subsubsection",
   CellTags->"ReduceGamma"],
  Cell[1669149, 67832, 194, 4, 43, "Text",
   CellTags->"ReduceGamma"],
  Cell[1669346, 67838, 179, 9, 25, "Text",
   CellTags->"ReduceGamma"]},
 "ReduceToScalars"->{
  Cell[1669540, 67850, 67, 1, 41, "ObjectName",
   CellTags->"ReduceToScalars"],
  Cell[1669632, 67855, 66, 1, 36, "Subsubsection",
   CellTags->"ReduceToScalars"],
  Cell[1669701, 67858, 210, 5, 43, "Text",
   CellTags->"ReduceToScalars"],
  Cell[1669914, 67865, 299, 14, 25, "Text",
   CellTags->"ReduceToScalars"]},
 "Rename"->{
  Cell[1670228, 67882, 49, 1, 41, "ObjectName",
   CellTags->"Rename"],
  Cell[1670302, 67887, 57, 1, 36, "Subsubsection",
   CellTags->"Rename"],
  Cell[1670362, 67890, 144, 4, 25, "Text",
   CellTags->"Rename"],
  Cell[1670509, 67896, 177, 9, 25, "Text",
   CellTags->"Rename"]},
 "RHI"->{
  Cell[1670701, 67908, 61, 1, 41, "ObjectName",
   CellTags->"RHI"],
  Cell[1670787, 67913, 54, 1, 36, "Subsubsection",
   CellTags->"RHI"],
  Cell[1670844, 67916, 611, 12, 147, "Text",
   CellTags->"RHI"],
  Cell[1671458, 67930, 168, 9, 25, "Text",
   CellTags->"RHI"],
  Cell[1671663, 67944, 51, 1, 36, "Subsubsection",
   CellTags->"RHI"],
  Cell[1671717, 67947, 70, 2, 28, "Input",
   CellTags->"RHI"]},
 "RHI2FC"->{
  Cell[1671802, 67952, 67, 1, 41, "ObjectName",
   CellTags->"RHI2FC"],
  Cell[1671894, 67957, 57, 1, 36, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1671954, 67960, 614, 12, 147, "Text",
   CellTags->"RHI2FC"],
  Cell[1672571, 67974, 171, 9, 25, "Text",
   CellTags->"RHI2FC"],
  Cell[1672779, 67988, 54, 1, 36, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1672836, 67991, 73, 2, 28, "Input",
   CellTags->"RHI2FC"]},
 "RHM"->{
  Cell[1672924, 67996, 61, 1, 41, "ObjectName",
   CellTags->"RHM"],
  Cell[1673010, 68001, 54, 1, 36, "Subsubsection",
   CellTags->"RHM"],
  Cell[1673067, 68004, 77, 1, 25, "Text",
   CellTags->"RHM"],
  Cell[1673147, 68007, 159, 9, 25, "Text",
   CellTags->"RHM"],
  Cell[1673343, 68021, 51, 1, 36, "Subsubsection",
   CellTags->"RHM"],
  Cell[1673397, 68024, 70, 2, 28, "Input",
   CellTags->"RHM"]},
 "RHO"->{
  Cell[1673482, 68029, 61, 1, 41, "ObjectName",
   CellTags->"RHO"],
  Cell[1673568, 68034, 54, 1, 36, "Subsubsection",
   CellTags->"RHO"],
  Cell[1673625, 68037, 267, 5, 43, "Text",
   CellTags->"RHO"],
  Cell[1673895, 68044, 159, 9, 25, "Text",
   CellTags->"RHO"],
  Cell[1674091, 68058, 51, 1, 36, "Subsubsection",
   CellTags->"RHO"],
  Cell[1674145, 68061, 70, 2, 28, "Input",
   CellTags->"RHO"]},
 "RHP"->{
  Cell[1674230, 68066, 45, 1, 41, "ObjectName",
   CellTags->"RHP"],
  Cell[1674300, 68071, 54, 1, 36, "Subsubsection",
   CellTags->"RHP"],
  Cell[1674357, 68074, 272, 5, 43, "Text",
   CellTags->"RHP"],
  Cell[1674632, 68081, 159, 9, 25, "Text",
   CellTags->"RHP"],
  Cell[1674828, 68095, 51, 1, 36, "Subsubsection",
   CellTags->"RHP"],
  Cell[1680146, 68325, 70, 2, 28, "Input",
   CellTags->"RHP"]},
 "RTL"->{
  Cell[1680231, 68330, 61, 1, 41, "ObjectName",
   CellTags->"RTL"],
  Cell[1680317, 68335, 54, 1, 36, "Subsubsection",
   CellTags->"RTL"],
  Cell[1680374, 68338, 188, 4, 43, "Text",
   CellTags->"RTL"],
  Cell[1680565, 68344, 266, 14, 25, "Text",
   CellTags->"RTL"],
  Cell[1680868, 68363, 51, 1, 36, "Subsubsection",
   CellTags->"RTL"],
  Cell[1680922, 68366, 70, 2, 28, "Input",
   CellTags->"RTL"]},
 "RightPartialD"->{
  Cell[1681007, 68371, 63, 1, 41, "ObjectName",
   CellTags->"RightPartialD"],
  Cell[1681095, 68376, 64, 1, 36, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1681162, 68379, 256, 10, 25, "Text",
   CellTags->"RightPartialD"],
  Cell[1681421, 68391, 434, 19, 25, "Text",
   CellTags->"RightPartialD"],
  Cell[1681892, 68415, 61, 1, 36, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1681978, 68420, 124, 3, 28, "Input",
   CellTags->"RightPartialD"],
  Cell[1682105, 68425, 252, 9, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1682394, 68439, 255, 7, 28, "Input",
   CellTags->"RightPartialD"],
  Cell[1682652, 68448, 308, 11, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1682997, 68464, 121, 3, 28, "Input",
   CellTags->"RightPartialD"],
  Cell[1683121, 68469, 356, 14, 27, "Output",
   CellTags->"RightPartialD"],
  Cell[1683514, 68488, 115, 3, 28, "Input",
   CellTags->"RightPartialD"],
  Cell[1683632, 68493, 292, 7, 42, "Output",
   CellTags->"RightPartialD"],
  Cell[1683961, 68505, 162, 5, 28, "Input",
   CellTags->"RightPartialD"],
  Cell[1684126, 68512, 178, 4, 42, "Output",
   CellTags->"RightPartialD"]},
 "RussianTrick"->{
  Cell[1684331, 68520, 61, 1, 41, "ObjectName",
   CellTags->"RussianTrick"],
  Cell[1684417, 68525, 63, 1, 36, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1684483, 68528, 318, 6, 61, "Text",
   CellTags->"RussianTrick"],
  Cell[1684804, 68536, 90, 1, 25, "Text",
   CellTags->"RussianTrick"],
  Cell[1684897, 68539, 326, 14, 25, "Text",
   CellTags->"RussianTrick"],
  Cell[1685260, 68558, 60, 1, 36, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1685345, 68563, 374, 12, 28, "Input",
   CellTags->"RussianTrick"],
  Cell[1685722, 68577, 501, 21, 27, "Output",
   CellTags->"RussianTrick"],
  Cell[1686260, 68603, 108, 3, 28, "Input",
   CellTags->"RussianTrick"],
  Cell[1686371, 68608, 661, 23, 48, "Output",
   CellTags->"RussianTrick"],
  Cell[1687069, 68636, 172, 5, 28, "Input",
   CellTags->"RussianTrick"],
  Cell[1687244, 68643, 3509, 122, 140, "Output",
   CellTags->"RussianTrick"],
  Cell[1690790, 68770, 112, 3, 28, "Input",
   CellTags->"RussianTrick"],
  Cell[1690905, 68775, 2675, 111, 27, "Output",
   CellTags->"RussianTrick"],
  Cell[1693617, 68891, 135, 4, 28, "Input",
   CellTags->"RussianTrick"],
  Cell[1693755, 68897, 2947, 112, 42, "Output",
   CellTags->"RussianTrick"],
  Cell[1696717, 69012, 111, 3, 28, "Input",
   CellTags->"RussianTrick"]},
 "ScalarGluonVertex"->{
  Cell[1696843, 69018, 71, 1, 41, "ObjectName",
   CellTags->"ScalarGluonVertex"],
  Cell[1696939, 69023, 68, 1, 36, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1697010, 69026, 430, 8, 95, "Text",
   CellTags->"ScalarGluonVertex"],
  Cell[1697465, 69038, 134, 3, 28, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1697602, 69043, 271, 8, 27, "Output",
   CellTags->"ScalarGluonVertex"],
  Cell[1697922, 69057, 65, 1, 36, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1698012, 69062, 274, 8, 28, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1698289, 69072, 479, 19, 27, "Output",
   CellTags->"ScalarGluonVertex"]},
 "ScalarProduct"->{
  Cell[1698795, 69095, 63, 1, 41, "ObjectName",
   CellTags->"ScalarProduct"],
  Cell[1698883, 69100, 64, 1, 36, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1698950, 69103, 836, 21, 79, "Text",
   CellTags->"ScalarProduct"],
  Cell[1699789, 69126, 880, 38, 25, "Text",
   CellTags->"ScalarProduct"],
  Cell[1700706, 69169, 61, 1, 36, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1700792, 69174, 143, 4, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1700938, 69180, 212, 8, 27, "Output",
   CellTags->"ScalarProduct"],
  Cell[1701187, 69193, 186, 6, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1701376, 69201, 301, 11, 27, "Output",
   CellTags->"ScalarProduct"],
  Cell[1701714, 69217, 143, 4, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1701860, 69223, 168, 6, 27, "Output",
   CellTags->"ScalarProduct"],
  Cell[1702065, 69234, 120, 3, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1702188, 69239, 168, 6, 27, "Output",
   CellTags->"ScalarProduct"],
  Cell[1702393, 69250, 179, 5, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1702575, 69257, 221, 6, 42, "Output",
   CellTags->"ScalarProduct"],
  Cell[1702833, 69268, 231, 7, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1703067, 69277, 271, 8, 42, "Output",
   CellTags->"ScalarProduct"],
  Cell[1703375, 69290, 248, 8, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1703626, 69300, 131, 4, 36, "Output",
   CellTags->"ScalarProduct"],
  Cell[1703794, 69309, 295, 9, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1704092, 69320, 546, 22, 36, "Output",
   CellTags->"ScalarProduct"],
  Cell[1704675, 69347, 280, 9, 28, "Input",
   CellTags->"ScalarProduct"],
  Cell[1704958, 69358, 546, 22, 36, "Output",
   CellTags->"ScalarProduct"],
  Cell[1705519, 69383, 99, 2, 28, "Input",
   CellTags->"ScalarProduct"]},
 "Info3387712142-8239904"->{
  Cell[1706955, 69441, 179, 3, 40, "Print",
   CellTags->{"Info3387712142-8239904", "ScalarProductCancel"}]},
 "ScaleMu"->{
  Cell[1717120, 69846, 51, 1, 41, "ObjectName",
   CellTags->"ScaleMu"],
  Cell[1717196, 69851, 58, 1, 36, "Subsubsection",
   CellTags->"ScaleMu"],
  Cell[1717257, 69854, 128, 4, 25, "Text",
   CellTags->"ScaleMu"],
  Cell[1717410, 69862, 81, 2, 28, "Input",
   CellTags->"ScaleMu"],
  Cell[1717494, 69866, 108, 3, 27, "Output",
   CellTags->"ScaleMu"]},
 "Schouten"->{
  Cell[1717629, 69873, 53, 1, 41, "ObjectName",
   CellTags->"Schouten"],
  Cell[1717707, 69878, 59, 1, 36, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1717769, 69881, 460, 8, 87, "Text",
   CellTags->"Schouten"],
  Cell[1718232, 69891, 289, 13, 25, "Text",
   CellTags->"Schouten"],
  Cell[1718558, 69909, 56, 1, 36, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1718639, 69914, 1938, 54, 167, "Input",
   CellTags->"Schouten"],
  Cell[1720580, 69970, 15656, 627, 174, "Output",
   CellTags->"Schouten"],
  Cell[1736273, 70602, 106, 3, 28, "Input",
   CellTags->"Schouten"],
  Cell[1736382, 70607, 105, 3, 27, "Output",
   CellTags->"Schouten"]},
 "SD"->{
  Cell[1736601, 70618, 41, 1, 41, "ObjectName",
   CellTags->"SD"],
  Cell[1736667, 70623, 53, 1, 36, "Subsubsection",
   CellTags->"SD"],
  Cell[1736723, 70626, 265, 7, 43, "Text",
   CellTags->"SD"],
  Cell[1736991, 70635, 173, 9, 25, "Text",
   CellTags->"SD"],
  Cell[1737201, 70649, 50, 1, 36, "Subsubsection",
   CellTags->"SD"],
  Cell[1737276, 70654, 121, 4, 28, "Input",
   CellTags->"SD"],
  Cell[1737400, 70660, 147, 5, 27, "Output",
   CellTags->"SD"],
  Cell[1737584, 70670, 130, 4, 28, "Input",
   CellTags->"SD"],
  Cell[1737717, 70676, 214, 6, 42, "Output",
   CellTags->"SD"],
  Cell[1737968, 70687, 130, 4, 28, "Input",
   CellTags->"SD"],
  Cell[1738101, 70693, 136, 4, 42, "Output",
   CellTags->"SD"]},
 "SelectFree"->{
  Cell[1738264, 70701, 57, 1, 41, "ObjectName",
   CellTags->"SelectFree"],
  Cell[1738346, 70706, 61, 1, 36, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1738410, 70709, 255, 5, 43, "Text",
   CellTags->"SelectFree"],
  Cell[1738668, 70716, 77, 1, 25, "Text",
   CellTags->"SelectFree"],
  Cell[1738748, 70719, 300, 14, 25, "Text",
   CellTags->"SelectFree"],
  Cell[1739085, 70738, 58, 1, 36, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1739168, 70743, 211, 6, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1739382, 70751, 130, 4, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1739549, 70760, 166, 5, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1739718, 70767, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1739862, 70775, 275, 8, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1740140, 70785, 130, 4, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1740307, 70794, 137, 4, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1740447, 70800, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1740591, 70808, 137, 4, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1740731, 70814, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1740875, 70822, 137, 4, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1741015, 70828, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1741159, 70836, 166, 5, 28, "Input",
   CellTags->"SelectFree"],
  Cell[1741328, 70843, 107, 3, 27, "Output",
   CellTags->"SelectFree"]},
 "SelectGraphs"->{
  Cell[1741462, 70850, 61, 1, 41, "ObjectName",
   CellTags->"SelectGraphs"],
  Cell[1741548, 70855, 63, 1, 36, "Subsubsection",
   CellTags->"SelectGraphs"],
  Cell[1741614, 70858, 486, 8, 79, "Text",
   CellTags->"SelectGraphs"],
  Cell[1742103, 70868, 189, 9, 25, "Text",
   CellTags->"SelectGraphs"]},
 "SelectNotFree"->{
  Cell[1742307, 70880, 63, 1, 41, "ObjectName",
   CellTags->"SelectNotFree"],
  Cell[1742395, 70885, 64, 1, 36, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1742462, 70888, 143, 4, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1742608, 70894, 270, 5, 43, "Text",
   CellTags->"SelectNotFree"],
  Cell[1742881, 70901, 83, 1, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1742967, 70904, 294, 14, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1743298, 70923, 61, 1, 36, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1743384, 70928, 207, 6, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1743594, 70936, 162, 5, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1743793, 70946, 276, 8, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1744072, 70956, 172, 5, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1744281, 70966, 143, 4, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1744427, 70972, 110, 3, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1744574, 70980, 167, 5, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1744744, 70987, 110, 3, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1744891, 70995, 143, 4, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1745037, 71001, 110, 3, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1745184, 71009, 143, 4, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1745330, 71015, 110, 3, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1745477, 71023, 172, 5, 28, "Input",
   CellTags->"SelectNotFree"],
  Cell[1745652, 71030, 138, 4, 27, "Output",
   CellTags->"SelectNotFree"]},
 "SelectSplit"->{
  Cell[1745817, 71038, 59, 1, 41, "ObjectName",
   CellTags->"SelectSplit"],
  Cell[1745901, 71043, 62, 1, 36, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1745966, 71046, 249, 5, 43, "Text",
   CellTags->"SelectSplit"],
  Cell[1746252, 71056, 59, 1, 36, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1746336, 71061, 804, 26, 50, "Input",
   CellTags->"SelectSplit"],
  Cell[1747143, 71089, 532, 19, 27, "Output",
   CellTags->"SelectSplit"],
  Cell[1747712, 71113, 652, 21, 28, "Input",
   CellTags->"SelectSplit"],
  Cell[1748367, 71136, 495, 16, 27, "Output",
   CellTags->"SelectSplit"]},
 "Select1"->{
  Cell[1748889, 71156, 51, 1, 41, "ObjectName",
   CellTags->"Select1"],
  Cell[1748965, 71161, 58, 1, 36, "Subsubsection",
   CellTags->"Select1"],
  Cell[1749026, 71164, 243, 5, 43, "Text",
   CellTags->"Select1"],
  Cell[1749272, 71171, 175, 9, 25, "Text",
   CellTags->"Select1"]},
 "Select2"->{
  Cell[1749462, 71183, 51, 1, 41, "ObjectName",
   CellTags->"Select2"],
  Cell[1749538, 71188, 58, 1, 36, "Subsubsection",
   CellTags->"Select2"],
  Cell[1749599, 71191, 245, 5, 43, "Text",
   CellTags->"Select2"],
  Cell[1749847, 71198, 175, 9, 25, "Text",
   CellTags->"Select2"]},
 "Series2"->{
  Cell[1750037, 71210, 51, 1, 41, "ObjectName",
   CellTags->"Series2"],
  Cell[1750113, 71215, 58, 1, 36, "Subsubsection",
   CellTags->"Series2"],
  Cell[1750174, 71218, 383, 9, 61, "Text",
   CellTags->"Series2"],
  Cell[1750582, 71231, 114, 3, 28, "Input",
   CellTags->"Series2"],
  Cell[1750699, 71236, 528, 15, 27, "Output",
   CellTags->"Series2"],
  Cell[1751276, 71257, 55, 1, 36, "Subsubsection",
   CellTags->"Series2"],
  Cell[1751356, 71262, 339, 12, 28, "Input",
   CellTags->"Series2"],
  Cell[1751698, 71276, 357, 12, 38, "Output",
   CellTags->"Series2"],
  Cell[1752092, 71293, 174, 5, 28, "Input",
   CellTags->"Series2"],
  Cell[1752269, 71300, 242, 8, 38, "Output",
   CellTags->"Series2"],
  Cell[1752548, 71313, 222, 7, 28, "Input",
   CellTags->"Series2"],
  Cell[1752773, 71322, 806, 25, 38, "Output",
   CellTags->"Series2"],
  Cell[1753616, 71352, 174, 5, 28, "Input",
   CellTags->"Series2"],
  Cell[1753793, 71359, 607, 23, 38, "Output",
   CellTags->"Series2"],
  Cell[1754437, 71387, 290, 8, 28, "Input",
   CellTags->"Series2"],
  Cell[1754730, 71397, 1223, 41, 40, "Output",
   CellTags->"Series2"],
  Cell[1756180, 71451, 460, 15, 28, "Input",
   CellTags->"Series2"],
  Cell[1756643, 71468, 1259, 42, 40, "Output",
   CellTags->"Series2"],
  Cell[1757917, 71513, 113, 3, 25, "Text",
   CellTags->"Series2"],
  Cell[1758055, 71520, 484, 15, 28, "Input",
   CellTags->"Series2"],
  Cell[1758542, 71537, 790, 29, 38, "Output",
   CellTags->"Series2"],
  Cell[1759369, 71571, 578, 17, 50, "Input",
   CellTags->"Series2"],
  Cell[1759950, 71590, 1149, 41, 38, "Output",
   CellTags->"Series2"],
  Cell[1761136, 71636, 242, 6, 28, "Input",
   CellTags->"Series2"],
  Cell[1761381, 71644, 1429, 52, 27, "Output",
   CellTags->"Series2"],
  Cell[1762847, 71701, 147, 4, 28, "Input",
   CellTags->"Series2"],
  Cell[1762997, 71707, 3671, 130, 94, "Output",
   CellTags->"Series2"],
  Cell[1766683, 71840, 351, 11, 25, "Text",
   CellTags->"Series2"]},
 "Series3"->{
  Cell[1767049, 71854, 51, 1, 41, "ObjectName",
   CellTags->"Series3"],
  Cell[1767125, 71859, 58, 1, 36, "Subsubsection",
   CellTags->"Series3"],
  Cell[1767186, 71862, 267, 5, 43, "Text",
   CellTags->"Series3"],
  Cell[1767478, 71871, 114, 3, 28, "Input",
   CellTags->"Series3"],
  Cell[1767595, 71876, 262, 8, 27, "Output",
   CellTags->"Series3"],
  Cell[1767872, 71887, 175, 9, 25, "Text",
   CellTags->"Series3"],
  Cell[1768084, 71901, 55, 1, 36, "Subsubsection",
   CellTags->"Series3"],
  Cell[1768164, 71906, 339, 12, 28, "Input",
   CellTags->"Series3"],
  Cell[1768506, 71920, 307, 11, 38, "Output",
   CellTags->"Series3"],
  Cell[1768850, 71936, 214, 7, 28, "Input",
   CellTags->"Series3"],
  Cell[1769067, 71945, 236, 8, 38, "Output",
   CellTags->"Series3"]},
 "SetMandelstam"->{
  Cell[1769330, 71957, 63, 1, 41, "ObjectName",
   CellTags->"SetMandelstam"],
  Cell[1769418, 71962, 64, 1, 36, "Subsubsection",
   CellTags->"SetMandelstam"],
  Cell[1769485, 71965, 462, 9, 87, "Text",
   CellTags->"SetMandelstam"],
  Cell[1769972, 71978, 126, 3, 28, "Input",
   CellTags->"SetMandelstam"],
  Cell[1770101, 71983, 237, 8, 27, "Output",
   CellTags->"SetMandelstam"],
  Cell[1770353, 71994, 190, 9, 25, "Text",
   CellTags->"SetMandelstam"]},
 "SimplifyDeltaFunction"->{
  Cell[1770558, 72006, 79, 1, 41, "ObjectName",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1770662, 72011, 72, 1, 36, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1770737, 72014, 299, 6, 43, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1771039, 72022, 347, 14, 25, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1771423, 72041, 69, 1, 36, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1771517, 72046, 203, 6, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1771723, 72054, 268, 9, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1772028, 72068, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1772195, 72074, 392, 13, 34, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1772624, 72092, 203, 6, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1772830, 72100, 301, 10, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1773168, 72115, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1773335, 72121, 815, 28, 34, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1774187, 72154, 220, 6, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1774410, 72162, 313, 10, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1774760, 72177, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1774927, 72183, 212, 7, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1775176, 72195, 263, 8, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1775442, 72205, 439, 16, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1775918, 72226, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1776085, 72232, 234, 8, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1776356, 72245, 298, 9, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1776657, 72256, 480, 17, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1777174, 72278, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1777341, 72284, 118, 3, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1777496, 72292, 263, 8, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1777762, 72302, 439, 16, 27, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1778238, 72323, 164, 4, 28, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1778405, 72329, 234, 8, 27, "Output",
   CellTags->"SimplifyDeltaFunction"]},
 "SimplifyGTI"->{
  Cell[1778666, 72341, 77, 1, 41, "ObjectName",
   CellTags->"SimplifyGTI"],
  Cell[1778768, 72346, 62, 1, 36, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1778833, 72349, 71, 1, 25, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1778907, 72352, 167, 9, 25, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1779111, 72366, 59, 1, 36, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1779173, 72369, 78, 2, 28, "Input",
   CellTags->"SimplifyGTI"]},
 "SimplifyPolyLog"->{
  Cell[1779266, 72374, 67, 1, 41, "ObjectName",
   CellTags->"SimplifyPolyLog"],
  Cell[1779358, 72379, 66, 1, 36, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1779427, 72382, 199, 4, 25, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1779629, 72388, 183, 9, 25, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1779849, 72402, 63, 1, 36, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1779915, 72405, 224, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1780164, 72415, 195, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1780362, 72423, 909, 31, 40, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1781308, 72459, 170, 5, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1781481, 72466, 650, 23, 27, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1782168, 72494, 221, 7, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1782392, 72503, 976, 35, 27, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1783405, 72543, 195, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1783603, 72551, 955, 34, 27, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1784595, 72590, 271, 9, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1784869, 72601, 890, 32, 38, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1785796, 72638, 248, 8, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1786047, 72648, 868, 31, 38, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1786952, 72684, 281, 9, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1787236, 72695, 732, 29, 38, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1788005, 72729, 216, 7, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1788224, 72738, 714, 26, 43, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1788975, 72769, 195, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1789173, 72777, 115, 3, 27, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1789325, 72785, 195, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1789523, 72793, 1658, 61, 27, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1791218, 72859, 271, 9, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1791492, 72870, 1796, 64, 70, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1793325, 72939, 221, 7, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1793549, 72948, 1818, 69, 43, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1795404, 73022, 271, 9, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1795678, 73033, 3421, 125, 95, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1799136, 73163, 191, 6, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1799330, 73171, 333, 11, 41, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1799700, 73187, 166, 5, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1799869, 73194, 251, 8, 40, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1800157, 73207, 146, 4, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1800306, 73213, 406, 14, 40, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1800749, 73232, 146, 4, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1800898, 73238, 355, 13, 40, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1801290, 73256, 146, 4, 28, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1801439, 73262, 355, 13, 40, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1801809, 73278, 116, 3, 28, "Input",
   CellTags->"SimplifyPolyLog"]},
 "Simplify2"->{
  Cell[1801940, 73284, 73, 1, 41, "ObjectName",
   CellTags->"Simplify2"],
  Cell[1802038, 73289, 60, 1, 36, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1802101, 73292, 81, 1, 25, "Text",
   CellTags->"Simplify2"],
  Cell[1802219, 73298, 57, 1, 36, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1802279, 73301, 76, 2, 28, "Input",
   CellTags->"Simplify2"]},
 "SmallDelta"->{
  Cell[1802370, 73306, 57, 1, 41, "ObjectName",
   CellTags->"SmallDelta"],
  Cell[1802452, 73311, 61, 1, 36, "Subsubsection",
   CellTags->"SmallDelta"],
  Cell[1802516, 73314, 87, 1, 25, "Text",
   CellTags->"SmallDelta"]},
 "SmallEpsilon"->{
  Cell[1802618, 73318, 61, 1, 41, "ObjectName",
   CellTags->"SmallEpsilon"],
  Cell[1802704, 73323, 63, 1, 36, "Subsubsection",
   CellTags->"SmallEpsilon"],
  Cell[1802770, 73326, 91, 1, 25, "Text",
   CellTags->"SmallEpsilon"]},
 "SmallVariable"->{
  Cell[1802876, 73330, 63, 1, 41, "ObjectName",
   CellTags->"SmallVariable"],
  Cell[1802964, 73335, 64, 1, 36, "Subsubsection",
   CellTags->"SmallVariable"],
  Cell[1803031, 73338, 281, 5, 43, "Text",
   CellTags->"SmallVariable"],
  Cell[1803315, 73345, 437, 19, 25, "Text",
   CellTags->"SmallVariable"]},
 "SmallVariables"->{
  Cell[1803767, 73367, 65, 1, 41, "ObjectName",
   CellTags->"SmallVariables"],
  Cell[1803857, 73372, 65, 1, 36, "Subsubsection",
   CellTags->"SmallVariables"],
  Cell[1803925, 73375, 225, 5, 43, "Text",
   CellTags->"SmallVariables"],
  Cell[1804153, 73382, 182, 9, 25, "Text",
   CellTags->"SmallVariables"]},
 "SMP"->{
  Cell[1804350, 73394, 43, 1, 41, "ObjectName",
   CellTags->"SMP"],
  Cell[1804418, 73399, 54, 1, 36, "Subsubsection",
   CellTags->"SMP"],
  Cell[1804475, 73402, 232, 5, 43, "Text",
   CellTags->"SMP"],
  Cell[1804732, 73411, 127, 4, 28, "Input",
   CellTags->"SMP"],
  Cell[1804862, 73417, 166, 5, 42, "Output",
   CellTags->"SMP"],
  Cell[1805043, 73425, 168, 8, 25, "Text",
   CellTags->"SMP"],
  Cell[1805248, 73438, 51, 1, 36, "Subsubsection",
   CellTags->"SMP"],
  Cell[1805324, 73443, 109, 3, 28, "Input",
   CellTags->"SMP"],
  Cell[1805436, 73448, 101, 3, 27, "Output",
   CellTags->"SMP"],
  Cell[1818466, 74041, 162, 8, 70, "Text",
   CellTags->"SMP"]},
 "SMVertex"->{
  Cell[1805564, 73455, 53, 1, 41, "ObjectName",
   CellTags->"SMVertex"],
  Cell[1805642, 73460, 59, 1, 36, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1805704, 73463, 304, 6, 43, "Text",
   CellTags->"SMVertex"],
  Cell[1806033, 73473, 112, 3, 28, "Input",
   CellTags->"SMVertex"],
  Cell[1806148, 73478, 235, 8, 27, "Output",
   CellTags->"SMVertex"],
  Cell[1806432, 73492, 56, 1, 36, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1806513, 73497, 223, 6, 28, "Input",
   CellTags->"SMVertex"],
  Cell[1806739, 73505, 1146, 43, 27, "Output",
   CellTags->"SMVertex"]},
 "Smu"->{
  Cell[1807912, 73552, 61, 1, 41, "ObjectName",
   CellTags->"Smu"],
  Cell[1807998, 73557, 54, 1, 36, "Subsubsection",
   CellTags->"Smu"],
  Cell[1808055, 73560, 45, 1, 25, "Text",
   CellTags->"Smu"]},
 "Sn"->{
  Cell[1808115, 73564, 41, 1, 41, "ObjectName",
   CellTags->"Sn"],
  Cell[1808181, 73569, 53, 1, 36, "Subsubsection",
   CellTags->"Sn"],
  Cell[1808237, 73572, 300, 14, 25, "Text",
   CellTags->"Sn"]},
 "SO"->{
  Cell[1808552, 73589, 41, 1, 41, "ObjectName",
   CellTags->"SO"],
  Cell[1808618, 73594, 53, 1, 36, "Subsubsection",
   CellTags->"SO"],
  Cell[1808674, 73597, 173, 4, 43, "Text",
   CellTags->"SO"],
  Cell[1808850, 73603, 580, 28, 25, "Text",
   CellTags->"SO"],
  Cell[1809467, 73636, 50, 1, 36, "Subsubsection",
   CellTags->"SO"],
  Cell[1809542, 73641, 98, 3, 28, "Input",
   CellTags->"SO"],
  Cell[1809643, 73646, 265, 10, 27, "Output",
   CellTags->"SO"],
  Cell[1809945, 73661, 121, 4, 28, "Input",
   CellTags->"SO"],
  Cell[1810069, 73667, 384, 15, 27, "Output",
   CellTags->"SO"],
  Cell[1810490, 73687, 163, 6, 28, "Input",
   CellTags->"SO"],
  Cell[1810656, 73695, 217, 6, 42, "Output",
   CellTags->"SO"]},
 "SOD"->{
  Cell[1810900, 73705, 43, 1, 70, "ObjectName",
   CellTags->"SOD"],
  Cell[1810968, 73710, 54, 1, 70, "Subsubsection",
   CellTags->"SOD"],
  Cell[1811025, 73713, 191, 5, 70, "Text",
   CellTags->"SOD"],
  Cell[1811219, 73720, 492, 24, 70, "Text",
   CellTags->"SOD"],
  Cell[1811748, 73749, 51, 1, 36, "Subsubsection",
   CellTags->"SOD"],
  Cell[1811824, 73754, 100, 3, 28, "Input",
   CellTags->"SOD"],
  Cell[1811927, 73759, 266, 10, 27, "Output",
   CellTags->"SOD"],
  Cell[1812230, 73774, 123, 4, 28, "Input",
   CellTags->"SOD"],
  Cell[1812356, 73780, 385, 15, 27, "Output",
   CellTags->"SOD"],
  Cell[1812778, 73800, 165, 6, 28, "Input",
   CellTags->"SOD"],
  Cell[1812946, 73808, 268, 8, 42, "Output",
   CellTags->"SOD"]},
 "Solve2"->{
  Cell[1813241, 73820, 49, 1, 70, "ObjectName",
   CellTags->"Solve2"],
  Cell[1813315, 73825, 57, 1, 70, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1813375, 73828, 346, 7, 61, "Text",
   CellTags->"Solve2"],
  Cell[1813746, 73839, 112, 3, 28, "Input",
   CellTags->"Solve2"],
  Cell[1813861, 73844, 264, 8, 27, "Output",
   CellTags->"Solve2"],
  Cell[1814174, 73858, 54, 1, 36, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1814253, 73863, 390, 14, 28, "Input",
   CellTags->"Solve2"],
  Cell[1814646, 73879, 373, 13, 40, "Output",
   CellTags->"Solve2"],
  Cell[1815034, 73895, 111, 3, 25, "Text",
   CellTags->"Solve2"],
  Cell[1815170, 73902, 153, 5, 28, "Input",
   CellTags->"Solve2"],
  Cell[1815326, 73909, 176, 6, 27, "Output",
   CellTags->"Solve2"],
  Cell[1815539, 73920, 267, 8, 28, "Input",
   CellTags->"Solve2"],
  Cell[1815809, 73930, 176, 6, 27, "Output",
   CellTags->"Solve2"],
  Cell[1816022, 73941, 443, 15, 28, "Input",
   CellTags->"Solve2"],
  Cell[1816468, 73958, 314, 11, 41, "Output",
   CellTags->"Solve2"],
  Cell[1816819, 73974, 389, 14, 28, "Input",
   CellTags->"Solve2"],
  Cell[1817211, 73990, 406, 14, 40, "Output",
   CellTags->"Solve2"]},
 "Solve3"->{
  Cell[1817644, 74008, 49, 1, 70, "ObjectName",
   CellTags->"Solve3"],
  Cell[1817718, 74013, 57, 1, 70, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1817778, 74016, 271, 5, 43, "Text",
   CellTags->"Solve3"],
  Cell[1818074, 74025, 112, 3, 28, "Input",
   CellTags->"Solve3"],
  Cell[1818189, 74030, 262, 8, 27, "Output",
   CellTags->"Solve3"],
  Cell[1818665, 74054, 54, 1, 36, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1818744, 74059, 390, 14, 28, "Input",
   CellTags->"Solve3"],
  Cell[1819137, 74075, 373, 13, 40, "Output",
   CellTags->"Solve3"]},
 "SP"->{
  Cell[1819537, 74092, 41, 1, 70, "ObjectName",
   CellTags->"SP"],
  Cell[1819603, 74097, 53, 1, 70, "Subsubsection",
   CellTags->"SP"],
  Cell[1819659, 74100, 327, 11, 70, "Text",
   CellTags->"SP"],
  Cell[1819989, 74113, 429, 19, 70, "Text",
   CellTags->"SP"],
  Cell[1820455, 74137, 50, 1, 36, "Subsubsection",
   CellTags->"SP"],
  Cell[1820530, 74142, 184, 6, 28, "Input",
   CellTags->"SP"],
  Cell[1820717, 74150, 366, 16, 27, "Output",
   CellTags->"SP"],
  Cell[1821120, 74171, 189, 7, 28, "Input",
   CellTags->"SP"],
  Cell[1821312, 74180, 297, 11, 27, "Output",
   CellTags->"SP"],
  Cell[1821646, 74196, 234, 8, 28, "Input",
   CellTags->"SP"],
  Cell[1821883, 74206, 388, 16, 27, "Output",
   CellTags->"SP"],
  Cell[1822308, 74227, 168, 5, 28, "Input",
   CellTags->"SP"],
  Cell[1822479, 74234, 209, 8, 28, "Output",
   CellTags->"SP"],
  Cell[1822725, 74247, 157, 5, 28, "Input",
   CellTags->"SP"],
  Cell[1822885, 74254, 136, 4, 42, "Output",
   CellTags->"SP"],
  Cell[1823058, 74263, 188, 7, 28, "Input",
   CellTags->"SP"],
  Cell[1823249, 74272, 210, 6, 42, "Output",
   CellTags->"SP"],
  Cell[1823496, 74283, 217, 8, 28, "Input",
   CellTags->"SP"],
  Cell[1823716, 74293, 136, 4, 42, "Output",
   CellTags->"SP"]},
 "SPC"->{
  Cell[1823879, 74301, 43, 1, 41, "ObjectName",
   CellTags->"SPC"],
  Cell[1823947, 74306, 54, 1, 36, "Subsubsection",
   CellTags->"SPC"],
  Cell[1824004, 74309, 81, 1, 25, "Text",
   CellTags->"SPC"],
  Cell[1824088, 74312, 207, 9, 25, "Text",
   CellTags->"SPC"]},
 "SPD"->{
  Cell[1824310, 74324, 43, 1, 41, "ObjectName",
   CellTags->"SPD"],
  Cell[1824378, 74329, 54, 1, 36, "Subsubsection",
   CellTags->"SPD"],
  Cell[1824435, 74332, 346, 11, 43, "Text",
   CellTags->"SPD"],
  Cell[1824784, 74345, 522, 24, 25, "Text",
   CellTags->"SPD"],
  Cell[1825343, 74374, 51, 1, 36, "Subsubsection",
   CellTags->"SPD"],
  Cell[1825419, 74379, 187, 6, 28, "Input",
   CellTags->"SPD"],
  Cell[1825609, 74387, 367, 16, 27, "Output",
   CellTags->"SPD"],
  Cell[1826013, 74408, 191, 7, 28, "Input",
   CellTags->"SPD"],
  Cell[1826207, 74417, 298, 11, 27, "Output",
   CellTags->"SPD"],
  Cell[1826542, 74433, 236, 8, 28, "Input",
   CellTags->"SPD"],
  Cell[1826781, 74443, 389, 16, 27, "Output",
   CellTags->"SPD"],
  Cell[1827207, 74464, 170, 5, 28, "Input",
   CellTags->"SPD"],
  Cell[1827380, 74471, 210, 8, 28, "Output",
   CellTags->"SPD"],
  Cell[1827627, 74484, 159, 5, 28, "Input",
   CellTags->"SPD"],
  Cell[1827789, 74491, 138, 4, 42, "Output",
   CellTags->"SPD"],
  Cell[1827964, 74500, 190, 7, 28, "Input",
   CellTags->"SPD"],
  Cell[1828157, 74509, 261, 8, 42, "Output",
   CellTags->"SPD"],
  Cell[1828455, 74522, 219, 8, 28, "Input",
   CellTags->"SPD"],
  Cell[1828677, 74532, 138, 4, 42, "Output",
   CellTags->"SPD"],
  Cell[1828852, 74541, 270, 9, 28, "Input",
   CellTags->"SPD"],
  Cell[1829125, 74552, 138, 4, 42, "Output",
   CellTags->"SPD"]},
 "Spinor"->{
  Cell[1829290, 74560, 49, 1, 41, "ObjectName",
   CellTags->"Spinor"],
  Cell[1829364, 74565, 57, 1, 36, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1829424, 74568, 1326, 49, 131, "Text",
   CellTags->"Spinor"],
  Cell[1830753, 74619, 766, 34, 25, "Text",
   CellTags->"Spinor"],
  Cell[1831556, 74658, 54, 1, 36, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1831635, 74663, 106, 3, 28, "Input",
   CellTags->"Spinor"],
  Cell[1831744, 74668, 175, 6, 27, "Output",
   CellTags->"Spinor"],
  Cell[1831956, 74679, 129, 4, 28, "Input",
   CellTags->"Spinor"],
  Cell[1832088, 74685, 185, 6, 27, "Output",
   CellTags->"Spinor"],
  Cell[1832288, 74694, 146, 4, 25, "Text",
   CellTags->"Spinor"],
  Cell[1832459, 74702, 250, 8, 28, "Input",
   CellTags->"Spinor"],
  Cell[1832712, 74712, 126, 4, 27, "Output",
   CellTags->"Spinor"],
  Cell[1832875, 74721, 254, 8, 28, "Input",
   CellTags->"Spinor"],
  Cell[1833132, 74731, 251, 9, 27, "Output",
   CellTags->"Spinor"],
  Cell[1833420, 74745, 141, 4, 28, "Input",
   CellTags->"Spinor"],
  Cell[1833564, 74751, 192, 6, 42, "Output",
   CellTags->"Spinor"],
  Cell[1833771, 74760, 158, 4, 25, "Text",
   CellTags->"Spinor"],
  Cell[1833954, 74768, 212, 7, 28, "Input",
   CellTags->"Spinor"],
  Cell[1834169, 74777, 192, 6, 42, "Output",
   CellTags->"Spinor"],
  Cell[1834398, 74788, 165, 5, 28, "Input",
   CellTags->"Spinor"],
  Cell[1834566, 74795, 192, 6, 42, "Output",
   CellTags->"Spinor"],
  Cell[1834773, 74804, 77, 1, 25, "Text",
   CellTags->"Spinor"],
  Cell[1834875, 74809, 210, 7, 28, "Input",
   CellTags->"Spinor"],
  Cell[1835088, 74818, 192, 6, 42, "Output",
   CellTags->"Spinor"]},
 "SpinorCollect"->{
  Cell[1835307, 74828, 63, 1, 41, "ObjectName",
   CellTags->"SpinorCollect"],
  Cell[1835395, 74833, 64, 1, 36, "Subsubsection",
   CellTags->"SpinorCollect"],
  Cell[1835462, 74836, 191, 4, 43, "Text",
   CellTags->"SpinorCollect"],
  Cell[1835656, 74842, 301, 13, 25, "Text",
   CellTags->"SpinorCollect"]},
 "SpinorUBar"->{
  Cell[1835972, 74858, 57, 1, 41, "ObjectName",
   CellTags->"SpinorUBar"],
  Cell[1836054, 74863, 61, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1836118, 74866, 170, 7, 25, "Text",
   CellTags->"SpinorUBar"],
  Cell[1836291, 74875, 505, 24, 25, "Text",
   CellTags->"SpinorUBar"],
  Cell[1836833, 74904, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1836894, 74907, 85, 1, 25, "Text",
   CellTags->"SpinorUBar"],
  Cell[1837004, 74912, 114, 3, 28, "Input",
   CellTags->"SpinorUBar"],
  Cell[1837121, 74917, 202, 7, 27, "Output",
   CellTags->"SpinorUBar"],
  Cell[1837360, 74929, 137, 4, 28, "Input",
   CellTags->"SpinorUBar"],
  Cell[1837500, 74935, 212, 7, 27, "Output",
   CellTags->"SpinorUBar"],
  Cell[1837749, 74947, 173, 5, 28, "Input",
   CellTags->"SpinorUBar"],
  Cell[1837925, 74954, 152, 4, 42, "Output",
   CellTags->"SpinorUBar"],
  Cell[1838114, 74963, 204, 7, 28, "Input",
   CellTags->"SpinorUBar"],
  Cell[1838321, 74972, 196, 6, 42, "Output",
   CellTags->"SpinorUBar"],
  Cell[1838554, 74983, 233, 8, 28, "Input",
   CellTags->"SpinorUBar"],
  Cell[1838790, 74993, 196, 6, 42, "Output",
   CellTags->"SpinorUBar"]},
 "SpinorU"->{
  Cell[1839013, 75003, 51, 1, 41, "ObjectName",
   CellTags->"SpinorU"],
  Cell[1839089, 75008, 58, 1, 70, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1839150, 75011, 79, 1, 70, "Text",
   CellTags->"SpinorU"],
  Cell[1839232, 75014, 511, 24, 70, "Text",
   CellTags->"SpinorU"],
  Cell[1839780, 75043, 55, 1, 36, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1839860, 75048, 108, 3, 28, "Input",
   CellTags->"SpinorU"],
  Cell[1839971, 75053, 132, 4, 27, "Output",
   CellTags->"SpinorU"],
  Cell[1840140, 75062, 131, 4, 28, "Input",
   CellTags->"SpinorU"],
  Cell[1840274, 75068, 142, 4, 27, "Output",
   CellTags->"SpinorU"],
  Cell[1840453, 75077, 167, 5, 28, "Input",
   CellTags->"SpinorU"],
  Cell[1840623, 75084, 146, 4, 42, "Output",
   CellTags->"SpinorU"],
  Cell[1840806, 75093, 198, 7, 28, "Input",
   CellTags->"SpinorU"],
  Cell[1841007, 75102, 193, 6, 42, "Output",
   CellTags->"SpinorU"],
  Cell[1841237, 75113, 227, 8, 28, "Input",
   CellTags->"SpinorU"],
  Cell[1841467, 75123, 193, 6, 42, "Output",
   CellTags->"SpinorU"]},
 "SpinorVBar"->{
  Cell[1841687, 75133, 57, 1, 70, "ObjectName",
   CellTags->"SpinorVBar"],
  Cell[1841769, 75138, 61, 1, 70, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1841833, 75141, 175, 7, 70, "Text",
   CellTags->"SpinorVBar"],
  Cell[1842011, 75150, 505, 24, 70, "Text",
   CellTags->"SpinorVBar"],
  Cell[1842553, 75179, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1842636, 75184, 114, 3, 28, "Input",
   CellTags->"SpinorVBar"],
  Cell[1842753, 75189, 202, 7, 27, "Output",
   CellTags->"SpinorVBar"],
  Cell[1842992, 75201, 137, 4, 28, "Input",
   CellTags->"SpinorVBar"],
  Cell[1843132, 75207, 212, 7, 27, "Output",
   CellTags->"SpinorVBar"],
  Cell[1843381, 75219, 173, 5, 28, "Input",
   CellTags->"SpinorVBar"],
  Cell[1843557, 75226, 152, 4, 42, "Output",
   CellTags->"SpinorVBar"],
  Cell[1843746, 75235, 204, 7, 28, "Input",
   CellTags->"SpinorVBar"],
  Cell[1843953, 75244, 217, 7, 42, "Output",
   CellTags->"SpinorVBar"],
  Cell[1844207, 75256, 233, 8, 28, "Input",
   CellTags->"SpinorVBar"],
  Cell[1844443, 75266, 217, 7, 42, "Output",
   CellTags->"SpinorVBar"]},
 "SpinorV"->{
  Cell[1844687, 75277, 51, 1, 70, "ObjectName",
   CellTags->"SpinorV"],
  Cell[1844763, 75282, 58, 1, 70, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1844824, 75285, 79, 1, 70, "Text",
   CellTags->"SpinorV"],
  Cell[1844906, 75288, 511, 24, 70, "Text",
   CellTags->"SpinorV"],
  Cell[1845454, 75317, 55, 1, 36, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1845534, 75322, 108, 3, 28, "Input",
   CellTags->"SpinorV"],
  Cell[1845645, 75327, 132, 4, 27, "Output",
   CellTags->"SpinorV"],
  Cell[1845814, 75336, 131, 4, 28, "Input",
   CellTags->"SpinorV"],
  Cell[1845948, 75342, 142, 4, 27, "Output",
   CellTags->"SpinorV"],
  Cell[1846127, 75351, 167, 5, 28, "Input",
   CellTags->"SpinorV"],
  Cell[1846297, 75358, 146, 4, 42, "Output",
   CellTags->"SpinorV"],
  Cell[1846480, 75367, 198, 7, 28, "Input",
   CellTags->"SpinorV"],
  Cell[1846681, 75376, 214, 7, 42, "Output",
   CellTags->"SpinorV"],
  Cell[1846932, 75388, 227, 8, 28, "Input",
   CellTags->"SpinorV"],
  Cell[1847162, 75398, 214, 7, 42, "Output",
   CellTags->"SpinorV"]},
 "SpinPolarizationSum"->{
  Cell[1847403, 75409, 75, 1, 41, "ObjectName",
   CellTags->"SpinPolarizationSum"],
  Cell[1847503, 75414, 70, 1, 36, "Subsubsection",
   CellTags->"SpinPolarizationSum"],
  Cell[1847576, 75417, 164, 4, 25, "Text",
   CellTags->"SpinPolarizationSum"],
  Cell[1847743, 75423, 203, 8, 25, "Text",
   CellTags->"SpinPolarizationSum"]},
 "SPL"->{
  Cell[1847961, 75434, 43, 1, 41, "ObjectName",
   CellTags->"SPL"],
  Cell[1848029, 75439, 54, 1, 36, "Subsubsection",
   CellTags->"SPL"],
  Cell[1848086, 75442, 77, 1, 25, "Text",
   CellTags->"SPL"],
  Cell[1848166, 75445, 195, 9, 25, "Text",
   CellTags->"SPL"]},
 "SplittingFunction"->{
  Cell[1848376, 75457, 71, 1, 41, "ObjectName",
   CellTags->"SplittingFunction"],
  Cell[1848472, 75462, 68, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1848543, 75465, 218, 7, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1848786, 75476, 134, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1848923, 75481, 181, 6, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1849119, 75490, 218, 9, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1849374, 75504, 65, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1849442, 75507, 65, 1, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1849510, 75510, 221, 6, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1849734, 75518, 170, 4, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1849929, 75526, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1850062, 75531, 553, 20, 41, "Output",
   CellTags->"SplittingFunction"],
  Cell[1850652, 75556, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1850785, 75561, 304, 11, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1851126, 75577, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1851259, 75582, 269, 9, 40, "Output",
   CellTags->"SplittingFunction"],
  Cell[1851565, 75596, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1851698, 75601, 859, 29, 41, "Output",
   CellTags->"SplittingFunction"],
  Cell[1852594, 75635, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1852727, 75640, 1128, 38, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1853892, 75683, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1854025, 75688, 594, 20, 40, "Output",
   CellTags->"SplittingFunction"],
  Cell[1854656, 75713, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1854789, 75718, 540, 19, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1855344, 75740, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1855477, 75745, 63, 1, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1855543, 75748, 221, 6, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1855789, 75758, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1855922, 75763, 553, 20, 41, "Output",
   CellTags->"SplittingFunction"],
  Cell[1856512, 75788, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1856645, 75793, 239, 9, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1856921, 75807, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1857054, 75812, 235, 8, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1857326, 75825, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1857459, 75830, 834, 29, 41, "Output",
   CellTags->"SplittingFunction"],
  Cell[1858330, 75864, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1858463, 75869, 1247, 42, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1859747, 75916, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1859880, 75921, 503, 18, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1860420, 75944, 131, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1860554, 75949, 445, 16, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1861036, 75970, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1861169, 75975, 445, 16, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1861651, 75996, 131, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1861785, 76001, 445, 16, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1862267, 76022, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1862400, 76027, 1362, 46, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1863799, 76078, 131, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1863933, 76083, 1362, 46, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1865332, 76134, 131, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1865466, 76139, 660, 23, 27, "Output",
   CellTags->"SplittingFunction"],
  Cell[1866163, 76167, 132, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1866298, 76172, 5987, 193, 215, "Output",
   CellTags->"SplittingFunction"],
  Cell[1872322, 76370, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1872455, 76375, 3011, 96, 93, "Output",
   CellTags->"SplittingFunction"],
  Cell[1875503, 76476, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1875636, 76481, 3710, 121, 136, "Output",
   CellTags->"SplittingFunction"],
  Cell[1879383, 76607, 130, 3, 28, "Input",
   CellTags->"SplittingFunction"],
  Cell[1879516, 76612, 4577, 151, 176, "Output",
   CellTags->"SplittingFunction"]},
 "StandardMatrixElement"->{
  Cell[1884120, 76767, 79, 1, 41, "ObjectName",
   CellTags->"StandardMatrixElement"],
  Cell[1884224, 76772, 72, 1, 36, "Subsubsection",
   CellTags->"StandardMatrixElement"],
  Cell[1884299, 76775, 134, 3, 25, "Text",
   CellTags->"StandardMatrixElement"],
  Cell[1884436, 76780, 189, 9, 25, "Text",
   CellTags->"StandardMatrixElement"]},
 "SubContext"->{
  Cell[1884640, 76792, 57, 1, 41, "ObjectName",
   CellTags->"SubContext"],
  Cell[1884722, 76797, 61, 1, 36, "Subsubsection",
   CellTags->"SubContext"],
  Cell[1884786, 76800, 120, 3, 25, "Text",
   CellTags->"SubContext"],
  Cell[1884909, 76805, 279, 14, 25, "Text",
   CellTags->"SubContext"]},
 "SubLoop"->{
  Cell[1885203, 76822, 51, 1, 41, "ObjectName",
   CellTags->"SubLoop"],
  Cell[1885279, 76827, 58, 1, 36, "Subsubsection",
   CellTags->"SubLoop"],
  Cell[1885340, 76830, 152, 4, 25, "Text",
   CellTags->"SubLoop"],
  Cell[1885495, 76836, 178, 9, 25, "Text",
   CellTags->"SubLoop"]},
 "SumP"->{
  Cell[1885688, 76848, 45, 1, 41, "ObjectName",
   CellTags->"SumP"],
  Cell[1885758, 76853, 55, 1, 36, "Subsubsection",
   CellTags->"SumP"],
  Cell[1885816, 76856, 767, 31, 28, "Text",
   CellTags->"SumP"],
  Cell[1886586, 76889, 261, 14, 25, "Text",
   CellTags->"SumP"],
  Cell[1886884, 76908, 52, 1, 36, "Subsubsection",
   CellTags->"SumP"],
  Cell[1886961, 76913, 149, 5, 28, "Input",
   CellTags->"SumP"],
  Cell[1887113, 76920, 263, 8, 27, "Output",
   CellTags->"SumP"],
  Cell[1887413, 76933, 149, 5, 28, "Input",
   CellTags->"SumP"],
  Cell[1887565, 76940, 263, 8, 27, "Output",
   CellTags->"SumP"],
  Cell[1887865, 76953, 125, 4, 28, "Input",
   CellTags->"SumP"],
  Cell[1887993, 76959, 238, 7, 27, "Output",
   CellTags->"SumP"],
  Cell[1888268, 76971, 125, 4, 28, "Input",
   CellTags->"SumP"],
  Cell[1888396, 76977, 124, 4, 38, "Output",
   CellTags->"SumP"],
  Cell[1888557, 76986, 301, 12, 51, "Input",
   CellTags->"SumP"],
  Cell[1888861, 77000, 124, 4, 38, "Output",
   CellTags->"SumP"],
  Cell[1889022, 77009, 186, 6, 28, "Input",
   CellTags->"SumP"],
  Cell[1889211, 77017, 764, 29, 40, "Output",
   CellTags->"SumP"],
  Cell[1890012, 77051, 124, 4, 28, "Input",
   CellTags->"SumP"],
  Cell[1890139, 77057, 124, 4, 38, "Output",
   CellTags->"SumP"]},
 "SumS"->{
  Cell[1890290, 77065, 45, 1, 41, "ObjectName",
   CellTags->"SumS"],
  Cell[1890360, 77070, 55, 1, 36, "Subsubsection",
   CellTags->"SumS"],
  Cell[1890418, 77073, 1438, 57, 43, "Text",
   CellTags->"SumS"],
  Cell[1891881, 77134, 108, 3, 28, "Input",
   CellTags->"SumS"],
  Cell[1891992, 77139, 165, 6, 27, "Output",
   CellTags->"SumS"],
  Cell[1892172, 77148, 261, 14, 25, "Text",
   CellTags->"SumS"],
  Cell[1892470, 77167, 52, 1, 36, "Subsubsection",
   CellTags->"SumS"],
  Cell[1892547, 77172, 149, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1892699, 77179, 211, 8, 27, "Output",
   CellTags->"SumS"],
  Cell[1892947, 77192, 149, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1893099, 77199, 211, 8, 27, "Output",
   CellTags->"SumS"],
  Cell[1893347, 77212, 144, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1893494, 77219, 172, 6, 27, "Output",
   CellTags->"SumS"],
  Cell[1893703, 77230, 173, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1893879, 77237, 261, 10, 39, "Output",
   CellTags->"SumS"],
  Cell[1894177, 77252, 197, 6, 28, "Input",
   CellTags->"SumS"],
  Cell[1894377, 77260, 340, 13, 45, "Output",
   CellTags->"SumS"],
  Cell[1894732, 77276, 192, 6, 28, "Input",
   CellTags->"SumS"],
  Cell[1894949, 77286, 149, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1895101, 77293, 509, 19, 45, "Output",
   CellTags->"SumS"],
  Cell[1895625, 77315, 193, 6, 28, "Input",
   CellTags->"SumS"],
  Cell[1895843, 77325, 125, 4, 28, "Input",
   CellTags->"SumS"],
  Cell[1895971, 77331, 124, 4, 38, "Output",
   CellTags->"SumS"],
  Cell[1896132, 77340, 174, 6, 51, "Input",
   CellTags->"SumS"],
  Cell[1896309, 77348, 124, 4, 38, "Output",
   CellTags->"SumS"],
  Cell[1896470, 77357, 159, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1896632, 77364, 212, 8, 27, "Output",
   CellTags->"SumS"],
  Cell[1896881, 77377, 146, 4, 28, "Input",
   CellTags->"SumS"],
  Cell[1897030, 77383, 149, 5, 38, "Output",
   CellTags->"SumS"],
  Cell[1897216, 77393, 144, 5, 28, "Input",
   CellTags->"SumS"],
  Cell[1897363, 77400, 142, 5, 38, "Output",
   CellTags->"SumS"],
  Cell[1897542, 77410, 125, 4, 28, "Input",
   CellTags->"SumS"],
  Cell[1897670, 77416, 142, 5, 38, "Output",
   CellTags->"SumS"]},
 "SumT"->{
  Cell[1897839, 77425, 45, 1, 41, "ObjectName",
   CellTags->"SumT"],
  Cell[1897909, 77430, 55, 1, 36, "Subsubsection",
   CellTags->"SumT"],
  Cell[1897967, 77433, 473, 15, 43, "Text",
   CellTags->"SumT"],
  Cell[1898443, 77450, 261, 14, 25, "Text",
   CellTags->"SumT"],
  Cell[1898741, 77469, 52, 1, 36, "Subsubsection",
   CellTags->"SumT"],
  Cell[1898818, 77474, 149, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1898970, 77481, 236, 9, 28, "Output",
   CellTags->"SumT"],
  Cell[1899243, 77495, 149, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1899395, 77502, 236, 9, 28, "Output",
   CellTags->"SumT"],
  Cell[1899668, 77516, 125, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1899796, 77522, 180, 7, 28, "Output",
   CellTags->"SumT"],
  Cell[1900013, 77534, 173, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1900189, 77541, 361, 14, 42, "Output",
   CellTags->"SumT"],
  Cell[1900587, 77560, 125, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1900715, 77566, 142, 5, 38, "Output",
   CellTags->"SumT"],
  Cell[1900894, 77576, 244, 9, 51, "Input",
   CellTags->"SumT"],
  Cell[1901141, 77587, 142, 5, 38, "Output",
   CellTags->"SumT"],
  Cell[1901320, 77597, 159, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1901482, 77604, 237, 9, 28, "Output",
   CellTags->"SumT"],
  Cell[1901756, 77618, 136, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1901895, 77624, 253, 7, 38, "Output",
   CellTags->"SumT"],
  Cell[1902185, 77636, 125, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1902313, 77642, 142, 5, 38, "Output",
   CellTags->"SumT"],
  Cell[1902492, 77652, 144, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1902639, 77659, 142, 5, 38, "Output",
   CellTags->"SumT"],
  Cell[1902818, 77669, 268, 10, 51, "Input",
   CellTags->"SumT"],
  Cell[1903089, 77681, 705, 26, 42, "Output",
   CellTags->"SumT"],
  Cell[1903831, 77712, 136, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1903970, 77718, 170, 6, 38, "Output",
   CellTags->"SumT"],
  Cell[1904177, 77729, 155, 5, 28, "Input",
   CellTags->"SumT"],
  Cell[1904335, 77736, 253, 7, 38, "Output",
   CellTags->"SumT"],
  Cell[1904625, 77748, 129, 4, 28, "Input",
   CellTags->"SumT"],
  Cell[1904757, 77754, 458, 16, 40, "Output",
   CellTags->"SumT"],
  Cell[1905252, 77775, 240, 9, 28, "Input",
   CellTags->"SumT"],
  Cell[1905495, 77786, 458, 16, 40, "Output",
   CellTags->"SumT"]},
 "SUND"->{
  Cell[1905980, 77806, 45, 1, 41, "ObjectName",
   CellTags->"SUND"],
  Cell[1906050, 77811, 55, 1, 36, "Subsubsection",
   CellTags->"SUND"],
  Cell[1906108, 77814, 224, 10, 25, "Text",
   CellTags->"SUND"],
  Cell[1906335, 77826, 392, 19, 25, "Text",
   CellTags->"SUND"],
  Cell[1906764, 77850, 52, 1, 36, "Subsubsection",
   CellTags->"SUND"],
  Cell[1906841, 77855, 135, 4, 28, "Input",
   CellTags->"SUND"],
  Cell[1906979, 77861, 147, 5, 27, "Output",
   CellTags->"SUND"],
  Cell[1907163, 77871, 211, 6, 28, "Input",
   CellTags->"SUND"],
  Cell[1907377, 77879, 1086, 44, 27, "Output",
   CellTags->"SUND"],
  Cell[1908500, 77928, 135, 4, 28, "Input",
   CellTags->"SUND"],
  Cell[1908638, 77934, 147, 5, 27, "Output",
   CellTags->"SUND"],
  Cell[1908822, 77944, 135, 4, 28, "Input",
   CellTags->"SUND"],
  Cell[1908960, 77950, 147, 5, 27, "Output",
   CellTags->"SUND"],
  Cell[1909144, 77960, 266, 8, 28, "Input",
   CellTags->"SUND"],
  Cell[1909413, 77970, 260, 9, 27, "Output",
   CellTags->"SUND"],
  Cell[1909710, 77984, 377, 12, 28, "Input",
   CellTags->"SUND"],
  Cell[1910090, 77998, 304, 11, 40, "Output",
   CellTags->"SUND"],
  Cell[1910431, 78014, 377, 12, 28, "Input",
   CellTags->"SUND"],
  Cell[1910811, 78028, 474, 19, 40, "Output",
   CellTags->"SUND"],
  Cell[1911322, 78052, 173, 6, 28, "Input",
   CellTags->"SUND"],
  Cell[1911498, 78060, 150, 4, 42, "Output",
   CellTags->"SUND"],
  Cell[1911685, 78069, 202, 7, 28, "Input",
   CellTags->"SUND"],
  Cell[1911890, 78078, 258, 7, 42, "Output",
   CellTags->"SUND"],
  Cell[1912185, 78090, 231, 8, 28, "Input",
   CellTags->"SUND"],
  Cell[1912419, 78100, 150, 4, 42, "Output",
   CellTags->"SUND"],
  Cell[1912584, 78107, 104, 3, 28, "Input",
   CellTags->"SUND"]},
 "SUNDelta"->{
  Cell[1912703, 78113, 53, 1, 41, "ObjectName",
   CellTags->"SUNDelta"],
  Cell[1912781, 78118, 59, 1, 36, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1912843, 78121, 172, 6, 25, "Text",
   CellTags->"SUNDelta"],
  Cell[1913018, 78129, 717, 33, 25, "Text",
   CellTags->"SUNDelta"],
  Cell[1913772, 78167, 56, 1, 36, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1913853, 78172, 133, 4, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1913989, 78178, 153, 5, 27, "Output",
   CellTags->"SUNDelta"],
  Cell[1914179, 78188, 262, 8, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1914444, 78198, 301, 13, 27, "Output",
   CellTags->"SUNDelta"],
  Cell[1914782, 78216, 205, 6, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1914990, 78224, 301, 13, 27, "Output",
   CellTags->"SUNDelta"],
  Cell[1915328, 78242, 169, 5, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1915500, 78249, 148, 4, 42, "Output",
   CellTags->"SUNDelta"],
  Cell[1915685, 78258, 200, 7, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1915888, 78267, 220, 6, 42, "Output",
   CellTags->"SUNDelta"],
  Cell[1916145, 78278, 229, 8, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1916377, 78288, 142, 4, 42, "Output",
   CellTags->"SUNDelta"],
  Cell[1916556, 78297, 194, 7, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1916753, 78306, 220, 6, 42, "Output",
   CellTags->"SUNDelta"],
  Cell[1916988, 78315, 256, 5, 43, "Text",
   CellTags->"SUNDelta"],
  Cell[1917269, 78324, 315, 10, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1917587, 78336, 460, 20, 27, "Output",
   CellTags->"SUNDelta"],
  Cell[1918084, 78361, 110, 3, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1918197, 78366, 395, 11, 42, "Output",
   CellTags->"SUNDelta"],
  Cell[1918629, 78382, 200, 7, 28, "Input",
   CellTags->"SUNDelta"],
  Cell[1918832, 78391, 236, 6, 42, "Output",
   CellTags->"SUNDelta"]},
 "SUNDeltaContract"->{
  Cell[1919095, 78401, 69, 1, 41, "ObjectName",
   CellTags->"SUNDeltaContract"],
  Cell[1919189, 78406, 67, 1, 36, "Subsubsection",
   CellTags->"SUNDeltaContract"],
  Cell[1919259, 78409, 397, 9, 61, "Text",
   CellTags->"SUNDeltaContract"],
  Cell[1919659, 78420, 297, 14, 25, "Text",
   CellTags->"SUNDeltaContract"]},
 "SUNF"->{
  Cell[1919971, 78437, 45, 1, 41, "ObjectName",
   CellTags->"SUNF"],
  Cell[1920041, 78442, 55, 1, 36, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1920099, 78445, 190, 6, 25, "Text",
   CellTags->"SUNF"],
  Cell[1920314, 78455, 108, 3, 28, "Input",
   CellTags->"SUNF"],
  Cell[1920425, 78460, 168, 6, 27, "Output",
   CellTags->"SUNF"],
  Cell[1920608, 78469, 701, 34, 25, "Text",
   CellTags->"SUNF"],
  Cell[1921346, 78508, 52, 1, 36, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1921423, 78513, 294, 10, 28, "Input",
   CellTags->"SUNF"],
  Cell[1921720, 78525, 272, 10, 27, "Output",
   CellTags->"SUNF"],
  Cell[1922029, 78540, 103, 3, 28, "Input",
   CellTags->"SUNF"],
  Cell[1922135, 78545, 764, 34, 27, "Output",
   CellTags->"SUNF"],
  Cell[1922936, 78584, 110, 3, 28, "Input",
   CellTags->"SUNF"],
  Cell[1923049, 78589, 455, 20, 27, "Output",
   CellTags->"SUNF"],
  Cell[1923541, 78614, 135, 4, 28, "Input",
   CellTags->"SUNF"],
  Cell[1923679, 78620, 147, 5, 27, "Output",
   CellTags->"SUNF"],
  Cell[1923863, 78630, 163, 5, 28, "Input",
   CellTags->"SUNF"],
  Cell[1924029, 78637, 101, 3, 27, "Output",
   CellTags->"SUNF"],
  Cell[1924145, 78643, 319, 10, 25, "Text",
   CellTags->"SUNF"],
  Cell[1924489, 78657, 185, 5, 28, "Input",
   CellTags->"SUNF"],
  Cell[1924677, 78664, 1147, 44, 27, "Output",
   CellTags->"SUNF"],
  Cell[1925861, 78713, 266, 8, 28, "Input",
   CellTags->"SUNF"],
  Cell[1926130, 78723, 353, 15, 27, "Output",
   CellTags->"SUNF"],
  Cell[1926520, 78743, 239, 7, 28, "Input",
   CellTags->"SUNF"],
  Cell[1926762, 78752, 1166, 45, 27, "Output",
   CellTags->"SUNF"],
  Cell[1927965, 78802, 173, 6, 28, "Input",
   CellTags->"SUNF"],
  Cell[1928141, 78810, 150, 4, 42, "Output",
   CellTags->"SUNF"],
  Cell[1928328, 78819, 202, 7, 28, "Input",
   CellTags->"SUNF"],
  Cell[1928533, 78828, 258, 7, 42, "Output",
   CellTags->"SUNF"],
  Cell[1928828, 78840, 231, 8, 28, "Input",
   CellTags->"SUNF"],
  Cell[1929062, 78850, 150, 4, 42, "Output",
   CellTags->"SUNF"],
  Cell[1929249, 78859, 135, 4, 28, "Input",
   CellTags->"SUNF"],
  Cell[1929387, 78865, 147, 5, 27, "Output",
   CellTags->"SUNF"],
  Cell[1929571, 78875, 162, 5, 28, "Input",
   CellTags->"SUNF"],
  Cell[1929736, 78882, 401, 18, 27, "Output",
   CellTags->"SUNF"]},
 "SUNFJacobi"->{
  Cell[1930164, 78904, 57, 1, 41, "ObjectName",
   CellTags->"SUNFJacobi"],
  Cell[1930246, 78909, 61, 1, 36, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1930310, 78912, 146, 4, 25, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1930459, 78918, 283, 13, 25, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1930779, 78936, 58, 1, 36, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1930862, 78941, 380, 12, 28, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1931245, 78955, 694, 32, 27, "Output",
   CellTags->"SUNFJacobi"],
  Cell[1931976, 78992, 379, 12, 28, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1932358, 79006, 1362, 62, 27, "Output",
   CellTags->"SUNFJacobi"]},
 "SUNIndex"->{
  Cell[1933747, 79072, 53, 1, 41, "ObjectName",
   CellTags->"SUNIndex"],
  Cell[1933825, 79077, 59, 1, 36, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1933887, 79080, 135, 6, 25, "Text",
   CellTags->"SUNIndex"],
  Cell[1934025, 79088, 405, 18, 25, "Text",
   CellTags->"SUNIndex"],
  Cell[1934467, 79111, 56, 1, 36, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1934548, 79116, 110, 3, 28, "Input",
   CellTags->"SUNIndex"],
  Cell[1934661, 79121, 105, 3, 27, "Output",
   CellTags->"SUNIndex"],
  Cell[1934803, 79129, 110, 3, 28, "Input",
   CellTags->"SUNIndex"],
  Cell[1934916, 79134, 125, 3, 42, "Output",
   CellTags->"SUNIndex"],
  Cell[1935078, 79142, 200, 7, 28, "Input",
   CellTags->"SUNIndex"],
  Cell[1935281, 79151, 220, 6, 42, "Output",
   CellTags->"SUNIndex"]},
 "SUNIndexRename"->{
  Cell[1935528, 79161, 65, 1, 41, "ObjectName",
   CellTags->"SUNIndexRename"],
  Cell[1935618, 79166, 65, 1, 36, "Subsubsection",
   CellTags->"SUNIndexRename"],
  Cell[1935686, 79169, 163, 4, 25, "Text",
   CellTags->"SUNIndexRename"],
  Cell[1935852, 79175, 304, 14, 25, "Text",
   CellTags->"SUNIndexRename"]},
 "SUNN"->{
  Cell[1936171, 79192, 45, 1, 41, "ObjectName",
   CellTags->"SUNN"],
  Cell[1936241, 79197, 55, 1, 36, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1936299, 79200, 203, 7, 25, "Text",
   CellTags->"SUNN"],
  Cell[1936505, 79209, 579, 29, 25, "Text",
   CellTags->"SUNN"],
  Cell[1937121, 79243, 52, 1, 36, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1937198, 79248, 317, 9, 28, "Input",
   CellTags->"SUNN"],
  Cell[1937518, 79259, 149, 5, 27, "Output",
   CellTags->"SUNN"]},
 "SUNNToCACF"->{
  Cell[1937694, 79268, 57, 1, 41, "ObjectName",
   CellTags->"SUNNToCACF"],
  Cell[1937776, 79273, 61, 1, 36, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1937840, 79276, 187, 4, 43, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1938030, 79282, 573, 29, 25, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1938640, 79316, 58, 1, 36, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1938723, 79321, 322, 9, 28, "Input",
   CellTags->"SUNNToCACF"],
  Cell[1939048, 79332, 186, 6, 27, "Output",
   CellTags->"SUNNToCACF"]},
 "SUNSimplify"->{
  Cell[1939261, 79342, 59, 1, 41, "ObjectName",
   CellTags->"SUNSimplify"],
  Cell[1939345, 79347, 62, 1, 36, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1939410, 79350, 376, 7, 61, "Text",
   CellTags->"SUNSimplify"],
  Cell[1939811, 79361, 122, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1939936, 79366, 512, 13, 49, "Output",
   CellTags->"SUNSimplify"],
  Cell[1940463, 79382, 299, 13, 25, "Text",
   CellTags->"SUNSimplify"],
  Cell[1940799, 79400, 59, 1, 36, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1940883, 79405, 247, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1941133, 79415, 231, 8, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1941401, 79428, 117, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1941521, 79433, 304, 13, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1941862, 79451, 189, 6, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1942054, 79459, 249, 10, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1942340, 79474, 117, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1942460, 79479, 130, 4, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1942627, 79488, 183, 5, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1942813, 79495, 203, 7, 40, "Output",
   CellTags->"SUNSimplify"],
  Cell[1943053, 79507, 254, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1943310, 79517, 227, 8, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1943574, 79530, 117, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1943694, 79535, 360, 15, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1944091, 79555, 269, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1944363, 79565, 227, 8, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1944627, 79578, 117, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1944747, 79583, 197, 6, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1944981, 79594, 259, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1945243, 79604, 227, 8, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1945507, 79617, 117, 3, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1945627, 79622, 360, 15, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1946024, 79642, 259, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1946286, 79652, 227, 8, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1946550, 79665, 180, 5, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1946733, 79672, 108, 3, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1946878, 79680, 384, 12, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1947265, 79694, 311, 11, 40, "Output",
   CellTags->"SUNSimplify"],
  Cell[1947613, 79710, 389, 12, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1948005, 79724, 448, 18, 40, "Output",
   CellTags->"SUNSimplify"],
  Cell[1948490, 79747, 246, 7, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1948739, 79756, 1173, 45, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1949949, 79806, 246, 7, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1950198, 79815, 1132, 44, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1951367, 79864, 273, 8, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1951643, 79874, 251, 8, 38, "Output",
   CellTags->"SUNSimplify"],
  Cell[1951931, 79887, 439, 14, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1952373, 79903, 379, 14, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1952789, 79922, 181, 5, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1952973, 79929, 705, 32, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1953715, 79966, 238, 6, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1953956, 79974, 1363, 62, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1955356, 80041, 200, 6, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1955559, 80049, 164, 5, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1955760, 80059, 341, 10, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1956104, 80071, 543, 21, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1956684, 80097, 181, 5, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1956868, 80104, 543, 21, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1957448, 80130, 180, 5, 28, "Input",
   CellTags->"SUNSimplify"],
  Cell[1957631, 80137, 571, 22, 27, "Output",
   CellTags->"SUNSimplify"],
  Cell[1958217, 80162, 208, 6, 28, "Input",
   CellTags->"SUNSimplify"]},
 "SUNT"->{
  Cell[1958440, 80171, 45, 1, 41, "ObjectName",
   CellTags->"SUNT"],
  Cell[1958510, 80176, 55, 1, 36, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1958568, 80179, 237, 10, 25, "Text",
   CellTags->"SUNT"],
  Cell[1958808, 80191, 674, 34, 25, "Text",
   CellTags->"SUNT"],
  Cell[1959519, 80230, 52, 1, 36, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1959596, 80235, 102, 3, 28, "Input",
   CellTags->"SUNT"],
  Cell[1959701, 80240, 157, 6, 27, "Output",
   CellTags->"SUNT"],
  Cell[1959873, 80249, 203, 7, 25, "Text",
   CellTags->"SUNT"],
  Cell[1960101, 80260, 207, 6, 28, "Input",
   CellTags->"SUNT"],
  Cell[1960311, 80268, 311, 13, 27, "Output",
   CellTags->"SUNT"],
  Cell[1960659, 80286, 145, 4, 28, "Input",
   CellTags->"SUNT"],
  Cell[1960807, 80292, 365, 16, 27, "Output",
   CellTags->"SUNT"],
  Cell[1961209, 80313, 242, 7, 28, "Input",
   CellTags->"SUNT"],
  Cell[1961454, 80322, 268, 11, 38, "Output",
   CellTags->"SUNT"],
  Cell[1961759, 80338, 184, 5, 28, "Input",
   CellTags->"SUNT"],
  Cell[1961946, 80345, 133, 4, 27, "Output",
   CellTags->"SUNT"],
  Cell[1962116, 80354, 174, 5, 28, "Input",
   CellTags->"SUNT"],
  Cell[1962293, 80361, 399, 16, 38, "Output",
   CellTags->"SUNT"],
  Cell[1962729, 80382, 242, 7, 28, "Input",
   CellTags->"SUNT"],
  Cell[1962974, 80391, 268, 11, 38, "Output",
   CellTags->"SUNT"],
  Cell[1963257, 80405, 422, 15, 25, "Text",
   CellTags->"SUNT"],
  Cell[1963704, 80424, 161, 5, 28, "Input",
   CellTags->"SUNT"],
  Cell[1963868, 80431, 328, 14, 40, "Output",
   CellTags->"SUNT"],
  Cell[1964211, 80448, 197, 7, 25, "Text",
   CellTags->"SUNT"],
  Cell[1964433, 80459, 197, 6, 28, "Input",
   CellTags->"SUNT"],
  Cell[1964633, 80467, 353, 15, 27, "Output",
   CellTags->"SUNT"],
  Cell[1965023, 80487, 198, 6, 28, "Input",
   CellTags->"SUNT"],
  Cell[1965224, 80495, 263, 8, 52, "Output",
   CellTags->"SUNT"],
  Cell[1965524, 80508, 167, 6, 28, "Input",
   CellTags->"SUNT"],
  Cell[1965694, 80516, 152, 4, 42, "Output",
   CellTags->"SUNT"],
  Cell[1965883, 80525, 195, 7, 28, "Input",
   CellTags->"SUNT"],
  Cell[1966081, 80534, 117, 3, 42, "Output",
   CellTags->"SUNT"]},
 "SUNTrace"->{
  Cell[1966225, 80541, 53, 1, 41, "ObjectName",
   CellTags->"SUNTrace"],
  Cell[1966303, 80546, 59, 1, 36, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1966365, 80549, 81, 1, 25, "Text",
   CellTags->"SUNTrace"],
  Cell[1966471, 80554, 116, 3, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1966590, 80559, 172, 6, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1966777, 80568, 280, 14, 25, "Text",
   CellTags->"SUNTrace"],
  Cell[1967094, 80587, 56, 1, 36, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1967175, 80592, 165, 5, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1967343, 80599, 332, 14, 40, "Output",
   CellTags->"SUNTrace"],
  Cell[1967712, 80618, 175, 5, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1967890, 80625, 524, 22, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1968451, 80652, 240, 7, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1968694, 80661, 811, 35, 40, "Output",
   CellTags->"SUNTrace"],
  Cell[1969542, 80701, 185, 5, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1969730, 80708, 641, 27, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1970408, 80740, 278, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1970689, 80750, 4229, 183, 40, "Output",
   CellTags->"SUNTrace"],
  Cell[1974955, 80938, 177, 5, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1975135, 80945, 641, 27, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1975813, 80977, 288, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1976104, 80987, 11403, 439, 173, "Output",
   CellTags->"SUNTrace"],
  Cell[1987544, 81431, 177, 5, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1987724, 81438, 758, 32, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1988519, 81475, 270, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1988792, 81485, 105, 3, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1988934, 81493, 270, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1989207, 81503, 105, 3, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1989349, 81511, 270, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1989622, 81521, 105, 3, 27, "Output",
   CellTags->"SUNTrace"],
  Cell[1989764, 81529, 270, 8, 28, "Input",
   CellTags->"SUNTrace"],
  Cell[1990037, 81539, 576, 23, 27, "Output",
   CellTags->"SUNTrace"]},
 "SymbolicSum2"->{
  Cell[1990640, 81566, 61, 1, 41, "ObjectName",
   CellTags->"SymbolicSum2"],
  Cell[1990726, 81571, 63, 1, 36, "Subsubsection",
   CellTags->"SymbolicSum2"],
  Cell[1990792, 81574, 284, 6, 43, "Text",
   CellTags->"SymbolicSum2"]},
 "SymbolicSum3"->{
  Cell[1991091, 81583, 61, 1, 41, "ObjectName",
   CellTags->"SymbolicSum3"],
  Cell[1991177, 81588, 63, 1, 36, "Subsubsection",
   CellTags->"SymbolicSum3"],
  Cell[1991243, 81591, 292, 6, 43, "Text",
   CellTags->"SymbolicSum3"]},
 "Symmetrize"->{
  Cell[1991550, 81600, 57, 1, 41, "ObjectName",
   CellTags->"Symmetrize"],
  Cell[1991632, 81605, 61, 1, 36, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1991696, 81608, 145, 4, 25, "Text",
   CellTags->"Symmetrize"],
  Cell[1991844, 81614, 193, 8, 25, "Text",
   CellTags->"Symmetrize"],
  Cell[1992074, 81627, 58, 1, 36, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1992157, 81632, 240, 8, 28, "Input",
   CellTags->"Symmetrize"],
  Cell[1992400, 81642, 317, 11, 38, "Output",
   CellTags->"Symmetrize"],
  Cell[1992754, 81658, 260, 8, 28, "Input",
   CellTags->"Symmetrize"],
  Cell[1993017, 81668, 652, 20, 38, "Output",
   CellTags->"Symmetrize"]},
 "TBox"->{
  Cell[1993696, 81692, 61, 1, 41, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1993782, 81697, 65, 1, 36, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1993850, 81700, 262, 5, 43, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tbox"->{
  Cell[1993696, 81692, 61, 1, 41, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1993782, 81697, 65, 1, 36, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1993850, 81700, 262, 5, 43, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tdec"->{
  Cell[1994127, 81708, 45, 1, 41, "ObjectName",
   CellTags->"Tdec"],
  Cell[1994197, 81713, 55, 1, 36, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1994255, 81716, 256, 5, 43, "Text",
   CellTags->"Tdec"],
  Cell[1994536, 81725, 108, 3, 28, "Input",
   CellTags->"Tdec"],
  Cell[1994647, 81730, 412, 11, 27, "Output",
   CellTags->"Tdec"],
  Cell[1995074, 81744, 389, 19, 25, "Text",
   CellTags->"Tdec"],
  Cell[1995500, 81768, 52, 1, 36, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1995555, 81771, 608, 22, 30, "Text",
   CellTags->"Tdec"],
  Cell[1996188, 81797, 202, 7, 28, "Input",
   CellTags->"Tdec"],
  Cell[1996393, 81806, 556, 21, 41, "Output",
   CellTags->"Tdec"],
  Cell[1996986, 81832, 199, 7, 28, "Input",
   CellTags->"Tdec"],
  Cell[1997188, 81841, 354, 14, 41, "Output",
   CellTags->"Tdec"],
  Cell[1997557, 81858, 776, 28, 25, "Text",
   CellTags->"Tdec"],
  Cell[1998336, 81888, 529, 18, 28, "Input",
   CellTags->"Tdec"],
  Cell[1998890, 81910, 199, 7, 28, "Input",
   CellTags->"Tdec"],
  Cell[1999092, 81919, 8001, 289, 137, "Output",
   CellTags->"Tdec"],
  Cell[2007130, 82213, 380, 13, 28, "Input",
   CellTags->"Tdec"],
  Cell[2007513, 82228, 4966, 181, 66, "Output",
   CellTags->"Tdec"],
  Cell[2012494, 82412, 121, 4, 28, "Input",
   CellTags->"Tdec"]},
 "TensorFunction"->{
  Cell[2012630, 82419, 65, 1, 41, "ObjectName",
   CellTags->"TensorFunction"],
  Cell[2012720, 82424, 65, 1, 36, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[2012788, 82427, 360, 7, 61, "Text",
   CellTags->"TensorFunction"],
  Cell[2013151, 82436, 191, 9, 25, "Text",
   CellTags->"TensorFunction"],
  Cell[2013379, 82450, 62, 1, 36, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[2013466, 82455, 178, 4, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2013647, 82461, 212, 6, 27, "Output",
   CellTags->"TensorFunction"],
  Cell[2013896, 82472, 116, 3, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2014015, 82477, 193, 4, 42, "Output",
   CellTags->"TensorFunction"],
  Cell[2014245, 82486, 198, 6, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2014446, 82494, 376, 14, 27, "Output",
   CellTags->"TensorFunction"],
  Cell[2014859, 82513, 116, 3, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2014978, 82518, 358, 10, 42, "Output",
   CellTags->"TensorFunction"],
  Cell[2015373, 82533, 228, 7, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2015604, 82542, 263, 8, 27, "Output",
   CellTags->"TensorFunction"],
  Cell[2015904, 82555, 228, 7, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2016135, 82564, 263, 8, 27, "Output",
   CellTags->"TensorFunction"],
  Cell[2016435, 82577, 116, 3, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2016554, 82582, 243, 7, 42, "Output",
   CellTags->"TensorFunction"],
  Cell[2016834, 82594, 118, 3, 28, "Input",
   CellTags->"TensorFunction"],
  Cell[2016955, 82599, 129, 4, 27, "Output",
   CellTags->"TensorFunction"],
  Cell[2017099, 82606, 154, 4, 28, "Input",
   CellTags->"TensorFunction"]},
 "Tf"->{
  Cell[2017268, 82613, 41, 1, 41, "ObjectName",
   CellTags->"Tf"],
  Cell[2017334, 82618, 53, 1, 36, "Subsubsection",
   CellTags->"Tf"],
  Cell[2017390, 82621, 214, 10, 25, "Text",
   CellTags->"Tf"],
  Cell[2017607, 82633, 414, 18, 25, "Text",
   CellTags->"Tf"]},
 "TFi"->{
  Cell[2018036, 82654, 43, 1, 41, "ObjectName",
   CellTags->"TFi"],
  Cell[2018104, 82659, 54, 1, 36, "Subsubsection",
   CellTags->"TFi"],
  Cell[2018701, 82680, 576, 10, 97, "Text",
   CellTags->"TFi"],
  Cell[2019280, 82692, 237, 8, 25, "Text",
   CellTags->"TFi"],
  Cell[2019520, 82702, 185, 4, 43, "Text",
   CellTags->"TFi"],
  Cell[2019708, 82708, 266, 13, 25, "Text",
   CellTags->"TFi"],
  Cell[2020011, 82726, 51, 1, 36, "Subsubsection",
   CellTags->"TFi"],
  Cell[2020087, 82731, 506, 17, 28, "Input",
   CellTags->"TFi"],
  Cell[2020596, 82750, 776, 24, 91, "Output",
   CellTags->"TFi"],
  Cell[2021409, 82779, 295, 10, 28, "Input",
   CellTags->"TFi"],
  Cell[2021707, 82791, 333, 11, 27, "Output",
   CellTags->"TFi"]},
 "ThreeVector"->{
  Cell[2022067, 82806, 59, 1, 41, "ObjectName",
   CellTags->"ThreeVector"],
  Cell[2022151, 82811, 62, 1, 36, "Subsubsection",
   CellTags->"ThreeVector"],
  Cell[2022216, 82814, 91, 1, 25, "Text",
   CellTags->"ThreeVector"],
  Cell[2022310, 82817, 304, 13, 25, "Text",
   CellTags->"ThreeVector"]},
 "TID"->{
  Cell[2022629, 82833, 43, 1, 41, "ObjectName",
   CellTags->"TID"],
  Cell[2022697, 82838, 54, 1, 36, "Subsubsection",
   CellTags->"TID"],
  Cell[2022754, 82841, 80, 1, 25, "Text",
   CellTags->"TID"],
  Cell[2022859, 82846, 106, 3, 28, "Input",
   CellTags->"TID"],
  Cell[2022968, 82851, 655, 15, 71, "Output",
   CellTags->"TID"],
  Cell[2023638, 82869, 293, 14, 25, "Text",
   CellTags->"TID"],
  Cell[2023968, 82888, 51, 1, 36, "Subsubsection",
   CellTags->"TID"],
  Cell[2024044, 82893, 374, 12, 28, "Input",
   CellTags->"TID"],
  Cell[2024421, 82907, 532, 21, 47, "Output",
   CellTags->"TID"],
  Cell[2024990, 82933, 181, 6, 28, "Input",
   CellTags->"TID"],
  Cell[2025174, 82941, 4307, 136, 58, "Output",
   CellTags->"TID"],
  Cell[2029518, 83082, 271, 9, 28, "Input",
   CellTags->"TID"],
  Cell[2029792, 83093, 389, 16, 47, "Output",
   CellTags->"TID"],
  Cell[2030218, 83114, 123, 4, 28, "Input",
   CellTags->"TID"],
  Cell[2030344, 83120, 625, 26, 46, "Output",
   CellTags->"TID"],
  Cell[2031006, 83151, 195, 6, 28, "Input",
   CellTags->"TID"],
  Cell[2031204, 83159, 625, 26, 46, "Output",
   CellTags->"TID"]},
 "TIDL"->{
  Cell[2031856, 83189, 45, 1, 41, "ObjectName",
   CellTags->"TIDL"],
  Cell[2031926, 83194, 55, 1, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2031984, 83197, 87, 1, 25, "Text",
   CellTags->"TIDL"],
  Cell[2032074, 83200, 154, 8, 25, "Text",
   CellTags->"TIDL"],
  Cell[2032243, 83211, 52, 1, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2032320, 83216, 154, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2032477, 83223, 414, 13, 25, "Text",
   CellTags->"TIDL"],
  Cell[2032916, 83240, 202, 7, 28, "Input",
   CellTags->"TIDL"],
  Cell[2033121, 83249, 465, 20, 43, "Output",
   CellTags->"TIDL"],
  Cell[2033635, 83275, 159, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2033819, 83284, 312, 11, 28, "Input",
   CellTags->"TIDL"],
  Cell[2034134, 83297, 1876, 74, 46, "Output",
   CellTags->"TIDL"],
  Cell[2036059, 83377, 154, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2036238, 83386, 277, 10, 28, "Input",
   CellTags->"TIDL"],
  Cell[2036518, 83398, 2812, 108, 45, "Output",
   CellTags->"TIDL"],
  Cell[2039379, 83512, 159, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2039563, 83521, 387, 14, 28, "Input",
   CellTags->"TIDL"],
  Cell[2039953, 83537, 24393, 841, 306, "Output",
   CellTags->"TIDL"],
  Cell[2064395, 84384, 165, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2064563, 84391, 434, 15, 28, "Input",
   CellTags->"TIDL"],
  Cell[2065034, 84411, 173, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2065210, 84418, 506, 17, 28, "Input",
   CellTags->"TIDL"],
  Cell[2065753, 84440, 154, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2065932, 84449, 312, 11, 28, "Input",
   CellTags->"TIDL"],
  Cell[2066247, 84462, 14684, 522, 131, "Output",
   CellTags->"TIDL"],
  Cell[2080980, 84990, 159, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2081142, 84997, 374, 13, 28, "Input",
   CellTags->"TIDL"],
  Cell[2081553, 85015, 165, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2081721, 85022, 444, 15, 28, "Input",
   CellTags->"TIDL"],
  Cell[2082202, 85042, 173, 5, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2082378, 85049, 529, 18, 28, "Input",
   CellTags->"TIDL"],
  Cell[2082932, 85071, 240, 7, 28, "Input",
   CellTags->"TIDL"],
  Cell[2083175, 85080, 161, 5, 27, "Output",
   CellTags->"TIDL"]},
 "TimedIntegrate"->{
  Cell[2083363, 85089, 65, 1, 41, "ObjectName",
   CellTags->"TimedIntegrate"],
  Cell[2083453, 85094, 65, 1, 36, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2083521, 85097, 225, 5, 43, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2083771, 85106, 128, 3, 28, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2083902, 85111, 424, 13, 27, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2084375, 85130, 62, 1, 36, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2084440, 85133, 73, 1, 25, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2084538, 85138, 305, 9, 28, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2084846, 85149, 129, 4, 27, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2084990, 85156, 59, 1, 25, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2085074, 85161, 389, 11, 28, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2085466, 85174, 449, 14, 27, "Output",
   CellTags->"TimedIntegrate"]},
 "TLI"->{
  Cell[2085942, 85192, 61, 1, 41, "ObjectName",
   CellTags->"TLI"],
  Cell[2086028, 85197, 54, 1, 36, "Subsubsection",
   CellTags->"TLI"],
  Cell[2086085, 85200, 616, 12, 121, "Text",
   CellTags->"TLI"],
  Cell[2086726, 85216, 106, 3, 28, "Input",
   CellTags->"TLI"],
  Cell[2086835, 85221, 229, 8, 27, "Output",
   CellTags->"TLI"],
  Cell[2087079, 85232, 168, 9, 25, "Text",
   CellTags->"TLI"],
  Cell[2087284, 85246, 51, 1, 36, "Subsubsection",
   CellTags->"TLI"],
  Cell[2087338, 85249, 70, 2, 28, "Input",
   CellTags->"TLI"]},
 "TLIFP"->{
  Cell[2087423, 85254, 65, 1, 41, "ObjectName",
   CellTags->"TLIFP"],
  Cell[2087513, 85259, 56, 1, 36, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2087572, 85262, 93, 1, 25, "Text",
   CellTags->"TLIFP"],
  Cell[2087690, 85267, 110, 3, 28, "Input",
   CellTags->"TLIFP"],
  Cell[2087803, 85272, 425, 12, 27, "Output",
   CellTags->"TLIFP"],
  Cell[2088243, 85287, 161, 9, 25, "Text",
   CellTags->"TLIFP"],
  Cell[2088441, 85301, 53, 1, 36, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2088497, 85304, 72, 2, 28, "Input",
   CellTags->"TLIFP"]},
 "TLIHYP"->{
  Cell[2088584, 85309, 67, 1, 41, "ObjectName",
   CellTags->"TLIHYP"],
  Cell[2088676, 85314, 57, 1, 36, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2088736, 85317, 136, 4, 25, "Text",
   CellTags->"TLIHYP"],
  Cell[2088897, 85325, 112, 3, 28, "Input",
   CellTags->"TLIHYP"],
  Cell[2089012, 85330, 241, 8, 27, "Output",
   CellTags->"TLIHYP"],
  Cell[2089268, 85341, 162, 9, 25, "Text",
   CellTags->"TLIHYP"],
  Cell[2089467, 85355, 54, 1, 36, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2089524, 85358, 73, 2, 28, "Input",
   CellTags->"TLIHYP"]},
 "TLI2"->{
  Cell[2089612, 85363, 45, 1, 41, "ObjectName",
   CellTags->"TLI2"],
  Cell[2089682, 85368, 55, 1, 36, "Subsubsection",
   CellTags->"TLI2"],
  Cell[2089740, 85371, 83, 1, 25, "Text",
   CellTags->"TLI2"],
  Cell[2089826, 85374, 160, 9, 25, "Text",
   CellTags->"TLI2"]},
 "TLI2FC"->{
  Cell[2090001, 85386, 67, 1, 41, "ObjectName",
   CellTags->"TLI2FC"],
  Cell[2090093, 85391, 57, 1, 36, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2090153, 85394, 109, 3, 25, "Text",
   CellTags->"TLI2FC"],
  Cell[2090287, 85401, 112, 3, 28, "Input",
   CellTags->"TLI2FC"],
  Cell[2090402, 85406, 355, 11, 27, "Output",
   CellTags->"TLI2FC"],
  Cell[2090772, 85420, 262, 13, 25, "Text",
   CellTags->"TLI2FC"],
  Cell[2091071, 85438, 54, 1, 36, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2091128, 85441, 73, 2, 28, "Input",
   CellTags->"TLI2FC"]},
 "ToDistribution"->{
  Cell[2091216, 85446, 65, 1, 41, "ObjectName",
   CellTags->"ToDistribution"],
  Cell[2091306, 85451, 65, 1, 36, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2091374, 85454, 424, 7, 61, "Text",
   CellTags->"ToDistribution"],
  Cell[2091823, 85465, 128, 3, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2091954, 85470, 189, 6, 27, "Output",
   CellTags->"ToDistribution"],
  Cell[2092158, 85479, 209, 9, 25, "Text",
   CellTags->"ToDistribution"],
  Cell[2092404, 85493, 62, 1, 36, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2092491, 85498, 217, 6, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2092711, 85506, 197, 6, 27, "Output",
   CellTags->"ToDistribution"],
  Cell[2092945, 85517, 277, 9, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2093225, 85528, 1338, 48, 46, "Output",
   CellTags->"ToDistribution"],
  Cell[2094600, 85581, 206, 5, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2094809, 85588, 189, 6, 27, "Output",
   CellTags->"ToDistribution"],
  Cell[2095035, 85599, 277, 9, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2095315, 85610, 1353, 49, 48, "Output",
   CellTags->"ToDistribution"],
  Cell[2096705, 85664, 461, 14, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2097169, 85680, 371, 13, 27, "Output",
   CellTags->"ToDistribution"],
  Cell[2097577, 85698, 118, 3, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2097698, 85703, 330, 9, 27, "Output",
   CellTags->"ToDistribution"],
  Cell[2098065, 85717, 409, 13, 28, "Input",
   CellTags->"ToDistribution"],
  Cell[2098477, 85732, 371, 13, 27, "Output",
   CellTags->"ToDistribution"]},
 "ToHypergeometric"->{
  Cell[2098875, 85749, 69, 1, 41, "ObjectName",
   CellTags->"ToHypergeometric"],
  Cell[2098969, 85754, 67, 1, 36, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2099039, 85757, 1686, 67, 43, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2100728, 85826, 479, 19, 25, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2101244, 85850, 64, 1, 36, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2101333, 85855, 395, 13, 28, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2101731, 85870, 1738, 58, 44, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2103506, 85933, 488, 16, 28, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2103997, 85951, 1555, 52, 44, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2105589, 86008, 395, 13, 28, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2105987, 86023, 1804, 60, 45, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2107828, 86088, 488, 16, 28, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2108319, 86106, 1641, 55, 45, "Output",
   CellTags->"ToHypergeometric"]},
 "ToLarin"->{
  Cell[2109987, 86165, 51, 1, 41, "ObjectName",
   CellTags->"ToLarin"],
  Cell[2110063, 86170, 58, 1, 36, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2110124, 86173, 605, 19, 25, "Text",
   CellTags->"ToLarin"],
  Cell[2110732, 86194, 163, 9, 25, "Text",
   CellTags->"ToLarin"],
  Cell[2110932, 86208, 55, 1, 36, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2111012, 86213, 229, 6, 28, "Input",
   CellTags->"ToLarin"],
  Cell[2111244, 86221, 389, 15, 27, "Output",
   CellTags->"ToLarin"],
  Cell[2111670, 86241, 108, 3, 28, "Input",
   CellTags->"ToLarin"],
  Cell[2111781, 86246, 1085, 45, 38, "Output",
   CellTags->"ToLarin"]},
 "ToTFi"->{
  Cell[2112893, 86295, 47, 1, 41, "ObjectName",
   CellTags->"ToTFi"],
  Cell[2112965, 86300, 56, 1, 36, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2113024, 86303, 274, 6, 43, "Text",
   CellTags->"ToTFi"],
  Cell[2113301, 86311, 262, 13, 25, "Text",
   CellTags->"ToTFi"],
  Cell[2113600, 86329, 53, 1, 36, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2113678, 86334, 333, 11, 28, "Input",
   CellTags->"ToTFi"],
  Cell[2114014, 86347, 1870, 49, 48, "Output",
   CellTags->"ToTFi"],
  Cell[2115921, 86401, 149, 4, 28, "Input",
   CellTags->"ToTFi"],
  Cell[2116073, 86407, 2126, 54, 53, "Output",
   CellTags->"ToTFi"],
  Cell[2118236, 86466, 107, 3, 28, "Input",
   CellTags->"ToTFi"],
  Cell[2118346, 86471, 491, 16, 42, "Output",
   CellTags->"ToTFi"],
  Cell[2118874, 86492, 468, 15, 28, "Input",
   CellTags->"ToTFi"],
  Cell[2119345, 86509, 916, 33, 49, "Output",
   CellTags->"ToTFi"]},
 "TR"->{
  Cell[2120310, 86548, 41, 1, 41, "ObjectName",
   CellTags->"TR"],
  Cell[2120354, 86551, 100, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2120491, 86559, 53, 1, 36, "Subsubsection",
   CellTags->"TR"],
  Cell[2120547, 86562, 420, 12, 61, "Text",
   CellTags->"TR"],
  Cell[2120992, 86578, 104, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2121099, 86583, 761, 18, 71, "Output",
   CellTags->"TR"],
  Cell[2121875, 86604, 536, 23, 25, "Text",
   CellTags->"TR"],
  Cell[2122448, 86632, 50, 1, 36, "Subsubsection",
   CellTags->"TR"],
  Cell[2122523, 86637, 129, 4, 28, "Input",
   CellTags->"TR"],
  Cell[2122655, 86643, 194, 6, 27, "Output",
   CellTags->"TR"],
  Cell[2122886, 86654, 98, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2122987, 86659, 175, 6, 27, "Output",
   CellTags->"TR"],
  Cell[2123199, 86670, 375, 12, 28, "Input",
   CellTags->"TR"],
  Cell[2123577, 86684, 1227, 49, 27, "Output",
   CellTags->"TR"],
  Cell[2124841, 86738, 205, 6, 28, "Input",
   CellTags->"TR"],
  Cell[2125049, 86746, 559, 23, 27, "Output",
   CellTags->"TR"],
  Cell[2125645, 86774, 240, 7, 28, "Input",
   CellTags->"TR"],
  Cell[2125888, 86783, 8674, 322, 49, "Output",
   CellTags->"TR"],
  Cell[2134599, 87110, 74, 2, 28, "Input",
   CellTags->"TR"],
  Cell[2134676, 87114, 102, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2134815, 87122, 91, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2134909, 87127, 377, 6, 78, "Print",
   CellTags->{"Info3387712206-4027155", "TR"}],
  Cell[2135301, 87136, 166, 4, 25, "Text",
   CellTags->"TR"],
  Cell[2135492, 87144, 100, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2135595, 87149, 103, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2135735, 87157, 421, 13, 28, "Input",
   CellTags->"TR"],
  Cell[2136159, 87172, 99, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2136295, 87180, 171, 5, 28, "Input",
   CellTags->"TR"],
  Cell[2136469, 87187, 991, 39, 27, "Output",
   CellTags->"TR"],
  Cell[2137497, 87231, 444, 14, 28, "Input",
   CellTags->"TR"],
  Cell[2137944, 87247, 342, 13, 27, "Output",
   CellTags->"TR"],
  Cell[2138323, 87265, 224, 7, 28, "Input",
   CellTags->"TR"],
  Cell[2138550, 87274, 180, 6, 28, "Output",
   CellTags->"TR"],
  Cell[2138767, 87285, 104, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2138874, 87290, 173, 5, 42, "Output",
   CellTags->"TR"],
  Cell[2139084, 87300, 347, 10, 28, "Input",
   CellTags->"TR"],
  Cell[2139434, 87312, 190, 6, 28, "Output",
   CellTags->"TR"],
  Cell[2139661, 87323, 104, 3, 28, "Input",
   CellTags->"TR"],
  Cell[2139768, 87328, 183, 5, 42, "Output",
   CellTags->"TR"],
  Cell[2139988, 87338, 339, 11, 28, "Input",
   CellTags->"TR"],
  Cell[2140330, 87351, 428, 16, 27, "Output",
   CellTags->"TR"],
  Cell[2140795, 87372, 214, 6, 28, "Input",
   CellTags->"TR"],
  Cell[2141012, 87380, 208, 7, 27, "Output",
   CellTags->"TR"],
  Cell[2141257, 87392, 213, 6, 28, "Input",
   CellTags->"TR"],
  Cell[2141473, 87400, 208, 7, 27, "Output",
   CellTags->"TR"],
  Cell[2141718, 87412, 214, 6, 28, "Input",
   CellTags->"TR"],
  Cell[2141935, 87420, 99, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2142071, 87428, 213, 6, 28, "Input",
   CellTags->"TR"],
  Cell[2142287, 87436, 99, 3, 27, "Output",
   CellTags->"TR"],
  Cell[2142730, 87458, 119, 4, 28, "Input",
   CellTags->"TR"]},
 "Info3387712206-4027155"->{
  Cell[2134909, 87127, 377, 6, 78, "Print",
   CellTags->{"Info3387712206-4027155", "TR"}]},
 "TraceDimension"->{
  Cell[2142864, 87465, 65, 1, 41, "ObjectName",
   CellTags->"TraceDimension"],
  Cell[2142954, 87470, 65, 1, 36, "Subsubsection",
   CellTags->"TraceDimension"],
  Cell[2143022, 87473, 142, 4, 25, "Text",
   CellTags->"TraceDimension"],
  Cell[2143167, 87479, 200, 9, 25, "Text",
   CellTags->"TraceDimension"]},
 "TraceOfOne"->{
  Cell[2143382, 87491, 57, 1, 41, "ObjectName",
   CellTags->"TraceOfOne"],
  Cell[2143464, 87496, 61, 1, 36, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2143528, 87499, 149, 4, 25, "Text",
   CellTags->"TraceOfOne"],
  Cell[2143680, 87505, 273, 13, 25, "Text",
   CellTags->"TraceOfOne"],
  Cell[2143990, 87523, 58, 1, 36, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2144073, 87528, 273, 7, 28, "Input",
   CellTags->"TraceOfOne"],
  Cell[2144349, 87537, 109, 3, 27, "Output",
   CellTags->"TraceOfOne"],
  Cell[2144495, 87545, 272, 7, 28, "Input",
   CellTags->"TraceOfOne"],
  Cell[2144770, 87554, 109, 3, 27, "Output",
   CellTags->"TraceOfOne"]},
 "Trick"->{
  Cell[2144906, 87561, 47, 1, 41, "ObjectName",
   CellTags->"Trick"],
  Cell[2144978, 87566, 56, 1, 36, "Subsubsection",
   CellTags->"Trick"],
  Cell[2145037, 87569, 174, 4, 43, "Text",
   CellTags->"Trick"],
  Cell[2145214, 87575, 625, 29, 25, "Text",
   CellTags->"Trick"],
  Cell[2145876, 87609, 62, 1, 35, "Subsubsection",
   CellTags->"Trick"],
  Cell[2145941, 87612, 376, 15, 25, "Text",
   CellTags->"Trick"],
  Cell[2146342, 87631, 350, 11, 28, "Input",
   CellTags->"Trick"],
  Cell[2146695, 87644, 264, 10, 27, "Output",
   CellTags->"Trick"],
  Cell[2146996, 87659, 330, 11, 28, "Input",
   CellTags->"Trick"],
  Cell[2147329, 87672, 426, 16, 27, "Output",
   CellTags->"Trick"],
  Cell[2147792, 87693, 104, 3, 28, "Input",
   CellTags->"Trick"],
  Cell[2147899, 87698, 549, 23, 27, "Output",
   CellTags->"Trick"],
  Cell[2148485, 87726, 227, 6, 28, "Input",
   CellTags->"Trick"],
  Cell[2148715, 87734, 391, 15, 28, "Output",
   CellTags->"Trick"],
  Cell[2149143, 87754, 133, 4, 28, "Input",
   CellTags->"Trick"],
  Cell[2149279, 87760, 213, 6, 42, "Output",
   CellTags->"Trick"]},
 "TrickIntegrate"->{
  Cell[2149519, 87770, 65, 1, 41, "ObjectName",
   CellTags->"TrickIntegrate"],
  Cell[2149609, 87775, 65, 1, 36, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2149677, 87778, 483, 8, 79, "Text",
   CellTags->"TrickIntegrate"],
  Cell[2150197, 87791, 62, 1, 36, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2150284, 87796, 364, 12, 28, "Input",
   CellTags->"TrickIntegrate"],
  Cell[2150651, 87810, 517, 18, 28, "Output",
   CellTags->"TrickIntegrate"]},
 "TrickMandelstam"->{
  Cell[2151195, 87832, 67, 1, 70, "ObjectName",
   CellTags->"TrickMandelstam"],
  Cell[2151287, 87837, 66, 1, 70, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2151356, 87840, 1340, 55, 70, "Text",
   CellTags->"TrickMandelstam"],
  Cell[2152733, 87900, 63, 1, 36, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2152821, 87905, 528, 18, 28, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2153352, 87925, 250, 9, 27, "Output",
   CellTags->"TrickMandelstam"],
  Cell[2153639, 87939, 568, 17, 28, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2154210, 87958, 235, 8, 27, "Output",
   CellTags->"TrickMandelstam"]},
 "Tr2"->{
  Cell[2154472, 87970, 43, 1, 70, "ObjectName",
   CellTags->"Tr2"],
  Cell[2154540, 87975, 54, 1, 70, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2154597, 87978, 347, 6, 61, "Text",
   CellTags->"Tr2"],
  Cell[2154969, 87988, 106, 3, 28, "Input",
   CellTags->"Tr2"],
  Cell[2155078, 87993, 168, 6, 27, "Output",
   CellTags->"Tr2"],
  Cell[2155261, 88002, 593, 28, 70, "Text",
   CellTags->"Tr2"],
  Cell[2155891, 88035, 51, 1, 36, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2156719, 88071, 187, 6, 28, "Input",
   CellTags->"Tr2"],
  Cell[2156909, 88079, 156, 5, 27, "Output",
   CellTags->"Tr2"]},
 "Twist2AlienOperator"->{
  Cell[2157092, 88088, 116, 2, 70, "ObjectName",
   CellTags->"Twist2AlienOperator"],
  Cell[2157233, 88094, 70, 1, 70, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2157306, 88097, 194, 4, 70, "Text",
   CellTags->"Twist2AlienOperator"],
  Cell[2157537, 88106, 67, 1, 36, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2157629, 88111, 160, 4, 28, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2157792, 88117, 887, 30, 45, "Output",
   CellTags->"Twist2AlienOperator"],
  Cell[2158716, 88152, 246, 7, 28, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2158965, 88161, 3113, 103, 48, "Output",
   CellTags->"Twist2AlienOperator"]},
 "Twist2CounterOperator"->{
  Cell[2162105, 88268, 120, 2, 70, "ObjectName",
   CellTags->"Twist2CounterOperator"],
  Cell[2162250, 88274, 72, 1, 70, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2162325, 88277, 239, 5, 70, "Text",
   CellTags->"Twist2CounterOperator"],
  Cell[2162601, 88287, 69, 1, 36, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2162695, 88292, 204, 5, 28, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2162902, 88299, 3999, 135, 77, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2166938, 88439, 159, 4, 28, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2167100, 88445, 1020, 35, 45, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2168157, 88485, 242, 6, 28, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2168402, 88493, 2610, 89, 48, "Output",
   CellTags->"Twist2CounterOperator"]},
 "Twist2GluonOperator"->{
  Cell[2171039, 88586, 75, 1, 70, "ObjectName",
   CellTags->"Twist2GluonOperator"],
  Cell[2171139, 88591, 70, 1, 70, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2171212, 88594, 964, 15, 70, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2172179, 88611, 223, 9, 70, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2172439, 88625, 99, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2172563, 88630, 352, 8, 50, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2172918, 88640, 444, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2173377, 88655, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2173464, 88660, 271, 8, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2173738, 88670, 894, 35, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2174669, 88710, 321, 9, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2174993, 88721, 1120, 45, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2176150, 88771, 221, 6, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2176374, 88779, 336, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2176747, 88796, 271, 7, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2177021, 88805, 705, 28, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2177741, 88836, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2177828, 88841, 355, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2178186, 88853, 1193, 48, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2179416, 88906, 325, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2179744, 88918, 492, 18, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2180273, 88941, 375, 11, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2180651, 88954, 11158, 407, 127, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2191824, 89364, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2191911, 89369, 431, 12, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2192345, 89383, 4934, 185, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2197316, 89573, 391, 12, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2197710, 89587, 606, 22, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2198353, 89614, 441, 13, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2198797, 89629, 24093, 861, 369, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2222939, 90496, 100, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2223064, 90501, 268, 6, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2223335, 90509, 444, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2223794, 90524, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2223881, 90529, 271, 8, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2224155, 90539, 904, 36, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2225096, 90580, 321, 9, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2225420, 90591, 2653, 101, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2228110, 90697, 221, 6, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2228334, 90705, 336, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2228707, 90722, 271, 7, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2228981, 90731, 2228, 83, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2231224, 90817, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2231311, 90822, 355, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2231669, 90834, 1267, 50, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2232973, 90889, 325, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2233301, 90901, 492, 18, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2233830, 90924, 375, 11, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2234208, 90937, 16156, 580, 200, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2250379, 91520, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2250466, 91525, 431, 12, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2250900, 91539, 4944, 186, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2255881, 91730, 391, 12, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2256275, 91744, 606, 22, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2256918, 91771, 441, 13, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2257362, 91786, 31649, 1133, 445, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2289060, 92925, 76, 1, 52, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2289139, 92928, 97, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2289239, 92931, 295, 7, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2289537, 92940, 380, 11, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2289954, 92956, 102, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2290081, 92961, 284, 6, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2290368, 92969, 443, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2290826, 92984, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2290913, 92989, 320, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2291236, 93001, 3606, 128, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2294857, 93132, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2294944, 93137, 355, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2295302, 93149, 53443, 1666, 714, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2348760, 94818, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2348847, 94823, 482, 13, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2349332, 94838, 4934, 185, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2354315, 95029, 104, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2354444, 95034, 284, 6, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2354731, 95042, 443, 12, 27, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2355189, 95057, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2355276, 95062, 320, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2355599, 95074, 3037, 114, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2358651, 95191, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2358716, 95194, 380, 11, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2359121, 95209, 325, 10, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2359449, 95221, 3038, 114, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2362524, 95340, 375, 11, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2362902, 95353, 3038, 114, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2365955, 95470, 62, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2366042, 95475, 482, 13, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2366527, 95490, 4944, 186, 38, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2371486, 95679, 78, 1, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2371589, 95684, 520, 16, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2372112, 95702, 10853, 374, 149, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2382980, 96079, 109, 3, 25, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2383092, 96084, 295, 7, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2383412, 96095, 428, 13, 28, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2383843, 96110, 11798, 376, 168, "Output",
   CellTags->"Twist2GluonOperator"]},
 "Twist2QuarkOperator"->{
  Cell[2395668, 96490, 75, 1, 70, "ObjectName",
   CellTags->"Twist2QuarkOperator"],
  Cell[2395768, 96495, 70, 1, 70, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2395841, 96498, 980, 15, 70, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2396824, 96515, 223, 9, 70, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2397062, 96527, 67, 1, 70, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2397154, 96532, 98, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2397277, 96537, 268, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2397548, 96545, 443, 12, 27, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2398006, 96560, 75, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2398106, 96565, 156, 4, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2398265, 96571, 600, 21, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2398880, 96595, 81, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2398986, 96600, 305, 9, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2399294, 96611, 1530, 55, 51, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400839, 96669, 87, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400951, 96674, 380, 11, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2401334, 96687, 4828, 159, 107, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406211, 96852, 100, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406336, 96857, 273, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406612, 96865, 443, 12, 27, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407070, 96880, 75, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407170, 96885, 156, 4, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407329, 96891, 481, 17, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407825, 96911, 81, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407931, 96916, 305, 9, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408239, 96927, 1467, 53, 51, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2409721, 96983, 87, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2409833, 96988, 380, 11, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410216, 97001, 4713, 156, 75, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2414944, 97160, 93, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2415062, 97165, 197, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2415262, 97173, 263, 8, 43, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2415562, 97186, 376, 10, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2415941, 97198, 1945, 71, 46, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2417923, 97274, 449, 12, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2418375, 97288, 9376, 296, 91, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2427800, 97590, 102, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2427927, 97595, 269, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2428199, 97603, 444, 12, 27, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2428658, 97618, 178, 4, 70, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2428839, 97624, 275, 5, 43, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2429139, 97633, 205, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2429347, 97641, 800, 28, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2430162, 97672, 100, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2430287, 97677, 305, 9, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2430595, 97688, 2280, 77, 51, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2432890, 97768, 87, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2433002, 97773, 156, 4, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2433161, 97779, 1102, 33, 64, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2434312, 97818, 104, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2434441, 97823, 269, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2434713, 97831, 444, 12, 27, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2435172, 97846, 75, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2435272, 97851, 205, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2435480, 97859, 800, 28, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2436295, 97890, 81, 1, 25, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2436401, 97895, 305, 9, 28, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2436709, 97906, 2280, 77, 51, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2439004, 97986, 210, 6, 28, "Input",
   CellTags->"Twist2QuarkOperator"]},
 "Twist3QuarkOperator"->{
  Cell[2439229, 97995, 75, 1, 57, "ObjectName",
   CellTags->"Twist3QuarkOperator"],
  Cell[2439329, 98000, 70, 1, 52, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2439402, 98003, 768, 12, 115, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2440195, 98019, 134, 3, 28, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2440332, 98024, 328, 10, 27, "Output",
   CellTags->"Twist3QuarkOperator"],
  Cell[2440675, 98037, 366, 14, 41, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2441078, 98056, 67, 1, 36, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2441170, 98061, 132, 3, 28, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2441305, 98066, 654, 23, 28, "Output",
   CellTags->"Twist3QuarkOperator"]},
 "Twist4GluonOperator"->{
  Cell[2441986, 98093, 75, 1, 57, "ObjectName",
   CellTags->"Twist4GluonOperator"],
  Cell[2442086, 98098, 70, 1, 52, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2442159, 98101, 150, 4, 41, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2442312, 98107, 509, 19, 41, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2442858, 98131, 67, 1, 36, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2442950, 98136, 529, 15, 28, "Input",
   CellTags->"Twist4GluonOperator"],
  Cell[2443482, 98153, 119831, 4588, 1110, "Output",
   CellTags->"Twist4GluonOperator"]},
 "TwoLoopSimplify"->{
  Cell[2563340, 102745, 104, 1, 57, "ObjectName",
   CellTags->"TwoLoopSimplify"],
  Cell[2563469, 102750, 66, 1, 52, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2563538, 102753, 335, 6, 59, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2563876, 102761, 207, 9, 41, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2564120, 102775, 63, 1, 36, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2564186, 102778, 82, 2, 28, "Input",
   CellTags->"TwoLoopSimplify"]},
 "Uncontract"->{
  Cell[2564283, 102783, 57, 1, 57, "ObjectName",
   CellTags->"Uncontract"],
  Cell[2564365, 102788, 61, 1, 52, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2564429, 102791, 256, 5, 43, "Text",
   CellTags->"Uncontract"],
  Cell[2564710, 102800, 120, 3, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2564833, 102805, 372, 11, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2565220, 102819, 181, 9, 41, "Text",
   CellTags->"Uncontract"],
  Cell[2565438, 102833, 58, 1, 36, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2565521, 102838, 234, 7, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2565758, 102847, 397, 17, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2566192, 102869, 143, 4, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2566338, 102875, 781, 31, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2567156, 102911, 138, 4, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2567297, 102917, 224, 8, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2567558, 102930, 143, 4, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2567704, 102936, 541, 20, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2568282, 102961, 153, 4, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2568438, 102967, 1138, 44, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2569591, 103014, 89, 1, 25, "Text",
   CellTags->"Uncontract"],
  Cell[2569705, 103019, 208, 6, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2569916, 103027, 209, 8, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2570140, 103038, 152, 4, 25, "Text",
   CellTags->"Uncontract"],
  Cell[2570317, 103046, 248, 7, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2570568, 103055, 570, 22, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2571175, 103082, 274, 8, 28, "Input",
   CellTags->"Uncontract"],
  Cell[2571452, 103092, 1030, 40, 27, "Output",
   CellTags->"Uncontract"],
  Cell[2572497, 103135, 134, 4, 28, "Input",
   CellTags->"Uncontract"]},
 "UnDeclareNonCommutative"->{
  Cell[2572646, 103142, 83, 1, 57, "ObjectName",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2572754, 103147, 74, 1, 52, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2572831, 103150, 198, 4, 41, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573032, 103156, 343, 14, 41, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573412, 103175, 71, 1, 36, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573486, 103178, 138, 3, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573627, 103183, 153, 4, 25, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573805, 103191, 107, 2, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2573915, 103195, 123, 3, 27, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574053, 103201, 103, 1, 25, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574159, 103204, 140, 3, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574324, 103211, 161, 4, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574488, 103217, 123, 3, 27, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574626, 103223, 161, 4, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574812, 103231, 171, 4, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2574986, 103237, 173, 5, 27, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2575174, 103245, 163, 4, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2575362, 103253, 171, 4, 28, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2575536, 103259, 173, 5, 27, "Output",
   CellTags->"UnDeclareNonCommutative"]},
 "Upper"->{
  Cell[2575736, 103268, 47, 1, 57, "ObjectName",
   CellTags->"Upper"],
  Cell[2575808, 103273, 56, 1, 52, "Subsubsection",
   CellTags->"Upper"],
  Cell[2575867, 103276, 126, 4, 41, "Text",
   CellTags->"Upper"],
  Cell[2575996, 103282, 397, 18, 41, "Text",
   CellTags->"Upper"]},
 "UVPart"->{
  Cell[2576408, 103303, 86, 1, 57, "ObjectName",
   CellTags->"UVPart"],
  Cell[2576519, 103308, 57, 1, 52, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2576579, 103311, 128, 4, 41, "Text",
   CellTags->"UVPart"],
  Cell[2576744, 103320, 54, 1, 36, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2576801, 103323, 73, 2, 28, "Input",
   CellTags->"UVPart"]},
 "Vectors"->{
  Cell[2576889, 103328, 51, 1, 57, "ObjectName",
   CellTags->"Vectors"],
  Cell[2576965, 103333, 58, 1, 52, "Subsubsection",
   CellTags->"Vectors"],
  Cell[2577026, 103336, 205, 5, 41, "Text",
   CellTags->"Vectors"],
  Cell[2577234, 103343, 188, 8, 41, "Text",
   CellTags->"Vectors"]},
 "WriteOut"->{
  Cell[2577437, 103354, 53, 1, 57, "ObjectName",
   CellTags->"WriteOut"],
  Cell[2577515, 103359, 59, 1, 52, "Subsubsection",
   CellTags->"WriteOut"],
  Cell[2577577, 103362, 216, 5, 41, "Text",
   CellTags->"WriteOut"],
  Cell[2577796, 103369, 171, 8, 41, "Text",
   CellTags->"WriteOut"]},
 "WriteOutPaVe"->{
  Cell[2577982, 103380, 61, 1, 57, "ObjectName",
   CellTags->"WriteOutPaVe"],
  Cell[2578068, 103385, 63, 1, 52, "Subsubsection",
   CellTags->"WriteOutPaVe"],
  Cell[2578134, 103388, 260, 5, 59, "Text",
   CellTags->"WriteOutPaVe"],
  Cell[2578397, 103395, 398, 18, 70, "Text",
   CellTags->"WriteOutPaVe"]},
 "Write2"->{
  Cell[2578810, 103416, 49, 1, 57, "ObjectName",
   CellTags->"Write2"],
  Cell[2578884, 103421, 57, 1, 52, "Subsubsection",
   CellTags->"Write2"],
  Cell[2578944, 103424, 297, 6, 70, "Text",
   CellTags->"Write2"],
  Cell[2579244, 103432, 153, 4, 25, "Text",
   CellTags->"Write2"],
  Cell[2579422, 103440, 136, 3, 28, "Input",
   CellTags->"Write2"],
  Cell[2579561, 103445, 248, 9, 43, "Output",
   CellTags->"Write2"],
  Cell[2579846, 103459, 112, 3, 28, "Input",
   CellTags->"Write2"],
  Cell[2579961, 103464, 637, 15, 49, "Output",
   CellTags->"Write2"],
  Cell[2580635, 103484, 115, 3, 28, "Input",
   CellTags->"Write2"],
  Cell[2580753, 103489, 133, 4, 27, "Output",
   CellTags->"Write2"],
  Cell[2580901, 103496, 290, 14, 41, "Text",
   CellTags->"Write2"],
  Cell[2581228, 103515, 54, 1, 36, "Subsubsection",
   CellTags->"Write2"],
  Cell[2581307, 103520, 429, 15, 34, "Input",
   CellTags->"Write2"],
  Cell[2581739, 103537, 855, 29, 28, "Output",
   CellTags->"Write2"],
  Cell[2582609, 103569, 208, 7, 28, "Input",
   CellTags->"Write2"],
  Cell[2582820, 103578, 78, 1, 25, "Text",
   CellTags->"Write2"],
  Cell[2582901, 103581, 184, 6, 28, "Input",
   CellTags->"Write2"],
  Cell[2583088, 103589, 73, 1, 25, "Text",
   CellTags->"Write2"],
  Cell[2583186, 103594, 363, 11, 28, "Input",
   CellTags->"Write2"],
  Cell[2583552, 103607, 801, 22, 75, "Output",
   CellTags->"Write2"],
  Cell[2584368, 103632, 286, 8, 28, "Input",
   CellTags->"Write2"],
  Cell[2584679, 103644, 233, 7, 28, "Input",
   CellTags->"Write2"],
  Cell[2584915, 103653, 700, 27, 28, "Output",
   CellTags->"Write2"],
  Cell[2585630, 103683, 185, 6, 28, "Input",
   CellTags->"Write2"],
  Cell[2585840, 103693, 363, 11, 28, "Input",
   CellTags->"Write2"],
  Cell[2586206, 103706, 1025, 29, 183, "Output",
   CellTags->"Write2"],
  Cell[2587246, 103738, 286, 8, 28, "Input",
   CellTags->"Write2"],
  Cell[2587535, 103748, 106, 3, 25, "Text",
   CellTags->"Write2"],
  Cell[2587644, 103753, 247, 8, 28, "Input",
   CellTags->"Write2"],
  Cell[2587916, 103765, 363, 11, 28, "Input",
   CellTags->"Write2"],
  Cell[2588282, 103778, 942, 25, 129, "Output",
   CellTags->"Write2"],
  Cell[2589239, 103806, 397, 11, 50, "Input",
   CellTags->"Write2"]},
 "XYT"->{
  Cell[2589651, 103820, 43, 1, 57, "ObjectName",
   CellTags->"XYT"],
  Cell[2589719, 103825, 54, 1, 52, "Subsubsection",
   CellTags->"XYT"],
  Cell[2589776, 103828, 215, 9, 41, "Text",
   CellTags->"XYT"]},
 "ZeroMomentumInsertion"->{
  Cell[2590006, 103840, 79, 1, 57, "ObjectName",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2590110, 103845, 72, 1, 52, "Subsubsection",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2590185, 103848, 154, 4, 41, "Text",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2590342, 103854, 473, 18, 41, "Text",
   CellTags->"ZeroMomentumInsertion"]},
 "Zeta2"->{
  Cell[2590830, 103875, 47, 1, 57, "ObjectName",
   CellTags->"Zeta2"],
  Cell[2590902, 103880, 56, 1, 52, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2590961, 103883, 94, 1, 41, "Text",
   CellTags->"Zeta2"],
  Cell[2591058, 103886, 191, 8, 41, "Text",
   CellTags->"Zeta2"],
  Cell[2591286, 103899, 53, 1, 36, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2591364, 103904, 77, 2, 28, "Input",
   CellTags->"Zeta2"],
  Cell[2591444, 103908, 136, 4, 27, "Output",
   CellTags->"Zeta2"],
  Cell[2591617, 103917, 104, 3, 28, "Input",
   CellTags->"Zeta2"],
  Cell[2591724, 103922, 120, 3, 27, "Output",
   CellTags->"Zeta2"],
  Cell[2591881, 103930, 138, 4, 28, "Input",
   CellTags->"Zeta2"],
  Cell[2592022, 103936, 160, 5, 27, "Output",
   CellTags->"Zeta2"]},
 "$Abbreviations"->{
  Cell[2592209, 103945, 65, 1, 41, "ObjectName",
   CellTags->"$Abbreviations"],
  Cell[2592299, 103950, 65, 1, 36, "Subsubsection",
   CellTags->"$Abbreviations"],
  Cell[2592367, 103953, 293, 6, 43, "Text",
   CellTags->"$Abbreviations"],
  Cell[2592685, 103963, 129, 3, 28, "Input",
   CellTags->"$Abbreviations"],
  Cell[2592817, 103968, 867, 17, 82, "Output",
   CellTags->"$Abbreviations"],
  Cell[2593699, 103988, 652, 29, 25, "Text",
   CellTags->"$Abbreviations"]},
 "$AL"->{
  Cell[2594366, 104020, 43, 1, 41, "ObjectName",
   CellTags->"$AL"],
  Cell[2594434, 104025, 54, 1, 36, "Subsubsection",
   CellTags->"$AL"],
  Cell[2594491, 104028, 114, 3, 25, "Text",
   CellTags->"$AL"],
  Cell[2594630, 104035, 73, 2, 28, "Input",
   CellTags->"$AL"],
  Cell[2594706, 104039, 102, 3, 27, "Output",
   CellTags->"$AL"],
  Cell[2594845, 104047, 241, 7, 28, "Input",
   CellTags->"$AL"],
  Cell[2595089, 104056, 563, 22, 27, "Output",
   CellTags->"$AL"],
  Cell[2595667, 104081, 117, 4, 28, "Input",
   CellTags->"$AL"],
  Cell[2595809, 104089, 241, 7, 28, "Input",
   CellTags->"$AL"],
  Cell[2596053, 104098, 477, 20, 27, "Output",
   CellTags->"$AL"],
  Cell[2596545, 104121, 109, 4, 28, "Input",
   CellTags->"$AL"]},
 "$BreitMaison"->{
  Cell[2596669, 104128, 61, 1, 41, "ObjectName",
   CellTags->"$BreitMaison"],
  Cell[2596755, 104133, 63, 1, 36, "Subsubsection",
   CellTags->"$BreitMaison"],
  Cell[2596821, 104136, 319, 9, 43, "Text",
   CellTags->"$BreitMaison"],
  Cell[2597143, 104147, 370, 19, 25, "Text",
   CellTags->"$BreitMaison"]},
 "$Color"->{
  Cell[2597528, 104169, 49, 1, 41, "ObjectName",
   CellTags->"$Color"],
  Cell[2597602, 104174, 57, 1, 36, "Subsubsection",
   CellTags->"$Color"],
  Cell[2597662, 104177, 130, 4, 25, "Text",
   CellTags->"$Color"]},
 "$Covariant"->{
  Cell[2597807, 104184, 57, 1, 41, "ObjectName",
   CellTags->"$Covariant"],
  Cell[2597889, 104189, 61, 1, 36, "Subsubsection",
   CellTags->"$Covariant"],
  Cell[2597953, 104192, 183, 4, 43, "Text",
   CellTags->"$Covariant"],
  Cell[2598161, 104200, 87, 2, 28, "Input",
   CellTags->"$Covariant"],
  Cell[2598251, 104204, 111, 3, 27, "Output",
   CellTags->"$Covariant"],
  Cell[2598399, 104212, 110, 3, 28, "Input",
   CellTags->"$Covariant"],
  Cell[2598512, 104217, 142, 4, 27, "Output",
   CellTags->"$Covariant"],
  Cell[2598669, 104224, 130, 4, 28, "Input",
   CellTags->"$Covariant"],
  Cell[2598824, 104232, 110, 3, 28, "Input",
   CellTags->"$Covariant"],
  Cell[2598937, 104237, 140, 4, 27, "Output",
   CellTags->"$Covariant"],
  Cell[2599092, 104244, 131, 4, 28, "Input",
   CellTags->"$Covariant"]},
 "$FCS"->{
  Cell[2599238, 104251, 45, 1, 41, "ObjectName",
   CellTags->"$FCS"],
  Cell[2599308, 104256, 55, 1, 36, "Subsubsection",
   CellTags->"$FCS"],
  Cell[2599366, 104259, 142, 4, 25, "Text",
   CellTags->"$FCS"],
  Cell[2599533, 104267, 75, 2, 28, "Input",
   CellTags->"$FCS"],
  Cell[2599611, 104271, 700, 14, 49, "Output",
   CellTags->"$FCS"]},
 "$FCT"->{
  Cell[2600338, 104289, 45, 1, 41, "ObjectName",
   CellTags->"$FCT"],
  Cell[2600408, 104294, 55, 1, 36, "Subsubsection",
   CellTags->"$FCT"],
  Cell[2600466, 104297, 177, 4, 43, "Text",
   CellTags->"$FCT"],
  Cell[2600646, 104303, 128, 4, 28, "Input",
   CellTags->"$FCT"],
  Cell[2600799, 104311, 170, 5, 28, "Input",
   CellTags->"$FCT"],
  Cell[2600972, 104318, 196, 6, 27, "Output",
   CellTags->"$FCT"],
  Cell[2601183, 104327, 129, 4, 28, "Input",
   CellTags->"$FCT"],
  Cell[2601337, 104335, 170, 5, 28, "Input",
   CellTags->"$FCT"],
  Cell[2601510, 104342, 196, 6, 27, "Output",
   CellTags->"$FCT"]},
 "$FeynCalcStuff"->{
  Cell[2601733, 104352, 65, 1, 41, "ObjectName",
   CellTags->"$FeynCalcStuff"],
  Cell[2601823, 104357, 65, 1, 36, "Subsubsection",
   CellTags->"$FeynCalcStuff"],
  Cell[2601891, 104360, 102, 1, 25, "Text",
   CellTags->"$FeynCalcStuff"]},
 "$FeynCalcVersion"->{
  Cell[2602008, 104364, 69, 1, 41, "ObjectName",
   CellTags->"$FeynCalcVersion"],
  Cell[2602102, 104369, 67, 1, 36, "Subsubsection",
   CellTags->"$FeynCalcVersion"],
  Cell[2602172, 104372, 124, 3, 25, "Text",
   CellTags->"$FeynCalcVersion"],
  Cell[2602321, 104379, 99, 2, 28, "Input",
   CellTags->"$FeynCalcVersion"],
  Cell[2602423, 104383, 125, 3, 27, "Output",
   CellTags->"$FeynCalcVersion"]},
 "$FortranContinuationCharacter"->{
  Cell[2602575, 104390, 95, 1, 41, "ObjectName",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2602695, 104395, 80, 1, 36, "Subsubsection",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2602778, 104398, 143, 3, 25, "Text",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2602946, 104405, 125, 2, 28, "Input",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2603074, 104409, 134, 3, 27, "Output",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2603223, 104415, 194, 9, 25, "Text",
   CellTags->"$FortranContinuationCharacter"]},
 "$Gauge"->{
  Cell[2603432, 104427, 49, 1, 41, "ObjectName",
   CellTags->"$Gauge"],
  Cell[2603506, 104432, 57, 1, 36, "Subsubsection",
   CellTags->"$Gauge"],
  Cell[2603566, 104435, 260, 5, 43, "Text",
   CellTags->"$Gauge"],
  Cell[2603829, 104442, 168, 9, 25, "Text",
   CellTags->"$Gauge"]},
 "$Larin"->{
  Cell[2604012, 104454, 49, 1, 41, "ObjectName",
   CellTags->"$Larin"],
  Cell[2604086, 104459, 57, 1, 36, "Subsubsection",
   CellTags->"$Larin"],
  Cell[2604146, 104462, 1133, 30, 115, "Text",
   CellTags->"$Larin"],
  Cell[2605304, 104496, 79, 2, 28, "Input",
   CellTags->"$Larin"],
  Cell[2605386, 104500, 107, 3, 27, "Output",
   CellTags->"$Larin"],
  Cell[2605508, 104506, 377, 18, 25, "Text",
   CellTags->"$Larin"]},
 "$LimitTo4"->{
  Cell[2605900, 104527, 55, 1, 57, "ObjectName",
   CellTags->"$LimitTo4"],
  Cell[2605980, 104532, 60, 1, 52, "Subsubsection",
   CellTags->"$LimitTo4"],
  Cell[2606043, 104535, 203, 4, 43, "Text",
   CellTags->"$LimitTo4"],
  Cell[2606271, 104543, 85, 2, 28, "Input",
   CellTags->"$LimitTo4"],
  Cell[2606359, 104547, 109, 3, 27, "Output",
   CellTags->"$LimitTo4"],
  Cell[2606483, 104553, 391, 19, 41, "Text",
   CellTags->"$LimitTo4"]},
 "$LorentzIndices"->{
  Cell[2606889, 104575, 67, 1, 57, "ObjectName",
   CellTags->"$LorentzIndices"],
  Cell[2606981, 104580, 66, 1, 52, "Subsubsection",
   CellTags->"$LorentzIndices"],
  Cell[2607050, 104583, 164, 4, 25, "Text",
   CellTags->"$LorentzIndices"],
  Cell[2607217, 104589, 140, 4, 28, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2607382, 104597, 333, 9, 28, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2607718, 104608, 669, 28, 28, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2608424, 104641, 117, 3, 28, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2608544, 104646, 457, 13, 42, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2609038, 104664, 302, 9, 28, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2609343, 104675, 884, 38, 28, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2610264, 104718, 117, 3, 28, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2610384, 104723, 655, 19, 62, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2611054, 104745, 141, 4, 28, "Input",
   CellTags->"$LorentzIndices"]},
 "$MIntegrate"->{
  Cell[2611210, 104752, 59, 1, 57, "ObjectName",
   CellTags->"$MIntegrate"],
  Cell[2611294, 104757, 62, 1, 52, "Subsubsection",
   CellTags->"$MIntegrate"],
  Cell[2611359, 104760, 136, 4, 25, "Text",
   CellTags->"$MIntegrate"],
  Cell[2611520, 104768, 89, 2, 28, "Input",
   CellTags->"$MIntegrate"],
  Cell[2611612, 104772, 126, 4, 27, "Output",
   CellTags->"$MIntegrate"],
  Cell[2611753, 104779, 191, 9, 41, "Text",
   CellTags->"$MIntegrate"]},
 "$MU"->{
  Cell[2611959, 104791, 43, 1, 57, "ObjectName",
   CellTags->"$MU"],
  Cell[2612027, 104796, 54, 1, 52, "Subsubsection",
   CellTags->"$MU"],
  Cell[2612084, 104799, 151, 4, 25, "Text",
   CellTags->"$MU"],
  Cell[2612260, 104807, 73, 2, 28, "Input",
   CellTags->"$MU"],
  Cell[2612336, 104811, 102, 3, 27, "Output",
   CellTags->"$MU"],
  Cell[2612453, 104817, 174, 9, 41, "Text",
   CellTags->"$MU"]},
 "$MemoryAvailable"->{
  Cell[2612642, 104829, 69, 1, 57, "ObjectName",
   CellTags->"$MemoryAvailable"],
  Cell[2612736, 104834, 67, 1, 52, "Subsubsection",
   CellTags->"$MemoryAvailable"],
  Cell[2612806, 104837, 434, 11, 61, "Text",
   CellTags->"$MemoryAvailable"],
  Cell[2613265, 104852, 99, 2, 28, "Input",
   CellTags->"$MemoryAvailable"],
  Cell[2613367, 104856, 115, 3, 27, "Output",
   CellTags->"$MemoryAvailable"]},
 "$NonComm"->{
  Cell[2613509, 104863, 53, 1, 57, "ObjectName",
   CellTags->"$NonComm"],
  Cell[2613587, 104868, 59, 1, 52, "Subsubsection",
   CellTags->"$NonComm"],
  Cell[2613649, 104871, 109, 3, 25, "Text",
   CellTags->"$NonComm"],
  Cell[2613783, 104878, 83, 2, 28, "Input",
   CellTags->"$NonComm"],
  Cell[2613869, 104882, 803, 15, 93, "Output",
   CellTags->"$NonComm"],
  Cell[2614687, 104900, 307, 14, 41, "Text",
   CellTags->"$NonComm"]},
 "$PairBrackets"->{
  Cell[2615009, 104917, 63, 1, 57, "ObjectName",
   CellTags->"$PairBrackets"],
  Cell[2615097, 104922, 64, 1, 52, "Subsubsection",
   CellTags->"$PairBrackets"],
  Cell[2615164, 104925, 155, 4, 25, "Text",
   CellTags->"$PairBrackets"],
  Cell[2615344, 104933, 93, 2, 28, "Input",
   CellTags->"$PairBrackets"],
  Cell[2615440, 104937, 114, 3, 27, "Output",
   CellTags->"$PairBrackets"],
  Cell[2615591, 104945, 204, 7, 28, "Input",
   CellTags->"$PairBrackets"],
  Cell[2615798, 104954, 425, 18, 27, "Output",
   CellTags->"$PairBrackets"],
  Cell[2616260, 104977, 118, 3, 28, "Input",
   CellTags->"$PairBrackets"],
  Cell[2616381, 104982, 113, 3, 27, "Output",
   CellTags->"$PairBrackets"],
  Cell[2616531, 104990, 204, 7, 28, "Input",
   CellTags->"$PairBrackets"],
  Cell[2616738, 104999, 445, 18, 27, "Output",
   CellTags->"$PairBrackets"],
  Cell[2617220, 105022, 119, 3, 28, "Input",
   CellTags->"$PairBrackets"],
  Cell[2617342, 105027, 114, 3, 27, "Output",
   CellTags->"$PairBrackets"]},
 "$SpinorMinimal"->{
  Cell[2617483, 105034, 65, 1, 57, "ObjectName",
   CellTags->"$SpinorMinimal"],
  Cell[2617573, 105039, 65, 1, 52, "Subsubsection",
   CellTags->"$SpinorMinimal"],
  Cell[2617641, 105042, 241, 5, 43, "Text",
   CellTags->"$SpinorMinimal"],
  Cell[2617907, 105051, 95, 2, 28, "Input",
   CellTags->"$SpinorMinimal"],
  Cell[2618005, 105055, 115, 3, 27, "Output",
   CellTags->"$SpinorMinimal"],
  Cell[2618135, 105061, 200, 9, 41, "Text",
   CellTags->"$SpinorMinimal"]},
 "$VeryVerbose"->{
  Cell[2618350, 105073, 61, 1, 57, "ObjectName",
   CellTags->"$VeryVerbose"],
  Cell[2618436, 105078, 63, 1, 52, "Subsubsection",
   CellTags->"$VeryVerbose"],
  Cell[2618502, 105081, 221, 5, 43, "Text",
   CellTags->"$VeryVerbose"],
  Cell[2618748, 105090, 91, 2, 28, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2618842, 105094, 109, 3, 27, "Output",
   CellTags->"$VeryVerbose"],
  Cell[2618966, 105100, 141, 4, 28, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2619132, 105108, 287, 10, 28, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2619422, 105120, 121, 3, 22, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2619546, 105125, 255, 7, 23, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2619804, 105134, 273, 7, 23, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2620092, 105144, 131, 4, 28, "Input",
   CellTags->"$VeryVerbose"]},
 "$V0"->{
  Cell[2620226, 105150, 159, 9, 41, "Text",
   CellTags->"$V0"]},
 "$West"->{
  Cell[2620400, 105162, 47, 1, 57, "ObjectName",
   CellTags->"$West"],
  Cell[2620472, 105167, 56, 1, 52, "Subsubsection",
   CellTags->"$West"],
  Cell[2620531, 105170, 468, 13, 59, "Text",
   CellTags->"$West"],
  Cell[2621002, 105185, 158, 8, 41, "Text",
   CellTags->"$West"]},
 "Experimental"->{
  Cell[2622183, 105246, 70, 1, 81, "Section",
   CellTags->"Experimental"],
  Cell[2643109, 106009, 122, 3, 25, "Text",
   CellTags->"Experimental"],
  Cell[2643256, 106016, 198, 6, 28, "Input",
   CellTags->"Experimental"],
  Cell[2643457, 106024, 193, 6, 27, "Output",
   CellTags->"Experimental"],
  Cell[2643665, 106033, 100, 1, 25, "Text",
   CellTags->"Experimental"],
  Cell[2643790, 106038, 222, 6, 28, "Input",
   CellTags->"Experimental"],
  Cell[2644015, 106046, 246, 7, 27, "Output",
   CellTags->"Experimental"]},
 "DoPolarizationSums"->{
  Cell[2622256, 105249, 73, 1, 57, "ObjectName",
   CellTags->"DoPolarizationSums"],
  Cell[2622354, 105254, 69, 1, 52, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2622426, 105257, 291, 7, 70, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2622720, 105266, 317, 14, 70, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2623074, 105285, 66, 1, 52, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2623143, 105288, 310, 6, 70, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2623456, 105296, 123, 3, 25, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2623604, 105303, 571, 17, 50, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2624178, 105322, 448, 16, 27, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2624663, 105343, 116, 3, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2624782, 105348, 133, 4, 27, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2624930, 105355, 1270, 48, 70, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2626203, 105405, 562, 21, 25, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2626768, 105428, 142, 4, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2626913, 105434, 311, 10, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2627249, 105448, 271, 8, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2627523, 105458, 733, 20, 73, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2628271, 105481, 213, 9, 25, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2628487, 105492, 276, 9, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2628766, 105503, 328, 10, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2629097, 105515, 323, 13, 41, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2629423, 105530, 697, 30, 26, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2630145, 105564, 434, 13, 51, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2630582, 105579, 773, 24, 73, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2631370, 105606, 624, 27, 26, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2632019, 105637, 433, 13, 51, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2632455, 105652, 773, 24, 73, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2633265, 105681, 859, 24, 72, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2634127, 105707, 606, 22, 28, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2634770, 105734, 130, 3, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2634903, 105739, 323, 12, 30, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2635263, 105756, 116, 3, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2635382, 105761, 133, 4, 27, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2635552, 105770, 120, 3, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2635675, 105775, 133, 4, 27, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2635845, 105784, 126, 3, 28, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2635974, 105789, 133, 4, 27, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2636122, 105796, 167, 4, 28, "Input",
   CellTags->"DoPolarizationSums"]},
 "FeynmanDoIntegrals"->{
  Cell[2636304, 105803, 90, 1, 41, "ObjectName",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2636419, 105808, 69, 1, 36, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2636491, 105811, 1518, 23, 195, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2638012, 105836, 347, 14, 25, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2638384, 105854, 136, 3, 28, "Input",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2638523, 105859, 1037, 29, 71, "Output",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2639609, 105894, 66, 1, 36, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2639678, 105897, 85, 2, 28, "Input",
   CellTags->"FeynmanDoIntegrals"]},
 "FeynmanParametrize1"->{
  Cell[2639778, 105902, 92, 1, 41, "ObjectName",
   CellTags->"FeynmanParametrize1"],
  Cell[2639895, 105907, 70, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2639968, 105910, 717, 12, 141, "Text",
   CellTags->"FeynmanParametrize1"],
  Cell[2640710, 105926, 138, 3, 28, "Input",
   CellTags->"FeynmanParametrize1"],
  Cell[2640851, 105931, 505, 13, 49, "Output",
   CellTags->"FeynmanParametrize1"],
  Cell[2641405, 105950, 67, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2641475, 105953, 86, 2, 28, "Input",
   CellTags->"FeynmanParametrize1"]},
 "FeynmanReduce"->{
  Cell[2641576, 105958, 80, 1, 41, "ObjectName",
   CellTags->"FeynmanReduce"],
  Cell[2641681, 105963, 64, 1, 36, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2641748, 105966, 723, 12, 141, "Text",
   CellTags->"FeynmanReduce"],
  Cell[2642496, 105982, 126, 3, 28, "Input",
   CellTags->"FeynmanReduce"],
  Cell[2642625, 105987, 288, 9, 27, "Output",
   CellTags->"FeynmanReduce"],
  Cell[2642962, 106002, 61, 1, 36, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2643026, 106005, 80, 2, 28, "Input",
   CellTags->"FeynmanReduce"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"ClickSave", 2646150, 106120},
 {"Functions", 2646241, 106123},
 {"Abbreviation", 2646327, 106126},
 {"Amplitude", 2646871, 106142},
 {"Amputate", 2648111, 106179},
 {"AnomalousDimension", 2648897, 106202},
 {"AntiCommutator", 2652273, 106289},
 {"AntiQuarkField", 2654365, 106346},
 {"AntiSymmetrize", 2654915, 106361},
 {"Anti5", 2655605, 106380},
 {"Apart2", 2657248, 106431},
 {"A0", 2657997, 106454},
 {"A0ToB0", 2659216, 106494},
 {"BackgroundGluonVertex", 2659672, 106508},
 {"Bracket", 2660929, 106539},
 {"BReduce", 2661220, 106548},
 {"B0", 2662010, 106572},
 {"B0Real", 2663039, 106605},
 {"B0Unique", 2663953, 106633},
 {"B00", 2664888, 106661},
 {"B1", 2666187, 106702},
 {"B11", 2667595, 106747},
 {"CA", 2669148, 106796},
 {"Calc", 2669858, 106819},
 {"CalcColorFactor", 2671125, 106858},
 {"Collecting", 2671453, 106867},
 {"CalculateCounterTerm", 2671845, 106878},
 {"Info3387711865-3148101", 2672400, 106891},
 {"CancelQP", 2672524, 106894},
 {"Cases2", 2673734, 106929},
 {"CF", 2674563, 106954},
 {"ChangeDimension", 2675411, 106981},
 {"ChargeConjugationMatrix", 2678172, 107054},
 {"ChargeConjugationMatrixInv", 2679473, 107085},
 {"CheckContext", 2681151, 107124},
 {"CheckDB", 2682118, 107151},
 {"ChiralityProjector", 2682899, 107174},
 {"Chisholm", 2683795, 107197},
 {"ChisholmSpinor", 2686173, 107266},
 {"ClearScalarProducts", 2686891, 107285},
 {"Collect2", 2688048, 107314},
 {"Collect3", 2689758, 107363},
 {"Combine", 2690559, 107386},
 {"CombineGraphs", 2691624, 107417},
 {"Commutator", 2691946, 107426},
 {"CommutatorExplicit", 2693486, 107469},
 {"CompleteSquare", 2694876, 107504},
 {"ComplexConjugate", 2695749, 107527},
 {"Contract", 2697799, 107580},
 {"Contract1", 2701174, 107677},
 {"Convolute", 2702127, 107704},
 {"ConvoluteTable", 2706345, 107823},
 {"CovariantFieldDerivative", 2707374, 107850},
 {"CounterT", 2708004, 107865},
 {"CounterTerm", 2708527, 107880},
 {"CouplingConstant", 2709075, 107895},
 {"CovariantD", 2709413, 107904},
 {"CrossProduct", 2712465, 107989},
 {"C0", 2713005, 108004},
 {"DataType", 2713736, 108027},
 {"DB0", 2716605, 108110},
 {"DB1", 2717082, 108125},
 {"DeclareNonCommutative", 2717579, 108140},
 {"DeltaFunction", 2718849, 108171},
 {"DeltaFunctionDoublePrime", 2720016, 108202},
 {"DeltaFunctionPrime", 2720391, 108211},
 {"DenominatorOrder", 2721306, 108234},
 {"Dimension", 2721641, 108243},
 {"DimensionalReduction", 2722877, 108278},
 {"DiracBasis", 2723229, 108287},
 {"DiracCanonical", 2723544, 108296},
 {"DiracEquation", 2723874, 108305},
 {"DiracGamma", 2724577, 108324},
 {"DiracGammaCombine", 2726554, 108379},
 {"DiracGammaExpand", 2727772, 108410},
 {"DiracGammaT", 2729507, 108455},
 {"DiracMatrix", 2730337, 108478},
 {"DiracOrder", 2732259, 108531},
 {"DiracReduce", 2734511, 108594},
 {"DiracSigma", 2736287, 108643},
 {"DiracSigmaExplicit", 2737472, 108676},
 {"DiracSimpCombine", 2738226, 108695},
 {"DiracSimplify", 2738564, 108704},
 {"DiracSimplify2", 2742333, 108805},
 {"DiracSlash", 2743046, 108824},
 {"DiracSpinor", 2744511, 108865},
 {"DiracSubstitute67", 2744833, 108874},
 {"DiracTrace", 2745173, 108883},
 {"DiracTraceEvaluate", 2746936, 108932},
 {"DiracTrick", 2747280, 108941},
 {"Divideout", 2749887, 109014},
 {"DOT", 2750188, 109023},
 {"DotExpand", 2750470, 109032},
 {"DotPower", 2751421, 109059},
 {"DotProduct", 2751724, 109068},
 {"DotSimplifyRelations", 2752268, 109083},
 {"DotSimplify", 2752620, 109092},
 {"DummyIndex", 2755266, 109165},
 {"D0", 2755569, 109174},
 {"D0Convention", 2756307, 109197},
 {"Eps", 2756620, 109206},
 {"EpsChisholm", 2758985, 109279},
 {"EpsContract", 2759673, 109298},
 {"EpsDiscard", 2760507, 109321},
 {"EpsEvaluate", 2760819, 109330},
 {"EpsUncontract", 2761656, 109353},
 {"Epsilon", 2761977, 109362},
 {"EpsilonOrder", 2762694, 109383},
 {"Expanding", 2763014, 109392},
 {"ExpandPartialD", 2763326, 109401},
 {"ScalarProductCancel", 2765572, 109460},
 {"ExpandScalarProduct", 2770133, 109557},
 {"Expand2", 2772594, 109604},
 {"Explicit", 2773038, 109617},
 {"ExplicitLorentzIndex", 2774271, 109652},
 {"ExplicitPartialD", 2775043, 109671},
 {"ExplicitSUNIndex", 2776248, 109702},
 {"ExtraFactor", 2776824, 109717},
 {"ExtraVariables", 2777143, 109726},
 {"FactorFull", 2777471, 109735},
 {"Factoring", 2777782, 109744},
 {"Factorout", 2778089, 109753},
 {"FactorTime", 2778395, 109762},
 {"Factor1", 2778703, 109771},
 {"Factor2", 2779828, 109804},
 {"FAD", 2781223, 109845},
 {"FC", 2782219, 109876},
 {"FCE", 2782941, 109899},
 {"FCI", 2784259, 109940},
 {"FCIntegral", 2785264, 109971},
 {"FCIntegrate", 2785577, 109980},
 {"FCNIntegrate", 2785894, 109989},
 {"FC2RHI", 2786209, 109998},
 {"FC2TLI", 2786775, 110015},
 {"Info3387711966-3304885", 2787357, 110032},
 {"FDS", 2787452, 110035},
 {"FermionSpinSum", 2787738, 110044},
 {"FeynAmp", 2790040, 110105},
 {"FeynAmpDenominator", 2790901, 110130},
 {"FeynAmpDenominatorCombine", 2792141, 110161},
 {"FeynAmpDenominatorSimplify", 2793141, 110184},
 {"FeynAmpDenominatorSplit", 2795556, 110239},
 {"FeynAmpList", 2797113, 110276},
 {"FeynCalc", 2797497, 110287},
 {"Info3387711973-8270308", 2797910, 110298},
 {"FeynCalcExternal", 2798033, 110301},
 {"FeynCalcForm", 2799545, 110340},
 {"FeynCalcInternal", 2800755, 110373},
 {"FeynCalc2FORM", 2802275, 110412},
 {"FeynCalcToLaTeX", 2804775, 110479},
 {"FeynmanParameterNames", 2805042, 110486},
 {"FeynmanParametrize", 2805406, 110495},
 {"FeynRule", 2805992, 110510},
 {"FI", 2810637, 110643},
 {"FieldDerivative", 2810922, 110652},
 {"FieldStrength", 2811806, 110675},
 {"FinalFunction", 2813415, 110718},
 {"FinalSubstitutions", 2813745, 110727},
 {"FORM", 2814083, 110736},
 {"FORMEpilog", 2814371, 110745},
 {"FORMProlog", 2814683, 110754},
 {"FORM2FeynCalc", 2814998, 110763},
 {"FourDivergence", 2817124, 110820},
 {"FourLaplacian", 2818216, 110849},
 {"FourVector", 2819371, 110880},
 {"FreeIndex", 2820692, 110917},
 {"FreeQ2", 2820996, 110926},
 {"FRH", 2821899, 110953},
 {"FromTFi", 2822767, 110980},
 {"FromTFI", 2823488, 111001},
 {"FUNCTION", 2823572, 111004},
 {"FunctionalD", 2823876, 111013},
 {"FV", 2830632, 111198},
 {"FVD", 2831800, 111235},
 {"GA", 2832984, 111272},
 {"GAD", 2834091, 111307},
 {"GammaEpsilon", 2835224, 111342},
 {"GammaExpand", 2836370, 111373},
 {"Gamma1", 2837208, 111396},
 {"Gamma2", 2837504, 111405},
 {"Gamma3", 2837800, 111414},
 {"Gauge", 2838095, 111423},
 {"GaugeField", 2838391, 111432},
 {"GaugeXi", 2839073, 111451},
 {"GA5", 2839369, 111460},
 {"GGV", 2839980, 111479},
 {"GhostPropagator", 2840271, 111488},
 {"GHP", 2841461, 111519},
 {"GluonField", 2842345, 111546},
 {"GluonGhostVertex", 2843034, 111565},
 {"GluonPropagator", 2844097, 111592},
 {"GluonSelfEnergy", 2846707, 111659},
 {"GluonVertex", 2847599, 111682},
 {"GO", 2849842, 111743},
 {"GP", 2850116, 111752},
 {"GrassmannParity", 2850403, 111761},
 {"GS", 2850730, 111770},
 {"GSD", 2851725, 111801},
 {"Gstrong", 2852739, 111832},
 {"GTI", 2853317, 111849},
 {"GV", 2853595, 111858},
 {"Hill", 2853871, 111867},
 {"HypergeometricAC", 2854777, 111894},
 {"HypergeometricIR", 2856400, 111935},
 {"HypergeometricSE", 2857464, 111962},
 {"HypExplicit", 2858364, 111985},
 {"HypInt", 2859354, 112012},
 {"IFPD", 2859932, 112029},
 {"IFPDOff", 2860219, 112038},
 {"IFPDOn", 2861152, 112065},
 {"IncludePair", 2861453, 112074},
 {"IndexPosition", 2861775, 112083},
 {"Indices", 2862098, 112092},
 {"InitialFunction", 2862406, 112101},
 {"InitialSubstitutions", 2862751, 112110},
 {"InsideDiracTrace", 2863113, 112119},
 {"IntegralTable", 2863456, 112128},
 {"IntegrateByParts", 2863790, 112137},
 {"Integratedx", 2864454, 112154},
 {"Integrate2", 2864774, 112163},
 {"Integrate3", 2870071, 112308},
 {"IntermediateSubstitutions", 2871356, 112343},
 {"InverseMellin", 2871734, 112352},
 {"Isolate", 2876091, 112467},
 {"Info3387712053-5834299", 2877865, 112516},
 {"IsolateHead", 2878079, 112521},
 {"IsolateNames", 2878399, 112530},
 {"IsolatePrint", 2878725, 112539},
 {"IsolateSplit", 2879050, 112548},
 {"KeepOnly", 2879371, 112557},
 {"KK", 2879671, 112566},
 {"Kummer", 2879951, 112575},
 {"Lagrangian", 2885380, 112730},
 {"LC", 2886567, 112763},
 {"LCD", 2887692, 112798},
 {"LeftPartialD", 2889104, 112841},
 {"LeftRightPartialD", 2890566, 112880},
 {"LeftRightPartialD2", 2891965, 112915},
 {"LeviCivita", 2893373, 112950},
 {"LeviCivitaSign", 2895447, 113007},
 {"ListIntegrals", 2895782, 113016},
 {"Li2", 2896184, 113027},
 {"Li3", 2896929, 113050},
 {"Loop", 2897807, 113077},
 {"LorentzIndex", 2898034, 113084},
 {"Lower", 2899696, 113129},
 {"MakeContext", 2899993, 113138},
 {"Mandelstam", 2900313, 113147},
 {"Map2", 2900625, 113156},
 {"MemoryAvailable", 2901526, 113183},
 {"MemSet", 2901858, 113192},
 {"MetricTensor", 2902159, 113201},
 {"MLimit", 2903752, 113244},
 {"Momentum", 2904196, 113257},
 {"MomentumCombine", 2906271, 113316},
 {"MomentumCombine2", 2907704, 113353},
 {"MomentumExpand", 2908449, 113372},
 {"MT", 2910084, 113415},
 {"MTD", 2911076, 113446},
 {"NegativeInteger", 2912096, 113477},
 {"Nf", 2912423, 113486},
 {"Nielsen", 2912703, 113495},
 {"NonCommFreeQ", 2914197, 113538},
 {"NonCommQ", 2914519, 113547},
 {"NonCommutative", 2914900, 113558},
 {"NTerms", 2915228, 113567},
 {"NumberOfMetricTensors", 2916094, 113592},
 {"NumericalFactor", 2916375, 113599},
 {"NumericQ1", 2916633, 113606},
 {"OneLoop", 2917379, 113627},
 {"OneLoopSimplify", 2919375, 113684},
 {"OneLoopSum", 2921051, 113727},
 {"OPE", 2921359, 113736},
 {"OPEDelta", 2921644, 113745},
 {"OPEi", 2922446, 113768},
 {"OPEInsert", 2923613, 113803},
 {"OPEInt", 2924068, 113816},
 {"OPEIntDelta", 2924510, 113829},
 {"OPEIntegrate", 2924984, 113842},
 {"OPEIntegrateDelta", 2925468, 113855},
 {"OPESumExplicit", 2925980, 113868},
 {"OPESum", 2927085, 113897},
 {"OPESumSimplify", 2928088, 113926},
 {"OPE1Loop", 2929272, 113957},
 {"OPE2TID", 2929722, 113970},
 {"OptionsSelect", 2930312, 113987},
 {"Pair", 2930559, 113994},
 {"PairCollect", 2932521, 114053},
 {"PairContract", 2932843, 114062},
 {"PairContract2", 2933170, 114071},
 {"PairContract3", 2933501, 114080},
 {"PartialD", 2933827, 114089},
 {"PartialDRelations", 2934645, 114112},
 {"PartialFourVector", 2935403, 114131},
 {"PartialIntegrate", 2936000, 114146},
 {"PartitHead", 2937376, 114181},
 {"PauliSigma", 2937620, 114188},
 {"PaVe", 2938007, 114199},
 {"PaVeOrderList", 2938771, 114222},
 {"PaVeOrder", 2939098, 114231},
 {"PaVeReduce", 2939923, 114254},
 {"Info3387712116-6749064", 2941968, 114309},
 {"PD", 2942078, 114312},
 {"PlusDistribution", 2942367, 114321},
 {"Polarization", 2943740, 114356},
 {"PolarizationSum", 2945196, 114395},
 {"PolarizationUncontract", 2946408, 114426},
 {"PolarizationVector", 2946780, 114435},
 {"PositiveInteger", 2948994, 114490},
 {"PositiveNumber", 2949333, 114499},
 {"PowerFactor", 2949665, 114508},
 {"PowerSimplify", 2950362, 114527},
 {"Power2", 2951375, 114554},
 {"PostFortranFile", 2952029, 114573},
 {"Prefactor", 2952364, 114582},
 {"PreFortranFile", 2952680, 114591},
 {"PropagatorDenominator", 2953023, 114600},
 {"PropagatorDenominatorExplicit", 2954746, 114641},
 {"QGV", 2955782, 114664},
 {"QO", 2956060, 114673},
 {"QP", 2956334, 114682},
 {"QuantumField", 2956619, 114691},
 {"QuarkField", 2959401, 114766},
 {"QuarkGluonVertex", 2959946, 114781},
 {"QuarkMass", 2961471, 114820},
 {"QuarkPropagator", 2961786, 114829},
 {"ReduceGamma", 2962828, 114856},
 {"ReduceToScalars", 2963152, 114865},
 {"Rename", 2963484, 114874},
 {"RHI", 2963776, 114883},
 {"RHI2FC", 2964197, 114896},
 {"RHM", 2964633, 114909},
 {"RHO", 2965048, 114922},
 {"RHP", 2965464, 114935},
 {"RTL", 2965880, 114948},
 {"RightPartialD", 2966307, 114961},
 {"RussianTrick", 2967477, 114992},
 {"ScalarGluonVertex", 2968793, 115027},
 {"ScalarProduct", 2969466, 115044},
 {"Info3387712142-8239904", 2971325, 115093},
 {"ScaleMu", 2971449, 115096},
 {"Schouten", 2971820, 115107},
 {"SD", 2972487, 115126},
 {"SelectFree", 2973228, 115149},
 {"SelectGraphs", 2974710, 115190},
 {"SelectNotFree", 2975036, 115199},
 {"SelectSplit", 2976651, 115242},
 {"Select1", 2977273, 115259},
 {"Select2", 2977573, 115268},
 {"Series2", 2977873, 115277},
 {"Series3", 2979867, 115334},
 {"SetMandelstam", 2980668, 115357},
 {"SimplifyDeltaFunction", 2981157, 115370},
 {"SimplifyGTI", 2983619, 115429},
 {"SimplifyPolyLog", 2984094, 115442},
 {"Simplify2", 2987477, 115529},
 {"SmallDelta", 2987865, 115540},
 {"SmallEpsilon", 2988110, 115547},
 {"SmallVariable", 2988362, 115554},
 {"SmallVariables", 2988694, 115563},
 {"SMP", 2989018, 115572},
 {"SMVertex", 2989701, 115593},
 {"Smu", 2990293, 115610},
 {"Sn", 2990507, 115617},
 {"SO", 2990720, 115624},
 {"SOD", 2991456, 115647},
 {"Solve2", 2992207, 115670},
 {"Solve3", 2993405, 115705},
 {"SP", 2993980, 115722},
 {"SPC", 2995235, 115761},
 {"SPD", 2995514, 115770},
 {"Spinor", 2996922, 115813},
 {"SpinorCollect", 2998599, 115862},
 {"SpinorUBar", 2998927, 115871},
 {"SpinorU", 3000114, 115904},
 {"SpinorVBar", 3001188, 115935},
 {"SpinorV", 3002305, 115966},
 {"SpinPolarizationSum", 3003388, 115997},
 {"SPL", 3003732, 116006},
 {"SplittingFunction", 3004025, 116015},
 {"StandardMatrixElement", 3008784, 116134},
 {"SubContext", 3009143, 116143},
 {"SubLoop", 3009456, 116152},
 {"SumP", 3009753, 116161},
 {"SumS", 3011046, 116200},
 {"SumT", 3013271, 116267},
 {"SUND", 3015631, 116338},
 {"SUNDelta", 3017397, 116391},
 {"SUNDeltaContract", 3019272, 116444},
 {"SUNF", 3019606, 116453},
 {"SUNFJacobi", 3021909, 116522},
 {"SUNIndex", 3022597, 116541},
 {"SUNIndexRename", 3023409, 116564},
 {"SUNN", 3023735, 116573},
 {"SUNNToCACF", 3024233, 116588},
 {"SUNSimplify", 3024774, 116603},
 {"SUNT", 3029085, 116720},
 {"SUNTrace", 3031248, 116785},
 {"SymbolicSum2", 3033482, 116848},
 {"SymbolicSum3", 3033734, 116855},
 {"Symmetrize", 3033984, 116862},
 {"TBox", 3034664, 116881},
 {"Tbox", 3034914, 116888},
 {"Tdec", 3035164, 116895},
 {"TensorFunction", 3036474, 116934},
 {"Tf", 3038182, 116979},
 {"TFi", 3038460, 116988},
 {"ThreeVector", 3039216, 117011},
 {"TID", 3039528, 117020},
 {"TIDL", 3040676, 117055},
 {"TimedIntegrate", 3043002, 117122},
 {"TLI", 3043944, 117147},
 {"TLIFP", 3044495, 117164},
 {"TLIHYP", 3045061, 117181},
 {"TLI2", 3045633, 117198},
 {"TLI2FC", 3045919, 117207},
 {"ToDistribution", 3046503, 117224},
 {"ToHypergeometric", 3048155, 117267},
 {"ToLarin", 3049219, 117294},
 {"ToTFi", 3049875, 117313},
 {"TR", 3050785, 117340},
 {"Info3387712206-4027155", 3054090, 117441},
 {"TraceDimension", 3054204, 117444},
 {"TraceOfOne", 3054535, 117453},
 {"Trick", 3055215, 117472},
 {"TrickIntegrate", 3056337, 117505},
 {"TrickMandelstam", 3056836, 117518},
 {"Tr2", 3057486, 117535},
 {"Twist2AlienOperator", 3058117, 117554},
 {"Twist2CounterOperator", 3058818, 117571},
 {"Twist2GluonOperator", 3059703, 117592},
 {"Twist2QuarkOperator", 3067504, 117781},
 {"Twist3QuarkOperator", 3072387, 117900},
 {"Twist4GluonOperator", 3073166, 117919},
 {"TwoLoopSimplify", 3073783, 117934},
 {"Uncontract", 3074285, 117947},
 {"UnDeclareNonCommutative", 3076245, 118000},
 {"Upper", 3077911, 118039},
 {"UVPart", 3078207, 118048},
 {"Vectors", 3078583, 118059},
 {"WriteOut", 3078888, 118068},
 {"WriteOutPaVe", 3079201, 118077},
 {"Write2", 3079525, 118086},
 {"XYT", 3081777, 118151},
 {"ZeroMomentumInsertion", 3082014, 118158},
 {"Zeta2", 3082373, 118167},
 {"$Abbreviations", 3083160, 118190},
 {"$AL", 3083645, 118203},
 {"$BreitMaison", 3084405, 118226},
 {"$Color", 3084729, 118235},
 {"$Covariant", 3084964, 118242},
 {"$FCS", 3085791, 118265},
 {"$FCT", 3086149, 118276},
 {"$FeynCalcStuff", 3086786, 118295},
 {"$FeynCalcVersion", 3087051, 118302},
 {"$FortranContinuationCharacter", 3087493, 118313},
 {"$Gauge", 3088069, 118326},
 {"$Larin", 3088368, 118335},
 {"$LimitTo4", 3088812, 118348},
 {"$LorentzIndices", 3089277, 118361},
 {"$MIntegrate", 3090328, 118388},
 {"$MU", 3090792, 118401},
 {"$MemoryAvailable", 3091221, 118414},
 {"$NonComm", 3091643, 118425},
 {"$PairBrackets", 3092101, 118438},
 {"$SpinorMinimal", 3093127, 118465},
 {"$VeryVerbose", 3093618, 118478},
 {"$V0", 3094465, 118501},
 {"$West", 3094543, 118504},
 {"Experimental", 3094845, 118513},
 {"DoPolarizationSums", 3095397, 118528},
 {"FeynmanDoIntegrals", 3098538, 118605},
 {"FeynmanParametrize1", 3099235, 118622},
 {"FeynmanReduce", 3099850, 118637}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 53, 1, 104, "Title"],
Cell[646, 26, 87, 1, 66, "Subtitle"],
Cell[736, 29, 123, 3, 66, "Subtitle",
 PageBreakBelow->True],
Cell[862, 34, 80, 1, 53, "Subsubtitle"],
Cell[945, 37, 244, 8, 53, "Subsubtitle"],
Cell[1192, 47, 799, 21, 36, NotebookDefault,
 CellTags->"ClickSave"],
Cell[CellGroupData[{
Cell[2016, 72, 147, 4, 28, "Input"],
Cell[2166, 78, 335, 9, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2538, 92, 52, 1, 65, "Section",
 CellTags->"Functions"],
Cell[2593, 95, 54, 0, 31, "Commentary"],
Cell[2650, 97, 87, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[2762, 103, 122, 3, 28, "Input"],
Cell[2887, 108, 120, 3, 27, "Output"]
}, Open  ]],
Cell[3022, 114, 69, 1, 28, "Input"],
Cell[3094, 117, 84, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[3203, 123, 63, 1, 36, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[3269, 126, 329, 6, 61, "Text",
 CellTags->"Abbreviation"],
Cell[3601, 134, 1032, 43, 43, "Text",
 CellTags->"Abbreviation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4670, 182, 60, 1, 36, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[4755, 187, 264, 8, 28, "Input",
 CellTags->"Abbreviation"],
Cell[5022, 197, 228, 8, 27, "Output",
 CellTags->"Abbreviation"]
}, Open  ]]
}, Open  ]],
Cell[5277, 209, 55, 1, 41, "ObjectName",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[5357, 214, 60, 1, 36, "Subsubsection",
 CellTags->"Amplitude"],
Cell[5420, 217, 529, 9, 79, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[5974, 230, 115, 3, 28, "Input",
 CellTags->"Amplitude"],
Cell[6092, 235, 328, 10, 27, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[6435, 248, 177, 9, 25, "Text",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6649, 262, 57, 1, 36, "Subsubsection",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[6731, 267, 133, 4, 28, "Input",
 CellTags->"Amplitude"],
Cell[6867, 273, 104, 3, 27, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[6986, 279, 93, 1, 25, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[7104, 284, 121, 3, 28, "Input",
 CellTags->"Amplitude"],
Cell[7228, 289, 2216, 95, 29, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9481, 389, 108, 3, 28, "Input",
 CellTags->"Amplitude"],
Cell[9592, 394, 5105, 187, 49, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[14712, 584, 211, 8, 25, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[14948, 596, 306, 9, 28, "Input",
 CellTags->"Amplitude"],
Cell[15257, 607, 667, 14, 40, "Message",
 CellTags->"Amplitude"],
Cell[15927, 623, 13529, 441, 135, "Output",
 CellTags->"Amplitude"]
}, Open  ]]
}, Open  ]],
Cell[29483, 1068, 53, 1, 41, "ObjectName",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[29561, 1073, 59, 1, 36, "Subsubsection",
 CellTags->"Amputate"],
Cell[29623, 1076, 215, 5, 43, "Text",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[29863, 1085, 114, 3, 28, "Input",
 CellTags->"Amputate"],
Cell[29980, 1090, 296, 10, 27, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[30291, 1103, 656, 28, 25, "Text",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30984, 1136, 56, 1, 36, "Subsubsection",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[31065, 1141, 170, 5, 28, "Input",
 CellTags->"Amputate"],
Cell[31238, 1148, 428, 16, 27, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31703, 1169, 131, 4, 28, "Input",
 CellTags->"Amputate"],
Cell[31837, 1175, 721, 27, 27, "Output",
 CellTags->"Amputate"]
}, Open  ]]
}, Open  ]],
Cell[32585, 1206, 73, 1, 41, "ObjectName",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[32683, 1211, 69, 1, 36, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[32755, 1214, 143, 4, 25, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[32923, 1222, 134, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[33060, 1227, 254, 8, 27, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[33329, 1238, 406, 18, 25, "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33772, 1261, 66, 1, 36, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[33841, 1264, 63, 1, 25, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[33929, 1269, 206, 5, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[34138, 1276, 254, 8, 27, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[34407, 1287, 241, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[34673, 1300, 133, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[34809, 1305, 462, 17, 40, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[35286, 1325, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[35551, 1338, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[35686, 1343, 284, 10, 40, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[35985, 1356, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[36250, 1369, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[36385, 1374, 283, 10, 40, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[36683, 1387, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[36948, 1400, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[37083, 1405, 594, 21, 40, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[37692, 1429, 241, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[37958, 1442, 133, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[38094, 1447, 689, 23, 47, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[38798, 1473, 241, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[39064, 1486, 133, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[39200, 1491, 6351, 214, 233, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[45566, 1708, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[45831, 1721, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[45966, 1726, 4090, 139, 141, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[50071, 1868, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[50336, 1881, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[50471, 1886, 6253, 212, 226, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[56739, 2101, 240, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[57004, 2114, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[57139, 2119, 4933, 167, 186, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[62087, 2289, 267, 9, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[62379, 2302, 132, 3, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[62514, 2307, 4422, 140, 151, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[66951, 2450, 326, 10, 26, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[67302, 2464, 295, 9, 28, "Input",
 CellTags->"AnomalousDimension"],
Cell[67600, 2475, 239, 7, 27, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]]
}, Open  ]],
Cell[67866, 2486, 65, 1, 41, "ObjectName",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[67956, 2491, 65, 1, 36, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[68024, 2494, 145, 4, 25, "Text",
 CellTags->"AntiCommutator"],
Cell[68172, 2500, 599, 24, 25, "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68808, 2529, 62, 1, 36, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[68873, 2532, 200, 9, 25, "Text",
 CellTags->"AntiCommutator"],
Cell[69076, 2543, 150, 4, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[69251, 2551, 143, 4, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[69397, 2557, 163, 5, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[69597, 2567, 124, 3, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[69724, 2572, 180, 6, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[69941, 2583, 265, 8, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[70209, 2593, 438, 16, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[70684, 2614, 258, 8, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[70945, 2624, 298, 10, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[71258, 2637, 218, 6, 33, "Input",
 CellTags->"AntiCommutator"],
Cell[71479, 2645, 104, 1, 25, "Text",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[71608, 2650, 177, 5, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[71788, 2657, 109, 3, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71934, 2665, 182, 5, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[72119, 2672, 190, 6, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72346, 2683, 250, 8, 33, "Input",
 CellTags->"AntiCommutator"],
Cell[72599, 2693, 132, 4, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72768, 2702, 252, 7, 33, "Input",
 CellTags->"AntiCommutator"],
Cell[73023, 2711, 315, 11, 30, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[73353, 2725, 240, 6, 33, "Input",
 CellTags->"AntiCommutator"],
Cell[73596, 2733, 176, 5, 28, "Input",
 CellTags->"AntiCommutator"],
Cell[73775, 2740, 226, 7, 33, "Input",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[74016, 2750, 65, 1, 41, "ObjectName",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[74106, 2755, 65, 1, 36, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[74174, 2758, 197, 4, 25, "Text",
 CellTags->"AntiQuarkField"],
Cell[74374, 2764, 313, 14, 25, "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74724, 2783, 62, 1, 36, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[74811, 2788, 93, 2, 28, "Input",
 CellTags->"AntiQuarkField"],
Cell[74907, 2792, 137, 4, 28, "Output",
 CellTags->"AntiQuarkField"]
}, Open  ]]
}, Open  ]],
Cell[75071, 2800, 65, 1, 41, "ObjectName",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[75161, 2805, 65, 1, 36, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[75229, 2808, 153, 4, 25, "Text",
 CellTags->"AntiSymmetrize"],
Cell[75385, 2814, 185, 8, 25, "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75607, 2827, 62, 1, 36, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[75694, 2832, 246, 8, 28, "Input",
 CellTags->"AntiSymmetrize"],
Cell[75943, 2842, 319, 11, 38, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76299, 2858, 266, 8, 28, "Input",
 CellTags->"AntiSymmetrize"],
Cell[76568, 2868, 654, 20, 38, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]]
}, Open  ]],
Cell[77249, 2892, 47, 1, 41, "ObjectName",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[77321, 2897, 56, 1, 36, "Subsubsection",
 CellTags->"Anti5"],
Cell[77380, 2900, 487, 16, 25, "Text",
 CellTags->"Anti5"],
Cell[77870, 2918, 163, 7, 25, "Text",
 CellTags->"Anti5"],
Cell[78036, 2927, 423, 19, 25, "Text",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78496, 2951, 53, 1, 36, "Subsubsection",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[78574, 2956, 154, 5, 28, "Input",
 CellTags->"Anti5"],
Cell[78731, 2963, 186, 6, 27, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78954, 2974, 102, 3, 28, "Input",
 CellTags->"Anti5"],
Cell[79059, 2979, 290, 11, 27, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[79386, 2995, 144, 5, 28, "Input",
 CellTags->"Anti5"],
Cell[79533, 3002, 265, 10, 27, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[79835, 3017, 194, 6, 28, "Input",
 CellTags->"Anti5"],
Cell[80032, 3025, 320, 9, 28, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[80389, 3039, 125, 4, 28, "Input",
 CellTags->"Anti5"],
Cell[80517, 3045, 320, 9, 28, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[80874, 3059, 133, 4, 28, "Input",
 CellTags->"Anti5"],
Cell[81010, 3065, 320, 9, 28, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81367, 3079, 151, 5, 28, "Input",
 CellTags->"Anti5"],
Cell[81521, 3086, 320, 9, 28, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[81856, 3098, 351, 14, 25, "Text",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[82232, 3116, 127, 4, 28, "Input",
 CellTags->"Anti5"],
Cell[82362, 3122, 191, 6, 27, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82590, 3133, 102, 3, 28, "Input",
 CellTags->"Anti5"],
Cell[82695, 3138, 290, 11, 27, "Output",
 CellTags->"Anti5"]
}, Open  ]]
}, Open  ]],
Cell[83012, 3153, 49, 1, 41, "ObjectName",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[83086, 3158, 57, 1, 36, "Subsubsection",
 CellTags->"Apart2"],
Cell[83146, 3161, 108, 3, 25, "Text",
 CellTags->"Apart2"],
Cell[83257, 3166, 302, 14, 25, "Text",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83596, 3185, 54, 1, 36, "Subsubsection",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[83675, 3190, 256, 9, 28, "Input",
 CellTags->"Apart2"],
Cell[83934, 3201, 1188, 30, 48, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85159, 3236, 104, 3, 28, "Input",
 CellTags->"Apart2"],
Cell[85266, 3241, 720, 26, 48, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86023, 3272, 140, 4, 28, "Input",
 CellTags->"Apart2"],
Cell[86166, 3278, 517, 19, 54, "Output",
 CellTags->"Apart2"]
}, Open  ]]
}, Open  ]],
Cell[86710, 3301, 64, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[86799, 3307, 53, 1, 36, "Subsubsection",
 CellTags->"A0"],
Cell[86855, 3310, 291, 11, 25, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[87171, 3325, 107, 3, 28, "Input",
 CellTags->"A0"],
Cell[87281, 3330, 161, 6, 27, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[87457, 3339, 437, 24, 25, "Text",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87931, 3368, 50, 1, 36, "Subsubsection",
 CellTags->"A0"],
Cell[87984, 3371, 246, 11, 25, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[88255, 3386, 120, 4, 28, "Input",
 CellTags->"A0"],
Cell[88378, 3392, 259, 10, 27, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[88652, 3405, 186, 6, 28, "Input",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[88863, 3415, 120, 4, 28, "Input",
 CellTags->"A0"],
Cell[88986, 3421, 324, 11, 27, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[89325, 3435, 187, 6, 28, "Input",
 CellTags->"A0"],
Cell[89515, 3443, 222, 8, 25, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[89762, 3455, 96, 3, 28, "Input",
 CellTags->"A0"],
Cell[89861, 3460, 97, 3, 27, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89995, 3468, 160, 5, 28, "Input",
 CellTags->"A0"],
Cell[90158, 3475, 97, 3, 27, "Output",
 CellTags->"A0"]
}, Open  ]]
}, Open  ]],
Cell[90282, 3482, 72, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[90379, 3488, 57, 1, 36, "Subsubsection",
 CellTags->"A0ToB0"],
Cell[90439, 3491, 614, 27, 25, "Text",
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[91078, 3522, 111, 3, 28, "Input",
 CellTags->"A0ToB0"],
Cell[91192, 3527, 165, 6, 27, "Output",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[91372, 3536, 533, 29, 25, "Text",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[91920, 3568, 79, 1, 41, "ObjectName",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[92024, 3573, 72, 1, 36, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[92099, 3576, 1042, 19, 151, "Text",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[93166, 3599, 140, 3, 28, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[93309, 3604, 318, 9, 27, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[93642, 3616, 201, 9, 25, "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93880, 3630, 69, 1, 36, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[93974, 3635, 360, 10, 28, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[94337, 3647, 1460, 57, 31, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[95834, 3709, 436, 12, 28, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[96273, 3723, 3843, 141, 29, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[100153, 3869, 411, 11, 28, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[100567, 3882, 1532, 59, 51, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[102136, 3946, 487, 13, 28, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[102626, 3961, 3921, 143, 46, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[106574, 4108, 51, 1, 41, "ObjectName",
 CellTags->"Bracket"],
Cell[CellGroupData[{
Cell[106650, 4113, 58, 1, 36, "Subsubsection",
 CellTags->"Bracket"],
Cell[106711, 4116, 72, 1, 25, "Text",
 CellTags->"Bracket"],
Cell[106786, 4119, 181, 9, 25, "Text",
 CellTags->"Bracket"]
}, Open  ]],
Cell[106982, 4131, 74, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[107081, 4137, 58, 1, 36, "Subsubsection",
 CellTags->"BReduce"],
Cell[107142, 4140, 148, 4, 25, "Text",
 CellTags->"BReduce"],
Cell[107293, 4146, 534, 29, 25, "Text",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[107864, 4180, 55, 1, 36, "Subsubsection",
 CellTags->"BReduce"],
Cell[107922, 4183, 251, 11, 25, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[108198, 4198, 134, 4, 28, "Input",
 CellTags->"BReduce"],
Cell[108335, 4204, 176, 6, 27, "Output",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108548, 4215, 81, 2, 28, "Input"],
Cell[108632, 4219, 263, 8, 27, "Output"]
}, Open  ]],
Cell[108910, 4230, 94, 3, 25, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[109029, 4237, 183, 5, 28, "Input",
 CellTags->"BReduce"],
Cell[109215, 4244, 260, 10, 40, "Output",
 CellTags->"BReduce"]
}, Open  ]]
}, Open  ]],
Cell[109502, 4258, 41, 1, 41, "ObjectName",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[109568, 4263, 53, 1, 36, "Subsubsection",
 CellTags->"B0"],
Cell[109624, 4266, 390, 9, 61, "Text",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[110039, 4279, 102, 3, 28, "Input",
 CellTags->"B0"],
Cell[110144, 4284, 280, 9, 27, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[110439, 4296, 443, 24, 25, "Text",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110919, 4325, 50, 1, 36, "Subsubsection",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[110994, 4330, 234, 8, 28, "Input",
 CellTags->"B0"],
Cell[111231, 4340, 316, 12, 27, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[111562, 4355, 237, 7, 28, "Input",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[111824, 4366, 154, 5, 28, "Input",
 CellTags->"B0"],
Cell[111981, 4373, 245, 8, 27, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112263, 4386, 179, 6, 28, "Input",
 CellTags->"B0"],
Cell[112445, 4394, 245, 8, 27, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112727, 4407, 177, 6, 28, "Input",
 CellTags->"B0"],
Cell[112907, 4415, 218, 7, 27, "Output",
 CellTags->"B0"]
}, Open  ]]
}, Open  ]],
Cell[113152, 4426, 72, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[113249, 4432, 57, 1, 36, "Subsubsection",
 CellTags->"B0Real"],
Cell[113309, 4435, 186, 4, 25, "Text",
 CellTags->"B0Real"],
Cell[113498, 4441, 153, 8, 25, "Text",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113688, 4454, 54, 1, 36, "Subsubsection",
 CellTags->"B0Real"],
Cell[113745, 4457, 83, 1, 25, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[113853, 4462, 133, 4, 28, "Input",
 CellTags->"B0Real"],
Cell[113989, 4468, 200, 7, 27, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[114226, 4480, 102, 3, 28, "Input",
 CellTags->"B0Real"],
Cell[114331, 4485, 282, 9, 27, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[114628, 4497, 92, 3, 25, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[114745, 4504, 181, 5, 28, "Input",
 CellTags->"B0Real"],
Cell[114929, 4511, 200, 7, 27, "Output",
 CellTags->"B0Real"]
}, Open  ]]
}, Open  ]],
Cell[115156, 4522, 76, 2, 41, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[115257, 4528, 59, 1, 36, "Subsubsection",
 CellTags->"B0Unique"],
Cell[115319, 4531, 184, 4, 25, "Text",
 CellTags->"B0Unique"],
Cell[115506, 4537, 155, 8, 25, "Text",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[115698, 4550, 56, 1, 36, "Subsubsection",
 CellTags->"B0Unique"],
Cell[115757, 4553, 73, 1, 25, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[115855, 4558, 135, 4, 28, "Input",
 CellTags->"B0Unique"],
Cell[115993, 4564, 202, 7, 27, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116232, 4576, 104, 3, 28, "Input",
 CellTags->"B0Unique"],
Cell[116339, 4581, 284, 9, 27, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[116638, 4593, 88, 3, 25, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[116751, 4600, 186, 5, 28, "Input",
 CellTags->"B0Unique"],
Cell[116940, 4607, 177, 6, 27, "Output",
 CellTags->"B0Unique"]
}, Open  ]]
}, Open  ]],
Cell[117144, 4617, 43, 1, 41, "ObjectName",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[117212, 4622, 54, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[117269, 4625, 290, 8, 43, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[117584, 4637, 104, 3, 28, "Input",
 CellTags->"B00"],
Cell[117691, 4642, 162, 6, 27, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[117868, 4651, 346, 19, 25, "Text",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118251, 4675, 51, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[118305, 4678, 202, 7, 25, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[118532, 4689, 97, 3, 28, "Input",
 CellTags->"B00"],
Cell[118632, 4694, 156, 6, 27, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118825, 4705, 211, 7, 28, "Input",
 CellTags->"B00"],
Cell[119039, 4714, 2678, 91, 79, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[121754, 4810, 211, 7, 28, "Input",
 CellTags->"B00"],
Cell[121968, 4819, 1452, 54, 40, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123457, 4878, 261, 8, 28, "Input",
 CellTags->"B00"],
Cell[123721, 4888, 516, 22, 27, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[124274, 4915, 181, 6, 28, "Input",
 CellTags->"B00"],
Cell[124458, 4923, 614, 22, 40, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[125109, 4950, 248, 8, 28, "Input",
 CellTags->"B00"],
Cell[125360, 4960, 641, 23, 40, "Output",
 CellTags->"B00"]
}, Open  ]]
}, Open  ]],
Cell[126028, 4987, 41, 1, 41, "ObjectName",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[126094, 4992, 53, 1, 36, "Subsubsection",
 CellTags->"B1"],
Cell[126150, 4995, 230, 7, 25, "Text",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[126405, 5006, 102, 3, 28, "Input",
 CellTags->"B1"],
Cell[126510, 5011, 161, 6, 27, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[126686, 5020, 559, 29, 25, "Text",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[127282, 5054, 50, 1, 36, "Subsubsection",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[127357, 5059, 209, 7, 28, "Input",
 CellTags->"B1"],
Cell[127569, 5068, 1153, 42, 44, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128759, 5115, 259, 8, 28, "Input",
 CellTags->"B1"],
Cell[129021, 5125, 316, 12, 27, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[129352, 5140, 187, 6, 28, "Input",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[129564, 5150, 209, 7, 28, "Input",
 CellTags->"B1"],
Cell[129776, 5159, 390, 15, 38, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[130203, 5179, 179, 6, 28, "Input",
 CellTags->"B1"],
Cell[130385, 5187, 403, 14, 38, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[130825, 5206, 154, 5, 28, "Input",
 CellTags->"B1"],
Cell[130982, 5213, 403, 14, 38, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[131422, 5232, 238, 7, 28, "Input",
 CellTags->"B1"],
Cell[131663, 5241, 791, 25, 43, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[132491, 5271, 279, 8, 28, "Input",
 CellTags->"B1"],
Cell[132773, 5281, 419, 14, 38, "Output",
 CellTags->"B1"]
}, Open  ]]
}, Open  ]],
Cell[133219, 5299, 43, 1, 57, "ObjectName",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[133287, 5304, 54, 1, 52, "Subsubsection",
 CellTags->"B11"],
Cell[133344, 5307, 495, 17, 43, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[133864, 5328, 104, 3, 28, "Input",
 CellTags->"B11"],
Cell[133971, 5333, 162, 6, 27, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[134148, 5342, 441, 24, 41, "Text",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[134626, 5371, 51, 1, 52, "Subsubsection",
 CellTags->"B11"],
Cell[134680, 5374, 215, 7, 25, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[134920, 5385, 97, 3, 28, "Input",
 CellTags->"B11"],
Cell[135020, 5390, 156, 6, 27, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135213, 5401, 212, 7, 28, "Input",
 CellTags->"B11"],
Cell[135428, 5410, 2677, 94, 52, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138142, 5509, 212, 7, 28, "Input",
 CellTags->"B11"],
Cell[138357, 5518, 1206, 47, 46, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[139600, 5570, 171, 5, 28, "Input",
 CellTags->"B11"],
Cell[139774, 5577, 164, 6, 27, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[139975, 5588, 212, 7, 28, "Input",
 CellTags->"B11"],
Cell[140190, 5597, 517, 22, 27, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140744, 5624, 170, 5, 28, "Input",
 CellTags->"B11"],
Cell[140917, 5631, 163, 6, 27, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[141117, 5642, 182, 6, 28, "Input",
 CellTags->"B11"],
Cell[141302, 5650, 269, 9, 38, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[141608, 5664, 249, 8, 28, "Input",
 CellTags->"B11"],
Cell[141860, 5674, 295, 10, 38, "Output",
 CellTags->"B11"]
}, Open  ]]
}, Open  ]],
Cell[142182, 5688, 41, 1, 57, "ObjectName",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[142248, 5693, 53, 1, 52, "Subsubsection",
 CellTags->"CA"],
Cell[142304, 5696, 198, 9, 41, "Text",
 CellTags->"CA"],
Cell[142505, 5707, 274, 14, 41, "Text",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142816, 5726, 50, 1, 36, "Subsubsection",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[142891, 5731, 70, 2, 28, "Input",
 CellTags->"CA"],
Cell[142964, 5735, 120, 4, 27, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[143121, 5744, 173, 5, 28, "Input",
 CellTags->"CA"],
Cell[143297, 5751, 98, 3, 27, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[143432, 5759, 72, 2, 28, "Input",
 CellTags->"CA"],
Cell[143507, 5763, 98, 3, 27, "Output",
 CellTags->"CA"]
}, Open  ]]
}, Open  ]],
Cell[143632, 5770, 45, 1, 41, "ObjectName",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[143702, 5775, 55, 1, 52, "Subsubsection",
 CellTags->"Calc"],
Cell[143760, 5778, 233, 5, 43, "Text",
 CellTags->"Calc"],
Cell[143996, 5785, 407, 19, 25, "Text",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[144440, 5809, 61, 1, 35, "Subsubsection",
 CellTags->"Calc"],
Cell[144504, 5812, 346, 13, 25, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[144875, 5829, 284, 9, 28, "Input",
 CellTags->"Calc"],
Cell[145162, 5840, 147, 5, 27, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[145324, 5848, 213, 9, 25, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[145562, 5861, 258, 8, 28, "Input",
 CellTags->"Calc"],
Cell[145823, 5871, 352, 15, 27, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146212, 5891, 328, 11, 28, "Input",
 CellTags->"Calc"],
Cell[146543, 5904, 424, 16, 27, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[147004, 5925, 101, 3, 28, "Input",
 CellTags->"Calc"],
Cell[147108, 5930, 547, 23, 27, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[147692, 5958, 215, 6, 28, "Input",
 CellTags->"Calc"],
Cell[147910, 5966, 985, 44, 27, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148932, 6015, 382, 13, 28, "Input",
 CellTags->"Calc"],
Cell[149317, 6030, 100, 3, 27, "Output",
 CellTags->"Calc"]
}, Open  ]]
}, Open  ]],
Cell[149444, 6037, 67, 1, 41, "ObjectName",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[149536, 6042, 66, 1, 36, "Subsubsection",
 CellTags->"CalcColorFactor"],
Cell[149605, 6045, 470, 9, 113, "Text",
 CellTags->"CalcColorFactor"],
Cell[150078, 6056, 195, 9, 25, "Text",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[150298, 6069, 250, 7, 28, "Input"],
Cell[150551, 6078, 333, 14, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150921, 6097, 99, 2, 28, "Input"],
Cell[151023, 6101, 1663, 55, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[152723, 6161, 118, 3, 28, "Input"],
Cell[152844, 6166, 2279, 77, 55, "Output"]
}, Open  ]],
Cell[155138, 6246, 411, 18, 25, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[155564, 6267, 94, 1, 41, "ObjectName",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[155683, 6272, 71, 1, 36, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[155757, 6275, 110, 1, 25, "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[155904, 6281, 77, 1, 35, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[156006, 6286, 123, 3, 28, "Input",
 CellTags->"CalculateCounterTerm"],
Cell[156132, 6291, 191, 3, 40, "Print",
 CellTags->{"CalculateCounterTerm", "Info3387711865-3148101"}]
}, Open  ]]
}, Open  ]],
Cell[156350, 6298, 53, 1, 41, "ObjectName",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[156428, 6303, 59, 1, 36, "Subsubsection",
 CellTags->"CancelQP"],
Cell[156490, 6306, 314, 13, 25, "Text",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156841, 6324, 65, 1, 35, "Subsubsection",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[156931, 6329, 549, 19, 28, "Input",
 CellTags->"CancelQP"],
Cell[157483, 6350, 667, 24, 50, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158187, 6379, 182, 5, 28, "Input",
 CellTags->"CancelQP"],
Cell[158372, 6386, 2183, 73, 43, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160592, 6464, 183, 5, 28, "Input",
 CellTags->"CancelQP"],
Cell[160778, 6471, 6388, 208, 166, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167203, 6684, 555, 18, 28, "Input",
 CellTags->"CancelQP"],
Cell[167761, 6704, 886, 33, 45, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[168684, 6742, 182, 5, 28, "Input",
 CellTags->"CancelQP"],
Cell[168869, 6749, 6250, 218, 88, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[175156, 6972, 183, 5, 28, "Input",
 CellTags->"CancelQP"],
Cell[175342, 6979, 7177, 246, 88, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[182534, 7228, 106, 3, 28, "Input",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[182655, 7234, 49, 1, 41, "ObjectName",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[182729, 7239, 57, 1, 52, "Subsubsection",
 CellTags->"Cases2"],
Cell[182789, 7242, 293, 6, 59, "Text",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[183119, 7253, 63, 1, 35, "Subsubsection",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[183207, 7258, 305, 10, 28, "Input",
 CellTags->"Cases2"],
Cell[183515, 7270, 275, 9, 27, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[183827, 7284, 312, 10, 28, "Input",
 CellTags->"Cases2"],
Cell[184142, 7296, 279, 9, 27, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[184458, 7310, 427, 14, 28, "Input",
 CellTags->"Cases2"],
Cell[184888, 7326, 355, 11, 27, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[185280, 7342, 327, 10, 28, "Input",
 CellTags->"Cases2"],
Cell[185610, 7354, 463, 17, 27, "Output",
 CellTags->"Cases2"]
}, Open  ]]
}, Open  ]],
Cell[186100, 7375, 41, 1, 57, "ObjectName",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[186166, 7380, 53, 1, 52, "Subsubsection",
 CellTags->"CF"],
Cell[186222, 7383, 287, 13, 41, "Text",
 CellTags->"CF"],
Cell[186512, 7398, 274, 14, 41, "Text",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186823, 7417, 50, 1, 36, "Subsubsection",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[186898, 7422, 70, 2, 28, "Input",
 CellTags->"CF"],
Cell[186971, 7426, 120, 4, 27, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187128, 7435, 178, 5, 28, "Input",
 CellTags->"CF"],
Cell[187309, 7442, 193, 7, 40, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187539, 7454, 83, 2, 28, "Input"],
Cell[187625, 7458, 76, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187738, 7464, 72, 2, 28, "Input",
 CellTags->"CF"],
Cell[187813, 7468, 98, 3, 27, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187948, 7476, 233, 7, 28, "Input",
 CellTags->"CF"],
Cell[188184, 7485, 177, 6, 27, "Output",
 CellTags->"CF"]
}, Open  ]]
}, Open  ]],
Cell[188388, 7495, 67, 1, 57, "ObjectName",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[188480, 7500, 66, 1, 52, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[188549, 7503, 210, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[188762, 7510, 519, 24, 41, "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[189318, 7539, 63, 1, 52, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[189384, 7542, 219, 5, 43, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[189628, 7551, 212, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[189843, 7559, 198, 7, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190078, 7571, 151, 4, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[190232, 7577, 198, 7, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190467, 7589, 116, 3, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[190586, 7594, 272, 8, 42, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[190873, 7605, 119, 3, 25, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[191017, 7612, 201, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[191221, 7620, 222, 6, 42, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[191458, 7629, 105, 1, 25, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[191588, 7634, 482, 15, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[192073, 7651, 571, 18, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[192681, 7674, 153, 4, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[192837, 7680, 755, 29, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193629, 7714, 222, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[193854, 7722, 755, 29, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[194624, 7754, 134, 4, 25, "Text",
 CellTags->"ChangeDimension"],
Cell[194761, 7760, 208, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[194972, 7768, 219, 5, 43, "Text",
 CellTags->"ChangeDimension"],
Cell[195194, 7775, 201, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[195420, 7785, 220, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[195643, 7793, 489, 21, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196169, 7819, 182, 5, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[196354, 7826, 489, 21, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196880, 7852, 176, 5, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[197059, 7859, 301, 10, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[197397, 7874, 141, 4, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[197541, 7880, 130, 4, 27, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[197686, 7887, 201, 6, 28, "Input",
 CellTags->"ChangeDimension"],
Cell[197890, 7895, 157, 5, 28, "Input",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[198062, 7903, 83, 1, 57, "ObjectName",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[198170, 7908, 74, 1, 52, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[198247, 7911, 176, 6, 41, "Text",
 CellTags->"ChargeConjugationMatrix"],
Cell[198426, 7919, 248, 9, 41, "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[198711, 7933, 71, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[198807, 7938, 214, 5, 28, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[199024, 7945, 306, 11, 27, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199367, 7961, 120, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[199490, 7966, 255, 9, 27, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199782, 7980, 209, 5, 28, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[199994, 7987, 228, 7, 27, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[200259, 7999, 120, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[200382, 8004, 192, 6, 27, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[200611, 8015, 134, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[200748, 8020, 137, 4, 27, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]]
}, Open  ]],
Cell[200912, 8028, 89, 1, 57, "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[201026, 8033, 77, 1, 52, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[201106, 8036, 195, 4, 41, "Text",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[201304, 8042, 242, 9, 41, "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201583, 8056, 74, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[201682, 8061, 217, 5, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[201902, 8068, 309, 11, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202248, 8084, 123, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[202374, 8089, 258, 9, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202669, 8103, 212, 5, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[202884, 8110, 231, 7, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203152, 8122, 123, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[203278, 8127, 195, 6, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203510, 8138, 165, 4, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[203678, 8144, 189, 6, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203904, 8155, 119, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[204026, 8160, 164, 5, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204227, 8170, 140, 3, 28, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[204370, 8175, 140, 4, 27, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]]
}, Open  ]],
Cell[204537, 8183, 61, 1, 57, "ObjectName",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[204623, 8188, 63, 1, 52, "Subsubsection",
 CellTags->"CheckContext"],
Cell[204689, 8191, 164, 4, 41, "Text",
 CellTags->"CheckContext"],
Cell[204856, 8197, 320, 14, 41, "Text",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205213, 8216, 60, 1, 52, "Subsubsection",
 CellTags->"CheckContext"],
Cell[205276, 8219, 142, 4, 25, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[205443, 8227, 138, 3, 28, "Input",
 CellTags->"CheckContext"],
Cell[205584, 8232, 112, 3, 27, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[205711, 8238, 87, 1, 25, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[205823, 8243, 92, 2, 28, "Input",
 CellTags->"CheckContext"],
Cell[205918, 8247, 121, 3, 27, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206076, 8255, 138, 3, 28, "Input",
 CellTags->"CheckContext"],
Cell[206217, 8260, 111, 3, 27, "Output",
 CellTags->"CheckContext"]
}, Open  ]]
}, Open  ]],
Cell[206355, 8267, 51, 1, 57, "ObjectName",
 CellTags->"CheckDB"],
Cell[CellGroupData[{
Cell[206431, 8272, 106, 3, 52, "Subsubsection",
 CellTags->"CheckDB"],
Cell[206540, 8277, 1081, 15, 151, "Text"],
Cell[CellGroupData[{
Cell[207646, 8296, 113, 3, 28, "Input",
 CellTags->"CheckDB"],
Cell[207762, 8301, 471, 14, 27, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208270, 8320, 116, 3, 28, "Input",
 CellTags->"CheckDB"],
Cell[208389, 8325, 134, 4, 27, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[208538, 8332, 115, 3, 25, "Text"],
Cell[CellGroupData[{
Cell[208678, 8339, 337, 11, 28, "Input"],
Cell[209018, 8352, 178, 5, 23, "Print"],
Cell[209199, 8359, 178, 5, 23, "Print"],
Cell[209380, 8366, 128, 4, 27, "Output"]
}, Open  ]],
Cell[209523, 8373, 133, 3, 25, "Text"],
Cell[CellGroupData[{
Cell[209681, 8380, 383, 13, 28, "Input",
 CellTags->"CheckDB"],
Cell[210067, 8395, 150, 5, 27, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[210232, 8403, 60, 0, 25, "Text"],
Cell[CellGroupData[{
Cell[210317, 8407, 303, 9, 28, "Input",
 CellTags->"CheckDB"],
Cell[210623, 8418, 119, 3, 49, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[210757, 8424, 268, 8, 28, "Input",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[211040, 8435, 73, 1, 57, "ObjectName",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[211138, 8440, 69, 1, 52, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[211210, 8443, 499, 19, 41, "Text",
 CellTags->"ChiralityProjector"],
Cell[211712, 8464, 409, 19, 41, "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[212158, 8488, 66, 1, 36, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[212249, 8493, 262, 8, 28, "Input",
 CellTags->"ChiralityProjector"],
Cell[212514, 8503, 227, 7, 27, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[212778, 8515, 114, 3, 28, "Input",
 CellTags->"ChiralityProjector"],
Cell[212895, 8520, 227, 7, 27, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[213159, 8532, 196, 5, 28, "Input",
 CellTags->"ChiralityProjector"],
Cell[213358, 8539, 380, 14, 42, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]]
}, Open  ]],
Cell[213765, 8557, 53, 1, 57, "ObjectName",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[213843, 8562, 59, 1, 52, "Subsubsection",
 CellTags->"Chisholm"],
Cell[213905, 8565, 142, 4, 41, "Text",
 CellTags->"Chisholm"],
Cell[214050, 8571, 316, 14, 41, "Text",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[214403, 8590, 56, 1, 36, "Subsubsection",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[214484, 8595, 206, 6, 28, "Input",
 CellTags->"Chisholm"],
Cell[214693, 8603, 1988, 78, 28, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216718, 8686, 183, 5, 28, "Input",
 CellTags->"Chisholm"],
Cell[216904, 8693, 236, 7, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[217177, 8705, 139, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[217319, 8711, 1372, 54, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[218706, 8768, 77, 1, 25, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[218808, 8773, 130, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[218941, 8779, 105, 3, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219083, 8787, 139, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[219225, 8793, 1372, 54, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220634, 8852, 130, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[220767, 8858, 105, 3, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220909, 8866, 169, 5, 28, "Input",
 CellTags->"Chisholm"],
Cell[221081, 8873, 590, 22, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[221708, 8900, 110, 3, 28, "Input",
 CellTags->"Chisholm"],
Cell[221821, 8905, 1457, 58, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[223315, 8968, 232, 7, 28, "Input",
 CellTags->"Chisholm"],
Cell[223550, 8977, 391, 10, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[223978, 8992, 139, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[224120, 8998, 11506, 439, 93, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[235663, 9442, 139, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[235805, 9448, 11506, 439, 93, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[247326, 9890, 85, 1, 25, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[247436, 9895, 130, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[247569, 9901, 125, 4, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[247731, 9910, 130, 4, 28, "Input",
 CellTags->"Chisholm"],
Cell[247864, 9916, 125, 4, 27, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[248004, 9923, 192, 6, 28, "Input",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[248211, 9932, 65, 1, 57, "ObjectName",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[248301, 9937, 65, 1, 52, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[248369, 9940, 296, 6, 59, "Text",
 CellTags->"ChisholmSpinor"],
Cell[248668, 9948, 304, 14, 41, "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249009, 9967, 62, 1, 36, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[249096, 9972, 312, 9, 28, "Input",
 CellTags->"ChisholmSpinor"],
Cell[249411, 9983, 427, 15, 27, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249875, 10003, 117, 3, 28, "Input",
 CellTags->"ChisholmSpinor"],
Cell[249995, 10008, 2722, 100, 85, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]]
}, Open  ]],
Cell[252744, 10112, 75, 1, 57, "ObjectName",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[252844, 10117, 70, 1, 52, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[252917, 10120, 145, 4, 41, "Text",
 CellTags->"ClearScalarProducts"],
Cell[253065, 10126, 484, 23, 41, "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[253586, 10154, 67, 1, 36, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[253678, 10159, 195, 6, 28, "Input",
 CellTags->"ClearScalarProducts"],
Cell[253876, 10167, 139, 4, 27, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[254052, 10176, 211, 6, 28, "Input",
 CellTags->"ClearScalarProducts"],
Cell[254266, 10184, 139, 4, 27, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[254420, 10191, 104, 2, 28, "Input",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[254549, 10197, 211, 6, 28, "Input",
 CellTags->"ClearScalarProducts"],
Cell[254763, 10205, 173, 6, 27, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[254973, 10216, 148, 4, 28, "Input",
 CellTags->"ClearScalarProducts"],
Cell[255124, 10222, 173, 6, 27, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]]
}, Open  ]],
Cell[255324, 10232, 57, 1, 57, "ObjectName",
 CellTags->"Collecting"],
Cell[CellGroupData[{
Cell[255406, 10237, 61, 1, 52, "Subsubsection",
 CellTags->"Collecting"],
Cell[255470, 10240, 207, 5, 41, "Text",
 CellTags->"Collecting"],
Cell[255680, 10247, 411, 18, 41, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[256106, 10268, 53, 1, 57, "ObjectName",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[256184, 10273, 59, 1, 52, "Subsubsection",
 CellTags->"Collect2"],
Cell[256246, 10276, 613, 10, 97, "Text",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[256884, 10290, 115, 3, 28, "Input",
 CellTags->"Collect2"],
Cell[257002, 10295, 399, 11, 27, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[257416, 10309, 176, 9, 41, "Text",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257629, 10323, 56, 1, 36, "Subsubsection",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[257710, 10328, 440, 14, 37, "Input",
 CellTags->"Collect2"],
Cell[258153, 10344, 451, 16, 38, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[258641, 10365, 185, 5, 28, "Input",
 CellTags->"Collect2"],
Cell[258829, 10372, 399, 14, 37, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[259265, 10391, 186, 5, 28, "Input",
 CellTags->"Collect2"],
Cell[259454, 10398, 451, 16, 38, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[259942, 10419, 525, 18, 28, "Input",
 CellTags->"Collect2"],
Cell[260470, 10439, 514, 17, 27, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[261021, 10461, 320, 10, 29, "Input",
 CellTags->"Collect2"],
Cell[261344, 10473, 815, 31, 28, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[262196, 10509, 112, 3, 28, "Input",
 CellTags->"Collect2"],
Cell[262311, 10514, 825, 27, 28, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[263151, 10544, 130, 4, 28, "Input",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[263306, 10552, 305, 9, 28, "Input",
 CellTags->"Collect2"],
Cell[263614, 10563, 935, 31, 27, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[264586, 10599, 149, 4, 28, "Input",
 CellTags->"Collect2"],
Cell[264738, 10605, 671, 21, 27, "Output",
 CellTags->"Collect2"]
}, Open  ]]
}, Open  ]],
Cell[265436, 10630, 53, 1, 41, "ObjectName",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[265514, 10635, 59, 1, 36, "Subsubsection",
 CellTags->"Collect3"],
Cell[265576, 10638, 719, 23, 61, "Text",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[266320, 10665, 115, 3, 28, "Input",
 CellTags->"Collect3"],
Cell[266438, 10670, 233, 8, 27, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[266686, 10681, 286, 14, 25, "Text",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[267009, 10700, 56, 1, 36, "Subsubsection",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[267090, 10705, 525, 18, 28, "Input",
 CellTags->"Collect3"],
Cell[267618, 10725, 704, 22, 27, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[268359, 10752, 289, 9, 29, "Input",
 CellTags->"Collect3"],
Cell[268651, 10763, 5476, 142, 137, "Output",
 CellTags->"Collect3"]
}, Open  ]]
}, Open  ]],
Cell[274154, 10909, 51, 1, 41, "ObjectName",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[274230, 10914, 58, 1, 36, "Subsubsection",
 CellTags->"Combine"],
Cell[274291, 10917, 315, 6, 61, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[274631, 10927, 113, 3, 28, "Input",
 CellTags->"Combine"],
Cell[274747, 10932, 171, 6, 27, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[274933, 10941, 175, 9, 25, "Text",
 CellTags->"Combine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[275145, 10955, 55, 1, 36, "Subsubsection",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[275225, 10960, 283, 10, 40, "Input",
 CellTags->"Combine"],
Cell[275511, 10972, 304, 11, 40, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[275830, 10986, 122, 3, 25, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[275977, 10993, 284, 10, 40, "Input",
 CellTags->"Combine"],
Cell[276264, 11005, 303, 10, 40, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[276582, 11018, 175, 4, 25, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[276782, 11026, 354, 12, 40, "Input",
 CellTags->"Combine"],
Cell[277139, 11040, 303, 10, 40, "Output",
 CellTags->"Combine"]
}, Open  ]]
}, Open  ]],
Cell[277469, 11054, 63, 1, 41, "ObjectName",
 CellTags->"CombineGraphs"],
Cell[CellGroupData[{
Cell[277557, 11059, 64, 1, 36, "Subsubsection",
 CellTags->"CombineGraphs"],
Cell[277624, 11062, 86, 1, 25, "Text",
 CellTags->"CombineGraphs"],
Cell[277713, 11065, 185, 8, 25, "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],
Cell[277913, 11076, 57, 1, 41, "ObjectName",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[277995, 11081, 61, 1, 36, "Subsubsection",
 CellTags->"Commutator"],
Cell[278059, 11084, 138, 4, 25, "Text",
 CellTags->"Commutator"],
Cell[278200, 11090, 607, 24, 25, "Text",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[278844, 11119, 58, 1, 36, "Subsubsection",
 CellTags->"Commutator"],
Cell[278905, 11122, 167, 4, 28, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[279097, 11130, 136, 4, 28, "Input",
 CellTags->"Commutator"],
Cell[279236, 11136, 184, 5, 27, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[279457, 11146, 121, 3, 28, "Input",
 CellTags->"Commutator"],
Cell[279581, 11151, 177, 6, 27, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[279795, 11162, 247, 8, 28, "Input",
 CellTags->"Commutator"],
Cell[280045, 11172, 381, 12, 27, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[280441, 11187, 169, 4, 28, "Input",
 CellTags->"Commutator"],
Cell[280613, 11193, 68, 1, 25, "Text",
 CellTags->"Commutator"],
Cell[280684, 11196, 228, 6, 28, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[280937, 11206, 463, 15, 28, "Input",
 CellTags->"Commutator"],
Cell[281403, 11223, 727, 19, 27, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[282167, 11247, 114, 3, 28, "Input",
 CellTags->"Commutator"],
Cell[282284, 11252, 106, 3, 27, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[282405, 11258, 113, 3, 28, "Input",
 CellTags->"Commutator"],
Cell[282521, 11263, 159, 4, 28, "Input",
 CellTags->"Commutator"]
}, Open  ]],
Cell[282695, 11270, 73, 1, 41, "ObjectName",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[282793, 11275, 69, 1, 36, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[282865, 11278, 157, 4, 25, "Text",
 CellTags->"CommutatorExplicit"],
Cell[283025, 11284, 296, 14, 25, "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[283358, 11303, 66, 1, 36, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[283427, 11306, 175, 4, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[283627, 11314, 144, 4, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[283774, 11320, 192, 5, 27, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[284003, 11330, 129, 3, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[284135, 11335, 185, 6, 27, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[284357, 11346, 196, 6, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[284556, 11354, 283, 10, 27, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[284876, 11369, 129, 3, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[285008, 11374, 389, 14, 27, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[285434, 11393, 167, 5, 28, "Input",
 CellTags->"CommutatorExplicit"],
Cell[285604, 11400, 389, 12, 27, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[286008, 11415, 177, 4, 28, "Input",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[286200, 11422, 65, 1, 57, "ObjectName",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[286290, 11427, 65, 1, 52, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[286358, 11430, 1107, 82, 59, "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[287502, 11517, 62, 1, 36, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[287589, 11522, 277, 10, 28, "Input"],
Cell[287869, 11534, 337, 12, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[288243, 11551, 170, 5, 28, "Input",
 CellTags->"CompleteSquare"],
Cell[288416, 11558, 685, 27, 42, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[289138, 11590, 180, 6, 28, "Input",
 CellTags->"CompleteSquare"],
Cell[289321, 11598, 110, 3, 27, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[289468, 11606, 155, 4, 28, "Input",
 CellTags->"CompleteSquare"],
Cell[289626, 11612, 515, 20, 42, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[290156, 11635, 137, 4, 28, "Input",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[290308, 11642, 69, 1, 57, "ObjectName",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[290402, 11647, 67, 1, 52, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[290472, 11650, 414, 7, 77, "Text",
 CellTags->"ComplexConjugate"],
Cell[290889, 11659, 449, 19, 41, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[291375, 11683, 64, 1, 36, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[291464, 11688, 196, 5, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[291663, 11695, 163, 5, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[291863, 11705, 121, 3, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[291987, 11710, 239, 6, 42, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[292263, 11721, 152, 4, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[292418, 11727, 249, 7, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[292704, 11739, 125, 3, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[292832, 11744, 427, 16, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[293296, 11765, 182, 5, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[293481, 11772, 531, 22, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[294049, 11799, 125, 3, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[294177, 11804, 531, 22, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[294745, 11831, 190, 5, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[294938, 11838, 380, 17, 27, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[295355, 11860, 270, 7, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[295628, 11869, 157, 4, 42, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[295822, 11878, 273, 8, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[296098, 11888, 335, 10, 28, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296470, 11903, 125, 3, 28, "Input",
 CellTags->"ComplexConjugate"],
Cell[296598, 11908, 431, 15, 28, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[297044, 11926, 166, 4, 41, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[297225, 11933, 53, 1, 57, "ObjectName",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[297303, 11938, 59, 1, 52, "Subsubsection",
 CellTags->"Contract"],
Cell[297365, 11941, 411, 7, 61, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[297801, 11952, 115, 3, 28, "Input",
 CellTags->"Contract"],
Cell[297919, 11957, 612, 15, 49, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[298546, 11975, 138, 3, 25, "Text"],
Cell[298687, 11980, 423, 19, 41, "Text",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[299147, 12004, 56, 1, 36, "Subsubsection",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[299228, 12009, 233, 7, 28, "Input",
 CellTags->"Contract"],
Cell[299464, 12018, 319, 13, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[299820, 12036, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[299932, 12041, 236, 10, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[300205, 12056, 199, 6, 28, "Input",
 CellTags->"Contract"],
Cell[300407, 12064, 376, 16, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[300820, 12085, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[300932, 12090, 221, 8, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[301190, 12103, 144, 4, 28, "Input",
 CellTags->"Contract"],
Cell[301337, 12109, 155, 5, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[301507, 12117, 83, 1, 25, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[301615, 12122, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[301727, 12127, 104, 3, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[301846, 12133, 208, 5, 43, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[302079, 12142, 231, 7, 28, "Input",
 CellTags->"Contract"],
Cell[302313, 12151, 233, 8, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[302583, 12164, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[302695, 12169, 104, 3, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[302836, 12177, 231, 7, 28, "Input",
 CellTags->"Contract"],
Cell[303070, 12186, 233, 8, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[303340, 12199, 227, 7, 28, "Input",
 CellTags->"Contract"],
Cell[303570, 12208, 405, 18, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304012, 12231, 114, 3, 28, "Input",
 CellTags->"Contract"],
Cell[304129, 12236, 206, 8, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304372, 12249, 277, 9, 28, "Input",
 CellTags->"Contract"],
Cell[304652, 12260, 513, 22, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[305202, 12287, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[305314, 12292, 575, 24, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[305926, 12321, 271, 8, 28, "Input",
 CellTags->"Contract"],
Cell[306200, 12331, 667, 29, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[306904, 12365, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[307016, 12370, 439, 19, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[307492, 12394, 313, 9, 28, "Input",
 CellTags->"Contract"],
Cell[307808, 12405, 922, 40, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308767, 12450, 109, 3, 28, "Input",
 CellTags->"Contract"],
Cell[308879, 12455, 204, 7, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[309120, 12467, 426, 13, 28, "Input",
 CellTags->"Contract"],
Cell[309549, 12482, 301, 9, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[309887, 12496, 139, 4, 28, "Input",
 CellTags->"Contract"],
Cell[310029, 12502, 352, 12, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[310396, 12517, 194, 6, 28, "Input",
 CellTags->"Contract"]
}, Open  ]],
Cell[310605, 12526, 55, 1, 57, "ObjectName",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[310685, 12531, 60, 1, 52, "Subsubsection",
 CellTags->"Contract1"],
Cell[310748, 12534, 459, 8, 119, "Text",
 CellTags->"Contract1"],
Cell[311210, 12544, 389, 18, 41, "Text",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311636, 12567, 57, 1, 36, "Subsubsection",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[311718, 12572, 349, 12, 28, "Input",
 CellTags->"Contract1"],
Cell[312070, 12586, 207, 8, 27, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312314, 12599, 349, 12, 28, "Input",
 CellTags->"Contract1"],
Cell[312666, 12613, 478, 20, 27, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[313181, 12638, 269, 9, 28, "Input",
 CellTags->"Contract1"],
Cell[313453, 12649, 105, 3, 27, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[313595, 12657, 269, 9, 28, "Input",
 CellTags->"Contract1"],
Cell[313867, 12668, 298, 11, 27, "Output",
 CellTags->"Contract1"]
}, Open  ]]
}, Open  ]],
Cell[314192, 12683, 55, 1, 57, "ObjectName",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[314272, 12688, 60, 1, 52, "Subsubsection",
 CellTags->"Convolute"],
Cell[314335, 12691, 1367, 50, 65, "Text",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[315727, 12745, 117, 3, 28, "Input",
 CellTags->"Convolute"],
Cell[315847, 12750, 400, 13, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[316262, 12766, 332, 14, 41, "Text",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[316631, 12785, 57, 1, 36, "Subsubsection",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[316713, 12790, 134, 4, 28, "Input",
 CellTags->"Convolute"],
Cell[316850, 12796, 154, 5, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317041, 12806, 134, 4, 28, "Input",
 CellTags->"Convolute"],
Cell[317178, 12812, 178, 6, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317393, 12823, 134, 4, 28, "Input",
 CellTags->"Convolute"],
Cell[317530, 12829, 128, 4, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317695, 12838, 181, 6, 39, "Input",
 CellTags->"Convolute"],
Cell[317879, 12846, 215, 7, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318131, 12858, 227, 7, 39, "Input",
 CellTags->"Convolute"],
Cell[318361, 12867, 215, 7, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318613, 12879, 181, 6, 39, "Input",
 CellTags->"Convolute"],
Cell[318797, 12887, 289, 10, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[319123, 12902, 228, 8, 39, "Input",
 CellTags->"Convolute"],
Cell[319354, 12912, 532, 19, 40, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[319923, 12936, 190, 6, 28, "Input",
 CellTags->"Convolute"],
Cell[320116, 12944, 431, 16, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[320584, 12965, 216, 7, 28, "Input",
 CellTags->"Convolute"],
Cell[320803, 12974, 273, 10, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321113, 12989, 237, 8, 39, "Input",
 CellTags->"Convolute"],
Cell[321353, 12999, 369, 12, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321759, 13016, 263, 9, 39, "Input",
 CellTags->"Convolute"],
Cell[322025, 13027, 597, 20, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322659, 13052, 239, 8, 39, "Input",
 CellTags->"Convolute"],
Cell[322901, 13062, 811, 28, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[323749, 13095, 190, 6, 28, "Input",
 CellTags->"Convolute"],
Cell[323942, 13103, 218, 8, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324197, 13116, 190, 6, 28, "Input",
 CellTags->"Convolute"],
Cell[324390, 13124, 317, 11, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324744, 13140, 260, 9, 39, "Input",
 CellTags->"Convolute"],
Cell[325007, 13151, 452, 16, 45, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325496, 13172, 165, 5, 28, "Input",
 CellTags->"Convolute"],
Cell[325664, 13179, 229, 9, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325930, 13193, 195, 6, 28, "Input",
 CellTags->"Convolute"],
Cell[326128, 13201, 239, 9, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[326404, 13215, 212, 7, 39, "Input",
 CellTags->"Convolute"],
Cell[326619, 13224, 546, 20, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[327202, 13249, 213, 7, 39, "Input",
 CellTags->"Convolute"],
Cell[327418, 13258, 422, 16, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[327877, 13279, 237, 8, 39, "Input",
 CellTags->"Convolute"],
Cell[328117, 13289, 685, 24, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[328839, 13318, 213, 7, 39, "Input",
 CellTags->"Convolute"],
Cell[329055, 13327, 526, 19, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329618, 13351, 190, 6, 28, "Input",
 CellTags->"Convolute"],
Cell[329811, 13359, 218, 8, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330066, 13372, 165, 5, 28, "Input",
 CellTags->"Convolute"],
Cell[330234, 13379, 188, 6, 27, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330459, 13390, 286, 10, 39, "Input",
 CellTags->"Convolute"],
Cell[330748, 13402, 986, 37, 45, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[331771, 13444, 295, 10, 39, "Input",
 CellTags->"Convolute"],
Cell[332069, 13456, 1733, 64, 61, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[333829, 13524, 66, 1, 57, "ObjectName",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[333920, 13529, 65, 1, 52, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[333988, 13532, 152, 4, 25, "Text",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[334165, 13540, 122, 3, 28, "Input",
 CellTags->"ConvoluteTable"],
Cell[334290, 13545, 405, 13, 27, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[334710, 13561, 322, 14, 41, "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335069, 13580, 62, 1, 36, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[335156, 13585, 144, 4, 28, "Input",
 CellTags->"ConvoluteTable"],
Cell[335303, 13591, 159, 5, 27, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335499, 13601, 144, 4, 28, "Input",
 CellTags->"ConvoluteTable"],
Cell[335646, 13607, 183, 6, 27, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335866, 13618, 144, 4, 28, "Input",
 CellTags->"ConvoluteTable"],
Cell[336013, 13624, 133, 4, 27, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[336183, 13633, 244, 8, 39, "Input",
 CellTags->"Convolute"],
Cell[336430, 13643, 551, 20, 38, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[337008, 13667, 85, 1, 57, "ObjectName",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[337118, 13672, 75, 1, 52, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[337196, 13675, 372, 6, 59, "Text",
 CellTags->"CovariantFieldDerivative"],
Cell[337571, 13683, 460, 19, 41, "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[338068, 13707, 72, 1, 36, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[338165, 13712, 329, 9, 28, "Input",
 CellTags->"CovariantFieldDerivative"],
Cell[338497, 13723, 543, 21, 27, "Output",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[339067, 13748, 53, 1, 57, "ObjectName",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[339145, 13753, 59, 1, 52, "Subsubsection",
 CellTags->"CounterT"],
Cell[339207, 13756, 146, 4, 41, "Text",
 CellTags->"CounterT"],
Cell[339356, 13762, 450, 19, 41, "Text",
 CellTags->"CounterT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339843, 13786, 56, 1, 36, "Subsubsection",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[339924, 13791, 284, 6, 28, "Input",
 CellTags->"CounterT"],
Cell[340211, 13799, 3127, 106, 54, "Output",
 CellTags->"CounterT"]
}, Open  ]]
}, Open  ]],
Cell[343365, 13909, 59, 1, 57, "ObjectName",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[343449, 13914, 62, 1, 36, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[343514, 13917, 216, 5, 43, "Text",
 CellTags->"CounterTerm"],
Cell[343733, 13924, 578, 24, 25, "Text",
 CellTags->"CounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[344348, 13953, 59, 1, 36, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[344432, 13958, 116, 3, 28, "Input",
 CellTags->"CounterTerm"],
Cell[344551, 13963, 1634, 53, 59, "Output",
 CellTags->"CounterTerm"]
}, Open  ]]
}, Open  ]],
Cell[346212, 14020, 69, 1, 41, "ObjectName",
 CellTags->"CouplingConstant"],
Cell[CellGroupData[{
Cell[346306, 14025, 67, 1, 36, "Subsubsection",
 CellTags->"CouplingConstant"],
Cell[346376, 14028, 657, 11, 105, "Text",
 CellTags->"CouplingConstant"],
Cell[347036, 14041, 318, 14, 25, "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],
Cell[347369, 14058, 57, 1, 41, "ObjectName",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[347451, 14063, 61, 1, 36, "Subsubsection",
 CellTags->"CovariantD"],
Cell[347515, 14066, 1212, 19, 229, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[348752, 14089, 119, 3, 28, "Input",
 CellTags->"CovariantD"],
Cell[348874, 14094, 439, 12, 27, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[349328, 14109, 185, 4, 43, "Text",
 CellTags->"CovariantD"],
Cell[349516, 14115, 455, 19, 25, "Text",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[350008, 14139, 58, 1, 36, "Subsubsection",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[350091, 14144, 117, 3, 28, "Input",
 CellTags->"CovariantD"],
Cell[350211, 14149, 132, 4, 27, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[350380, 14158, 150, 4, 28, "Input",
 CellTags->"CovariantD"],
Cell[350533, 14164, 161, 5, 27, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[350731, 14174, 180, 5, 28, "Input",
 CellTags->"CovariantD"],
Cell[350914, 14181, 992, 39, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[351921, 14223, 164, 4, 25, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[352110, 14231, 120, 3, 28, "Input",
 CellTags->"CovariantD"],
Cell[352233, 14236, 176, 6, 27, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[352446, 14247, 153, 4, 28, "Input",
 CellTags->"CovariantD"],
Cell[352602, 14253, 205, 7, 27, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[352844, 14265, 203, 5, 28, "Input",
 CellTags->"CovariantD"],
Cell[353050, 14272, 939, 36, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[354026, 14313, 183, 5, 28, "Input",
 CellTags->"CovariantD"],
Cell[354212, 14320, 932, 35, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[355181, 14360, 187, 5, 28, "Input",
 CellTags->"CovariantD"],
Cell[355371, 14367, 1339, 48, 36, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[356725, 14418, 602, 22, 31, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[357352, 14444, 215, 6, 28, "Input",
 CellTags->"CovariantD"],
Cell[357570, 14452, 513, 20, 34, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[358098, 14475, 285, 10, 25, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[358408, 14489, 215, 6, 28, "Input",
 CellTags->"CovariantD"],
Cell[358626, 14497, 2262, 80, 51, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[360903, 14580, 189, 7, 25, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[361117, 14591, 215, 6, 28, "Input",
 CellTags->"CovariantD"],
Cell[361335, 14599, 5963, 196, 54, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367335, 14800, 181, 5, 28, "Input",
 CellTags->"CovariantD"],
Cell[367519, 14807, 261, 9, 28, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367817, 14821, 195, 6, 28, "Input",
 CellTags->"CovariantD"],
Cell[368015, 14829, 4522, 146, 54, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[372574, 14980, 223, 7, 28, "Input",
 CellTags->"CovariantD"],
Cell[372800, 14989, 788, 24, 62, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373625, 15018, 240, 7, 28, "Input",
 CellTags->"CovariantD"],
Cell[373868, 15027, 803, 25, 62, "Output",
 CellTags->"CovariantD"]
}, Open  ]]
}, Open  ]],
Cell[374698, 15056, 61, 1, 57, "ObjectName",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[374784, 15061, 63, 1, 52, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[374850, 15064, 145, 4, 41, "Text",
 CellTags->"CrossProduct"],
Cell[374998, 15070, 308, 14, 41, "Text",
 CellTags->"CrossProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[375343, 15089, 60, 1, 36, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[375428, 15094, 326, 9, 28, "Input",
 CellTags->"CrossProduct"],
Cell[375757, 15105, 639, 22, 30, "Output",
 CellTags->"CrossProduct"]
}, Open  ]]
}, Open  ]],
Cell[376423, 15131, 41, 1, 57, "ObjectName",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[376489, 15136, 53, 1, 52, "Subsubsection",
 CellTags->"C0"],
Cell[376545, 15139, 448, 10, 59, "Text",
 CellTags->"C0"],
Cell[376996, 15151, 458, 24, 41, "Text",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[377491, 15180, 50, 1, 36, "Subsubsection",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[377566, 15185, 174, 5, 28, "Input",
 CellTags->"C0"],
Cell[377743, 15192, 260, 8, 27, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[378040, 15205, 203, 6, 28, "Input",
 CellTags->"C0"],
Cell[378246, 15213, 260, 8, 27, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[378543, 15226, 326, 10, 28, "Input",
 CellTags->"C0"],
Cell[378872, 15238, 260, 8, 27, "Output",
 CellTags->"C0"]
}, Open  ]]
}, Open  ]],
Cell[379159, 15250, 53, 1, 41, "ObjectName",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[379237, 15255, 59, 1, 36, "Subsubsection",
 CellTags->"DataType"],
Cell[379299, 15258, 344, 6, 61, "Text",
 CellTags->"DataType"],
Cell[379646, 15266, 835, 32, 42, "Text",
 CellTags->"DataType"],
Cell[380484, 15300, 291, 13, 25, "Text",
 CellTags->"DataType"],
Cell[380778, 15315, 218, 9, 25, "Text",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[381033, 15329, 56, 1, 36, "Subsubsection",
 CellTags->"DataType"],
Cell[381092, 15332, 74, 1, 25, "Text",
 CellTags->"DataType"],
Cell[381169, 15335, 221, 7, 28, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[381415, 15346, 224, 8, 28, "Input",
 CellTags->"DataType"],
Cell[381642, 15356, 238, 9, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[381895, 15368, 179, 4, 43, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[382099, 15376, 112, 3, 28, "Input",
 CellTags->"DataType"],
Cell[382214, 15381, 210, 7, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[382439, 15391, 271, 9, 28, "Input",
 CellTags->"DataType"],
Cell[382713, 15402, 412, 14, 28, "Input",
 CellTags->"DataType"],
Cell[383128, 15418, 395, 14, 28, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[383548, 15436, 340, 13, 28, "Input",
 CellTags->"DataType"],
Cell[383891, 15451, 350, 13, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384278, 15469, 107, 3, 28, "Input",
 CellTags->"DataType"],
Cell[384388, 15474, 298, 11, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384723, 15490, 107, 3, 28, "Input",
 CellTags->"DataType"],
Cell[384833, 15495, 172, 6, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[385020, 15504, 191, 6, 28, "Input",
 CellTags->"DataType"],
Cell[385214, 15512, 187, 6, 28, "Input",
 CellTags->"DataType"],
Cell[385404, 15520, 429, 15, 28, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[385858, 15539, 290, 11, 28, "Input",
 CellTags->"DataType"],
Cell[386151, 15552, 296, 11, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[386484, 15568, 105, 3, 28, "Input",
 CellTags->"DataType"],
Cell[386592, 15573, 228, 9, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[386835, 15585, 151, 5, 28, "Input",
 CellTags->"DataType"],
Cell[386989, 15592, 222, 7, 28, "Input",
 CellTags->"DataType"],
Cell[387214, 15601, 272, 9, 28, "Input",
 CellTags->"DataType"],
Cell[387489, 15612, 114, 3, 25, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[387628, 15619, 149, 4, 28, "Input",
 CellTags->"DataType"],
Cell[387780, 15625, 107, 3, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[387902, 15631, 83, 1, 25, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[388010, 15636, 233, 8, 28, "Input",
 CellTags->"DataType"],
Cell[388246, 15646, 104, 3, 27, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[388387, 15654, 226, 7, 28, "Input",
 CellTags->"DataType"],
Cell[388616, 15663, 435, 16, 27, "Output",
 CellTags->"DataType"]
}, Open  ]]
}, Open  ]],
Cell[389078, 15683, 43, 1, 41, "ObjectName",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[389146, 15688, 54, 1, 36, "Subsubsection",
 CellTags->"DB0"],
Cell[389203, 15691, 146, 4, 25, "Text",
 CellTags->"DB0"],
Cell[389352, 15697, 156, 9, 25, "Text",
 CellTags->"DB0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[389545, 15711, 51, 1, 36, "Subsubsection",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[389621, 15716, 339, 12, 28, "Input",
 CellTags->"DB0"],
Cell[389963, 15730, 255, 8, 27, "Output",
 CellTags->"DB0"]
}, Open  ]]
}, Open  ]],
Cell[390245, 15742, 43, 1, 41, "ObjectName",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[390313, 15747, 54, 1, 36, "Subsubsection",
 CellTags->"DB1"],
Cell[390370, 15750, 117, 3, 25, "Text",
 CellTags->"DB1"],
Cell[390490, 15755, 156, 9, 41, "Text",
 CellTags->"DB1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[390683, 15769, 51, 1, 36, "Subsubsection",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[390759, 15774, 339, 12, 28, "Input",
 CellTags->"DB1"],
Cell[391101, 15788, 1369, 43, 44, "Output",
 CellTags->"DB1"]
}, Open  ]]
}, Open  ]],
Cell[392497, 15835, 79, 1, 57, "ObjectName",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[392601, 15840, 72, 1, 52, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[392676, 15843, 192, 4, 41, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[392871, 15849, 347, 14, 41, "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[393255, 15868, 69, 1, 36, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[393327, 15871, 135, 3, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[393465, 15876, 152, 4, 25, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[393642, 15884, 158, 4, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[393803, 15890, 120, 3, 27, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[393938, 15896, 158, 4, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[394121, 15904, 191, 5, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[394315, 15911, 199, 6, 27, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[394529, 15920, 170, 4, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[394724, 15928, 191, 5, 28, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[394918, 15935, 199, 6, 27, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[395144, 15945, 63, 1, 41, "ObjectName",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[395232, 15950, 64, 1, 36, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[395299, 15953, 353, 11, 51, "Text",
 CellTags->"DeltaFunction"],
Cell[395655, 15966, 592, 24, 25, "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[396284, 15995, 61, 1, 36, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[396370, 16000, 142, 4, 28, "Input",
 CellTags->"DeltaFunction"],
Cell[396515, 16006, 203, 7, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[396755, 16018, 320, 10, 28, "Input",
 CellTags->"DeltaFunction"],
Cell[397078, 16030, 137, 4, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[397252, 16039, 294, 9, 28, "Input",
 CellTags->"DeltaFunction"],
Cell[397549, 16050, 137, 4, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[397723, 16059, 320, 10, 28, "Input",
 CellTags->"DeltaFunction"],
Cell[398046, 16071, 137, 4, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[398220, 16080, 204, 6, 28, "Input",
 CellTags->"DeltaFunction"],
Cell[398427, 16088, 109, 3, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[398563, 16095, 85, 1, 41, "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[CellGroupData[{
Cell[398673, 16100, 75, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[398751, 16103, 248, 8, 25, "Text",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[399002, 16113, 219, 8, 25, "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],
Cell[399236, 16124, 73, 1, 41, "ObjectName",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[399334, 16129, 69, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[399406, 16132, 227, 7, 25, "Text",
 CellTags->"DeltaFunctionPrime"],
Cell[399636, 16141, 576, 23, 25, "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400249, 16169, 66, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[400340, 16174, 152, 4, 28, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[400495, 16180, 240, 8, 27, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400772, 16193, 330, 10, 28, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[401105, 16205, 203, 6, 27, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[401345, 16216, 326, 10, 28, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[401674, 16228, 114, 3, 27, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]]
}, Open  ]],
Cell[401815, 16235, 69, 1, 41, "ObjectName",
 CellTags->"DenominatorOrder"],
Cell[CellGroupData[{
Cell[401909, 16240, 67, 1, 36, "Subsubsection",
 CellTags->"DenominatorOrder"],
Cell[401979, 16243, 175, 4, 25, "Text",
 CellTags->"DenominatorOrder"],
Cell[402157, 16249, 327, 13, 25, "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],
Cell[402499, 16265, 55, 1, 41, "ObjectName",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[402579, 16270, 60, 1, 36, "Subsubsection",
 CellTags->"Dimension"],
Cell[402642, 16273, 246, 5, 43, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[402913, 16282, 120, 3, 28, "Input",
 CellTags->"Dimension"],
Cell[403036, 16287, 243, 8, 27, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403316, 16300, 313, 9, 28, "Input",
 CellTags->"Dimension"],
Cell[403632, 16311, 286, 11, 27, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[403933, 16325, 131, 4, 25, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[404089, 16333, 357, 11, 28, "Input",
 CellTags->"Dimension"],
Cell[404449, 16346, 437, 13, 42, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[404901, 16362, 166, 4, 25, "Text",
 CellTags->"Dimension"],
Cell[405070, 16368, 133, 4, 28, "Input",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[405228, 16376, 326, 9, 28, "Input",
 CellTags->"Dimension"],
Cell[405557, 16387, 662, 28, 28, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[406256, 16420, 110, 3, 28, "Input",
 CellTags->"Dimension"],
Cell[406369, 16425, 450, 13, 42, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[406834, 16441, 134, 4, 28, "Input",
 CellTags->"Dimension"]
}, Open  ]],
Cell[406983, 16448, 77, 1, 57, "ObjectName",
 CellTags->"DimensionalReduction"],
Cell[CellGroupData[{
Cell[407085, 16453, 71, 1, 52, "Subsubsection",
 CellTags->"DimensionalReduction"],
Cell[407159, 16456, 121, 3, 41, "Text",
 CellTags->"DimensionalReduction"],
Cell[407283, 16461, 301, 13, 41, "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],
Cell[407599, 16477, 57, 1, 57, "ObjectName",
 CellTags->"DiracBasis"],
Cell[CellGroupData[{
Cell[407681, 16482, 61, 1, 52, "Subsubsection",
 CellTags->"DiracBasis"],
Cell[407745, 16485, 299, 6, 59, "Text",
 CellTags->"DiracBasis"],
Cell[408047, 16493, 184, 8, 41, "Text",
 CellTags->"DiracBasis"]
}, Open  ]],
Cell[408246, 16504, 65, 1, 57, "ObjectName",
 CellTags->"DiracCanonical"],
Cell[CellGroupData[{
Cell[408336, 16509, 65, 1, 52, "Subsubsection",
 CellTags->"DiracCanonical"],
Cell[408404, 16512, 171, 4, 41, "Text",
 CellTags->"DiracCanonical"],
Cell[408578, 16518, 194, 8, 41, "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],
Cell[408787, 16529, 63, 1, 57, "ObjectName",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[408875, 16534, 64, 1, 52, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[408942, 16537, 160, 4, 41, "Text",
 CellTags->"DiracEquation"],
Cell[409105, 16543, 199, 9, 41, "Text",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[409341, 16557, 61, 1, 36, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[409427, 16562, 279, 8, 28, "Input",
 CellTags->"DiracEquation"],
Cell[409709, 16572, 368, 13, 27, "Output",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[410114, 16590, 119, 3, 28, "Input",
 CellTags->"DiracEquation"],
Cell[410236, 16595, 217, 7, 27, "Output",
 CellTags->"DiracEquation"]
}, Open  ]]
}, Open  ]],
Cell[410480, 16606, 57, 1, 57, "ObjectName",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[410562, 16611, 61, 1, 52, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[410626, 16614, 766, 25, 59, "Text",
 CellTags->"DiracGamma"],
Cell[411395, 16641, 681, 29, 41, "Text",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[412113, 16675, 58, 1, 36, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[412196, 16680, 113, 3, 28, "Input",
 CellTags->"DiracGamma"],
Cell[412312, 16685, 137, 4, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[412486, 16694, 159, 4, 28, "Input",
 CellTags->"DiracGamma"],
Cell[412648, 16700, 214, 8, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[412877, 16711, 332, 13, 25, "Text",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[413234, 16728, 167, 5, 28, "Input",
 CellTags->"DiracGamma"],
Cell[413404, 16735, 223, 8, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[413664, 16748, 280, 8, 28, "Input",
 CellTags->"DiracGamma"],
Cell[413947, 16758, 468, 17, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[414452, 16780, 218, 7, 28, "Input",
 CellTags->"DiracGamma"],
Cell[414673, 16789, 223, 8, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[414933, 16802, 208, 7, 28, "Input",
 CellTags->"DiracGamma"],
Cell[415144, 16811, 341, 12, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[415522, 16828, 144, 4, 28, "Input",
 CellTags->"DiracGamma"],
Cell[415669, 16834, 593, 22, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[416299, 16861, 300, 9, 28, "Input",
 CellTags->"DiracGamma"],
Cell[416602, 16872, 430, 13, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[417069, 16890, 138, 4, 28, "Input",
 CellTags->"DiracGamma"],
Cell[417210, 16896, 430, 13, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[417677, 16914, 141, 4, 28, "Input",
 CellTags->"DiracGamma"],
Cell[417821, 16920, 1123, 42, 27, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[418959, 16965, 168, 5, 28, "Input",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[419142, 16973, 71, 1, 57, "ObjectName",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[419238, 16978, 68, 1, 52, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[419309, 16981, 135, 4, 41, "Text",
 CellTags->"DiracGammaCombine"],
Cell[419447, 16987, 804, 34, 41, "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[420288, 17026, 65, 1, 36, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[420378, 17031, 220, 6, 28, "Input",
 CellTags->"DiracGammaCombine"],
Cell[420601, 17039, 265, 9, 27, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[420903, 17053, 122, 3, 28, "Input",
 CellTags->"DiracGammaCombine"],
Cell[421028, 17058, 194, 5, 42, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[421259, 17068, 270, 8, 28, "Input",
 CellTags->"DiracGammaCombine"],
Cell[421532, 17078, 317, 11, 27, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[421886, 17094, 122, 3, 28, "Input",
 CellTags->"DiracGammaCombine"],
Cell[422011, 17099, 244, 7, 42, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[422292, 17111, 127, 3, 28, "Input",
 CellTags->"DiracGammaCombine"],
Cell[422422, 17116, 438, 16, 27, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]]
}, Open  ]],
Cell[422887, 17136, 69, 1, 57, "ObjectName",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[422981, 17141, 67, 1, 52, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[423051, 17144, 190, 4, 41, "Text",
 CellTags->"DiracGammaExpand"],
Cell[423244, 17150, 806, 34, 41, "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424087, 17189, 64, 1, 36, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[424176, 17194, 314, 9, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[424493, 17205, 474, 17, 27, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[424982, 17225, 125, 3, 25, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[425132, 17232, 121, 3, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[425256, 17237, 291, 8, 42, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[425562, 17248, 101, 1, 25, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[425688, 17253, 125, 3, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[425816, 17258, 599, 22, 27, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[426430, 17283, 90, 1, 25, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[426545, 17288, 209, 5, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[426757, 17295, 291, 8, 42, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[427085, 17308, 164, 4, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[427252, 17314, 394, 11, 42, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[427661, 17328, 106, 1, 25, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[427792, 17333, 122, 3, 28, "Input",
 CellTags->"DiracGammaExpand"],
Cell[427917, 17338, 534, 20, 27, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[428466, 17361, 114, 3, 28, "Input",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[428595, 17367, 59, 1, 57, "ObjectName",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[428679, 17372, 62, 1, 52, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[428744, 17375, 208, 5, 59, "Text",
 CellTags->"DiracGammaT"],
Cell[428955, 17382, 188, 9, 41, "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[429180, 17396, 59, 1, 36, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[429264, 17401, 158, 4, 28, "Input",
 CellTags->"DiracGammaT"],
Cell[429425, 17407, 220, 8, 27, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[429682, 17420, 113, 3, 28, "Input",
 CellTags->"DiracGammaT"],
Cell[429798, 17425, 212, 8, 27, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[430047, 17438, 167, 6, 28, "Input",
 CellTags->"DiracGammaT"],
Cell[430217, 17446, 219, 7, 29, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]]
}, Open  ]],
Cell[430463, 17457, 59, 1, 57, "ObjectName",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[430547, 17462, 62, 1, 52, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[430612, 17465, 1220, 46, 59, "Text",
 CellTags->"DiracMatrix"],
Cell[431835, 17513, 1053, 49, 41, "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[432925, 17567, 59, 1, 36, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[433009, 17572, 119, 3, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[433131, 17577, 212, 8, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[433358, 17588, 539, 20, 43, "Text",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[433922, 17612, 184, 5, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[434109, 17619, 350, 14, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[434496, 17638, 160, 5, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[434659, 17645, 175, 4, 42, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[434871, 17654, 145, 4, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[435019, 17660, 142, 4, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[435198, 17669, 112, 3, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[435313, 17674, 125, 3, 42, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[435475, 17682, 111, 3, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[435589, 17687, 142, 4, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[435768, 17696, 138, 4, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[435909, 17702, 221, 6, 42, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[436167, 17713, 152, 4, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[436322, 17719, 249, 7, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[436608, 17731, 166, 5, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[436777, 17738, 342, 14, 27, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[437156, 17757, 112, 3, 28, "Input",
 CellTags->"DiracMatrix"],
Cell[437271, 17762, 269, 7, 42, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]]
}, Open  ]],
Cell[437567, 17773, 57, 1, 57, "ObjectName",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[437649, 17778, 61, 1, 52, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[437713, 17781, 204, 5, 41, "Text",
 CellTags->"DiracOrder"],
Cell[437920, 17788, 312, 14, 41, "Text",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[438269, 17807, 58, 1, 36, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[438352, 17812, 176, 5, 28, "Input",
 CellTags->"DiracOrder"],
Cell[438531, 17819, 206, 6, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[438774, 17830, 114, 3, 28, "Input",
 CellTags->"DiracOrder"],
Cell[438891, 17835, 472, 18, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[439378, 17856, 93, 1, 25, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[439496, 17861, 176, 5, 28, "Input",
 CellTags->"DiracOrder"],
Cell[439675, 17868, 247, 7, 27, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[439959, 17880, 114, 3, 28, "Input",
 CellTags->"DiracOrder"],
Cell[440076, 17885, 472, 18, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440585, 17908, 171, 5, 28, "Input",
 CellTags->"DiracOrder"],
Cell[440759, 17915, 243, 7, 27, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[441017, 17925, 196, 8, 25, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[441238, 17937, 114, 3, 28, "Input",
 CellTags->"DiracOrder"],
Cell[441355, 17942, 391, 15, 27, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441783, 17962, 167, 5, 28, "Input",
 CellTags->"DiracOrder"],
Cell[441953, 17969, 239, 7, 27, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[442229, 17981, 114, 3, 28, "Input",
 CellTags->"DiracOrder"],
Cell[442346, 17986, 271, 10, 27, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[442654, 18001, 183, 5, 28, "Input",
 CellTags->"DiracOrder"],
Cell[442840, 18008, 255, 7, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[443110, 18018, 288, 10, 25, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[443423, 18032, 219, 7, 28, "Input",
 CellTags->"DiracOrder"],
Cell[443645, 18041, 1207, 45, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[444889, 18091, 113, 3, 28, "Input",
 CellTags->"DiracOrder"],
Cell[445005, 18096, 477, 19, 28, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[445497, 18118, 188, 6, 28, "Input",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[445700, 18127, 59, 1, 57, "ObjectName",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[445784, 18132, 62, 1, 52, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[445849, 18135, 662, 11, 97, "Text",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[446536, 18150, 121, 3, 28, "Input",
 CellTags->"DiracReduce"],
Cell[446660, 18155, 323, 10, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[446998, 18168, 197, 9, 41, "Text",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447232, 18182, 59, 1, 36, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[447316, 18187, 172, 5, 28, "Input",
 CellTags->"DiracReduce"],
Cell[447491, 18194, 202, 6, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447730, 18205, 116, 3, 28, "Input",
 CellTags->"DiracReduce"],
Cell[447849, 18210, 281, 9, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448167, 18224, 186, 5, 28, "Input",
 CellTags->"DiracReduce"],
Cell[448356, 18231, 239, 7, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448632, 18243, 116, 3, 28, "Input",
 CellTags->"DiracReduce"],
Cell[448751, 18248, 919, 28, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[449707, 18281, 207, 6, 28, "Input",
 CellTags->"DiracReduce"],
Cell[449917, 18289, 283, 8, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[450237, 18302, 140, 4, 28, "Input",
 CellTags->"DiracReduce"],
Cell[450380, 18308, 1811, 54, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[452228, 18367, 253, 7, 28, "Input",
 CellTags->"DiracReduce"],
Cell[452484, 18376, 127, 4, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[452648, 18385, 133, 4, 28, "Input",
 CellTags->"DiracReduce"],
Cell[452784, 18391, 127, 4, 27, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[452926, 18398, 156, 4, 28, "Input",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[453097, 18405, 57, 1, 57, "ObjectName",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[453179, 18410, 61, 1, 52, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[453243, 18413, 213, 5, 41, "Text",
 CellTags->"DiracSigma"],
Cell[453459, 18420, 211, 9, 41, "Text",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[453707, 18434, 58, 1, 36, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[453790, 18439, 236, 7, 28, "Input",
 CellTags->"DiracSigma"],
Cell[454029, 18448, 169, 5, 28, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[454235, 18458, 122, 3, 28, "Input",
 CellTags->"DiracSigma"],
Cell[454360, 18463, 784, 29, 38, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[455181, 18497, 236, 7, 28, "Input",
 CellTags->"DiracSigma"],
Cell[455420, 18506, 189, 6, 28, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[455646, 18517, 223, 7, 28, "Input",
 CellTags->"DiracSigma"],
Cell[455872, 18526, 156, 5, 27, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[456065, 18536, 122, 3, 28, "Input",
 CellTags->"DiracSigma"],
Cell[456190, 18541, 950, 33, 38, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[457155, 18577, 144, 4, 28, "Input",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[457314, 18584, 73, 1, 57, "ObjectName",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[457412, 18589, 69, 1, 52, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[457484, 18592, 186, 4, 25, "Text",
 CellTags->"DiracSigmaExplicit"],
Cell[457673, 18598, 195, 9, 41, "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[457905, 18612, 66, 1, 36, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[457996, 18617, 217, 6, 28, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[458216, 18625, 177, 5, 28, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[458430, 18635, 129, 3, 28, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[458562, 18640, 792, 29, 38, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]]
}, Open  ]],
Cell[459381, 18673, 69, 1, 41, "ObjectName",
 CellTags->"DiracSimpCombine"],
Cell[CellGroupData[{
Cell[459475, 18678, 67, 1, 36, "Subsubsection",
 CellTags->"DiracSimpCombine"],
Cell[459545, 18681, 209, 5, 25, "Text",
 CellTags->"DiracSimpCombine"],
Cell[459757, 18688, 196, 8, 25, "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],
Cell[459968, 18699, 63, 1, 41, "ObjectName",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[460056, 18704, 64, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[460123, 18707, 382, 7, 61, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[460530, 18718, 125, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[460658, 18723, 604, 14, 49, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[461277, 18740, 422, 19, 25, "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[461736, 18764, 61, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[461800, 18767, 99, 1, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[461924, 18772, 178, 5, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[462105, 18779, 250, 7, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[462392, 18791, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[462515, 18796, 261, 10, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[462791, 18809, 96, 1, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[462912, 18814, 179, 5, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[463094, 18821, 250, 7, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[463381, 18833, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[463504, 18838, 261, 10, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[463802, 18853, 174, 5, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[463979, 18860, 246, 7, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[464240, 18870, 179, 7, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[464444, 18881, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[464567, 18886, 394, 15, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[464998, 18906, 170, 5, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[465171, 18913, 242, 7, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465450, 18925, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[465573, 18930, 274, 10, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465884, 18945, 326, 10, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[466213, 18957, 570, 18, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[466798, 18978, 120, 3, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[466943, 18985, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[467066, 18990, 1908, 72, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[469011, 19067, 117, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[469131, 19072, 570, 18, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[469738, 19095, 272, 7, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[470013, 19104, 407, 14, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[470457, 19123, 120, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[470580, 19128, 109, 3, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[470726, 19136, 373, 12, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[471102, 19150, 624, 18, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[471763, 19173, 119, 3, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[471885, 19178, 13907, 530, 93, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[485807, 19711, 115, 3, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[485947, 19718, 251, 7, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[486201, 19727, 465, 18, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[486681, 19748, 141, 4, 25, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[486847, 19756, 289, 8, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[487139, 19766, 556, 19, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[487732, 19790, 268, 7, 28, "Input",
 CellTags->"DiracSimplify"],
Cell[488003, 19799, 109, 3, 27, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[488127, 19805, 183, 5, 28, "Input",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[488325, 19813, 65, 1, 41, "ObjectName",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[488415, 19818, 65, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[488483, 19821, 224, 7, 25, "Text",
 CellTags->"DiracSimplify2"],
Cell[488710, 19830, 200, 9, 25, "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[488947, 19844, 62, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[489034, 19849, 222, 6, 28, "Input",
 CellTags->"DiracSimplify2"],
Cell[489259, 19857, 439, 11, 28, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[489735, 19873, 121, 3, 28, "Input",
 CellTags->"DiracSimplify2"],
Cell[489859, 19878, 883, 35, 28, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]]
}, Open  ]],
Cell[490769, 19917, 57, 1, 41, "ObjectName",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[490851, 19922, 61, 1, 36, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[490915, 19925, 469, 16, 43, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[491409, 19945, 119, 3, 28, "Input",
 CellTags->"DiracSlash"],
Cell[491531, 19950, 244, 8, 27, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[491790, 19961, 868, 39, 25, "Text",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[492695, 20005, 58, 1, 36, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[492756, 20008, 234, 9, 25, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[493015, 20021, 113, 3, 28, "Input",
 CellTags->"DiracSlash"],
Cell[493131, 20026, 223, 8, 27, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[493391, 20039, 173, 5, 28, "Input",
 CellTags->"DiracSlash"],
Cell[493567, 20046, 431, 16, 27, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[494035, 20067, 136, 4, 28, "Input",
 CellTags->"DiracSlash"],
Cell[494174, 20073, 431, 16, 27, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[494642, 20094, 105, 3, 28, "Input",
 CellTags->"DiracSlash"],
Cell[494750, 20099, 147, 4, 27, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[494934, 20108, 148, 4, 28, "Input",
 CellTags->"DiracSlash"],
Cell[495085, 20114, 163, 4, 42, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[495285, 20123, 214, 7, 28, "Input",
 CellTags->"DiracSlash"],
Cell[495502, 20132, 212, 6, 42, "Output",
 CellTags->"DiracSlash"]
}, Open  ]]
}, Open  ]],
Cell[495741, 20142, 59, 1, 41, "ObjectName",
 CellTags->"DiracSpinor"],
Cell[CellGroupData[{
Cell[495825, 20147, 62, 1, 36, "Subsubsection",
 CellTags->"DiracSpinor"],
Cell[495890, 20150, 288, 6, 43, "Text",
 CellTags->"DiracSpinor"],
Cell[496181, 20158, 170, 8, 41, "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],
Cell[496366, 20169, 71, 1, 57, "ObjectName",
 CellTags->"DiracSubstitute67"],
Cell[CellGroupData[{
Cell[496462, 20174, 68, 1, 52, "Subsubsection",
 CellTags->"DiracSubstitute67"],
Cell[496533, 20177, 219, 5, 41, "Text",
 CellTags->"DiracSubstitute67"],
Cell[496755, 20184, 313, 13, 41, "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],
Cell[497083, 20200, 57, 1, 57, "ObjectName",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[497165, 20205, 61, 1, 52, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[497229, 20208, 336, 6, 61, "Text",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[497590, 20218, 119, 3, 28, "Input",
 CellTags->"DiracTrace"],
Cell[497712, 20223, 664, 16, 49, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[498391, 20242, 283, 12, 41, "Text",
 CellTags->"DiracTrace"],
Cell[498677, 20256, 163, 9, 41, "Text",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[498877, 20270, 58, 1, 36, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[498960, 20275, 174, 5, 28, "Input",
 CellTags->"DiracTrace"],
Cell[499137, 20282, 270, 9, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[499444, 20296, 209, 6, 28, "Input",
 CellTags->"DiracTrace"],
Cell[499656, 20304, 370, 11, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500063, 20320, 149, 4, 28, "Input",
 CellTags->"DiracTrace"],
Cell[500215, 20326, 635, 22, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500887, 20353, 296, 8, 28, "Input",
 CellTags->"DiracTrace"],
Cell[501186, 20363, 566, 23, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[501789, 20391, 334, 9, 28, "Input",
 CellTags->"DiracTrace"],
Cell[502126, 20402, 8726, 322, 52, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[510889, 20729, 186, 5, 28, "Input",
 CellTags->"DiracTrace"],
Cell[511078, 20736, 498, 15, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[511613, 20756, 294, 8, 28, "Input",
 CellTags->"DiracTrace"],
Cell[511910, 20766, 182, 6, 27, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[512129, 20777, 111, 3, 28, "Input",
 CellTags->"DiracTrace"],
Cell[512243, 20782, 175, 5, 42, "Output",
 CellTags->"DiracTrace"]
}, Open  ]]
}, Open  ]],
Cell[512445, 20791, 73, 1, 57, "ObjectName",
 CellTags->"DiracTraceEvaluate"],
Cell[CellGroupData[{
Cell[512543, 20796, 69, 1, 52, "Subsubsection",
 CellTags->"DiracTraceEvaluate"],
Cell[512615, 20799, 162, 4, 41, "Text",
 CellTags->"DiracTraceEvaluate"],
Cell[512780, 20805, 281, 13, 41, "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],
Cell[513076, 20821, 57, 1, 57, "ObjectName",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[513158, 20826, 61, 1, 52, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[513222, 20829, 222, 5, 43, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[513469, 20838, 119, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[513591, 20843, 174, 6, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[513780, 20852, 428, 19, 70, "Text",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[514245, 20876, 58, 1, 70, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[514306, 20879, 96, 1, 25, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[514427, 20884, 175, 5, 28, "Input",
 CellTags->"DiracTrick"],
Cell[514605, 20891, 247, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[514889, 20903, 114, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[515006, 20908, 247, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[515268, 20918, 93, 1, 25, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[515386, 20923, 176, 5, 28, "Input",
 CellTags->"DiracTrick"],
Cell[515565, 20930, 247, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[515849, 20942, 114, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[515966, 20947, 247, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[516250, 20959, 171, 5, 28, "Input",
 CellTags->"DiracTrick"],
Cell[516424, 20966, 243, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[516682, 20976, 176, 7, 25, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[516883, 20987, 114, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[517000, 20992, 243, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[517280, 21004, 167, 5, 28, "Input",
 CellTags->"DiracTrick"],
Cell[517450, 21011, 239, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[517726, 21023, 114, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[517843, 21028, 239, 7, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[518119, 21040, 323, 10, 28, "Input",
 CellTags->"DiracTrick"],
Cell[518445, 21052, 567, 18, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[519049, 21075, 114, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[519166, 21080, 567, 18, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[519770, 21103, 108, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[519881, 21108, 1905, 72, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[521823, 21185, 370, 12, 28, "Input",
 CellTags->"DiracTrick"],
Cell[522196, 21199, 621, 18, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[522854, 21222, 113, 3, 28, "Input",
 CellTags->"DiracTrick"],
Cell[522970, 21227, 621, 18, 27, "Output",
 CellTags->"DiracTrick"]
}, Open  ]]
}, Open  ]],
Cell[523618, 21249, 55, 1, 57, "ObjectName",
 CellTags->"Divideout"],
Cell[CellGroupData[{
Cell[523698, 21254, 60, 1, 52, "Subsubsection",
 CellTags->"Divideout"],
Cell[523761, 21257, 137, 4, 41, "Text",
 CellTags->"Divideout"],
Cell[523901, 21263, 281, 13, 41, "Text",
 CellTags->"Divideout"]
}, Open  ]],
Cell[524197, 21279, 43, 1, 57, "ObjectName",
 CellTags->"DOT"],
Cell[CellGroupData[{
Cell[524265, 21284, 54, 1, 52, "Subsubsection",
 CellTags->"DOT"],
Cell[524322, 21287, 348, 8, 93, "Text",
 CellTags->"DOT"],
Cell[524673, 21297, 177, 8, 41, "Text",
 CellTags->"DOT"]
}, Open  ]],
Cell[524865, 21308, 55, 1, 57, "ObjectName",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[524945, 21313, 60, 1, 52, "Subsubsection",
 CellTags->"DotExpand"],
Cell[525008, 21316, 85, 1, 41, "Text",
 CellTags->"DotExpand"],
Cell[525096, 21319, 582, 23, 41, "Text",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[525715, 21347, 57, 1, 36, "Subsubsection",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[525797, 21352, 313, 10, 28, "Input",
 CellTags->"DotExpand"],
Cell[526113, 21364, 504, 13, 27, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[526632, 21380, 229, 7, 28, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[526886, 21391, 313, 10, 28, "Input",
 CellTags->"DotExpand"],
Cell[527202, 21403, 639, 22, 27, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[527856, 21428, 231, 7, 28, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[528112, 21439, 313, 10, 28, "Input",
 CellTags->"DotExpand"],
Cell[528428, 21451, 504, 13, 27, "Output",
 CellTags->"DotExpand"]
}, Open  ]]
}, Open  ]],
Cell[528959, 21468, 53, 1, 57, "ObjectName",
 CellTags->"DotPower"],
Cell[CellGroupData[{
Cell[529037, 21473, 59, 1, 52, "Subsubsection",
 CellTags->"DotPower"],
Cell[529099, 21476, 189, 4, 41, "Text",
 CellTags->"DotPower"],
Cell[529291, 21482, 182, 8, 70, "Text",
 CellTags->"DotPower"]
}, Open  ]],
Cell[529488, 21493, 57, 1, 70, "ObjectName",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[529570, 21498, 61, 1, 70, "Subsubsection",
 CellTags->"DotProduct"],
Cell[529634, 21501, 219, 5, 70, "Text",
 CellTags->"DotProduct"],
Cell[529856, 21508, 312, 14, 41, "Text",
 CellTags->"DotProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[530205, 21527, 58, 1, 36, "Subsubsection",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[530288, 21532, 234, 7, 28, "Input",
 CellTags->"DotProduct"],
Cell[530525, 21541, 314, 11, 30, "Output",
 CellTags->"DotProduct"]
}, Open  ]]
}, Open  ]],
Cell[530866, 21556, 77, 1, 57, "ObjectName",
 CellTags->"DotSimplifyRelations"],
Cell[CellGroupData[{
Cell[530968, 21561, 71, 1, 52, "Subsubsection",
 CellTags->"DotSimplifyRelations"],
Cell[531042, 21564, 241, 5, 41, "Text",
 CellTags->"DotSimplifyRelations"],
Cell[531286, 21571, 194, 8, 41, "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],
Cell[531495, 21582, 59, 1, 57, "ObjectName",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[531579, 21587, 62, 1, 52, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[531644, 21590, 322, 6, 61, "Text",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[531991, 21600, 121, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[532115, 21605, 381, 11, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[532511, 21619, 414, 19, 41, "Text",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[532962, 21643, 59, 1, 36, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[533046, 21648, 342, 11, 28, "Input",
 CellTags->"DotSimplify"],
Cell[533391, 21661, 376, 11, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[533804, 21677, 116, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[533923, 21682, 1013, 39, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[534951, 21724, 158, 4, 28, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[535134, 21732, 209, 7, 28, "Input",
 CellTags->"DotSimplify"],
Cell[535346, 21741, 217, 8, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[535600, 21754, 116, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[535719, 21759, 223, 7, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[535979, 21771, 161, 5, 28, "Input",
 CellTags->"DotSimplify"],
Cell[536143, 21778, 107, 3, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[536287, 21786, 116, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[536406, 21791, 279, 10, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[536700, 21804, 161, 5, 28, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[536886, 21813, 116, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[537005, 21818, 223, 7, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[537265, 21830, 165, 5, 28, "Input",
 CellTags->"DotSimplify"],
Cell[537433, 21837, 107, 3, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[537577, 21845, 116, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[537696, 21850, 257, 8, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[537968, 21861, 165, 5, 28, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[538158, 21870, 299, 9, 28, "Input",
 CellTags->"DotSimplify"],
Cell[538460, 21881, 166, 6, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[538641, 21890, 125, 3, 28, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[538791, 21897, 148, 4, 28, "Input",
 CellTags->"DotSimplify"],
Cell[538942, 21903, 140, 4, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[539119, 21912, 213, 6, 28, "Input",
 CellTags->"DotSimplify"],
Cell[539335, 21920, 140, 4, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[539490, 21927, 170, 4, 28, "Input",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[539675, 21934, 57, 1, 57, "ObjectName",
 CellTags->"DummyIndex"],
Cell[CellGroupData[{
Cell[539757, 21939, 61, 1, 52, "Subsubsection",
 CellTags->"DummyIndex"],
Cell[539821, 21942, 191, 4, 41, "Text",
 CellTags->"DummyIndex"],
Cell[540015, 21948, 181, 8, 41, "Text",
 CellTags->"DummyIndex"]
}, Open  ]],
Cell[540211, 21959, 41, 1, 57, "ObjectName",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[540277, 21964, 53, 1, 52, "Subsubsection",
 CellTags->"D0"],
Cell[540333, 21967, 524, 12, 77, "Text",
 CellTags->"D0"],
Cell[540860, 21981, 458, 24, 41, "Text",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[541355, 22010, 50, 1, 36, "Subsubsection",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[541430, 22015, 315, 9, 28, "Input",
 CellTags->"D0"],
Cell[541748, 22026, 550, 20, 27, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[542335, 22051, 484, 14, 28, "Input",
 CellTags->"D0"],
Cell[542822, 22067, 550, 20, 27, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[543409, 22092, 104, 3, 28, "Input",
 CellTags->"D0"],
Cell[543516, 22097, 550, 20, 27, "Output",
 CellTags->"D0"]
}, Open  ]]
}, Open  ]],
Cell[544093, 22121, 61, 1, 57, "ObjectName",
 CellTags->"D0Convention"],
Cell[CellGroupData[{
Cell[544179, 22126, 63, 1, 52, "Subsubsection",
 CellTags->"D0Convention"],
Cell[544245, 22129, 307, 6, 59, "Text",
 CellTags->"D0Convention"],
Cell[544555, 22137, 263, 13, 41, "Text",
 CellTags->"D0Convention"]
}, Open  ]],
Cell[544833, 22153, 43, 1, 57, "ObjectName",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[544901, 22158, 54, 1, 52, "Subsubsection",
 CellTags->"Eps"],
Cell[544958, 22161, 347, 9, 43, "Text",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[545330, 22174, 105, 3, 28, "Input",
 CellTags->"Eps"],
Cell[545438, 22179, 160, 5, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[545613, 22187, 486, 24, 41, "Text",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[546136, 22216, 51, 1, 36, "Subsubsection",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[546212, 22221, 322, 8, 28, "Input",
 CellTags->"Eps"],
Cell[546537, 22231, 477, 21, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[547051, 22257, 314, 8, 28, "Input",
 CellTags->"Eps"],
Cell[547368, 22267, 435, 19, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[547840, 22291, 181, 6, 28, "Input",
 CellTags->"Eps"],
Cell[548024, 22299, 176, 5, 42, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548237, 22309, 142, 4, 28, "Input",
 CellTags->"Eps"],
Cell[548382, 22315, 99, 3, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548518, 22323, 142, 4, 28, "Input",
 CellTags->"Eps"],
Cell[548663, 22329, 117, 4, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[548795, 22336, 189, 6, 28, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[549009, 22346, 460, 13, 28, "Input",
 CellTags->"Eps"],
Cell[549472, 22361, 477, 21, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[549986, 22387, 107, 3, 28, "Input",
 CellTags->"Eps"],
Cell[550096, 22392, 102, 3, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[550213, 22398, 189, 6, 28, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[550427, 22408, 460, 13, 28, "Input",
 CellTags->"Eps"],
Cell[550890, 22423, 477, 21, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[551404, 22449, 137, 4, 28, "Input",
 CellTags->"Eps"],
Cell[551544, 22455, 102, 3, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[551683, 22463, 293, 9, 28, "Input"],
Cell[551979, 22474, 366, 9, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[552382, 22488, 77, 2, 28, "Input"],
Cell[552462, 22492, 974, 41, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[553473, 22538, 433, 14, 40, "Input",
 CellTags->"Eps"],
Cell[553909, 22554, 1119, 45, 38, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[555065, 22604, 978, 28, 40, "Input",
 CellTags->"Eps"],
Cell[556046, 22634, 1679, 61, 38, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[557762, 22700, 188, 6, 28, "Input",
 CellTags->"Eps"],
Cell[557953, 22708, 342, 12, 38, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[558332, 22725, 124, 4, 28, "Input",
 CellTags->"Eps"],
Cell[558459, 22731, 99, 3, 27, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[558573, 22737, 164, 5, 28, "Input",
 CellTags->"Eps"]
}, Open  ]],
Cell[558752, 22745, 59, 1, 57, "ObjectName",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[558836, 22750, 62, 1, 52, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[558901, 22753, 166, 4, 41, "Text",
 CellTags->"EpsChisholm"],
Cell[559070, 22759, 182, 9, 41, "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[559289, 22773, 59, 1, 36, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[559373, 22778, 208, 6, 28, "Input",
 CellTags->"EpsChisholm"],
Cell[559584, 22786, 1989, 78, 27, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561610, 22869, 115, 3, 28, "Input",
 CellTags->"EpsChisholm"],
Cell[561728, 22874, 594, 24, 27, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]]
}, Open  ]],
Cell[562349, 22902, 59, 1, 57, "ObjectName",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[562433, 22907, 62, 1, 52, "Subsubsection",
 CellTags->"EpsContract"],
Cell[562498, 22910, 227, 5, 59, "Text",
 CellTags->"EpsContract"],
Cell[562728, 22917, 277, 14, 41, "Text",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[563042, 22936, 59, 1, 36, "Subsubsection",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[563126, 22941, 468, 13, 28, "Input",
 CellTags->"EpsContract"],
Cell[563597, 22956, 485, 21, 27, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[564119, 22982, 205, 6, 28, "Input",
 CellTags->"EpsContract"],
Cell[564327, 22990, 569, 23, 28, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[564933, 23018, 204, 6, 28, "Input",
 CellTags->"EpsContract"],
Cell[565140, 23026, 320, 11, 27, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[565475, 23040, 84, 2, 28, "Input"]
}, Open  ]],
Cell[565574, 23045, 57, 1, 57, "ObjectName",
 CellTags->"EpsDiscard"],
Cell[CellGroupData[{
Cell[565656, 23050, 61, 1, 36, "Subsubsection",
 CellTags->"EpsDiscard"],
Cell[565720, 23053, 169, 4, 25, "Text",
 CellTags->"EpsDiscard"],
Cell[565892, 23059, 196, 9, 25, "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],
Cell[566103, 23071, 59, 1, 41, "ObjectName",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[566187, 23076, 62, 1, 36, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[566252, 23079, 173, 4, 25, "Text",
 CellTags->"EpsEvaluate"],
Cell[566428, 23085, 494, 24, 25, "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[566959, 23114, 59, 1, 36, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[567043, 23119, 335, 10, 28, "Input",
 CellTags->"EpsEvaluate"],
Cell[567381, 23131, 466, 20, 27, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[567884, 23156, 115, 3, 28, "Input",
 CellTags->"EpsEvaluate"],
Cell[568002, 23161, 466, 20, 27, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[568505, 23186, 116, 3, 28, "Input",
 CellTags->"EpsEvaluate"],
Cell[568624, 23191, 434, 12, 42, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]]
}, Open  ]],
Cell[569085, 23207, 63, 1, 41, "ObjectName",
 CellTags->"EpsUncontract"],
Cell[CellGroupData[{
Cell[569173, 23212, 64, 1, 36, "Subsubsection",
 CellTags->"EpsUncontract"],
Cell[569240, 23215, 115, 3, 25, "Text",
 CellTags->"EpsUncontract"],
Cell[569358, 23220, 285, 14, 25, "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],
Cell[569658, 23237, 51, 1, 41, "ObjectName",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[569734, 23242, 58, 1, 36, "Subsubsection",
 CellTags->"Epsilon"],
Cell[569795, 23245, 237, 9, 25, "Text",
 CellTags->"Epsilon"],
Cell[570035, 23256, 175, 9, 25, "Text",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[570247, 23270, 55, 1, 36, "Subsubsection",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[570327, 23275, 80, 2, 28, "Input",
 CellTags->"Epsilon"],
Cell[570410, 23279, 148, 5, 27, "Output",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[570573, 23287, 114, 3, 25, "Text",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[570712, 23294, 587, 19, 28, "Input",
 CellTags->"Epsilon"],
Cell[571302, 23315, 199, 6, 27, "Output",
 CellTags->"Epsilon"]
}, Open  ]]
}, Open  ]],
Cell[571528, 23325, 61, 1, 41, "ObjectName",
 CellTags->"EpsilonOrder"],
Cell[CellGroupData[{
Cell[571614, 23330, 63, 1, 36, "Subsubsection",
 CellTags->"EpsilonOrder"],
Cell[571680, 23333, 305, 12, 25, "Text",
 CellTags->"EpsilonOrder"],
Cell[571988, 23347, 287, 14, 25, "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],
Cell[572290, 23364, 55, 1, 41, "ObjectName",
 CellTags->"Expanding"],
Cell[CellGroupData[{
Cell[572370, 23369, 60, 1, 36, "Subsubsection",
 CellTags->"Expanding"],
Cell[572433, 23372, 378, 7, 61, "Text",
 CellTags->"Expanding"],
Cell[572814, 23381, 641, 29, 25, "Text",
 CellTags->"Expanding"]
}, Open  ]],
Cell[573470, 23413, 65, 1, 41, "ObjectName",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[573560, 23418, 65, 1, 36, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[573628, 23421, 188, 4, 43, "Text",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[573841, 23429, 98, 2, 28, "Input"],
Cell[573942, 23433, 1041, 36, 29, "Output"]
}, Open  ]],
Cell[574998, 23472, 724, 28, 25, "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575759, 23505, 62, 1, 36, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[575846, 23510, 373, 10, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[576222, 23522, 345, 12, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[576604, 23539, 121, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[576728, 23544, 773, 30, 27, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[577538, 23579, 115, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[577656, 23584, 798, 21, 62, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[578491, 23610, 259, 7, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[578753, 23619, 312, 11, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[579102, 23635, 121, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[579226, 23640, 704, 26, 38, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[579967, 23671, 433, 13, 50, "Input",
 CellTags->"ExpandPartialD"],
Cell[580403, 23686, 332, 10, 35, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[580772, 23701, 121, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[580896, 23706, 1940, 67, 38, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582873, 23778, 180, 6, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[583056, 23786, 267, 8, 35, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[583360, 23799, 123, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[583486, 23804, 438, 14, 38, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[583961, 23823, 121, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[584085, 23828, 1877, 59, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[585999, 23892, 314, 9, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[586316, 23903, 1192, 47, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[587545, 23955, 121, 3, 28, "Input",
 CellTags->"ExpandPartialD"],
Cell[587669, 23960, 110, 3, 27, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]]
}, Open  ]],
Cell[587806, 23967, 121, 1, 41, "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[587952, 23972, 95, 1, 36, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[588050, 23975, 252, 5, 69, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[588327, 23984, 158, 3, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[588488, 23989, 214, 6, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[588717, 23998, 462, 19, 25, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[589216, 24022, 92, 1, 36, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[589333, 24027, 257, 7, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[589593, 24036, 338, 10, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[589968, 24051, 152, 3, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[590123, 24056, 338, 10, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[590498, 24071, 186, 5, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[590687, 24078, 277, 9, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[591001, 24092, 156, 3, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[591160, 24097, 277, 9, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[591474, 24111, 190, 5, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[591667, 24118, 227, 7, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[591931, 24130, 156, 3, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[592090, 24135, 227, 7, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[592354, 24147, 239, 7, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[592596, 24156, 312, 10, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[592945, 24171, 184, 4, 28, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[593132, 24177, 565, 22, 27, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]]
}, Open  ]],
Cell[593724, 24203, 51, 1, 41, "ObjectName",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[593800, 24208, 58, 1, 36, "Subsubsection",
 CellTags->"Expand2"],
Cell[593861, 24211, 164, 4, 25, "Text",
 CellTags->"Expand2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[594062, 24220, 55, 1, 36, "Subsubsection",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[594142, 24225, 579, 21, 28, "Input",
 CellTags->"Expand2"],
Cell[594724, 24248, 574, 19, 27, "Output",
 CellTags->"Expand2"]
}, Open  ]]
}, Open  ]],
Cell[595325, 24271, 53, 1, 41, "ObjectName",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[595403, 24276, 59, 1, 36, "Subsubsection",
 CellTags->"Explicit"],
Cell[595465, 24279, 401, 9, 61, "Text",
 CellTags->"Explicit"],
Cell[595869, 24290, 331, 14, 25, "Text",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[596225, 24308, 88, 2, 28, "Input"],
Cell[596316, 24312, 345, 10, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[596710, 24328, 56, 1, 36, "Subsubsection",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[596791, 24333, 134, 3, 28, "Input"],
Cell[596928, 24338, 183, 6, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[597148, 24349, 86, 2, 28, "Input"],
Cell[597237, 24353, 241, 8, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[597515, 24366, 233, 6, 28, "Input"],
Cell[597751, 24374, 776, 28, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[598564, 24407, 230, 6, 28, "Input",
 CellTags->"Explicit"],
Cell[598797, 24415, 940, 42, 27, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[599774, 24462, 109, 3, 28, "Input",
 CellTags->"Explicit"],
Cell[599886, 24467, 1969, 78, 27, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[601892, 24550, 184, 5, 28, "Input",
 CellTags->"Explicit"],
Cell[602079, 24557, 892, 36, 38, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[603008, 24598, 109, 3, 28, "Input",
 CellTags->"Explicit"],
Cell[603120, 24603, 2641, 101, 38, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[605798, 24709, 155, 4, 28, "Input",
 CellTags->"Explicit"],
Cell[605956, 24715, 163, 5, 27, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[606156, 24725, 109, 3, 28, "Input",
 CellTags->"Explicit"],
Cell[606268, 24730, 1742, 77, 27, "Output",
 CellTags->"Explicit"]
}, Open  ]]
}, Open  ]],
Cell[608037, 24811, 77, 1, 41, "ObjectName",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[608139, 24816, 71, 1, 36, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[608213, 24819, 141, 4, 25, "Text",
 CellTags->"ExplicitLorentzIndex"],
Cell[608357, 24825, 295, 13, 25, "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[608689, 24843, 68, 1, 36, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[608782, 24848, 224, 6, 28, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[609009, 24856, 199, 7, 27, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[609245, 24868, 121, 3, 28, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[609369, 24873, 247, 6, 42, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]]
}, Open  ]],
Cell[609643, 24883, 69, 1, 41, "ObjectName",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[609737, 24888, 67, 1, 36, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[609807, 24891, 162, 4, 25, "Text",
 CellTags->"ExplicitPartialD"],
Cell[609972, 24897, 476, 18, 25, "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[610485, 24920, 64, 1, 36, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[610574, 24925, 183, 4, 28, "Input",
 CellTags->"ExplicitPartialD"],
Cell[610760, 24931, 525, 19, 38, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[611322, 24955, 184, 4, 28, "Input",
 CellTags->"ExplicitPartialD"],
Cell[611509, 24961, 430, 16, 33, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[611976, 24982, 186, 4, 28, "Input",
 CellTags->"ExplicitPartialD"],
Cell[612165, 24988, 461, 15, 38, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[612663, 25008, 183, 6, 28, "Input",
 CellTags->"ExplicitPartialD"],
Cell[612849, 25016, 270, 8, 35, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[613156, 25029, 125, 3, 28, "Input",
 CellTags->"ExplicitPartialD"],
Cell[613284, 25034, 440, 14, 38, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]]
}, Open  ]],
Cell[613751, 25052, 69, 1, 41, "ObjectName",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[613845, 25057, 67, 1, 36, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[613915, 25060, 178, 6, 25, "Text",
 CellTags->"ExplicitSUNIndex"],
Cell[614096, 25068, 484, 23, 25, "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614617, 25096, 64, 1, 36, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[614706, 25101, 125, 3, 28, "Input",
 CellTags->"ExplicitSUNIndex"],
Cell[614834, 25106, 112, 3, 27, "Output",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614983, 25114, 176, 6, 28, "Input"],
Cell[615162, 25122, 204, 5, 42, "Output"]
}, Open  ]]
}, Open  ]],
Cell[615393, 25131, 59, 1, 41, "ObjectName",
 CellTags->"ExtraFactor"],
Cell[CellGroupData[{
Cell[615477, 25136, 62, 1, 36, "Subsubsection",
 CellTags->"ExtraFactor"],
Cell[615542, 25139, 199, 4, 43, "Text",
 CellTags->"ExtraFactor"],
Cell[615744, 25145, 194, 8, 25, "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],
Cell[615953, 25156, 65, 1, 41, "ObjectName",
 CellTags->"ExtraVariables"],
Cell[CellGroupData[{
Cell[616043, 25161, 65, 1, 36, "Subsubsection",
 CellTags->"ExtraVariables"],
Cell[616111, 25164, 213, 5, 43, "Text",
 CellTags->"ExtraVariables"],
Cell[616327, 25171, 298, 14, 25, "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],
Cell[616640, 25188, 57, 1, 41, "ObjectName",
 CellTags->"FactorFull"],
Cell[CellGroupData[{
Cell[616722, 25193, 61, 1, 36, "Subsubsection",
 CellTags->"FactorFull"],
Cell[616786, 25196, 509, 24, 25, "Text",
 CellTags->"FactorFull"],
Cell[617298, 25222, 178, 9, 25, "Text",
 CellTags->"FactorFull"]
}, Open  ]],
Cell[617491, 25234, 55, 1, 41, "ObjectName",
 CellTags->"Factoring"],
Cell[CellGroupData[{
Cell[617571, 25239, 60, 1, 36, "Subsubsection",
 CellTags->"Factoring"],
Cell[617634, 25242, 235, 5, 43, "Text",
 CellTags->"Factoring"],
Cell[617872, 25249, 382, 19, 25, "Text",
 CellTags->"Factoring"]
}, Open  ]],
Cell[618269, 25271, 55, 1, 41, "ObjectName",
 CellTags->"Factorout"],
Cell[CellGroupData[{
Cell[618349, 25276, 60, 1, 36, "Subsubsection",
 CellTags->"Factorout"],
Cell[618412, 25279, 74, 1, 25, "Text",
 CellTags->"Factorout"],
Cell[618489, 25282, 174, 9, 25, "Text",
 CellTags->"Factorout"]
}, Open  ]],
Cell[618678, 25294, 57, 1, 41, "ObjectName",
 CellTags->"FactorTime"],
Cell[CellGroupData[{
Cell[618760, 25299, 61, 1, 36, "Subsubsection",
 CellTags->"FactorTime"],
Cell[618824, 25302, 166, 4, 25, "Text",
 CellTags->"FactorTime"],
Cell[618993, 25308, 178, 9, 25, "Text",
 CellTags->"FactorTime"]
}, Open  ]],
Cell[619186, 25320, 51, 1, 41, "ObjectName",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[619262, 25325, 58, 1, 36, "Subsubsection",
 CellTags->"Factor1"],
Cell[619323, 25328, 144, 4, 25, "Text",
 CellTags->"Factor1"],
Cell[619470, 25334, 175, 9, 25, "Text",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[619682, 25348, 55, 1, 36, "Subsubsection",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[619762, 25353, 213, 8, 28, "Input",
 CellTags->"Factor1"],
Cell[619978, 25363, 224, 8, 27, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[620239, 25376, 222, 7, 28, "Input",
 CellTags->"Factor1"],
Cell[620464, 25385, 435, 16, 27, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[620936, 25406, 251, 9, 28, "Input",
 CellTags->"Factor1"],
Cell[621190, 25417, 176, 6, 27, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[621403, 25428, 107, 3, 28, "Input",
 CellTags->"Factor1"],
Cell[621513, 25433, 224, 8, 27, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[621774, 25446, 108, 3, 28, "Input",
 CellTags->"Factor1"],
Cell[621885, 25451, 176, 6, 27, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[622076, 25460, 141, 4, 28, "Input",
 CellTags->"Factor1"]
}, Open  ]],
Cell[622232, 25467, 51, 1, 41, "ObjectName",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[622308, 25472, 58, 1, 36, "Subsubsection",
 CellTags->"Factor2"],
Cell[622369, 25475, 398, 7, 61, "Text",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[622792, 25486, 113, 3, 28, "Input",
 CellTags->"Factor2"],
Cell[622908, 25491, 230, 7, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[623153, 25501, 178, 9, 25, "Text",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[623368, 25515, 55, 1, 36, "Subsubsection",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[623448, 25520, 213, 8, 28, "Input",
 CellTags->"Factor2"],
Cell[623664, 25530, 224, 8, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[623925, 25543, 222, 7, 28, "Input",
 CellTags->"Factor2"],
Cell[624150, 25552, 435, 16, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[624622, 25573, 251, 9, 28, "Input",
 CellTags->"Factor2"],
Cell[624876, 25584, 176, 6, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[625089, 25595, 107, 3, 28, "Input",
 CellTags->"Factor2"],
Cell[625199, 25600, 224, 8, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[625460, 25613, 108, 3, 28, "Input",
 CellTags->"Factor2"],
Cell[625571, 25618, 176, 6, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[625784, 25629, 173, 5, 28, "Input",
 CellTags->"Factor2"],
Cell[625960, 25636, 224, 8, 27, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[626199, 25647, 141, 4, 28, "Input",
 CellTags->"Factor2"]
}, Open  ]],
Cell[626355, 25654, 43, 1, 70, "ObjectName",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[626423, 25659, 54, 1, 70, "Subsubsection",
 CellTags->"FAD"],
Cell[626480, 25662, 335, 6, 70, "Text",
 CellTags->"FAD"],
Cell[626818, 25670, 802, 34, 70, "Text",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[627657, 25709, 51, 1, 36, "Subsubsection",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[627733, 25714, 146, 5, 28, "Input",
 CellTags->"FAD"],
Cell[627882, 25721, 700, 20, 48, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[628619, 25746, 197, 7, 28, "Input",
 CellTags->"FAD"],
Cell[628819, 25755, 819, 22, 48, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[629675, 25782, 245, 9, 28, "Input",
 CellTags->"FAD"],
Cell[629923, 25793, 161, 5, 42, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[630121, 25803, 215, 8, 28, "Input",
 CellTags->"FAD"],
Cell[630339, 25813, 527, 16, 62, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[630903, 25834, 280, 9, 28, "Input",
 CellTags->"FAD"],
Cell[631186, 25845, 527, 16, 62, "Output",
 CellTags->"FAD"]
}, Open  ]]
}, Open  ]],
Cell[631740, 25865, 41, 1, 70, "ObjectName",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[631806, 25870, 53, 1, 70, "Subsubsection",
 CellTags->"FC"],
Cell[631862, 25873, 117, 3, 70, "Text",
 CellTags->"FC"],
Cell[631982, 25878, 545, 24, 70, "Text",
 CellTags->"FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[632564, 25907, 50, 1, 36, "Subsubsection",
 CellTags->"FC"],
Cell[632617, 25910, 70, 2, 28, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[632712, 25916, 227, 7, 28, "Input",
 CellTags->"FC"],
Cell[632942, 25925, 100, 2, 26, "Output",
 CellTags->"FC"]
}, Open  ]],
Cell[633057, 25930, 70, 2, 28, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[633152, 25936, 227, 7, 28, "Input",
 CellTags->"FC"],
Cell[633382, 25945, 305, 11, 27, "Output",
 CellTags->"FC"]
}, Open  ]]
}, Open  ]],
Cell[633714, 25960, 43, 1, 41, "ObjectName",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[633782, 25965, 54, 1, 36, "Subsubsection",
 CellTags->"FCE"],
Cell[633839, 25968, 126, 4, 25, "Text",
 CellTags->"FCE"],
Cell[633968, 25974, 72, 1, 25, "Text",
 CellTags->"FCE"],
Cell[634043, 25977, 427, 19, 25, "Text",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[634507, 26001, 51, 1, 36, "Subsubsection",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[634583, 26006, 262, 8, 28, "Input",
 CellTags->"FCE"],
Cell[634848, 26016, 225, 8, 27, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[635110, 26029, 104, 3, 28, "Input",
 CellTags->"FCE"],
Cell[635217, 26034, 190, 6, 42, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[635444, 26045, 437, 13, 28, "Input",
 CellTags->"FCE"],
Cell[635884, 26060, 547, 18, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[636468, 26083, 104, 3, 28, "Input",
 CellTags->"FCE"],
Cell[636575, 26088, 452, 13, 42, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[637064, 26106, 95, 3, 28, "Input",
 CellTags->"FCE"],
Cell[637162, 26111, 862, 34, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[638061, 26150, 104, 3, 28, "Input",
 CellTags->"FCE"],
Cell[638168, 26155, 893, 24, 82, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639098, 26184, 134, 4, 28, "Input",
 CellTags->"FCE"],
Cell[639235, 26190, 452, 13, 42, "Output",
 CellTags->"FCE"]
}, Open  ]]
}, Open  ]],
Cell[639714, 26207, 43, 1, 41, "ObjectName",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[639782, 26212, 54, 1, 36, "Subsubsection",
 CellTags->"FCI"],
Cell[639839, 26215, 159, 4, 25, "Text",
 CellTags->"FCI"],
Cell[640001, 26221, 427, 19, 25, "Text",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640465, 26245, 51, 1, 36, "Subsubsection",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[640541, 26250, 437, 13, 28, "Input",
 CellTags->"FCI"],
Cell[640981, 26265, 547, 18, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[641565, 26288, 104, 3, 28, "Input",
 CellTags->"FCI"],
Cell[641672, 26293, 452, 13, 42, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642161, 26311, 95, 3, 28, "Input",
 CellTags->"FCI"],
Cell[642259, 26316, 862, 34, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[643158, 26355, 104, 3, 28, "Input",
 CellTags->"FCI"],
Cell[643265, 26360, 893, 24, 82, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644195, 26389, 134, 4, 28, "Input",
 CellTags->"FCI"],
Cell[644332, 26395, 452, 13, 42, "Output",
 CellTags->"FCI"]
}, Open  ]]
}, Open  ]],
Cell[644811, 26412, 57, 1, 41, "ObjectName",
 CellTags->"FCIntegral"],
Cell[CellGroupData[{
Cell[644893, 26417, 61, 1, 36, "Subsubsection",
 CellTags->"FCIntegral"],
Cell[644957, 26420, 209, 5, 43, "Text",
 CellTags->"FCIntegral"],
Cell[645169, 26427, 360, 14, 25, "Text",
 CellTags->"FCIntegral"]
}, Open  ]],
Cell[645544, 26444, 59, 1, 41, "ObjectName",
 CellTags->"FCIntegrate"],
Cell[CellGroupData[{
Cell[645628, 26449, 62, 1, 36, "Subsubsection",
 CellTags->"FCIntegrate"],
Cell[645693, 26452, 287, 6, 43, "Text",
 CellTags->"FCIntegrate"],
Cell[645983, 26460, 194, 9, 25, "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],
Cell[646192, 26472, 61, 1, 41, "ObjectName",
 CellTags->"FCNIntegrate"],
Cell[CellGroupData[{
Cell[646278, 26477, 63, 1, 36, "Subsubsection",
 CellTags->"FCNIntegrate"],
Cell[646344, 26480, 418, 7, 61, "Text",
 CellTags->"FCNIntegrate"],
Cell[646765, 26489, 192, 9, 25, "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],
Cell[646972, 26501, 50, 1, 41, "ObjectName",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[647047, 26506, 57, 1, 36, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[647107, 26509, 306, 6, 61, "Text",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[647438, 26519, 111, 3, 28, "Input",
 CellTags->"FC2RHI"],
Cell[647552, 26524, 277, 8, 27, "Output",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[647866, 26537, 349, 12, 28, "Input"],
Cell[648218, 26551, 506, 18, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[648761, 26574, 119, 3, 28, "Input"],
Cell[648883, 26579, 473, 20, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[649393, 26604, 83, 2, 28, "Input"],
Cell[649479, 26608, 94, 1, 42, "Output"]
}, Open  ]],
Cell[649588, 26612, 162, 9, 25, "Text",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[649787, 26626, 54, 1, 36, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[649844, 26629, 72, 2, 28, "Input",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[649931, 26634, 50, 1, 41, "ObjectName",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[650006, 26639, 57, 1, 36, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[650066, 26642, 256, 5, 43, "Text",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[650347, 26651, 111, 3, 28, "Input",
 CellTags->"FC2TLI"],
Cell[650461, 26656, 277, 8, 27, "Output",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[650775, 26669, 71, 2, 28, "Input"],
Cell[650849, 26673, 321, 5, 116, "Print",
 CellTags->"Info3387711966-3304885"]
}, Open  ]],
Cell[CellGroupData[{
Cell[651207, 26683, 365, 12, 28, "Input"],
Cell[651575, 26697, 1483, 42, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[653095, 26744, 119, 3, 28, "Input"],
Cell[653217, 26749, 699, 29, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[653953, 26783, 83, 2, 28, "Input"],
Cell[654039, 26787, 100, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654176, 26793, 84, 2, 28, "Input"],
Cell[654263, 26797, 588, 21, 48, "Output"]
}, Open  ]],
Cell[654866, 26821, 162, 9, 25, "Text",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[655065, 26835, 54, 1, 36, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[655122, 26838, 72, 2, 28, "Input",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[655209, 26843, 43, 1, 41, "ObjectName",
 CellTags->"FDS"],
Cell[CellGroupData[{
Cell[655277, 26848, 54, 1, 36, "Subsubsection",
 CellTags->"FDS"],
Cell[655334, 26851, 82, 1, 25, "Text",
 CellTags->"FDS"],
Cell[655419, 26854, 228, 9, 25, "Text",
 CellTags->"FDS"]
}, Open  ]],
Cell[655662, 26866, 65, 1, 41, "ObjectName",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[655752, 26871, 65, 1, 36, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[655820, 26874, 118, 3, 25, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[655963, 26881, 127, 3, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[656093, 26886, 314, 9, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[656422, 26898, 521, 24, 25, "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[656980, 26927, 62, 1, 36, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[657045, 26930, 261, 12, 25, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[657331, 26946, 296, 9, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[657630, 26957, 353, 13, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658020, 26975, 143, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[658166, 26981, 330, 12, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658533, 26998, 143, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[658679, 27004, 133, 4, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658849, 27013, 296, 9, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[659148, 27024, 353, 13, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[659538, 27042, 143, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[659684, 27048, 330, 12, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660051, 27065, 143, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[660197, 27071, 152, 5, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660386, 27081, 334, 10, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[660723, 27093, 508, 17, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[661246, 27113, 164, 4, 25, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[661435, 27121, 147, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[661585, 27127, 540, 18, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[662162, 27150, 145, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[662310, 27156, 1074, 36, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[663421, 27197, 148, 4, 28, "Input",
 CellTags->"FermionSpinSum"],
Cell[663572, 27203, 837, 32, 27, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[664424, 27238, 141, 4, 28, "Input",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[664580, 27245, 51, 1, 41, "ObjectName",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[664656, 27250, 58, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[664717, 27253, 243, 5, 43, "Text",
 CellTags->"FeynAmp"],
Cell[664963, 27260, 165, 4, 25, "Text",
 CellTags->"FeynAmp"],
Cell[665131, 27266, 181, 9, 25, "Text",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[CellGroupData[{
Cell[665349, 27280, 55, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[665407, 27283, 125, 4, 25, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[665557, 27291, 846, 22, 50, "Input",
 CellTags->"FeynAmp"],
Cell[666406, 27315, 2586, 102, 41, "Output",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[669007, 27420, 73, 1, 25, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[669105, 27425, 198, 6, 28, "Input",
 CellTags->"FeynAmp"],
Cell[669306, 27433, 414, 14, 41, "Output",
 CellTags->"FeynAmp"]
}, Open  ]]
}, Open  ]],
Cell[669747, 27451, 73, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[669845, 27456, 69, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[669917, 27459, 256, 5, 43, "Text",
 CellTags->"FeynAmpDenominator"],
Cell[670176, 27466, 338, 14, 25, "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[670551, 27485, 66, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[670642, 27490, 201, 5, 28, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[670846, 27497, 211, 7, 42, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671094, 27509, 326, 9, 28, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[671423, 27520, 430, 15, 48, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[671868, 27538, 247, 7, 28, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[672140, 27549, 149, 4, 28, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[672292, 27555, 279, 7, 42, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[672608, 27567, 149, 4, 28, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[672760, 27573, 176, 5, 42, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[672951, 27581, 134, 4, 28, "Input",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[673100, 27588, 87, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[673212, 27593, 76, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[673291, 27596, 228, 5, 43, "Text",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[673522, 27603, 241, 9, 25, "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[673800, 27617, 73, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[673898, 27622, 234, 7, 28, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[674135, 27631, 765, 23, 45, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[674937, 27659, 208, 6, 28, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[675148, 27667, 203, 6, 42, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[675388, 27678, 206, 6, 28, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[675597, 27686, 233, 7, 42, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]]
}, Open  ]],
Cell[675857, 27697, 89, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[675971, 27702, 77, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[676051, 27705, 398, 7, 61, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[676452, 27714, 85, 1, 25, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[676562, 27719, 95, 2, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[676660, 27723, 147, 3, 27, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[676822, 27729, 218, 9, 25, "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[677077, 27743, 74, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[677154, 27746, 432, 14, 25, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[677611, 27764, 280, 8, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[677894, 27774, 122, 3, 27, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[678031, 27780, 468, 18, 27, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[678524, 27802, 328, 10, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[678855, 27814, 336, 11, 45, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679228, 27830, 260, 7, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[679491, 27839, 308, 10, 45, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679836, 27854, 339, 10, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[680178, 27866, 362, 12, 47, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[680577, 27883, 449, 14, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[681029, 27899, 1091, 40, 48, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[682157, 27944, 215, 6, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[682375, 27952, 795, 29, 47, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[683207, 27986, 157, 4, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[683367, 27992, 795, 29, 47, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[684199, 28026, 373, 13, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[684575, 28041, 1361, 50, 48, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[685951, 28094, 147, 4, 28, "Input",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[686113, 28101, 83, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[686221, 28106, 74, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[686298, 28109, 357, 7, 61, "Text",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[686658, 28118, 245, 9, 25, "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[686940, 28132, 71, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[687036, 28137, 308, 9, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[687347, 28148, 489, 16, 45, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687873, 28169, 116, 3, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[687992, 28174, 136, 3, 27, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[688165, 28182, 139, 3, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[688307, 28187, 489, 16, 45, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[688833, 28208, 150, 4, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[688986, 28214, 425, 13, 42, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[689448, 28232, 230, 7, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[689681, 28241, 368, 12, 42, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[690086, 28258, 206, 6, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[690295, 28266, 305, 9, 42, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[690615, 28278, 121, 3, 28, "Input",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[690751, 28284, 59, 1, 41, "ObjectName",
 CellTags->"FeynAmpList"],
Cell[CellGroupData[{
Cell[690835, 28289, 62, 1, 36, "Subsubsection",
 CellTags->"FeynAmpList"],
Cell[690900, 28292, 145, 4, 25, "Text",
 CellTags->"FeynAmpList"],
Cell[691048, 28298, 117, 3, 25, "Text",
 CellTags->"FeynAmpList"],
Cell[691168, 28303, 179, 9, 25, "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],
Cell[691362, 28315, 53, 1, 41, "ObjectName",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[691440, 28320, 59, 1, 36, "Subsubsection",
 CellTags->"FeynCalc"],
Cell[691502, 28323, 91, 1, 25, "Text",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[691618, 28328, 99, 3, 28, "Input",
 CellTags->"FeynCalc"],
Cell[691720, 28333, 703, 11, 344, "Print",
 CellTags->{"FeynCalc", "Info3387711973-8270308"}]
}, Open  ]]
}, Open  ]],
Cell[692450, 28348, 69, 1, 41, "ObjectName",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[692544, 28353, 67, 1, 36, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[692614, 28356, 156, 4, 25, "Text",
 CellTags->"FeynCalcExternal"],
Cell[692773, 28362, 211, 9, 25, "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[693021, 28376, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[693110, 28381, 162, 4, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[693275, 28387, 143, 4, 27, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[693455, 28396, 117, 3, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[693575, 28401, 126, 3, 42, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[693738, 28409, 450, 13, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[694191, 28424, 560, 18, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[694788, 28447, 117, 3, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[694908, 28452, 465, 13, 42, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[695410, 28470, 121, 3, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[695534, 28475, 875, 34, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[696446, 28514, 117, 3, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[696566, 28519, 906, 24, 82, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[697509, 28548, 163, 5, 28, "Input",
 CellTags->"FeynCalcExternal"],
Cell[697675, 28555, 465, 13, 42, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]]
}, Open  ]],
Cell[698167, 28572, 61, 1, 41, "ObjectName",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[698253, 28577, 63, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[698319, 28580, 534, 10, 79, "Text",
 CellTags->"FeynCalcForm"],
Cell[698856, 28592, 433, 19, 25, "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[699326, 28616, 60, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[699389, 28619, 79, 1, 25, "Text",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[699493, 28624, 242, 7, 28, "Input",
 CellTags->"FeynCalcForm"],
Cell[699738, 28633, 527, 22, 27, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[700280, 28658, 112, 3, 25, "Text",
 CellTags->"FeynCalcForm"],
Cell[700395, 28663, 148, 4, 28, "Input",
 CellTags->"FeynCalcForm"],
Cell[700546, 28669, 647, 19, 72, "Input",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[701218, 28692, 242, 7, 28, "Input",
 CellTags->"FeynCalcForm"],
Cell[701463, 28701, 191, 6, 32, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[701669, 28710, 76, 1, 25, "Text",
 CellTags->"FeynCalcForm"],
Cell[701748, 28713, 123, 4, 28, "Input",
 CellTags->"FeynCalcForm"],
Cell[701874, 28719, 652, 19, 50, "Input",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[702541, 28741, 69, 1, 41, "ObjectName",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[702635, 28746, 67, 1, 36, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[702705, 28749, 157, 4, 25, "Text",
 CellTags->"FeynCalcInternal"],
Cell[702865, 28755, 401, 19, 25, "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[703303, 28779, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[703392, 28784, 450, 13, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[703845, 28799, 560, 18, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[704442, 28822, 117, 3, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[704562, 28827, 465, 13, 42, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[705064, 28845, 121, 3, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[705188, 28850, 875, 34, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[706100, 28889, 117, 3, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[706220, 28894, 906, 24, 82, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[707163, 28923, 163, 5, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[707329, 28930, 465, 13, 42, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[707831, 28948, 517, 15, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[708351, 28965, 1461, 63, 42, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[709849, 29033, 117, 3, 28, "Input",
 CellTags->"FeynCalcInternal"],
Cell[709969, 29038, 1128, 30, 102, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]]
}, Open  ]],
Cell[711124, 29072, 63, 1, 41, "ObjectName",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[711212, 29077, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[711279, 29080, 226, 5, 43, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[711508, 29087, 199, 9, 25, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[711732, 29100, 125, 3, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[711860, 29105, 1022, 22, 71, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[712931, 29133, 61, 1, 36, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[713017, 29138, 273, 9, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[713293, 29149, 276, 9, 41, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[713606, 29163, 137, 4, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[713746, 29169, 68, 1, 22, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[713851, 29175, 199, 6, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[714053, 29183, 199, 6, 28, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[714289, 29194, 137, 4, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[714429, 29200, 66, 1, 22, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[714532, 29206, 212, 6, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[714747, 29214, 373, 11, 27, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[715157, 29230, 137, 4, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[715297, 29236, 80, 1, 22, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[715414, 29242, 298, 9, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[715715, 29253, 473, 16, 27, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[716225, 29274, 137, 4, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[716365, 29280, 80, 1, 22, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[716460, 29284, 328, 10, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[716813, 29298, 332, 10, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[717148, 29310, 154, 4, 27, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[717339, 29319, 314, 10, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[717656, 29331, 6179, 240, 49, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[723850, 29574, 209, 6, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[724084, 29584, 376, 11, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[724463, 29597, 1338, 36, 233, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[725816, 29636, 313, 10, 28, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[726132, 29648, 129, 4, 28, "Input",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[726276, 29655, 67, 1, 41, "ObjectName",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[726368, 29660, 66, 1, 36, "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],
Cell[726437, 29663, 300, 6, 69, "Text",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[726762, 29673, 158, 4, 28, "Input"],
Cell[726923, 29679, 477, 18, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[727437, 29702, 93, 2, 28, "Input"],
Cell[727533, 29706, 413, 7, 93, "Output"]
}, Open  ]]
}, Open  ]],
Cell[727973, 29717, 79, 1, 41, "ObjectName",
 CellTags->"FeynmanParameterNames"],
Cell[CellGroupData[{
Cell[728077, 29722, 72, 1, 36, "Subsubsection",
 CellTags->"FeynmanParameterNames"],
Cell[728152, 29725, 143, 4, 25, "Text",
 CellTags->"FeynmanParameterNames"],
Cell[728298, 29731, 362, 14, 25, "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],
Cell[728675, 29748, 90, 1, 41, "ObjectName",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[728790, 29753, 69, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[728862, 29756, 176, 4, 25, "Text",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[729063, 29764, 135, 3, 28, "Input",
 CellTags->"FeynmanParametrize"],
Cell[729201, 29769, 251, 8, 27, "Output",
 CellTags->"FeynmanParametrize"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[729501, 29783, 66, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[729570, 29786, 84, 2, 28, "Input",
 CellTags->"FeynmanParametrize"]
}, Open  ]],
Cell[729669, 29791, 53, 1, 41, "ObjectName",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[729747, 29796, 59, 1, 36, "Subsubsection",
 CellTags->"FeynRule"],
Cell[729809, 29799, 168, 4, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[730002, 29807, 115, 3, 28, "Input",
 CellTags->"FeynRule"],
Cell[730120, 29812, 630, 16, 49, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[730765, 29831, 92, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[730860, 29834, 133, 4, 25, "Text",
 CellTags->"FeynRule"],
Cell[730996, 29840, 185, 9, 25, "Text",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[731218, 29854, 56, 1, 36, "Subsubsection",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[731299, 29859, 146, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[731448, 29865, 680, 23, 38, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732165, 29893, 146, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[732314, 29899, 1003, 36, 38, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[733354, 29940, 111, 3, 28, "Input",
 CellTags->"FeynRule"],
Cell[733468, 29945, 4550, 176, 38, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[738055, 30126, 141, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[738199, 30132, 2654, 113, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[740868, 30248, 74, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[740967, 30253, 503, 16, 28, "Input",
 CellTags->"FeynRule"],
Cell[741473, 30271, 700, 30, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[742210, 30306, 307, 9, 28, "Input",
 CellTags->"FeynRule"],
Cell[742520, 30317, 3760, 137, 47, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[746295, 30457, 72, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[746392, 30462, 503, 16, 28, "Input",
 CellTags->"FeynRule"],
Cell[746898, 30480, 700, 30, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[747635, 30515, 307, 9, 28, "Input",
 CellTags->"FeynRule"],
Cell[747945, 30526, 4465, 151, 47, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752447, 30682, 218, 7, 28, "Input",
 CellTags->"FeynRule"],
Cell[752668, 30691, 1044, 42, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[753727, 30736, 110, 3, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[753862, 30743, 360, 10, 28, "Input",
 CellTags->"FeynRule"],
Cell[754225, 30755, 1108, 45, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[755348, 30803, 77, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[755450, 30808, 145, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[755598, 30814, 618, 22, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[756231, 30839, 90, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[756346, 30844, 145, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[756494, 30850, 618, 22, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[757149, 30877, 696, 21, 50, "Input",
 CellTags->"FeynRule"],
Cell[757848, 30900, 893, 37, 29, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[758778, 30942, 227, 7, 28, "Input",
 CellTags->"FeynRule"],
Cell[759008, 30951, 4903, 175, 107, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[763948, 31131, 420, 12, 28, "Input",
 CellTags->"FeynRule"],
Cell[764371, 31145, 4701, 156, 75, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[769109, 31306, 362, 12, 28, "Input",
 CellTags->"FeynRule"],
Cell[769474, 31320, 104, 3, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[769593, 31326, 210, 5, 43, "Text",
 CellTags->"FeynRule"],
Cell[769806, 31333, 144, 4, 28, "Input",
 CellTags->"FeynRule"],
Cell[769953, 31339, 51, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[770029, 31344, 693, 21, 50, "Input",
 CellTags->"FeynRule"],
Cell[770725, 31367, 984, 42, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[771746, 31414, 212, 7, 28, "Input",
 CellTags->"FeynRule"],
Cell[771961, 31423, 1969, 78, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[773967, 31506, 385, 11, 28, "Input",
 CellTags->"FeynRule"],
Cell[774355, 31519, 1419, 57, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[775811, 31581, 197, 6, 28, "Input",
 CellTags->"FeynRule"],
Cell[776011, 31589, 104, 3, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[776152, 31597, 884, 26, 50, "Input",
 CellTags->"FeynRule"],
Cell[777039, 31625, 1270, 54, 29, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[778346, 31684, 212, 7, 28, "Input",
 CellTags->"FeynRule"],
Cell[778561, 31693, 3296, 130, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[781894, 31828, 461, 13, 28, "Input",
 CellTags->"FeynRule"],
Cell[782358, 31843, 3461, 131, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[785856, 31979, 197, 6, 28, "Input",
 CellTags->"FeynRule"],
Cell[786056, 31987, 104, 3, 27, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[786175, 31993, 418, 9, 50, "Input",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[786608, 32005, 41, 1, 41, "ObjectName",
 CellTags->"FI"],
Cell[CellGroupData[{
Cell[786674, 32010, 53, 1, 36, "Subsubsection",
 CellTags->"FI"],
Cell[786730, 32013, 195, 4, 43, "Text",
 CellTags->"FI"],
Cell[786928, 32019, 545, 24, 25, "Text",
 CellTags->"FI"]
}, Open  ]],
Cell[787488, 32046, 67, 1, 41, "ObjectName",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[787580, 32051, 66, 1, 36, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[787649, 32054, 549, 9, 79, "Text",
 CellTags->"FieldDerivative"],
Cell[788201, 32065, 314, 14, 25, "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[788552, 32084, 63, 1, 36, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[788640, 32089, 620, 19, 50, "Input",
 CellTags->"FieldDerivative"],
Cell[789263, 32110, 465, 17, 27, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[789765, 32132, 218, 6, 28, "Input",
 CellTags->"FieldDerivative"],
Cell[789986, 32140, 1529, 55, 27, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[791552, 32200, 218, 6, 28, "Input",
 CellTags->"FieldDerivative"],
Cell[791773, 32208, 4269, 153, 49, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[796069, 32365, 63, 1, 41, "ObjectName",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[796157, 32370, 64, 1, 36, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[796224, 32373, 1285, 37, 61, "Text",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[797534, 32414, 125, 3, 28, "Input",
 CellTags->"FieldStrength"],
Cell[797662, 32419, 529, 15, 49, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[798206, 32437, 196, 9, 25, "Text",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[798439, 32451, 61, 1, 36, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[798525, 32456, 150, 4, 28, "Input",
 CellTags->"FieldStrength"],
Cell[798678, 32462, 174, 5, 27, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[798889, 32472, 160, 4, 28, "Input",
 CellTags->"FieldStrength"],
Cell[799052, 32478, 168, 5, 27, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[799257, 32488, 200, 5, 28, "Input",
 CellTags->"FieldStrength"],
Cell[799460, 32495, 722, 30, 27, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[800219, 32530, 210, 5, 28, "Input",
 CellTags->"FieldStrength"],
Cell[800432, 32537, 1503, 65, 27, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[801972, 32607, 241, 6, 28, "Input",
 CellTags->"FieldStrength"],
Cell[802216, 32615, 547, 15, 62, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[802800, 32635, 244, 6, 28, "Input",
 CellTags->"FieldStrength"],
Cell[803047, 32643, 545, 15, 62, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[803629, 32663, 291, 7, 28, "Input",
 CellTags->"FieldStrength"],
Cell[803923, 32672, 1503, 65, 27, "Output",
 CellTags->"FieldStrength"]
}, Open  ]]
}, Open  ]],
Cell[805453, 32741, 63, 1, 41, "ObjectName",
 CellTags->"FinalFunction"],
Cell[CellGroupData[{
Cell[805541, 32746, 64, 1, 36, "Subsubsection",
 CellTags->"FinalFunction"],
Cell[805608, 32749, 86, 1, 25, "Text",
 CellTags->"FinalFunction"],
Cell[805697, 32752, 190, 9, 25, "Text",
 CellTags->"FinalFunction"]
}, Open  ]],
Cell[805902, 32764, 73, 1, 41, "ObjectName",
 CellTags->"FinalSubstitutions"],
Cell[CellGroupData[{
Cell[806000, 32769, 69, 1, 36, "Subsubsection",
 CellTags->"FinalSubstitutions"],
Cell[806072, 32772, 204, 4, 43, "Text",
 CellTags->"FinalSubstitutions"],
Cell[806279, 32778, 406, 19, 25, "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],
Cell[806700, 32800, 45, 1, 41, "ObjectName",
 CellTags->"FORM"],
Cell[CellGroupData[{
Cell[806770, 32805, 55, 1, 36, "Subsubsection",
 CellTags->"FORM"],
Cell[806828, 32808, 204, 5, 43, "Text",
 CellTags->"FORM"],
Cell[807035, 32815, 285, 14, 25, "Text",
 CellTags->"FORM"]
}, Open  ]],
Cell[807335, 32832, 57, 1, 41, "ObjectName",
 CellTags->"FORMEpilog"],
Cell[CellGroupData[{
Cell[807417, 32837, 61, 1, 36, "Subsubsection",
 CellTags->"FORMEpilog"],
Cell[807481, 32840, 161, 4, 25, "Text",
 CellTags->"FORMEpilog"],
Cell[807645, 32846, 312, 14, 25, "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],
Cell[807972, 32863, 57, 1, 41, "ObjectName",
 CellTags->"FORMProlog"],
Cell[CellGroupData[{
Cell[808054, 32868, 61, 1, 36, "Subsubsection",
 CellTags->"FORMProlog"],
Cell[808118, 32871, 178, 4, 25, "Text",
 CellTags->"FORMProlog"],
Cell[808299, 32877, 312, 14, 25, "Text",
 CellTags->"FORMProlog"]
}, Open  ]],
Cell[808626, 32894, 63, 1, 41, "ObjectName",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[808714, 32899, 64, 1, 36, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[808781, 32902, 470, 8, 79, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[809254, 32912, 199, 9, 25, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[809478, 32925, 125, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[809606, 32930, 662, 18, 49, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[810317, 32954, 61, 1, 36, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[810403, 32959, 139, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[810545, 32964, 354, 14, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[810936, 32983, 114, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[811053, 32988, 243, 8, 42, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[811311, 32999, 159, 4, 25, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[811495, 33007, 314, 8, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[811812, 33017, 405, 14, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[812254, 33036, 114, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[812371, 33041, 303, 9, 43, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[812711, 33055, 150, 4, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[812864, 33061, 305, 10, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[813206, 33076, 117, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[813326, 33081, 237, 8, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[813600, 33094, 150, 4, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[813753, 33100, 257, 9, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[814047, 33114, 114, 3, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[814164, 33119, 307, 10, 42, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[814508, 33134, 333, 9, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[814844, 33145, 269, 9, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[815150, 33159, 195, 5, 28, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[815348, 33166, 583, 18, 27, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[815958, 33188, 65, 1, 41, "ObjectName",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[816048, 33193, 65, 1, 36, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[816116, 33196, 242, 5, 43, "Text",
 CellTags->"FourDivergence"],
Cell[816361, 33203, 197, 9, 25, "Text",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[816595, 33217, 62, 1, 36, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[816682, 33222, 167, 5, 28, "Input",
 CellTags->"FourDivergence"],
Cell[816852, 33229, 212, 8, 27, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[817101, 33242, 211, 6, 28, "Input",
 CellTags->"FourDivergence"],
Cell[817315, 33250, 242, 10, 27, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[817594, 33265, 192, 6, 28, "Input",
 CellTags->"FourDivergence"],
Cell[817789, 33273, 247, 9, 27, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[818073, 33287, 237, 7, 28, "Input",
 CellTags->"FourDivergence"],
Cell[818313, 33296, 110, 3, 27, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[818438, 33302, 112, 3, 28, "Input",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[818565, 33308, 63, 1, 41, "ObjectName",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[818653, 33313, 64, 1, 36, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[818720, 33316, 426, 16, 25, "Text",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[819171, 33336, 125, 3, 28, "Input",
 CellTags->"FourLaplacian"],
Cell[819299, 33341, 170, 5, 27, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[819484, 33349, 324, 14, 25, "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[819845, 33368, 61, 1, 36, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[819931, 33373, 131, 4, 28, "Input",
 CellTags->"FourLaplacian"],
Cell[820065, 33379, 167, 6, 27, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[820269, 33390, 152, 4, 28, "Input",
 CellTags->"FourLaplacian"],
Cell[820424, 33396, 132, 4, 27, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[820593, 33405, 264, 9, 28, "Input",
 CellTags->"FourLaplacian"],
Cell[820860, 33416, 440, 16, 48, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[821337, 33437, 157, 4, 28, "Input",
 CellTags->"FourLaplacian"],
Cell[821497, 33443, 3872, 136, 95, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]]
}, Open  ]],
Cell[825396, 33583, 57, 1, 41, "ObjectName",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[825478, 33588, 61, 1, 36, "Subsubsection",
 CellTags->"FourVector"],
Cell[825542, 33591, 339, 10, 43, "Text",
 CellTags->"FourVector"],
Cell[825884, 33603, 356, 19, 25, "Text",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[826277, 33627, 58, 1, 36, "Subsubsection",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[826360, 33632, 140, 4, 28, "Input",
 CellTags->"FourVector"],
Cell[826503, 33638, 238, 10, 27, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[826778, 33653, 164, 5, 28, "Input",
 CellTags->"FourVector"],
Cell[826945, 33660, 290, 12, 27, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[827272, 33677, 180, 5, 28, "Input",
 CellTags->"FourVector"],
Cell[827455, 33684, 225, 6, 42, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[827717, 33695, 229, 6, 28, "Input",
 CellTags->"FourVector"],
Cell[827949, 33703, 275, 8, 42, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[828261, 33716, 212, 6, 28, "Input",
 CellTags->"FourVector"],
Cell[828476, 33724, 147, 4, 42, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[828638, 33731, 193, 4, 43, "Text",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[828856, 33739, 212, 6, 28, "Input",
 CellTags->"FourVector"],
Cell[829071, 33747, 407, 18, 27, "Output",
 CellTags->"FourVector"]
}, Open  ]]
}, Open  ]],
Cell[829505, 33769, 55, 1, 41, "ObjectName",
 CellTags->"FreeIndex"],
Cell[CellGroupData[{
Cell[829585, 33774, 60, 1, 36, "Subsubsection",
 CellTags->"FreeIndex"],
Cell[829648, 33777, 152, 4, 25, "Text",
 CellTags->"FreeIndex"],
Cell[829803, 33783, 290, 14, 25, "Text",
 CellTags->"FreeIndex"]
}, Open  ]],
Cell[830108, 33800, 49, 1, 41, "ObjectName",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[830182, 33805, 57, 1, 36, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[830242, 33808, 228, 5, 43, "Text",
 CellTags->"FreeQ2"],
Cell[830473, 33815, 308, 14, 25, "Text",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[830818, 33834, 54, 1, 36, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[830897, 33839, 246, 8, 28, "Input",
 CellTags->"FreeQ2"],
Cell[831146, 33849, 106, 3, 27, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831289, 33857, 241, 8, 28, "Input",
 CellTags->"FreeQ2"],
Cell[831533, 33867, 105, 3, 27, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831675, 33875, 133, 4, 28, "Input",
 CellTags->"FreeQ2"],
Cell[831811, 33881, 105, 3, 27, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831953, 33889, 162, 5, 28, "Input",
 CellTags->"FreeQ2"],
Cell[832118, 33896, 106, 3, 27, "Output",
 CellTags->"FreeQ2"]
}, Open  ]]
}, Open  ]],
Cell[832251, 33903, 43, 1, 41, "ObjectName",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[832319, 33908, 54, 1, 36, "Subsubsection",
 CellTags->"FRH"],
Cell[832376, 33911, 134, 4, 25, "Text",
 CellTags->"FRH"],
Cell[832513, 33917, 171, 9, 25, "Text",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[832721, 33931, 51, 1, 36, "Subsubsection",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[832797, 33936, 200, 6, 28, "Input",
 CellTags->"FRH"],
Cell[833000, 33944, 248, 9, 27, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[833285, 33958, 99, 3, 28, "Input",
 CellTags->"FRH"],
Cell[833387, 33963, 99, 3, 27, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[833523, 33971, 304, 10, 28, "Input",
 CellTags->"FRH"],
Cell[833830, 33983, 525, 20, 27, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[834392, 34008, 128, 4, 28, "Input",
 CellTags->"FRH"],
Cell[834523, 34014, 536, 21, 52, "Output",
 CellTags->"FRH"]
}, Open  ]]
}, Open  ]],
Cell[835086, 34039, 51, 1, 41, "ObjectName",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[835162, 34044, 58, 1, 36, "Subsubsection",
 CellTags->"FromTFi"],
Cell[835223, 34047, 200, 5, 43, "Text",
 CellTags->"FromTFi"],
Cell[835426, 34054, 258, 13, 25, "Text",
 CellTags->"FromTFI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[835721, 34072, 55, 1, 36, "Subsubsection",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[835801, 34077, 431, 14, 28, "Input",
 CellTags->"FromTFi"],
Cell[836235, 34093, 945, 36, 28, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[837217, 34134, 693, 22, 28, "Input",
 CellTags->"FromTFi"],
Cell[837913, 34158, 639, 22, 48, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[838589, 34185, 756, 24, 28, "Input",
 CellTags->"FromTFi"],
Cell[839348, 34211, 842, 30, 49, "Output",
 CellTags->"FromTFi"]
}, Open  ]]
}, Open  ]],
Cell[840217, 34245, 53, 1, 41, "ObjectName",
 CellTags->"FUNCTION"],
Cell[CellGroupData[{
Cell[840295, 34250, 59, 1, 36, "Subsubsection",
 CellTags->"FUNCTION"],
Cell[840357, 34253, 160, 4, 25, "Text",
 CellTags->"FUNCTION"],
Cell[840520, 34259, 173, 9, 25, "Text",
 CellTags->"FUNCTION"]
}, Open  ]],
Cell[840708, 34271, 59, 1, 41, "ObjectName",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[840792, 34276, 62, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[840857, 34279, 416, 7, 61, "Text",
 CellTags->"FunctionalD"],
Cell[841276, 34288, 95, 1, 25, "Text",
 CellTags->"FunctionalD"],
Cell[841374, 34291, 304, 14, 25, "Text",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[841715, 34310, 59, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[841777, 34313, 610, 20, 42, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[842412, 34337, 228, 6, 28, "Input",
 CellTags->"FunctionalD"],
Cell[842643, 34345, 107, 3, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[842787, 34353, 253, 7, 28, "Input",
 CellTags->"FunctionalD"],
Cell[843043, 34362, 135, 4, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[843193, 34369, 1269, 44, 49, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[844487, 34417, 295, 8, 28, "Input",
 CellTags->"FunctionalD"],
Cell[844785, 34427, 273, 10, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[845073, 34440, 778, 24, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[845876, 34468, 668, 22, 50, "Input",
 CellTags->"FunctionalD"],
Cell[846547, 34492, 735, 26, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[847319, 34523, 217, 6, 28, "Input",
 CellTags->"FunctionalD"],
Cell[847539, 34531, 626, 25, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[848180, 34559, 489, 15, 25, "Text",
 CellTags->"FunctionalD"],
Cell[848672, 34576, 97, 2, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[848794, 34582, 315, 8, 28, "Input",
 CellTags->"FunctionalD"],
Cell[849112, 34592, 1303, 56, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[850452, 34653, 315, 8, 28, "Input",
 CellTags->"FunctionalD"],
Cell[850770, 34663, 1303, 56, 28, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[852110, 34724, 216, 8, 28, "Input",
 CellTags->"FunctionalD"],
Cell[852329, 34734, 2956, 115, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[855300, 34852, 560, 13, 61, "Text",
 CellTags->"FunctionalD"],
Cell[855863, 34867, 1383, 49, 25, "Text",
 CellTags->"FunctionalD"],
Cell[857249, 34918, 445, 13, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[857719, 34935, 231, 7, 28, "Input",
 CellTags->"FunctionalD"],
Cell[857953, 34944, 286, 12, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[858276, 34961, 165, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[858444, 34968, 2260, 95, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[860719, 35066, 120, 3, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[860864, 35073, 176, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[861043, 35080, 2260, 95, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[863318, 35178, 290, 8, 43, "Text",
 CellTags->"FunctionalD"],
Cell[863611, 35188, 523, 16, 28, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[864159, 35208, 230, 7, 28, "Input",
 CellTags->"FunctionalD"],
Cell[864392, 35217, 384, 17, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864813, 35239, 240, 7, 28, "Input",
 CellTags->"FunctionalD"],
Cell[865056, 35248, 416, 18, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[865509, 35271, 583, 17, 50, "Input",
 CellTags->"FunctionalD"],
Cell[866095, 35290, 2900, 126, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[869032, 35421, 184, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[869219, 35428, 1405, 62, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[870661, 35495, 352, 11, 28, "Input",
 CellTags->"FunctionalD"],
Cell[871016, 35508, 1159, 50, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[872190, 35561, 269, 9, 25, "Text",
 CellTags->"FunctionalD"],
Cell[872462, 35572, 98, 2, 25, "Text",
 CellTags->"FunctionalD"],
Cell[872563, 35576, 167, 4, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[872755, 35584, 105, 3, 28, "Input",
 CellTags->"FunctionalD"],
Cell[872863, 35589, 2956, 115, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[875856, 35709, 194, 6, 28, "Input",
 CellTags->"FunctionalD"],
Cell[876053, 35717, 23406, 1004, 140, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[899474, 36724, 227, 5, 43, "Text",
 CellTags->"FunctionalD"],
Cell[899704, 36731, 345, 6, 61, "Text",
 CellTags->"FunctionalD"],
Cell[900052, 36739, 132, 4, 28, "Input",
 CellTags->"FunctionalD"],
Cell[900187, 36745, 2583, 67, 138, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[902795, 36816, 169, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[902967, 36823, 12134, 525, 72, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915138, 37353, 164, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[915305, 37360, 6459, 285, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[921779, 37648, 212, 8, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[922016, 37660, 315, 11, 28, "Input",
 CellTags->"FunctionalD"],
Cell[922334, 37673, 2679, 119, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[925050, 37797, 651, 21, 28, "Input",
 CellTags->"FunctionalD"],
Cell[925704, 37820, 1310, 60, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[927051, 37885, 154, 4, 28, "Input",
 CellTags->"FunctionalD"],
Cell[927208, 37891, 5152, 227, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[932375, 38121, 131, 4, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[932531, 38129, 79, 2, 28, "Input",
 CellTags->"FunctionalD"],
Cell[932613, 38133, 1159, 50, 27, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[933809, 38188, 798, 21, 50, "Input",
 CellTags->"FunctionalD"],
Cell[934610, 38211, 1064, 41, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[935711, 38257, 137, 4, 28, "Input",
 CellTags->"FunctionalD"],
Cell[935851, 38263, 4174, 167, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[940062, 38435, 204, 6, 28, "Input",
 CellTags->"FunctionalD"],
Cell[940269, 38443, 6023, 266, 28, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[946329, 38714, 138, 4, 28, "Input",
 CellTags->"FunctionalD"],
Cell[946470, 38720, 11159, 491, 49, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[957666, 39216, 163, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[957832, 39223, 10288, 452, 49, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[968135, 39678, 66, 1, 25, "Text",
 CellTags->"FunctionalD"],
Cell[968204, 39681, 539, 13, 50, "Input",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[968780, 39699, 110, 2, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[968893, 39703, 79, 1, 25, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[968997, 39708, 1073, 26, 94, "Input"],
Cell[970073, 39736, 1337, 59, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[971447, 39800, 765, 22, 50, "Input",
 CellTags->"FunctionalD"],
Cell[972215, 39824, 8894, 328, 61, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[981146, 40157, 165, 5, 28, "Input",
 CellTags->"FunctionalD"],
Cell[981314, 40164, 2064, 82, 38, "Output",
 CellTags->"FunctionalD"]
}, Open  ]]
}, Open  ]],
Cell[983405, 40250, 41, 1, 41, "ObjectName",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[983471, 40255, 53, 1, 36, "Subsubsection",
 CellTags->"FV"],
Cell[983527, 40258, 275, 13, 25, "Text",
 CellTags->"FV"],
Cell[983805, 40273, 562, 29, 25, "Text",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[984404, 40307, 50, 1, 36, "Subsubsection",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[984479, 40312, 124, 4, 28, "Input",
 CellTags->"FV"],
Cell[984606, 40318, 126, 4, 27, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[984769, 40327, 148, 5, 28, "Input",
 CellTags->"FV"],
Cell[984920, 40334, 185, 7, 27, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[985142, 40346, 160, 5, 28, "Input",
 CellTags->"FV"],
Cell[985305, 40353, 139, 4, 42, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[985481, 40362, 195, 7, 28, "Input",
 CellTags->"FV"],
Cell[985679, 40371, 217, 6, 42, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[985911, 40380, 130, 4, 25, "Text",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[986066, 40388, 196, 6, 28, "Input",
 CellTags->"FV"],
Cell[986265, 40396, 185, 7, 27, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[986487, 40408, 107, 3, 28, "Input",
 CellTags->"FV"],
Cell[986597, 40413, 163, 5, 42, "Output",
 CellTags->"FV"]
}, Open  ]]
}, Open  ]],
Cell[986787, 40422, 43, 1, 41, "ObjectName",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[986855, 40427, 54, 1, 36, "Subsubsection",
 CellTags->"FVD"],
Cell[986912, 40430, 312, 15, 25, "Text",
 CellTags->"FVD"],
Cell[987227, 40447, 560, 29, 25, "Text",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[987824, 40481, 51, 1, 36, "Subsubsection",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[987900, 40486, 126, 4, 28, "Input",
 CellTags->"FVD"],
Cell[988029, 40492, 127, 4, 27, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[988193, 40501, 150, 5, 28, "Input",
 CellTags->"FVD"],
Cell[988346, 40508, 186, 7, 27, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[988569, 40520, 162, 5, 28, "Input",
 CellTags->"FVD"],
Cell[988734, 40527, 141, 4, 42, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[988912, 40536, 197, 7, 28, "Input",
 CellTags->"FVD"],
Cell[989112, 40545, 268, 8, 42, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[989395, 40556, 88, 1, 25, "Text",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[989508, 40561, 198, 6, 28, "Input",
 CellTags->"FVD"],
Cell[989709, 40569, 186, 7, 27, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[989932, 40581, 108, 3, 28, "Input",
 CellTags->"FVD"],
Cell[990043, 40586, 165, 5, 42, "Output",
 CellTags->"FVD"]
}, Open  ]]
}, Open  ]],
Cell[990235, 40595, 41, 1, 41, "ObjectName",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[990301, 40600, 53, 1, 36, "Subsubsection",
 CellTags->"GA"],
Cell[990357, 40603, 654, 24, 43, "Text",
 CellTags->"GA"],
Cell[991014, 40629, 369, 19, 25, "Text",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[991420, 40653, 50, 1, 36, "Subsubsection",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[991495, 40658, 101, 3, 28, "Input",
 CellTags->"GA"],
Cell[991599, 40663, 133, 4, 27, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[991769, 40672, 213, 7, 28, "Input",
 CellTags->"GA"],
Cell[991985, 40681, 317, 10, 27, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[992339, 40696, 171, 5, 28, "Input",
 CellTags->"GA"],
Cell[992513, 40703, 163, 4, 42, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[992713, 40712, 163, 5, 28, "Input",
 CellTags->"GA"],
Cell[992879, 40719, 289, 8, 27, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[993205, 40732, 203, 6, 28, "Input",
 CellTags->"GA"],
Cell[993411, 40740, 262, 7, 42, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[993710, 40752, 250, 8, 28, "Input",
 CellTags->"GA"],
Cell[993963, 40762, 301, 9, 28, "Output",
 CellTags->"GA"]
}, Open  ]]
}, Open  ]],
Cell[994291, 40775, 43, 1, 41, "ObjectName",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[994359, 40780, 54, 1, 36, "Subsubsection",
 CellTags->"GAD"],
Cell[994416, 40783, 671, 24, 43, "Text",
 CellTags->"GAD"],
Cell[995090, 40809, 367, 19, 25, "Text",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[995494, 40833, 51, 1, 36, "Subsubsection",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[995570, 40838, 103, 3, 28, "Input",
 CellTags->"GAD"],
Cell[995676, 40843, 134, 4, 27, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[995847, 40852, 216, 7, 28, "Input",
 CellTags->"GAD"],
Cell[996066, 40861, 318, 10, 27, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[996421, 40876, 173, 5, 28, "Input",
 CellTags->"GAD"],
Cell[996597, 40883, 213, 6, 42, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[996847, 40894, 165, 5, 28, "Input",
 CellTags->"GAD"],
Cell[997015, 40901, 290, 8, 27, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[997342, 40914, 205, 6, 28, "Input",
 CellTags->"GAD"],
Cell[997550, 40922, 267, 7, 42, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[997854, 40934, 254, 8, 28, "Input",
 CellTags->"GAD"],
Cell[998111, 40944, 302, 9, 28, "Output",
 CellTags->"GAD"]
}, Open  ]]
}, Open  ]],
Cell[998440, 40957, 61, 1, 41, "ObjectName",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[998526, 40962, 63, 1, 36, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[998592, 40965, 167, 4, 25, "Text",
 CellTags->"GammaEpsilon"],
Cell[998762, 40971, 299, 14, 25, "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[999098, 40990, 60, 1, 36, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[999161, 40993, 141, 4, 25, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[999327, 41001, 170, 5, 28, "Input",
 CellTags->"GammaEpsilon"],
Cell[999500, 41008, 2073, 75, 40, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1001610, 41088, 170, 5, 28, "Input",
 CellTags->"GammaEpsilon"],
Cell[1001783, 41095, 1834, 69, 43, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[1003632, 41167, 91, 1, 25, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1003748, 41172, 123, 3, 28, "Input",
 CellTags->"GammaEpsilon"],
Cell[1003874, 41177, 3083, 109, 90, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1006994, 41291, 117, 3, 28, "Input",
 CellTags->"GammaEpsilon"],
Cell[1007114, 41296, 150, 4, 27, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]]
}, Open  ]],
Cell[1007291, 41304, 59, 1, 41, "ObjectName",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1007375, 41309, 62, 1, 36, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[1007440, 41312, 123, 3, 25, "Text",
 CellTags->"GammaExpand"],
Cell[1007566, 41317, 194, 9, 25, "Text",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1007797, 41331, 59, 1, 36, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1007881, 41336, 187, 5, 28, "Input",
 CellTags->"GammaExpand"],
Cell[1008071, 41343, 349, 12, 27, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1008457, 41360, 197, 6, 28, "Input",
 CellTags->"GammaExpand"],
Cell[1008657, 41368, 652, 23, 44, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1009346, 41396, 187, 5, 28, "Input",
 CellTags->"GammaExpand"],
Cell[1009536, 41403, 222, 7, 27, "Output",
 CellTags->"GammaExpand"]
}, Open  ]]
}, Open  ]],
Cell[1009785, 41414, 49, 1, 41, "ObjectName",
 CellTags->"Gamma1"],
Cell[CellGroupData[{
Cell[1009859, 41419, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma1"],
Cell[1009919, 41422, 139, 4, 25, "Text",
 CellTags->"Gamma1"],
Cell[1010061, 41428, 275, 14, 25, "Text",
 CellTags->"Gamma1"]
}, Open  ]],
Cell[1010351, 41445, 49, 1, 41, "ObjectName",
 CellTags->"Gamma2"],
Cell[CellGroupData[{
Cell[1010425, 41450, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma2"],
Cell[1010485, 41453, 172, 4, 43, "Text",
 CellTags->"Gamma2"],
Cell[1010660, 41459, 275, 14, 25, "Text",
 CellTags->"Gamma2"]
}, Open  ]],
Cell[1010950, 41476, 49, 1, 41, "ObjectName",
 CellTags->"Gamma3"],
Cell[CellGroupData[{
Cell[1011024, 41481, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma3"],
Cell[1011084, 41484, 237, 5, 43, "Text",
 CellTags->"Gamma3"],
Cell[1011324, 41491, 275, 14, 25, "Text",
 CellTags->"Gamma3"]
}, Open  ]],
Cell[1011614, 41508, 47, 1, 41, "ObjectName",
 CellTags->"Gauge"],
Cell[CellGroupData[{
Cell[1011686, 41513, 56, 1, 36, "Subsubsection",
 CellTags->"Gauge"],
Cell[1011745, 41516, 133, 4, 25, "Text",
 CellTags->"Gauge"],
Cell[1011881, 41522, 301, 14, 25, "Text",
 CellTags->"Gauge"]
}, Open  ]],
Cell[1012197, 41539, 57, 1, 41, "ObjectName",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1012279, 41544, 61, 1, 36, "Subsubsection",
 CellTags->"GaugeField"],
Cell[1012343, 41547, 212, 5, 43, "Text",
 CellTags->"GaugeField"],
Cell[1012558, 41554, 318, 14, 25, "Text",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1012913, 41573, 58, 1, 36, "Subsubsection",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1012996, 41578, 86, 2, 28, "Input",
 CellTags->"GaugeField"],
Cell[1013085, 41582, 106, 3, 27, "Output",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1013228, 41590, 237, 6, 28, "Input",
 CellTags->"GaugeField"],
Cell[1013468, 41598, 316, 14, 27, "Output",
 CellTags->"GaugeField"]
}, Open  ]]
}, Open  ]],
Cell[1013811, 41616, 51, 1, 41, "ObjectName",
 CellTags->"GaugeXi"],
Cell[CellGroupData[{
Cell[1013887, 41621, 58, 1, 36, "Subsubsection",
 CellTags->"GaugeXi"],
Cell[1013948, 41624, 77, 1, 25, "Text",
 CellTags->"GaugeXi"],
Cell[1014028, 41627, 285, 14, 25, "Text",
 CellTags->"GaugeXi"]
}, Open  ]],
Cell[1014328, 41644, 43, 1, 41, "ObjectName",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1014396, 41649, 54, 1, 36, "Subsubsection",
 CellTags->"GA5"],
Cell[1014453, 41652, 88, 1, 25, "Text",
 CellTags->"GA5"],
Cell[1014544, 41655, 364, 19, 25, "Text",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1014945, 41679, 51, 1, 36, "Subsubsection",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1015021, 41684, 72, 2, 28, "Input",
 CellTags->"GA5"],
Cell[1015096, 41688, 130, 4, 27, "Output",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1015263, 41697, 94, 3, 28, "Input",
 CellTags->"GA5"],
Cell[1015360, 41702, 99, 3, 27, "Output",
 CellTags->"GA5"]
}, Open  ]]
}, Open  ]],
Cell[1015486, 41709, 43, 1, 41, "ObjectName",
 CellTags->"GGV"],
Cell[CellGroupData[{
Cell[1015554, 41714, 54, 1, 36, "Subsubsection",
 CellTags->"GGV"],
Cell[1015611, 41717, 72, 1, 25, "Text",
 CellTags->"GGV"],
Cell[1015686, 41720, 198, 9, 25, "Text",
 CellTags->"GGV"]
}, Open  ]],
Cell[1015899, 41732, 67, 1, 41, "ObjectName",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1015991, 41737, 66, 1, 36, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[1016060, 41740, 324, 9, 25, "Text",
 CellTags->"GhostPropagator"],
Cell[1016387, 41751, 436, 19, 25, "Text",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1016848, 41774, 99, 2, 28, "Input"],
Cell[1016950, 41778, 148, 5, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1017147, 41789, 63, 1, 36, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1017235, 41794, 156, 4, 28, "Input",
 CellTags->"GhostPropagator"],
Cell[1017394, 41800, 193, 6, 27, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1017624, 41811, 206, 5, 28, "Input",
 CellTags->"GhostPropagator"],
Cell[1017833, 41818, 409, 16, 44, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1018279, 41839, 123, 3, 28, "Input",
 CellTags->"GhostPropagator"],
Cell[1018405, 41844, 176, 6, 27, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1018618, 41855, 188, 5, 28, "Input",
 CellTags->"GhostPropagator"],
Cell[1018809, 41862, 413, 16, 44, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1019259, 41883, 285, 9, 28, "Input",
 CellTags->"GhostPropagator"],
Cell[1019547, 41894, 228, 6, 42, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1019802, 41904, 43, 1, 41, "ObjectName",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1019870, 41909, 54, 1, 36, "Subsubsection",
 CellTags->"GHP"],
Cell[1019927, 41912, 288, 9, 25, "Text",
 CellTags->"GHP"],
Cell[1020218, 41923, 460, 19, 25, "Text",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1020715, 41947, 51, 1, 36, "Subsubsection",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1020791, 41952, 132, 4, 28, "Input",
 CellTags->"GHP"],
Cell[1020926, 41958, 181, 6, 27, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1021144, 41969, 130, 4, 28, "Input",
 CellTags->"GHP"],
Cell[1021277, 41975, 157, 5, 43, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1021471, 41985, 132, 4, 28, "Input",
 CellTags->"GHP"],
Cell[1021606, 41991, 185, 6, 27, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1021828, 42002, 264, 9, 28, "Input",
 CellTags->"GHP"],
Cell[1022095, 42013, 216, 6, 42, "Output",
 CellTags->"GHP"]
}, Open  ]]
}, Open  ]],
Cell[1022338, 42023, 57, 1, 41, "ObjectName",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1022420, 42028, 61, 1, 36, "Subsubsection",
 CellTags->"GluonField"],
Cell[1022484, 42031, 79, 1, 25, "Text",
 CellTags->"GluonField"],
Cell[1022566, 42034, 187, 9, 25, "Text",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1022790, 42048, 58, 1, 36, "Subsubsection",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1022873, 42053, 86, 2, 28, "Input",
 CellTags->"GluonField"],
Cell[1022962, 42057, 106, 3, 27, "Output",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1023105, 42065, 237, 6, 28, "Input",
 CellTags->"GluonField"],
Cell[1023345, 42073, 316, 14, 27, "Output",
 CellTags->"GluonField"]
}, Open  ]]
}, Open  ]],
Cell[1023688, 42091, 69, 1, 41, "ObjectName",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1023782, 42096, 67, 1, 36, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[1023852, 42099, 422, 7, 61, "Text",
 CellTags->"GluonGhostVertex"],
Cell[1024277, 42108, 583, 23, 25, "Text",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1024885, 42135, 131, 3, 28, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1025019, 42140, 324, 10, 27, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1025392, 42156, 64, 1, 36, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1025481, 42161, 348, 10, 28, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1025832, 42173, 517, 21, 30, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1026386, 42199, 86, 2, 28, "Input"],
Cell[1026475, 42203, 578, 26, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1027090, 42234, 121, 3, 28, "Input"],
Cell[1027214, 42239, 181, 6, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1027432, 42250, 86, 2, 28, "Input"],
Cell[1027521, 42254, 286, 12, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1027844, 42271, 148, 4, 28, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1027995, 42277, 333, 10, 42, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1028365, 42292, 191, 5, 28, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1028559, 42299, 521, 21, 32, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[1029107, 42324, 67, 1, 41, "ObjectName",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1029199, 42329, 66, 1, 36, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[1029268, 42332, 535, 10, 70, "Text",
 CellTags->"GluonPropagator"],
Cell[1029806, 42344, 99, 1, 25, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1029930, 42349, 83, 2, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1030016, 42353, 125, 3, 27, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1030178, 42361, 129, 3, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1030310, 42366, 482, 13, 27, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1030807, 42382, 591, 24, 70, "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1031435, 42411, 72, 1, 35, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1031532, 42416, 187, 5, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1031722, 42423, 233, 8, 27, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1031992, 42436, 189, 5, 28, "Input"],
Cell[1032184, 42443, 477, 18, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1032698, 42466, 271, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1032972, 42475, 1035, 39, 51, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1034044, 42519, 357, 10, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1034404, 42531, 2748, 104, 52, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1037189, 42640, 199, 6, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1037391, 42648, 271, 9, 43, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1037677, 42660, 149, 4, 25, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1037851, 42668, 242, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1038096, 42677, 1028, 39, 54, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1039161, 42721, 267, 9, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1039431, 42732, 314, 9, 42, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1039782, 42746, 275, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1040060, 42755, 1274, 46, 46, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1041371, 42806, 275, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1041649, 42815, 1283, 47, 42, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1042969, 42867, 275, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1043247, 42876, 1282, 46, 46, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044566, 42927, 275, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1044844, 42936, 1274, 46, 46, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1046155, 42987, 275, 7, 28, "Input",
 CellTags->"GluonPropagator"],
Cell[1046433, 42996, 2422, 88, 46, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1048882, 43088, 67, 1, 41, "ObjectName",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1048974, 43093, 66, 1, 36, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[1049043, 43096, 122, 3, 25, "Text",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1049190, 43103, 129, 3, 28, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1049322, 43108, 803, 24, 27, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[1050140, 43135, 591, 24, 25, "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1050768, 43164, 72, 1, 35, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1050865, 43169, 252, 8, 28, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1051120, 43179, 2388, 91, 40, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1053545, 43275, 300, 9, 28, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1053848, 43286, 2755, 103, 46, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]]
}, Open  ]],
Cell[1056630, 43393, 59, 1, 41, "ObjectName",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1056714, 43398, 62, 1, 36, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[1056779, 43401, 88, 1, 25, "Text",
 CellTags->"GluonVertex"],
Cell[1056870, 43404, 621, 10, 97, "Text",
 CellTags->"GluonVertex"],
Cell[1057494, 43416, 91, 1, 25, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1057610, 43421, 79, 2, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1057692, 43425, 117, 3, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1057846, 43433, 121, 3, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1057970, 43438, 378, 11, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1058363, 43452, 337, 14, 25, "Text",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1058737, 43471, 59, 1, 36, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1058821, 43476, 338, 10, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1059162, 43488, 943, 42, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1060142, 43535, 379, 12, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1060524, 43549, 1972, 78, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1062533, 43632, 308, 10, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1062844, 43644, 272, 9, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1063153, 43658, 349, 12, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1063505, 43672, 1682, 65, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1065224, 43742, 414, 12, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1065641, 43756, 352, 11, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1066030, 43772, 431, 13, 28, "Input"],
Cell[1066464, 43787, 3438, 130, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1069939, 43922, 374, 12, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1070316, 43936, 160, 5, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1070513, 43946, 259, 7, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1070775, 43955, 3470, 131, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1074260, 44089, 200, 4, 43, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1074485, 44097, 172, 5, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1074660, 44104, 942, 42, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1075639, 44151, 230, 7, 28, "Input",
 CellTags->"GluonVertex"],
Cell[1075872, 44160, 3422, 131, 27, "Output",
 CellTags->"GluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1079321, 44295, 41, 1, 41, "ObjectName",
 CellTags->"GO"],
Cell[CellGroupData[{
Cell[1079387, 44300, 53, 1, 36, "Subsubsection",
 CellTags->"GO"],
Cell[1079443, 44303, 73, 1, 25, "Text",
 CellTags->"GO"],
Cell[1079519, 44306, 206, 9, 25, "Text",
 CellTags->"GO"]
}, Open  ]],
Cell[1079740, 44318, 41, 1, 41, "ObjectName",
 CellTags->"GP"],
Cell[CellGroupData[{
Cell[1079806, 44323, 53, 1, 36, "Subsubsection",
 CellTags->"GP"],
Cell[1079862, 44326, 69, 1, 25, "Text",
 CellTags->"GP"],
Cell[1079934, 44329, 194, 9, 25, "Text",
 CellTags->"GP"]
}, Open  ]],
Cell[1080143, 44341, 67, 1, 41, "ObjectName",
 CellTags->"GrassmannParity"],
Cell[CellGroupData[{
Cell[1080235, 44346, 66, 1, 36, "Subsubsection",
 CellTags->"GrassmannParity"],
Cell[1080304, 44349, 216, 5, 43, "Text",
 CellTags->"GrassmannParity"],
Cell[1080523, 44356, 186, 9, 25, "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],
Cell[1080724, 44368, 41, 1, 41, "ObjectName",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1080790, 44373, 53, 1, 36, "Subsubsection",
 CellTags->"GS"],
Cell[1080846, 44376, 635, 18, 43, "Text",
 CellTags->"GS"],
Cell[1081484, 44396, 482, 24, 25, "Text",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082003, 44425, 50, 1, 36, "Subsubsection",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1082078, 44430, 97, 3, 28, "Input",
 CellTags->"GS"],
Cell[1082178, 44435, 139, 4, 27, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082354, 44444, 162, 6, 28, "Input",
 CellTags->"GS"],
Cell[1082519, 44452, 155, 4, 42, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082711, 44461, 140, 4, 28, "Input",
 CellTags->"GS"],
Cell[1082854, 44467, 412, 13, 27, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1083303, 44485, 179, 6, 28, "Input",
 CellTags->"GS"],
Cell[1083485, 44493, 242, 7, 42, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1083764, 44505, 237, 8, 28, "Input",
 CellTags->"GS"],
Cell[1084004, 44515, 361, 12, 27, "Output",
 CellTags->"GS"]
}, Open  ]]
}, Open  ]],
Cell[1084392, 44531, 43, 1, 41, "ObjectName",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1084460, 44536, 54, 1, 36, "Subsubsection",
 CellTags->"GSD"],
Cell[1084517, 44539, 644, 18, 43, "Text",
 CellTags->"GSD"],
Cell[1085164, 44559, 483, 24, 25, "Text",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1085684, 44588, 51, 1, 36, "Subsubsection",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1085760, 44593, 99, 3, 28, "Input",
 CellTags->"GSD"],
Cell[1085862, 44598, 140, 4, 27, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1086039, 44607, 164, 6, 28, "Input",
 CellTags->"GSD"],
Cell[1086206, 44615, 205, 6, 42, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1086448, 44626, 142, 4, 28, "Input",
 CellTags->"GSD"],
Cell[1086593, 44632, 413, 13, 27, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1087043, 44650, 181, 6, 28, "Input",
 CellTags->"GSD"],
Cell[1087227, 44658, 247, 7, 42, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1087511, 44670, 241, 8, 28, "Input",
 CellTags->"GSD"],
Cell[1087755, 44680, 362, 12, 27, "Output",
 CellTags->"GSD"]
}, Open  ]]
}, Open  ]],
Cell[1088144, 44696, 51, 1, 41, "ObjectName",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1088220, 44701, 58, 1, 36, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1088281, 44704, 83, 1, 25, "Text",
 CellTags->"Gstrong"],
Cell[1088367, 44707, 334, 15, 25, "Text",
 CellTags->"Gstrong"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1088738, 44727, 55, 1, 36, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1088796, 44730, 119, 3, 25, "Text",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1088940, 44737, 80, 2, 28, "Input",
 CellTags->"Gstrong"],
Cell[1089023, 44741, 125, 4, 27, "Output",
 CellTags->"Gstrong"]
}, Open  ]]
}, Open  ]],
Cell[1089175, 44749, 43, 1, 41, "ObjectName",
 CellTags->"GTI"],
Cell[CellGroupData[{
Cell[1089243, 44754, 54, 1, 36, "Subsubsection",
 CellTags->"GTI"],
Cell[1089300, 44757, 85, 1, 25, "Text",
 CellTags->"GTI"],
Cell[1089388, 44760, 159, 9, 25, "Text",
 CellTags->"GTI"]
}, Open  ]],
Cell[1089562, 44772, 41, 1, 41, "ObjectName",
 CellTags->"GV"],
Cell[CellGroupData[{
Cell[1089628, 44777, 53, 1, 36, "Subsubsection",
 CellTags->"GV"],
Cell[1089684, 44780, 65, 1, 25, "Text",
 CellTags->"GV"],
Cell[1089752, 44783, 182, 9, 25, "Text",
 CellTags->"GV"]
}, Open  ]],
Cell[1089949, 44795, 45, 1, 41, "ObjectName",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1090019, 44800, 55, 1, 36, "Subsubsection",
 CellTags->"Hill"],
Cell[1090077, 44803, 129, 4, 25, "Text",
 CellTags->"Hill"],
Cell[1090209, 44809, 196, 9, 25, "Text",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1090442, 44823, 52, 1, 36, "Subsubsection",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1090519, 44828, 124, 4, 28, "Input",
 CellTags->"Hill"],
Cell[1090646, 44834, 2639, 95, 83, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1093322, 44934, 270, 8, 28, "Input",
 CellTags->"Hill"],
Cell[1093595, 44944, 100, 3, 27, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1093732, 44952, 262, 9, 28, "Input",
 CellTags->"Hill"],
Cell[1093997, 44963, 1517, 57, 43, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1095551, 45025, 273, 9, 28, "Input",
 CellTags->"Hill"],
Cell[1095827, 45036, 100, 3, 27, "Output",
 CellTags->"Hill"]
}, Open  ]]
}, Open  ]],
Cell[1095954, 45043, 69, 1, 41, "ObjectName",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1096048, 45048, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1096118, 45051, 553, 9, 79, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1096696, 45064, 131, 3, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1096830, 45069, 177, 6, 27, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[1097022, 45078, 598, 24, 25, "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1097657, 45107, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1097724, 45110, 101, 1, 25, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1097850, 45115, 264, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1098117, 45124, 3978, 124, 48, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1102132, 45253, 264, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1102399, 45262, 3941, 124, 49, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1106377, 45391, 264, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1106644, 45400, 3997, 126, 49, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1110678, 45531, 264, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1110945, 45540, 4289, 133, 94, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1115271, 45678, 264, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1115538, 45687, 1560, 55, 36, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1117135, 45747, 265, 7, 28, "Input",
 CellTags->"HypergeometricAC"],
Cell[1117403, 45756, 1561, 55, 36, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]]
}, Open  ]],
Cell[1118991, 45815, 69, 1, 41, "ObjectName",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1119085, 45820, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[1119155, 45823, 269, 5, 43, "Text",
 CellTags->"HypergeometricIR"],
Cell[1119427, 45830, 479, 19, 25, "Text",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1119931, 45853, 131, 3, 28, "Input",
 CellTags->"HypergeometricIR"],
Cell[1120065, 45858, 182, 6, 27, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1120296, 45870, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1120385, 45875, 241, 7, 28, "Input",
 CellTags->"HypergeometricIR"],
Cell[1120629, 45884, 676, 22, 47, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1121342, 45911, 394, 14, 28, "Input",
 CellTags->"HypergeometricIR"],
Cell[1121739, 45927, 1737, 58, 44, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1123513, 45990, 148, 4, 28, "Input",
 CellTags->"HypergeometricIR"],
Cell[1123664, 45996, 352, 13, 27, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[1124043, 46013, 69, 1, 41, "ObjectName",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1124137, 46018, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[1124207, 46021, 420, 13, 43, "Text",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1124652, 46038, 131, 3, 28, "Input",
 CellTags->"HypergeometricSE"],
Cell[1124786, 46043, 186, 6, 27, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[1124987, 46052, 211, 9, 25, "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1125235, 46066, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1125324, 46071, 245, 7, 28, "Input",
 CellTags->"HypergeometricSE"],
Cell[1125572, 46080, 679, 18, 44, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1126288, 46103, 351, 11, 28, "Input",
 CellTags->"HypergeometricSE"],
Cell[1126642, 46116, 955, 24, 44, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]]
}, Open  ]],
Cell[1127624, 46144, 59, 1, 41, "ObjectName",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1127708, 46149, 62, 1, 36, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[1127773, 46152, 325, 10, 43, "Text",
 CellTags->"HypExplicit"],
Cell[1128101, 46164, 206, 9, 25, "Text",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1128344, 46178, 59, 1, 36, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1128428, 46183, 164, 4, 28, "Input",
 CellTags->"HypExplicit"],
Cell[1128595, 46189, 1241, 46, 27, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1129873, 46240, 142, 4, 28, "Input",
 CellTags->"HypExplicit"],
Cell[1130018, 46246, 674, 18, 44, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1130729, 46269, 262, 8, 28, "Input",
 CellTags->"HypExplicit"],
Cell[1130994, 46279, 1495, 55, 27, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1132526, 46339, 142, 4, 28, "Input",
 CellTags->"HypExplicit"],
Cell[1132671, 46345, 950, 24, 44, "Output",
 CellTags->"HypExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1133648, 46373, 49, 1, 41, "ObjectName",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1133722, 46378, 57, 1, 36, "Subsubsection",
 CellTags->"HypInt"],
Cell[1133782, 46381, 206, 5, 43, "Text",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1134025, 46391, 54, 1, 36, "Subsubsection",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1134104, 46396, 159, 4, 28, "Input",
 CellTags->"HypInt"],
Cell[1134266, 46402, 1236, 46, 27, "Output",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1135539, 46453, 128, 4, 28, "Input",
 CellTags->"HypInt"],
Cell[1135670, 46459, 981, 34, 50, "Output",
 CellTags->"HypInt"]
}, Open  ]]
}, Open  ]],
Cell[1136678, 46497, 45, 1, 41, "ObjectName",
 CellTags->"IFPD"],
Cell[CellGroupData[{
Cell[1136748, 46502, 55, 1, 36, "Subsubsection",
 CellTags->"IFPD"],
Cell[1136806, 46505, 66, 1, 25, "Text",
 CellTags->"IFPD"],
Cell[1136875, 46508, 214, 9, 25, "Text",
 CellTags->"IFPD"]
}, Open  ]],
Cell[1137104, 46520, 51, 1, 41, "ObjectName",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1137180, 46525, 58, 1, 36, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[1137241, 46528, 182, 4, 43, "Text",
 CellTags->"IFPDOff"],
Cell[1137426, 46534, 270, 14, 25, "Text",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1137733, 46553, 55, 1, 36, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1137813, 46558, 163, 5, 28, "Input",
 CellTags->"IFPDOff"],
Cell[1137979, 46565, 384, 16, 27, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1138400, 46586, 108, 3, 28, "Input",
 CellTags->"IFPDOff"],
Cell[1138511, 46591, 178, 5, 42, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1138726, 46601, 130, 4, 28, "Input",
 CellTags->"IFPDOff"],
Cell[1138859, 46607, 212, 8, 27, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1139108, 46620, 108, 3, 28, "Input",
 CellTags->"IFPDOff"],
Cell[1139219, 46625, 283, 9, 42, "Output",
 CellTags->"IFPDOff"]
}, Open  ]]
}, Open  ]],
Cell[1139529, 46638, 49, 1, 41, "ObjectName",
 CellTags->"IFPDOn"],
Cell[CellGroupData[{
Cell[1139603, 46643, 57, 1, 36, "Subsubsection",
 CellTags->"IFPDOn"],
Cell[1139663, 46646, 317, 6, 61, "Text",
 CellTags->"IFPDOn"],
Cell[1139983, 46654, 272, 14, 25, "Text",
 CellTags->"IFPDOn"]
}, Open  ]],
Cell[1140270, 46671, 59, 1, 41, "ObjectName",
 CellTags->"IncludePair"],
Cell[CellGroupData[{
Cell[1140354, 46676, 62, 1, 36, "Subsubsection",
 CellTags->"IncludePair"],
Cell[1140419, 46679, 124, 3, 25, "Text",
 CellTags->"IncludePair"],
Cell[1140546, 46684, 176, 9, 25, "Text",
 CellTags->"IncludePair"]
}, Open  ]],
Cell[1140737, 46696, 63, 1, 41, "ObjectName",
 CellTags->"IndexPosition"],
Cell[CellGroupData[{
Cell[1140825, 46701, 64, 1, 36, "Subsubsection",
 CellTags->"IndexPosition"],
Cell[1140892, 46704, 89, 1, 25, "Text",
 CellTags->"IndexPosition"],
Cell[1140984, 46707, 199, 9, 25, "Text",
 CellTags->"IndexPosition"]
}, Open  ]],
Cell[1141198, 46719, 51, 1, 41, "ObjectName",
 CellTags->"Indices"],
Cell[CellGroupData[{
Cell[1141274, 46724, 58, 1, 36, "Subsubsection",
 CellTags->"Indices"],
Cell[1141335, 46727, 205, 5, 43, "Text",
 CellTags->"Indices"],
Cell[1141543, 46734, 193, 9, 25, "Text",
 CellTags->"Indices"]
}, Open  ]],
Cell[1141751, 46746, 67, 1, 41, "ObjectName",
 CellTags->"InitialFunction"],
Cell[CellGroupData[{
Cell[1141843, 46751, 66, 1, 36, "Subsubsection",
 CellTags->"InitialFunction"],
Cell[1141912, 46754, 183, 4, 25, "Text",
 CellTags->"InitialFunction"],
Cell[1142098, 46760, 186, 9, 25, "Text",
 CellTags->"InitialFunction"]
}, Open  ]],
Cell[1142299, 46772, 77, 1, 41, "ObjectName",
 CellTags->"InitialSubstitutions"],
Cell[CellGroupData[{
Cell[1142401, 46777, 71, 1, 36, "Subsubsection",
 CellTags->"InitialSubstitutions"],
Cell[1142475, 46780, 208, 4, 25, "Text",
 CellTags->"InitialSubstitutions"],
Cell[1142686, 46786, 408, 19, 25, "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],
Cell[1143109, 46808, 69, 1, 41, "ObjectName",
 CellTags->"InsideDiracTrace"],
Cell[CellGroupData[{
Cell[1143203, 46813, 67, 1, 36, "Subsubsection",
 CellTags->"InsideDiracTrace"],
Cell[1143273, 46816, 498, 9, 61, "Text",
 CellTags->"InsideDiracTrace"],
Cell[1143774, 46827, 318, 14, 25, "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],
Cell[1144107, 46844, 63, 1, 41, "ObjectName",
 CellTags->"IntegralTable"],
Cell[CellGroupData[{
Cell[1144195, 46849, 64, 1, 36, "Subsubsection",
 CellTags->"IntegralTable"],
Cell[1144262, 46852, 233, 5, 25, "Text",
 CellTags->"IntegralTable"],
Cell[1144498, 46859, 500, 19, 25, "Text",
 CellTags->"IntegralTable"]
}, Open  ]],
Cell[1145013, 46881, 69, 1, 41, "ObjectName",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1145107, 46886, 67, 1, 36, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[1145177, 46889, 184, 4, 25, "Text",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1145386, 46897, 131, 3, 28, "Input",
 CellTags->"IntegrateByParts"],
Cell[1145520, 46902, 172, 5, 27, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1145741, 46913, 64, 1, 36, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1145830, 46918, 539, 19, 28, "Input",
 CellTags->"IntegrateByParts"],
Cell[1146372, 46939, 648, 23, 41, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[1147047, 46966, 59, 1, 41, "ObjectName",
 CellTags->"Integratedx"],
Cell[CellGroupData[{
Cell[1147131, 46971, 62, 1, 36, "Subsubsection",
 CellTags->"Integratedx"],
Cell[1147196, 46974, 154, 4, 25, "Text",
 CellTags->"Integratedx"],
Cell[1147353, 46980, 852, 34, 43, "Text",
 CellTags->"Integratedx"]
}, Open  ]],
Cell[1148220, 47017, 57, 1, 41, "ObjectName",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1148302, 47022, 61, 1, 36, "Subsubsection",
 CellTags->"Integrate2"],
Cell[1148366, 47025, 866, 17, 97, "Text",
 CellTags->"Integrate2"],
Cell[1149235, 47044, 508, 24, 25, "Text",
 CellTags->"Integrate2"],
Cell[1149746, 47070, 405, 7, 61, "Text",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1150188, 47082, 58, 1, 36, "Subsubsection",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1150271, 47087, 425, 15, 28, "Input",
 CellTags->"Integrate2"],
Cell[1150699, 47104, 594, 22, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1151308, 47129, 190, 6, 25, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1151523, 47139, 281, 9, 28, "Input",
 CellTags->"Integrate2"],
Cell[1151807, 47150, 232, 8, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1152076, 47163, 276, 9, 28, "Input",
 CellTags->"Integrate2"],
Cell[1152355, 47174, 574, 22, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1152966, 47201, 336, 11, 28, "Input",
 CellTags->"Integrate2"],
Cell[1153305, 47214, 775, 28, 41, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1154117, 47247, 317, 10, 28, "Input",
 CellTags->"Integrate2"],
Cell[1154437, 47259, 134, 4, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1154586, 47266, 1589, 53, 55, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1156200, 47323, 270, 9, 28, "Input",
 CellTags->"Integrate2"],
Cell[1156473, 47334, 106, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1156594, 47340, 649, 24, 44, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1157268, 47368, 317, 10, 28, "Input",
 CellTags->"Integrate2"],
Cell[1157588, 47380, 106, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1157731, 47388, 386, 13, 28, "Input",
 CellTags->"Integrate2"],
Cell[1158120, 47403, 164, 5, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1158321, 47413, 417, 14, 28, "Input",
 CellTags->"Integrate2"],
Cell[1158741, 47429, 397, 16, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1159175, 47450, 328, 11, 28, "Input",
 CellTags->"Integrate2"],
Cell[1159506, 47463, 369, 15, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1159912, 47483, 354, 12, 28, "Input",
 CellTags->"Integrate2"],
Cell[1160269, 47497, 527, 20, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1160833, 47522, 313, 10, 28, "Input",
 CellTags->"Integrate2"],
Cell[1161149, 47534, 189, 6, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1161375, 47545, 281, 9, 28, "Input",
 CellTags->"Integrate2"],
Cell[1161659, 47556, 514, 20, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1162210, 47581, 445, 15, 28, "Input",
 CellTags->"Integrate2"],
Cell[1162658, 47598, 598, 21, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1163293, 47624, 721, 24, 28, "Input",
 CellTags->"Integrate2"],
Cell[1164017, 47650, 5640, 201, 129, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1169694, 47856, 471, 16, 28, "Input",
 CellTags->"Integrate2"],
Cell[1170168, 47874, 1013, 37, 42, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171218, 47916, 567, 19, 39, "Input",
 CellTags->"Integrate2"],
Cell[1171788, 47937, 6108, 208, 121, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1177933, 48150, 142, 4, 28, "Input",
 CellTags->"Integrate2"],
Cell[1178078, 48156, 2556, 85, 77, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1180671, 48246, 104, 3, 28, "Input",
 CellTags->"Integrate2"],
Cell[1180778, 48251, 124, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1180939, 48259, 418, 13, 39, "Input"],
Cell[1181360, 48274, 100, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1181497, 48281, 684, 23, 39, "Input",
 CellTags->"Integrate2"],
Cell[1182184, 48306, 4132, 157, 81, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1186353, 48468, 107, 3, 28, "Input"],
Cell[1186463, 48473, 626, 23, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1187126, 48501, 79, 2, 28, "Input"],
Cell[1187208, 48505, 100, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1187345, 48512, 656, 22, 39, "Input"],
Cell[1188004, 48536, 100, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1188141, 48543, 151, 4, 28, "Input",
 CellTags->"Integrate2"],
Cell[1188295, 48549, 109, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1188441, 48557, 368, 12, 28, "Input",
 CellTags->"Integrate2"],
Cell[1188812, 48571, 106, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1188933, 48577, 137, 4, 25, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1189095, 48585, 147, 4, 28, "Input",
 CellTags->"Integrate2"],
Cell[1189245, 48591, 5979, 193, 215, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1195261, 48789, 230, 8, 28, "Input",
 CellTags->"Integrate2"],
Cell[1195494, 48799, 174, 6, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1195683, 48808, 165, 4, 25, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1195873, 48816, 326, 11, 28, "Input",
 CellTags->"Integrate2"],
Cell[1196202, 48829, 175, 6, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1196392, 48838, 126, 4, 28, "Input",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1196543, 48846, 317, 10, 28, "Input",
 CellTags->"Integrate2"],
Cell[1196863, 48858, 134, 4, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1197034, 48867, 325, 11, 28, "Input",
 CellTags->"Integrate2"],
Cell[1197362, 48880, 260, 9, 40, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1197659, 48894, 99, 3, 28, "Input",
 CellTags->"Integrate2"],
Cell[1197761, 48899, 125, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1197923, 48907, 325, 11, 28, "Input",
 CellTags->"Integrate2"],
Cell[1198251, 48920, 125, 3, 27, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1198413, 48928, 381, 13, 28, "Input",
 CellTags->"Integrate2"],
Cell[1198797, 48943, 4862, 184, 125, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1203696, 49132, 107, 3, 28, "Input"],
Cell[1203806, 49137, 230, 8, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1204073, 49150, 79, 2, 28, "Input"],
Cell[1204155, 49154, 99, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1204291, 49161, 353, 12, 28, "Input"],
Cell[1204647, 49175, 99, 2, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[1204773, 49181, 57, 1, 70, "ObjectName",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1204855, 49186, 61, 1, 70, "Subsubsection",
 CellTags->"Integrate3"],
Cell[1204919, 49189, 331, 6, 70, "Text",
 CellTags->"Integrate3"],
Cell[1205253, 49197, 187, 9, 70, "Text",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1205477, 49211, 58, 1, 36, "Subsubsection",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1205560, 49216, 280, 9, 28, "Input",
 CellTags->"Integrate3"],
Cell[1205843, 49227, 225, 8, 45, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1206105, 49240, 400, 13, 41, "Input",
 CellTags->"Integrate3"],
Cell[1206508, 49255, 722, 29, 27, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1207267, 49289, 676, 23, 41, "Input",
 CellTags->"Integrate3"],
Cell[1207946, 49314, 2758, 99, 70, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1210741, 49418, 266, 8, 28, "Input",
 CellTags->"Integrate3"],
Cell[1211010, 49428, 106, 3, 27, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1211153, 49436, 322, 10, 28, "Input",
 CellTags->"Integrate3"],
Cell[1211478, 49448, 195, 6, 27, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1211710, 49459, 270, 9, 28, "Input",
 CellTags->"Integrate3"],
Cell[1211983, 49470, 106, 3, 27, "Output",
 CellTags->"Integrate3"]
}, Open  ]]
}, Open  ]],
Cell[1212116, 49477, 87, 1, 70, "ObjectName",
 CellTags->"IntermediateSubstitutions"],
Cell[CellGroupData[{
Cell[1212228, 49482, 76, 1, 70, "Subsubsection",
 CellTags->"IntermediateSubstitutions"],
Cell[1212307, 49485, 206, 4, 70, "Text",
 CellTags->"IntermediateSubstitutions"],
Cell[1212516, 49491, 193, 9, 70, "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],
Cell[1212724, 49503, 63, 1, 70, "ObjectName",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1212812, 49508, 64, 1, 70, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[1212879, 49511, 531, 14, 70, "Text",
 CellTags->"InverseMellin"],
Cell[1213413, 49527, 609, 29, 70, "Text",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1214059, 49561, 61, 1, 36, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1214145, 49566, 169, 5, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1214317, 49573, 157, 5, 27, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1214511, 49583, 220, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1214734, 49592, 157, 5, 27, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1214928, 49602, 142, 4, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1215073, 49608, 280, 10, 27, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1215390, 49623, 194, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1215587, 49631, 293, 11, 38, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1215917, 49647, 194, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1216114, 49655, 360, 13, 27, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1216511, 49673, 203, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1216717, 49682, 360, 13, 27, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1217092, 49698, 113, 3, 25, "Text",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1217230, 49705, 294, 9, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1217527, 49716, 131, 4, 39, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1217673, 49723, 117, 3, 25, "Text",
 CellTags->"InverseMellin"],
Cell[1217793, 49728, 3132, 100, 164, "Input",
 CellTags->"InverseMellin"],
Cell[1220928, 49830, 244, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1221197, 49841, 179, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1221379, 49849, 398, 14, 40, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1221814, 49868, 179, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1221996, 49876, 356, 12, 40, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1222389, 49893, 172, 5, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1222564, 49900, 796, 28, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1223375, 49931, 81, 2, 28, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1223481, 49937, 192, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1223676, 49945, 500, 19, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224213, 49969, 270, 9, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1224486, 49980, 502, 18, 41, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1225025, 50003, 270, 9, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1225298, 50014, 693, 24, 41, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1226028, 50043, 267, 9, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1226298, 50054, 838, 30, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1227173, 50089, 246, 9, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1227422, 50100, 549, 20, 41, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1228008, 50125, 274, 10, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1228285, 50137, 1140, 39, 45, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1229462, 50181, 192, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1229657, 50189, 700, 25, 43, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1230394, 50219, 221, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1230618, 50228, 841, 30, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1231496, 50263, 192, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1231691, 50271, 1047, 38, 45, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1232775, 50314, 202, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1232980, 50322, 544, 20, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233561, 50347, 223, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1233787, 50356, 1354, 48, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1235178, 50409, 223, 7, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1235404, 50418, 1299, 46, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1236740, 50469, 212, 6, 28, "Input",
 CellTags->"InverseMellin"],
Cell[1236955, 50477, 632, 23, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1237602, 50503, 158, 5, 28, "Input",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1237775, 50511, 51, 1, 41, "ObjectName",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1237851, 50516, 58, 1, 36, "Subsubsection",
 CellTags->"Isolate"],
Cell[1237912, 50519, 422, 7, 61, "Text",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1238359, 50530, 114, 3, 28, "Input",
 CellTags->"Isolate"],
Cell[1238476, 50535, 404, 12, 49, "Output",
 CellTags->"Isolate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1238929, 50553, 55, 1, 36, "Subsubsection",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1239009, 50558, 156, 5, 28, "Input",
 CellTags->"Isolate"],
Cell[1239168, 50565, 158, 6, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1239363, 50576, 378, 13, 28, "Input",
 CellTags->"Isolate"],
Cell[1239744, 50591, 305, 12, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240086, 50608, 114, 3, 28, "Input",
 CellTags->"Isolate"],
Cell[1240203, 50613, 282, 11, 42, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240522, 50629, 182, 5, 28, "Input",
 CellTags->"Isolate"],
Cell[1240707, 50636, 640, 25, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1241384, 50666, 561, 19, 28, "Input",
 CellTags->"Isolate"],
Cell[1241948, 50687, 355, 14, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1242340, 50706, 94, 3, 28, "Input",
 CellTags->"Isolate"],
Cell[1242437, 50711, 90, 1, 41, "Print",
 CellTags->{"Info3387712053-5834299", "Isolate"}],
Cell[1242530, 50714, 1014, 30, 52, "Print",
 CellTags->{"Info3387712053-5834299", "Isolate"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[1243581, 50749, 278, 7, 28, "Input",
 CellTags->"Isolate"],
Cell[1243862, 50758, 157, 6, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1244056, 50769, 178, 6, 28, "Input",
 CellTags->"Isolate"],
Cell[1244237, 50777, 206, 7, 27, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1244458, 50787, 152, 4, 28, "Input",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1244625, 50794, 59, 1, 41, "ObjectName",
 CellTags->"IsolateHead"],
Cell[CellGroupData[{
Cell[1244709, 50799, 62, 1, 36, "Subsubsection",
 CellTags->"IsolateHead"],
Cell[1244774, 50802, 84, 1, 25, "Text",
 CellTags->"IsolateHead"],
Cell[1244861, 50805, 194, 9, 25, "Text",
 CellTags->"IsolateHead"]
}, Open  ]],
Cell[1245070, 50817, 61, 1, 41, "ObjectName",
 CellTags->"IsolateNames"],
Cell[CellGroupData[{
Cell[1245156, 50822, 63, 1, 36, "Subsubsection",
 CellTags->"IsolateNames"],
Cell[1245222, 50825, 220, 5, 43, "Text",
 CellTags->"IsolateNames"],
Cell[1245445, 50832, 290, 14, 25, "Text",
 CellTags->"IsolateNames"]
}, Open  ]],
Cell[1245750, 50849, 61, 1, 41, "ObjectName",
 CellTags->"IsolatePrint"],
Cell[CellGroupData[{
Cell[1245836, 50854, 63, 1, 36, "Subsubsection",
 CellTags->"IsolatePrint"],
Cell[1245902, 50857, 284, 6, 43, "Text",
 CellTags->"IsolatePrint"],
Cell[1246189, 50865, 180, 9, 25, "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],
Cell[1246384, 50877, 61, 1, 41, "ObjectName",
 CellTags->"IsolateSplit"],
Cell[CellGroupData[{
Cell[1246470, 50882, 63, 1, 36, "Subsubsection",
 CellTags->"IsolateSplit"],
Cell[1246536, 50885, 413, 7, 61, "Text",
 CellTags->"IsolateSplit"],
Cell[1246952, 50894, 180, 9, 25, "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],
Cell[1247147, 50906, 53, 1, 41, "ObjectName",
 CellTags->"KeepOnly"],
Cell[CellGroupData[{
Cell[1247225, 50911, 59, 1, 36, "Subsubsection",
 CellTags->"KeepOnly"],
Cell[1247287, 50914, 293, 6, 43, "Text",
 CellTags->"KeepOnly"],
Cell[1247583, 50922, 452, 24, 25, "Text",
 CellTags->"KeepOnly"]
}, Open  ]],
Cell[1248050, 50949, 41, 1, 41, "ObjectName",
 CellTags->"KK"],
Cell[CellGroupData[{
Cell[1248116, 50954, 53, 1, 36, "Subsubsection",
 CellTags->"KK"],
Cell[1248172, 50957, 235, 5, 43, "Text",
 CellTags->"KK"],
Cell[1248410, 50964, 292, 14, 25, "Text",
 CellTags->"KK"]
}, Open  ]],
Cell[1248717, 50981, 49, 1, 41, "ObjectName",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1248791, 50986, 57, 1, 36, "Subsubsection",
 CellTags->"Kummer"],
Cell[1248851, 50989, 260, 5, 43, "Text",
 CellTags->"Kummer"],
Cell[1249114, 50996, 201, 9, 25, "Text",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1249352, 51010, 54, 1, 36, "Subsubsection",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1249431, 51015, 337, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1249771, 51026, 2763, 98, 29, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1252571, 51129, 337, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1252911, 51140, 2781, 99, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1255729, 51244, 337, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1256069, 51255, 2781, 99, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1258887, 51359, 388, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1259278, 51372, 2771, 98, 27, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1262086, 51475, 479, 13, 28, "Input",
 CellTags->"Kummer"],
Cell[1262568, 51490, 2864, 99, 27, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1265469, 51594, 388, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1265860, 51607, 2758, 98, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1268655, 51710, 479, 13, 28, "Input",
 CellTags->"Kummer"],
Cell[1269137, 51725, 2873, 100, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1272047, 51830, 388, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1272438, 51843, 2758, 98, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1275233, 51946, 479, 13, 28, "Input",
 CellTags->"Kummer"],
Cell[1275715, 51961, 2873, 100, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1278625, 52066, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1279115, 52083, 2940, 104, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1282092, 52192, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1282724, 52213, 3068, 107, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1285829, 52325, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1286319, 52342, 2915, 104, 41, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1289271, 52451, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1289903, 52472, 3043, 107, 41, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1292983, 52584, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1293473, 52601, 2891, 103, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1296401, 52709, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1297033, 52730, 3100, 108, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1300170, 52843, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1300660, 52860, 2940, 104, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1303637, 52969, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1304269, 52990, 3125, 109, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1307431, 53104, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1307921, 53121, 2865, 102, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1310823, 53228, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1311455, 53249, 3050, 107, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1314542, 53361, 487, 15, 28, "Input",
 CellTags->"Kummer"],
Cell[1315032, 53378, 2865, 102, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1317934, 53485, 629, 19, 50, "Input",
 CellTags->"Kummer"],
Cell[1318566, 53506, 3167, 110, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1321770, 53621, 531, 15, 50, "Input",
 CellTags->"Kummer"],
Cell[1322304, 53638, 2914, 102, 29, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1325255, 53745, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1325596, 53756, 2763, 98, 29, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1328396, 53859, 531, 15, 50, "Input",
 CellTags->"Kummer"],
Cell[1328930, 53876, 2985, 104, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1331952, 53985, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1332293, 53996, 2753, 98, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1335083, 54099, 531, 15, 50, "Input",
 CellTags->"Kummer"],
Cell[1335617, 54116, 2985, 104, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1338639, 54225, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1338980, 54236, 2753, 98, 36, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1341770, 54339, 582, 17, 50, "Input",
 CellTags->"Kummer"],
Cell[1342355, 54358, 2967, 103, 27, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1345359, 54466, 389, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1345751, 54479, 2771, 98, 27, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1348559, 54582, 582, 17, 50, "Input",
 CellTags->"Kummer"],
Cell[1349144, 54601, 3083, 107, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1352264, 54713, 389, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1352656, 54726, 2816, 100, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1355509, 54831, 582, 17, 50, "Input",
 CellTags->"Kummer"],
Cell[1356094, 54850, 3025, 105, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1359156, 54960, 389, 11, 28, "Input",
 CellTags->"Kummer"],
Cell[1359548, 54973, 2758, 98, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1362343, 55076, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1362684, 55087, 5080, 165, 71, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1367801, 55257, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1368142, 55268, 5159, 169, 48, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1373338, 55442, 338, 9, 28, "Input",
 CellTags->"Kummer"],
Cell[1373679, 55453, 5177, 169, 48, "Output",
 CellTags->"Kummer"]
}, Open  ]]
}, Open  ]],
Cell[1378883, 55626, 57, 1, 70, "ObjectName",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1378965, 55631, 61, 1, 36, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[1379029, 55634, 454, 9, 70, "Text",
 CellTags->"Lagrangian"],
Cell[1379486, 55645, 181, 9, 25, "Text",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1379704, 55659, 58, 1, 36, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1379787, 55664, 124, 3, 28, "Input",
 CellTags->"Lagrangian"],
Cell[1379914, 55669, 332, 11, 38, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[1380261, 55683, 85, 1, 25, "Text",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1380371, 55688, 124, 3, 28, "Input",
 CellTags->"Lagrangian"],
Cell[1380498, 55693, 683, 23, 38, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1381218, 55721, 124, 3, 28, "Input",
 CellTags->"Lagrangian"],
Cell[1381345, 55726, 1006, 36, 38, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1382388, 55767, 124, 3, 28, "Input",
 CellTags->"Lagrangian"],
Cell[1382515, 55772, 621, 22, 28, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1383173, 55799, 124, 3, 28, "Input",
 CellTags->"Lagrangian"],
Cell[1383300, 55804, 664, 23, 28, "Output",
 CellTags->"Lagrangian"]
}, Open  ]]
}, Open  ]],
Cell[1383991, 55831, 41, 1, 41, "ObjectName",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1384057, 55836, 53, 1, 36, "Subsubsection",
 CellTags->"LC"],
Cell[1384113, 55839, 240, 5, 43, "Text",
 CellTags->"LC"],
Cell[1384356, 55846, 274, 14, 25, "Text",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1384667, 55865, 50, 1, 36, "Subsubsection",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1384742, 55870, 164, 5, 28, "Input",
 CellTags->"LC"],
Cell[1384909, 55877, 184, 6, 27, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1385130, 55888, 95, 3, 28, "Input",
 CellTags->"LC"],
Cell[1385228, 55893, 477, 21, 27, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1385742, 55919, 104, 3, 28, "Input",
 CellTags->"LC"],
Cell[1385849, 55924, 337, 8, 42, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1386223, 55937, 181, 6, 28, "Input",
 CellTags->"LC"],
Cell[1386407, 55945, 169, 5, 27, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1386613, 55955, 130, 4, 28, "Input",
 CellTags->"LC"],
Cell[1386746, 55961, 367, 10, 42, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387150, 55976, 381, 13, 28, "Input",
 CellTags->"LC"],
Cell[1387534, 55991, 396, 16, 27, "Output",
 CellTags->"LC"]
}, Open  ]]
}, Open  ]],
Cell[1387957, 56011, 43, 1, 41, "ObjectName",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1388025, 56016, 54, 1, 36, "Subsubsection",
 CellTags->"LCD"],
Cell[1388082, 56019, 234, 5, 43, "Text",
 CellTags->"LCD"],
Cell[1388319, 56026, 161, 4, 25, "Text",
 CellTags->"LCD"],
Cell[1388483, 56032, 272, 14, 25, "Text",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1388792, 56051, 51, 1, 36, "Subsubsection",
 CellTags->"LCD"],
Cell[1388846, 56054, 190, 6, 28, "Input",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1389061, 56064, 166, 5, 28, "Input",
 CellTags->"LCD"],
Cell[1389230, 56071, 185, 6, 27, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1389452, 56082, 96, 3, 28, "Input",
 CellTags->"LCD"],
Cell[1389551, 56087, 478, 21, 27, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1390066, 56113, 105, 3, 28, "Input",
 CellTags->"LCD"],
Cell[1390174, 56118, 438, 12, 42, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1390649, 56135, 183, 6, 28, "Input",
 CellTags->"LCD"],
Cell[1390835, 56143, 170, 5, 27, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391042, 56153, 131, 4, 28, "Input",
 CellTags->"LCD"],
Cell[1391176, 56159, 418, 12, 42, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391631, 56176, 400, 13, 28, "Input",
 CellTags->"LCD"],
Cell[1392034, 56191, 397, 15, 27, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392468, 56211, 170, 5, 28, "Input",
 CellTags->"LCD"],
Cell[1392641, 56218, 161, 5, 27, "Output",
 CellTags->"LCD"]
}, Open  ]]
}, Open  ]],
Cell[1392829, 56227, 61, 1, 41, "ObjectName",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1392915, 56232, 63, 1, 36, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[1392981, 56235, 307, 12, 31, "Text",
 CellTags->"LeftPartialD"],
Cell[1393291, 56249, 573, 24, 25, "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1393901, 56278, 69, 1, 35, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1393995, 56283, 253, 7, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1394251, 56292, 306, 11, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1394594, 56308, 120, 3, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1394717, 56313, 355, 14, 27, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1395109, 56332, 118, 3, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1395230, 56337, 291, 7, 42, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1395558, 56349, 161, 4, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1395722, 56355, 176, 4, 42, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1395935, 56364, 372, 10, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1396310, 56376, 344, 12, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1396691, 56393, 120, 3, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1396814, 56398, 774, 30, 27, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1397625, 56433, 118, 3, 28, "Input",
 CellTags->"LeftPartialD"],
Cell[1397746, 56438, 799, 21, 62, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1398572, 56463, 71, 1, 41, "ObjectName",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1398668, 56468, 68, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[1398739, 56471, 592, 21, 49, "Text",
 CellTags->"LeftRightPartialD"],
Cell[1399334, 56494, 831, 33, 25, "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1400202, 56532, 65, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1400292, 56537, 132, 3, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1400427, 56542, 260, 9, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1400724, 56556, 127, 3, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1400854, 56561, 527, 19, 38, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1401418, 56585, 263, 7, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1401684, 56594, 316, 11, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1402037, 56610, 125, 3, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1402165, 56615, 708, 26, 38, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1402910, 56646, 382, 10, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1403295, 56658, 354, 12, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1403686, 56675, 125, 3, 28, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1403814, 56680, 902, 33, 38, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1404743, 56717, 73, 1, 70, "ObjectName",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1404841, 56722, 69, 1, 70, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[1404913, 56725, 591, 21, 70, "Text",
 CellTags->"LeftRightPartialD2"],
Cell[1405507, 56748, 698, 29, 70, "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1406242, 56782, 66, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1406333, 56787, 134, 3, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1406470, 56792, 261, 9, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1406768, 56806, 128, 3, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1406899, 56811, 433, 16, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1407369, 56832, 265, 7, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1407637, 56841, 317, 11, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1407991, 56857, 126, 3, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1408120, 56862, 600, 23, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1408757, 56890, 384, 10, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1409144, 56902, 355, 12, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1409536, 56919, 126, 3, 28, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1409665, 56924, 780, 30, 27, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]]
}, Open  ]],
Cell[1410472, 56958, 58, 1, 70, "ObjectName",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1410555, 56963, 61, 1, 36, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[1410619, 56966, 1212, 41, 79, "Text",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1411856, 57011, 120, 3, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1411979, 57016, 168, 5, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1412162, 57024, 463, 24, 25, "Text",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1412662, 57053, 58, 1, 36, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1412745, 57058, 187, 5, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1412935, 57065, 492, 21, 28, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1413464, 57091, 204, 6, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1413671, 57099, 411, 17, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414119, 57121, 180, 5, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1414302, 57128, 313, 13, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414652, 57146, 186, 5, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1414841, 57153, 107, 3, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414985, 57161, 242, 7, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1415230, 57170, 332, 8, 42, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1415599, 57183, 365, 10, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1415967, 57195, 932, 40, 28, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1416936, 57240, 113, 3, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1417052, 57245, 126, 4, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1417193, 57252, 316, 9, 28, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1417534, 57265, 403, 11, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1417940, 57278, 297, 10, 27, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1418252, 57291, 316, 9, 28, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1418593, 57304, 179, 5, 28, "Input",
 CellTags->"LeviCivita"],
Cell[1418775, 57311, 199, 6, 28, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1418989, 57320, 110, 3, 28, "Input",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1419114, 57326, 65, 1, 41, "ObjectName",
 CellTags->"LeviCivitaSign"],
Cell[CellGroupData[{
Cell[1419204, 57331, 65, 1, 36, "Subsubsection",
 CellTags->"LeviCivitaSign"],
Cell[1419272, 57334, 203, 4, 43, "Text",
 CellTags->"LeviCivitaSign"],
Cell[1419478, 57340, 310, 14, 25, "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],
Cell[1419803, 57357, 81, 1, 41, "ObjectName",
 CellTags->"ListIntegrals"],
Cell[CellGroupData[{
Cell[1419909, 57362, 64, 1, 36, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1419976, 57365, 470, 8, 79, "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1420483, 57378, 61, 1, 36, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1420547, 57381, 80, 2, 28, "Input",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[1420642, 57386, 44, 1, 41, "ObjectName",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1420711, 57391, 54, 1, 36, "Subsubsection",
 CellTags->"Li2"],
Cell[1420768, 57394, 117, 3, 25, "Text",
 CellTags->"Li2"],
Cell[1420888, 57399, 153, 8, 25, "Text",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1421078, 57412, 51, 1, 36, "Subsubsection",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1421154, 57417, 100, 3, 28, "Input",
 CellTags->"Li2"],
Cell[1421257, 57422, 236, 9, 27, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1421530, 57436, 107, 3, 28, "Input",
 CellTags->"Li2"],
Cell[1421640, 57441, 162, 5, 42, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1421839, 57451, 253, 9, 40, "Input",
 CellTags->"Li2"],
Cell[1422095, 57462, 236, 9, 27, "Output",
 CellTags->"Li2"]
}, Open  ]]
}, Open  ]],
Cell[1422358, 57475, 44, 1, 41, "ObjectName",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1422427, 57480, 54, 1, 36, "Subsubsection",
 CellTags->"Li3"],
Cell[1422484, 57483, 118, 3, 25, "Text",
 CellTags->"Li3"],
Cell[1422605, 57488, 153, 8, 25, "Text",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1422795, 57501, 51, 1, 36, "Subsubsection",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1422871, 57506, 100, 3, 28, "Input",
 CellTags->"Li3"],
Cell[1422974, 57511, 236, 9, 27, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423247, 57525, 107, 3, 28, "Input",
 CellTags->"Li3"],
Cell[1423357, 57530, 162, 5, 42, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423556, 57540, 152, 5, 28, "Input",
 CellTags->"Li3"],
Cell[1423711, 57547, 266, 11, 40, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1424014, 57563, 204, 7, 41, "Input",
 CellTags->"Li3"],
Cell[1424221, 57572, 236, 9, 27, "Output",
 CellTags->"Li3"]
}, Open  ]]
}, Open  ]],
Cell[1424484, 57585, 45, 1, 41, "ObjectName",
 CellTags->"Loop"],
Cell[CellGroupData[{
Cell[1424554, 57590, 55, 1, 36, "Subsubsection",
 CellTags->"Loop"],
Cell[1424612, 57593, 94, 1, 25, "Text",
 CellTags->"Loop"]
}, Open  ]],
Cell[1424721, 57597, 61, 1, 41, "ObjectName",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1424807, 57602, 63, 1, 36, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1424873, 57605, 243, 5, 43, "Text",
 CellTags->"LorentzIndex"],
Cell[1425119, 57612, 314, 14, 25, "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1425470, 57631, 60, 1, 36, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1425533, 57634, 89, 1, 25, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1425647, 57639, 125, 3, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1425775, 57644, 116, 3, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1425906, 57650, 125, 3, 25, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1426056, 57657, 148, 4, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1426207, 57663, 116, 3, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1426360, 57671, 106, 2, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1426469, 57675, 140, 4, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1426646, 57684, 165, 4, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1426814, 57690, 113, 3, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1426964, 57698, 207, 5, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1427174, 57705, 139, 4, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1427328, 57712, 141, 4, 25, "Text",
 CellTags->"LorentzIndex"],
Cell[1427472, 57718, 137, 4, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1427634, 57726, 347, 10, 28, "Input",
 CellTags->"LorentzIndex"],
Cell[1427984, 57738, 385, 16, 27, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1428384, 57757, 148, 4, 28, "Input",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1428547, 57764, 47, 1, 41, "ObjectName",
 CellTags->"Lower"],
Cell[CellGroupData[{
Cell[1428619, 57769, 56, 1, 36, "Subsubsection",
 CellTags->"Lower"],
Cell[1428678, 57772, 120, 3, 25, "Text",
 CellTags->"Lower"],
Cell[1428801, 57777, 402, 19, 25, "Text",
 CellTags->"Lower"]
}, Open  ]],
Cell[1429218, 57799, 59, 1, 41, "ObjectName",
 CellTags->"MakeContext"],
Cell[CellGroupData[{
Cell[1429302, 57804, 62, 1, 36, "Subsubsection",
 CellTags->"MakeContext"],
Cell[1429367, 57807, 238, 5, 43, "Text",
 CellTags->"MakeContext"],
Cell[1429608, 57814, 530, 24, 25, "Text",
 CellTags->"MakeContext"]
}, Open  ]],
Cell[1430153, 57841, 57, 1, 41, "ObjectName",
 CellTags->"Mandelstam"],
Cell[CellGroupData[{
Cell[1430235, 57846, 61, 1, 36, "Subsubsection",
 CellTags->"Mandelstam"],
Cell[1430299, 57849, 1575, 73, 61, "Text",
 CellTags->"Mandelstam"],
Cell[1431877, 57924, 758, 34, 25, "Text",
 CellTags->"Mandelstam"]
}, Open  ]],
Cell[1432650, 57961, 45, 1, 41, "ObjectName",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1432720, 57966, 55, 1, 36, "Subsubsection",
 CellTags->"Map2"],
Cell[1432778, 57969, 135, 4, 25, "Text",
 CellTags->"Map2"],
Cell[1432916, 57975, 169, 9, 25, "Text",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1433122, 57989, 52, 1, 36, "Subsubsection",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1433199, 57994, 149, 5, 28, "Input",
 CellTags->"Map2"],
Cell[1433351, 58001, 202, 7, 27, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1433590, 58013, 125, 4, 28, "Input",
 CellTags->"Map2"],
Cell[1433718, 58019, 129, 4, 27, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1433884, 58028, 184, 6, 28, "Input",
 CellTags->"Map2"],
Cell[1434071, 58036, 191, 7, 27, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1434299, 58048, 125, 4, 28, "Input",
 CellTags->"Map2"],
Cell[1434427, 58054, 129, 4, 27, "Output",
 CellTags->"Map2"]
}, Open  ]]
}, Open  ]],
Cell[1434583, 58062, 67, 1, 41, "ObjectName",
 CellTags->"MemoryAvailable"],
Cell[CellGroupData[{
Cell[1434675, 58067, 66, 1, 36, "Subsubsection",
 CellTags->"MemoryAvailable"],
Cell[1434744, 58070, 230, 5, 43, "Text",
 CellTags->"MemoryAvailable"],
Cell[1434977, 58077, 314, 14, 25, "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],
Cell[1435306, 58094, 49, 1, 41, "ObjectName",
 CellTags->"MemSet"],
Cell[CellGroupData[{
Cell[1435380, 58099, 57, 1, 36, "Subsubsection",
 CellTags->"MemSet"],
Cell[1435440, 58102, 345, 9, 43, "Text",
 CellTags->"MemSet"],
Cell[1435788, 58113, 198, 9, 25, "Text",
 CellTags->"MemSet"]
}, Open  ]],
Cell[1436001, 58125, 61, 1, 41, "ObjectName",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1436087, 58130, 63, 1, 36, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[1436153, 58133, 214, 7, 25, "Text",
 CellTags->"MetricTensor"],
Cell[1436370, 58142, 584, 29, 25, "Text",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1436991, 58176, 60, 1, 36, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1437076, 58181, 154, 4, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1437233, 58187, 165, 5, 28, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1437435, 58197, 137, 4, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1437575, 58203, 109, 3, 27, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1437721, 58211, 202, 5, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1437926, 58218, 165, 5, 28, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1438128, 58228, 137, 4, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1438268, 58234, 109, 3, 27, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1438414, 58242, 181, 5, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1438598, 58249, 228, 6, 42, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1438863, 58260, 230, 6, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1439096, 58268, 278, 8, 42, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1439411, 58281, 213, 6, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1439627, 58289, 146, 4, 42, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1439810, 58298, 263, 7, 28, "Input",
 CellTags->"MetricTensor"],
Cell[1440076, 58307, 147, 4, 42, "Output",
 CellTags->"MetricTensor"]
}, Open  ]]
}, Open  ]],
Cell[1440250, 58315, 49, 1, 41, "ObjectName",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1440324, 58320, 57, 1, 36, "Subsubsection",
 CellTags->"MLimit"],
Cell[1440384, 58323, 116, 3, 25, "Text",
 CellTags->"MLimit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1440537, 58331, 54, 1, 36, "Subsubsection",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1440616, 58336, 419, 14, 28, "Input",
 CellTags->"MLimit"],
Cell[1441038, 58352, 430, 14, 43, "Output",
 CellTags->"MLimit"]
}, Open  ]]
}, Open  ]],
Cell[1441495, 58370, 53, 1, 41, "ObjectName",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1441573, 58375, 59, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1441635, 58378, 258, 5, 43, "Text",
 CellTags->"Momentum"],
Cell[1441896, 58385, 530, 24, 25, "Text",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1442463, 58414, 56, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1442522, 58417, 75, 1, 25, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1442622, 58422, 110, 3, 28, "Input",
 CellTags->"Momentum"],
Cell[1442735, 58427, 105, 3, 27, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1442855, 58433, 138, 4, 25, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1443018, 58441, 133, 4, 28, "Input",
 CellTags->"Momentum"],
Cell[1443154, 58447, 105, 3, 27, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1443274, 58453, 86, 1, 25, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1443385, 58458, 169, 5, 28, "Input",
 CellTags->"Momentum"],
Cell[1443557, 58465, 148, 4, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1443742, 58474, 153, 5, 28, "Input",
 CellTags->"Momentum"],
Cell[1443898, 58481, 123, 4, 27, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1444058, 58490, 111, 3, 28, "Input",
 CellTags->"Momentum"],
Cell[1444172, 58495, 143, 4, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1444352, 58504, 259, 8, 28, "Input",
 CellTags->"Momentum"],
Cell[1444614, 58514, 201, 7, 27, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1444852, 58526, 111, 3, 28, "Input",
 CellTags->"Momentum"],
Cell[1444966, 58531, 231, 7, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1445234, 58543, 148, 4, 28, "Input",
 CellTags->"Momentum"],
Cell[1445385, 58549, 183, 5, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1445605, 58559, 149, 4, 28, "Input",
 CellTags->"Momentum"],
Cell[1445757, 58565, 231, 7, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1446025, 58577, 216, 7, 28, "Input",
 CellTags->"Momentum"],
Cell[1446244, 58586, 148, 4, 42, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1446407, 58593, 132, 4, 28, "Input",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1446554, 58600, 67, 1, 41, "ObjectName",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1446646, 58605, 66, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[1446715, 58608, 183, 4, 43, "Text",
 CellTags->"MomentumCombine"],
Cell[1446901, 58614, 585, 23, 25, "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1447523, 58642, 63, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1447611, 58647, 303, 9, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1447917, 58658, 214, 6, 42, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1448168, 58669, 288, 9, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1448459, 58680, 218, 7, 27, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1448714, 58692, 125, 3, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1448842, 58697, 218, 7, 27, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1449097, 58709, 163, 5, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1449263, 58716, 259, 8, 42, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1449559, 58729, 170, 5, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1449732, 58736, 218, 7, 27, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1449987, 58748, 121, 3, 28, "Input",
 CellTags->"MomentumCombine"],
Cell[1450111, 58753, 259, 8, 42, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1450385, 58764, 115, 3, 28, "Input",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1450515, 58770, 69, 1, 41, "ObjectName",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1450609, 58775, 67, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[1450679, 58778, 192, 4, 43, "Text",
 CellTags->"MomentumCombine2"],
Cell[1450874, 58784, 589, 23, 25, "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1451500, 58812, 64, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1451589, 58817, 434, 13, 28, "Input",
 CellTags->"MomentumCombine2"],
Cell[1452026, 58832, 476, 20, 27, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1452539, 58857, 164, 5, 28, "Input",
 CellTags->"MomentumCombine2"],
Cell[1452706, 58864, 309, 9, 42, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]]
}, Open  ]],
Cell[1453042, 58877, 65, 1, 41, "ObjectName",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1453132, 58882, 65, 1, 36, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[1453200, 58885, 149, 4, 25, "Text",
 CellTags->"MomentumExpand"],
Cell[1453352, 58891, 349, 14, 25, "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1453738, 58910, 62, 1, 36, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1453825, 58915, 218, 6, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1454046, 58923, 189, 5, 42, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1454272, 58933, 168, 5, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1454443, 58940, 248, 9, 27, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1454728, 58954, 116, 3, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1454847, 58959, 247, 7, 42, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1455131, 58971, 211, 6, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1455345, 58979, 358, 15, 27, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1455740, 58999, 116, 3, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1455859, 59004, 284, 8, 42, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1456180, 59017, 236, 7, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1456419, 59026, 384, 16, 27, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1456840, 59047, 116, 3, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1456959, 59052, 367, 11, 42, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1457363, 59068, 112, 3, 28, "Input",
 CellTags->"MomentumExpand"],
Cell[1457478, 59073, 558, 23, 27, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]]
}, Open  ]],
Cell[1458063, 59100, 41, 1, 41, "ObjectName",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1458129, 59105, 53, 1, 36, "Subsubsection",
 CellTags->"MT"],
Cell[1458185, 59108, 182, 7, 25, "Text",
 CellTags->"MT"],
Cell[1458370, 59117, 604, 29, 25, "Text",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1459011, 59151, 50, 1, 36, "Subsubsection",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1459086, 59156, 134, 4, 28, "Input",
 CellTags->"MT"],
Cell[1459223, 59162, 155, 5, 28, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1459415, 59172, 263, 8, 28, "Input",
 CellTags->"MT"],
Cell[1459681, 59182, 99, 3, 27, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1459817, 59190, 157, 5, 28, "Input",
 CellTags->"MT"],
Cell[1459977, 59197, 136, 4, 42, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1460150, 59206, 189, 6, 28, "Input",
 CellTags->"MT"],
Cell[1460342, 59214, 218, 6, 42, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1460597, 59225, 225, 8, 28, "Input",
 CellTags->"MT"],
Cell[1460825, 59235, 136, 4, 42, "Output",
 CellTags->"MT"]
}, Open  ]]
}, Open  ]],
Cell[1460988, 59243, 43, 1, 41, "ObjectName",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1461056, 59248, 54, 1, 36, "Subsubsection",
 CellTags->"MTD"],
Cell[1461113, 59251, 184, 7, 25, "Text",
 CellTags->"MTD"],
Cell[1461300, 59260, 602, 29, 25, "Text",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1461939, 59294, 51, 1, 36, "Subsubsection",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1462015, 59299, 136, 4, 28, "Input",
 CellTags->"MTD"],
Cell[1462154, 59305, 156, 5, 28, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1462347, 59315, 266, 8, 28, "Input",
 CellTags->"MTD"],
Cell[1462616, 59325, 100, 3, 27, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1462753, 59333, 175, 6, 28, "Input",
 CellTags->"MTD"],
Cell[1462931, 59341, 151, 4, 42, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1463119, 59350, 207, 7, 28, "Input",
 CellTags->"MTD"],
Cell[1463329, 59359, 282, 8, 42, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1463648, 59372, 235, 8, 28, "Input",
 CellTags->"MTD"],
Cell[1463886, 59382, 146, 4, 42, "Output",
 CellTags->"MTD"]
}, Open  ]]
}, Open  ]],
Cell[1464059, 59390, 67, 1, 41, "ObjectName",
 CellTags->"NegativeInteger"],
Cell[CellGroupData[{
Cell[1464151, 59395, 66, 1, 36, "Subsubsection",
 CellTags->"NegativeInteger"],
Cell[1464220, 59398, 141, 4, 25, "Text",
 CellTags->"NegativeInteger"],
Cell[1464364, 59404, 186, 9, 25, "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],
Cell[1464565, 59416, 41, 1, 41, "ObjectName",
 CellTags->"Nf"],
Cell[CellGroupData[{
Cell[1464631, 59421, 53, 1, 36, "Subsubsection",
 CellTags->"Nf"],
Cell[1464687, 59424, 66, 1, 25, "Text",
 CellTags->"Nf"],
Cell[1464756, 59427, 685, 29, 25, "Text",
 CellTags->"Nf"]
}, Open  ]],
Cell[1465456, 59459, 51, 1, 41, "ObjectName",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1465532, 59464, 58, 1, 36, "Subsubsection",
 CellTags->"Nielsen"],
Cell[1465593, 59467, 86, 1, 25, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1465704, 59472, 114, 3, 28, "Input",
 CellTags->"Nielsen"],
Cell[1465821, 59477, 169, 6, 27, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1466005, 59486, 199, 9, 25, "Text",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1466241, 59500, 55, 1, 36, "Subsubsection",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1466321, 59505, 141, 4, 28, "Input",
 CellTags->"Nielsen"],
Cell[1466465, 59511, 251, 10, 27, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1466731, 59524, 176, 4, 25, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1466932, 59532, 172, 5, 28, "Input",
 CellTags->"Nielsen"],
Cell[1467107, 59539, 123, 3, 27, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1467245, 59545, 71, 1, 25, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1467341, 59550, 467, 14, 28, "Input",
 CellTags->"Nielsen"],
Cell[1467811, 59566, 992, 40, 42, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1468840, 59611, 190, 5, 28, "Input",
 CellTags->"Nielsen"],
Cell[1469033, 59618, 984, 38, 38, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1470054, 59661, 190, 5, 28, "Input",
 CellTags->"Nielsen"],
Cell[1470247, 59668, 1144, 42, 40, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1471428, 59715, 190, 5, 28, "Input",
 CellTags->"Nielsen"],
Cell[1471621, 59722, 240, 9, 27, "Output",
 CellTags->"Nielsen"]
}, Open  ]]
}, Open  ]],
Cell[1471888, 59735, 61, 1, 70, "ObjectName",
 CellTags->"NonCommFreeQ"],
Cell[CellGroupData[{
Cell[1471974, 59740, 63, 1, 70, "Subsubsection",
 CellTags->"NonCommFreeQ"],
Cell[1472040, 59743, 244, 5, 70, "Text",
 CellTags->"NonCommFreeQ"],
Cell[1472287, 59750, 519, 24, 70, "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],
Cell[1472821, 59777, 53, 1, 70, "ObjectName",
 CellTags->"NonCommQ"],
Cell[CellGroupData[{
Cell[1472899, 59782, 59, 1, 70, "Subsubsection",
 CellTags->"NonCommQ"],
Cell[1472961, 59785, 203, 4, 70, "Text",
 CellTags->"NonCommQ"],
Cell[1473167, 59791, 527, 24, 70, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1473709, 59818, 65, 1, 70, "ObjectName",
 CellTags->"NonCommutative"],
Cell[CellGroupData[{
Cell[1473799, 59823, 65, 1, 70, "Subsubsection",
 CellTags->"NonCommutative"],
Cell[1473867, 59826, 150, 4, 70, "Text",
 CellTags->"NonCommutative"],
Cell[1474020, 59832, 333, 14, 70, "Text",
 CellTags->"NonCommutative"]
}, Open  ]],
Cell[1474368, 59849, 49, 1, 70, "ObjectName",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1474442, 59854, 57, 1, 70, "Subsubsection",
 CellTags->"NTerms"],
Cell[1474502, 59857, 154, 4, 70, "Text",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1474693, 59866, 54, 1, 36, "Subsubsection",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1474772, 59871, 129, 4, 28, "Input",
 CellTags->"NTerms"],
Cell[1474904, 59877, 103, 3, 27, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1475044, 59885, 139, 4, 28, "Input",
 CellTags->"NTerms"],
Cell[1475186, 59891, 103, 3, 27, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1475326, 59899, 106, 3, 28, "Input",
 CellTags->"NTerms"],
Cell[1475435, 59904, 103, 3, 27, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1475575, 59912, 106, 3, 28, "Input",
 CellTags->"NTerms"],
Cell[1475684, 59917, 103, 3, 27, "Output",
 CellTags->"NTerms"]
}, Open  ]]
}, Open  ]],
Cell[1475814, 59924, 79, 1, 70, "ObjectName",
 CellTags->"NumberOfMetricTensors"],
Cell[CellGroupData[{
Cell[1475918, 59929, 72, 1, 70, "Subsubsection",
 CellTags->"NumberOfMetricTensors"],
Cell[1475993, 59932, 95, 1, 70, "Text",
 CellTags->"NumberOfMetricTensors"],
Cell[1476091, 59935, 167, 9, 70, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1476273, 59947, 67, 1, 70, "ObjectName",
 CellTags->"NumericalFactor"],
Cell[CellGroupData[{
Cell[1476365, 59952, 66, 1, 52, "Subsubsection",
 CellTags->"NumericalFactor"],
Cell[1476434, 59955, 119, 3, 70, "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],
Cell[1476568, 59961, 55, 1, 57, "ObjectName",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1476648, 59966, 60, 1, 52, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[1476711, 59969, 139, 4, 25, "Text",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1476887, 59978, 57, 1, 36, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1476969, 59983, 224, 7, 28, "Input",
 CellTags->"NumericQ1"],
Cell[1477196, 59992, 110, 3, 27, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1477343, 60000, 271, 9, 28, "Input",
 CellTags->"NumericQ1"],
Cell[1477617, 60011, 110, 3, 27, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1477764, 60019, 311, 10, 28, "Input",
 CellTags->"NumericQ1"],
Cell[1478078, 60031, 109, 3, 27, "Output",
 CellTags->"NumericQ1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1478236, 60040, 69, 1, 41, "ObjectName",
 CellTags->"OneLoop"],
Cell[1478308, 60043, 619, 16, 61, "Text",
 CellTags->"OneLoop"],
Cell[1478930, 60061, 968, 26, 61, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1479935, 60092, 58, 1, 36, "Subsubsection",
 CellTags->"OneLoop"],
Cell[1479996, 60095, 391, 12, 43, "Text",
 CellTags->"OneLoop"],
Cell[1480390, 60109, 352, 6, 61, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1480767, 60119, 114, 3, 28, "Input",
 CellTags->"OneLoop"],
Cell[1480884, 60124, 1275, 29, 93, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1482174, 60156, 781, 39, 25, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1482992, 60200, 55, 1, 36, "Subsubsection",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1483072, 60205, 275, 10, 28, "Input",
 CellTags->"OneLoop"],
Cell[1483350, 60217, 323, 11, 43, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1483710, 60233, 131, 4, 28, "Input",
 CellTags->"OneLoop"],
Cell[1483844, 60239, 266, 10, 27, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1484125, 60252, 363, 11, 28, "Input",
 CellTags->"OneLoop"],
Cell[1484491, 60265, 588, 20, 27, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1485104, 60289, 908, 30, 70, "Input",
 CellTags->"OneLoop"],
Cell[1486015, 60321, 1804, 62, 52, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1487834, 60386, 219, 5, 43, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1488078, 60395, 131, 4, 28, "Input",
 CellTags->"OneLoop"],
Cell[1488212, 60401, 2148, 72, 50, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1490397, 60478, 113, 3, 28, "Input",
 CellTags->"OneLoop"],
Cell[1490513, 60483, 1540, 53, 43, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1492090, 60541, 333, 11, 28, "Input",
 CellTags->"OneLoop"],
Cell[1492426, 60554, 442, 17, 46, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1492905, 60576, 131, 4, 28, "Input",
 CellTags->"OneLoop"],
Cell[1493039, 60582, 1917, 67, 44, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1494971, 60652, 106, 3, 28, "Input",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1495092, 60658, 67, 1, 41, "ObjectName",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1495184, 60663, 66, 1, 36, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[1495253, 60666, 172, 4, 25, "Text",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1495450, 60674, 130, 3, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1495583, 60679, 737, 18, 49, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[1496335, 60700, 376, 19, 25, "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496748, 60724, 63, 1, 36, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1496836, 60729, 341, 11, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1497180, 60742, 450, 17, 46, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1497667, 60764, 147, 4, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1497817, 60770, 450, 17, 46, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1498304, 60792, 203, 6, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1498510, 60800, 369, 14, 46, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1498916, 60819, 401, 12, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1499320, 60833, 580, 22, 47, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1499937, 60860, 152, 4, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1500092, 60866, 7072, 260, 94, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1507201, 61131, 253, 9, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1507457, 61142, 4303, 163, 48, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1511797, 61310, 424, 15, 28, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1512224, 61327, 1510, 58, 49, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1513761, 61389, 57, 1, 41, "ObjectName",
 CellTags->"OneLoopSum"],
Cell[CellGroupData[{
Cell[1513843, 61394, 61, 1, 36, "Subsubsection",
 CellTags->"OneLoopSum"],
Cell[1513907, 61397, 190, 4, 43, "Text",
 CellTags->"OneLoopSum"],
Cell[1514100, 61403, 178, 9, 25, "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],
Cell[1514293, 61415, 43, 1, 41, "ObjectName",
 CellTags->"OPE"],
Cell[CellGroupData[{
Cell[1514361, 61420, 54, 1, 36, "Subsubsection",
 CellTags->"OPE"],
Cell[1514418, 61423, 170, 4, 51, "Text",
 CellTags->"OPE"],
Cell[1514591, 61429, 86, 2, 25, "Text"],
Cell[CellGroupData[{
Cell[1514702, 61435, 60, 1, 28, "Input"],
Cell[1514765, 61438, 156, 5, 27, "Output"]
}, Open  ]],
Cell[1514936, 61446, 174, 9, 25, "Text",
 CellTags->"OPE"]
}, Open  ]],
Cell[1515125, 61458, 53, 1, 41, "ObjectName",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1515203, 61463, 59, 1, 36, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[1515265, 61466, 140, 4, 25, "Text",
 CellTags->"OPEDelta"],
Cell[1515408, 61472, 212, 9, 25, "Text",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1515657, 61486, 56, 1, 36, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1515738, 61491, 146, 4, 28, "Input",
 CellTags->"OPEDelta"],
Cell[1515887, 61497, 251, 10, 27, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1516175, 61512, 133, 4, 28, "Input",
 CellTags->"OPEDelta"],
Cell[1516311, 61518, 105, 3, 27, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1516453, 61526, 141, 4, 28, "Input",
 CellTags->"OPEDelta"],
Cell[1516597, 61532, 105, 3, 27, "Output",
 CellTags->"OPEDelta"]
}, Open  ]]
}, Open  ]],
Cell[1516729, 61539, 81, 1, 41, "ObjectName",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1516835, 61544, 55, 1, 36, "Subsubsection",
 CellTags->"OPEi"],
Cell[1516893, 61547, 171, 4, 25, "Text",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1517101, 61556, 52, 1, 36, "Subsubsection",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1517178, 61561, 75, 2, 28, "Input",
 CellTags->"OPEi"],
Cell[1517256, 61565, 101, 3, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1517394, 61573, 259, 7, 28, "Input",
 CellTags->"OPEi"],
Cell[1517656, 61582, 228, 7, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1517921, 61594, 1076, 40, 50, "Input",
 CellTags->"OPEi"],
Cell[1519000, 61636, 202, 6, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[1519217, 61645, 55, 1, 25, "Text",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1519297, 61650, 519, 17, 28, "Input",
 CellTags->"OPEi"],
Cell[1519819, 61669, 529, 19, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1520385, 61693, 534, 19, 28, "Input",
 CellTags->"OPEi"],
Cell[1520922, 61714, 413, 15, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1521372, 61734, 519, 17, 28, "Input",
 CellTags->"OPEi"],
Cell[1521894, 61753, 529, 19, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]]
}, Open  ]],
Cell[1522450, 61776, 73, 1, 41, "ObjectName",
 CellTags->"OPEInsert"],
Cell[CellGroupData[{
Cell[1522548, 61781, 60, 1, 36, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1522611, 61784, 264, 5, 43, "Text",
 CellTags->"OPEInsert"],
Cell[1522878, 61791, 165, 9, 25, "Text",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1523080, 61805, 57, 1, 36, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1523140, 61808, 76, 2, 28, "Input",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[1523231, 61813, 67, 1, 41, "ObjectName",
 CellTags->"OPEInt"],
Cell[CellGroupData[{
Cell[1523323, 61818, 57, 1, 36, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1523383, 61821, 257, 5, 43, "Text",
 CellTags->"OPEInt"],
Cell[1523643, 61828, 162, 9, 25, "Text",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1523842, 61842, 54, 1, 36, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1523899, 61845, 73, 2, 28, "Input",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[1523987, 61850, 77, 1, 41, "ObjectName",
 CellTags->"OPEIntDelta"],
Cell[CellGroupData[{
Cell[1524089, 61855, 62, 1, 36, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1524154, 61858, 478, 10, 87, "Text",
 CellTags->"OPEIntDelta"],
Cell[1524635, 61870, 167, 9, 25, "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1524839, 61884, 59, 1, 36, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1524901, 61887, 78, 2, 28, "Input",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[1524994, 61892, 79, 1, 41, "ObjectName",
 CellTags->"OPEIntegrate"],
Cell[CellGroupData[{
Cell[1525098, 61897, 63, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1525164, 61900, 266, 5, 43, "Text",
 CellTags->"OPEIntegrate"],
Cell[1525433, 61907, 168, 9, 25, "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1525638, 61921, 60, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1525701, 61924, 79, 2, 28, "Input",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[1525795, 61929, 89, 1, 41, "ObjectName",
 CellTags->"OPEIntegrateDelta"],
Cell[CellGroupData[{
Cell[1525909, 61934, 68, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1525980, 61937, 499, 10, 87, "Text",
 CellTags->"OPEIntegrateDelta"],
Cell[1526482, 61949, 173, 9, 25, "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1526692, 61963, 65, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1526760, 61966, 84, 2, 28, "Input",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[1526859, 61971, 65, 1, 41, "ObjectName",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1526949, 61976, 65, 1, 36, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[1527017, 61979, 85, 1, 25, "Text",
 CellTags->"OPESumExplicit"],
Cell[1527105, 61982, 307, 14, 25, "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1527449, 62001, 62, 1, 36, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1527536, 62006, 387, 13, 28, "Input",
 CellTags->"OPESumExplicit"],
Cell[1527926, 62021, 385, 14, 51, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1528348, 62040, 123, 3, 28, "Input",
 CellTags->"OPESumExplicit"],
Cell[1528474, 62045, 408, 16, 40, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1528919, 62066, 554, 18, 28, "Input",
 CellTags->"OPESumExplicit"],
Cell[1529476, 62086, 678, 26, 53, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1530191, 62117, 123, 3, 28, "Input",
 CellTags->"OPESumExplicit"],
Cell[1530317, 62122, 2216, 83, 86, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1532548, 62208, 157, 5, 28, "Input",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1532720, 62216, 49, 1, 70, "ObjectName",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1532794, 62221, 57, 1, 70, "Subsubsection",
 CellTags->"OPESum"],
Cell[1532854, 62224, 127, 4, 70, "Text",
 CellTags->"OPESum"],
Cell[1532984, 62230, 323, 14, 70, "Text",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1533344, 62249, 54, 1, 36, "Subsubsection",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1533423, 62254, 460, 15, 28, "Input",
 CellTags->"OPESum"],
Cell[1533886, 62271, 833, 30, 51, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1534756, 62306, 115, 3, 28, "Input",
 CellTags->"OPESum"],
Cell[1534874, 62311, 1604, 60, 45, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1536515, 62376, 546, 18, 28, "Input",
 CellTags->"OPESum"],
Cell[1537064, 62396, 670, 26, 53, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1537771, 62427, 115, 3, 28, "Input",
 CellTags->"OPESum"],
Cell[1537889, 62432, 2208, 83, 86, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1540112, 62518, 149, 5, 28, "Input",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1540276, 62526, 65, 1, 70, "ObjectName",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1540366, 62531, 65, 1, 70, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[1540434, 62534, 92, 1, 70, "Text",
 CellTags->"OPESumSimplify"],
Cell[1540529, 62537, 307, 14, 70, "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1540873, 62556, 62, 1, 36, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1540960, 62561, 143, 4, 28, "Input",
 CellTags->"OPESumSimplify"],
Cell[1541106, 62567, 1052, 37, 50, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542195, 62609, 89, 2, 28, "Input",
 CellTags->"OPESumSimplify"],
Cell[1542287, 62613, 1099, 38, 50, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1543423, 62656, 273, 8, 28, "Input",
 CellTags->"OPESumSimplify"],
Cell[1543699, 62666, 283, 11, 50, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1544019, 62682, 439, 14, 28, "Input",
 CellTags->"OPESumSimplify"],
Cell[1544461, 62698, 553, 22, 50, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1545051, 62725, 116, 3, 28, "Input",
 CellTags->"OPESumSimplify"],
Cell[1545170, 62730, 404, 13, 42, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1545601, 62747, 54, 1, 41, "ObjectName",
 CellTags->"OPE1Loop"],
Cell[CellGroupData[{
Cell[1545680, 62752, 59, 1, 36, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1545742, 62755, 119, 3, 25, "Text",
 CellTags->"OPE1Loop"],
Cell[1545864, 62760, 173, 9, 25, "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1546074, 62774, 56, 1, 36, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1546133, 62777, 75, 2, 28, "Input",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[1546223, 62782, 69, 1, 41, "ObjectName",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1546317, 62787, 58, 1, 36, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1546378, 62790, 184, 4, 43, "Text",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1546587, 62798, 114, 3, 28, "Input",
 CellTags->"OPE2TID"],
Cell[1546704, 62803, 343, 10, 27, "Output",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1547062, 62816, 172, 9, 25, "Text",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1547271, 62830, 55, 1, 36, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1547329, 62833, 74, 2, 28, "Input",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1547418, 62838, 63, 1, 41, "ObjectName",
 CellTags->"OptionsSelect"],
Cell[CellGroupData[{
Cell[1547506, 62843, 64, 1, 36, "Subsubsection",
 CellTags->"OptionsSelect"],
Cell[1547573, 62846, 219, 5, 43, "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],
Cell[1547807, 62854, 45, 1, 41, "ObjectName",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1547877, 62859, 55, 1, 36, "Subsubsection",
 CellTags->"Pair"],
Cell[1547935, 62862, 499, 11, 61, "Text",
 CellTags->"Pair"],
Cell[1548437, 62875, 989, 49, 25, "Text",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1549463, 62929, 52, 1, 36, "Subsubsection",
 CellTags->"Pair"],
Cell[1549518, 62932, 84, 1, 25, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1549627, 62937, 218, 6, 28, "Input",
 CellTags->"Pair"],
Cell[1549848, 62945, 157, 5, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[1550020, 62953, 73, 1, 25, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1550118, 62958, 268, 8, 28, "Input",
 CellTags->"Pair"],
Cell[1550389, 62968, 157, 5, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1550583, 62978, 208, 6, 28, "Input",
 CellTags->"Pair"],
Cell[1550794, 62986, 236, 10, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1551067, 63001, 197, 6, 28, "Input",
 CellTags->"Pair"],
Cell[1551267, 63009, 203, 8, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1551507, 63022, 197, 6, 28, "Input",
 CellTags->"Pair"],
Cell[1551707, 63030, 159, 6, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1551903, 63041, 222, 7, 28, "Input",
 CellTags->"Pair"],
Cell[1552128, 63050, 238, 9, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1552403, 63064, 223, 7, 28, "Input",
 CellTags->"Pair"],
Cell[1552629, 63073, 223, 9, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1552889, 63087, 223, 7, 28, "Input",
 CellTags->"Pair"],
Cell[1553115, 63096, 223, 9, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1553375, 63110, 308, 10, 28, "Input",
 CellTags->"Pair"],
Cell[1553686, 63122, 101, 3, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1553824, 63130, 272, 8, 28, "Input",
 CellTags->"Pair"],
Cell[1554099, 63140, 292, 12, 27, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1554428, 63157, 383, 12, 28, "Input",
 CellTags->"Pair"],
Cell[1554814, 63171, 101, 3, 27, "Output",
 CellTags->"Pair"]
}, Open  ]]
}, Open  ]],
Cell[1554942, 63178, 59, 1, 41, "ObjectName",
 CellTags->"PairCollect"],
Cell[CellGroupData[{
Cell[1555026, 63183, 62, 1, 36, "Subsubsection",
 CellTags->"PairCollect"],
Cell[1555091, 63186, 153, 4, 25, "Text",
 CellTags->"PairCollect"],
Cell[1555247, 63192, 286, 14, 25, "Text",
 CellTags->"PairCollect"]
}, Open  ]],
Cell[1555548, 63209, 61, 1, 41, "ObjectName",
 CellTags->"PairContract"],
Cell[CellGroupData[{
Cell[1555634, 63214, 63, 1, 36, "Subsubsection",
 CellTags->"PairContract"],
Cell[1555700, 63217, 118, 3, 25, "Text",
 CellTags->"PairContract"],
Cell[1555821, 63222, 275, 13, 25, "Text",
 CellTags->"PairContract"]
}, Open  ]],
Cell[1556111, 63238, 63, 1, 41, "ObjectName",
 CellTags->"PairContract2"],
Cell[CellGroupData[{
Cell[1556199, 63243, 64, 1, 36, "Subsubsection",
 CellTags->"PairContract2"],
Cell[1556266, 63246, 142, 4, 25, "Text",
 CellTags->"PairContract2"],
Cell[1556411, 63252, 398, 18, 25, "Text",
 CellTags->"PairContract2"]
}, Open  ]],
Cell[1556824, 63273, 63, 1, 41, "ObjectName",
 CellTags->"PairContract3"],
Cell[CellGroupData[{
Cell[1556912, 63278, 64, 1, 36, "Subsubsection",
 CellTags->"PairContract3"],
Cell[1556979, 63281, 142, 4, 25, "Text",
 CellTags->"PairContract3"],
Cell[1557124, 63287, 398, 18, 25, "Text",
 CellTags->"PairContract3"]
}, Open  ]],
Cell[1557537, 63308, 53, 1, 41, "ObjectName",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1557615, 63313, 59, 1, 36, "Subsubsection",
 CellTags->"PartialD"],
Cell[1557677, 63316, 864, 35, 43, "Text",
 CellTags->"PartialD"],
Cell[1558544, 63353, 581, 24, 25, "Text",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1559162, 63382, 65, 1, 35, "Subsubsection",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1559252, 63387, 233, 7, 28, "Input",
 CellTags->"PartialD"],
Cell[1559488, 63396, 302, 11, 33, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1559827, 63412, 116, 3, 28, "Input",
 CellTags->"PartialD"],
Cell[1559946, 63417, 351, 14, 27, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1560334, 63436, 114, 3, 28, "Input",
 CellTags->"PartialD"],
Cell[1560451, 63441, 287, 7, 42, "Output",
 CellTags->"PartialD"]
}, Open  ]]
}, Open  ]],
Cell[1560765, 63452, 71, 1, 41, "ObjectName",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1560861, 63457, 68, 1, 36, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[1560932, 63460, 191, 5, 25, "Text",
 CellTags->"PartialDRelations"],
Cell[1561126, 63467, 700, 29, 25, "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1561863, 63501, 74, 1, 35, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1561962, 63506, 344, 10, 28, "Input",
 CellTags->"PartialDRelations"],
Cell[1562309, 63518, 348, 12, 33, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1562694, 63535, 256, 7, 28, "Input",
 CellTags->"PartialDRelations"],
Cell[1562953, 63544, 855, 34, 27, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]]
}, Open  ]],
Cell[1563835, 63582, 71, 1, 41, "ObjectName",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1563931, 63587, 68, 1, 36, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[1564002, 63590, 277, 6, 43, "Text",
 CellTags->"PartialFourVector"],
Cell[1564282, 63598, 402, 19, 25, "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1564721, 63622, 74, 1, 35, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1564820, 63627, 382, 11, 33, "Input",
 CellTags->"PartialFourVector"],
Cell[1565205, 63640, 399, 15, 28, "Output",
 CellTags->"PartialFourVector"]
}, Open  ]]
}, Open  ]],
Cell[1565631, 63659, 69, 1, 41, "ObjectName",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1565725, 63664, 67, 1, 36, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[1565795, 63667, 283, 6, 43, "Text",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1566103, 63677, 132, 3, 28, "Input",
 CellTags->"PartialIntegrate"],
Cell[1566238, 63682, 185, 6, 27, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566472, 63694, 73, 1, 35, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1566570, 63699, 326, 10, 28, "Input",
 CellTags->"PartialIntegrate"],
Cell[1566899, 63711, 1239, 41, 45, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1568175, 63757, 84, 2, 28, "Input"],
Cell[1568262, 63761, 220, 5, 82, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1568519, 63771, 275, 9, 28, "Input",
 CellTags->"PartialIntegrate"],
Cell[1568797, 63782, 507, 19, 38, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1569341, 63806, 293, 10, 41, "Input",
 CellTags->"PartialIntegrate"],
Cell[1569637, 63818, 378, 14, 44, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1570052, 63837, 443, 14, 28, "Input",
 CellTags->"PartialIntegrate"],
Cell[1570498, 63853, 302, 10, 46, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1570837, 63868, 320, 11, 28, "Input",
 CellTags->"PartialIntegrate"],
Cell[1571160, 63881, 246, 8, 42, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1571421, 63892, 138, 4, 28, "Input",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1571574, 63899, 57, 1, 41, "ObjectName",
 CellTags->"PartitHead"],
Cell[CellGroupData[{
Cell[1571656, 63904, 61, 1, 36, "Subsubsection",
 CellTags->"PartitHead"],
Cell[1571720, 63907, 167, 4, 25, "Text",
 CellTags->"PartitHead"]
}, Open  ]],
Cell[1571902, 63914, 57, 1, 41, "ObjectName",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1571984, 63919, 61, 1, 36, "Subsubsection",
 CellTags->"PauliSigma"],
Cell[1572048, 63922, 252, 5, 43, "Text",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1572325, 63931, 141, 4, 28, "Input",
 CellTags->"PauliSigma"],
Cell[1572469, 63937, 3957, 118, 123, "Output",
 CellTags->"PauliSigma"]
}, Open  ]]
}, Open  ]],
Cell[1576453, 64059, 45, 1, 41, "ObjectName",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1576523, 64064, 55, 1, 36, "Subsubsection",
 CellTags->"PaVe"],
Cell[1576581, 64067, 430, 8, 61, "Text",
 CellTags->"PaVe"],
Cell[1577014, 64077, 266, 8, 25, "Text",
 CellTags->"PaVe"],
Cell[1577283, 64087, 181, 9, 25, "Text",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1577501, 64101, 52, 1, 36, "Subsubsection",
 CellTags->"PaVe"],
Cell[1577556, 64104, 78, 1, 25, "Text",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1577659, 64109, 303, 10, 28, "Input",
 CellTags->"PaVe"],
Cell[1577965, 64121, 304, 9, 27, "Output",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1578306, 64135, 271, 9, 28, "Input",
 CellTags->"PaVe"],
Cell[1578580, 64146, 311, 11, 27, "Output",
 CellTags->"PaVe"]
}, Open  ]]
}, Open  ]],
Cell[1578918, 64161, 63, 1, 41, "ObjectName",
 CellTags->"PaVeOrderList"],
Cell[CellGroupData[{
Cell[1579006, 64166, 64, 1, 36, "Subsubsection",
 CellTags->"PaVeOrderList"],
Cell[1579073, 64169, 176, 4, 25, "Text",
 CellTags->"PaVeOrderList"],
Cell[1579252, 64175, 487, 23, 25, "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],
Cell[1579754, 64201, 55, 1, 41, "ObjectName",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1579834, 64206, 60, 1, 36, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[1579897, 64209, 564, 16, 61, "Text",
 CellTags->"PaVeOrder"],
Cell[1580464, 64227, 186, 9, 25, "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1580687, 64241, 57, 1, 36, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1580769, 64246, 413, 12, 28, "Input",
 CellTags->"PaVeOrder"],
Cell[1581185, 64260, 382, 12, 27, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1581604, 64277, 413, 12, 28, "Input",
 CellTags->"PaVeOrder"],
Cell[1582020, 64291, 382, 12, 27, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1582439, 64308, 706, 20, 50, "Input",
 CellTags->"PaVeOrder"],
Cell[1583148, 64330, 623, 18, 27, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]]
}, Open  ]],
Cell[1583798, 64352, 57, 1, 41, "ObjectName",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1583880, 64357, 61, 1, 36, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[1583944, 64360, 167, 4, 25, "Text",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1584136, 64368, 120, 3, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1584259, 64373, 446, 13, 27, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1584742, 64391, 106, 3, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1584851, 64396, 332, 5, 59, "Print",
 CellTags->{"Info3387712116-6749064", "PaVeReduce"}]
}, Open  ]],
Cell[1585198, 64404, 279, 14, 25, "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1585514, 64423, 58, 1, 36, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1585597, 64428, 520, 16, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1586120, 64446, 3016, 104, 92, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1589173, 64555, 106, 3, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1589282, 64560, 136, 4, 27, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1589455, 64569, 107, 3, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1589565, 64574, 136, 4, 27, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1589738, 64583, 491, 15, 50, "Input",
 CellTags->"PaVeReduce"],
Cell[1590232, 64600, 445, 14, 40, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1590714, 64619, 398, 12, 50, "Input",
 CellTags->"PaVeReduce"],
Cell[1591115, 64633, 792, 23, 75, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1591922, 64659, 209, 7, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1592134, 64668, 162, 5, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1592321, 64677, 633, 17, 50, "Input",
 CellTags->"PaVeReduce"],
Cell[1592957, 64696, 8271, 259, 311, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1601243, 64958, 267, 8, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1601535, 64970, 375, 11, 28, "Input",
 CellTags->"PaVeReduce"],
Cell[1601913, 64983, 2978, 58, 659, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1604906, 65044, 253, 7, 28, "Input",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1605174, 65054, 41, 1, 41, "ObjectName",
 CellTags->"PD"],
Cell[CellGroupData[{
Cell[1605240, 65059, 53, 1, 36, "Subsubsection",
 CellTags->"PD"],
Cell[1605296, 65062, 81, 1, 25, "Text",
 CellTags->"PD"],
Cell[1605380, 65065, 310, 14, 25, "Text",
 CellTags->"PD"]
}, Open  ]],
Cell[1605705, 65082, 69, 1, 41, "ObjectName",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1605799, 65087, 67, 1, 36, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[1605869, 65090, 147, 4, 25, "Text",
 CellTags->"PlusDistribution"],
Cell[1606019, 65096, 193, 9, 25, "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1606249, 65110, 64, 1, 36, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1606338, 65115, 198, 6, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1606539, 65123, 208, 7, 41, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1606784, 65135, 254, 8, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1607041, 65145, 268, 9, 44, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1607346, 65159, 329, 10, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1607678, 65171, 113, 3, 27, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1607828, 65179, 389, 12, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1608220, 65193, 113, 3, 27, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608370, 65201, 417, 13, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1608790, 65216, 113, 3, 27, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608940, 65224, 307, 10, 28, "Input",
 CellTags->"PlusDistribution"],
Cell[1609250, 65236, 376, 13, 44, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]]
}, Open  ]],
Cell[1609653, 65253, 61, 1, 41, "ObjectName",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1609739, 65258, 63, 1, 36, "Subsubsection",
 CellTags->"Polarization"],
Cell[1609805, 65261, 769, 12, 115, "Text",
 CellTags->"Polarization"],
Cell[1610577, 65275, 484, 19, 25, "Text",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1611098, 65299, 60, 1, 36, "Subsubsection",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1611183, 65304, 118, 3, 28, "Input",
 CellTags->"Polarization"],
Cell[1611304, 65309, 151, 4, 27, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1611492, 65318, 153, 4, 28, "Input",
 CellTags->"Polarization"],
Cell[1611648, 65324, 168, 4, 42, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1611853, 65333, 160, 5, 28, "Input",
 CellTags->"Polarization"],
Cell[1612016, 65340, 487, 19, 27, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1612540, 65364, 196, 6, 28, "Input",
 CellTags->"Polarization"],
Cell[1612739, 65372, 187, 5, 42, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1612963, 65382, 155, 4, 28, "Input",
 CellTags->"Polarization"],
Cell[1613121, 65388, 270, 9, 27, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1613428, 65402, 194, 6, 28, "Input",
 CellTags->"Polarization"],
Cell[1613625, 65410, 243, 6, 42, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1613905, 65421, 302, 8, 28, "Input",
 CellTags->"Polarization"],
Cell[1614210, 65431, 109, 3, 27, "Output",
 CellTags->"Polarization"]
}, Open  ]]
}, Open  ]],
Cell[1614346, 65438, 67, 1, 41, "ObjectName",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1614438, 65443, 66, 1, 36, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[1614507, 65446, 277, 5, 43, "Text",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1614809, 65455, 130, 3, 28, "Input",
 CellTags->"PolarizationSum"],
Cell[1614942, 65460, 173, 5, 27, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[1615130, 65468, 198, 9, 25, "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1615365, 65482, 63, 1, 36, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1615453, 65487, 155, 4, 28, "Input",
 CellTags->"PolarizationSum"],
Cell[1615611, 65493, 183, 6, 27, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1615831, 65504, 165, 4, 28, "Input",
 CellTags->"PolarizationSum"],
Cell[1615999, 65510, 605, 25, 43, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1616641, 65540, 175, 4, 28, "Input",
 CellTags->"PolarizationSum"],
Cell[1616819, 65546, 1775, 76, 43, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1618631, 65627, 247, 7, 28, "Input",
 CellTags->"PolarizationSum"],
Cell[1618881, 65636, 4326, 175, 48, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]]
}, Open  ]],
Cell[1623234, 65815, 81, 1, 41, "ObjectName",
 CellTags->"PolarizationUncontract"],
Cell[CellGroupData[{
Cell[1623340, 65820, 73, 1, 36, "Subsubsection",
 CellTags->"PolarizationUncontract"],
Cell[1623416, 65823, 152, 4, 25, "Text",
 CellTags->"PolarizationUncontract"],
Cell[1623571, 65829, 321, 14, 25, "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],
Cell[1623907, 65846, 73, 1, 41, "ObjectName",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1624005, 65851, 69, 1, 36, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[1624077, 65854, 103, 1, 25, "Text",
 CellTags->"PolarizationVector"],
Cell[1624183, 65857, 409, 18, 25, "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1624629, 65880, 66, 1, 36, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1624720, 65885, 157, 4, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1624880, 65891, 258, 9, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1625175, 65905, 194, 5, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1625372, 65912, 268, 9, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1625655, 65924, 241, 9, 25, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1625921, 65937, 190, 5, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1626114, 65944, 310, 8, 42, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1626439, 65955, 440, 18, 25, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1626904, 65977, 284, 8, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1627191, 65987, 426, 17, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1627654, 66009, 121, 3, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1627778, 66014, 115, 3, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1627930, 66022, 310, 9, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1628243, 66033, 488, 20, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1628768, 66058, 121, 3, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1628892, 66063, 336, 12, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1629243, 66078, 1511, 51, 42, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1630779, 66133, 310, 9, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1631092, 66144, 488, 20, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1631617, 66169, 121, 3, 28, "Input",
 CellTags->"PolarizationVector"],
Cell[1631741, 66174, 336, 12, 27, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1632092, 66189, 153, 4, 28, "Input",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1632260, 66196, 67, 1, 41, "ObjectName",
 CellTags->"PositiveInteger"],
Cell[CellGroupData[{
Cell[1632352, 66201, 66, 1, 36, "Subsubsection",
 CellTags->"PositiveInteger"],
Cell[1632421, 66204, 136, 4, 25, "Text",
 CellTags->"PositiveInteger"],
Cell[1632560, 66210, 186, 9, 25, "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],
Cell[1632761, 66222, 65, 1, 41, "ObjectName",
 CellTags->"PositiveNumber"],
Cell[CellGroupData[{
Cell[1632851, 66227, 65, 1, 36, "Subsubsection",
 CellTags->"PositiveNumber"],
Cell[1632919, 66230, 146, 4, 25, "Text",
 CellTags->"PositiveNumber"],
Cell[1633068, 66236, 185, 9, 25, "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],
Cell[1633268, 66248, 59, 1, 41, "ObjectName",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1633352, 66253, 62, 1, 36, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[1633417, 66256, 375, 16, 25, "Text",
 CellTags->"PowerFactor"],
Cell[1633795, 66274, 197, 9, 25, "Text",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1634029, 66288, 59, 1, 36, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1634113, 66293, 144, 5, 28, "Input",
 CellTags->"PowerFactor"],
Cell[1634260, 66300, 181, 6, 27, "Output",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1634478, 66311, 116, 3, 28, "Input",
 CellTags->"PowerFactor"],
Cell[1634597, 66316, 181, 6, 27, "Output",
 CellTags->"PowerFactor"]
}, Open  ]]
}, Open  ]],
Cell[1634805, 66326, 63, 1, 41, "ObjectName",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1634893, 66331, 64, 1, 36, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[1634960, 66334, 2559, 112, 63, "Text",
 CellTags->"PowerSimplify"],
Cell[1637522, 66448, 282, 14, 25, "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1637841, 66467, 61, 1, 36, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1637927, 66472, 234, 8, 28, "Input",
 CellTags->"PowerSimplify"],
Cell[1638164, 66482, 110, 3, 27, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1638311, 66490, 239, 8, 28, "Input",
 CellTags->"PowerSimplify"],
Cell[1638553, 66500, 178, 6, 27, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1638768, 66511, 239, 8, 28, "Input",
 CellTags->"PowerSimplify"],
Cell[1639010, 66521, 178, 6, 27, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1639225, 66532, 190, 6, 28, "Input",
 CellTags->"PowerSimplify"],
Cell[1639418, 66540, 178, 6, 27, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1639623, 66550, 49, 1, 41, "ObjectName",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1639697, 66555, 57, 1, 36, "Subsubsection",
 CellTags->"Power2"],
Cell[1639757, 66558, 213, 5, 43, "Text",
 CellTags->"Power2"],
Cell[1639973, 66565, 186, 9, 25, "Text",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1640196, 66579, 54, 1, 36, "Subsubsection",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1640275, 66584, 147, 5, 28, "Input",
 CellTags->"Power2"],
Cell[1640425, 66591, 171, 6, 29, "Output",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1640633, 66602, 148, 5, 28, "Input",
 CellTags->"Power2"],
Cell[1640784, 66609, 257, 10, 29, "Output",
 CellTags->"Power2"]
}, Open  ]]
}, Open  ]],
Cell[1641068, 66623, 67, 1, 41, "ObjectName",
 CellTags->"PostFortranFile"],
Cell[CellGroupData[{
Cell[1641160, 66628, 66, 1, 36, "Subsubsection",
 CellTags->"PostFortranFile"],
Cell[1641229, 66631, 229, 5, 43, "Text",
 CellTags->"PostFortranFile"],
Cell[1641461, 66638, 308, 14, 25, "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],
Cell[1641784, 66655, 55, 1, 41, "ObjectName",
 CellTags->"Prefactor"],
Cell[CellGroupData[{
Cell[1641864, 66660, 60, 1, 36, "Subsubsection",
 CellTags->"Prefactor"],
Cell[1641927, 66663, 290, 6, 43, "Text",
 CellTags->"Prefactor"],
Cell[1642220, 66671, 293, 14, 25, "Text",
 CellTags->"Prefactor"]
}, Open  ]],
Cell[1642528, 66688, 65, 1, 41, "ObjectName",
 CellTags->"PreFortranFile"],
Cell[CellGroupData[{
Cell[1642618, 66693, 65, 1, 36, "Subsubsection",
 CellTags->"PreFortranFile"],
Cell[1642686, 66696, 232, 5, 43, "Text",
 CellTags->"PreFortranFile"],
Cell[1642921, 66703, 310, 14, 25, "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],
Cell[1643246, 66720, 79, 1, 41, "ObjectName",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1643350, 66725, 72, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[1643425, 66728, 549, 16, 61, "Text",
 CellTags->"PropagatorDenominator"],
Cell[1643977, 66746, 493, 19, 25, "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1644507, 66770, 69, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1644601, 66775, 178, 5, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1644782, 66782, 215, 7, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1645034, 66794, 136, 3, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1645173, 66799, 164, 5, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1645374, 66809, 189, 5, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1645566, 66816, 215, 7, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1645818, 66828, 158, 4, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1645979, 66834, 235, 6, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1646251, 66845, 194, 5, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1646448, 66852, 235, 6, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1646720, 66863, 145, 3, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1646868, 66868, 253, 9, 42, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1647158, 66882, 127, 3, 28, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1647288, 66887, 325, 10, 56, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1647628, 66900, 121, 3, 28, "Input",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1647764, 66906, 95, 1, 70, "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1647884, 66911, 80, 1, 70, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1647967, 66914, 243, 5, 70, "Text",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1648213, 66921, 379, 14, 70, "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1648629, 66940, 77, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1648731, 66945, 308, 11, 28, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1649042, 66958, 357, 12, 45, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1649436, 66975, 181, 5, 28, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1649620, 66982, 766, 29, 42, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1650423, 67016, 131, 3, 28, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1650557, 67021, 547, 18, 56, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1651131, 67043, 43, 1, 70, "ObjectName",
 CellTags->"QGV"],
Cell[CellGroupData[{
Cell[1651199, 67048, 54, 1, 70, "Subsubsection",
 CellTags->"QGV"],
Cell[1651256, 67051, 72, 1, 70, "Text",
 CellTags->"QGV"],
Cell[1651331, 67054, 198, 9, 70, "Text",
 CellTags->"QGV"]
}, Open  ]],
Cell[1651544, 67066, 41, 1, 70, "ObjectName",
 CellTags->"QO"],
Cell[CellGroupData[{
Cell[1651610, 67071, 53, 1, 70, "Subsubsection",
 CellTags->"QO"],
Cell[1651666, 67074, 73, 1, 70, "Text",
 CellTags->"QO"],
Cell[1651742, 67077, 206, 9, 70, "Text",
 CellTags->"QO"]
}, Open  ]],
Cell[1651963, 67089, 41, 1, 70, "ObjectName",
 CellTags->"QP"],
Cell[CellGroupData[{
Cell[1652029, 67094, 53, 1, 52, "Subsubsection",
 CellTags->"QP"],
Cell[1652085, 67097, 169, 4, 70, "Text",
 CellTags->"QP"],
Cell[1652257, 67103, 194, 9, 41, "Text",
 CellTags->"QP"]
}, Open  ]],
Cell[1652466, 67115, 61, 1, 57, "ObjectName",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1652552, 67120, 63, 1, 52, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1652618, 67123, 404, 9, 59, "Text",
 CellTags->"QuantumField"],
Cell[1653025, 67134, 421, 19, 41, "Text",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1653483, 67158, 60, 1, 52, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1653546, 67161, 71, 1, 25, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1653642, 67166, 118, 3, 28, "Input",
 CellTags->"QuantumField"],
Cell[1653763, 67171, 109, 3, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1653909, 67179, 131, 3, 28, "Input",
 CellTags->"QuantumField"],
Cell[1654043, 67184, 137, 4, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1654217, 67193, 127, 3, 28, "Input",
 CellTags->"QuantumField"],
Cell[1654347, 67198, 114, 3, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1654476, 67204, 80, 1, 25, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1654581, 67209, 169, 5, 28, "Input",
 CellTags->"QuantumField"],
Cell[1654753, 67216, 135, 4, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1654903, 67223, 94, 1, 25, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1655022, 67228, 212, 6, 28, "Input",
 CellTags->"QuantumField"],
Cell[1655237, 67236, 319, 14, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1655593, 67255, 114, 3, 28, "Input",
 CellTags->"QuantumField"],
Cell[1655710, 67260, 255, 6, 42, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1655980, 67269, 358, 13, 25, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1656363, 67286, 206, 6, 28, "Input",
 CellTags->"QuantumField"],
Cell[1656572, 67294, 259, 10, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1656846, 67307, 185, 4, 43, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1657056, 67315, 244, 7, 28, "Input",
 CellTags->"QuantumField"],
Cell[1657303, 67324, 245, 6, 42, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1657563, 67333, 88, 1, 25, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1657676, 67338, 219, 6, 28, "Input",
 CellTags->"QuantumField"],
Cell[1657898, 67346, 355, 14, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1658290, 67365, 184, 5, 28, "Input",
 CellTags->"QuantumField"],
Cell[1658477, 67372, 257, 9, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1658771, 67386, 259, 7, 28, "Input",
 CellTags->"QuantumField"],
Cell[1659033, 67395, 404, 16, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1659474, 67416, 291, 9, 28, "Input",
 CellTags->"QuantumField"],
Cell[1659768, 67427, 413, 16, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1660218, 67448, 232, 6, 28, "Input",
 CellTags->"QuantumField"],
Cell[1660453, 67456, 112, 3, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]]
}, Open  ]],
Cell[1660592, 67463, 57, 1, 57, "ObjectName",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1660674, 67468, 61, 1, 36, "Subsubsection",
 CellTags->"QuarkField"],
Cell[1660738, 67471, 189, 4, 43, "Text",
 CellTags->"QuarkField"],
Cell[1660930, 67477, 321, 14, 25, "Text",
 CellTags->"QuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1661288, 67496, 58, 1, 36, "Subsubsection",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1661371, 67501, 87, 2, 28, "Input",
 CellTags->"QuarkField"],
Cell[1661461, 67505, 112, 3, 27, "Output",
 CellTags->"QuarkField"]
}, Open  ]]
}, Open  ]],
Cell[1661600, 67512, 69, 1, 41, "ObjectName",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1661694, 67517, 67, 1, 36, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[1661764, 67520, 260, 9, 25, "Text",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1662049, 67533, 132, 3, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1662184, 67538, 491, 13, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[1662690, 67554, 196, 9, 25, "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1662923, 67568, 64, 1, 36, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1663012, 67573, 153, 4, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1663168, 67579, 149, 4, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663354, 67588, 208, 5, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1663565, 67595, 149, 4, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663751, 67604, 208, 5, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1663962, 67611, 149, 4, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1664148, 67620, 208, 5, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1664359, 67627, 149, 4, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1664545, 67636, 315, 9, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1664863, 67647, 366, 11, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1665266, 67663, 316, 9, 28, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1665585, 67674, 367, 11, 27, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1665979, 67689, 55, 1, 41, "ObjectName",
 CellTags->"QuarkMass"],
Cell[CellGroupData[{
Cell[1666059, 67694, 60, 1, 36, "Subsubsection",
 CellTags->"QuarkMass"],
Cell[1666122, 67697, 76, 1, 25, "Text",
 CellTags->"QuarkMass"],
Cell[1666201, 67700, 183, 9, 25, "Text",
 CellTags->"QuarkMass"]
}, Open  ]],
Cell[1666399, 67712, 67, 1, 41, "ObjectName",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1666491, 67717, 66, 1, 36, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[1666560, 67720, 174, 4, 25, "Text",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1666759, 67728, 130, 3, 28, "Input",
 CellTags->"QuarkPropagator"],
Cell[1666892, 67733, 534, 14, 27, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[1667441, 67750, 341, 14, 25, "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1667819, 67769, 63, 1, 36, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1667907, 67774, 124, 3, 28, "Input",
 CellTags->"QuarkPropagator"],
Cell[1668034, 67779, 177, 6, 27, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1668248, 67790, 171, 5, 28, "Input",
 CellTags->"QuarkPropagator"],
Cell[1668422, 67797, 177, 6, 27, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1668636, 67808, 157, 4, 28, "Input",
 CellTags->"QuarkPropagator"],
Cell[1668796, 67814, 177, 6, 27, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1669000, 67824, 59, 1, 41, "ObjectName",
 CellTags->"ReduceGamma"],
Cell[CellGroupData[{
Cell[1669084, 67829, 62, 1, 36, "Subsubsection",
 CellTags->"ReduceGamma"],
Cell[1669149, 67832, 194, 4, 43, "Text",
 CellTags->"ReduceGamma"],
Cell[1669346, 67838, 179, 9, 25, "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],
Cell[1669540, 67850, 67, 1, 41, "ObjectName",
 CellTags->"ReduceToScalars"],
Cell[CellGroupData[{
Cell[1669632, 67855, 66, 1, 36, "Subsubsection",
 CellTags->"ReduceToScalars"],
Cell[1669701, 67858, 210, 5, 43, "Text",
 CellTags->"ReduceToScalars"],
Cell[1669914, 67865, 299, 14, 25, "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],
Cell[1670228, 67882, 49, 1, 41, "ObjectName",
 CellTags->"Rename"],
Cell[CellGroupData[{
Cell[1670302, 67887, 57, 1, 36, "Subsubsection",
 CellTags->"Rename"],
Cell[1670362, 67890, 144, 4, 25, "Text",
 CellTags->"Rename"],
Cell[1670509, 67896, 177, 9, 25, "Text",
 CellTags->"Rename"]
}, Open  ]],
Cell[1670701, 67908, 61, 1, 41, "ObjectName",
 CellTags->"RHI"],
Cell[CellGroupData[{
Cell[1670787, 67913, 54, 1, 36, "Subsubsection",
 CellTags->"RHI"],
Cell[1670844, 67916, 611, 12, 147, "Text",
 CellTags->"RHI"],
Cell[1671458, 67930, 168, 9, 25, "Text",
 CellTags->"RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1671663, 67944, 51, 1, 36, "Subsubsection",
 CellTags->"RHI"],
Cell[1671717, 67947, 70, 2, 28, "Input",
 CellTags->"RHI"]
}, Open  ]],
Cell[1671802, 67952, 67, 1, 41, "ObjectName",
 CellTags->"RHI2FC"],
Cell[CellGroupData[{
Cell[1671894, 67957, 57, 1, 36, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1671954, 67960, 614, 12, 147, "Text",
 CellTags->"RHI2FC"],
Cell[1672571, 67974, 171, 9, 25, "Text",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1672779, 67988, 54, 1, 36, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1672836, 67991, 73, 2, 28, "Input",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[1672924, 67996, 61, 1, 41, "ObjectName",
 CellTags->"RHM"],
Cell[CellGroupData[{
Cell[1673010, 68001, 54, 1, 36, "Subsubsection",
 CellTags->"RHM"],
Cell[1673067, 68004, 77, 1, 25, "Text",
 CellTags->"RHM"],
Cell[1673147, 68007, 159, 9, 25, "Text",
 CellTags->"RHM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1673343, 68021, 51, 1, 36, "Subsubsection",
 CellTags->"RHM"],
Cell[1673397, 68024, 70, 2, 28, "Input",
 CellTags->"RHM"]
}, Open  ]],
Cell[1673482, 68029, 61, 1, 41, "ObjectName",
 CellTags->"RHO"],
Cell[CellGroupData[{
Cell[1673568, 68034, 54, 1, 36, "Subsubsection",
 CellTags->"RHO"],
Cell[1673625, 68037, 267, 5, 43, "Text",
 CellTags->"RHO"],
Cell[1673895, 68044, 159, 9, 25, "Text",
 CellTags->"RHO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1674091, 68058, 51, 1, 36, "Subsubsection",
 CellTags->"RHO"],
Cell[1674145, 68061, 70, 2, 28, "Input",
 CellTags->"RHO"]
}, Open  ]],
Cell[1674230, 68066, 45, 1, 41, "ObjectName",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1674300, 68071, 54, 1, 36, "Subsubsection",
 CellTags->"RHP"],
Cell[1674357, 68074, 272, 5, 43, "Text",
 CellTags->"RHP"],
Cell[1674632, 68081, 159, 9, 25, "Text",
 CellTags->"RHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1674828, 68095, 51, 1, 36, "Subsubsection",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1674904, 68100, 133, 3, 28, "Input"],
Cell[1675040, 68105, 1937, 77, 46, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1677014, 68187, 133, 3, 28, "Input"],
Cell[1677150, 68192, 713, 29, 46, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1677900, 68226, 133, 3, 28, "Input"],
Cell[1678036, 68231, 1415, 59, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1679488, 68295, 133, 3, 28, "Input"],
Cell[1679624, 68300, 507, 22, 43, "Output"]
}, Open  ]],
Cell[1680146, 68325, 70, 2, 28, "Input",
 CellTags->"RHP"]
}, Open  ]],
Cell[1680231, 68330, 61, 1, 41, "ObjectName",
 CellTags->"RTL"],
Cell[CellGroupData[{
Cell[1680317, 68335, 54, 1, 36, "Subsubsection",
 CellTags->"RTL"],
Cell[1680374, 68338, 188, 4, 43, "Text",
 CellTags->"RTL"],
Cell[1680565, 68344, 266, 14, 25, "Text",
 CellTags->"RTL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1680868, 68363, 51, 1, 36, "Subsubsection",
 CellTags->"RTL"],
Cell[1680922, 68366, 70, 2, 28, "Input",
 CellTags->"RTL"]
}, Open  ]],
Cell[1681007, 68371, 63, 1, 41, "ObjectName",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1681095, 68376, 64, 1, 36, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[1681162, 68379, 256, 10, 25, "Text",
 CellTags->"RightPartialD"],
Cell[1681421, 68391, 434, 19, 25, "Text",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1681892, 68415, 61, 1, 36, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1681978, 68420, 124, 3, 28, "Input",
 CellTags->"RightPartialD"],
Cell[1682105, 68425, 252, 9, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1682394, 68439, 255, 7, 28, "Input",
 CellTags->"RightPartialD"],
Cell[1682652, 68448, 308, 11, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1682997, 68464, 121, 3, 28, "Input",
 CellTags->"RightPartialD"],
Cell[1683121, 68469, 356, 14, 27, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1683514, 68488, 115, 3, 28, "Input",
 CellTags->"RightPartialD"],
Cell[1683632, 68493, 292, 7, 42, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1683961, 68505, 162, 5, 28, "Input",
 CellTags->"RightPartialD"],
Cell[1684126, 68512, 178, 4, 42, "Output",
 CellTags->"RightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1684331, 68520, 61, 1, 41, "ObjectName",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1684417, 68525, 63, 1, 36, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[1684483, 68528, 318, 6, 61, "Text",
 CellTags->"RussianTrick"],
Cell[1684804, 68536, 90, 1, 25, "Text",
 CellTags->"RussianTrick"],
Cell[1684897, 68539, 326, 14, 25, "Text",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1685260, 68558, 60, 1, 36, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1685345, 68563, 374, 12, 28, "Input",
 CellTags->"RussianTrick"],
Cell[1685722, 68577, 501, 21, 27, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1686260, 68603, 108, 3, 28, "Input",
 CellTags->"RussianTrick"],
Cell[1686371, 68608, 661, 23, 48, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1687069, 68636, 172, 5, 28, "Input",
 CellTags->"RussianTrick"],
Cell[1687244, 68643, 3509, 122, 140, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1690790, 68770, 112, 3, 28, "Input",
 CellTags->"RussianTrick"],
Cell[1690905, 68775, 2675, 111, 27, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1693617, 68891, 135, 4, 28, "Input",
 CellTags->"RussianTrick"],
Cell[1693755, 68897, 2947, 112, 42, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1696717, 69012, 111, 3, 28, "Input",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1696843, 69018, 71, 1, 41, "ObjectName",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1696939, 69023, 68, 1, 36, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[1697010, 69026, 430, 8, 95, "Text",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1697465, 69038, 134, 3, 28, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1697602, 69043, 271, 8, 27, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1697922, 69057, 65, 1, 36, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1698012, 69062, 274, 8, 28, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1698289, 69072, 479, 19, 27, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1698795, 69095, 63, 1, 41, "ObjectName",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1698883, 69100, 64, 1, 36, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[1698950, 69103, 836, 21, 79, "Text",
 CellTags->"ScalarProduct"],
Cell[1699789, 69126, 880, 38, 25, "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1700706, 69169, 61, 1, 36, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1700792, 69174, 143, 4, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1700938, 69180, 212, 8, 27, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1701187, 69193, 186, 6, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1701376, 69201, 301, 11, 27, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1701714, 69217, 143, 4, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1701860, 69223, 168, 6, 27, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702065, 69234, 120, 3, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1702188, 69239, 168, 6, 27, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702393, 69250, 179, 5, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1702575, 69257, 221, 6, 42, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702833, 69268, 231, 7, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1703067, 69277, 271, 8, 42, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703375, 69290, 248, 8, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1703626, 69300, 131, 4, 36, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703794, 69309, 295, 9, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1704092, 69320, 546, 22, 36, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1704675, 69347, 280, 9, 28, "Input",
 CellTags->"ScalarProduct"],
Cell[1704958, 69358, 546, 22, 36, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1705519, 69383, 99, 2, 28, "Input",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1705633, 69388, 75, 1, 41, "ObjectName",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1705733, 69393, 70, 1, 36, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1705806, 69396, 186, 4, 25, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1706017, 69404, 89, 2, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1706109, 69408, 134, 3, 27, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1706280, 69416, 134, 3, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1706417, 69421, 392, 10, 49, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1706846, 69436, 106, 3, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1706955, 69441, 179, 3, 40, "Print",
 CellTags->{"Info3387712142-8239904", "ScalarProductCancel"}]
}, Open  ]],
Cell[1707149, 69447, 743, 33, 25, "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1707929, 69485, 67, 1, 36, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1707999, 69488, 244, 9, 25, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1708268, 69501, 429, 15, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1708700, 69518, 454, 17, 43, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1709191, 69540, 156, 4, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1709350, 69546, 874, 33, 52, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1710261, 69584, 162, 4, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1710426, 69590, 708, 26, 52, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1711171, 69621, 185, 5, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1711359, 69628, 708, 26, 52, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1712104, 69659, 578, 18, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1712685, 69679, 898, 33, 45, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1713620, 69717, 197, 5, 28, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1713820, 69724, 3108, 111, 109, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1716943, 69838, 162, 5, 28, "Input",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1717120, 69846, 51, 1, 41, "ObjectName",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1717196, 69851, 58, 1, 36, "Subsubsection",
 CellTags->"ScaleMu"],
Cell[1717257, 69854, 128, 4, 25, "Text",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1717410, 69862, 81, 2, 28, "Input",
 CellTags->"ScaleMu"],
Cell[1717494, 69866, 108, 3, 27, "Output",
 CellTags->"ScaleMu"]
}, Open  ]]
}, Open  ]],
Cell[1717629, 69873, 53, 1, 41, "ObjectName",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1717707, 69878, 59, 1, 36, "Subsubsection",
 CellTags->"Schouten"],
Cell[1717769, 69881, 460, 8, 87, "Text",
 CellTags->"Schouten"],
Cell[1718232, 69891, 289, 13, 25, "Text",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1718558, 69909, 56, 1, 36, "Subsubsection",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1718639, 69914, 1938, 54, 167, "Input",
 CellTags->"Schouten"],
Cell[1720580, 69970, 15656, 627, 174, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1736273, 70602, 106, 3, 28, "Input",
 CellTags->"Schouten"],
Cell[1736382, 70607, 105, 3, 27, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[1736502, 70613, 84, 2, 28, "Input"]
}, Open  ]],
Cell[1736601, 70618, 41, 1, 41, "ObjectName",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1736667, 70623, 53, 1, 36, "Subsubsection",
 CellTags->"SD"],
Cell[1736723, 70626, 265, 7, 43, "Text",
 CellTags->"SD"],
Cell[1736991, 70635, 173, 9, 25, "Text",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737201, 70649, 50, 1, 36, "Subsubsection",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1737276, 70654, 121, 4, 28, "Input",
 CellTags->"SD"],
Cell[1737400, 70660, 147, 5, 27, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737584, 70670, 130, 4, 28, "Input",
 CellTags->"SD"],
Cell[1737717, 70676, 214, 6, 42, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737968, 70687, 130, 4, 28, "Input",
 CellTags->"SD"],
Cell[1738101, 70693, 136, 4, 42, "Output",
 CellTags->"SD"]
}, Open  ]]
}, Open  ]],
Cell[1738264, 70701, 57, 1, 41, "ObjectName",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1738346, 70706, 61, 1, 36, "Subsubsection",
 CellTags->"SelectFree"],
Cell[1738410, 70709, 255, 5, 43, "Text",
 CellTags->"SelectFree"],
Cell[1738668, 70716, 77, 1, 25, "Text",
 CellTags->"SelectFree"],
Cell[1738748, 70719, 300, 14, 25, "Text",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739085, 70738, 58, 1, 36, "Subsubsection",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1739168, 70743, 211, 6, 28, "Input",
 CellTags->"SelectFree"],
Cell[1739382, 70751, 130, 4, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739549, 70760, 166, 5, 28, "Input",
 CellTags->"SelectFree"],
Cell[1739718, 70767, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739862, 70775, 275, 8, 28, "Input",
 CellTags->"SelectFree"],
Cell[1740140, 70785, 130, 4, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1740307, 70794, 137, 4, 28, "Input",
 CellTags->"SelectFree"],
Cell[1740447, 70800, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1740591, 70808, 137, 4, 28, "Input",
 CellTags->"SelectFree"],
Cell[1740731, 70814, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1740875, 70822, 137, 4, 28, "Input",
 CellTags->"SelectFree"],
Cell[1741015, 70828, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1741159, 70836, 166, 5, 28, "Input",
 CellTags->"SelectFree"],
Cell[1741328, 70843, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]]
}, Open  ]],
Cell[1741462, 70850, 61, 1, 41, "ObjectName",
 CellTags->"SelectGraphs"],
Cell[CellGroupData[{
Cell[1741548, 70855, 63, 1, 36, "Subsubsection",
 CellTags->"SelectGraphs"],
Cell[1741614, 70858, 486, 8, 79, "Text",
 CellTags->"SelectGraphs"],
Cell[1742103, 70868, 189, 9, 25, "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],
Cell[1742307, 70880, 63, 1, 41, "ObjectName",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1742395, 70885, 64, 1, 36, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[1742462, 70888, 143, 4, 25, "Text",
 CellTags->"SelectNotFree"],
Cell[1742608, 70894, 270, 5, 43, "Text",
 CellTags->"SelectNotFree"],
Cell[1742881, 70901, 83, 1, 25, "Text",
 CellTags->"SelectNotFree"],
Cell[1742967, 70904, 294, 14, 25, "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743298, 70923, 61, 1, 36, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1743384, 70928, 207, 6, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1743594, 70936, 162, 5, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743793, 70946, 276, 8, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1744072, 70956, 172, 5, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1744281, 70966, 143, 4, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1744427, 70972, 110, 3, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1744574, 70980, 167, 5, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1744744, 70987, 110, 3, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1744891, 70995, 143, 4, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1745037, 71001, 110, 3, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1745184, 71009, 143, 4, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1745330, 71015, 110, 3, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1745477, 71023, 172, 5, 28, "Input",
 CellTags->"SelectNotFree"],
Cell[1745652, 71030, 138, 4, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]]
}, Open  ]],
Cell[1745817, 71038, 59, 1, 41, "ObjectName",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1745901, 71043, 62, 1, 36, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[1745966, 71046, 249, 5, 43, "Text",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1746252, 71056, 59, 1, 36, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1746336, 71061, 804, 26, 50, "Input",
 CellTags->"SelectSplit"],
Cell[1747143, 71089, 532, 19, 27, "Output",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1747712, 71113, 652, 21, 28, "Input",
 CellTags->"SelectSplit"],
Cell[1748367, 71136, 495, 16, 27, "Output",
 CellTags->"SelectSplit"]
}, Open  ]]
}, Open  ]],
Cell[1748889, 71156, 51, 1, 41, "ObjectName",
 CellTags->"Select1"],
Cell[CellGroupData[{
Cell[1748965, 71161, 58, 1, 36, "Subsubsection",
 CellTags->"Select1"],
Cell[1749026, 71164, 243, 5, 43, "Text",
 CellTags->"Select1"],
Cell[1749272, 71171, 175, 9, 25, "Text",
 CellTags->"Select1"]
}, Open  ]],
Cell[1749462, 71183, 51, 1, 41, "ObjectName",
 CellTags->"Select2"],
Cell[CellGroupData[{
Cell[1749538, 71188, 58, 1, 36, "Subsubsection",
 CellTags->"Select2"],
Cell[1749599, 71191, 245, 5, 43, "Text",
 CellTags->"Select2"],
Cell[1749847, 71198, 175, 9, 25, "Text",
 CellTags->"Select2"]
}, Open  ]],
Cell[1750037, 71210, 51, 1, 41, "ObjectName",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1750113, 71215, 58, 1, 36, "Subsubsection",
 CellTags->"Series2"],
Cell[1750174, 71218, 383, 9, 61, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1750582, 71231, 114, 3, 28, "Input",
 CellTags->"Series2"],
Cell[1750699, 71236, 528, 15, 27, "Output",
 CellTags->"Series2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1751276, 71257, 55, 1, 36, "Subsubsection",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1751356, 71262, 339, 12, 28, "Input",
 CellTags->"Series2"],
Cell[1751698, 71276, 357, 12, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1752092, 71293, 174, 5, 28, "Input",
 CellTags->"Series2"],
Cell[1752269, 71300, 242, 8, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1752548, 71313, 222, 7, 28, "Input",
 CellTags->"Series2"],
Cell[1752773, 71322, 806, 25, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1753616, 71352, 174, 5, 28, "Input",
 CellTags->"Series2"],
Cell[1753793, 71359, 607, 23, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1754437, 71387, 290, 8, 28, "Input",
 CellTags->"Series2"],
Cell[1754730, 71397, 1223, 41, 40, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1755968, 71441, 187, 6, 25, "Text"],
Cell[CellGroupData[{
Cell[1756180, 71451, 460, 15, 28, "Input",
 CellTags->"Series2"],
Cell[1756643, 71468, 1259, 42, 40, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1757917, 71513, 113, 3, 25, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1758055, 71520, 484, 15, 28, "Input",
 CellTags->"Series2"],
Cell[1758542, 71537, 790, 29, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1759369, 71571, 578, 17, 50, "Input",
 CellTags->"Series2"],
Cell[1759950, 71590, 1149, 41, 38, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1761136, 71636, 242, 6, 28, "Input",
 CellTags->"Series2"],
Cell[1761381, 71644, 1429, 52, 27, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1762847, 71701, 147, 4, 28, "Input",
 CellTags->"Series2"],
Cell[1762997, 71707, 3671, 130, 94, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1766683, 71840, 351, 11, 25, "Text",
 CellTags->"Series2"]
}, Open  ]],
Cell[1767049, 71854, 51, 1, 41, "ObjectName",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1767125, 71859, 58, 1, 36, "Subsubsection",
 CellTags->"Series3"],
Cell[1767186, 71862, 267, 5, 43, "Text",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1767478, 71871, 114, 3, 28, "Input",
 CellTags->"Series3"],
Cell[1767595, 71876, 262, 8, 27, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[1767872, 71887, 175, 9, 25, "Text",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1768084, 71901, 55, 1, 36, "Subsubsection",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1768164, 71906, 339, 12, 28, "Input",
 CellTags->"Series3"],
Cell[1768506, 71920, 307, 11, 38, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1768850, 71936, 214, 7, 28, "Input",
 CellTags->"Series3"],
Cell[1769067, 71945, 236, 8, 38, "Output",
 CellTags->"Series3"]
}, Open  ]]
}, Open  ]],
Cell[1769330, 71957, 63, 1, 41, "ObjectName",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1769418, 71962, 64, 1, 36, "Subsubsection",
 CellTags->"SetMandelstam"],
Cell[1769485, 71965, 462, 9, 87, "Text",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1769972, 71978, 126, 3, 28, "Input",
 CellTags->"SetMandelstam"],
Cell[1770101, 71983, 237, 8, 27, "Output",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1770353, 71994, 190, 9, 25, "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1770558, 72006, 79, 1, 41, "ObjectName",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1770662, 72011, 72, 1, 36, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[1770737, 72014, 299, 6, 43, "Text",
 CellTags->"SimplifyDeltaFunction"],
Cell[1771039, 72022, 347, 14, 25, "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1771423, 72041, 69, 1, 36, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1771517, 72046, 203, 6, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1771723, 72054, 268, 9, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1772028, 72068, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1772195, 72074, 392, 13, 34, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1772624, 72092, 203, 6, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1772830, 72100, 301, 10, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1773168, 72115, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1773335, 72121, 815, 28, 34, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1774187, 72154, 220, 6, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1774410, 72162, 313, 10, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1774760, 72177, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1774927, 72183, 212, 7, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1775176, 72195, 263, 8, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1775442, 72205, 439, 16, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1775918, 72226, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1776085, 72232, 234, 8, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1776356, 72245, 298, 9, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1776657, 72256, 480, 17, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1777174, 72278, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1777341, 72284, 118, 3, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1777496, 72292, 263, 8, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1777762, 72302, 439, 16, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1778238, 72323, 164, 4, 28, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1778405, 72329, 234, 8, 27, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[1778666, 72341, 77, 1, 41, "ObjectName",
 CellTags->"SimplifyGTI"],
Cell[CellGroupData[{
Cell[1778768, 72346, 62, 1, 36, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1778833, 72349, 71, 1, 25, "Text",
 CellTags->"SimplifyGTI"],
Cell[1778907, 72352, 167, 9, 25, "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1779111, 72366, 59, 1, 36, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1779173, 72369, 78, 2, 28, "Input",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[1779266, 72374, 67, 1, 41, "ObjectName",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1779358, 72379, 66, 1, 36, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1779427, 72382, 199, 4, 25, "Text",
 CellTags->"SimplifyPolyLog"],
Cell[1779629, 72388, 183, 9, 25, "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1779849, 72402, 63, 1, 36, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1779915, 72405, 224, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1780164, 72415, 195, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1780362, 72423, 909, 31, 40, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1781308, 72459, 170, 5, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1781481, 72466, 650, 23, 27, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782168, 72494, 221, 7, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1782392, 72503, 976, 35, 27, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1783405, 72543, 195, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1783603, 72551, 955, 34, 27, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1784595, 72590, 271, 9, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1784869, 72601, 890, 32, 38, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1785796, 72638, 248, 8, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1786047, 72648, 868, 31, 38, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1786952, 72684, 281, 9, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1787236, 72695, 732, 29, 38, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1788005, 72729, 216, 7, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1788224, 72738, 714, 26, 43, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1788975, 72769, 195, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1789173, 72777, 115, 3, 27, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1789325, 72785, 195, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1789523, 72793, 1658, 61, 27, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1791218, 72859, 271, 9, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1791492, 72870, 1796, 64, 70, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1793325, 72939, 221, 7, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1793549, 72948, 1818, 69, 43, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1795404, 73022, 271, 9, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1795678, 73033, 3421, 125, 95, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799136, 73163, 191, 6, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1799330, 73171, 333, 11, 41, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799700, 73187, 166, 5, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1799869, 73194, 251, 8, 40, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1800157, 73207, 146, 4, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1800306, 73213, 406, 14, 40, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1800749, 73232, 146, 4, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1800898, 73238, 355, 13, 40, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1801290, 73256, 146, 4, 28, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1801439, 73262, 355, 13, 40, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1801809, 73278, 116, 3, 28, "Input",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1801940, 73284, 73, 1, 41, "ObjectName",
 CellTags->"Simplify2"],
Cell[CellGroupData[{
Cell[1802038, 73289, 60, 1, 36, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1802101, 73292, 81, 1, 25, "Text",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802219, 73298, 57, 1, 36, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1802279, 73301, 76, 2, 28, "Input",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[1802370, 73306, 57, 1, 41, "ObjectName",
 CellTags->"SmallDelta"],
Cell[CellGroupData[{
Cell[1802452, 73311, 61, 1, 36, "Subsubsection",
 CellTags->"SmallDelta"],
Cell[1802516, 73314, 87, 1, 25, "Text",
 CellTags->"SmallDelta"]
}, Open  ]],
Cell[1802618, 73318, 61, 1, 41, "ObjectName",
 CellTags->"SmallEpsilon"],
Cell[CellGroupData[{
Cell[1802704, 73323, 63, 1, 36, "Subsubsection",
 CellTags->"SmallEpsilon"],
Cell[1802770, 73326, 91, 1, 25, "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],
Cell[1802876, 73330, 63, 1, 41, "ObjectName",
 CellTags->"SmallVariable"],
Cell[CellGroupData[{
Cell[1802964, 73335, 64, 1, 36, "Subsubsection",
 CellTags->"SmallVariable"],
Cell[1803031, 73338, 281, 5, 43, "Text",
 CellTags->"SmallVariable"],
Cell[1803315, 73345, 437, 19, 25, "Text",
 CellTags->"SmallVariable"]
}, Open  ]],
Cell[1803767, 73367, 65, 1, 41, "ObjectName",
 CellTags->"SmallVariables"],
Cell[CellGroupData[{
Cell[1803857, 73372, 65, 1, 36, "Subsubsection",
 CellTags->"SmallVariables"],
Cell[1803925, 73375, 225, 5, 43, "Text",
 CellTags->"SmallVariables"],
Cell[1804153, 73382, 182, 9, 25, "Text",
 CellTags->"SmallVariables"]
}, Open  ]],
Cell[1804350, 73394, 43, 1, 41, "ObjectName",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1804418, 73399, 54, 1, 36, "Subsubsection",
 CellTags->"SMP"],
Cell[1804475, 73402, 232, 5, 43, "Text",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1804732, 73411, 127, 4, 28, "Input",
 CellTags->"SMP"],
Cell[1804862, 73417, 166, 5, 42, "Output",
 CellTags->"SMP"]
}, Open  ]],
Cell[1805043, 73425, 168, 8, 25, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1805248, 73438, 51, 1, 36, "Subsubsection",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1805324, 73443, 109, 3, 28, "Input",
 CellTags->"SMP"],
Cell[1805436, 73448, 101, 3, 27, "Output",
 CellTags->"SMP"]
}, Open  ]]
}, Open  ]],
Cell[1805564, 73455, 53, 1, 41, "ObjectName",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1805642, 73460, 59, 1, 36, "Subsubsection",
 CellTags->"SMVertex"],
Cell[1805704, 73463, 304, 6, 43, "Text",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1806033, 73473, 112, 3, 28, "Input",
 CellTags->"SMVertex"],
Cell[1806148, 73478, 235, 8, 27, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1806432, 73492, 56, 1, 36, "Subsubsection",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1806513, 73497, 223, 6, 28, "Input",
 CellTags->"SMVertex"],
Cell[1806739, 73505, 1146, 43, 27, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[1807912, 73552, 61, 1, 41, "ObjectName",
 CellTags->"Smu"],
Cell[CellGroupData[{
Cell[1807998, 73557, 54, 1, 36, "Subsubsection",
 CellTags->"Smu"],
Cell[1808055, 73560, 45, 1, 25, "Text",
 CellTags->"Smu"]
}, Open  ]],
Cell[1808115, 73564, 41, 1, 41, "ObjectName",
 CellTags->"Sn"],
Cell[CellGroupData[{
Cell[1808181, 73569, 53, 1, 36, "Subsubsection",
 CellTags->"Sn"],
Cell[1808237, 73572, 300, 14, 25, "Text",
 CellTags->"Sn"]
}, Open  ]],
Cell[1808552, 73589, 41, 1, 41, "ObjectName",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1808618, 73594, 53, 1, 36, "Subsubsection",
 CellTags->"SO"],
Cell[1808674, 73597, 173, 4, 43, "Text",
 CellTags->"SO"],
Cell[1808850, 73603, 580, 28, 25, "Text",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1809467, 73636, 50, 1, 36, "Subsubsection",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1809542, 73641, 98, 3, 28, "Input",
 CellTags->"SO"],
Cell[1809643, 73646, 265, 10, 27, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1809945, 73661, 121, 4, 28, "Input",
 CellTags->"SO"],
Cell[1810069, 73667, 384, 15, 27, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1810490, 73687, 163, 6, 28, "Input",
 CellTags->"SO"],
Cell[1810656, 73695, 217, 6, 42, "Output",
 CellTags->"SO"]
}, Open  ]]
}, Open  ]],
Cell[1810900, 73705, 43, 1, 70, "ObjectName",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1810968, 73710, 54, 1, 70, "Subsubsection",
 CellTags->"SOD"],
Cell[1811025, 73713, 191, 5, 70, "Text",
 CellTags->"SOD"],
Cell[1811219, 73720, 492, 24, 70, "Text",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1811748, 73749, 51, 1, 36, "Subsubsection",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1811824, 73754, 100, 3, 28, "Input",
 CellTags->"SOD"],
Cell[1811927, 73759, 266, 10, 27, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1812230, 73774, 123, 4, 28, "Input",
 CellTags->"SOD"],
Cell[1812356, 73780, 385, 15, 27, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1812778, 73800, 165, 6, 28, "Input",
 CellTags->"SOD"],
Cell[1812946, 73808, 268, 8, 42, "Output",
 CellTags->"SOD"]
}, Open  ]]
}, Open  ]],
Cell[1813241, 73820, 49, 1, 70, "ObjectName",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1813315, 73825, 57, 1, 70, "Subsubsection",
 CellTags->"Solve2"],
Cell[1813375, 73828, 346, 7, 61, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1813746, 73839, 112, 3, 28, "Input",
 CellTags->"Solve2"],
Cell[1813861, 73844, 264, 8, 27, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1814174, 73858, 54, 1, 36, "Subsubsection",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1814253, 73863, 390, 14, 28, "Input",
 CellTags->"Solve2"],
Cell[1814646, 73879, 373, 13, 40, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[1815034, 73895, 111, 3, 25, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1815170, 73902, 153, 5, 28, "Input",
 CellTags->"Solve2"],
Cell[1815326, 73909, 176, 6, 27, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1815539, 73920, 267, 8, 28, "Input",
 CellTags->"Solve2"],
Cell[1815809, 73930, 176, 6, 27, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1816022, 73941, 443, 15, 28, "Input",
 CellTags->"Solve2"],
Cell[1816468, 73958, 314, 11, 41, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1816819, 73974, 389, 14, 28, "Input",
 CellTags->"Solve2"],
Cell[1817211, 73990, 406, 14, 40, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[1817644, 74008, 49, 1, 70, "ObjectName",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1817718, 74013, 57, 1, 70, "Subsubsection",
 CellTags->"Solve3"],
Cell[1817778, 74016, 271, 5, 43, "Text",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1818074, 74025, 112, 3, 28, "Input",
 CellTags->"Solve3"],
Cell[1818189, 74030, 262, 8, 27, "Output",
 CellTags->"Solve3"]
}, Open  ]],
Cell[1818466, 74041, 162, 8, 70, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1818665, 74054, 54, 1, 36, "Subsubsection",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1818744, 74059, 390, 14, 28, "Input",
 CellTags->"Solve3"],
Cell[1819137, 74075, 373, 13, 40, "Output",
 CellTags->"Solve3"]
}, Open  ]]
}, Open  ]],
Cell[1819537, 74092, 41, 1, 70, "ObjectName",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1819603, 74097, 53, 1, 70, "Subsubsection",
 CellTags->"SP"],
Cell[1819659, 74100, 327, 11, 70, "Text",
 CellTags->"SP"],
Cell[1819989, 74113, 429, 19, 70, "Text",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1820455, 74137, 50, 1, 36, "Subsubsection",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1820530, 74142, 184, 6, 28, "Input",
 CellTags->"SP"],
Cell[1820717, 74150, 366, 16, 27, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1821120, 74171, 189, 7, 28, "Input",
 CellTags->"SP"],
Cell[1821312, 74180, 297, 11, 27, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1821646, 74196, 234, 8, 28, "Input",
 CellTags->"SP"],
Cell[1821883, 74206, 388, 16, 27, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1822308, 74227, 168, 5, 28, "Input",
 CellTags->"SP"],
Cell[1822479, 74234, 209, 8, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1822725, 74247, 157, 5, 28, "Input",
 CellTags->"SP"],
Cell[1822885, 74254, 136, 4, 42, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1823058, 74263, 188, 7, 28, "Input",
 CellTags->"SP"],
Cell[1823249, 74272, 210, 6, 42, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1823496, 74283, 217, 8, 28, "Input",
 CellTags->"SP"],
Cell[1823716, 74293, 136, 4, 42, "Output",
 CellTags->"SP"]
}, Open  ]]
}, Open  ]],
Cell[1823879, 74301, 43, 1, 41, "ObjectName",
 CellTags->"SPC"],
Cell[CellGroupData[{
Cell[1823947, 74306, 54, 1, 36, "Subsubsection",
 CellTags->"SPC"],
Cell[1824004, 74309, 81, 1, 25, "Text",
 CellTags->"SPC"],
Cell[1824088, 74312, 207, 9, 25, "Text",
 CellTags->"SPC"]
}, Open  ]],
Cell[1824310, 74324, 43, 1, 41, "ObjectName",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1824378, 74329, 54, 1, 36, "Subsubsection",
 CellTags->"SPD"],
Cell[1824435, 74332, 346, 11, 43, "Text",
 CellTags->"SPD"],
Cell[1824784, 74345, 522, 24, 25, "Text",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1825343, 74374, 51, 1, 36, "Subsubsection",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1825419, 74379, 187, 6, 28, "Input",
 CellTags->"SPD"],
Cell[1825609, 74387, 367, 16, 27, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1826013, 74408, 191, 7, 28, "Input",
 CellTags->"SPD"],
Cell[1826207, 74417, 298, 11, 27, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1826542, 74433, 236, 8, 28, "Input",
 CellTags->"SPD"],
Cell[1826781, 74443, 389, 16, 27, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1827207, 74464, 170, 5, 28, "Input",
 CellTags->"SPD"],
Cell[1827380, 74471, 210, 8, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1827627, 74484, 159, 5, 28, "Input",
 CellTags->"SPD"],
Cell[1827789, 74491, 138, 4, 42, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1827964, 74500, 190, 7, 28, "Input",
 CellTags->"SPD"],
Cell[1828157, 74509, 261, 8, 42, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1828455, 74522, 219, 8, 28, "Input",
 CellTags->"SPD"],
Cell[1828677, 74532, 138, 4, 42, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1828852, 74541, 270, 9, 28, "Input",
 CellTags->"SPD"],
Cell[1829125, 74552, 138, 4, 42, "Output",
 CellTags->"SPD"]
}, Open  ]]
}, Open  ]],
Cell[1829290, 74560, 49, 1, 41, "ObjectName",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1829364, 74565, 57, 1, 36, "Subsubsection",
 CellTags->"Spinor"],
Cell[1829424, 74568, 1326, 49, 131, "Text",
 CellTags->"Spinor"],
Cell[1830753, 74619, 766, 34, 25, "Text",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1831556, 74658, 54, 1, 36, "Subsubsection",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1831635, 74663, 106, 3, 28, "Input",
 CellTags->"Spinor"],
Cell[1831744, 74668, 175, 6, 27, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1831956, 74679, 129, 4, 28, "Input",
 CellTags->"Spinor"],
Cell[1832088, 74685, 185, 6, 27, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1832288, 74694, 146, 4, 25, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1832459, 74702, 250, 8, 28, "Input",
 CellTags->"Spinor"],
Cell[1832712, 74712, 126, 4, 27, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1832875, 74721, 254, 8, 28, "Input",
 CellTags->"Spinor"],
Cell[1833132, 74731, 251, 9, 27, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1833420, 74745, 141, 4, 28, "Input",
 CellTags->"Spinor"],
Cell[1833564, 74751, 192, 6, 42, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1833771, 74760, 158, 4, 25, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1833954, 74768, 212, 7, 28, "Input",
 CellTags->"Spinor"],
Cell[1834169, 74777, 192, 6, 42, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1834398, 74788, 165, 5, 28, "Input",
 CellTags->"Spinor"],
Cell[1834566, 74795, 192, 6, 42, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1834773, 74804, 77, 1, 25, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1834875, 74809, 210, 7, 28, "Input",
 CellTags->"Spinor"],
Cell[1835088, 74818, 192, 6, 42, "Output",
 CellTags->"Spinor"]
}, Open  ]]
}, Open  ]],
Cell[1835307, 74828, 63, 1, 41, "ObjectName",
 CellTags->"SpinorCollect"],
Cell[CellGroupData[{
Cell[1835395, 74833, 64, 1, 36, "Subsubsection",
 CellTags->"SpinorCollect"],
Cell[1835462, 74836, 191, 4, 43, "Text",
 CellTags->"SpinorCollect"],
Cell[1835656, 74842, 301, 13, 25, "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],
Cell[1835972, 74858, 57, 1, 41, "ObjectName",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1836054, 74863, 61, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1836118, 74866, 170, 7, 25, "Text",
 CellTags->"SpinorUBar"],
Cell[1836291, 74875, 505, 24, 25, "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1836833, 74904, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1836894, 74907, 85, 1, 25, "Text",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1837004, 74912, 114, 3, 28, "Input",
 CellTags->"SpinorUBar"],
Cell[1837121, 74917, 202, 7, 27, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1837360, 74929, 137, 4, 28, "Input",
 CellTags->"SpinorUBar"],
Cell[1837500, 74935, 212, 7, 27, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1837749, 74947, 173, 5, 28, "Input",
 CellTags->"SpinorUBar"],
Cell[1837925, 74954, 152, 4, 42, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1838114, 74963, 204, 7, 28, "Input",
 CellTags->"SpinorUBar"],
Cell[1838321, 74972, 196, 6, 42, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1838554, 74983, 233, 8, 28, "Input",
 CellTags->"SpinorUBar"],
Cell[1838790, 74993, 196, 6, 42, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]]
}, Open  ]],
Cell[1839013, 75003, 51, 1, 41, "ObjectName",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1839089, 75008, 58, 1, 70, "Subsubsection",
 CellTags->"SpinorU"],
Cell[1839150, 75011, 79, 1, 70, "Text",
 CellTags->"SpinorU"],
Cell[1839232, 75014, 511, 24, 70, "Text",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1839780, 75043, 55, 1, 36, "Subsubsection",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1839860, 75048, 108, 3, 28, "Input",
 CellTags->"SpinorU"],
Cell[1839971, 75053, 132, 4, 27, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1840140, 75062, 131, 4, 28, "Input",
 CellTags->"SpinorU"],
Cell[1840274, 75068, 142, 4, 27, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1840453, 75077, 167, 5, 28, "Input",
 CellTags->"SpinorU"],
Cell[1840623, 75084, 146, 4, 42, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1840806, 75093, 198, 7, 28, "Input",
 CellTags->"SpinorU"],
Cell[1841007, 75102, 193, 6, 42, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1841237, 75113, 227, 8, 28, "Input",
 CellTags->"SpinorU"],
Cell[1841467, 75123, 193, 6, 42, "Output",
 CellTags->"SpinorU"]
}, Open  ]]
}, Open  ]],
Cell[1841687, 75133, 57, 1, 70, "ObjectName",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1841769, 75138, 61, 1, 70, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[1841833, 75141, 175, 7, 70, "Text",
 CellTags->"SpinorVBar"],
Cell[1842011, 75150, 505, 24, 70, "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1842553, 75179, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1842636, 75184, 114, 3, 28, "Input",
 CellTags->"SpinorVBar"],
Cell[1842753, 75189, 202, 7, 27, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1842992, 75201, 137, 4, 28, "Input",
 CellTags->"SpinorVBar"],
Cell[1843132, 75207, 212, 7, 27, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1843381, 75219, 173, 5, 28, "Input",
 CellTags->"SpinorVBar"],
Cell[1843557, 75226, 152, 4, 42, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1843746, 75235, 204, 7, 28, "Input",
 CellTags->"SpinorVBar"],
Cell[1843953, 75244, 217, 7, 42, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1844207, 75256, 233, 8, 28, "Input",
 CellTags->"SpinorVBar"],
Cell[1844443, 75266, 217, 7, 42, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]]
}, Open  ]],
Cell[1844687, 75277, 51, 1, 70, "ObjectName",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1844763, 75282, 58, 1, 70, "Subsubsection",
 CellTags->"SpinorV"],
Cell[1844824, 75285, 79, 1, 70, "Text",
 CellTags->"SpinorV"],
Cell[1844906, 75288, 511, 24, 70, "Text",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1845454, 75317, 55, 1, 36, "Subsubsection",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1845534, 75322, 108, 3, 28, "Input",
 CellTags->"SpinorV"],
Cell[1845645, 75327, 132, 4, 27, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1845814, 75336, 131, 4, 28, "Input",
 CellTags->"SpinorV"],
Cell[1845948, 75342, 142, 4, 27, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1846127, 75351, 167, 5, 28, "Input",
 CellTags->"SpinorV"],
Cell[1846297, 75358, 146, 4, 42, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1846480, 75367, 198, 7, 28, "Input",
 CellTags->"SpinorV"],
Cell[1846681, 75376, 214, 7, 42, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1846932, 75388, 227, 8, 28, "Input",
 CellTags->"SpinorV"],
Cell[1847162, 75398, 214, 7, 42, "Output",
 CellTags->"SpinorV"]
}, Open  ]]
}, Open  ]],
Cell[1847403, 75409, 75, 1, 41, "ObjectName",
 CellTags->"SpinPolarizationSum"],
Cell[CellGroupData[{
Cell[1847503, 75414, 70, 1, 36, "Subsubsection",
 CellTags->"SpinPolarizationSum"],
Cell[1847576, 75417, 164, 4, 25, "Text",
 CellTags->"SpinPolarizationSum"],
Cell[1847743, 75423, 203, 8, 25, "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],
Cell[1847961, 75434, 43, 1, 41, "ObjectName",
 CellTags->"SPL"],
Cell[CellGroupData[{
Cell[1848029, 75439, 54, 1, 36, "Subsubsection",
 CellTags->"SPL"],
Cell[1848086, 75442, 77, 1, 25, "Text",
 CellTags->"SPL"],
Cell[1848166, 75445, 195, 9, 25, "Text",
 CellTags->"SPL"]
}, Open  ]],
Cell[1848376, 75457, 71, 1, 41, "ObjectName",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1848472, 75462, 68, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1848543, 75465, 218, 7, 25, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1848786, 75476, 134, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1848923, 75481, 181, 6, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1849119, 75490, 218, 9, 25, "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1849374, 75504, 65, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1849442, 75507, 65, 1, 25, "Text",
 CellTags->"SplittingFunction"],
Cell[1849510, 75510, 221, 6, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1849734, 75518, 170, 4, 25, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1849929, 75526, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1850062, 75531, 553, 20, 41, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1850652, 75556, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1850785, 75561, 304, 11, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1851126, 75577, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1851259, 75582, 269, 9, 40, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1851565, 75596, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1851698, 75601, 859, 29, 41, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1852594, 75635, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1852727, 75640, 1128, 38, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1853892, 75683, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1854025, 75688, 594, 20, 40, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1854656, 75713, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1854789, 75718, 540, 19, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1855344, 75740, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1855477, 75745, 63, 1, 25, "Text",
 CellTags->"SplittingFunction"],
Cell[1855543, 75748, 221, 6, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1855789, 75758, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1855922, 75763, 553, 20, 41, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856512, 75788, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1856645, 75793, 239, 9, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856921, 75807, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1857054, 75812, 235, 8, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1857326, 75825, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1857459, 75830, 834, 29, 41, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1858330, 75864, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1858463, 75869, 1247, 42, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859747, 75916, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1859880, 75921, 503, 18, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860420, 75944, 131, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1860554, 75949, 445, 16, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1861036, 75970, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1861169, 75975, 445, 16, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1861651, 75996, 131, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1861785, 76001, 445, 16, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1862267, 76022, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1862400, 76027, 1362, 46, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1863799, 76078, 131, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1863933, 76083, 1362, 46, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865332, 76134, 131, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1865466, 76139, 660, 23, 27, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1866163, 76167, 132, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1866298, 76172, 5987, 193, 215, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1872322, 76370, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1872455, 76375, 3011, 96, 93, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1875503, 76476, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1875636, 76481, 3710, 121, 136, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1879383, 76607, 130, 3, 28, "Input",
 CellTags->"SplittingFunction"],
Cell[1879516, 76612, 4577, 151, 176, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]]
}, Open  ]],
Cell[1884120, 76767, 79, 1, 41, "ObjectName",
 CellTags->"StandardMatrixElement"],
Cell[CellGroupData[{
Cell[1884224, 76772, 72, 1, 36, "Subsubsection",
 CellTags->"StandardMatrixElement"],
Cell[1884299, 76775, 134, 3, 25, "Text",
 CellTags->"StandardMatrixElement"],
Cell[1884436, 76780, 189, 9, 25, "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],
Cell[1884640, 76792, 57, 1, 41, "ObjectName",
 CellTags->"SubContext"],
Cell[CellGroupData[{
Cell[1884722, 76797, 61, 1, 36, "Subsubsection",
 CellTags->"SubContext"],
Cell[1884786, 76800, 120, 3, 25, "Text",
 CellTags->"SubContext"],
Cell[1884909, 76805, 279, 14, 25, "Text",
 CellTags->"SubContext"]
}, Open  ]],
Cell[1885203, 76822, 51, 1, 41, "ObjectName",
 CellTags->"SubLoop"],
Cell[CellGroupData[{
Cell[1885279, 76827, 58, 1, 36, "Subsubsection",
 CellTags->"SubLoop"],
Cell[1885340, 76830, 152, 4, 25, "Text",
 CellTags->"SubLoop"],
Cell[1885495, 76836, 178, 9, 25, "Text",
 CellTags->"SubLoop"]
}, Open  ]],
Cell[1885688, 76848, 45, 1, 41, "ObjectName",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1885758, 76853, 55, 1, 36, "Subsubsection",
 CellTags->"SumP"],
Cell[1885816, 76856, 767, 31, 28, "Text",
 CellTags->"SumP"],
Cell[1886586, 76889, 261, 14, 25, "Text",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1886884, 76908, 52, 1, 36, "Subsubsection",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1886961, 76913, 149, 5, 28, "Input",
 CellTags->"SumP"],
Cell[1887113, 76920, 263, 8, 27, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1887413, 76933, 149, 5, 28, "Input",
 CellTags->"SumP"],
Cell[1887565, 76940, 263, 8, 27, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1887865, 76953, 125, 4, 28, "Input",
 CellTags->"SumP"],
Cell[1887993, 76959, 238, 7, 27, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1888268, 76971, 125, 4, 28, "Input",
 CellTags->"SumP"],
Cell[1888396, 76977, 124, 4, 38, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1888557, 76986, 301, 12, 51, "Input",
 CellTags->"SumP"],
Cell[1888861, 77000, 124, 4, 38, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1889022, 77009, 186, 6, 28, "Input",
 CellTags->"SumP"],
Cell[1889211, 77017, 764, 29, 40, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1890012, 77051, 124, 4, 28, "Input",
 CellTags->"SumP"],
Cell[1890139, 77057, 124, 4, 38, "Output",
 CellTags->"SumP"]
}, Open  ]]
}, Open  ]],
Cell[1890290, 77065, 45, 1, 41, "ObjectName",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1890360, 77070, 55, 1, 36, "Subsubsection",
 CellTags->"SumS"],
Cell[1890418, 77073, 1438, 57, 43, "Text",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1891881, 77134, 108, 3, 28, "Input",
 CellTags->"SumS"],
Cell[1891992, 77139, 165, 6, 27, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1892172, 77148, 261, 14, 25, "Text",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892470, 77167, 52, 1, 36, "Subsubsection",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1892547, 77172, 149, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1892699, 77179, 211, 8, 27, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892947, 77192, 149, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1893099, 77199, 211, 8, 27, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1893347, 77212, 144, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1893494, 77219, 172, 6, 27, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1893703, 77230, 173, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1893879, 77237, 261, 10, 39, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1894177, 77252, 197, 6, 28, "Input",
 CellTags->"SumS"],
Cell[1894377, 77260, 340, 13, 45, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1894732, 77276, 192, 6, 28, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1894949, 77286, 149, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1895101, 77293, 509, 19, 45, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1895625, 77315, 193, 6, 28, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1895843, 77325, 125, 4, 28, "Input",
 CellTags->"SumS"],
Cell[1895971, 77331, 124, 4, 38, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1896132, 77340, 174, 6, 51, "Input",
 CellTags->"SumS"],
Cell[1896309, 77348, 124, 4, 38, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1896470, 77357, 159, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1896632, 77364, 212, 8, 27, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1896881, 77377, 146, 4, 28, "Input",
 CellTags->"SumS"],
Cell[1897030, 77383, 149, 5, 38, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1897216, 77393, 144, 5, 28, "Input",
 CellTags->"SumS"],
Cell[1897363, 77400, 142, 5, 38, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1897542, 77410, 125, 4, 28, "Input",
 CellTags->"SumS"],
Cell[1897670, 77416, 142, 5, 38, "Output",
 CellTags->"SumS"]
}, Open  ]]
}, Open  ]],
Cell[1897839, 77425, 45, 1, 41, "ObjectName",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1897909, 77430, 55, 1, 36, "Subsubsection",
 CellTags->"SumT"],
Cell[1897967, 77433, 473, 15, 43, "Text",
 CellTags->"SumT"],
Cell[1898443, 77450, 261, 14, 25, "Text",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1898741, 77469, 52, 1, 36, "Subsubsection",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1898818, 77474, 149, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1898970, 77481, 236, 9, 28, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1899243, 77495, 149, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1899395, 77502, 236, 9, 28, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1899668, 77516, 125, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1899796, 77522, 180, 7, 28, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900013, 77534, 173, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1900189, 77541, 361, 14, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900587, 77560, 125, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1900715, 77566, 142, 5, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900894, 77576, 244, 9, 51, "Input",
 CellTags->"SumT"],
Cell[1901141, 77587, 142, 5, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1901320, 77597, 159, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1901482, 77604, 237, 9, 28, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1901756, 77618, 136, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1901895, 77624, 253, 7, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902185, 77636, 125, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1902313, 77642, 142, 5, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902492, 77652, 144, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1902639, 77659, 142, 5, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902818, 77669, 268, 10, 51, "Input",
 CellTags->"SumT"],
Cell[1903089, 77681, 705, 26, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1903831, 77712, 136, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1903970, 77718, 170, 6, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1904177, 77729, 155, 5, 28, "Input",
 CellTags->"SumT"],
Cell[1904335, 77736, 253, 7, 38, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1904625, 77748, 129, 4, 28, "Input",
 CellTags->"SumT"],
Cell[1904757, 77754, 458, 16, 40, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1905252, 77775, 240, 9, 28, "Input",
 CellTags->"SumT"],
Cell[1905495, 77786, 458, 16, 40, "Output",
 CellTags->"SumT"]
}, Open  ]]
}, Open  ]],
Cell[1905980, 77806, 45, 1, 41, "ObjectName",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1906050, 77811, 55, 1, 36, "Subsubsection",
 CellTags->"SUND"],
Cell[1906108, 77814, 224, 10, 25, "Text",
 CellTags->"SUND"],
Cell[1906335, 77826, 392, 19, 25, "Text",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1906764, 77850, 52, 1, 36, "Subsubsection",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1906841, 77855, 135, 4, 28, "Input",
 CellTags->"SUND"],
Cell[1906979, 77861, 147, 5, 27, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1907163, 77871, 211, 6, 28, "Input",
 CellTags->"SUND"],
Cell[1907377, 77879, 1086, 44, 27, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1908500, 77928, 135, 4, 28, "Input",
 CellTags->"SUND"],
Cell[1908638, 77934, 147, 5, 27, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1908822, 77944, 135, 4, 28, "Input",
 CellTags->"SUND"],
Cell[1908960, 77950, 147, 5, 27, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1909144, 77960, 266, 8, 28, "Input",
 CellTags->"SUND"],
Cell[1909413, 77970, 260, 9, 27, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1909710, 77984, 377, 12, 28, "Input",
 CellTags->"SUND"],
Cell[1910090, 77998, 304, 11, 40, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1910431, 78014, 377, 12, 28, "Input",
 CellTags->"SUND"],
Cell[1910811, 78028, 474, 19, 40, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1911322, 78052, 173, 6, 28, "Input",
 CellTags->"SUND"],
Cell[1911498, 78060, 150, 4, 42, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1911685, 78069, 202, 7, 28, "Input",
 CellTags->"SUND"],
Cell[1911890, 78078, 258, 7, 42, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1912185, 78090, 231, 8, 28, "Input",
 CellTags->"SUND"],
Cell[1912419, 78100, 150, 4, 42, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[1912584, 78107, 104, 3, 28, "Input",
 CellTags->"SUND"]
}, Open  ]],
Cell[1912703, 78113, 53, 1, 41, "ObjectName",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1912781, 78118, 59, 1, 36, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[1912843, 78121, 172, 6, 25, "Text",
 CellTags->"SUNDelta"],
Cell[1913018, 78129, 717, 33, 25, "Text",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1913772, 78167, 56, 1, 36, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1913853, 78172, 133, 4, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1913989, 78178, 153, 5, 27, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1914179, 78188, 262, 8, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1914444, 78198, 301, 13, 27, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1914782, 78216, 205, 6, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1914990, 78224, 301, 13, 27, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1915328, 78242, 169, 5, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1915500, 78249, 148, 4, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1915685, 78258, 200, 7, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1915888, 78267, 220, 6, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1916145, 78278, 229, 8, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1916377, 78288, 142, 4, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1916556, 78297, 194, 7, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1916753, 78306, 220, 6, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[1916988, 78315, 256, 5, 43, "Text",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1917269, 78324, 315, 10, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1917587, 78336, 460, 20, 27, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1918084, 78361, 110, 3, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1918197, 78366, 395, 11, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1918629, 78382, 200, 7, 28, "Input",
 CellTags->"SUNDelta"],
Cell[1918832, 78391, 236, 6, 42, "Output",
 CellTags->"SUNDelta"]
}, Open  ]]
}, Open  ]],
Cell[1919095, 78401, 69, 1, 41, "ObjectName",
 CellTags->"SUNDeltaContract"],
Cell[CellGroupData[{
Cell[1919189, 78406, 67, 1, 36, "Subsubsection",
 CellTags->"SUNDeltaContract"],
Cell[1919259, 78409, 397, 9, 61, "Text",
 CellTags->"SUNDeltaContract"],
Cell[1919659, 78420, 297, 14, 25, "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],
Cell[1919971, 78437, 45, 1, 41, "ObjectName",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1920041, 78442, 55, 1, 36, "Subsubsection",
 CellTags->"SUNF"],
Cell[1920099, 78445, 190, 6, 25, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1920314, 78455, 108, 3, 28, "Input",
 CellTags->"SUNF"],
Cell[1920425, 78460, 168, 6, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1920608, 78469, 701, 34, 25, "Text",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1921346, 78508, 52, 1, 36, "Subsubsection",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1921423, 78513, 294, 10, 28, "Input",
 CellTags->"SUNF"],
Cell[1921720, 78525, 272, 10, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1922029, 78540, 103, 3, 28, "Input",
 CellTags->"SUNF"],
Cell[1922135, 78545, 764, 34, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1922936, 78584, 110, 3, 28, "Input",
 CellTags->"SUNF"],
Cell[1923049, 78589, 455, 20, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1923541, 78614, 135, 4, 28, "Input",
 CellTags->"SUNF"],
Cell[1923679, 78620, 147, 5, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1923863, 78630, 163, 5, 28, "Input",
 CellTags->"SUNF"],
Cell[1924029, 78637, 101, 3, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1924145, 78643, 319, 10, 25, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1924489, 78657, 185, 5, 28, "Input",
 CellTags->"SUNF"],
Cell[1924677, 78664, 1147, 44, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1925861, 78713, 266, 8, 28, "Input",
 CellTags->"SUNF"],
Cell[1926130, 78723, 353, 15, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1926520, 78743, 239, 7, 28, "Input",
 CellTags->"SUNF"],
Cell[1926762, 78752, 1166, 45, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1927965, 78802, 173, 6, 28, "Input",
 CellTags->"SUNF"],
Cell[1928141, 78810, 150, 4, 42, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1928328, 78819, 202, 7, 28, "Input",
 CellTags->"SUNF"],
Cell[1928533, 78828, 258, 7, 42, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1928828, 78840, 231, 8, 28, "Input",
 CellTags->"SUNF"],
Cell[1929062, 78850, 150, 4, 42, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1929249, 78859, 135, 4, 28, "Input",
 CellTags->"SUNF"],
Cell[1929387, 78865, 147, 5, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1929571, 78875, 162, 5, 28, "Input",
 CellTags->"SUNF"],
Cell[1929736, 78882, 401, 18, 27, "Output",
 CellTags->"SUNF"]
}, Open  ]]
}, Open  ]],
Cell[1930164, 78904, 57, 1, 41, "ObjectName",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1930246, 78909, 61, 1, 36, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[1930310, 78912, 146, 4, 25, "Text",
 CellTags->"SUNFJacobi"],
Cell[1930459, 78918, 283, 13, 25, "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1930779, 78936, 58, 1, 36, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1930862, 78941, 380, 12, 28, "Input",
 CellTags->"SUNFJacobi"],
Cell[1931245, 78955, 694, 32, 27, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1931976, 78992, 379, 12, 28, "Input",
 CellTags->"SUNFJacobi"],
Cell[1932358, 79006, 1362, 62, 27, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]]
}, Open  ]],
Cell[1933747, 79072, 53, 1, 41, "ObjectName",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1933825, 79077, 59, 1, 36, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[1933887, 79080, 135, 6, 25, "Text",
 CellTags->"SUNIndex"],
Cell[1934025, 79088, 405, 18, 25, "Text",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1934467, 79111, 56, 1, 36, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1934548, 79116, 110, 3, 28, "Input",
 CellTags->"SUNIndex"],
Cell[1934661, 79121, 105, 3, 27, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1934803, 79129, 110, 3, 28, "Input",
 CellTags->"SUNIndex"],
Cell[1934916, 79134, 125, 3, 42, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1935078, 79142, 200, 7, 28, "Input",
 CellTags->"SUNIndex"],
Cell[1935281, 79151, 220, 6, 42, "Output",
 CellTags->"SUNIndex"]
}, Open  ]]
}, Open  ]],
Cell[1935528, 79161, 65, 1, 41, "ObjectName",
 CellTags->"SUNIndexRename"],
Cell[CellGroupData[{
Cell[1935618, 79166, 65, 1, 36, "Subsubsection",
 CellTags->"SUNIndexRename"],
Cell[1935686, 79169, 163, 4, 25, "Text",
 CellTags->"SUNIndexRename"],
Cell[1935852, 79175, 304, 14, 25, "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],
Cell[1936171, 79192, 45, 1, 41, "ObjectName",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1936241, 79197, 55, 1, 36, "Subsubsection",
 CellTags->"SUNN"],
Cell[1936299, 79200, 203, 7, 25, "Text",
 CellTags->"SUNN"],
Cell[1936505, 79209, 579, 29, 25, "Text",
 CellTags->"SUNN"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1937121, 79243, 52, 1, 36, "Subsubsection",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1937198, 79248, 317, 9, 28, "Input",
 CellTags->"SUNN"],
Cell[1937518, 79259, 149, 5, 27, "Output",
 CellTags->"SUNN"]
}, Open  ]]
}, Open  ]],
Cell[1937694, 79268, 57, 1, 41, "ObjectName",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1937776, 79273, 61, 1, 36, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[1937840, 79276, 187, 4, 43, "Text",
 CellTags->"SUNNToCACF"],
Cell[1938030, 79282, 573, 29, 25, "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1938640, 79316, 58, 1, 36, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1938723, 79321, 322, 9, 28, "Input",
 CellTags->"SUNNToCACF"],
Cell[1939048, 79332, 186, 6, 27, "Output",
 CellTags->"SUNNToCACF"]
}, Open  ]]
}, Open  ]],
Cell[1939261, 79342, 59, 1, 41, "ObjectName",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1939345, 79347, 62, 1, 36, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[1939410, 79350, 376, 7, 61, "Text",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1939811, 79361, 122, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1939936, 79366, 512, 13, 49, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1940463, 79382, 299, 13, 25, "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1940799, 79400, 59, 1, 36, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1940883, 79405, 247, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1941133, 79415, 231, 8, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1941401, 79428, 117, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1941521, 79433, 304, 13, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1941862, 79451, 189, 6, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1942054, 79459, 249, 10, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1942340, 79474, 117, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1942460, 79479, 130, 4, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1942627, 79488, 183, 5, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1942813, 79495, 203, 7, 40, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1943053, 79507, 254, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1943310, 79517, 227, 8, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1943574, 79530, 117, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1943694, 79535, 360, 15, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1944091, 79555, 269, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1944363, 79565, 227, 8, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1944627, 79578, 117, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1944747, 79583, 197, 6, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1944981, 79594, 259, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1945243, 79604, 227, 8, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1945507, 79617, 117, 3, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1945627, 79622, 360, 15, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1946024, 79642, 259, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1946286, 79652, 227, 8, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1946550, 79665, 180, 5, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1946733, 79672, 108, 3, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1946878, 79680, 384, 12, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1947265, 79694, 311, 11, 40, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1947613, 79710, 389, 12, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1948005, 79724, 448, 18, 40, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1948490, 79747, 246, 7, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1948739, 79756, 1173, 45, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1949949, 79806, 246, 7, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1950198, 79815, 1132, 44, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1951367, 79864, 273, 8, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1951643, 79874, 251, 8, 38, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1951931, 79887, 439, 14, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1952373, 79903, 379, 14, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1952789, 79922, 181, 5, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1952973, 79929, 705, 32, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1953715, 79966, 238, 6, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1953956, 79974, 1363, 62, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1955356, 80041, 200, 6, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1955559, 80049, 164, 5, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1955760, 80059, 341, 10, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1956104, 80071, 543, 21, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1956684, 80097, 181, 5, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1956868, 80104, 543, 21, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1957448, 80130, 180, 5, 28, "Input",
 CellTags->"SUNSimplify"],
Cell[1957631, 80137, 571, 22, 27, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1958217, 80162, 208, 6, 28, "Input",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1958440, 80171, 45, 1, 41, "ObjectName",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1958510, 80176, 55, 1, 36, "Subsubsection",
 CellTags->"SUNT"],
Cell[1958568, 80179, 237, 10, 25, "Text",
 CellTags->"SUNT"],
Cell[1958808, 80191, 674, 34, 25, "Text",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1959519, 80230, 52, 1, 36, "Subsubsection",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1959596, 80235, 102, 3, 28, "Input",
 CellTags->"SUNT"],
Cell[1959701, 80240, 157, 6, 27, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1959873, 80249, 203, 7, 25, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1960101, 80260, 207, 6, 28, "Input",
 CellTags->"SUNT"],
Cell[1960311, 80268, 311, 13, 27, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1960659, 80286, 145, 4, 28, "Input",
 CellTags->"SUNT"],
Cell[1960807, 80292, 365, 16, 27, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1961209, 80313, 242, 7, 28, "Input",
 CellTags->"SUNT"],
Cell[1961454, 80322, 268, 11, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1961759, 80338, 184, 5, 28, "Input",
 CellTags->"SUNT"],
Cell[1961946, 80345, 133, 4, 27, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1962116, 80354, 174, 5, 28, "Input",
 CellTags->"SUNT"],
Cell[1962293, 80361, 399, 16, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1962729, 80382, 242, 7, 28, "Input",
 CellTags->"SUNT"],
Cell[1962974, 80391, 268, 11, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1963257, 80405, 422, 15, 25, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1963704, 80424, 161, 5, 28, "Input",
 CellTags->"SUNT"],
Cell[1963868, 80431, 328, 14, 40, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1964211, 80448, 197, 7, 25, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1964433, 80459, 197, 6, 28, "Input",
 CellTags->"SUNT"],
Cell[1964633, 80467, 353, 15, 27, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1965023, 80487, 198, 6, 28, "Input",
 CellTags->"SUNT"],
Cell[1965224, 80495, 263, 8, 52, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1965524, 80508, 167, 6, 28, "Input",
 CellTags->"SUNT"],
Cell[1965694, 80516, 152, 4, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1965883, 80525, 195, 7, 28, "Input",
 CellTags->"SUNT"],
Cell[1966081, 80534, 117, 3, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]]
}, Open  ]],
Cell[1966225, 80541, 53, 1, 41, "ObjectName",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1966303, 80546, 59, 1, 36, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[1966365, 80549, 81, 1, 25, "Text",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1966471, 80554, 116, 3, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1966590, 80559, 172, 6, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[1966777, 80568, 280, 14, 25, "Text",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1967094, 80587, 56, 1, 36, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1967175, 80592, 165, 5, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1967343, 80599, 332, 14, 40, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1967712, 80618, 175, 5, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1967890, 80625, 524, 22, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1968451, 80652, 240, 7, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1968694, 80661, 811, 35, 40, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1969542, 80701, 185, 5, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1969730, 80708, 641, 27, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1970408, 80740, 278, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1970689, 80750, 4229, 183, 40, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1974955, 80938, 177, 5, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1975135, 80945, 641, 27, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1975813, 80977, 288, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1976104, 80987, 11403, 439, 173, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1987544, 81431, 177, 5, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1987724, 81438, 758, 32, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1988519, 81475, 270, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1988792, 81485, 105, 3, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1988934, 81493, 270, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1989207, 81503, 105, 3, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1989349, 81511, 270, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1989622, 81521, 105, 3, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1989764, 81529, 270, 8, 28, "Input",
 CellTags->"SUNTrace"],
Cell[1990037, 81539, 576, 23, 27, "Output",
 CellTags->"SUNTrace"]
}, Open  ]]
}, Open  ]],
Cell[1990640, 81566, 61, 1, 41, "ObjectName",
 CellTags->"SymbolicSum2"],
Cell[CellGroupData[{
Cell[1990726, 81571, 63, 1, 36, "Subsubsection",
 CellTags->"SymbolicSum2"],
Cell[1990792, 81574, 284, 6, 43, "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],
Cell[1991091, 81583, 61, 1, 41, "ObjectName",
 CellTags->"SymbolicSum3"],
Cell[CellGroupData[{
Cell[1991177, 81588, 63, 1, 36, "Subsubsection",
 CellTags->"SymbolicSum3"],
Cell[1991243, 81591, 292, 6, 43, "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],
Cell[1991550, 81600, 57, 1, 41, "ObjectName",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1991632, 81605, 61, 1, 36, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[1991696, 81608, 145, 4, 25, "Text",
 CellTags->"Symmetrize"],
Cell[1991844, 81614, 193, 8, 25, "Text",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1992074, 81627, 58, 1, 36, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1992157, 81632, 240, 8, 28, "Input",
 CellTags->"Symmetrize"],
Cell[1992400, 81642, 317, 11, 38, "Output",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1992754, 81658, 260, 8, 28, "Input",
 CellTags->"Symmetrize"],
Cell[1993017, 81668, 652, 20, 38, "Output",
 CellTags->"Symmetrize"]
}, Open  ]]
}, Open  ]],
Cell[1993696, 81692, 61, 1, 41, "ObjectName",
 CellTags->{"TBox", "Tbox"}],
Cell[CellGroupData[{
Cell[1993782, 81697, 65, 1, 36, "Subsubsection",
 CellTags->{"TBox", "Tbox"}],
Cell[1993850, 81700, 262, 5, 43, "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],
Cell[1994127, 81708, 45, 1, 41, "ObjectName",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1994197, 81713, 55, 1, 36, "Subsubsection",
 CellTags->"Tdec"],
Cell[1994255, 81716, 256, 5, 43, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1994536, 81725, 108, 3, 28, "Input",
 CellTags->"Tdec"],
Cell[1994647, 81730, 412, 11, 27, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1995074, 81744, 389, 19, 25, "Text",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1995500, 81768, 52, 1, 36, "Subsubsection",
 CellTags->"Tdec"],
Cell[1995555, 81771, 608, 22, 30, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1996188, 81797, 202, 7, 28, "Input",
 CellTags->"Tdec"],
Cell[1996393, 81806, 556, 21, 41, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1996986, 81832, 199, 7, 28, "Input",
 CellTags->"Tdec"],
Cell[1997188, 81841, 354, 14, 41, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1997557, 81858, 776, 28, 25, "Text",
 CellTags->"Tdec"],
Cell[1998336, 81888, 529, 18, 28, "Input",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1998890, 81910, 199, 7, 28, "Input",
 CellTags->"Tdec"],
Cell[1999092, 81919, 8001, 289, 137, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2007130, 82213, 380, 13, 28, "Input",
 CellTags->"Tdec"],
Cell[2007513, 82228, 4966, 181, 66, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[2012494, 82412, 121, 4, 28, "Input",
 CellTags->"Tdec"]
}, Open  ]],
Cell[2012630, 82419, 65, 1, 41, "ObjectName",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[2012720, 82424, 65, 1, 36, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[2012788, 82427, 360, 7, 61, "Text",
 CellTags->"TensorFunction"],
Cell[2013151, 82436, 191, 9, 25, "Text",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2013379, 82450, 62, 1, 36, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[2013466, 82455, 178, 4, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2013647, 82461, 212, 6, 27, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2013896, 82472, 116, 3, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2014015, 82477, 193, 4, 42, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2014245, 82486, 198, 6, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2014446, 82494, 376, 14, 27, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2014859, 82513, 116, 3, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2014978, 82518, 358, 10, 42, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2015373, 82533, 228, 7, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2015604, 82542, 263, 8, 27, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2015904, 82555, 228, 7, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2016135, 82564, 263, 8, 27, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2016435, 82577, 116, 3, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2016554, 82582, 243, 7, 42, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2016834, 82594, 118, 3, 28, "Input",
 CellTags->"TensorFunction"],
Cell[2016955, 82599, 129, 4, 27, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[2017099, 82606, 154, 4, 28, "Input",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[2017268, 82613, 41, 1, 41, "ObjectName",
 CellTags->"Tf"],
Cell[CellGroupData[{
Cell[2017334, 82618, 53, 1, 36, "Subsubsection",
 CellTags->"Tf"],
Cell[2017390, 82621, 214, 10, 25, "Text",
 CellTags->"Tf"],
Cell[2017607, 82633, 414, 18, 25, "Text",
 CellTags->"Tf"]
}, Open  ]],
Cell[2018036, 82654, 43, 1, 41, "ObjectName",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[2018104, 82659, 54, 1, 36, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[2018183, 82664, 72, 2, 28, "Input"],
Cell[2018258, 82668, 428, 9, 21, "Message"]
}, Open  ]],
Cell[2018701, 82680, 576, 10, 97, "Text",
 CellTags->"TFi"],
Cell[2019280, 82692, 237, 8, 25, "Text",
 CellTags->"TFi"],
Cell[2019520, 82702, 185, 4, 43, "Text",
 CellTags->"TFi"],
Cell[2019708, 82708, 266, 13, 25, "Text",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2020011, 82726, 51, 1, 36, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[2020087, 82731, 506, 17, 28, "Input",
 CellTags->"TFi"],
Cell[2020596, 82750, 776, 24, 91, "Output",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2021409, 82779, 295, 10, 28, "Input",
 CellTags->"TFi"],
Cell[2021707, 82791, 333, 11, 27, "Output",
 CellTags->"TFi"]
}, Open  ]]
}, Open  ]],
Cell[2022067, 82806, 59, 1, 41, "ObjectName",
 CellTags->"ThreeVector"],
Cell[CellGroupData[{
Cell[2022151, 82811, 62, 1, 36, "Subsubsection",
 CellTags->"ThreeVector"],
Cell[2022216, 82814, 91, 1, 25, "Text",
 CellTags->"ThreeVector"],
Cell[2022310, 82817, 304, 13, 25, "Text",
 CellTags->"ThreeVector"]
}, Open  ]],
Cell[2022629, 82833, 43, 1, 41, "ObjectName",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[2022697, 82838, 54, 1, 36, "Subsubsection",
 CellTags->"TID"],
Cell[2022754, 82841, 80, 1, 25, "Text",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[2022859, 82846, 106, 3, 28, "Input",
 CellTags->"TID"],
Cell[2022968, 82851, 655, 15, 71, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[2023638, 82869, 293, 14, 25, "Text",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2023968, 82888, 51, 1, 36, "Subsubsection",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[2024044, 82893, 374, 12, 28, "Input",
 CellTags->"TID"],
Cell[2024421, 82907, 532, 21, 47, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2024990, 82933, 181, 6, 28, "Input",
 CellTags->"TID"],
Cell[2025174, 82941, 4307, 136, 58, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2029518, 83082, 271, 9, 28, "Input",
 CellTags->"TID"],
Cell[2029792, 83093, 389, 16, 47, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2030218, 83114, 123, 4, 28, "Input",
 CellTags->"TID"],
Cell[2030344, 83120, 625, 26, 46, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2031006, 83151, 195, 6, 28, "Input",
 CellTags->"TID"],
Cell[2031204, 83159, 625, 26, 46, "Output",
 CellTags->"TID"]
}, Open  ]]
}, Open  ]],
Cell[2031856, 83189, 45, 1, 41, "ObjectName",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2031926, 83194, 55, 1, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2031984, 83197, 87, 1, 25, "Text",
 CellTags->"TIDL"],
Cell[2032074, 83200, 154, 8, 25, "Text",
 CellTags->"TIDL"]
}, Open  ]],
Cell[2032243, 83211, 52, 1, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2032320, 83216, 154, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2032477, 83223, 414, 13, 25, "Text",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2032916, 83240, 202, 7, 28, "Input",
 CellTags->"TIDL"],
Cell[2033121, 83249, 465, 20, 43, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2033635, 83275, 159, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2033819, 83284, 312, 11, 28, "Input",
 CellTags->"TIDL"],
Cell[2034134, 83297, 1876, 74, 46, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2036059, 83377, 154, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2036238, 83386, 277, 10, 28, "Input",
 CellTags->"TIDL"],
Cell[2036518, 83398, 2812, 108, 45, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2039379, 83512, 159, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2039563, 83521, 387, 14, 28, "Input",
 CellTags->"TIDL"],
Cell[2039953, 83537, 24393, 841, 306, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2064395, 84384, 165, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2064563, 84391, 434, 15, 28, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2065034, 84411, 173, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2065210, 84418, 506, 17, 28, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2065753, 84440, 154, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2065932, 84449, 312, 11, 28, "Input",
 CellTags->"TIDL"],
Cell[2066247, 84462, 14684, 522, 131, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2080980, 84990, 159, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2081142, 84997, 374, 13, 28, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2081553, 85015, 165, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2081721, 85022, 444, 15, 28, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2082202, 85042, 173, 5, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[2082378, 85049, 529, 18, 28, "Input",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2082932, 85071, 240, 7, 28, "Input",
 CellTags->"TIDL"],
Cell[2083175, 85080, 161, 5, 27, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[2083363, 85089, 65, 1, 41, "ObjectName",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2083453, 85094, 65, 1, 36, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2083521, 85097, 225, 5, 43, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2083771, 85106, 128, 3, 28, "Input",
 CellTags->"TimedIntegrate"],
Cell[2083902, 85111, 424, 13, 27, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2084375, 85130, 62, 1, 36, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2084440, 85133, 73, 1, 25, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2084538, 85138, 305, 9, 28, "Input",
 CellTags->"TimedIntegrate"],
Cell[2084846, 85149, 129, 4, 27, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]],
Cell[2084990, 85156, 59, 1, 25, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2085074, 85161, 389, 11, 28, "Input",
 CellTags->"TimedIntegrate"],
Cell[2085466, 85174, 449, 14, 27, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2085942, 85192, 61, 1, 41, "ObjectName",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2086028, 85197, 54, 1, 36, "Subsubsection",
 CellTags->"TLI"],
Cell[2086085, 85200, 616, 12, 121, "Text",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2086726, 85216, 106, 3, 28, "Input",
 CellTags->"TLI"],
Cell[2086835, 85221, 229, 8, 27, "Output",
 CellTags->"TLI"]
}, Open  ]],
Cell[2087079, 85232, 168, 9, 25, "Text",
 CellTags->"TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2087284, 85246, 51, 1, 36, "Subsubsection",
 CellTags->"TLI"],
Cell[2087338, 85249, 70, 2, 28, "Input",
 CellTags->"TLI"]
}, Open  ]],
Cell[2087423, 85254, 65, 1, 41, "ObjectName",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2087513, 85259, 56, 1, 36, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2087572, 85262, 93, 1, 25, "Text",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2087690, 85267, 110, 3, 28, "Input",
 CellTags->"TLIFP"],
Cell[2087803, 85272, 425, 12, 27, "Output",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2088243, 85287, 161, 9, 25, "Text",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2088441, 85301, 53, 1, 36, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2088497, 85304, 72, 2, 28, "Input",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2088584, 85309, 67, 1, 41, "ObjectName",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2088676, 85314, 57, 1, 36, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2088736, 85317, 136, 4, 25, "Text",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2088897, 85325, 112, 3, 28, "Input",
 CellTags->"TLIHYP"],
Cell[2089012, 85330, 241, 8, 27, "Output",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2089268, 85341, 162, 9, 25, "Text",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2089467, 85355, 54, 1, 36, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2089524, 85358, 73, 2, 28, "Input",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2089612, 85363, 45, 1, 41, "ObjectName",
 CellTags->"TLI2"],
Cell[CellGroupData[{
Cell[2089682, 85368, 55, 1, 36, "Subsubsection",
 CellTags->"TLI2"],
Cell[2089740, 85371, 83, 1, 25, "Text",
 CellTags->"TLI2"],
Cell[2089826, 85374, 160, 9, 25, "Text",
 CellTags->"TLI2"]
}, Open  ]],
Cell[2090001, 85386, 67, 1, 41, "ObjectName",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2090093, 85391, 57, 1, 36, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2090153, 85394, 109, 3, 25, "Text",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2090287, 85401, 112, 3, 28, "Input",
 CellTags->"TLI2FC"],
Cell[2090402, 85406, 355, 11, 27, "Output",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2090772, 85420, 262, 13, 25, "Text",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2091071, 85438, 54, 1, 36, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2091128, 85441, 73, 2, 28, "Input",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2091216, 85446, 65, 1, 41, "ObjectName",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2091306, 85451, 65, 1, 36, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[2091374, 85454, 424, 7, 61, "Text",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2091823, 85465, 128, 3, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2091954, 85470, 189, 6, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[2092158, 85479, 209, 9, 25, "Text",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2092404, 85493, 62, 1, 36, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2092491, 85498, 217, 6, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2092711, 85506, 197, 6, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2092945, 85517, 277, 9, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2093225, 85528, 1338, 48, 46, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2094600, 85581, 206, 5, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2094809, 85588, 189, 6, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2095035, 85599, 277, 9, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2095315, 85610, 1353, 49, 48, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2096705, 85664, 461, 14, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2097169, 85680, 371, 13, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2097577, 85698, 118, 3, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2097698, 85703, 330, 9, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2098065, 85717, 409, 13, 28, "Input",
 CellTags->"ToDistribution"],
Cell[2098477, 85732, 371, 13, 27, "Output",
 CellTags->"ToDistribution"]
}, Open  ]]
}, Open  ]],
Cell[2098875, 85749, 69, 1, 41, "ObjectName",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2098969, 85754, 67, 1, 36, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[2099039, 85757, 1686, 67, 43, "Text",
 CellTags->"ToHypergeometric"],
Cell[2100728, 85826, 479, 19, 25, "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2101244, 85850, 64, 1, 36, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2101333, 85855, 395, 13, 28, "Input",
 CellTags->"ToHypergeometric"],
Cell[2101731, 85870, 1738, 58, 44, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2103506, 85933, 488, 16, 28, "Input",
 CellTags->"ToHypergeometric"],
Cell[2103997, 85951, 1555, 52, 44, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2105589, 86008, 395, 13, 28, "Input",
 CellTags->"ToHypergeometric"],
Cell[2105987, 86023, 1804, 60, 45, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107828, 86088, 488, 16, 28, "Input",
 CellTags->"ToHypergeometric"],
Cell[2108319, 86106, 1641, 55, 45, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]]
}, Open  ]],
Cell[2109987, 86165, 51, 1, 41, "ObjectName",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2110063, 86170, 58, 1, 36, "Subsubsection",
 CellTags->"ToLarin"],
Cell[2110124, 86173, 605, 19, 25, "Text",
 CellTags->"ToLarin"],
Cell[2110732, 86194, 163, 9, 25, "Text",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2110932, 86208, 55, 1, 36, "Subsubsection",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2111012, 86213, 229, 6, 28, "Input",
 CellTags->"ToLarin"],
Cell[2111244, 86221, 389, 15, 27, "Output",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2111670, 86241, 108, 3, 28, "Input",
 CellTags->"ToLarin"],
Cell[2111781, 86246, 1085, 45, 38, "Output",
 CellTags->"ToLarin"]
}, Open  ]]
}, Open  ]],
Cell[2112893, 86295, 47, 1, 41, "ObjectName",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2112965, 86300, 56, 1, 36, "Subsubsection",
 CellTags->"ToTFi"],
Cell[2113024, 86303, 274, 6, 43, "Text",
 CellTags->"ToTFi"],
Cell[2113301, 86311, 262, 13, 25, "Text",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2113600, 86329, 53, 1, 36, "Subsubsection",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2113678, 86334, 333, 11, 28, "Input",
 CellTags->"ToTFi"],
Cell[2114014, 86347, 1870, 49, 48, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2115921, 86401, 149, 4, 28, "Input",
 CellTags->"ToTFi"],
Cell[2116073, 86407, 2126, 54, 53, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2118236, 86466, 107, 3, 28, "Input",
 CellTags->"ToTFi"],
Cell[2118346, 86471, 491, 16, 42, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2118874, 86492, 468, 15, 28, "Input",
 CellTags->"ToTFi"],
Cell[2119345, 86509, 916, 33, 49, "Output",
 CellTags->"ToTFi"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2120310, 86548, 41, 1, 41, "ObjectName",
 CellTags->"TR"],
Cell[2120354, 86551, 100, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2120491, 86559, 53, 1, 36, "Subsubsection",
 CellTags->"TR"],
Cell[2120547, 86562, 420, 12, 61, "Text",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2120992, 86578, 104, 3, 28, "Input",
 CellTags->"TR"],
Cell[2121099, 86583, 761, 18, 71, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[2121875, 86604, 536, 23, 25, "Text",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2122448, 86632, 50, 1, 36, "Subsubsection",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2122523, 86637, 129, 4, 28, "Input",
 CellTags->"TR"],
Cell[2122655, 86643, 194, 6, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2122886, 86654, 98, 3, 28, "Input",
 CellTags->"TR"],
Cell[2122987, 86659, 175, 6, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2123199, 86670, 375, 12, 28, "Input",
 CellTags->"TR"],
Cell[2123577, 86684, 1227, 49, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2124841, 86738, 205, 6, 28, "Input",
 CellTags->"TR"],
Cell[2125049, 86746, 559, 23, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2125645, 86774, 240, 7, 28, "Input",
 CellTags->"TR"],
Cell[2125888, 86783, 8674, 322, 49, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2134599, 87110, 74, 2, 28, "Input",
 CellTags->"TR"],
Cell[2134676, 87114, 102, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2134815, 87122, 91, 3, 28, "Input",
 CellTags->"TR"],
Cell[2134909, 87127, 377, 6, 78, "Print",
 CellTags->{"Info3387712206-4027155", "TR"}]
}, Open  ]],
Cell[2135301, 87136, 166, 4, 25, "Text",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2135492, 87144, 100, 3, 28, "Input",
 CellTags->"TR"],
Cell[2135595, 87149, 103, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2135735, 87157, 421, 13, 28, "Input",
 CellTags->"TR"],
Cell[2136159, 87172, 99, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2136295, 87180, 171, 5, 28, "Input",
 CellTags->"TR"],
Cell[2136469, 87187, 991, 39, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2137497, 87231, 444, 14, 28, "Input",
 CellTags->"TR"],
Cell[2137944, 87247, 342, 13, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2138323, 87265, 224, 7, 28, "Input",
 CellTags->"TR"],
Cell[2138550, 87274, 180, 6, 28, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2138767, 87285, 104, 3, 28, "Input",
 CellTags->"TR"],
Cell[2138874, 87290, 173, 5, 42, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2139084, 87300, 347, 10, 28, "Input",
 CellTags->"TR"],
Cell[2139434, 87312, 190, 6, 28, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2139661, 87323, 104, 3, 28, "Input",
 CellTags->"TR"],
Cell[2139768, 87328, 183, 5, 42, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2139988, 87338, 339, 11, 28, "Input",
 CellTags->"TR"],
Cell[2140330, 87351, 428, 16, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2140795, 87372, 214, 6, 28, "Input",
 CellTags->"TR"],
Cell[2141012, 87380, 208, 7, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2141257, 87392, 213, 6, 28, "Input",
 CellTags->"TR"],
Cell[2141473, 87400, 208, 7, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2141718, 87412, 214, 6, 28, "Input",
 CellTags->"TR"],
Cell[2141935, 87420, 99, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2142071, 87428, 213, 6, 28, "Input",
 CellTags->"TR"],
Cell[2142287, 87436, 99, 3, 27, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2142423, 87444, 139, 4, 28, "Input"],
Cell[2142565, 87450, 150, 5, 27, "Output"]
}, Open  ]],
Cell[2142730, 87458, 119, 4, 28, "Input",
 CellTags->"TR"]
}, Open  ]],
Cell[2142864, 87465, 65, 1, 41, "ObjectName",
 CellTags->"TraceDimension"],
Cell[CellGroupData[{
Cell[2142954, 87470, 65, 1, 36, "Subsubsection",
 CellTags->"TraceDimension"],
Cell[2143022, 87473, 142, 4, 25, "Text",
 CellTags->"TraceDimension"],
Cell[2143167, 87479, 200, 9, 25, "Text",
 CellTags->"TraceDimension"]
}, Open  ]],
Cell[2143382, 87491, 57, 1, 41, "ObjectName",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2143464, 87496, 61, 1, 36, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[2143528, 87499, 149, 4, 25, "Text",
 CellTags->"TraceOfOne"],
Cell[2143680, 87505, 273, 13, 25, "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2143990, 87523, 58, 1, 36, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2144073, 87528, 273, 7, 28, "Input",
 CellTags->"TraceOfOne"],
Cell[2144349, 87537, 109, 3, 27, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2144495, 87545, 272, 7, 28, "Input",
 CellTags->"TraceOfOne"],
Cell[2144770, 87554, 109, 3, 27, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]]
}, Open  ]],
Cell[2144906, 87561, 47, 1, 41, "ObjectName",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2144978, 87566, 56, 1, 36, "Subsubsection",
 CellTags->"Trick"],
Cell[2145037, 87569, 174, 4, 43, "Text",
 CellTags->"Trick"],
Cell[2145214, 87575, 625, 29, 25, "Text",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2145876, 87609, 62, 1, 35, "Subsubsection",
 CellTags->"Trick"],
Cell[2145941, 87612, 376, 15, 25, "Text",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2146342, 87631, 350, 11, 28, "Input",
 CellTags->"Trick"],
Cell[2146695, 87644, 264, 10, 27, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2146996, 87659, 330, 11, 28, "Input",
 CellTags->"Trick"],
Cell[2147329, 87672, 426, 16, 27, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2147792, 87693, 104, 3, 28, "Input",
 CellTags->"Trick"],
Cell[2147899, 87698, 549, 23, 27, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2148485, 87726, 227, 6, 28, "Input",
 CellTags->"Trick"],
Cell[2148715, 87734, 391, 15, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2149143, 87754, 133, 4, 28, "Input",
 CellTags->"Trick"],
Cell[2149279, 87760, 213, 6, 42, "Output",
 CellTags->"Trick"]
}, Open  ]]
}, Open  ]],
Cell[2149519, 87770, 65, 1, 41, "ObjectName",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2149609, 87775, 65, 1, 36, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[2149677, 87778, 483, 8, 79, "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2150197, 87791, 62, 1, 36, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2150284, 87796, 364, 12, 28, "Input",
 CellTags->"TrickIntegrate"],
Cell[2150651, 87810, 517, 18, 28, "Output",
 CellTags->"TrickIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2151195, 87832, 67, 1, 70, "ObjectName",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2151287, 87837, 66, 1, 70, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[2151356, 87840, 1340, 55, 70, "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2152733, 87900, 63, 1, 36, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2152821, 87905, 528, 18, 28, "Input",
 CellTags->"TrickMandelstam"],
Cell[2153352, 87925, 250, 9, 27, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2153639, 87939, 568, 17, 28, "Input",
 CellTags->"TrickMandelstam"],
Cell[2154210, 87958, 235, 8, 27, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]]
}, Open  ]],
Cell[2154472, 87970, 43, 1, 70, "ObjectName",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2154540, 87975, 54, 1, 70, "Subsubsection",
 CellTags->"Tr2"],
Cell[2154597, 87978, 347, 6, 61, "Text",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2154969, 87988, 106, 3, 28, "Input",
 CellTags->"Tr2"],
Cell[2155078, 87993, 168, 6, 27, "Output",
 CellTags->"Tr2"]
}, Open  ]],
Cell[2155261, 88002, 593, 28, 70, "Text",
 CellTags->"Tr2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2155891, 88035, 51, 1, 36, "Subsubsection",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2155967, 88040, 164, 4, 28, "Input"],
Cell[2156134, 88046, 548, 20, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2156719, 88071, 187, 6, 28, "Input",
 CellTags->"Tr2"],
Cell[2156909, 88079, 156, 5, 27, "Output",
 CellTags->"Tr2"]
}, Open  ]]
}, Open  ]],
Cell[2157092, 88088, 116, 2, 70, "ObjectName",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2157233, 88094, 70, 1, 70, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[2157306, 88097, 194, 4, 70, "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2157537, 88106, 67, 1, 36, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2157629, 88111, 160, 4, 28, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2157792, 88117, 887, 30, 45, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2158716, 88152, 246, 7, 28, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2158965, 88161, 3113, 103, 48, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]]
}, Open  ]],
Cell[2162105, 88268, 120, 2, 70, "ObjectName",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2162250, 88274, 72, 1, 70, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[2162325, 88277, 239, 5, 70, "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2162601, 88287, 69, 1, 36, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2162695, 88292, 204, 5, 28, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2162902, 88299, 3999, 135, 77, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2166938, 88439, 159, 4, 28, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2167100, 88445, 1020, 35, 45, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2168157, 88485, 242, 6, 28, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2168402, 88493, 2610, 89, 48, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]]
}, Open  ]],
Cell[2171039, 88586, 75, 1, 70, "ObjectName",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2171139, 88591, 70, 1, 70, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2171212, 88594, 964, 15, 70, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2172179, 88611, 223, 9, 70, "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2172439, 88625, 99, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2172563, 88630, 352, 8, 50, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2172918, 88640, 444, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2173377, 88655, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2173464, 88660, 271, 8, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2173738, 88670, 894, 35, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2174669, 88710, 321, 9, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2174993, 88721, 1120, 45, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2176150, 88771, 221, 6, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2176374, 88779, 336, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2176747, 88796, 271, 7, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2177021, 88805, 705, 28, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2177741, 88836, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2177828, 88841, 355, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2178186, 88853, 1193, 48, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2179416, 88906, 325, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2179744, 88918, 492, 18, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2180273, 88941, 375, 11, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2180651, 88954, 11158, 407, 127, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2191824, 89364, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2191911, 89369, 431, 12, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2192345, 89383, 4934, 185, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2197316, 89573, 391, 12, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2197710, 89587, 606, 22, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2198353, 89614, 441, 13, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2198797, 89629, 24093, 861, 369, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2222939, 90496, 100, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2223064, 90501, 268, 6, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2223335, 90509, 444, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2223794, 90524, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2223881, 90529, 271, 8, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2224155, 90539, 904, 36, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2225096, 90580, 321, 9, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2225420, 90591, 2653, 101, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2228110, 90697, 221, 6, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2228334, 90705, 336, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2228707, 90722, 271, 7, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2228981, 90731, 2228, 83, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2231224, 90817, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2231311, 90822, 355, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2231669, 90834, 1267, 50, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2232973, 90889, 325, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2233301, 90901, 492, 18, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2233830, 90924, 375, 11, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2234208, 90937, 16156, 580, 200, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2250379, 91520, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2250466, 91525, 431, 12, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2250900, 91539, 4944, 186, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2255881, 91730, 391, 12, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2256275, 91744, 606, 22, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2256918, 91771, 441, 13, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2257362, 91786, 31649, 1133, 445, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2289060, 92925, 76, 1, 52, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2289139, 92928, 97, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2289239, 92931, 295, 7, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2289537, 92940, 380, 11, 28, "Input",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2289954, 92956, 102, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2290081, 92961, 284, 6, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2290368, 92969, 443, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2290826, 92984, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2290913, 92989, 320, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2291236, 93001, 3606, 128, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2294857, 93132, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2294944, 93137, 355, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2295302, 93149, 53443, 1666, 714, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2348760, 94818, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2348847, 94823, 482, 13, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2349332, 94838, 4934, 185, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2354315, 95029, 104, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2354444, 95034, 284, 6, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2354731, 95042, 443, 12, 27, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2355189, 95057, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2355276, 95062, 320, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2355599, 95074, 3037, 114, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2358651, 95191, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2358716, 95194, 380, 11, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2359121, 95209, 325, 10, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2359449, 95221, 3038, 114, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2362524, 95340, 375, 11, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2362902, 95353, 3038, 114, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2365955, 95470, 62, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2366042, 95475, 482, 13, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2366527, 95490, 4944, 186, 38, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2371486, 95679, 78, 1, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2371589, 95684, 520, 16, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2372112, 95702, 10853, 374, 149, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2382980, 96079, 109, 3, 25, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2383092, 96084, 295, 7, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2383412, 96095, 428, 13, 28, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2383843, 96110, 11798, 376, 168, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2395668, 96490, 75, 1, 70, "ObjectName",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2395768, 96495, 70, 1, 70, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[2395841, 96498, 980, 15, 70, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2396824, 96515, 223, 9, 70, "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2397062, 96527, 67, 1, 70, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2397154, 96532, 98, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2397277, 96537, 268, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2397548, 96545, 443, 12, 27, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2398006, 96560, 75, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2398106, 96565, 156, 4, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2398265, 96571, 600, 21, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2398880, 96595, 81, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2398986, 96600, 305, 9, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2399294, 96611, 1530, 55, 51, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2400839, 96669, 87, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2400951, 96674, 380, 11, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2401334, 96687, 4828, 159, 107, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2406211, 96852, 100, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2406336, 96857, 273, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2406612, 96865, 443, 12, 27, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2407070, 96880, 75, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2407170, 96885, 156, 4, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2407329, 96891, 481, 17, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2407825, 96911, 81, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2407931, 96916, 305, 9, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2408239, 96927, 1467, 53, 51, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2409721, 96983, 87, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2409833, 96988, 380, 11, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2410216, 97001, 4713, 156, 75, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2414944, 97160, 93, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2415062, 97165, 197, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2415262, 97173, 263, 8, 43, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2415562, 97186, 376, 10, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2415941, 97198, 1945, 71, 46, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2417923, 97274, 449, 12, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2418375, 97288, 9376, 296, 91, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2427800, 97590, 102, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2427927, 97595, 269, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2428199, 97603, 444, 12, 27, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2428658, 97618, 178, 4, 70, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2428839, 97624, 275, 5, 43, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2429139, 97633, 205, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2429347, 97641, 800, 28, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2430162, 97672, 100, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2430287, 97677, 305, 9, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2430595, 97688, 2280, 77, 51, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2432890, 97768, 87, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2433002, 97773, 156, 4, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2433161, 97779, 1102, 33, 64, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2434312, 97818, 104, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2434441, 97823, 269, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2434713, 97831, 444, 12, 27, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2435172, 97846, 75, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2435272, 97851, 205, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2435480, 97859, 800, 28, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2436295, 97890, 81, 1, 25, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2436401, 97895, 305, 9, 28, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2436709, 97906, 2280, 77, 51, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2439004, 97986, 210, 6, 28, "Input",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2439229, 97995, 75, 1, 57, "ObjectName",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2439329, 98000, 70, 1, 52, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[2439402, 98003, 768, 12, 115, "Text",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2440195, 98019, 134, 3, 28, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2440332, 98024, 328, 10, 27, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[2440675, 98037, 366, 14, 41, "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2441078, 98056, 67, 1, 36, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2441170, 98061, 132, 3, 28, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2441305, 98066, 654, 23, 28, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[2441986, 98093, 75, 1, 57, "ObjectName",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2442086, 98098, 70, 1, 52, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[2442159, 98101, 150, 4, 41, "Text",
 CellTags->"Twist4GluonOperator"],
Cell[2442312, 98107, 509, 19, 41, "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2442858, 98131, 67, 1, 36, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2442950, 98136, 529, 15, 28, "Input",
 CellTags->"Twist4GluonOperator"],
Cell[2443482, 98153, 119831, 4588, 1110, "Output",
 CellTags->"Twist4GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2563340, 102745, 104, 1, 57, "ObjectName",
 CellTags->"TwoLoopSimplify"],
Cell[CellGroupData[{
Cell[2563469, 102750, 66, 1, 52, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2563538, 102753, 335, 6, 59, "Text",
 CellTags->"TwoLoopSimplify"],
Cell[2563876, 102761, 207, 9, 41, "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2564120, 102775, 63, 1, 36, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2564186, 102778, 82, 2, 28, "Input",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[2564283, 102783, 57, 1, 57, "ObjectName",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2564365, 102788, 61, 1, 52, "Subsubsection",
 CellTags->"Uncontract"],
Cell[2564429, 102791, 256, 5, 43, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2564710, 102800, 120, 3, 28, "Input",
 CellTags->"Uncontract"],
Cell[2564833, 102805, 372, 11, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2565220, 102819, 181, 9, 41, "Text",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2565438, 102833, 58, 1, 36, "Subsubsection",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2565521, 102838, 234, 7, 28, "Input",
 CellTags->"Uncontract"],
Cell[2565758, 102847, 397, 17, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2566192, 102869, 143, 4, 28, "Input",
 CellTags->"Uncontract"],
Cell[2566338, 102875, 781, 31, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2567156, 102911, 138, 4, 28, "Input",
 CellTags->"Uncontract"],
Cell[2567297, 102917, 224, 8, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2567558, 102930, 143, 4, 28, "Input",
 CellTags->"Uncontract"],
Cell[2567704, 102936, 541, 20, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2568282, 102961, 153, 4, 28, "Input",
 CellTags->"Uncontract"],
Cell[2568438, 102967, 1138, 44, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2569591, 103014, 89, 1, 25, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2569705, 103019, 208, 6, 28, "Input",
 CellTags->"Uncontract"],
Cell[2569916, 103027, 209, 8, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2570140, 103038, 152, 4, 25, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2570317, 103046, 248, 7, 28, "Input",
 CellTags->"Uncontract"],
Cell[2570568, 103055, 570, 22, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2571175, 103082, 274, 8, 28, "Input",
 CellTags->"Uncontract"],
Cell[2571452, 103092, 1030, 40, 27, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2572497, 103135, 134, 4, 28, "Input",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2572646, 103142, 83, 1, 57, "ObjectName",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2572754, 103147, 74, 1, 52, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2572831, 103150, 198, 4, 41, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2573032, 103156, 343, 14, 41, "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2573412, 103175, 71, 1, 36, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2573486, 103178, 138, 3, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2573627, 103183, 153, 4, 25, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2573805, 103191, 107, 2, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2573915, 103195, 123, 3, 27, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2574053, 103201, 103, 1, 25, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2574159, 103204, 140, 3, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2574324, 103211, 161, 4, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2574488, 103217, 123, 3, 27, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2574626, 103223, 161, 4, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2574812, 103231, 171, 4, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2574986, 103237, 173, 5, 27, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2575174, 103245, 163, 4, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2575362, 103253, 171, 4, 28, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2575536, 103259, 173, 5, 27, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[2575736, 103268, 47, 1, 57, "ObjectName",
 CellTags->"Upper"],
Cell[CellGroupData[{
Cell[2575808, 103273, 56, 1, 52, "Subsubsection",
 CellTags->"Upper"],
Cell[2575867, 103276, 126, 4, 41, "Text",
 CellTags->"Upper"],
Cell[2575996, 103282, 397, 18, 41, "Text",
 CellTags->"Upper"]
}, Open  ]],
Cell[2576408, 103303, 86, 1, 57, "ObjectName",
 CellTags->"UVPart"],
Cell[CellGroupData[{
Cell[2576519, 103308, 57, 1, 52, "Subsubsection",
 CellTags->"UVPart"],
Cell[2576579, 103311, 128, 4, 41, "Text",
 CellTags->"UVPart"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2576744, 103320, 54, 1, 36, "Subsubsection",
 CellTags->"UVPart"],
Cell[2576801, 103323, 73, 2, 28, "Input",
 CellTags->"UVPart"]
}, Open  ]],
Cell[2576889, 103328, 51, 1, 57, "ObjectName",
 CellTags->"Vectors"],
Cell[CellGroupData[{
Cell[2576965, 103333, 58, 1, 52, "Subsubsection",
 CellTags->"Vectors"],
Cell[2577026, 103336, 205, 5, 41, "Text",
 CellTags->"Vectors"],
Cell[2577234, 103343, 188, 8, 41, "Text",
 CellTags->"Vectors"]
}, Open  ]],
Cell[2577437, 103354, 53, 1, 57, "ObjectName",
 CellTags->"WriteOut"],
Cell[CellGroupData[{
Cell[2577515, 103359, 59, 1, 52, "Subsubsection",
 CellTags->"WriteOut"],
Cell[2577577, 103362, 216, 5, 41, "Text",
 CellTags->"WriteOut"],
Cell[2577796, 103369, 171, 8, 41, "Text",
 CellTags->"WriteOut"]
}, Open  ]],
Cell[2577982, 103380, 61, 1, 57, "ObjectName",
 CellTags->"WriteOutPaVe"],
Cell[CellGroupData[{
Cell[2578068, 103385, 63, 1, 52, "Subsubsection",
 CellTags->"WriteOutPaVe"],
Cell[2578134, 103388, 260, 5, 59, "Text",
 CellTags->"WriteOutPaVe"],
Cell[2578397, 103395, 398, 18, 70, "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],
Cell[2578810, 103416, 49, 1, 57, "ObjectName",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2578884, 103421, 57, 1, 52, "Subsubsection",
 CellTags->"Write2"],
Cell[2578944, 103424, 297, 6, 70, "Text",
 CellTags->"Write2"],
Cell[2579244, 103432, 153, 4, 25, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2579422, 103440, 136, 3, 28, "Input",
 CellTags->"Write2"],
Cell[2579561, 103445, 248, 9, 43, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2579846, 103459, 112, 3, 28, "Input",
 CellTags->"Write2"],
Cell[2579961, 103464, 637, 15, 49, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2580635, 103484, 115, 3, 28, "Input",
 CellTags->"Write2"],
Cell[2580753, 103489, 133, 4, 27, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2580901, 103496, 290, 14, 41, "Text",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2581228, 103515, 54, 1, 36, "Subsubsection",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2581307, 103520, 429, 15, 34, "Input",
 CellTags->"Write2"],
Cell[2581739, 103537, 855, 29, 28, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2582609, 103569, 208, 7, 28, "Input",
 CellTags->"Write2"],
Cell[2582820, 103578, 78, 1, 25, "Text",
 CellTags->"Write2"],
Cell[2582901, 103581, 184, 6, 28, "Input",
 CellTags->"Write2"],
Cell[2583088, 103589, 73, 1, 25, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2583186, 103594, 363, 11, 28, "Input",
 CellTags->"Write2"],
Cell[2583552, 103607, 801, 22, 75, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2584368, 103632, 286, 8, 28, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2584679, 103644, 233, 7, 28, "Input",
 CellTags->"Write2"],
Cell[2584915, 103653, 700, 27, 28, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2585630, 103683, 185, 6, 28, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2585840, 103693, 363, 11, 28, "Input",
 CellTags->"Write2"],
Cell[2586206, 103706, 1025, 29, 183, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2587246, 103738, 286, 8, 28, "Input",
 CellTags->"Write2"],
Cell[2587535, 103748, 106, 3, 25, "Text",
 CellTags->"Write2"],
Cell[2587644, 103753, 247, 8, 28, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2587916, 103765, 363, 11, 28, "Input",
 CellTags->"Write2"],
Cell[2588282, 103778, 942, 25, 129, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2589239, 103806, 397, 11, 50, "Input",
 CellTags->"Write2"]
}, Open  ]],
Cell[2589651, 103820, 43, 1, 57, "ObjectName",
 CellTags->"XYT"],
Cell[CellGroupData[{
Cell[2589719, 103825, 54, 1, 52, "Subsubsection",
 CellTags->"XYT"],
Cell[2589776, 103828, 215, 9, 41, "Text",
 CellTags->"XYT"]
}, Open  ]],
Cell[2590006, 103840, 79, 1, 57, "ObjectName",
 CellTags->"ZeroMomentumInsertion"],
Cell[CellGroupData[{
Cell[2590110, 103845, 72, 1, 52, "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],
Cell[2590185, 103848, 154, 4, 41, "Text",
 CellTags->"ZeroMomentumInsertion"],
Cell[2590342, 103854, 473, 18, 41, "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],
Cell[2590830, 103875, 47, 1, 57, "ObjectName",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2590902, 103880, 56, 1, 52, "Subsubsection",
 CellTags->"Zeta2"],
Cell[2590961, 103883, 94, 1, 41, "Text",
 CellTags->"Zeta2"],
Cell[2591058, 103886, 191, 8, 41, "Text",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2591286, 103899, 53, 1, 36, "Subsubsection",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2591364, 103904, 77, 2, 28, "Input",
 CellTags->"Zeta2"],
Cell[2591444, 103908, 136, 4, 27, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2591617, 103917, 104, 3, 28, "Input",
 CellTags->"Zeta2"],
Cell[2591724, 103922, 120, 3, 27, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2591881, 103930, 138, 4, 28, "Input",
 CellTags->"Zeta2"],
Cell[2592022, 103936, 160, 5, 27, "Output",
 CellTags->"Zeta2"]
}, Open  ]]
}, Open  ]],
Cell[2592209, 103945, 65, 1, 41, "ObjectName",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2592299, 103950, 65, 1, 36, "Subsubsection",
 CellTags->"$Abbreviations"],
Cell[2592367, 103953, 293, 6, 43, "Text",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2592685, 103963, 129, 3, 28, "Input",
 CellTags->"$Abbreviations"],
Cell[2592817, 103968, 867, 17, 82, "Output",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2593699, 103988, 652, 29, 25, "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2594366, 104020, 43, 1, 41, "ObjectName",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2594434, 104025, 54, 1, 36, "Subsubsection",
 CellTags->"$AL"],
Cell[2594491, 104028, 114, 3, 25, "Text",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2594630, 104035, 73, 2, 28, "Input",
 CellTags->"$AL"],
Cell[2594706, 104039, 102, 3, 27, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2594845, 104047, 241, 7, 28, "Input",
 CellTags->"$AL"],
Cell[2595089, 104056, 563, 22, 27, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2595667, 104081, 117, 4, 28, "Input",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2595809, 104089, 241, 7, 28, "Input",
 CellTags->"$AL"],
Cell[2596053, 104098, 477, 20, 27, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2596545, 104121, 109, 4, 28, "Input",
 CellTags->"$AL"]
}, Open  ]],
Cell[2596669, 104128, 61, 1, 41, "ObjectName",
 CellTags->"$BreitMaison"],
Cell[CellGroupData[{
Cell[2596755, 104133, 63, 1, 36, "Subsubsection",
 CellTags->"$BreitMaison"],
Cell[2596821, 104136, 319, 9, 43, "Text",
 CellTags->"$BreitMaison"],
Cell[2597143, 104147, 370, 19, 25, "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],
Cell[2597528, 104169, 49, 1, 41, "ObjectName",
 CellTags->"$Color"],
Cell[CellGroupData[{
Cell[2597602, 104174, 57, 1, 36, "Subsubsection",
 CellTags->"$Color"],
Cell[2597662, 104177, 130, 4, 25, "Text",
 CellTags->"$Color"]
}, Open  ]],
Cell[2597807, 104184, 57, 1, 41, "ObjectName",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2597889, 104189, 61, 1, 36, "Subsubsection",
 CellTags->"$Covariant"],
Cell[2597953, 104192, 183, 4, 43, "Text",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2598161, 104200, 87, 2, 28, "Input",
 CellTags->"$Covariant"],
Cell[2598251, 104204, 111, 3, 27, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2598399, 104212, 110, 3, 28, "Input",
 CellTags->"$Covariant"],
Cell[2598512, 104217, 142, 4, 27, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2598669, 104224, 130, 4, 28, "Input",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2598824, 104232, 110, 3, 28, "Input",
 CellTags->"$Covariant"],
Cell[2598937, 104237, 140, 4, 27, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2599092, 104244, 131, 4, 28, "Input",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2599238, 104251, 45, 1, 41, "ObjectName",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2599308, 104256, 55, 1, 36, "Subsubsection",
 CellTags->"$FCS"],
Cell[2599366, 104259, 142, 4, 25, "Text",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2599533, 104267, 75, 2, 28, "Input",
 CellTags->"$FCS"],
Cell[2599611, 104271, 700, 14, 49, "Output",
 CellTags->"$FCS"]
}, Open  ]]
}, Open  ]],
Cell[2600338, 104289, 45, 1, 41, "ObjectName",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2600408, 104294, 55, 1, 36, "Subsubsection",
 CellTags->"$FCT"],
Cell[2600466, 104297, 177, 4, 43, "Text",
 CellTags->"$FCT"],
Cell[2600646, 104303, 128, 4, 28, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2600799, 104311, 170, 5, 28, "Input",
 CellTags->"$FCT"],
Cell[2600972, 104318, 196, 6, 27, "Output",
 CellTags->"$FCT"]
}, Open  ]],
Cell[2601183, 104327, 129, 4, 28, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2601337, 104335, 170, 5, 28, "Input",
 CellTags->"$FCT"],
Cell[2601510, 104342, 196, 6, 27, "Output",
 CellTags->"$FCT"]
}, Open  ]]
}, Open  ]],
Cell[2601733, 104352, 65, 1, 41, "ObjectName",
 CellTags->"$FeynCalcStuff"],
Cell[CellGroupData[{
Cell[2601823, 104357, 65, 1, 36, "Subsubsection",
 CellTags->"$FeynCalcStuff"],
Cell[2601891, 104360, 102, 1, 25, "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],
Cell[2602008, 104364, 69, 1, 41, "ObjectName",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2602102, 104369, 67, 1, 36, "Subsubsection",
 CellTags->"$FeynCalcVersion"],
Cell[2602172, 104372, 124, 3, 25, "Text",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2602321, 104379, 99, 2, 28, "Input",
 CellTags->"$FeynCalcVersion"],
Cell[2602423, 104383, 125, 3, 27, "Output",
 CellTags->"$FeynCalcVersion"]
}, Open  ]]
}, Open  ]],
Cell[2602575, 104390, 95, 1, 41, "ObjectName",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2602695, 104395, 80, 1, 36, "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],
Cell[2602778, 104398, 143, 3, 25, "Text",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2602946, 104405, 125, 2, 28, "Input",
 CellTags->"$FortranContinuationCharacter"],
Cell[2603074, 104409, 134, 3, 27, "Output",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2603223, 104415, 194, 9, 25, "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2603432, 104427, 49, 1, 41, "ObjectName",
 CellTags->"$Gauge"],
Cell[CellGroupData[{
Cell[2603506, 104432, 57, 1, 36, "Subsubsection",
 CellTags->"$Gauge"],
Cell[2603566, 104435, 260, 5, 43, "Text",
 CellTags->"$Gauge"],
Cell[2603829, 104442, 168, 9, 25, "Text",
 CellTags->"$Gauge"]
}, Open  ]],
Cell[2604012, 104454, 49, 1, 41, "ObjectName",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2604086, 104459, 57, 1, 36, "Subsubsection",
 CellTags->"$Larin"],
Cell[2604146, 104462, 1133, 30, 115, "Text",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2605304, 104496, 79, 2, 28, "Input",
 CellTags->"$Larin"],
Cell[2605386, 104500, 107, 3, 27, "Output",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2605508, 104506, 377, 18, 25, "Text",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2605900, 104527, 55, 1, 57, "ObjectName",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2605980, 104532, 60, 1, 52, "Subsubsection",
 CellTags->"$LimitTo4"],
Cell[2606043, 104535, 203, 4, 43, "Text",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2606271, 104543, 85, 2, 28, "Input",
 CellTags->"$LimitTo4"],
Cell[2606359, 104547, 109, 3, 27, "Output",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2606483, 104553, 391, 19, 41, "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2606889, 104575, 67, 1, 57, "ObjectName",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2606981, 104580, 66, 1, 52, "Subsubsection",
 CellTags->"$LorentzIndices"],
Cell[2607050, 104583, 164, 4, 25, "Text",
 CellTags->"$LorentzIndices"],
Cell[2607217, 104589, 140, 4, 28, "Input",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2607382, 104597, 333, 9, 28, "Input",
 CellTags->"$LorentzIndices"],
Cell[2607718, 104608, 669, 28, 28, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2608424, 104641, 117, 3, 28, "Input",
 CellTags->"$LorentzIndices"],
Cell[2608544, 104646, 457, 13, 42, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2609038, 104664, 302, 9, 28, "Input",
 CellTags->"$LorentzIndices"],
Cell[2609343, 104675, 884, 38, 28, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2610264, 104718, 117, 3, 28, "Input",
 CellTags->"$LorentzIndices"],
Cell[2610384, 104723, 655, 19, 62, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2611054, 104745, 141, 4, 28, "Input",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2611210, 104752, 59, 1, 57, "ObjectName",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2611294, 104757, 62, 1, 52, "Subsubsection",
 CellTags->"$MIntegrate"],
Cell[2611359, 104760, 136, 4, 25, "Text",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2611520, 104768, 89, 2, 28, "Input",
 CellTags->"$MIntegrate"],
Cell[2611612, 104772, 126, 4, 27, "Output",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2611753, 104779, 191, 9, 41, "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2611959, 104791, 43, 1, 57, "ObjectName",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2612027, 104796, 54, 1, 52, "Subsubsection",
 CellTags->"$MU"],
Cell[2612084, 104799, 151, 4, 25, "Text",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2612260, 104807, 73, 2, 28, "Input",
 CellTags->"$MU"],
Cell[2612336, 104811, 102, 3, 27, "Output",
 CellTags->"$MU"]
}, Open  ]],
Cell[2612453, 104817, 174, 9, 41, "Text",
 CellTags->"$MU"]
}, Open  ]],
Cell[2612642, 104829, 69, 1, 57, "ObjectName",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2612736, 104834, 67, 1, 52, "Subsubsection",
 CellTags->"$MemoryAvailable"],
Cell[2612806, 104837, 434, 11, 61, "Text",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2613265, 104852, 99, 2, 28, "Input",
 CellTags->"$MemoryAvailable"],
Cell[2613367, 104856, 115, 3, 27, "Output",
 CellTags->"$MemoryAvailable"]
}, Open  ]]
}, Open  ]],
Cell[2613509, 104863, 53, 1, 57, "ObjectName",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2613587, 104868, 59, 1, 52, "Subsubsection",
 CellTags->"$NonComm"],
Cell[2613649, 104871, 109, 3, 25, "Text",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2613783, 104878, 83, 2, 28, "Input",
 CellTags->"$NonComm"],
Cell[2613869, 104882, 803, 15, 93, "Output",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2614687, 104900, 307, 14, 41, "Text",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2615009, 104917, 63, 1, 57, "ObjectName",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2615097, 104922, 64, 1, 52, "Subsubsection",
 CellTags->"$PairBrackets"],
Cell[2615164, 104925, 155, 4, 25, "Text",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2615344, 104933, 93, 2, 28, "Input",
 CellTags->"$PairBrackets"],
Cell[2615440, 104937, 114, 3, 27, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2615591, 104945, 204, 7, 28, "Input",
 CellTags->"$PairBrackets"],
Cell[2615798, 104954, 425, 18, 27, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2616260, 104977, 118, 3, 28, "Input",
 CellTags->"$PairBrackets"],
Cell[2616381, 104982, 113, 3, 27, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2616531, 104990, 204, 7, 28, "Input",
 CellTags->"$PairBrackets"],
Cell[2616738, 104999, 445, 18, 27, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2617220, 105022, 119, 3, 28, "Input",
 CellTags->"$PairBrackets"],
Cell[2617342, 105027, 114, 3, 27, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]]
}, Open  ]],
Cell[2617483, 105034, 65, 1, 57, "ObjectName",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2617573, 105039, 65, 1, 52, "Subsubsection",
 CellTags->"$SpinorMinimal"],
Cell[2617641, 105042, 241, 5, 43, "Text",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2617907, 105051, 95, 2, 28, "Input",
 CellTags->"$SpinorMinimal"],
Cell[2618005, 105055, 115, 3, 27, "Output",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2618135, 105061, 200, 9, 41, "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2618350, 105073, 61, 1, 57, "ObjectName",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2618436, 105078, 63, 1, 52, "Subsubsection",
 CellTags->"$VeryVerbose"],
Cell[2618502, 105081, 221, 5, 43, "Text",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2618748, 105090, 91, 2, 28, "Input",
 CellTags->"$VeryVerbose"],
Cell[2618842, 105094, 109, 3, 27, "Output",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2618966, 105100, 141, 4, 28, "Input",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2619132, 105108, 287, 10, 28, "Input",
 CellTags->"$VeryVerbose"],
Cell[2619422, 105120, 121, 3, 22, "Print",
 CellTags->"$VeryVerbose"],
Cell[2619546, 105125, 255, 7, 23, "Print",
 CellTags->"$VeryVerbose"],
Cell[2619804, 105134, 273, 7, 23, "Print",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2620092, 105144, 131, 4, 28, "Input",
 CellTags->"$VeryVerbose"],
Cell[2620226, 105150, 159, 9, 41, "Text",
 CellTags->"$V0"]
}, Open  ]],
Cell[2620400, 105162, 47, 1, 57, "ObjectName",
 CellTags->"$West"],
Cell[CellGroupData[{
Cell[2620472, 105167, 56, 1, 52, "Subsubsection",
 CellTags->"$West"],
Cell[2620531, 105170, 468, 13, 59, "Text",
 CellTags->"$West"],
Cell[2621002, 105185, 158, 8, 41, "Text",
 CellTags->"$West"],
Cell[2621163, 105195, 54, 0, 31, "Commentary"],
Cell[CellGroupData[{
Cell[2621242, 105199, 125, 3, 28, "Input"],
Cell[2621370, 105204, 109, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2621516, 105211, 85, 2, 28, "Input"],
Cell[2621604, 105215, 89, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2621730, 105222, 87, 2, 28, "Input"],
Cell[2621820, 105226, 89, 2, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2621946, 105233, 82, 2, 28, "Input"],
Cell[2622031, 105237, 91, 2, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2622183, 105246, 70, 1, 81, "Section",
 CellTags->"Experimental"],
Cell[2622256, 105249, 73, 1, 57, "ObjectName",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2622354, 105254, 69, 1, 52, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2622426, 105257, 291, 7, 70, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2622720, 105266, 317, 14, 70, "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2623074, 105285, 66, 1, 52, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2623143, 105288, 310, 6, 70, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2623456, 105296, 123, 3, 25, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2623604, 105303, 571, 17, 50, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2624178, 105322, 448, 16, 27, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2624663, 105343, 116, 3, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2624782, 105348, 133, 4, 27, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2624930, 105355, 1270, 48, 70, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2626203, 105405, 562, 21, 25, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2626768, 105428, 142, 4, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2626913, 105434, 311, 10, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2627249, 105448, 271, 8, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2627523, 105458, 733, 20, 73, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2628271, 105481, 213, 9, 25, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2628487, 105492, 276, 9, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2628766, 105503, 328, 10, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2629097, 105515, 323, 13, 41, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2629423, 105530, 697, 30, 26, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2630145, 105564, 434, 13, 51, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2630582, 105579, 773, 24, 73, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2631370, 105606, 624, 27, 26, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2632019, 105637, 433, 13, 51, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2632455, 105652, 773, 24, 73, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2633265, 105681, 859, 24, 72, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2634127, 105707, 606, 22, 28, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2634770, 105734, 130, 3, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2634903, 105739, 323, 12, 30, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2635263, 105756, 116, 3, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2635382, 105761, 133, 4, 27, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2635552, 105770, 120, 3, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2635675, 105775, 133, 4, 27, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2635845, 105784, 126, 3, 28, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2635974, 105789, 133, 4, 27, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2636122, 105796, 167, 4, 28, "Input",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2636304, 105803, 90, 1, 41, "ObjectName",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2636419, 105808, 69, 1, 36, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2636491, 105811, 1518, 23, 195, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[2638012, 105836, 347, 14, 25, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2638384, 105854, 136, 3, 28, "Input",
 CellTags->"FeynmanDoIntegrals"],
Cell[2638523, 105859, 1037, 29, 71, "Output",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2639609, 105894, 66, 1, 36, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2639678, 105897, 85, 2, 28, "Input",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]],
Cell[2639778, 105902, 92, 1, 41, "ObjectName",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2639895, 105907, 70, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2639968, 105910, 717, 12, 141, "Text",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2640710, 105926, 138, 3, 28, "Input",
 CellTags->"FeynmanParametrize1"],
Cell[2640851, 105931, 505, 13, 49, "Output",
 CellTags->"FeynmanParametrize1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2641405, 105950, 67, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2641475, 105953, 86, 2, 28, "Input",
 CellTags->"FeynmanParametrize1"]
}, Open  ]],
Cell[2641576, 105958, 80, 1, 41, "ObjectName",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2641681, 105963, 64, 1, 36, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2641748, 105966, 723, 12, 141, "Text",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2642496, 105982, 126, 3, 28, "Input",
 CellTags->"FeynmanReduce"],
Cell[2642625, 105987, 288, 9, 27, "Output",
 CellTags->"FeynmanReduce"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2642962, 106002, 61, 1, 36, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2643026, 106005, 80, 2, 28, "Input",
 CellTags->"FeynmanReduce"],
Cell[2643109, 106009, 122, 3, 25, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2643256, 106016, 198, 6, 28, "Input",
 CellTags->"Experimental"],
Cell[2643457, 106024, 193, 6, 27, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[2643665, 106033, 100, 1, 25, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2643790, 106038, 222, 6, 28, "Input",
 CellTags->"Experimental"],
Cell[2644015, 106046, 246, 7, 27, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2644298, 106058, 75, 1, 28, "Input"],
Cell[2644376, 106061, 109, 2, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
