(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   1001597,      32346]
NotebookOptionsPosition[    916031,      29766]
NotebookOutlinePosition[    938444,      30508]
CellTagsIndexPosition[    932850,      30324]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["TARCER 2.0.0", "Title",
 TextAlignment->Center,
 CellTags->"T1.1"],

Cell[TextData[{
 "A ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " program for the reduction of two-loop propagator integrals "
}], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "Rolf Mertig\n",
 ButtonBox["GluonVision GmbH\n",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.gluonvision.com"], None}],
 "and\nRainer Scharf\n",
 ButtonBox["Universit\[ADoubleDot]t Leipzig",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.physik.uni-leipzig.de/~scharf/"], None}]
}], "Text",
 Editable->False,
 TextAlignment->Center,
 CellTags->"T1.2"],

Cell["April 2nd, 2009", "Text",
 TextAlignment->Center],

Cell[TextData[{
 "COMMENTS (RM 2.4.2009) fixed notebook for ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 7."
}], "Text"],

Cell["\<\
COMMENTS (RM 14.3..2004) uncommented a global variable assignement.changed \
default setting of $RankLimit to {2,5}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 31.10.2003) Change default settings of TarcerExpand, modify \
$BasisIntegras for massless integrals,
changed default setting of $RankLimit to {1,6}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 28.10.2003) Fixed a bug in TarcerExpand (only introduced \
recently),
changed default setting of $RankLimit to {1,5}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 24.9.2003) Relaxed a condition in equation (61) and (6153)\
\>", "Text"],

Cell["\<\
COMMENTS (RM 18.9.2003): Fixed things for 5.0, eliminated a TVI symmetry \
(causing infinite recursion) and added
TFI decomposition into products of TAI's (was somehow missing but shows up \
in, e.g., the 2-loop Higss selfenergy).
Putting TFI back into the Tarcer context, because the fcloops context created \
trouble ...
Changed default of $Comment to False.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 8.9.2002.): Changed context of TFI to \
HighEnergyPhysics`fcloops`TFI` and updated other HighEnergyPhysics`* contexts.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed PreFactor into Prefactor1 to avoid conflict \
with FeynArts and FeynCalc.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed context of Zeta2 and made definition \
dependent on not having loaded FeynCalc.\
\>", "Text"],

Cell[TextData[{
 "COMMENTS (RM 25.4.): ",
 ButtonBox["TFISimplify",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.7.1.1"],
 " (like TFIRecurse, but without trule[9])."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 23.4.): bug fixed in ",
 ButtonBox["tlrule[17]",
  BaseStyle->"Hyperlink",
  ButtonData:>"tlrule17"],
 "."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 6.4.):  ",
 ButtonBox["special TVR relations",
  BaseStyle->"Hyperlink",
  ButtonData:>"specialTVR"],
 " added to TVR."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 22.3.):  ",
 ButtonBox["TVI Symmetry",
  BaseStyle->"Hyperlink",
  ButtonData:>"Tsymmetry"],
 " added to TVR."
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 18.3.): More rules added to ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". "
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 18.2.): More rules added to ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". "
}], "Text"],

Cell[TextData[{
 "COMMENTS (RM 21.2.): Included ",
 ButtonBox["$TTable",
  BaseStyle->"Hyperlink",
  ButtonData:>"$TTable"],
 ". added tlrule[25]"
}], "Text"],

Cell["\<\
COMMENTS (RM 20.2.): $RankLimit = {0,6} ;  Changed TarcerRecurse such that \
special TJR and TVR (reducing D -> D-2) are used first.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 15.2.): $RankLimit = {0,8} ;  
                                              The following test now works:
                                              
                                                 Print[\"test 1 :  TFI[D, pp, \
{0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,MW}}] = \",
                                                            Timing[t1 \
=TarcerRecurse[TFI[D, pp, {0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, \
MZ}, {1,MW}}]]
                                                                       ] // \
InputForm
                                                         ]

                                                  Print[\"test 2 :  TFI[d, \
pp, 0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}] = \",
                                                           Timing[t2 \
=TarcerRecurse[TFI[D, pp,  0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}]];
                                                                       ] // \
InputForm
                                                          ]
\
\>", "Text"],

Cell["\<\
COMMENTS (RM 13.2.): tlrule[11] and trule[12] made more restrictive: \
inhibiting thus a recursion if {n5,m5} equals {0,0}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 12.2.): tlrule[15] made safer (in order to not get   0 :> 0 );
                                             TFIRecurse made safer (to not \
get into a recursion);
                                                  There is still something \
wrong with the tlrule's ; since 
                                         TFIRecurse[TFI[D, pp, {0, 0, 0, 1, \
3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,   MW}}]]
                                         doesn't work ..\
\>", "Text"],

Cell["\<\
COMMENTS (RM 5.2.): tlrule[10] slightly changed to cover more cases.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 2.2.):  FunctionExpand added in tlrule[9]. Option TimeConstraint \
added to TarcerRecurse (because TimeConstrained is buggy under Linux).\
\>", "Text"],

Cell[TextData[{
 "COMMENTS (RM 1.2.):  Changed TarcerRecurse. New TLR ",
 ButtonBox["definition",
  BaseStyle->"Hyperlink",
  ButtonData:>"TLRdef"],
 ". \n                          default $RankLimit={0,6};"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Remember", "=", "True"}], ";"}]], "Input"],

Cell["\<\
COMMENTS (RM 19.1.):   changes are:  tlrule[22] and tlrule[23], minor \
efficiency changes (Expand3 = Identity) and other small programming \
optimizations.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 22.1.):   changes are: bug in tlrule[23] fixed, new tlrule[24], \
new option Factor to TarcerReduce (doesn't change anything by default, but \
allows for improved Factor-ing).\
\>", "Text"],

Cell[TextData[{
 "See also: ",
 ButtonBox["hep-ph/9801383",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}],
 ", published in Comput. Phys. Commun. 111 (1998) 265-273"
}], "Text",
 Editable->False],

Cell[CellGroupData[{

Cell["Table of contents", "Section",
 CellDingbat->"\[FilledSmallCircle]"],

Cell[TextData[ButtonBox["Preliminaries",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.1.1"]], "Text"],

Cell[TextData[ButtonBox["Usages",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.2.1"]], "Text"],

Cell[TextData[ButtonBox["Creating the tarcerij.mx file",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.2.1.1"]], "Text"],

Cell[TextData[ButtonBox["Prologue (setting of $RankLimit)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.1"]], "Text"],

Cell[TextData[ButtonBox["Typesetting rules",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.1.1"]], "Text"],

Cell[TextData[ButtonBox["The Palettes",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.2.1"]], "Text"],

Cell[TextData[ButtonBox["OperatorApply",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.3.1"]], "Text"],

Cell[TextData[ButtonBox["MassDerivative",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.4.1"]], "Text"],

Cell[TextData[ButtonBox["TarasovT",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.5.1"]], "Text"],

Cell[TextData[ButtonBox["ApplyTarasovT",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.6.1"]], "Text"],

Cell[TextData[ButtonBox["Misc",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.1"]], "Text"],

Cell[TextData[ButtonBox["Colors",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["PQ",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["PNQ",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.3.1"]], "SmallText"],

Cell[TextData[ButtonBox["FactorC",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.4.1"]], "SmallText"],

Cell[TextData[ButtonBox["TFI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.5.1"]], "SmallText"],

Cell[TextData[ButtonBox["TJI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.6.1"]], "SmallText"],

Cell[TextData[ButtonBox["TBI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.7.1"]], "SmallText"],

Cell[TextData[ButtonBox["TAI",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.3.7.8.1"]], "SmallText"],

Cell[TextData[ButtonBox["Definitions and notation",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.1"]], "Text"],

Cell[TextData[StyleBox[ButtonBox["ToArgs",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.1.1"],
 FontColor->RGBColor[0.2, 0.6, 1]]], "Text"],

Cell[TextData[StyleBox[ButtonBox["Cayley",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.4.2.1"],
 FontColor->RGBColor[1, 0, 0.4]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      StyleBox["Cayley",
       FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], "=", 
     SubscriptBox["\[CapitalDelta]", "i"]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.4.3.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{
      StyleBox["CayleyD",
       FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], "=", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", "i"], "/", "\[CapitalDelta]"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.4.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[ButtonBox["Utility Functions",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun and friends",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1.1"]], "Text"],

Cell[TextData[ButtonBox["Interchange",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.1.1.1"]], "SmallText"],

Cell[TextData[{
 ButtonBox["\[Nu]",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.5.2.1"],
 ButtonBox["explicit",
  BaseStyle->"Hyperlink",
  ButtonData:>"T1.5.2.1"]
}], "Text"],

Cell[TextData[ButtonBox["TA",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.2.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["CheckRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.1"]], "Text"],

Cell[TextData[ButtonBox["CheckTVIRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["CheckTJIRecursion",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.5.3.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["Recurrence Relations Input and Recursion Function \
Generation",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.1"]], "Text"],

Cell[TextData[ButtonBox["Input (35)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.1.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (35)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.2.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["1", 
       RowBox[{"+", " "}]],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"35", "_"}], "1"}], ")"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.1.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["2", 
       RowBox[{"+", " "}]],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"35", "_"}], "2"}], ")"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.2.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["3", 
       RowBox[{"+", " "}]],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"35", "_"}], "3"}], ")"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.3.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     FormBox[
      SuperscriptBox["4", 
       RowBox[{"+", " "}]],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"35", "_"}], "4"}], ")"}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.2.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[ButtonBox["Input (3501)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.3.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{"MakeFun", " ", 
      RowBox[{"(", "3501", ")"}], " ", "\[CapitalDelta]"}], "=", 
     RowBox[{"0", " ", 
      FormBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["p", "2"], "=", 
         SuperscriptBox["m", "2"]}], ")"}],
       TraditionalForm]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.4.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (3502)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.5.1"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{"MakeFun", " ", 
      RowBox[{"(", "3502", ")"}], " ", "\[CapitalDelta]"}], "=", 
     RowBox[{"0", " ", 
      FormBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["p", "2"], "=", 
         SuperscriptBox["m", "2"]}], ")"}],
       TraditionalForm]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.6.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{"MakeFun", " ", 
      RowBox[{"(", "3503", ")"}], " ", "\[CapitalDelta]"}], "=", 
     RowBox[{"0", " ", 
      FormBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["p", "2"], "=", 
         SuperscriptBox["m", "2"]}], ")"}],
       TraditionalForm]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.7.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  ButtonBox[
   StyleBox[
    RowBox[{
     RowBox[{"MakeFun", " ", 
      RowBox[{"(", "3504", ")"}], " ", "\[CapitalDelta]"}], "=", 
     RowBox[{"0", " ", 
      FormBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["p", "2"], "=", 
         SuperscriptBox["m", "2"]}], ")"}],
       TraditionalForm]}]}],
    SingleLetterItalics->False],
   Active->True,
   Background->GrayLevel[1],
   ButtonData:>"T1.6.8.1",
   ButtonFunction:>(FrontEndExecute[{
      FrontEnd`NotebookLocate[#2]}]& ),
   ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (41)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.9.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (41)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.10.1"]], "Text"],

Cell[TextData[ButtonBox["Input (43)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.11.1"]], "Text"],

Cell[TextData[ButtonBox["Input (44)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.12.1"]], "Text"],

Cell[TextData[ButtonBox["Input (45)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.13.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (43) and (44)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.14.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (50)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.15.1"]], "Text"],

Cell[TextData[ButtonBox["Input (51)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.16.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (51)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.17.1"]], "Text"],

Cell[TextData[ButtonBox["Input (52)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.18.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (52)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.19.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (53)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.20.1"]], "Text"],

Cell[TextData[ButtonBox["Input (54)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.21.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (54)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.22.1"]], "Text"],

Cell[TextData[ButtonBox["Input (55)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.23.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (55)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.24.1"]], "Text"],

Cell[TextData[ButtonBox["Input (56)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.25.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (56)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.26.1"]], "Text"],

Cell[TextData[ButtonBox["Input (59)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.27.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (59)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.28.1"]], "Text"],

Cell[TextData[ButtonBox["Input (60)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.29.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (60)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.30.1"]], "Text"],

Cell[TextData[ButtonBox["Input (61)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.31.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (61)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.32.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.33.1"]], "Text"],

Cell[TextData[ButtonBox["Input (62)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.34.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (62)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.35.1"]], "Text"],

Cell[TextData[ButtonBox["Input (63)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.36.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (63)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.37.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.38.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.39.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6553)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.40.1"]], "Text"],

Cell[TextData[ButtonBox["Input (66)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.41.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (66)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.42.1"]], "Text"],

Cell[TextData[ButtonBox["Input (alternative67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.43.1"]], "Text"],

Cell[TextData[ButtonBox["Input (67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.44.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (67)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.45.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6713)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.46.1"]], "Text"],

Cell[TextData[ButtonBox["Input (68)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.47.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (68)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.48.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6813)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.49.1"]], "Text"],

Cell[TextData[ButtonBox["Input (70)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.50.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (70)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.51.1"]], "Text"],

Cell[TextData[ButtonBox["Input (71)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.52.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (71)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.53.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7123)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.54.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7113)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.55.1"]], "Text"],

Cell[TextData[ButtonBox["Input (78)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.56.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (78)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.57.1"]], "Text"],

Cell[TextData[ButtonBox["Input (80)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.58.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (80)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.59.1"]], "Text"],

Cell[TextData[ButtonBox["Input (81)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.60.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (81)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.61.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8123)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.62.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8113)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.63.1"]], "Text"],

Cell[TextData[ButtonBox["Input (82)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.64.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (82)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.65.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8213)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.66.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8212)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.67.1"]], "Text"],

Cell[TextData[ButtonBox["Input (83)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.68.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (83)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.69.1"]], "Text"],

Cell[TextData[ButtonBox["Input (87)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.1"]], "Text"],

Cell[TextData[ButtonBox["(88)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.1.1"]], "SmallText"],

Cell[TextData[ButtonBox["(89)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.70.2.1"]], "SmallText"],

Cell[TextData[ButtonBox["MakeFun (87)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.71.1"]], "Text"],

Cell[TextData[ButtonBox["Input (90)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.72.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (90)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.73.1"]], "Text"],

Cell[TextData[ButtonBox["Input (91)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.74.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (911)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.75.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (912)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.76.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (913)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.77.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (914)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.78.1"]], "Text"],

Cell[TextData[ButtonBox["Input (92)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.79.1"]], "Text"],

Cell[TextData[ButtonBox["Input (93)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.80.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (93)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.81.1"]], "Text"],

Cell[TextData[ButtonBox["Input (94)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.82.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (94)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.83.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9401)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.84.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9402)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.85.1"]], "Text"],

Cell[TextData[ButtonBox["Input (95)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.86.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (95)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.87.1"]], "Text"],

Cell[TextData[ButtonBox["Input (96)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.88.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (96)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.89.1"]], "Text"],

Cell[TextData[ButtonBox["Input (97)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.90.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (97)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.91.1"]], "Text"],

Cell[TextData[ButtonBox["Input (98)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.92.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (98)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.93.1"]], "Text"],

Cell[TextData[ButtonBox["Input (99)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.94.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (99)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.95.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9912)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.96.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9913)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.97.1"]], "Text"],

Cell[TextData[ButtonBox["Input (100)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.98.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (100)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.99.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1002)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.100.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1003)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.101.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1004)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.102.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1005)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.103.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1006)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.104.1"]], "Text"],

Cell[TextData[ButtonBox["Input (150)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.105.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (150)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.106.1"]], "Text"],

Cell[TextData[ButtonBox["Input (151)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.107.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (151)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.108.1"]], "Text"],

Cell[TextData[ButtonBox["Input (152)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.109.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (152)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.110.1"]], "Text"],

Cell[TextData[ButtonBox["Input (153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.111.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (153)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.112.1"]], "Text"],

Cell[TextData[ButtonBox["Input (200)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.113.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (200)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.114.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (201)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.115.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (202)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.116.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (203)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.117.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (204)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.118.1"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (205)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.119.1"]], "Text"],

Cell[TextData[ButtonBox["Input (1xxxx)",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.120.1"]], "Text"],

Cell[TextData[ButtonBox["five -> four",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.121.1"]], "Text"],

Cell[TextData[ButtonBox["four -> three",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.122.1"]], "Text"],

Cell[TextData[ButtonBox["three -> two",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.123.1"]], "Text"],

Cell[TextData[ButtonBox["two -> one",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.6.124.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse and TFIRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.1"]], "Text"],

Cell[TextData[ButtonBox["TFIRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.1.1"]], "Text"],

Cell[TextData[ButtonBox["T*IS",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.2.1"]], "Text"],

Cell[TextData[ButtonBox["TComment",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.3.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.7.4.1"]], "Text"],

Cell[TextData[ButtonBox["TarcerExpand",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.8.1"]], "Text"],

Cell[TextData[ButtonBox["Construct T's",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.9.1"]], "Text"],

Cell[TextData[ButtonBox["$BasisIntegrals",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.10.1"]], "Text"],

Cell[TextData[ButtonBox["Save",
 BaseStyle->"Hyperlink",
 ButtonData:>"T1.9.1"]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.1.1"],

Cell[TextData[{
 "This is free software.\nYou may use it, distribute it, change it, improve \
it, etc..\nIf you have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " questions, ask them to support@wolfram.com or to your local ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " reseller."
}], "SmallText",
 Editable->False],

Cell["\<\
 We consider the following type of integrals, where a,b,u,v,r,s,t are \
non-negative integers.\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", 
     SuperscriptBox["p", "2"], ",", "\[CapitalDelta]p", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"u", ",", "v", ",", "r", ",", "s", ",", "t"}], "}"}]}]}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               SubscriptBox["\[Nu]", "1"], ",", 
               SubscriptBox["m", "1"]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               SubscriptBox["\[Nu]", "2"], ","}]}]}]}],
           TraditionalForm], 
          SubscriptBox["m", "2"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], ",", 
          SubscriptBox["m", "3"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "4"], ",", 
          SubscriptBox["m", "4"]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Nu]", "5"], ",", 
          SubscriptBox["m", "5"]}], "}"}]}], "}"}], "]"}], " ", "="}], " "}], 
   TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SuperscriptBox["\[Pi]", "d"]], TraditionalForm]]],
 "\[Integral] \[Integral] ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       SuperscriptBox["d", "d"], 
       SubscriptBox["k", "1"], " ", 
       SuperscriptBox["d", "d"], 
       SubscriptBox["k", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
          SubscriptBox["k", "1"]}], ")"}], "a"], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
          SubscriptBox["k", "1"]}], ")"}], "b"]}],
      TraditionalForm], 
     SuperscriptBox[
      RowBox[{"(", 
       SubsuperscriptBox["k", "1", "2"], ")"}], "u"], 
     SuperscriptBox[
      RowBox[{"(", 
       SubsuperscriptBox["k", "2", "2"], ")"}], "v"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "\[CenterDot]", 
        SubscriptBox["k", "1"]}], ")"}], "r"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "\[CenterDot]", 
        SubscriptBox["k", "2"]}], ")"}], "s"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SubscriptBox["k", "2"]}], ")"}], "t"]}], 
    SuperscriptBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{
            SuperscriptBox[
             RowBox[{"[", 
              RowBox[{
               SubsuperscriptBox["k", "1", "2"], "-", 
               SubsuperscriptBox["m", "1", "2"]}], "]"}], 
             SubscriptBox["\[Nu]", "1"]], "[", 
            RowBox[{
             SubsuperscriptBox["k", "2", "2"], "-", 
             SubsuperscriptBox["m", "2", "2"]}], "]"}], 
           SubscriptBox["\[Nu]", "2"]], "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SubscriptBox["k", "1"], "-", "p"}], ")"}], "2"], "-", 
           SubsuperscriptBox["m", "3", "2"]}], "]"}], 
         SubscriptBox["\[Nu]", "3"]], "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["k", "2"], "-", "p"}], ")"}], "2"], "-", 
         SubsuperscriptBox["m", "4", "2"]}], "]"}], 
       SubscriptBox["\[Nu]", "4"]], "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["k", "1"], "-", 
          SubscriptBox["k", "2"]}], ")"}], "2"], "-", 
       SubsuperscriptBox["m", "5", "2"]}], "]"}], 
     SubscriptBox["\[Nu]", "5"]]], TraditionalForm]]]
}], "Text"],

Cell["\<\
Putting the        $Post = (#;)&;       instruction after   
Unset[$Post]   suppresses all output cells.
Putting Unset[$Post] behind  $Post=(#:)&;
will produce a lot of typsetted reduction formulae.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unset", "[", "$Post", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Post", " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"#", ";"}], ")"}], "&"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$Notebooks", "=!=", "True"}], ",", 
    RowBox[{"$Post", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"#", ";"}], ")"}], "&"}]}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$HistoryLength", "=", "0"}], ";"}]], "Input"],

Cell["\<\
This determines the directory into which the resulting file tarcer.mx is \
written.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$Notebooks", ",", 
   RowBox[{"SetDirectory", "[", 
    RowBox[{"ToFileName", "[", 
     RowBox[{"First", "[", 
      RowBox[{"\"\<FileName\>\"", "/.", 
       RowBox[{"NotebookInformation", "[", 
        RowBox[{"InputNotebook", "[", "]"}], "]"}]}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input"],

Cell[TextData[{
 "Somehow ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " crashes on my (R.M.) good old NeXTStation if I don't open all cells first \
...; this opens all cells automatically:"
}], "SmallText"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"$System", "===", "\"\<NeXT\>\""}], ",", 
   RowBox[{
    RowBox[{"SelectionMove", "[", 
     RowBox[{
      RowBox[{"EvaluationNotebook", "[", "]"}], ",", "All", ",", "Notebook"}],
      "]"}], ";", 
    RowBox[{"FrontEndExecute", "[", 
     RowBox[{"{", 
      RowBox[{"FrontEnd`FrontEndToken", "[", 
       RowBox[{
        RowBox[{"EvaluationNotebook", "[", "]"}], ",", 
        "\"\<SelectionOpenAllGroups\>\""}], "]"}], "}"}], "]"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell1"}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Creating the tarcerij.mx file", "Subsubsection",
 CellTags->"T1.2.1.1"],

Cell[TextData[{
 "Running this notebook will produce a tarcerij.mx file where ij will be \
replaced by \nthe two number set by the variable $RankLimit (default 33). \n\
In the same directory as this notebook which can be loaded  into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by    <<tarcer33.mx,\nor, when running in Terminal mode, on Unix systems, \
already when starting ",
 StyleBox["Mathematica, ",
  FontSlant->"Italic"],
 "as \nmyshellprompt>  math -initfile tarcer33.mx"
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usages", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.2.1"],

Cell[BoxData[""], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", "\"\<HighEnergyPhysics`Tarcer`\>\"", "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerRecurse", "::", "usage"}], "=", 
   "\"\<TarcerRecurse[expr] applies recurrence relations for all TFI's (and \
TVI, TJI, TBI, TAI) in expr. For evaluation and Laurent expansion of the \
resulting basis integrals use TarcerExpand.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerExpand", "::", "usage"}], "=", 
   "\"\<TarcerExpand[expr, d -> (4+e), order] or TarcerExpand[epxr, d -> \
(4-e), order] or TarcerExpand[expr, d -> (4 - 2 om)substitutes integrals as \
specified by the option TarcerRules, and does a Laurent expansion in the \
variable around 0 at the right hand side of the second argument (i.e., you \
can use whatever convention you like ...). The same effect as \
TarcerExpand[TarcerRecurse[expr], ...] can be achieved by setting the option \
TarcerRecurse of TarcerExpand to True. TarcerExpand[expr, d -> (4+e)] is \
equivalent to TarcerExpand[expr, d -> (4+e), 0].\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerReduce", "::", "usage"}], "=", 
   "\"\<TarcerReduce is an option to TarcerExpand. It should be set to a list \
of basic integrals.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cayley", "::", "usage"}], "=", 
   "\"\<Cayley[m1,m2,m3,m4,m5,Sqrt[pp]] is the Cayley determinant.\>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cayleyu", "::", "usage"}], "=", 
   "\"\<Cayleyu are the sub-Cayley-determinants as described in Tarasov's \
paper.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CayleyD", "::", "usage"}], "=", 
   "\"\<CayleyD are the sub-Cayley-determinants as described in Tarasov's \
paper.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Prefactor1", "::", "usage"}], "=", 
   "\"\<Prefactor1[expr] may be used to prohibit TarcerExpand from \
Taylor-expanding expr.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SEpsilon", "::", "usage"}], "=", 
   "\"\<Sepsilon[d] is an abbreviation for Exp[(d-4)/2 EulerGamma].\>\""}], 
  ";"}]], "Input"],

Cell["\<\
TFI::\"usage\" =
\"TFI[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is
the 2-loop d-dimensional integral
1/( (q1^2 - m1^2)^n1  (q2^2 - m2^2)^n2 ((q1-p)^2 - m3^2)^n3 *
    ((q2-p)^2 - m4^2)^n4  ((q1-q2)^2 - m5^2)^n5 ) .

 TFI[d, pp, {x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}]
has as additional factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z*
(q2.p)^v*(q1.q2)^w .

 TFI[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}]
has as additional factors in the numerator  (OPEDelta.q1)^a * \
(OPEDelta.q2)^b;
dp is (OPEDelta.p).\";\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFIRecurse", "::", "usage"}], "=", 
   "\"\<TFIRecurse[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFISimplify", "::", "usage"}], "=", 
   "\"\<TFISimplify[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr, except k1.k2.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVI", "::", "usage"}], "=", 
   "\"\<TVI[d,pp,{ {n1,m1},{n2,m2},{n3,m3},{n4,m4}}] is the massive 2-loop \
integral with four propagators in d dimensions. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJI", "::", "usage"}], "=", 
   "\"\<TJI[d, pp, { {n1,m1},{n2,m2},{n3,m3} }] is the massive 2-loop \
integral with three propagators in d dimensions. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TKI", "::", "usage"}], "=", 
   "\"\<TKI[d, 0, { {n1,m1},{n2,m2},{n3,m3} }] is the bubble massive 2-loop \
selfenergy type integral with three propagators in d dimensions. \>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBI", "::", "usage"}], "=", 
   "\"\<TBI[d, pp, { {n1,m1},{n2,m2} }] is the massive 1-loop selfenergy type \
integral with two propagators in d dimensions.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "::", "usage"}], "=", 
   "\"\<TAI[d, pp, { {n1,m1} }] is the massive 1-loop selfenergy type \
integral with one propagator in d dimensions.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFR", "::", "usage"}], "=", 
   "\"\<TFR is similar in syntax to TFI but applies once the recurrence \
relations derived for TFI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVR", "::", "usage"}], "=", 
   "\"\<TVR is similar in syntax to TVI but applies once the recurrence \
relations derived for TVI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJR", "::", "usage"}], "=", 
   "\"\<TJR is similar in syntax to TJI but applies once the recurrence \
relations derived for TJI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBR", "::", "usage"}], "=", 
   "\"\<TBR is similar in syntax to TBI but applies once the recurrence \
relations derived for TBI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAR", "::", "usage"}], "=", 
   "\"\<TAR is similar in syntax to TAI but applies once the recurrence \
relations derived for TAI.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$RankLimit", "::", "usage"}], "=", 
   "\"\<$RankLimit is the list of integers specifying the highest possible \
sums of exponents of scalar products; i.e., {a+b, r+s} means that at most \
(Delta.k1)^a * (Delta.k2)^2 * (p.k1)^r * (p.k2)^s in the numerator will be \
calculated (a,b,r,s have to be non-negative integares). $RankLimit has to be \
set in the TARCER.nb notebook. In the tarcer.mx file this is not possible \
anymore.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$TarcerRecursed", "::", "usage"}], "=", 
   "\"\<If $Comment is set to True all integrals\\n\\t\\twhich are reduced \
are collected in the global variable $TarcerRecured.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$BasisIntegrals", "::", "usage"}], "=", 
   "\"\<$BasisIntegrals is a list of some basis integrals. It is the default \
setting of TarcerReduce of TarcerExpand.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$Comment", "::", "usage"}], "=", 
   "\"\<$Comment can be set to True or False. If set to False no comments are \
generated.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$CommentNotebook", "::", "usage"}], "=", 
   "\"\<$Comment can be set to True or False. Setting it to True will \
generate a separate notebook (if and only if $Comment is True) where the used \
recursion relations are displayed.\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$RecursionStop", "::", "usage"}], "=", 
   "\"\<$RecursionStop sets an upper limit on the total number of recursion \
levels. Its purpose is to inhibit erroneous infinite recursions.\>\""}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$TTable", "::", "usage"}], "=", 
   "\"\<$TTable is a list of precomputed reductions. $TTable is the default \
setting of the option Table of TarcerRecurse. \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Expand3", "=", "Identity"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prologue (setting of $RankLimit)", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.3.1"],

Cell[TextData[{
 "This sets the limit up to which the ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 "-operators are constructed automatically, i.e., $RankLimit = {a+b, r+s} , \
concerning: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
       SubscriptBox["k", "1"]}], ")"}], "a"], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", "\[CenterDot]", 
      SubscriptBox["k", "2"]}], ")"}], "b"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "\[CenterDot]", 
       SubscriptBox["k", "1"]}],
      TraditionalForm], ")"}], "r"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "\[CenterDot]", 
       SubscriptBox["k", "2"]}],
      TraditionalForm], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "6"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "6"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"$RankLimit", "=", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5"}], "}"}]}], ";"}]}], "Input"],

Cell["Check recursion relations by changing False to True here:", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$CheckRecursion", "=", "False"}], ";"}]], "Input"],

Cell["\<\
Change False to True for getting the input palettes for entering new \
recursion relations.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PutPalettes", "=", "False"}], ";"}]], "Input"],

Cell[TextData[{
 "Notice:  Instead of using shift operators",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"  ", 
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"]}],
     TraditionalForm], ",", 
    StyleBox[
     SuperscriptBox["2", "+"],
     FontWeight->"Bold"]}], TraditionalForm]]],
 " etc. the \[Nu]-indices are colored blue; instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"  ", 
      StyleBox[
       SuperscriptBox["1", "-"],
       FontWeight->"Bold"]}],
     TraditionalForm], ",", 
    StyleBox[
     SuperscriptBox["2", "-"],
     FontWeight->"Bold"]}], TraditionalForm]]],
 " etc. the \[Nu]-indices are colored red."
}], "Text"],

Cell[CellGroupData[{

Cell["Typesetting rules", "Subsection",
 CellTags->"T1.3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"atime", "=", 
   RowBox[{"AbsoluteTime", "[", "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"SEpsilon", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"SEpsilon", "[", "d_", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"StyleBox", "[", 
      RowBox[{
       RowBox[{"SubscriptBox", "[", 
        RowBox[{"\"\<S\>\"", ",", 
         RowBox[{"First", "[", 
          RowBox[{"Variables", "[", "d", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}]}], "]"}], ",", 
     RowBox[{"SEpsilon", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "\[Rule]", "False"}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"SEpsilon", "[", "d_", "]"}], "n_"], ",", "fmt_"}], "]"}], ":=", 
  
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"StyleBox", "[", 
      RowBox[{
       RowBox[{"SubsuperscriptBox", "[", 
        RowBox[{"\"\<S\>\"", ",", 
         RowBox[{"First", "[", 
          RowBox[{"Variables", "[", "d", "]"}], "]"}], ",", "n"}], "]"}], ",", 
       RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}]}], "]"}], ",", 
     RowBox[{"SEpsilon", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "\[Rule]", "False"}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"Prefactor1", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"Prefactor1", "[", "d_", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"MakeBoxes", "[", 
      RowBox[{"d", ",", "fmt"}], "]"}], ",", 
     RowBox[{"Prefactor1", "[", "d", "]"}], ",", 
     RowBox[{"Editable", "->", "False"}]}], "}"}]}]}]], "Input"],

Cell["\<\
Don't mess with the InterpretationBox's !! (important for the palette and \
OperatorApply).\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"Schiebe", "[", 
     RowBox[{"i_", ",", "pm_String"}], "]"}], " ", ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "[", 
   RowBox[{
    RowBox[{"StyleBox", "[", 
     RowBox[{
      RowBox[{"SuperscriptBox", "[", 
       RowBox[{"i", ",", "pm"}], "]"}], ",", 
      RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
    RowBox[{"Schiebe", "[", 
     RowBox[{"i", ",", "pm"}], "]"}], ",", 
    RowBox[{"Editable", "->", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", "i_Integer", "]"}], "[", "args_", "]"}], ",", 
     "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", " ", "i"}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", 
       RowBox[{"i_Integer", ",", "j_Integer"}], "]"}], "[", "args_", "]"}], 
     ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
       RowBox[{
        RowBox[{"10", " ", "i"}], "+", "j"}]}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CayleyD", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CayleyD", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"FractionBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
       RowBox[{
        RowBox[{"100", "  ", "i"}], "+", 
        RowBox[{"10", "j"}], "+", "k"}]}], "]"}], ",", 
     "\"\<\[CapitalDelta]\>\""}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", "i_Integer", "]"}], "[", "args_", "]"}], ",", 
     "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", " ", "i"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", 
       RowBox[{"i_Integer", ",", "j_Integer"}], "]"}], "[", "args_", "]"}], 
     ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
     RowBox[{
      RowBox[{"10", " ", "i"}], "+", "j"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayley", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[CapitalDelta]\>\"", ",", 
     RowBox[{
      RowBox[{"100", "  ", "i"}], "+", 
      RowBox[{"10", "j"}], "+", "k"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayleyu", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Cayleyu", "[", 
       RowBox[{"i_Integer", ",", "j_Integer", ",", "k_Integer"}], "]"}], "[", 
      "args_", "]"}], ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<u\>\"", ",", 
     RowBox[{
      RowBox[{"100", "  ", "i"}], "+", 
      RowBox[{"10", "j"}], "+", "k"}]}], "]"}]}], ";"}]], "Input"],

Cell["\<\
MakeBoxes[PP, fm_] := InterpretationBox[SuperscriptBox[p, 2], PP]; 
MakeBoxes[m1^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 1, p], m1^p]; \

MakeBoxes[m2^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 2, p], m2^p]; \

MakeBoxes[m3^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 3, p], m3^p]; \

MakeBoxes[m4^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 4, p], m4^p]; \

MakeBoxes[m5^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 5, p], m5^p]; \

m6 = Sqrt[PP]; \
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m1", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "1"}], "]"}], ",", "m1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m2", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "2"}], "]"}], ",", "m2"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m3", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "3"}], "]"}], ",", "m3"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m4", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "4"}], "]"}], ",", "m4"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m5", ",", "_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"m", ",", "5"}], "]"}], ",", "m5"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"mbt", "[", "z_", "]"}], ":=", 
   RowBox[{"ToBoxes", "[", 
    RowBox[{"z", ",", "TraditionalForm"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "z", "]"}], "=!=", "Plus"}]}], "]"}], ":=", 
   RowBox[{"mbt", "[", "z", "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "1"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"mbt", "[", "z", "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_Subscript", "-", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"z", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         ",", "\"\<_\>\""}], "]"}], ",", 
       RowBox[{"z", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
      "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"OverscriptBox", "[", 
      RowBox[{
       RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "-", "3"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"UnderscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
       "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "+", "1"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"mbt", "[", "z", "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_", "+", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"OverscriptBox", "[", 
      RowBox[{
       RowBox[{"mbt", "[", "z", "]"}], ",", "\"\<_\>\""}], "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"redblue", "[", 
    RowBox[{"z_Subscript", "+", "2"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{
       RowBox[{"OverscriptBox", "[", 
        RowBox[{
         RowBox[{"z", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         ",", "\"\<_\>\""}], "]"}], ",", 
       RowBox[{"z", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
      "]"}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", "ur__", "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "ur"}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFI", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "ur", "}"}], ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"d_", ",", "pp_", ",", "dp_", ",", "any__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<F\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"",
               ",", "\"\< \>\"", ",", 
              RowBox[{"Apply", "[", 
               RowBox[{"Sequence", ",", 
                RowBox[{"Flatten", "[", 
                 RowBox[{"{", "any", "}"}], "]"}]}], "]"}]}], "}"}], "]"}]}], 
          "]"}], ",", "\n", "\t\t", 
         RowBox[{"TFI", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", "any", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "/;", 
    RowBox[{
     RowBox[{"Head", "[", "dp", "]"}], "=!=", "List"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFI", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFI", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVI", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVI", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVI", "]"}], "->", "TVI"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]", ",", "\[Gamma]", ",", 
       "\[Delta]"}], "}"}]}], "]"}], ",", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]", ",", "\[Gamma]", ",", 
       "\[Delta]"}], "}"}]}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJI", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJI", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJI", "]"}], "->", "TJI"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJI", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJI", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJI", "]"}], "->", "TJI"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TKI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TKI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<K\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TKI", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TKI", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<K\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TKI", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TKI", "]"}], "->", "TKI"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]"}], "}"}]}], "]"}], 
   ",", 
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]", ",", "\[Gamma]"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"TKI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]", ",", "\[Gamma]"}], 
      "}"}]}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBI", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "}"}]}], "]"}], ",", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "+", "1"}], ",", "\[Beta]"}], "}"}]}], "]"}], ",", 
   
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "-", "1"}], ",", "\[Beta]"}], "}"}]}], "]"}]}], 
  "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAI", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAI", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAI", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", "\[Alpha]", "}"}]}], "]"}], ",", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", "+", "1"}], "}"}]}], "]"}], ",", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", "-", "1"}], "}"}]}], "]"}]}], "}"}]], "Input"],

Cell["\<\
T*IC are just like T*I , but used only for commentary in the 
recursion (TComment).\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "i_", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "i", ",", "\"\<0\>\""}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "i_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "\"\<0\>\"", ",", "i"}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFIC", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFIC", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVIC", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVIC", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVIC", "]"}], "->", "TVIC"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJIC", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJIC", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJIC", "]"}], "->", "TJIC"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJIC", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJIC", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJIC", "]"}], "->", "TJIC"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBIC", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAIC", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAIC", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAIC", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "i_", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "i", ",", "\"\<0\>\""}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "i_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\"", 
             ",", "\"\< \>\"", ",", "\"\<0\>\"", ",", "i"}], "}"}], "]"}]}], 
         "]"}], ",", "\n", "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TFR", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<F\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{
               RowBox[{"First", "[", 
                RowBox[{"{", "dpp", "}"}], "]"}], ",", "TraditionalForm"}], 
              "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
        "\t\t", 
        RowBox[{"TFR", "[", 
         RowBox[{"dpp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TVR", "[", 
      RowBox[{"dpp__", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", "\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"InterpretationBox", "@@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"SubsuperscriptBox", "[", 
           RowBox[{
            RowBox[{"StyleBox", "[", 
             RowBox[{"\"\<V\>\"", ",", 
              RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
              RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
            RowBox[{"RowBox", "@@", 
             RowBox[{"{", 
              RowBox[{"(", 
               RowBox[{"redblue", "/@", 
                RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
            RowBox[{"RowBox", "[", 
             RowBox[{"{", 
              RowBox[{"\"\<(\>\"", ",", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"{", "dpp", "}"}], "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
                "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
          "\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TVR", "]"}], "[", 
           RowBox[{"dpp", ",", 
            RowBox[{"{", "den", "}"}]}], "]"}], ",", 
          RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
      RowBox[{
       RowBox[{"Hold", "[", "TVR", "]"}], "->", "TVR"}]}], ")"}], "/;", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}], "&&", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"Last", "[", 
        RowBox[{"{", "dpp", "}"}], "]"}], "]"}], "=!=", "List"}]}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"TJR", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", "0", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", "\n", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"InterpretationBox", "@@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"SubsuperscriptBox", "[", 
            RowBox[{
             RowBox[{"StyleBox", "[", 
              RowBox[{"\"\<K\>\"", ",", 
               RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
               RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
             RowBox[{"RowBox", "@@", 
              RowBox[{"{", 
               RowBox[{"(", 
                RowBox[{"redblue", "/@", 
                 RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
             RowBox[{"RowBox", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<(\>\"", ",", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
                "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
           RowBox[{
            RowBox[{"Hold", "[", "TJR", "]"}], "[", 
            RowBox[{"d", ",", "0", ",", 
             RowBox[{"{", "den", "}"}]}], "]"}], ",", 
           RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
       RowBox[{
        RowBox[{"Hold", "[", "TJR", "]"}], "->", "TJR"}]}], ")"}], "/;", 
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}],
       "]"}]}], "\n", "\t", ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJR", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\t", 
  
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"InterpretationBox", "@@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"SubsuperscriptBox", "[", 
          RowBox[{
           RowBox[{"StyleBox", "[", 
            RowBox[{"\"\<J\>\"", ",", 
             RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
             RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
           RowBox[{"RowBox", "@@", 
            RowBox[{"{", 
             RowBox[{"(", 
              RowBox[{"redblue", "/@", 
               RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
           RowBox[{"RowBox", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<(\>\"", ",", 
              RowBox[{"ToBoxes", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", "dpp", "}"}], "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "TraditionalForm"}], 
               "]"}], ",", "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", 
         "\t\t", 
         RowBox[{
          RowBox[{"Hold", "[", "TJR", "]"}], "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", "den", "}"}]}], "]"}], ",", 
         RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/.", 
     RowBox[{
      RowBox[{"Hold", "[", "TJR", "]"}], "->", "TJR"}]}], ")"}], "/;", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"SubsuperscriptBox", "[", 
      RowBox[{
       RowBox[{"StyleBox", "[", 
        RowBox[{"\"\<B\>\"", ",", 
         RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
         RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
       RowBox[{"RowBox", "@@", 
        RowBox[{"{", 
         RowBox[{"(", 
          RowBox[{"redblue", "/@", 
           RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
       RowBox[{"RowBox", "[", 
        RowBox[{"{", 
         RowBox[{"\"\<(\>\"", ",", 
          RowBox[{"ToBoxes", "[", 
           RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", "\"\<)\>\""}], 
         "}"}], "]"}]}], "]"}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TBR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<B\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TBR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAR", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"TAR", "[", 
      RowBox[{"d_", ",", "pp___", ",", 
       RowBox[{"{", "den__", "}"}]}], "]"}], ",", "fmt_"}], "]"}], ":=", "\n",
    "\t", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"InterpretationBox", "@@", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SubsuperscriptBox", "[", 
         RowBox[{
          RowBox[{"StyleBox", "[", 
           RowBox[{"\"\<A\>\"", ",", 
            RowBox[{"SingleLetterItalics", "->", "False"}], ",", 
            RowBox[{"FontWeight", "->", "\"\<Bold\>\""}]}], "]"}], ",", 
          RowBox[{"RowBox", "@@", 
           RowBox[{"{", 
            RowBox[{"(", 
             RowBox[{"redblue", "/@", 
              RowBox[{"{", "den", "}"}]}], ")"}], "}"}]}], ",", 
          RowBox[{"RowBox", "[", 
           RowBox[{"{", 
            RowBox[{"\"\<(\>\"", ",", 
             RowBox[{"ToBoxes", "[", 
              RowBox[{"d", ",", "TraditionalForm"}], "]"}], ",", 
             "\"\<)\>\""}], "}"}], "]"}]}], "]"}], ",", "\n", "\t\t", 
        RowBox[{"TAR", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", "den", "}"}]}], "]"}], ",", 
        RowBox[{"Editable", "->", "True"}]}], "}"}]}], ")"}], "/;", 
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"StandardForm", ",", "TraditionalForm"}], "}"}], ",", "fmt"}], 
     "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n1", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n1", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n2", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n2", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "2"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n3", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n3", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "3"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n4", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n4", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "4"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"n5", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"n5", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[Nu]\>\"", ",", "5"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"ParD", "[", "i_", "]"}], "j_"], ",", "TraditionalForm"}], 
    "]"}], ":=", 
   RowBox[{"SubsuperscriptBox", "[", 
    RowBox[{"\"\<\[PartialD]\>\"", ",", "i", ",", "j"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"ParD", "[", "i_", "]"}], ",", "TraditionalForm"}], "]"}], "^=", 
   
   RowBox[{"SubscriptBox", "[", 
    RowBox[{"\"\<\[PartialD]\>\"", ",", "i"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"DPlus", ",", "TraditionalForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"SuperscriptBox", "[", 
      RowBox[{"\"\<d\>\"", ",", "\"\<+\>\""}], "]"}], ",", 
     RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}], ",", 
     RowBox[{"SingleLetterItalics", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{"TpD", "[", 
     RowBox[{"_", ",", "b_"}], "]"}], ",", "TraditionalForm"}], "]"}], ":=", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"b", ",", "TraditionalForm"}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Palettes", "Subsection",
 CellTags->"T1.3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$PutPalettes", "===", "True"}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"tobut", "[", "a_", "]"}], ":=", 
        RowBox[{"ButtonBox", "[", 
         RowBox[{
          RowBox[{"ToBoxes", "[", 
           RowBox[{"a", ",", "StandardForm"}], "]"}], ",", 
          RowBox[{"BaseStyle", "\[Rule]", "\"\<Paste\>\""}]}], "]"}]}], ";"}],
       ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Head", "[", "pal1", "]"}], "===", "NotebookObject"}], ",", 
         
         RowBox[{"NotebookClose", "[", "pal1", "]"}]}], "]"}], ";"}], ")"}], 
     " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"pal1", "=", 
        RowBox[{"NotebookPut", "[", 
         RowBox[{"Notebook", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Cell", "[", 
             RowBox[{
              RowBox[{"BoxData", "[", 
               RowBox[{"GridBox", "[", 
                RowBox[{
                 RowBox[{"Join", "[", 
                  RowBox[{
                   RowBox[{"Transpose", "[", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "[", 
                    SubsuperscriptBox["m", "j", "2"], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"j", ",", "5"}], "}"}]}], "]"}], ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "[", 
                    SubscriptBox["\[Nu]", "j"], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"j", ",", "5"}], "}"}]}], "]"}], ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "[", 
                    RowBox[{"Schiebe", "[", 
                    RowBox[{"i", ",", "\"\<+\>\""}], "]"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"i", ",", "5"}], "}"}]}], "]"}], ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"tobut", "[", 
                    RowBox[{"Schiebe", "[", 
                    RowBox[{"i", ",", "\"\<-\>\""}], "]"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"i", ",", "5"}], "}"}]}], "]"}]}], "}"}], "]"}], 
                   ",", 
                   RowBox[{"{", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{"PP", ",", 
                    SubscriptBox["\[CapitalDelta]", "\[Placeholder]"], ",", 
                    SubscriptBox["u", "\[Placeholder]"], ",", 
                    "\[ScriptCapitalC]"}], "}"}]}], "}"}]}], "]"}], ",", 
                 RowBox[{"RowSpacings", "\[Rule]", "0"}], ",", 
                 RowBox[{"ColumnSpacings", "\[Rule]", "0"}]}], "]"}], "]"}], 
              ",", "NotebookDefault"}], "]"}], "}"}], ",", 
           RowBox[{"Editable", "\[Rule]", "True"}], ",", 
           RowBox[{"WindowToolbars", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"WindowSize", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"Fit", ",", "Fit"}], "}"}]}], ",", 
           RowBox[{"WindowMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"WindowFrame", "\[Rule]", "\"\<Palette\>\""}], ",", 
           RowBox[{"WindowElements", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"WindowFrameElements", "\[Rule]", "\"\<CloseBox\>\""}], 
           ",", 
           RowBox[{"WindowClickSelect", "\[Rule]", "False"}], ",", 
           RowBox[{"ScrollingOptions", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\"\<PagewiseScrolling\>\"", "\[Rule]", "True"}], 
             "}"}]}], ",", 
           RowBox[{"ShowCellBracket", "\[Rule]", "False"}], ",", 
           RowBox[{"CellMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Enabled", "\[Rule]", "True"}], ",", 
           RowBox[{"CellOpen", "\[Rule]", "True"}], ",", 
           RowBox[{"ShowCellLabel", "\[Rule]", "False"}], ",", 
           RowBox[{"ShowCellTags", "\[Rule]", "False"}], ",", 
           RowBox[{"ImageMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Magnification", "\[Rule]", "2"}], ",", 
           RowBox[{"Visible", "\[Rule]", "True"}]}], "]"}], "]"}]}], ";"}], 
      ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"SetOptions", "[", 
        RowBox[{"pal1", ",", 
         RowBox[{"WindowMargins", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Automatic", ",", "0"}], "}"}]}], "}"}]}]}], "]"}], 
       ";"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"SetOptions", "[", 
        RowBox[{"pal1", ",", 
         RowBox[{"Visible", "\[Rule]", "True"}], ",", 
         RowBox[{"WindowSize", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}], ";"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Head", "[", "pal2", "]"}], "===", "NotebookObject"}], ",", 
         
         RowBox[{"NotebookClose", "[", "pal2", "]"}]}], "]"}], ";"}], ")"}], 
     " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"pal2", "=", 
        RowBox[{"NotebookPut", "[", 
         RowBox[{"Notebook", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Cell", "[", 
             RowBox[{
              RowBox[{"BoxData", "[", 
               RowBox[{"GridBox", "[", 
                RowBox[{
                 RowBox[{"JJJ", "=", 
                  RowBox[{"Join", "[", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"TFI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"], ",", 
                    SubscriptBox["\[Nu]", "5"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TVI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TJI", "[", 
                    RowBox[{
                    RowBox[{"d", "+", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TKI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TBI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TAI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    SubscriptBox["\[Nu]", "1"], "}"}]}], "]"}]}], "}"}]}], 
                    "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"tobut", "/@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"TFI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"], ",", 
                    SubscriptBox["\[Nu]", "5"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TVI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"], ",", 
                    SubscriptBox["\[Nu]", "4"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TJI", "[", 
                    RowBox[{"d", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TKI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"], ",", 
                    SubscriptBox["\[Nu]", "3"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TBI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    SubscriptBox["\[Nu]", "1"], ",", 
                    SubscriptBox["\[Nu]", "2"]}], "}"}]}], "]"}], ",", 
                    RowBox[{"TAI", "[", 
                    RowBox[{
                    RowBox[{"d", "-", "2"}], ",", 
                    RowBox[{"{", 
                    SubscriptBox["\[Nu]", "1"], "}"}]}], "]"}]}], "}"}]}], 
                    "}"}]}], "]"}]}], ",", 
                 RowBox[{"RowSpacings", "\[Rule]", "0"}], ",", 
                 RowBox[{"ColumnSpacings", "\[Rule]", "0"}]}], "]"}], "]"}], 
              ",", "NotebookDefault"}], "]"}], "}"}], ",", 
           RowBox[{"Editable", "\[Rule]", "True"}], ",", 
           RowBox[{"WindowToolbars", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"WindowSize", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"FitAll", ",", "FitAll"}], "}"}]}], ",", 
           RowBox[{"WindowMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"WindowFrame", "\[Rule]", "\"\<Palette\>\""}], ",", 
           RowBox[{"WindowElements", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"WindowFrameElements", "\[Rule]", "\"\<CloseBox\>\""}], 
           ",", 
           RowBox[{"WindowClickSelect", "\[Rule]", "False"}], ",", 
           RowBox[{"ScrollingOptions", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\"\<PagewiseScrolling\>\"", "\[Rule]", "True"}], 
             "}"}]}], ",", 
           RowBox[{"ShowCellBracket", "\[Rule]", "False"}], ",", 
           RowBox[{"CellMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Enabled", "\[Rule]", "True"}], ",", 
           RowBox[{"CellOpen", "\[Rule]", "True"}], ",", 
           RowBox[{"ShowCellLabel", "\[Rule]", "False"}], ",", 
           RowBox[{"ShowCellTags", "\[Rule]", "False"}], ",", 
           RowBox[{"ImageMargins", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Magnification", "\[Rule]", "2"}], ",", 
           RowBox[{"Visible", "\[Rule]", "True"}]}], "]"}], "]"}]}], ";"}], 
      ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"SetOptions", "[", 
        RowBox[{"pal2", ",", 
         RowBox[{"WindowMargins", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Automatic", ",", "15"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Automatic", ",", "4"}], "}"}]}], "}"}]}]}], "]"}], 
       ";"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"SetOptions", "[", 
        RowBox[{"pal2", ",", 
         RowBox[{"Visible", "\[Rule]", "True"}], ",", 
         RowBox[{"WindowSize", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}], ";"}], ")"}], " ", 
     RowBox[{"SetOptions", "[", 
      RowBox[{"pal2", ",", 
       RowBox[{"WindowSize", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Fit", ",", "Fit"}], "}"}]}]}], "]"}]}]}], "]"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["OperatorApply", "Subsection",
 CellTags->"T1.3.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyF", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TFI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TFI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TFI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TFI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TFI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyV", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TVI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TVI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TVI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TVI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TVI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyJ", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TJI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TJI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TJI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TJI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TJI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyK", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TKI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyK", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TKI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TKI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TKI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyB", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TBI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TBI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TBI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TBI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TBI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"OperatorApplyA", "[", 
    RowBox[{"x_", "==", "y_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "sch", "}"}], ",", "\n", "\t\t", 
     RowBox[{
      RowBox[{"sch", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<+\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TAI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TAI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "+", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}], ",", "\n", "\t\t\t\t", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"Schiebe", "[", 
             RowBox[{"i_", ",", "\"\<-\>\""}], "]"}], "ni_."], " ", 
           RowBox[{"TAI", "[", 
            RowBox[{"d_", ",", 
             RowBox[{"{", "n15__", "}"}]}], "]"}]}], ":>", "\n", "\t\t\t\t\t", 
          RowBox[{"TAI", "[", 
           RowBox[{"d", ",", 
            RowBox[{"ReplacePart", "[", 
             RowBox[{
              RowBox[{"{", "n15", "}"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"{", "n15", "}"}], "[", 
                RowBox[{"[", "i", "]"}], "]"}], "-", "ni"}], ",", "i"}], 
             "]"}]}], "]"}]}]}], "\n", "\t\t\t", "}"}]}], ";", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Collect", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Expand", "[", "#", "]"}], "//.", "sch"}], ",", 
           RowBox[{"TAI", "[", "__", "]"}], ",", "Factor"}], "]"}], "&"}], 
        ",", "\n", "\t\t\t", 
        RowBox[{"x", "==", "y"}]}], "]"}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MassDerivative", "Subsection",
 CellTags->"T1.3.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MassDerivative", "[", 
    RowBox[{
     RowBox[{"STLI", "[", 
      RowBox[{"any__", ",", 
       RowBox[{"{", "\n", "       ", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"al_", ",", "m1_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"be_", ",", "m2_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"ga_", ",", "m3_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"de_", ",", "m4_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"ep_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "d1_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d2_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d3_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d4_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"0", ",", "d5_"}], "}"}]}], "}"}]}], "]"}], 
   StyleBox[" ",
    FontWeight->"Plain"], 
   StyleBox[":=",
    FontWeight->"Plain"], "\n", 
   RowBox[{
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"Pochhammer", "[", 
        RowBox[{"al", ",", "d1"}], "]"}], " ", 
       RowBox[{"Pochhammer", "[", 
        RowBox[{"be", ",", "d2"}], "]"}], " ", 
       RowBox[{"Pochhammer", "[", 
        RowBox[{"ga", ",", "d3"}], "]"}]}], "*", "\n", "  ", 
      RowBox[{"Pochhammer", "[", 
       RowBox[{"de", ",", "d4"}], "]"}], " ", 
      RowBox[{"Pochhammer", "[", 
       RowBox[{"ep", ",", "d5"}], "]"}]}], " ", ")"}], " ", "*", "\n", " ", 
    RowBox[{"STLI", "[", 
     RowBox[{"any", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"al", "+", "d1"}], ",", "m1"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"be", "+", "d2"}], ",", "m2"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"ga", "+", "d3"}], ",", "m3"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"de", "+", "d4"}], ",", "m4"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"ep", "+", "d5"}], ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}],
   ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarasovT", "Subsection",
 CellTags->"T1.3.5.1"],

Cell["\<\
TarasovT[abrs__, qq_ /; Head[qq] =!= List] := 
  TarasovT[abrs, qq, {\"g\", \"g\", \"g\", \"g\", \"g\"}]\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarasovT", "[", 
    RowBox[{"r_", ",", " ", "s_", ",", " ", "qq_", ",", " ", 
     RowBox[{"{", 
      RowBox[{
      "alp1_", ",", "alp2_", ",", " ", "alp3_", ",", "alp4_", ",", "alp5_"}], 
      "}"}]}], "]"}], " ", ":=", "\n", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", " ", 
      RowBox[{
      "al1", ",", "al2", ",", "al3", ",", "al4", ",", "al5", ",", "alrul", 
       ",", "new", ",", "Q1", ",", "Q11", ",", "Q12", ",", "Q2", ",", "Q22", 
       ",", "dum1", ",", "dum2"}], "}"}], ",", "\n", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp1", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al1", " ", "=", " ", "0"}]}], "]"}], ";", " ", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp2", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al2", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n",
                     " ", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp3", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al3", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp4", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al4", " ", "=", " ", "0"}]}], "]"}], ";", " ", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"alp5", " ", "===", " ", "0"}], ",", " ", 
                    RowBox[{"al5", " ", "=", " ", "0"}]}], "]"}], ";"}], "\n",
                     " ", "alrul"}], " ", "=", " ", 
                    RowBox[{"Select", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"al1", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "1", "]"}]}]}], ",", " ", 
                    RowBox[{"al2", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "2", "]"}]}]}], ",", " ", 
                    RowBox[{"al3", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "3", "]"}]}]}], ",", "\n", 
                    "                 ", 
                    RowBox[{"al4", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "4", "]"}]}]}], ",", " ", 
                    RowBox[{"al5", ":>", 
                    RowBox[{"I", " ", 
                    RowBox[{"ParD", "[", "5", "]"}]}]}]}], "}"}], ",", " ", 
                    RowBox[{
                    RowBox[{"!", 
                    RowBox[{"MatchQ", "[", 
                    RowBox[{"#", ",", " ", 
                    RowBox[{"0", ":>", "_"}]}], "]"}]}], "&"}]}], "]"}]}], 
                    ";"}], "\n", " ", "Q1"}], " ", "=", " ", 
                    RowBox[{
                    RowBox[{"al3", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al4", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al2", " ", "al3"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al4"}]}]}], ";"}], "\n", " ", "Q2"}],
                    " ", "=", " ", 
                   RowBox[{
                    RowBox[{"al4", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al5"}], " ", "+", " ", 
                    RowBox[{"al1", " ", "al4"}], " ", "+", " ", 
                    RowBox[{"al3", " ", "al4"}]}]}], ";"}], "\n", " ", 
                 "Q11"}], " ", "=", 
                RowBox[{
                 RowBox[{"1", "/", 
                  RowBox[{"(", 
                   RowBox[{"-", "4"}], ")"}]}], " ", 
                 RowBox[{"(", " ", 
                  RowBox[{"al2", " ", "+", " ", "al4", " ", "+", " ", "al5"}],
                   " ", ")"}]}]}], ";"}], "\n", " ", "Q22"}], " ", "=", 
             RowBox[{
              RowBox[{"1", "/", 
               RowBox[{"(", 
                RowBox[{"-", "4"}], ")"}]}], " ", 
              RowBox[{"(", " ", 
               RowBox[{"al1", " ", "+", " ", "al3", " ", "+", " ", "al5"}], 
               " ", ")"}]}]}], ";"}], "\n", " ", "Q12"}], " ", "=", 
          RowBox[{
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"-", "2"}], ")"}]}], " ", "al5"}]}], ";"}], "\n", "\t\t",
         "new"}], "=", "\n", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"I", "^", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "r"}], "-", "s"}], ")"}]}], 
         RowBox[{"(", " ", 
          RowBox[{"D", "[", " ", 
           RowBox[{
            RowBox[{"Exp", "[", " ", 
             RowBox[{"I", " ", "qq", " ", 
              RowBox[{"(", "\n", "    ", 
               RowBox[{
                RowBox[{"be1", " ", "Q1"}], " ", "+", " ", 
                RowBox[{"be2", " ", "Q2"}], " ", "+", 
                RowBox[{
                 RowBox[{"be1", "^", "2"}], " ", "Q11"}], " ", "+", 
                RowBox[{
                 RowBox[{"be2", "^", "2"}], " ", "Q22"}], " ", "+", " ", 
                RowBox[{"be1", " ", "be2", " ", "Q12"}]}], "\n", 
               "                                 ", ")"}], " ", "rho"}], " ", 
             "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"be1", ",", "r"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"be2", ",", "s"}], "}"}]}], "\n", "\t\t\t\t", "]"}], " ",
           ")"}]}], "/.", "\n", " ", 
        RowBox[{"(", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"alrul", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"be1", " ", ":>", " ", "0"}], ",", 
             RowBox[{"be2", " ", ":>", " ", "0"}], ",", " ", 
             RowBox[{"rho", " ", ":>", " ", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"Pi", "^", "2"}]}], "/", 
                RowBox[{"Pi", "^", "2"}]}], " ", "DPlus"}]}]}], " ", "}"}]}], 
          "]"}], ")"}]}]}], ";", 
      RowBox[{
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"#", ",", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#", ",", "ParD"}], "]"}], "&&", 
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#", ",", "DPlus"}], "]"}]}], ")"}], "&"}]}], "]"}],
             " ", 
            RowBox[{"TpD", "[", 
             RowBox[{"qq", ",", 
              RowBox[{"Select", "[", 
               RowBox[{"#", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"!", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "ParD"}], "]"}], "&&", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "DPlus"}], "]"}]}], ")"}]}], ")"}], 
                 "&"}]}], "]"}]}], "]"}]}], ")"}], "&"}], ",", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Expand", "[", 
            RowBox[{"new", ",", "ParD"}], "]"}], "+", " ", 
           RowBox[{"dum1", " ", "dum2"}]}], ")"}]}], "\n", "\t\t", "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"dum1", ":>", "0"}], ",", 
         RowBox[{"dum2", ":>", "0"}]}], "}"}]}]}]}], "\n", "\t\t", "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{"TarasovT", "[", 
    RowBox[{"r_", ",", " ", "s_", ",", " ", "pp_", ",", " ", 
     RowBox[{"dp_", "/;", 
      RowBox[{
       RowBox[{"Head", "[", "dp", "]"}], "=!=", "List"}]}]}], "]"}], " ", ":=",
    " ", 
   RowBox[{"TarasovT", "[", 
    RowBox[{"r", ",", "s", ",", "pp", ",", "dp", ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<g\>\"", ",", "\"\<g\>\"", ",", "\"\<g\>\"", ",", "\"\<g\>\"", ",", 
       "\"\<g\>\""}], "}"}]}], "]"}]}]}]], "Input"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", " ", 
      SubscriptBox["q", "1"]}], ")"}], "a"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"\[CapitalDelta]", " ", 
      SubscriptBox["q", "2"]}], ")"}], "b"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"p", " ", 
      SubscriptBox["q", "1"]}], ")"}], "r"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"p", " ", 
      SubscriptBox["q", "2"]}], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"TarasovT", "[", 
     RowBox[{
     "a_", ",", "b_", ",", "r_", ",", "s_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
       "alp1_", ",", "alp2_", ",", "alp3_", ",", "alp4_", ",", "alp5_"}], 
       "}"}]}], "]"}], ":=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "al1", ",", "al2", ",", "al3", ",", "al4", ",", "al5", ",", "alrul", 
        ",", "new", ",", "Q1", ",", "Q11", ",", "Q12", ",", "Q2", ",", "Q22", 
        ",", "dum1", ",", "dum2"}], "}"}], ",", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp1", "===", "0"}], ",", 
         RowBox[{"al1", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp2", "===", "0"}], ",", 
         RowBox[{"al2", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp3", "===", "0"}], ",", 
         RowBox[{"al3", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp4", "===", "0"}], ",", 
         RowBox[{"al4", "=", "0"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"alp5", "===", "0"}], ",", 
         RowBox[{"al5", "=", "0"}]}], "]"}], ";", 
       RowBox[{"alrul", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"al1", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "1", "]"}]}]}], ",", 
            RowBox[{"al2", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "2", "]"}]}]}], ",", 
            RowBox[{"al3", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "3", "]"}]}]}], ",", 
            RowBox[{"al4", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "4", "]"}]}]}], ",", 
            RowBox[{"al5", "\[RuleDelayed]", 
             RowBox[{"I", " ", 
              RowBox[{"ParD", "[", "5", "]"}]}]}]}], "}"}], ",", 
          "\[InvisibleSpace]", 
          RowBox[{
           RowBox[{"!", 
            RowBox[{"(", 
             RowBox[{"MatchQ", "[", 
              RowBox[{"#1", ",", 
               RowBox[{"0", "\[RuleDelayed]", "_"}]}], "]"}], ")"}]}], 
           "&"}]}], "]"}]}], ";", 
       RowBox[{"Q1", "=", 
        RowBox[{
         RowBox[{"al3", " ", "al5"}], "+", 
         RowBox[{"al4", " ", "al5"}], "+", 
         RowBox[{"al2", " ", "al3"}], "+", 
         RowBox[{"al3", " ", "al4"}]}]}], ";", 
       RowBox[{"Q2", "=", 
        RowBox[{
         RowBox[{"al4", " ", "al5"}], "+", 
         RowBox[{"al3", " ", "al5"}], "+", 
         RowBox[{"al1", " ", "al4"}], "+", 
         RowBox[{"al3", " ", "al4"}]}]}], ";", 
       RowBox[{"Q11", "=", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         RowBox[{"(", 
          RowBox[{"al2", "+", "al4", "+", "al5"}], ")"}]}]}], ";", 
       RowBox[{"Q22", "=", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
         RowBox[{"(", 
          RowBox[{"al1", "+", "al3", "+", "al5"}], ")"}]}]}], ";", 
       RowBox[{"Q12", "=", 
        RowBox[{"-", 
         FractionBox["al5", "2"]}]}], ";", 
       RowBox[{"new", "=", 
        RowBox[{
         RowBox[{
          SuperscriptBox["I", 
           RowBox[{
            RowBox[{"-", "r"}], "-", "s", "-", "a", "-", "b"}]], " ", 
          RowBox[{
           SubscriptBox["\[PartialD]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"be1", ",", "r"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"be2", ",", "s"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ga1", ",", "a"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ga2", ",", "b"}], "}"}]}]], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "pp"}], "+", 
                  RowBox[{"ga1", " ", "dp"}]}], ")"}], " ", "Q1"}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be2", " ", "pp"}], "+", 
                  RowBox[{"ga2", " ", "dp"}]}], ")"}], " ", "Q2"}], "+", 
               RowBox[{"be1", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "pp"}], "+", 
                  RowBox[{"2", " ", "ga1", " ", "dp"}]}], ")"}], " ", "Q11"}],
                "+", 
               RowBox[{"be2", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be2", " ", "pp"}], "+", 
                  RowBox[{"2", " ", "ga2", " ", "dp"}]}], ")"}], " ", "Q22"}],
                "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"be1", " ", "be2", " ", "pp"}], "+", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"be1", " ", "ga2"}], "+", 
                    RowBox[{"be2", " ", "ga1"}]}], ")"}], " ", "dp"}]}], 
                 ")"}], " ", "Q12"}]}], ")"}], " ", "rho"}], "]"}]}]}], "/.", 
         
         RowBox[{"Join", "[", 
          RowBox[{"alrul", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"be1", "\[RuleDelayed]", "0"}], ",", 
             RowBox[{"be2", "\[RuleDelayed]", "0"}], ",", 
             RowBox[{"ga1", ":>", "0"}], ",", 
             RowBox[{"ga2", ":>", "0"}], ",", 
             RowBox[{"rho", "\[RuleDelayed]", 
              RowBox[{"-", 
               FractionBox[
                RowBox[{
                 SuperscriptBox["\[Pi]", "2"], " ", "DPlus"}], 
                SuperscriptBox["\[Pi]", "2"]]}]}]}], "}"}]}], "]"}]}]}], ";", 
       
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"#1", ",", 
              RowBox[{
               RowBox[{
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#1", ",", "ParD"}], "]"}], "&&", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#1", ",", "DPlus"}], "]"}]}], "&"}]}], "]"}], " ", 
            RowBox[{"TpD", "[", 
             RowBox[{"pp", ",", 
              RowBox[{"Select", "[", 
               RowBox[{"#1", ",", "\[InvisibleSpace]", 
                RowBox[{
                 RowBox[{"!", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#1", ",", "ParD"}], "]"}], "&&", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#1", ",", "DPlus"}], "]"}]}], ")"}]}], "&"}]}], 
               "]"}]}], "]"}]}], "&"}], ")"}], "/@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Expand", "[", "new", "]"}], "+", 
           RowBox[{"dum1", " ", "dum2"}]}], ")"}]}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"dum1", "\[RuleDelayed]", "0"}], ",", 
          RowBox[{"dum2", "\[RuleDelayed]", "0"}]}], "}"}]}]}]}], "]"}]}], 
   ";"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["ApplyTarasovT", "Subsection",
 CellTags->"T1.3.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ApplyTarasovT", "[", 
   RowBox[{"tij_", ",", " ", "exp_"}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "dump", "}"}], ",", "\n", "\t\t", 
    RowBox[{
     RowBox[{"dump", "=", 
      RowBox[{"Product", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ParD", "[", "j", "]"}], "^", "dummy"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "5"}], "}"}]}], "]"}]}], ";", "\n", "\t\t", 
     RowBox[{
      RowBox[{"Expand", "[", 
       RowBox[{"tij", " ", "exp"}], "]"}], "/.", "\n", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"STLI", "[", 
          RowBox[{"de_", ",", "pp_", ",", "pe__List"}], "]"}], " ", 
         RowBox[{"TpD", "[", 
          RowBox[{"tpp_", ",", 
           RowBox[{
            RowBox[{"DPlus", "^", "i_."}], " ", "t_"}]}], "]"}]}], ")"}], ":>",
        "\n", "\t", 
       RowBox[{"MassDerivative", "[", 
        RowBox[{
         RowBox[{"STLI", "[", " ", 
          RowBox[{
           RowBox[{"de", "/.", 
            RowBox[{"de", ":>", 
             RowBox[{"(", 
              RowBox[{"de", " ", "+", " ", 
               RowBox[{"2", " ", "i"}]}], ")"}]}]}], ",", "pp", ",", "pe"}], 
          "]"}], ",", "\n", "\t\t\t\t\t\t\t", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"t", " ", "dump"}], " ", "/.", 
            RowBox[{"Times", "->", "List"}]}], "/.", 
           RowBox[{
            RowBox[{"Power", "[", 
             RowBox[{"_", ",", "n_"}], "]"}], ":>", 
            RowBox[{"{", 
             RowBox[{"0", ",", "n"}], "}"}]}]}], "/.", 
          RowBox[{"dummy", "->", "0"}]}]}], "]"}]}]}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Misc", "Subsection",
 CellTags->"T1.3.7.1"],

Cell[BoxData[
 RowBox[{"Prefactor1", "/:", "\t\t", 
  RowBox[{
   RowBox[{"Prefactor1", "[", "a_", "]"}], "^", "n_"}], ":=", 
  RowBox[{"Prefactor1", "[", 
   RowBox[{"a", "^", "n"}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell["Colors", "Subsubsection",
 CellTags->"T1.3.7.1.1"],

Cell[TextData[{
 StyleBox["TFI",
  FontColor->RGBColor[1, 0.6, 0]],
 " ",
 StyleBox["TVI ",
  FontColor->RGBColor[0.2, 0.4, 0.8]],
 StyleBox["TFR ",
  FontColor->RGBColor[0, 0.6, 0]],
 StyleBox["TVR ",
  FontColor->RGBColor[0.2, 0.8, 1]],
 StyleBox["MakeFun ",
  FontColor->RGBColor[1, 0.6, 0.6]],
 StyleBox["Cayley ",
  FontColor->RGBColor[1, 0, 0.4]],
 StyleBox["CayleyD ",
  FontColor->RGBColor[1, 0.2, 1]],
 StyleBox["Cayleyu",
  FontColor->RGBColor[0.8, 0.8, 0.4]],
 StyleBox[" ",
  FontColor->RGBColor[1, 0.2, 1]],
 StyleBox["IFF ",
  FontColor->RGBColor[0.6, 0.4, 1]],
 StyleBox["ToArgs ",
  FontColor->RGBColor[0.2, 0.6, 1]],
 StyleBox["Block",
  FontSize->16,
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0.2, 0.2]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Block", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Block\>\"", ",", 
     RowBox[{"FontSize", "\[Rule]", "16"}], ",", 
     RowBox[{"FontWeight", "\[Rule]", "\"\<Bold\>\""}], ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.2", ",", "0.2"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Cayley", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Cayley\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0", ",", "0.4"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"CayleyD", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<CayleyD\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.2", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Cayleyu", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<Cayleyu\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.8", ",", "0.8", ",", "0.4"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"IFF", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<IFF\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.6", ",", "0.4", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"MakeFun", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<MakeFun\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.6", ",", "0.6"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TFI", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TFI\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"1", ",", "0.6", ",", "0"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TFR", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TFR\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", "0.6", ",", "0"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"ToArgs", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<ToArgs\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.2", ",", "0.6", ",", "1"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"TVI", ",", "StandardForm"}], "]"}], ":=", 
   RowBox[{"StyleBox", "[", 
    RowBox[{"\"\<TVI\>\"", ",", 
     RowBox[{"FontColor", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.2", ",", "0.4", ",", "0.8"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"TVR", ",", "StandardForm"}], "]"}], ":=", 
  RowBox[{"StyleBox", "[", 
   RowBox[{"\"\<TVR\>\"", ",", 
    RowBox[{"FontColor", "\[Rule]", 
     RowBox[{"RGBColor", "[", 
      RowBox[{"0.2", ",", "0.8", ",", "1"}], "]"}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PQ", "Subsubsection",
 CellTags->"T1.3.7.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PQ", "[", 
    RowBox[{"_Integer", "?", "Positive"}], "]"}], " ", ":=", " ", "True"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PNQ", "Subsubsection",
 CellTags->"T1.3.7.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PNQ", "[", "0", "]"}], "=", "True"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"PNQ", "[", 
    RowBox[{"_Integer", "?", "Positive"}], "]"}], " ", ":=", " ", "True"}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FactorC", "Subsubsection",
 CellTags->"T1.3.7.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FactorC", "[", "z_", "]"}], ":=", 
  RowBox[{
   RowBox[{"FactorC", "[", "z", "]"}], "=", 
   RowBox[{"Factor", "[", "z", "]"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TFI", "Subsubsection",
 CellTags->"T1.3.7.5.1"],

Cell["\<\

TFI[depp__,{a___,b_Integer,c___}]:=TFI[depp,{a,{b,0},c}];
TFI[depp__,{a___,{0,m_/;m=!=0},b___}]:=TFI[depp,{a,{0,0},b}];
TFI[a__,{0,0},b__List]:=TFI[a,b];
TFI[d_,pp_,dp_,{0,0},{x1_,x2_,x3_,x4_,x5_},list_List]:=
    TFI[d,pp,{x1,x2,x3,x4,x5},list];
TFI[d_,pp_,dp_,{a_,b_},{0,0,0,0,0},list_List]:=TFI[d,pp,dp,{a,b},list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,{0,0,0,0,0},list_List]:=TFI[d,pp,list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,list_List]:=TFI[d,pp,list];
TFI[d_,pp_,dp_/;Head[dp]=!=List,{x1_,x2_,x3_,x4_,x5_},list_List]:=
    TFI[d,pp,{x1,x2,x3,x4,x5},list];
TFI[__,{{_,_},{0,0},{_,_},{0,0},{_,0}}]:=0;
TFI[__,{{0,0},{_,_},{0,0},{_,_},{_,0}}]:=0;
TFI[__,{{0,0},{_,_},{_,0},{_,_},{0,0}}]:=0;
TFI[__,{{_,0},{_,_},{0,0},{_,_},{0,0}}]:=0;
TFI[__,{{_,_},{_,0},{_,_},{0,0},{0,0}}]:=0;
TFI[__,{{_,_},{0,0},{_,_},{_,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{0,0},{_,0},{_,_}}]:=0;
TFI[__,{{0,0},{0,0},{_,0},{0,0},{_,_}}]:=0;
TFI[__,{{0,0},{_,0},{0,0},{0,0},{_,_}}]:=0;
TFI[__,{{_,0},{0,0},{0,0},{0,0},{_,_}}]:=0;
TFI[__,{{_,_},{_,0},{0,0},{0,0},{0,0}}]:=0;
TFI[__,{{_,0},{_,_},{0,0},{0,0},{0,0}}]:=0;
TFI[__,{{_,_},{0,0},{0,0},{_,0},{0,0}}]:=0;
TFI[__,{{_,0},{0,0},{0,0},{_,_},{0,0}}]:=0;
TFI[__,{{0,0},{_,0},{_,_},{0,0},{0,0}}]:=0;
TFI[__,{{0,0},{_,_},{_,0},{0,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{_,_},{_,0},{0,0}}]:=0;
TFI[__,{{0,0},{0,0},{_,0},{_,_},{0,0}}]:=0;

If[$Notebooks,

mbt[z_] := ToBoxes[z, TraditionalForm];
  redblue[z_ /; Head[z] =!= Plus] := mbt[z];
  redblue[(z_) - 1] := StyleBox[mbt[z], FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_Subscript) - 2] :=
   StyleBox[SubscriptBox[OverscriptBox[z[[1]], \"_\"], z[[2]]],
    FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) - 2] :=
   StyleBox[OverscriptBox[mbt[z], \"_\"], FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) - 3] :=
   StyleBox[UnderscriptBox[OverscriptBox[mbt[z], \"_\"], \"_\"],
    FontColor -> RGBColor[1, 0, 0]];
  redblue[(z_) + 1] := StyleBox[mbt[z], FontColor -> RGBColor[0, 0, 1]];
  redblue[(z_) + 2] :=
   StyleBox[OverscriptBox[mbt[z], \"_\"], FontColor -> RGBColor[0, 0, 1]];
  redblue[(z_Subscript) + 2] :=
   StyleBox[SubscriptBox[OverscriptBox[z[[1]], \"_\"], z[[2]]],
    FontColor -> RGBColor[0, 0, 1]];

  TFI/: MakeBoxes[TFI[d_, pp_, {ur__}, {den__}], TraditionalForm] :=
    InterpretationBox @@
      {SubsuperscriptBox[StyleBox[\"F\", Rule[SingleLetterItalics, False],
         Rule[FontWeight, \"Bold\"]], RowBox @@ {redblue /@ {den}},
        RowBox[{\"(\", ToBoxes[d, TraditionalForm], \")\", \" \", ur}]],
       TFI[d, pp, {ur}, {den}], Editable -> True} /;
     MemberQ[{StandardForm, TraditionalForm}, TraditionalForm];

  TFI/: MakeBoxes[TFI[d_, pp_, dp_, any__, {den__}], TraditionalForm] :=
    (InterpretationBox @@
       {SubsuperscriptBox[StyleBox[\"F\",
          Rule[SingleLetterItalics, False], Rule[FontWeight, \"Bold\"]],
         RowBox @@ {redblue /@ {den}},
         RowBox[{\"(\", ToBoxes[d, TraditionalForm], \")\", \" \",
           Sequence @@ Flatten[{any}]}]], TFI[d, pp, dp, any, {den}],
        Editable -> True}) /; Head[dp] =!= List;

TFI/:
   MakeBoxes[TFI[dpp__, {den__}], TraditionalForm] :=
    InterpretationBox @@
      {SubsuperscriptBox[StyleBox[\"F\", Rule[SingleLetterItalics, False],
         Rule[FontWeight, \"Bold\"]], RowBox @@ {redblue /@ {den}},
        RowBox[{\"(\", ToBoxes[First[{dpp}], TraditionalForm], \")\"}]],
       TFI[dpp, {den}], Editable -> True} /;
     MemberQ[{StandardForm, TraditionalForm}, TraditionalForm] &&
      Head[Last[{dpp}]] =!= List;

];\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TJI", "Subsubsection",
 CellTags->"T1.3.7.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"de_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_Integer", ",", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3_Integer", ",", "m3_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}]}], "}"}]}], "]"}], "=", 
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"Reverse", "[", 
        RowBox[{"Sort", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"n1", ",", "m1"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n2", ",", "m2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n3", ",", "m3"}], "}"}]}], "}"}], "]"}], "]"}]}], 
      "]"}]}], ")"}], "/;", "\[InvisibleSpace]", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}], "]"}], 
     ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"TJI", "[", 
  RowBox[{"d", ",", "pp", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "m1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TBI", "Subsubsection",
 CellTags->"T1.3.7.7.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["TBI",
    FontColor->RGBColor[1, 0.6, 0]], "[", 
   RowBox[{"de_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_Integer", ",", "m2_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"TJI", "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}]}], "}"}]}], "]"}], "=", 
     RowBox[{
      StyleBox["TBI",
       FontColor->RGBColor[1, 0.6, 0]], "[", 
      RowBox[{"de", ",", "pp", ",", 
       RowBox[{"Reverse", "@", 
        RowBox[{"Sort", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"n1", ",", "m1"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"n2", ",", "m2"}], "}"}]}], "}"}]}]}]}], "]"}]}], ")"}], "/;",
    "\[InvisibleSpace]", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"OrderedQ", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}], "]"}], 
     ")"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TAI", "Subsubsection",
 CellTags->"T1.3.7.8.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["TAI",
     FontColor->RGBColor[1, 0.6, 0]], "[", 
    RowBox[{"de_", ",", "pp_", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1_Integer", ",", "0"}], "}"}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Definitions and notation", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalDelta]\[ScriptCapitalC]", "=", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], 
    RowBox[{"Det", "[", " ", 
     RowBox[{"(", GridBox[{
        {"0", "1", "1", "1", "1"},
        {"1", "0", 
         SubscriptBox["\[Mu]", "6"], 
         SubscriptBox["\[Mu]", "4"], 
         SubscriptBox["\[Mu]", "3"]},
        {"1", 
         SubscriptBox["\[Mu]", "6"], "0", 
         SubscriptBox["\[Mu]", "2"], 
         SubscriptBox["\[Mu]", "1"]},
        {"1", 
         SubscriptBox["\[Mu]", "4"], 
         SubscriptBox["\[Mu]", "2"], "0", 
         SubscriptBox["\[Mu]", "5"]},
        {"1", 
         SubscriptBox["\[Mu]", "3"], 
         SubscriptBox["\[Mu]", "1"], 
         SubscriptBox["\[Mu]", "5"], "0"}
       }], ")"}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkli", " ", "=", " ", 
   RowBox[{"{", "\n", "\t\t", 
    RowBox[{
     RowBox[{"\[ScriptCapitalC]", "->", "\[CapitalDelta]\[ScriptCapitalC]"}], 
     ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", "i_"], ":>", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["\[Mu]", "i"]], "\[CapitalDelta]\[ScriptCapitalC]"}]}], 
     ",", "\n", "\t\t", 
     RowBox[{
      SubscriptBox["\[Mu]", "6"], ":>", "PP"}], ",", 
     RowBox[{
      SubscriptBox["\[Mu]", "i_"], ":>", 
      RowBox[{
       RowBox[{"ToExpression", "[", 
        RowBox[{"\"\<m\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "^", "2"}]}], ",", 
     "\n", "\t\t", 
     RowBox[{
      SubscriptBox["m", "1"], ":>", 
      SqrtBox[
       SuperscriptBox["m1", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "2"], ":>", 
      SqrtBox[
       SuperscriptBox["m2", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "3"], ":>", 
      SqrtBox[
       SuperscriptBox["m3", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "4"], ":>", 
      SqrtBox[
       SuperscriptBox["m4", "2"]]}], ",", 
     RowBox[{
      SubscriptBox["m", "5"], ":>", 
      SqrtBox[
       SuperscriptBox["m5", "2"]]}], ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       SubsuperscriptBox["\[Mu]", "i", "2"], "+", 
       SubsuperscriptBox["\[Mu]", "j", "2"], "+", 
       SubsuperscriptBox["\[Mu]", "k", "2"], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Mu]", "i"], 
           SubscriptBox["\[Mu]", "j"]}], "+", 
          RowBox[{
           SubscriptBox["\[Mu]", "j"], 
           SubscriptBox["\[Mu]", "k"]}], "+", 
          RowBox[{
           SubscriptBox["\[Mu]", "k"], 
           SubscriptBox["\[Mu]", "i"]}]}], ")"}]}]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       SubscriptBox["\[Mu]", "i"], "-", 
       SubscriptBox["\[Mu]", "j"], "-", 
       SubscriptBox["\[Mu]", "k"]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
      RowBox[{
       RowBox[{"3", " ", "d"}], " ", "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"j", "=", "1"}], "i"], 
          SubscriptBox["\[Nu]", "j"]}], ")"}]}], "-", "2"}]}], ",", "\n", 
     "\t", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}]}], "}"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "Define symmetry property for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"i", ",", "j", ",", "k"}]], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"i_", ",", "j_", ",", "k_"}]], ":=", 
  RowBox[{
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"Sequence", "@@", 
     RowBox[{"Sort", "[", 
      RowBox[{"{", 
       RowBox[{"i", ",", "j", ",", "k"}], "}"}], "]"}]}]], "/;", 
   RowBox[{"!", 
    RowBox[{"OrderedQ", "[", 
     RowBox[{"{", 
      RowBox[{"i", ",", "j", ",", "k"}], "}"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 SubscriptBox["\[CapitalDelta]", 
  RowBox[{"3", ",", "1", ",", "2"}]]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["ToArgs",
  FontColor->RGBColor[0.2, 0.6, 1]]], "Subsection",
 CellTags->"T1.4.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["ToArgs",
     FontColor->RGBColor[0.2, 0.6, 1]], "[", "z_", "]"}], ":=", 
   RowBox[{"Expand", "[", 
    RowBox[{"z", "//.", "checkli"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ToArgs", "[", 
   SubscriptBox["\[CapitalSigma]", "4"], "]"}], "//", "InputForm"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["Cayley",
  FontColor->RGBColor[1, 0, 0.4]]], "Subsection",
 CellTags->"T1.4.2.1"],

Cell[TextData[{
 "This is (40).  ( p = ",
 Cell[BoxData[
  FormBox[
   SqrtBox[
    SuperscriptBox["p", "2"]], TraditionalForm]]],
 ")"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{"hCayley", "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "2"],
        m2], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], ":", "_"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "5"],
        m5], ":", "_"}], ",", "p_"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{"hCayley", "[", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ",", 
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ",", 
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ",", 
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ",", 
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ",", "p"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       RowBox[{
        RowBox[{
         StyleBox["ToArgs",
          FontColor->RGBColor[0.2, 0.6, 1]], "[", "\[ScriptCapitalC]", "]"}], 
        "/.", 
        RowBox[{
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], "\[Rule]", 
         SuperscriptBox["p", "2"]}]}], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayley", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}]}], "}"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["Cayley",
    FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], "=", 
  SubscriptBox["\[CapitalDelta]", "i"]}]], "Subsection",
 CellTags->"T1.4.3.1"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"setd", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayley", "[", "i", "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
      RowBox[{"set", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hCayley", "[", "i", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           InterpretationBox[
            SubscriptBox["m", "1"],
            m1], ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            m2], ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            m3], ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            m4], ",", 
           InterpretationBox[
            SubscriptBox["m", "5"],
            m5], ",", "p"}], "}"}], "]"}], ",", 
        RowBox[{"factor", "[", 
         RowBox[{
          RowBox[{
           StyleBox["ToArgs",
            FontColor->RGBColor[0.2, 0.6, 1]], "[", 
           SubscriptBox["\[CapitalDelta]", "i"], "]"}], "/.", 
          RowBox[{
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], "\[Rule]", 
           SuperscriptBox["p", "2"]}]}], "]"}]}], "]"}]}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"hCayley", "\[RuleDelayed]", 
       StyleBox["Cayley",
        FontColor->RGBColor[1, 0, 0.4]]}], ",", 
      RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
      RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
      RowBox[{"factor", "\[RuleDelayed]", "Factor"}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"1", ",", "2", ",", "6"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]], "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "m_List", "]"}], ":=", 
   
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["Cayley",
       FontColor->RGBColor[1, 0, 0.4]], "[", 
      RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "m", "]"}], "=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"mm", "=", "m"}], "}"}], ",", "\n", "\t\t\t", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "mm", "]"}], "<", "7"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"mm", ",", 
           RowBox[{"Last", "[", "mm", "]"}]}], "]"}]}], "]"}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"FactorC", "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], "4"], 
         "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
           "2"]}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
           "2"]}], "+", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "4"], 
         "+", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], "4"], 
         "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"],
           " ", 
          SuperscriptBox[
           RowBox[{
           "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
           "2"]}]}], "]"}]}]}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["Cayleyu",
      FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "m_List", "]"}], ":=", 
   
   RowBox[{
    RowBox[{
     RowBox[{
      StyleBox["Cayleyu",
       FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
      RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", "mm", "]"}], "=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"mm", "=", "m"}], "}"}], ",", "\n", "\t\t\t\t\t", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "mm", "]"}], "<", "7"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"mm", ",", 
           RowBox[{"Last", "[", "mm", "]"}]}], "]"}]}], "]"}], ";", 
       RowBox[{"FactorC", "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], "2"], 
         "-", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], "2"], 
         "-", 
         SuperscriptBox[
          RowBox[{
          "mm", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
          "2"]}], "]"}]}]}], "]"}]}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["CayleyD",
    FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], "=", 
  FractionBox[
   SubscriptBox["\[CapitalDelta]", "i"], "\[CapitalDelta]"]}]], "Subsection",
 CellTags->"T1.4.4.1"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"setd", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", "i", "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ":", "_"}], ",", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
      RowBox[{"set", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hCayleyD", "[", "i", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           InterpretationBox[
            SubscriptBox["m", "1"],
            m1], ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            m2], ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            m3], ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            m4], ",", 
           InterpretationBox[
            SubscriptBox["m", "5"],
            m5], ",", "p"}], "}"}], "]"}], ",", 
        RowBox[{"factor", "[", 
         FractionBox[
          RowBox[{
           RowBox[{
            StyleBox["ToArgs",
             FontColor->RGBColor[0.2, 0.6, 1]], "[", 
            SubscriptBox["\[CapitalDelta]", "i"], "]"}], "/.", 
           RowBox[{
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], "\[Rule]", 
            SuperscriptBox["p", "2"]}]}], 
          RowBox[{"\[ScriptCapitalC]", "[", 
           RowBox[{
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1], ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2], ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3], ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4], ",", 
            InterpretationBox[
             SubscriptBox["m", "5"],
             m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"hCayleyD", "\[RuleDelayed]", 
       StyleBox["CayleyD",
        FontColor->RGBColor[1, 0.2, 1]]}], ",", 
      RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
      RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
      RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
      RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
       StyleBox["Cayley",
        FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"1", ",", "2", ",", "5"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"1", ",", "2", ",", "5"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"1", ",", "2", ",", "5"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"1", ",", "3", ",", "6"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"1", ",", "3", ",", "6"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"1", ",", "3", ",", "6"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setd", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hCayleyD", "[", 
      RowBox[{"3", ",", "4", ",", "5"}], "]"}], "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "1"],
         m1], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "2"],
         m2], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "3"],
         m3], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "4"],
         m4], ":", "_"}], ",", 
       RowBox[{
        InterpretationBox[
         SubscriptBox["m", "5"],
         m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
    RowBox[{"set", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"hCayleyD", "[", 
        RowBox[{"3", ",", "4", ",", "5"}], "]"}], "[", 
       RowBox[{"{", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", "p"}], "}"}], "]"}], ",", 
      RowBox[{"factor", "[", 
       FractionBox[
        RowBox[{
         RowBox[{
          StyleBox["ToArgs",
           FontColor->RGBColor[0.2, 0.6, 1]], "[", 
          SubscriptBox["\[CapitalDelta]", 
           RowBox[{"3", ",", "4", ",", "5"}]], "]"}], "/.", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "\[Rule]", 
          SuperscriptBox["p", "2"]}]}], 
        RowBox[{"\[ScriptCapitalC]", "[", 
         RowBox[{
          InterpretationBox[
           SubscriptBox["m", "1"],
           m1], ",", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           m2], ",", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           m3], ",", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           m4], ",", 
          InterpretationBox[
           SubscriptBox["m", "5"],
           m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"hCayleyD", "\[RuleDelayed]", 
     StyleBox["CayleyD",
      FontColor->RGBColor[1, 0.2, 1]]}], ",", 
    RowBox[{"setd", "\[RuleDelayed]", "SetDelayed"}], ",", 
    RowBox[{"set", "\[RuleDelayed]", "Set"}], ",", 
    RowBox[{"factor", "\[RuleDelayed]", "Factor"}], ",", 
    RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
     StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"i", ",", "j", ",", "k"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["\[CapitalDelta]", 
   RowBox[{"1", ",", "2", ",", "6"}]], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["ToArgs",
   FontColor->RGBColor[0.2, 0.6, 1]], "[", 
  SubscriptBox["u", 
   RowBox[{"i", ",", "j", ",", "k"}]], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Utility Functions", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.5.1"],

Cell[CellGroupData[{

Cell["MakeFun and friends", "Subsection",
 CellTags->"T1.5.1.1"],

Cell["\<\
This function generates the recursion function definitions for 
TARCER starting from the entered relations.\
\>", "Text"],

Cell["The idea is to recurse TFR into TFI's step by step.", "Text"],

Cell[TextData[{
 "Define: S cD[ijk__] := ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[ScriptCapitalC]", 
     RowBox[{"-", "1"}]], 
    SubscriptBox["\[CapitalDelta]", "ijk"]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutonand\[CapitalDelta]u", "[", 
    RowBox[{"w_", ",", "5"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"w", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"\[ScriptCapitalC]", " ", 
         RowBox[{"cD", "[", "ijk", "]"}]}]}], ",", 
       RowBox[{
        SubscriptBox["u", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cu", "[", "ijk", "]"}]}], ",", "\n", "\t\t", 
       RowBox[{
        SubscriptBox["D", "ijk__"], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"D", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Rho]", "ijk__"], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Rho]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Phi]", 
         RowBox[{"ijk__", " "}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "ijk"}], "]"}]}], ",", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["\[Sigma]", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Sigma]", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["h", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"h", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
        RowBox[{"TA", "[", 
         RowBox[{"S", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
        RowBox[{"ToArgs", "[", 
         SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "\n", "\t\t", 
      "}"}]}], "/.", 
    RowBox[{"{", "\t", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
      RowBox[{
       SubscriptBox["m", "1"], ":>", "m1"}], ",", 
      RowBox[{
       SubscriptBox["m", "2"], ":>", "m2"}], ",", 
      RowBox[{
       SubscriptBox["m", "3"], ":>", "m3"}], ",", 
      RowBox[{
       SubscriptBox["m", "4"], ":>", "m4"}], ",", 
      RowBox[{
       SubscriptBox["m", "5"], ":>", "m5"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutonand\[CapitalDelta]u", "[", "w_", "]"}], ":=", 
   RowBox[{
    RowBox[{"w", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
       RowBox[{
        SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cD", "[", "ijk", "]"}]}], ",", 
       RowBox[{
        SubscriptBox["u", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"cu", "[", "ijk", "]"}]}], ",", 
       RowBox[{
        SubscriptBox["D", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"D", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Rho]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Rho]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Phi]", "ijk__"], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "ijk"}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[Sigma]", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"\[Sigma]", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["h", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"h", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"i_", ",", "j_", ",", "k_"}]], "\[RuleDelayed]", 
        RowBox[{"TA", "[", 
         RowBox[{"S", ",", "d", ",", "i", ",", "j", ",", "k", ",", 
          SubscriptBox["\[Nu]", "i"], ",", 
          SubscriptBox["\[Nu]", "j"], ",", 
          SubscriptBox["\[Nu]", "k"]}], "]"}]}], ",", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
        RowBox[{"ToArgs", "[", 
         SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "\[RuleDelayed]", "n1"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "\[RuleDelayed]", "n2"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "\[RuleDelayed]", "n3"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "4"], "\[RuleDelayed]", "n4"}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "5"], "\[RuleDelayed]", "n5"}], ",", 
      RowBox[{
       SubscriptBox["m", "1"], "\[RuleDelayed]", "m1"}], ",", 
      RowBox[{
       SubscriptBox["m", "2"], "\[RuleDelayed]", "m2"}], ",", 
      RowBox[{
       SubscriptBox["m", "3"], "\[RuleDelayed]", "m3"}], ",", 
      RowBox[{
       SubscriptBox["m", "4"], "\[RuleDelayed]", "m4"}], ",", 
      RowBox[{
       SubscriptBox["m", "5"], "\[RuleDelayed]", "m5"}]}], "}"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_", ",", "rest_"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "pp", ",", "li1", ",", "rest"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "pp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_Symbol", ",", "li1_", ",", "rest_"}], 
     "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "pp", ",", "dp", ",", "li1", ",", "rest"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_Symbol", ",", "li1_List"}], "]"}], 
    "]"}], ":=", 
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "pp", ",", "dp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVI", "[", 
     RowBox[{"d_", ",", "pp_", ",", "li1_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "pp", ",", "li1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nnnnnaddPeP", "[", 
   RowBox[{"any_", "[", 
    RowBox[{"d_", ",", "pp_", ",", "li1_List", ",", "rest__List"}], "]"}], 
   "]"}], ":=", 
  RowBox[{
   RowBox[{"any", "[", 
    RowBox[{"d", ",", "pp", ",", "li1", ",", "rest"}], "]"}], "/;", 
   RowBox[{"any", "=!=", "List"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", "z_List", "]"}], ":=", 
   RowBox[{"Append", "[", 
    RowBox[{"z", ",", 
     SqrtBox["PP"]}], "]"}]}], ";"}]], "Input"],

Cell["addPeP[TFI[d_, z_List]] := TFI[d, PP, z]; ", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVI", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TJI", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TJI", "[", 
    RowBox[{"d", ",", "PP", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TFR", "[", 
     RowBox[{"d_", ",", "pl_List", ",", "z_List"}], "]"}], "]"}], ":=", 
   RowBox[{"TFR", "[", 
    RowBox[{"d", ",", "PP_", ",", "pl", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TVR", "[", 
     RowBox[{"d_", ",", "z__List"}], "]"}], "]"}], ":=", 
   RowBox[{"TVR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addPeP", "[", 
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", "z__List"}], "]"}], "]"}], ":=", 
   RowBox[{"TJR", "[", 
    RowBox[{"d", ",", "PP_", ",", "z"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nutomass", "[", "w_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"w", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "+", "i1_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n1", "+", "i1"}], ",", "m1"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "2"], "+", "i2_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n2", "+", "i2"}], ",", "m2"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "+", "i3_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n3", "+", "i3"}], ",", "m3"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "4"], "+", "i4_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n4", "+", "i4"}], ",", "m4"}], "}"}]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Nu]", "5"], "+", "i5_."}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"n5", "+", "i5"}], ",", "m5"}], "}"}]}]}], "}"}]}], "/.", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "in1_Integer", ",", "in2_Integer", ",", "in3_Integer", ",", 
        "in4_Integer"}], "}"}], "\[RuleDelayed]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"in1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in2", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"in4", ",", "m4"}], "}"}]}], "}"}]}]}], "/.", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"in1_Integer", ",", "in2_Integer", ",", "in3_Integer"}], "}"}], 
     "\[RuleDelayed]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"in1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"in2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"in3", ",", "m3"}], "}"}]}], "}"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "maybeF", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"maybeF", "[", "z_Times", "]"}], ":=", 
   RowBox[{
    RowBox[{"FactorC", "[", "z", "]"}], "/;", 
    RowBox[{
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TFI"}], "]"}], "&&", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TVI"}], "]"}], "&&", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"z", ",", "TJI"}], "]"}], "&&", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"Cases", "[", 
        RowBox[{"z", ",", 
         RowBox[{"cD", "[", "__", "]"}], ",", 
         RowBox[{"-", "1"}]}], "]"}], "]"}], ">", "1"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"maybeF", "[", "z_", "]"}], ":=", "z"}], ";", 
  RowBox[{"SetAttributes", "[", 
   RowBox[{"top", ",", "Listable"}], "]"}], ";"}]], "Input"],

Cell["This is the function for the information of side conditions.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{
    StyleBox["IFF",
     FontColor->RGBColor[0.6, 0.4, 1]], ",", "HoldAll"}], "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==", "rhs_"}],
      ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     CellFrame->True,
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", "nm", "}"}], ",", 
     RowBox[{
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"HoldForm", "@@", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TFR",
                    FontColor->RGBColor[0, 0.6, 0]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    "\t", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TFR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TFR", "->", "TFIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te", "]"}], "]"}], 
                    "]"}]}]}], ")"}]}], ",", "5"}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    ",", "5"}], "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", "DP", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\n", 
                    "\t\t\t\t\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                 RowBox[{
                  RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["CayleyD",
                    FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}], "]"}],
                    "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{"BLA", "=", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                "CompoundExpression"}]}], "/.", 
              RowBox[{
               RowBox[{
                RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                RowBox[{"a_", ",", 
                 RowBox[{
                  StyleBox["IFF",
                   FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                "]"}], "\[Rule]", 
               RowBox[{"a", "/;", "b"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["CayleyD",
               FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{"{", 
           RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], ")"}],
         "/.", 
        RowBox[{"Identity", "->", "identit"}]}], "/.", 
       RowBox[{
        RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==", "rhs_"}],
      ",", "eqn_String", ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     StyleBox["Block",
      CellFrame->True,
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", "nm", "}"}], ",", 
      RowBox[{
       RowBox[{"nmrel", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "mrel", "}"}], "]"}], ",", 
          RowBox[{
           RowBox[{"FreeQ", "[", 
            RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TFR",
                    FontColor->RGBColor[0, 0.6, 0]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TFR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TFR", "->", "TFIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "te", "]"}], "]"}], 
                    "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cD", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", "DP", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["CayleyD",
                    FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}], "]"}],
                     "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", 
                    RowBox[{"addPeP", "->", "Identity"}]}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]",
                   "CompoundExpression"}]}], "/.", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                  RowBox[{"a_", ",", 
                   RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                  "]"}], "\[Rule]", 
                 RowBox[{"a", "/;", "b"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", 
               RowBox[{
                StyleBox["CayleyD",
                 FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
              "\[Rule]", 
              RowBox[{
               StyleBox["CayleyD",
                FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
            
            RowBox[{"{", 
             RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], 
           ")"}], "/.", 
          RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
         RowBox[{"Identity", "->", "identit"}]}], "/.", 
        RowBox[{
         RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}], "/;", 
    
    RowBox[{
     RowBox[{"Head", "[", "mrel", "]"}], "=!=", 
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", "nm", "}"}], ",", 
     RowBox[{
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"nmrel", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            SubscriptBox["m", "1"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1]}], ",", 
           RowBox[{
            SubscriptBox["m", "2"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2]}], ",", 
           RowBox[{
            SubscriptBox["m", "3"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3]}], ",", 
           RowBox[{
            SubscriptBox["m", "4"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4]}], ",", 
           RowBox[{
            SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
            RowBox[{"ToArgs", "[", 
             SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
        "]"}]}], ";", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"HoldForm", "@@", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TVR",
                    FontColor->RGBColor[0.2, 0.8, 1]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TVR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TVR", "->", "TVIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te", "]"}], 
                    "]"}], "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "->", "1"}]}], "/.", 
                    "nmrel"}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", "\t", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", "\n", "\t\t\t", 
                    RowBox[{
                    RowBox[{"HoldPattern", "[", 
                    RowBox[{"TA", "[", "aijk__", "]"}], "]"}], "\[Rule]", 
                    RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{"TA", "[", "aijk", "]"}], "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", "\n", "\t\t\t", 
                   RowBox[{
                    RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                    RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{"Cayley", "[", "ijk", "]"}], "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                  RowBox[{
                   RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                    "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", 
                    RowBox[{"addPeP", "->", "Identity"}]}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]",
                   "CompoundExpression"}]}], "/.", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                  RowBox[{"a_", ",", 
                   RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                  "]"}], "\[Rule]", 
                 RowBox[{"a", "/;", "b"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", 
              RowBox[{
               StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
              "]"}], "\[Rule]", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayley",
               FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", "]"}]}]}], "/.", 
          "\n", "\t\t\t", 
          RowBox[{
           RowBox[{"Hold", "[", "TA", "]"}], "->", "TA"}]}], "/.", 
         RowBox[{"{", 
          RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], "/.", 
        RowBox[{"Identity", "->", "identit"}]}], "/.", 
       RowBox[{
        RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     StyleBox["Block",
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", "nm", "}"}], ",", 
      RowBox[{
       RowBox[{"nmrel", "=", 
        RowBox[{"Select", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "mrel", "}"}], "]"}], ",", 
          RowBox[{
           RowBox[{"FreeQ", "[", 
            RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
       RowBox[{"nmrel", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"nmrel", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             SubscriptBox["m", "1"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "1"],
              m1]}], ",", 
            RowBox[{
             SubscriptBox["m", "2"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              m2]}], ",", 
            RowBox[{
             SubscriptBox["m", "3"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              m3]}], ",", 
            RowBox[{
             SubscriptBox["m", "4"], "\[RuleDelayed]", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              m4]}], ",", 
            RowBox[{
             SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
             RowBox[{
              StyleBox["ToArgs",
               FontColor->RGBColor[0.2, 0.6, 1]], "[", 
              SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
         "]"}]}], ";", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    StyleBox["TVR",
                    FontColor->RGBColor[0.2, 0.8, 1]], "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", "t", "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TVR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TVR", "->", "TVIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], "\n", "\t\t\t\t\t\t", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te__", "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "te", "]"}], 
                    "]"}], "]"}]}]}], ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "\[Rule]", "1"}]}]}], 
                    "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}], "]"}],
                     "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                    "]"}], "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{
                 RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                 "CompoundExpression"}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
              RowBox[{
               RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
             RowBox[{"addPeP", "->", "Identity"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", 
              RowBox[{
               StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
              "]"}], "\[Rule]", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayley",
               FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", "]"}], "]"}], 
            "\[Rule]", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{"{", 
           RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}], "/.", 
         
         RowBox[{"Identity", "->", "identit"}]}], "/.", 
        RowBox[{
         RowBox[{"identit", "[", "a_", "]"}], "->", "a"}]}]}]}], "]"}], "/;", 
    
    RowBox[{
     RowBox[{"Head", "[", "mrel", "]"}], "=!=", 
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{"TJI", "[", "t__", "]"}]}], "==", "rhs_"}], ",", "eqn_String", 
     ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", 
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"nm", ",", "nmrel", ",", " ", "result"}], "}"}], ",", 
     RowBox[{
      RowBox[{"\[ScriptCapitalC]", "=", "1"}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Flatten", "[", 
          RowBox[{"{", "mrel", "}"}], "]"}], ",", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{"#1", ",", "Factor"}], "]"}], "&"}]}], "]"}]}], ";", 
      RowBox[{"nmrel", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"nmrel", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            SubscriptBox["m", "1"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "1"],
             m1]}], ",", 
           RowBox[{
            SubscriptBox["m", "2"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             m2]}], ",", 
           RowBox[{
            SubscriptBox["m", "3"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             m3]}], ",", 
           RowBox[{
            SubscriptBox["m", "4"], "\[RuleDelayed]", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             m4]}], ",", 
           RowBox[{
            SubscriptBox["\[CapitalSigma]", "i_"], "\[RuleDelayed]", 
            RowBox[{
             StyleBox["ToArgs",
              FontColor->RGBColor[0.2, 0.6, 1]], "[", 
             SubscriptBox["\[CapitalSigma]", "i"], "]"}]}]}], "}"}]}], 
        "]"}]}], ";", "\n", "\t\t", 
      RowBox[{"result", " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"HoldForm", "@@", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"setdel", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"TJR", "@@", 
                    RowBox[{"top", "/@", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"{", 
                    RowBox[{"tT", "=", "t"}], "}"}], "]"}]}]}], "]"}], "/.", 
                    "nmrel"}], ",", 
                    RowBox[{
                    RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "CompoundExpression", "]"}], "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TComment", "]"}], "[", 
                    RowBox[{"eqn", ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TJR", "[", "t", "]"}], "]"}], "]"}], "/.", 
                    RowBox[{"TJR", "->", "TJIC"}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Pattern", ":>", "pat"}], ",", 
                    RowBox[{"Blank", ":>", "blank"}]}], "}"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"PP", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ":>", "PP"}]}], "/.",
                     "nmrel"}]}], "]"}], "\t", ",", 
                    RowBox[{"TJ", "=", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"TT", "=", 
                    RowBox[{"Expand3", "[", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    FractionBox["1", "c"], 
                    RowBox[{"(", 
                    RowBox[{"rhs", "/.", 
                    RowBox[{
                    RowBox[{"TJI", "[", "te__", "]"}], "\[RuleDelayed]", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{"nutomass", "[", 
                    RowBox[{"TJI", "[", "te", "]"}], "]"}], "]"}]}]}], 
                    ")"}]}], "]"}], "]"}]}], ",", 
                    RowBox[{"TJI", "[", "__", "]"}], ",", 
                    RowBox[{
                    RowBox[{"maybeF", "[", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Numerator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}], 
                    "/", 
                    RowBox[{"Collect", "[", 
                    RowBox[{
                    RowBox[{"Denominator", "[", "#1", "]"}], ",", 
                    RowBox[{"cu", "[", "__", "]"}], ",", "Factor"}], "]"}]}], 
                    "&"}], ")"}], "/@", 
                    RowBox[{"Factor", "[", "#1", "]"}]}], "]"}], "&"}]}], 
                    "]"}], "/.", 
                    RowBox[{"\[ScriptCapitalC]", "\[Rule]", "1"}]}], "/.", 
                    "nmrel"}]}]}], "]"}], ",", 
                    RowBox[{"nutonand\[CapitalDelta]u", "[", 
                    RowBox[{
                    StyleBox["IFF",
                    FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds", "]"}], 
                    "]"}]}], "]"}]}], "]"}], "/.", 
                    RowBox[{
                    RowBox[{"top", "[", "i_Integer", "]"}], "\[Rule]", 
                    "i"}]}], "/.", 
                    RowBox[{"top", "\[RuleDelayed]", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "PQ"}], "]"}], 
                    "&"}], ")"}]}]}], "/.", 
                    RowBox[{
                    RowBox[{"patternTest", "[", 
                    RowBox[{
                    RowBox[{"pattern", "[", 
                    RowBox[{
                    RowBox[{"dm_", "/;", 
                    RowBox[{"MemberQ", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"d", ",", 
                    InterpretationBox[
                    SuperscriptBox["p", "2"],
                    PP], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3], ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", "dm"}], "]"}]}], ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}], ",", "_"}], "]"}], 
                    "\[RuleDelayed]", 
                    RowBox[{"pattern", "[", 
                    RowBox[{"dm", ",", 
                    RowBox[{"blank", "[", "]"}]}], "]"}]}]}], "/.", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"blank", "\[RuleDelayed]", "Blank"}], ",", 
                    RowBox[{"pattern", "\[RuleDelayed]", "Pattern"}], ",", 
                    RowBox[{
                    "patternTest", "\[RuleDelayed]", "PatternTest"}]}], 
                    "}"}]}], "}"}]}], "/.", 
                  RowBox[{
                   RowBox[{"HoldPattern", "[", 
                    RowBox[{"TA", "[", "aijk__", "]"}], "]"}], "\[Rule]", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Hold", "[", "TA", "]"}], "[", "aijk", "]"}], "[", 
                    RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_"}], "}"}], "\[RuleDelayed]", 
                    "ma"}]}], "]"}], "]"}]}]}], "/.", 
                 RowBox[{
                  RowBox[{"cD", "[", "ijk__", "]"}], "\[Rule]", 
                  RowBox[{
                   RowBox[{"Hold", "[", 
                    RowBox[{
                    StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}], "]"}],
                    "[", 
                   RowBox[{"addPeP", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
                RowBox[{
                 RowBox[{"cu", "[", "ijk__", "]"}], "\[Rule]", 
                 RowBox[{
                  RowBox[{"Hold", "[", 
                   RowBox[{
                    StyleBox["Cayleyu",
                    FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}], 
                   "]"}], "[", 
                  RowBox[{"addPeP", "[", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"nutomass", "[", 
                    RowBox[{"Last", "[", 
                    RowBox[{"{", "t", "}"}], "]"}], "]"}], "/.", "nmrel"}], "/.", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"nuu_", ",", "ma_Symbol"}], "}"}], 
                    "\[RuleDelayed]", "ma"}]}], "]"}], "]"}]}]}], "/.", 
               RowBox[{
                RowBox[{"Hold", "[", "CompoundExpression", "]"}], "\[Rule]", 
                "CompoundExpression"}]}], "/.", 
              RowBox[{
               RowBox[{
                RowBox[{"Hold", "[", "Condition", "]"}], "[", 
                RowBox[{"a_", ",", 
                 RowBox[{
                  StyleBox["IFF",
                   FontColor->RGBColor[0.6, 0.4, 1]], "[", "b_", "]"}]}], 
                "]"}], "\[Rule]", 
               RowBox[{"a", "/;", "b"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Hold", "[", "Set", "]"}], "\[Rule]", "Set"}]}], "/.", 
            RowBox[{
             RowBox[{"Hold", "[", "TComment", "]"}], "->", "TComment"}]}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", 
             RowBox[{
              StyleBox["Cayleyu",
               FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk__", "]"}], 
             "]"}], "\[Rule]", 
            RowBox[{
             StyleBox["Cayleyu",
              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
          RowBox[{
           RowBox[{"Hold", "[", 
            RowBox[{
             StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", "]"}], "]"}], 
           "\[Rule]", 
           RowBox[{
            StyleBox["Cayley",
             FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", "]"}]}]}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "TA", "]"}], "\[Rule]", "TA"}]}], "/.", 
        RowBox[{"{", 
         RowBox[{"setdel", "\[RuleDelayed]", "SetDelayed"}], "}"}]}]}], ";", 
      RowBox[{"\[ScriptCapitalC]", "=", "."}], ";", " ", "result"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"paTT", ",", "HoldFirst"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"c_.", " ", 
      RowBox[{
       StyleBox["TKI",
        FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
     "rhs_"}], ",", "eqn_String", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
    RowBox[{"mrel_", ":", 
     RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"MakeFun", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"c", " ", 
           RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
         RowBox[{"TKI", "->", "TJI"}]}], ",", "eqn", ",", 
        RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PP", ":>", "0"}], ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "PP", "]"}], ":>", "0"}], ",", 
        RowBox[{"Pattern", ":>", "paTT"}]}], "}"}]}], "/.", 
     RowBox[{
      RowBox[{"paTT", "[", 
       RowBox[{"0", ",", "_"}], "]"}], "->", "0"}]}], "/.", 
    RowBox[{"paTT", "->", "Pattern"}]}], "/.", 
   RowBox[{
    RowBox[{"TJIC", "[", 
     RowBox[{"d_", ",", "li_List"}], "]"}], ":>", 
    RowBox[{"TJIC", "[", 
     RowBox[{"d", ",", "0", ",", "li"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TBI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"c", " ", 
         RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
       RowBox[{"TBI", "->", "TJI"}]}], ",", "eqn", ",", 
      RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
    RowBox[{"{", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"TJI", ":>", "TBI"}], ",", 
      RowBox[{"TJR", ":>", "TBR"}], ",", 
      RowBox[{"TJIC", ":>", "TBIC"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["MakeFun",
     FontColor->RGBColor[1, 0.6, 0.6]], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"c_.", " ", 
       RowBox[{
        StyleBox["TAI",
         FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==", 
      "rhs_"}], ",", "eqn_String", ",", 
     RowBox[{
      StyleBox["IFF",
       FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
     RowBox[{"mrel_", ":", 
      RowBox[{"{", "}"}]}]}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"c", " ", 
         RowBox[{"TJI", "[", "t", "]"}]}], "==", "rhs"}], "/.", 
       RowBox[{"TAI", "->", "TJI"}]}], ",", "eqn", ",", 
      RowBox[{"IFF", "[", "conds", "]"}], ",", "mrel"}], "]"}], "/.", 
    RowBox[{"{", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"TJI", ":>", "TAI"}], ",", 
      RowBox[{"TJR", ":>", "TAR"}], ",", 
      RowBox[{"TJIC", ":>", "TAIC"}]}], "}"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Interchange", "Subsubsection",
 CellTags->"T1.5.1.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Interchange", "[", 
   RowBox[{"z_", ",", 
    RowBox[{"i_", "\[LeftRightArrow]", "j_"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", "z", "]"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "i"], ":>", 
       SubscriptBox["\[Nu]", "j"]}], ",", 
      RowBox[{
       SubscriptBox["\[Nu]", "j"], ":>", 
       SubscriptBox["\[Nu]", "i"]}], ",", 
      RowBox[{
       SubscriptBox["m", "i"], ":>", 
       SubscriptBox["m", "j"]}], ",", 
      RowBox[{
       SubscriptBox["m", "j"], ":>", 
       SubscriptBox["m", "i"]}], ",", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[CapitalDelta]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["u", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["u", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", "\n", "\t\t", 
      RowBox[{
       SubscriptBox["h", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Phi]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Phi]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Rho]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Rho]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["\[Sigma]", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Sigma]", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}], ",", 
      RowBox[{
       SubscriptBox["D", "ij__"], ":>", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["D", "ij"], "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"i", ":>", "j"}], ",", 
           RowBox[{"j", ":>", "i"}]}], "}"}]}], ")"}]}]}], "}"}]}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       StyleBox["TFI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{
       StyleBox["TVI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TVI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{
       StyleBox["TJI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TJI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}], ",", "\n", 
     "\t\t\t", 
     RowBox[{
      RowBox[{
       StyleBox["TBI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"depp__", ",", "args_List"}], "]"}], ":>", 
      RowBox[{
       RowBox[{
        StyleBox["TBI",
         FontColor->RGBColor[1, 0.6, 0]], "[", 
        RowBox[{"depp", ",", 
         RowBox[{"Sort", "[", "args", "]"}]}], "]"}], "/;", 
       "\[InvisibleSpace]", 
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"OrderedQ", "[", "args", "]"}], ")"}]}]}]}]}], 
    "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"i_", "\[LeftRightArrow]", "j_"}], ",", "vw__"}], "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{"z", ",", 
       RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], ",", "vw"}], "]"}]}],
   ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"{", 
      RowBox[{"i_", "\[LeftRightArrow]", "j_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{"Interchange", "[", 
     RowBox[{"z", ",", 
      RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"i_", "\[LeftRightArrow]", "j_"}], ",", "vw__"}], "}"}]}], 
    "]"}], ":=", 
   RowBox[{"Interchange", "[", 
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{"z", ",", 
       RowBox[{"i", "\[LeftRightArrow]", "j"}]}], "]"}], ",", "vw"}], "]"}]}],
   ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Interchange", "[", 
  RowBox[{
   RowBox[{"test", "[", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"x", ",", "b", ",", "y"}]], ",", 
     SubscriptBox["\[Nu]", "x"], ",", 
     SubscriptBox["\[Nu]", "y"], ",", 
     SubscriptBox["\[Nu]", "a"], ",", 
     SubscriptBox["\[Nu]", "b"], ",", 
     SubscriptBox["m", "x"]}], "]"}], ",", " ", 
   RowBox[{"x", "\[LeftRightArrow]", "y"}], ",", 
   RowBox[{"a", "\[LeftRightArrow]", "b"}]}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\[Nu]explicit", "Subsection",
 CellTags->"T1.5.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Nu]explicit", "[", "z_", "]"}], ":=", 
  RowBox[{
   RowBox[{"z", "/.", 
    RowBox[{
     SubscriptBox["\[CapitalSigma]", "i_"], ":>", 
     RowBox[{"(", 
      RowBox[{"Expand", "[", 
       RowBox[{
        RowBox[{"3", " ", "d"}], " ", "-", "2", " ", "-", 
        RowBox[{"2", " ", 
         RowBox[{
          SubsuperscriptBox["\[Sum]", 
           RowBox[{"j", "=", "1"}], "i"], 
          SubscriptBox["\[Nu]", "j"]}]}]}], "]"}], ")"}]}]}], "/.", "\n", 
   "\t", 
   RowBox[{"{", "\t", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Sigma]", 
       RowBox[{"i_", ",", "j_", ",", "k_"}]], ":>", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "4"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}], "-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "i"]}], "-", 
          SubscriptBox["\[Nu]", "j"]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"j", ",", "i", ",", "k"}]]}], "-", "\n", "\t\t\t\t\t", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "i"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          SubscriptBox["\[Nu]", "k"], "-", "1"}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}]}]}], ",", "\n", "\t\t\t", 
     RowBox[{
      SubscriptBox["h", 
       RowBox[{"i_", ",", "j_", ",", "k_", " "}]], ":>", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          SubscriptBox["\[Nu]", "k"]}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "k"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}], "-", "1"}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "i"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}], "+", "\n", "\t\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "j"], "-", 
          RowBox[{"2", " ", 
           SubscriptBox["\[Nu]", "k"]}]}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}]}]}], ",", "\n", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"i_", ",", "j_", ",", "k_", " "}]], ":>", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"2", 
            SubscriptBox["\[Nu]", "j"]}], "-", 
           SubscriptBox["\[Nu]", "k"]}], ")"}]}], 
        SuperscriptBox[
         SubscriptBox["m", "k"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"j", ",", "i", ",", "k"}]]}], " ", "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "j"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}]}], ")"}], 
        SuperscriptBox[
         SubscriptBox["m", "j"], "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"k", ",", "i", ",", "j"}]]}], "+", "\n", "\t\t\t\t\t", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          SubscriptBox["\[Nu]", "i"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "j"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "k"]}], "-", "1"}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"i", ",", "j", ",", "k"}]]}]}]}]}], "}"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell["TA", "Subsubsection",
 CellTags->"T1.5.2.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"TA", "[", 
      RowBox[{"a_", ",", "d___Symbol", ",", "ijk___Integer"}], "]"}], "[", 
     "__", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"SubscriptBox", "[", 
   RowBox[{"a", ",", 
    RowBox[{"RowBox", "[", 
     RowBox[{"{", "ijk", "}"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBoxes", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"TA", "[", 
      RowBox[{
      "a_", ",", "d___Symbol", ",", "ijk___Integer", ",", "___Symbol"}], 
      "]"}], "[", "__", "]"}], ",", "fmt_"}], "]"}], ":=", 
  RowBox[{"SubscriptBox", "[", 
   RowBox[{"a", ",", 
    RowBox[{"RowBox", "[", 
     RowBox[{"{", "ijk", "}"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"TA", "[", "args__", "]"}], "[", "mar_List", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"TA", "[", "args", "]"}], "[", "mar", "]"}], "=", 
     RowBox[{
      RowBox[{"ta", "[", "args", "]"}], "[", "mar", "]"}]}], ")"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "Assume that the last argument of {em__} is always p =",
 Cell[BoxData[
  FormBox[
   SqrtBox[
    RowBox[{" ", 
     FormBox[
      SuperscriptBox["p", "2"],
      TraditionalForm]}]], TraditionalForm]]],
 "."
}], "Text"],

Cell["(72)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"D", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       SuperscriptBox["q", "8"], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["q", "6"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "2"], "+", 
          SuperscriptBox["mj", "2"], "+", 
          SuperscriptBox["mk", "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["q", "4"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "+", 
          RowBox[{"4", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["mi", "2"], " ", 
              SuperscriptBox["mj", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mi", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mj", "2"]}]}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["q", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "6"], "+", 
          SuperscriptBox["mj", "6"], "+", 
          SuperscriptBox["mk", "6"], "-", 
          RowBox[{
           SuperscriptBox["mi", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
          RowBox[{
           SuperscriptBox["mj", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
          RowBox[{
           SuperscriptBox["mk", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"]}], ")"}]}], "+", 
          RowBox[{"10", " ", 
           SuperscriptBox["mi", "2"], " ", 
           SuperscriptBox["mj", "2"], " ", 
           SuperscriptBox["mk", "2"]}]}], ")"}]}], "+", 
       SuperscriptBox[
        RowBox[{
         RowBox[{
          StyleBox["Cayley",
           FontColor->RGBColor[1, 0, 0.4]], "[", 
          RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}], "2"]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(73)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"\[Rho]", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}],
      "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["q", "6"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["q", "4"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["mi", "2"], "+", 
          SuperscriptBox["mj", "2"], "+", 
          SuperscriptBox["mk", "2"]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox["q", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["mi", "4"], "+", 
             SuperscriptBox["mj", "4"], "+", 
             SuperscriptBox["mk", "4"]}], ")"}]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SuperscriptBox["mi", "2"], " ", 
              SuperscriptBox["mj", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mi", "2"]}], "+", 
             RowBox[{
              SuperscriptBox["mk", "2"], " ", 
              SuperscriptBox["mj", "2"]}]}], ")"}]}]}], ")"}]}], "+", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["mi", "6"], "+", 
         SuperscriptBox["mj", "6"], "+", 
         SuperscriptBox["mk", "6"], "-", 
         RowBox[{
          SuperscriptBox["mi", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mj", "4"], "+", 
            SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox["mj", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mi", "4"], "+", 
            SuperscriptBox["mk", "4"]}], ")"}]}], "-", 
         RowBox[{
          SuperscriptBox["mk", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["mi", "4"], "+", 
            SuperscriptBox["mj", "4"]}], ")"}]}], "+", 
         RowBox[{"10", " ", 
          SuperscriptBox["mi", "2"], " ", 
          SuperscriptBox["mj", "2"], " ", 
          SuperscriptBox["mk", "2"]}]}], ")"}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(74)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"\[Phi]", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer"}],
      "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"4", 
      RowBox[{"Factor", "[", 
       RowBox[{
        SuperscriptBox["q", "4"], " ", "+", 
        RowBox[{"2", 
         SuperscriptBox["q", "2"], 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["mi", "2"], "-", 
           SuperscriptBox["mj", "2"], "-", 
           SuperscriptBox["mk", "2"]}], ")"}]}], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["mj", "2"], "-", 
           SuperscriptBox["mk", "2"]}], ")"}], "2"], "+", 
        RowBox[{
         SuperscriptBox["mi", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SuperscriptBox["mj", "2"]}], "+", 
           RowBox[{"2", 
            SuperscriptBox["mk", "2"]}], "-", 
           RowBox[{"3", 
            SuperscriptBox["mi", "2"]}]}], ")"}]}]}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(75)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "\[Sigma]", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", 
      "k_Integer", ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", 
      "\[Nu]k_Integer"}], "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", 
   RowBox[{"Factor", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], 
       RowBox[{"(", 
        RowBox[{"d", "-", "\[Nu]i", "-", 
         RowBox[{"2", "\[Nu]j"}]}], ")"}], 
       RowBox[{
        RowBox[{"TA", "[", 
         RowBox[{"\[Phi]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
        RowBox[{"{", "em", "}"}], "]"}]}], " ", 
      RowBox[{"-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", "\[Nu]i"}], "-", "\[Nu]j"}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "j", ",", "i", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}]}], "-", 
     RowBox[{
      FractionBox["1", "4"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", "d"}], "-", 
        RowBox[{"2", "\[Nu]i"}], "-", 
        RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k", "-", "1"}], ")"}], 
      RowBox[{
       RowBox[{"TA", "[", 
        RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
       RowBox[{"{", "em", "}"}], "]"}]}]}], " ", "  ", "]"}]}], 
  ";"}]], "Input"],

Cell["(76)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "h", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer", 
      ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", "\[Nu]k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", "\n", "\t", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k"}], ")"}], 
        SuperscriptBox["mk", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", "\[Nu]i", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", 
          RowBox[{"2", "\[Nu]k"}], "-", "1"}], ")"}], 
        SuperscriptBox["mi", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "\[Nu]j", "-", 
          RowBox[{"2", " ", "\[Nu]k"}]}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Rho]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}], " ", "  ", "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell["(79)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{
     "S", ",", "d_", ",", "i_Integer", ",", "j_Integer", ",", "k_Integer", 
      ",", "\[Nu]i_Integer", ",", "\[Nu]j_Integer", ",", "\[Nu]k_Integer"}], 
     "]"}], "[", 
    RowBox[{"{", "em__", "}"}], "]"}], ":=", "\n", "\t\t", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q", "=", 
        RowBox[{"Last", "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "i", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mj", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}]}], ",", 
       RowBox[{"mk", "=", 
        RowBox[{
         RowBox[{"{", "em", "}"}], "\[LeftDoubleBracket]", "k", 
         "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
     RowBox[{"Factor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"2", "\[Nu]j"}], "-", "\[Nu]k"}], ")"}]}], 
        SuperscriptBox["mk", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "j", ",", "i", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}], " ", "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "\[Nu]j", "-", 
          RowBox[{"2", "\[Nu]k"}]}], ")"}], 
        SuperscriptBox["mj", "2"], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Phi]", ",", "k", ",", "i", ",", "j"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}], "test", "\n", 
        FormBox["test",
         TraditionalForm], "2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", "\[Nu]i", "-", 
          RowBox[{"2", "\[Nu]j"}], "-", 
          RowBox[{"2", "\[Nu]k"}], "-", "1"}], ")"}], 
        RowBox[{
         RowBox[{"TA", "[", 
          RowBox[{"\[Rho]", ",", "i", ",", "j", ",", "k"}], "]"}], "[", 
         RowBox[{"{", "em", "}"}], "]"}]}]}], "   ", "]"}]}], "]"}]}], 
  ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckRecursion", "Subsection",
 CellTags->"T1.5.3.1"],

Cell["\<\
The following functions check the recurrence relations for J and V (using the \
final form as generated by MakeFun) via their Mellin-Integral Kernels.\
\>", "Text"],

Cell["\<\
This test is applicable in the general case  and if some of the 
masses vanish, but not for other special cases !!!\
\>", "Text"],

Cell[CellGroupData[{

Cell["CheckTVIRecursion", "Subsubsection",
 CellTags->"T1.5.3.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVIKernel", "[", 
    RowBox[{"d_", ",", "PP", ",", " ", "lis_"}], "]"}], ":=", "\n", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "PP"}], ")"}], 
     RowBox[{"d", "-", "n1", "-", "n2", "-", "n3", "-", "n4"}]], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m1", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s1"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m2", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s2"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m3", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s3"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m4", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s4"], "*", 
    RowBox[{"TVIpsi", " ", "[", 
     RowBox[{"d", ",", " ", 
      RowBox[{"(", 
       RowBox[{"lis", "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"m1", "->", "s1"}], ",", 
          RowBox[{"m2", "->", "s2"}], ",", 
          RowBox[{"m3", "->", "s3"}], ",", 
          RowBox[{"m4", "->", "s4"}]}], "}"}]}], ")"}]}], "]"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TVIPsi", "[", 
    RowBox[{"d_", ",", "\n", "\t", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1_", ",", " ", "s1_"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"n2_", ",", "s2_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3_", ",", "s3_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4_", ",", "s4_"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], 
     RowBox[{"n1", "+", "n2", "+", "n3", "+", "n4", "+", "1"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s1"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n1", "-", "s1"}], "]"}]}], 
     RowBox[{"gamma", "[", "n1", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s2"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n2", "-", "s2"}], "]"}]}], 
     RowBox[{"gamma", "[", "n2", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s3"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "n3", "-", "s3"}], "]"}]}], 
     RowBox[{"gamma", "[", "n3", "]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"gamma", "[", 
       RowBox[{"-", "s4"}], "]"}], 
      RowBox[{"gamma", "[", 
       RowBox[{"n4", "+", "s4"}], "]"}]}], 
     RowBox[{"gamma", "[", "n4", "]"}]], 
    RowBox[{
     RowBox[{"gamma", "[", 
      RowBox[{
      "n1", "+", "n2", "+", "n3", "+", "n4", "+", "s1", "+", "s2", "+", "s3", 
       "+", "s4", "-", "d"}], "]"}], "/", 
     RowBox[{"gamma", "[", 
      RowBox[{
       RowBox[{"3", 
        RowBox[{"d", "/", "2"}]}], "-", "n1", "-", "n2", "-", "n3", "-", "n4",
        "-", "s1", "-", "s2", "-", "s3", "-", "s4"}], "]"}]}], 
    FractionBox[
     RowBox[{"gamma", "[", 
      RowBox[{"n1", "+", "n3", "+", "s1", "+", "s3", "-", 
       RowBox[{"d", "/", "2"}]}], "]"}], 
     RowBox[{"gamma", "[", 
      RowBox[{
      "n1", "+", "n3", "+", "n4", "+", "s1", "+", "s3", "+", "s4", "-", 
       RowBox[{"d", "/", "2"}]}], "]"}]], 
    FractionBox[
     RowBox[{"gamma", "[", 
      RowBox[{
      "d", "-", "n1", "-", "n3", "-", "n4", "-", "s1", "-", "s3", "-", "s4"}],
       "]"}], 
     RowBox[{"gamma", "[", 
      RowBox[{"d", "-", "n1", "-", "n3", "-", "s1", "-", "s3"}], "]"}]]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CheckTVIRecursion", "[", "expr_", "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$CheckRecursion", "===", "True"}], ",", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rel", ",", "kern", ",", "nkern"}], "}"}], ",", 
      RowBox[{
       RowBox[{"rel", "=", 
        RowBox[{
         RowBox[{"Extract", "[", 
          RowBox[{
           RowBox[{"expr", "/.", "\[InvisibleSpace]", 
            RowBox[{"TVIC", "\[Rule]", "TVIkernel"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "1", ",", "1", ",", "2"}], "}"}]}], 
          "]"}], "-", 
         RowBox[{"Extract", "[", 
          RowBox[{
           RowBox[{"expr", "/.", "\[InvisibleSpace]", 
            RowBox[{"TVI", "\[Rule]", "TVIkernel"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], 
       ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", "TVIkernel", "}"}], "===", 
          RowBox[{"Union", "[", 
           RowBox[{"Head", "/@", 
            RowBox[{"Cases", "[", 
             RowBox[{"rel", ",", 
              RowBox[{
               RowBox[{"ff_", "[", "__", "]"}], "/;", 
               RowBox[{
                RowBox[{"Context", "[", "ff", "]"}], "=!=", 
                "\"\<System`\>\""}]}], ",", "4"}], "]"}]}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"rel", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"Numerator", "[", 
              RowBox[{"Together", "[", "rel", "]"}], "]"}], ",", 
             RowBox[{"TVI", "[", "__", "]"}], ",", "Factor"}], "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"rel", "/.", "\[InvisibleSpace]", 
            RowBox[{"TVIkernel", "\[Rule]", "TVIKernel"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"PP", "\[Rule]", 
               RowBox[{"-", "1"}]}], ",", 
              RowBox[{"m1", "\[Rule]", 
               SqrtBox["z1"]}], ",", 
              RowBox[{"m2", "\[Rule]", 
               SqrtBox["z2"]}], ",", 
              RowBox[{"m3", "\[Rule]", 
               SqrtBox["z3"]}], ",", 
              RowBox[{"m4", "\[Rule]", 
               SqrtBox["z4"]}]}], "}"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"PowerExpand", "[", "kern", "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"Expand", "[", "kern", "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               SuperscriptBox["z1", 
                RowBox[{"s1", "+", "w1_."}]], " ", 
               SuperscriptBox["z2", 
                RowBox[{"s2", "+", "w2_."}]], " ", 
               SuperscriptBox["z3", 
                RowBox[{"s3", "+", "w3_."}]], " ", 
               SuperscriptBox["z4", 
                RowBox[{"s4", "+", "w4_."}]], " ", "fac_"}], "\[RuleDelayed]", 
              RowBox[{"(", 
               RowBox[{"fac", "/.", "\[InvisibleSpace]", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"s1", "\[Rule]", 
                   RowBox[{"s1", "-", "w1"}]}], ",", 
                  RowBox[{"s2", "\[Rule]", 
                   RowBox[{"s2", "-", "w2"}]}], ",", 
                  RowBox[{"s3", "\[Rule]", 
                   RowBox[{"s3", "-", "w3"}]}], ",", 
                  RowBox[{"s4", "\[Rule]", 
                   RowBox[{"s4", "-", "w4"}]}]}], "}"}]}], ")"}]}], "}"}]}]}],
           ";", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"kern", "/.", "\[InvisibleSpace]", 
              RowBox[{"TVIpsi", "\[Rule]", "TVIPsi"}]}], "/.", 
             "\[InvisibleSpace]", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"gamma", "[", "zz_", "]"}], "\[RuleDelayed]", 
               RowBox[{"gamma", "[", 
                RowBox[{"Expand", "[", "zz", "]"}], "]"}]}], "}"}]}], "/.", 
            "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", 
               RowBox[{"zz_", "+", "nn_Integer"}], "]"}], "\[RuleDelayed]", 
              RowBox[{
               RowBox[{"gamma", "[", "zz", "]"}], " ", 
               RowBox[{"Pochhammer", "[", 
                RowBox[{"zz", ",", "nn"}], "]"}]}]}], "}"}]}]}], ";", 
          RowBox[{"KERN", "=", "kern"}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", "zz_", "]"}], "\[RuleDelayed]", "1"}], 
             "}"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{"kern", "/.", "\[InvisibleSpace]", 
             RowBox[{"{", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], 
                RowBox[{"nn_", "+", "ww_Integer"}]], "\[RuleDelayed]", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], "ww"]}], "}"}]}], "/.", 
            "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "nn_"], "\[RuleDelayed]", "1"}], 
             "}"}]}]}], ";", 
          RowBox[{"nkern", "=", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"kern", "/.", "\[InvisibleSpace]", 
              RowBox[{"Thread", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "d", ",", "n1", ",", "n2", ",", "n3", ",", "n4", ",", "s1", 
                  ",", "s2", ",", "s3", ",", "s4"}], "}"}], "\[Rule]", 
                RowBox[{"RandomReal", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "1"}], "}"}], ",", "9"}], "]"}]}], 
               "]"}]}], ",", 
             RowBox[{"{", "3", "}"}]}], "]"}]}], ";", "nkern"}], ",", 
         RowBox[{"Print", "[", "\"\<Matching Error\>\"", "]"}]}], "]"}]}]}], 
     "]"}], ",", 
    RowBox[{"If", "[", 
     RowBox[{"$Notebooks", ",", 
      RowBox[{"Print", "[", "\"\<Off\>\"", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckTJIRecursion", "Subsubsection",
 CellTags->"T1.5.3.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJIKernel", "[", 
    RowBox[{"d_", ",", "PP", ",", " ", "lis_"}], "]"}], ":=", "\n", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "PP"}], ")"}], 
     RowBox[{"d", "-", "n1", "-", "n2", "-", "n3"}]], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m1", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s1"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m2", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s2"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox[
       SuperscriptBox["m3", "2"], 
       RowBox[{"-", "PP"}]], ")"}], "s3"], "*", 
    RowBox[{"TJIpsi", " ", "[", 
     RowBox[{"d", ",", " ", 
      RowBox[{"(", 
       RowBox[{"lis", "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"m1", "->", "s1"}], ",", 
          RowBox[{"m2", "->", "s2"}], ",", 
          RowBox[{"m3", "->", "s3"}]}], "}"}]}], ")"}]}], "]"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIPsi", " ", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "s1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", "s2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", "s3_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"Evaluate", "[", 
   RowBox[{
    RowBox[{"TVIPsi", "[", 
     RowBox[{"d", ",", "\n", "\t", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", " ", "s1"}], "}"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{"n2", ",", "s2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "s3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "/", 
    RowBox[{"gamma", "[", "0", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "CheckTJIRecursion", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CheckTJIRecursion", "[", "expr_", "]"}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$CheckRecursion", "===", "True"}], ",", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rel", ",", "kern", ",", "nkern"}], "}"}], ",", 
      RowBox[{
       RowBox[{"rel", "=", 
        RowBox[{
         RowBox[{"Extract", "[", 
          RowBox[{
           RowBox[{"expr", "/.", "\[InvisibleSpace]", 
            RowBox[{"TJIC", "\[Rule]", "TJIkernel"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "1", ",", "1", ",", "2"}], "}"}]}], 
          "]"}], "-", 
         RowBox[{"Extract", "[", 
          RowBox[{
           RowBox[{"expr", "/.", "\[InvisibleSpace]", 
            RowBox[{"TJI", "\[Rule]", "TJIkernel"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}]}], "]"}]}]}], 
       ";", 
       RowBox[{"REL", "=", "rel"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", "TJIkernel", "}"}], "===", 
          RowBox[{"Union", "[", 
           RowBox[{"Head", "/@", 
            RowBox[{"Cases", "[", 
             RowBox[{"rel", ",", 
              RowBox[{
               RowBox[{"ff_", "[", "__", "]"}], "/;", 
               RowBox[{
                RowBox[{"Context", "[", "ff", "]"}], "=!=", 
                "\"\<System`\>\""}]}], ",", "4"}], "]"}]}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"rel", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"Numerator", "[", 
              RowBox[{"Together", "[", "rel", "]"}], "]"}], ",", 
             RowBox[{"TJI", "[", "__", "]"}], ",", "Factor"}], "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"rel", "/.", "\[InvisibleSpace]", 
            RowBox[{"TJIkernel", "\[Rule]", "TJIKernel"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"PP", "\[Rule]", 
               RowBox[{"-", "1"}]}], ",", 
              RowBox[{"m1", "\[Rule]", 
               SqrtBox["z1"]}], ",", 
              RowBox[{"m2", "\[Rule]", 
               SqrtBox["z2"]}], ",", 
              RowBox[{"m3", "\[Rule]", 
               SqrtBox["z3"]}]}], "}"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"PowerExpand", "[", "kern", "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"Expand", "[", "kern", "]"}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               SuperscriptBox["z1", 
                RowBox[{"s1", "+", "w1_."}]], " ", 
               SuperscriptBox["z2", 
                RowBox[{"s2", "+", "w2_."}]], " ", 
               SuperscriptBox["z3", 
                RowBox[{"s3", "+", "w3_."}]], " ", "fac_"}], "\[RuleDelayed]", 
              RowBox[{"(", 
               RowBox[{"fac", "/.", "\[InvisibleSpace]", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"s1", "\[Rule]", 
                   RowBox[{"s1", "-", "w1"}]}], ",", 
                  RowBox[{"s2", "\[Rule]", 
                   RowBox[{"s2", "-", "w2"}]}], ",", 
                  RowBox[{"s3", "\[Rule]", 
                   RowBox[{"s3", "-", "w3"}]}]}], "}"}]}], ")"}]}], "}"}]}]}],
           ";", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"kern", "/.", "\[InvisibleSpace]", 
              RowBox[{"TJIpsi", "\[Rule]", "TJIPsi"}]}], "/.", 
             "\[InvisibleSpace]", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"gamma", "[", "zz_", "]"}], "\[RuleDelayed]", 
               RowBox[{"gamma", "[", 
                RowBox[{"Expand", "[", "zz", "]"}], "]"}]}], "}"}]}], "/.", 
            "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", 
               RowBox[{"zz_", "+", "nn_Integer"}], "]"}], "\[RuleDelayed]", 
              RowBox[{
               RowBox[{"gamma", "[", "zz", "]"}], " ", 
               RowBox[{"Pochhammer", "[", 
                RowBox[{"zz", ",", "nn"}], "]"}]}]}], "}"}]}]}], ";", 
          RowBox[{"KERN", "=", "kern"}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{"kern", "/.", "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"gamma", "[", "zz_", "]"}], "\[RuleDelayed]", "1"}], 
             "}"}]}]}], ";", 
          RowBox[{"kern", "=", 
           RowBox[{
            RowBox[{"kern", "/.", "\[InvisibleSpace]", 
             RowBox[{"{", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], 
                RowBox[{"nn_", "+", "ww_Integer"}]], "\[RuleDelayed]", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], "ww"]}], "}"}]}], "/.", 
            "\[InvisibleSpace]", 
            RowBox[{"{", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "nn_"], "\[RuleDelayed]", "1"}], 
             "}"}]}]}], ";", 
          RowBox[{"nkern", "=", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"kern", "/.", "\[InvisibleSpace]", 
              RowBox[{"Thread", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "d", ",", "n1", ",", "n2", ",", "n3", ",", "s1", ",", "s2", 
                  ",", "s3"}], "}"}], "\[Rule]", 
                RowBox[{"RandomReal", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "1"}], "}"}], ",", "7"}], "]"}]}], 
               "]"}]}], ",", 
             RowBox[{"{", "3", "}"}]}], "]"}]}], ";", "nkern"}], ",", 
         RowBox[{"Print", "[", "\"\<Matching Error\>\"", "]"}]}], "]"}]}]}], 
     "]"}], ",", 
    RowBox[{"If", "[", 
     RowBox[{"$Notebooks", ",", 
      RowBox[{"Print", "[", "\"\<Off\>\"", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Recurrence Relations Input and Recursion Function Generation", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.6.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "pp_", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "pp", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[TextData[{
 "Most ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", 
    RowBox[{"...", " "}]], TraditionalForm]]],
 "equations below were entered by the palettes which show up when \
$PutPalettes is set to True.\nExamples of alternative (hand) input are  (50) \
, (201) -- (205)."
}], "Text"],

Cell[CellGroupData[{

Cell["Input (35)", "Subsection",
 CellTags->"T1.6.1.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "35"], "=", "\n", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["\[Nu]", "1"], "\[ScriptCapitalC]", " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}], 
        SubscriptBox["\[CapitalDelta]", "1"]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"3", ",", "4", ",", "5"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True]}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "5"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "6"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "35"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "35"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (35)", "Subsection",
 CellTags->"T1.6.2.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "351"], "=", 
  RowBox[{
   SubscriptBox["s", "35"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}], ")"}]}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["1", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_1)"
}], "Subsubsection",
 CellTags->"T1.6.2.1.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "351"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "351"], ",", "\"\<(35_1)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_2)"
}], "Subsubsection",
 CellTags->"T1.6.2.2.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "352"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "2"}], ",", 
      RowBox[{"3", "\[LeftRightArrow]", "4"}]}], "]"}], ",", "\"\<(35_2)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["3", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_3)"
}], "Subsubsection",
 CellTags->"T1.6.2.3.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "353"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "4"}]}], "]"}], ",", "\"\<(35_3)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SuperscriptBox["4", 
    RowBox[{"+", " "}]], TraditionalForm]]],
 " (35_4)"
}], "Subsubsection",
 CellTags->"T1.6.2.4.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "354"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["h", "351"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "4"}], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], ",", "\"\<(35_4)\>\"",
     ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3501)", "Subsection",
 CellTags->"T1.6.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"m1", ",", "m5", ",", "m5", ",", "m4", ",", "m5", ",", "m5"}], 
   "]"}], "//", "Factor"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "1"], "=", "0"}], "  ", ";", "  ", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", 
     SubscriptBox["m", "5"]}]}], ";", "   ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "5", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "3501"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       SubscriptBox["\[Nu]", "5"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "5"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["5", "+"],
          FontWeight->"Bold"],
         Schiebe[5, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         RowBox[{
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"], " "}],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "3501"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "3501"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3501) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.4.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3501"], "=", 
  SubscriptBox["s", "3501"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3501"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "3501"], ",", "\"\<(3501)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "m5"}], ",", 
      RowBox[{"m3", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3502)", "Subsection",
 CellTags->"T1.6.5.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"m1", ",", "m5", ",", "m5", ",", "m4", ",", "m5", ",", "m5"}], 
   "]"}], "//", "Factor"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "4"], "=", "0"}], "  ", ";", "  ", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", 
     SubscriptBox["m", "5"]}]}], ";", "   ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "5", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "3502"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       SubscriptBox["\[Nu]", "5"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "5"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["5", "+"],
          FontWeight->"Bold"],
         Schiebe[5, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         RowBox[{
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"], " "}],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "3502"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "3502"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3502) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.6.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3502"], "=", 
  SubscriptBox["s", "3502"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3502"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3502"], ",", "\"\<(3502)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m4", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "m5"}], ",", 
      RowBox[{"m3", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3503) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.7.1"],

Cell[BoxData[
 RowBox[{"Cayley", "[", 
  RowBox[{"m5", ",", "0", ",", "m3", ",", "m5", ",", "m5", ",", "m5"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3503"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "3502"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}], ",", " ", 
    RowBox[{"3", "\[LeftRightArrow]", "4"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3503"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3503"], ",", "\"\<(3503)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "0"}], ",", 
      RowBox[{"m1", ":>", "m5"}], ",", 
      RowBox[{"m4", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "MakeFun (3504) \[CapitalDelta]=0 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["p", "2"], "=", 
     SuperscriptBox["m", "2"]}], ")"}], TraditionalForm]]]
}], "Subsection",
 CellTags->"T1.6.8.1"],

Cell[BoxData[
 RowBox[{"Cayley", "[", 
  RowBox[{"m5", ",", "m2", ",", "0", ",", "m5", ",", "m5", ",", "m5"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "3504"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "3502"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", " ", 
    RowBox[{"2", "\[LeftRightArrow]", "4"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "3504"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "3504"], ",", "\"\<(3504)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{"PP", "===", 
      RowBox[{"m5", "^", "2"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m3", ":>", "0"}], ",", 
      RowBox[{"m1", ":>", "m5"}], ",", 
      RowBox[{"m4", ":>", "m5"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (41)", "Subsection",
 CellTags->"T1.6.9.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "41"], "=", "\n", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["\[Nu]", "5"], " ", "\[ScriptCapitalC]", " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["5", "+"],
       FontWeight->"Bold"],
      Schiebe[5, "+"]], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}]]}], " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"1", ",", "3", ",", "6"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"]]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
        SubscriptBox["\[CapitalDelta]", "5"]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"]]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"]], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"]]}], "+", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", "4"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"]], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"]]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"]], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"]], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"]]}], ")"}]}], "+", 
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "5"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "41"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "41"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (41)", "Subsection",
 CellTags->"T1.6.10.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "41"], "=", 
  RowBox[{
   SubscriptBox["s", "41"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "5"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "5"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "41"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "41"], ",", "\"\<(41)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "5"], ">", "1"}], "&&", 
      RowBox[{
       RowBox[{
        StyleBox["Cayley",
         FontColor->RGBColor[1, 0, 0.4]], "[", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "1"],
          m1], ",", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          m2], ",", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          m3], ",", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          m4], ",", 
         InterpretationBox[
          SubscriptBox["m", "5"],
          m5], ",", 
         SqrtBox[
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (43)", "Subsection",
 CellTags->"T1.6.11.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SuperscriptBox["p", "2"]]],
 " and ( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["m", "5", "2"], "==", "0"}], TraditionalForm]]],
 "  &&  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
 "==",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 " && ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "== ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "43"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "43"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "43"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (44)", "Subsection",
 CellTags->"T1.6.12.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SuperscriptBox["p", "2"]]],
 " and ( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["m", "5", "2"], "==", "0"}], TraditionalForm]]],
 "  &&  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
 "==",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 " && ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "== ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "44"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "4"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["5", "-"],
            FontWeight->"Bold"],
           Schiebe[5, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "44"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "44"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (45)", "Subsection",
 CellTags->"T1.6.13.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "45"], "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "3", ",", "6"}]], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}], "\[Equal]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "1", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "6"}]], 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True]}], ")"}]}], "+", " ", 
          RowBox[{
           SubscriptBox["\[Nu]", "4"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["4", "+"],
             FontWeight->"Bold"],
            Schiebe[4, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["5", "-"],
               FontWeight->"Bold"],
              Schiebe[5, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{"-", 
           SubscriptBox["\[CapitalSigma]", "5"]}], "-", 
          SubscriptBox["\[Nu]", "1"], "+", 
          RowBox[{"3", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "5"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["5", "+"],
             FontWeight->"Bold"],
            Schiebe[5, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], "+", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["F",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TFI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4], 
        Subscript[\[Nu], 5]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "45"], "=", 
  RowBox[{"OperatorApplyF", "[", 
   SubscriptBox["e", "45"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (43) and (44)", "Subsection",
 CellTags->"T1.6.14.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "43"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["s", "43"], ",", "\"\<(43)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", 
       SubscriptBox["\[Nu]", "3"], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "5"]}], "-", 
       RowBox[{"(", 
        RowBox[{"d", "/.", 
         RowBox[{"dd_Symbol", ":>", "4"}]}], ")"}]}], "=!=", "0"}], "]"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m5", ":>", "0"}], ",", 
      RowBox[{"m4", ":>", "m3"}], ",", 
      RowBox[{"m2", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "44"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["s", "44"], ",", "\"\<(44)\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m5", ":>", "0"}], ",", 
      RowBox[{"m4", ":>", "m3"}], ",", 
      RowBox[{"m2", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (50)", "Subsection",
 CellTags->"T1.6.15.1"],

Cell["\<\
Subscript[f, 50] = 
   HoldForm[TFR[d_, PP_, 
      {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {(n3_)?PQ, m3_}, 
       {(n4_)?PQ, m4_}, {(n5_)?PQ, m5_}}] := 
     (TComment[\"(50)\", TFIC[d, PP, 
          {{n1, m1}, {n2, m2}, {n3, m3}, {n4, m4}, 
           {n5, m5}}]]; 
        Block[{r1 = 3 - d, r2 = 4 - d, 
          r3 = 
           -(m1^2*m2^2*m3^2) + m2^4*m3^2 + m2^2*m3^4 + 
            m1^4*m4^2 - m1^2*m2^2*m4^2 - 
            m1^2*m3^2*m4^2 - m2^2*m3^2*m4^2 + 
            m1^2*m4^4 + m1^2*m2^2*m5^2 - 
            m2^2*m3^2*m5^2 - m1^2*m4^2*m5^2 + 
            m3^2*m4^2*m5^2 + m1^2*m3^2*PP - 
            m2^2*m3^2*PP - m1^2*m4^2*PP + m2^2*m4^2*PP - 
            m1^2*m5^2*PP - m2^2*m5^2*PP - m3^2*m5^2*PP - 
            m4^2*m5^2*PP + m5^4*PP + m5^2*PP^2, 
          r4 = -4 + d, r5 = -3 + d, r6 = -7 + 2*d, 
          r7 = -(m2^2*r4) + m3^2*r5 - m5^2*r5 + m4^2*r6, 
          r8 = -10 + 3*d, r9 = -m3^2 + m4^2, 
          r10 = -11 + 3*d, 
          r11 = -(m4^2*r10) - 3*m5^2*r5 + m3^2*r6, 
          r12 = 
           m2^2*r11 + m5^4*r5 + m2^4*r8 + m4^2*r4*r9 + 
            m5^2*r5*r9, r13 = -6 + d, 
          r14 = -(m4^2*r13) + m5^2*r5, 
          r15 = -3*m3^2 - m4^2*r10 + m5^2*r5, 
          r16 = 
           m3^2*r14 + m2^2*r15 + m2^4*r4 - 2*m3^4*r5 - 
            3*m4^2*m5^2*r5 + m5^4*r5 + m4^4*r8, 
          r17 = m3^2 + 2*m4^2, r18 = -1 + d, 
          r19 = -(m3^2*r13) - 2*m4^2*r18 + 2*m5^2*r5, 
          r20 = 
           2*m2^4 - 3*m3^4 + d*m3^4 - 3*m3^2*m4^2 + 
            2*m4^4 + m2^2*r19 - 2*m5^4*r5 + m5^2*r17*r5, 
          r21 = 2 + d, r22 = 
           2*m5^2 - m4^2*r21 + m2^2*r4 - m3^2*r4, 
          r23 = -7 + d, r24 = 6*m4^2 - 4*m5^2, 
          r25 = 3*m3^2 + m4^2*r18 + m5^2*r23, 
          r26 = 
           -6*m4^4 - 2*m4^2*m5^2*r23 + m3^2*r24 + 
            2*m2^2*r25 - 2*m2^4*r4 - m3^4*r4, 
          r27 = m3^4 + m3^2*m4^2 - 2*m4^4, 
          r28 = m3^2 + m4^2*r23, 
          r29 = -2*m4^2*r18 + m3^2*r21 + 2*m5^2*r23, 
          r30 = -6*m2^4 + 2*m5^2*r28 - m2^2*r29 + r27*r4, 
          r31 = m1^4*r22 + m1^2*r26 + m3^2*r30 + m1^6*r4, 
          r32 = m3 - m4, r33 = m3 + m4, 
          r34 = 2*m3^2 + m4^2, r35 = -17 + 3*d, 
          r36 = m3^2*r23 + m4^2*r35, r37 = -9 + d, 
          r38 = 17 - 3*d, 
          r39 = 5*m3^2 + m4^2*r37 + m5^2*r38, 
          r40 = 1 + d, r41 = 
           5*m4^2 - m5^2*r23 + m2^2*r4 - m3^2*r40, 
          r42 = 
           m5^4*r35 - m5^2*r36 + m2^2*r39 - 2*m1^4*r4 + 
            m2^4*r4 - r32*r33*r34*r4 + m1^2*r41, 
          r43 = m1^2 - m2^2 + m3^2 - m4^2, 
          r44 = 2*m5^2*r13 + r4*r43, 
          r45 = 
           m3^4*r12 + m1^4*r16 + m1^2*m3^2*r20 + 
            PP*r31 + PP^2*r42 + PP^3*r44 + m1^6*r7, 
          r46 = m2^2 + m4^2, r47 = 13 - 3*d, 
          r48 = m4^2*r47 - m5^2*r5 + m3^2*r6, 
          r49 = m5^4*r5 + m5^2*r32*r33*r5 + m4^2*r4*r9, 
          r50 = -18 + 4*d, r51 = -(m4^2*r13) - 3*m5^2*r5, 
          r52 = 
           2*m3^4*r5 + m4^2*m5^2*r5 + m5^4*r5 + 
            m4^4*r50 + m3^2*r51, 
          r53 = -3*m4^2 + 2*m5^2*r5, 
          r54 = 
           m4^4*r47 + 2*m3^4*r5 + m4^2*m5^2*r5 - 
            2*m5^4*r5 + m3^2*r53, 
          r55 = m4^2*r13 + 6*m3^2*r5 - 2*m5^2*r5, 
          r56 = 3*m4^2 + m3^2*r5 - m5^2*r5, 
          r57 = 7 - 2*d, 
          r58 = m3^2*r5 + 3*m5^2*r5 + m4^2*r57, 
          r59 = 
           m2^6*r4 + m2^2*m4^2*r55 + m2^4*r56 + m4^4*r58, 
          r60 = m4^2 - m3^2*r5, r61 = 10 - 3*d, 
          r62 = 2*m5^2 + 3*m4^2*r4 + m3^2*r61, 
          r63 = m4^2 + m5^2*r5, 
          r64 = 
           -4*m4^2*m5^2 + 3*m4^4*r4 - 2*m3^4*r5 + 
            2*m3^2*r63, r65 = m4^2 + m3^2*r5 - m5^2*r5, 
          r66 = 2*m3^2*r5 + 2*m5^2*r5 + m4^2*r61, 
          r67 = 3*m2^4*r4 + 2*m2^2*r65 + m4^2*r66, 
          r68 = 
           -3*m2^6*r4 - 2*m1^4*m4^2*r5 + 
            2*m4^2*m5^2*r60 + m2^4*r62 + m2^2*r64 + 
            m1^2*r67 - 3*m4^4*r4*r9, r69 = -13 + 3*d, 
          r70 = m3^2*r5 + m4^2*r69, 
          r71 = m4^2 - 3*m2^2*r4 - m3^2*r5 + m5^2*r5, 
          r72 = -5 + d, r73 = m3^2 + m5^2*r69 + m4^2*r72, 
          r74 = 
           3*m2^4*r4 - m5^4*r5 + m5^2*r70 + m1^2*r71 + 
            m2^2*r73 + 3*m4^2*r4*r9, 
          r75 = m1^2 - m2^2 + m3^2 - m4^2 - 2*m5^2, 
          r76 = 
           m2^8*r4 + m2^6*r48 + m4^4*r49 + 
            2*m1^4*m4^2*r46*r5 + m2^4*r52 + 
            m2^2*m4^2*r54 - m1^2*r59 + PP*r68 + 
            PP^2*r74 + PP^3*r4*r75, 
          r77 = m3^2 - m4^2 + m5^2, 
          r78 = -2*m2^2 + m4^2 + m5^2, 
          r79 = -2*m1^2 - m3^2 - m4^2 + m5^2, 
          r80 = 
           m1^4 + PP^2 - m1^2*r77 + m3^2*r78 + PP*r79, 
          r81 = m3^2 + m4^2, r82 = m3^2 + m4^2 + m5^2, 
          r83 = m1^2 + m3^2 - m5^2, 
          r84 = 
           m1^4 + 2*m3^2*m4^2 - m4^4 + PP^2 - m2^2*r77 + 
            m5^2*r81 - m1^2*r82 - 2*PP*r83, 
          r85 = m1 - m3, r86 = -PP + r85^2, 
          r87 = m1 + m3, r88 = -PP + r87^2, 
          r89 = m2 - m4, r90 = m2 + m4, 
          r91 = -m1^2 - m2^2 - m3^2 - m4^2 + 2*m5^2, 
          r92 = PP^2 + r85*r87*r89*r90 + PP*r91, 
          r93 = m2^2 - 3*m4^2, r94 = m4 - m5, 
          r95 = m4 + m5, r96 = 2*m3^2 - m4^2 + m5^2, 
          r97 = -m1^2 - m2^2 - 2*m3^2 + 3*m5^2, 
          r98 = 
           PP^2 + m1^2*r93 + m2^2*r94*r95 + m4^2*r96 + 
            PP*r97, r99 = -2 + d, r100 = -m1^2 + m2^2, 
          r101 = m2^2 + m1^2*r72, 
          r102 = m3^2*r101 - 2*m3^4*r4 + m2^2*r100*r4, 
          r103 = 9 - 2*d, r104 = m2^2*r23 + m3^2*r72, 
          r105 = 
           2*m1^4 + m2^4*r103 + m1^2*r104 + m2^2*m3^2*r5, 
          r106 = -9 + 2*d, r107 = m1 - m2, 
          r108 = m1 + m2, 
          r109 = -(r106*r107*r108) + m3^2*r4, 
          r110 = 
           m2^2*r102 + m4^2*r105 + m4^4*r109 - m4^6*r4, 
          r111 = m1^2 + m2^2, 
          r112 = m2^2*r4 - 2*m1^2*r72, r113 = -19 + 5*d, 
          r114 = m3^2*r112 + m3^4*r113 + m2^2*r111*r4, 
          r115 = m3^2*r5 + m2^2*r6, 
          r116 = 
           2*m1^4 + 2*m1^2*r115 + m2^4*r13 + m3^4*r72 - 
            2*m2^2*m3^2*r99, 
          r117 = m2^2*r106 + m3^2*r4 - m1^2*r99, 
          r118 = 
           m2^2*r114 - m4^2*r116 + m4^4*r117 - m4^6*r6, 
          r119 = -(m2^2*r5) + 2*m3^2*r57 + m1^2*r72, 
          r120 = m1^2*r18 + 2*m3^2*r4 - m2^2*r5, 
          r121 = m2^2*r119 + m4^2*r120 + 2*m4^4*r5, 
          r122 = m3^2*r5 + m4^2*r72, 
          r123 = -2*m2^2*r4 - m4^2*r69 + m3^2*r72, 
          r124 = 
           m2^2*r122 + m1^2*r123 + 2*m2^4*r4 + 
            2*m4^2*r32*r33*r4, 
          r125 = -3*m4^2 + m2^2*r4 - m3^2*r72, 
          r126 = m3^2 - m4^2*r72, 
          r127 = 
           -2*m3^2*m4^2 - 2*m1^2*r125 - 2*m2^2*r126 + 
            m4^4*r18 - 4*m2^4*r4 + m3^4*r72, 
          r128 = 
           -(m2^2*r18) + 2*m4^2*r5 + m1^2*r72 + 
            2*m3^2*r72, 
          r129 = 
           m5^2*r127 - m5^4*r128 - r124*r32*r33 + 
            m5^6*r72, r130 = -m1^2 + m2^2 + m3^2 - m4^2, 
          r131 = -14 + 3*d, 
          r132 = 
           -(m3^2*r13) + m4^2*r131 + m1^2*r4 + 5*m2^2*r4, 
          r133 = -2*m5^4 + m5^2*r132 + r130*r32*r33*r4, 
          r134 = 
           m5^2*r118 + m5^4*r121 + PP*r129 + PP^2*r133 + 
            r110*r32*r33 - 2*m5^2*PP^3*r4 + 
            m5^6*r5*r89*r90, r135 = m3 - m4 - m5, 
          r136 = m3 + m4 - m5, r137 = m3 - m4 + m5, 
          r138 = m3 + m4 + m5, r139 = -10 + d, 
          r140 = 13 - 2*d, r141 = d*m4^2 + 2*m3^2*r5, 
          r142 = -8 + d, 
          r143 = m4^2*r142 - m3^2*r99 + m5^2*r99, 
          r144 = 
           m3^2*m4^2*r139 + m4^4*r140 - m5^2*r141 + 
            m2^2*r143 + m3^4*r5 + m5^4*r5, 
          r145 = 3*m3^4 - 12*m3^2*m4^2 + 7*m4^4, 
          r146 = 3*m3^2 + 4*m4^2, r147 = m3^2 - 11*m4^2, 
          r148 = 
           7*m3^4 + 10*m3^2*m4^2 - 17*m4^4 - 5*m5^4 - 
            2*m5^2*r147, r149 = m2^2 + 2*m3^2 + m4^2, 
          r150 = 
           -m2^4 + m2^2*m3^2 + m3^4 + 3*m2^2*m4^2 + 
            m3^2*m4^2 - m4^4 + m5^4 - m5^2*r149, 
          r151 = 
           3*m3^6 + 6*m3^4*m4^2 - 17*m3^2*m4^4 + 
            8*m4^6 + 3*m5^6 - m5^2*r145 - m5^4*r146 + 
            m2^2*r148 - d*r150*r77 + 6*m2^4*r94*r95, 
          r152 = -(m3^2*m4) + m4^3, r153 = 11 - 2*d, 
          r154 = -(d*m3^2*m4^2) + m4^4*r153 + m3^4*r5, 
          r155 = m4^2 + m3^2*r5, 
          r156 = r142*r32*r33 + m5^2*r99, 
          r157 = 11 - 3*d, r158 = -8 + 3*d, 
          r159 = m5^2*r158 + m4^2*r21, 
          r160 = 
           -3*m3^4 + 2*m4^2*m5^2*r106 + m5^4*r157 + 
            m3^2*r159 - m4^4*r18, 
          r161 = 
           -2*m2^6 + m5^2*r154 - 2*m5^4*r155 + 
            m2^4*r156 + m2^2*r160 + r152^2*r4 + m5^6*r5, 
          r162 = m4^2*r23 - 3*m3^2*r72, 
          r163 = -4*m2^2 + m4^2*r18 + m3^2*r23, 
          r164 = -m3^4 + m4^4, 
          r165 = 
           -(m2^4*r13) + m2^2*r162 + m1^2*r163 + 
            r164*r4 + m1^4*r99, r166 = -19 + 4*d, 
          r167 = m4^2 + m3^2*r72, r168 = m3^2 + m4^2*r69, 
          r169 = -2*r168 + 4*m2^2*r4, 
          r170 = 
           -m3^4 + m2^4*r13 - m4^4*r166 + 2*m2^2*r167 + 
            m1^2*r169 + 2*m3^2*m4^2*r72 + m1^4*r99, 
          r171 = m1^2 - m2^2 + 4*m4^2, 
          r172 = 
           -(m5^2*r170) + r165*r32*r33 + m5^6*r72 - 
            m5^4*r171*r72, r173 = m2^2 + m3^2, 
          r174 = -(m4^2*r13) + r131*r173 + m1^2*r8, 
          r175 = m5^2*r174 - 2*m5^4*r4 + r130*r32*r33*r4, 
          r176 = 
           2*m1^6*m4^2 + m1^4*r144 + m1^2*r151 + 
            m3^2*r161 + PP*r172 + PP^2*r175 - 
            2*m5^2*PP^3*r4, r177 = m3^2 + m4^2 - m5^2, 
          r178 = -2*m1^2 + m3^2 - m4^2 + m5^2, 
          r179 = -m3^2 + m4^2 + m5^2, 
          r180 = m2^2*r177 + m4^2*r178 + PP*r179, 
          r181 = -2*m2^2 - m3^2 + m4^2 + m5^2, 
          r182 = m1^2*r177 + m3^2*r181 + PP*r77, 
          r183 = -(m2^2*r13) - 2*m3^2*r5, 
          r184 = 
           m1^2*r183 - 2*m3^4*r4 + m2^4*r57 + m1^4*r72 - 
            m2^2*m3^2*r99, r185 = 3*m2^2*r4 + m3^2*r72, 
          r186 = 
           m2^4*r106 + 2*m1^2*r185 + m1^4*r40 - 
            2*m2^2*m3^2*r6, r187 = r111*r13 - m3^2*r4, 
          r188 = 
           m2^2*r184 + m4^2*r186 - m4^4*r187 + m4^6*r4, 
          r189 = 2*m1^2 + m3^2*r10 + 2*m2^2*r5, 
          r190 = m2^2*r5 + m3^2*r72 + 2*m1^2*r99, 
          r191 = m2^2*r189 - m4^2*r190 - m4^4*r5, 
          r192 = -(m3^2*r5) - m4^2*r72, 
          r193 = 2*m2^2*r4 + m4^2*r69 - m3^2*r72, 
          r194 = 
           m2^2*r192 + m1^2*r193 - 2*m2^4*r4 + 
            2*m4^2*r4*r9, r195 = m2^2 + m4^2 + m3^2*r72, 
          r196 = -(m3^2*r6) - m4^2*r72, 
          r197 = 
           m2^4*r157 + 2*m1^2*r195 + 2*m2^2*r196 + 
            2*m4^2*r17*r4 + m1^4*r72, 
          r198 = 
           m4^2*r10 - 2*m2^2*r5 + 2*m1^2*r72 + m3^2*r72, 
          r199 = 
           r107*r108*r194 + m5^2*r197 - m5^4*r198 + 
            m5^6*r72, r200 = m1^2 - m2^2 - m3^2 + m4^2, 
          r201 = m1^2 + m2^2 + m3^2 + 5*m4^2, 
          r202 = 2*m5^4 - r107*r108*r200 - m5^2*r201, 
          r203 = 
           r107*r108*r110 - m5^2*r188 - m5^4*r191 + 
            PP*r199 + 2*m5^2*PP^3*r4 + PP^2*r202*r4 + 
            m5^6*r5*r89*r90, r204 = m4^2 + m5^2, 
          r205 = -2*m5^2*r5 + m4^2*r99, r206 = 18 - 5*d, 
          r207 = m4^2*r206 + m5^2*r8 - 2*m3^2*r99, 
          r208 = 
           m2^2*r207 + 2*m2^4*r4 - m3^4*r5 - 
            m3^2*r204*r5 + r205*r94*r95, 
          r209 = 2*m3^2*m4^2 + m4^4*r142 + 2*m3^4*r5, 
          r210 = -(m4^2*r18) + m3^2*r5, r211 = 3 - 2*d, 
          r212 = m5^2 + m4^2*r157 + m3^2*r211, 
          r213 = m4^2*r106 + m3^2*r4, 
          r214 = 
           -(d*m3^4) + 10*m3^2*m4^2 - 2*d*m3^2*m4^2 - 
            2*m4^4 + d*m4^4 + 2*m5^2*r213 - 2*m5^4*r4, 
          r215 = 
           d*m3^2*m4^4 - 2*m4^6 - m5^2*r209 + 
            m5^4*r210 + m2^4*r212 + m2^2*r214 + 
            m2^6*r4 + m5^6*r5 - m3^4*m4^2*r99, 
          r216 = m4^2*r13 + m3^2*r99, 
          r217 = m3^2*r5 - m4^2*r72, r218 = 13 - 4*d, 
          r219 = 3*m3^2 + m5^2*r218 + m4^2*r23, 
          r220 = m4^2*r142 + m5^2*r61, 
          r221 = 
           2*m3^4 + 2*m4^2*m5^2 - m4^4*r13 + m3^2*r220 + 
            2*m5^4*r6, r222 = 
           m4^2*m5^2*r216 + m5^4*r217 + m2^4*r219 + 
            m2^2*r221 - m5^6*r5 + m2^6*r99, 
          r223 = -(m3^2*r10) + m4^2*r5, 
          r224 = 
           4*m3^2*m4^2 + m1^2*r122 + m4^4*r13 + 
            m2^2*r223 + m1^4*r4 - m2^4*r4 - m3^4*r99, 
          r225 = -m3^2 + m2^2*r72 - m4^2*r72, 
          r226 = -2*m4^2*r106 + 2*m3^2*r69, 
          r227 = 
           m1^4*r106 + m2^4*r106 - m4^4*r13 - 
            2*m1^2*r225 + m2^2*r226 - 4*m3^2*m4^2*r4 - 
            m3^4*r99, r228 = m3^2 + 3*m4^2, 
          r229 = 
           r107*r108*r224 - m5^2*r227 + m5^6*r72 - 
            m5^4*r228*r72, 
          r230 = m4^2*r131 + m1^2*r4 + m2^2*r4 + m3^2*r8, 
          r231 = 
           2*m5^4*r106 - m5^2*r230 - r107*r108*r200*r4, 
          r232 = 
           m1^4*r208 - m1^2*r215 - m3^2*r222 + PP*r229 + 
            PP^2*r231 + 2*m5^2*PP^3*r4 + m1^6*r7, 
          r233 = -2*m3^2 + m4^2, 
          r234 = -m2^2 + m4^2 - 2*m5^2, 
          r235 = -m1^2 + m2^2 + m5^2, 
          r236 = 
           m1^4 + m5^4 + m2^2*r233 + m1^2*r234 + 
            PP*r235 - m5^2*r46, r237 = -m2^2 + m5^2, 
          r238 = m2^2 + m3^2 - 2*m4^2 + m5^2, 
          r239 = -m1^2 + m2^2 - m5^2, 
          r240 = m1^4 + m3^2*r237 - m1^2*r238 + PP*r239, 
          r241 = m3^2 + 3*m4^2 - m5^2, 
          r242 = -3*m3^2 - m4^2 + m5^2, 
          r243 = 
           m5^4 - m5^2*r228 + m1^2*r241 + m2^2*r242 + 
            2*PP*r32*r33 + 2*m4^2*r9, 
          r244 = 2*m1^2 + m2^2 + m4^2, 
          r245 = -2*m1^2 - m3^2 - m4^2 + 3*m5^2, 
          r246 = 
           m1^4 - 3*m2^2*m3^2 + m5^4 + PP^2 + 
            m4^2*r173 - m5^2*r244 + PP*r245, 
          r247 = -2*m3^2 + m4^2 + m5^2, 
          r248 = 
           -m2^4 - m4^2*m5^2 + PP*r235 + m2^2*r247 + 
            m1^2*r46, r249 = m2^2*r153 + m1^2*r5, 
          r250 = 2*m1^2 + m2^2*r4, 
          r251 = m2^2*r13 + m1^2*r139, 
          r252 = r107*r108*r250 + m3^2*r251 - m3^4*r4, 
          r253 = m1^2*r140 + 2*m3^2*r4 + m2^2*r72, 
          r254 = 
           m3^4*r249 + m4^2*r252 + m4^4*r253 - m4^6*r4 + 
            m2^2*m3^2*r100*r99, r255 = 2*m3^2 + m4^2*r13, 
          r256 = -2*m3^2*r5 + m2^2*r99 - m4^2*r99, 
          r257 = -11 + 2*d, 
          r258 = 
           -(m4^4*r106) + 2*m3^2*m4^2*r23 - m3^4*r257, 
          r259 = 4*m4^2 + m3^2*r13, 
          r260 = 
           -2*m2^4 + 6*m3^2*m4^2 + 2*m2^2*r259 + 
            m4^4*r37 + m3^4*r5, 
          r261 = 
           -5*m3^4*m4^2 + m2^4*r255 + m1^4*r256 + 
            m2^2*r258 + m1^2*r260 - m2^6*r4 + m3^6*r5 + 
            m4^6*r6 - 3*m3^2*m4^4*r72, 
          r262 = m3^4 + m3^2*m4^2 + m4^4, 
          r263 = 
           m1^4 + m2^4 - 2*r262 + m1^2*r81 + m2^2*r81, 
          r264 = m1^2 + m2^2 - m3^2 - m4^2, 
          r265 = -(r13*r32*r33) + m2^2*r99, 
          r266 = 
           -2*m1^2*r265 - r32^2*r33^2*r4 + 
            2*m2^2*r13*r9 + m1^4*r99 + m2^4*r99, 
          r267 = m3^2*r13 + m4^2*r4, r268 = -16 + 3*d, 
          r269 = m3^2*r13 - m4^2*r268 + m2^2*r4, 
          r270 = 
           2*m2^2*r267 + 2*m1^2*r269 + 
            r13*r228*r32*r33 + m2^4*r61 + m1^4*r99, 
          r271 = 2*m2^2 + m4^2*r23 - m3^2*r72, 
          r272 = 
           -(m5^2*r270) - 2*m5^4*r271 + r266*r32*r33, 
          r273 = 2*m3^2 - 2*m4^2 - m5^2, 
          r274 = m1^2*r5 - m2^2*r5 + r273*r72, 
          r275 = 
           r200*r254 + m5^2*r261 + PP*r272 + 
            2*m5^2*PP^2*r274 + m5^4*r263*r5 - 
            m5^6*r264*r5, 
          r276 = 
           -2*m2^2*m3^2 - 2*m3^4 + 3*m3^2*m4^2 - m4^4 + 
            3*m3^2*m5^2 + 2*m4^2*m5^2 - m5^4 + 
            m1^2*r177 + PP*r77, 
          r277 = -(m4^2*r106) + m2^2*r4 - m3^2*r5, 
          r278 = 
           -2*m3^4 - m3^2*m4^2*r13 + m2^2*r196 + 
            m2^4*r4 + m4^4*r4, 
          r279 = m4^2*r131 + m3^2*r8, 
          r280 = 
           m2^2*r279 - 2*m2^4*r4 + m4^2*r32*r33*r99, 
          r281 = m1^4*r277 + m2^2*r278 + m1^2*r280, 
          r282 = 11*m3^2 - 3*d*m3^2 + 9*m4^2 - 2*d*m4^2, 
          r283 = 15 - 4*d, 
          r284 = m4^2*r106 + m2^2*r283 + m3^2*r5, 
          r285 = m4^4*r13 + 4*m3^2*m4^2*r5 - m3^4*r99, 
          r286 = -m4^2 + m3^2*r5, 
          r287 = -(m4^2*r5) + m3^2*r6, 
          r288 = 2*m2^2*r287 + m2^4*r72 - 2*r286*r81, 
          r289 = 
           -2*m3^2*m4^4 + m2^4*r282 + m1^4*r284 + 
            m2^2*r285 + m1^2*r288 - m4^6*r4 + m1^6*r5 + 
            m2^6*r6 + m3^4*m4^2*r99, 
          r290 = 2*m2^2 - m3^2 - m4^2, 
          r291 = 
           2*m1^4 + 2*m2^4 - m3^4 - m4^4 + m1^2*r290 - 
            m2^2*r81, r292 = m2^2 - m3^2 + m4^2, 
          r293 = 
           m1^4*r4 + m2^4*r4 - 2*m1^2*r292*r4 + 
            2*m2^2*r4*r9 - r32^2*r33^2*r99, 
          r294 = m3^2*r13 - m4^2*r131, 
          r295 = m2^2*r13 - r4*r81, 
          r296 = 
           m1^4*r131 + 2*m2^2*r294 - 2*m1^2*r295 - 
            2*m3^2*m4^2*r4 + m4^4*r8 - m2^4*r99 - 
            m3^4*r99, r297 = 2*m4^2 - m2^2*r5 + m1^2*r72, 
          r298 = 
           r107*r108*r293 - m5^2*r296 + 2*m5^4*r297, 
          r299 = -2*m5^2*r32*r33*r5 + 2*m5^4*r72, 
          r300 = 
           r130*r281 - m5^2*r289 + PP*r298 + PP^2*r299 - 
            m5^6*r264*r5 + m5^4*r291*r5, 
          r301 = 
           -2*m2^2*m3^2 - m3^2*m4^2 + m4^4 + m5^4 - 
            m5^2*r17 + m1^2*r177 + PP*r77, 
          r302 = m2^2 + 2*m3^2 - 3*m4^2, 
          r303 = -m1^2 + m2^2 - 3*m5^2, 
          r304 = 
           m1^4 - m2^2*m4^2 - m5^4 + m5^2*r149 - 
            m1^2*r302 + PP*r303, 
          r305 = 
           -((r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
                 TBI[d, PP, {{2, m1}, {1, m3}}])/(4*r3)) 
+ (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/(4*r3) - 
            (r80*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/2 - 
            (r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/(4*r3) - 
            (r84*TBI[d, PP, {{2, m2}, {1, m4}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{2, m3}, {2, m1}}] + 
            (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/(4*r3) - 
            (r92*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            (r98*TBI[d, PP, {{2, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m1}, {1, m3}}] - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m3}, {1, m1}}] - 
            r3*TFI[r99, PP, 
              {{1, m1}, {1, m2}, {1, m3}, {1, m4}, 
               {1, m5}}] - 
            (r134*TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {1, m3}}])/
             (4*r3) - (r135*r136*r137*r138*
               TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {2, m3}}])/2 
- (r176*TVI[d, PP, {{1, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/(4*r3) + 
            (r180*TVI[d, PP, 
                {{1, m5}, {1, m2}, {2, m3}, {2, m4}}])/2 
- r182*TVI[d, PP, {{1, m5}, {1, m2}, {3, m3}, 
               {1, m4}}] + 
            (r203*TVI[d, PP, 
                {{1, m5}, {1, m3}, {2, m2}, {1, m1}}])/
             (4*r3) + (r232*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {2, m2}}])/2 
+ r240*TVI[d, PP, {{1, m5}, {1, m4}, {3, m1}, 
               {1, m2}}] + 
            (r45*TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r86*r88*
               TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {2, m3}}])/2 
+ (r92*TVI[d, PP, {{1, m5}, {2, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m2}, {1, m3}, {1, m4}}])/
             (4*r3) - (r243*
               TVI[d, PP, 
                {{1, m5}, {2, m2}, {2, m3}, {1, m4}}])/2 
+ (r45*TVI[d, PP, {{1, m5}, {2, m3}, {1, m2}, 
                 {1, m1}}])/(4*r3) + 
            (r86*r88*TVI[d, PP, 
                {{1, m5}, {2, m3}, {1, m2}, {2, m1}}])/2 
+ (r246*TVI[d, PP, {{1, m5}, {2, m3}, {2, m2}, 
                 {1, m1}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r248*
               TVI[d, PP, 
                {{1, m5}, {2, m4}, {2, m1}, {1, m2}}])/2 
+ r86*r88*TVI[d, PP, {{1, m5}, {3, m1}, {1, m4}, 
               {1, m3}}] + 
            r86*r88*TVI[d, PP, 
              {{1, m5}, {3, m3}, {1, m2}, {1, m1}}] + 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r276*
               TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {2, m3}}])/2 
- (r180*TVI[d, PP, {{2, m5}, {1, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {1, m4}}])/
             (4*r3) + (r180*
               TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {2, m4}}])/2 
- (r276*TVI[d, PP, {{2, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/2 - 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {1, m1}}])/
             (4*r3) - (r240*
               TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {2, m1}}])/2 
- (r236*TVI[d, PP, {{2, m5}, {1, m3}, {2, m2}, 
                 {1, m1}}])/2 + 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {2, m2}}])/2 
+ (r240*TVI[d, PP, {{2, m5}, {1, m4}, {2, m1}, 
                 {1, m2}}])/2 + 
            (r182*TVI[d, PP, 
                {{2, m5}, {2, m1}, {1, m4}, {1, m3}}])/2 
- (r301*TVI[d, PP, {{2, m5}, {2, m2}, {1, m3}, 
                 {1, m4}}])/2 - 
            (r304*TVI[d, PP, 
                {{2, m5}, {2, m3}, {1, m2}, {1, m1}}])/2 
+ (r248*TVI[d, PP, {{2, m5}, {2, m4}, {1, m1}, 
                 {1, m2}}])/2, r306}, 
         r306 = Factor /@ r305/(PP*r1*r2); r306]) /; 
         n1 === 1 && n2 === 1 && n3 === 1 && n4 === 1 && n5 === 1 &&  
       MatchQ[d, _Symbol + _Integer?Positive] && PP =!= 0 &&
       Cayley[m1, m2, m3, m4, m5, Sqrt[PP]] =!= 0]; \
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (51)", "Subsection",
 CellTags->"T1.6.16.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "51"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "3", ",", "4"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "4"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "51"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "51"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (51)", "Subsection",
 CellTags->"T1.6.17.1"],

Cell[BoxData[
 SubscriptBox["s", "51"]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "51"], "=", 
  RowBox[{
   SubscriptBox["s", "51"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "51"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "51"], ",", "\"\<(51)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "3", ",", "4"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "51"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (52)", "Subsection",
 CellTags->"T1.6.18.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "52"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"2", ",", "4", ",", "6"}]], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "4", "2"], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", 
           SubsuperscriptBox["m", "2", "2"]}], "-", 
          SubscriptBox["u", 
           RowBox[{"4", ",", "2", ",", "6"}]]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "4"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"2", ",", "4", ",", "6"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"3", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "4", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "4"], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "52"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "52"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (52)", "Subsection",
 CellTags->"T1.6.19.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "52"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "52"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "52"], ",", "\"\<(52)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "52"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (53)", "Subsection",
 CellTags->"T1.6.20.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "53"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "51"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "53"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "53"], ",", "\"\<(53)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"3", ",", "1", ",", "4"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "53"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (54)", "Subsection",
 CellTags->"T1.6.21.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "54"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "3", "2"], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "1"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "54"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "54"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (54)", "Subsection",
 CellTags->"T1.6.22.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "54"], "=", 
  RowBox[{
   SubscriptBox["s", "54"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "54"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "54"], ",", "\"\<(54)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", " ", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "54"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (55)", "Subsection",
 CellTags->"T1.6.23.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "55"], "=", 
   RowBox[{
    RowBox[{"2", " ", 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "4"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"3", ",", "1", ",", "4"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "1"], "-", 
         SubscriptBox["\[Nu]", "2"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "55"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "55"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (55)", "Subsection",
 CellTags->"T1.6.24.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "55"], "=", 
  RowBox[{
   SubscriptBox["s", "55"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "55"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "55"], ",", "\"\<(55)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", " ", 
      RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "55"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (56)", "Subsection",
 CellTags->"T1.6.25.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "56"], "=", 
   RowBox[{
    RowBox[{"2", " ", 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "4"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "d"}], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         SubscriptBox["\[Nu]", "2"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "3"]}], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "56"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "56"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (56)", "Subsection",
 CellTags->"T1.6.26.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "56"], "=", 
  RowBox[{
   SubscriptBox["s", "56"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "56"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "56"], ",", "\"\<(56)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "4"], ">", "1"}], "&&", " ", 
      RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "===", "1"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "56"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (59)", "Subsection",
 CellTags->"T1.6.27.1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 but all  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " != 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "59"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["u", 
         RowBox[{"1", ",", "3", ",", "4"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
        SubsuperscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}], "2"]}], "+", "\n", "\t", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "-", "\n", "\t", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"4", 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "3", "2"]}], "+", 
          SubsuperscriptBox["u", 
           RowBox[{"4", ",", "1", ",", "3"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "-", "\n", "\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"], "+", "1"}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "+", "1"}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "59"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "59"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (59)", "Subsection",
 CellTags->"T1.6.28.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "59"], "=", 
  SubscriptBox["s", "59"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "59"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "59"], ",", "\"\<(59)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "3", ",", "4"}]], "==", "0"}], "&&", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", 
          SubsuperscriptBox["m", "3", "2"], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"4", " ", 
          SubsuperscriptBox["m", "3", "2"], " ", 
          SubscriptBox["\[Nu]", "3"], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "+", 
         RowBox[{"2", " ", "d", " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"4", " ", 
          SubscriptBox["\[Nu]", "3"], " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "-", 
         RowBox[{"2", " ", 
          SubscriptBox["u", 
           RowBox[{"1", ",", "3", ",", "4"}]], " ", 
          SubsuperscriptBox["m", "1", "2"]}], "+", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], " ", 
          SubsuperscriptBox["u", 
           RowBox[{"4", ",", "1", ",", "3"}], "2"]}], "-", 
         SubsuperscriptBox["u", 
          RowBox[{"4", ",", "1", ",", "3"}], "2"]}], ")"}], "  ", "=!=", 
       "0"}], "&&", 
      RowBox[{"n1", " ", "===", " ", "1"}], "&&", " ", 
      RowBox[{"n2", " ", "===", " ", "1"}], "&&", " ", 
      RowBox[{"n4", " ", "===", " ", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Integer"}], "]"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (60) ", "Subsection",
 CellTags->"T1.6.29.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "60"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"4", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         SubscriptBox["\[Nu]", "3"], "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "60"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "60"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (60)", "Subsection",
 CellTags->"T1.6.30.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "60"], "=", 
  RowBox[{
   SubscriptBox["s", "60"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["notnecessaryf", "60"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "60"], ",", "\"\<(60)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", " ", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["notnecessaryf", "60"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (61) ", "Subsection",
 CellTags->"T1.6.31.1"],

Cell[TextData[{
 " ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "61"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["4", "-"],
         FontWeight->"Bold"],
        Schiebe[4, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "61"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "61"], "]"}]}]], "Input"],

Cell[TextData[{
 "For  ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )\nuse ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3], 
         Subscript[\[Nu], 4]}],
       Editable->True], "==", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3], 
         Subscript[\[Nu], 4]}],
       Editable->True]}], ")"}], " "}], TextForm]]],
 " and switch the masses accordingly."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (61)", "Subsection",
 CellTags->"T1.6.32.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " )  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "61"], "=", 
  SubscriptBox["s", "61"]}]], "Input"],

Cell["\<\
Don't need the restriction to n1=1 and n4=1 (I hope ...). Changed September \
24th 2003.\
\>", "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "61"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "61"], ",", "\"\<(61)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n1", ":>", "1"}], ",", 
      RowBox[{"n4", ":>", "1"}]}], "}"}]}], "]"}]}]], "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "61"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "61"], ",", "\"\<(61)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6153)", "Subsection",
 CellTags->"T1.6.33.1"],

Cell[TextData[{
 " ( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 ")  \[DoubleLongRightArrow] ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "3", ",", "4"}]], TraditionalForm]]],
 "== 0 )"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6153"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "61"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell["\<\
Don't need the restriction to n1=1 and n4=1 (I hope ...). Changed September \
24th 2003.\
\>", "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6153"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6153"], ",", "\"\<(6153)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n3", ":>", "1"}], ",", 
      RowBox[{"n4", ":>", "1"}]}], "}"}]}], "]"}]}]], "SmallText"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6153"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6153"], ",", "\"\<(6153)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "===", "0"}], "&&", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1], "===", 
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4]}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (62) ", "Subsection",
 CellTags->"T1.6.34.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0 && ",
 Cell[BoxData[
  SubsuperscriptBox["m", "1", "2"]]],
 "== ",
 Cell[BoxData[
  SubsuperscriptBox["m", "3", "2"]]],
 " ) \[DoubleLongRightArrow] (",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 "== 0 )\nlower ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Nu]", "2"], TraditionalForm]]],
 " via (52) to one first"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "62"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "-", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"]}], "+", "2"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["4", "+"],
       FontWeight->"Bold"],
      Schiebe[4, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", "\n", "\t\t", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "1", "2"], "-", 
          SubscriptBox["u", 
           RowBox[{"6", ",", "1", ",", "2"}]]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "4"], "-", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True]}], "-", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       SubscriptBox["\[CapitalSigma]", "4"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "62"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "62"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (62)", "Subsection",
 CellTags->"T1.6.35.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "62"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "62"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "4"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "4"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "62"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "62"], ",", "\"\<(62)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "=!=", 
       InterpretationBox[
        SubsuperscriptBox["m", "2", "2"],
        m2^2]}], "&&", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "2"]}], "+", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "4"]}]}], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "3"],
        m3], "\[RuleDelayed]", 
       InterpretationBox[
        SubscriptBox["m", "1"],
        m1]}], ",", 
      RowBox[{"n2", "\[RuleDelayed]", "1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (63) ", "Subsection",
 CellTags->"T1.6.36.1"],

Cell[TextData[{
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0  && ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 " != 0\nreplace (55) by the following"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "63"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], "-", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "3", "2"], "-", 
          SubsuperscriptBox["m", "1", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "3"], "-", 
          SubscriptBox["\[Nu]", "1"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", "\n", "\t\t", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "1", "2"], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubsuperscriptBox["m", "1", "2"], 
              SubscriptBox["\[Nu]", "1"], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True]}], "+", 
             RowBox[{"2", 
              SubsuperscriptBox["m", "3", "2"], 
              SubscriptBox["\[Nu]", "3"], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "+"],
                FontWeight->"Bold"],
               Schiebe[3, "+"],
               Editable->True]}], "-", 
             SubscriptBox["\[CapitalSigma]", "4"], "-", "2"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "-", 
          RowBox[{"4", 
           SubsuperscriptBox["m", "2", "2"], 
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["4", "-"],
             FontWeight->"Bold"],
            Schiebe[4, "-"],
            Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           SubscriptBox["\[Nu]", "3"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              StyleBox[
               SuperscriptBox["4", "-"],
               FontWeight->"Bold"],
              Schiebe[4, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "63"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "63"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (63)", "Subsection",
 CellTags->"T1.6.37.1"],

Cell[TextData[{
 Cell[BoxData[
  SubsuperscriptBox["m", "4", "2"]]],
 "== 0  && ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"1", ",", "3", ",", "4"}]]}], TraditionalForm]]],
 " != 0"
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "63"], "=", 
  RowBox[{"\[Nu]explicit", "[", 
   SubscriptBox["s", "63"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "63"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "63"], ",", "\"\<(63)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", " ", "=!=", " ", 
       RowBox[{"m2", "^", "2"}]}], "&&", 
      RowBox[{
       RowBox[{
        RowBox[{"Cayley", "[", 
         RowBox[{"1", ",", "3", ",", "4"}], "]"}], "[", 
        RowBox[{"{", 
         RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}], "]"}], "=!=",
        "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      InterpretationBox[
       SubscriptBox["m", "4"],
       m4], "\[RuleDelayed]", "0"}], "}"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "63"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "63"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "63"], ",", "\"\<(63)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", " ", "=!=", " ", 
       RowBox[{"m2", "^", "2"}]}], "&&", 
      RowBox[{
       RowBox[{
        RowBox[{"Cayley", "[", 
         RowBox[{"1", ",", "3", ",", "4"}], "]"}], "[", 
        RowBox[{"{", 
         RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}], "]"}], "=!=",
        "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       InterpretationBox[
        SubscriptBox["m", "4"],
        m4], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{"n1", ":>", "1"}], ",", 
      RowBox[{"n3", ":>", "1"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

CheckTVIRecursion not applicable for special indices since A's and 
B's appear .
\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
 CellTags->"T1.6.38.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["e", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"2", ",", "4", ",", "6"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"formal", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["e", "55"], "/.", 
       RowBox[{
        RowBox[{"TVI", "[", "__", "]"}], "->", "1"}]}], ",", 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["4", "+"],
        FontWeight->"Bold"],
       Schiebe[4, "+"],
       Editable->True]}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "/.", "formal"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   RowBox[{
    SubscriptBox["r", "6552"], "/.", "formal"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "4"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "6552"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "6552"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "4"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "4"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6552"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "6552"], ",", "\"\<(6552)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["\[Nu]", "2"], "-", 
            SubscriptBox["\[Nu]", "4"]}], ")"}], " ", 
          InterpretationBox[
           SubsuperscriptBox["m", "4", "2"],
           m4^2]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             SubscriptBox["\[Nu]", "2"]}], "-", "d"}], ")"}], " ", 
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]]}], "+", 
         RowBox[{
          SubscriptBox["\[Nu]", "2"], " ", 
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]]}]}], "]"}], "=!=", "0"}]}], 
     "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
 CellTags->"T1.6.39.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6552"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["e", "52"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[CapitalDelta]", 
     RowBox[{"2", ",", "4", ",", "6"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["r", "6552"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "6552"], "[", 
  RowBox[{"[", 
   RowBox[{"2", ",", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "6552"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "6552"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "5"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["n", "6552"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "5"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6552"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["s", "6552"], ",", "\"\<(6552)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "4", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "2"}], " ", 
           SubscriptBox["\[Nu]", "2"], " ", 
           SubsuperscriptBox["m", "4", "2"]}], "+", 
          RowBox[{"2", " ", 
           SubscriptBox["\[Nu]", "4"], " ", 
           SubsuperscriptBox["m", "4", "2"]}], "+", 
          RowBox[{"d", " ", 
           SubscriptBox["u", 
            RowBox[{"2", ",", "4", ",", "6"}]]}], "-", 
          RowBox[{"3", " ", 
           SubscriptBox["\[Nu]", "2"], " ", 
           SubscriptBox["u", 
            RowBox[{"2", ",", "4", ",", "6"}]]}]}], ")"}], "]"}], "=!=", 
       "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6553)", "Subsection",
 CellTags->"T1.6.40.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "6553"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Nu]explicit", "[", 
      SubscriptBox["s", "55"], "]"}], "/.", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"3", ",", "1", ",", "4"}]], "->", "0"}]}], "/.", 
    RowBox[{
     SubscriptBox["u", 
      RowBox[{"6", ",", "2", ",", "4"}]], "->", "0"}]}], "/.", 
   RowBox[{
    SubscriptBox["m", "4"], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6553"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["r", "6553"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "==", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["r", "6553"], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "-", 
     RowBox[{
      SubscriptBox["r", "6553"], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6553"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6553"], ",", "\"\<(6553)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "2", ",", "4"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"1", ",", "3", ",", "4"}]], "===", "0"}]}], "]"}], ",", 
    RowBox[{"m4", ":>", "0"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (66) ", "Subsection",
 CellTags->"T1.6.41.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "66"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", "d"}], "-", 
          RowBox[{"6", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"5", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"6", 
           SubscriptBox["\[Nu]", "3"]}], "-", 
          RowBox[{"4", 
           SubscriptBox["\[Nu]", "4"]}], "-", 
          RowBox[{"6", 
           SubsuperscriptBox["m", "1", "2"], 
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True]}]}], ")"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", 
            RowBox[{"6", ",", "2", ",", "4"}]], "-", 
           RowBox[{"4", 
            SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
         SubscriptBox["\[Nu]", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{"6", 
         SubsuperscriptBox["m", "3", "2"], 
         SubscriptBox["\[Nu]", "3"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"],
          Schiebe[3, "+"],
          Editable->True]}], "+", 
        RowBox[{"2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", 
            RowBox[{"6", ",", "2", ",", "4"}]], "-", 
           SubsuperscriptBox["m", "4", "2"]}], ")"}], 
         SubscriptBox["\[Nu]", "4"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True]}], "-", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "-"],
           FontWeight->"Bold"],
          Schiebe[4, "-"],
          Editable->True]}], "-", 
        RowBox[{"2", 
         SubscriptBox["\[Nu]", "4"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "-"],
           FontWeight->"Bold"],
          Schiebe[2, "-"],
          Editable->True]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "66"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "66"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (66)", "Subsection",
 CellTags->"T1.6.42.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "66"], "=", 
  RowBox[{
   SubscriptBox["s", "66"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "66"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "66"], ",", "\"\<(66)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "66"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (alternative67)", "Subsection",
 CellTags->"T1.6.43.1"],

Cell["\<\
we dereived an alternative formula for Tarasov's (67), but do not save it by \
default.\
\>", "Text"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "67"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], " ", "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"6", "d"}], "-", 
         RowBox[{"6", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         RowBox[{"5", 
          SubscriptBox["\[Nu]", "2"]}], "-", 
         RowBox[{"6", 
          SubscriptBox["\[Nu]", "3"]}], "-", 
         RowBox[{"4", 
          SubscriptBox["\[Nu]", "4"]}]}], ")"}], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]], "-", 
          RowBox[{"4", 
           SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"6", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"6", ",", "2", ",", "4"}]], "-", 
          SubsuperscriptBox["m", "4", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "-"],
          FontWeight->"Bold"],
         Schiebe[4, "-"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "67"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "67"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (67)", "Subsection",
 CellTags->"T1.6.44.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "67"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], " ", "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", "d"}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}], "-", 
          SubscriptBox["\[Nu]", "4"]}], ")"}]}], "-", 
       RowBox[{"2", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["u", 
           RowBox[{"2", ",", "4", ",", "6"}]], "+", 
          RowBox[{"2", 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"-", "4"}], 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "-", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "4"}]], 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["4", "-"],
            FontWeight->"Bold"],
           Schiebe[4, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "4"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["4", "+"],
          FontWeight->"Bold"],
         Schiebe[4, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "4"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True], " ", 
    SubsuperscriptBox["m", "2", "2"]}], "-", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "4"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "d"}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "1"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}], "-", 
      SubscriptBox["\[Nu]", "4"]}], ")"}], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["m", "2", "2"], "+", 
      SubsuperscriptBox["m", "4", "2"], "-", 
      SuperscriptBox["p", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "4"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"4", " ", 
    SubsuperscriptBox["m", "3", "2"], " ", 
    SubscriptBox["\[Nu]", "3"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 1 + Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "+", 
   RowBox[{"2", " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        FontColor->RGBColor[1, 0, 0]]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], -1 + Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", 
       SubsuperscriptBox["m", "1", "2"]}], "+", 
      SubsuperscriptBox["m", "2", "2"], "-", 
      SubsuperscriptBox["m", "4", "2"], "-", 
      SuperscriptBox["p", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TVI[d, {1 + Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox["p", "2"], " ", 
    SubscriptBox["\[Nu]", "1"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["V",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontWeight->"Bold",
        FontColor->RGBColor[0, 0, 1]], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox[
        RowBox[{"d", "+", "2"}],
        TraditionalForm], ")"}]],
     TVI[2 + d, {2 + Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3], 
       Subscript[\[Nu], 4]}],
     Editable->True]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "67"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "67"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (67)", "Subsection",
 CellTags->"T1.6.45.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "67"], "=", 
  RowBox[{
   SubscriptBox["s", "67"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "67"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "67"], ",", "\"\<(67)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "67"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6713)", "Subsection",
 CellTags->"T1.6.46.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6713"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "67"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6713"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6713"], ",", "\"\<(6713)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "6713"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (68)", "Subsection",
 CellTags->"T1.6.47.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "68"], "=", 
   RowBox[{
    RowBox[{"4", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubsuperscriptBox["m", "4", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TVI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubsuperscriptBox["m", "1", "2"], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "4"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "4", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "+", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}], "-", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"1", ",", "3", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SubsuperscriptBox["m", "1", "2"], 
            SubscriptBox["\[Nu]", "1"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"2", ",", "4", ",", "6"}]], 
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}]}], ")"}]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"3", ",", "1", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", 
            SubsuperscriptBox["m", "3", "2"], 
            SubscriptBox["\[Nu]", "3"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["3", "+"],
              FontWeight->"Bold"],
             Schiebe[3, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "-"],
              FontWeight->"Bold"],
             Schiebe[4, "-"],
             Editable->True]}]}], ")"}]}], "-", 
        RowBox[{"2", 
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["\[Nu]", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            SubscriptBox["\[Nu]", "4"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "+"],
              FontWeight->"Bold"],
             Schiebe[4, "+"],
             Editable->True]}]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True], "-", 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], ")"}]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3], 
        Subscript[\[Nu], 4]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "68"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "68"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (68)", "Subsection",
 CellTags->"T1.6.48.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "68"], "=", 
  RowBox[{
   SubscriptBox["s", "68"], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "1"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "2"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "68"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "68"], ",", "\"\<(68)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{"m4", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "68"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6813)", "Subsection",
 CellTags->"T1.6.49.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "6813"], "=", 
  RowBox[{
   RowBox[{"Interchange", "[", 
    RowBox[{
     SubscriptBox["s", "68"], ",", 
     RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Nu]", "2"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "2"}]}], ",", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], ":>", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}], ",", 
     RowBox[{"d", ":>", 
      RowBox[{"d", "-", "2"}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "6813"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "6813"], ",", "\"\<(6813)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"m4", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTVIRecursion", "[", 
  SubscriptBox["f", "6813"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (70)", "Subsection",
 CellTags->"T1.6.50.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "70"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], "2"], 
     SubsuperscriptBox["m", "4", "2"], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["V",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TVI[d, {1, 1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{
      FractionBox["1", 
       RowBox[{"4", 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP]}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", "d"}], "-", "10"}], ")"}], 
         SubscriptBox["\[CapitalDelta]", 
          RowBox[{"1", ",", "3", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["\[CapitalDelta]", 
            RowBox[{"2", ",", "4", ",", "6"}]], "+", 
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"2", ",", "4", ",", "6"}]], 
            RowBox[{"(", 
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["2", "-"],
                FontWeight->"Bold"],
               Schiebe[2, "-"],
               Editable->True], "-", 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["4", "-"],
                FontWeight->"Bold"],
               Schiebe[4, "-"],
               Editable->True]}], ")"}]}]}], ")"}]}], "-", 
        RowBox[{"8", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "2"], 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], 
         SubsuperscriptBox["m", "3", "2"], 
         SubsuperscriptBox["m", "4", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "-"],
           FontWeight->"Bold"],
          Schiebe[4, "-"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "-", "2"}],
          TraditionalForm], ")"}]],
       TVI[-2 + d, {1, 1, 1, 1}],
       Editable->True]}], "+", "\n", "\t\t\t", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "3"}], ")"}], 
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"4", 
            SubsuperscriptBox["m", "1", "2"], 
            RowBox[{"(", 
             RowBox[{"d", "-", "4"}], ")"}], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}], "-", 
           RowBox[{"8", 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            SubsuperscriptBox["m", "2", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}]}], ")"}]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"6", ",", "2", ",", "4"}]], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FractionBox["1", "2"], 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            SubscriptBox["u", 
             RowBox[{"1", ",", "3", ",", "4"}]], 
            RowBox[{"(", 
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], "-", 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "+"],
                FontWeight->"Bold"],
               Schiebe[3, "+"],
               Editable->True]}], ")"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"d", "-", "2"}], ")"}], 
            RowBox[{"(", 
             RowBox[{"d", "-", "3"}], ")"}], 
            SubsuperscriptBox["m", "4", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["1", "+"],
              FontWeight->"Bold"],
             Schiebe[1, "+"],
             Editable->True]}]}], ")"}], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{
         SubsuperscriptBox["m", "4", "2"], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"d", "-", "4"}], ")"}], "2"], 
            SubscriptBox["u", 
             RowBox[{"1", ",", "3", ",", "4"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"3", "d"}], "-", "10"}], ")"}], 
            SubscriptBox["u", 
             RowBox[{"4", ",", "1", ",", "3"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["3", "+"],
              FontWeight->"Bold"],
             Schiebe[3, "+"],
             Editable->True]}]}], ")"}], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["4", "-"],
        FontWeight->"Bold"],
       Schiebe[4, "-"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], 
      SubsuperscriptBox["m", "4", "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", 
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], 
         SubsuperscriptBox["m", "1", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "2"}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["u", 
             RowBox[{"6", ",", "2", ",", "4"}]], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["2", "+"],
              FontWeight->"Bold"],
             Schiebe[2, "+"],
             Editable->True]}], "-", 
           RowBox[{"2", 
            SubsuperscriptBox["m", "4", "2"], 
            InterpretationBox[
             StyleBox[
              SuperscriptBox["4", "+"],
              FontWeight->"Bold"],
             Schiebe[4, "+"],
             Editable->True]}]}], ")"}]}]}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["3", "-"],
        FontWeight->"Bold"],
       Schiebe[3, "-"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", "d"}], "-", "10"}], ")"}], 
      SubscriptBox["u", 
       RowBox[{"1", ",", "3", ",", "4"}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"6", ",", "2", ",", "4"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "+"],
           FontWeight->"Bold"],
          Schiebe[2, "+"],
          Editable->True]}], "-", 
        RowBox[{"2", 
         SubsuperscriptBox["m", "4", "2"], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["4", "+"],
           FontWeight->"Bold"],
          Schiebe[4, "+"],
          Editable->True]}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "+"],
           FontWeight->"Bold"],
          Schiebe[3, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "-"],
           FontWeight->"Bold"],
          Schiebe[1, "-"],
          Editable->True]}], "-", 
        RowBox[{
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "+"],
           FontWeight->"Bold"],
          Schiebe[1, "+"],
          Editable->True], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "-"],
           FontWeight->"Bold"],
          Schiebe[3, "-"],
          Editable->True]}]}], ")"}], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["V",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm], 
         FormBox["1",
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TVI[d, {1, 1, 1, 1}],
       Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "70"], "=", 
  RowBox[{"OperatorApplyV", "[", 
   SubscriptBox["e", "70"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (70)", "Subsection",
 CellTags->"T1.6.51.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "70"], "=", 
  SubscriptBox["s", "70"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "70"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "70"], ",", "\"\<(70)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m4", "=!=", "0"}], " ", "&&", " ", 
       RowBox[{"PP", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["\<\
CheckTVIRecursion not applicable for special indices since A's and 
B's appear .\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (71)", "Subsection",
 CellTags->"T1.6.52.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ",", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], ">", "0"}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "71"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["D", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubscriptBox["h", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubscriptBox["h", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Sigma]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Rho]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        FractionBox["1", "2"], 
        SubscriptBox["\[CapitalSigma]", "3"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["\[Phi]", 
         RowBox[{"3", ",", "2", ",", "1"}]]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "71"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "71"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (71)", "Subsection",
 CellTags->"T1.6.53.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "71"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "71"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "2"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "71"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "71"], ",", "\"\<(71)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "71"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7123)", "Subsection",
 CellTags->"T1.6.54.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "7123"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "71"], ",", 
      RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "7123"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "7123"], ",", "\"\<(7123)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "3", ",", "2"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "7123"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7113)", "Subsection",
 CellTags->"T1.6.55.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "7113"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "71"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "7113"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "7113"], ",", "\"\<(7113)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"3", ",", "2", ",", "1"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "7113"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (78)", "Subsection",
 CellTags->"T1.6.56.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "78"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], " ", 
     SubscriptBox["D", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubscriptBox["\[CapitalSigma]", "3"]}], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["\[Rho]", 
         RowBox[{"1", ",", "2", ",", "3"}]]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Phi]", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "2", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
        SubscriptBox["D", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["S", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["S", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["S", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "78"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "78"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (78)", "Subsection",
 CellTags->"T1.6.57.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "78"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "78"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "78"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "78"], ",", "\"\<(78)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
       RowBox[{"m1", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["D", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "78"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (80)", "Subsection",
 CellTags->"T1.6.58.1"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["m", "1", "2"], "==", "0"}]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "80"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "-", "2"}], ")"}], 
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"2", ",", "3", ",", "6"}]], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "4"}], 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}]}], "-", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "2"], "-", "1"}], ")"}]}]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t  ", 
       RowBox[{"2", " ", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}]}], "-", 
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}]}]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}], 
        SubscriptBox["u", 
         RowBox[{"6", ",", "2", ",", "3"}]]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "80"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "80"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (80)", "Subsection",
 CellTags->"T1.6.59.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "80"], "=", 
  RowBox[{
   RowBox[{"\[Nu]explicit", "[", 
    SubscriptBox["s", "80"], "]"}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "\[Rule]", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "80"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "80"], ",", "\"\<(80)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"2", ",", "3", ",", "6"}]], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "80"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (81)", "Subsection",
 CellTags->"T1.6.60.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ",", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], ">", "0"}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "81"], "=", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TJI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "1", "2"], "-", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "2", "2"]}], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "3", "2"]}], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "+", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "1", "2"]}], "-", 
          SubsuperscriptBox["m", "2", "2"], "+", 
          RowBox[{"3", 
           SubsuperscriptBox["m", "3", "2"]}], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "+", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "81"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "81"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (81)", "Subsection",
 CellTags->"T1.6.61.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "81"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Nu]explicit", "[", 
      SubscriptBox["s", "81"], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "81"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "81"], ",", "\"\<(81)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "81"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8123)", "Subsection",
 CellTags->"T1.6.62.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8123"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{
       SubscriptBox["s", "81"], ",", 
       RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8123"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8123"], ",", "\"\<(8123)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8123"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8113)", "Subsection",
 CellTags->"T1.6.63.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8113"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Interchange", "[", 
      RowBox[{
       SubscriptBox["s", "81"], ",", 
       RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "\[Rule]", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "\[Rule]", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "\[Rule]", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8113"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8113"], ",", "\"\<(8113)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8113"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (82)", "Subsection",
 CellTags->"T1.6.64.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "82"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP]}], 
      SubsuperscriptBox["m", "1", "2"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
      SubscriptBox["\[Nu]", "1"], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], "+", "1"}], ")"}], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "+", "2"}],
          TraditionalForm], ")"}]],
       TJI[2 + d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      InterpretationBox[
       SuperscriptBox["p", "2"],
       PP], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalSigma]", "3"], "+", "4"}], ")"}], 
      SubscriptBox["\[Nu]", "1"], 
      InterpretationBox[
       StyleBox[
        SuperscriptBox["1", "+"],
        FontWeight->"Bold"],
       Schiebe[1, "+"],
       Editable->True], 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox[
          RowBox[{"d", "+", "2"}],
          TraditionalForm], ")"}]],
       TJI[2 + d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        FractionBox["1", "2"], 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "1"]}], "+", 
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "2"]}], "+", 
             RowBox[{"6", 
              SubscriptBox["\[Nu]", "3"]}], "-", 
             RowBox[{"7", "d"}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "2", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "3", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "-", 
       RowBox[{"4", 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], ")"}], "2"]}], "-", 
          RowBox[{
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", 
              SubsuperscriptBox["m", "1", "2"]}], "+", 
             SubsuperscriptBox["m", "2", "2"], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "1", "2"], 
           RowBox[{"(", 
            RowBox[{
             SubsuperscriptBox["m", "2", "2"], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{"3", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              SubsuperscriptBox["m", "2", "2"], "-", 
              SubsuperscriptBox["m", "3", "2"]}], ")"}], "2"]}]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "2", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "3", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", 
       RowBox[{
        FractionBox["1", "2"], 
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "-", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["m", "2", "2"], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "82"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "82"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["r", "82"], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["s", "82"], "=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["r", "82"], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], "==", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["r", "82"], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "+", 
      RowBox[{
       SubscriptBox["r", "82"], "[", 
       RowBox[{"[", "2", "]"}], "]"}]}]}], ")"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (82)", "Subsection",
 CellTags->"T1.6.65.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "82"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "82"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "82"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "82"], ",", "\"\<(82)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
       RowBox[{"m1", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "82"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8213)", "Subsection",
 CellTags->"T1.6.66.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8213"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "82"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8213"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8213"], ",", "\"\<(8213)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"m3", "=!=", "0"}]}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8213"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8212)", "Subsection",
 CellTags->"T1.6.67.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "8212"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Interchange", "[", 
     RowBox[{
      SubscriptBox["s", "82"], ",", 
      RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "2"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "8212"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "8212"], ",", "\"\<(8212)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
       RowBox[{"m2", "=!=", "0"}]}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "8212"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (83)", "Subsection",
 CellTags->"T1.6.68.1"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["m", "1", "2"], "==", "0"}]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Nu]", "1"], ">", "0"}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "83"], "=", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalSigma]", "3"], "+", "4"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "+", "2"}],
         TraditionalForm], ")"}]],
      TJI[2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], ")"}], "2"]}], 
        RowBox[{"(", 
         RowBox[{"d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubsuperscriptBox["m", "2", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}], "+", 
          RowBox[{
           SubsuperscriptBox["m", "3", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "1"]}], "-", 
             RowBox[{"10", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             RowBox[{"4", 
              SubscriptBox["\[Nu]", "3"]}]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "2", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          SubsuperscriptBox["m", "3", "2"], ")"}], "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "-", 
       RowBox[{"8", 
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"], "-", 
          SubscriptBox["\[Nu]", "3"]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "2", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["3", "-"],
               FontWeight->"Bold"],
              Schiebe[3, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "2", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             PP], ")"}], "2"], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "-", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["1", "-"],
               FontWeight->"Bold"],
              Schiebe[1, "-"],
              Editable->True], "+", 
             InterpretationBox[
              StyleBox[
               SuperscriptBox["2", "-"],
               FontWeight->"Bold"],
              Schiebe[2, "-"],
              Editable->True]}], ")"}]}], "-", 
          SuperscriptBox[
           RowBox[{"(", 
            SubsuperscriptBox["m", "3", "2"], ")"}], "2"]}], ")"}], 
        SubscriptBox["\[Nu]", "3"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[CapitalSigma]", "3"], "+", "2"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "83"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "83"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (83)", "Subsection",
 CellTags->"T1.6.69.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "83"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Nu]explicit", "[", 
     SubscriptBox["s", "83"], "]"}], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], "/.", 
   RowBox[{"d", "->", 
    RowBox[{"d", "-", "2"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "83"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "83"], ",", "\"\<(83)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "83"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (87)", "Subsection",
 CellTags->"T1.6.70.1"],

Cell[CellGroupData[{

Cell["(88)", "Subsubsection",
 CellTags->"T1.6.70.1.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "1"], "=", 
   RowBox[{
    SubsuperscriptBox["m", "1", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "+", 
    SubsuperscriptBox["m", "3", "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "2"], "=", 
   RowBox[{
    RowBox[{"3", 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "1", "4"], "+", 
       SubsuperscriptBox["m", "2", "4"], "+", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{"2", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "2", "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "1", "2"], 
        SubsuperscriptBox["m", "3", "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], 
        SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["t", "3"], "=", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "2", "4"], "-", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "4"], "-", 
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "3", "4"]}], ")"}]}], "+", 
    RowBox[{
     SubsuperscriptBox["m", "3", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "3", "4"], "-", 
       SubsuperscriptBox["m", "1", "4"], "-", 
       SubsuperscriptBox["m", "2", "4"]}], ")"}]}], "+", 
    RowBox[{"10", 
     SubsuperscriptBox["m", "1", "2"], 
     SubsuperscriptBox["m", "2", "2"], 
     SubsuperscriptBox["m", "3", "2"]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"\[Mu]1_", ",", "\[Mu]2_", ",", " ", "\[Mu]3_"}], "]"}], ":=", 
   RowBox[{"\[Mu]1", 
    RowBox[{"(", 
     RowBox[{
      InterpretationBox[
       SuperscriptBox["p", "2"],
       PP], "-", "\[Mu]1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], ")"}], "2"]}], "+", 
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", 
          SubscriptBox["t", "1"], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"5", "d"}], "-", "18"}], ")"}]}], "-", 
         RowBox[{"24", "\[Mu]1", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", "d"}], "-", "7"}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{"2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", "d"}], "-", "13"}], ")"}], 
       SuperscriptBox[
        SubscriptBox["t", "1"], "2"]}], "+", 
      RowBox[{"2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"9", "d"}], "-", "31"}], ")"}], 
       SubscriptBox["t", "2"]}], "-", 
      RowBox[{"24", "\[Mu]2", " ", "\[Mu]3", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", "d"}], "-", "13"}], ")"}]}], "-", 
      RowBox[{"24", " ", 
       SuperscriptBox["\[Mu]1", "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "d"}], "-", "7"}], ")"}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"1", ",", "2", ",", "3"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"2", ",", "1", ",", "3"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"3", ",", "2", ",", "1"}]], "=", 
   RowBox[{"f", "[", 
    RowBox[{
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["(89)", "Subsubsection",
 CellTags->"T1.6.70.2.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"\[Mu]1_", ",", "\[Mu]2_", ",", " ", "\[Mu]3_"}], "]"}], ":=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[Mu]1", " ", "\[Mu]2"}], 
     RowBox[{"d", "-", "4"}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          PP], ")"}], "2"]}], "-", 
      RowBox[{"4", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"7", "d"}], "-", "24"}], ")"}], 
       InterpretationBox[
        SuperscriptBox["p", "2"],
        PP], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", "\[Mu]3"}], "-", 
         RowBox[{"2", 
          SubscriptBox["t", "1"]}]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        SubscriptBox["t", "1"], "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"23", "d"}], "-", "80"}], ")"}]}], "+", 
      RowBox[{
       SubscriptBox["t", "2"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"9", "d"}], "-", "32"}], ")"}]}], "-", 
      RowBox[{"12", 
       SuperscriptBox["\[Mu]3", "2"], 
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}]}], "+", 
      RowBox[{"12", "\[Mu]1", " ", "\[Mu]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"7", "d"}], "-", "24"}], ")"}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"1", ",", "2", ",", "3"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"1", ",", "3", ",", "2"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", 
    RowBox[{"2", ",", "3", ",", "1"}]], "=", 
   RowBox[{"g", "[", 
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], ",", 
     SubsuperscriptBox["m", "3", "2"], ",", 
     SubsuperscriptBox["m", "1", "2"]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 SubscriptBox["g", 
  RowBox[{"1", ",", "2", ",", "3"}]]], "Input"],

Cell[TextData[{
 "general  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "87"], "=", 
   RowBox[{
    RowBox[{"3", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", "3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "8"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], "2"], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "3"]}], "-", 
       RowBox[{"2", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"], 
        SubscriptBox["t", "1"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", "d"}], "-", "23"}], ")"}]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"5", 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"15", 
              SuperscriptBox["d", "2"]}], "-", 
             RowBox[{"117", "d"}], "+", "224"}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["t", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"42", 
              SuperscriptBox["d", "2"]}], "-", 
             RowBox[{"331", "d"}], "+", "640"}], ")"}]}]}], ")"}]}], "-", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "5"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["t", "3"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"27", "d"}], "-", "90"}], ")"}]}], "-", 
          RowBox[{
           SubscriptBox["t", "1"], 
           SubscriptBox["t", "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "2"}], ")"}]}], "-", 
          RowBox[{"2", 
           SuperscriptBox[
            SubscriptBox["t", "1"], "3"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "d"}], "-", "26"}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"2", ",", "1", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["f", 
         RowBox[{"3", ",", "2", ",", "1"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "-"],
          FontWeight->"Bold"],
         Schiebe[3, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"1", ",", "3", ",", "2"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["g", 
         RowBox[{"2", ",", "3", ",", "1"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TJI[-2 + d, {1, 1, 1}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "87"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "87"], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (87)", "Subsection",
 CellTags->"T1.6.71.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "87"], "=", 
  SubscriptBox["s", "87"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["f", "87"], "=", 
   RowBox[{"MakeFun", "[", 
    RowBox[{
     SubscriptBox["h", "87"], ",", "\"\<(87)\>\"", ",", 
     RowBox[{"IFF", "[", 
      RowBox[{
       RowBox[{"PP", "=!=", "0"}], "&&", 
       RowBox[{"MatchQ", "[", 
        RowBox[{"d", ",", 
         RowBox[{"_Symbol", "+", 
          RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (90)", "Subsection",
 CellTags->"T1.6.72.1"],

Cell[TextData[{
 "general  ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]],
 " "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "90"], "=", 
   RowBox[{
    RowBox[{"3", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", "3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "d"}], "-", "10"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1, 1, 1}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubsuperscriptBox["m", "1", "2"]}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], ")"}], "2"], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "-", 
          RowBox[{"2", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            PP], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"4", "d"}], "-", "15"}], ")"}], 
              SubscriptBox["t", "1"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "5"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}]}], ")"}]}], "+", 
          RowBox[{
           FractionBox["3", "2"], 
           RowBox[{"(", 
            RowBox[{"d", "-", "5"}], ")"}], 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"]}], "+", 
          RowBox[{
           FractionBox["5", "2"], 
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           SubscriptBox["t", "2"]}], "-", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"5", "d"}], "-", "17"}], ")"}], 
           SuperscriptBox[
            RowBox[{"(", 
             SubsuperscriptBox["m", "1", "2"], ")"}], "2"]}], "-", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"13", "d"}], "-", "45"}], ")"}], 
           SubsuperscriptBox["m", "2", "2"], 
           SubsuperscriptBox["m", "3", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "+", 
             SubsuperscriptBox["m", "2", "2"], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "25"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True]}], "+", "\n", "\t\t\t\t", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "3", "2"], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], "-", 
          SubsuperscriptBox["m", "3", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             InterpretationBox[
              SuperscriptBox["p", "2"],
              PP], "-", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"]}], "+", 
             SubsuperscriptBox["m", "3", "2"]}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "25"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["3", "+"],
          FontWeight->"Bold"],
         Schiebe[3, "+"],
         Editable->True]}], "-", "\n", "\t\t\t\t", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", "3"}], ")"}], 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}], 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           PP], ")"}], "2"]}], "+", 
       RowBox[{
        InterpretationBox[
         SuperscriptBox["p", "2"],
         PP], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "30"}], ")"}], 
           RowBox[{"(", 
            RowBox[{"d", "-", "3"}], ")"}], 
           SubscriptBox["t", "1"]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"7", "d"}], "-", "31"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "10"}], ")"}], 
           SubsuperscriptBox["m", "1", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        FractionBox["1", "4"], 
        RowBox[{"(", 
         RowBox[{"d", "-", "5"}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"17", "d"}], "-", "66"}], ")"}], 
           SuperscriptBox[
            SubscriptBox["t", "1"], "2"]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "14"}], ")"}], 
           SubscriptBox["t", "2"]}], "-", 
          RowBox[{"4", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"3", "d"}], "-", "10"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               SubsuperscriptBox["m", "1", "2"], ")"}], "2"], "+", 
             RowBox[{"5", 
              SubsuperscriptBox["m", "2", "2"], 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}]}]}], ")"}]}], "+", 
       "\t", 
       RowBox[{
        FractionBox["2", 
         RowBox[{"d", "-", "4"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "2", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"5", "d"}], "-", "18"}], ")"}], 
              SubsuperscriptBox["m", "3", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "2", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}], "+", "\n", "\t\t\t\t\t\t", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "3", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"7", "d"}], "-", "24"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}], "-", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"5", "d"}], "-", "18"}], ")"}], 
              SubsuperscriptBox["m", "2", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "1", "2"], 
           SubsuperscriptBox["m", "3", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}], "-", "\n", "\t\t\t\t\t\t", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                InterpretationBox[
                 SuperscriptBox["p", "2"],
                 PP], "+", 
                SubsuperscriptBox["m", "2", "2"], "+", 
                SubsuperscriptBox["m", "3", "2"]}], ")"}], 
              RowBox[{"(", 
               RowBox[{"d", "-", "3"}], ")"}]}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"d", "-", "4"}], ")"}], 
              SubsuperscriptBox["m", "1", "2"]}]}], ")"}], 
           SubsuperscriptBox["m", "2", "2"], 
           SubsuperscriptBox["m", "3", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "+"],
             FontWeight->"Bold"],
            Schiebe[3, "+"],
            Editable->True]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm], 
        FormBox["1",
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TJI[-2 + d, {1, 1, 1}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "90"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "90"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (90)", "Subsection",
 CellTags->"T1.6.73.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["h", "90"], "=", 
   SubscriptBox["s", "90"]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "90"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "90"], ",", "\"\<(90)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"PP", "=!=", "0"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (91)", "Subsection",
 CellTags->"T1.6.74.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.1"], "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "3"}]]}], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["2", "-"],
         FontWeight->"Bold"],
        Schiebe[2, "-"],
        Editable->True], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["3", "-"],
         FontWeight->"Bold"],
        Schiebe[3, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.1"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.1"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.2"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     SubscriptBox["\[Nu]", "2"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "+"],
       FontWeight->"Bold"],
      Schiebe[2, "+"],
      Editable->True], 
     SubscriptBox["\[Nu]", "3"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["3", "+"],
       FontWeight->"Bold"],
      Schiebe[3, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], 
        SubsuperscriptBox["m", "1", "2"], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True]}], "+", 
       RowBox[{"(", 
        RowBox[{"d", "-", "2", "-", 
         RowBox[{"2", 
          SubscriptBox["\[Nu]", "1"]}]}], ")"}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.2"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.2"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.3"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[CapitalDelta]", 
         RowBox[{"1", ",", "2", ",", "3"}]], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"1", ",", "2", ",", "3"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["1", "-"],
           FontWeight->"Bold"],
          Schiebe[1, "-"],
          Editable->True]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"2", ",", "1", ",", "3"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["2", "-"],
           FontWeight->"Bold"],
          Schiebe[2, "-"],
          Editable->True]}], "+", 
        RowBox[{
         SubscriptBox["u", 
          RowBox[{"3", ",", "1", ",", "2"}]], 
         InterpretationBox[
          StyleBox[
           SuperscriptBox["3", "-"],
           FontWeight->"Bold"],
          Schiebe[3, "-"],
          Editable->True]}]}], ")"}]}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TKI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.3"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.3"], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "91.4"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "3"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "3"}]], 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          SubscriptBox["\[Nu]", "1"], "-", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], "-", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"3", ",", "1", ",", "2"}]], 
        SubscriptBox["\[Nu]", "1"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "+"],
          FontWeight->"Bold"],
         Schiebe[1, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["2", "-"],
            FontWeight->"Bold"],
           Schiebe[2, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        SubscriptBox["\[Nu]", "2"], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "+"],
          FontWeight->"Bold"],
         Schiebe[2, "+"],
         Editable->True], 
        RowBox[{"(", 
         RowBox[{
          InterpretationBox[
           StyleBox[
            SuperscriptBox["1", "-"],
            FontWeight->"Bold"],
           Schiebe[1, "-"],
           Editable->True], "-", 
          InterpretationBox[
           StyleBox[
            SuperscriptBox["3", "-"],
            FontWeight->"Bold"],
           Schiebe[3, "-"],
           Editable->True]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "91.4"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "91.4"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (911)", "Subsection",
 CellTags->"T1.6.75.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "911"], "=", 
  RowBox[{
   SubscriptBox["s", "91.1"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "911"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "911"], ",", "\"\<(911)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (912)", "Subsection",
 CellTags->"T1.6.76.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "912"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["s", "91.2"], "/.", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], "->", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "912"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "912"], ",", "\"\<(912)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (913)", "Subsection",
 CellTags->"T1.6.77.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "913"], "=", 
  SubscriptBox["s", "91.3"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "913"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "913"], ",", "\"\<(913)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"n2", "===", "1"}], " ", "||", " ", 
        RowBox[{"n3", "===", "1"}]}], ")"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}], ",", 
    RowBox[{"n1", ":>", "1"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (914)", "Subsection",
 CellTags->"T1.6.78.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "914"], "=", 
  RowBox[{
   SubscriptBox["s", "91.4"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "914"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "914"], ",", "\"\<(914)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "3"}]], "=!=", "0"}], "&&", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], " ", ">", " ", "1"}], " ", ")"}], "&&", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Symbol"}], "]"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (92)", "Subsection",
 CellTags->"T1.6.79.1"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalDelta]", 
    RowBox[{"1", ",", "2", ",", "3"}]], TraditionalForm]]],
 " = Cayley[1,2,3][{m1,m2,m3}]"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cayley", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"m1", ",", "m2", ",", "m3"}], "}"}], "]"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", " ", 
    RowBox[{
     SubsuperscriptBox["m", "1", "2"], "==", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "2"], "+", 
        SubscriptBox["m", "3"]}], ")"}], "2"]}], " ", ")"}], " ", 
   "\[DoubleLongRightArrow]", " ", 
   RowBox[{"(", " ", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "3"}]], "==", "0"}], " ", ")"}]}], 
  TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "92"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["m", "2"], 
     SubscriptBox["m", "3"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["m", "2"], "+", 
       SubscriptBox["m", "3"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["m", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", 
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "2"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "3"]}], "-", "1"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["2", "-"],
                FontWeight->"Bold"],
               Schiebe[2, "-"],
               Editable->True]}], "-", "1"}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "2"], "+", "1"}], ")"}], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["3", "-"],
             FontWeight->"Bold"],
            Schiebe[3, "-"],
            Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t", 
       RowBox[{
        SubscriptBox["m", "3"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"d", "-", 
             SubscriptBox["\[Nu]", "1"], "-", 
             RowBox[{"2", 
              SubscriptBox["\[Nu]", "2"]}], "-", 
             SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              InterpretationBox[
               StyleBox[
                SuperscriptBox["1", "+"],
                FontWeight->"Bold"],
               Schiebe[1, "+"],
               Editable->True], 
              InterpretationBox[
               StyleBox[
                SuperscriptBox["3", "-"],
                FontWeight->"Bold"],
               Schiebe[3, "-"],
               Editable->True]}], "-", "1"}], ")"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\[Nu]", "1"], "-", 
             SubscriptBox["\[Nu]", "3"], "+", "1"}], ")"}], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True]}]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "92"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "92"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (93)", "Subsection",
 CellTags->"T1.6.80.1"],

Cell[TextData[{
 "( ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "1"], "==", "0"}], ")"}], " ", "&&", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["m", "2"], "==", 
        SubscriptBox["m", "3"]}], ")"}]}], " ", " ", ")"}], " ", 
    "\[DoubleLongRightArrow]", " ", 
    RowBox[{"(", " ", 
     RowBox[{
      SubscriptBox["\[CapitalDelta]", 
       RowBox[{"1", ",", "2", ",", "3"}]], "==", "0"}], " ", ")"}]}], 
   TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "93"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "2", "2"], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "3"]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}], "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TKI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "93"], "=", 
  RowBox[{"OperatorApplyK", "[", 
   SubscriptBox["e", "93"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (93)", "Subsection",
 CellTags->"T1.6.81.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "93"], "=", 
  RowBox[{
   SubscriptBox["s", "93"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "93"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "93"], ",", "\"\<(93)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{
       SubscriptBox["m", "2"], "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "m2"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (94)", "Subsection",
 CellTags->"T1.6.82.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "94"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "6"}]], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"6", ",", "1", ",", "2"}]], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "1"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "+"],
             FontWeight->"Bold"],
            Schiebe[1, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "-"],
             FontWeight->"Bold"],
            Schiebe[2, "-"],
            Editable->True]}], "-", "d", "+", 
          SubscriptBox["\[Nu]", "1"], "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], "+", 
       RowBox[{"2", 
        SubsuperscriptBox["m", "2", "2"], 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Nu]", "2"], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["2", "+"],
             FontWeight->"Bold"],
            Schiebe[2, "+"],
            Editable->True], 
           InterpretationBox[
            StyleBox[
             SuperscriptBox["1", "-"],
             FontWeight->"Bold"],
            Schiebe[1, "-"],
            Editable->True]}], "-", "d", "+", 
          RowBox[{"2", 
           SubscriptBox["\[Nu]", "1"]}], "+", 
          SubscriptBox["\[Nu]", "2"]}], ")"}]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "94"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "94"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (94)", "Subsection",
 CellTags->"T1.6.83.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "94"], "=", 
  RowBox[{
   SubscriptBox["s", "94"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "94"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "94"], ",", "\"\<(94)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "=!=", "0"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9401)", "Subsection",
 CellTags->"T1.6.84.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "9401"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["s", "94"], "/.", 
    RowBox[{
     SubscriptBox["\[CapitalDelta]", 
      RowBox[{"1", ",", "2", ",", "6"}]], "->", "0"}]}], "/.", 
   RowBox[{
    SubscriptBox["u", 
     RowBox[{"6", ",", "1", ",", "2"}]], "->", "0"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9401"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "9401"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}]}], "==", 
   RowBox[{
    SubscriptBox["n", "9401"], "[", 
    RowBox[{"[", 
     RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9401"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9401"], ",", "\"\<(9401)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "1", ",", "2"}]], "===", "0"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9402)", "Subsection",
 CellTags->"T1.6.85.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["n", "9402"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["s", "94"], "/.", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "->", "0"}]}], "/.", 
     RowBox[{
      SubscriptBox["u", 
       RowBox[{"6", ",", "1", ",", "2"}]], "->", "0"}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9402"], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["n", "9402"], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "==", 
   RowBox[{
    SubscriptBox["n", "9402"], "[", 
    RowBox[{"[", 
     RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9402"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9402"], ",", "\"\<(9402)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "===", "0"}], "&&", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"6", ",", "1", ",", "2"}]], "===", "0"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (95)", "Subsection",
 CellTags->"T1.6.86.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "95"], "=", 
   RowBox[{
    RowBox[{"2", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["u", 
        RowBox[{"1", ",", "2", ",", "6"}]], "+", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["1", "-"],
         FontWeight->"Bold"],
        Schiebe[1, "-"],
        Editable->True], "-", 
       InterpretationBox[
        StyleBox[
         SuperscriptBox["2", "-"],
         FontWeight->"Bold"],
        Schiebe[2, "-"],
        Editable->True]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TBI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "95"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "95"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (95)", "Subsection",
 CellTags->"T1.6.87.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "95"], "=", 
  RowBox[{
   SubscriptBox["s", "95"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "95"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "95"], ",", "\"\<(95)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], " ", "&&", " ", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", 
        RowBox[{"_Symbol", "+", 
         RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (96)", "Subsection",
 CellTags->"T1.6.88.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "96"], "=", 
   RowBox[{
    RowBox[{"2", 
     InterpretationBox[
      SuperscriptBox["p", "2"],
      PP], 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", "1"}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TBI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[CapitalDelta]", 
        RowBox[{"1", ",", "2", ",", "6"}]], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"1", ",", "2", ",", "6"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["1", "-"],
          FontWeight->"Bold"],
         Schiebe[1, "-"],
         Editable->True]}], "+", 
       RowBox[{
        SubscriptBox["u", 
         RowBox[{"2", ",", "1", ",", "6"}]], 
        InterpretationBox[
         StyleBox[
          SuperscriptBox["2", "-"],
          FontWeight->"Bold"],
         Schiebe[2, "-"],
         Editable->True]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TBI[-2 + d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "96"], "=", 
  RowBox[{"OperatorApplyB", "[", 
   SubscriptBox["e", "96"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (96)", "Subsection",
 CellTags->"T1.6.89.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "96"], "=", 
  SubscriptBox["s", "96"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "96"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "96"], ",", "\"\<(96)\>\"", ",", 
    RowBox[{"IFF", "[", " ", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"d", ",", 
       RowBox[{"_Symbol", "+", 
        RowBox[{"_Integer", " ", "?", "Positive"}]}]}], "]"}], "]"}], ",", 
    RowBox[{"n1", ":>", "1"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (97)", "Subsection",
 CellTags->"T1.6.90.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "97"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubsuperscriptBox["m", "1", "2"], 
     SubscriptBox["\[Nu]", "1"], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "97"], "=", 
  RowBox[{"OperatorApplyA", "[", 
   SubscriptBox["e", "97"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (97)", "Subsection",
 CellTags->"T1.6.91.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "97"], "=", 
  RowBox[{
   SubscriptBox["s", "97"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "97"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "97"], ",", "\"\<(97)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", " ", 
      RowBox[{"MatchQ", "[", 
       RowBox[{"d", ",", "_Symbol"}], " ", "]"}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (98)", "Subsection",
 CellTags->"T1.6.92.1"],

Cell[TextData[{
 "general ",
 Cell[BoxData[
  SubsuperscriptBox["m", "i", "2"]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["e", "98"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", 
        SubscriptBox["\[Nu]", "1"]}]}], ")"}], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TAI[d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{"-", "2"}], 
     SubsuperscriptBox["m", "1", "2"], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox[
         RowBox[{"d", "-", "2"}],
         TraditionalForm], ")"}]],
      TAI[-2 + d, {
        Subscript[\[Nu], 1]}],
      Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "98"], "=", 
  RowBox[{"OperatorApplyA", "[", 
   SubscriptBox["e", "98"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (98)", "Subsection",
 CellTags->"T1.6.93.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "98"], "=", 
  SubscriptBox["s", "98"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "98"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "98"], ",", "\"\<(98)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{"MatchQ", "[", 
      RowBox[{"d", ",", 
       RowBox[{"_Symbol", " ", "+", 
        RowBox[{"_Integer", "?", "Positive"}]}]}], "]"}], "]"}]}], 
   "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (99)", "Subsection",
 CellTags->"T1.6.94.1"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "1"], "=", "M"}], ",", " ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SuperscriptBox["M", "2"]}], ",", 
   RowBox[{
    SubscriptBox["m", "2"], "=", 
    RowBox[{
     SubscriptBox["m", "3"], "=", "0"}]}]}], TraditionalForm]]]], "Text"],

Cell[TextData[{
 "Herleitung in three.nb ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubscriptBox["\[Nu]", "1"]}], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], " ", 
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]]}], "==", 
    RowBox[{
     SubscriptBox["\[Nu]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "+", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", "d"}], "+", 
       SubscriptBox["\[Nu]", "1"], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "3"]}], "+", "2"}], ")"}], " ", 
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]]}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["e", "99"], "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Nu]", "1"], " ", 
    RowBox[{"(", 
     RowBox[{"d", "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "-", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"d", "-", 
      SubscriptBox["\[Nu]", "2"], "-", 
      SubscriptBox["\[Nu]", "3"], "-", "1"}], ")"}], " ", 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["1", "+"],
      FontWeight->"Bold"],
     Schiebe[1, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{"-", 
     SubscriptBox["\[Nu]", "2"]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "d"}], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "+", "2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "d"}], "+", 
      SubscriptBox["\[Nu]", "1"], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "2"]}], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Nu]", "3"]}], "+", "2"}], ")"}], 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["2", "+"],
      FontWeight->"Bold"],
     Schiebe[2, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "99"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "99"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (99)", "Subsection",
 CellTags->"T1.6.95.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "99"], "=", 
  RowBox[{
   SubscriptBox["s", "99"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "99"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "99"], ",", "\"\<(99)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m1", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9912)", "Subsection",
 CellTags->"T1.6.96.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9912"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["s", "99"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9912"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9912"], ",", "\"\<(9912)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m2", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m3", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9913)", "Subsection",
 CellTags->"T1.6.97.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "9913"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["s", "99"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "9913"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "9913"], ",", "\"\<(9913)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m3", "^", "2"}], "===", "PP"}], "&&", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "1"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m1", ":>", "0"}], ",", 
      RowBox[{"m2", ":>", "0"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (100)", "Subsection",
 CellTags->"T1.6.98.1"],

Cell["Herleitung in three.nb ", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SubsuperscriptBox["m", "2", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       StyleBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        FontColor->RGBColor[0, 0, 1]], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "-", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "+", "2"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         FontColor->RGBColor[0, 0, 1]], 
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         FontColor->RGBColor[1, 0, 0]], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1 + Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}], TraditionalForm]], "Input"]], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["m", "1"], "=", "0"}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["e", "100"], "=", 
  RowBox[{
   RowBox[{"2", " ", 
    SubsuperscriptBox["m", "2", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
    SubscriptBox["\[Nu]", "2"], " ", 
    InterpretationBox[
     StyleBox[
      SuperscriptBox["2", "+"],
      FontWeight->"Bold"],
     Schiebe[2, "+"],
     Editable->True], 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d",
        TraditionalForm], ")"}]],
     TJI[d, {
       Subscript[\[Nu], 1], 
       Subscript[\[Nu], 2], 
       Subscript[\[Nu], 3]}],
     Editable->True]}], "==", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SubscriptBox["\[Nu]", "2"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "-", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "d"}], "+", 
       RowBox[{"2", " ", 
        SubscriptBox["\[Nu]", "1"]}], "+", "2"}], ")"}], " ", 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["1", "+"],
       FontWeight->"Bold"],
      Schiebe[1, "+"],
      Editable->True], 
     InterpretationBox[
      StyleBox[
       SuperscriptBox["2", "-"],
       FontWeight->"Bold"],
      Schiebe[2, "-"],
      Editable->True], 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {
        Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "100"], "=", 
  RowBox[{"OperatorApplyJ", "[", 
   SubscriptBox["e", "100"], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (100)", "Subsection",
 CellTags->"T1.6.99.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "100"], "=", 
  SubscriptBox["s", "100"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "100"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "100"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], ",", "\"\<(100)\>\"", ",", 
    
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "100"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1002)", "Subsection",
 CellTags->"T1.6.100.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1002"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1002"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1002"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "2"], "-", "1"}]}]}], ",", "\"\<(1002)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "2"], ">", "2"}], "&&", 
      RowBox[{"m2", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m3", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1002"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1003)", "Subsection",
 CellTags->"T1.6.101.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1003"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "3"}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1003"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1003"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", "\"\<(1003)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m3", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1003"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1004)", "Subsection",
 CellTags->"T1.6.102.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1004"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1004"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1004"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}], ",", "\"\<(1004)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "2"}], "&&", 
      RowBox[{"m1", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m2", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1004"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1005)", "Subsection",
 CellTags->"T1.6.103.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1005"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"2", "\[LeftRightArrow]", "3"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1005"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1005"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], ",", "\"\<(1005)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m1", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1005"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1006)", "Subsection",
 CellTags->"T1.6.104.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "1006"], "=", 
  RowBox[{"Interchange", "[", 
   RowBox[{
    SubscriptBox["s", "100"], ",", 
    RowBox[{"2", "\[LeftRightArrow]", "3"}], ",", 
    RowBox[{"1", "\[LeftRightArrow]", "2"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "1006"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1006"], "/.", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], "->", 
      RowBox[{
       SubscriptBox["\[Nu]", "3"], "-", "1"}]}]}], ",", "\"\<(1006)\>\"", ",", 
    RowBox[{"IFF", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "3"], ">", "2"}], "&&", 
      RowBox[{"m3", "=!=", "0"}]}], "]"}], ",", 
    RowBox[{"m2", ":>", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckTJIRecursion", "[", 
  SubscriptBox["f", "1006"], "]"}]], "Input"],

Cell[BoxData["\<\"Off\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (150)", "Subsection",
 CellTags->"T1.6.105.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "150"], "=", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", "PP", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (150)", "Subsection",
 CellTags->"T1.6.106.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "150"], "=", 
  SubscriptBox["s", "150"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "150"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "150"], ",", "\"\<(150)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (151)", "Subsection",
 CellTags->"T1.6.107.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "151"], "=", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"d", ",", "PP", ",", "DP", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", "PP", ",", "DP", ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "b"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", "DP", ",", 
       RowBox[{"{", 
        RowBox[{"a", ",", "b"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", "DP", ",", 
       RowBox[{"{", 
        RowBox[{"a", ",", "b"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (151)", "Subsection",
 CellTags->"T1.6.108.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "151"], "=", 
  SubscriptBox["s", "151"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "151"], "=", 
  RowBox[{"ReplacePart", "[", 
   RowBox[{
    RowBox[{"MakeFun", "[", 
     RowBox[{
      SubscriptBox["h", "151"], ",", "\"\<(151)\>\""}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "4"}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (152)", "Subsection",
 CellTags->"T1.6.109.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "4"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["q", "2"], "-", "p"}], " ", ")"}], "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}], "=", " ", 
   RowBox[{
    SubsuperscriptBox["q", "2", "2"], "-", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    RowBox[{"2", "p", " ", 
     SubscriptBox["q", "2"]}], "+", 
    SuperscriptBox["p", "2"], "+", 
    SubsuperscriptBox["m", "2", "2"], "-", 
    SubsuperscriptBox["m", "4", "2"]}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "152"], "=", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m4"}], "}"}]}], "}"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"TVI", "[", 
     RowBox[{"d", ",", "PP", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2", "-", "1"}], ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "-", 
    RowBox[{"2", 
     RowBox[{"TFI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}]}], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"PP", "+", 
       RowBox[{"m2", "^", "2"}], "-", 
       RowBox[{"m4", "^", "2"}]}], ")"}], 
     RowBox[{"TVI", "[", 
      RowBox[{"d", ",", "PP", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2", ",", "m2"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (152)", "Subsection",
 CellTags->"T1.6.110.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "152"], "=", 
  SubscriptBox["s", "152"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "152"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "152"], ",", "\"\<(152)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (153)", "Subsection",
 CellTags->"T1.6.111.1"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Nu]", "2"], "=", 
   RowBox[{"-", "1"}]}], TextForm]], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["s", "153"], "=", 
  RowBox[{
   RowBox[{"TVI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n1", ",", "m1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n3", ",", "m3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}], "==", "\n", 
   "\t\t", 
   RowBox[{
    RowBox[{"TFI", "[", 
     RowBox[{"d", ",", " ", "PP", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3", ",", "m3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n1", ",", "m1"}], "}"}]}], "}"}]}], "]"}], "+", 
    RowBox[{
     RowBox[{"m2", "^", "2"}], " ", 
     RowBox[{"TVI", "[", 
      RowBox[{"d", ",", " ", "PP", ",", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1", ",", "m1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n3", ",", "m3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (153)", "Subsection",
 CellTags->"T1.6.112.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "153"], "=", 
  SubscriptBox["s", "153"]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "153"], "=", 
  RowBox[{"MakeFun", "[", 
   RowBox[{
    SubscriptBox["h", "153"], ",", "\"\<(153)\>\""}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (200)", "Subsection",
 CellTags->"T1.6.113.1"],

Cell["Derivation in TJRspecial.nb", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["m", "3"], "=", 
    RowBox[{
     SubscriptBox["m", "2"], "=", 
     SubscriptBox["m", "1"]}]}], ",", " ", 
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SubsuperscriptBox["m", "1", "2"]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["s", "200"], "=", 
   RowBox[{
    RowBox[{"16", " ", 
     SubsuperscriptBox["m", "1", "2"], " ", 
     SubscriptBox["\[Nu]", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", 
       SubscriptBox["\[Nu]", "1"], "-", 
       SubscriptBox["\[Nu]", "2"], "-", 
       SubscriptBox["\[Nu]", "3"]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        StyleBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         FontColor->RGBColor[0, 0, 1]], 
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm], 
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      TJI[d, {1 + Subscript[\[Nu], 1], 
        Subscript[\[Nu], 2], 
        Subscript[\[Nu], 3]}],
      Editable->True]}], "==", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "d"}], "-", 
         RowBox[{"3", " ", 
          SubscriptBox["\[Nu]", "1"]}], "-", 
         RowBox[{"4", " ", 
          SubscriptBox["\[Nu]", "2"]}]}], ")"}]}], " ", 
      SubscriptBox["\[Nu]", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {-1 + Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 1 + Subscript[\[Nu], 3]}],
       Editable->True]}], "-", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"3", " ", "d"}], "-", 
        RowBox[{"3", " ", 
         SubscriptBox["\[Nu]", "1"]}], "-", 
        RowBox[{"4", " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {-1 + Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "d"}], "-", 
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "2"]}]}], ")"}], " ", 
      SubscriptBox["\[Nu]", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], -1 + Subscript[\[Nu], 2], 1 + 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "6"}], " ", 
         SuperscriptBox["d", "2"]}], "+", 
        RowBox[{"22", " ", "d", " ", 
         SubscriptBox["\[Nu]", "1"]}], "-", 
        RowBox[{"16", " ", 
         SubsuperscriptBox["\[Nu]", "1", "2"]}], "-", 
        SubscriptBox["\[Nu]", "2"], "+", 
        RowBox[{"6", " ", "d", " ", 
         SubscriptBox["\[Nu]", "2"]}], "-", 
        RowBox[{"13", " ", 
         SubscriptBox["\[Nu]", "1"], " ", 
         SubscriptBox["\[Nu]", "2"]}], "-", 
        SubscriptBox["\[Nu]", "3"], "+", 
        RowBox[{"6", " ", "d", " ", 
         SubscriptBox["\[Nu]", "3"]}], "-", 
        RowBox[{"13", " ", 
         SubscriptBox["\[Nu]", "1"], " ", 
         SubscriptBox["\[Nu]", "3"]}], "-", 
        RowBox[{"4", " ", 
         SubscriptBox["\[Nu]", "2"], " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], 
         Subscript[\[Nu], 2], 
         Subscript[\[Nu], 3]}],
       Editable->True]}], "+", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "d"}], "-", 
        SubscriptBox["\[Nu]", "1"], "-", 
        RowBox[{"2", " ", 
         SubscriptBox["\[Nu]", "3"]}]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["J",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        RowBox[{
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          FontColor->RGBColor[0, 0, 1]], 
         StyleBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          FontColor->RGBColor[1, 0, 0]]}], 
        RowBox[{"(", 
         FormBox["d",
          TraditionalForm], ")"}]],
       TJI[d, {
         Subscript[\[Nu], 1], 1 + Subscript[\[Nu], 2], -1 + 
         Subscript[\[Nu], 3]}],
       Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Due", " ", "to", " ", "sorting", "  ", 
    SubscriptBox["\[Nu]", "1"]}], "\[GreaterSlantEqual]", 
   SubscriptBox["\[Nu]", "2"], "\[GreaterSlantEqual]", 
   RowBox[{
    SubscriptBox["\[Nu]", "3"], "  ", "by", " ", "TJI", " ", "only", " ", 
    "this", " ", "rule", " ", "is", " ", 
    RowBox[{"necessary", "."}]}]}], TextForm]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (200)", "Subsection",
 CellTags->"T1.6.114.1"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "200"], "=", 
  RowBox[{
   SubscriptBox["s", "200"], "/.", 
   RowBox[{
    SubscriptBox["\[Nu]", "1"], "->", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], "-", "1"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "200"], "=", 
  RowBox[{
   StyleBox["MakeFun",
    FontColor->RGBColor[1, 0.6, 0.6]], "[", 
   RowBox[{
    SubscriptBox["h", "200"], ",", "\"\<(200)\>\"", ",", 
    RowBox[{
     StyleBox["IFF",
      FontColor->RGBColor[0.6, 0.4, 1]], "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Nu]", "1"], ">", "1"}], "&&", 
      RowBox[{"m1", "=!=", "0"}], "&&", 
      RowBox[{"PP", "===", 
       RowBox[{"m1", "^", "2"}]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m2", ":>", "m1"}], ",", 
      RowBox[{"m3", ":>", "m1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (201)", "Subsection",
 CellTags->"T1.6.115.1"],

Cell["\<\
Subscript[f, 201] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n2_)?PQ, 0}, 
      {(n3_)?PQ, m3_}}] := 
    (TComment[\"(201)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 
         2*n1 + 2*n2)*
          (3*d - 2*n1 - 
         2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, PP, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}]
)/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (202)", "Subsection",
 CellTags->"T1.6.116.1"],

Cell["\<\
Subscript[f, 202] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n3_)?PQ, m3_}, 
      {(n2_)?PQ, 0}}] := 
    (TComment[\"(202)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 2*n1 + 
          2*n2)*
          (3*d - 2*n1 - 
          2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, m3^2, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}
])/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (203)", "Subsection",
 CellTags->"T1.6.117.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "203"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}]}],
        "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t\t\t\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(203)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], "]"}],
       ";", " ", "\n", "\t\t\t\t", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", "\n", "\t\t\t", 
     ")"}]}], "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (204)", "Subsection",
 CellTags->"T1.6.118.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "204"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", " ", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], 
        ",", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}]}],
        "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(204)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", " ", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], "]"}],
       ";", " ", "\n", "\t", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", ")"}]}], 
   "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (205)", "Subsection",
 CellTags->"T1.6.119.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subscript", "[", 
   RowBox[{"f", ",", " ", "205"}], "]"}], " ", "=", 
  RowBox[{"HoldForm", "[", "\n", "\t\t ", 
   RowBox[{
    RowBox[{"TJR", "[", 
     RowBox[{"d_", ",", " ", "0", ",", " ", 
      RowBox[{"{", " ", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n2_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", "\n", "    ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n3_", ")"}], "?", "PQ"}], ",", " ", "m2_"}], "}"}], 
        ",", " ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", "n1_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}]}], 
       "}"}]}], "]"}], " ", ":=", 
    RowBox[{"(", "\n", "\t ", 
     RowBox[{
      RowBox[{"TComment", "[", 
       RowBox[{"\"\<(205)\>\"", ",", " ", "\n", "        ", 
        RowBox[{"TJIC", "[", 
         RowBox[{"d", ",", "0", ",", " ", 
          RowBox[{"{", " ", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"n3", ",", " ", "m2"}], "}"}], ",", " ", 
            RowBox[{"{", 
             RowBox[{"n1", ",", " ", "0"}], "}"}]}], "}"}]}], "]"}]}], "]"}], 
      ";", " ", "\n", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n2"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{
          "1", " ", "+", " ", "d", " ", "-", " ", "n1", " ", "-", " ", "n2", 
           " ", "-", " ", "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], " ", "+", " ", "n1", " ", "+", " ", "n3"}], 
             ")"}]}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"TJR", "[", 
          RowBox[{"d", ",", " ", "0", ",", " ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "1"}], " ", "+", " ", "n1"}], ",", " ", "0"}], 
              "}"}], ",", " ", 
             RowBox[{"{", 
              RowBox[{"n2", ",", " ", "m2"}], "}"}], ",", " ", "\n", 
             "       ", 
             RowBox[{"{", 
              RowBox[{"n3", ",", " ", "m2"}], "}"}]}], "}"}]}], "]"}]}], 
        ")"}], "/", "\n", "   ", 
       RowBox[{"(", 
        RowBox[{"2", "*", 
         RowBox[{"m2", "^", "2"}], "*", 
         RowBox[{"(", 
          RowBox[{"d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}]}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}], "*", "\n", "     ", 
         RowBox[{"(", 
          RowBox[{"2", " ", "+", " ", "d", " ", "-", " ", 
           RowBox[{"2", "*", "n1"}], " ", "-", " ", "n2", " ", "-", " ", 
           "n3"}], ")"}]}], ")"}]}]}], "\n", "\t\t\t", ")"}]}], 
   "]"}]}]], "Input",
 Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (1xxxx)", "Subsection",
 CellTags->"T1.6.120.1"],

Cell["\<\
If[MatchQ[$RankLimit, {_Integer?NonNegative, 
     _Integer?NonNegative}], 
   $TarasovTdeltaplimit = $RankLimit[[1]]; 
    ($TarasovTplimit = $RankLimit[[2]]; ), 
   $TarasovTdeltaplimit = 2; $TarasovTplimit = 2]; \
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tarti", "=", 
  RowBox[{"Timing", "[", 
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"0", "<", 
        RowBox[{"a", "+", "b", "+", "r", "+", "es"}]}], ",", 
       RowBox[{
        RowBox[{"WriteString", "[", 
         RowBox[{"\"\<stdout\>\"", ",", 
          RowBox[{"{", 
           RowBox[{"a", ",", "b", ",", "r", ",", "es"}], "}"}]}], "]"}], ";", 
        
        RowBox[{"Set", "@@", 
         RowBox[{"{", 
          RowBox[{
           SubscriptBox["s", 
            RowBox[{
             SuperscriptBox["10", "4"], "+", 
             RowBox[{"1000", " ", "a"}], "+", 
             RowBox[{"100", " ", "b"}], "+", 
             RowBox[{"10", " ", "r"}], "+", "es"}]], ",", 
           RowBox[{
            RowBox[{
             RowBox[{
              StyleBox["TFI",
               FontColor->RGBColor[1, 0.6, 0]], "[", 
              RowBox[{"d", ",", 
               InterpretationBox[
                SuperscriptBox["p", "2"],
                PP], ",", "DP", ",", 
               RowBox[{"{", 
                RowBox[{"a", ",", "b"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0", ",", "r", ",", "es", ",", "0"}], "}"}],
                ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"n1", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n2", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n3", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n4", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"n5", ",", 
                   InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}]}], "}"}]}], "]"}], "==", 
             RowBox[{"ApplyTarasovT", "[", 
              RowBox[{
               RowBox[{"TarasovT", "[", 
                RowBox[{"a", ",", "b", ",", "r", ",", "es", ",", 
                 InterpretationBox[
                  SuperscriptBox["p", "2"],
                  PP], ",", "DP"}], "]"}], ",", 
               RowBox[{"STLI", "[", 
                RowBox[{"d", ",", 
                 InterpretationBox[
                  SuperscriptBox["p", "2"],
                  PP], ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"n1", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "1"],
                    m1]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n2", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "2"],
                    m2]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n3", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "3"],
                    m3]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n4", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "4"],
                    m4]}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"n5", ",", 
                    InterpretationBox[
                    SubscriptBox["m", "5"],
                    m5]}], "}"}]}], "}"}]}], "]"}]}], "]"}]}], "/.", 
            RowBox[{"STLI", "\[Rule]", 
             StyleBox["TFI",
              FontColor->RGBColor[1, 0.6, 0]]}]}]}], "}"}]}]}]}], "]"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"a", ",", "0", ",", "$TarasovTdeltaplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "0", ",", 
       RowBox[{"$TarasovTdeltaplimit", "-", "a"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0", ",", "$TarasovTplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"es", ",", "0", ",", 
       RowBox[{"$TarasovTplimit", "-", "r"}]}], "}"}]}], "]"}], 
   "]"}]}]], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, 1, 0}{0, \
0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, \
2}{0, 0, 2, 3}{0, 0, 3, 0}{0, 0, 3, 1}{0, 0, 3, 2}{0, 0, 4, 0}{0, 0, 4, 1}{0, \
0, 5, 0}{0, 1, 0, 0}{0, 1, 0, 1}{0, 1, 0, 2}{0, 1, 0, 3}{0, 1, 0, 4}{0, 1, 0, \
5}{0, 1, 1, 0}{0, 1, 1, 1}{0, 1, 1, 2}{0, 1, 1, 3}{0, 1, 1, 4}{0, 1, 2, 0}{0, \
1, 2, 1}{0, 1, 2, 2}{0, 1, 2, 3}{0, 1, 3, 0}{0, 1, 3, 1}{0, 1, 3, 2}{0, 1, 4, \
0}{0, 1, 4, 1}{0, 1, 5, 0}{0, 2, 0, 0}{0, 2, 0, 1}{0, 2, 0, 2}{0, 2, 0, 3}{0, \
2, 0, 4}{0, 2, 0, 5}{0, 2, 1, 0}{0, 2, 1, 1}{0, 2, 1, 2}{0, 2, 1, 3}{0, 2, 1, \
4}{0, 2, 2, 0}{0, 2, 2, 1}{0, 2, 2, 2}{0, 2, 2, 3}{0, 2, 3, 0}{0, 2, 3, 1}{0, \
2, 3, 2}{0, 2, 4, 0}{0, 2, 4, 1}{0, 2, 5, 0}{1, 0, 0, 0}{1, 0, 0, 1}{1, 0, 0, \
2}{1, 0, 0, 3}{1, 0, 0, 4}{1, 0, 0, 5}{1, 0, 1, 0}{1, 0, 1, 1}{1, 0, 1, 2}{1, \
0, 1, 3}{1, 0, 1, 4}{1, 0, 2, 0}{1, 0, 2, 1}{1, 0, 2, 2}{1, 0, 2, 3}{1, 0, 3, \
0}{1, 0, 3, 1}{1, 0, 3, 2}{1, 0, 4, 0}{1, 0, 4, 1}{1, 0, 5, 0}{1, 1, 0, 0}{1, \
1, 0, 1}{1, 1, 0, 2}{1, 1, 0, 3}{1, 1, 0, 4}{1, 1, 0, 5}{1, 1, 1, 0}{1, 1, 1, \
1}{1, 1, 1, 2}{1, 1, 1, 3}{1, 1, 1, 4}{1, 1, 2, 0}{1, 1, 2, 1}{1, 1, 2, 2}{1, \
1, 2, 3}{1, 1, 3, 0}{1, 1, 3, 1}{1, 1, 3, 2}{1, 1, 4, 0}{1, 1, 4, 1}{1, 1, 5, \
0}{2, 0, 0, 0}{2, 0, 0, 1}{2, 0, 0, 2}{2, 0, 0, 3}{2, 0, 0, 4}{2, 0, 0, 5}{2, \
0, 1, 0}{2, 0, 1, 1}{2, 0, 1, 2}{2, 0, 1, 3}{2, 0, 1, 4}{2, 0, 2, 0}{2, 0, 2, \
1}{2, 0, 2, 2}{2, 0, 2, 3}{2, 0, 3, 0}{2, 0, 3, 1}{2, 0, 3, 2}{2, 0, 4, 0}{2, \
0, 4, 1}{2, 0, 5, 0}\
\>", "Print"]
}, Open  ]],

Cell["Share[]", "Text"],

Cell[BoxData[
 RowBox[{"matarti", "=", 
  RowBox[{"Timing", "[", 
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"0", "<", 
        RowBox[{"ia", "+", "ib", "+", "ir", "+", "es"}]}], ",", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"WriteString", "[", 
          RowBox[{"\"\<stdout\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"ia", ",", "ib", ",", "ir", ",", "es"}], "}"}]}], "]"}], 
         ";"}], "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"PrintTemporary", "[", 
         RowBox[{"{", 
          RowBox[{"ia", ",", "ib", ",", "ir", ",", "es"}], "}"}], "]"}], ";", 
        
        RowBox[{"Set", "[", 
         RowBox[{
          SubscriptBox["f", 
           RowBox[{
            SuperscriptBox["10", "4"], "+", 
            RowBox[{"1000", " ", "ia"}], "+", 
            RowBox[{"100", " ", "ib"}], "+", 
            RowBox[{"10", " ", "ir"}], "+", "es"}]], ",", 
          RowBox[{
           RowBox[{"MakeFun", "[", 
            RowBox[{
             SubscriptBox["s", 
              RowBox[{
               SuperscriptBox["10", "4"], "+", 
               RowBox[{"1000", " ", "ia"}], "+", 
               RowBox[{"100", " ", "ib"}], "+", 
               RowBox[{"10", " ", "ir"}], "+", "es"}]], ",", 
             RowBox[{"\"\<(1\>\"", "<>", 
              RowBox[{"ToString", "[", "ia", "]"}], "<>", 
              RowBox[{"ToString", "[", "ib", "]"}], "<>", 
              RowBox[{"ToString", "[", "ir", "]"}], "<>", 
              RowBox[{"ToString", "[", "es", "]"}], "<>", "\"\<)\>\""}]}], 
            "]"}], "/.", 
           RowBox[{"PQ", ":>", "PNQ"}]}]}], "]"}]}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"ia", ",", "0", ",", " ", "$TarasovTdeltaplimit"}], "}"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"ib", ",", "0", ",", 
       RowBox[{"$TarasovTdeltaplimit", "-", "ia"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"ir", ",", "0", ",", "  ", "$TarasovTplimit"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"es", ",", "0", ",", 
       RowBox[{"$TarasovTplimit", "-", "ir"}]}], "}"}]}], "]"}], 
   "]"}]}]], "Input"],

Cell["Share[]", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["symmetry", "Subsection",
 CellTags->"Tsymmetry"],

Cell["Uncomment this ...", "Text"],

Cell["\<\
TVR[d_, pp_, {a_, b_, c_, z_}] := 
  TVR[d, pp, {c, b, a, z}] /;  !OrderedQ[{a, c}]\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["special TVR reductions", "Subsection",
 CellTags->"specialTVR"],

Cell["\<\
TVR[2 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   ((-6*m1^2 + 2*d*m1^2 + 3*pp)*TAI[d, 0, {{1, m1}}]^2)/
      (12*(-3 + d)*(-1 + d)*m1^2*pp) + 
     ((4*m1^2 - pp)*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-3 + d)*(-1 + d)*m1^2) - 
     ((6*m1^2 - 2*d*m1^2 - 7*pp + 2*d*pp)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/(6*(-2 + d)*(-1 + d)*pp) + 
     ((-4*m1^2 + pp)*(-6*m1^2 + 2*d*m1^2 + 3*pp)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (6*(-3 + d)*(-2 + d)*(-1 + d)*pp) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   -((-96*d*m1^6 + 352*d^2*m1^6 - 384*d^3*m1^6 + 128*d^4*m1^6 - 
           192*m1^4*pp + 1008*d*m1^4*pp - 1632*d^2*m1^4*pp + 
           912*d^3*m1^4*pp - 144*d^4*m1^4*pp + 288*m1^2*pp^2 - 
           1536*d*m1^2*pp^2 + 2316*d^2*m1^2*pp^2 - 1086*d^3*m1^2*pp^2 + 
           66*d^4*m1^2*pp^2 + 72*d*pp^3 - 166*d^2*pp^3 + 81*d^3*pp^3 + 
           4*d^4*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (36*(-1 + d)^2*d^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-1 + d)^2*d*(1 + d)) - 
     ((288*m1^8 - 1152*d*m1^8 + 1504*d^2*m1^8 - 768*d^3*m1^8 + 
          128*d^4*m1^8 - 816*m1^6*pp + 2480*d*m1^6*pp - 
          2224*d^2*m1^6*pp + 736*d^3*m1^6*pp - 80*d^4*m1^6*pp + 
          864*m1^4*pp^2 - 2196*d*m1^4*pp^2 + 1446*d^2*m1^4*pp^2 - 
          252*d^3*m1^4*pp^2 - 6*d^4*m1^4*pp^2 - 432*m1^2*pp^3 + 
          1086*d*m1^2*pp^3 - 895*d^2*m1^2*pp^3 + 315*d^3*m1^2*pp^3 - 
          44*d^4*m1^2*pp^3 + 4*d*pp^4 - 2*d^2*pp^4 - 4*d^3*pp^4 + 
          2*d^4*pp^4)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     (m1^2*(4*m1^2 - pp)*(-96*m1^6 + 352*d*m1^6 - 384*d^2*m1^6 + 
          128*d^3*m1^6 + 240*m1^4*pp - 624*d*m1^4*pp + 384*d^2*m1^4*pp - 
          48*d^3*m1^4*pp - 192*m1^2*pp^2 + 420*d*m1^2*pp^2 - 
          162*d^2*m1^2*pp^2 - 18*d^3*m1^2*pp^2 + 72*pp^3 - 154*d*pp^3 + 
          81*d^2*pp^3 - 8*d^3*pp^3)*TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   ((-72*d^2*m1^6 + 144*d^3*m1^6 - 88*d^4*m1^6 + 16*d^5*m1^6 - 
          240*d*m1^4*pp + 572*d^2*m1^4*pp - 398*d^3*m1^4*pp + 
          84*d^4*m1^4*pp - 2*d^5*m1^4*pp - 96*m1^2*pp^2 + 
          480*d*m1^2*pp^2 - 744*d^2*m1^2*pp^2 + 390*d^3*m1^2*pp^2 - 
          50*d^4*m1^2*pp^2 + 4*d^5*m1^2*pp^2 - 24*d*pp^3 + 54*d^2*pp^3 - 
          27*d^3*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (24*(-3 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*m1^2*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (8*(-3 + d)*(-1 + d)^2*(1 + d)*m1^2) + 
     ((-72*d*m1^6 + 144*d^2*m1^6 - 88*d^3*m1^6 + 16*d^4*m1^6 - 
          96*m1^4*pp + 260*d*m1^4*pp - 166*d^2*m1^4*pp + 
          12*d^3*m1^4*pp + 6*d^4*m1^4*pp + 160*m1^2*pp^2 - 
          300*d*m1^2*pp^2 + 50*d^2*m1^2*pp^2 + 90*d^3*m1^2*pp^2 - 
          24*d^4*m1^2*pp^2 - 48*pp^3 + 114*d*pp^3 - 67*d^2*pp^3 + 
          4*d^3*pp^3 + 2*d^4*pp^3)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (12*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) - 
     ((-4*m1^2 + pp)*(72*d*m1^6 - 144*d^2*m1^6 + 88*d^3*m1^6 - 
          16*d^4*m1^6 + 96*m1^4*pp - 236*d*m1^4*pp + 122*d^2*m1^4*pp + 
          12*d^3*m1^4*pp - 10*d^4*m1^4*pp - 96*m1^2*pp^2 + 
          180*d*m1^2*pp^2 - 30*d^2*m1^2*pp^2 - 46*d^3*m1^2*pp^2 + 
          8*d^4*m1^2*pp^2 + 24*pp^3 - 54*d*pp^3 + 27*d^2*pp^3)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
(12*(-3 + d)*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[6 + d_Symbol, pp_, 
    {{1, m1_}, {1, m1_}, {4, 0}, {1, m1_}}] := 
   ((2304*d*m1^10 - 4608*d^2*m1^10 - 3520*d^3*m1^10 + 6976*d^4*m1^10 + 
          1472*d^5*m1^10 - 2624*d^6*m1^10 - 256*d^7*m1^10 + 
          256*d^8*m1^10 + 4608*m1^8*pp - 9600*d*m1^8*pp - 
          7680*d^2*m1^8*pp + 9312*d^3*m1^8*pp + 11680*d^4*m1^8*pp - 
          6560*d^5*m1^8*pp - 2880*d^6*m1^8*pp + 1088*d^7*m1^8*pp + 
          32*d^8*m1^8*pp + 6912*m1^6*pp^2 - 35712*d*m1^6*pp^2 + 
          1056*d^2*m1^6*pp^2 + 85296*d^3*m1^6*pp^2 - 
          37032*d^4*m1^6*pp^2 - 20484*d^5*m1^6*pp^2 + 
          8700*d^6*m1^6*pp^2 - 12*d^7*m1^6*pp^2 - 84*d^8*m1^6*pp^2 - 
          20736*m1^4*pp^3 + 58752*d*m1^4*pp^3 + 40560*d^2*m1^4*pp^3 - 
          150536*d^3*m1^4*pp^3 + 18980*d^4*m1^4*pp^3 + 
          48946*d^5*m1^4*pp^3 - 6748*d^6*m1^4*pp^3 - 146*d^7*m1^4*pp^3 + 
          128*d^8*m1^4*pp^3 + 3456*m1^2*pp^4 - 13824*d*m1^2*pp^4 - 
          4176*d^2*m1^2*pp^4 + 35784*d^3*m1^2*pp^4 - 
          8332*d^4*m1^2*pp^4 - 10162*d^5*m1^2*pp^4 + 
          1122*d^6*m1^2*pp^4 - 80*d^7*m1^2*pp^4 - 8*d^8*m1^2*pp^4 + 
          864*d*pp^5 - 216*d^2*pp^5 - 2268*d^3*pp^5 + 486*d^4*pp^5 + 
          729*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (432*(-3 + d)*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-3 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((-1152*m1^10 + 2880*d*m1^10 + 320*d^2*m1^10 - 3648*d^3*m1^10 + 
          1088*d^4*m1^10 + 768*d^5*m1^10 - 256*d^6*m1^10 - 192*m1^8*pp + 
          1312*d*m1^8*pp + 2080*d^2*m1^8*pp - 6880*d^3*m1^8*pp + 
          4032*d^4*m1^8*pp - 192*d^5*m1^8*pp - 160*d^6*m1^8*pp + 
          6336*m1^6*pp^2 - 6960*d*m1^6*pp^2 - 16960*d^2*m1^6*pp^2 + 
          17292*d^3*m1^6*pp^2 - 1828*d^4*m1^6*pp^2 - 828*d^5*m1^6*pp^2 + 
          68*d^6*m1^6*pp^2 - 9504*m1^4*pp^3 + 3624*d*m1^4*pp^3 + 
          26104*d^2*m1^4*pp^3 - 10938*d^3*m1^4*pp^3 - 
          4976*d^4*m1^4*pp^3 + 330*d^5*m1^4*pp^3 + 400*d^6*m1^4*pp^3 + 
          6336*m1^2*pp^4 - 3640*d*m1^2*pp^4 - 14584*d^2*m1^2*pp^4 + 
          9190*d^3*m1^2*pp^4 + 1374*d^4*m1^2*pp^4 - 600*d^5*m1^2*pp^4 - 
          56*d^6*m1^2*pp^4 - 864*pp^5 + 732*d*pp^5 + 1864*d^2*pp^5 - 
          1695*d^3*pp^5 + 148*d^4*pp^5 + 36*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (216*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((-4*m1^2 + pp)*
        (1152*m1^10 - 2880*d*m1^10 - 320*d^2*m1^10 + 3648*d^3*m1^10 - 
          1088*d^4*m1^10 - 768*d^5*m1^10 + 256*d^6*m1^10 + 576*m1^8*pp - 
          2208*d*m1^8*pp - 2304*d^2*m1^8*pp + 8000*d^3*m1^8*pp - 
          4256*d^4*m1^8*pp - 32*d^5*m1^8*pp + 224*d^6*m1^8*pp - 
          6336*m1^6*pp^2 + 6288*d*m1^6*pp^2 + 16320*d^2*m1^6*pp^2 - 
          14868*d^3*m1^6*pp^2 + 684*d^4*m1^6*pp^2 + 804*d^5*m1^6*pp^2 - 
          12*d^6*m1^6*pp^2 + 6912*m1^4*pp^3 - 3192*d*m1^4*pp^3 - 
          18160*d^2*m1^4*pp^3 + 7950*d^3*m1^4*pp^3 + 3128*d^4*m1^4*pp^3 - 
          78*d^5*m1^4*pp^3 - 160*d^6*m1^4*pp^3 - 3456*m1^2*pp^4 + 
          1800*d*m1^2*pp^4 + 8136*d^2*m1^2*pp^4 - 4322*d^3*m1^2*pp^4 - 
          1042*d^4*m1^2*pp^4 + 128*d^5*m1^2*pp^4 + 16*d^6*m1^2*pp^4 + 
          432*pp^5 - 324*d*pp^5 - 972*d^2*pp^5 + 729*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (216*(-3 + d)*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
 TVR[6 + d_Symbol, pp_, {{2, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   -((-1536*d*m1^10 + 2560*d^2*m1^10 + 3200*d^3*m1^10 - 3584*d^4*m1^10 - 
           2176*d^5*m1^10 + 1024*d^6*m1^10 + 512*d^7*m1^10 - 
           3072*m1^8*pp + 6528*d*m1^8*pp + 5376*d^2*m1^8*pp - 
           7520*d^3*m1^8*pp - 5568*d^4*m1^8*pp + 3232*d^5*m1^8*pp + 
           1344*d^6*m1^8*pp - 320*d^7*m1^8*pp - 2304*m1^6*pp^2 + 
           13824*d*m1^6*pp^2 + 192*d^2*m1^6*pp^2 - 31584*d^3*m1^6*pp^2 + 
           11232*d^4*m1^6*pp^2 + 8088*d^5*m1^6*pp^2 - 
           2304*d^6*m1^6*pp^2 - 24*d^7*m1^6*pp^2 + 6912*m1^4*pp^3 - 
           19584*d*m1^4*pp^3 - 12688*d^2*m1^4*pp^3 + 
           50584*d^3*m1^4*pp^3 - 7516*d^4*m1^4*pp^3 - 
           16982*d^5*m1^4*pp^3 + 2564*d^6*m1^4*pp^3 + 310*d^7*m1^4*pp^3 - 
           1152*m1^2*pp^4 + 4608*d*m1^2*pp^4 + 1392*d^2*m1^2*pp^4 - 
           11896*d^3*m1^2*pp^4 + 2820*d^4*m1^2*pp^4 + 
           3374*d^5*m1^2*pp^4 - 414*d^6*m1^2*pp^4 + 8*d^7*m1^2*pp^4 - 
           288*d*pp^5 + 72*d^2*pp^5 + 756*d^3*pp^5 - 162*d^4*pp^5 - 
           243*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (144*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((2304*m1^10 - 5760*d*m1^10 - 640*d^2*m1^10 + 7296*d^3*m1^10 - 
          2176*d^4*m1^10 - 1536*d^5*m1^10 + 512*d^6*m1^10 - 
          1344*m1^8*pp + 1120*d*m1^8*pp - 1376*d^2*m1^8*pp + 
          5280*d^3*m1^8*pp - 4896*d^4*m1^8*pp + 1280*d^5*m1^8*pp - 
          64*d^6*m1^8*pp - 5184*m1^6*pp^2 + 7584*d*m1^6*pp^2 + 
          14800*d^2*m1^6*pp^2 - 21496*d^3*m1^6*pp^2 + 
          6120*d^4*m1^6*pp^2 + 280*d^5*m1^6*pp^2 - 184*d^6*m1^6*pp^2 + 
          9504*m1^4*pp^3 - 3720*d*m1^4*pp^3 - 27296*d^2*m1^4*pp^3 + 
          15378*d^3*m1^4*pp^3 + 4114*d^4*m1^4*pp^3 - 666*d^5*m1^4*pp^3 - 
          674*d^6*m1^4*pp^3 - 6336*m1^2*pp^4 + 5720*d*m1^2*pp^4 + 
          13712*d^2*m1^2*pp^4 - 13926*d^3*m1^2*pp^4 + 
          1284*d^4*m1^2*pp^4 + 946*d^5*m1^2*pp^4 - 80*d^6*m1^2*pp^4 + 
          864*pp^5 - 1020*d*pp^5 - 1636*d^2*pp^5 + 2311*d^3*pp^5 - 
          693*d^4*pp^5 + 20*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((4*m1^2 - pp)*
        (-768*m1^10 + 1664*d*m1^10 + 768*d^2*m1^10 - 2176*d^3*m1^10 + 
          512*d^5*m1^10 + 192*m1^8*pp + 288*d*m1^8*pp + 704*d^2*m1^8*pp - 
          2272*d^3*m1^8*pp + 1024*d^4*m1^8*pp + 64*d^5*m1^8*pp + 
          1920*m1^6*pp^2 - 1824*d*m1^6*pp^2 - 5472*d^2*m1^6*pp^2 + 
          4584*d^3*m1^6*pp^2 - 168*d^5*m1^6*pp^2 - 2304*m1^4*pp^3 + 
          616*d*m1^4*pp^3 + 6192*d^2*m1^4*pp^3 - 1946*d^3*m1^4*pp^3 - 
          1128*d^4*m1^4*pp^3 - 230*d^5*m1^4*pp^3 + 1152*m1^2*pp^4 - 
          600*d*m1^2*pp^4 - 2744*d^2*m1^2*pp^4 + 1414*d^3*m1^2*pp^4 + 
          374*d^4*m1^2*pp^4 - 16*d^5*m1^2*pp^4 - 144*pp^5 + 108*d*pp^5 + 
          324*d^2*pp^5 - 243*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["five -> four", "Subsection",
 CellTags->"T1.6.121.1"],

Cell["\<\
TFI[d_, PP_/;Head[PP]=!=List,{ {0, m1_}, {n2_, m2_}, 
                               {n3_, m3_}, {n4_, m4_}, 
                               {n5_, m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", " ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n5_", ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,  PP_/;Head[PP]=!=List,{ {(n2_), m2_},{0, m1_},
                               {(n4_), m4_}, {(n3_), m3_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "  ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,   PP_/;Head[PP]=!=List,{ {(n3_), m3_}, {(n4_), m4_},
\t\t                           {0, m1_}, {(n2_), m2_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "   ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", "\n", 
      "\t\t                           ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\
TFI[d_,   PP_/;Head[PP]=!=List,{ {(n4_), m4_},{(n3_), m3_},
\t\t                           {(n2_), m2_}, {0, m1_}, 
                               {(n5_), m5_} }] := 
TVI[d,PP, {{n5,m5}, {n2,m2}, {n3,m3}, {n4,m4}}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TFR", "[", 
   RowBox[{"d_", ",", "   ", 
    RowBox[{"PP_", "/;", 
     RowBox[{
      RowBox[{"Head", "[", "PP", "]"}], "=!=", "List"}]}], ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n4_", ")"}], ",", " ", "m4_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}], ",", "\n", 
      "\t\t                           ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n5_", ")"}], ",", " ", "m5_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TVI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n5", ",", "m5"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

TFI[d_,  PP_/;Head[PP]=!=List,{{(n1_), m1_}, 

    {(n2_), m2_}, 

    {(n3_), m3_}, 

    {(n4_), m4_}, {0, m5_}}] := 

  TBI[d, PP, {{n1, m1}, {n3, m3}}]*

   TBI[d, PP, {{n2, m2}, {n4, m4}}]
\
\>", "Text"],

Cell["\<\
TFR[d_, PP_ /; Head[PP] =!= List, 
   {{n1_, m1_}, {n2_, m2_}, {n3_, m3_}, {n4_, m4_}, 
    {0, m5_}}] := TBI[d, PP, {{n1, m1}, {n3, m3}}]*
   TBI[d, PP, {{n2, m2}, {n4, m4}}]\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["four -> three", "Subsection",
 CellTags->"T1.6.122.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n4_", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " ", 
   RowBox[{"TBI", "[", 
    RowBox[{"d", ",", "PP", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n2", ",", "m2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n4", ",", "m4"}], "}"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell["\<\
TVI[d_, PP_, {{n1_, m1_}, {0, m2_}, {n3_, m3_}, {n4_, m4_}}] := 
  TJI[d, 0, {{n1, m1}, {n3, m3}, {n4, m4}}]\
\>", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TVI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1_", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"n2_", ",", " ", "m2_"}], "}"}], ",", " ", "\n", 
      "                               ", 
      RowBox[{"{", 
       RowBox[{"n3_", ",", " ", "m3_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m4_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TJI", "[", 
   RowBox[{"d", ",", "PP", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}]}], " ", "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["three -> two", "Subsection",
 CellTags->"T1.6.123.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n3_", ")"}], ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n3", ",", "m3"}], "}"}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m3_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], "  ", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["two -> one", "Subsection",
 CellTags->"T1.6.124.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n2_", ")"}], ",", " ", "m2_"}], "}"}]}], " ", "}"}]}], 
   "]"}], " ", ":=", " ", "\n", 
  RowBox[{"TAI", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"n2", ",", "m2"}], "}"}], "}"}]}], "]"}], " "}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBI", "[", 
   RowBox[{"d_", ",", " ", "PP_", ",", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"(", "n1_", ")"}], ",", " ", "m1_"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "m2_"}], "}"}]}], " ", "}"}]}], "]"}], " ", ":=",
   " ", "\n", 
  RowBox[{"TAI", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"n1", ",", "m1"}], "}"}], "}"}]}], "]"}], " "}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n_Integer", ",", "m_"}], "}"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"{", 
      RowBox[{"n_Integer", ",", "m_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TAI", "[", 
    RowBox[{"d_", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"n_Integer", ",", "m_"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TAI", "[", 
    RowBox[{"d", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"n", ",", "m"}], "}"}], "}"}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse and TFIRecurse", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.7.1"],

Cell[CellGroupData[{

Cell["TFIRecurse & TFISimplify", "Subsection",
 CellTags->"T1.7.1.1"],

Cell["This cancels scalar products.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$TLRComment", "=", "False"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLRComment", "[", "s_String", "]"}], ":=", 
   RowBox[{
    RowBox[{"WriteString", "[", 
     RowBox[{"\"\<stdout\>\"", ",", "s", ",", "\"\< \>\""}], "]"}], "/;", 
    RowBox[{"$TLRComment", "===", "True"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"applytlrules", "[", 
    RowBox[{"y_", ",", "fun_"}], "]"}], " ", ":=", 
   RowBox[{"fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"y", " ", "/.", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", "TLRules"}], 
     "/.", 
     RowBox[{"TLR", "->", 
      StyleBox["TFI",
       FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFIRecurse", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"f_", ":", "Identity"}]}], "]"}], " ", ":=", " ", 
   RowBox[{"FixedPoint", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"applytlrules", "[", 
       RowBox[{"#", ",", "f"}], "]"}], "&"}], ",", "z", ",", "1000"}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"applytlrules2", "[", 
    RowBox[{"y_", ",", "fun_"}], "]"}], " ", ":=", 
   RowBox[{"fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"y", " ", "/.", 
       RowBox[{
        StyleBox["TFI",
         FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", "TLRules2"}],
      "/.", 
     RowBox[{"TLR", "->", 
      StyleBox["TFI",
       FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TFISimplify", "[", 
    RowBox[{"z_", ",", 
     RowBox[{"f_", ":", "Identity"}]}], "]"}], " ", ":=", " ", 
   RowBox[{"FixedPoint", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"applytlrules2", "[", 
       RowBox[{"#", ",", "f"}], "]"}], "&"}], ",", "z", ",", "1000"}], 
    "]"}]}], ";"}]], "Input"],

Cell["\<\
TFIRecurse[z_]:=z/.TFI\[Rule]TLR//.TLRules/.TLR\[Rule]TFI;\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ExpandMaybe", "=", "Identity"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fN", "[", "li_List", "]"}], ":=", 
   RowBox[{
    RowBox[{"fN", "[", "li", "]"}], "=", 
    RowBox[{"li", "/.", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], "\[RuleDelayed]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}]}]}]}]}], ";"}]], "Input"],

Cell["\<\
RM: new Feb. 1999  
need TLR[ ...]:= rule;
(copy from TFI definition) (in order to get e.g. 
TarcerRecurse[TFI[D,pp,{1,1,1,1,1},{{1,0},{1,M},{0,0},{1,m},{1,M}}]]
to work)\
\>", "Text",
 CellTags->"TLRdef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"depp__", ",", 
     RowBox[{"{", 
      RowBox[{"a___", ",", "b_Integer", ",", "c___"}], "}"}]}], "]"}], ":=", 
   RowBox[{"TLR", "[", 
    RowBox[{"depp", ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"b", ",", "0"}], "}"}], ",", "c"}], "}"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "1", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      "pr_"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"NoTLRComment", "[", "\"\<tlrule1\>\"", "]"}], ";", 
      RowBox[{
       StyleBox["TFI",
        FontColor->RGBColor[1, 0.6, 0]], "[", 
       RowBox[{"d", ",", "pp", ",", 
        RowBox[{"pr", "/.", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "_"}], "}"}], "\[RuleDelayed]", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}]}]}]}], "]"}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell["(* q1^2 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "2", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n1_", "?", "PQ"}], ",", "m1_"}], "}"}], ",", "nm2_", ",", 
        "nm3_", ",", "nm4_", ",", "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule2\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"v", "\[LessEqual]", "n1"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m1", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "w", ",", "x", ",", "y", ",", "z"}], "}"}], 
             ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n1", "-", "v"}], ",", "0"}], "}"}], ",", "nm2", ",",
                "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "v"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"v", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", " ", 
               RowBox[{"{", 
                RowBox[{"0", ",", "w", ",", "x", ",", "y", ",", "z"}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n1", "-", "i"}], ",", "m1"}], "}"}], "]"}], ",", 
                 "nm2", ",", "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], 
             " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m1", "2"], ")"}], 
              RowBox[{"v", "-", "i"}]]}]}]}], "]"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m1", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"v", "-", "n1"}], ",", "w", ",", "x", ",", "y", ",", 
               "z"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", "nm3", ",", 
               "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "n1"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"n1", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"v", "-", "n1"}], ",", "w", ",", "x", ",", "y", ",", 
                 "z"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n1", "-", "i"}], ",", "m1"}], "}"}], "]"}], ",", 
                 "nm2", ",", "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], 
             " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m1", "2"], ")"}], 
              RowBox[{"n1", "-", "i"}]]}]}]}], "]"}]}], "]"}], "]"}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["q", "2", "2"], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "3", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2_", "?", "PQ"}], ",", "m2_"}], "}"}], ",", "nm3_", ",", 
        "nm4_", ",", "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule3\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"w", "\[LessEqual]", "n2"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m2", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"v", ",", "0", ",", "x", ",", "y", ",", "z"}], "}"}], 
             ",", 
             RowBox[{"{", 
              RowBox[{"nm1", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n2", "-", "w"}], ",", "0"}], "}"}], ",", "nm3", ",",
                "nm4", ",", "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "w"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"w", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{"v", ",", "0", ",", "x", ",", "y", ",", "z"}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"nm1", ",", 
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n2", "-", "i"}], ",", "m2"}], "}"}], "]"}], ",", 
                 "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m2", "2"], ")"}], 
              RowBox[{"w", "-", "i"}]]}]}]}], "]"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"m2", "===", "0"}], ",", 
           RowBox[{"TLR", "[", 
            RowBox[{"dpp", ",", 
             RowBox[{"{", 
              RowBox[{"v", ",", 
               RowBox[{"w", "-", "n2"}], ",", "x", ",", "y", ",", "z"}], 
              "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"nm1", ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", "nm4", ",", 
               "nm5"}], "}"}]}], "]"}], ",", 
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "n2"], 
            RowBox[{
             RowBox[{"Binomial", "[", 
              RowBox[{"n2", ",", "i"}], "]"}], " ", 
             RowBox[{"TLR", "[", 
              RowBox[{"dpp", ",", 
               RowBox[{"{", 
                RowBox[{"v", ",", 
                 RowBox[{"w", "-", "n2"}], ",", "x", ",", "y", ",", "z"}], 
                "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"nm1", ",", 
                 RowBox[{"fN", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"n2", "-", "i"}], ",", "m2"}], "}"}], "]"}], ",", 
                 "nm3", ",", "nm4", ",", "nm5"}], "}"}]}], "]"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox["m2", "2"], ")"}], 
              RowBox[{"n2", "-", "i"}]]}]}]}], "]"}]}], "]"}], "]"}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"p", "\[CenterDot]", 
   SubscriptBox["q", "1"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "4", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", "y_", ",", "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n1_", "?", "PQ"}], ",", "m1_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule4\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "+", "1"}], ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], "]"}], ",", 
             "nm4", ",", "nm5"}], "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", "dpp", "}"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "-", 
           SuperscriptBox["m3", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", 
             RowBox[{"x", "-", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"p", "\[CenterDot]", 
   SubscriptBox["q", "2"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "5", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n2_", "?", "PQ"}], ",", "m2_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule5\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "+", "1"}], ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], "]"}], ",", 
             "nm5"}], "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", "dpp", "}"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "-", 
           SuperscriptBox["m4", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", 
             RowBox[{"y", "-", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["q", "1"], "\[CenterDot]", 
   SubscriptBox["q", "2"]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "6", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"dpp__", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", 
        RowBox[{"z_", "?", "PQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n5_", "?", "PQ"}], ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule6\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "+", "1"}], ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"fN", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"n5", "-", "1"}], ",", "m5"}], "}"}], "]"}]}], 
            "}"}]}], "]"}]}], "+", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "+", "1"}], ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         SuperscriptBox["m5", "2"], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"dpp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", "w", ",", "x", ",", "y", ",", 
             RowBox[{"z", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n1", ",", "m1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], "]"}]}],
      ")"}]}]}], ";"}]], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "7", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp___", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "z_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule7\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"v", "-", "1"}], ",", "w", ",", "x", ",", "y", ",", "z"}],
            "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
            RowBox[{"fN", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], "]"}], ",", 
            "nm4", ",", "nm5"}], "}"}]}], "]"}], "+", 
        RowBox[{"2", " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "-", "1"}], ",", "w", ",", 
             RowBox[{"x", "+", "1"}], ",", "y", ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"pp", "-", 
           SuperscriptBox["m3", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"v", "-", "1"}], ",", "w", ",", "x", ",", "y", ",", 
             "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", 
             RowBox[{"{", 
              RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", "nm5"}], 
            "}"}]}], "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* q2.q2  --> q2.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "8", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp___", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule8\>\"", "]"}], ";", 
      RowBox[{"ExpandMaybe", "[", 
       RowBox[{
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", 
            RowBox[{"w", "-", "1"}], ",", "x", ",", "y", ",", "z"}], "}"}], 
          ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
            RowBox[{"fN", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], "]"}], ",", 
            "nm5"}], "}"}]}], "]"}], "+", 
        RowBox[{"2", " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "-", "1"}], ",", "x", ",", 
             RowBox[{"y", "+", "1"}], ",", "z"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}], "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"pp", "-", 
           SuperscriptBox["m4", "2"]}], ")"}], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"v", ",", 
             RowBox[{"w", "-", "1"}], ",", "x", ",", "y", ",", "z"}], "}"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm5"}], "}"}]}], 
          "]"}]}]}], "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* eq. (3) - (5) *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "9", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", 
        RowBox[{"z_", "?", "PQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule9\>\"", "]"}], ";", 
      RowBox[{"FunctionExpand", "[", 
       RowBox[{"ExpandMaybe", "[", 
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"k", "=", "0"}], 
          RowBox[{"Floor", "[", 
           FractionBox["z", "2"], "]"}]], 
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], "k"], 
          RowBox[{
           UnderoverscriptBox["\[Sum]", 
            RowBox[{"j", "=", "0"}], "k"], 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Gamma", "[", 
               RowBox[{"k", "+", 
                FractionBox["1", "2"]}], "]"}], " ", 
              RowBox[{"Gamma", "[", 
               RowBox[{
                FractionBox["d", "2"], "-", 
                FractionBox["1", "2"]}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"z", ",", 
                RowBox[{"2", " ", "k"}]}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"k", ",", "i"}], "]"}], " ", 
              RowBox[{"Binomial", "[", 
               RowBox[{"k", ",", "j"}], "]"}], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], 
               RowBox[{"i", "+", "j"}]], " ", 
              SuperscriptBox["pp", 
               RowBox[{"i", "+", "j", "-", "z"}]], " ", 
              RowBox[{"TLR", "[", 
               RowBox[{"d", ",", "pp", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"v", "+", "i"}], ",", 
                  RowBox[{"w", "+", "j"}], ",", 
                  RowBox[{"x", "+", "z", "-", 
                   RowBox[{"2", " ", "i"}]}], ",", 
                  RowBox[{"y", "+", "z", "-", 
                   RowBox[{"2", " ", "j"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], ")"}], 
            "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Gamma", "[", 
               FractionBox["1", "2"], "]"}], " ", 
              RowBox[{"Gamma", "[", 
               RowBox[{"k", "+", 
                FractionBox["d", "2"], "-", 
                FractionBox["1", "2"]}], "]"}]}], ")"}]}]}]}]}], "]"}], 
       "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[" OLD trule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
(* (9) and (10) *)
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {{n1_?PQ,m1_},{n2_?PQ,m2_}, 
                                {0,_},{0,_},nm5_}] :>  
( TLRComment[\"tlrule10\"]; 0 /; OddQ[x+y] );\
\>", "Text"],

Cell["New tlrule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
   TLR[d_, pp_, {v_, w_, (x_)?PNQ, (y_)?PNQ, z_}, 
     {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {0, _}, {0, _}, nm5_}] :> \
(TLRComment[\"tlrule10\"]; 0 /; OddQ[x + y]); \
\>", "Input"],

Cell["\<\
Bug in  tlrule[11] and tlrule[12] fixed on Feb. 26th 1998 by R.Scharf 
with thanks to York Schroeder for locating the bug.\
\>", "Text"],

Cell["\<\

tlrule[11] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule11\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (x-1) TLR[d, pp, {v+1,w,x-2,y,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   y TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && x > 1
);\
\>", "Input"],

Cell["\<\

tlrule[12] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule12\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (y-1) TLR[d, pp, {v,w+1,x,y-2,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   x TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && y > 1
);\
\>", "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "13", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0", ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n4_", "?", "PQ"}], ",", "m4_"}], "}"}], ",", "nm5_"}], 
       "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule13\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "y"], 
       RowBox[{
        RowBox[{"Binomial", "[", 
         RowBox[{"y", ",", "i"}], "]"}], " ", 
        SuperscriptBox["pp", 
         RowBox[{"y", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "i", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm3", ",", 
            RowBox[{"{", 
             RowBox[{"n4", ",", "m4"}], "}"}], ",", "nm1", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm5"}], "}"}]}], 
         "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "14", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n3_", "?", "PQ"}], ",", "m3_"}], "}"}], ",", "nm4_", ",", 
        "nm5_"}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule14\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "x"], 
       RowBox[{
        RowBox[{"Binomial", "[", 
         RowBox[{"x", ",", "i"}], "]"}], " ", 
        SuperscriptBox["pp", 
         RowBox[{"x", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "i", ",", "0", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"n3", ",", "m3"}], "}"}], ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", "nm2", ",", "nm5"}], 
           "}"}]}], "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["(* 0's *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "15", "]"}], "=", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"TLR", "[", 
      RowBox[{"d_", ",", "pp_", ",", 
       RowBox[{"{", 
        RowBox[{"v_", ",", "w_", ",", "x_", ",", "y_", ",", "z_"}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"n1_", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"n2_", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule15\>\"", "]"}], ";", "0"}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "16", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", 
        RowBox[{"x_", "?", "PQ"}], ",", 
        RowBox[{"y_", "?", "PQ"}], ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule16\>\"", "]"}], ";", 
      RowBox[{"0", "/;", 
       RowBox[{
        RowBox[{"OddQ", "[", "x", "]"}], "||", 
        RowBox[{"OddQ", "[", "y", "]"}]}]}]}], ")"}]}]}], ";"}]], "Input"],

Cell["bug in tlrule[17] fixedApril 23rd 1999", "Text",
 CellTags->"tlrule17"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "17", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PQ"}], ",", "b_", ",", "0", ",", "de_", ",", 
        "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule17\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "i"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"i", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"]}], ")"}], " ", 
         SuperscriptBox["2", 
          RowBox[{"j", "-", "1"}]], " ", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a", "-", "i"}], ",", 
             RowBox[{"b", "+", "i", "-", "j"}], ",", "0", ",", "de", ",", 
             "j"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"n5", ",", "m5"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n2", ",", "m2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"n4", ",", "m4"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "18", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PNQ"}], ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n2_", ",", "m2_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3_", ",", "m3_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule18\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "b"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"b", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"pp", "+", 
              SuperscriptBox["m3", "2"]}], ")"}], " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n2", ",", "m2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
           RowBox[{"TLR", "[", 
            RowBox[{"d", ",", "pp", ",", "dp", ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "j"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n3", "-", "1"}], ",", "m3"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n4", ",", "m4"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n2", ",", "m2"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0", ",", "1", ",", "0", ",", "0"}], "}"}], 
              ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n2", ",", "m2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], 
          ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"i", "+", "j"}]], " ", 
         SuperscriptBox["dp", 
          RowBox[{"a", "+", "b", "-", "i", "-", "j"}]]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "19", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"a_", "?", "PNQ"}], ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n1_", ",", "m1_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n3_", ",", "m3_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n4_", ",", "m4_"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n5_", ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule18\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], "a"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], "b"], 
        RowBox[{
         RowBox[{"Binomial", "[", 
          RowBox[{"a", ",", "i"}], "]"}], " ", 
         RowBox[{"Binomial", "[", 
          RowBox[{"b", ",", "j"}], "]"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"pp", "+", 
              SuperscriptBox["m4", "2"]}], ")"}], " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n1", ",", "m1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}], "+", 
           RowBox[{"TLR", "[", 
            RowBox[{"d", ",", "pp", ",", "dp", ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "j"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"n3", ",", "m3"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"n4", "-", "1"}], ",", "m4"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n1", ",", "m1"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "0"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}], "-", 
           RowBox[{"2", " ", 
            RowBox[{"TLR", "[", 
             RowBox[{"d", ",", "pp", ",", "dp", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "j"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], 
              ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"n3", ",", "m3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n4", ",", "m4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n1", ",", "m1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"n5", ",", "m5"}], "}"}]}], "}"}]}], "]"}]}]}], 
          ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"i", "+", "j"}]], " ", 
         SuperscriptBox["dp", 
          RowBox[{"a", "+", "b", "-", "i", "-", "j"}]]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "20", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"v_", "?", "PQ"}], ",", "w_", ",", "x_", ",", "y_", ",", 
        "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm2_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"n5_", "?", "PQ"}], ",", "m5_"}], "}"}]}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule20\>\"", "]"}], ";", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], "x"], 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], "v"], 
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"r", "=", "0"}], "i"], 
         RowBox[{
          RowBox[{"Binomial", "[", 
           RowBox[{"i", ",", "r"}], "]"}], " ", 
          RowBox[{"Binomial", "[", 
           RowBox[{"v", ",", "i"}], "]"}], " ", 
          RowBox[{"Binomial", "[", 
           RowBox[{"x", ",", "j"}], "]"}], " ", 
          SuperscriptBox["2", "r"], " ", 
          RowBox[{"TLR", "[", 
           RowBox[{"d", ",", "pp", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"v", "-", "i"}], ",", 
              RowBox[{
               RowBox[{"-", "r"}], "+", "i", "+", "w"}], ",", "j", ",", 
              RowBox[{"x", "-", "j", "+", "y"}], ",", "r"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"n5", ",", "m5"}], "}"}], ",", "nm2", ",", "0", ",", 
              "nm4", ",", "0"}], "}"}]}], "]"}]}]}]}]}]}], ")"}]}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "21", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", 
        RowBox[{"w_", "?", "PQ"}], ",", "x_", ",", "y_", ",", "0"}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm3_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}], ",", "nm5_"}], "}"}]}], "]"}], 
    "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule21\>\"", "]"}], ";", 
      RowBox[{"TLR", "[", 
       RowBox[{"d", ",", "pp", ",", 
        RowBox[{"{", 
         RowBox[{"w", ",", "v", ",", "y", ",", "x", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", "nm1", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", "nm3", ",", "nm5"}], "}"}]}], 
       "]"}]}], ")"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "22", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule22\>\"", "]"}], ";", 
      RowBox[{
       FractionBox["1", "pp"], 
       RowBox[{"(", 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "w", ",", "1", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[{
 "Using  FeynCalc : \n\n",
 Cell[BoxData[
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           FormBox[
            RowBox[{
             FormBox["k1",
              TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], ")"}], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"],
 "\n",
 Cell[BoxData[
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Calc", "[", " ", 
     RowBox[{
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"k2", ",", "\[Nu]"}], "]"}], "%"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k1", ",", "k2"}], "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "-", 
    FractionBox[
     RowBox[{"D", " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}]}], 
     RowBox[{"1", "-", "D"}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]]}], TraditionalForm]], "Output"],
 "\n"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "23", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule23\>\"", "]"}], ";", 
      RowBox[{
       RowBox[{
        FractionBox["1", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"b", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "2", ",", "y", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox[
         RowBox[{"d", "  ", "dp"}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "2"]}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "b"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "2", ",", 
             RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox["1", 
         RowBox[{"1", "-", "d"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"b", "+", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "0", ",", "y", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "+", 
       RowBox[{
        FractionBox["dp", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "0", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["k1",
         TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"D", "+", "2"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            FormBox[
             RowBox[{
              FormBox["k1",
               TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
              FormBox["p",
               TraditionalForm]}],
             TraditionalForm], ")"}], "2"]}], "-", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           FormBox["k1",
            TraditionalForm], "2"], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"]}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "D"}], ")"}], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "6"]}]]}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[NoBreak]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox["k1",
             TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            FormBox["p",
             TraditionalForm]}],
           TraditionalForm], ")"}], "2"], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["k1",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"],
 "\n",
 Cell[BoxData[
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Calc", "[", " ", 
     RowBox[{
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"OPEDelta", ",", "\[Nu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"k2", ",", "\[Rho]"}], "]"}], "%"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k1", ",", "k2"}], "}"}]}], "]"}]], "Input"],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "3"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]], "-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "+", "2"}], ")"}], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["k1",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "3"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "6"]}]], "-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]], "+", 
    FractionBox[
     RowBox[{"3", " ", 
      SuperscriptBox[
       FormBox["k1",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k2",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], ")"}], "2"], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       FormBox["p",
        TraditionalForm]}]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "4"]}]]}], TraditionalForm]], "Output"]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tlrule", "[", "24", "]"}], "=", 
   RowBox[{
    RowBox[{"TLR", "[", 
     RowBox[{"d_", ",", "pp_", ",", "dp_", ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", 
        RowBox[{"b_", "?", "PNQ"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"w_", "?", "PNQ"}], ",", "0", ",", 
        RowBox[{"y_", "?", "PNQ"}], ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"nm1_", ",", "nm2_", ",", "nm3_", ",", "nm4_", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"TLRComment", "[", "\"\<tlrule24\>\"", "]"}], ";", 
      RowBox[{
       RowBox[{"2", "dp", 
        RowBox[{
         FractionBox["1", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", "-", "d"}], ")"}], 
           SuperscriptBox["pp", "2"]}]], 
         RowBox[{"(", 
          RowBox[{"TLR", "[", 
           RowBox[{"d", ",", "pp", ",", "dp", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", 
              RowBox[{"b", "+", "1"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "w", ",", "3", ",", "y", ",", "0"}], "}"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}]}], 
       "-", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{"d", "+", "2"}], ")"}], "  ", 
          SuperscriptBox["dp", "2"]}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "3"]}]], 
        RowBox[{"(", 
         RowBox[{"TLR", "[", 
          RowBox[{"d", ",", "pp", ",", "dp", ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "b"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "w", ",", "3", ",", 
             RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}], ")"}]}], "-", 
       
       RowBox[{
        FractionBox[
         RowBox[{"2", "dp"}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], "pp"}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"b", "+", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "1", ",", "y", ",", "0"}], "}"}], ",", 
          
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "+", 
       RowBox[{
        FractionBox[
         RowBox[{"3", 
          SuperscriptBox["dp", "2"]}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "d"}], ")"}], 
          SuperscriptBox["pp", "2"]}]], 
        RowBox[{"TLR", "[", 
         RowBox[{"d", ",", "pp", ",", "dp", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "w", ",", "1", ",", 
            RowBox[{"y", "+", "1"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"nm1", ",", "nm2", ",", "nm3", ",", "nm4", ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}]}], 
     ")"}]}]}], ";"}]], "Input"],

Cell["Symmetry 1 <--> 2; added February 21 1999 by R.M.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tlrule", "[", "25", "]"}], "=", 
    RowBox[{
     RowBox[{"TLR", "[", 
      RowBox[{"dim_", ",", "pp_", ",", " ", 
       RowBox[{"{", 
        RowBox[{"a_", ",", " ", "b_", ",", " ", 
         RowBox[{
          RowBox[{"(", "c_", ")"}], "?", "PNQ"}], ",", " ", 
         RowBox[{
          RowBox[{"(", "d_", ")"}], "?", "PQ"}], ",", " ", "0"}], "}"}], ",", 
       " ", 
       RowBox[{"{", 
        RowBox[{"nm1_", ",", " ", "nm2_", ",", " ", "nm3_", ",", " ", 
         RowBox[{"{", 
          RowBox[{"0", ",", " ", "_"}], "}"}], ",", " ", "nm5_"}], "}"}]}], 
      "\n", "   ", "]"}], " ", ":>", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"TLRComment", "[", "\"\<tlrule25\>\"", "]"}], ";", " ", 
        RowBox[{"TLR", "[", 
         RowBox[{"dim", ",", " ", "pp", ",", " ", 
          RowBox[{"{", 
           RowBox[{
           "b", ",", "a", ",", " ", "d", ",", " ", "c", ",", " ", "0"}], 
           "}"}], ",", " ", 
          RowBox[{"{", 
           RowBox[{"nm2", ",", " ", "nm1", ",", " ", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", " ", "nm3", ",", " ", 
            "nm5"}], "}"}]}], "]"}]}], ")"}], " ", "/;", " ", 
      RowBox[{"d", " ", ">", " ", "c"}]}]}]}], ";"}], "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TLRules", "=", 
   RowBox[{"Array", "[", 
    RowBox[{"tlrule", ",", "25"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TLRules2", "=", 
   RowBox[{"Drop", "[", 
    RowBox[{"TLRules", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "9"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TLR", "[", 
    RowBox[{"__", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "_"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "_"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"seeetDelayed", ",", "HoldAll"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"seeetDelayed", "[", 
   RowBox[{
    RowBox[{"TLR", "[", "a__", "]"}], ",", "b_"}], "]"}], ":=", 
  RowBox[{"SetDelayed", "@@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"TLR", "[", "a", "]"}], ",", "b"}], "}"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lis1", "=", 
   RowBox[{"Permutations", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"_", ",", "0"}], "}"}]}], "}"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lis2", "=", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"TLR", "[", 
        RowBox[{"__", ",", 
         RowBox[{
         "lis1", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}]}], 
        "]"}], "\[RuleDelayed]", "0"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "lis1", "]"}]}], "}"}]}], "]"}], "/.", 
    RowBox[{"RuleDelayed", "\[Rule]", "seeetDelayed"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"lis1", ",", "lis2"}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["T*IS", "Subsection",
 CellTags->"T1.7.2.1"],

Cell["\<\
These functions below are never used.
They can serve as tests.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TJIS", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"Power", "[", 
      RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{
    SuperscriptBox["SMu", "2"], 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", " ", 
          SuperscriptBox["M", "2"]}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "I"}], " ", "Pi", " ", 
        RowBox[{"(", 
         RowBox[{"d", "-", "4"}], ")"}]}], "]"}], " ", 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], 
        "]"}], 
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{
          FractionBox["3", "2"], "d"}], "-", "\[Alpha]", "-", "\[Beta]", "-", 
         "\[Gamma]"}], "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Alpha]", "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Beta]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Beta]", "]"}]], 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         FractionBox["d", "2"], "-", "\[Gamma]"}], "]"}], 
       RowBox[{"Gamma", "[", "\[Gamma]", "]"}]]}], ")"}]}]}], "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  "\t", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", 
    RowBox[{"M", "^", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  "\t", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", 
    RowBox[{"M", "^", "2"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", 
    RowBox[{"Power", "[", 
     RowBox[{"M_", ",", " ", "2"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Gamma]"}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Beta]"}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{"d", "-", "\[Beta]", "-", "\[Gamma]"}], "]"}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Beta]", "+", "\[Gamma]", "-", 
       FractionBox["d", "2"]}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"2", " ", "d"}], "-", "\[Alpha]", "-", 
       RowBox[{"2", "\[Beta]"}], "-", 
       RowBox[{"2", "\[Gamma]"}]}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{
      FractionBox[
       RowBox[{"3", "d"}], "2"], "-", "\[Alpha]", "-", "\[Beta]", "-", 
      "\[Gamma]"}], "]"}]]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "0"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Gamma", "[", 
       RowBox[{"\[Beta]", "+", "\[Gamma]", "-", 
        RowBox[{"d", "/", "2"}]}], "]"}], 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "\[Beta]"}], "]"}], 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"d", "/", "2"}], "-", "\[Gamma]"}], "]"}]}], ")"}], "/", 
    RowBox[{"Gamma", "[", 
     RowBox[{"d", "/", "2"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "M"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{"TJIS", "[", 
   RowBox[{"d", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]", ",", "M"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]", ",", "M"}], "}"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", "\n", 
  
  RowBox[{
   FractionBox[
    SuperscriptBox["SMu", "2"], 
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["M", "2"], ")"}], 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]]], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
    RowBox[{
     RowBox[{"Gamma", "[", "\[Beta]", "]"}], 
     RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}]], 
   FractionBox[
    RowBox[{"Gamma", "[", 
     RowBox[{
      FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], 
    RowBox[{"Gamma", "[", 
     FractionBox["d", "2"], "]"}]], 
   FractionBox[
    RowBox[{
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Gamma]", "-", 
       FractionBox["d", "2"]}], "]"}], 
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
       FractionBox["d", "2"]}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{
      RowBox[{"2", "\[Alpha]"}], "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], 
     "]"}]]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Beta]_", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Gamma]_", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["SMu", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "+", "1"}]], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "d"}], "]"}], 
     " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{
       FractionBox["d", "2"], "-", "\[Alpha]"}], "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Gamma]", "-", 
         FractionBox["d", "2"]}], "]"}], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
         FractionBox["d", "2"]}], "]"}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["M", "2"], ")"}], 
      RowBox[{"\[Alpha]", "+", "\[Beta]", "+", "\[Gamma]", "-", "4"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", "\[Beta]", "]"}], " ", 
       RowBox[{"Gamma", "[", "\[Gamma]", "]"}]}], ")"}], " ", 
     RowBox[{"Gamma", "[", 
      FractionBox["d", "2"], "]"}], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"2", " ", "\[Alpha]"}], "+", "\[Beta]", "+", "\[Gamma]", "-", 
       "d"}], "]"}]}], ")"}]}]}]], "Input"],

Cell["(2.65)", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TAIS", "[", 
   RowBox[{"d_", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"1", ",", "M_"}], "}"}], "}"}]}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"-", "I"}], "  ", 
   SuperscriptBox["M", "2"], "  ", "SMu", " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", "d"}], ")"}], "/", "2"}], "-", "1"}], 
    "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBIS", "[", 
    RowBox[{"d_", ",", "0", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", "0"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TBIS", "[", 
    RowBox[{"d_", ",", 
     RowBox[{"Power", "[", 
      RowBox[{"M_", ",", " ", "2"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Alpha]_", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]_", ",", "0"}], "}"}]}], "}"}]}], "]"}], ":=", 
   RowBox[{"I", " ", "SMu", " ", "*", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]"}]], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", " ", 
          SuperscriptBox["M", "2"]}], ")"}], 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "2"}]]], 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "I"}], " ", "Pi", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], "/", "2"}]}], "]"}], " ", 
      FractionBox[
       RowBox[{"Gamma", "[", 
        RowBox[{"\[Alpha]", "+", "\[Beta]", "-", 
         RowBox[{"d", "/", "2"}]}], "]"}], 
       RowBox[{
        RowBox[{"Gamma", "[", "\[Alpha]", "]"}], 
        RowBox[{"Gamma", "[", "\[Beta]", "]"}]}]], 
      FractionBox[
       RowBox[{
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"d", "/", "2"}], "-", "\[Alpha]"}], "]"}], 
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"d", "/", "2"}], "-", "\[Beta]"}], "]"}]}], 
       RowBox[{"Gamma", "[", 
        RowBox[{"d", "-", "\[Alpha]", "-", "\[Beta]"}], "]"}]]}], ")"}]}]}], 
  "\n"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TJIS", "[", 
   RowBox[{"d_Symbol", ",", 
    SuperscriptBox["M_", "2"], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "M_"}], "}"}]}], "}"}]}], "]"}], ":=", 
  RowBox[{
   SuperscriptBox["M", "2"], " ", 
   SuperscriptBox["SMu", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["6", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"]]}], "+", 
     FractionBox["17", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "4"}], "+", "d"}], ")"}]}]], "+", 
     RowBox[{
      FractionBox["1", "8"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "59"}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "4"}], "+", "d"}], ")"}], "3"], " ", "TJI111e"}], "+", 
     RowBox[{
      FractionBox["1", "1920"], 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{"16755", "-", 
          RowBox[{"4750", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", 
          RowBox[{"14", " ", 
           SuperscriptBox["\[Pi]", "4"]}], "+", 
          RowBox[{"3840", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"Log", "[", "2", "]"}]}], "-", 
          RowBox[{"12080", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}], ")"}]}], "+", 
     RowBox[{
      FractionBox["1", "96"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "4"}], "+", "d"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"195", "+", 
        RowBox[{"98", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", 
        RowBox[{"48", " ", 
         RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TComment", "Subsection",
 CellTags->"T1.7.3.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$CommentNotebook", "=", "True"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TComment", "[", 
   RowBox[{"s_String", ",", "b_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"ValueQ", "[", "$TarcerRecursed", "]"}]}], ",", 
        RowBox[{"$TarcerRecursed", "=", 
         RowBox[{"{", "}"}]}]}], "]"}], ";", 
      RowBox[{"!", 
       RowBox[{"MemberQ", "[", 
        RowBox[{"$TarcerRecursed", ",", 
         RowBox[{"{", 
          RowBox[{"s", ",", "b"}], "}"}]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"AppendTo", "[", 
       RowBox[{"$TarcerRecursed", ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "b"}], "}"}]}], "]"}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"$Notebooks", "&&", 
         RowBox[{"$CommentNotebook", "===", "True"}]}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"$commentnb", "===", "False"}], ",", 
          RowBox[{
           RowBox[{"$commentnb", "=", "True"}], ";", 
           RowBox[{"$cnb", "=", 
            RowBox[{"CreateWindow", "[", 
             RowBox[{
              RowBox[{
              "StyleDefinitions", "\[Rule]", "\"\<NaturalColor.nb\>\""}], ",", 
              RowBox[{"Magnification", "\[Rule]", "1.5`"}]}], "]"}]}], ";", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"WindowMargins", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "Automatic"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"Automatic", ",", "0"}], "}"}]}], "}"}]}]}], "]"}], 
           ";", 
           RowBox[{"SetOptions", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"WindowSize", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"500", ",", "300"}], "}"}]}]}], "]"}], ";", 
           RowBox[{"NotebookWrite", "[", 
            RowBox[{"$cnb", ",", 
             RowBox[{"Cell", "[", 
              RowBox[{
               RowBox[{"BoxData", "[", 
                RowBox[{"ToBoxes", "[", 
                 RowBox[{
                  RowBox[{"b", "/.", "\[InvisibleSpace]", 
                   RowBox[{"TFIC", "\[Rule]", "TFI"}]}], ",", 
                  "StandardForm"}], "]"}], "]"}], ",", "\"\<Input\>\"", ",", 
               RowBox[{"CellDingbat", "\[Rule]", 
                RowBox[{"s", "<>", "\"\< \>\""}]}]}], "]"}]}], "]"}]}], ",", 
          RowBox[{"NotebookWrite", "[", 
           RowBox[{"$cnb", ",", 
            RowBox[{"Cell", "[", 
             RowBox[{
              RowBox[{"BoxData", "[", 
               RowBox[{"ToBoxes", "[", 
                RowBox[{
                 RowBox[{"b", "/.", "\[InvisibleSpace]", 
                  RowBox[{"TFIC", "\[Rule]", "TFI"}]}], ",", "StandardForm"}],
                 "]"}], "]"}], ",", "\"\<Input\>\"", ",", 
              RowBox[{"CellDingbat", "\[Rule]", 
               RowBox[{"s", "<>", "\"\< \>\""}]}]}], "]"}]}], "]"}]}], "]"}], 
        ",", 
        RowBox[{"Print", "[", 
         RowBox[{"s", ",", 
          RowBox[{"b", "/.", "\[InvisibleSpace]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"TFIC", "\[RuleDelayed]", "TFI"}], ",", 
             RowBox[{"TVIC", "\[RuleDelayed]", "TVI"}], ",", 
             RowBox[{"TJIC", "\[RuleDelayed]", "TJI"}], ",", 
             RowBox[{"TBIC", "\[RuleDelayed]", "TBI"}], ",", 
             RowBox[{"TAIC", "\[RuleDelayed]", "TAI"}]}], "}"}]}]}], "]"}]}], 
       "]"}]}]}], "]"}], "/;", 
   RowBox[{"$Comment", "===", "True"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$TTable", "Subsection",
 CellTags->"$TTable"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$TTable", "=", 
   RowBox[{"{", "}"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse", "Subsection",
 CellTags->"T1.7.4.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FEPrint", "[", "a__String", "]"}], ":=", 
   RowBox[{
    RowBox[{"Print", "[", "a", "]"}], " ", "/;", " ", 
    RowBox[{"$Notebooks", "===", "False"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FEPrint", "[", "a__String", "]"}], ":=", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "$cnb", "]"}], "===", "NotebookObject"}], ",", 
      "\n", "\t\t", 
      RowBox[{
       RowBox[{"NotebookWrite", "[", 
        RowBox[{"$cnb", ",", 
         RowBox[{"Cell", "[", 
          RowBox[{
           RowBox[{"StringJoin", "@@", 
            RowBox[{"{", "a", "}"}]}], ",", "\"\<SmallText\>\""}], "]"}]}], 
        "]"}], ";", "\n", "\t", 
       RowBox[{"SelectionMove", "[", 
        RowBox[{"$cnb", ",", "After", ",", "Cell"}], "]"}]}]}], "]"}], "/;", 
    RowBox[{"$Notebooks", "===", "True"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "TarcerRecurse", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Factor", "\[Rule]", "Factor"}], ",", 
     RowBox[{"TimeConstraint", "->", "Infinity"}], ",", 
     RowBox[{"Table", ":>", "$TTable"}], ",", 
     RowBox[{"Together", "->", "Together"}], ",", 
     RowBox[{"Replace", "->", 
      RowBox[{"{", "}"}]}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerRecurse", "[", 
    RowBox[{"z_", ",", "opts___"}], "]"}], ":=", 
   RowBox[{"Catch", "[", 
    RowBox[{
     StyleBox["Block",
      FontSize->16,
      FontWeight->"Bold",
      FontColor->RGBColor[1, 0.2, 0.2]], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "basisdone", ",", "factorfun", ",", "fertig", ",", "dummm1", ",", 
        "dummm2", ",", "getvars", ",", "te", ",", "time", ",", "cutoff", ",", 
        "tog", ",", "ttable", ",", "facfun", ",", "nexp", ",", "oldexpr", ",",
         "rec", ",", "special", ",", "sti", ",", "tl", ",", "tcheck", ",", 
        "tjrd", ",", "tvrd", ",", "together", ",", "vars", ",", 
        RowBox[{"$cc", "=", "0"}], ",", "$cv"}], "}"}], ",", "\n", "\t\t\t", 
      RowBox[{
       RowBox[{"time", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "]"}]}], ";", "\n", "\t\t\t", 
       RowBox[{"ttable", "=", 
        RowBox[{
         RowBox[{"Table", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"MatchQ", "[", 
           RowBox[{"ttable", ",", 
            RowBox[{"{", "__RuleDelayed", "}"}]}], "]"}]}], ",", 
         RowBox[{"ttable", "=", 
          RowBox[{"{", "}"}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"special", "=", 
        RowBox[{
         RowBox[{"Replace", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"cutoff", "=", 
        RowBox[{
         RowBox[{"TimeConstraint", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", "\n", 
       "\t\t\t", 
       RowBox[{"together", "=", 
        RowBox[{
         RowBox[{"Together", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", 
       RowBox[{"factorfun", "=", 
        RowBox[{
         RowBox[{"Factor", "/.", 
          RowBox[{"{", "opts", "}"}]}], "/.", 
         RowBox[{"Options", "[", "TarcerRecurse", "]"}]}]}], ";", 
       RowBox[{"$commentnb", "=", "False"}], ";", 
       RowBox[{"tog", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"#1", ">", "1001"}], ",", 
                RowBox[{"If", "[", 
                 RowBox[{"$Comment", ",", 
                  RowBox[{"FEPrint", "[", 
                   RowBox[{
                    RowBox[{"\"\<coefficient # \>\"", "<>", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"ValueQ", "[", "$cv", "]"}], ",", 
                    RowBox[{"ToString", "[", "$cc", "]"}], ",", "\"\<\>\""}], 
                    "]"}], "<>", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"ValueQ", "[", "$cv", "]"}], ",", 
                    RowBox[{"\"\< (\>\"", "<>", 
                    RowBox[{"ToString", "[", "$cv", "]"}], "<>", 
                    "\"\<)\>\""}], ",", "\"\<\>\""}], "]"}], "<>", 
                    "\"\< simplified, \>\""}], ",", 
                    RowBox[{"ToString", "[", "#1", "]"}]}], "]"}]}], "]"}]}], 
               "]"}], ";", "#2"}], ")"}], "&"}], ")"}], "[", 
          RowBox[{
           RowBox[{"LeafCount", "[", "#1", "]"}], ",", 
           RowBox[{
            RowBox[{"$cc", "++"}], ";", 
            RowBox[{"together", "[", "#1", "]"}]}]}], "]"}], "&"}]}], ";", 
       RowBox[{"If", "[", "\[InvisibleSpace]", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{"ValueQ", "[", "$TarcerRecursed", "]"}], ")"}]}], ",", 
         RowBox[{"$TarcerRecursed", "=", 
          RowBox[{"{", "}"}]}]}], "]"}], ";", 
       RowBox[{"$TarcerRecursed", "=", 
        RowBox[{"{", "}"}]}], ";", 
       RowBox[{"If", "[", "\[InvisibleSpace]", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"(", 
           RowBox[{"MemberQ", "[", 
            RowBox[{
            "$ContextPath", ",", "\"\<HighEnergyPhysics`fcloops`ToTFi`\>\""}],
             "]"}], ")"}]}], ",", 
         RowBox[{"t0", "=", "z"}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{
           RowBox[{"HighEnergyPhysics`fcloops`ToTFi`ToTFi", "[", "z", "]"}], "/.", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"HighEnergyPhysics`fcloops`TFi`TFi", "\[RuleDelayed]", 
              StyleBox["TFI",
               FontColor->RGBColor[1, 0.6, 0]]}], ",", 
             RowBox[{
             "HighEnergyPhysics`fcloops`ToTFi`ToTFi", "\[RuleDelayed]", 
              "Identity"}]}], "}"}]}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"ttable", "=!=", 
          RowBox[{"{", "}"}]}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{"t0", "/.", "ttable"}]}]}], "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"getvars", "=", 
        RowBox[{
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"Variables", "[", "#1", "]"}], ",", 
           RowBox[{
            RowBox[{"MatchQ", "[", 
             RowBox[{"#1", ",", 
              RowBox[{
               RowBox[{
                StyleBox["TFI",
                 FontColor->RGBColor[1, 0.6, 0]], "[", "__", "]"}], "|", 
               RowBox[{
                StyleBox["TVI",
                 FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "__", "]"}], "|", 
               RowBox[{"TJI", "[", "__", "]"}], "|", 
               RowBox[{"TBI", "[", "__", "]"}], "|", 
               RowBox[{"TAI", "[", "__", "]"}]}]}], "]"}], "&"}]}], "]"}], 
         "&"}]}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", "TFI"}], "]"}]}], ",", 
         RowBox[{"t0", "=", 
          RowBox[{"Collect", "[", 
           RowBox[{
            RowBox[{"t0", "=", 
             RowBox[{
              RowBox[{"TFIRecurse", "[", "t0", "]"}], "/.", "ttable"}]}], ",", 
            RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
        "]"}], ";", "\n", "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", 
            RowBox[{"TVI", "[", 
             RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\n", 
         "\t\t\t", 
         RowBox[{
          RowBox[{"tvrd", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{
              RowBox[{"DownValues", "[", "TVR", "]"}], ",", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#", ",", 
                  RowBox[{"TVI", "[", 
                   RowBox[{
                    RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
               "&"}]}], "]"}], "/.", 
            RowBox[{"TVR", "->", "TVI"}]}]}], ";", "\n", "\t\t\t\t", 
          RowBox[{"t0", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"t0", "=", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"t0", "/.", "tvrd"}], "/.", "tvrd"}], "/.", "tvrd"}],
                "/.", "tvrd"}]}], ",", 
             RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}], ";", 
          "\n", "\t\t\t\t", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"t0", ",", 
               RowBox[{"TVI", "[", 
                RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\t", 
            RowBox[{"t0", "=", 
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"t0", "=", 
                RowBox[{"t0", "//.", "tvrd"}]}], ",", 
               RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
           "]"}]}]}], "]"}], ";", "\n", "\t\t\t\t", "\n", "\t\t\t", 
       RowBox[{"tjrd", "=", 
        RowBox[{
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"DownValues", "[", "TJR", "]"}], ",", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"#", ",", 
               RowBox[{"TJI", "[", 
                RowBox[{
                 RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
            "&"}]}], "]"}], "/.", 
         RowBox[{"TJR", "->", "TJI"}]}]}], ";", "\n", "\t\t\t", "\n", 
       "\t\t\t", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"FreeQ", "[", 
           RowBox[{"t0", ",", 
            RowBox[{"TJI", "[", 
             RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\n", 
         "\t\t\t", 
         RowBox[{
          RowBox[{"tjrd", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{
              RowBox[{"DownValues", "[", "TJR", "]"}], ",", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"#", ",", 
                  RowBox[{"TJI", "[", 
                   RowBox[{
                    RowBox[{"_", "-", "2"}], ",", "__"}], "]"}]}], "]"}]}], 
               "&"}]}], "]"}], "/.", 
            RowBox[{"TJR", "->", "TJI"}]}]}], ";", "\n", "\t\t\t\t", 
          RowBox[{"t0", "=", 
           RowBox[{"Collect", "[", 
            RowBox[{
             RowBox[{"t0", "=", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"t0", "/.", "tjrd"}], "/.", "tjrd"}], "/.", "tjrd"}],
                "/.", "tjrd"}]}], ",", 
             RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}], ";", 
          "\n", "\t\t\t\t", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"FreeQ", "[", 
              RowBox[{"t0", ",", 
               RowBox[{"TJI", "[", 
                RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", "\t", 
            RowBox[{"t0", "=", 
             RowBox[{"Collect", "[", 
              RowBox[{
               RowBox[{"t0", "=", 
                RowBox[{"t0", "//.", "tjrd"}]}], ",", 
               RowBox[{"getvars", "[", "t0", "]"}], ",", "tog"}], "]"}]}]}], 
           "]"}]}]}], "]"}], ";", "\n", "\t\t\t", "\n", "\t\t\t", 
       RowBox[{
        RowBox[{"rec", "[", 
         RowBox[{"expr_", ",", 
          RowBox[{"{", 
           RowBox[{"ti_", ",", "tr_"}], "}"}]}], "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sti", "=", 
           RowBox[{"ToString", "[", "ti", "]"}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"FreeQ", "[", 
             RowBox[{"expr", ",", "ti"}], "]"}], ",", "expr", ",", 
            RowBox[{
             RowBox[{"nexp", "=", "expr"}], ";", 
             RowBox[{"If", "[", 
              RowBox[{"$Comment", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"#1", "===", "1"}], ",", 
                    RowBox[{"FEPrint", "[", 
                    RowBox[{
                    "sti", "<>", "\"\< level, there is 1 \>\"", "<>", "sti"}],
                     "]"}], ",", 
                    RowBox[{"FEPrint", "[", 
                    RowBox[{
                    RowBox[{"sti", "<>", "\"\< level, there are \>\""}], ",", 
                    
                    RowBox[{"ToString", "[", "#1", "]"}], ",", 
                    RowBox[{"\"\< \>\"", "<>", "sti", "<>", "\"\<'s \>\""}]}],
                     "]"}]}], "]"}], "&"}], ")"}], "[", 
                RowBox[{"Count", "[", 
                 RowBox[{"nexp", ",", 
                  RowBox[{"ti", "[", "__", "]"}], ",", 
                  RowBox[{"-", "1"}]}], "]"}], "]"}]}], "]"}], ";", "\n", 
             "\t\t\t\t", "\n", "\t\t\t\t\t\t", 
             RowBox[{"nexp", "=", 
              RowBox[{
               RowBox[{"nexp", "/.", 
                RowBox[{"ti", "\[Rule]", "tr"}]}], "/.", 
               RowBox[{"tr", "\[Rule]", "ti"}]}]}], ";", "\n", 
             "\t\t\t\t\t\t\t", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"FreeQ", "[", 
                 RowBox[{"nexp", ",", 
                  RowBox[{"TJI", "[", 
                   RowBox[{"_Plus", ",", "__"}], "]"}]}], "]"}]}], ",", 
               RowBox[{"nexp", "=", 
                RowBox[{"nexp", "//.", "tjrd"}]}]}], "]"}], ";", "\n", 
             "\t\t\t\t\t\t", "\n", "\t\t\t\t\t\t", 
             RowBox[{"tvars", "=", 
              RowBox[{"getvars", "[", "nexp", "]"}]}], ";", 
             RowBox[{"$cc", "=", "0"}], ";", 
             RowBox[{"$cv", "=", 
              RowBox[{"Length", "[", "tvars", "]"}]}], ";", 
             RowBox[{"nexp", "=", 
              RowBox[{"Collect", "[", 
               RowBox[{"nexp", ",", "tvars", ",", "tog"}], "]"}]}], ";", "\n",
              "\t\t\t\t\t\t", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"cutoff", "=!=", "Infinity"}], ",", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", "]"}], "-", "time"}], ">", 
                  "cutoff"}], ",", 
                 RowBox[{"Throw", "[", "$Failed", "]"}]}], "]"}]}], "]"}], 
             ";", "\n", "\t\t\t\t\t\t", "nexp"}]}], "]"}]}], ")"}]}], ";", 
       RowBox[{"basisdone", "=", "0"}], ";", 
       RowBox[{"tl", "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            StyleBox["TFI",
             FontColor->RGBColor[1, 0.6, 0]], ",", 
            StyleBox["TFR",
             FontColor->RGBColor[0, 0.6, 0]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            StyleBox["TVI",
             FontColor->RGBColor[0.2, 0.4, 0.8]], ",", 
            StyleBox["TVR",
             FontColor->RGBColor[0.2, 0.8, 1]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TJI", ",", "TJR"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TBI", ",", "TBR"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"TAI", ",", "TAR"}], "}"}]}], "}"}]}], ";", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"t0", "=", " ", 
           RowBox[{"FixedPoint", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"rec", "[", 
               RowBox[{"#", ",", 
                RowBox[{"tl", "[", 
                 RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "&"}], ",", "t0", 
             ",", "100"}], "]"}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Head", "[", "t0", "]"}], "===", "Plus"}], ",", 
            RowBox[{
             RowBox[{"fertig", "=", 
              RowBox[{"Select", "[", 
               RowBox[{"t0", ",", "\[InvisibleSpace]", 
                RowBox[{"!", 
                 RowBox[{"(", 
                  RowBox[{"FreeQ", "[", 
                   RowBox[{"#1", ",", 
                    RowBox[{"tl", "\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}]}], 
                   "]"}], ")"}]}]}], "]"}]}], ";", 
             RowBox[{"basisdone", "=", 
              RowBox[{"basisdone", "+", "fertig"}]}], ";", 
             RowBox[{"t0", "=", 
              RowBox[{"t0", "-", "fertig"}]}]}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Length", "[", "tl", "]"}]}], "}"}]}], "]"}], ";", 
       RowBox[{"t0", "=", 
        RowBox[{"t0", "+", "basisdone"}]}], ";", 
       RowBox[{"$cv", "=", "."}], ";", 
       RowBox[{"t0", "=", 
        RowBox[{"Collect", "[", 
         RowBox[{"t0", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"TAI", "[", "__", "]"}], ",", 
            RowBox[{
             RowBox[{"TAI", "[", "a1__", "]"}], " ", 
             RowBox[{"TAI", "[", "a2__", "]"}]}], ",", 
            RowBox[{"TAI", "[", "__", "]"}], ",", 
            RowBox[{"TBI", "[", "__", "]"}], ",", 
            RowBox[{"TJI", "[", "__", "]"}], ",", 
            RowBox[{"TVI", "[", "__", "]"}], ",", 
            RowBox[{"TFI", "[", "__", "]"}]}], "}"}], ",", "factorfun"}], 
         "]"}]}], ";", 
       RowBox[{"If", "[", 
        RowBox[{"$Comment", ",", 
         RowBox[{"FEPrint", "[", 
          RowBox[{"\"\<the result has \>\"", ",", 
           RowBox[{"ToString", "[", 
            RowBox[{
             RowBox[{"Length", "[", 
              RowBox[{"t0", "+", "dummm1", "+", "dummm2"}], "]"}], "-", "2"}],
             "]"}], ",", "\"\< terms\>\""}], "]"}]}], "]"}], ";", 
       RowBox[{"If", "[", 
        RowBox[{"$Notebooks", ",", 
         RowBox[{"SetSelectedNotebook", "[", 
          RowBox[{"EvaluationNotebook", "[", "]"}], "]"}]}], "]"}], ";", 
       "t0"}]}], "]"}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerExpand", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.8.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "TarcerExpand", "]"}], "=", 
   RowBox[{"{", "\n", "\t\t", 
    RowBox[{
     RowBox[{"TarcerReduce", ":>", "$BasisIntegrals"}], ",", "\n", "\t", 
     RowBox[{"TarcerRecurse", "->", "False"}], ",", 
     RowBox[{"Zeta", "->", "True"}], ",", 
     RowBox[{"Replace", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Prefactor1", "\[Rule]", "Identity"}], ",", 
        RowBox[{"EulerGamma", " ", "\[Rule]", "0"}]}], "}"}]}]}], "}"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"TarcerExpand", "[", 
    RowBox[{"expr_", ",", 
     RowBox[{"d_Symbol", "->", 
      RowBox[{"fe_", "/;", 
       RowBox[{
        RowBox[{"Length", "[", 
         RowBox[{"Variables", "[", "fe", "]"}], "]"}], "===", "1"}]}]}], ",", 
     "ru___Rule"}], "]"}], ":=", 
   RowBox[{"TarcerExpand", "[", 
    RowBox[{"expr", ",", 
     RowBox[{"d", " ", "->", "fe"}], ",", "0", ",", "ru"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TarcerExpand", "[", 
   RowBox[{"expr_", ",", 
    RowBox[{"d_Symbol", "\[Rule]", "fe_"}], ",", 
    RowBox[{"order_Integer", "?", "NonNegative"}], ",", "opts___Rule"}], 
   "]"}], ":=", 
  RowBox[{
   RowBox[{
    StyleBox["Block",
     FontSize->16,
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0.2, 0.2]], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "numfa", ",", "z2rule", ",",
        "restfa", ",", "ta0", ",", "tarcNumericalFactor", ",", "tarcFactor1", 
       ",", "eeps"}], 
      RowBox[{"(*", 
       RowBox[{",", "extracoll"}], "*)"}], "}"}], ",", 
     RowBox[{
      RowBox[{"eeps", "=", 
       RowBox[{
        RowBox[{"Variables", "[", "fe", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"tarcNumericalFactor", "[", "x_", "]"}], ":=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"NumberQ", "[", "x", "]"}], ",", "x", ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "x", "]"}], "===", "Times"}], ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"NumberQ", "[", 
              RowBox[{"First", "[", "x", "]"}], "]"}], ",", 
             RowBox[{"First", "[", "x", "]"}], ",", "1"}], "]"}], ",", "1"}], 
          "]"}]}], "]"}]}], ";", "\n", 
      RowBox[{
       RowBox[{"tarcFactor1", "[", "x_", "]"}], ":=", 
       RowBox[{"Block", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "factor1t1", ",", "factor1t2", ",", "factor1t3", ",", "mt", ",", 
           "mi", ",", "m1", ",", "mp1", ",", 
           RowBox[{"nx", "=", "x"}], ",", "iIii"}], "}"}], ",", 
         RowBox[{
          RowBox[{"mt", "=", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                 RowBox[{"Plus", "\[Rule]", "mi"}]}], "/.", 
                "\[InvisibleSpace]", 
                RowBox[{"mi", "\[Rule]", "Plus"}]}], "/.", 
               "\[InvisibleSpace]", 
               RowBox[{"m1", "\[Rule]", 
                RowBox[{"-", "1"}]}]}], "/.", "\[InvisibleSpace]", 
              RowBox[{"mp1", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"+", "##1"}], ")"}]}], "&"}], ")"}]}]}], "/.", 
             "\[InvisibleSpace]", 
             RowBox[{"iIii", "\[Rule]", "\[ImaginaryI]"}]}], "&"}]}], ";", 
          RowBox[{
           RowBox[{"mi", "[", 
            RowBox[{"y_", ",", "z__"}], "]"}], ":=", 
           RowBox[{
            RowBox[{"m1", " ", 
             RowBox[{"mp1", "[", 
              RowBox[{"y", ",", "z"}], "]"}]}], "/;", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Head", "[", "#1", "]"}], "===", "Complex"}], ",", 
                 "False", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"#1", "<", "0"}], ",", "True", ",", "False"}], 
                  "]"}]}], "]"}], "&"}], ")"}], "[", 
             RowBox[{"tarcNumericalFactor", "[", "y", "]"}], "]"}]}]}], ";", 
          RowBox[{"nx", "=", 
           RowBox[{"x", "/.", "\[InvisibleSpace]", 
            RowBox[{
             RowBox[{"Complex", "[", 
              RowBox[{"0", ",", "b_"}], "]"}], "\[Rule]", 
             RowBox[{"b", " ", "iIii"}]}]}]}], ";", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Head", "[", "nx", "]"}], "=!=", "Plus"}], ",", 
            RowBox[{"mt", "[", 
             RowBox[{"nx", "/.", "\[InvisibleSpace]", 
              RowBox[{"Plus", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Factor1", "[", 
                  RowBox[{"+", "##1"}], "]"}], "&"}], ")"}]}]}], "]"}], ",", 
            RowBox[{
             RowBox[{"factor1t1", "=", 
              RowBox[{"List", "@@", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                   RowBox[{"Plus", "\[Rule]", "factor1t3"}]}], "&"}], ")"}], "/@",
                 "nx"}]}]}], ";", 
             RowBox[{"factor1t2", "=", 
              RowBox[{
               RowBox[{"PolynomialGCD", "@@", "factor1t1"}], "/.", 
               "\[InvisibleSpace]", 
               RowBox[{"factor1t3", "\[Rule]", "Plus"}]}]}], ";", 
             RowBox[{"mt", "[", 
              RowBox[{"factor1t2", " ", 
               RowBox[{"Plus", "@@", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   FractionBox[
                    RowBox[{"#1", "/.", "\[InvisibleSpace]", 
                    RowBox[{"factor1t3", "\[Rule]", "Plus"}]}], "factor1t2"], 
                   "&"}], ")"}], "/@", "factor1t1"}]}]}], "]"}]}]}], 
           "]"}]}]}], "]"}]}], ";", 
      RowBox[{"Catch", "[", 
       RowBox[{
        RowBox[{"e", "=", "eeps"}], ";", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"ints", ",", "recurse", ",", "zeta2replace"}], "}"}], "=", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"TarcerReduce", ",", "TarcerRecurse", ",", "Zeta"}], "}"}],
           "/.", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", 
            RowBox[{"opts", ",", 
             RowBox[{"Options", "[", "TarcerExpand", "]"}]}], "}"}], 
           "]"}]}]}], ";", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"zeta2replace", "=!=", "True"}], "&&", 
           RowBox[{"MemberQ", "[", 
            RowBox[{
            "$ContextPath", ",", "\"\<HighEnergyPhysics`general`Zeta2`\>\""}],
             "]"}]}], ",", 
          RowBox[{"z2rule", "=", 
           RowBox[{"{", "}"}]}], ",", 
          RowBox[{"z2rule", "=", 
           RowBox[{
            SuperscriptBox["\[Pi]", "2"], "\[RuleDelayed]", 
            RowBox[{"6", " ", 
             RowBox[{
             "ToExpression", "[", 
              "\"\<HighEnergyPhysics`general`Zeta2`Zeta2\>\"", "]"}]}]}]}]}], 
         "]"}], ";", 
        RowBox[{"ints", "=", 
         RowBox[{"Flatten", "[", "ints", "]"}]}], ";", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"recurse", "=!=", "True"}], ",", 
          RowBox[{"ta0", "=", "expr"}], ",", 
          RowBox[{"ta0", "=", 
           RowBox[{"TarcerRecurse", "[", "expr", "]"}]}]}], "]"}], ";", 
        RowBox[{"t1", "=", 
         RowBox[{
          RowBox[{"ta0", "/.", "ints"}], "/.", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"SEpsilon", "[", "de_", "]"}], "\[RuleDelayed]", 
             RowBox[{"SEpsilon", "[", 
              RowBox[{"ToString", "[", "d", "]"}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"Prefactor1", "[", "de_", "]"}], "\[RuleDelayed]", 
             RowBox[{"Prefactor1", "[", " ", 
              RowBox[{"ToString", "[", 
               RowBox[{"de", ",", "InputForm"}], "]"}], "]"}]}]}], "}"}]}]}], 
        ";", 
        RowBox[{"vars", "=", 
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"Variables", "[", "t1", "]"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "#1", "]"}], ">", "0"}], "&"}]}], "]"}]}],
         ";", 
        RowBox[{"If", "[", "\[InvisibleSpace]", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"(", 
            RowBox[{"FreeQ", "[", 
             RowBox[{"vars", ",", 
              RowBox[{
               StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]], "|", 
               StyleBox["TVI",
                FontColor->RGBColor[0.2, 0.4, 0.8]], "|", "TJI", "|", "TKI", 
               "|", "TBI", "|", "TAI"}]}], "]"}], ")"}]}], ",", 
          RowBox[{
           RowBox[{
           "Print", "[", "\"\<Returning the list of remaining integrals.\>\"",
             "]"}], ";", 
           RowBox[{"Throw", "[", 
            RowBox[{"Select", "[", 
             RowBox[{"vars", ",", "\[InvisibleSpace]", 
              RowBox[{
               RowBox[{"!", 
                RowBox[{"(", 
                 RowBox[{"FreeQ", "[", 
                  RowBox[{"#1", ",", 
                   RowBox[{
                    StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "|", 
                    StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "|", "TJI", "|", 
                    "TKI", "|", "TBI", "|", "TAI"}]}], "]"}], ")"}]}], 
               "&"}]}], "]"}], "]"}]}]}], "]"}], ";", 
        RowBox[{"t1", "=", 
         RowBox[{"t1", "/.", 
          RowBox[{"d", "\[Rule]", "fe"}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{
          RowBox[{"-", "2"}], " ", 
          RowBox[{"Zeta", "[", "3", "]"}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"2", ",", "2"}], "]"}], "=", 
         RowBox[{"2", "-", 
          RowBox[{"2", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}]}], ";", 
        RowBox[{
         RowBox[{"polygafu", "[", 
          RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{"polygafu", "[", 
           RowBox[{"x", ",", "y"}], "]"}], "=", 
          RowBox[{"FunctionExpand", "[", 
           RowBox[{"PolyGamma", "[", 
            RowBox[{"x", ",", "y"}], "]"}], "]"}]}]}], ";", 
        RowBox[{"t2", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"Normal", "[", 
            RowBox[{"t1", "+", 
             SuperscriptBox[
              RowBox[{"O", "[", "e", "]"}], 
              RowBox[{"order", "+", "1"}]]}], "]"}], "/.", 
           RowBox[{"PolyGamma", "\[Rule]", "polygafu"}]}], "/.", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Replace", "/.", 
             RowBox[{"{", "opts", "}"}]}], "/.", 
            RowBox[{"Options", "[", "TarcerExpand", "]"}]}], ")"}]}]}], ";", 
        RowBox[{"t3", "=", 
         RowBox[{"Collect", "[", 
          RowBox[{"t2", ",", "e", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Expand", "[", "#1", "]"}], "/.", "z2rule"}], ")"}], 
            "&"}]}], "]"}]}], ";", 
        RowBox[{"t3", "=", 
         RowBox[{"t2", "/.", 
          RowBox[{
           RowBox[{"ToString", "[", "d", "]"}], "\[RuleDelayed]", 
           RowBox[{"(", "fe", ")"}]}]}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Global`T3", "=", "t3"}], ";"}], "*)"}], 
        RowBox[{"t4", "=", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"tarcFactor1", "[", 
               RowBox[{"Expand", "[", "t3", "]"}], "]"}], "//.", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"pre", ":", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"Prefactor1", "|", "SEpsilon"}], ")"}], "[", "a_",
                    "]"}], " ", 
                  RowBox[{"Prefactor1", "[", "b_", "]"}]}]}], ")"}], 
               "\[RuleDelayed]", 
               RowBox[{"Prefactor1", "[", 
                RowBox[{"a", " ", "b"}], "]"}]}]}], "/.", 
             RowBox[{
              RowBox[{"Prefactor1", "[", "1", "]"}], "\[RuleDelayed]", 
              "1"}]}], "/.", 
            RowBox[{
             RowBox[{"Prefactor1", "[", "xy_", "]"}], "\[RuleDelayed]", 
             RowBox[{"Prefactor1", "[", 
              RowBox[{"xy", "/.", 
               RowBox[{"st_String", "\[RuleDelayed]", 
                RowBox[{"ToExpression", "[", "st", "]"}]}]}], "]"}]}]}], "/.", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Prefactor1", "[", 
               RowBox[{
                RowBox[{"m_", "^", "e"}], " ", "xx_"}], "]"}], 
              RowBox[{"m_", "^", "any_"}]}], " ", ")"}], "\[RuleDelayed]", 
            RowBox[{"Prefactor1", "[", 
             RowBox[{
              RowBox[{"m", "^", 
               RowBox[{"(", 
                RowBox[{"e", "+", "any"}], ")"}]}], "xx"}], "]"}]}]}], "/.", 
          "z2rule"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Global`t4", "=", "t4"}], ";"}], "*)"}], 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Head", "[", "t4", "]"}], "===", "Times"}], ",", 
             RowBox[{
              RowBox[{"numfa", "=", 
               RowBox[{"tarcNumericalFactor", "[", "t4", "]"}]}], ";", 
              RowBox[{"restfa", " ", "=", " ", 
               RowBox[{"Select", "[", 
                RowBox[{"t4", ",", 
                 RowBox[{
                  RowBox[{"FreeQ", "[", 
                   RowBox[{"#", ",", 
                    RowBox[{"eeps", "|", "Log", "|", "Zeta"}]}], "]"}], 
                  "&"}]}], "]"}]}], ";", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"Global`RR", "=", "restfa"}], ";", 
                RowBox[{"Global`NN", "=", "numfa"}], ";"}], "*)"}], 
              RowBox[{
               RowBox[{"extracoll", "[", 
                RowBox[{"z_", ",", "e"}], "]"}], ":=", 
               RowBox[{"Block", "[", 
                RowBox[{
                 RowBox[{"{", "}"}], ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Head", "[", "z", "]"}], "===", "Plus"}], ",", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"extracoll", "[", 
                    RowBox[{"#", ",", "e"}], "]"}], "&"}], "/@", "z"}], ",", 
                   RowBox[{"z", "/.", 
                    RowBox[{"xy_Plus", "\[RuleDelayed]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Length", "[", 
                    RowBox[{"Variables", "[", "xy", "]"}], "]"}], "===", 
                    "0"}], ",", "xy", ",", 
                    RowBox[{"Collect", "[", 
                    RowBox[{"xy", ",", 
                    RowBox[{"Complement", "[", 
                    RowBox[{
                    RowBox[{"Variables", "[", "xy", "]"}], ",", 
                    RowBox[{"{", "e", "}"}]}], "]"}]}], "]"}]}], "]"}]}]}]}], 
                  "]"}]}], " ", "]"}]}], ";", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"Select", "[", 
                  RowBox[{"t4", ",", 
                   RowBox[{
                    RowBox[{"!", 
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "Prefactor1"}], "]"}]}], "&"}]}], "]"}], 
                 RowBox[{"(", 
                  FractionBox["restfa", "numfa"], ")"}]}], ")"}], ".", 
               RowBox[{"extracoll", "[", " ", 
                RowBox[{
                 RowBox[{"Collect", "[", 
                  RowBox[{
                   RowBox[{"Expand", "[", 
                    RowBox[{
                    FractionBox["1", 
                    RowBox[{"restfa", " "}]], 
                    RowBox[{"numfa", "  ", 
                    RowBox[{"Select", "[", " ", 
                    RowBox[{"t4", ",", 
                    RowBox[{
                    RowBox[{"FreeQ", "[", 
                    RowBox[{"#", ",", "Prefactor1"}], "]"}], "&"}]}], 
                    "]"}]}]}], "]"}], ",", "e"}], "]"}], ",", "e"}], 
                "]"}]}]}], ",", "t4"}], "]"}], "/.", 
           RowBox[{"s_String", "\[RuleDelayed]", 
            RowBox[{"ToExpression", "[", "s", "]"}]}]}], "/.", 
          RowBox[{"d", "\[Rule]", "fe"}]}], "/.", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Replace", "/.", 
            RowBox[{"{", "opts", "}"}]}], "/.", 
           RowBox[{"Options", "[", "TarcerExpand", "]"}]}], ")"}]}]}], 
       "]"}]}]}], "]"}], "/;", 
   RowBox[{
    RowBox[{"Length", "[", 
     RowBox[{"Variables", "[", "fe", "]"}], "]"}], "===", "1"}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Construct T's", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.9.1"],

Cell[CellGroupData[{

Cell["remember", "Subsection",
 CellTags->"T1.9.1.1"],

Cell["The function remember is not used by default.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tfch", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"TFR", ":>", 
      RowBox[{"Hold", "[", "TFR", "]"}]}], ",", 
     RowBox[{"TVR", ":>", 
      RowBox[{"Hold", "[", "TVR", "]"}]}], ",", 
     RowBox[{"TJR", ":>", 
      RowBox[{"Hold", "[", "TJR", "]"}]}], ",", 
     RowBox[{"TBR", ":>", 
      RowBox[{"Hold", "[", "TBR", "]"}]}], ",", 
     RowBox[{"TAR", ":>", 
      RowBox[{"Hold", "[", "TAR", "]"}]}], ",", 
     RowBox[{"TComment", ":>", 
      RowBox[{"Hold", "[", "TComment", "]"}]}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rtfch", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Reverse", ",", "tfch"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"condrest", ",", "HoldRest"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "remember", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"remember", "[", "zi_HoldForm", "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z", "=", 
       RowBox[{"zi", "/.", "tfch"}]}], "}"}], ",", 
     RowBox[{"If", "[", "\[InvisibleSpace]", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"FreeQ", "[", 
          RowBox[{"z", ",", "Condition"}], "]"}], ")"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "[", 
            RowBox[{
             RowBox[{"Hold", "[", "SetDelayed", "]"}], "[", 
             RowBox[{
              RowBox[{"z", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"ReplacePart", "[", 
               RowBox[{
                RowBox[{"z", "\[LeftDoubleBracket]", 
                 RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], ",", 
                RowBox[{"Prepend", "[", 
                 RowBox[{
                  RowBox[{"Extract", "[", 
                   RowBox[{"z", ",", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", "Hold"}], 
                   "]"}], ",", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"z", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "/.", 
                    
                    RowBox[{"PatternTest", "\[Rule]", "pt"}]}], "/.", 
                    RowBox[{
                    RowBox[{"pt", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}], "/.", 
                    RowBox[{"Pattern", "\[Rule]", "pat"}]}], "/.", 
                   RowBox[{
                    RowBox[{"pat", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}]}], "]"}], ",", "1"}], "]"}]}], "]"}], "]"}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "\[Rule]", "HoldForm"}]}],
           "/.", "rtfch"}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "SetDelayed", "]"}], "\[Rule]", 
          "SetDelayed"}]}], "/.", 
        RowBox[{"Hold", "\[Rule]", "Set"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "[", 
            RowBox[{
             RowBox[{"Hold", "[", "SetDelayed", "]"}], "[", 
             RowBox[{
              RowBox[{"z", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"Prepend", "[", 
               RowBox[{
                RowBox[{"Extract", "[", 
                 RowBox[{"z", ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}], ",", 
                
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"z", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "/.", 
                    
                    RowBox[{"PatternTest", "\[Rule]", "pt"}]}], "/.", 
                   RowBox[{
                    RowBox[{"pt", "[", 
                    RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                    "a"}]}], "/.", 
                  RowBox[{"Pattern", "\[Rule]", "pat"}]}], "/.", 
                 RowBox[{
                  RowBox[{"pat", "[", 
                   RowBox[{"a_", ",", "b_"}], "]"}], "\[RuleDelayed]", 
                  "a"}]}]}], "]"}]}], "]"}], "]"}], "/.", 
           RowBox[{
            RowBox[{"Hold", "[", "HoldForm", "]"}], "\[Rule]", "HoldForm"}]}],
           "/.", "rtfch"}], "/.", 
         RowBox[{
          RowBox[{"Hold", "[", "SetDelayed", "]"}], "\[Rule]", 
          "SetDelayed"}]}], "/.", 
        RowBox[{"Hold", "\[Rule]", "Set"}]}]}], "]"}]}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData["$TarasovTdeltaplimit"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aa2", " ", "=", " ", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", "#"], "&"}], ",", " ", 
     RowBox[{"10000", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"$TarasovTdeltaplimit", "+", "1"}], ")"}], " ", "1000"}]}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bb12", "=", 
   RowBox[{"Select", "[", 
    RowBox[{"aa2", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "#", "]"}], "=!=", "Subscript"}], "&"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "aa2", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Print", "[", 
  RowBox[{"\"\<there are \>\"", ",", 
   RowBox[{"Length", "[", "bb12", "]"}], ",", 
   "\"\< recursion equations for $RankLimit = \>\"", ",", "$RankLimit"}], 
  "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"there are \"\>", "\[InvisibleSpace]", "201", 
   "\[InvisibleSpace]", "\<\" recursion equations for $RankLimit = \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5"}], "}"}]}],
  SequenceForm[
  "there are ", 201, " recursion equations for $RankLimit = ", {2, 5}],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"Global`$Remember", " ", "===", " ", "True"}], ",", 
    RowBox[{
     RowBox[{"Global`RCHECK", "=", "True"}], ";", " ", 
     RowBox[{"nb", " ", "=", 
      RowBox[{"Map", "[", 
       RowBox[{"remember", ",", "bb12"}], "]"}]}]}], ",", 
    RowBox[{"nb", "=", "bb12"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "bb12", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Unset", "[", "$Post", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ReleaseHold", "[", "nb", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Comment", "=", "False"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"!", 
     RowBox[{
     "ValueQ", "[", "HighEnergyPhysics`FeynCalc`$FeynCalcDirectory", "]"}]}], 
    ",", 
    RowBox[{
    "BeginPackage", "[", "\"\<HighEnergyPhysics`general`Zeta2`\>\"", "]"}]}], 
   "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Zeta2", "::", "usage"}], "=", "\n", 
   "\"\<Zeta2 denotes Zeta[2].\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeta2", " ", "/:", " ", 
   RowBox[{"N", "[", "Zeta2", "]"}], " ", "=", " ", 
   RowBox[{"N", "[", 
    RowBox[{"Zeta", "[", "2", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Zeta2", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"Zeta2", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "@@", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"RowBox", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<\[Zeta]\>\"", ",", "\"\<(\>\"", ",", "2", ",", "\"\<)\>\""}], 
        "}"}], "]"}], ",", "Zeta2", ",", 
      RowBox[{"Editable", "->", "False"}]}], "}"}]}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"EndPackage", "[", "]"}], ";"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$BasisIntegrals", "Section",
 CellDingbat->"\[EmptySmallCircle]",
 CellTags->"T1.10.1"],

Cell["\<\
This is a (incomplete) set of basis integrals. You can add your favorite \
integrals here.\
\>", "Text"],

Cell["eta=Gamma[d/2-1]/Gamma[d-3]Gamma[3-d/2];", "Text"],

Cell[CellGroupData[{

Cell["\<\
$BasisIntegrals =
  {TAI[d_, 0, {{1, M_}}] -> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     Prefactor1[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] -> 
   (I*E^(((4 - d)*EulerGamma)/2)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      Prefactor1[ (-pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] ->-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  Prefactor1[ 
        (-pp)^(-4 + d)*SEpsilon[d]^2])/
    (Gamma[-3 + (3*d)/2] )
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] ->(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]->M^2*Prefactor1[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]->(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] ->(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["A",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      FormBox[
       RowBox[{"{", 
        RowBox[{"1", ",", "M_"}], "}"}],
       TraditionalForm], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TAI[
      Pattern[$CellContext`d, 
       Blank[]], 0, {{1, 
        Pattern[$CellContext`M, 
         Blank[]]}}],
     Editable->True], "\[Rule]", 
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", 
     SuperscriptBox["M", "2"], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       FractionBox[
        RowBox[{"4", "-", "d"}], "2"]}], "]"}], " ", 
     RowBox[{"(", 
      InterpretationBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["M", "2"], ")"}], 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "4"}], "+", "d"}], ")"}]}]], " ", 
        InterpretationBox[
         StyleBox[
          SubscriptBox["S", "Global`d"],
          FontWeight->"Bold"],
         HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
         Editable->False]}],
       HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(
          Rational[1, 2] (-4 + $CellContext`d)) 
        HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]],
       Editable->False], ")"}]}]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["B",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TBI[
      Pattern[$CellContext`d, 
       Blank[]], 
      Pattern[$CellContext`pp, 
       Blank[]], {{1, 0}, {1, 0}}],
     Editable->True], "\[Rule]", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"2", "-", 
        FractionBox["d", "2"]}], "]"}], " ", 
      SuperscriptBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         FractionBox["d", "2"]}], "]"}], "2"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "pp"}], ")"}], 
          RowBox[{
           FractionBox["1", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "4"}], "+", "d"}], ")"}]}]], " ", 
         InterpretationBox[
          StyleBox[
           SubscriptBox["S", "Global`d"],
           FontWeight->"Bold"],
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
          Editable->False]}],
        HighEnergyPhysics`Tarcer`Prefactor1[(-$CellContext`pp)^(
           Rational[1, 2] (-4 + $CellContext`d)) 
         HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]],
        Editable->False], ")"}]}], 
     RowBox[{"Gamma", "[", 
      RowBox[{
       RowBox[{"-", "2"}], "+", "d"}], "]"}]]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TJI[
      Pattern[$CellContext`d, 
       Blank[]], 
      Pattern[$CellContext`pp, 
       Blank[]], {{1, 0}, {1, 0}, {1, 0}}],
     Editable->True], "\[Rule]", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", "pp", 
       " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{"3", "-", "d"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          FractionBox["d", "2"]}], "]"}], "3"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "pp"}], ")"}], 
           RowBox[{
            RowBox[{"-", "4"}], "+", "d"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "Global`d", "2"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[(-$CellContext`pp)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
         Editable->False], ")"}]}], 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"-", "3"}], "+", 
        FractionBox[
         RowBox[{"3", " ", "d"}], "2"]}], "]"}]]}]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TJI[
      Pattern[$CellContext`d, 
       Blank[]], Pattern[$CellContext`M, 
        Blank[]]^2, {{1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 0}, {1, 0}}],
     Editable->True], "\[Rule]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", 
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{"3", "-", "d"}], "]"}], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{"Gamma", "[", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          FractionBox["d", "2"]}], "]"}], "2"], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"-", "5"}], "+", 
         RowBox[{"2", " ", "d"}]}], "]"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{
            RowBox[{"-", "4"}], "+", "d"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "Global`d", "2"],
            FontWeight->"Bold"],
           HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
           Editable->False]}],
         HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
         Editable->False], ")"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"-", "2"}], "+", "d"}], "]"}], " ", 
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"-", "3"}], "+", 
         FractionBox[
          RowBox[{"3", " ", "d"}], "2"]}], "]"}]}], ")"}]}]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["J",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_Symbol",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TJI[
      Pattern[$CellContext`d, 
       Blank[Symbol]], Pattern[$CellContext`M, 
        Blank[]]^2, {{1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 
        Pattern[$CellContext`M, 
         Blank[]]}}],
     Editable->True], "\[Rule]", 
    RowBox[{
     SuperscriptBox["M", "2"], " ", 
     RowBox[{"(", 
      InterpretationBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["M", "2"], ")"}], 
         RowBox[{
          RowBox[{"-", "4"}], "+", "d"}]], " ", 
        InterpretationBox[
         StyleBox[
          SubsuperscriptBox["S", "Global`d", "2"],
          FontWeight->"Bold"],
         HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
         Editable->False]}],
       HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
       Editable->False], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["6", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"]]}], "+", 
       FractionBox["17", 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}], ")"}]}]], "+", 
       RowBox[{
        FractionBox["1", "8"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "59"}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}], ")"}], "3"], " ", "TJI111e"}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "4"}], "+", "d"}], ")"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{"16755", "-", 
           RowBox[{"4750", " ", 
            SuperscriptBox["\[Pi]", "2"]}], "-", 
           RowBox[{"14", " ", 
            SuperscriptBox["\[Pi]", "4"]}], "+", 
           RowBox[{"3840", " ", 
            SuperscriptBox["\[Pi]", "2"], " ", 
            RowBox[{"Log", "[", "2", "]"}]}], "-", 
           RowBox[{"12080", " ", 
            RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}], "1920"], "+", 
       RowBox[{
        FractionBox["1", "96"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "4"}], "+", "d"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"195", "+", 
          RowBox[{"98", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", 
          RowBox[{"48", " ", 
           RowBox[{"Zeta", "[", "3", "]"}]}]}], ")"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["K",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TJI[
      Pattern[$CellContext`d, 
       Blank[]], 0, {{1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 0}}],
     Editable->True], "\[Rule]", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", 
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"3", "-", "d"}], "]"}], " ", 
      SuperscriptBox[
       RowBox[{"Gamma", "[", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], "]"}], "2"], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        FractionBox["d", "2"]}], "]"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}]], " ", 
         InterpretationBox[
          StyleBox[
           SubsuperscriptBox["S", "Global`d", "2"],
           FontWeight->"Bold"],
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
          Editable->False]}],
        HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
        Editable->False], ")"}]}], 
     RowBox[{
      RowBox[{"Gamma", "[", 
       RowBox[{"4", "-", "d"}], "]"}], " ", 
      RowBox[{"Gamma", "[", 
       FractionBox["d", "2"], "]"}]}]]}], ",", 
   RowBox[{
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["K",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["d_",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TJI[
      Pattern[$CellContext`d, 
       Blank[]], 0, {{1, 
        Pattern[$CellContext`M, 
         Blank[]]}, {1, 0}, {1, 0}}],
     Editable->True], "\[Rule]", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}], " ", "EulerGamma"}]], " ", 
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"3", "-", "d"}], "]"}], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"2", "-", 
        FractionBox["d", "2"]}], "]"}], " ", 
      SuperscriptBox[
       RowBox[{"Gamma", "[", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         FractionBox["d", "2"]}], "]"}], "2"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "d"}]], " ", 
         InterpretationBox[
          StyleBox[
           SubsuperscriptBox["S", "Global`d", "2"],
           FontWeight->"Bold"],
          HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d],
          Editable->False]}],
        HighEnergyPhysics`Tarcer`Prefactor1[($CellContext`M^2)^(-4 + \
$CellContext`d) HighEnergyPhysics`Tarcer`SEpsilon[$CellContext`d]^2],
        Editable->False], ")"}]}], 
     RowBox[{"Gamma", "[", 
      FractionBox["d", "2"], "]"}]]}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
$BasisIntegrals = 
  {TAI[d_, 0, {{1, M_}}] :> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     Prefactor1[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] :> 
   (I*E^(((4 - d)*EulerGamma)/2 - 
         I/2*(-4 + d)*Pi)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      Prefactor1[(pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] :>-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  Prefactor1[
        (pp)^(-4 + d)*SEpsilon[d]^2])/
    (E^(I*(-4 + d)*Pi)*Gamma[-3 + (3*d)/2])
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]:>M^2*Prefactor1[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]:>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["A",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "M_"}], "}"}],
        TraditionalForm], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TAI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"4", "-", "d"}], ")"}], " ", 
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]]}]], " ", 
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{
        FractionBox[
         RowBox[{"4", "-", "d"}], "2"], "-", "1"}], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          FractionBox[
           RowBox[{"d", "-", "4"}], "2"]], " ", 
         InterpretationBox[
          StyleBox[
           SubscriptBox["S", "d"],
           FontWeight->"Bold"],
          SEpsilon[d],
          Editable->False]}],
        Prefactor1[(M^2)^(Rational[1, 2] (-4 + d)) SEpsilon[d]],
        Editable->False], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TBI[
       Pattern[d, 
        Blank[]], 
       Pattern[pp, 
        Blank[]], {{1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"4", "-", "d"}], ")"}], " ", 
          TagBox["\[DoubledGamma]",
           Function[{}, EulerGamma]]}], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], " ", "\[Pi]"}]}]], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox["pp", 
           FractionBox[
            RowBox[{"d", "-", "4"}], "2"]], " ", 
          InterpretationBox[
           StyleBox[
            SubscriptBox["S", "d"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[pp^(Rational[1, 2] (-4 + d)) SEpsilon[d]],
         Editable->False], ")"}]}], 
      RowBox[{"\[CapitalGamma]", "(", 
       RowBox[{"d", "-", "2"}], ")"}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 
       Pattern[pp, 
        Blank[]], {{1, 0}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"pp", " ", 
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{"3", "-", "d"}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
           FractionBox["d", "2"], "-", "1"}], ")"}], "3"], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"4", "-", "d"}], ")"}], " ", 
          TagBox["\[DoubledGamma]",
           Function[{}, EulerGamma]]}]], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{
           SuperscriptBox["pp", 
            RowBox[{"d", "-", "4"}]], " ", 
           InterpretationBox[
            StyleBox[
             SubsuperscriptBox["S", "d", "2"],
             FontWeight->"Bold"],
            SEpsilon[d],
            Editable->False]}],
          Prefactor1[pp^(-4 + d) SEpsilon[d]^2],
          Editable->False], ")"}]}], 
       RowBox[{
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], " ", "\[Pi]"}]], " ", 
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox[
           RowBox[{"3", " ", "d"}], "2"], "-", "3"}], ")"}]}]]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], Pattern[M, 
         Blank[]]^2, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         RowBox[{"2", " ", "d"}], "-", "5"}], ")"}], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"d", "-", "2"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         FractionBox[
          RowBox[{"3", " ", "d"}], "2"], "-", "3"}], ")"}]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_Symbol",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[Symbol]], Pattern[M, 
         Blank[]]^2, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}}],
      Editable->True], "\[RuleDelayed]", 
     RowBox[{
      SuperscriptBox["M", "2"], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SuperscriptBox["M", "2"], ")"}], 
          RowBox[{"d", "-", "4"}]], " ", 
         InterpretationBox[
          StyleBox[
           SubsuperscriptBox["S", "d", "2"],
           FontWeight->"Bold"],
          SEpsilon[d],
          Editable->False]}],
        Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"TJI111e", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "3"]}], "+", 
        RowBox[{
         FractionBox["1", "8"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "59"}], "-", 
           RowBox[{"2", " ", 
            SuperscriptBox["\[Pi]", "2"]}]}], ")"}]}], "+", 
        RowBox[{
         FractionBox["1", "96"], " ", 
         RowBox[{"(", 
          RowBox[{"d", "-", "4"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "48"}], " ", 
            TagBox[
             RowBox[{"\[Zeta]", "(", 
              TagBox["3",
               Zeta,
               Editable->True], ")"}],
             InterpretTemplate[
              Function[BoxForm`e$, 
               Zeta[BoxForm`e$]]]]}], "+", 
           RowBox[{"98", " ", 
            SuperscriptBox["\[Pi]", "2"]}], "+", "195"}], ")"}]}], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3840", " ", 
             SuperscriptBox["\[Pi]", "2"], " ", 
             RowBox[{"log", "(", "2", ")"}]}], "-", 
            RowBox[{"14", " ", 
             SuperscriptBox["\[Pi]", "4"]}], "-", 
            RowBox[{"4750", " ", 
             SuperscriptBox["\[Pi]", "2"]}], "-", 
            RowBox[{"12080", " ", 
             TagBox[
              RowBox[{"\[Zeta]", "(", 
               TagBox["3",
                Zeta,
                Editable->True], ")"}],
              InterpretTemplate[
               Function[BoxForm`e$, 
                Zeta[BoxForm`e$]]]]}], "+", "16755"}], ")"}]}], "1920"], "+", 
        
        FractionBox["17", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}]}]], "-", 
        FractionBox["6", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"d", "-", "4"}], ")"}], "2"]]}], ")"}]}]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 
         Pattern[M, 
          Blank[]]}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{"2", "-", 
          FractionBox["d", "2"]}], ")"}], "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
         FractionBox["d", "2"], "-", "1"}], ")"}], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"4", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        FractionBox["d", "2"], ")"}]}]]}], ",", 
    RowBox[{
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["K",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M_"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d_",
         TraditionalForm], ")"}]],
      TJI[
       Pattern[d, 
        Blank[]], 0, {{1, 
         Pattern[M, 
          Blank[]]}, {1, 0}, {1, 0}}],
      Editable->True], "\[RuleDelayed]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["M", "2"], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"3", "-", "d"}], ")"}], " ", 
       RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{"2", "-", 
         FractionBox["d", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"\[CapitalGamma]", "(", 
         RowBox[{
          FractionBox["d", "2"], "-", "1"}], ")"}], "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"4", "-", "d"}], ")"}], " ", 
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]]}]], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["M", "2"], ")"}], 
           RowBox[{"d", "-", "4"}]], " ", 
          InterpretationBox[
           StyleBox[
            SubsuperscriptBox["S", "d", "2"],
            FontWeight->"Bold"],
           SEpsilon[d],
           Editable->False]}],
         Prefactor1[(M^2)^(-4 + d) SEpsilon[d]^2],
         Editable->False], ")"}]}], 
      RowBox[{"\[CapitalGamma]", "(", 
       FractionBox["d", "2"], ")"}]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Save tarcer*.mx file", "Section",
 CellDingbat->"\[FilledSmallCircle]",
 CellTags->"T1.11.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", "]"}], ";"}]], "Input"],

Cell["\<\
DumpSave[StringJoin[\"tarcer\", StringJoin @@ 
     ToString /@ $RankLimit, \".mx\"], 
   {$ContextPath, HighEnergyPhysics`Tarcer`Cayley, 
    HighEnergyPhysics`Tarcer`CayleyD, 
    HighEnergyPhysics`Tarcer`Cayleyu, 
    HighEnergyPhysics`Tarcer`Prefactor1, 
    HighEnergyPhysics`Tarcer`TarcerRecurse, 
    HighEnergyPhysics`Tarcer`TFIRecurse, 
    HighEnergyPhysics`Tarcer`TFISimplify, 
    HighEnergyPhysics`Tarcer`SEpsilon, 
    HighEnergyPhysics`Tarcer`TarcerExpand, 
    HighEnergyPhysics`Tarcer`TarcerReduce, 
    HighEnergyPhysics`Tarcer`TAI, 
    HighEnergyPhysics`Tarcer`TAR, 
    HighEnergyPhysics`Tarcer`TBI, 
    HighEnergyPhysics`Tarcer`TBR, 
    HighEnergyPhysics`Tarcer`TFI, 
    HighEnergyPhysics`Tarcer`TFR, 
    HighEnergyPhysics`Tarcer`TJI, 
    HighEnergyPhysics`Tarcer`TJR, 
    HighEnergyPhysics`Tarcer`TKI, 
    HighEnergyPhysics`Tarcer`TVI, 
    HighEnergyPhysics`Tarcer`TVR, 
    HighEnergyPhysics`Tarcer`$BasisIntegrals, 
    HighEnergyPhysics`Tarcer`$Comment, 
    HighEnergyPhysics`Tarcer`$CommentNotebook, 
    HighEnergyPhysics`Tarcer`$RankLimit, 
    HighEnergyPhysics`Tarcer`$RecursionStop, 
    HighEnergyPhysics`Tarcer`$TTable, 
    HighEnergyPhysics`Tarcer`Private`applytlrules, 
    HighEnergyPhysics`Tarcer`Private`applytlrules2,
    HighEnergyPhysics`Tarcer`Private`ExpandMaybe, 
    HighEnergyPhysics`Tarcer`Private`FactorC, 
    HighEnergyPhysics`Tarcer`Private`FEPrint, 
    HighEnergyPhysics`Tarcer`Private`fN, 
    HighEnergyPhysics`Tarcer`Private`mbt, 
    HighEnergyPhysics`Tarcer`Private`PNQ, 
    HighEnergyPhysics`Tarcer`Private`PQ, 
    HighEnergyPhysics`Tarcer`Private`ta, 
    HighEnergyPhysics`Tarcer`Private`TA, 
    HighEnergyPhysics`Tarcer`Private`TAIC, 
    HighEnergyPhysics`Tarcer`Private`TBIC, 
    HighEnergyPhysics`Tarcer`Private`TComment, 
    HighEnergyPhysics`Tarcer`Private`TLRComment, 
    HighEnergyPhysics`Tarcer`Private`TFIC, 
    HighEnergyPhysics`Tarcer`Private`TJIC, 
    HighEnergyPhysics`Tarcer`Private`TLR, 
    HighEnergyPhysics`Tarcer`Private`tlrule, 
    HighEnergyPhysics`Tarcer`Private`redblue, 
    HighEnergyPhysics`Tarcer`Private`TLRules, 
    HighEnergyPhysics`Tarcer`Private`TLRules2, 
    HighEnergyPhysics`Tarcer`Private`TVIC, 
    HighEnergyPhysics`Tarcer`Private`$TLRComment, 
    HighEnergyPhysics`general`Zeta2`Zeta2}]; \
\>", "Input",
 Editable->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"$HistoryLength", "=", "\[Infinity]"}], ";"}]], "Input"],

Cell["\<\
This gives the absolute time (in minutes) to run this notebook.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Round", "[", 
    RowBox[{
     RowBox[{"AbsoluteTime", "[", "]"}], "-", 
     "HighEnergyPhysics`Tarcer`Private`atime"}], "]"}], "/", "60."}], " ", 
  "min"}]], "Input"],

Cell[BoxData[
 RowBox[{"1.7666666666666666`", " ", "min"}]], "Output"]
}, Open  ]],

Cell["Display the size of the generated file:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"filename", " ", "=", " ", 
  RowBox[{"\"\<tarcer\>\"", "<>", 
   RowBox[{"(", 
    RowBox[{"Apply", "[", 
     RowBox[{"StringJoin", ",", 
      RowBox[{"ToString", "/@", "$RankLimit"}]}], "]"}], ")"}], "<>", 
   "\"\<.mx\>\""}]}]], "Input"],

Cell[BoxData["\<\"tarcer25.mx\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FileByteCount", "[", "filename", "]"}], " ", "Byte"}]], "Input"],

Cell[BoxData[
 RowBox[{"5388936", " ", "Byte"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Directory", "[", "]"}]], "Input"],

Cell[BoxData["\<\"/home/rolfm/fccvs/fc7/HighEnergyPhysics/Tarcer\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input"],

Cell[BoxData["99177464"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemoryInUse", "[", "]"}]], "Input"],

Cell[BoxData["25627688"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData["87.87000000000002`"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{1430, 849},
WindowMargins->{{56, Automatic}, {Automatic, 0}},
ShowSelection->True,
CellLabelAutoDelete->True,
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.5, 1.5 Inherited],
FrontEndVersion->"7.0 for Linux x86 (64-bit) (February 25, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "T1.1"->{
  Cell[567, 22, 72, 2, 188, "Title",
   CellTags->"T1.1"]},
 "T1.2"->{
  Cell[816, 34, 402, 14, 223, "Text",
   CellTags->"T1.2"]},
 "T1.1.1"->{
  Cell[29513, 1116, 91, 2, 160, "Section",
   CellTags->"T1.1.1"]},
 "T1.2.1.1"->{
  Cell[36281, 1354, 77, 1, 61, "Subsubsection",
   CellTags->"T1.2.1.1"]},
 "T1.2.1"->{
  Cell[36913, 1374, 84, 2, 160, "Section",
   CellTags->"T1.2.1"]},
 "T1.3.1"->{
  Cell[44789, 1618, 110, 2, 160, "Section",
   CellTags->"T1.3.1"]},
 "T1.3.1.1"->{
  Cell[47363, 1721, 62, 1, 83, "Subsection",
   CellTags->"T1.3.1.1"]},
 "T1.3.2.1"->{
  Cell[110436, 3594, 57, 1, 83, "Subsection",
   CellTags->"T1.3.2.1"]},
 "T1.3.3.1"->{
  Cell[124706, 3941, 58, 1, 83, "Subsection",
   CellTags->"T1.3.3.1"]},
 "T1.3.4.1"->{
  Cell[139557, 4360, 59, 1, 83, "Subsection",
   CellTags->"T1.3.4.1"]},
 "T1.3.5.1"->{
  Cell[141922, 4436, 53, 1, 83, "Subsection",
   CellTags->"T1.3.5.1"]},
 "T1.3.6.1"->{
  Cell[158817, 4890, 58, 1, 83, "Subsection",
   CellTags->"T1.3.6.1"]},
 "T1.3.7.1"->{
  Cell[160614, 4946, 49, 1, 83, "Subsection",
   CellTags->"T1.3.7.1"]},
 "T1.3.7.1.1"->{
  Cell[160893, 4958, 56, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.1.1"]},
 "T1.3.7.2.1"->{
  Cell[165598, 5126, 52, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.2.1"]},
 "T1.3.7.3.1"->{
  Cell[165843, 5139, 53, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.3.1"]},
 "T1.3.7.4.1"->{
  Cell[166194, 5157, 57, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.4.1"]},
 "T1.3.7.5.1"->{
  Cell[166468, 5170, 53, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.5.1"]},
 "T1.3.7.6.1"->{
  Cell[170087, 5257, 53, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.6.1"]},
 "T1.3.7.7.1"->{
  Cell[172400, 5339, 53, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.7.1"]},
 "T1.3.7.8.1"->{
  Cell[174130, 5403, 53, 1, 61, "Subsubsection",
   CellTags->"T1.3.7.8.1"]},
 "T1.4.1"->{
  Cell[174530, 5423, 102, 2, 160, "Section",
   CellTags->"T1.4.1"]},
 "T1.4.1.1"->{
  Cell[179170, 5574, 108, 3, 85, "Subsection",
   CellTags->"T1.4.1.1"]},
 "T1.4.2.1"->{
  Cell[179673, 5596, 106, 3, 85, "Subsection",
   CellTags->"T1.4.2.1"]},
 "T1.4.3.1"->{
  Cell[181806, 5678, 196, 6, 85, "Subsection",
   CellTags->"T1.4.3.1"]},
 "T1.4.4.1"->{
  Cell[188041, 5881, 233, 7, 119, "Subsection",
   CellTags->"T1.4.4.1"]},
 "T1.5.1"->{
  Cell[200189, 6288, 95, 2, 157, "Section",
   CellTags->"T1.5.1"]},
 "T1.5.1.1"->{
  Cell[200309, 6294, 64, 1, 157, "Subsection",
   CellTags->"T1.5.1.1"]},
 "T1.5.1.1.1"->{
  Cell[274094, 8238, 61, 1, 61, "Subsubsection",
   CellTags->"T1.5.1.1.1"]},
 "T1.5.2.1"->{
  Cell[280627, 8452, 58, 1, 83, "Subsection",
   CellTags->"T1.5.2.1"]},
 "T1.5.2.1.1"->{
  Cell[285004, 8586, 52, 1, 61, "Subsubsection",
   CellTags->"T1.5.2.1.1"]},
 "T1.5.3.1"->{
  Cell[300414, 9071, 59, 1, 157, "Subsection",
   CellTags->"T1.5.3.1"]},
 "T1.5.3.1.1"->{
  Cell[300817, 9086, 67, 1, 61, "Subsubsection",
   CellTags->"T1.5.3.1.1"]},
 "T1.5.3.2.1"->{
  Cell[311153, 9374, 67, 1, 61, "Subsubsection",
   CellTags->"T1.5.3.2.1"]},
 "T1.6.1"->{
  Cell[319500, 9607, 138, 2, 220, "Section",
   CellTags->"T1.6.1"]},
 "T1.6.1.1"->{
  Cell[320760, 9652, 55, 1, 157, "Subsection",
   CellTags->"T1.6.1.1"]},
 "T1.6.2.1"->{
  Cell[327661, 9889, 57, 1, 83, "Subsection",
   CellTags->"T1.6.2.1"]},
 "T1.6.2.1.1"->{
  Cell[328001, 9905, 167, 7, 61, "Subsubsection",
   CellTags->"T1.6.2.1.1"]},
 "T1.6.2.2.1"->{
  Cell[329271, 9956, 167, 7, 61, "Subsubsection",
   CellTags->"T1.6.2.2.1"]},
 "T1.6.2.3.1"->{
  Cell[330710, 10012, 167, 7, 61, "Subsubsection",
   CellTags->"T1.6.2.3.1"]},
 "T1.6.2.4.1"->{
  Cell[332149, 10068, 167, 7, 61, "Subsubsection",
   CellTags->"T1.6.2.4.1"]},
 "T1.6.3.1"->{
  Cell[333600, 10125, 57, 1, 83, "Subsection",
   CellTags->"T1.6.3.1"]},
 "T1.6.4.1"->{
  Cell[337480, 10272, 246, 9, 90, "Subsection",
   CellTags->"T1.6.4.1"]},
 "T1.6.5.1"->{
  Cell[338407, 10310, 57, 1, 83, "Subsection",
   CellTags->"T1.6.5.1"]},
 "T1.6.6.1"->{
  Cell[342287, 10457, 246, 9, 90, "Subsection",
   CellTags->"T1.6.6.1"]},
 "T1.6.7.1"->{
  Cell[343214, 10495, 246, 9, 90, "Subsection",
   CellTags->"T1.6.7.1"]},
 "T1.6.8.1"->{
  Cell[344435, 10542, 246, 9, 90, "Subsection",
   CellTags->"T1.6.8.1"]},
 "T1.6.9.1"->{
  Cell[345656, 10589, 55, 1, 157, "Subsection",
   CellTags->"T1.6.9.1"]},
 "T1.6.10.1"->{
  Cell[352049, 10808, 58, 1, 83, "Subsection",
   CellTags->"T1.6.10.1"]},
 "T1.6.11.1"->{
  Cell[353441, 10863, 56, 1, 157, "Subsection",
   CellTags->"T1.6.11.1"]},
 "T1.6.12.1"->{
  Cell[357613, 11025, 56, 1, 157, "Subsection",
   CellTags->"T1.6.12.1"]},
 "T1.6.13.1"->{
  Cell[361785, 11187, 56, 1, 83, "Subsection",
   CellTags->"T1.6.13.1"]},
 "T1.6.14.1"->{
  Cell[368467, 11416, 67, 1, 83, "Subsection",
   CellTags->"T1.6.14.1"]},
 "T1.6.15.1"->{
  Cell[369565, 11457, 58, 1, 83, "Subsection",
   CellTags->"T1.6.15.1"]},
 "T1.6.16.1"->{
  Cell[392317, 12001, 56, 1, 157, "Subsection",
   CellTags->"T1.6.16.1"]},
 "T1.6.17.1"->{
  Cell[396384, 12153, 58, 1, 83, "Subsection",
   CellTags->"T1.6.17.1"]},
 "T1.6.18.1"->{
  Cell[397453, 12200, 56, 1, 157, "Subsection",
   CellTags->"T1.6.18.1"]},
 "T1.6.19.1"->{
  Cell[403027, 12403, 58, 1, 83, "Subsection",
   CellTags->"T1.6.19.1"]},
 "T1.6.20.1"->{
  Cell[404086, 12448, 58, 1, 83, "Subsection",
   CellTags->"T1.6.20.1"]},
 "T1.6.21.1"->{
  Cell[405211, 12495, 56, 1, 157, "Subsection",
   CellTags->"T1.6.21.1"]},
 "T1.6.22.1"->{
  Cell[408503, 12622, 58, 1, 83, "Subsection",
   CellTags->"T1.6.22.1"]},
 "T1.6.23.1"->{
  Cell[409447, 12663, 56, 1, 157, "Subsection",
   CellTags->"T1.6.23.1"]},
 "T1.6.24.1"->{
  Cell[413478, 12815, 58, 1, 83, "Subsection",
   CellTags->"T1.6.24.1"]},
 "T1.6.25.1"->{
  Cell[414496, 12858, 56, 1, 157, "Subsection",
   CellTags->"T1.6.25.1"]},
 "T1.6.26.1"->{
  Cell[418284, 13001, 58, 1, 83, "Subsection",
   CellTags->"T1.6.26.1"]},
 "T1.6.27.1"->{
  Cell[419304, 13044, 56, 1, 157, "Subsection",
   CellTags->"T1.6.27.1"]},
 "T1.6.28.1"->{
  Cell[425502, 13263, 58, 1, 83, "Subsection",
   CellTags->"T1.6.28.1"]},
 "T1.6.29.1"->{
  Cell[427618, 13327, 57, 1, 157, "Subsection",
   CellTags->"T1.6.29.1"]},
 "T1.6.30.1"->{
  Cell[430911, 13454, 58, 1, 83, "Subsection",
   CellTags->"T1.6.30.1"]},
 "T1.6.31.1"->{
  Cell[431874, 13495, 57, 1, 157, "Subsection",
   CellTags->"T1.6.31.1"]},
 "T1.6.32.1"->{
  Cell[436805, 13691, 58, 1, 157, "Subsection",
   CellTags->"T1.6.32.1"]},
 "T1.6.33.1"->{
  Cell[438888, 13779, 60, 1, 157, "Subsection",
   CellTags->"T1.6.33.1"]},
 "T1.6.34.1"->{
  Cell[441092, 13870, 57, 1, 157, "Subsection",
   CellTags->"T1.6.34.1"]},
 "T1.6.35.1"->{
  Cell[446833, 14080, 58, 1, 83, "Subsection",
   CellTags->"T1.6.35.1"]},
 "T1.6.36.1"->{
  Cell[448397, 14140, 57, 1, 157, "Subsection",
   CellTags->"T1.6.36.1"]},
 "T1.6.37.1"->{
  Cell[455094, 14368, 58, 1, 157, "Subsection",
   CellTags->"T1.6.37.1"]},
 "T1.6.38.1"->{
  Cell[457524, 14465, 60, 1, 83, "Subsection",
   CellTags->"T1.6.38.1"]},
 "T1.6.39.1"->{
  Cell[460615, 14581, 60, 1, 83, "Subsection",
   CellTags->"T1.6.39.1"]},
 "T1.6.40.1"->{
  Cell[462917, 14665, 60, 1, 83, "Subsection",
   CellTags->"T1.6.40.1"]},
 "T1.6.41.1"->{
  Cell[464578, 14729, 57, 1, 157, "Subsection",
   CellTags->"T1.6.41.1"]},
 "T1.6.42.1"->{
  Cell[469831, 14921, 58, 1, 83, "Subsection",
   CellTags->"T1.6.42.1"]},
 "T1.6.43.1"->{
  Cell[471418, 14983, 67, 1, 157, "Subsection",
   CellTags->"T1.6.43.1"]},
 "T1.6.44.1"->{
  Cell[477531, 15208, 56, 1, 157, "Subsection",
   CellTags->"T1.6.44.1"]},
 "T1.6.45.1"->{
  Cell[491504, 15715, 58, 1, 83, "Subsection",
   CellTags->"T1.6.45.1"]},
 "T1.6.46.1"->{
  Cell[492897, 15771, 60, 1, 83, "Subsection",
   CellTags->"T1.6.46.1"]},
 "T1.6.47.1"->{
  Cell[494410, 15830, 56, 1, 157, "Subsection",
   CellTags->"T1.6.47.1"]},
 "T1.6.48.1"->{
  Cell[500801, 16053, 58, 1, 83, "Subsection",
   CellTags->"T1.6.48.1"]},
 "T1.6.49.1"->{
  Cell[502235, 16110, 60, 1, 83, "Subsection",
   CellTags->"T1.6.49.1"]},
 "T1.6.50.1"->{
  Cell[503808, 16171, 56, 1, 157, "Subsection",
   CellTags->"T1.6.50.1"]},
 "T1.6.51.1"->{
  Cell[515982, 16591, 58, 1, 83, "Subsection",
   CellTags->"T1.6.51.1"]},
 "T1.6.52.1"->{
  Cell[516876, 16628, 56, 1, 157, "Subsection",
   CellTags->"T1.6.52.1"]},
 "T1.6.53.1"->{
  Cell[522635, 16837, 58, 1, 83, "Subsection",
   CellTags->"T1.6.53.1"]},
 "T1.6.54.1"->{
  Cell[523774, 16885, 60, 1, 83, "Subsection",
   CellTags->"T1.6.54.1"]},
 "T1.6.55.1"->{
  Cell[524993, 16935, 60, 1, 83, "Subsection",
   CellTags->"T1.6.55.1"]},
 "T1.6.56.1"->{
  Cell[526212, 16985, 56, 1, 157, "Subsection",
   CellTags->"T1.6.56.1"]},
 "T1.6.57.1"->{
  Cell[532675, 17215, 58, 1, 83, "Subsection",
   CellTags->"T1.6.57.1"]},
 "T1.6.58.1"->{
  Cell[533668, 17258, 56, 1, 157, "Subsection",
   CellTags->"T1.6.58.1"]},
 "T1.6.59.1"->{
  Cell[539205, 17449, 58, 1, 83, "Subsection",
   CellTags->"T1.6.59.1"]},
 "T1.6.60.1"->{
  Cell[540189, 17490, 56, 1, 157, "Subsection",
   CellTags->"T1.6.60.1"]},
 "T1.6.61.1"->{
  Cell[545988, 17698, 58, 1, 83, "Subsection",
   CellTags->"T1.6.61.1"]},
 "T1.6.62.1"->{
  Cell[547303, 17751, 60, 1, 83, "Subsection",
   CellTags->"T1.6.62.1"]},
 "T1.6.63.1"->{
  Cell[548700, 17806, 60, 1, 83, "Subsection",
   CellTags->"T1.6.63.1"]},
 "T1.6.64.1"->{
  Cell[550112, 17861, 56, 1, 157, "Subsection",
   CellTags->"T1.6.64.1"]},
 "T1.6.65.1"->{
  Cell[563628, 18314, 58, 1, 83, "Subsection",
   CellTags->"T1.6.65.1"]},
 "T1.6.66.1"->{
  Cell[564793, 18362, 60, 1, 83, "Subsection",
   CellTags->"T1.6.66.1"]},
 "T1.6.67.1"->{
  Cell[566036, 18412, 60, 1, 83, "Subsection",
   CellTags->"T1.6.67.1"]},
 "T1.6.68.1"->{
  Cell[567281, 18462, 56, 1, 157, "Subsection",
   CellTags->"T1.6.68.1"]},
 "T1.6.69.1"->{
  Cell[575409, 18738, 58, 1, 83, "Subsection",
   CellTags->"T1.6.69.1"]},
 "T1.6.70.1"->{
  Cell[576545, 18784, 56, 1, 157, "Subsection",
   CellTags->"T1.6.70.1"]},
 "T1.6.70.1.1"->{
  Cell[576626, 18789, 55, 1, 61, "Subsubsection",
   CellTags->"T1.6.70.1.1"]},
 "T1.6.70.2.1"->{
  Cell[581140, 18950, 55, 1, 61, "Subsubsection",
   CellTags->"T1.6.70.2.1"]},
 "T1.6.71.1"->{
  Cell[589906, 19272, 58, 1, 83, "Subsection",
   CellTags->"T1.6.71.1"]},
 "T1.6.72.1"->{
  Cell[590597, 19301, 56, 1, 157, "Subsection",
   CellTags->"T1.6.72.1"]},
 "T1.6.73.1"->{
  Cell[602600, 19691, 58, 1, 83, "Subsection",
   CellTags->"T1.6.73.1"]},
 "T1.6.74.1"->{
  Cell[603281, 19720, 56, 1, 157, "Subsection",
   CellTags->"T1.6.74.1"]},
 "T1.6.75.1"->{
  Cell[614700, 20150, 59, 1, 83, "Subsection",
   CellTags->"T1.6.75.1"]},
 "T1.6.76.1"->{
  Cell[615528, 20184, 59, 1, 83, "Subsection",
   CellTags->"T1.6.76.1"]},
 "T1.6.77.1"->{
  Cell[616562, 20225, 59, 1, 83, "Subsection",
   CellTags->"T1.6.77.1"]},
 "T1.6.78.1"->{
  Cell[617363, 20256, 59, 1, 83, "Subsection",
   CellTags->"T1.6.78.1"]},
 "T1.6.79.1"->{
  Cell[618278, 20291, 56, 1, 157, "Subsection",
   CellTags->"T1.6.79.1"]},
 "T1.6.80.1"->{
  Cell[624184, 20498, 56, 1, 157, "Subsection",
   CellTags->"T1.6.80.1"]},
 "T1.6.81.1"->{
  Cell[627567, 20626, 58, 1, 83, "Subsection",
   CellTags->"T1.6.81.1"]},
 "T1.6.82.1"->{
  Cell[628416, 20661, 56, 1, 157, "Subsection",
   CellTags->"T1.6.82.1"]},
 "T1.6.83.1"->{
  Cell[631547, 20779, 58, 1, 83, "Subsection",
   CellTags->"T1.6.83.1"]},
 "T1.6.84.1"->{
  Cell[632277, 20809, 60, 1, 83, "Subsection",
   CellTags->"T1.6.84.1"]},
 "T1.6.85.1"->{
  Cell[633539, 20858, 60, 1, 83, "Subsection",
   CellTags->"T1.6.85.1"]},
 "T1.6.86.1"->{
  Cell[634922, 20910, 56, 1, 157, "Subsection",
   CellTags->"T1.6.86.1"]},
 "T1.6.87.1"->{
  Cell[637113, 21001, 58, 1, 83, "Subsection",
   CellTags->"T1.6.87.1"]},
 "T1.6.88.1"->{
  Cell[637892, 21033, 56, 1, 157, "Subsection",
   CellTags->"T1.6.88.1"]},
 "T1.6.89.1"->{
  Cell[640249, 21127, 58, 1, 83, "Subsection",
   CellTags->"T1.6.89.1"]},
 "T1.6.90.1"->{
  Cell[640840, 21151, 56, 1, 157, "Subsection",
   CellTags->"T1.6.90.1"]},
 "T1.6.91.1"->{
  Cell[642371, 21217, 58, 1, 83, "Subsection",
   CellTags->"T1.6.91.1"]},
 "T1.6.92.1"->{
  Cell[643073, 21246, 56, 1, 157, "Subsection",
   CellTags->"T1.6.92.1"]},
 "T1.6.93.1"->{
  Cell[644478, 21307, 58, 1, 83, "Subsection",
   CellTags->"T1.6.93.1"]},
 "T1.6.94.1"->{
  Cell[645031, 21331, 56, 1, 157, "Subsection",
   CellTags->"T1.6.94.1"]},
 "T1.6.95.1"->{
  Cell[650058, 21522, 58, 1, 83, "Subsection",
   CellTags->"T1.6.95.1"]},
 "T1.6.96.1"->{
  Cell[650857, 21555, 60, 1, 83, "Subsection",
   CellTags->"T1.6.96.1"]},
 "T1.6.97.1"->{
  Cell[651781, 21591, 60, 1, 83, "Subsection",
   CellTags->"T1.6.97.1"]},
 "T1.6.98.1"->{
  Cell[652705, 21627, 57, 1, 157, "Subsection",
   CellTags->"T1.6.98.1"]},
 "T1.6.99.1"->{
  Cell[658806, 21858, 59, 1, 83, "Subsection",
   CellTags->"T1.6.99.1"]},
 "T1.6.100.1"->{
  Cell[659686, 21897, 61, 1, 83, "Subsection",
   CellTags->"T1.6.100.1"]},
 "T1.6.101.1"->{
  Cell[660673, 21938, 61, 1, 83, "Subsection",
   CellTags->"T1.6.101.1"]},
 "T1.6.102.1"->{
  Cell[661711, 21980, 61, 1, 83, "Subsection",
   CellTags->"T1.6.102.1"]},
 "T1.6.103.1"->{
  Cell[662698, 22021, 61, 1, 83, "Subsection",
   CellTags->"T1.6.103.1"]},
 "T1.6.104.1"->{
  Cell[663685, 22062, 61, 1, 83, "Subsection",
   CellTags->"T1.6.104.1"]},
 "T1.6.105.1"->{
  Cell[664723, 22104, 58, 1, 157, "Subsection",
   CellTags->"T1.6.105.1"]},
 "T1.6.106.1"->{
  Cell[667757, 22206, 60, 1, 83, "Subsection",
   CellTags->"T1.6.106.1"]},
 "T1.6.107.1"->{
  Cell[668126, 22224, 58, 1, 157, "Subsection",
   CellTags->"T1.6.107.1"]},
 "T1.6.108.1"->{
  Cell[671466, 22334, 60, 1, 83, "Subsection",
   CellTags->"T1.6.108.1"]},
 "T1.6.109.1"->{
  Cell[672027, 22358, 58, 1, 157, "Subsection",
   CellTags->"T1.6.109.1"]},
 "T1.6.110.1"->{
  Cell[674846, 22454, 60, 1, 83, "Subsection",
   CellTags->"T1.6.110.1"]},
 "T1.6.111.1"->{
  Cell[675215, 22472, 58, 1, 157, "Subsection",
   CellTags->"T1.6.111.1"]},
 "T1.6.112.1"->{
  Cell[676991, 22534, 60, 1, 83, "Subsection",
   CellTags->"T1.6.112.1"]},
 "T1.6.113.1"->{
  Cell[677360, 22552, 58, 1, 157, "Subsection",
   CellTags->"T1.6.113.1"]},
 "T1.6.114.1"->{
  Cell[685032, 22809, 60, 1, 83, "Subsection",
   CellTags->"T1.6.114.1"]},
 "T1.6.115.1"->{
  Cell[685989, 22847, 60, 1, 83, "Subsection",
   CellTags->"T1.6.115.1"]},
 "T1.6.116.1"->{
  Cell[686780, 22884, 60, 1, 83, "Subsection",
   CellTags->"T1.6.116.1"]},
 "T1.6.117.1"->{
  Cell[687575, 22921, 61, 1, 83, "Subsection",
   CellTags->"T1.6.117.1"]},
 "T1.6.118.1"->{
  Cell[691061, 23020, 61, 1, 83, "Subsection",
   CellTags->"T1.6.118.1"]},
 "T1.6.119.1"->{
  Cell[694517, 23119, 61, 1, 83, "Subsection",
   CellTags->"T1.6.119.1"]},
 "T1.6.120.1"->{
  Cell[697967, 23218, 60, 1, 83, "Subsection",
   CellTags->"T1.6.120.1"]},
 "Tsymmetry"->{
  Cell[706578, 23440, 54, 1, 83, "Subsection",
   CellTags->"Tsymmetry"]},
 "specialTVR"->{
  Cell[706816, 23453, 69, 1, 83, "Subsection",
   CellTags->"specialTVR"]},
 "T1.6.121.1"->{
  Cell[717604, 23641, 59, 1, 83, "Subsection",
   CellTags->"T1.6.121.1"]},
 "T1.6.122.1"->{
  Cell[723996, 23850, 60, 1, 83, "Subsection",
   CellTags->"T1.6.122.1"]},
 "T1.6.123.1"->{
  Cell[726982, 23950, 59, 1, 83, "Subsection",
   CellTags->"T1.6.123.1"]},
 "T1.6.124.1"->{
  Cell[729465, 24037, 57, 1, 83, "Subsection",
   CellTags->"T1.6.124.1"]},
 "T1.7.1"->{
  Cell[731649, 24120, 106, 2, 160, "Section",
   CellTags->"T1.7.1"]},
 "T1.7.1.1"->{
  Cell[731780, 24126, 69, 1, 83, "Subsection",
   CellTags->"T1.7.1.1"]},
 "TLRdef"->{
  Cell[734399, 24225, 215, 7, 223, "Text",
   CellTags->"TLRdef"]},
 "tlrule17"->{
  Cell[767526, 25190, 77, 1, 64, "Text",
   CellTags->"tlrule17"]},
 "T1.7.2.1"->{
  Cell[810561, 26571, 49, 1, 157, "Subsection",
   CellTags->"T1.7.2.1"]},
 "T1.7.3.1"->{
  Cell[828135, 27145, 53, 1, 83, "Subsection",
   CellTags->"T1.7.3.1"]},
 "$TTable"->{
  Cell[831948, 27247, 51, 1, 83, "Subsection",
   CellTags->"$TTable"]},
 "T1.7.4.1"->{
  Cell[832131, 27258, 58, 1, 83, "Subsection",
   CellTags->"T1.7.4.1"]},
 "T1.8.1"->{
  Cell[850885, 27744, 90, 2, 160, "Section",
   CellTags->"T1.8.1"]},
 "T1.9.1"->{
  Cell[868422, 28202, 91, 2, 157, "Section",
   CellTags->"T1.9.1"]},
 "T1.9.1.1"->{
  Cell[868538, 28208, 53, 1, 157, "Subsection",
   CellTags->"T1.9.1.1"]},
 "T1.10.1"->{
  Cell[877297, 28491, 93, 2, 157, "Section",
   CellTags->"T1.10.1"]},
 "T1.11.1"->{
  Cell[911765, 29616, 99, 2, 160, "Section",
   CellTags->"T1.11.1"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"T1.1", 916488, 29781},
 {"T1.2", 916559, 29784},
 {"T1.1.1", 916633, 29787},
 {"T1.2.1.1", 916716, 29790},
 {"T1.2.1", 916804, 29793},
 {"T1.3.1", 916885, 29796},
 {"T1.3.1.1", 916969, 29799},
 {"T1.3.2.1", 917056, 29802},
 {"T1.3.3.1", 917144, 29805},
 {"T1.3.4.1", 917232, 29808},
 {"T1.3.5.1", 917320, 29811},
 {"T1.3.6.1", 917408, 29814},
 {"T1.3.7.1", 917496, 29817},
 {"T1.3.7.1.1", 917586, 29820},
 {"T1.3.7.2.1", 917681, 29823},
 {"T1.3.7.3.1", 917776, 29826},
 {"T1.3.7.4.1", 917871, 29829},
 {"T1.3.7.5.1", 917966, 29832},
 {"T1.3.7.6.1", 918061, 29835},
 {"T1.3.7.7.1", 918156, 29838},
 {"T1.3.7.8.1", 918251, 29841},
 {"T1.4.1", 918342, 29844},
 {"T1.4.1.1", 918427, 29847},
 {"T1.4.2.1", 918516, 29850},
 {"T1.4.3.1", 918605, 29853},
 {"T1.4.4.1", 918694, 29856},
 {"T1.5.1", 918782, 29859},
 {"T1.5.1.1", 918866, 29862},
 {"T1.5.1.1.1", 918957, 29865},
 {"T1.5.2.1", 919050, 29868},
 {"T1.5.2.1.1", 919140, 29871},
 {"T1.5.3.1", 919233, 29874},
 {"T1.5.3.1.1", 919324, 29877},
 {"T1.5.3.2.1", 919419, 29880},
 {"T1.6.1", 919510, 29883},
 {"T1.6.1.1", 919595, 29886},
 {"T1.6.2.1", 919684, 29889},
 {"T1.6.2.1.1", 919774, 29892},
 {"T1.6.2.2.1", 919870, 29895},
 {"T1.6.2.3.1", 919966, 29898},
 {"T1.6.2.4.1", 920063, 29901},
 {"T1.6.3.1", 920158, 29904},
 {"T1.6.4.1", 920247, 29907},
 {"T1.6.5.1", 920337, 29910},
 {"T1.6.6.1", 920426, 29913},
 {"T1.6.7.1", 920516, 29916},
 {"T1.6.8.1", 920606, 29919},
 {"T1.6.9.1", 920696, 29922},
 {"T1.6.10.1", 920787, 29925},
 {"T1.6.11.1", 920878, 29928},
 {"T1.6.12.1", 920970, 29931},
 {"T1.6.13.1", 921062, 29934},
 {"T1.6.14.1", 921153, 29937},
 {"T1.6.15.1", 921244, 29940},
 {"T1.6.16.1", 921335, 29943},
 {"T1.6.17.1", 921427, 29946},
 {"T1.6.18.1", 921518, 29949},
 {"T1.6.19.1", 921610, 29952},
 {"T1.6.20.1", 921701, 29955},
 {"T1.6.21.1", 921792, 29958},
 {"T1.6.22.1", 921884, 29961},
 {"T1.6.23.1", 921975, 29964},
 {"T1.6.24.1", 922067, 29967},
 {"T1.6.25.1", 922158, 29970},
 {"T1.6.26.1", 922250, 29973},
 {"T1.6.27.1", 922341, 29976},
 {"T1.6.28.1", 922433, 29979},
 {"T1.6.29.1", 922524, 29982},
 {"T1.6.30.1", 922616, 29985},
 {"T1.6.31.1", 922707, 29988},
 {"T1.6.32.1", 922799, 29991},
 {"T1.6.33.1", 922891, 29994},
 {"T1.6.34.1", 922983, 29997},
 {"T1.6.35.1", 923075, 30000},
 {"T1.6.36.1", 923166, 30003},
 {"T1.6.37.1", 923258, 30006},
 {"T1.6.38.1", 923350, 30009},
 {"T1.6.39.1", 923441, 30012},
 {"T1.6.40.1", 923532, 30015},
 {"T1.6.41.1", 923623, 30018},
 {"T1.6.42.1", 923715, 30021},
 {"T1.6.43.1", 923806, 30024},
 {"T1.6.44.1", 923898, 30027},
 {"T1.6.45.1", 923990, 30030},
 {"T1.6.46.1", 924081, 30033},
 {"T1.6.47.1", 924172, 30036},
 {"T1.6.48.1", 924264, 30039},
 {"T1.6.49.1", 924355, 30042},
 {"T1.6.50.1", 924446, 30045},
 {"T1.6.51.1", 924538, 30048},
 {"T1.6.52.1", 924629, 30051},
 {"T1.6.53.1", 924721, 30054},
 {"T1.6.54.1", 924812, 30057},
 {"T1.6.55.1", 924903, 30060},
 {"T1.6.56.1", 924994, 30063},
 {"T1.6.57.1", 925086, 30066},
 {"T1.6.58.1", 925177, 30069},
 {"T1.6.59.1", 925269, 30072},
 {"T1.6.60.1", 925360, 30075},
 {"T1.6.61.1", 925452, 30078},
 {"T1.6.62.1", 925543, 30081},
 {"T1.6.63.1", 925634, 30084},
 {"T1.6.64.1", 925725, 30087},
 {"T1.6.65.1", 925817, 30090},
 {"T1.6.66.1", 925908, 30093},
 {"T1.6.67.1", 925999, 30096},
 {"T1.6.68.1", 926090, 30099},
 {"T1.6.69.1", 926182, 30102},
 {"T1.6.70.1", 926273, 30105},
 {"T1.6.70.1.1", 926367, 30108},
 {"T1.6.70.2.1", 926465, 30111},
 {"T1.6.71.1", 926561, 30114},
 {"T1.6.72.1", 926652, 30117},
 {"T1.6.73.1", 926744, 30120},
 {"T1.6.74.1", 926835, 30123},
 {"T1.6.75.1", 926927, 30126},
 {"T1.6.76.1", 927018, 30129},
 {"T1.6.77.1", 927109, 30132},
 {"T1.6.78.1", 927200, 30135},
 {"T1.6.79.1", 927291, 30138},
 {"T1.6.80.1", 927383, 30141},
 {"T1.6.81.1", 927475, 30144},
 {"T1.6.82.1", 927566, 30147},
 {"T1.6.83.1", 927658, 30150},
 {"T1.6.84.1", 927749, 30153},
 {"T1.6.85.1", 927840, 30156},
 {"T1.6.86.1", 927931, 30159},
 {"T1.6.87.1", 928023, 30162},
 {"T1.6.88.1", 928114, 30165},
 {"T1.6.89.1", 928206, 30168},
 {"T1.6.90.1", 928297, 30171},
 {"T1.6.91.1", 928389, 30174},
 {"T1.6.92.1", 928480, 30177},
 {"T1.6.93.1", 928572, 30180},
 {"T1.6.94.1", 928663, 30183},
 {"T1.6.95.1", 928755, 30186},
 {"T1.6.96.1", 928846, 30189},
 {"T1.6.97.1", 928937, 30192},
 {"T1.6.98.1", 929028, 30195},
 {"T1.6.99.1", 929120, 30198},
 {"T1.6.100.1", 929212, 30201},
 {"T1.6.101.1", 929305, 30204},
 {"T1.6.102.1", 929398, 30207},
 {"T1.6.103.1", 929491, 30210},
 {"T1.6.104.1", 929584, 30213},
 {"T1.6.105.1", 929677, 30216},
 {"T1.6.106.1", 929771, 30219},
 {"T1.6.107.1", 929864, 30222},
 {"T1.6.108.1", 929958, 30225},
 {"T1.6.109.1", 930051, 30228},
 {"T1.6.110.1", 930145, 30231},
 {"T1.6.111.1", 930238, 30234},
 {"T1.6.112.1", 930332, 30237},
 {"T1.6.113.1", 930425, 30240},
 {"T1.6.114.1", 930519, 30243},
 {"T1.6.115.1", 930612, 30246},
 {"T1.6.116.1", 930705, 30249},
 {"T1.6.117.1", 930798, 30252},
 {"T1.6.118.1", 930891, 30255},
 {"T1.6.119.1", 930984, 30258},
 {"T1.6.120.1", 931077, 30261},
 {"Tsymmetry", 931169, 30264},
 {"specialTVR", 931261, 30267},
 {"T1.6.121.1", 931354, 30270},
 {"T1.6.122.1", 931447, 30273},
 {"T1.6.123.1", 931540, 30276},
 {"T1.6.124.1", 931633, 30279},
 {"T1.7.1", 931722, 30282},
 {"T1.7.1.1", 931808, 30285},
 {"TLRdef", 931895, 30288},
 {"tlrule17", 931978, 30291},
 {"T1.7.2.1", 932061, 30294},
 {"T1.7.3.1", 932151, 30297},
 {"$TTable", 932239, 30300},
 {"T1.7.4.1", 932327, 30303},
 {"T1.8.1", 932414, 30306},
 {"T1.9.1", 932497, 30309},
 {"T1.9.1.1", 932582, 30312},
 {"T1.10.1", 932671, 30315},
 {"T1.11.1", 932756, 30318}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 72, 2, 188, "Title",
 CellTags->"T1.1"],
Cell[642, 26, 171, 6, 64, "Text"],
Cell[816, 34, 402, 14, 223, "Text",
 CellTags->"T1.2"],
Cell[1221, 50, 55, 1, 64, "Text"],
Cell[1279, 53, 130, 5, 64, "Text"],
Cell[1412, 60, 140, 3, 104, "Text"],
Cell[1555, 65, 185, 4, 144, "Text"],
Cell[1743, 71, 154, 4, 144, "Text"],
Cell[1900, 77, 95, 2, 64, "Text"],
Cell[1998, 81, 384, 8, 303, "Text"],
Cell[2385, 91, 155, 3, 144, "Text"],
Cell[2543, 96, 131, 3, 104, "Text"],
Cell[2677, 101, 137, 3, 104, "Text"],
Cell[2817, 106, 177, 6, 64, "Text"],
Cell[2997, 114, 149, 6, 64, "Text"],
Cell[3149, 122, 162, 6, 64, "Text"],
Cell[3314, 130, 153, 6, 64, "Text"],
Cell[3470, 138, 153, 6, 64, "Text"],
Cell[3626, 146, 153, 6, 64, "Text"],
Cell[3782, 154, 158, 6, 64, "Text"],
Cell[3943, 162, 156, 3, 104, "Text"],
Cell[4102, 167, 1081, 21, 661, "Text"],
Cell[5186, 190, 147, 3, 104, "Text"],
Cell[5336, 195, 497, 9, 263, "Text"],
Cell[5836, 206, 92, 2, 64, "Text"],
Cell[5931, 210, 174, 3, 144, "Text"],
Cell[6108, 215, 219, 6, 104, "Text"],
Cell[6330, 223, 78, 2, 96, "Input"],
Cell[6411, 227, 180, 4, 104, "Text"],
Cell[6594, 233, 212, 4, 144, "Text"],
Cell[6809, 239, 249, 8, 104, "Text"],
Cell[CellGroupData[{
Cell[7083, 251, 74, 1, 160, "Section"],
Cell[7160, 254, 98, 2, 64, "Text"],
Cell[7261, 258, 91, 2, 64, "Text"],
Cell[7355, 262, 116, 2, 64, "Text"],
Cell[7474, 266, 117, 2, 64, "Text"],
Cell[7594, 270, 104, 2, 64, "Text"],
Cell[7701, 274, 99, 2, 64, "Text"],
Cell[7803, 278, 100, 2, 64, "Text"],
Cell[7906, 282, 101, 2, 64, "Text"],
Cell[8010, 286, 95, 2, 64, "Text"],
Cell[8108, 290, 100, 2, 64, "Text"],
Cell[8211, 294, 91, 2, 64, "Text"],
Cell[8305, 298, 100, 2, 53, "SmallText"],
Cell[8408, 302, 96, 2, 53, "SmallText"],
Cell[8507, 306, 97, 2, 53, "SmallText"],
Cell[8607, 310, 101, 2, 53, "SmallText"],
Cell[8711, 314, 97, 2, 53, "SmallText"],
Cell[8811, 318, 97, 2, 53, "SmallText"],
Cell[8911, 322, 97, 2, 53, "SmallText"],
Cell[9011, 326, 97, 2, 53, "SmallText"],
Cell[9111, 330, 109, 2, 64, "Text"],
Cell[9223, 334, 138, 3, 64, "Text"],
Cell[9364, 339, 136, 3, 64, "Text"],
Cell[9503, 344, 457, 15, 82, "Text"],
Cell[9963, 361, 499, 16, 82, "Text"],
Cell[10465, 379, 102, 2, 64, "Text"],
Cell[10570, 383, 106, 2, 64, "Text"],
Cell[10679, 387, 105, 2, 53, "SmallText"],
Cell[10787, 391, 175, 7, 64, "Text"],
Cell[10965, 400, 96, 2, 53, "SmallText"],
Cell[11064, 404, 101, 2, 64, "Text"],
Cell[11168, 408, 111, 2, 53, "SmallText"],
Cell[11282, 412, 111, 2, 53, "SmallText"],
Cell[11396, 416, 147, 3, 64, "Text"],
Cell[11546, 421, 97, 2, 64, "Text"],
Cell[11646, 425, 99, 2, 64, "Text"],
Cell[11748, 429, 501, 18, 78, "SmallText"],
Cell[12252, 449, 501, 18, 78, "SmallText"],
Cell[12756, 469, 501, 18, 78, "SmallText"],
Cell[13260, 489, 501, 18, 78, "SmallText"],
Cell[13764, 509, 99, 2, 64, "Text"],
Cell[13866, 513, 604, 20, 87, "Text"],
Cell[14473, 535, 99, 2, 64, "Text"],
Cell[14575, 539, 604, 20, 87, "Text"],
Cell[15182, 561, 604, 20, 87, "Text"],
Cell[15789, 583, 604, 20, 87, "Text"],
Cell[16396, 605, 97, 2, 64, "Text"],
Cell[16496, 609, 100, 2, 64, "Text"],
Cell[16599, 613, 98, 2, 64, "Text"],
Cell[16700, 617, 98, 2, 64, "Text"],
Cell[16801, 621, 98, 2, 64, "Text"],
Cell[16902, 625, 109, 2, 64, "Text"],
Cell[17014, 629, 100, 2, 64, "Text"],
Cell[17117, 633, 98, 2, 64, "Text"],
Cell[17218, 637, 100, 2, 64, "Text"],
Cell[17321, 641, 98, 2, 64, "Text"],
Cell[17422, 645, 100, 2, 64, "Text"],
Cell[17525, 649, 100, 2, 64, "Text"],
Cell[17628, 653, 98, 2, 64, "Text"],
Cell[17729, 657, 100, 2, 64, "Text"],
Cell[17832, 661, 98, 2, 64, "Text"],
Cell[17933, 665, 100, 2, 64, "Text"],
Cell[18036, 669, 98, 2, 64, "Text"],
Cell[18137, 673, 100, 2, 64, "Text"],
Cell[18240, 677, 98, 2, 64, "Text"],
Cell[18341, 681, 100, 2, 64, "Text"],
Cell[18444, 685, 98, 2, 64, "Text"],
Cell[18545, 689, 100, 2, 64, "Text"],
Cell[18648, 693, 98, 2, 64, "Text"],
Cell[18749, 697, 100, 2, 64, "Text"],
Cell[18852, 701, 102, 2, 64, "Text"],
Cell[18957, 705, 98, 2, 64, "Text"],
Cell[19058, 709, 100, 2, 64, "Text"],
Cell[19161, 713, 98, 2, 64, "Text"],
Cell[19262, 717, 100, 2, 64, "Text"],
Cell[19365, 721, 102, 2, 64, "Text"],
Cell[19470, 725, 102, 2, 64, "Text"],
Cell[19575, 729, 102, 2, 64, "Text"],
Cell[19680, 733, 98, 2, 64, "Text"],
Cell[19781, 737, 100, 2, 64, "Text"],
Cell[19884, 741, 109, 2, 64, "Text"],
Cell[19996, 745, 98, 2, 64, "Text"],
Cell[20097, 749, 100, 2, 64, "Text"],
Cell[20200, 753, 102, 2, 64, "Text"],
Cell[20305, 757, 98, 2, 64, "Text"],
Cell[20406, 761, 100, 2, 64, "Text"],
Cell[20509, 765, 102, 2, 64, "Text"],
Cell[20614, 769, 98, 2, 64, "Text"],
Cell[20715, 773, 100, 2, 64, "Text"],
Cell[20818, 777, 98, 2, 64, "Text"],
Cell[20919, 781, 100, 2, 64, "Text"],
Cell[21022, 785, 102, 2, 64, "Text"],
Cell[21127, 789, 102, 2, 64, "Text"],
Cell[21232, 793, 98, 2, 64, "Text"],
Cell[21333, 797, 100, 2, 64, "Text"],
Cell[21436, 801, 98, 2, 64, "Text"],
Cell[21537, 805, 100, 2, 64, "Text"],
Cell[21640, 809, 98, 2, 64, "Text"],
Cell[21741, 813, 100, 2, 64, "Text"],
Cell[21844, 817, 102, 2, 64, "Text"],
Cell[21949, 821, 102, 2, 64, "Text"],
Cell[22054, 825, 98, 2, 64, "Text"],
Cell[22155, 829, 100, 2, 64, "Text"],
Cell[22258, 833, 102, 2, 64, "Text"],
Cell[22363, 837, 102, 2, 64, "Text"],
Cell[22468, 841, 98, 2, 64, "Text"],
Cell[22569, 845, 100, 2, 64, "Text"],
Cell[22672, 849, 98, 2, 64, "Text"],
Cell[22773, 853, 99, 2, 53, "SmallText"],
Cell[22875, 857, 99, 2, 53, "SmallText"],
Cell[22977, 861, 100, 2, 64, "Text"],
Cell[23080, 865, 98, 2, 64, "Text"],
Cell[23181, 869, 100, 2, 64, "Text"],
Cell[23284, 873, 98, 2, 64, "Text"],
Cell[23385, 877, 101, 2, 64, "Text"],
Cell[23489, 881, 101, 2, 64, "Text"],
Cell[23593, 885, 101, 2, 64, "Text"],
Cell[23697, 889, 101, 2, 64, "Text"],
Cell[23801, 893, 98, 2, 64, "Text"],
Cell[23902, 897, 98, 2, 64, "Text"],
Cell[24003, 901, 100, 2, 64, "Text"],
Cell[24106, 905, 98, 2, 64, "Text"],
Cell[24207, 909, 100, 2, 64, "Text"],
Cell[24310, 913, 102, 2, 64, "Text"],
Cell[24415, 917, 102, 2, 64, "Text"],
Cell[24520, 921, 98, 2, 64, "Text"],
Cell[24621, 925, 100, 2, 64, "Text"],
Cell[24724, 929, 98, 2, 64, "Text"],
Cell[24825, 933, 100, 2, 64, "Text"],
Cell[24928, 937, 98, 2, 64, "Text"],
Cell[25029, 941, 100, 2, 64, "Text"],
Cell[25132, 945, 98, 2, 64, "Text"],
Cell[25233, 949, 100, 2, 64, "Text"],
Cell[25336, 953, 98, 2, 64, "Text"],
Cell[25437, 957, 100, 2, 64, "Text"],
Cell[25540, 961, 102, 2, 64, "Text"],
Cell[25645, 965, 102, 2, 64, "Text"],
Cell[25750, 969, 99, 2, 64, "Text"],
Cell[25852, 973, 101, 2, 64, "Text"],
Cell[25956, 977, 103, 2, 64, "Text"],
Cell[26062, 981, 103, 2, 64, "Text"],
Cell[26168, 985, 103, 2, 64, "Text"],
Cell[26274, 989, 103, 2, 64, "Text"],
Cell[26380, 993, 103, 2, 64, "Text"],
Cell[26486, 997, 100, 2, 64, "Text"],
Cell[26589, 1001, 102, 2, 64, "Text"],
Cell[26694, 1005, 100, 2, 64, "Text"],
Cell[26797, 1009, 102, 2, 64, "Text"],
Cell[26902, 1013, 100, 2, 64, "Text"],
Cell[27005, 1017, 102, 2, 64, "Text"],
Cell[27110, 1021, 100, 2, 64, "Text"],
Cell[27213, 1025, 102, 2, 64, "Text"],
Cell[27318, 1029, 100, 2, 64, "Text"],
Cell[27421, 1033, 102, 2, 64, "Text"],
Cell[27526, 1037, 102, 2, 64, "Text"],
Cell[27631, 1041, 102, 2, 64, "Text"],
Cell[27736, 1045, 102, 2, 64, "Text"],
Cell[27841, 1049, 102, 2, 64, "Text"],
Cell[27946, 1053, 102, 2, 64, "Text"],
Cell[28051, 1057, 102, 2, 64, "Text"],
Cell[28156, 1061, 101, 2, 64, "Text"],
Cell[28260, 1065, 102, 2, 64, "Text"],
Cell[28365, 1069, 101, 2, 64, "Text"],
Cell[28469, 1073, 99, 2, 64, "Text"],
Cell[28571, 1077, 113, 2, 64, "Text"],
Cell[28687, 1081, 97, 2, 64, "Text"],
Cell[28787, 1085, 91, 2, 64, "Text"],
Cell[28881, 1089, 95, 2, 64, "Text"],
Cell[28979, 1093, 100, 2, 64, "Text"],
Cell[29082, 1097, 97, 2, 64, "Text"],
Cell[29182, 1101, 98, 2, 64, "Text"],
Cell[29283, 1105, 101, 2, 64, "Text"],
Cell[29387, 1109, 89, 2, 64, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29513, 1116, 91, 2, 160, "Section",
 CellTags->"T1.1.1"],
Cell[29607, 1120, 332, 10, 152, "SmallText"],
Cell[29942, 1132, 118, 3, 104, "Text"],
Cell[30063, 1137, 3986, 130, 221, "Text"],
Cell[34052, 1269, 222, 5, 184, "Text"],
Cell[34277, 1276, 80, 2, 96, "Input"],
Cell[34360, 1280, 146, 5, 96, "Input"],
Cell[34509, 1287, 246, 8, 96, "Input"],
Cell[34758, 1297, 80, 2, 96, "Input"],
Cell[34841, 1301, 107, 3, 64, "Text"],
Cell[34951, 1306, 339, 9, 217, "Input"],
Cell[35293, 1317, 215, 6, 86, "SmallText"],
Cell[35511, 1325, 520, 15, 257, "Input"],
Cell[36034, 1342, 109, 3, 96, "Input"],
Cell[36146, 1347, 110, 3, 96, "Input"],
Cell[CellGroupData[{
Cell[36281, 1354, 77, 1, 61, "Subsubsection",
 CellTags->"T1.2.1.1"],
Cell[36361, 1357, 503, 11, 263, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[36913, 1374, 84, 2, 160, "Section",
 CellTags->"T1.2.1"],
Cell[37000, 1378, 26, 0, 96, "Input"],
Cell[37029, 1380, 118, 3, 96, "Input"],
Cell[37150, 1385, 307, 6, 257, "Input"],
Cell[37460, 1393, 681, 11, 497, "Input"],
Cell[38144, 1406, 209, 5, 177, "Input"],
Cell[38356, 1413, 173, 5, 137, "Input"],
Cell[38532, 1420, 187, 5, 177, "Input"],
Cell[38722, 1427, 187, 5, 177, "Input"],
Cell[38912, 1434, 199, 5, 177, "Input"],
Cell[39114, 1441, 176, 5, 137, "Input"],
Cell[39293, 1448, 580, 15, 507, "Input"],
Cell[39876, 1465, 263, 6, 217, "Input"],
Cell[40142, 1473, 279, 6, 217, "Input"],
Cell[40424, 1481, 226, 5, 177, "Input"],
Cell[40653, 1488, 222, 5, 177, "Input"],
Cell[40878, 1495, 247, 6, 217, "Input"],
Cell[41128, 1503, 227, 5, 177, "Input"],
Cell[41358, 1510, 218, 5, 177, "Input"],
Cell[41579, 1517, 201, 5, 177, "Input"],
Cell[41783, 1524, 201, 5, 177, "Input"],
Cell[41987, 1531, 201, 5, 177, "Input"],
Cell[42191, 1538, 201, 5, 177, "Input"],
Cell[42395, 1545, 201, 5, 177, "Input"],
Cell[42599, 1552, 501, 9, 377, "Input"],
Cell[43103, 1563, 242, 5, 177, "Input"],
Cell[43348, 1570, 232, 5, 177, "Input"],
Cell[43583, 1577, 196, 5, 177, "Input"],
Cell[43782, 1584, 300, 6, 257, "Input"],
Cell[44085, 1592, 257, 6, 217, "Input"],
Cell[44345, 1600, 229, 5, 177, "Input"],
Cell[44577, 1607, 92, 2, 96, "Input"],
Cell[44672, 1611, 80, 2, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44789, 1618, 110, 2, 160, "Section",
 CellTags->"T1.3.1"],
Cell[44902, 1622, 999, 36, 108, "Text"],
Cell[45904, 1660, 345, 12, 177, "Input"],
Cell[46252, 1674, 73, 0, 64, "Text"],
Cell[46328, 1676, 85, 2, 96, "Input"],
Cell[46416, 1680, 115, 3, 64, "Text"],
Cell[46534, 1685, 82, 2, 96, "Input"],
Cell[46619, 1689, 719, 28, 104, "Text"],
Cell[CellGroupData[{
Cell[47363, 1721, 62, 1, 83, "Subsection",
 CellTags->"T1.3.1.1"],
Cell[47428, 1724, 106, 3, 96, "Input"],
Cell[47537, 1729, 632, 16, 217, "Input"],
Cell[48172, 1747, 660, 18, 217, "Input"],
Cell[48835, 1767, 418, 11, 177, "Input"],
Cell[49256, 1780, 115, 3, 64, "Text"],
Cell[49374, 1785, 551, 15, 177, "Input"],
Cell[49928, 1802, 425, 12, 137, "Input"],
Cell[50356, 1816, 514, 15, 137, "Input"],
Cell[50873, 1833, 570, 16, 137, "Input"],
Cell[51446, 1851, 330, 10, 137, "Input"],
Cell[51779, 1863, 412, 12, 137, "Input"],
Cell[52194, 1877, 466, 13, 137, "Input"],
Cell[52663, 1892, 454, 13, 137, "Input"],
Cell[53120, 1907, 513, 13, 302, "Input"],
Cell[53636, 1922, 271, 8, 96, "Input"],
Cell[53910, 1932, 271, 8, 96, "Input"],
Cell[54184, 1942, 271, 8, 96, "Input"],
Cell[54458, 1952, 271, 8, 96, "Input"],
Cell[54732, 1962, 271, 8, 96, "Input"],
Cell[55006, 1972, 174, 5, 96, "Input"],
Cell[55183, 1979, 227, 7, 96, "Input"],
Cell[55413, 1988, 340, 10, 96, "Input"],
Cell[55756, 2000, 630, 17, 177, "Input"],
Cell[56389, 2019, 420, 12, 137, "Input"],
Cell[56812, 2033, 513, 15, 177, "Input"],
Cell[57328, 2050, 340, 10, 96, "Input"],
Cell[57671, 2062, 420, 12, 137, "Input"],
Cell[58094, 2076, 630, 17, 177, "Input"],
Cell[58727, 2095, 1573, 42, 377, "Input"],
Cell[60303, 2139, 1836, 47, 417, "Input"],
Cell[62142, 2188, 1695, 48, 417, "Input"],
Cell[63840, 2238, 1940, 53, 377, "Input"],
Cell[65783, 2293, 629, 19, 96, "Input"],
Cell[66415, 2314, 1876, 50, 377, "Input"],
Cell[68294, 2366, 1709, 47, 337, "Input"],
Cell[70006, 2415, 1130, 32, 297, "Input"],
Cell[71139, 2449, 1709, 47, 337, "Input"],
Cell[72851, 2498, 573, 19, 96, "Input"],
Cell[73427, 2519, 1130, 32, 297, "Input"],
Cell[74560, 2553, 1454, 39, 377, "Input"],
Cell[76017, 2594, 511, 18, 96, "Input"],
Cell[76531, 2614, 1443, 39, 377, "Input"],
Cell[77977, 2655, 403, 13, 96, "Input"],
Cell[78383, 2670, 107, 3, 104, "Text"],
Cell[78493, 2675, 1723, 45, 377, "Input"],
Cell[80219, 2722, 1723, 45, 377, "Input"],
Cell[81945, 2769, 1698, 48, 417, "Input"],
Cell[83646, 2819, 1945, 53, 377, "Input"],
Cell[85594, 2874, 1881, 50, 417, "Input"],
Cell[87478, 2926, 1714, 47, 337, "Input"],
Cell[89195, 2975, 1132, 32, 297, "Input"],
Cell[90330, 3009, 1457, 39, 377, "Input"],
Cell[91790, 3050, 1446, 39, 377, "Input"],
Cell[93239, 3091, 1720, 45, 377, "Input"],
Cell[94962, 3138, 1720, 45, 377, "Input"],
Cell[96685, 3185, 1695, 48, 417, "Input"],
Cell[98383, 3235, 1940, 53, 377, "Input"],
Cell[100326, 3290, 1876, 50, 377, "Input"],
Cell[102205, 3342, 1709, 47, 337, "Input"],
Cell[103917, 3391, 1130, 32, 297, "Input"],
Cell[105050, 3425, 1454, 39, 377, "Input"],
Cell[106507, 3466, 1443, 39, 377, "Input"],
Cell[107953, 3507, 234, 6, 96, "Input"],
Cell[108190, 3515, 234, 6, 96, "Input"],
Cell[108427, 3523, 234, 6, 96, "Input"],
Cell[108664, 3531, 234, 6, 96, "Input"],
Cell[108901, 3539, 234, 6, 96, "Input"],
Cell[109138, 3547, 314, 10, 102, "Input"],
Cell[109455, 3559, 266, 8, 96, "Input"],
Cell[109724, 3569, 415, 11, 177, "Input"],
Cell[110142, 3582, 257, 7, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110436, 3594, 57, 1, 83, "Subsection",
 CellTags->"T1.3.2.1"],
Cell[110496, 3597, 14173, 339, 2087, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[124706, 3941, 58, 1, 83, "Subsection",
 CellTags->"T1.3.3.1"],
Cell[124767, 3944, 2105, 57, 363, "Input"],
Cell[126875, 4003, 2105, 57, 363, "Input"],
Cell[128983, 4062, 2105, 57, 363, "Input"],
Cell[131091, 4121, 2105, 57, 363, "Input"],
Cell[133199, 4180, 2105, 57, 363, "Input"],
Cell[135307, 4239, 2105, 57, 363, "Input"],
Cell[137415, 4298, 2105, 57, 363, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[139557, 4360, 59, 1, 83, "Subsection",
 CellTags->"T1.3.4.1"],
Cell[139619, 4363, 2266, 68, 337, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[141922, 4436, 53, 1, 83, "Subsection",
 CellTags->"T1.3.5.1"],
Cell[141978, 4439, 129, 3, 131, "Input"],
Cell[142110, 4444, 8160, 199, 1097, "Input"],
Cell[150273, 4645, 512, 14, 177, "Input"],
Cell[150788, 4661, 662, 25, 68, "Text"],
Cell[151453, 4688, 7327, 197, 1082, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158817, 4890, 58, 1, 83, "Subsection",
 CellTags->"T1.3.6.1"],
Cell[158878, 4893, 1699, 48, 337, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160614, 4946, 49, 1, 83, "Subsection",
 CellTags->"T1.3.7.1"],
Cell[160666, 4949, 202, 5, 96, "Input"],
Cell[CellGroupData[{
Cell[160893, 4958, 56, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.1.1"],
Cell[160952, 4961, 738, 28, 76, "Text"],
Cell[161693, 4991, 454, 12, 177, "Input"],
Cell[162150, 5005, 342, 10, 137, "Input"],
Cell[162495, 5017, 344, 10, 137, "Input"],
Cell[162842, 5029, 348, 10, 137, "Input"],
Cell[163193, 5041, 338, 10, 137, "Input"],
Cell[163534, 5053, 346, 10, 137, "Input"],
Cell[163883, 5065, 336, 10, 137, "Input"],
Cell[164222, 5077, 336, 10, 137, "Input"],
Cell[164561, 5089, 344, 10, 137, "Input"],
Cell[164908, 5101, 340, 10, 137, "Input"],
Cell[165251, 5113, 310, 8, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[165598, 5126, 52, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.2.1"],
Cell[165653, 5129, 153, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[165843, 5139, 53, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.3.1"],
Cell[165899, 5142, 101, 3, 96, "Input"],
Cell[166003, 5147, 154, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166194, 5157, 57, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.4.1"],
Cell[166254, 5160, 177, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166468, 5170, 53, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.5.1"],
Cell[166524, 5173, 3526, 79, 2734, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[170087, 5257, 53, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.6.1"],
Cell[170143, 5260, 1562, 49, 257, "Input"],
Cell[171708, 5311, 311, 10, 96, "Input"],
Cell[172022, 5323, 341, 11, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[172400, 5339, 53, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.7.1"],
Cell[172456, 5342, 1358, 45, 217, "Input"],
Cell[173817, 5389, 276, 9, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[174130, 5403, 53, 1, 61, "Subsubsection",
 CellTags->"T1.3.7.8.1"],
Cell[174186, 5406, 283, 10, 96, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[174530, 5423, 102, 2, 160, "Section",
 CellTags->"T1.4.1"],
Cell[174635, 5427, 810, 25, 230, "Input"],
Cell[175448, 5454, 2976, 89, 517, "Input"],
Cell[178427, 5545, 182, 6, 67, "Text"],
Cell[178612, 5553, 435, 13, 99, "Input"],
Cell[179050, 5568, 95, 2, 98, "Input"],
Cell[CellGroupData[{
Cell[179170, 5574, 108, 3, 85, "Subsection",
 CellTags->"T1.4.1.1"],
Cell[179281, 5579, 224, 7, 96, "Input"],
Cell[179508, 5588, 128, 3, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179673, 5596, 106, 3, 85, "Subsection",
 CellTags->"T1.4.2.1"],
Cell[179782, 5601, 147, 7, 81, "Text"],
Cell[179932, 5610, 1837, 63, 190, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181806, 5678, 196, 6, 85, "Subsection",
 CellTags->"T1.4.3.1"],
Cell[182005, 5686, 2244, 71, 283, "Input"],
Cell[184252, 5759, 179, 5, 98, "Input"],
Cell[184434, 5766, 2260, 68, 236, "Input"],
Cell[186697, 5836, 1307, 40, 189, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188041, 5881, 233, 7, 119, "Subsection",
 CellTags->"T1.4.4.1"],
Cell[188277, 5890, 2949, 92, 373, "Input"],
Cell[191229, 5984, 2791, 91, 355, "Input"],
Cell[194023, 6077, 2791, 91, 355, "Input"],
Cell[196817, 6170, 2791, 91, 355, "Input"],
Cell[199611, 6263, 179, 5, 99, "Input"],
Cell[199793, 6270, 179, 5, 98, "Input"],
Cell[199975, 6277, 165, 5, 99, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[200189, 6288, 95, 2, 157, "Section",
 CellTags->"T1.5.1"],
Cell[CellGroupData[{
Cell[200309, 6294, 64, 1, 157, "Subsection",
 CellTags->"T1.5.1.1"],
Cell[200376, 6297, 131, 3, 157, "Text"],
Cell[200510, 6302, 67, 0, 157, "Text"],
Cell[200580, 6304, 226, 8, 69, "Text"],
Cell[200809, 6314, 3442, 89, 472, "Input"],
Cell[204254, 6405, 3420, 86, 388, "Input"],
Cell[207677, 6493, 275, 8, 96, "Input"],
Cell[207955, 6503, 249, 7, 96, "Input"],
Cell[208207, 6512, 310, 9, 96, "Input"],
Cell[208520, 6523, 283, 8, 96, "Input"],
Cell[208806, 6533, 249, 7, 96, "Input"],
Cell[209058, 6542, 325, 9, 137, "Input"],
Cell[209386, 6553, 182, 6, 115, "Input"],
Cell[209571, 6561, 59, 0, 96, "Input"],
Cell[209633, 6563, 233, 7, 96, "Input"],
Cell[209869, 6572, 233, 7, 96, "Input"],
Cell[210105, 6581, 234, 7, 96, "Input"],
Cell[210342, 6590, 261, 7, 96, "Input"],
Cell[210606, 6599, 235, 7, 96, "Input"],
Cell[210844, 6608, 235, 7, 96, "Input"],
Cell[211082, 6617, 2167, 65, 377, "Input"],
Cell[213252, 6684, 81, 2, 96, "Input"],
Cell[213336, 6688, 610, 18, 177, "Input"],
Cell[213949, 6708, 186, 5, 96, "Input"],
Cell[214138, 6715, 76, 0, 64, "Text"],
Cell[214217, 6717, 173, 6, 96, "Input"],
Cell[214393, 6725, 134, 4, 96, "Input"],
Cell[214530, 6731, 9844, 243, 1599, "Input"],
Cell[224377, 6976, 9528, 235, 1559, "Input"],
Cell[233908, 7213, 12850, 315, 1937, "Input"],
Cell[246761, 7530, 11337, 282, 1617, "Input"],
Cell[258101, 7814, 12420, 304, 2017, "Input"],
Cell[270524, 8120, 119, 3, 96, "Input"],
Cell[270646, 8125, 1389, 43, 217, "Input"],
Cell[272038, 8170, 1014, 31, 177, "Input"],
Cell[273055, 8203, 1014, 31, 177, "Input"],
Cell[CellGroupData[{
Cell[274094, 8238, 61, 1, 61, "Subsubsection",
 CellTags->"T1.5.1.1.1"],
Cell[274158, 8241, 4767, 148, 628, "Input"],
Cell[278928, 8391, 370, 11, 96, "Input"],
Cell[279301, 8404, 355, 11, 96, "Input"],
Cell[279659, 8417, 420, 14, 137, "Input"],
Cell[280082, 8433, 496, 13, 99, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[280627, 8452, 58, 1, 83, "Subsection",
 CellTags->"T1.5.2.1"],
Cell[280688, 8455, 4291, 127, 684, "Input"],
Cell[CellGroupData[{
Cell[285004, 8586, 52, 1, 61, "Subsubsection",
 CellTags->"T1.5.2.1.1"],
Cell[285059, 8589, 367, 11, 137, "Input"],
Cell[285429, 8602, 393, 12, 137, "Input"],
Cell[285825, 8616, 330, 11, 96, "Input"],
Cell[286158, 8629, 239, 10, 157, "Text"],
Cell[286400, 8641, 20, 0, 64, "Text"],
Cell[286423, 8643, 3284, 95, 331, "Input"],
Cell[289710, 8740, 20, 0, 64, "Text"],
Cell[289733, 8742, 2974, 87, 331, "Input"],
Cell[292710, 8831, 20, 0, 64, "Text"],
Cell[292733, 8833, 1741, 53, 238, "Input"],
Cell[294477, 8888, 20, 0, 64, "Text"],
Cell[294500, 8890, 1475, 44, 445, "Input"],
Cell[295978, 8936, 20, 0, 64, "Text"],
Cell[296001, 8938, 2177, 62, 424, "Input"],
Cell[298181, 9002, 20, 0, 64, "Text"],
Cell[298204, 9004, 2161, 61, 317, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[300414, 9071, 59, 1, 157, "Subsection",
 CellTags->"T1.5.3.1"],
Cell[300476, 9074, 174, 3, 157, "Text"],
Cell[300653, 9079, 139, 3, 157, "Text"],
Cell[CellGroupData[{
Cell[300817, 9086, 67, 1, 61, "Subsubsection",
 CellTags->"T1.5.3.1.1"],
Cell[300887, 9089, 1210, 40, 242, "Input"],
Cell[302100, 9131, 2525, 76, 672, "Input"],
Cell[304628, 9209, 6488, 160, 1099, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311153, 9374, 67, 1, 61, "Subsubsection",
 CellTags->"T1.5.3.2.1"],
Cell[311223, 9377, 1018, 34, 242, "Input"],
Cell[312244, 9413, 854, 26, 137, "Input"],
Cell[313101, 9441, 92, 2, 96, "Input"],
Cell[313196, 9445, 6243, 155, 1119, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[319500, 9607, 138, 2, 220, "Section",
 CellTags->"T1.6.1"],
Cell[319641, 9611, 793, 26, 137, "Input"],
Cell[320437, 9639, 298, 9, 157, "Text"],
Cell[CellGroupData[{
Cell[320760, 9652, 55, 1, 157, "Subsection",
 CellTags->"T1.6.1.1"],
Cell[320818, 9655, 92, 4, 64, "Text"],
Cell[320913, 9661, 6571, 217, 274, "Input"],
Cell[327487, 9880, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[327661, 9889, 57, 1, 83, "Subsection",
 CellTags->"T1.6.2.1"],
Cell[327721, 9892, 255, 9, 96, "Input"],
Cell[CellGroupData[{
Cell[328001, 9905, 167, 7, 61, "Subsubsection",
 CellTags->"T1.6.2.1.1"],
Cell[328171, 9914, 1063, 37, 187, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329271, 9956, 167, 7, 61, "Subsubsection",
 CellTags->"T1.6.2.2.1"],
Cell[329441, 9965, 1232, 42, 187, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330710, 10012, 167, 7, 61, "Subsubsection",
 CellTags->"T1.6.2.3.1"],
Cell[330880, 10021, 1232, 42, 187, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332149, 10068, 167, 7, 61, "Subsubsection",
 CellTags->"T1.6.2.4.1"],
Cell[332319, 10077, 1232, 42, 187, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[333600, 10125, 57, 1, 83, "Subsection",
 CellTags->"T1.6.3.1"],
Cell[333660, 10128, 162, 4, 96, "Input"],
Cell[333825, 10134, 377, 12, 66, "Text"],
Cell[334205, 10148, 3094, 113, 151, "Input"],
Cell[337302, 10263, 141, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[337480, 10272, 246, 9, 90, "Subsection",
 CellTags->"T1.6.4.1"],
Cell[337729, 10283, 99, 3, 96, "Input"],
Cell[337831, 10288, 539, 17, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[338407, 10310, 57, 1, 83, "Subsection",
 CellTags->"T1.6.5.1"],
Cell[338467, 10313, 162, 4, 96, "Input"],
Cell[338632, 10319, 377, 12, 66, "Text"],
Cell[339012, 10333, 3094, 113, 151, "Input"],
Cell[342109, 10448, 141, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[342287, 10457, 246, 9, 90, "Subsection",
 CellTags->"T1.6.6.1"],
Cell[342536, 10468, 99, 3, 96, "Input"],
Cell[342638, 10473, 539, 17, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[343214, 10495, 246, 9, 90, "Subsection",
 CellTags->"T1.6.7.1"],
Cell[343463, 10506, 130, 3, 96, "Input"],
Cell[343596, 10511, 260, 7, 96, "Input"],
Cell[343859, 10520, 539, 17, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[344435, 10542, 246, 9, 90, "Subsection",
 CellTags->"T1.6.8.1"],
Cell[344684, 10553, 130, 3, 96, "Input"],
Cell[344817, 10558, 260, 7, 96, "Input"],
Cell[345080, 10567, 539, 17, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[345656, 10589, 55, 1, 157, "Subsection",
 CellTags->"T1.6.9.1"],
Cell[345714, 10592, 92, 4, 64, "Text"],
Cell[345809, 10598, 6063, 199, 274, "Input"],
Cell[351875, 10799, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[352049, 10808, 58, 1, 83, "Subsection",
 CellTags->"T1.6.10.1"],
Cell[352110, 10811, 232, 8, 96, "Input"],
Cell[352345, 10821, 1059, 37, 187, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[353441, 10863, 56, 1, 157, "Subsection",
 CellTags->"T1.6.11.1"],
Cell[353500, 10866, 624, 26, 66, "Text"],
Cell[354127, 10894, 3309, 120, 151, "Input"],
Cell[357439, 11016, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357613, 11025, 56, 1, 157, "Subsection",
 CellTags->"T1.6.12.1"],
Cell[357672, 11028, 624, 26, 66, "Text"],
Cell[358299, 11056, 3309, 120, 151, "Input"],
Cell[361611, 11178, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361785, 11187, 56, 1, 83, "Subsection",
 CellTags->"T1.6.13.1"],
Cell[361844, 11190, 6446, 215, 239, "Input"],
Cell[368293, 11407, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[368467, 11416, 67, 1, 83, "Subsection",
 CellTags->"T1.6.14.1"],
Cell[368537, 11419, 662, 21, 137, "Input"],
Cell[369202, 11442, 326, 10, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[369565, 11457, 58, 1, 83, "Subsection",
 CellTags->"T1.6.15.1"],
Cell[369626, 11460, 22654, 536, 18352, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[392317, 12001, 56, 1, 157, "Subsection",
 CellTags->"T1.6.16.1"],
Cell[392376, 12004, 92, 4, 64, "Text"],
Cell[392471, 12010, 3736, 132, 198, "Input"],
Cell[396210, 12144, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[396384, 12153, 58, 1, 83, "Subsection",
 CellTags->"T1.6.17.1"],
Cell[396445, 12156, 49, 1, 96, "Input"],
Cell[396497, 12159, 232, 8, 96, "Input"],
Cell[396732, 12169, 514, 17, 98, "Input"],
Cell[CellGroupData[{
Cell[397271, 12190, 93, 2, 96, "Input"],
Cell[397367, 12194, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[397453, 12200, 56, 1, 157, "Subsection",
 CellTags->"T1.6.18.1"],
Cell[397512, 12203, 92, 4, 64, "Text"],
Cell[397607, 12209, 5243, 183, 245, "Input"],
Cell[402853, 12394, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403027, 12403, 58, 1, 83, "Subsection",
 CellTags->"T1.6.19.1"],
Cell[403088, 12406, 274, 9, 96, "Input"],
Cell[403365, 12417, 514, 17, 98, "Input"],
Cell[CellGroupData[{
Cell[403904, 12438, 93, 2, 96, "Input"],
Cell[404000, 12442, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[404086, 12448, 58, 1, 83, "Subsection",
 CellTags->"T1.6.20.1"],
Cell[404147, 12451, 340, 11, 96, "Input"],
Cell[404490, 12464, 514, 17, 98, "Input"],
Cell[CellGroupData[{
Cell[405029, 12485, 93, 2, 96, "Input"],
Cell[405125, 12489, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[405211, 12495, 56, 1, 157, "Subsection",
 CellTags->"T1.6.21.1"],
Cell[405270, 12498, 92, 4, 64, "Text"],
Cell[405365, 12504, 2961, 107, 151, "Input"],
Cell[408329, 12613, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[408503, 12622, 58, 1, 83, "Subsection",
 CellTags->"T1.6.22.1"],
Cell[408564, 12625, 232, 8, 96, "Input"],
Cell[408799, 12635, 441, 14, 96, "Input"],
Cell[CellGroupData[{
Cell[409265, 12653, 93, 2, 96, "Input"],
Cell[409361, 12657, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[409447, 12663, 56, 1, 157, "Subsection",
 CellTags->"T1.6.23.1"],
Cell[409506, 12666, 92, 4, 64, "Text"],
Cell[409601, 12672, 3700, 132, 151, "Input"],
Cell[413304, 12806, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[413478, 12815, 58, 1, 83, "Subsection",
 CellTags->"T1.6.24.1"],
Cell[413539, 12818, 232, 8, 96, "Input"],
Cell[413774, 12828, 515, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[414314, 12848, 93, 2, 96, "Input"],
Cell[414410, 12852, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[414496, 12858, 56, 1, 157, "Subsection",
 CellTags->"T1.6.25.1"],
Cell[414555, 12861, 92, 4, 64, "Text"],
Cell[414650, 12867, 3457, 123, 151, "Input"],
Cell[418110, 12992, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[418284, 13001, 58, 1, 83, "Subsection",
 CellTags->"T1.6.26.1"],
Cell[418345, 13004, 232, 8, 96, "Input"],
Cell[418580, 13014, 517, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[419122, 13034, 93, 2, 96, "Input"],
Cell[419218, 13038, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[419304, 13044, 56, 1, 157, "Subsection",
 CellTags->"T1.6.27.1"],
Cell[419363, 13047, 230, 9, 64, "Text"],
Cell[419596, 13058, 5729, 194, 245, "Input"],
Cell[425328, 13254, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[425502, 13263, 58, 1, 83, "Subsection",
 CellTags->"T1.6.28.1"],
Cell[425563, 13266, 95, 3, 96, "Input"],
Cell[425661, 13271, 1867, 49, 331, "Input"],
Cell[427531, 13322, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[427618, 13327, 57, 1, 157, "Subsection",
 CellTags->"T1.6.29.1"],
Cell[427678, 13330, 92, 4, 64, "Text"],
Cell[427773, 13336, 2961, 107, 151, "Input"],
Cell[430737, 13445, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[430911, 13454, 58, 1, 83, "Subsection",
 CellTags->"T1.6.30.1"],
Cell[430972, 13457, 232, 8, 96, "Input"],
Cell[431207, 13467, 448, 14, 98, "Input"],
Cell[CellGroupData[{
Cell[431680, 13485, 105, 2, 98, "Input"],
Cell[431788, 13489, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[431874, 13495, 57, 1, 157, "Subsection",
 CellTags->"T1.6.31.1"],
Cell[431934, 13498, 382, 16, 64, "Text"],
Cell[432319, 13516, 2278, 86, 103, "Input"],
Cell[434600, 13604, 137, 4, 96, "Input"],
Cell[434740, 13610, 2028, 76, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[436805, 13691, 58, 1, 157, "Subsection",
 CellTags->"T1.6.32.1"],
Cell[436866, 13694, 381, 16, 64, "Text"],
Cell[437250, 13712, 95, 3, 96, "Input"],
Cell[437348, 13717, 117, 3, 157, "SmallText"],
Cell[437468, 13722, 725, 26, 51, "SmallText"],
Cell[438196, 13750, 602, 22, 96, "Input"],
Cell[438801, 13774, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[438888, 13779, 60, 1, 157, "Subsection",
 CellTags->"T1.6.33.1"],
Cell[438951, 13782, 381, 16, 64, "Text"],
Cell[439335, 13800, 202, 6, 96, "Input"],
Cell[439540, 13808, 117, 3, 157, "SmallText"],
Cell[439660, 13813, 731, 26, 51, "SmallText"],
Cell[440394, 13841, 608, 22, 96, "Input"],
Cell[441005, 13865, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441092, 13870, 57, 1, 157, "Subsection",
 CellTags->"T1.6.34.1"],
Cell[441152, 13873, 512, 21, 104, "Text"],
Cell[441667, 13896, 4989, 173, 245, "Input"],
Cell[446659, 14071, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446833, 14080, 58, 1, 83, "Subsection",
 CellTags->"T1.6.35.1"],
Cell[446894, 14083, 274, 9, 96, "Input"],
Cell[447171, 14094, 1136, 39, 149, "Input"],
Cell[448310, 14135, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448397, 14140, 57, 1, 157, "Subsection",
 CellTags->"T1.6.36.1"],
Cell[448457, 14143, 277, 10, 104, "Text"],
Cell[448737, 14155, 6180, 202, 198, "Input"],
Cell[454920, 14359, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[455094, 14368, 58, 1, 157, "Subsection",
 CellTags->"T1.6.37.1"],
Cell[455155, 14371, 246, 10, 64, "Text"],
Cell[455404, 14383, 136, 4, 96, "Input"],
Cell[455543, 14389, 782, 25, 137, "Input"],
Cell[CellGroupData[{
Cell[456350, 14418, 93, 2, 96, "Input"],
Cell[456446, 14422, 37, 0, 46, "Print"]
}, Open  ]],
Cell[456498, 14425, 880, 28, 177, "Input"],
Cell[457381, 14455, 106, 5, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[457524, 14465, 60, 1, 83, "Subsection",
 CellTags->"T1.6.38.1"],
Cell[457587, 14468, 263, 8, 98, "Input"],
Cell[457853, 14478, 596, 21, 96, "Input"],
Cell[458452, 14501, 80, 2, 96, "Input"],
Cell[458535, 14505, 172, 5, 96, "Input"],
Cell[458710, 14512, 126, 4, 96, "Input"],
Cell[458839, 14518, 479, 17, 96, "Input"],
Cell[459321, 14537, 1204, 37, 149, "Input"],
Cell[460528, 14576, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[460615, 14581, 60, 1, 83, "Subsection",
 CellTags->"T1.6.39.1"],
Cell[460678, 14584, 263, 8, 98, "Input"],
Cell[460944, 14594, 141, 4, 96, "Input"],
Cell[461088, 14600, 126, 4, 96, "Input"],
Cell[461217, 14606, 479, 17, 96, "Input"],
Cell[461699, 14625, 1128, 33, 149, "Input"],
Cell[462830, 14660, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[462917, 14665, 60, 1, 83, "Subsection",
 CellTags->"T1.6.40.1"],
Cell[462980, 14668, 447, 15, 98, "Input"],
Cell[463430, 14685, 479, 17, 96, "Input"],
Cell[463912, 14704, 576, 18, 98, "Input"],
Cell[464491, 14724, 50, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[464578, 14729, 57, 1, 157, "Subsection",
 CellTags->"T1.6.41.1"],
Cell[464638, 14732, 92, 4, 64, "Text"],
Cell[464733, 14738, 4921, 172, 198, "Input"],
Cell[469657, 14912, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[469831, 14921, 58, 1, 83, "Subsection",
 CellTags->"T1.6.42.1"],
Cell[469892, 14924, 588, 20, 96, "Input"],
Cell[470483, 14946, 728, 23, 177, "Input"],
Cell[CellGroupData[{
Cell[471236, 14973, 93, 2, 96, "Input"],
Cell[471332, 14977, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[471418, 14983, 67, 1, 157, "Subsection",
 CellTags->"T1.6.43.1"],
Cell[471488, 14986, 111, 3, 157, "Text"],
Cell[471602, 14991, 92, 4, 64, "Text"],
Cell[471697, 14997, 5657, 200, 245, "Input"],
Cell[477357, 15199, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[477531, 15208, 56, 1, 157, "Subsection",
 CellTags->"T1.6.44.1"],
Cell[477590, 15211, 92, 4, 64, "Text"],
Cell[477685, 15217, 5038, 179, 198, "Input"],
Cell[482726, 15398, 8601, 306, 181, "Text"],
Cell[491330, 15706, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[491504, 15715, 58, 1, 83, "Subsection",
 CellTags->"T1.6.45.1"],
Cell[491565, 15718, 463, 16, 96, "Input"],
Cell[492031, 15736, 659, 21, 137, "Input"],
Cell[CellGroupData[{
Cell[492715, 15761, 93, 2, 96, "Input"],
Cell[492811, 15765, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[492897, 15771, 60, 1, 83, "Subsection",
 CellTags->"T1.6.46.1"],
Cell[492960, 15774, 573, 19, 96, "Input"],
Cell[493536, 15795, 665, 21, 137, "Input"],
Cell[CellGroupData[{
Cell[494226, 15820, 95, 2, 96, "Input"],
Cell[494324, 15824, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[494410, 15830, 56, 1, 157, "Subsection",
 CellTags->"T1.6.47.1"],
Cell[494469, 15833, 92, 4, 64, "Text"],
Cell[494564, 15839, 6060, 203, 245, "Input"],
Cell[500627, 16044, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500801, 16053, 58, 1, 83, "Subsection",
 CellTags->"T1.6.48.1"],
Cell[500862, 16056, 463, 16, 96, "Input"],
Cell[501328, 16074, 700, 22, 177, "Input"],
Cell[CellGroupData[{
Cell[502053, 16100, 93, 2, 96, "Input"],
Cell[502149, 16104, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[502235, 16110, 60, 1, 83, "Subsection",
 CellTags->"T1.6.49.1"],
Cell[502298, 16113, 573, 19, 96, "Input"],
Cell[502874, 16134, 725, 23, 177, "Input"],
Cell[CellGroupData[{
Cell[503624, 16161, 95, 2, 96, "Input"],
Cell[503722, 16165, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[503808, 16171, 56, 1, 157, "Subsection",
 CellTags->"T1.6.50.1"],
Cell[503867, 16174, 92, 4, 64, "Text"],
Cell[503962, 16180, 11843, 400, 649, "Input"],
Cell[515808, 16582, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[515982, 16591, 58, 1, 83, "Subsection",
 CellTags->"T1.6.51.1"],
Cell[516043, 16594, 95, 3, 96, "Input"],
Cell[516141, 16599, 591, 19, 137, "Input"],
Cell[516735, 16620, 104, 3, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[516876, 16628, 56, 1, 157, "Subsection",
 CellTags->"T1.6.52.1"],
Cell[516935, 16631, 258, 11, 64, "Text"],
Cell[517196, 16644, 5262, 182, 313, "Input"],
Cell[522461, 16828, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[522635, 16837, 58, 1, 83, "Subsection",
 CellTags->"T1.6.53.1"],
Cell[522696, 16840, 407, 14, 96, "Input"],
Cell[523106, 16856, 461, 15, 98, "Input"],
Cell[CellGroupData[{
Cell[523592, 16875, 93, 2, 96, "Input"],
Cell[523688, 16879, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[523774, 16885, 60, 1, 83, "Subsection",
 CellTags->"T1.6.54.1"],
Cell[523837, 16888, 477, 16, 96, "Input"],
Cell[524317, 16906, 467, 15, 98, "Input"],
Cell[CellGroupData[{
Cell[524809, 16925, 95, 2, 96, "Input"],
Cell[524907, 16929, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[524993, 16935, 60, 1, 83, "Subsection",
 CellTags->"T1.6.55.1"],
Cell[525056, 16938, 477, 16, 96, "Input"],
Cell[525536, 16956, 467, 15, 98, "Input"],
Cell[CellGroupData[{
Cell[526028, 16975, 95, 2, 96, "Input"],
Cell[526126, 16979, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[526212, 16985, 56, 1, 157, "Subsection",
 CellTags->"T1.6.56.1"],
Cell[526271, 16988, 204, 9, 64, "Text"],
Cell[526478, 16999, 6020, 205, 286, "Input"],
Cell[532501, 17206, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[532675, 17215, 58, 1, 83, "Subsection",
 CellTags->"T1.6.57.1"],
Cell[532736, 17218, 269, 9, 96, "Input"],
Cell[533008, 17229, 453, 15, 98, "Input"],
Cell[CellGroupData[{
Cell[533486, 17248, 93, 2, 96, "Input"],
Cell[533582, 17252, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[533668, 17258, 56, 1, 157, "Subsection",
 CellTags->"T1.6.58.1"],
Cell[533727, 17261, 216, 9, 64, "Text"],
Cell[533946, 17272, 5082, 166, 285, "Input"],
Cell[539031, 17440, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[539205, 17449, 58, 1, 83, "Subsection",
 CellTags->"T1.6.59.1"],
Cell[539266, 17452, 274, 9, 96, "Input"],
Cell[539543, 17463, 439, 13, 98, "Input"],
Cell[CellGroupData[{
Cell[540007, 17480, 93, 2, 96, "Input"],
Cell[540103, 17484, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[540189, 17490, 56, 1, 157, "Subsection",
 CellTags->"T1.6.60.1"],
Cell[540248, 17493, 257, 11, 64, "Text"],
Cell[540508, 17506, 5303, 181, 245, "Input"],
Cell[545814, 17689, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[545988, 17698, 58, 1, 83, "Subsection",
 CellTags->"T1.6.61.1"],
Cell[546049, 17701, 493, 17, 96, "Input"],
Cell[546545, 17720, 551, 17, 137, "Input"],
Cell[CellGroupData[{
Cell[547121, 17741, 93, 2, 96, "Input"],
Cell[547217, 17745, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[547303, 17751, 60, 1, 83, "Subsection",
 CellTags->"T1.6.62.1"],
Cell[547366, 17754, 565, 19, 96, "Input"],
Cell[547934, 17775, 557, 17, 137, "Input"],
Cell[CellGroupData[{
Cell[548516, 17796, 95, 2, 96, "Input"],
Cell[548614, 17800, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[548700, 17806, 60, 1, 83, "Subsection",
 CellTags->"T1.6.63.1"],
Cell[548763, 17809, 580, 19, 96, "Input"],
Cell[549346, 17830, 557, 17, 137, "Input"],
Cell[CellGroupData[{
Cell[549928, 17851, 95, 2, 96, "Input"],
Cell[550026, 17855, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[550112, 17861, 56, 1, 157, "Subsection",
 CellTags->"T1.6.64.1"],
Cell[550171, 17864, 203, 9, 64, "Text"],
Cell[550377, 17875, 12433, 402, 716, "Input"],
Cell[562813, 18279, 137, 4, 96, "Input"],
Cell[562953, 18285, 124, 4, 96, "Input"],
Cell[563080, 18291, 511, 18, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[563628, 18314, 58, 1, 83, "Subsection",
 CellTags->"T1.6.65.1"],
Cell[563689, 18317, 350, 12, 96, "Input"],
Cell[564042, 18331, 544, 17, 177, "Input"],
Cell[CellGroupData[{
Cell[564611, 18352, 93, 2, 96, "Input"],
Cell[564707, 18356, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[564793, 18362, 60, 1, 83, "Subsection",
 CellTags->"T1.6.66.1"],
Cell[564856, 18365, 420, 14, 96, "Input"],
Cell[565279, 18381, 548, 17, 177, "Input"],
Cell[CellGroupData[{
Cell[565852, 18402, 95, 2, 96, "Input"],
Cell[565950, 18406, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[566036, 18412, 60, 1, 83, "Subsection",
 CellTags->"T1.6.67.1"],
Cell[566099, 18415, 420, 14, 96, "Input"],
Cell[566522, 18431, 550, 17, 177, "Input"],
Cell[CellGroupData[{
Cell[567097, 18452, 95, 2, 96, "Input"],
Cell[567195, 18456, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[567281, 18462, 56, 1, 157, "Subsection",
 CellTags->"T1.6.68.1"],
Cell[567340, 18465, 216, 9, 64, "Text"],
Cell[567559, 18476, 7673, 251, 390, "Input"],
Cell[575235, 18729, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575409, 18738, 58, 1, 83, "Subsection",
 CellTags->"T1.6.69.1"],
Cell[575470, 18741, 350, 12, 96, "Input"],
Cell[575823, 18755, 515, 15, 137, "Input"],
Cell[CellGroupData[{
Cell[576363, 18774, 93, 2, 96, "Input"],
Cell[576459, 18778, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[576545, 18784, 56, 1, 157, "Subsection",
 CellTags->"T1.6.70.1"],
Cell[CellGroupData[{
Cell[576626, 18789, 55, 1, 61, "Subsubsection",
 CellTags->"T1.6.70.1.1"],
Cell[576684, 18792, 226, 7, 96, "Input"],
Cell[576913, 18801, 688, 22, 102, "Input"],
Cell[577604, 18825, 957, 29, 102, "Input"],
Cell[578564, 18856, 1648, 56, 238, "Input"],
Cell[580215, 18914, 294, 9, 102, "Input"],
Cell[580512, 18925, 294, 9, 102, "Input"],
Cell[580809, 18936, 294, 9, 102, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[581140, 18950, 55, 1, 61, "Subsubsection",
 CellTags->"T1.6.70.2.1"],
Cell[581198, 18953, 1446, 50, 227, "Input"],
Cell[582647, 19005, 294, 9, 102, "Input"],
Cell[582944, 19016, 294, 9, 102, "Input"],
Cell[583241, 19027, 294, 9, 102, "Input"],
Cell[583538, 19038, 81, 2, 98, "Input"],
Cell[583622, 19042, 99, 5, 64, "Text"],
Cell[583724, 19049, 5993, 211, 427, "Input"],
Cell[589720, 19262, 137, 4, 96, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[589906, 19272, 58, 1, 83, "Subsection",
 CellTags->"T1.6.71.1"],
Cell[589967, 19275, 95, 3, 96, "Input"],
Cell[590065, 19280, 443, 14, 137, "Input"],
Cell[590511, 19296, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[590597, 19301, 56, 1, 157, "Subsection",
 CellTags->"T1.6.72.1"],
Cell[590656, 19304, 99, 5, 64, "Text"],
Cell[590758, 19311, 11665, 369, 882, "Input"],
Cell[602426, 19682, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[602600, 19691, 58, 1, 83, "Subsection",
 CellTags->"T1.6.73.1"],
Cell[602661, 19694, 115, 4, 96, "Input"],
Cell[602779, 19700, 413, 13, 137, "Input"],
Cell[603195, 19715, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[603281, 19720, 56, 1, 157, "Subsection",
 CellTags->"T1.6.74.1"],
Cell[603340, 19723, 92, 4, 64, "Text"],
Cell[603435, 19729, 2266, 85, 103, "Input"],
Cell[605704, 19816, 141, 4, 96, "Input"],
Cell[605848, 19822, 2281, 85, 103, "Input"],
Cell[608132, 19909, 141, 4, 96, "Input"],
Cell[608276, 19915, 2582, 92, 151, "Input"],
Cell[610861, 20009, 141, 4, 96, "Input"],
Cell[611005, 20015, 3514, 124, 198, "Input"],
Cell[614522, 20141, 141, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614700, 20150, 59, 1, 83, "Subsection",
 CellTags->"T1.6.75.1"],
Cell[614762, 20153, 230, 8, 96, "Input"],
Cell[614995, 20163, 444, 14, 137, "Input"],
Cell[615442, 20179, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[615528, 20184, 59, 1, 83, "Subsection",
 CellTags->"T1.6.76.1"],
Cell[615590, 20187, 367, 13, 96, "Input"],
Cell[615960, 20202, 513, 16, 137, "Input"],
Cell[616476, 20220, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[616562, 20225, 59, 1, 83, "Subsection",
 CellTags->"T1.6.77.1"],
Cell[616624, 20228, 98, 3, 96, "Input"],
Cell[616725, 20233, 549, 16, 177, "Input"],
Cell[617277, 20251, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[617363, 20256, 59, 1, 83, "Subsection",
 CellTags->"T1.6.78.1"],
Cell[617425, 20259, 230, 8, 96, "Input"],
Cell[617658, 20269, 531, 15, 139, "Input"],
Cell[618192, 20286, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[618278, 20291, 56, 1, 157, "Subsection",
 CellTags->"T1.6.79.1"],
Cell[618337, 20294, 181, 6, 64, "Text"],
Cell[618521, 20302, 181, 5, 96, "Input"],
Cell[618705, 20309, 501, 16, 71, "Text"],
Cell[619209, 20327, 4798, 160, 191, "Input"],
Cell[624010, 20489, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[624184, 20498, 56, 1, 157, "Subsection",
 CellTags->"T1.6.80.1"],
Cell[624243, 20501, 551, 20, 64, "Text"],
Cell[624797, 20523, 2593, 92, 151, "Input"],
Cell[627393, 20617, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[627567, 20626, 58, 1, 83, "Subsection",
 CellTags->"T1.6.81.1"],
Cell[627628, 20629, 227, 8, 96, "Input"],
Cell[627858, 20639, 469, 15, 96, "Input"],
Cell[628330, 20656, 49, 0, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[628416, 20661, 56, 1, 157, "Subsection",
 CellTags->"T1.6.82.1"],
Cell[628475, 20664, 92, 4, 64, "Text"],
Cell[628570, 20670, 2800, 98, 151, "Input"],
Cell[631373, 20770, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[631547, 20779, 58, 1, 83, "Subsection",
 CellTags->"T1.6.83.1"],
Cell[631608, 20782, 227, 8, 96, "Input"],
Cell[631838, 20792, 402, 12, 98, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[632277, 20809, 60, 1, 83, "Subsection",
 CellTags->"T1.6.84.1"],
Cell[632340, 20812, 335, 11, 98, "Input"],
Cell[632678, 20825, 335, 12, 96, "Input"],
Cell[633016, 20839, 486, 14, 98, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[633539, 20858, 60, 1, 83, "Subsection",
 CellTags->"T1.6.85.1"],
Cell[633602, 20861, 456, 14, 98, "Input"],
Cell[634061, 20877, 335, 12, 96, "Input"],
Cell[634399, 20891, 486, 14, 98, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[634922, 20910, 56, 1, 157, "Subsection",
 CellTags->"T1.6.86.1"],
Cell[634981, 20913, 92, 4, 64, "Text"],
Cell[635076, 20919, 1860, 71, 103, "Input"],
Cell[636939, 20992, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[637113, 21001, 58, 1, 83, "Subsection",
 CellTags->"T1.6.87.1"],
Cell[637174, 21004, 227, 8, 96, "Input"],
Cell[637404, 21014, 451, 14, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[637892, 21033, 56, 1, 157, "Subsection",
 CellTags->"T1.6.88.1"],
Cell[637951, 21036, 92, 4, 64, "Text"],
Cell[638046, 21042, 2026, 74, 103, "Input"],
Cell[640075, 21118, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640249, 21127, 58, 1, 83, "Subsection",
 CellTags->"T1.6.89.1"],
Cell[640310, 21130, 95, 3, 96, "Input"],
Cell[640408, 21135, 395, 11, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640840, 21151, 56, 1, 157, "Subsection",
 CellTags->"T1.6.90.1"],
Cell[640899, 21154, 92, 4, 64, "Text"],
Cell[640994, 21160, 1200, 46, 103, "Input"],
Cell[642197, 21208, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642371, 21217, 58, 1, 83, "Subsection",
 CellTags->"T1.6.91.1"],
Cell[642432, 21220, 227, 8, 96, "Input"],
Cell[642662, 21230, 374, 11, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[643073, 21246, 56, 1, 157, "Subsection",
 CellTags->"T1.6.92.1"],
Cell[643132, 21249, 92, 4, 64, "Text"],
Cell[643227, 21255, 1074, 41, 103, "Input"],
Cell[644304, 21298, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644478, 21307, 58, 1, 83, "Subsection",
 CellTags->"T1.6.93.1"],
Cell[644539, 21310, 95, 3, 96, "Input"],
Cell[644637, 21315, 357, 11, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[645031, 21331, 56, 1, 157, "Subsection",
 CellTags->"T1.6.94.1"],
Cell[645090, 21334, 330, 11, 157, "Text"],
Cell[645423, 21347, 2031, 72, 128, "Text"],
Cell[647457, 21421, 2424, 90, 151, "Input"],
Cell[649884, 21513, 137, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[650058, 21522, 58, 1, 83, "Subsection",
 CellTags->"T1.6.95.1"],
Cell[650119, 21525, 227, 8, 96, "Input"],
Cell[650349, 21535, 471, 15, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[650857, 21555, 60, 1, 83, "Subsection",
 CellTags->"T1.6.96.1"],
Cell[650920, 21558, 344, 11, 96, "Input"],
Cell[651267, 21571, 477, 15, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[651781, 21591, 60, 1, 83, "Subsection",
 CellTags->"T1.6.97.1"],
Cell[651844, 21594, 344, 11, 96, "Input"],
Cell[652191, 21607, 477, 15, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[652705, 21627, 57, 1, 157, "Subsection",
 CellTags->"T1.6.98.1"],
Cell[652765, 21630, 39, 0, 157, "Text"],
Cell[652807, 21632, 2717, 96, 157, "Text"],
Cell[655527, 21730, 116, 3, 64, "Text"],
Cell[655646, 21735, 2981, 112, 151, "Input"],
Cell[658630, 21849, 139, 4, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658806, 21858, 59, 1, 83, "Subsection",
 CellTags->"T1.6.99.1"],
Cell[658868, 21861, 97, 3, 96, "Input"],
Cell[658968, 21866, 510, 17, 96, "Input"],
Cell[CellGroupData[{
Cell[659503, 21887, 94, 2, 96, "Input"],
Cell[659600, 21891, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[659686, 21897, 61, 1, 83, "Subsection",
 CellTags->"T1.6.100.1"],
Cell[659750, 21900, 203, 6, 96, "Input"],
Cell[659956, 21908, 508, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[660489, 21928, 95, 2, 96, "Input"],
Cell[660587, 21932, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[660673, 21938, 61, 1, 83, "Subsection",
 CellTags->"T1.6.101.1"],
Cell[660737, 21941, 254, 7, 96, "Input"],
Cell[660994, 21950, 508, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[661527, 21970, 95, 2, 96, "Input"],
Cell[661625, 21974, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[661711, 21980, 61, 1, 83, "Subsection",
 CellTags->"T1.6.102.1"],
Cell[661775, 21983, 203, 6, 96, "Input"],
Cell[661981, 21991, 508, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[662514, 22011, 95, 2, 96, "Input"],
Cell[662612, 22015, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[662698, 22021, 61, 1, 83, "Subsection",
 CellTags->"T1.6.103.1"],
Cell[662762, 22024, 203, 6, 96, "Input"],
Cell[662968, 22032, 508, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[663501, 22052, 95, 2, 96, "Input"],
Cell[663599, 22056, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[663685, 22062, 61, 1, 83, "Subsection",
 CellTags->"T1.6.104.1"],
Cell[663749, 22065, 254, 7, 96, "Input"],
Cell[664006, 22074, 508, 16, 96, "Input"],
Cell[CellGroupData[{
Cell[664539, 22094, 95, 2, 96, "Input"],
Cell[664637, 22098, 37, 0, 46, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[664723, 22104, 58, 1, 157, "Subsection",
 CellTags->"T1.6.105.1"],
Cell[664784, 22107, 116, 4, 157, "Text"],
Cell[664903, 22113, 525, 16, 67, "Text"],
Cell[665431, 22131, 2289, 70, 297, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[667757, 22206, 60, 1, 83, "Subsection",
 CellTags->"T1.6.106.1"],
Cell[667820, 22209, 97, 3, 96, "Input"],
Cell[667920, 22214, 169, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[668126, 22224, 58, 1, 157, "Subsection",
 CellTags->"T1.6.107.1"],
Cell[668187, 22227, 116, 4, 157, "Text"],
Cell[668306, 22233, 525, 16, 67, "Text"],
Cell[668834, 22251, 2595, 78, 417, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671466, 22334, 60, 1, 83, "Subsection",
 CellTags->"T1.6.108.1"],
Cell[671529, 22337, 97, 3, 96, "Input"],
Cell[671629, 22342, 361, 11, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[672027, 22358, 58, 1, 157, "Subsection",
 CellTags->"T1.6.109.1"],
Cell[672088, 22361, 116, 4, 157, "Text"],
Cell[672207, 22367, 525, 16, 67, "Text"],
Cell[672735, 22385, 2074, 64, 257, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[674846, 22454, 60, 1, 83, "Subsection",
 CellTags->"T1.6.110.1"],
Cell[674909, 22457, 97, 3, 96, "Input"],
Cell[675009, 22462, 169, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[675215, 22472, 58, 1, 157, "Subsection",
 CellTags->"T1.6.111.1"],
Cell[675276, 22475, 116, 4, 64, "Text"],
Cell[675395, 22481, 1559, 48, 257, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[676991, 22534, 60, 1, 83, "Subsection",
 CellTags->"T1.6.112.1"],
Cell[677054, 22537, 97, 3, 96, "Input"],
Cell[677154, 22542, 169, 5, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[677360, 22552, 58, 1, 157, "Subsection",
 CellTags->"T1.6.113.1"],
Cell[677421, 22555, 43, 0, 157, "Text"],
Cell[677467, 22557, 305, 10, 66, "Text"],
Cell[677775, 22569, 6831, 224, 294, "Input"],
Cell[684609, 22795, 386, 9, 157, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[685032, 22809, 60, 1, 83, "Subsection",
 CellTags->"T1.6.114.1"],
Cell[685095, 22812, 229, 8, 96, "Input"],
Cell[685327, 22822, 625, 20, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[685989, 22847, 60, 1, 83, "Subsection",
 CellTags->"T1.6.115.1"],
Cell[686052, 22850, 691, 29, 987, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[686780, 22884, 60, 1, 83, "Subsection",
 CellTags->"T1.6.116.1"],
Cell[686843, 22887, 695, 29, 987, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687575, 22921, 61, 1, 83, "Subsection",
 CellTags->"T1.6.117.1"],
Cell[687639, 22924, 3385, 91, 657, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[691061, 23020, 61, 1, 83, "Subsection",
 CellTags->"T1.6.118.1"],
Cell[691125, 23023, 3355, 91, 617, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[694517, 23119, 61, 1, 83, "Subsection",
 CellTags->"T1.6.119.1"],
Cell[694581, 23122, 3349, 91, 617, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[697967, 23218, 60, 1, 83, "Subsection",
 CellTags->"T1.6.120.1"],
Cell[698030, 23221, 240, 6, 233, "Input"],
Cell[CellGroupData[{
Cell[698295, 23231, 4436, 118, 512, "Input"],
Cell[702734, 23351, 1563, 21, 46, "Print"]
}, Open  ]],
Cell[704312, 23375, 23, 0, 64, "Text"],
Cell[704338, 23377, 2177, 56, 538, "Input"],
Cell[706518, 23435, 23, 0, 64, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[706578, 23440, 54, 1, 83, "Subsection",
 CellTags->"Tsymmetry"],
Cell[706635, 23443, 34, 0, 64, "Text"],
Cell[706672, 23445, 107, 3, 104, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[706816, 23453, 69, 1, 83, "Subsection",
 CellTags->"specialTVR"],
Cell[706888, 23456, 579, 11, 405, "Input"],
Cell[707470, 23469, 1683, 27, 918, "Input"],
Cell[709156, 23498, 1597, 26, 884, "Input"],
Cell[710756, 23526, 3547, 56, 1946, "Input"],
Cell[714306, 23584, 3261, 52, 1809, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[717604, 23641, 59, 1, 83, "Subsection",
 CellTags->"T1.6.121.1"],
Cell[717666, 23644, 230, 5, 184, "Text"],
Cell[717899, 23651, 1143, 33, 217, "Input"],
Cell[719045, 23686, 237, 5, 184, "Text"],
Cell[719285, 23693, 1251, 37, 217, "Input"],
Cell[720539, 23732, 239, 5, 184, "Text"],
Cell[720781, 23739, 1257, 37, 217, "Input"],
Cell[722041, 23778, 238, 5, 184, "Text"],
Cell[722282, 23785, 1252, 37, 217, "Input"],
Cell[723537, 23824, 219, 14, 541, "Text"],
Cell[723759, 23840, 200, 5, 199, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[723996, 23850, 60, 1, 83, "Subsection",
 CellTags->"T1.6.122.1"],
Cell[724059, 23853, 949, 29, 177, "Input"],
Cell[725011, 23884, 949, 29, 177, "Input"],
Cell[725963, 23915, 133, 3, 131, "Input"],
Cell[726099, 23920, 846, 25, 177, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[726982, 23950, 59, 1, 83, "Subsection",
 CellTags->"T1.6.123.1"],
Cell[727044, 23953, 793, 25, 137, "Input"],
Cell[727840, 23980, 793, 25, 137, "Input"],
Cell[728636, 24007, 792, 25, 137, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[729465, 24037, 57, 1, 83, "Subsection",
 CellTags->"T1.6.124.1"],
Cell[729525, 24040, 512, 16, 137, "Input"],
Cell[730040, 24058, 511, 16, 137, "Input"],
Cell[730554, 24076, 362, 12, 96, "Input"],
Cell[730919, 24090, 334, 11, 96, "Input"],
Cell[731256, 24103, 344, 11, 96, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[731649, 24120, 106, 2, 160, "Section",
 CellTags->"T1.7.1"],
Cell[CellGroupData[{
Cell[731780, 24126, 69, 1, 83, "Subsection",
 CellTags->"T1.7.1.1"],
Cell[731852, 24129, 45, 0, 64, "Text"],
Cell[731900, 24131, 81, 2, 96, "Input"],
Cell[731984, 24135, 275, 7, 137, "Input"],
Cell[732262, 24144, 451, 15, 96, "Input"],
Cell[732716, 24161, 354, 11, 137, "Input"],
Cell[733073, 24174, 453, 15, 96, "Input"],
Cell[733529, 24191, 356, 11, 137, "Input"],
Cell[733888, 24204, 82, 2, 64, "Text"],
Cell[733973, 24208, 84, 2, 96, "Input"],
Cell[734060, 24212, 336, 11, 96, "Input"],
Cell[734399, 24225, 215, 7, 223, "Text",
 CellTags->"TLRdef"],
Cell[734617, 24234, 388, 13, 96, "Input"],
Cell[735008, 24249, 755, 23, 137, "Input"],
Cell[735766, 24274, 26, 0, 64, "Text"],
Cell[735795, 24276, 3735, 100, 907, "Input"],
Cell[739533, 24378, 102, 2, 66, "Text"],
Cell[739638, 24382, 3693, 97, 907, "Input"],
Cell[743334, 24481, 127, 3, 64, "Text"],
Cell[743464, 24486, 2718, 74, 725, "Input"],
Cell[746185, 24562, 127, 3, 64, "Text"],
Cell[746315, 24567, 2980, 82, 725, "Input"],
Cell[749298, 24651, 150, 4, 64, "Text"],
Cell[749451, 24657, 3363, 91, 850, "Input"],
Cell[752817, 24750, 37, 0, 64, "Text"],
Cell[752857, 24752, 2534, 69, 524, "Input"],
Cell[755394, 24823, 37, 0, 64, "Text"],
Cell[755434, 24825, 2524, 69, 524, "Input"],
Cell[757961, 24896, 35, 0, 64, "Text"],
Cell[757999, 24898, 2979, 76, 857, "Input"],
Cell[760981, 24976, 31, 0, 157, "Text"],
Cell[761015, 24978, 224, 6, 157, "Text"],
Cell[761242, 24986, 31, 0, 64, "Text"],
Cell[761276, 24988, 204, 5, 165, "Input"],
Cell[761483, 24995, 146, 3, 104, "Text"],
Cell[761632, 25000, 443, 15, 507, "Input"],
Cell[762078, 25017, 443, 15, 507, "Input"],
Cell[762524, 25034, 49, 0, 64, "Text"],
Cell[762576, 25036, 1495, 43, 473, "Input"],
Cell[764074, 25081, 49, 0, 64, "Text"],
Cell[764126, 25083, 1498, 43, 473, "Input"],
Cell[765627, 25128, 25, 0, 64, "Text"],
Cell[765655, 25130, 876, 27, 217, "Input"],
Cell[766534, 25159, 989, 29, 217, "Input"],
Cell[767526, 25190, 77, 1, 64, "Text",
 CellTags->"tlrule17"],
Cell[767606, 25193, 2195, 64, 542, "Input"],
Cell[769804, 25259, 3938, 106, 755, "Input"],
Cell[773745, 25367, 3938, 106, 755, "Input"],
Cell[777686, 25475, 37, 0, 64, "Text"],
Cell[777726, 25477, 1908, 54, 542, "Input"],
Cell[779637, 25533, 1071, 31, 257, "Input"],
Cell[780711, 25566, 1289, 35, 421, "Input"],
Cell[782003, 25603, 5310, 191, 336, "Text"],
Cell[787316, 25796, 3323, 91, 841, "Input"],
Cell[790642, 25889, 9990, 351, 344, "Text"],
Cell[800635, 26242, 3684, 103, 870, "Input"],
Cell[804322, 26347, 65, 0, 64, "Text"],
Cell[804390, 26349, 1327, 35, 377, "Input"],
Cell[805720, 26386, 137, 4, 96, "Input"],
Cell[805860, 26392, 190, 6, 96, "Input"],
Cell[806053, 26400, 496, 17, 96, "Input"],
Cell[806552, 26419, 496, 17, 96, "Input"],
Cell[807051, 26438, 496, 17, 96, "Input"],
Cell[807550, 26457, 496, 17, 96, "Input"],
Cell[808049, 26476, 496, 17, 96, "Input"],
Cell[808548, 26495, 496, 17, 96, "Input"],
Cell[809047, 26514, 125, 3, 96, "Input"],
Cell[809175, 26519, 263, 8, 96, "Input"],
Cell[809441, 26529, 466, 15, 96, "Input"],
Cell[809910, 26546, 505, 15, 137, "Input"],
Cell[810418, 26563, 106, 3, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[810561, 26571, 49, 1, 157, "Subsection",
 CellTags->"T1.7.2.1"],
Cell[810613, 26574, 86, 3, 104, "Text"],
Cell[810702, 26579, 1925, 57, 424, "Input"],
Cell[812630, 26638, 788, 25, 137, "Input"],
Cell[813421, 26665, 788, 25, 137, "Input"],
Cell[814212, 26692, 1878, 56, 345, "Input"],
Cell[816093, 26750, 682, 21, 137, "Input"],
Cell[816778, 26773, 682, 21, 137, "Input"],
Cell[817463, 26796, 1459, 43, 250, "Input"],
Cell[818925, 26841, 683, 21, 137, "Input"],
Cell[819611, 26864, 683, 21, 137, "Input"],
Cell[820297, 26887, 1536, 47, 355, "Input"],
Cell[821836, 26936, 1690, 49, 345, "Input"],
Cell[823529, 26987, 22, 0, 64, "Text"],
Cell[823554, 26989, 435, 15, 96, "Input"],
Cell[823992, 27006, 321, 11, 96, "Input"],
Cell[824316, 27019, 1605, 49, 370, "Input"],
Cell[825924, 27070, 2174, 70, 418, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[828135, 27145, 53, 1, 83, "Subsection",
 CellTags->"T1.7.3.1"],
Cell[828191, 27148, 85, 2, 96, "Input"],
Cell[828279, 27152, 3632, 90, 777, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831948, 27247, 51, 1, 83, "Subsection",
 CellTags->"$TTable"],
Cell[832002, 27250, 92, 3, 96, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[832131, 27258, 58, 1, 83, "Subsection",
 CellTags->"T1.7.4.1"],
Cell[832192, 27261, 220, 6, 96, "Input"],
Cell[832415, 27269, 713, 20, 177, "Input"],
Cell[833131, 27291, 415, 11, 177, "Input"],
Cell[833549, 27304, 17287, 434, 3350, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[850885, 27744, 90, 2, 160, "Section",
 CellTags->"T1.8.1"],
Cell[850978, 27748, 532, 14, 217, "Input"],
Cell[851513, 27764, 463, 14, 137, "Input"],
Cell[851979, 27780, 16406, 417, 2953, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[868422, 28202, 91, 2, 157, "Section",
 CellTags->"T1.9.1"],
Cell[CellGroupData[{
Cell[868538, 28208, 53, 1, 157, "Subsection",
 CellTags->"T1.9.1.1"],
Cell[868594, 28211, 61, 0, 64, "Text"],
Cell[868658, 28213, 565, 16, 137, "Input"],
Cell[869226, 28231, 136, 4, 96, "Input"],
Cell[869365, 28237, 122, 3, 96, "Input"],
Cell[869490, 28242, 83, 2, 96, "Input"],
Cell[869576, 28246, 4161, 103, 577, "Input"],
Cell[873740, 28351, 46, 0, 96, "Input"],
Cell[873789, 28353, 330, 11, 96, "Input"],
Cell[874122, 28366, 230, 8, 96, "Input"],
Cell[874355, 28376, 78, 2, 96, "Input"],
Cell[CellGroupData[{
Cell[874458, 28382, 216, 5, 137, "Input"],
Cell[874677, 28389, 367, 9, 46, "Print"]
}, Open  ]],
Cell[875059, 28401, 366, 10, 137, "Input"],
Cell[875428, 28413, 79, 2, 96, "Input"],
Cell[875510, 28417, 62, 1, 96, "Input"],
Cell[875575, 28420, 83, 2, 96, "Input"],
Cell[875661, 28424, 78, 2, 96, "Input"],
Cell[875742, 28428, 69, 2, 96, "Input"],
Cell[875814, 28432, 76, 2, 96, "Input"],
Cell[875893, 28436, 297, 10, 137, "Input"],
Cell[876193, 28448, 139, 4, 137, "Input"],
Cell[876335, 28454, 92, 2, 96, "Input"],
Cell[876430, 28458, 648, 18, 217, "Input"],
Cell[877081, 28478, 69, 2, 96, "Input"],
Cell[877153, 28482, 95, 3, 96, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[877297, 28491, 93, 2, 157, "Section",
 CellTags->"T1.10.1"],
Cell[877393, 28495, 114, 3, 157, "Text"],
Cell[877510, 28500, 56, 0, 64, "Text"],
Cell[CellGroupData[{
Cell[877591, 28504, 1634, 53, 1740, "Input"],
Cell[879228, 28559, 15628, 499, 1234, "Output"]
}, Open  ]],
Cell[894871, 29061, 1675, 54, 157, "Text"],
Cell[896549, 29117, 15179, 494, 157, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[911765, 29616, 99, 2, 160, "Section",
 CellTags->"T1.11.1"],
Cell[911867, 29620, 71, 2, 96, "Input"],
Cell[911941, 29624, 2354, 56, 1912, "Input"],
Cell[914298, 29682, 90, 2, 96, "Input"],
Cell[914391, 29686, 87, 2, 64, "Text"],
Cell[CellGroupData[{
Cell[914503, 29692, 217, 7, 137, "Input"],
Cell[914723, 29701, 70, 1, 92, "Output"]
}, Open  ]],
Cell[914808, 29705, 55, 0, 64, "Text"],
Cell[CellGroupData[{
Cell[914888, 29709, 265, 7, 137, "Input"],
Cell[915156, 29718, 46, 0, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915239, 29723, 99, 2, 96, "Input"],
Cell[915341, 29727, 59, 1, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915437, 29733, 57, 1, 96, "Input"],
Cell[915497, 29736, 83, 1, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915617, 29742, 61, 1, 96, "Input"],
Cell[915681, 29745, 35, 0, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915753, 29750, 59, 1, 96, "Input"],
Cell[915815, 29753, 35, 0, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915887, 29758, 56, 1, 96, "Input"],
Cell[915946, 29761, 45, 0, 92, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
