(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    203169,       6611]
NotebookOptionsPosition[    197129,       6405]
NotebookOutlinePosition[    197817,       6432]
CellTagsIndexPosition[    197687,       6426]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["QCD 1 - loop example: gluon self energy", "Subsection"],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolf/fccvs/fc7/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900414478009*^9}],

Cell["\<\
Loading TARCER /home/rolf/fccvs/fc7/HighEnergyPhysics/Tarcer/tarcer25.mx\
\>", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900414651064*^9}],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "7.0.0",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900415775507*^9}],

Cell["Loading PHI ", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900415948844*^9}],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900416660928*^9}],

Cell["FeynArts 3.5 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.472900416818763*^9}],

Cell[CellGroupData[{

Cell["The graphs", "Subsubsection"],

Cell["\<\
This generates the 1-loop self energy diagrams using a model file for QCD \
from the Models directory in HighEnergyPhyiscs.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Paint", "[", 
   RowBox[{
    RowBox[{"inserts", "=", 
     RowBox[{"InsertFields", "[", 
      RowBox[{
       RowBox[{"CreateTopologies", "[", 
        RowBox[{"1", ",", 
         RowBox[{"1", "\[Rule]", "1"}], ",", 
         RowBox[{"ExcludeTopologies", "\[Rule]", "Tadpoles"}]}], "]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"V", "[", "5", "]"}], "}"}], "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"V", "[", "5", "]"}], "}"}]}], ",", 
       RowBox[{"InsertionLevel", "\[Rule]", 
        RowBox[{"{", "Classes", "}"}]}], ",", 
       RowBox[{"GenericModel", "->", "\"\<FCQCDLorentz\>\""}], ",", 
       RowBox[{"Model", "\[Rule]", "\"\<FCQCD\>\""}]}], "]"}]}], ",", 
    RowBox[{"ColumnsXRows", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"4", ",", "1"}], "}"}]}], ",", 
    RowBox[{"SheetHeader", "\[Rule]", "None"}], ",", 
    RowBox[{"Numbering", "\[Rule]", "Simple"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 FormBox[
  GraphicsBox[{InsetBox[
     GraphicsBox[
      {Thickness[0.005], LineBox[CompressedData["
1:eJxFlwtcTOkbx8d0Ualp0r1Edc4hbW7bH60u50npQlYukdamWreWahIrNluW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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5., 9.0548}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9l3s4VOsXx6dRDBVRHYrjsvd2zWXvLVFJq3REKlO6ELqcVChkKKEyCY1b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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {15., 8.354800000000001}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9mHk4Vdv/xzUnhAaS4cxnn+MMlXQr4ztJGpB5aDAkJBoUFVGKikolikxF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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {10., 16.4452}, {0, -1}], 
       {PointSize[0.04], PointBox[{10., 10.}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox[
           FormBox["1",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          FontFamily->"Times",
          FontSize->6.597818181818183],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          6.597818181818183]& ], {10., -0.5}, {0, -1}]},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {0, 0}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], LineBox[CompressedData["
1:eJw9lQtUTHkcx2emiSkjhOixYu69IdqkYnaU+1P0QBTZkiR5TV6FXhSyOsyU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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9lQtUTVkYx6+YXBViQpKRc84o6XXOKYkaH2rWJa6Wkoqe0wtNqqlb0Ut6
pwdNuaS6oqe4WZNUHqmUpElaohAlobw1PZDu7GNZ+6x111m/dc/e397f4/9f
6rZvm4ccj8cLRT/uHT0pQ88K4H1/KPBRvvN5yFQHeJIvZUYsBSG1wPeLRsxv
677pQIGja8oc7TbE1zVvVAZSoFdrKf1jsS7wvGMU/4mkIEopd9XqPxGXZ5zW
O0ABLyl8fkMN4g/1U9y9KJhnVBNBT9UDnt3U5dYWFPh3KZ37sgFxOr+mYj4F
Sw4zjaP7ESsHS/t7SIgnAyrCChEbMC+u5ZAQa/iyJLgRsWDRxCF7EiqfEspB
DxCPX/IoUCFhjdMz69LHiIund+d0ELBMq0X3UyfiXlbYkEXAY51nR+tuINaY
9niROwFd0b6PVM5y8e7nrDQjwEpqpmKbjDgk7aQCRYBzpaTJzJtjI8+ehQSY
HktrkZojVo06ySwhoOZO4GAmiVjSK2xkCWhYZmxYMx1xe19Qsi0BwqvDn/KH
0f01BtwaYgkQ8yx4AYOIIU+kUEuAnOSd0a0hxOP3xg/ICNBSi996cRwx/0r0
/Q0kZGwPsJH8jPZLcFItTCAhJc3VZPMabv/qdr8WEhSKfGWdvoirlt7u/4mC
HH315XVliKP26EwaUyA9d+KjwjCXr8oxYycKDFQ0hOZr9YH3am1rvoiC5jr5
3RdSEWvt6ZWLokArY4vM5wni5qujT0IpqLazfN00zwDlXxAR507Brofr2EIT
xCGKaTPXUTArqdszwx5xlGj0tzkUlBd7SKICEV8P48c+IKE9fKBIksitd3Nt
ySIhqyPAyl/MfX9Ux2YbCTFKhxXqJYglBV3HZpJQFhFuW5mP2Ltx56xWAqZN
E4815CJWnm05/QgBV9TbZWWZiF3Oz4hwJkCgN3OKKAGx2MJk2ISA1sua/XYh
iPnznTYTBOzu6uJf9ERcpWF/GdXPZW9C+jcbxKrUyHkNAiYPmmUVmiP2WyD8
dyUBym/rvecac+fZt3PcAcXffulMpC5iQazbrmQCtK9fUP+sxa33y4pvJOCq
sshJoI3YanZFojwJH9xSDr2mERdXTVGzJCH93JK7s4A7n8vNujQSiILG0CQu
foKm2oy7JLxwK8597svtz686rUhB2wpb0Y5ULn8B0mxTCsJP8TY9usCdR1Ya
7EqBifPCZraLy0/lR4cQCr6Pr4xbXxu+GNWvL/uFcasmjf+PcJJdpIQ0Xq93
rySx2J/G+xuKd7guOkLj+A8dVscFnaXx+Xy29ep9rKXx+bXFAWEqbTS+32/v
E09d6qTx/TvWCvvlH9A4P/MKBTujOmicvzdzzY69baZxfr+F1++2v0zj/Nek
Be+T4+L/qE+GpubYpuM0rp+BeZd6UTyN69vR+mkyMZjG9Yd+fZ71Hhr3x/hX
L76pK437x+pgdMlGRxr3V8jwXyNmzjTuvxitFGmqF437Uydazc5ZROP+3bzj
hHNkMo37W8l/2XGfQhr3f96v11rtm2g8H+uLKlRMhmg8P7G0fdQtZQbP14K8
MMuiVQyev+cj77scXRg8n+vPMPH74xg8v7qv/PfmlTB4vkVlAl5mM4Pn35rd
2m//jMH6wHQ2deqPMlg/XgaY2hVMZbG+fJXyPbIVWKw/5ZE9st8VWaxPAepx
cXryLNavEz7PB4gJBuvbL2ILRaN3DNa/A/pVasU9DNbH1jNS9b7bDNZPG62+
EZsaBuvr+WrB0aBSButvymDqyrwcBuvzq9dqg/KZDNbviduz7pceYbC+a6S/
C7b4m8H6X+2htMKcW//DH2wHmwrqyhjsH2+qPXNi6hnsLw0ZoSVyTxjsP/4D
+eA1yWB/mlhdF5hLsNi/uv8r73fbyGJ/c9dw0BL6s9j/tmw1GRsSs9gfHbOf
fr15jcX+2RSseiipj8X+WvPddw2x//4Pj3TtGg==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 10.9452}, {0, -1}], 
       CircleBox[{9.999999999999995, 10.899999999999997`}, 
        4.099999999999994, {3.3629070959375835`, 6.0618708648317945`}], 
       PolygonBox[{{9.399999999999995, 6.800000000000004}, {
        10.599999999999994`, 6.400000000000004}, {10.599999999999994`, 
        7.200000000000005}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.8548000000000044`}, \
{0, 1}], CircleBox[{9.999999999999995, 9.100000000000003}, 
        4.099999999999994, {0.2213144423477913, 2.9202782112420023`}], 
       PolygonBox[{{10.599999999999994`, 13.199999999999996`}, {
        9.399999999999995, 13.599999999999996`}, {9.399999999999995, 
        12.799999999999995`}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 14.145199999999996`}, \
{0, -1}], 
       {PointSize[0.04], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox[
           FormBox["2",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          FontFamily->"Times",
          FontSize->6.597818181818183],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          6.597818181818183]& ], {10., -0.5}, {0, -1}]},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {22, 0}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], LineBox[CompressedData["
1:eJw9lQtUTHkcx2emiSkjhOixYu69IdqkYnaU+1P0QBTZkiR5TV6FXhSyOsyU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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9lQtUTVkYx6+YXBViQpKRc84o6XXOKYkaH2rWJa6Wkoqe0wtNqqlb0Ut6
pwdNuaS6oqe4WZNUHqmUpElaohAlobw1PZDu7GNZ+6x111m/dc/e397f4/9f
6rZvm4ccj8cLRT/uHT0pQ88K4H1/KPBRvvN5yFQHeJIvZUYsBSG1wPeLRsxv
677pQIGja8oc7TbE1zVvVAZSoFdrKf1jsS7wvGMU/4mkIEopd9XqPxGXZ5zW
O0ABLyl8fkMN4g/1U9y9KJhnVBNBT9UDnt3U5dYWFPh3KZ37sgFxOr+mYj4F
Sw4zjaP7ESsHS/t7SIgnAyrCChEbMC+u5ZAQa/iyJLgRsWDRxCF7EiqfEspB
DxCPX/IoUCFhjdMz69LHiIund+d0ELBMq0X3UyfiXlbYkEXAY51nR+tuINaY
9niROwFd0b6PVM5y8e7nrDQjwEpqpmKbjDgk7aQCRYBzpaTJzJtjI8+ehQSY
HktrkZojVo06ySwhoOZO4GAmiVjSK2xkCWhYZmxYMx1xe19Qsi0BwqvDn/KH
0f01BtwaYgkQ8yx4AYOIIU+kUEuAnOSd0a0hxOP3xg/ICNBSi996cRwx/0r0
/Q0kZGwPsJH8jPZLcFItTCAhJc3VZPMabv/qdr8WEhSKfGWdvoirlt7u/4mC
HH315XVliKP26EwaUyA9d+KjwjCXr8oxYycKDFQ0hOZr9YH3am1rvoiC5jr5
3RdSEWvt6ZWLokArY4vM5wni5qujT0IpqLazfN00zwDlXxAR507Brofr2EIT
xCGKaTPXUTArqdszwx5xlGj0tzkUlBd7SKICEV8P48c+IKE9fKBIksitd3Nt
ySIhqyPAyl/MfX9Ux2YbCTFKhxXqJYglBV3HZpJQFhFuW5mP2Ltx56xWAqZN
E4815CJWnm05/QgBV9TbZWWZiF3Oz4hwJkCgN3OKKAGx2MJk2ISA1sua/XYh
iPnznTYTBOzu6uJf9ERcpWF/GdXPZW9C+jcbxKrUyHkNAiYPmmUVmiP2WyD8
dyUBym/rvecac+fZt3PcAcXffulMpC5iQazbrmQCtK9fUP+sxa33y4pvJOCq
sshJoI3YanZFojwJH9xSDr2mERdXTVGzJCH93JK7s4A7n8vNujQSiILG0CQu
foKm2oy7JLxwK8597svtz686rUhB2wpb0Y5ULn8B0mxTCsJP8TY9usCdR1Ya
7EqBifPCZraLy0/lR4cQCr6Pr4xbXxu+GNWvL/uFcasmjf+PcJJdpIQ0Xq93
rySx2J/G+xuKd7guOkLj+A8dVscFnaXx+Xy29ep9rKXx+bXFAWEqbTS+32/v
E09d6qTx/TvWCvvlH9A4P/MKBTujOmicvzdzzY69baZxfr+F1++2v0zj/Nek
Be+T4+L/qE+GpubYpuM0rp+BeZd6UTyN69vR+mkyMZjG9Yd+fZ71Hhr3x/hX
L76pK437x+pgdMlGRxr3V8jwXyNmzjTuvxitFGmqF437Uydazc5ZROP+3bzj
hHNkMo37W8l/2XGfQhr3f96v11rtm2g8H+uLKlRMhmg8P7G0fdQtZQbP14K8
MMuiVQyev+cj77scXRg8n+vPMPH74xg8v7qv/PfmlTB4vkVlAl5mM4Pn35rd
2m//jMH6wHQ2deqPMlg/XgaY2hVMZbG+fJXyPbIVWKw/5ZE9st8VWaxPAepx
cXryLNavEz7PB4gJBuvbL2ILRaN3DNa/A/pVasU9DNbH1jNS9b7bDNZPG62+
EZsaBuvr+WrB0aBSButvymDqyrwcBuvzq9dqg/KZDNbviduz7pceYbC+a6S/
C7b4m8H6X+2htMKcW//DH2wHmwrqyhjsH2+qPXNi6hnsLw0ZoSVyTxjsP/4D
+eA1yWB/mlhdF5hLsNi/uv8r73fbyGJ/c9dw0BL6s9j/tmw1GRsSs9gfHbOf
fr15jcX+2RSseiipj8X+WvPddw2x//4Pj3TtGg==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 10.9452}, {0, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 10.899999999999997`}, 
         4.099999999999994, {3.3629070959375835`, 6.0618708648317945`}]}, 
       PolygonBox[{{9.399999999999995, 6.800000000000004}, {
        10.599999999999994`, 6.400000000000004}, {10.599999999999994`, 
        7.200000000000005}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.8548000000000044`}, \
{0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 9.100000000000003}, 4.099999999999994, {
         0.2213144423477913, 2.9202782112420023`}]}, 
       PolygonBox[{{10.599999999999994`, 13.199999999999996`}, {
        9.399999999999995, 13.599999999999996`}, {9.399999999999995, 
        12.799999999999995`}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 14.145199999999996`}, \
{0, -1}], 
       {PointSize[0.04], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox[
           FormBox["3",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          FontFamily->"Times",
          FontSize->6.597818181818183],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          6.597818181818183]& ], {10., -0.5}, {0, -1}]},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {44, 0}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], LineBox[CompressedData["
1:eJw9lQtUTHkcx2emiSkjhOixYu69IdqkYnaU+1P0QBTZkiR5TV6FXhSyOsyU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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9lQtUTVkYx6+YXBViQpKRc84o6XXOKYkaH2rWJa6Wkoqe0wtNqqlb0Ut6
pwdNuaS6oqe4WZNUHqmUpElaohAlobw1PZDu7GNZ+6x111m/dc/e397f4/9f
6rZvm4ccj8cLRT/uHT0pQ88K4H1/KPBRvvN5yFQHeJIvZUYsBSG1wPeLRsxv
677pQIGja8oc7TbE1zVvVAZSoFdrKf1jsS7wvGMU/4mkIEopd9XqPxGXZ5zW
O0ABLyl8fkMN4g/1U9y9KJhnVBNBT9UDnt3U5dYWFPh3KZ37sgFxOr+mYj4F
Sw4zjaP7ESsHS/t7SIgnAyrCChEbMC+u5ZAQa/iyJLgRsWDRxCF7EiqfEspB
DxCPX/IoUCFhjdMz69LHiIund+d0ELBMq0X3UyfiXlbYkEXAY51nR+tuINaY
9niROwFd0b6PVM5y8e7nrDQjwEpqpmKbjDgk7aQCRYBzpaTJzJtjI8+ehQSY
HktrkZojVo06ySwhoOZO4GAmiVjSK2xkCWhYZmxYMx1xe19Qsi0BwqvDn/KH
0f01BtwaYgkQ8yx4AYOIIU+kUEuAnOSd0a0hxOP3xg/ICNBSi996cRwx/0r0
/Q0kZGwPsJH8jPZLcFItTCAhJc3VZPMabv/qdr8WEhSKfGWdvoirlt7u/4mC
HH315XVliKP26EwaUyA9d+KjwjCXr8oxYycKDFQ0hOZr9YH3am1rvoiC5jr5
3RdSEWvt6ZWLokArY4vM5wni5qujT0IpqLazfN00zwDlXxAR507Brofr2EIT
xCGKaTPXUTArqdszwx5xlGj0tzkUlBd7SKICEV8P48c+IKE9fKBIksitd3Nt
ySIhqyPAyl/MfX9Ux2YbCTFKhxXqJYglBV3HZpJQFhFuW5mP2Ltx56xWAqZN
E4815CJWnm05/QgBV9TbZWWZiF3Oz4hwJkCgN3OKKAGx2MJk2ISA1sua/XYh
iPnznTYTBOzu6uJf9ERcpWF/GdXPZW9C+jcbxKrUyHkNAiYPmmUVmiP2WyD8
dyUBym/rvecac+fZt3PcAcXffulMpC5iQazbrmQCtK9fUP+sxa33y4pvJOCq
sshJoI3YanZFojwJH9xSDr2mERdXTVGzJCH93JK7s4A7n8vNujQSiILG0CQu
foKm2oy7JLxwK8597svtz686rUhB2wpb0Y5ULn8B0mxTCsJP8TY9usCdR1Ya
7EqBifPCZraLy0/lR4cQCr6Pr4xbXxu+GNWvL/uFcasmjf+PcJJdpIQ0Xq93
rySx2J/G+xuKd7guOkLj+A8dVscFnaXx+Xy29ep9rKXx+bXFAWEqbTS+32/v
E09d6qTx/TvWCvvlH9A4P/MKBTujOmicvzdzzY69baZxfr+F1++2v0zj/Nek
Be+T4+L/qE+GpubYpuM0rp+BeZd6UTyN69vR+mkyMZjG9Yd+fZ71Hhr3x/hX
L76pK437x+pgdMlGRxr3V8jwXyNmzjTuvxitFGmqF437Uydazc5ZROP+3bzj
hHNkMo37W8l/2XGfQhr3f96v11rtm2g8H+uLKlRMhmg8P7G0fdQtZQbP14K8
MMuiVQyev+cj77scXRg8n+vPMPH74xg8v7qv/PfmlTB4vkVlAl5mM4Pn35rd
2m//jMH6wHQ2deqPMlg/XgaY2hVMZbG+fJXyPbIVWKw/5ZE9st8VWaxPAepx
cXryLNavEz7PB4gJBuvbL2ILRaN3DNa/A/pVasU9DNbH1jNS9b7bDNZPG62+
EZsaBuvr+WrB0aBSButvymDqyrwcBuvzq9dqg/KZDNbviduz7pceYbC+a6S/
C7b4m8H6X+2htMKcW//DH2wHmwrqyhjsH2+qPXNi6hnsLw0ZoSVyTxjsP/4D
+eA1yWB/mlhdF5hLsNi/uv8r73fbyGJ/c9dw0BL6s9j/tmw1GRsSs9gfHbOf
fr15jcX+2RSseiipj8X+WvPddw2x//4Pj3TtGg==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 10.9452}, {0, -1}], 
       LineBox[CompressedData["
1:eJw9lwtUTOsbxrsjESWNaWb27JlpmpmiQse1eqRCHakoTqKLRCIkXQ6pkEtF
yr0UFaV0IZGECrlUuki6KBWnor+DIQnFf85Z32nWmrXXrD1773d/3/s+z++h
PTc7rlWQk5N7Kfv+c/z+65+PPob+PfLxflarqh3XAHPVzT9fu8nHq6XLGRVT
DdC93vJegB8fqacVs1tMDPBihFagB4ePZ5HJ3xpEBrh3kn1RVM2DWfT955PU
DFDb8uTCpQgeBpQjl4/o1EfH306vU+by0B0n+tWRqQ8vrup4UwUeBObPgo74
6KOlLP142zMaVoFN5av4+jjy5mRl5zUae7TCNp9vkWBHmPTnsQs05ihkbouP
l4DTrOtqmkZj2Wu1vd8WS9A/MnrG/BwaxmfzpSbqEjzKnzXTp4yGbfpvseIm
MdzWTOOptdHwn7Wp1jJLDO8Fe8s9hmgUqO/ffmKfGCL/45aXKVk90rhFLZvE
kF7Z+yJ4Hg8bbb0jI9zFSI9+svioOw+UeX9g/moxFgW8o2bt4GHZkUYjrBdj
Z8zsdvkjPGSunuCbtFOM73HNPfVJPCQoUG+qE8SY//C4Wk0aD/xF9CmPMjEm
PDC7PyOFhy7f2bfXvRdj46pJcT7HeXicKtUKoyRIK7AfaAjjQbPRQKvTQYLG
8tFZGrLnd/x0Sw3dLUHiQNmrLyY8hG1riebmSnA0qdPqljwPTz0sLVPqJDBY
FDff6D6NtVTYHJ1eCY519Vo/3kVD1+nmL8kXCZScKhWLptE4yphaLJbK7r+u
5UB7FxfJWxqCV7dJ4GwdrpZ2iouK6T1SnZsSuDTaK+Uu4SKVMre7d0CCzdVh
hYljubil3GN/0VaCG7mPXM2vUFg/slO7WVmCmOfOdb7uFPSD69zrborB2O89
t3YyBTul6F26fmIsPku1uzEprLIOPTGVL8buHJc5HwUU9s3ZOtjeKoJeSeBJ
fxsKRt7LjEOSRDife+260z4KG1NVNqd6izDn+fSUoacUgh0U5nnNEsFj+21D
rhYXJzxNLlpPFGFBf5fobwsuNiWFe/gP6UHoHZD9ZjMXnaqsWTXv9aC7RfOT
+2kuqox/RTJ69cBvCasddZsL+QM5C/+Snd9yUjlbtZkLjaFRdNKgHj5rSlPl
3nGxwndGtL+GCPSUgYf9fVyc++4otTQUQWOO+sXln7kIsGisP+ogwr0645zW
bi4m2ahf3hYkwmPba6fCa7hI2KVUG5ssQsrZpKqQHC6o1E6X2fdFsDgzNNs8
ggvvGN6M6C4RFN1Z9SdtuZAIOYUxv0Tg3UiuuDyGizmB3wO3qYvh3F3DXh8v
W1/FtePbNMS46qZdlS1HQcn34Qr1kWL48WwGM305mFsb89HwvQgfxZcsiprZ
MEs1N/AtF2FVldzKfAc2igZLauRjRTiRJ3/Hs4WF7sNxkpmLRVD29dP/GMLC
O3nzDl1FETg+t1TMTVg4s7x6n2+BHnIueeoHqrNg5a13vMlLD55vHN0KRrFg
GKqwWp0hW7+GsVbyfBbWNiWNN6kTIiBxf1bQChZmKLu+vxEvhF5lQvmZCyw8
sB1dNnO1ECVPG4fqlNjQ8jv7ffd0If5y/WZavZkN7phQlytaQhROUYhW72Dj
dfqW8KXyQsR0Xcmas4SDpdOs6bP9uhBuk4/fWMhB98ipY8/06WKcs2idjyaF
plttXpbfdeEiPbinejUFp2zr/cdUhHjxM+mc8ASFseeeWEQxhFD1LNvef51C
uF+vcKKhENWW7LvFJRSSLRLqlRYJka7RlteZR6H//Mp9PV5CPDU55ZO7n4J3
//7pihFCWPI/JxoupKBi1Drr90QhKuqHJht85mDj4pIt/leEKGqtqqOiOWis
f/rmZ6kQ0UkbJs3T5KByZ8udiIdCTH3wxP1QLBu+bkdGVd0VIsr5WEmtbD0E
15f9aSi73mvDSJOkYBby1150L44TIouh2y6Q6qB33JW4TbJ6PJKc2z4E6OA6
d93zxRIhMm5trPZT1YF4BC94Y5cuHs3QzA4oZGKPq/m20lO6mN/caXMpnAk1
Hal1zwJdREq19Jk+TDhXfuZV9wuQvmxBScNWJgw2SBrtMwWYfL20ves0E6K5
PePaPAVoyXN56P2SiaO/5YboCQVYKXdHpXW2Dn4ljgwf7ONj4dOX1WmZOkib
YDvWuJYPR30X8XYBC/sWJ2deKeRjzOykrIbzLDwZDMnLzOKDo5ISGiJgQ/fM
H35FGXywOsbXHEhiQ6X3bYNRDh+/26W8qlLlYJe+23SrIj4M2rKOVK7jYHt0
1c7ZFXw4eCzobLrMgWrF1MAdL/kIF5jGCV5z8E6of/isrJ7VZmG/ygdl55uf
bXUfLRj+/cHkycG9tABh5P8RAdP2lJsIYE/uF/zsXOVNa8Hw8z5qTZTUOApg
Q+qpW5MeZ71CACapV6y4z97ZSQAWeZ/N+R7OdgsFUCXv6/i/Amm1oQAOZD2+
rvkapiirx5qsl9djh31NzXy4kPWsvMf4wEvio5ms96PjDaI8Z/7wfiRPV2sz
HsVHBtmvt3+NLWLc4GE/2U//tNIY83U8WJH9rhpo23+UyUMl6QfT9lPi/AYa
OaRfRn+w84tPlPkG6SerhOVnLf1oZJN+q1t0KbLWjoY36ceoc5v7p5nSOEz6
VX+P7rxdM2n8RvqZMpisLW9OI5b0e1Q+Yg/Jri8l8yCt4T+46kWjmszLkVHB
lYMyn7Im8yS38nwIQ1ZPA5m3FbfXGvy4TiOLzGNQnFbz21oaT8m89rpNvb29
hx6e57SlRcG+32i0knnnzuclFivzhvVA4+sT6ZrRPGgQvThfVfTSfZSMA4ie
TPwjNiz7J40oojfPP+2+YNcr4waiRx1x7sqhlTReEb0yKugze5hCo4zoWen3
tPELNtEQEb07rOf5vmIKjSCihw92D6V87JH5BtHLy6ebNuglynyA6OkfF41D
jsl84DLRW+3BCPOCbxRoosfMV61ZsWspqBG9/lMl3H2wk4MzRM+3FIdniLdz
4EX0PsCtm3OOz8Eg8YM34q12nR/YCCZ+cfKbSZtVi6xu4ifBmqM1JnWy4UX8
5szVe3HdchzoET8S9tyMezudA2XiV/nFc22igziwJn5WqvDVreMuBxeI35X6
dp5ljKdQRfxQdYfDiiZXCuXELwtUXdiBKRQmED+1EMe9Zb6gwCd+26PQ/9JT
kYs+4scDQY6FOuO5w36tXy33Z4+MA/7z84lfXj8wkp3/z++jPMOSDylwh3ng
/uzjef711DAvDLr+CJ1xiMIawhOaxZ/o4qkUzAhvHJy2PGlMuex9CI+0H2CP
gDUHEsIr+tuU6xOK2NhLeMbm61uFa3w2HAnvHFZfZbYqigUe4SHGB9PaxX06
OEp4yb/PeHXeeh2UEp5KL41YE/Keie2Et1xOGLMeRTPhRngsj+M0dr4VU8ZR
hNeqEw9ms5loIzyX4RQtl6XFxAjCe6Iy+YOP9Zk4SXjQfm5Q0G1XJowIL/Yz
bN/cSWPiFOHJmQooVvrBxDnCm7UO6Q2mrjpoIzzqdZP1TL5MB+mEVyWi6Egj
WR/7EZ7N3XS7Zm4oCxMJ75Z9LzP0rJBxA+HhNItGcbkKG4OEl39vt08OM2Qj
lPD01QhNjUQLNmwJbzN9pn0ZNGMjg/B4Brt1o61MZ/sIr0d2SCfUf2JBTHhe
7ZCuW+Al1jDvO12N78xewhrOA1FlCaxjnTrDecFiVvdXNS8dDJA8Efgjh/Gi
hQma5I3uZ/J0oQ0TYSSPeGkuCbAsmoR4kldSIpaPazeYhA6SZ3itufJm2Qz4
kLzz8/MXe6kpA10kD/m/DXSs6tHGc5KXFvbdVVuWq43HJE9lVw1k2BzTxiuS
t6ascKjxTdDGO5LHmDELeteUaMOK5DW5fz8MKPx7FOD/4E/baw==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.8548000000000044`}, \
{0, 1}], LineBox[CompressedData["
1:eJw9lws0VOsbxl0qKaUSpmGYMTN79t6cXKJUynMKFYcQkYREF7eKQld0UR2K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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 14.145199999999996`}, \
{0, -1}], 
       {PointSize[0.04], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox[
           FormBox["4",
            TraditionalForm],
           TraditionalForm,
           Editable->True],
          FontFamily->"Times",
          FontSize->6.597818181818183],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          6.597818181818183]& ], {10., -0.5}, {0, -1}]},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {66, 0}, {0, 0}, {22, 22}]},
   AspectRatio->NCache[
     Rational[1, 4], 0.25],
   PlotRange->{{0, 88}, {0, 22}}], TraditionalForm]], "Print",
 CellChangeTimes->{3.472899866735516*^9, 3.472900417710916*^9}]
}, Open  ]],

Cell["\<\
Inserting the analytical expressions. The momentum flowing into the loop is \
p, since the outgoing momentum is k1 and by definition from the Model file \
p1+k1=0 we have to replace k1 by -p. The loop momentum is q and the Lorentz \
indices of the external gluons are \[Mu] and \[Nu].\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"amps", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"CreateFeynAmp", "[", 
       RowBox[{"inserts", ",", 
        RowBox[{"Truncated", "\[Rule]", "True"}], ",", 
        RowBox[{"PreFactor", "\[Rule]", "1"}], ",", 
        RowBox[{"AmplitudeLevel", "\[Rule]", 
         RowBox[{"{", "Classes", "}"}]}]}], "]"}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"p1", "\[RuleDelayed]", "p"}], ",", 
        RowBox[{"q1", "\[RuleDelayed]", "q"}], ",", 
        RowBox[{"li1", "\[RuleDelayed]", "\[Mu]"}], ",", 
        RowBox[{"li2", "\[RuleDelayed]", "\[Nu]"}]}], "}"}]}], "/.", 
     RowBox[{
      RowBox[{"FeynAmpList", "[", "__", "]"}], "\[RuleDelayed]", "List"}]}], 
    " ", "/.", " ", 
    RowBox[{
     RowBox[{"FeynAmp", "[", 
      RowBox[{"_", ",", "_", ",", "x_"}], "]"}], "\[RuleDelayed]", "x"}]}]}], 
  ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Calculating the bare 2-loop on-shell gluon selfenergy diagrams\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", "amps", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", "q", ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["V", 
          RowBox[{"ci1", "ci2", "ci4", "ci4"}], 
          RowBox[{"\[Mu]", "\[Nu]", "li3", "li4"}]], "(", 
         RowBox[{"p", 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"-", "p"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"-", "q"}],
           TraditionalForm], 
          RowBox[{",", " "}], "q"}], ")"}]}]},
      {
       RowBox[{"-", 
        RowBox[{"tr", "(", 
         FormBox[
          RowBox[{"2", " ", "Tf", " ", 
           SubscriptBox["N", "f"], " ", 
           RowBox[{
            RowBox[{
             SubscriptBox["\[CapitalPi]", "q"], "(", "q", ")"}], ".", 
            SubscriptBox["T", 
             FormBox["ci2",
              TraditionalForm]], ".", 
            SuperscriptBox["Q", "\[Nu]"], ".", 
            RowBox[{
             SubscriptBox["\[CapitalPi]", "q"], "(", 
             FormBox[
              RowBox[{"q", "-", "p"}],
              TraditionalForm], ")"}], ".", 
            SubscriptBox["T", 
             FormBox["ci1",
              TraditionalForm]], ".", 
            SuperscriptBox["Q", "\[Mu]"]}]}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          SubscriptBox["\[CapitalPi]", "u"], "(", "q", ")"}]}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", "q", ")"}], 
        " ", 
        SubscriptBox["f", 
         RowBox[{"ci5", "ci3", "ci1"}]], " ", 
        SubscriptBox["f", 
         RowBox[{"ci3", "ci5", "ci2"}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
         FormBox[
          RowBox[{"q", "-", "p"}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{
        FractionBox["1", "2"], " ", 
        SubscriptBox["f", 
         RowBox[{"ci1", "ci4", "ci6"}]], " ", 
        SubscriptBox["f", 
         RowBox[{"ci2", "ci4", "ci6"}]], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", "q", ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li5", "li6"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox["V", 
          RowBox[{"\[Mu]", "li3", "li5"}]], "(", 
         RowBox[{"p", 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"-", "q"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"q", "-", "p"}],
           TraditionalForm]}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["V", 
          RowBox[{"\[Nu]", "li4", "li6"}]], "(", 
         RowBox[{
          FormBox[
           RowBox[{"-", "p"}],
           TraditionalForm], 
          RowBox[{",", " "}], "q", 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"p", "-", "q"}],
           TraditionalForm]}], ")"}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472899890759707*^9, 3.472900417842574*^9}]
}, Open  ]],

Cell["\<\
We recognize the combinatorical factors and minus signs provided by FeynArts.\
\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The ghost loop", "Subsection"],

Cell["\<\
Explicitly inserting the Feynman rules for a general gauge and summing the \
amplitudes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gh1", " ", "=", 
  RowBox[{"amps", "[", 
   RowBox[{"[", "3", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["\[CapitalPi]", "u"], "(", "q", ")"}]}], " ", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", "q", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{"ci5", "ci3", "ci1"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"ci3", "ci5", "ci2"}]], " ", 
   RowBox[{
    SubscriptBox["\[CapitalPi]", "u"], "(", 
    FormBox[
     RowBox[{"q", "-", "p"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
    FormBox[
     RowBox[{"q", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4728999043309803`*^9, 3.472900418178856*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gh2", "=", 
  RowBox[{"SUNSimplify", "[", 
   RowBox[{"Explicit", "[", 
    RowBox[{"gh1", ",", 
     RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], "-", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.4728999092836037`*^9, 3.47290041837123*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gh3", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Collect2", "[", 
      RowBox[{
       RowBox[{"Contract", "[", "gh2", "]"}], ",", "\[Xi]"}], "]"}], "/.", 
     RowBox[{"CA", "\[Rule]", "1"}]}], "/.", 
    RowBox[{"Gstrong", "\[Rule]", "1"}]}], "/.", 
   RowBox[{
    RowBox[{"SUNDelta", "[", "__", "]"}], "\[RuleDelayed]", 
    "1"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     SuperscriptBox["q", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     SuperscriptBox["q", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.472899935853232*^9, 3.472900418503943*^9}]
}, Open  ]],

Cell["\<\
Do a tensor integral decomposition by using the function TID.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gh4", "=", 
  RowBox[{"TID", "[", 
   RowBox[{"gh3", ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], 
        RowBox[{"1", "-", "D"}]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
         SuperscriptBox[
          FormBox[
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"],
           TraditionalForm], "2"]}]], "-", 
       FractionBox[
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.472899938247054*^9, 3.4729004186235647`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ToFI"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"ToFI[expr, {q1, q2}, {p}] translates all non-tensorial \\nloop \
integrals in expr into TFI notation from Tarcer. \\nToFI[expr, {q}, {p}] \
introduces TBI B0-like integrals. ToFI can be extended to more external \
particles and more loops if needed.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.4729004195876513`*^9},
 CellTags->"Info3472904018-5213457"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ghostloop", "=", 
  RowBox[{
   RowBox[{"ToFI", "[", 
    RowBox[{"gh4", ",", 
     RowBox[{"{", "q", "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}], "//", "Factor2"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"D", " ", 
        SuperscriptBox["p", "\[Mu]"], " ", 
        SuperscriptBox["p", "\[Nu]"]}]}], "-", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"]}]}], ")"}], " ", 
    InterpretationBox[
     SubsuperscriptBox[
      StyleBox["B",
       SingleLetterItalics->False,
       FontWeight->"Bold"], 
      RowBox[{
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}],
        TraditionalForm]}], 
      RowBox[{"(", 
       FormBox["D",
        TraditionalForm], ")"}]],
     HighEnergyPhysics`Tarcer`TBI[D, 
      HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], {{1,
       0}, {1, 0}}],
     Editable->True]}], 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "D"}], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.4728999429308*^9, 3.472900420222782*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input"],

Cell["\<\
((2*FVD[p, \[Mu]]*FVD[p, \[Nu]] - D*FVD[p, \[Mu]]*FVD[p, \[Nu]] - MTD[\[Mu], \
\[Nu]]*SPD[p, p])*
  TBI[D, SPD[p, p], {{1, 0}, {1, 0}}])/(4*(1 - D))\
\>", "Output",
 CellChangeTimes->{3.472899946452875*^9, 3.4729004202325153`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The gluon loop", "Subsection"],

Cell["\<\
Explicitly inserting the Feynman rules for a general gauge and summing the \
amplitudes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl1", " ", "=", 
  RowBox[{"amps", "[", 
   RowBox[{"[", "4", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SubscriptBox["f", 
    RowBox[{"ci1", "ci4", "ci6"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"ci2", "ci4", "ci6"}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", "g", 
     RowBox[{"li3", "li4"}]], "(", "q", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", "g", 
     RowBox[{"li5", "li6"}]], "(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{"\[Mu]", "li3", "li5"}]], "(", 
    RowBox[{"p", 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"-", "q"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{"\[Nu]", "li4", "li6"}]], "(", 
    RowBox[{
     FormBox[
      RowBox[{"-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], "q", 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4728999500211277`*^9, 3.472900420362858*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl2", "=", 
  RowBox[{"SUNSimplify", "[", 
   RowBox[{"Explicit", "[", 
    RowBox[{"gl1", ",", 
     RowBox[{"Gauge", "\[Rule]", 
      RowBox[{"1", "-", "\[Xi]"}]}], ",", 
     RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      SuperscriptBox["q", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"]}]], 
    RowBox[{
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"\[Xi]", " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["li3",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["li4",
            TraditionalForm],
           TraditionalForm]]}], 
        SuperscriptBox["q", "2"]], "-", 
       SuperscriptBox["g", 
        RowBox[{"li3", "li4"}]]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li5", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["li3",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["li3",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li3", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["li5",
             TraditionalForm],
            TraditionalForm]], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["li5",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li3", "li5"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li6", "\[Nu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["li4",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["li4",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li4", "\[Nu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["li6",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["li6",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"li4", "li6"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"\[Xi]", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["li5",
              TraditionalForm],
             TraditionalForm]], "-", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["li5",
              TraditionalForm],
             TraditionalForm]]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["li6",
              TraditionalForm],
             TraditionalForm]], "-", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["li6",
              TraditionalForm],
             TraditionalForm]]}], ")"}]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       SuperscriptBox["g", 
        RowBox[{"li5", "li6"}]]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472899956559909*^9, 3.472900420583275*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl3", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Collect2", "[", 
      RowBox[{
       RowBox[{"Contract", "[", "gl2", "]"}], ",", "\[Xi]"}], "]"}], "/.", 
     RowBox[{"CA", "\[Rule]", "1"}]}], "/.", 
    RowBox[{"Gstrong", "\[Rule]", "1"}]}], "/.", 
   RowBox[{
    RowBox[{"SUNDelta", "[", "__", "]"}], "\[RuleDelayed]", 
    "1"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox["\[Xi]", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"]], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["1", 
       SuperscriptBox["q", "2"]], ")"}], "2"]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"D", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"2", " ", "D", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"2", " ", "D", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"4", " ", "D", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"5", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], "2"]}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
     SuperscriptBox["q", "2"]}]], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
      SuperscriptBox["q", "2"]}]], 
    RowBox[{"\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
          SuperscriptBox[
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm], "2"]}], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"p", "-", "q"}], ")"}], "2"]]}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "+", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}],
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          FormBox[
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], "2"],
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          FormBox[
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], "2"],
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "+", 
       FractionBox[
        RowBox[{"3", " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], 
        SuperscriptBox["q", "2"]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "+", 
       FractionBox[
        RowBox[{"3", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], 
        SuperscriptBox["q", "2"]], "-", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], 
        SuperscriptBox["q", "2"]], "-", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "2"]], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], " ", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], 
        SuperscriptBox["q", "2"]]}], ")"}]}]}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.472899961253099*^9, 3.472900420724175*^9}]
}, Open  ]],

Cell[TextData[{
 "This is the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "\[Mu]\[Nu]"], TraditionalForm]]],
 "in eq. (A.3) from T.Muta's book \"Foundation of Quantum Chromodynamics\"."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"Coefficient", "[", 
       RowBox[{"gl3", ",", "\[Xi]", ",", "0"}], "]"}]}], "/.", 
     RowBox[{"p", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{"-", "k"}], ")"}]}]}], "/.", 
    RowBox[{
     RowBox[{"FeynAmpDenominator", "[", "__", "]"}], "\[RuleDelayed]", 
     "1"}]}], ",", "LorentzIndex", ",", 
   RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox[
      FormBox[
       TagBox[
        StyleBox["\<\"k\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox[
      FormBox[
       TagBox[
        StyleBox["\<\"k\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "6"}], ")"}], " ", 
    SuperscriptBox[
     FormBox[
      FormBox[
       TagBox[
        StyleBox["\<\"k\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox[
      FormBox[
       TagBox[
        StyleBox["\<\"k\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", "D"}], "-", "6"}], ")"}], " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], "2"]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472899967428982*^9, 3.472900420773696*^9}]
}, Open  ]],

Cell[TextData[{
 "This should be the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "\[Mu]\[Nu]"], TraditionalForm]]],
 "in eq. (A.4) from T.Muta's book \"Foundation of Quantum Chromodynamics\"."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bmunu", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"FCI", "[", 
     RowBox[{"FCE", "[", 
      RowBox[{
       RowBox[{"FDS", "[", 
        RowBox[{
         RowBox[{"Coefficient", "[", 
          RowBox[{
           RowBox[{"2", " ", "gl3"}], ",", "\[Xi]", ",", "1"}], "]"}], ",", 
         "q"}], "]"}], "/.", 
       RowBox[{"p", "\[Rule]", 
        RowBox[{"(", 
         RowBox[{"-", "k"}], ")"}]}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          RowBox[{
           FormBox[
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm],
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], "+", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], ")"}], "2"]}], 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm], "+", "q"}], ")"}], "2"]}]]}], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox[
       FormBox[
        TagBox[
         StyleBox["\<\"k\"\>",
          FontSlant->"Italic"],
         StyleForm[#, FontSlant -> "Italic"]& ],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox[
       FormBox[
        TagBox[
         StyleBox["\<\"k\"\>",
          FontSlant->"Italic"],
         StyleForm[#, FontSlant -> "Italic"]& ],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm], "+", "q"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", 
         RowBox[{
          FormBox[
           FormBox[
            TagBox[
             StyleBox["\<\"k\"\>",
              FontSlant->"Italic"],
             StyleForm[#, FontSlant -> "Italic"]& ],
            TraditionalForm],
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}]}], "+", 
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], "2"], "-", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm], "+", "q"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox[
       FormBox[
        TagBox[
         StyleBox["\<\"k\"\>",
          FontSlant->"Italic"],
         StyleForm[#, FontSlant -> "Italic"]& ],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", 
        RowBox[{
         FormBox[
          FormBox[
           TagBox[
            StyleBox["\<\"k\"\>",
             FontSlant->"Italic"],
            StyleForm[#, FontSlant -> "Italic"]& ],
           TraditionalForm],
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm], "+", "q"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox[
       FormBox[
        TagBox[
         StyleBox["\<\"k\"\>",
          FontSlant->"Italic"],
         StyleForm[#, FontSlant -> "Italic"]& ],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", 
        RowBox[{
         FormBox[
          FormBox[
           TagBox[
            StyleBox["\<\"k\"\>",
             FontSlant->"Italic"],
            StyleForm[#, FontSlant -> "Italic"]& ],
           TraditionalForm],
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm], "+", "q"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.472900109456677*^9, 3.4729004212486277`*^9}]
}, Open  ]],

Cell["\<\
However, it is not evident that it is really the same. Therefore we enter eq. \
(A.4) and subtract it from the above.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mutabmunu1", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"q", "+", "k"}]}], "]"}], " ", ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"FAD", "[", "q", "]"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"SPD", "[", "q", "]"}], "+", 
         RowBox[{"2", 
          RowBox[{"SPD", "[", 
           RowBox[{"k", ",", "q"}], "]"}]}]}], ")"}], "^", "2"}], 
      RowBox[{"MTD", "[", 
       RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], " ", "+", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"FAD", "[", "q", "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"q", ",", "\[Nu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"q", ",", "\[Mu]"}], "]"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SPD", "[", "q", "]"}], "+", 
        RowBox[{"2", 
         RowBox[{"SPD", "[", 
          RowBox[{"k", ",", "q"}], "]"}]}], "-", 
        RowBox[{"SPD", "[", "k", "]"}]}], ")"}]}], "+", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"FAD", "[", "q", "]"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SPD", "[", "q", "]"}], "+", 
        RowBox[{"3", 
         RowBox[{"SPD", "[", 
          RowBox[{"k", ",", "q"}], "]"}]}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"FVD", "[", 
          RowBox[{"q", ",", "\[Mu]"}], "]"}], 
         RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Nu]"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{"FVD", "[", 
          RowBox[{"q", ",", "\[Nu]"}], "]"}], 
         RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}], ")"}]}], "-", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"FVD", "[", 
       RowBox[{"k", ",", "\[Mu]"}], "]"}], 
      RowBox[{"FVD", "[", 
       RowBox[{"k", ",", "\[Nu]"}], "]"}]}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", $CellContext`q^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm], "+", "q"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", ($CellContext`k + $CellContext`q)^2, "]"],
        Editable->False], ")"}]}]],
    TraditionalForm], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          FormBox[
           RowBox[{
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}],
           TraditionalForm]}], "+", 
         FormBox[
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], "2"],
          TraditionalForm]}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        FormBox[
         FractionBox["1", 
          InterpretationBox[
           RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
            SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
           SequenceForm["[", $CellContext`q^2, "]"],
           Editable->False]],
         TraditionalForm]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["q", "\[Mu]"], " ", 
      SuperscriptBox["q", "\[Nu]"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         FormBox[
          RowBox[{
           FormBox[
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm],
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm]}], "-", 
        FormBox[
         SuperscriptBox[
          FormBox[
           FormBox[
            TagBox[
             StyleBox["\<\"k\"\>",
              FontSlant->"Italic"],
             StyleForm[#, FontSlant -> "Italic"]& ],
            TraditionalForm],
           TraditionalForm], "2"],
         TraditionalForm], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}], " ", 
      FormBox[
       FractionBox["1", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q^2, "]"],
         Editable->False]],
       TraditionalForm]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", " ", 
         FormBox[
          RowBox[{
           FormBox[
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm],
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm]}], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["q", "\[Nu]"], " ", 
         SuperscriptBox[
          FormBox[
           FormBox[
            TagBox[
             StyleBox["\<\"k\"\>",
              FontSlant->"Italic"],
             StyleForm[#, FontSlant -> "Italic"]& ],
            TraditionalForm],
           TraditionalForm], "\[Mu]"]}], "+", 
        RowBox[{
         SuperscriptBox["q", "\[Mu]"], " ", 
         SuperscriptBox[
          FormBox[
           FormBox[
            TagBox[
             StyleBox["\<\"k\"\>",
              FontSlant->"Italic"],
             StyleForm[#, FontSlant -> "Italic"]& ],
            TraditionalForm],
           TraditionalForm], "\[Nu]"]}]}], ")"}], " ", 
      FormBox[
       FractionBox["1", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q^2, "]"],
         Editable->False]],
       TraditionalForm]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "\[Mu]"], " ", 
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729001854770813`*^9, 3.4729004213008223`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mutabmunu", "=", 
  RowBox[{"FCI", "[", 
   RowBox[{"mutabmunu1", "+", " ", 
    RowBox[{"(", 
     RowBox[{"mutabmunu1", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", ":>", 
         RowBox[{"(", 
          RowBox[{"q", "+", "k"}], ")"}]}], ",", 
        RowBox[{"k", "\[RuleDelayed]", 
         RowBox[{"-", "k"}]}]}], "}"}]}], ")"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm], "+", "q"}], ")"}], "2"]}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             RowBox[{
              FormBox[
               FormBox[
                TagBox[
                 StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                 StyleForm[#, FontSlant -> "Italic"]& ],
                TraditionalForm],
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], "+", 
            SuperscriptBox[
             FormBox["q",
              TraditionalForm], "2"]}], ")"}], "2"]}], 
        SuperscriptBox["q", "2"]]}], "+", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           RowBox[{
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], "-", 
          SuperscriptBox[
           FormBox[
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm],
            TraditionalForm], "2"], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], "2"]}], ")"}]}], 
       SuperscriptBox["q", "2"]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", 
           RowBox[{
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}], 
       SuperscriptBox["q", "2"]], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm], "+", "q"}], ")"}], "2"]}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              FormBox[
               RowBox[{
                FormBox[
                 TagBox[
                  StyleBox["\<\"k\"\>",
                   FontSlant->"Italic"],
                  StyleForm[#, FontSlant -> "Italic"]& ],
                 TraditionalForm], "+", "q"}],
               TraditionalForm], ")"}], "2"], "-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox[
               FormBox[
                TagBox[
                 StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                 StyleForm[#, FontSlant -> "Italic"]& ],
                TraditionalForm],
               TraditionalForm], "\[CenterDot]", "(", 
              FormBox[
               RowBox[{
                FormBox[
                 TagBox[
                  StyleBox["\<\"k\"\>",
                   FontSlant->"Italic"],
                  StyleForm[#, FontSlant -> "Italic"]& ],
                 TraditionalForm], "+", "q"}],
               TraditionalForm], ")"}]}]}], ")"}], "2"]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox[
            TagBox[
             StyleBox["\<\"k\"\>",
              FontSlant->"Italic"],
             StyleForm[#, FontSlant -> "Italic"]& ],
            TraditionalForm], "+", "q"}], ")"}], "2"]]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            FormBox[
             RowBox[{
              FormBox[
               TagBox[
                StyleBox["\<\"k\"\>",
                 FontSlant->"Italic"],
                StyleForm[#, FontSlant -> "Italic"]& ],
               TraditionalForm], "+", "q"}],
             TraditionalForm], ")"}], "2"], "-", 
          RowBox[{"3", " ", 
           RowBox[{
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], "\[CenterDot]", "(", 
            FormBox[
             RowBox[{
              FormBox[
               TagBox[
                StyleBox["\<\"k\"\>",
                 FontSlant->"Italic"],
                StyleForm[#, FontSlant -> "Italic"]& ],
               TraditionalForm], "+", "q"}],
             TraditionalForm], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           RowBox[{"(", 
            RowBox[{"-", 
             SuperscriptBox[
              RowBox[{"(", 
               FormBox[
                RowBox[{
                 FormBox[
                  TagBox[
                   StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                   StyleForm[#, FontSlant -> "Italic"]& ],
                  TraditionalForm], "+", "q"}],
                TraditionalForm], ")"}], 
              FormBox[
               FormBox["\[Mu]",
                TraditionalForm],
               TraditionalForm]]}], ")"}]}], "-", 
          RowBox[{
           SuperscriptBox[
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             FormBox[
              RowBox[{
               FormBox[
                TagBox[
                 StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                 StyleForm[#, FontSlant -> "Italic"]& ],
                TraditionalForm], "+", "q"}],
              TraditionalForm], ")"}], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox[
           TagBox[
            StyleBox["\<\"k\"\>",
             FontSlant->"Italic"],
            StyleForm[#, FontSlant -> "Italic"]& ],
           TraditionalForm], "+", "q"}], ")"}], "2"]], "+", 
      FractionBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm], "+", "q"}],
           TraditionalForm], ")"}], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          FormBox[
           RowBox[{
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm], "+", "q"}],
           TraditionalForm], ")"}], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            FormBox[
             RowBox[{
              FormBox[
               TagBox[
                StyleBox["\<\"k\"\>",
                 FontSlant->"Italic"],
                StyleForm[#, FontSlant -> "Italic"]& ],
               TraditionalForm], "+", "q"}],
             TraditionalForm], ")"}], "2"], "-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox[
             FormBox[
              TagBox[
               StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
               StyleForm[#, FontSlant -> "Italic"]& ],
              TraditionalForm],
             TraditionalForm], "\[CenterDot]", "(", 
            FormBox[
             RowBox[{
              FormBox[
               TagBox[
                StyleBox["\<\"k\"\>",
                 FontSlant->"Italic"],
                StyleForm[#, FontSlant -> "Italic"]& ],
               TraditionalForm], "+", "q"}],
             TraditionalForm], ")"}]}], "-", 
          SuperscriptBox[
           FormBox[
            FormBox[
             TagBox[
              StyleBox["\<\"k\"\>",
               FontSlant->"Italic"],
              StyleForm[#, FontSlant -> "Italic"]& ],
             TraditionalForm],
            TraditionalForm], "2"]}], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox[
           TagBox[
            StyleBox["\<\"k\"\>",
             FontSlant->"Italic"],
            StyleForm[#, FontSlant -> "Italic"]& ],
           TraditionalForm], "+", "q"}], ")"}], "2"]], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox[
         FormBox[
          TagBox[
           StyleBox["\<\"k\"\>",
            FontSlant->"Italic"],
           StyleForm[#, FontSlant -> "Italic"]& ],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900193539386*^9, 3.472900421405242*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"ScalarProductCancel", "@", 
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
     RowBox[{
      RowBox[{"Calc", "[", 
       RowBox[{"mutabmunu", "-", " ", "bmunu"}], "]"}], ",", "q"}], "]"}]}], 
   ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.472900196331191*^9, 3.4729004221254587`*^9}]
}, Open  ]],

Cell["This proves that bmunu is equivalent to (A.4) of T. Muta.", "Text"],

Cell[TextData[{
 "Project out  the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Mu]\[Nu]"], TraditionalForm]]],
 "in eq. (A.5) from T.Muta's book."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", 
  RowBox[{
   RowBox[{"FCE", "[", 
    RowBox[{
     RowBox[{"Coefficient", "[", " ", 
      RowBox[{
       RowBox[{"2", " ", "gl3"}], ",", "\[Xi]", ",", "2"}], "]"}], "/.", 
     RowBox[{"p", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{"-", "k"}], ")"}]}]}], "]"}], "/.", 
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q", ",", "q", ",", 
      RowBox[{"q", "+", "k"}], ",", 
      RowBox[{"q", "+", "k"}]}], "]"}], "\[RuleDelayed]", 
    RowBox[{"FAD", "[", 
     RowBox[{"q", ",", 
      RowBox[{"q", "+", "k"}]}], "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      SuperscriptBox["q", "2"]], ")"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FormBox[
           TagBox[
            StyleBox["\<\"k\"\>",
             FontSlant->"Italic"],
            StyleForm[#, FontSlant -> "Italic"]& ],
           TraditionalForm]}], "-", "q"}], ")"}], "2"]], ")"}], "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        FormBox[
         TagBox[
          StyleBox["\<\"k\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ],
         TraditionalForm],
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900422429133*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl4", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
     RowBox[{"gl3", ",", "q"}], "]"}], ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "\[Xi]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", "\[Xi]", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", "\[Xi]", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", "\[Xi]", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"4", " ", "\[Xi]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"D", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"5", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.472900422632856*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TID"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"TID[amp, q] does a 1-loop tensor integral decomposition, \
transforming the\\nLorentz indices away from the integration momentum q.\"\>",
   "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.4729004232306643`*^9},
 CellTags->"Info3472904022-5473604"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl4", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.4729004237100763`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl4", "//", "Length"}]], "Input"],

Cell[BoxData[
 FormBox["21", TraditionalForm]], "Output",
 CellChangeTimes->{3.472900423769051*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gl4", "[", 
    RowBox[{"[", "11", "]"}], "]"}], "//", "FCE"}], "//", 
  "InputForm"}]], "Input"],

Cell["\<\
-(\[Xi]^2*FAD[q, q, -p + q, -p + q]*FVD[p, \[Nu]]*FVD[q, \[Mu]]*SPD[p, \
p]*SPD[p, q])/2\
\>", "Output",
 CellChangeTimes->{3.472900423777227*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl5", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"TID", "[", 
     RowBox[{"gl4", ",", "q"}], "]"}], ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", "\[Xi]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"]}]}], ")"}], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm], "3"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "1"}], ")"}], " ", 
      RowBox[{
       SuperscriptBox["q", "2"], ".", 
       SuperscriptBox["q", "2"], ".", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
      SuperscriptBox[
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "2"]}]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "5"}], " ", "D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"6", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"4", " ", "D", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"5", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", "\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "D"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"2", " ", "D", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"D", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"D", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{"5", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Xi]", "2"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{"\[Xi]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", "D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{"3", " ", "D", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900423935884*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gl6", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"ToFI", "[", 
     RowBox[{"gl5", ",", 
      RowBox[{"{", "q", "}"}], ",", 
      RowBox[{"{", "p", "}"}]}], "]"}], "//", "FCE"}], "//", 
   "Factor2"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"8", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}]}]], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "6"], " ", 
       RowBox[{"(", 
        RowBox[{"-", 
         SuperscriptBox["\[Xi]", "2"]}], ")"}], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {2, 0}}],
        Editable->True]}], "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "4"], " ", 
       SuperscriptBox["\[Xi]", "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"2", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       SuperscriptBox["\[Xi]", "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"8", " ", "D", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "4"], " ", "\[Xi]", " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"12", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "4"], " ", "\[Xi]", " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"8", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", "\[Xi]", " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"12", " ", "D", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"10", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "4"], " ", 
       SuperscriptBox["\[Xi]", "2"], " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {2, 0}}],
        Editable->True]}], "-", 
      RowBox[{"4", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       SuperscriptBox["\[Xi]", "2"], " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"2", " ", 
       SuperscriptBox["\[Xi]", "2"], " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"8", " ", "D", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", "\[Xi]", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"12", " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", "\[Xi]", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{2, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"8", " ", "\[Xi]", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "-", 
      RowBox[{"14", " ", "D", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}], "+", 
      RowBox[{"12", " ", 
       SuperscriptBox["p", "\[Mu]"], " ", 
       SuperscriptBox["p", "\[Nu]"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[D, 
         HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, 0}, {1, 0}}],
        Editable->True]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729004249115467`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gluonloop", "=", 
  RowBox[{
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{"TarcerRecurse", "[", "gl6", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}]}], "]"}], "/.", 
   RowBox[{"a_Plus", "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"Collect2", "[", 
      RowBox[{"a", ",", "\[Xi]"}], "]"}], "/;", 
     RowBox[{"FreeQ", "[", 
      RowBox[{"a", ",", "TBI"}], "]"}]}]}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"D", "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "D"}], "-", "7"}], ")"}], " ", "\[Xi]"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"7", " ", "D"}], "-", "6"}], ")"}]}]}], ")"}], " ", 
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["p", "\[Nu]"], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["D",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TBI[D, 
       HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], {{
       1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"D", "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "D"}], "-", "7"}], ")"}], " ", "\[Xi]"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"6", " ", "D"}], "-", "5"}], ")"}]}]}], ")"}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["D",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TBI[D, 
       HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], {{
       1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729004258697567`*^9}]
}, Open  ]],

Cell["\<\
Only by adding the ghost contribution transversality is restored.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nonfermionicresult", "=", 
  RowBox[{
   RowBox[{"Factor2", "[", 
    RowBox[{"gluonloop", "+", "ghostloop"}], "]"}], "/.", 
   RowBox[{"h_Plus", "\[RuleDelayed]", 
    RowBox[{"Collect2", "[", 
     RowBox[{"h", ",", "\[Xi]"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"D", "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "D"}], "-", "7"}], ")"}], " ", "\[Xi]"}], "-", 
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["p", "\[Mu]"], " ", 
        SuperscriptBox["p", "\[Nu]"]}], "-", 
       RowBox[{
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["B",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["D",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TBI[D, 
       HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], {{
       1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900426085343*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "nonfermionicresult"}], 
  "]"}]], "Input"],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.4729004264776154`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The fermion loop", "Subsection"],

Cell["\<\
Explicitly inserting the Feynman rules for a general gauge and summing the \
amplitudes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fl1", "=", 
  RowBox[{
  "amps", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{"2", " ", "Tf", " ", 
      SubscriptBox["N", "f"], " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[CapitalPi]", "q"], "(", "q", ")"}], ".", 
       SubscriptBox["T", 
        FormBox["ci2",
         TraditionalForm]], ".", 
       SuperscriptBox["Q", "\[Nu]"], ".", 
       RowBox[{
        SubscriptBox["\[CapitalPi]", "q"], "(", 
        FormBox[
         RowBox[{"q", "-", "p"}],
         TraditionalForm], ")"}], ".", 
       SubscriptBox["T", 
        FormBox["ci1",
         TraditionalForm]], ".", 
       SuperscriptBox["Q", "\[Mu]"]}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729004267083282`*^9}]
}, Open  ]],

Cell["\<\
Replace the \"QuarkMass\" variable from the FCQCDLorentz Model file by \"M\".\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fl2", "=", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"Explicit", "[", 
     RowBox[{"fl1", ",", 
      RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], "/.", 
   RowBox[{"QuarkMass", " ", "\[Rule]", " ", "M"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"M", "+", 
          RowBox[{
           FormBox["\<\"\[Gamma]\"\>",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], ".", 
        RowBox[{"(", 
         RowBox[{"M", "-", 
          RowBox[{
           FormBox["\<\"\[Gamma]\"\>",
            TraditionalForm], "\[CenterDot]", "(", 
           FormBox[
            RowBox[{"p", "-", "q"}],
            TraditionalForm], ")"}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900426826913*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fl3", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Contract", "[", 
      RowBox[{"fl2", "/.", "\[InvisibleSpace]", 
       RowBox[{"DiracTrace", "\[Rule]", "TR"}]}], "]"}], "/.", 
     "\[InvisibleSpace]", 
     RowBox[{"CA", "\[Rule]", "1"}]}], "/.", "\[InvisibleSpace]", 
    RowBox[{"Gstrong", "\[Rule]", "1"}]}], "/.", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"SUNDelta", "[", "__", "]"}], "\[RuleDelayed]", 
    "1"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"4", " ", 
      SuperscriptBox["M", "2"], " ", "Tf", " ", 
      SubscriptBox["N", "f"], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}]}]]}], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"], " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"4", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"4", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"8", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900426944899*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fl4", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"TID", "[", 
     RowBox[{"fl3", ",", "q"}], "]"}], ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      FormBox["q",
       TraditionalForm], "2"], " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}]}]}]], "-", 
   FractionBox[
    RowBox[{"8", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}]}]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox["M", "2"], " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}]}]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.472900427058379*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cc", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"SPC", "[", 
     RowBox[{
      RowBox[{"SPC", "[", 
       RowBox[{"fl4", ",", "q", ",", 
        RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}], ",", "q", ",", 
      RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}], ",", "q"}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"q", "-", "p"}], ")"}], "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}]}]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.472900427218793*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fermionloop", "=", 
  RowBox[{"Factor2", "[", 
   RowBox[{"ToTFI", "[", 
    RowBox[{"fl4", ",", "q", ",", "p"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "Tf", " ", 
     SubscriptBox["N", "f"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["p", "\[Mu]"], " ", 
        SuperscriptBox["p", "\[Nu]"]}], "-", 
       RowBox[{
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", "D", " ", 
         InterpretationBox[
          SubsuperscriptBox[
           StyleBox["A",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm], 
           RowBox[{"(", 
            FormBox["D",
             TraditionalForm], ")"}]],
          HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`M}}],
          Editable->True]}]}], "+", 
       RowBox[{"4", " ", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["A",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "M"}], "}"}],
           TraditionalForm], 
          RowBox[{"(", 
           FormBox["D",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`M}}],
         Editable->True]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["M", "2"], " ", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["B",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          RowBox[{
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm]}], 
          RowBox[{"(", 
           FormBox["D",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TBI[D, 
          
          HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, $CellContext`M}, {1, $CellContext`M}}],
         Editable->True]}], "+", 
       RowBox[{"D", " ", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], " ", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["B",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          RowBox[{
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm]}], 
          RowBox[{"(", 
           FormBox["D",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TBI[D, 
          
          HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, $CellContext`M}, {1, $CellContext`M}}],
         Editable->True]}], "-", 
       RowBox[{"2", " ", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], " ", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["B",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          RowBox[{
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}],
            TraditionalForm]}], 
          RowBox[{"(", 
           FormBox["D",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TBI[D, 
          
          HighEnergyPhysics`FeynCalc`SPD`SPD[$CellContext`p, $CellContext`p], \
{{1, $CellContext`M}, {1, $CellContext`M}}],
         Editable->True]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472900427779101*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["5.829999999999999`", TraditionalForm]], "Output",
 CellChangeTimes->{3.472900427832282*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{853, 634},
WindowMargins->{{0, Automatic}, {-66, Automatic}},
ShowSelection->True,
Magnification->1,
FrontEndVersion->"7.0 for Linux x86 (64-bit) (February 25, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3472904018-5213457"->{
  Cell[58657, 1399, 397, 6, 104, "Print",
   CellTags->"Info3472904018-5213457"]},
 "Info3472904022-5473604"->{
  Cell[137218, 4259, 282, 5, 64, "Print",
   CellTags->"Info3472904022-5473604"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3472904018-5213457", 197469, 6417},
 {"Info3472904022-5473604", 197581, 6420}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 61, 0, 39, "Subsection"],
Cell[631, 24, 71, 1, 32, "Input"],
Cell[705, 27, 214, 4, 18, "Text"],
Cell[922, 33, 234, 6, 18, "Text"],
Cell[1159, 41, 670, 22, 18, "Text"],
Cell[1832, 65, 166, 4, 18, "Text"],
Cell[2001, 71, 371, 12, 18, "Text"],
Cell[2375, 85, 196, 4, 18, "Text"],
Cell[CellGroupData[{
Cell[2596, 93, 35, 0, 29, "Subsubsection"],
Cell[2634, 95, 147, 3, 51, "Text"],
Cell[CellGroupData[{
Cell[2806, 102, 968, 24, 77, "Input"],
Cell[3777, 128, 41603, 780, 98, "Print"]
}, Open  ]],
Cell[45395, 911, 308, 5, 71, "Text"],
Cell[45706, 918, 897, 24, 77, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46640, 947, 95, 2, 29, "Subsubsection"],
Cell[CellGroupData[{
Cell[46760, 953, 65, 1, 32, "Input"],
Cell[46828, 956, 4083, 125, 131, "Output"]
}, Open  ]],
Cell[50926, 1084, 103, 3, 31, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[51078, 1093, 36, 0, 39, "Subsection"],
Cell[51117, 1095, 112, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[51254, 1102, 111, 3, 32, "Input"],
Cell[51368, 1107, 766, 24, 35, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52171, 1136, 192, 5, 32, "Input"],
Cell[52366, 1143, 1182, 46, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53585, 1194, 387, 12, 32, "Input"],
Cell[53975, 1208, 1119, 46, 55, "Output"]
}, Open  ]],
Cell[55109, 1257, 85, 2, 31, "Text"],
Cell[CellGroupData[{
Cell[55219, 1263, 107, 3, 32, "Input"],
Cell[55329, 1268, 3241, 123, 71, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58607, 1396, 47, 1, 32, "Input"],
Cell[58657, 1399, 397, 6, 104, "Print",
 CellTags->"Info3472904018-5213457"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59091, 1410, 208, 6, 32, "Input"],
Cell[59302, 1418, 1326, 45, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60665, 1468, 58, 1, 32, "Input"],
Cell[60726, 1471, 240, 5, 63, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[61015, 1482, 36, 0, 39, "Subsection"],
Cell[61054, 1484, 112, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[61191, 1491, 111, 3, 32, "Input"],
Cell[61305, 1496, 1167, 41, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62509, 1542, 269, 7, 32, "Input"],
Cell[62781, 1551, 5860, 208, 113, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68678, 1764, 387, 12, 32, "Input"],
Cell[69068, 1778, 16451, 605, 265, "Output"]
}, Open  ]],
Cell[85534, 2386, 203, 6, 31, "Text"],
Cell[CellGroupData[{
Cell[85762, 2396, 485, 14, 55, "Input"],
Cell[86250, 2412, 3252, 129, 35, "Output"]
}, Open  ]],
Cell[89517, 2544, 210, 6, 31, "Text"],
Cell[CellGroupData[{
Cell[89752, 2554, 550, 17, 32, "Input"],
Cell[90305, 2573, 6503, 239, 59, "Output"]
}, Open  ]],
Cell[96823, 2815, 141, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[96989, 2822, 1905, 59, 99, "Input"],
Cell[98897, 2883, 5653, 181, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[104587, 3069, 399, 12, 32, "Input"],
Cell[104989, 3083, 13806, 452, 165, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118832, 3540, 291, 8, 55, "Input"],
Cell[119126, 3550, 122, 2, 32, "Output"]
}, Open  ]],
Cell[119263, 3555, 73, 0, 31, "Text"],
Cell[119339, 3557, 167, 6, 31, "Text"],
Cell[CellGroupData[{
Cell[119531, 3567, 590, 18, 32, "Input"],
Cell[120124, 3587, 2869, 108, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123030, 3700, 193, 5, 32, "Input"],
Cell[123226, 3707, 13906, 544, 214, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137169, 4256, 46, 1, 32, "Input"],
Cell[137218, 4259, 282, 5, 64, "Print",
 CellTags->"Info3472904022-5473604"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137537, 4269, 79, 2, 32, "Input"],
Cell[137619, 4273, 729, 29, 55, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138385, 4307, 57, 1, 32, "Input"],
Cell[138445, 4310, 99, 2, 32, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138581, 4317, 144, 5, 32, "Input"],
Cell[138728, 4324, 156, 4, 46, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138921, 4333, 170, 5, 32, "Input"],
Cell[139094, 4340, 17165, 637, 271, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156296, 4982, 237, 8, 32, "Input"],
Cell[156536, 4992, 14173, 452, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[170746, 5449, 449, 13, 32, "Input"],
Cell[171198, 5464, 3006, 100, 114, "Output"]
}, Open  ]],
Cell[174219, 5567, 89, 2, 31, "Text"],
Cell[CellGroupData[{
Cell[174333, 5573, 275, 7, 32, "Input"],
Cell[174611, 5582, 1797, 60, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[176445, 5647, 164, 5, 32, "Input"],
Cell[176612, 5654, 100, 2, 32, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[176761, 5662, 38, 0, 39, "Subsection"],
Cell[176802, 5664, 112, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[176939, 5671, 124, 3, 32, "Input"],
Cell[177066, 5676, 754, 24, 34, "Output"]
}, Open  ]],
Cell[177835, 5703, 103, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[177963, 5710, 268, 7, 32, "Input"],
Cell[178234, 5719, 1655, 57, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179926, 5781, 478, 13, 32, "Input"],
Cell[180407, 5796, 3951, 145, 111, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[184395, 5946, 170, 5, 32, "Input"],
Cell[184568, 5953, 4764, 173, 114, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[189369, 6131, 333, 10, 32, "Input"],
Cell[189705, 6143, 2641, 97, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[192383, 6245, 163, 4, 32, "Input"],
Cell[192549, 6251, 4341, 140, 64, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196927, 6396, 56, 1, 32, "Input"],
Cell[196986, 6399, 115, 2, 32, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
